local s_meta = {
  ["generic/pst-abspos/pst-abspos-doc.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/bidi/bidi-doc.pdf"] = {["details"] = "Package user documentation", },
  ["generic/fixjfm/README"] = {["details"] = "Readme", },
  ["generic/randomlist/README.txt"] = {["details"] = "Readme", },
  ["latex/edmargin/edmargin.pdf"] = {["details"] = "Package documentation", },
  ["latex/pdfmarginpar/README"] = {["details"] = "Readme", },
  ["latex/stellenbosch/UStitle-1.0.pdf"] = {["lang"] = "en", ["details"] = "UStitle package documentation", },
  ["latex/hrlatex/README"] = {["details"] = "Readme", },
  ["latex/iwhdp/README"] = {["details"] = "Readme", },
  ["latex/marginnote/marginnote.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries/README"] = {["details"] = "Readme", },
  ["latex/komacv-rg/komacv-multilang.pdf"] = {["details"] = "Package documentation komacv-multilang", },
  ["luatex/interpreter/interpreter-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/movie15/overlay-example.pdf"] = {["details"] = "Overlay example", },
  ["latex/svninfo/svninfo.pdf"] = {["details"] = "Package documentation", },
  ["latex/makecell/makecell-rus.pdf"] = {["lang"] = "ru", ["details"] = "Package documentation", },
  ["latex/envlab/envlab.pdf"] = {["details"] = "Documentation source", },
  ["latex/localloc/localloc.pdf"] = {["details"] = "Package documentation", },
  ["latex/mylatexformat/README"] = {["details"] = "Readme", },
  ["latex/leadsheets/README"] = {["details"] = "Readme", },
  ["latex/greekdates/greekdates.pdf"] = {["details"] = "Package documentation", },
  ["latex/nddiss/nddiss2e.pdf"] = {["details"] = "Package documentation", },
  ["latex/frege/frege.pdf"] = {["details"] = "Package documentation", },
  ["latex/morehype/blog.pdf"] = {["details"] = "Package documentation: blog", },
  ["fonts/libertinus-otf/README.md"] = {["details"] = "Readme", },
  ["latex/hyphenat/README"] = {["details"] = "Readme", },
  ["latex/dk-bib/dk-bib.pdf"] = {["details"] = "Package documentation", },
  ["latex/latexcheat-esmx/README"] = {["lang"] = "es", ["details"] = "Readme", },
  ["metapost/featpost/doc/metapostpropaganda.pdf"] = {["details"] = "Promotional stuff", },
  ["latex/schwalbe-chess/schwalbe.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/lazylist/lazylist.pdf"] = {["details"] = "Paper prepared for TUGboat", },
  ["fonts/carlito/Carlito-Regular.pdf"] = {["details"] = "Font tables", },
  ["fonts/ocr-b-outline/README"] = {["details"] = "Readme", },
  ["latex/revquantum/README.md"] = {["details"] = "Readme", },
  ["latex/txgreeks/README"] = {["details"] = "Readme", },
  ["latex/gmp/gmp.pdf"] = {["details"] = "Package documentation", },
  ["latex/graphicxbox/doc/graphicxbox.pdf"] = {["details"] = "Package documentation", },
  ["fonts/dejavu-otf/README.md"] = {["details"] = "Readme", },
  ["latex/ionumbers/README"] = {["details"] = "Readme", },
  ["support/epspdf/index.html"] = {["details"] = "Package documentation", },
  ["latex/pdfscreen/manual-screen.pdf"] = {["details"] = "Manual, for screen viewing", },
  ["fonts/yhmath/README"] = {["details"] = "Readme", },
  ["context/third/visualcounter/visualcounter.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries-irish/README"] = {["details"] = "Readme", },
  ["generic/babel-latin/latin.pdf"] = {["details"] = "Package documentation", },
  ["fonts/crimson/README"] = {["details"] = "Readme", },
  ["latex/tikz-relay/doc/tikz-relay-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/fullblck/README"] = {["details"] = "Readme", },
  ["latex/refstyle/refstyle.pdf"] = {["details"] = "Users manual", },
  ["platex/pxchfon/pxchfon.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation", },
  ["fonts/levy/README"] = {["details"] = "Readme", },
  ["latex/hagenberg-thesis/examples/HgbThesisDE/main.pdf"] = {["lang"] = "de", ["details"] = "Example document / Tutorial (German)", },
  ["latex/graphicxpsd/README.md"] = {["details"] = "Readme", },
  ["latex/tikz-opm/README"] = {["details"] = "Readme", },
  ["fonts/bbm/README"] = {["details"] = "Readme", },
  ["bibtex/bath-bst/bath-bst-v1.pdf"] = {["details"] = "Documentation of version 1 style", },
  ["latex/dinbrief/dinbrief.pdf"] = {["details"] = "User Guide and Implementation", },
  ["latex/comprehensive/symbols-a4.pdf"] = {["details"] = "The document (for A4 paper)", },
  ["latex/datetime2-slovene/README"] = {["details"] = "Readme", },
  ["latex/nag/nag.pdf"] = {["details"] = "Package documentation", },
  ["latex/havannah/havannah.pdf"] = {["details"] = "Package documentation", },
  ["latex/gu/gudoc.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/uantwerpendocs/uantwerpenmasterthesis-example.pdf"] = {["details"] = "Example master thesis", },
  ["latex/sesamanuel/sesamath-doc-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/univie-ling/univie-ling-thesis.pdf"] = {["details"] = "Package documentation (thesis class)", },
  ["latex/mfpic4ode/README"] = {["details"] = "Readme", },
  ["latex/screenplay-pkg/README"] = {["details"] = "Readme", },
  ["latex/hhtensor/hhtensor.pdf"] = {["details"] = "Package documentation", },
  ["fonts/baskervald/baskervaldadf.pdf"] = {["details"] = "Package documentation", },
  ["latex/acmconf/README"] = {["details"] = "Readme", },
  ["latex/efbox/README"] = {["details"] = "Readme", },
  ["latex/basicarith/basicarith.pdf"] = {["details"] = "Package documentation", },
  ["latex/preview/README"] = {["details"] = "Readme", },
  ["latex/cjhebrew/README"] = {["details"] = "Readme", },
  ["latex/beamerswitch/beamerswitch-example-article.pdf"] = {["details"] = "Example of use (article)", },
  ["latex/biblatex-iso690/README.md"] = {["details"] = "Readme", },
  ["latex/enotez/enotez_en.pdf"] = {["details"] = "Package documentation", },
  ["support/lua-alt-getopt/README"] = {["details"] = "Readme", },
  ["fonts/gillius/gillius2-samples.pdf"] = {["details"] = "Gillius font samples", },
  ["latex/latexcheat-de/latexsheet-de.pdf"] = {["lang"] = "de", ["details"] = "The document itself", },
  ["latex/phonenumbers/phonenumbers-en.pdf"] = {["lang"] = "en-gb", ["details"] = "English package documentation", },
  ["latex/polynom/README"] = {["details"] = "Readme", },
  ["latex/mdputu/README"] = {["details"] = "Readme", },
  ["support/ascii-chart/ascii.pdf"] = {["details"] = "The chart itself", },
  ["latex/nih/example-biosketch.pdf"] = {["details"] = "Example -- an NIH biographical sketch", },
  ["bibtex/iopart-num/iopart-num.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/xepersian/xepersian-doc.pdf"] = {["details"] = "Package user documentation", },
  ["latex/interval/interval.pdf"] = {["details"] = "Package documentation", },
  ["latex/functan/functan.pdf"] = {["details"] = "Package documentation", },
  ["latex/floatflt/floatflt.pdf"] = {["details"] = "Package documentation", },
  ["metapost/drv/README"] = {["details"] = "Readme", },
  ["latex/assoccnt/assoccnt_example.pdf"] = {["details"] = "Example of use", },
  ["latex/hitec/hitec_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-czech/README"] = {["details"] = "Readme", },
  ["latex/lshort-ukr/lshort-ukr.pdf"] = {["details"] = "The document itself", },
  ["latex/cascadilla/example.pdf"] = {["details"] = "Example document", },
  ["generic/pst-pulley/README.md"] = {["details"] = "Readme", },
  ["latex/ecv/template/CV-template_de.pdf"] = {["lang"] = "de", ["details"] = "Template (German)", },
  ["metapost/latexmp/latexmp.pdf"] = {["details"] = "Package documentation", },
  ["support/lua2dox/lua2dox-refm.pdf"] = {["details"] = "Package documentation", },
  ["latex/bitpattern/README.md"] = {["details"] = "Readme", },
  ["generic/musixtnt/msxlint.pdf"] = {["details"] = "Package documentation", },
  ["latex/philosophersimprint/philosophersimprint.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/luamesh/README"] = {["details"] = "Readme", },
  ["latex/ladder/ladder.pdf"] = {["details"] = "Package documentation", },
  ["latex/intro-scientific/README"] = {["details"] = "Readme", },
  ["latex/chemexec/README"] = {["details"] = "Readme", },
  ["fonts/uptex-fonts/README.md"] = {["details"] = "Readme", },
  ["latex/pittetd/pittetd.pdf"] = {["details"] = "Package documentation", },
  ["latex/footbib/footbib.pdf"] = {["details"] = "Package documentation", },
  ["otherformats/startex/ideas.pdf"] = {["details"] = "TUG'96 paper", },
  ["fonts/dejavu/dejavu.pdf"] = {["details"] = "Package documentation", },
  ["latex/gitinfo/gitinfo.pdf"] = {["details"] = "Package documentation", },
  ["latex/tui/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["bibtex/bestpapers/README"] = {["details"] = "Readme", },
  ["latex/euenc/README"] = {["details"] = "Readme", },
  ["context/third/letter/README"] = {["details"] = "Readme", },
  ["cstex/cstexman.pdf"] = {["lang"] = "cs", ["details"] = "User manual", },
  ["latex/bxjscls/bxjscls-manual.pdf"] = {["lang"] = "ja", ["details"] = "User guide (Japanese)", },
  ["latex/seuthesis/seuthesis.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation (Chinese)", },
  ["fonts/boondox/README"] = {["details"] = "Readme", },
  ["latex/multiobjective/README"] = {["details"] = "Readme", },
  ["latex/pseudocode/pseudocode.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex2e-help-texinfo-fr/README"] = {["details"] = "Readme", },
  ["latex/fancyref/fancyref.pdf"] = {["details"] = "Package documentation", },
  ["latex/addfont/README"] = {["details"] = "Readme", },
  ["latex/umich-thesis/README"] = {["details"] = "Readme", },
  ["latex/translation-europecv-de/europecv-de.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/tkz-base/tkz-base-screen.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/tkz-orm/tkz-orm.pdf"] = {["details"] = "Package documentation", },
  ["latex/minorrevision/README"] = {["details"] = "Readme", },
  ["latex/conteq/conteq.pdf"] = {["details"] = "Package documentation", },
  ["latex/doipubmed/doipubmed-manual.html"] = {["details"] = "Package documentation (HTML)", },
  ["latex/biblatex-shortfields/README"] = {["details"] = "Readme", },
  ["latex/skeycommand/skeycommand-guide.pdf"] = {["details"] = "Package documentation", },
  ["latex/directory/directory.pdf"] = {["details"] = "Package documentation", },
  ["latex/ltb2bib/ltb2bib.pdf"] = {["details"] = "Package documentation", },
  ["generic/mfpic/README"] = {["details"] = "Readme", },
  ["latex/pdfprivacy/pdfprivacy.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-sfc/tikz-sfc-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/phfsvnwatermark/README.md"] = {["details"] = "Readme", },
  ["latex/hyperxmp/README"] = {["details"] = "Readme", },
  ["fonts/dantelogo/dantelogo-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/proposal/eu/strep/README"] = {["details"] = "Readme", },
  ["fonts/yinit-otf/README.md"] = {["details"] = "Readme", },
  ["latex/drac/drac.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/cquthesis/README"] = {["details"] = "Readme", },
  ["latex/harvard/README"] = {["details"] = "Package Readme", },
  ["latex/errata/errata.pdf"] = {["details"] = "Package documentation", },
  ["latex/pagecolor/README"] = {["details"] = "Readme", },
  ["generic/babel-bulgarian/bulgarian.pdf"] = {["details"] = "Package documentation", },
  ["latex/active-conf/README"] = {["details"] = "Readme", },
  ["latex/uaclasses/README"] = {["details"] = "Readme", },
  ["latex/ryethesis/README"] = {["details"] = "Readme", },
  ["support/ctan_chk/README"] = {["details"] = "Readme", },
  ["latex/tcldoc/tools/README.txt"] = {["details"] = "Readme", },
  ["generic/musixtex/musixtex-install.pdf"] = {["details"] = "Package installation notes", },
  ["latex/quotchap/README"] = {["details"] = "Readme", },
  ["latex/platex-tools/pxeveryshi.pdf"] = {["lang"] = "ja", ["details"] = "Documentation of the pxeveryshi package", },
  ["latex/decimal/decimal.pdf"] = {["details"] = "Package documentation", },
  ["fonts/skaknew/SkakNew.pdf"] = {["details"] = "Package documentation", },
  ["fonts/chemarrow/README"] = {["lang"] = "en", ["details"] = "Package README", },
  ["latex/sectionbreak/sectionbreak-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/he-she/README"] = {["details"] = "Readme", },
  ["latex/gamebook/README"] = {["details"] = "Readme", },
  ["latex/ticket/README.txt"] = {["details"] = "Package Readme", },
  ["latex/tablor/tablor.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/lsc/README"] = {["details"] = "Readme", },
  ["latex/bxeepic/README"] = {["details"] = "Readme", },
  ["latex/moresize/moresize.pdf"] = {["details"] = "Package documentation", },
  ["latex/elocalloc/elocalloc.pdf"] = {["details"] = "Package documentation", },
  ["latex/noindentafter/noindentafter.pdf"] = {["details"] = "Package documentation", },
  ["fonts/ebgaramond/samples.pdf"] = {["details"] = "Font samples", },
  ["lualatex/lilyglyphs/README"] = {["details"] = "Readme", },
  ["latex/curve2e/curve2e.pdf"] = {["details"] = "Package documentation", },
  ["latex/csquotes/csquotes.pdf"] = {["details"] = "Tutorial on use of the package", },
  ["latex/easyformat/README"] = {["details"] = "Readme", },
  ["latex/tikz-relay/doc/BeamerAnimation.pdf"] = {["details"] = "Further documentation", },
  ["luatex/chickenize/README.md"] = {["details"] = "Readme", },
  ["generic/pst-dart/README.md"] = {["details"] = "Readme", },
  ["generic/xint/README.pdf"] = {["details"] = "Readme (PDF)", },
  ["latex/biblatex-gost/README.md"] = {["details"] = "Readme", },
  ["fonts/frcursive/README"] = {["details"] = "Readme", },
  ["latex/datetime2-scottish/README"] = {["details"] = "Readme", },
  ["latex/minutes/minutes.pdf"] = {["details"] = "Package documentation", },
  ["latex/ifetex/ifetex.pdf"] = {["details"] = "Package documentation", },
  ["latex/ejpecp/ejpecp.pdf"] = {["details"] = "Package documentation", },
  ["latex/combine/README"] = {["details"] = "Package Readme", },
  ["latex/labelschanged/labelschanged.pdf"] = {["details"] = "Package documentation", },
  ["latex/feynmf/fmfman.pdf"] = {["details"] = "Package documentation", },
  ["latex/typoaid/README.md"] = {["details"] = "Readme", },
  ["latex/unicode-math/unimath-example.pdf"] = {["details"] = "Example of use", },
  ["latex/udesoftec/udesoftec-doc.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/storebox/storebox.pdf"] = {["details"] = "Package documentation", },
  ["latex/flipbook/README"] = {["details"] = "Readme", },
  ["latex/eskdx/manual/eskdx.pdf"] = {["lang"] = "ru", ["details"] = "Package documentation", },
  ["latex/termcal/termcal.pdf"] = {["details"] = "Package documentation", },
  ["latex/tufte-latex/sample-handout.pdf"] = {["details"] = "Sample handout", },
  ["xelatex/imsproc/README"] = {["details"] = "Readme", },
  ["latex/biblatex/biblatex.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/gmdoc/README"] = {["details"] = "README", },
  ["latex/endiagram/endiagram_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/epsdice/epsdice.pdf"] = {["details"] = "Package Documentation", },
  ["latex/munich/README"] = {["details"] = "Readme", },
  ["latex/tcolorbox/tcolorbox-example.pdf"] = {["lang"] = "en", ["details"] = "Examples of use", },
  ["latex/inputtrc/inputtrc.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/langsci/documentation/langsci-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ltxdockit/README"] = {["details"] = "Readme", },
  ["latex/fifinddo-info/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/smartdiagram/smartdiagram.pdf"] = {["details"] = "Package documentation", },
  ["support/mf2pt1/README"] = {["details"] = "Readme", },
  ["latex/units/units.pdf"] = {["details"] = "Package documentation", },
  ["latex/itnumpar/itnumpar.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-slovak/README"] = {["details"] = "Readme", },
  ["latex/thinsp/README"] = {["details"] = "Readme", },
  ["latex/powerdot/powerdot.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["plain/newsletr/sample.pdf"] = {["details"] = "Another sample Newsletter", },
  ["fonts/chemarrow/Liesmich.txt"] = {["lang"] = "de", ["details"] = "Package Liesmich", },
  ["latex/koma-script-sfs/SFSesim.pdf"] = {["lang"] = "fi", ["details"] = "Package documentation (Finnish)", },
  ["latex/topletter/TOPletter.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/lualatex-math/lualatex-math.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/ecgdraw/ecgdraw_ita.pdf"] = {["lang"] = "it", ["details"] = "Package documentation", },
  ["latex/etoolbox-de/etoolbox-DE.pdf"] = {["details"] = "The document itself", },
  ["latex/codepage/LISEZMOI"] = {["lang"] = "fr", ["details"] = "Readme", },
  ["fonts/eco/README"] = {["details"] = "Readme", },
  ["latex/idxcmds/idxcmds_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/psnfss/psnfss2e.pdf"] = {["details"] = "Package documentation", },
  ["plain/gentle/gentle.pdf"] = {["details"] = "The document itself", },
  ["latex/everypage/README"] = {["details"] = "Readme", },
  ["fonts/epigrafica/epigrafica.pdf"] = {["details"] = "Package documentation", },
  ["latex/aguplus/aguplus.pdf"] = {["details"] = "Package documentation", },
  ["latex/powerdot/README"] = {["details"] = "Readme", },
  ["support/latex2man/latex2man.html"] = {["details"] = "Package documentation (HTML)", },
  ["latex/fp/README"] = {["details"] = "Readme", },
  ["metapost/mpman-ru/README.ru.koi8-r"] = {["lang"] = "ru", ["details"] = "Readme (Russian, KOI-8 encoded)", },
  ["latex/makebase/makebase.pdf"] = {["details"] = "Package documentation", },
  ["latex/rec-thy/rec-thy.pdf"] = {["details"] = "Package documentation", },
  ["support/texfot/README"] = {["details"] = "Readme", },
  ["latex/protocol/protocol.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/chscite/README"] = {["details"] = "Readme", },
  ["generic/multiexpand/multiexpand.pdf"] = {["details"] = "Package documentation", },
  ["generic/expex/README"] = {["details"] = "Readme", },
  ["latex/warning/warning-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/cascadilla/README"] = {["details"] = "Readme", },
  ["latex/forms16be/doc/forms16be-man.pdf"] = {["details"] = "Package documentation", },
  ["latex/changelayout/changelayout-guide.pdf"] = {["details"] = "Package example (see code for details)", },
  ["latex/powerdot-tuliplab/README.md"] = {["details"] = "Readme", },
  ["support/pdfbook2/README"] = {["details"] = "Readme", },
  ["fonts/playfair/README"] = {["details"] = "Readme", },
  ["fonts/fontname/fontname.html"] = {["details"] = "Outline of the mapping scheme (HTML)", },
  ["latex/qtree/README"] = {["details"] = "Readme", },
  ["generic/pst-bar/pst-bar-docDE.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/mathfixs/mathfixs.pdf"] = {["details"] = "Package documentation", },
  ["latex/backnaur/README"] = {["details"] = "Readme", },
  ["generic/babel-icelandic/icelandic.pdf"] = {["details"] = "Package documentation", },
  ["latex/logicpuzzle/logicpuzzle.pdf"] = {["details"] = "Package documentation", },
  ["fonts/fontinst/README"] = {["details"] = "Readme", },
  ["metapost/base/mpman.pdf"] = {["details"] = "METAPOST Users Manual", },
  ["latex/gincltex/README"] = {["details"] = "Readme", },
  ["latex/multirow/README"] = {["details"] = "Readme", },
  ["latex/bosisio/index.html"] = {["details"] = "Outline of the collection", },
  ["latex/parallel/parallel.pdf"] = {["details"] = "Package documentation", },
  ["support/mkpic/mkpicdoc.pdf"] = {["details"] = "Package documentation", },
  ["generic/musixtnt/README"] = {["details"] = "Readme", },
  ["latex/breakurl/README"] = {["details"] = "Package README", },
  ["latex/datetime2-basque/README.txt"] = {["details"] = "Readme", },
  ["latex/esdiff/esdiff.pdf"] = {["details"] = "Package documentation", },
  ["otherformats/startex/startex.pdf"] = {["details"] = "Documented source code", },
  ["latex/bibarts/ba-short.pdf"] = {["details"] = "Package short documentation", },
  ["latex/varsfromjobname/README.md"] = {["details"] = "Readme", },
  ["latex/screenplay-pkg/screenplay-pkg.pdf"] = {["details"] = "Package documentation", },
  ["latex/texpower/manual.pdf"] = {["details"] = "Manual", },
  ["latex/unswcover/README"] = {["details"] = "Readme", },
  ["generic/ifptex/README.md"] = {["details"] = "Readme", },
  ["latex/ribbonproofs/ribbonproofsmanual.pdf"] = {["details"] = "Package documentation", },
  ["latex/ellipse/README"] = {["details"] = "Readme", },
  ["fonts/bartel-chess-fonts/README"] = {["details"] = "Readme", },
  ["generic/e-french/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/adfathesis/adfathesis.pdf"] = {["details"] = "Package documentation", },
  ["fonts/spark-otf/spark-otf-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-fun/pst-fun-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/anysize/README"] = {["details"] = "Package Readme", },
  ["latex/svninfo/README"] = {["details"] = "Readme", },
  ["latex/auto-pst-pdf/auto-pst-pdf-DE.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/ifmslide/README"] = {["details"] = "Readme", },
  ["latex/eqparbox/README"] = {["details"] = "Readme", },
  ["fonts/heuristica/README"] = {["details"] = "Readme", },
  ["latex/titlepic/README.md"] = {["details"] = "Readme", },
  ["generic/apnum/README"] = {["details"] = "Readme", },
  ["latex/abntex2/abntex2.pdf"] = {["lang"] = "pt-br", ["details"] = "Package documentation (Brazilian Portuguese)", },
  ["lualatex/novel/novel-documentation.html"] = {["details"] = "Package documentation (HTML)", },
  ["fonts/pxtxalfa/README"] = {["details"] = "Readme", },
  ["latex/confproc/README"] = {["details"] = "Readme", },
  ["latex/fifo-stack/fifo-stack.pdf"] = {["details"] = "Package documentation", },
  ["metapost/repere/repere-doc.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/curve/README"] = {["details"] = "Readme", },
  ["latex/gmdoc-enhance/README"] = {["details"] = "Readme", },
  ["latex/xyling/README"] = {["details"] = "README file", },
  ["latex/scratchx/Explications_ScratchX.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/export/export.pdf"] = {["details"] = "Package documentation", },
  ["latex/bankstatement/bankstatement.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamerswitch/beamerswitch-example-trans.pdf"] = {["details"] = "Example of use (transparencies, without overlays)", },
  ["latex/ellipsis/ellipsis.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-solides3d/pst-solides3d-doc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["support/asymptote-manual-zh-cn/asymptote-manual-zh-cn.pdf"] = {["lang"] = "zh", ["details"] = "The document itself", },
  ["latex/short-math-guide/short-math-guide.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamerswitch/beamerswitch-example.pdf"] = {["details"] = "Example of use (slides, with overlays)", },
  ["fonts/fonetika/README"] = {["details"] = "Package Readme", },
  ["latex/datetime2-catalan/datetime2-catalan.pdf"] = {["details"] = "Package documentation", },
  ["latex/abnt/abnt-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cns/cns40-1/README"] = {["details"] = "Readme", },
  ["fonts/dad/README"] = {["details"] = "Readme", },
  ["support/gitfile-info/README"] = {["details"] = "Readme", },
  ["latex/cmap/README"] = {["details"] = "Readme", },
  ["luatex/nodetree/README.md"] = {["details"] = "Readme", },
  ["latex/isomath/isomath.html"] = {["details"] = "Package documentation (in HTML format)", },
  ["latex/abstract/abstract.pdf"] = {["details"] = "Package documentation", },
  ["luatex/luamplib/README"] = {["details"] = "Readme", },
  ["latex/arraysort/README"] = {["details"] = "Readme", },
  ["latex/hepnames/README"] = {["details"] = "Readme", },
  ["latex/syllogism/README"] = {["details"] = "Readme", },
  ["latex/iscram/iscram-class-doc.pdf"] = {["details"] = "Package documenatation", },
  ["latex/simpleinvoice/README.md"] = {["details"] = "Readme", },
  ["latex/mdframed/mdframed.pdf"] = {["details"] = "Package documentation", },
  ["latex/callouts/callouts.pdf"] = {["details"] = "Package documentation", },
  ["fonts/caladea/README"] = {["details"] = "Readme", },
  ["latex/nostarch/README"] = {["details"] = "Readme", },
  ["latex/carlisle/README.txt"] = {["details"] = "Bundle README", },
  ["latex/memexsupp/memexsupp.pdf"] = {["details"] = "Package documentation", },
  ["latex/cleanthesis/thesis-example.pdf"] = {["details"] = "Sample document", },
  ["latex/esk/eskman.pdf"] = {["details"] = "Package manual", },
  ["latex/versonotes/README"] = {["details"] = "Readme", },
  ["latex/kix/kix.pdf"] = {["details"] = "Package documentation", },
  ["support/latexmk/latexmk.pdf"] = {["details"] = "Manual page, PDF", },
  ["latex/draftcopy/README"] = {["details"] = "Readme", },
  ["generic/pst-magneticfield/pst-magneticfield-docEN.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/notes2bib/README"] = {["details"] = "Readme", },
  ["latex/mathastext/README.md"] = {["details"] = "Readme", },
  ["latex/stex/omtext/README"] = {["details"] = "Readme", },
  ["support/asymptote-manual-zh-cn/README"] = {["details"] = "Readme", },
  ["latex/ebezier/ebezier.pdf"] = {["details"] = "Package documentation", },
  ["latex/enumitem/README"] = {["details"] = "Readme", },
  ["fonts/baskervaldx/README"] = {["details"] = "Readme", },
  ["latex/stealcaps/stealcaps.pdf"] = {["details"] = "Package documentation", },
  ["latex/SIunits/SIunits.pdf"] = {["details"] = "Package documentation", },
  ["fonts/morisawa/morisawa.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation", },
  ["latex/pxufont/README.md"] = {["details"] = "Readme", },
  ["latex/simpleinvoice/doc/simpleinvoice.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-kalender/README.md"] = {["details"] = "Readme", },
  ["latex/sslides/land-sample-slide.pdf"] = {["details"] = "Sample (landscape) slides", },
  ["latex/multenum/README"] = {["details"] = "Readme", },
  ["latex/cjkpunct/README.md"] = {["details"] = "Readme", },
  ["latex/graphicx-psmin/README"] = {["details"] = "Package README", },
  ["generic/xlop/xlop-doc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/komacv/README"] = {["details"] = "Readme", },
  ["generic/pgf/pgfmanual.pdf"] = {["details"] = "PGF Manual", },
  ["latex/nidanfloat/nidanfloat.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation (Japanese)", },
  ["fonts/fontmfizz/fontmfizz.pdf"] = {["details"] = "List of symbols", },
  ["latex/sanskrit/sktdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/booktabs-de/README-DE"] = {["lang"] = "de", ["details"] = "Readme (German)", },
  ["latex/ticollege/README"] = {["lang"] = "fr", ["details"] = "Readme", },
  ["latex/codicefiscaleitaliano/README"] = {["lang"] = "it", ["details"] = "Readme (English and Italian)", },
  ["fonts/iwona/00readme.eng"] = {["details"] = "Outline in English:", },
  ["generic/pst-gantt/README"] = {["details"] = "Readme", },
  ["latex/emptypage/emptypage.pdf"] = {["details"] = "Package documentation", },
  ["latex/reledmac/reledmac.pdf"] = {["details"] = "Package documentation", },
  ["latex/stex/tikzinput/README"] = {["details"] = "Readme", },
  ["latex/subdepth/README"] = {["details"] = "Readme", },
  ["latex/canoniclayout/README"] = {["details"] = "Readme", },
  ["latex/topletter/EsempioLettera.pdf"] = {["lang"] = "it", ["details"] = "Example of use (Italian)", },
  ["latex/invoice2/invoice2.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamersubframe/README"] = {["details"] = "Readme", },
  ["latex/qstest/README"] = {["details"] = "Readme", },
  ["plain/hyplain/hydoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/tucv/tucv_ex.pdf"] = {["details"] = "Package sampler", },
  ["generic/pgf/README"] = {["details"] = "Readme", },
  ["latex/kdgdocs/README"] = {["details"] = "Readme", },
  ["latex/colorspace/README"] = {["details"] = "Readme", },
  ["latex/xnewcommand/xnewcommand.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-caspervector/caspervector.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["latex/minorrevision/minorrevision.pdf"] = {["details"] = "Package documentation", },
  ["latex/foekfont/foekfont.pdf"] = {["details"] = "Package documentation", },
  ["generic/2up/2up-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/libertinus/README.txt"] = {["details"] = "Readme", },
  ["fonts/gofonts/go-samples.pdf"] = {["details"] = "Font samples", },
  ["support/match_parens/match_parens.pdf"] = {["details"] = "Package documentation", },
  ["latex/erdc/erdc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ptex2pdf/README.md"] = {["details"] = "Readme", },
  ["fonts/doublestroke/README"] = {["details"] = "Package Readme", },
  ["xelatex/arabxetex/arabxetex.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-usorbian/datetime2-usorbian.pdf"] = {["details"] = "Package documentation", },
  ["latex/technics/technics.pdf"] = {["details"] = "Package documentation", },
  ["latex/psu-thesis/mssample.pdf"] = {["details"] = "MS thesis layout demonstration", },
  ["latex/cquthesis/cquthesis.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["luatex/lua-visual-debug/sample.pdf"] = {["details"] = "Sample output", },
  ["latex/proposal/base/README"] = {["details"] = "Readme", },
  ["latex/gitinfo2/README"] = {["details"] = "Readme", },
  ["latex/l2tabu/l2tabu.pdf"] = {["lang"] = "de", ["details"] = "The document itself", },
  ["latex/crossreference/crossreference.pdf"] = {["details"] = "Package documentation", },
  ["latex/greektonoi/greektonoi-en.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-macedonian/README.md"] = {["details"] = "Readme", },
  ["generic/babel-belarusian/README"] = {["details"] = "Readme", },
  ["latex/l3build/README.md"] = {["details"] = "Readme", },
  ["latex/lps/README"] = {["details"] = "Readme", },
  ["latex/mathfixs/README.txt"] = {["details"] = "Readme", },
  ["latex/sduthesis/sduthesis-demo.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation (demo thesis)", },
  ["generic/pst-poly/README"] = {["details"] = "Package Readme", },
  ["fonts/carlito/samples.pdf"] = {["details"] = "Font samples", },
  ["latex/jkmath/README.md"] = {["details"] = "Readme", },
  ["latex/fduthesis/README.md"] = {["details"] = "Readme", },
  ["latex/setspace/README"] = {["details"] = "Readme", },
  ["latex/xcolor/xcolor2.pdf"] = {["details"] = "Examples of usage with pstricks", },
  ["generic/pst-uml/pst-uml-exemples.pdf"] = {["lang"] = "fr", ["details"] = "Examples", },
  ["latex/bankstatement/bankstatement-example.pdf"] = {["details"] = "Package example", },
  ["latex/duckuments/README.md"] = {["details"] = "Readme", },
  ["latex/afparticle/afpsample.pdf"] = {["details"] = "Example of usage", },
  ["latex/graph35/README.md"] = {["details"] = "Readme", },
  ["latex/pixelart/README.md"] = {["details"] = "Readme", },
  ["latex/amsmath/subeqn.pdf"] = {["lang"] = "en", ["details"] = "Sub-equation usage", },
  ["latex/cnbwp/cnbwp-manual-en.pdf"] = {["details"] = "Package documentation (English)", },
  ["latex/aguplus/README"] = {["details"] = "Readme", },
  ["latex/morefloats/morefloats.pdf"] = {["details"] = "Package documentation", },
  ["plain/gfnotation/README"] = {["details"] = "Readme", },
  ["metapost/mptrees/README.md"] = {["details"] = "Readme", },
  ["latex/tex-overview/README.md"] = {["details"] = "Readme", },
  ["metapost/mparrows/README"] = {["details"] = "Readme", },
  ["latex/uspatent/PatentApplicationGuide.pdf"] = {["details"] = "Package description", },
  ["fonts/blacklettert1/README"] = {["details"] = "Readme", },
  ["generic/pst-thick/README"] = {["details"] = "Readme", },
  ["latex/cascade/cascade.pdf"] = {["details"] = "Package documentation", },
  ["latex/csvsimple/csvsimple.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/paralist/paralist.pdf"] = {["details"] = "Package documentation", },
  ["latex/volumes/volumes.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/bidipagegrid/README"] = {["details"] = "README", },
  ["xelatex/xgreek/README"] = {["details"] = "README file", },
  ["plain/epsf-dvipdfmx/README"] = {["details"] = "Readme", },
  ["latex/titleref/titleref.pdf"] = {["details"] = "Package documentation", },
  ["latex/plweb/pl.pdf"] = {["details"] = "Package documentation", },
  ["latex/fancynum/fancynum.pdf"] = {["details"] = "Package documentation", },
  ["latex/apa/apacls.html"] = {["details"] = "Class documentation", },
  ["latex/endofproofwd/endofproofwd.pdf"] = {["details"] = "Package documentation", },
  ["latex/catechis/README"] = {["details"] = "Readme", },
  ["latex/beamerdarkthemes/README"] = {["details"] = "Readme", },
  ["latex/biblatex-swiss-legal/README"] = {["details"] = "Readme (French and English)", },
  ["latex/horoscop/horoscop.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/interchar/README"] = {["details"] = "Readme", },
  ["eplain/README"] = {["details"] = "Package Readme", },
  ["latex/musixguit/musixguit_de.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/thmbox/README"] = {["details"] = "Readme", },
  ["latex/tikz-qtree/tikz-qtree-manual.pdf"] = {["details"] = "Package documentation", },
  ["fonts/figbas/README"] = {["details"] = "Readme", },
  ["generic/circuitikz/README.md"] = {["details"] = "Readme", },
  ["latex/grid/grid.pdf"] = {["details"] = "Package documentation", },
  ["generic/enctex/README"] = {["details"] = "Readme", },
  ["latex/testhyphens/README"] = {["details"] = "Readme", },
  ["generic/german/betatest/00readme.1st"] = {["lang"] = "de", ["details"] = "Readme", },
  ["latex/apa6e/README"] = {["details"] = "Readme", },
  ["latex/moderncv/README"] = {["details"] = "Readme", },
  ["latex/cprotect/cprotect.pdf"] = {["details"] = "Package documentation", },
  ["latex/cv/ApplicationLetter.pdf"] = {["details"] = "Example application letter", },
  ["latex/gtrcrd/README"] = {["details"] = "Readme", },
  ["metapost/textpath/textpath.pdf"] = {["details"] = "Package documentation", },
  ["fonts/jablantile/README"] = {["details"] = "Readme", },
  ["latex/scratchx/Explanations_ScratchX.pdf"] = {["details"] = "Package documentation", },
  ["context/third/cmttbf/cmttbf-demo.pdf"] = {["details"] = "Demo of use", },
  ["latex/tcldoc/README.txt"] = {["details"] = "Readme", },
  ["latex/greekdates/README"] = {["details"] = "Readme", },
  ["latex/caption/README"] = {["details"] = "Readme", },
  ["latex/timing-diagrams/diagrams-examples.pdf"] = {["details"] = "Examples of use", },
  ["latex/makeglos/makeglos.pdf"] = {["details"] = "Package documentation", },
  ["fonts/chess/readme"] = {["details"] = "Readme", },
  ["latex/abntex2/abntex2cite.pdf"] = {["lang"] = "pt-br", ["details"] = "Citation support package documentation (Brazilian Portuguese)", },
  ["latex/morehype/README"] = {["details"] = "Readme", },
  ["latex/sdrt/sdrt-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/aobs-tikz/aobs-tikz.pdf"] = {["details"] = "Package documentation", },
  ["latex/hagenberg-thesis/examples/Manual/main.pdf"] = {["lang"] = "de", ["details"] = "Example document / Tutorial (German)", },
  ["latex/progress/progress.pdf"] = {["details"] = "Package documentation", },
  ["latex/polynom/polynom.pdf"] = {["details"] = "Package documentation", },
  ["latex/minitoc/README.md"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/einfuehrung2/README"] = {["details"] = "Readme", },
  ["generic/enctex/encdoc-e.pdf"] = {["lang"] = "en", ["details"] = "Reference manual (in English)", },
  ["generic/listofitems/README"] = {["details"] = "Readme", },
  ["latex/geometry/README.md"] = {["details"] = "Package Readme", },
  ["latex/nicetext/mdoccheat.pdf"] = {["details"] = "Cheat sheet for the makedoc package", },
  ["generic/babel-romansh/romansh.pdf"] = {["details"] = "Package documentation", },
  ["latex/svn/svn.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-pad/pst-pad-docDE.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["generic/midnight/midnight.pdf"] = {["details"] = "Package documentation", },
  ["latex/kotex-utf/kotexdoc.pdf"] = {["lang"] = "ko", ["details"] = "Package documentation (Korean)", },
  ["latex/turnstile/turnstile_artigo.pdf"] = {["lang"] = "pt", ["details"] = "Article about the package (Portuguese)", },
  ["latex/beamertheme-focus/README.md"] = {["details"] = "Readme", },
  ["bibtex/bib-fr/README"] = {["details"] = "Readme", },
  ["latex/termcal-de/README.md"] = {["details"] = "Readme", },
  ["latex/tdclock/README"] = {["details"] = "Readme", },
  ["latex/bxjscls/README.md"] = {["details"] = "Readme", },
  ["latex/grant/README.md"] = {["details"] = "Package README", },
  ["latex/reotex/reotex.pdf"] = {["details"] = "Package documentation", },
  ["latex/formation-latex-ul/README.md"] = {["details"] = "Readme", },
  ["latex/pagerange/README"] = {["details"] = "Readme", },
  ["generic/m-tx/mtxdoc.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-esperanto/esperanto.pdf"] = {["details"] = "Package documentation", },
  ["latex/boolexpr/README"] = {["details"] = "Readme", },
  ["support/latexmk/README"] = {["details"] = "Readme", },
  ["latex/booklet/booklet.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-bookinother/documentation/biblatex-bookinother.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/bibhtml/README"] = {["details"] = "Readme", },
  ["latex/apptools/apptools.pdf"] = {["details"] = "Package documentation", },
  ["latex/notestex/README.md"] = {["details"] = "Readme", },
  ["latex/asypictureb/README"] = {["details"] = "Readme", },
  ["latex/datenumber/doc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/mnotes/mnotes.pdf"] = {["details"] = "Readme", },
  ["latex/datenumber/README.txt"] = {["lang"] = "en", ["details"] = "Package README", },
  ["latex/scalerel/scalerel.pdf"] = {["details"] = "Package documentation", },
  ["latex/pdfreview/README.md"] = {["details"] = "Readme", },
  ["latex/akletter/lettereng.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/tikzmarmots/README.md"] = {["details"] = "Readme", },
  ["latex/currfile/currfile.pdf"] = {["details"] = "Package documentation", },
  ["fonts/variablelm/control_scaling_lm_fonts.pdf"] = {["details"] = "Package documentation", },
  ["latex/shadow/shadow-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/bigints/README"] = {["details"] = "Readme", },
  ["latex/gmdoc-enhance/gmdoc-enhance.pdf"] = {["details"] = "Package documentation", },
  ["latex/cleanthesis/README.md"] = {["details"] = "Readme", },
  ["latex/handin/handin-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/flashmovie/README"] = {["details"] = "Readme", },
  ["latex/mugsthesis/README"] = {["details"] = "Readme", },
  ["generic/pst-spinner/README"] = {["details"] = "Readme", },
  ["latex/curve/curve.pdf"] = {["details"] = "Package documentation", },
  ["latex/leading/README"] = {["details"] = "Readme", },
  ["latex/setdeck/setdeck.pdf"] = {["details"] = "Package documentation", },
  ["latex/ametsoc/README"] = {["details"] = "Readme", },
  ["fonts/asapsym/asapsym.pdf"] = {["details"] = "Package documentation", },
  ["latex/psbao/README"] = {["details"] = "Readme", },
  ["latex/pxtatescale/README"] = {["details"] = "Readme", },
  ["fonts/sansmathaccent/sansmathaccent.pdf"] = {["details"] = "Package documentation", },
  ["latex/tabls/tabls.pdf"] = {["details"] = "Package documentation", },
  ["latex/cd-cover/cd-cover.pdf"] = {["details"] = "Package documentation", },
  ["latex/komacv/examples/komacv_example_oldstyle.pdf"] = {["details"] = "Example of use (old style)", },
  ["plain/js-misc/README"] = {["details"] = "Readme", },
  ["latex/diadia/diadia-example.pdf"] = {["details"] = "Example of use", },
  ["generic/termmenu/README"] = {["details"] = "Readme", },
  ["latex/esami/doc/esami-doc-it.pdf"] = {["lang"] = "it", ["details"] = "Package documentation (Italian)", },
  ["latex/bpchem/README.md"] = {["details"] = "Readme", },
  ["latex/mnotes/README"] = {["details"] = "Readme", },
  ["latex/oberdiek/ifluatex.pdf"] = {["details"] = "Package documentation", },
  ["latex/metalogo/metalogo.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-stru/pst-stru-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/thalie/README.md"] = {["details"] = "Readme", },
  ["fonts/boisik/example/table.pdf"] = {["details"] = "Font table", },
  ["latex/comicneue/comicneue.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-knot/README"] = {["details"] = "Readme", },
  ["latex/scsnowman/scsnowman-sample.pdf"] = {["lang"] = "ja", ["details"] = "Sample of use", },
  ["latex/lipsum/README"] = {["details"] = "Readme", },
  ["latex/ltxnew/README"] = {["details"] = "Readme", },
  ["latex/docmute/docmute.pdf"] = {["details"] = "Package documentation", },
  ["latex/amsldoc-vn/amsldoc-print-vi.pdf"] = {["lang"] = "vi", ["details"] = "For printing", },
  ["context/third/bnf/README"] = {["details"] = "Readme", },
  ["luatex/nodetree/nodetree.pdf"] = {["details"] = "Package documentation", },
  ["fonts/courier-scaled/Couriers.pdf"] = {["details"] = "Package documentation", },
  ["latex/euro/euro.pdf"] = {["details"] = "Package documentation", },
  ["latex/feupphdteses/README"] = {["details"] = "Readme", },
  ["latex/abntex2/examples/abntex2-modelo-livro.pdf"] = {["lang"] = "pt-br", ["details"] = "Book template", },
  ["latex/refstyle/refconfig.pdf"] = {["details"] = "Configuration manual", },
  ["latex/paresse/paresse-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/footmisx/README"] = {["details"] = "Readme", },
  ["latex/pgf-blur/pgf-blur.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamer/beameruserguide.pdf"] = {["details"] = "Package documentation", },
  ["latex/figsize/figsize.pdf"] = {["details"] = "Package documentation", },
  ["latex/fvextra/fvextra.pdf"] = {["details"] = "Package documentation", },
  ["latex/probsoln/README"] = {["details"] = "Package README", },
  ["fonts/morisawa/README.md"] = {["details"] = "Readme", },
  ["generic/markdown/markdown.pdf"] = {["details"] = "Technical documentation", },
  ["latex/opcit/opcit.pdf"] = {["details"] = "Package documentation", },
  ["latex/phfqit/phfqit.pdf"] = {["details"] = "Package documentation", },
  ["latex/harnon-cv/sample.pdf"] = {["details"] = "Sample of use", },
  ["latex/multibib/multibib.pdf"] = {["details"] = "Package documentation", },
  ["latex/toptesi/toptesi.pdf"] = {["details"] = "Package documentation", },
  ["latex/lstaddons/lstautogobble.pdf"] = {["details"] = "lstautogobble package documentation", },
  ["latex/lps/lps.pdf"] = {["details"] = "Package documentation", },
  ["latex/verbments/README"] = {["details"] = "Readme", },
  ["generic/pst-light3d/README"] = {["details"] = "Readme", },
  ["latex/rutitlepage/README.md"] = {["details"] = "Readme", },
  ["fonts/typicons/README"] = {["details"] = "Readme", },
  ["latex/ndsu-thesis/ndsu-thesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/gridslides/example.pdf"] = {["details"] = "Example of use", },
  ["fonts/merriweather/merriweather-samples.pdf"] = {["details"] = "Font samples", },
  ["latex/environ/README"] = {["details"] = "Readme", },
  ["latex/stubs/stubs_ex.pdf"] = {["details"] = "Example of use", },
  ["latex/lastpage/lastpage.pdf"] = {["details"] = "Package documentation", },
  ["latex/datatool/datatool-user.pdf"] = {["details"] = "User guide", },
  ["metapost/mp3d/tugboat-geometry-space.pdf"] = {["lang"] = "en", ["details"] = "Paper on geometry, for TUGboat", },
  ["latex/fmtcount/fmtcount.pdf"] = {["details"] = "Package documentation", },
  ["latex/wtref/README.md"] = {["details"] = "Readme", },
  ["latex/capt-of/README"] = {["details"] = "Readme", },
  ["latex/abntex2/README"] = {["details"] = "Readme", },
  ["latex/noitcrul/README"] = {["details"] = "Readme", },
  ["latex/shuffle/shuffle.pdf"] = {["details"] = "Package documentation", },
  ["latex/nonumonpart/LISEZMOI"] = {["lang"] = "fr", ["details"] = "Readme", },
  ["latex/mhequ/README"] = {["details"] = "Readme", },
  ["latex/chextras/chextras.pdf"] = {["details"] = "Package documentation", },
  ["generic/dirtree/README"] = {["details"] = "Readme", },
  ["latex/lshort-vietnamese/README"] = {["details"] = "Readme", },
  ["generic/dcpic/manDCPiC.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["fonts/asapsym/README.md"] = {["details"] = "Readme", },
  ["latex/mugsthesis/mugsthesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/texproposal/README.md"] = {["details"] = "Readme", },
  ["xelatex/na-box/README"] = {["details"] = "Readme", },
  ["metapost/expressg/aamfigs.pdf"] = {["details"] = "Examples of use", },
  ["latex/paracol/paracol-man.pdf"] = {["details"] = "Package documentation", },
  ["generic/navigator/navigator-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/cdpbundl/00readme.txt"] = {["details"] = "Basic introduction", },
  ["fonts/fira/README"] = {["details"] = "Readme", },
  ["fonts/xits/user-guide.pdf"] = {["details"] = "User guide", },
  ["latex/nuc/README"] = {["details"] = "Readme", },
  ["latex/l3packages/xfp/xfp.pdf"] = {["details"] = "xfp documentation", },
  ["xelatex/philokalia/philokalia.pdf"] = {["details"] = "Package documentation", },
  ["latex/polexpr/polexpr.html"] = {["details"] = "Package documentation (HTML)", },
  ["latex/biblatex-abnt/biblatex-abnt.pdf"] = {["lang"] = "pt-br", ["details"] = "Package documentation", },
  ["latex/combinedgraphics/combinedgraphics.pdf"] = {["details"] = "Package documentation", },
  ["latex/moreverb/moreverb.pdf"] = {["details"] = "Package documentation", },
  ["latex/maker/README"] = {["details"] = "Readme", },
  ["fonts/kixfont/kixtable.pdf"] = {["details"] = "Font table", },
  ["generic/babel-english/README"] = {["details"] = "Readme", },
  ["latex/penrose/README.txt"] = {["details"] = "Readme", },
  ["latex/fnumprint/README"] = {["details"] = "Readme", },
  ["latex/keyreader/keyreader-guide.pdf"] = {["details"] = "Package documentation", },
  ["fonts/semaphor/README"] = {["details"] = "Readme", },
  ["generic/babel-estonian/estonian.pdf"] = {["details"] = "Package documentation", },
  ["latex/cjk/doc/cjk/README"] = {["details"] = "Readme file", },
  ["latex/leadsheets/leadsheets_en.pdf"] = {["details"] = "Package documentation", },
  ["fonts/rosario/README.md"] = {["details"] = "Readme", },
  ["latex/fancyslides/README"] = {["details"] = "Readme", },
  ["latex/mensa-tex/doc/mensa-tex-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/oldstandard/oldstand-manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/knittingpattern/README"] = {["details"] = "Readme", },
  ["lualatex/cloze/README"] = {["details"] = "Readme", },
  ["latex/bibarts/bibarts.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["context/third/title/title-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/amsfonts/euscript.pdf"] = {["details"] = "Eucal and euscript package documentation", },
  ["latex/totcount/README"] = {["details"] = "Readme", },
  ["latex/epspdfconversion/README"] = {["details"] = "Readme", },
  ["generic/ulem/README"] = {["details"] = "Readme", },
  ["support/adhocfilelist/adhocfilelist.htm"] = {["details"] = "Package documentation", },
  ["latex/cmpj/README"] = {["details"] = "Readme", },
  ["latex/ydoc/README"] = {["details"] = "Readme", },
  ["support/texdoc/README.md"] = {["details"] = "Readme", },
  ["latex/gb4e/gb4e-doc.pdf"] = {["details"] = "Package description", },
  ["latex/jumplines/jumplines_example.pdf"] = {["details"] = "Package example", },
  ["context/third/fullpage/fullpage-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/here/README"] = {["details"] = "Readme", },
  ["latex/epigraph/epigraph.pdf"] = {["details"] = "Package documentation", },
  ["generic/e-french/manuels/frenchle.pdf"] = {["details"] = "Package documentation (lightweight version)", },
  ["latex/mtgreek/mtgreek.pdf"] = {["details"] = "Package documentation", },
  ["latex/forarray/README.txt"] = {["details"] = "Readme", },
  ["latex/casyl/casyldoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ecothesis/README.txt"] = {["details"] = "Readme", },
  ["latex/sugconf/README"] = {["details"] = "Readme", },
  ["support/makedtx/makedtx.pdf"] = {["details"] = "Package documentation (PDF format)", },
  ["latex/gzt/README.md"] = {["details"] = "Readme", },
  ["latex/t-angles/README"] = {["details"] = "Readme", },
  ["latex/har2nat/har2nat.pdf"] = {["details"] = "Package documentation", },
  ["latex/ecclesiastic/README"] = {["details"] = "Readme", },
  ["latex/biochemistry-colors/README.txt"] = {["details"] = "Readme", },
  ["latex/SIunits/README"] = {["details"] = "Readme", },
  ["latex/h2020proposal/template-fet/template-fet.pdf"] = {["details"] = "Template (FET)", },
  ["xetex/xetex-pstricks/README"] = {["details"] = "Readme", },
  ["latex/pdftricks/manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/booktabs-de/booktabs-de.pdf"] = {["details"] = "Package documentation", },
  ["support/mendex/mendex.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation", },
  ["latex/vhistory/vh_sets_de.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (both packages)", },
  ["latex/filecontents/README"] = {["details"] = "Package Readme", },
  ["latex/xpatch/xpatch.pdf"] = {["details"] = "Package documentation", },
  ["fonts/wadalab/mc2j/README"] = {["details"] = "Readme", },
  ["fonts/libertine/README"] = {["details"] = "Readme", },
  ["bibtex/ijqc/xampl.pdf"] = {["details"] = "Example output", },
  ["latex/abntex2/examples/abntex2-modelo-slides.pdf"] = {["lang"] = "pt-br", ["details"] = "Presentation template", },
  ["plain/texbytopic/TeXbyTopic.pdf"] = {["details"] = "The book itself", },
  ["latex/glossaries-italian/README"] = {["details"] = "Readme", },
  ["xetex/font-change-xetex/font-change-xetex.pdf"] = {["details"] = "Package documentation", },
  ["metapost/automata/example.pdf"] = {["details"] = "Example of use", },
  ["latex/biblatex-oxref/oxalph-doc.pdf"] = {["details"] = "Style documentation for oxalph", },
  ["latex/hagenberg-thesis/examples/HgbThesisEN/main.pdf"] = {["lang"] = "de", ["details"] = "Example document / Tutorial (German)", },
  ["latex/ltablex/README"] = {["details"] = "README file", },
  ["generic/pst-math/README"] = {["details"] = "Readme", },
  ["bibtex/tamethebeast/README"] = {["details"] = "Readme", },
  ["latex/invoice/invoice.pdf"] = {["details"] = "Package documentation", },
  ["metapost/mcf2graph/mcf_mplib_exa.pdf"] = {["details"] = "MCF Example for LuaLaTeX", },
  ["latex/xcjk2uni/xCJK2uni.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation (Chinese)", },
  ["latex/bytefield/bytefield.pdf"] = {["details"] = "Package documentation", },
  ["latex/cool/Content_LaTeX_Package_Demo.pdf"] = {["details"] = "Tables of macros functions", },
  ["context/third/typearea/typearea-demo.pdf"] = {["details"] = "Package demo", },
  ["latex/keyreader/README"] = {["details"] = "Readme", },
  ["latex/fnspe/fnspe.pdf"] = {["details"] = "Package documentation", },
  ["latex/isorot/README"] = {["details"] = "Package Readme", },
  ["xelatex/unisugar/unisugar.pdf"] = {["details"] = "Package documentation", },
  ["fonts/fge/fge-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/elsarticle/README"] = {["details"] = "Readme", },
  ["latex/prosper/prosper-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/meetingmins/meetingmins.pdf"] = {["details"] = "Package documentation", },
  ["plain/graphics-pln/README.txt"] = {["details"] = "Readme", },
  ["latex/xkeyval/xkeyval.pdf"] = {["details"] = "Package documentation", },
  ["latex/boites/demo.pdf"] = {["details"] = "Examples of use", },
  ["generic/pst-fill/pst-fill.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikzsymbols/tikzsymbols.pdf"] = {["details"] = "Package documentation", },
  ["latex/binomexp/binomexp.pdf"] = {["details"] = "Package documentation", },
  ["latex/mgltex/README"] = {["details"] = "Readme", },
  ["fonts/bengali/bengdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/lapdf/lapdf.pdf"] = {["details"] = "Package documentation", },
  ["latex/elteikthesis/README"] = {["details"] = "Readme", },
  ["generic/babel-estonian/README.txt"] = {["details"] = "Readme", },
  ["latex/zxjafbfont/README"] = {["details"] = "Readme", },
  ["fonts/gfsdidot/README"] = {["details"] = "Readme", },
  ["generic/ofs/ofsdoc.pdf"] = {["lang"] = "cs", ["details"] = "Package documentation", },
  ["latex/latexcheat-ptbr/latexsheet-ptbr.pdf"] = {["lang"] = "pt", ["details"] = "The document itself", },
  ["latex/datetime2-latin/datetime2-latin.pdf"] = {["details"] = "Package documentation", },
  ["latex/ekaia/ekaia_EUS.pdf"] = {["lang"] = "eu", ["details"] = "Package documentation", },
  ["latex/limecv/README.md"] = {["details"] = "Readme", },
  ["latex/eiad-ltx/README"] = {["details"] = "Readme", },
  ["latex/ibycus-babel/README"] = {["details"] = "Readme", },
  ["fonts/cherokee/README"] = {["details"] = "Readme", },
  ["latex/theatre/doc/Help_Theatre.pdf"] = {["details"] = "Package documentation (French and English)", },
  ["latex/currfile/README"] = {["details"] = "Readme", },
  ["latex/engrec/engrec.pdf"] = {["details"] = "Package documentation", },
  ["latex/sr-vorl/sr-vorl.pdf"] = {["details"] = "Package documentation", },
  ["latex/fnpct/fnpct_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/fontspec/fontspec.pdf"] = {["details"] = "Package documentation", },
  ["latex/tablestyles/README"] = {["details"] = "Readme", },
  ["latex/stex/sproof/README"] = {["details"] = "Readme", },
  ["latex/appendixnumberbeamer/README.md"] = {["details"] = "Readme", },
  ["latex/genealogytree/README"] = {["details"] = "Readme", },
  ["latex/etaremune/etaremune.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/lualatex-truncate/lualatex-truncate-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/asaetr/asaetr.pdf"] = {["details"] = "Package documentation", },
  ["latex/endnotesj/endnotesj-ja.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation", },
  ["latex/ot-tableau/ot-tableau.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-latvian/latvian.pdf"] = {["details"] = "Package documentation", },
  ["latex/einfuehrung/README"] = {["details"] = "Readme", },
  ["latex/substances/substances_en.pdf"] = {["details"] = "Package documentation", },
  ["context/third/enigma/enigma/enigma-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/competences/README.md"] = {["details"] = "Readme", },
  ["latex/vhistory/vh_set_example.pdf"] = {["details"] = "Short example", },
  ["latex/tablor/README"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/msu-thesis/README"] = {["details"] = "Readme", },
  ["fonts/fonts-churchslavonic/README"] = {["details"] = "Readme", },
  ["latex/tabto-ltx/tabto-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/teubner/teubner.pdf"] = {["details"] = "Package documentation", },
  ["fonts/padauk/README"] = {["details"] = "Readme", },
  ["latex/coolstr/coolstr.pdf"] = {["details"] = "Package documentation", },
  ["plain/impatient-fr/fbook.pdf"] = {["lang"] = "fr", ["details"] = "A copy of the book (French translation)", },
  ["latex/biblatex-oxref/oxnotes-doc.pdf"] = {["details"] = "Style documentation for oxnotes", },
  ["latex/glossaries-spanish/glossaries-spanish.pdf"] = {["details"] = "Package documentation", },
  ["latex/pfarrei/pfarrei.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["fonts/gentium-tug/gentium.pdf"] = {["details"] = "Package documentation", },
  ["latex/lt3graph/README"] = {["details"] = "Readme", },
  ["latex/numname/README"] = {["details"] = "Readme", },
  ["latex/liturg/README"] = {["details"] = "Readme", },
  ["latex/komacv/komacv.pdf"] = {["details"] = "Package documentation", },
  ["metapost/latexmp/README"] = {["details"] = "Readme", },
  ["latex/eskd/eskd.pdf"] = {["details"] = "Package documentation", },
  ["latex/optional/optional.pdf"] = {["details"] = "Package documentation", },
  ["fonts/zlmtt/README"] = {["details"] = "Readme", },
  ["latex/dataref/dataref-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/selectp/selectp-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/calculator/calculator.pdf"] = {["details"] = "Package documentation", },
  ["latex/exam/examdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ppr-prv/README"] = {["details"] = "Readme", },
  ["latex/jslectureplanner/README"] = {["details"] = "Readme", },
  ["latex/petiteannonce/petiteannonce.doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/luatex85/README.md"] = {["details"] = "Readme", },
  ["latex/pst-optexp/pst-optexp.pdf"] = {["lang"] = "en", ["details"] = "Package manual (English)", },
  ["latex/univie-ling/README"] = {["details"] = "Readme", },
  ["latex/datetime2-welsh/datetime2-welsh.pdf"] = {["details"] = "Package documentation", },
  ["latex/isonums/isonums.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/odsfile/odsfile.pdf"] = {["details"] = "Package documentation", },
  ["fonts/newtx/README"] = {["details"] = "Readme", },
  ["latex/uiucthesis/README"] = {["details"] = "Readme", },
  ["latex/nature/README"] = {["details"] = "Readme", },
  ["latex/emf/emf.pdf"] = {["details"] = "Package documentation", },
  ["latex/digiconfigs/digiconfigs.pdf"] = {["details"] = "Package documentation", },
  ["latex/iwhdp/iwhdp_Manual.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/bangorcsthesis/bangorcsthesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/coolthms/coolthms.pdf"] = {["details"] = "Package documentation", },
  ["latex/notoccite/notoccite.pdf"] = {["details"] = "Package documentation", },
  ["generic/e-french/test/tests-formats/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/latexdemo/latexdemo.pdf"] = {["details"] = "Package documentation", },
  ["latex/pictexsum/pictexsum.pdf"] = {["details"] = "The document itself", },
  ["latex/abntex2/examples/abntex2-modelo-glossarios.pdf"] = {["lang"] = "pt-br", ["details"] = "Glossary template", },
  ["fonts/cns/cns40-7/README"] = {["details"] = "Readme", },
  ["latex/ascelike/ascexmpl.pdf"] = {["details"] = "Example output", },
  ["fonts/hacm/README"] = {["details"] = "Readme", },
  ["support/latex-make/latex-make.pdf"] = {["details"] = "Package documentation", },
  ["latex/mlist/README"] = {["details"] = "Readme", },
  ["fonts/tempora/README"] = {["details"] = "Readme", },
  ["latex/frontespizio/README"] = {["details"] = "Readme", },
  ["latex/ocgx/README"] = {["details"] = "Readme", },
  ["fonts/cns/cns40-2/README"] = {["details"] = "Readme", },
  ["latex/pgf-spectra/pgf-spectraManual.pdf"] = {["details"] = "Package documentation", },
  ["latex/caption/caption-rus.pdf"] = {["lang"] = "ru", ["details"] = "Russian documentation", },
  ["latex/maths-symbols/README"] = {["details"] = "Readme", },
  ["generic/t2/etc/rubibtex/README"] = {["details"] = "Readme", },
  ["support/asymptote-faq-zh-cn/README"] = {["details"] = "Readme", },
  ["latex/translator/translator.pdf"] = {["details"] = "Package documentation", },
  ["latex/diadia/README"] = {["details"] = "Readme", },
  ["generic/c-pascal/README.eng"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/adjmulticol/adjmulticol.pdf"] = {["details"] = "Package documentation", },
  ["latex/xduthesis/README.md"] = {["details"] = "Readme", },
  ["latex/minipage-marginpar/README"] = {["details"] = "README file", },
  ["latex/bardiag/README"] = {["details"] = "Readme", },
  ["latex/datetime2-hebrew/README"] = {["details"] = "Readme", },
  ["fonts/sanskrit-t1/sktdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/guitarchordschemes/guitarchordschemes_en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/uml/README"] = {["details"] = "Package Readme", },
  ["latex/colophon/README"] = {["details"] = "Readme", },
  ["latex/hecthese/README.md"] = {["details"] = "Readme", },
  ["fonts/ebgaramond-maths/ebgaramond-maths.pdf"] = {["details"] = "Package documentation", },
  ["latex/fancyhdr/fancyhdr.pdf"] = {["details"] = "Users' manual (English)", },
  ["latex/tkz-linknodes/tkz-linknodes-screen.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/musixguit/README"] = {["details"] = "Readme", },
  ["latex/synttree/synttree.pdf"] = {["details"] = "Package documentation", },
  ["latex/historische-zeitschrift/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/beamertheme-saintpetersburg/README.md"] = {["details"] = "Readme", },
  ["latex/musicography/musicography.pdf"] = {["details"] = "Package documentation", },
  ["fonts/droid/droidsans-samples.pdf"] = {["details"] = "Font samples (sans)", },
  ["latex/tikzinclude/README"] = {["details"] = "Readme", },
  ["latex/beamersubframe/beamersubframe.pdf"] = {["details"] = "Package documentation", },
  ["latex/download/download.pdf"] = {["details"] = "Package documentation", },
  ["latex/bxcalc/sample-bxcalc.pdf"] = {["details"] = "Examples", },
  ["generic/pmx/README"] = {["details"] = "Readme", },
  ["latex/gentombow/README.md"] = {["details"] = "Readme", },
  ["latex/calculation/calculation.pdf"] = {["details"] = "Package documentation", },
  ["latex/doi/README"] = {["details"] = "Readme", },
  ["latex/br-lex/README.md"] = {["details"] = "Readme", },
  ["support/gitfile-info/gitfile-info.pdf"] = {["details"] = "Package documentation", },
  ["latex/tcolorbox/README"] = {["details"] = "Readme", },
  ["fonts/dozenal/README"] = {["details"] = "Readme", },
  ["latex/rvwrite/rvwrite-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/eukdate/README"] = {["details"] = "Readme", },
  ["latex/templates-fenn/templates-fenn-en.txt"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["metapost/threeddice/README"] = {["details"] = "Readme", },
  ["latex/bigfoot/bigfoot.pdf"] = {["details"] = "Package documentation", },
  ["latex/cv/README"] = {["details"] = "Readme", },
  ["latex/jneurosci/jneurosci.pdf"] = {["details"] = "Package documentation", },
  ["xetex/xetexref/xetex-reference.pdf"] = {["details"] = "The document itself", },
  ["latex/isodate/README"] = {["details"] = "Package Readme", },
  ["fonts/cabin/README"] = {["details"] = "Readme (primary documentation)", },
  ["latex/ticket/manual.pdf"] = {["details"] = "Package manual", },
  ["fonts/ibygrk/README"] = {["details"] = "General and installation Readme", },
  ["latex/harnon-cv/README"] = {["details"] = "Readme", },
  ["latex/hobby/hobby.pdf"] = {["details"] = "Package documentation", },
  ["latex/lastpage/lastpage-example.pdf"] = {["details"] = "Package examples", },
  ["latex/statistics/statistics.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/luainputenc/luainputenc.pdf"] = {["details"] = "Package documentation", },
  ["latex/namespc/namespc.pdf"] = {["details"] = "Package documentation", },
  ["latex/econometrics/README.txt"] = {["details"] = "Readme", },
  ["latex/thumb/README"] = {["details"] = "Package Readme", },
  ["latex/bewerbung/bewerbung.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries-finnish/glossaries-finnish.pdf"] = {["details"] = "Package documentation", },
  ["latex/docsurvey/README.txt"] = {["details"] = "Readme", },
  ["fonts/baskervald/README"] = {["details"] = "Readme", },
  ["support/typeoutfileinfo/README"] = {["details"] = "Readme", },
  ["latex/tagpair/sample.pdf"] = {["details"] = "Sample of use", },
  ["latex/listliketab/listliketab.pdf"] = {["details"] = "Package documentation", },
  ["latex/yagusylo/yagusylo.pdf"] = {["details"] = "Source documentation", },
  ["generic/autosp/README"] = {["details"] = "Readme", },
  ["latex/fast-diagram/README"] = {["lang"] = "fr", ["details"] = "Readme", },
  ["luatex/ctablestack/ctablestack.pdf"] = {["details"] = "Package documentation", },
  ["latex/documentation/README"] = {["details"] = "Readme", },
  ["latex/romanbarpagenumber/romanbarpagenumber.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-turkish/README"] = {["details"] = "Readme", },
  ["latex/gtrlib-largetrees/gtrlib.largetrees.pdf"] = {["details"] = "Package documentation", },
  ["fonts/feyn/feyn.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-occitan/README"] = {["details"] = "Readme", },
  ["latex/svg/README.md"] = {["details"] = "Readme", },
  ["latex/datetime2-samin/datetime2-samin.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/ijqc/README"] = {["details"] = "Package Readme", },
  ["latex/cookingsymbols/cookingsymbols.pdf"] = {["details"] = "Package documentation", },
  ["latex/tengwarscript/tengwarscript.pdf"] = {["details"] = "Package manual", },
  ["latex/efbox/efbox.pdf"] = {["details"] = "Package documentation", },
  ["latex/notes/notes.pdf"] = {["details"] = "Package documentation", },
  ["latex/lcyw/README"] = {["details"] = "Readme", },
  ["otherformats/lollipop/manual/lollipop-manual.pdf"] = {["details"] = "Package manual", },
  ["latex/rotfloat/rotfloat.pdf"] = {["details"] = "Package documentation", },
  ["latex/pagerange/pagerange-guide.pdf"] = {["details"] = "Samples of usage", },
  ["latex/steinmetz/README"] = {["details"] = "Readme", },
  ["latex/l3packages/l3keys2e/l3keys2e.pdf"] = {["details"] = "l3keys2e documentation", },
  ["latex/subfig/subfig.pdf"] = {["details"] = "Package documentation", },
  ["latex/chemexec/chemexec_en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["context/third/animation/README"] = {["details"] = "Readme", },
  ["metapost/expressg/README"] = {["details"] = "Package Readme", },
  ["latex/counttexruns/counttexruns.pdf"] = {["details"] = "Package documentation", },
  ["latex/morehype/texlinks.pdf"] = {["details"] = "Package documentation: texlinks", },
  ["latex/preprint/README"] = {["details"] = "Readme", },
  ["generic/babel-belarusian/belarusianb.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-pad/README"] = {["details"] = "Readme", },
  ["latex/tagpair/tagpair.pdf"] = {["details"] = "Package documentation", },
  ["support/checkcites/README"] = {["details"] = "Readme", },
  ["latex/biblatex-science/README.md"] = {["details"] = "Readme", },
  ["latex/prerex/prerex.pdf"] = {["details"] = "Package documentation", },
  ["latex/rterface/rterface.pdf"] = {["details"] = "Package documentation", },
  ["latex/ltb2bib/README"] = {["details"] = "Readme", },
  ["latex/contour/contour.pdf"] = {["details"] = "Package documentation", },
  ["latex/harvard/harvard.pdf"] = {["details"] = "Package documentation", },
  ["latex/spotcolor/README"] = {["details"] = "Readme", },
  ["fonts/libris/librisadf.pdf"] = {["details"] = "Details of distribution", },
  ["latex/aeguill/README"] = {["details"] = "Readme", },
  ["latex/crossword/cwpuzzle.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-vietnamese/vietnamese.pdf"] = {["details"] = "Package documentation", },
  ["fonts/duerer/README"] = {["details"] = "Readme", },
  ["latex/latex-brochure/README"] = {["details"] = "Readme", },
  ["bibtex/ietfbibs/README.md"] = {["details"] = "Readme", },
  ["latex/milog/milog.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/notespages/notespages.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-coil/pst-coil-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/diffcoeff/diffcoeffx.pdf"] = {["details"] = "Package documentation (diffcoeffx)", },
  ["latex/gene-logic/gn-logic14.pdf"] = {["details"] = "Package documentation", },
  ["latex/fnpara/fnpara-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/jfmutil/README.md"] = {["details"] = "Readme", },
  ["latex/biblist/biblist.pdf"] = {["details"] = "Package documentation", },
  ["luatex/lua-visual-debug/lvdebug-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/txfontsb/txfontsb.pdf"] = {["details"] = "Package documentation", },
  ["latex/prerex/intro.pdf"] = {["details"] = "Package introduction", },
  ["latex/mathpunctspace/mathpunctspace.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-publist/README"] = {["details"] = "Readme", },
  ["latex/thucoursework/thucoursework.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["latex/nath/README"] = {["details"] = "Package Readme", },
  ["generic/babel-german/germanb.pdf"] = {["details"] = "Package documentation (traditional orthography)", },
  ["fonts/junicode/Junicode.pdf"] = {["details"] = "Package documentation", },
  ["latex/papermas/papermas-example.pdf"] = {["details"] = "Package examples", },
  ["latex/hang/sample.pdf"] = {["details"] = "Sample of use", },
  ["latex/makeplot/README"] = {["details"] = "Readme", },
  ["latex/komacv-rg/komacv-addons.pdf"] = {["details"] = "Package documentation komacv-addons", },
  ["latex/hletter/README"] = {["details"] = "Readme", },
  ["latex/bizcard/bizcard.pdf"] = {["details"] = "Package documentation", },
  ["latex/eqnalign/README.md"] = {["details"] = "Readme", },
  ["latex/colorinfo/README"] = {["details"] = "Readme", },
  ["fonts/frederika2016/FontInfo.pdf"] = {["details"] = "Package documentation", },
  ["latex/padcount/padcount.pdf"] = {["details"] = "Package documentation", },
  ["latex/splitbib/splitbib.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-diffraction/pst-diffraction-docDE.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/boxhandler/README"] = {["details"] = "README file", },
  ["latex/fifo-stack/README"] = {["details"] = "Readme", },
  ["fonts/wadalab/README"] = {["details"] = "Readme", },
  ["support/texosquery/texosquery.pdf"] = {["details"] = "Package documentation", },
  ["latex/penrose/penrose_code.pdf"] = {["details"] = "Code documentation", },
  ["latex/llncsconf/README.md"] = {["details"] = "Readme", },
  ["lualatex/luasseq/luasseq.pdf"] = {["details"] = "Package documentation", },
  ["latex/statistik/statistik.pdf"] = {["details"] = "Package documentation", },
  ["fonts/fge/README"] = {["details"] = "Readme", },
  ["latex/lastpackage/README"] = {["details"] = "Readme", },
  ["latex/lmake/lmake.pdf"] = {["details"] = "Package documentation", },
  ["latex/boxedminipage/boxedminipage.pdf"] = {["details"] = "Package documentation", },
  ["latex/phfnote/phfnote.pdf"] = {["details"] = "Package documentation", },
  ["latex/cquthesis/figures/README"] = {["details"] = "Readme", },
  ["latex/umoline/umoline.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-mla/doc/biblatex-mla.pdf"] = {["details"] = "Package documentation", },
  ["latex/alterqcm/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["fonts/bookhands/README.PRW"] = {["details"] = "Readme", },
  ["latex/russ/russ_doc.pdf"] = {["lang"] = "ru", ["details"] = "Package documentation", },
  ["latex/lcd/00readme"] = {["details"] = "Package README", },
  ["latex/silence/silence-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-luh-ipw/README"] = {["details"] = "Readme", },
  ["latex/iitem/README"] = {["details"] = "Readme", },
  ["latex/combelow/README"] = {["details"] = "Readme", },
  ["latex/rtkinenc/rtkinenc-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/FAQ-en/newfaq.pdf"] = {["details"] = "FAQ formatted for A4 paper", },
  ["latex/latexbangla/README"] = {["details"] = "Readme", },
  ["fonts/libertinust1math/README"] = {["details"] = "Readme", },
  ["generic/pst-text/README"] = {["details"] = "Package README", },
  ["generic/barr/README"] = {["details"] = "Readme", },
  ["latex/mgltex/mgltex.pdf"] = {["details"] = "Package documentation", },
  ["support/fragmaster/README.de"] = {["lang"] = "de", ["details"] = "Readme (German)", },
  ["latex/yathesis/README.md"] = {["details"] = "Readme", },
  ["latex/shadethm/shadethm-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/concrete/README"] = {["details"] = "Readme", },
  ["latex/codesection/codesection.pdf"] = {["details"] = "Package documentation", },
  ["fonts/adforn/adforn.pdf"] = {["details"] = "Package documentation and font tables", },
  ["generic/modulus/README"] = {["details"] = "Readme", },
  ["latex/jnuexam/README"] = {["details"] = "Readme", },
  ["latex/hyperref/manual.html"] = {["details"] = "Manual, HTML version", },
  ["latex/hepthesis/example/example.pdf"] = {["details"] = "Part of a thesis, seen as for printing", },
  ["latex/program/program-doc.pdf"] = {["details"] = "Generated documentation", },
  ["latex/ecv/ecv.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/lua-check-hyphen/doc/sample.pdf"] = {["details"] = "Sample of use", },
  ["latex/chletter/chletter.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["fonts/dictsym/README"] = {["details"] = "Readme", },
  ["latex/outline/outline.pdf"] = {["details"] = "Package documentation", },
  ["fonts/newtx/newtxdoc.pdf"] = {["details"] = "Package documentation", },
  ["context/third/account/account-doc.pdf"] = {["details"] = "Package documented source", },
  ["fonts/kpfonts/Kpfonts-Doc-French.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/arabi-add/arabi-add-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/forms16be/README.md"] = {["details"] = "Readme", },
  ["latex/bodegraph/bodegraph.pdf"] = {["details"] = "Package documentation (French with some English)", },
  ["latex/emarks/README"] = {["details"] = "Readme", },
  ["latex/exercisebank/exercisebank-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/skeycommand/README"] = {["details"] = "Readme", },
  ["latex/komacv/examples/komacv_example_casual.pdf"] = {["details"] = "Example of use (casual)", },
  ["latex/changepage/changepage.pdf"] = {["details"] = "Package documentation", },
  ["latex/intopdf/README.md"] = {["details"] = "Readme", },
  ["lualatex/luahyphenrules/luahyphenrules.pdf"] = {["details"] = "Package documentation", },
  ["latex/ot-tableau/README"] = {["details"] = "Readme", },
  ["latex/pstricks_calcnotes/README"] = {["details"] = "Readme", },
  ["latex/biblatex-apa/biblatex-apa-test.pdf"] = {["details"] = "The examples in the APA style guide", },
  ["generic/pst-ob3d/pst-ob3d.pdf"] = {["details"] = "Package documentation", },
  ["latex/appendix/README"] = {["details"] = "Package Readme", },
  ["latex/biblatex-ext/biblatex-ext.pdf"] = {["details"] = "Package documentation", },
  ["latex/image-gallery/README"] = {["details"] = "Readme", },
  ["latex/labelschanged/README.txt"] = {["details"] = "Readme", },
  ["latex/lstaddons/README"] = {["details"] = "Readme", },
  ["fonts/fira/fira-samples.pdf"] = {["details"] = "Font samples", },
  ["latex/hitec/README"] = {["details"] = "Readme", },
  ["latex/fancytabs/fancytabs.pdf"] = {["details"] = "Package documentation", },
  ["generic/texapi/texapi-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/musuos/musuos.pdf"] = {["details"] = "Package documentation", },
  ["latex/autobreak/autobreak.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex2e-help-texinfo-fr/latex2e-fr.html"] = {["lang"] = "fr", ["details"] = "HTML conversion of the source (French)", },
  ["latex/g-brief/beispiel.pdf"] = {["lang"] = "de", ["details"] = "Example, old version", },
  ["generic/babel-romanian/romanian.pdf"] = {["details"] = "Package documentation", },
  ["latex/lcd/example.pdf"] = {["details"] = "Examples", },
  ["plain/mkpattern/README"] = {["details"] = "Package README", },
  ["latex/pdfcomment/example.pdf"] = {["details"] = "Example output", },
  ["support/ctanupload/README"] = {["details"] = "Readme", },
  ["latex/bgteubner/LIESMICH"] = {["lang"] = "de", ["details"] = "Readme", },
  ["context/third/account/README"] = {["details"] = "Readme", },
  ["latex/graphicxbox/README"] = {["details"] = "Readme", },
  ["support/srcredact/srcredact.pdf"] = {["details"] = "Package documentation", },
  ["latex/synproof/synproof-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/gcite/README"] = {["details"] = "Readme", },
  ["latex/foreign/foreign.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-vue3d/pst-vue3d-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/blowup/blowup.pdf"] = {["details"] = "Package documentation", },
  ["generic/velthuis/README"] = {["details"] = "Readme", },
  ["generic/epsf/epsf-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/eqnalign/eqnalign.pdf"] = {["details"] = "Package documentation", },
  ["fonts/bera/README"] = {["details"] = "Readme", },
  ["latex/wsemclassic/wsemclassic.pdf"] = {["details"] = "Package documentation", },
  ["fonts/gfsartemisia/README"] = {["details"] = "Readme", },
  ["latex/thalie/thalie.pdf"] = {["details"] = "Package documentation", },
  ["latex/stackengine/stackengine.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-osci/README"] = {["details"] = "Readme", },
  ["latex/magaz/magaz.pdf"] = {["details"] = "Package documentation", },
  ["fonts/alegreya/README"] = {["details"] = "Readme", },
  ["latex/bizcard/bizex.pdf"] = {["details"] = "Example of use", },
  ["latex/quicktype/Quicktype.pdf"] = {["details"] = "Package documentation", },
  ["latex/bewerbung/README_DE"] = {["lang"] = "de", ["details"] = "Readme(German)", },
  ["latex/listing/listing.pdf"] = {["details"] = "Package documentation", },
  ["latex/tengwarscript/tengfonts.pdf"] = {["details"] = "Font sampler", },
  ["latex/threeparttablex/README"] = {["details"] = "Readme", },
  ["fonts/librecaslon/README"] = {["details"] = "Readme", },
  ["latex/denisbdoc/README.md"] = {["details"] = "Readme", },
  ["generic/variations/README"] = {["lang"] = "en", ["details"] = "Package README", },
  ["fonts/memdesign/README"] = {["details"] = "Readme", },
  ["latex/biblatex-realauthor/documentation/example-realauthor.pdf"] = {["details"] = "Example of use", },
  ["plain/mkpattern/mkpatter.pdf"] = {["details"] = "Package documentation", },
  ["latex/pgfopts/README"] = {["details"] = "Readme", },
  ["lualatex/cloze/cloze.pdf"] = {["details"] = "Package documentation", },
  ["generic/mathdots/mathdots.pdf"] = {["details"] = "Package documentation", },
  ["latex/reledmac/README"] = {["details"] = "Readme", },
  ["fonts/gfscomplutum/README"] = {["details"] = "Readme", },
  ["latex/stdclsdv/README"] = {["details"] = "Readme", },
  ["fonts/zhmetrics-uptex/doc/README.md"] = {["details"] = "Readme", },
  ["latex/epsdice/dice.pdf"] = {["details"] = "Sample dice", },
  ["latex/showexpl/README"] = {["details"] = "README file", },
  ["latex/fancypar/fancypar.pdf"] = {["details"] = "Package documentation", },
  ["latex/svn-multi/svn-multi.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-scottish/scottish.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/luabidi/README"] = {["details"] = "Readme", },
  ["latex/texvc/texvc.pdf"] = {["details"] = "Package documentation", },
  ["generic/fntproof/fntproof-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikzducks/README.md"] = {["details"] = "Readme", },
  ["generic/encxvlna/encxvlna.pdf"] = {["details"] = "Package documentation", },
  ["generic/ifxetex/README"] = {["details"] = "Readme", },
  ["latex/braille/summary.pdf"] = {["details"] = "Package documentation (PDF)", },
  ["latex/flowchart/README"] = {["details"] = "Readme", },
  ["bibtex/gbt7714/README.md"] = {["lang"] = "zh", ["details"] = "Readme", },
  ["latex/ijsra/ijsra.pdf"] = {["details"] = "Package documentation", },
  ["latex/newunicodechar/README"] = {["details"] = "Readme", },
  ["latex/oubraces/oubraces.pdf"] = {["details"] = "Package documentation", },
  ["latex/lplfitch/README"] = {["details"] = "Readme", },
  ["latex/elteikthesis/elteikthesis.pdf"] = {["lang"] = "hu", ["details"] = "Package documentation (mostly in Hungarian, but with English summary)", },
  ["latex/gmverse/gmverse.pdf"] = {["details"] = "Package documentation", },
  ["latex/epigraph/README"] = {["details"] = "Readme", },
  ["plain/metatex/README"] = {["details"] = "Readme", },
  ["latex/uri/README"] = {["details"] = "Readme", },
  ["latex/comment/README"] = {["details"] = "Readme", },
  ["latex/dpfloat/README"] = {["details"] = "Readme", },
  ["latex/apa6/apa6.pdf"] = {["details"] = "Package documentation", },
  ["latex/flowfram/README"] = {["details"] = "Readme", },
  ["latex/acmart/README"] = {["details"] = "Readme", },
  ["latex/engpron/LISEZMOI"] = {["lang"] = "fr", ["details"] = "Lisez moi", },
  ["latex/beilstein/README.md"] = {["details"] = "Readme", },
  ["latex/sa-tikz/sa-tikz-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/amsrefs/amsrdoc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-rputover/README.md"] = {["details"] = "Readme", },
  ["fonts/txfonts/txfontsdoc.pdf"] = {["details"] = "Package documentation (Letter)", },
  ["luatex/luatexbase/README.md"] = {["details"] = "Readme", },
  ["fonts/arphic-ttf/README.md"] = {["details"] = "Readme", },
  ["latex/bxcalc/README.md"] = {["details"] = "Readme", },
  ["latex/memexsupp/README"] = {["details"] = "Readme", },
  ["latex/classics/classics.pdf"] = {["details"] = "Package documentation", },
  ["latex/circ/circ.pdf"] = {["details"] = "Package documentation", },
  ["latex/linguex/ps-trees-doc.pdf"] = {["details"] = "PS-trees manual", },
  ["generic/kastrup/binhex.pdf"] = {["details"] = "Package documentation", },
  ["latex/komacv/examples/komacv_example.pdf"] = {["details"] = "Example of use (basic)", },
  ["fonts/baskervaldx/baskervaldx-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/poster-mac/poster-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamerswitch/beamerswitch-example-handout.pdf"] = {["details"] = "Example of use (handout)", },
  ["latex/jumplines/jumplines_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/dataref/README.md"] = {["details"] = "Readme", },
  ["fonts/adfsymbols/adfsymbols.pdf"] = {["details"] = "Package documentation", },
  ["latex/changes/changes.english.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/dsptricks/dspTricksManual.pdf"] = {["details"] = "Package documentation", },
  ["latex/finstrut/README"] = {["details"] = "Readme", },
  ["latex/nicematrix/nicematrix.pdf"] = {["details"] = "Package documentation", },
  ["latex/hobete/README"] = {["lang"] = "de", ["details"] = "Readme (German)", },
  ["latex/splitindex/README"] = {["details"] = "Readme", },
  ["latex/hackthefootline/doc/hackthefootline-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/dashundergaps/dashundergaps.pdf"] = {["details"] = "Package documentation", },
  ["fonts/inconsolata/README"] = {["details"] = "Readme", },
  ["latex/biblatex-juradiss/README"] = {["details"] = "Readme", },
  ["xelatex/resumecls/resumecls.pdf"] = {["details"] = "Package listing", },
  ["latex/lisp-on-tex/README.md"] = {["details"] = "Readme", },
  ["latex/ecgdraw/ecgdraw.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-sbl/README.md"] = {["details"] = "Readme", },
  ["generic/gates/README"] = {["details"] = "Readme", },
  ["generic/yax/yax-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/bguq/bguq-doc.pdf"] = {["details"] = "Package documentation", },
  ["platex/pxjahyper/pxjahyper.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation (Japanese)", },
  ["latex/emp/emp.pdf"] = {["details"] = "Documented source", },
  ["metapost/cmarrows/cmarrows.pdf"] = {["details"] = "Package documentation", },
  ["latex/guitartabs/nothingelsematters.pdf"] = {["details"] = "Example of use", },
  ["generic/babel-greek/usage.pdf"] = {["details"] = "Usage documentation", },
  ["generic/pst-solides3d/README"] = {["details"] = "Readme", },
  ["latex/tabvar/demo.pdf"] = {["details"] = "Package demo", },
  ["latex/asciilist/asciilist.pdf"] = {["details"] = "Package documentation", },
  ["latex/cjk/doc/japanese/README"] = {["details"] = "Readme file", },
  ["latex/MemoirChapStyles/README"] = {["details"] = "Readme", },
  ["fonts/gofonts/README"] = {["details"] = "Readme", },
  ["fonts/amiri/documentation-arabic.pdf"] = {["lang"] = "ar", ["details"] = "Package documentation (Arabic)", },
  ["fonts/cbfonts/grmn1000table.pdf"] = {["details"] = "Sample font table", },
  ["latex/pdftricks/test.pdf"] = {["details"] = "Test example (source available)", },
  ["latex/biblatex-gost/biblatex-gost.pdf"] = {["lang"] = "ru", ["details"] = "Package documentation", },
  ["latex/simplewick/simplewick.pdf"] = {["details"] = "Package documentation", },
  ["fonts/libertinus-otf/libertinus-otf-doc.pdf"] = {["details"] = "Package documentation", },
  ["support/bib2gls/bib2gls.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/sort-by-letters/README"] = {["details"] = "Readme", },
  ["latex/ifthenx/README"] = {["details"] = "Readme", },
  ["latex/dynblocks/dynblocks.pdf"] = {["details"] = "Package documentation", },
  ["latex/etdipa/README"] = {["details"] = "Readme", },
  ["latex/tex-font-errors-cheatsheet/README"] = {["details"] = "Readme", },
  ["fonts/poltawski/ap-info.pdf"] = {["details"] = "Technical documentation", },
  ["latex/logicproof/README"] = {["details"] = "Readme", },
  ["latex/cutwin/cutwin.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/philokalia/README"] = {["details"] = "Readme", },
  ["generic/latex-notes-zh-cn/latex-notes-zh-cn.pdf"] = {["lang"] = "zh", ["details"] = "The document itself", },
  ["latex/guitartabs/README.md"] = {["details"] = "Readme", },
  ["latex/cmtiup/README"] = {["details"] = "Readme", },
  ["generic/babel-greek/babel-greek.pdf"] = {["details"] = "Documentation", },
  ["latex/mathastext/mathastext.pdf"] = {["details"] = "Package documentation", },
  ["fonts/fetamont/fetamont.pdf"] = {["details"] = "Package documentation", },
  ["latex/turabian-formatting/turabian-formatting-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/kpfonts/kpfonts.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/storecmd/storecmd-guide.pdf"] = {["details"] = "Package documentation", },
  ["generic/mkjobtexmf/mkjobtexmf.pdf"] = {["details"] = "Package documentation (PDF format)", },
  ["latex/lapdf/README"] = {["details"] = "Readme", },
  ["latex/tableof/README"] = {["details"] = "Readme", },
  ["metapost/shapes/README"] = {["details"] = "Readme", },
  ["latex/tikzscale/tikzscale.pdf"] = {["details"] = "Package documentation", },
  ["latex/termcal-de/termcal-de-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/uspace/README.md"] = {["details"] = "Readme", },
  ["latex/interfaces/README"] = {["details"] = "Readme", },
  ["latex/keystroke/key-test.pdf"] = {["details"] = "Package demo", },
  ["latex/yaletter/README"] = {["details"] = "Readme", },
  ["latex/bloques/README"] = {["details"] = "Readme", },
  ["latex/acronym/acronym.pdf"] = {["details"] = "Package documentation", },
  ["latex/uhrzeit/README.txt"] = {["details"] = "Readme", },
  ["latex/docmfp/docmfp.pdf"] = {["details"] = "Package documentation", },
  ["latex/clock/README"] = {["details"] = "Readme", },
  ["fonts/playfair/playfair-samples.pdf"] = {["details"] = "Font samples", },
  ["latex/graphbox/graphbox.pdf"] = {["details"] = "Package documentation", },
  ["latex/overpic/overpic.pdf"] = {["lang"] = "en", ["details"] = "Documentation", },
  ["metapost/garrigues/README"] = {["details"] = "Readme", },
  ["fonts/cmsrb/cmsrb-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/footnotebackref/README"] = {["details"] = "Readme", },
  ["latex/ekaia/README"] = {["details"] = "Readme", },
  ["latex/footnpag/footnpag-user.pdf"] = {["details"] = "Package documentation", },
  ["latex/ling-macros/README.txt"] = {["details"] = "Readme", },
  ["latex/graphbox/README.txt"] = {["details"] = "Readme", },
  ["latex/callouts/README.md"] = {["details"] = "Readme", },
  ["latex/bezos/bezos.pdf"] = {["details"] = "Bundle outline", },
  ["latex/bclogo/LISEZ-MOI"] = {["lang"] = "fr", ["details"] = "Lisez moi (French)", },
  ["latex/gauss/gauss-ex.pdf"] = {["details"] = "Example of use (Gaussian elimination)", },
  ["latex/footnoterange/README"] = {["details"] = "Readme", },
  ["generic/babel-friulan/friulan.pdf"] = {["details"] = "Package documentation", },
  ["context/third/bnf/t-bnf.pdf"] = {["details"] = "Package documentation", },
  ["latex/abnt/README.md"] = {["details"] = "Readme", },
  ["fonts/amsfonts/amsfndoc.pdf"] = {["details"] = "Users' guide to the fonts (for version 2.2d)", },
  ["latex/breqn/breqn.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-karnaugh/tikz-karnaugh-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/sageep/README"] = {["details"] = "Readme", },
  ["latex/datetime2/datetime2.pdf"] = {["details"] = "Package documentation", },
  ["latex/amscls-doc/Author_Handbook_Mono.pdf"] = {["details"] = "AMS Author Handbook for Monograph Classes", },
  ["latex/platex-tools/README.md"] = {["details"] = "Readme", },
  ["latex/emp/empman.pdf"] = {["details"] = "Package manual", },
  ["latex/biblatex-opcit-booktitle/documentation/biblatex-opcit-booktitle.pdf"] = {["details"] = "Package documentation", },
  ["latex/mathtools/mathtools.pdf"] = {["details"] = "Package documentation", },
  ["latex/curves/curves.pdf"] = {["details"] = "Package documentation", },
  ["latex/footmisc/footmisc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/biblatex-cheatsheet/README"] = {["details"] = "Readme", },
  ["latex/makecmds/README"] = {["details"] = "Readme", },
  ["latex/onlyamsmath/onlyamsmath.pdf"] = {["details"] = "Package documentation", },
  ["fonts/erewhon/README"] = {["details"] = "Readme", },
  ["context/third/inifile/inifile-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/drac/drac-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/datetime2-norsk/datetime2-norsk.pdf"] = {["details"] = "Package documentation", },
  ["latex/dlfltxb/dlfltxbcodetips.pdf"] = {["details"] = "dlfltxbcodetips package documentation", },
  ["generic/pst-diffraction/pst-diffraction-docE.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/fancyref/README"] = {["details"] = "Package Readme", },
  ["latex/hc/README"] = {["details"] = "Readme", },
  ["latex/amsrefs/amsrefs.pdf"] = {["details"] = "Documented source", },
  ["latex/copyedit/README"] = {["details"] = "Readme", },
  ["latex/jslectureplanner/examples/English/lecturepresentation-english.pdf"] = {["details"] = "Example of use", },
  ["latex/longnamefilelist/README"] = {["details"] = "Readme", },
  ["latex/l2tabu-french/l2tabufr.pdf"] = {["lang"] = "fr", ["details"] = "The document itself", },
  ["generic/pst-asr/pst-asr-examples.pdf"] = {["details"] = "Examples of usage", },
  ["latex/b1encoding/b1encoding.pdf"] = {["details"] = "Package documentation", },
  ["latex/miller/miller.pdf"] = {["details"] = "Package documentation", },
  ["latex/cclicenses/README"] = {["details"] = "Readme", },
  ["latex/komacv/examples/komacv_example_classic.pdf"] = {["details"] = "Example of use (classic)", },
  ["latex/modroman/modroman-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/articleingud/README"] = {["details"] = "Readme", },
  ["latex/greek-fontenc/README"] = {["details"] = "Readme", },
  ["latex/bankstatement/README.md"] = {["details"] = "Readme", },
  ["latex/phfthm/README.md"] = {["details"] = "Readme", },
  ["latex/varsfromjobname/varsfromjobname.pdf"] = {["details"] = "Package documentation", },
  ["latex/epsdice/README"] = {["details"] = "Readme", },
  ["fonts/mathdesign/mathdesign-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/pdfslide/demo.pdf"] = {["details"] = "Narrative demonstration of the package", },
  ["latex/hyper/README"] = {["details"] = "Readme", },
  ["latex/biblatex-ext/README.md"] = {["details"] = "Readme", },
  ["latex/chscite/chscite.pdf"] = {["details"] = "Package documentation", },
  ["latex/xdoc/README"] = {["details"] = "Package Readme", },
  ["latex/cv/CVCTAN.pdf"] = {["details"] = "Example CV", },
  ["latex/mathpunctspace/README.md"] = {["details"] = "Readme", },
  ["generic/pst-optic/pst-optic-doc.pdf"] = {["details"] = "Package manual:", },
  ["latex/cancel/cancel.pdf"] = {["details"] = "Package documentation", },
  ["latex/hagenberg-thesis/examples/HgbTermReport/main.pdf"] = {["lang"] = "de", ["details"] = "Example document / Tutorial (German)", },
  ["latex/pst-poker/README"] = {["details"] = "Readme", },
  ["latex/fnbreak/README"] = {["details"] = "Readme", },
  ["support/make4ht/README"] = {["details"] = "Readme", },
  ["platex/jsclasses/jsclasses.pdf"] = {["lang"] = "ja", ["details"] = "Documentation of the classes (Japanese)", },
  ["metapost/roundrect/README"] = {["details"] = "Readme", },
  ["latex/greek-inputenc/README"] = {["details"] = "Readme", },
  ["latex/codedoc/README"] = {["details"] = "Readme", },
  ["latex/filemod/README"] = {["details"] = "Readme", },
  ["latex/glossaries-serbian/glossaries-serbian.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-french/README"] = {["details"] = "Readme", },
  ["generic/babel-serbian/serbian.pdf"] = {["details"] = "Package documentation", },
  ["generic/shapepar/shapepar.pdf"] = {["details"] = "Package documentation", },
  ["latex/fancyvrb/README"] = {["details"] = "Readme", },
  ["latex/gradientframe/README"] = {["details"] = "Readme", },
  ["latex/doipubmed/doipubmed.pdf"] = {["details"] = "Package documentation (PDF)", },
  ["generic/bitelist/README"] = {["details"] = "Readme", },
  ["fonts/chivo/README.md"] = {["details"] = "Readme", },
  ["latex/cryst/cryst1.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/octavo/tub-octavo.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-ladder/README.txt"] = {["details"] = "Readme", },
  ["luatex/base/luatex.pdf"] = {["details"] = "LuaTeX Reference", },
  ["latex/typeface/typeface.pdf"] = {["details"] = "Package documentation", },
  ["metapost/suanpan/README"] = {["details"] = "Readme", },
  ["fonts/jablantile/jablantile.pdf"] = {["details"] = "Example of use (letter to Jablan)", },
  ["generic/pst-antiprism/pst-antiprism-doc.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/lua-check-hyphen/doc/README.md"] = {["details"] = "Readme", },
  ["latex/elpres/README"] = {["details"] = "Readme", },
  ["latex/ellipsis/README"] = {["details"] = "Readme", },
  ["latex/datetime/README"] = {["details"] = "Package README", },
  ["latex/bewerbung/README"] = {["details"] = "Readme", },
  ["latex/cnbwp/cnbwp-manual-cs.pdf"] = {["details"] = "Package documentation (Czech)", },
  ["latex/graphics-def/README.md"] = {["details"] = "Readme", },
  ["latex/includernw/README.txt"] = {["details"] = "Readme", },
  ["latex/fancyslides/doc/fancyslides.pdf"] = {["details"] = "Package documentation", },
  ["latex/silence/README"] = {["details"] = "Readme", },
  ["latex/xsim/xsim_manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/mfpic4ode/demo/demo.pdf"] = {["details"] = "Examples of use", },
  ["latex/chemgreek/chemgreek_en.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cochineal/cochineal-doc.pdf"] = {["details"] = "Package manual", },
  ["latex/dtxdescribe/README.txt"] = {["details"] = "Readme", },
  ["fonts/academicons/academicons.pdf"] = {["details"] = "Package documentation", },
  ["plain/fontch/README"] = {["details"] = "Readme and usage details", },
  ["latex/ushort/ushort.pdf"] = {["details"] = "Package documentation", },
  ["latex/moreverb/README"] = {["details"] = "Package Readme", },
  ["fonts/japanese-otf-uptex/README"] = {["details"] = "Readme", },
  ["latex/crossreference/README"] = {["details"] = "Readme", },
  ["fonts/baskervillef/README"] = {["details"] = "Readme", },
  ["latex/fixme/fixme.pdf"] = {["details"] = "Package documentation", },
  ["latex/alnumsec/README"] = {["details"] = "Readme", },
  ["lualatex/luainputenc/README"] = {["details"] = "Readme", },
  ["fonts/cmll/README"] = {["details"] = "Readme", },
  ["latex/sourcecodepro/sourcecodepro.pdf"] = {["details"] = "Package documentation", },
  ["support/de-macro/user-guide.pdf"] = {["details"] = "Package documentation", },
  ["latex/achemso/achemso.pdf"] = {["details"] = "Package documentation", },
  ["fonts/bbold-type1/README"] = {["details"] = "Readme", },
  ["generic/olsak-misc/README"] = {["details"] = "Readme", },
  ["metapost/epsincl/README"] = {["details"] = "Readme", },
  ["latex/calrsfs/README"] = {["details"] = "Readme", },
  ["latex/linop/linop.pdf"] = {["details"] = "Package documentation", },
  ["latex/lshort-english/README"] = {["details"] = "Readme", },
  ["latex/feupphdteses/Template_EN.pdf"] = {["details"] = "Template showing construction and appearance", },
  ["latex/easylist/easylist-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/achemso/README.md"] = {["details"] = "Readme", },
  ["latex/hagenberg-thesis/hagenberg-thesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/lni/lni.pdf"] = {["details"] = "Package documentation", },
  ["latex/erdc/sample.pdf"] = {["details"] = "Sample of usage", },
  ["latex/romannum/romannum.pdf"] = {["details"] = "Package documentation", },
  ["latex/asyfig/asyfig.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-labo/pst-labo-docDE.pdf"] = {["details"] = "German documentation:", },
  ["latex/datetime2-croatian/datetime2-croatian.pdf"] = {["details"] = "Package documentation", },
  ["latex/acro/README"] = {["details"] = "Readme", },
  ["latex/pgfgantt/README"] = {["details"] = "Readme", },
  ["lualatex/luacode/README"] = {["details"] = "Readme", },
  ["latex/alterqcm/doc_aq-screen.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/elbioimp/elbioimp.pdf"] = {["details"] = "Package documentation", },
  ["generic/colortab/colortab-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/formlett/prog_manual.pdf"] = {["details"] = "Programmers manual", },
  ["latex/isopt/isopt-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/elzcards/README"] = {["details"] = "Readme", },
  ["latex/flagderiv/flagderiv.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/interchar/interchar.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cfr-lm/README"] = {["details"] = "Readme", },
  ["latex/chemmacros/chemmacros_en.pdf"] = {["lang"] = "en", ["details"] = "Chemmacros package documentation", },
  ["latex/mceinleger/mceinleger.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-sfc/README.txt"] = {["details"] = "Readme", },
  ["plain/tugboat-plain/tubguide.pdf"] = {["details"] = "TUGboat Authors' Guide", },
  ["context/third/gantt/gantt.txt"] = {["details"] = "Package documentation", },
  ["generic/tex-ewd/README"] = {["details"] = "Readme", },
  ["latex/russ/readme.RU.txt"] = {["lang"] = "ru", ["details"] = "Readme", },
  ["latex/biblatex-science/biblatex-science.pdf"] = {["details"] = "Package documentation", },
  ["latex/hyperref/README.pdf"] = {["details"] = "Readme, hyper-crossreferenced", },
  ["context/third/enigma/enigma/README"] = {["details"] = "Readme", },
  ["latex/leading/leading.pdf"] = {["details"] = "Package documentation", },
  ["latex/etextools/README"] = {["details"] = "Readme", },
  ["latex/tensor/tensor.pdf"] = {["details"] = "Package documentation", },
  ["latex/thesis-ekf/thesis-ekf.pdf"] = {["lang"] = "hu", ["details"] = "Package documentation", },
  ["latex/ean13isbn/README"] = {["details"] = "Readme", },
  ["support/classpack/README"] = {["details"] = "Readme", },
  ["latex/easyreview/doc/easyReview.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-morenames/README"] = {["details"] = "Readme", },
  ["latex/dijkstra/dijkstra-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["generic/pst-spinner/pst-spinner-doc.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/apalike-german/README.html"] = {["details"] = "Readme (HTML)", },
  ["latex/contracard/README"] = {["details"] = "Readme", },
  ["latex/econometrics/econometrics.pdf"] = {["details"] = "Package documentation", },
  ["latex/ebong/README"] = {["details"] = "Readme", },
  ["fonts/cbfonts-fd/README"] = {["details"] = "Readme", },
  ["latex/latex2e-help-texinfo/latex2e.pdf"] = {["details"] = "PDF conversion of the source (English)", },
  ["support/arara/README.md"] = {["details"] = "Readme", },
  ["latex/comment/comment.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/xepersian/xepersian.pdf"] = {["details"] = "Package source documentation", },
  ["latex/captcont/captcont.pdf"] = {["details"] = "Package documentation", },
  ["latex/coolstr/README"] = {["details"] = "Readme", },
  ["support/latex-papersize/README"] = {["details"] = "Readme", },
  ["latex/duerer-latex/duerer.pdf"] = {["details"] = "Package documentation", },
  ["fonts/amiri/amiri-table.pdf"] = {["details"] = "Font table", },
  ["latex/bxenclose/README.md"] = {["details"] = "Readme", },
  ["latex/graphicxpsd/graphicxpsd.pdf"] = {["details"] = "Package documentation", },
  ["latex/coverpage/README"] = {["details"] = "README file", },
  ["latex/grundgesetze/README"] = {["details"] = "Readme", },
  ["generic/FAQ-en/README"] = {["details"] = "Package Readme", },
  ["latex/dynkin-diagrams/dynkin-diagrams.pdf"] = {["details"] = "Package documentation", },
  ["latex/colortbl/colortbl-DE.pdf"] = {["lang"] = "de", ["details"] = "Package manual", },
  ["latex/beamerdarkthemes/beamerdarkthemesuserguide.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/typewriter/typewriter-guide.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-mirror/pst-mirror-doc.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/thaienum/thaienum.pdf"] = {["details"] = "Package documentation", },
  ["latex/blochsphere/example.pdf"] = {["details"] = "Example diagram", },
  ["generic/ifxptex/README"] = {["details"] = "Readme", },
  ["latex/glossaries-danish/glossaries-danish.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-greek/README.html"] = {["details"] = "Readme (HTML)", },
  ["latex/enumitem/enumitem.pdf"] = {["details"] = "Package documentation", },
  ["fonts/paratype/README"] = {["details"] = "Readme", },
  ["latex/bxcalc/bxcalcux.pdf"] = {["details"] = "Further Package Documentation", },
  ["latex/jneurosci/README"] = {["details"] = "Readme", },
  ["latex/xtab/README"] = {["details"] = "Readme", },
  ["latex/twoup/twoup.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamer-FUBerlin/doc/exampleClass.pdf"] = {["details"] = "Example of use of the class", },
  ["latex/manyind/mind.html"] = {["details"] = "Package documentation", },
  ["latex/cellspace/cellspace.pdf"] = {["details"] = "Package documentation", },
  ["latex/translation-moreverb-de/moreverb-de.dtx.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["metapost/drv/drv-guide.pdf"] = {["details"] = "User guide", },
  ["support/make4ht/make4ht-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/collcell/collcell.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-bookinarticle/README"] = {["details"] = "Readme", },
  ["latex/listlbls/listlbls.pdf"] = {["details"] = "Package documentation", },
  ["latex/tagging/README"] = {["details"] = "Readme", },
  ["latex/cuisine/cuisine.pdf"] = {["details"] = "Package documentation", },
  ["latex/tocbibind/tocbibind.pdf"] = {["details"] = "Package documentation", },
  ["latex/uiucthesis/thesis-ex.pdf"] = {["details"] = "Example of use", },
  ["latex/phfnote/README.md"] = {["details"] = "Readme", },
  ["latex/cbcoptic/README"] = {["details"] = "Readme", },
  ["latex/fundus-calligra/calligra.pdf"] = {["details"] = "Package documentation", },
  ["latex/cals/tb101parashchenko.pdf"] = {["details"] = "Users Guide", },
  ["latex/adjustbox/README"] = {["details"] = "Readme", },
  ["latex/ydoc/ydoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/afparticle/afparticle.pdf"] = {["details"] = "Package documentation", },
  ["latex/filecontents/filecontents.pdf"] = {["details"] = "Package documentation", },
  ["latex/pstring/pstring-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst-eucl-translation-bg/pst-eucl-docBG.pdf"] = {["lang"] = "bg", ["details"] = "Package documentation", },
  ["latex/hrefhide/README"] = {["details"] = "Readme", },
  ["latex/memoir/memman.pdf"] = {["details"] = "Basic user manual", },
  ["latex/etoolbox/etoolbox.pdf"] = {["details"] = "Package documentation", },
  ["generic/xint/CHANGES.html"] = {["details"] = "Changes (HTML)", },
  ["latex/presentations/README"] = {["details"] = "Readme", },
  ["fonts/eulerpx/README"] = {["details"] = "Readme", },
  ["latex/counttexruns/README"] = {["details"] = "Readme", },
  ["latex/logreq/README"] = {["details"] = "Readme", },
  ["latex/etsvthor/README.md"] = {["details"] = "Readme", },
  ["latex/revquantum/example.pdf"] = {["details"] = "Example of use", },
  ["latex/gradientframe/gradientframe.pdf"] = {["details"] = "Package documentation", },
  ["latex/avremu/avremu.pdf"] = {["details"] = "Package documentation", },
  ["latex/coollist/coollist.pdf"] = {["details"] = "Package documentation", },
  ["latex/translation-tabbing-fr/f-Tabbing.pdf"] = {["lang"] = "fr", ["details"] = "The document itself", },
  ["latex/guitlogo/guit.pdf"] = {["lang"] = "it", ["details"] = "Package documentation", },
  ["amstex/base/amsguide.pdf"] = {["details"] = "User's guide and installation details", },
  ["bibtex/apacite/README"] = {["details"] = "Package Readme", },
  ["latex/gitinfo2/gitinfo2.pdf"] = {["details"] = "Package documentation", },
  ["latex/newverbs/newverbs.pdf"] = {["details"] = "Package documentation", },
  ["latex/makebox/makebox.pdf"] = {["details"] = "Package documentation", },
  ["latex/fixfoot/fixfoot.pdf"] = {["details"] = "Package documentation", },
  ["latex/assoccnt/assoccnt_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamertheme-upenn-bc/README"] = {["details"] = "Readme", },
  ["latex/biblatex-bwl/doc/bwl-FU.pdf"] = {["details"] = "Package documentation", },
  ["latex/crossreftools/README"] = {["details"] = "Readme", },
  ["latex/media9/media9.pdf"] = {["details"] = "Package documentation", },
  ["latex/logpap/README"] = {["details"] = "Package Readme", },
  ["latex/mychemistry/mychemistry_en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/adtrees/README"] = {["details"] = "Readme", },
  ["support/fragmaster/README"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/mafr/ALIRE"] = {["lang"] = "fr", ["details"] = "Package ALIRE", },
  ["latex/datetime2-russian/README"] = {["details"] = "Readme", },
  ["xelatex/na-box/na-box_doc.pdf"] = {["lang"] = "ar", ["details"] = "Package documentation (in Arabic)", },
  ["latex/texments/texments.pdf"] = {["details"] = "Package documentation", },
  ["latex/eqnarray/README"] = {["details"] = "Readme", },
  ["bibtex/gost/README.md"] = {["lang"] = "en", ["details"] = "Readme", },
  ["generic/pictex/README"] = {["details"] = "Readme", },
  ["bibtex/perception/README"] = {["details"] = "Readme", },
  ["latex/prettyref/prettyref.pdf"] = {["details"] = "Package documentation", },
  ["latex/lettrine/demo.pdf"] = {["lang"] = "fr", ["details"] = "Package examples (French)", },
  ["fonts/courier-scaled/README"] = {["details"] = "Readme", },
  ["latex/fmp/fmp.pdf"] = {["details"] = "Package documentation", },
  ["latex/resphilosophica/resphilosophica.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-spanish/spanish.pdf"] = {["lang"] = "es", ["details"] = "Package documentation", },
  ["bibtex/babelbib/README"] = {["details"] = "Readme", },
  ["latex/dvgloss/README"] = {["details"] = "Readme", },
  ["latex/flowchart/flowchart.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-bookinarticle/documentation/biblatex-bookinarticle.pdf"] = {["details"] = "Package documentation", },
  ["latex/pecha/pecha_docu.pdf"] = {["details"] = "Package documentation", },
  ["latex/comprehensive/README"] = {["details"] = "Package README", },
  ["latex/linegoal/linegoal.pdf"] = {["details"] = "Package documentation", },
  ["latex/catechis/catechis.pdf"] = {["details"] = "Package documentation", },
  ["latex/active-conf/active-conf.pdf"] = {["details"] = "Class documentation", },
  ["latex/docmute/README"] = {["details"] = "Readme", },
  ["latex/tikz-layers/README"] = {["details"] = "Readme", },
  ["generic/xcomment/README"] = {["details"] = "Readme", },
  ["generic/pst-bezier/README.md"] = {["details"] = "Readme", },
  ["latex/anysize/anysize.pdf"] = {["details"] = "Package documentation", },
  ["fonts/ipaex-type1/sample-ipaex-type1.pdf"] = {["lang"] = "ja", ["details"] = "Font samples", },
  ["plain/impatient-cn/cnbook.pdf"] = {["lang"] = "zh", ["details"] = "A copy of the book (Chinese translation)", },
  ["latex/bxpdfver/README.md"] = {["details"] = "Readme", },
  ["latex/l3experimental/xgalley/xgalley.pdf"] = {["details"] = "galley documentation", },
  ["generic/pst-vehicle/README.md"] = {["details"] = "Readme", },
  ["support/ltxfileinfo/README"] = {["details"] = "README", },
  ["generic/babel-spanish/README.md"] = {["details"] = "Readme", },
  ["latex/tugboat/ltubguid.pdf"] = {["lang"] = "en", ["details"] = "Instructions for authors", },
  ["latex/makeshape/makeshape.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-tree/pst-tree-doc.pdf"] = {["lang"] = "en", ["details"] = "Package manual", },
  ["latex/tabfigures/README"] = {["details"] = "Readme", },
  ["latex/engpron/engpron-ex-fr.pdf"] = {["lang"] = "fr", ["details"] = "Usage example (French)", },
  ["generic/poster-mac/README"] = {["details"] = "Readme", },
  ["latex/eso-pic/eso-pic.pdf"] = {["details"] = "Package documentation", },
  ["latex/ctib/ctib4tex.pdf"] = {["details"] = "Package documentation", },
  ["latex/coolthms/README"] = {["details"] = "Readme", },
  ["fonts/libertinegc/libertinegc-doc.pdf"] = {["details"] = "Package documentation", },
  ["metapost/mptrees/mptrees-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/fncychap/README"] = {["details"] = "Readme", },
  ["support/texdraw/README"] = {["details"] = "Readme", },
  ["latex/hepnames/heppennames-rm.pdf"] = {["details"] = "Enumeration of available pennames", },
  ["latex/pixelart/pixelart.pdf"] = {["details"] = "Package documentation", },
  ["latex/pythontex/pythontex_gallery.pdf"] = {["details"] = "Gallery of use", },
  ["latex/germkorr/germkorr.pdf"] = {["details"] = "Package documentation", },
  ["latex/numspell/numspell.pdf"] = {["details"] = "Package documentation", },
  ["latex/cool/cool.pdf"] = {["details"] = "Implementation details", },
  ["latex/formation-latex-ul/formation-latex-ul.pdf"] = {["lang"] = "fr", ["details"] = "The course itself (Manual)", },
  ["latex/makeshape/ontesting.pdf"] = {["details"] = "Package test procedure", },
  ["fonts/archaic/asamples.pdf"] = {["details"] = "Collection of font samples", },
  ["latex/changebar/changebar.pdf"] = {["details"] = "Package documentation", },
  ["latex/thucoursework/iihw.pdf"] = {["details"] = "Example of use (English)", },
  ["generic/hlist/README"] = {["details"] = "Readme", },
  ["latex/textualicomma/README.md"] = {["details"] = "Readme", },
  ["latex/12many/12many.pdf"] = {["details"] = "Package documentation", },
  ["latex/revtex4/README"] = {["details"] = "Package Readme", },
  ["latex/shuffle/README"] = {["details"] = "Readme", },
  ["latex/pstring/README.md"] = {["details"] = "Readme", },
  ["latex/hepunits/README"] = {["details"] = "Readme", },
  ["latex/cje/README.txt"] = {["details"] = "Readme", },
  ["latex/engpron/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["fonts/opensans/opensans.pdf"] = {["details"] = "Package documentation", },
  ["latex/ucdavisthesis/README"] = {["details"] = "Readme", },
  ["fonts/schulschriften/README"] = {["details"] = "Readme", },
  ["generic/babel-piedmontese/piedmontese.pdf"] = {["details"] = "Package documentation", },
  ["latex/algorithmicx/algorithmicx.pdf"] = {["details"] = "Package documentation", },
  ["platex/jsclasses/okuverb.pdf"] = {["lang"] = "ja", ["details"] = "Okuverb package documentation (Japanese)", },
  ["fonts/gfsbaskerville/README"] = {["details"] = "Readme", },
  ["latex/etdipa/Help/etdipa_doc_DE.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["xelatex/bidishadowtext/bidishadowtext-demo.pdf"] = {["details"] = "Example of use", },
  ["bibtex/gost/gost.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/substances/README"] = {["details"] = "Readme", },
  ["latex/fink/README"] = {["details"] = "Readme", },
  ["latex/datetime2-lsorbian/README"] = {["details"] = "Readme", },
  ["generic/gobble/gobble.pdf"] = {["details"] = "Package documentation", },
  ["context/third/context-notes-zh-cn/README"] = {["details"] = "Readme", },
  ["latex/genmpage/genmpage.pdf"] = {["details"] = "Package documentation", },
  ["latex/qsymbols/README"] = {["details"] = "Readme", },
  ["latex/gcite/gcite.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries-portuges/README"] = {["details"] = "Readme", },
  ["latex/l2tabu-english/l2tabuen.pdf"] = {["details"] = "The document itself", },
  ["fonts/universalis/universalis-samples.pdf"] = {["details"] = "Font samples", },
  ["generic/babel-kurmanji/kurmanji.pdf"] = {["details"] = "Package documentation", },
  ["latex/layaureo/README"] = {["details"] = "Readme and summary", },
  ["latex/nddiss/README.md"] = {["details"] = "Readme", },
  ["fonts/mintspirit/README"] = {["details"] = "Readme", },
  ["generic/vntex/vntex-man.pdf"] = {["lang"] = "vi", ["details"] = "Package manual (Vietnamese)", },
  ["latex/ESIEEcv/ESIEEcv.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamertheme-phnompenh/beamerthemePhnomPenh.pdf"] = {["details"] = "small sample of use", },
  ["latex/complexity/README"] = {["details"] = "Readme", },
  ["latex/gatherenum/gatherenum.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-finnish/datetime2-finnish.pdf"] = {["details"] = "Package documentation", },
  ["latex/epiolmec/GlyphAccessCommands.pdf"] = {["details"] = "Package documentation", },
  ["cstex/cspsfonts-gen/README"] = {["lang"] = "cs", ["details"] = "Readme", },
  ["latex/booktabs/booktabs.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/tablestyles/tablestyles.pdf"] = {["details"] = "Package documentation", },
  ["platex/pxrubrica/pxrubrica-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/nrc/00-2013-feb-authors.txt"] = {["details"] = "Notes for authors", },
  ["latex/extsizes/README"] = {["details"] = "Readme", },
  ["generic/frame/README"] = {["details"] = "Readme", },
  ["latex/flabels/flabels.pdf"] = {["details"] = "Package documentation", },
  ["latex/etdipa/Help/etdipa_doc_EN.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/epspdfconversion/epspdfconversion.pdf"] = {["details"] = "Package documentation", },
  ["context/third/cmscbf/cmscbf-demo.pdf"] = {["details"] = "Demo of use", },
  ["latex/dccpaper/README.md"] = {["details"] = "Readme", },
  ["latex/environ/environ.pdf"] = {["details"] = "Package documentation", },
  ["fonts/plex-otf/plex-otf-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-contourplot/README"] = {["details"] = "Readme", },
  ["latex/fix2col/README"] = {["details"] = "Readme", },
  ["fonts/beuron/README"] = {["details"] = "Readme", },
  ["fonts/frcursive/frcursive.pdf"] = {["details"] = "Package documentation", },
  ["latex/gaceta/plantilla-articulo-suelto.pdf"] = {["lang"] = "es", ["details"] = "Example 1: how to write an article", },
  ["fonts/greenpoint/README"] = {["details"] = "Readme", },
  ["latex/ejpecp/sample.pdf"] = {["details"] = "Introduction, as a paper", },
  ["latex/translation-array-fr/README"] = {["details"] = "Readme", },
  ["fonts/cm-lgc/INSTALL"] = {["details"] = "Installation instructions", },
  ["latex/chngcntr/chngcntr.pdf"] = {["details"] = "Package documentation", },
  ["latex/fontaxes/README"] = {["details"] = "Readme", },
  ["latex/biblatex-manuscripts-philology/README"] = {["details"] = "Readme", },
  ["support/findhyph/README"] = {["details"] = "Readme", },
  ["latex/standalone/standalone.pdf"] = {["details"] = "Package documentation", },
  ["support/pkfix-helper/encoding-samples.pdf"] = {["details"] = "Encoding tables and how to use them", },
  ["fonts/almfixed/README"] = {["details"] = "Readme", },
  ["latex/blochsphere/blochsphere.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-estonian/datetime2-estonian.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-electricfield/pst-electricfield-docFR.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/askmaps/README"] = {["details"] = "Readme", },
  ["pdftex/manual/pdftex-a.pdf"] = {["details"] = "User manual", },
  ["latex/substitutefont/README"] = {["details"] = "Readme", },
  ["latex/natbib/natnotes.pdf"] = {["lang"] = "en", ["details"] = "Reference Sheet", },
  ["latex/logical-markup-utils/README"] = {["details"] = "Readme", },
  ["context/third/handlecsv/handlecsv.pdf"] = {["details"] = "Package documentation", },
  ["metapost/splines/splines.pdf"] = {["details"] = "Package documentation", },
  ["latex/axessibility/axessibility.pdf"] = {["details"] = "Package documentation", },
  ["latex/acmart/acmart.pdf"] = {["details"] = "Documentation", },
  ["latex/psu-thesis/phdsample.pdf"] = {["details"] = "PhD thesis layout demonstration", },
  ["generic/pstricks/pst-news18.pdf"] = {["details"] = "Latest news bulletin", },
  ["latex/aiaa/author_guide.pdf"] = {["details"] = "Author guide", },
  ["generic/babel-slovak/slovak.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-hebrew/heb209.pdf"] = {["details"] = "Package documentation (LaTeX 2.09 compatibility style files)", },
  ["latex/clrscode/clrscode.pdf"] = {["details"] = "Package documentation", },
  ["fonts/fontinst/manual/intro98.pdf"] = {["details"] = "Package introductioon", },
  ["latex/jurabib/docs/english/jbendoc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/realscripts/realscripts.pdf"] = {["details"] = "Package documentation", },
  ["latex/show2e/show2e-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["generic/encxvlna/README"] = {["details"] = "Readme", },
  ["latex/koma-script-sfs/README"] = {["details"] = "Readme", },
  ["support/texdef/texdef.pdf"] = {["details"] = "Package documentation", },
  ["latex/lettrine/lettrine.pdf"] = {["details"] = "Package documentation", },
  ["latex/eulervm/README.eulervm"] = {["details"] = "Package Readme", },
  ["latex/cweb-latex/cweb-user.pdf"] = {["details"] = "User manual", },
  ["latex/chessboard/README"] = {["details"] = "Readme", },
  ["latex/fei/fei.pdf"] = {["lang"] = "pt-br", ["details"] = "Package documentation", },
  ["fonts/Type1fonts/README"] = {["details"] = "Readme", },
  ["latex/biblatex-nejm/biblatex-nejm.pdf"] = {["details"] = "Package documentation", },
  ["fonts/universalis/README"] = {["details"] = "Readme", },
  ["latex/iso10303/README"] = {["details"] = "Readme", },
  ["latex/ncctools/README"] = {["details"] = "Readme", },
  ["fonts/latex-fonts/README"] = {["details"] = "Readme", },
  ["latex/songbook/README"] = {["details"] = "Readme", },
  ["latex/gmutils/README"] = {["details"] = "README", },
  ["latex/csvsimple/csvsimple-example.pdf"] = {["lang"] = "en", ["details"] = "example of use", },
  ["latex/multibibliography/README"] = {["details"] = "Readme", },
  ["latex/tocloft/tocloft.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-dutch/README"] = {["details"] = "Readme", },
  ["latex/diagmac2/doc/diagmac2.pdf"] = {["details"] = "Package documentation", },
  ["latex/serbian-date-lat/SerbianDateLat.pdf"] = {["details"] = "Package documentation", },
  ["latex/citeall/citeall.pdf"] = {["details"] = "Package documentation", },
  ["latex/tagpdf/README.md"] = {["details"] = "Readme", },
  ["support/texloganalyser/README"] = {["details"] = "Readme", },
  ["generic/pst-grad/pst-grad-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/scratch/README"] = {["details"] = "Readme", },
  ["latex/beamer/README.md"] = {["details"] = "Readme", },
  ["latex/xhfill/xhfill-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/chemcono/chemcono.pdf"] = {["details"] = "Package documentation", },
  ["latex/modular/README.md"] = {["details"] = "Readme", },
  ["latex/examplep/eurotex_2005_examplep.pdf"] = {["details"] = "Article from EuroTeX 2005 talk", },
  ["latex/hulipsum/hulipsum.pdf"] = {["details"] = "Package documentation", },
  ["latex/notes2bib/notes2bib.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-english/datetime2-english.pdf"] = {["details"] = "Package documentation", },
  ["latex/postage/README.txt"] = {["details"] = "Readme", },
  ["generic/enctex/encdoc.pdf"] = {["lang"] = "cs", ["details"] = "Reference manual (in Czech)", },
  ["latex/mflogo/mflogo.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst2pdf/README"] = {["details"] = "Readme", },
  ["latex/combinedgraphics/README"] = {["details"] = "Readme", },
  ["latex/algobox/algobox.pdf"] = {["details"] = "Package documentation", },
  ["latex/milsymb/README.md"] = {["details"] = "Readme", },
  ["latex/exsol/exsol.pdf"] = {["details"] = "Package documentation", },
  ["fonts/kurier/00readme.eng"] = {["lang"] = "en", ["details"] = "Outline in English", },
  ["latex/esdiff/README"] = {["details"] = "Readme", },
  ["latex/iffont/README.md"] = {["details"] = "Readme", },
  ["latex/dickimaw/dickimaw-thesis.pdf"] = {["details"] = "Using LaTeX to Write a PhD Thesis", },
  ["generic/pst-diffraction/pst-diffraction-docFR.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/sasnrdisplay/README"] = {["details"] = "Readme", },
  ["latex/amscls/amsclass.pdf"] = {["details"] = "Code with documentation", },
  ["latex/parrun/parrun.pdf"] = {["details"] = "Package documentation", },
  ["latex/recipe/README"] = {["details"] = "Readme", },
  ["plain/getoptk/guide.pdf"] = {["details"] = "Package documentation", },
  ["latex/unitsdef/unitsdef.pdf"] = {["details"] = "Package documentation", },
  ["latex/eltex/man_en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/tudscr/README.md"] = {["details"] = "Readme", },
  ["latex/uni-wtal-lin/LIESMICH"] = {["lang"] = "de", ["details"] = "Readme", },
  ["latex/bangorcsthesis/README"] = {["details"] = "Readme", },
  ["latex/glossaries-dutch/README"] = {["details"] = "Readme", },
  ["latex/dramatist/dramatist.pdf"] = {["details"] = "Package documentation", },
  ["latex/cntperchap/cntperchap_example.pdf"] = {["details"] = "Example of use", },
  ["latex/munich/documenation_munich_Bibtex_style.pdf"] = {["details"] = "Package documentation", },
  ["latex/dtk/doc/beispiel.pdf"] = {["lang"] = "de", ["details"] = "Example of use", },
  ["context/third/mathsets/mathsets-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/hustthesis/hustthesis.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-rubans/pst-rubans-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/varindex/varindex.pdf"] = {["details"] = "Package documentation", },
  ["fonts/beuron/beuron-de.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/basque-book/README"] = {["details"] = "Readme", },
  ["latex/easyreview/README.txt"] = {["details"] = "readme", },
  ["fonts/libertine/libertine.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-malay/malay.pdf"] = {["details"] = "Documentation for Bahasa Malaysia version", },
  ["generic/expex/expex-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/chet/chetdoc.pdf"] = {["details"] = "Package documentation", },
  ["generic/autoaligne/README"] = {["details"] = "Readme", },
  ["latex/presentations-en/README"] = {["details"] = "Readme", },
  ["latex/glossaries-finnish/README"] = {["details"] = "Readme", },
  ["latex/first-latex-doc/first-latex-doc.pdf"] = {["details"] = "The document itself", },
  ["latex/cntperchap/README"] = {["details"] = "Readme", },
  ["latex/subfigure/README"] = {["details"] = "Readme", },
  ["platex/jsclasses/okumacro.pdf"] = {["lang"] = "ja", ["details"] = "Okumacro package documentation (Japanese)", },
  ["latex/abstract/README"] = {["details"] = "Readme", },
  ["latex/gitlog/README.md"] = {["details"] = "Readme", },
  ["latex/glossaries-portuges/glossaries-portuges.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-slovene/datetime2-slovene.pdf"] = {["details"] = "Package documentation", },
  ["latex/mla-paper/README"] = {["details"] = "Readme (with package use details)", },
  ["latex/lion-msc/README.md"] = {["details"] = "Readme", },
  ["metapost/makecirc/MakeCirc-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation in English", },
  ["latex/gcard/README"] = {["details"] = "Readme", },
  ["xelatex/fontwrap/fontwrap.pdf"] = {["details"] = "Package documentation", },
  ["latex/binomexp/README"] = {["details"] = "Readme", },
  ["latex/labbook/labboode.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/fixmetodonotes/README"] = {["details"] = "Readme", },
  ["latex/aligned-overset/README.md"] = {["details"] = "Readme", },
  ["latex/tipfr/README"] = {["details"] = "Readme", },
  ["latex/pdfpages/pdfpages.pdf"] = {["details"] = "Package documentation", },
  ["latex/versions/versions-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex2e-help-texinfo/README"] = {["details"] = "Package Readme", },
  ["latex/philex/README"] = {["details"] = "Readme", },
  ["latex/lshort-slovenian/README"] = {["details"] = "Readme", },
  ["latex/datetime2-greek/README"] = {["details"] = "Readme", },
  ["latex/pict2e/pict2e.pdf"] = {["details"] = "Package documentation", },
  ["latex/frankenstein/README"] = {["details"] = "Bundle README", },
  ["latex/pas-crosswords/pas-crosswords.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/semproc/semproc.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-serbian/README"] = {["details"] = "Readme", },
  ["latex/pdfcomment/pdfcomment_de.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["luatex/luatexja/luatexja-ja.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation (Japanese)", },
  ["latex/exam-n/sample_exam.pdf"] = {["details"] = "Sample exam", },
  ["latex/robustindex/README.md"] = {["details"] = "Readme", },
  ["latex/biblatex-fiwi/biblatex-fiwi.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/hepnames/hepnames.pdf"] = {["details"] = "Details of the package bundle", },
  ["latex/psbao/psbaomanual.pdf"] = {["details"] = "Package manual", },
  ["latex/macroswap/macroswap.pdf"] = {["details"] = "Package documentation", },
  ["latex/pseudocode/README"] = {["details"] = "Readme", },
  ["latex/dtxgallery/README"] = {["details"] = "Readme", },
  ["latex/assignment/README"] = {["details"] = "Package README", },
  ["latex/xpunctuate/README"] = {["details"] = "Readme", },
  ["latex/svg-inkscape/README"] = {["details"] = "Readme", },
  ["latex/kerkis/README.html"] = {["details"] = "Readme", },
  ["latex/hvindex/README"] = {["details"] = "Readme", },
  ["latex/dickimaw/dickimaw-minexample.pdf"] = {["details"] = "Creating a LaTeX minimal example", },
  ["latex/rubik/README.txt"] = {["details"] = "Readme", },
  ["latex/arydshln/arydshln-man.pdf"] = {["details"] = "User manual", },
  ["fonts/cm-lgc/README"] = {["details"] = "Bundle Readme", },
  ["support/latex-make/texdepends.pdf"] = {["details"] = "texdepends Package documentation", },
  ["latex/calxxxx-yyyy/cal2018-2035_DK.pdf"] = {["lang"] = "da", ["details"] = "Sample (Danish) Calendar", },
  ["support/dosepsbin/dosepsbin.txt"] = {["details"] = "Package documentation (text format)", },
  ["latex/polytable/README"] = {["details"] = "Readme", },
  ["latex/ginpenc/ginpenc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ifmtarg/ifmtarg.pdf"] = {["details"] = "Package documentation", },
  ["latex/flowfram/ffuserguide.pdf"] = {["details"] = "User guide", },
  ["latex/pdfpc-movie/pdfpc-movie-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/visualfaq/source/README"] = {["details"] = "Readme", },
  ["latex/prosper/README"] = {["details"] = "Package Readme", },
  ["lualatex/bezierplot/bezierplot-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/matc3/matc3.pdf"] = {["lang"] = "it", ["details"] = "Package documentation (in Italian)", },
  ["latex/makebarcode/makebarcode.pdf"] = {["details"] = "Package documentation", },
  ["latex/a4wide/a4wide.pdf"] = {["details"] = "Package documentation", },
  ["eplain/eplain.html"] = {["details"] = "Package documentation (HTML)", },
  ["latex/knowledge/README.md"] = {["details"] = "Readme", },
  ["latex/apxproof/README.md"] = {["details"] = "Readme", },
  ["plain/texbytopic/README"] = {["details"] = "Readme", },
  ["latex/biblatex-shortfields/documentation/biblatex-shortfields.pdf"] = {["details"] = "Package documentation", },
  ["generic/chemfig/chemfig-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/forloop/forloop.pdf"] = {["details"] = "Package documentation", },
  ["generic/variations/ALIRE"] = {["lang"] = "fr", ["details"] = "Package ALIRE", },
  ["latex/latex-veryshortguide/README.md"] = {["details"] = "Readme", },
  ["generic/elhyphen/README"] = {["details"] = "Readme", },
  ["luatex/luatexbase/luatexbase.pdf"] = {["details"] = "Package documentation", },
  ["latex/logbox/README"] = {["details"] = "Readme", },
  ["support/ctanupload/ctanupload.pdf"] = {["details"] = "Package documentation", },
  ["latex/tabvar/README"] = {["details"] = "Readme", },
  ["latex/clefval/example.pdf"] = {["details"] = "Package example of use (English)", },
  ["context/third/simpleslides/simpleslides.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/bibexport/bibexport.pdf"] = {["details"] = "Package documentation", },
  ["context/third/annotation/annotation-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/filedate/filedate.pdf"] = {["details"] = "Package documentation", },
  ["latex/caption/caption-deu.pdf"] = {["lang"] = "de", ["details"] = "German documentation", },
  ["latex/catchfilebetweentags/catchfilebetweentags.pdf"] = {["details"] = "Package documentation", },
  ["latex/classicthesis/README"] = {["details"] = "Package README", },
  ["latex/dsptricks/README"] = {["details"] = "Readme", },
  ["fonts/cinzel/README"] = {["details"] = "Readme", },
  ["latex/churchslavonic/README"] = {["details"] = "Readme", },
  ["latex/bropd/README"] = {["details"] = "Readme", },
  ["latex/europecv/examples/minimal.pdf"] = {["details"] = "Simple example of use", },
  ["latex/doipubmed/README"] = {["details"] = "Readme", },
  ["latex/diagmac2/doc/diagmactest.pdf"] = {["details"] = "Example output", },
  ["latex/datetime2-serbian/datetime2-serbian.pdf"] = {["details"] = "Package documentation", },
  ["latex/apa6e/apa6e.pdf"] = {["details"] = "Package documentation", },
  ["latex/gmverb/gmverb.pdf"] = {["details"] = "Package documentation", },
  ["latex/geschichtsfrkl/geschichtsfrkl.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/pst-intersect/README"] = {["details"] = "Readme", },
  ["fonts/cabin/samples-condensed.pdf"] = {["details"] = "Samples for condensed fonts", },
  ["latex/latex-bib2-ex/README"] = {["details"] = "Readme", },
  ["support/crossrefware/README"] = {["details"] = "Readme", },
  ["latex/tikzscale/README"] = {["details"] = "Readme", },
  ["latex/hulipsum/README"] = {["details"] = "Readme", },
  ["latex/arydshln/arydshln.pdf"] = {["details"] = "Manual, including code listing", },
  ["latex/extsizes/extsizes.pdf"] = {["details"] = "Package documentation", },
  ["latex/webguide/webguide.html"] = {["details"] = "The document, as HTML", },
  ["latex/lsc/lsc.pdf"] = {["details"] = "Package documentation", },
  ["latex/fdsymbol/fdsymbol.pdf"] = {["details"] = "Package documentation", },
  ["latex/stex/hwexam/README"] = {["details"] = "Readme", },
  ["support/latex-make/figlatex.pdf"] = {["details"] = "figlatex Package documentation", },
  ["latex/duotenzor/README"] = {["details"] = "Readme", },
  ["latex/isodate/testdate.pdf"] = {["details"] = "Test document showing styles", },
  ["latex/uothesis/uothesis.pdf"] = {["details"] = "Package documentation", },
  ["generic/ean/README"] = {["details"] = "Readme", },
  ["generic/dcpic/manDCPiCpt.pdf"] = {["lang"] = "pt", ["details"] = "Package documentation (Portuguese)", },
  ["latex/picinpar/picinpar-en.pdf"] = {["lang"] = "en", ["details"] = "Package usage notes", },
  ["latex/fundus-sueterlin/suetterl.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-indonesian/indonesian.pdf"] = {["details"] = "Documentation for Bahasa indonesia version", },
  ["latex/gender/gender.pdf"] = {["details"] = "Package documentation", },
  ["latex/dirtytalk/README"] = {["details"] = "Readme", },
  ["latex/program/README"] = {["details"] = "Readme", },
  ["latex/ESIEEcv/cvtest.pdf"] = {["details"] = "Test/sample document", },
  ["latex/nomencl/nomencl.pdf"] = {["details"] = "Package documentation", },
  ["latex/sdrt/README"] = {["details"] = "Readme", },
  ["latex/beamertheme-epyt/epyt-demo.pdf"] = {["details"] = "Package documentation", },
  ["latex/duckuments/duckuments.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-swedish/datetime2-swedish.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex-graphics-companion/README"] = {["details"] = "Readme", },
  ["fonts/accfonts/README"] = {["details"] = "Package Readme and documentation", },
  ["latex/gchords/README"] = {["details"] = "Package Readme", },
  ["support/adhocfilelist/README"] = {["details"] = "Readme", },
  ["latex/lineno/README.txt"] = {["details"] = "Readme", },
  ["latex/fibeamer/fibeamer.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cmcyr/README"] = {["details"] = "Readme", },
  ["support/ltximg/ltximg-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/chronology/example.pdf"] = {["details"] = "Package documentation", },
  ["context/third/cmscbf/cmscbf-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/addlines/addlines.pdf"] = {["details"] = "Package documentation", },
  ["generic/xlop/LISEZMOI"] = {["lang"] = "fr", ["details"] = "Readme", },
  ["latex/wrapfig/wrapfig-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/elpres/elpres-manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/psnfss/00readme.txt"] = {["details"] = "Readme", },
  ["fonts/cm-super/FAQ"] = {["details"] = "Frequently Asked Questions", },
  ["fonts/cns/cns40-6/README"] = {["details"] = "Readme", },
  ["latex/stex/example/README"] = {["details"] = "Readme", },
  ["latex/mylatexformat/mylatexformat.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/kurdishlipsum/README"] = {["details"] = "Readme", },
  ["latex/exp-testopt/exp-testopt.pdf"] = {["lang"] = "eo", ["details"] = "Package documentation (Esperanto)", },
  ["latex/lshort-mongol/lshort-mn.pdf"] = {["lang"] = "mn", ["details"] = "The document itself", },
  ["latex/dotarrow/DotArrow.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-philosophy/biblatex-philosophy.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-manuscripts-philology/documentation/biblatex-manuscripts-philology.pdf"] = {["details"] = "Package documentation", },
  ["fonts/esint-type1/README.plainTeX"] = {["details"] = "Using the fonts with Plain TeX", },
  ["fonts/fetamont/README"] = {["details"] = "Readme", },
  ["latex/numberedblock/testnumberedblock.pdf"] = {["details"] = "Demonstration of use", },
  ["fonts/dictsym/dictsym.pdf"] = {["details"] = "Package documentation", },
  ["latex/fvextra/README"] = {["details"] = "Readme", },
  ["latex/biblatex-fiwi/README"] = {["details"] = "Readme", },
  ["latex/multilang/README.md"] = {["details"] = "Readme", },
  ["latex/tikz-page/tikz-page.pdf"] = {["details"] = "Package documentation", },
  ["latex/dirtytalk/dirtytalk.pdf"] = {["details"] = "Package documentation", },
  ["latex/flippdf/flippdf.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-socialscienceshuberlin/README.md"] = {["details"] = "Readme", },
  ["generic/FAQ-en/letterfaq.pdf"] = {["details"] = "FAQ formatted for letter paper", },
  ["latex/boites/boites.pdf"] = {["details"] = "English translation", },
  ["latex/colordoc/README"] = {["details"] = "Readme", },
  ["latex/skb/user-guide.pdf"] = {["details"] = "User guide", },
  ["latex/censor/censor.pdf"] = {["details"] = "Package documentation", },
  ["latex/dviasm/README"] = {["details"] = "Readme", },
  ["latex/glossaries-polish/glossaries-polish.pdf"] = {["details"] = "Package documentation", },
  ["latex/hypernat/hypernat.pdf"] = {["details"] = "Package documentation", },
  ["latex/sduthesis/README"] = {["details"] = "Readme", },
  ["support/tex4ebook/tex4ebook-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/lpform/README"] = {["details"] = "Readme", },
  ["latex/exercise/README"] = {["details"] = "Readme", },
  ["latex/datetime2-galician/datetime2-galician.pdf"] = {["details"] = "Package documentation", },
  ["latex/thucoursework/README.md"] = {["details"] = "Readme", },
  ["generic/dowith/README"] = {["details"] = "Readme", },
  ["latex/background/README"] = {["details"] = "Readme", },
  ["xelatex/bidi/bidi.pdf"] = {["details"] = "Package source documentation", },
  ["metapost/metaobj/momanual.pdf"] = {["details"] = "Package documentation", },
  ["generic/docbytex/README"] = {["details"] = "Readme", },
  ["latex/archaeologie/archaeologie-ger.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["fonts/staves/staves.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-socialscienceshuberlin/socialscienceshuberlin.pdf"] = {["details"] = "Package documentation", },
  ["latex/tdsfrmath/LISEZMOI"] = {["lang"] = "fr", ["details"] = "Lisez-moi", },
  ["latex/latex4wp-it/README"] = {["lang"] = "it", ["details"] = "Readme", },
  ["latex/bardiag/bardiag2.pdf"] = {["details"] = "Package documentation, appendix", },
  ["latex/checklistings/README.md"] = {["details"] = "Readme", },
  ["latex/ftcap/ftcap.pdf"] = {["details"] = "Package documentation", },
  ["latex/csbulletin/csbulletin.pdf"] = {["details"] = "Package manual", },
  ["generic/es-tex-faq/FAQ-CervanTeX.html"] = {["lang"] = "es", ["details"] = "The document itself (HTML)", },
  ["latex/mparhack/README"] = {["details"] = "Package README", },
  ["latex/basicarith/README"] = {["details"] = "Readme", },
  ["latex/fixcmex/fixcmex.pdf"] = {["details"] = "Package documentation", },
  ["latex/edmargin/README"] = {["details"] = "Readme", },
  ["latex/typehtml/README"] = {["details"] = "Readme", },
  ["fonts/mintspirit/mintspirit-samples.pdf"] = {["details"] = "MintSpirit samples", },
  ["latex/fancyhandout/fancyhandout-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/metafont-beginners/metafont-for-beginners.pdf"] = {["lang"] = "en", ["details"] = "The introductory document itself:", },
  ["generic/pst-plot/pst-plot-doc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["fonts/zhmetrics-uptex/README.md"] = {["details"] = "Readme", },
  ["latex/cmtiup/cmtiup.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-3d/README"] = {["details"] = "Readme", },
  ["xelatex/fixlatvian/README"] = {["details"] = "Readme", },
  ["latex/kdgdocs/kdgmasterthesis-example.pdf"] = {["details"] = "Master thesis example", },
  ["generic/multiexpand/README.md"] = {["details"] = "Readme", },
  ["latex/checklistings/checklistings.pdf"] = {["details"] = "Package documentation", },
  ["metapost/blockdraw_mp/blockdraw_mp.pdf"] = {["details"] = "Outline package documentation", },
  ["latex/constants/README"] = {["details"] = "Readme", },
  ["support/dosepsbin/dosepsbin.pdf"] = {["details"] = "Package documentation (PDF format)", },
  ["generic/pst-dbicons/README"] = {["details"] = "Package README", },
  ["generic/babel-french/frenchb.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cm-unicode/cmunti.pdf"] = {["details"] = "Italic font tables", },
  ["latex/articleingud/articleingud.pdf"] = {["lang"] = "es", ["details"] = "Package documentation (Spanish)", },
  ["fonts/cm-unicode/cmunrm.pdf"] = {["details"] = "Upright font tables", },
  ["latex/pdfcomment/README.md"] = {["details"] = "Readme", },
  ["latex/biblatex/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/easyfig/easyfig.pdf"] = {["details"] = "Package documentation", },
  ["fonts/electrum/electrumadf.pdf"] = {["details"] = "Documentation: installation and use", },
  ["latex/indextools/indextools.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-hebrew/hebrew.pdf"] = {["details"] = "Package documentation", },
  ["latex/polynom/polydemo.pdf"] = {["details"] = "Example of package use", },
  ["latex/philosophersimprint/README"] = {["details"] = "Readme", },
  ["support/asymptote-by-example-zh-cn/asymptote-by-example-zh-cn.pdf"] = {["lang"] = "zh", ["details"] = "The document itself", },
  ["context/third/rst/manual.pdf"] = {["details"] = "Manual", },
  ["latex/biblatex-phys/biblatex-phys.pdf"] = {["details"] = "Package documentation", },
  ["latex/graphics/README.md"] = {["details"] = "Readme", },
  ["bibtex/inlinebib/inlinebib.htm"] = {["details"] = "Package documentation", },
  ["latex/ccaption/README"] = {["details"] = "Readme", },
  ["latex/placeins/placeins-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/Type1fonts/fontinstallationguide.pdf"] = {["details"] = "The document itself", },
  ["metapost/textpath/README"] = {["details"] = "README file", },
  ["latex/moodle/moodle.pdf"] = {["details"] = "Package documentation", },
  ["latex/ascelike/README"] = {["details"] = "Readme", },
  ["latex/ted/ted-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/amsmath/diffs-m.txt"] = {["lang"] = "en", ["details"] = "Differences from previous versions", },
  ["generic/babel-latvian/README.md"] = {["details"] = "Readme", },
  ["latex/parselines/README"] = {["details"] = "Readme", },
  ["generic/babel-galician/galician.pdf"] = {["details"] = "Package documentation", },
  ["latex/smartref/README"] = {["details"] = "Readme and command details", },
  ["latex/optidef/README.md"] = {["details"] = "Readme", },
  ["latex/plantslabels/doc/pdf/plantslabels.pdf"] = {["details"] = "Package documentation", },
  ["latex/theatre/README"] = {["details"] = "Readme", },
  ["latex/cookingsymbols/README"] = {["details"] = "Readme", },
  ["latex/tocloft/README"] = {["details"] = "Readme", },
  ["fonts/plex-otf/README.md"] = {["details"] = "Readme", },
  ["latex/SIstyle/SIstyle-2.3a.pdf"] = {["details"] = "Package documentation", },
  ["latex/pict2e/p2e-drivers.pdf"] = {["details"] = "Documentation of graphics drivers", },
  ["latex/delim/README"] = {["details"] = "Readme", },
  ["latex/forest/forest-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-estonian/README"] = {["details"] = "Readme", },
  ["latex/ltabptch/README"] = {["details"] = "Readme", },
  ["latex/genealogytree/genealogytree-example-2.pdf"] = {["details"] = "Example 2", },
  ["latex/duotenzor/duotenzormanual.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cbfonts/README"] = {["details"] = "Readme", },
  ["fonts/gfsneohellenic/README"] = {["details"] = "Readme", },
  ["bibtex/dinat/dinat-index.html"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/lshort-korean/lshort-kr.pdf"] = {["lang"] = "ko", ["details"] = "The document itself", },
  ["latex/cheatsheet/README.md"] = {["details"] = "Readme", },
  ["latex/datetime2-latin/README"] = {["details"] = "Readme", },
  ["generic/xint/CHANGES.pdf"] = {["details"] = "Changes (PDF)", },
  ["latex/math-e/README"] = {["details"] = "Readme", },
  ["generic/pst-fun/README"] = {["details"] = "Readme", },
  ["latex/delimseasy/delimseasyMan.pdf"] = {["details"] = "Package documentation", },
  ["fonts/feyn/exercise-font.pdf"] = {["details"] = "Font details", },
  ["latex/bigfoot/README"] = {["details"] = "Readme", },
  ["latex/bropd/bropd.pdf"] = {["details"] = "Package documentation", },
  ["latex/phonrule/phonrule-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/toptesi/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/changelayout/README"] = {["details"] = "Readme", },
  ["latex/biblatex-true-citepages-omit/biblatex-true-citepages-omit.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-bahasai/README"] = {["details"] = "Readme", },
  ["latex/cweb-latex/src/README"] = {["details"] = "Readme", },
  ["latex/qrcode/README"] = {["details"] = "Readme", },
  ["latex/auto-pst-pdf-lua/README"] = {["details"] = "Readme", },
  ["latex/logpap/example.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-morenames/documentation/example-maineditor.pdf"] = {["details"] = "Example of use", },
  ["latex/subsupscripts/SubSupScripts.pdf"] = {["details"] = "Package documentation", },
  ["latex/lkproof/lkproof-doc.pdf"] = {["details"] = "Summary documentation", },
  ["latex/axodraw2/README"] = {["details"] = "Readme", },
  ["latex/cellspace/README"] = {["details"] = "Readme", },
  ["fonts/linearA/linearA_glyphs.pdf"] = {["details"] = "Font samples", },
  ["fonts/gfsneohellenicmath/README"] = {["details"] = "Readme", },
  ["latex/beamer2thesis/beamer2thesis_ita.pdf"] = {["lang"] = "it", ["details"] = "Package documentation (as presentation; in Italian)", },
  ["support/latexfileversion/README"] = {["details"] = "Readme", },
  ["latex/paralist/README"] = {["details"] = "Readme", },
  ["latex/showcharinbox/showcharinbox.pdf"] = {["details"] = "Package documentation", },
  ["latex/ccfonts/readme"] = {["details"] = "Readme", },
  ["context/third/simplefonts/README"] = {["details"] = "Readme", },
  ["latex/onedown/examples/onedown-ref.pdf"] = {["details"] = "Reference Guide", },
  ["xetex/xesearch/xesearch.pdf"] = {["details"] = "Package documentation", },
  ["latex/tabu/tabu.pdf"] = {["details"] = "Package documentation", },
  ["latex/graphics-cfg/README.md"] = {["details"] = "Readme", },
  ["latex/biblatex-multiple-dm/biblatex-multiple-dm.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-enc/README"] = {["details"] = "Readme", },
  ["latex/cnltx/cnltx_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/hepunits/hepunits.pdf"] = {["details"] = "Package documentation", },
  ["context/third/degrade/degrade-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/colorweb/README"] = {["details"] = "Readme", },
  ["latex/archaeologie/README.md"] = {["details"] = "Readme", },
  ["latex/pkuthss/readme.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["latex/papermas/papermas.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikzmarmots/tikzmarmots-doc.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/bezierplot/README"] = {["details"] = "Readme", },
  ["latex/aligned-overset/aligned-overset.pdf"] = {["details"] = "Package documentation", },
  ["latex/xargs/xargs-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/gnuplottex/gnuplottex.pdf"] = {["details"] = "Package documentation", },
  ["latex/ribbonproofs/README"] = {["details"] = "Readme", },
  ["generic/babel-sorbian/usorbian.pdf"] = {["details"] = "Package documentation (Upper Sorbian)", },
  ["latex/isomath/isomath.sty.html"] = {["details"] = "Package source (in HTML format)", },
  ["lualatex/arabluatex/samples/al-Jahiz.pdf"] = {["details"] = "Example of use", },
  ["lualatex/luahyphenrules/README.md"] = {["details"] = "Readme", },
  ["latex/edfnotes/edfnotes.pdf"] = {["details"] = "Package documentation", },
  ["latex/sides/sides-sample.pdf"] = {["details"] = "A (distressing) sample play", },
  ["latex/ebook/README"] = {["details"] = "Readme", },
  ["latex/paper/paper.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["metapost/mpman-ru/mpman-ru.pdf"] = {["details"] = "The document itself", },
  ["latex/geometry/geometry.pdf"] = {["details"] = "Users' manual", },
  ["latex/crop/crop.pdf"] = {["details"] = "Package documentation", },
  ["latex/cals/README"] = {["details"] = "Readme", },
  ["latex/babel/babel.pdf"] = {["details"] = "Package documentation", },
  ["latex/powerdot/powerdotDE.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/datetime2-romanian/datetime2-romanian.pdf"] = {["details"] = "Package documentation", },
  ["latex/ascmac/ascmac.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation (Japanese)", },
  ["latex/pst-pdf/pst-pdf.pdf"] = {["lang"] = "en", ["details"] = "English documentation", },
  ["fonts/amsfonts/amssymb.pdf"] = {["details"] = "Details of the fonts msam and msbm", },
  ["latex/cals/cals.pdf"] = {["details"] = "Package documentation", },
  ["latex/lshort-thai/readme"] = {["details"] = "Readme", },
  ["latex/authorarchive/README.md"] = {["details"] = "Readme", },
  ["latex/tcldoc/tclldoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/concepts/concepts.pdf"] = {["details"] = "Package documentation", },
  ["latex/covington/README"] = {["details"] = "Readme", },
  ["latex/biblatex-sbl/biblatex-sbl.pdf"] = {["details"] = "Package documentation", },
  ["latex/bez123/bez123.pdf"] = {["details"] = "Package documentation", },
  ["latex/fetchcls/README.md"] = {["details"] = "Readme", },
  ["latex/parnotes/README.md"] = {["details"] = "Readme", },
  ["fonts/dad/dad-user-guide.pdf"] = {["details"] = "Package documentation", },
  ["generic/bitelist/bitelist.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-irish/README"] = {["details"] = "Readme", },
  ["latex/carbohydrates/carbohydrates_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/bookcover/bookcover-example2.pdf"] = {["details"] = "Examples of use 2", },
  ["latex/l3kernel/README.md"] = {["details"] = "Readme", },
  ["latex/gotoh/README.md"] = {["details"] = "Readme", },
  ["latex/estcpmm/estcpmm.pdf"] = {["details"] = "Package documentation", },
  ["latex/statex2/statex2-example.pdf"] = {["details"] = "Package documentation (by example)", },
  ["latex/dtxtut/README"] = {["details"] = "Readme", },
  ["latex/edfnotes/README"] = {["details"] = "Readme", },
  ["luatex/lualibs/lualibs.pdf"] = {["details"] = "Package documentation", },
  ["latex/ocgx/demo-ocgx.pdf"] = {["details"] = "Package demo", },
  ["lualatex/combofont/README.md"] = {["details"] = "Readme", },
  ["latex/ellipse/ellipse.pdf"] = {["details"] = "Package documentation", },
  ["latex/boites/README"] = {["details"] = "Readme", },
  ["support/texdraw/texdraw.ps"] = {["details"] = "Package documentation", },
  ["latex/celtic/celtic.pdf"] = {["details"] = "Package documentation", },
  ["latex/thucoursework/ithw.pdf"] = {["lang"] = "zh", ["details"] = "Example of use (Chinese)", },
  ["latex/fouridx/fouridx.pdf"] = {["details"] = "Package documentation", },
  ["latex/upmethodology/upmethodology-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/factura/factura-ejemplo-prefactura.pdf"] = {["details"] = "Compiled sample", },
  ["latex/eqell/eqell.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-nottsclassic/README.md"] = {["details"] = "Readme", },
  ["latex/bxbase/README.md"] = {["details"] = "Readme", },
  ["latex/gmverb/README"] = {["details"] = "README", },
  ["latex/fullwidth/fullwidth.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-dw/de-biblatex-dw.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["generic/babel-friulan/README.txt"] = {["details"] = "Readme", },
  ["generic/termmenu/termmenu.pdf"] = {["details"] = "Package documentation", },
  ["latex/cdpbundl/cdpbundl-doc.pdf"] = {["details"] = "Package overview", },
  ["latex/microtype/README.md"] = {["details"] = "Package README", },
  ["latex/simpler-wick/README.md"] = {["details"] = "Readme", },
  ["generic/pstricks-add/README"] = {["details"] = "Readme", },
  ["fonts/sansmathaccent/README"] = {["details"] = "Readme", },
  ["latex/covington/covington.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["generic/ulem/ulem.pdf"] = {["details"] = "Package documentation", },
  ["latex/menukeys/menukeys.pdf"] = {["details"] = "Package documentation", },
  ["latex/issuulinks/README"] = {["details"] = "Readme", },
  ["bibtex/gbt7714/gbt7714.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["generic/tds/tds.html"] = {["details"] = "The standard, converted to HTML", },
  ["generic/arrayjobx/arrayjob.pdf"] = {["details"] = "Package documentation", },
  ["latex/afparticle/README"] = {["details"] = "Readme", },
  ["latex/nwejm/README.md"] = {["details"] = "Readme.md", },
  ["latex/srbook-mem/README"] = {["details"] = "Readme", },
  ["latex/filecontentsdef/filecontentsdef.pdf"] = {["details"] = "Package documentation", },
  ["latex/uafthesis/README.md"] = {["details"] = "Readme", },
  ["latex/mathtools/README"] = {["details"] = "Package README", },
  ["latex/multidef/README"] = {["details"] = "Readme", },
  ["latex/stex/smultiling/README"] = {["details"] = "Readme", },
  ["latex/ghsystem/ghsystem_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/poetry/poetry.pdf"] = {["details"] = "Package documentation", },
  ["latex/cjk/README"] = {["details"] = "Readme file", },
  ["latex/sitem/sitem.pdf"] = {["details"] = "Package documentation", },
  ["latex/multilang/multilang.pdf"] = {["details"] = "Package documentation", },
  ["latex/serbian-apostrophe/apostrophe-list.pdf"] = {["details"] = "List of word commands and their expansion", },
  ["latex/bibleref-mouth/README"] = {["details"] = "Readme", },
  ["latex/nicetext/fifinddo.pdf"] = {["details"] = "Fifinddo documentation", },
  ["latex/uspatent/PatentApplication.pdf"] = {["details"] = "Package 'dummy' sample", },
  ["generic/ukrhyph/rules90.pdf"] = {["lang"] = "uk", ["details"] = "New hyphenation rules", },
  ["latex/csquotes-de/csquotes-DE.pdf"] = {["details"] = "The document itself", },
  ["latex/isorot/rotman.pdf"] = {["details"] = "Package documentation", },
  ["latex/noindentafter/README"] = {["details"] = "Readme", },
  ["latex/bondgraphs/README"] = {["details"] = "Readme", },
  ["latex/modiagram/README"] = {["details"] = "Readme", },
  ["latex/arabi/user_guide.pdf"] = {["details"] = "Package documentation", },
  ["latex/ifnextok/README"] = {["details"] = "Readme", },
  ["latex/ecobiblatex/ecobiblatex.pdf"] = {["details"] = "Package documentation", },
  ["latex/scsnowman/README.md"] = {["details"] = "Readme", },
  ["latex/blox/README"] = {["details"] = "Readme", },
  ["latex/outline/README"] = {["details"] = "Readme", },
  ["latex/newlfm/newlfm.pdf"] = {["details"] = "Package documentation", },
  ["latex/colordoc/colordoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/lshort-chinese/README.md"] = {["details"] = "Readme", },
  ["latex/dateiliste/dateiliste.pdf"] = {["lang"] = "de", ["details"] = "Benutzerdoku", },
  ["latex/gauss/README"] = {["details"] = "Readme", },
  ["latex/binarytree/binarytree.pdf"] = {["details"] = "Package documentation", },
  ["latex/bredzenie/README"] = {["details"] = "Readme", },
  ["fonts/algolrevived/README"] = {["details"] = "Readme", },
  ["latex/germkorr/README"] = {["details"] = "Readme", },
  ["latex/engtlc/README"] = {["details"] = "Readme", },
  ["latex/yfonts/readme"] = {["details"] = "Readme", },
  ["latex/getmap/getmap.pdf"] = {["details"] = "Package documentation", },
  ["support/asymptote-by-example-zh-cn/README"] = {["details"] = "Readme", },
  ["latex/hhtensor/README"] = {["details"] = "Package Readme", },
  ["latex/tugboat/tugboat.pdf"] = {["lang"] = "en", ["details"] = "Program documentation", },
  ["generic/fenixpar/README"] = {["details"] = "Readme", },
  ["fonts/comfortaa/comfortaa-samples.pdf"] = {["details"] = "Font samples", },
  ["latex/apa/apaenum.txt"] = {["details"] = "Outline of list facilities", },
  ["latex/hyperref/README"] = {["details"] = "Readme", },
  ["latex/calctab/calctab_manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-bulgarian/datetime2-bulgarian.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-math/pst-math-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-english/datetime2-english-sample.pdf"] = {["details"] = "Sample article", },
  ["latex/iso/isoman.pdf"] = {["details"] = "Package manual", },
  ["generic/formlett/user_manual.pdf"] = {["details"] = "User manual", },
  ["latex/texshade/README"] = {["details"] = "Readme", },
  ["latex/lshort-persian/lshort.pdf"] = {["lang"] = "fa", ["details"] = "The document itself", },
  ["latex/multienv/README"] = {["details"] = "Readme", },
  ["context/third/gnuplot/gnuplot-context-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/bibleref-mouth/bibleref-mouth.pdf"] = {["details"] = "Package documentation", },
  ["latex/citeref/README.txt"] = {["details"] = "Readme", },
  ["latex/churchslavonic/churchslavonic-ru.pdf"] = {["lang"] = "ru", ["details"] = "Package documentation", },
  ["fonts/pxfonts/pxfontsdocA4.pdf"] = {["details"] = "Package documentation (A4 paper format", },
  ["latex/codepage/frintro.pdf"] = {["lang"] = "fr", ["details"] = "Introduction to package", },
  ["latex/svn/README"] = {["details"] = "Package README", },
  ["latex/bxcjkjatype/README.md"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/erw-l3/erw-l3.pdf"] = {["details"] = "Package documentation", },
  ["latex/ifetex/README"] = {["details"] = "Readme", },
  ["latex/datetime2-portuges/datetime2-portuges.pdf"] = {["details"] = "Package documentation", },
  ["uplatex/base/README.md"] = {["details"] = "Readme", },
  ["latex/nowidow/nowidow.pdf"] = {["details"] = "Package documentation", },
  ["latex/index/README"] = {["details"] = "Package Readme", },
  ["latex/xcolor-solarized/xcolor-solarized.pdf"] = {["details"] = "Package documentation", },
  ["latex/boldtensors/boldtensors.pdf"] = {["details"] = "Package documentation", },
  ["latex/bloques/example.pdf"] = {["details"] = "Command listing and examples", },
  ["latex/seuthesix/README"] = {["details"] = "Readme", },
  ["latex/fnbreak/fnbreak.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-calculate/README.md"] = {["details"] = "Readme", },
  ["fonts/punknova/documentation/sample.pdf"] = {["details"] = "Text sample", },
  ["latex/cntformats/README"] = {["details"] = "Readme", },
  ["latex/recipebook/LongSample.pdf"] = {["details"] = "Example (1)", },
  ["generic/pst-contourplot/examples/README"] = {["details"] = "Readme", },
  ["latex/doclicense/doclicense.pdf"] = {["details"] = "Package documentation", },
  ["latex/drawstack/README"] = {["details"] = "Readme", },
  ["latex/engpron/engpron-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/incgraph/incgraph.pdf"] = {["details"] = "Package documentation", },
  ["latex/numericplots/README"] = {["details"] = "Readme", },
  ["fonts/musixtex-fonts/musixtex-fonts-install.pdf"] = {["details"] = "Installation details", },
  ["latex/cmpj/template.pdf"] = {["details"] = "Template and instructions to authors", },
  ["latex/aomart/README"] = {["details"] = "Readme", },
  ["latex/feynmp-auto/feynmp-auto.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex2e-help-texinfo-fr/latex2e-fr.pdf"] = {["lang"] = "fr", ["details"] = "PDF conversion of the source (French)", },
  ["latex/handout/handout.pdf"] = {["details"] = "Package documentation", },
  ["fonts/belleek/README"] = {["details"] = "Readme", },
  ["generic/latex-notes-zh-cn/README"] = {["details"] = "Readme", },
  ["metapost/featpost/doc/featpostmanual.pdf"] = {["details"] = "Manual", },
  ["latex/childdoc/childdoc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/prodint/prodint.pdf"] = {["details"] = "Demonstration of use", },
  ["latex/biblatex-arthistory-bonn/arthistory-bonn.pdf"] = {["details"] = "Package documentation", },
  ["latex/examdesign/README"] = {["details"] = "Package README", },
  ["latex/currency/README.md"] = {["details"] = "Readme", },
  ["latex/doctools/doctools.pdf"] = {["details"] = "Package documentation", },
  ["latex/galois/galois.pdf"] = {["details"] = "Package description", },
  ["latex/biblatex-musuos/README"] = {["details"] = "Readme", },
  ["latex/dijkstra/README"] = {["details"] = "Readme", },
  ["latex/dtxtut/dtxtut.pdf"] = {["details"] = "Package documentation", },
  ["latex/ifnextok/ifnextok.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-welsh/README"] = {["details"] = "Readme", },
  ["latex/mafr/docmafr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/biblatex-mla/README"] = {["details"] = "Readme", },
  ["latex/phonenumbers/phonenumbers-de.pdf"] = {["lang"] = "de", ["details"] = "German package documentation", },
  ["fonts/electrum/README"] = {["details"] = "Readme", },
  ["latex/bpchem/bpchem.pdf"] = {["details"] = "Package documentation", },
  ["latex/progressbar/README"] = {["details"] = "Readme", },
  ["latex/fgruler/README"] = {["details"] = "Readme", },
  ["latex/listofsymbols/listofsymbols.pdf"] = {["details"] = "Package documentation", },
  ["latex/minipage-marginpar/minipage-marginpar.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/historische-zeitschrift/LIESMICH"] = {["lang"] = "de", ["details"] = "Readme", },
  ["fonts/overlock/overlock-samples.pdf"] = {["details"] = "Package documentation", },
  ["latex/polyglossia/polyglossia.pdf"] = {["details"] = "Package documentation", },
  ["latex/qstest/qstest.pdf"] = {["details"] = "qstest Package documentation", },
  ["latex/everypage/everypage.pdf"] = {["details"] = "Package documentation", },
  ["latex/spotcolor/readme.pdf"] = {["details"] = "Package documentation", },
  ["latex/cmdstring/README"] = {["details"] = "Readme", },
  ["latex/overlays/sample.pdf"] = {["details"] = "Example of use", },
  ["latex/pagecont/README"] = {["details"] = "Readme", },
  ["latex/nath/nathguide.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/lualatex-truncate/README.md"] = {["details"] = "Readme", },
  ["latex/arabtex/arabtex.htm"] = {["details"] = "Outline of the project", },
  ["latex/hepnames/hepnicenames-rm.pdf"] = {["details"] = "Enumeration of available nicenames", },
  ["latex/forloop/README"] = {["details"] = "Package README", },
  ["latex/gentombow/bounddvi-en.pdf"] = {["details"] = "Documentation of the bounddvi package (English)", },
  ["latex/mfirstuc/mfirstuc-manual.html"] = {["details"] = "Package documentation (HTML)", },
  ["latex/gillcm/README"] = {["details"] = "Readme", },
  ["latex/hausarbeit-jura/README.md"] = {["details"] = "Readme", },
  ["fonts/anonymouspro/README"] = {["details"] = "Readme", },
  ["latex/cachepic/README"] = {["details"] = "Readme", },
  ["latex/algobox/README.md"] = {["details"] = "Readme", },
  ["generic/chronosys/README"] = {["details"] = "Readme", },
  ["latex/exercisebank/README.txt"] = {["details"] = "Readme", },
  ["fonts/iwona/00readme.pol"] = {["details"] = "Outline in Polish:", },
  ["latex/examplep/README"] = {["details"] = "Readme", },
  ["latex/braids/braids_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/monofill/monofill.pdf"] = {["details"] = "Package documentation", },
  ["fonts/opensans/README"] = {["details"] = "Readme", },
  ["latex/mattens/mattens.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/luaindex/luaindex.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-oxref/oxnum-doc.pdf"] = {["details"] = "Style documentation for oxnum", },
  ["latex/bookcover/bookcover.pdf"] = {["details"] = "Package documentation", },
  ["latex/l2tabu-spanish/README"] = {["details"] = "Readme", },
  ["lualatex/luabibentry/luabibentry.pdf"] = {["details"] = "Package documentation", },
  ["fonts/feyn/overheads.pdf"] = {["details"] = "Brief sample", },
  ["fonts/cns/cns40-4/README"] = {["details"] = "Readme", },
  ["latex/fix2col/fix2col.pdf"] = {["details"] = "Package documentation", },
  ["latex/crbox/crbox-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/dingbat/README"] = {["details"] = "Readme", },
  ["latex/lshort-mongol/README"] = {["details"] = "Readme", },
  ["latex/ltxindex/ltxindex.pdf"] = {["details"] = "Package documentation", },
  ["plain/plipsum/plipsum-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/drm/small_specimen.pdf"] = {["details"] = "Font samples (1)", },
  ["fonts/sanskrit-t1/README"] = {["details"] = "Readme", },
  ["latex/g-brief/g-brief.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["fonts/cabin/samples.pdf"] = {["details"] = "Font samples", },
  ["latex/sesstime/README.txt"] = {["details"] = "Readme", },
  ["latex/newspaper/newspaper.pdf"] = {["details"] = "Package documentation", },
  ["generic/simplekv/README"] = {["details"] = "Readme", },
  ["latex/scsnowman/scsnowman.pdf"] = {["details"] = "Package documentation", },
  ["latex/natded/natded.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-norsk/README"] = {["details"] = "Readme", },
  ["generic/components-of-TeX/README"] = {["details"] = "Package README", },
  ["latex/tdclock/tdclock-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/l3experimental/README.md"] = {["details"] = "Readme", },
  ["latex/nonumonpart/nonumonpart-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/statmath/README"] = {["details"] = "Readme", },
  ["latex/visualfaq/visualFAQ.pdf"] = {["details"] = "The visual FAQ itself", },
  ["latex/miniplot/miniplot.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/fontwrap/README"] = {["details"] = "Readme", },
  ["latex/mlist/mlist.pdf"] = {["details"] = "Package documentation", },
  ["latex/numprint/numprint.pdf"] = {["details"] = "Package documentation", },
  ["latex/lshort-finnish/lyhyt2e.pdf"] = {["lang"] = "fi", ["details"] = "The document itself", },
  ["fonts/ae/README"] = {["details"] = "Readme", },
  ["generic/ukrhyph/README"] = {["details"] = "Readme", },
  ["latex/nwejm/french/documentation/nwejm-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/koma-moderncvclassic/cvbasic.pdf"] = {["details"] = "Example output", },
  ["latex/documentation/documentation.pdf"] = {["details"] = "Package documentation", },
  ["metapost/expressg/expressg.pdf"] = {["details"] = "Package documentation", },
  ["latex/mathalfa/README"] = {["details"] = "Readme", },
  ["latex/sr-vorl/README"] = {["details"] = "Readme", },
  ["latex/cjk-ko/README"] = {["details"] = "Readme", },
  ["uplatex/base/uplatex.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation", },
  ["latex/gotoh/gotoh.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-soroban/README"] = {["details"] = "Readme", },
  ["fonts/fouriernc/test_fouriernc.pdf"] = {["details"] = "FourierNC test example:", },
  ["fonts/lh/README"] = {["details"] = "Readme", },
  ["latex/widetable/widetable.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-chem/README.md"] = {["details"] = "Readme", },
  ["latex/gsemthesis/README"] = {["details"] = "Readme", },
  ["latex/nucleardata/nucleardata.pdf"] = {["details"] = "Package documentation", },
  ["latex/realscripts/README.md"] = {["details"] = "Readme", },
  ["latex/msg/msg.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-3dplot/README"] = {["details"] = "Readme", },
  ["latex/HA-prosper/README"] = {["details"] = "Package README", },
  ["fonts/linearA/README"] = {["details"] = "Readme", },
  ["support/pdfxup/README"] = {["details"] = "Readme", },
  ["latex/numnameru/numnameru.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-spectra/README"] = {["details"] = "Package README", },
  ["latex/randbild/randbild.pdf"] = {["lang"] = "de", ["details"] = "Benutzerdoku", },
  ["generic/pst-cox/pst-coxeterp/pst-coxeterp_doc.pdf"] = {["lang"] = "en", ["details"] = "Documentation for pst-coxeterp", },
  ["latex/quotchap/quotchap.pdf"] = {["details"] = "Package documentation", },
  ["latex/pxgreeks/README"] = {["details"] = "Readme", },
  ["support/latexmk/latexmk.txt"] = {["details"] = "Manual page, text format", },
  ["latex/maybemath/README"] = {["details"] = "Package README", },
  ["latex/showlabels/README"] = {["details"] = "Readme", },
  ["bibtex/base/btxhak.pdf"] = {["details"] = "Writing BibTeX styles", },
  ["latex/fncychap/fncychap.pdf"] = {["details"] = "Package documentation, with examples", },
  ["support/match_parens/README"] = {["details"] = "Readme", },
  ["latex/hyperref/manual.pdf"] = {["lang"] = "en", ["details"] = "Manual, PDF version", },
  ["latex/tikzcodeblocks/README.md"] = {["details"] = "Readme", },
  ["latex/bophook/bophook.pdf"] = {["details"] = "Package documentation", },
  ["latex/jurarsp/jurarsp.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["support/ltximg/README.md"] = {["details"] = "Readme", },
  ["latex/glossaries/glossaries-user.pdf"] = {["details"] = "User manual (PDF)", },
  ["latex/spot/spot.pdf"] = {["details"] = "Package documentation", },
  ["latex/cesenaexam/README.md"] = {["details"] = "Readme", },
  ["latex/padcount/README"] = {["details"] = "Readme", },
  ["latex/milsymb/milsymb.pdf"] = {["details"] = "Package documentation", },
  ["latex/abntex2/abntex2cite-alf.pdf"] = {["lang"] = "pt-br", ["details"] = "Citation support package documentation (supplementation)", },
  ["latex/bodegraph/README"] = {["details"] = "Readme (bilingual)", },
  ["plain/getoptk/README"] = {["details"] = "Readme", },
  ["platex/pxrubrica/README.md"] = {["lang"] = "en", ["details"] = "Readme", },
  ["xetex/xetexko/README"] = {["details"] = "Readme", },
  ["latex/tikz-ladder/tikz-ladder-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/ofs/eurotex2003-ofs.pdf"] = {["lang"] = "en", ["details"] = "EuroTeX paper", },
  ["latex/background/background.pdf"] = {["details"] = "Package documentation", },
  ["latex/lshort-spanish/lshort-a4.pdf"] = {["lang"] = "es", ["details"] = "The document itself (A4 paper)", },
  ["latex/cuisine/README"] = {["details"] = "Package Readme", },
  ["latex/polyglossia/README"] = {["details"] = "Readme", },
  ["generic/pst-3dplot/README"] = {["details"] = "Readme", },
  ["latex/rviewport/README"] = {["details"] = "Readme", },
  ["fonts/ocr-b/README"] = {["details"] = "Readme", },
  ["fonts/old-arrows/old-arrows.pdf"] = {["details"] = "Package documentation", },
  ["latex/lewis/lewis.pdf"] = {["details"] = "Package documentation", },
  ["latex/longfigure/README"] = {["details"] = "Readme", },
  ["latex/figsize/README"] = {["details"] = "Package Readme", },
  ["latex/imac/imac.pdf"] = {["details"] = "Package documentation", },
  ["latex/oscola/README.md"] = {["details"] = "Readme", },
  ["latex/hyper/hyper.pdf"] = {["details"] = "Package documentation", },
  ["latex/l3kernel/l3syntax-changes.pdf"] = {["details"] = "Syntax changes", },
  ["latex/jvlisting/jvlisting.pdf"] = {["details"] = "Package documentation", },
  ["fonts/missaali/README"] = {["details"] = "Readme", },
  ["latex/onedown/examples/onedown-examples.pdf"] = {["details"] = "Examples of use", },
  ["latex/datetime2-breton/README"] = {["details"] = "Readme", },
  ["context/third/typearea/typearea-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/stmaryrd/README"] = {["details"] = "Original README:", },
  ["latex/exp-testopt/README"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/gastex/README"] = {["details"] = "Package Readme", },
  ["latex/bohr/README"] = {["details"] = "Readme", },
  ["generic/catcodes/README"] = {["details"] = "Readme", },
  ["latex/robustindex/robustmanual.pdf"] = {["details"] = "Package documentation", },
  ["latex/smartref/smartref-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/tkz-orm/README"] = {["details"] = "Readme", },
  ["latex/endfloat/README"] = {["details"] = "Readme", },
  ["latex/easy/README"] = {["details"] = "Readme", },
  ["latex/jamtimes/jamtimes.pdf"] = {["details"] = "Package documentation", },
  ["latex/pinlabel/pinlabdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/titlecaps/README"] = {["details"] = "Readme", },
  ["latex/bytefield/README"] = {["details"] = "Readme", },
  ["latex/circ/README"] = {["details"] = "Readme", },
  ["latex/pb-diagram/pb-manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-greek/datetime2-greek.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-fr3d/README"] = {["details"] = "Readme", },
  ["latex/xytree/README"] = {["details"] = "Package README", },
  ["lualatex/luaindex/README"] = {["details"] = "Readme", },
  ["latex/simpler-wick/simpler-wick.pdf"] = {["details"] = "Package documentation", },
  ["latex/index/index.pdf"] = {["details"] = "Package documentation", },
  ["latex/lroundrect/README"] = {["details"] = "Readme", },
  ["latex/grid-system/README"] = {["details"] = "Readme", },
  ["latex/spot/README"] = {["details"] = "Readme", },
  ["latex/arraysort/arraysort.pdf"] = {["details"] = "Package documentation", },
  ["latex/songbook/songbook.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries-irish/glossaries-irish.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-english/README"] = {["details"] = "Readme", },
  ["latex/longdivision/README"] = {["details"] = "Readme", },
  ["latex/revtex/source/revtex4-1.pdf"] = {["details"] = "Package documentation", },
  ["fonts/urwchancal/urwchancal-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/drm/README"] = {["details"] = "Readme", },
  ["latex/bondgraph/README"] = {["details"] = "Readme", },
  ["latex/mwcls/README"] = {["lang"] = "en", ["details"] = "Package README", },
  ["latex/gradstudentresume/README.txt"] = {["details"] = "Readme", },
  ["fonts/miama/miama.pdf"] = {["details"] = "Package documentation", },
  ["latex/childdoc/README.txt"] = {["details"] = "Readme", },
  ["latex/upquote/README"] = {["details"] = "Readme", },
  ["latex/montex/montex.pdf"] = {["details"] = "Package documentation", },
  ["latex/chemnum/README"] = {["details"] = "Readme", },
  ["latex/cals/notes.pdf"] = {["details"] = "Additional notes", },
  ["latex/currvita/README"] = {["details"] = "Package Readme", },
  ["latex/stex/statements/README"] = {["details"] = "Readme", },
  ["latex/icsv/icsv.pdf"] = {["details"] = "Package documentation", },
  ["fonts/txuprcal/txuprcal-doc.pdf"] = {["details"] = "Package documentation", },
  ["xetex/font-change-xetex/README.txt"] = {["details"] = "Readme", },
  ["latex/warpcol/README"] = {["details"] = "Package Readme", },
  ["latex/mftinc/mftinc.pdf"] = {["details"] = "Package documentation", },
  ["latex/zxjafont/zxjafont.pdf"] = {["details"] = "Package documentation", },
  ["latex/calculator/README"] = {["details"] = "Readme", },
  ["latex/zwpagelayout/zwpagelayout.pdf"] = {["details"] = "Package documentation", },
  ["latex/ptptex/README"] = {["details"] = "Readme", },
  ["fonts/fonts-churchslavonic/fonts-churchslavonic.pdf"] = {["details"] = "Package documentation", },
  ["latex/zwpagelayout/README"] = {["details"] = "Readme", },
  ["fonts/brushscr/README"] = {["details"] = "Readme", },
  ["latex/bibunits/README"] = {["details"] = "Readme", },
  ["latex/tkz-euclide/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/zwgetfdate/zwgetfdate.pdf"] = {["details"] = "Package documentation", },
  ["generic/vaucanson-g/VCManual.pdf"] = {["details"] = "Package documentation", },
  ["latex/fixltxhyph/README"] = {["details"] = "Readme", },
  ["latex/cellprops/README.md"] = {["details"] = "Readme", },
  ["latex/pdfcomment/pdfcomment.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/ordinalpt/ordinalpt.pdf"] = {["details"] = "Package documentation", },
  ["latex/ziffer/README"] = {["details"] = "Readme", },
  ["latex/ifoddpage/README"] = {["details"] = "Readme", },
  ["latex/l2tabu/README"] = {["details"] = "Readme", },
  ["generic/zhspacing/zhs-man.pdf"] = {["details"] = "Package documentation", },
  ["generic/zhspacing/README.md"] = {["details"] = "Readme", },
  ["latex/zhnumber/zhnumber.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["latex/br-lex/leis-exemplo.pdf"] = {["lang"] = "pt-br", ["details"] = "Example of use", },
  ["latex/draftcopy/draftcopy.pdf"] = {["details"] = "Package documentation", },
  ["fonts/zhmetrics/README"] = {["details"] = "Readme", },
  ["latex/menu/README"] = {["details"] = "Readme", },
  ["latex/bibleref-lds/README"] = {["details"] = "Readme", },
  ["latex/zhlipsum/zhlipsum.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["latex/zhlipsum/zhlipsum-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/recipebook/README"] = {["details"] = "Readme", },
  ["latex/kotex-oblivoir/README"] = {["details"] = "Readme", },
  ["latex/verbatimcopy/README"] = {["details"] = "Readme", },
  ["latex/aurl/README.md"] = {["details"] = "Readme", },
  ["latex/outlining/README.md"] = {["details"] = "Readme", },
  ["latex/datatool/README"] = {["details"] = "Readme", },
  ["latex/glossaries-extra/README"] = {["details"] = "Readme", },
  ["latex/powerdot-FUBerlin/exampleClass.pdf"] = {["details"] = "Example of use of the class", },
  ["latex/ebsthesis/ebsthesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/gtrlib-largetrees/README.md"] = {["details"] = "Readme", },
  ["latex/lni/README.md"] = {["details"] = "Readme", },
  ["latex/zed-csp/csp2e.pdf"] = {["details"] = "CSP usage documentation", },
  ["xelatex/bidi-atbegshi/bidi-atbegshi-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/fancyhdr-it/itfancyhdr.pdf"] = {["details"] = "The document itself", },
  ["generic/babel-polish/polish.pdf"] = {["details"] = "Package documentation", },
  ["latex/univie-ling/univie-ling-wlg.pdf"] = {["details"] = "Package documentation (Wiener Linguistische Gazette)", },
  ["fonts/mathabx-type1/README"] = {["details"] = "Readme", },
  ["latex/calxxxx-yyyy/cal2018-2035_EN.pdf"] = {["lang"] = "en", ["details"] = "Sample (English) Calendar", },
  ["lualatex/lualatex-doc/lualatex-doc.pdf"] = {["details"] = "The document itself", },
  ["latex/scientific-thesis-cover/README.md"] = {["details"] = "Readme", },
  ["latex/ytableau/README"] = {["details"] = "Readme", },
  ["latex/youngtab/youngtab.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-magneticfield/pst-magneticfield-docFR.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["generic/librarian/librarian-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/youngtab/README"] = {["details"] = "Readme", },
  ["latex/york-thesis/york-thesis.pdf"] = {["details"] = "Package documentation", },
  ["metapost/bbcard/README.scorecard"] = {["details"] = "Scorecard readme", },
  ["latex/nucleardata/README"] = {["details"] = "Readme", },
  ["latex/york-thesis/README"] = {["details"] = "Package Readme", },
  ["latex/biblatex-source-division/README"] = {["details"] = "Readme", },
  ["latex/egameps/egameps.pdf"] = {["details"] = "Package documentation", },
  ["latex/gatherenum/README.md"] = {["details"] = "Readme", },
  ["latex/bbold/README"] = {["details"] = "Readme", },
  ["fonts/yhmath/yhmath.pdf"] = {["details"] = "Sample and documentation", },
  ["fonts/yfonts-t1/README"] = {["details"] = "Package README", },
  ["generic/babel-german/README"] = {["details"] = "Readme", },
  ["latex/ycbook/ycbook-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ycbook/README.md"] = {["details"] = "Readme", },
  ["latex/fixfoot/README"] = {["details"] = "Readme", },
  ["generic/yax/README"] = {["details"] = "Readme", },
  ["latex/yathesis/french/documentation/yathesis-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["fonts/yannisgr/README"] = {["details"] = "Readme", },
  ["latex/method/method.pdf"] = {["details"] = "Package documentation", },
  ["latex/xifthen/xifthen.pdf"] = {["details"] = "Package documentation", },
  ["context/third/npp-for-context/doc/npp-context-manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/chemschemex/README"] = {["details"] = "Readme", },
  ["latex/thumbs/thumbs.pdf"] = {["details"] = "Package documentation", },
  ["latex/yaletter/yaletter.pdf"] = {["details"] = "Package documentation", },
  ["latex/tabriz-thesis/README"] = {["details"] = "Readme", },
  ["latex/yagusylo/yagusylo-en.pdf"] = {["lang"] = "en", ["details"] = "User documentation (English)", },
  ["latex/forest-quickstart/README"] = {["details"] = "Readme", },
  ["latex/relenc/README"] = {["details"] = "Readme", },
  ["latex/yagusylo/LISEZMOI"] = {["lang"] = "fr", ["details"] = "Lisez moi", },
  ["latex/nih/example-nih-cls.pdf"] = {["details"] = "Example -- the class in normal use", },
  ["latex/bxjscls/bxjscls.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation (Japanese)", },
  ["latex/yafoot/README"] = {["details"] = "Bundle Readme", },
  ["latex/l3experimental/xcoffins/xcoffins.pdf"] = {["details"] = "coffins documentation", },
  ["latex/lshort-finnish/README"] = {["details"] = "Readme", },
  ["latex/maybemath/maybemath.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-gb7714-2015/biblatex-gb7714-2015.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation (Chinese)", },
  ["support/ltxfileinfo/ltxfileinfo.pdf"] = {["details"] = "Package documentation", },
  ["latex/xytree/xytree-doc-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["xelatex/fontbook/fontbook.pdf"] = {["details"] = "Package documentation", },
  ["generic/xypic-tut-pt/xypic-tutorial.pdf"] = {["lang"] = "pt", ["details"] = "The document itself", },
  ["generic/xypic/xysource.pdf"] = {["lang"] = "en", ["details"] = "Documented source", },
  ["latex/navydocs/README.txt"] = {["details"] = "Readme", },
  ["latex/gchords/gchords_doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/xypic/xyrefer.pdf"] = {["lang"] = "en", ["details"] = "Reference manual", },
  ["fonts/dsserif/README"] = {["details"] = "Readme", },
  ["generic/xypic/xyguide.pdf"] = {["lang"] = "en", ["details"] = "User guide", },
  ["latex/xyling/xyli-doc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["fonts/gfsartemisia/gfsartemisia.pdf"] = {["details"] = "Package documentation", },
  ["latex/xwatermark/xwatermark-guide.pdf"] = {["details"] = "Package documentation", },
  ["latex/xwatermark/README"] = {["details"] = "Readme", },
  ["generic/pst-knot/pst-knot-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/xurl/xurl.pdf"] = {["details"] = "Package documentation", },
  ["latex/xurl/README.md"] = {["details"] = "Readme", },
  ["latex/gtrcrd/gtrcrd-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ocg-p/ocg-p.pdf"] = {["details"] = "Package documentation", },
  ["latex/physics/README"] = {["details"] = "Readme", },
  ["latex/ntheorem-vn/ntheorem-doc-vn.pdf"] = {["lang"] = "vi", ["details"] = "The document itself", },
  ["xelatex/xunicode/README"] = {["details"] = "Readme", },
  ["latex/xtuthesis/xtuthesis.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["latex/xtuthesis/README.md"] = {["lang"] = "zh", ["details"] = "Readme", },
  ["latex/figbib/README"] = {["details"] = "Readme", },
  ["generic/xstring/xstring_doc_fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["generic/xstring/xstring_doc_en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/noconflict/README"] = {["details"] = "Readme", },
  ["latex/metalogo/README"] = {["details"] = "Readme", },
  ["latex/confproc/confproc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-fit/README"] = {["details"] = "Readme", },
  ["latex/xskak/README"] = {["details"] = "Readme", },
  ["latex/xsim/README"] = {["details"] = "Readme", },
  ["latex/cite/cite.pdf"] = {["details"] = "Package documentation", },
  ["latex/ethiop/README"] = {["details"] = "Readme", },
  ["latex/xsavebox/xsavebox.pdf"] = {["details"] = "Package documentation", },
  ["latex/imtekda/README"] = {["lang"] = "de", ["details"] = "Readme (German)", },
  ["fonts/xq/xqexample.pdf"] = {["details"] = "Example of use", },
  ["generic/pst-vehicle/pst-vehicle-doc-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/xpunctuate/xpunctuate.pdf"] = {["details"] = "Package documentation", },
  ["generic/textmerg/textmerg.pdf"] = {["details"] = "Package documentation", },
  ["plain/impatient/README"] = {["details"] = "Basic project information", },
  ["latex/titlepic/titlepic-manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/uowthesis/mythesis.pdf"] = {["details"] = "Demonstration document", },
  ["latex/aiaa/pre2004/README"] = {["details"] = "Package README", },
  ["latex/xpinyin/README.md"] = {["details"] = "Readme", },
  ["latex/wordlike/wordlike.pdf"] = {["details"] = "Package documentation", },
  ["latex/xpicture/README"] = {["details"] = "Readme", },
  ["latex/cjhebrew/cjhebrew.pdf"] = {["details"] = "Package manual", },
  ["latex/pst-vectorian/psvectorian.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/xpiano/xpiano.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-optic/README"] = {["details"] = "Readme", },
  ["latex/xpeek/xpeek.pdf"] = {["details"] = "Package documentation", },
  ["latex/chbibref/chbibref.pdf"] = {["details"] = "Package documentation", },
  ["latex/xpeek/README"] = {["details"] = "Readme", },
  ["fonts/lobster2/README"] = {["details"] = "Readme", },
  ["generic/pst-gr3d/README"] = {["details"] = "Readme", },
  ["latex/xpatch/README"] = {["details"] = "Readme", },
  ["latex/xoptarg/xoptarg.pdf"] = {["details"] = "Package documentation", },
  ["latex/koma-script-examples/README.txt"] = {["lang"] = "en", ["details"] = "English Readme", },
  ["latex/filemod/filemod.pdf"] = {["details"] = "Package documentation", },
  ["luatex/kanaparser/description.pdf"] = {["details"] = "Package documentation", },
  ["latex/xoptarg/README"] = {["details"] = "Readme", },
  ["latex/arcs/arcs.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-solarsystem/README"] = {["details"] = "Readme", },
  ["latex/xnewcommand/README"] = {["details"] = "Readme", },
  ["latex/gmdoc/gmdoc.pdf"] = {["details"] = "Package documentation", },
  ["luatex/luatexko/README"] = {["details"] = "Readme", },
  ["xelatex/na-position/README"] = {["details"] = "Readme", },
  ["latex/bewerbung/bewerbung-example.pdf"] = {["details"] = "Example of use", },
  ["latex/tengwarscript/README"] = {["details"] = "Readme", },
  ["latex/gentombow/gentombow.pdf"] = {["details"] = "Documentation of the gentombow package (English)", },
  ["latex/manuscript/manuscript.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-platon/README"] = {["details"] = "Readme", },
  ["latex/beamercolorthemeowl/beamercolorthemeowl.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-magyar/README"] = {["details"] = "Readme", },
  ["latex/intro-scientific/scidoc.pdf"] = {["details"] = "The document itself", },
  ["uplatex/base/uplatex-en.pdf"] = {["details"] = "Package documentation", },
  ["latex/xmpincl/README"] = {["details"] = "Package Readme", },
  ["otherformats/xmltex/base/readme.txt"] = {["details"] = "Readme", },
  ["otherformats/xmltex/base/manual.html"] = {["details"] = "Package documentation", },
  ["latex/expdlist/expdlist.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/cweb-latex/README"] = {["details"] = "Readme", },
  ["latex/facsimile/README"] = {["details"] = "Readme", },
  ["latex/custom-bib/merlin.pdf"] = {["details"] = "Master style file documentation", },
  ["fonts/lato/README"] = {["details"] = "Readme", },
  ["latex/simplified-latex/simplified-intro.pdf"] = {["details"] = "The document itself", },
  ["latex/graphviz/README"] = {["details"] = "Readme", },
  ["latex/latex-course/LaTeX-Course.pdf"] = {["details"] = "The course itself", },
  ["latex/xltxtra/xltxtra.pdf"] = {["details"] = "Package documentation", },
  ["fonts/ipaex-type1/README.md"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/xltxtra/README"] = {["details"] = "Readme", },
  ["latex/ucharcat/ucharcat.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/xeindex/README"] = {["details"] = "Readme", },
  ["latex/cmdstring/cmdstring.pdf"] = {["details"] = "Package documentation", },
  ["latex/sanitize-umlaut/sanitize-umlaut.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamerswitch/beamerswitch.pdf"] = {["details"] = "Package documentation", },
  ["latex/dlfltxb/dlfltxbmisc.pdf"] = {["details"] = "dlfltxbmisc package documentation", },
  ["latex/xltabular/README.md"] = {["details"] = "Readme", },
  ["fonts/esint-type1/table.pdf"] = {["details"] = "Table of integral signs", },
  ["latex/multienv/multienv.pdf"] = {["details"] = "Package documentation", },
  ["latex/newlfm/README"] = {["details"] = "Readme", },
  ["latex/amscls-doc/Author_Handbook_Memo.pdf"] = {["details"] = "AMS Author Handbook for the Memoirs Class", },
  ["fonts/fontawesome5/fontawesome5.pdf"] = {["details"] = "Package documentation", },
  ["latex/conv-xkv/doc/conv-xkv.pdf"] = {["details"] = "Package documentation", },
  ["latex/rectopma/TestTitle.pdf"] = {["details"] = "Package documentation (and usage example)", },
  ["generic/xlop/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/msu-thesis/msu-thesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/uni-wtal-ger/LIESMICH"] = {["lang"] = "de", ["details"] = "Readme", },
  ["latex/xkeyval/README"] = {["details"] = "Package README", },
  ["latex/ntheorem-vn/README"] = {["details"] = "Readme", },
  ["latex/fancyhandout/README.md"] = {["details"] = "Readme", },
  ["latex/przechlewski-book/README.pl"] = {["lang"] = "pl", ["details"] = "Readme (Polish)", },
  ["latex/chapterfolder/chapterfolder.pdf"] = {["details"] = "Package documentation", },
  ["fonts/xits/README"] = {["details"] = "Readme", },
  ["latex/ncclatex/ncclatex.pdf"] = {["details"] = "Package documentation", },
  ["generic/xint/xint.pdf"] = {["details"] = "Package documentation", },
  ["latex/ebproof/README.md"] = {["details"] = "Readme", },
  ["latex/liturg/liturg.pdf"] = {["details"] = "Package documentation", },
  ["latex/moodle/README.md"] = {["details"] = "Readme", },
  ["latex/changepage/README"] = {["details"] = "Readme", },
  ["support/convbkmk/README"] = {["details"] = "Readme", },
  ["generic/xint/sourcexint.pdf"] = {["details"] = "Documented source code", },
  ["latex/undolabl/README"] = {["details"] = "Readme", },
  ["latex/fancyvrb/fancyvrb.pdf"] = {["details"] = "Package documentation", },
  ["latex/multibibliography/tug-paper.pdf"] = {["details"] = "TUGboat paper", },
  ["latex/hardwrap/README"] = {["details"] = "Readme", },
  ["latex/keycommand/keycommand-example.pdf"] = {["details"] = "Example of usage", },
  ["latex/turnstile/turnstile-en.pdf"] = {["details"] = "Package documentation (English)", },
  ["latex/backnaur/backnaur.pdf"] = {["details"] = "Package documentation", },
  ["latex/uantwerpendocs/uantwerpenexam-example2.pdf"] = {["details"] = "Example exam (bilingual)", },
  ["latex/glossaries-german/README"] = {["details"] = "Readme", },
  ["support/mendex/README.md"] = {["details"] = "Readme", },
  ["latex/excludeonly/excludeonly.pdf"] = {["details"] = "Package documentation", },
  ["xindy/manual.html"] = {["details"] = "Xindy reference manual", },
  ["fonts/lato/lato.pdf"] = {["details"] = "Package documentation", },
  ["xindy/faq.html"] = {["details"] = "Xindy FAQ, or xindy by topic", },
  ["plain/xii-lat/README.txt"] = {["details"] = "Readme", },
  ["luatex/spelling/spelling-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/thumbs/thumbs-example.pdf"] = {["details"] = "Package examples", },
  ["latex/trajan/trytrajan.pdf"] = {["details"] = "Font sample", },
  ["generic/pst-osci/oscilloscope.pdf"] = {["lang"] = "fr", ["details"] = "French documentation", },
  ["metapost/metauml/metauml_manual_0.2.5.pdf"] = {["details"] = "Package documentation", },
  ["latex/tools/README.txt"] = {["details"] = "Readme", },
  ["latex/xfor/xfor.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-support/README"] = {["details"] = "Readme", },
  ["generic/pst-electricfield/pst-electricfield-docDE.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/munich/example_munich_Bibtex_style.pdf"] = {["details"] = "Usage example", },
  ["xelatex/xevlna/xevlna.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/xevlna/README"] = {["details"] = "Readme", },
  ["fonts/charter/readme.charter"] = {["details"] = "Readme", },
  ["xetex/xetexref/README"] = {["details"] = "Readme", },
  ["xetex/xetexko/xetexko-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/natbib/README.v831b"] = {["lang"] = "en", ["details"] = "Readme for this release", },
  ["latex/unswcover/thesis.pdf"] = {["details"] = "Example document", },
  ["otherformats/lollipop/README"] = {["details"] = "Readme", },
  ["latex/latex-tds/README"] = {["details"] = "Readme", },
  ["latex/dot2texi/README"] = {["details"] = "Readme", },
  ["xetex/xetex-tibetan/README"] = {["details"] = "Readme", },
  ["latex/combelow/combelow.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/xetex-itrans/README"] = {["details"] = "Readme", },
  ["xetex/base/XeTeX-notes.pdf"] = {["details"] = "About XeTeX", },
  ["latex/guitlogo/README"] = {["lang"] = "it", ["details"] = "Package README", },
  ["latex/actuarialangle/actuarialangle.pdf"] = {["details"] = "Package documentation", },
  ["latex/xprintlen/xprintlen.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/xespotcolor/README"] = {["details"] = "Readme", },
  ["latex/elements/README"] = {["details"] = "Readme", },
  ["fonts/cns/cns40-b5/README"] = {["details"] = "Readme", },
  ["xetex/xesearch/README"] = {["details"] = "Readme", },
  ["xelatex/xepersian/README"] = {["details"] = "Readme", },
  ["latex/xellipsis/xellipsis.pdf"] = {["details"] = "Package documentation", },
  ["latex/xellipsis/README"] = {["details"] = "Readme", },
  ["latex/marginfit/marginfit.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-fit/pst-fit-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/xltabular/xltabular-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/threadcol/README"] = {["details"] = "Readme", },
  ["latex/biblatex-iso690/biblatex-iso690.pdf"] = {["details"] = "Package documentation", },
  ["latex/aiaa/aiaa.pdf"] = {["details"] = "Package documentation", },
  ["latex/adfathesis/README"] = {["details"] = "Package README", },
  ["latex/interfaces/interfaces.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/xecyr/xecyr-doc-ru.pdf"] = {["lang"] = "ru", ["details"] = "Package documentation (Russian)", },
  ["xelatex/xecyr/README"] = {["details"] = "Readme", },
  ["xelatex/xecolor/README"] = {["details"] = "Readme", },
  ["xelatex/xecjk/xeCJK.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation (Chinese)", },
  ["latex/arcs/README"] = {["details"] = "Package README", },
  ["xelatex/xechangebar/README"] = {["details"] = "Readme", },
  ["latex/afthesis/README"] = {["details"] = "Readme", },
  ["latex/xebaposter/xebaposter-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ulqda/README"] = {["details"] = "Readme", },
  ["latex/linop/README.md"] = {["details"] = "Readme", },
  ["luatex/luatexko/luatexko-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries-polish/README"] = {["details"] = "Readme", },
  ["latex/xduthesis/xduthesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/xdoc/xdocdemo.pdf"] = {["details"] = "Demonstration of usage", },
  ["latex/xdoc/xdoc2.pdf"] = {["details"] = "xdoc2 documentation", },
  ["latex/xdoc/docindex.pdf"] = {["details"] = "docindex documentation", },
  ["latex/papertex/example/example.pdf"] = {["details"] = "Example newspaper page", },
  ["latex/quotmark/README"] = {["details"] = "Readme", },
  ["latex/tikz-qtree/README"] = {["details"] = "Readme", },
  ["generic/tex-refs/tex-refs.html"] = {["details"] = "The references themselves", },
  ["latex/stex/stex/stex.pdf"] = {["details"] = "Package documentation", },
  ["generic/xcomment/xcomment-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/beuron/beuron-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/datetime2-galician/README"] = {["details"] = "Readme", },
  ["latex/mathspic/README.txt"] = {["details"] = "Readme (Perl version)", },
  ["generic/pst-abspos/README"] = {["details"] = "Readme", },
  ["latex/dyntree/README"] = {["details"] = "Readme", },
  ["latex/datetime2-spanish/datetime2-spanish.pdf"] = {["details"] = "Package documentation", },
  ["latex/xcolor-solarized/README"] = {["details"] = "Readme", },
  ["latex/underoverlap/underoverlap.pdf"] = {["details"] = "Package documentation", },
  ["latex/undergradmath/README"] = {["details"] = "Readme", },
  ["latex/xcolor-material/README.md"] = {["details"] = "Readme", },
  ["latex/pythontex/README"] = {["details"] = "Readme", },
  ["latex/xcolor/README"] = {["details"] = "Readme", },
  ["latex/xcntperchap/xcntperchap_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/xcntperchap/README"] = {["details"] = "Readme", },
  ["latex/xcjk2uni/README.md"] = {["details"] = "Readme", },
  ["latex/xcite/xcite.pdf"] = {["details"] = "Package documentation", },
  ["latex/xcite/README"] = {["details"] = "Readme", },
  ["latex/eqnarray/eqnarray.pdf"] = {["details"] = "Package documentation", },
  ["generic/plainpkg/README"] = {["details"] = "Readme", },
  ["fonts/xcharter/xcharter-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/floatrow/floatrow.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["fonts/xcharter/README"] = {["details"] = "Readme", },
  ["latex/xbmks/docs/xbmksman.pdf"] = {["details"] = "Package documentation", },
  ["latex/cclicenses/cclicenses_short.pdf"] = {["details"] = "Brief package documentation", },
  ["latex/morefloats/README"] = {["details"] = "Readme", },
  ["latex/xbmks/README.md"] = {["details"] = "Readme", },
  ["fonts/quattrocento/samples.pdf"] = {["details"] = "Font samples", },
  ["latex/songbook/contrib/README"] = {["details"] = "Readme", },
  ["latex/chet/README"] = {["details"] = "Readme", },
  ["xelatex/facture/facture.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/scalebar/scalebar_examples.pdf"] = {["details"] = "Package usage guide and examples", },
  ["generic/pst-spirograph/README"] = {["details"] = "Readme", },
  ["latex/thmbox/thmbox.pdf"] = {["details"] = "Package documentation", },
  ["latex/import/import.pdf"] = {["details"] = "Package documentation", },
  ["latex/wtref/wtref.pdf"] = {["details"] = "Package documentation", },
  ["latex/newfile/README"] = {["details"] = "Readme", },
  ["latex/wtref/wtref-ja.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation (Japanese)", },
  ["latex/parnotes/parnotes.pdf"] = {["details"] = "Package documentation", },
  ["fonts/wsuipa/ipaman.ps"] = {["details"] = "Package documentation and font details", },
  ["context/third/vim/vim.txt"] = {["details"] = "Package usage notes", },
  ["fonts/wsuipa/README"] = {["details"] = "Package Readme", },
  ["latex/wsemclassic/README"] = {["details"] = "Readme", },
  ["latex/subfig/README"] = {["details"] = "Readme", },
  ["latex/xpicture/xpicture-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/barcodes/README"] = {["details"] = "Package Readme", },
  ["latex/cjk-ko/cjk-ko-doc.pdf"] = {["lang"] = "ko", ["details"] = "Package documentation (Korean)", },
  ["latex/wordcount/README"] = {["details"] = "Readme", },
  ["lualatex/selnolig/README"] = {["details"] = "Readme", },
  ["latex/getitems/getitems.pdf"] = {["details"] = "Package documentation", },
  ["latex/wnri-latex/README"] = {["details"] = "Readme", },
  ["latex/dynamicnumber/dynamicnumber.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamertheme-detlevcm/README.txt"] = {["details"] = "Readme", },
  ["latex/qtree/qarrows.pdf"] = {["lang"] = "en", ["details"] = "Use of tree-dvips with qtree", },
  ["latex/chemcompounds/chemcompounds.pdf"] = {["details"] = "Package documentation", },
  ["fonts/wnri/README"] = {["details"] = "Readme", },
  ["latex/carbohydrates/README"] = {["details"] = "Readme", },
  ["latex/moreenum/moreenum-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/witharrows/witharrows.pdf"] = {["details"] = "Package documentation", },
  ["latex/datenumber/docgerman.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/bookest/README"] = {["details"] = "Readme", },
  ["latex/notespages/np-test.pdf"] = {["details"] = "Example of use", },
  ["latex/bibleref-german/LIESMICH"] = {["lang"] = "de", ["details"] = "Liesmich", },
  ["latex/witharrows/README.md"] = {["details"] = "Readme", },
  ["latex/withargs/withargs.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-inet/README"] = {["details"] = "Readme", },
  ["latex/flacards/README"] = {["details"] = "Readme", },
  ["latex/milog/milog-example.pdf"] = {["details"] = "Example", },
  ["luatex/luaintro/README"] = {["details"] = "Readme", },
  ["latex/webguide/README"] = {["details"] = "README", },
  ["latex/hvfloat/hvfloat.pdf"] = {["details"] = "Package documentation", },
  ["latex/diffcoeff/diffcoeff.pdf"] = {["details"] = "Package documentation (diffcoeff)", },
  ["latex/optidef/optidef.pdf"] = {["details"] = "Package documentation", },
  ["latex/keystroke/README"] = {["details"] = "Readme", },
  ["latex/keyvaltable/README.md"] = {["details"] = "Readme", },
  ["latex/algorithm2e/algorithm2e.pdf"] = {["details"] = "Package documentation", },
  ["latex/warpcol/warpcol.pdf"] = {["details"] = "Package documentation", },
  ["latex/wallpaper/wallpapermanual.pdf"] = {["details"] = "Package documentation", },
  ["latex/octave/octave.pdf"] = {["details"] = "Package documentation", },
  ["latex/wallpaper/README"] = {["details"] = "Readme", },
  ["latex/testhyphens/testhyphens.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/wallcalendar/wallcalendar.pdf"] = {["details"] = "User Manual", },
  ["lualatex/wallcalendar/wallcalendar-code.pdf"] = {["details"] = "Code documentation", },
  ["lualatex/wallcalendar/README.md"] = {["details"] = "Readme", },
  ["latex/b1encoding/README"] = {["details"] = "Readme", },
  ["latex/har2nat/README"] = {["details"] = "Readme", },
  ["fonts/wadalab/wadalab-sampler.pdf"] = {["details"] = "Font sampler", },
  ["latex/cleveref/cleveref.pdf"] = {["details"] = "Package documentation", },
  ["latex/a0poster/a0_eng.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/poetry/README"] = {["details"] = "Readme", },
  ["fonts/wadalab/mr2j/README"] = {["details"] = "Readme", },
  ["latex/tocdata/tocdata.pdf"] = {["details"] = "Package documentation", },
  ["latex/simplecv/README"] = {["details"] = "Readme", },
  ["latex/continue/continue.pdf"] = {["details"] = "Package documentation", },
  ["latex/elsarticle/elsdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex2e-help-texinfo/graphics/README"] = {["details"] = "Package Readme", },
  ["latex/locality/README"] = {["details"] = "Readme", },
  ["fonts/merriweather/README"] = {["details"] = "Readme", },
  ["latex/nicematrix/README.md"] = {["details"] = "Readme", },
  ["fonts/wadalab/dmj/README"] = {["details"] = "Readme", },
  ["generic/babel-norsk/norsk.pdf"] = {["details"] = "Package documentation", },
  ["fonts/wadalab/dgj/README"] = {["details"] = "Readme", },
  ["latex/vwcol/vwcol.pdf"] = {["details"] = "Package documentation", },
  ["fonts/stix/README"] = {["details"] = "Readme", },
  ["latex/vwcol/README"] = {["details"] = "Readme", },
  ["support/epspdf/epspdf.pdf"] = {["details"] = "User Manual", },
  ["latex/eqnnumwarn/README.md"] = {["details"] = "Readme", },
  ["latex/nicetext/niceverb.pdf"] = {["details"] = "Niceverb documentation", },
  ["latex/vruler/vruler.pdf"] = {["details"] = "Package documentation", },
  ["latex/envlab/elguide.pdf"] = {["details"] = "User guide", },
  ["latex/vpe/README"] = {["details"] = "Readme", },
  ["latex/voss-mathcol/mathCol.pdf"] = {["details"] = "The document itself", },
  ["latex/vocaltract/vtLatex_FigureDemo.pdf"] = {["details"] = "Example model", },
  ["latex/vocaltract/README"] = {["details"] = "Readme", },
  ["generic/vntex/vntex.pdf"] = {["lang"] = "en", ["details"] = "Package manual (English)", },
  ["latex/esvect/README"] = {["details"] = "Readme", },
  ["latex/endfloat/endfloat.pdf"] = {["details"] = "Package documentation", },
  ["latex/clipboard/clipboard.pdf"] = {["details"] = "Package documentation", },
  ["latex/dialogl/README"] = {["details"] = "Readme", },
  ["latex/biblatex-apa/biblatex-apa.pdf"] = {["details"] = "Package documentation", },
  ["latex/bchart/README.md"] = {["details"] = "Readme", },
  ["latex/emptypage/README"] = {["details"] = "Readme", },
  ["latex/visualtikz/VisualTikZ.pdf"] = {["details"] = "The document itself", },
  ["latex/visualtikz/VisualTikZ-fr.pdf"] = {["lang"] = "fr", ["details"] = "The document itself (French)", },
  ["latex/autobreak/README.md"] = {["lang"] = "en", ["details"] = "README", },
  ["latex/visualtikz/README.txt"] = {["details"] = "Readme", },
  ["latex/plari/plari.pdf"] = {["details"] = "Package documentation", },
  ["latex/texments/README"] = {["details"] = "Readme", },
  ["latex/randomwalk/README.md"] = {["details"] = "Readme", },
  ["latex/eskdx/README"] = {["details"] = "Package README", },
  ["latex/visualpstricks/README.txt"] = {["details"] = "Readme", },
  ["latex/visualfaq/troubleshoot-vlf.pdf"] = {["details"] = "Dealing with visual FAQ problems:", },
  ["latex/isodoc/README"] = {["details"] = "Readme", },
  ["fonts/junicode/README"] = {["details"] = "Readme", },
  ["latex/visualfaq/README"] = {["details"] = "Readme", },
  ["latex/vhistory/vh_sets_en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (both packages)", },
  ["latex/vhistory/README"] = {["details"] = "Readme", },
  ["latex/vgrid/vgrid.pdf"] = {["details"] = "Package documentation", },
  ["latex/abntex2/examples/abntex2-modelo-artigo.pdf"] = {["lang"] = "pt-br", ["details"] = "Article template", },
  ["latex/philosophersimprint/sample.pdf"] = {["details"] = "Sample of usage", },
  ["latex/calligra/testfont.pdf"] = {["details"] = "Font table", },
  ["latex/cooltooltips/README"] = {["details"] = "Readme", },
  ["latex/nonfloat/nonfloat-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["fonts/uppunctlm/README.md"] = {["details"] = "Readme", },
  ["latex/assoccnt/README"] = {["details"] = "Readme", },
  ["latex/versonotes/versonotes.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-geo/pst-geo-doc.pdf"] = {["details"] = "Documentation", },
  ["latex/versonotes/sample.pdf"] = {["details"] = "Example of usage", },
  ["latex/labyrinth/README"] = {["details"] = "Readme", },
  ["latex/auto-pst-pdf/auto-pst-pdf.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/rterface/README"] = {["details"] = "Readme", },
  ["latex/url/url.pdf"] = {["details"] = "Package documentation", },
  ["latex/verse/README"] = {["details"] = "Readme", },
  ["latex/easy-todo/README"] = {["details"] = "Readme", },
  ["latex/verbments/verbments.pdf"] = {["details"] = "Package documentation", },
  ["latex/verbdef/verbdef.pdf"] = {["details"] = "Package documentation", },
  ["latex/bussproofs/testbp2.pdf"] = {["details"] = "Test/example of use", },
  ["fonts/hfbright/README"] = {["details"] = "Readme", },
  ["latex/ted/README"] = {["details"] = "Readme", },
  ["generic/pst-eps/pst-eps-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/fontname/fontname.pdf"] = {["details"] = "Outline of the mapping scheme (PDF)", },
  ["latex/verbatimbox/verbatimbox.pdf"] = {["details"] = "package documentation", },
  ["latex/bibleref/README"] = {["details"] = "Readme", },
  ["latex/msc/maintenance.pdf"] = {["details"] = "Maintenance manual", },
  ["latex/verbasef/verbasef-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/aiaa/pre2004/aiaa.pdf"] = {["details"] = "Package documentation", },
  ["latex/keyval2e/README"] = {["details"] = "Readme", },
  ["latex/trajan/trajan.pdf"] = {["details"] = "Package documentation", },
  ["latex/bxbase/bxbase-ja.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation", },
  ["latex/collectbox/README"] = {["details"] = "Readme", },
  ["fonts/fourier/README"] = {["details"] = "Readme", },
  ["latex/ionumbers/ionumbers.pdf"] = {["details"] = "Package documentation", },
  ["latex/apa/apacls.txt"] = {["details"] = "Outline of the class", },
  ["latex/everyhook/README"] = {["details"] = "Readme", },
  ["fonts/venturisadf/venturisadf.pdf"] = {["details"] = "Documentation: installation and use", },
  ["fonts/venturisadf/README"] = {["details"] = "Readme", },
  ["latex/dblfloatfix/dblfloatfix.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-bosnian/bosnian.pdf"] = {["details"] = "Package documentation", },
  ["latex/pmgraph/pmgraph.pdf"] = {["details"] = "Package documentation", },
  ["latex/venndiagram/venndiagram.pdf"] = {["details"] = "Package documentation", },
  ["latex/venndiagram/samples/venn-sample.pdf"] = {["details"] = "Example of use", },
  ["latex/latex-veryshortguide/veryshortguide.pdf"] = {["details"] = "The document itself", },
  ["latex/beamer-tut-pt/tutorialbeamer.pdf"] = {["lang"] = "pt", ["details"] = "The document itself (in Portuguese)", },
  ["lualatex/luabibentry/README"] = {["details"] = "Readme", },
  ["latex/latex4wp/latex4wp.pdf"] = {["details"] = "The document itself", },
  ["latex/rcsinfo/README"] = {["details"] = "Readme", },
  ["latex/matc3mem/matc3mem.pdf"] = {["lang"] = "it", ["details"] = "Package documentation (in Italian)", },
  ["latex/schemabloc/schemabloc.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["support/checkcites/checkcites-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/venndiagram/README"] = {["details"] = "Readme", },
  ["latex/beamerswitch/README.md"] = {["details"] = "Readme", },
  ["latex/havannah/README"] = {["details"] = "Readme", },
  ["latex/totcount/totcount.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst-intersect/pst-intersect.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["generic/velthuis/manual.pdf"] = {["details"] = "Package manual", },
  ["latex/tikz-karnaugh/README.txt"] = {["details"] = "Readme", },
  ["latex/mathpazo/pazotest.pdf"] = {["details"] = "Font tables:", },
  ["generic/vaucanson-g/README"] = {["details"] = "Readme", },
  ["fonts/arimo/README"] = {["details"] = "Readme", },
  ["latex/varwidth/varwidth-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/varindex/README"] = {["details"] = "Readme", },
  ["generic/variations/docvariations.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["fonts/variablelm/README.md"] = {["details"] = "Readme", },
  ["bibtex/vancouver/vancouver.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/vancouver/README"] = {["details"] = "Readme", },
  ["bibtex/vak/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/uwthesis/uwthesis.pdf"] = {["details"] = "Package documentation/example of use", },
  ["latex/uwthesis/README"] = {["details"] = "Readme", },
  ["latex/uwmslide/README"] = {["details"] = "Readme", },
  ["latex/jurarsp/README"] = {["details"] = "Readme", },
  ["generic/pst-contourplot/pst-contourplot-docEN.pdf"] = {["details"] = "Package documentation (English)", },
  ["bibtex/bath-bst/bath-bst.pdf"] = {["details"] = "Package documentation", },
  ["latex/begingreek/begingreek.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-labo/pst-labo-docEN.pdf"] = {["details"] = "English documentation:", },
  ["latex/multicap/multicap.pdf"] = {["details"] = "Package documentation", },
  ["mex/utf8mex/README"] = {["details"] = "Readme", },
  ["latex/ut-thesis/README"] = {["details"] = "Readme", },
  ["latex/uspatent/README"] = {["details"] = "Readme", },
  ["latex/petiteannonce/petiteannonceexample.pdf"] = {["details"] = "Example of use", },
  ["latex/ushort/README"] = {["details"] = "Readme", },
  ["latex/parrun/readme"] = {["details"] = "Readme", },
  ["latex/biblatex-publist/biblatex-publist.pdf"] = {["details"] = "Package documentation", },
  ["latex/talk/README"] = {["details"] = "Readme", },
  ["fonts/urwchancal/README"] = {["details"] = "Readme", },
  ["latex/signchart/README.txt"] = {["details"] = "Readme", },
  ["latex/kantlipsum/kantlipsum.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/urlbst/urlbst.html"] = {["details"] = "Package documentation (HTML format)", },
  ["xelatex/mathspec/mathspec.pdf"] = {["details"] = "Package documentation", },
  ["latex/modroman/modroman.pdf"] = {["details"] = "Documentated source (bilingual)", },
  ["latex/verse/verse.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/biber/biber.pdf"] = {["details"] = "Manual", },
  ["latex/recipecard/recipecard.pdf"] = {["details"] = "Package documentation", },
  ["latex/urcls/URletter-DEMO.pdf"] = {["details"] = "Example of usage (letter)", },
  ["latex/mafr/README"] = {["lang"] = "en", ["details"] = "Package README", },
  ["latex/drs/README"] = {["details"] = "Readme", },
  ["latex/urcls/URbeamer-DEMO.pdf"] = {["details"] = "Example of usage (presentation)", },
  ["latex/spalign/README.md"] = {["details"] = "Readme", },
  ["latex/normalcolor/README"] = {["details"] = "Readme", },
  ["fonts/old-arrows/README"] = {["details"] = "Readme", },
  ["latex/papertex/README"] = {["details"] = "Readme", },
  ["latex/todonotes/README"] = {["details"] = "Readme", },
  ["generic/upzhkinsoku/upzhkinsoku-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex-via-exemplos/README.md"] = {["details"] = "Readme", },
  ["fonts/overlock/README"] = {["details"] = "Readme", },
  ["latex/scanpages/scanpages-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/mwcls/mwclsdoc.pdf"] = {["lang"] = "pl", ["details"] = "Package documentation", },
  ["fonts/countriesofeurope/CountriesOfEurope.pdf"] = {["details"] = "Package documentation", },
  ["latex/trajan/README"] = {["details"] = "Readme", },
  ["fonts/gnu-freefont/README"] = {["details"] = "Readme", },
  ["latex/aucklandthesis/aucklandthesis_userguide.pdf"] = {["details"] = "Package documentation", },
  ["uptex/uptex-base/README.md"] = {["details"] = "Readme", },
  ["latex/upquote/upquote.pdf"] = {["details"] = "Package documentation", },
  ["latex/photo/photo.pdf"] = {["details"] = "Package documentation", },
  ["fonts/uppunctlm/always_upright_punctuation_marks.pdf"] = {["details"] = "Package documentation", },
  ["latex/upmethodology/README"] = {["details"] = "Readme", },
  ["generic/upca/README"] = {["details"] = "Readme", },
  ["latex/uowthesistitlepage/uowthesistitlepage_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/uowthesistitlepage/README.md"] = {["details"] = "Readme", },
  ["latex/xpinyin/xpinyin.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["latex/threadcol/threadcol.pdf"] = {["details"] = "Package documentation", },
  ["support/ketcindy/ketmanual/KeTCindyReferenceJ.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation (japanese)", },
  ["latex/tabularcalc/README"] = {["details"] = "Readme", },
  ["latex/uowthesis/README"] = {["details"] = "Readme", },
  ["latex/uothesis/README"] = {["details"] = "Readme", },
  ["latex/hyphenat/hyphenat.pdf"] = {["details"] = "Package documentation", },
  ["fonts/fouriernc/README"] = {["details"] = "Readme", },
  ["latex/engpron/engpron-ex-en.pdf"] = {["lang"] = "en", ["details"] = "Usage example (English)", },
  ["generic/pst-geometrictools/pst-geometrictools-doc.pdf"] = {["details"] = "Package documentation", },
  ["xetex/xetexfontinfo/README"] = {["details"] = "Readme", },
  ["latex/unravel/unravel.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries/glossaries-code.pdf"] = {["details"] = "Package code and documentation (PDF)", },
  ["latex/mathfam256/README.md"] = {["details"] = "Readme", },
  ["latex/libgreek/libgreek.pdf"] = {["details"] = "Package documentation", },
  ["latex/lt3graph/lt3graph.pdf"] = {["details"] = "Package documentation", },
  ["latex/zebra-goodies/README.md"] = {["details"] = "Readme", },
  ["support/pkfix/README"] = {["details"] = "Readme", },
  ["latex/tram/tram-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/univie-ling/univie-ling-paper.pdf"] = {["details"] = "Package documentation (paper class)", },
  ["latex/univie-ling/univie-ling-expose.pdf"] = {["details"] = "Package documentation (expose class)", },
  ["generic/babel-spanglish/README"] = {["details"] = "Readme", },
  ["latex/elbioimp/README"] = {["details"] = "Readme", },
  ["latex/blochsphere/README.md"] = {["details"] = "Readme", },
  ["latex/ltxindex/README"] = {["details"] = "Readme", },
  ["latex/stellenbosch/USsummary-1.0a.pdf"] = {["lang"] = "en", ["details"] = "USsummary package documentation", },
  ["latex/nameauth/nameauth.pdf"] = {["details"] = "Package documentation", },
  ["generic/fltpoint/fltpoint.pdf"] = {["details"] = "Package documentation", },
  ["latex/tabularcalc/tabularcalc_doc_fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/unitn-bimrep/examples/example.pdf"] = {["details"] = "Example of use", },
  ["latex/unitn-bimrep/README.pdf"] = {["details"] = "Package documentation", },
  ["latex/bxdpx-beamer/README.md"] = {["details"] = "Readme", },
  ["latex/modernposter/demo.pdf"] = {["details"] = "Example of use", },
  ["latex/mslapa/README"] = {["details"] = "Readme", },
  ["xelatex/mynsfc/mynsfc.pdf"] = {["details"] = "Package documentation", },
  ["latex/unicode-math/unicode-math.pdf"] = {["details"] = "Package documentation (for the user)", },
  ["latex/unicode-math/unicode-math-code.pdf"] = {["details"] = "Documentation of the source code", },
  ["latex/overlays/README"] = {["details"] = "Readme", },
  ["latex/unicode-math/README.md"] = {["details"] = "Readme", },
  ["generic/unicode-data/README.md"] = {["details"] = "Readme", },
  ["latex/colorwav/colorwav.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/unicode-bidi/README.md"] = {["details"] = "Readme", },
  ["latex/uni-wtal-lin/uni-wtal-lin.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/datetime2-dutch/datetime2-dutch.pdf"] = {["details"] = "Package documentation", },
  ["latex/lineno/ulineno.pdf"] = {["details"] = "User manual:", },
  ["fonts/mathabx/README"] = {["details"] = "Readme", },
  ["latex/yagusylo/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/chemschemex/chemschemex.pdf"] = {["details"] = "Package documentation", },
  ["latex/ijmart/ijmsample.pdf"] = {["details"] = "Sample paper", },
  ["generic/mfpic/mfpcard.pdf"] = {["lang"] = "en", ["details"] = "Quick reference", },
  ["latex/uni-wtal-ger/germanistik.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/studenthandouts/studenthandouts-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/unfonts-extra/README.md"] = {["details"] = "Readme", },
  ["metapost/mparrows/mparrows.pdf"] = {["details"] = "Package documentation", },
  ["fonts/gfscomplutum/gfscomplutum.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/abstyles/README"] = {["details"] = "Readme", },
  ["latex/sparklines/sparklines.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cns/cns40-5/README"] = {["details"] = "Readme", },
  ["generic/markdown/markdown.html"] = {["details"] = "User Manual", },
  ["latex/hausarbeit-jura/hausarbeit-jura.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/version/version-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/progressbar/progressbar.pdf"] = {["details"] = "Package documentation", },
  ["latex/underlin/underlin.pdf"] = {["details"] = "Package documentation", },
  ["latex/undergradmath/undergradmath.pdf"] = {["details"] = "Package documentation", },
  ["latex/xcolor-material/xcolor-material.pdf"] = {["details"] = "Package documentation", },
  ["latex/unamthesis/UNAMThesis.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/cqubeamer/README.md"] = {["details"] = "Readme", },
  ["latex/regcount/regcount.pdf"] = {["details"] = "Package documentation", },
  ["latex/unamth-template/README.md"] = {["details"] = "Readme", },
  ["latex/umthesis/example.pdf"] = {["details"] = "Example of use", },
  ["latex/latexbug/latexbug.pdf"] = {["details"] = "Package documentation", },
  ["latex/umthesis/README"] = {["details"] = "Readme", },
  ["latex/thumby/README"] = {["details"] = "Readme", },
  ["latex/modroman/modroman-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/pdfcprot/README.txt"] = {["details"] = "Package Readme", },
  ["latex/umlaute/umlaute.pdf"] = {["details"] = "Package documentation", },
  ["latex/amsldoc-vn/README"] = {["details"] = "Readme", },
  ["latex/uml/uml.pdf"] = {["details"] = "Package documentation", },
  ["latex/umbclegislation/README"] = {["details"] = "Readme", },
  ["fonts/grotesq/ugq.txt"] = {["details"] = "Package documentation", },
  ["generic/pst-asr/pst-asr-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ulthese/ulthese.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["fonts/oldlatin/README"] = {["details"] = "Readme", },
  ["latex/ulthese/README.md"] = {["lang"] = "fr", ["details"] = "Readme (in French)", },
  ["latex/awesomebox/README.md"] = {["details"] = "Readme", },
  ["latex/ulqda/ulqda.pdf"] = {["details"] = "Package documentation", },
  ["latex/dinbrief/readme"] = {["lang"] = "en", ["details"] = "Package Readme", },
  ["generic/pst-tools/pst-tools-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/adtrees/adtreesdoc.pdf"] = {["details"] = "Package documentation", },
  ["luatex/spelling/README"] = {["details"] = "Readme", },
  ["xelatex/resumecls/README.md"] = {["details"] = "Readme", },
  ["generic/ukrhyph/rules60.pdf"] = {["lang"] = "uk", ["details"] = "Old hyphenation rules", },
  ["latex/uiucthesis/uiucthesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/changes/changes.ngerman.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["fonts/memdesign/memdesign.pdf"] = {["details"] = "The document itself", },
  ["latex/uiucredborder/uiucredborder.pdf"] = {["details"] = "Package documentation", },
  ["latex/uhrzeit/uhrzeit-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/listings-ext/README"] = {["details"] = "Readme", },
  ["latex/uhhassignment/README.md"] = {["details"] = "Readme", },
  ["latex/uestcthesis/doc/uestcthesis-doc.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation (Chinese)", },
  ["latex/uestcthesis/README"] = {["details"] = "Readme (in English and Chinese)", },
  ["latex/uebungsblatt/uebungsblatt-doc.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/commath/README"] = {["details"] = "Readme", },
  ["bibtex/germbib/README.bibgerm"] = {["details"] = "Readme", },
  ["latex/mcite/README"] = {["details"] = "Readme", },
  ["latex/kdgdocs/kdgdocs.pdf"] = {["details"] = "Package documentation", },
  ["latex/grid-system/grid-system.pdf"] = {["details"] = "Package documentation", },
  ["latex/ucthesis/README"] = {["details"] = "Readme and basic documentation", },
  ["latex/regstats/regstats.pdf"] = {["details"] = "Package documentation", },
  ["latex/ucs/unsupported/README"] = {["details"] = "Bundle readme", },
  ["latex/archaeologie/archaeologie.pdf"] = {["details"] = "Package documentation", },
  ["latex/ucs/ucs.pdf"] = {["details"] = "Package documentation", },
  ["latex/thmtools/README"] = {["details"] = "Readme", },
  ["latex/ucs/INSTALL"] = {["details"] = "Installation details", },
  ["latex/pauldoc/pauldoc.pdf"] = {["lang"] = "de", ["details"] = "Benutzerdoku", },
  ["fonts/fontinst/manual/fontinst.pdf"] = {["details"] = "Package documentation", },
  ["latex/ucs/FAQ"] = {["details"] = "Frequently Asked Questions", },
  ["latex/pdfx/pdfx.pdf"] = {["details"] = "Package documentation", },
  ["latex/lshort-estonian/README"] = {["details"] = "Readme", },
  ["latex/amscls-doc/Author_Handbook_Journals.pdf"] = {["details"] = "AMS Author Handbook for Journal Classes", },
  ["latex/codepage/codepage.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/latexgit/README.md"] = {["details"] = "Readme", },
  ["latex/knittingpattern/introduction.pdf"] = {["details"] = "Package introduction", },
  ["latex/draftfigure/draftfigure.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/ucharclasses/README"] = {["details"] = "Readme", },
  ["latex/postage/postage.pdf"] = {["details"] = "Package documentation", },
  ["latex/ucdavisthesis/ucdavisthesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/stex/metakeys/README"] = {["details"] = "Readme", },
  ["latex/pressrelease/README"] = {["details"] = "Readme", },
  ["latex/emulateapj/README"] = {["details"] = "Readme", },
  ["latex/hagenberg-thesis/README.md"] = {["details"] = "Readme", },
  ["fonts/droid/droidserif-samples.pdf"] = {["details"] = "Font samples (serif)", },
  ["platex/base/platex.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation", },
  ["latex/ucbthesis/ucbthesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/lettrine/README"] = {["details"] = "Readme", },
  ["latex/dhua/dhua.pdf"] = {["details"] = "Package documentation", },
  ["latex/ucbthesis/README"] = {["details"] = "Readme", },
  ["latex/uassign/uassign.pdf"] = {["details"] = "Package documentation", },
  ["fonts/librebaskerville/librebaskerville-samples.pdf"] = {["details"] = "Font samples", },
  ["latex/uassign/README.md"] = {["details"] = "Package README", },
  ["latex/uantwerpendocs/uantwerpenletter-example.pdf"] = {["details"] = "Example letter", },
  ["latex/fixme/README.md"] = {["details"] = "Package Readme", },
  ["latex/uantwerpendocs/uantwerpenexam-example1.pdf"] = {["details"] = "Example exam (monolingual)", },
  ["latex/proposal/dfg/README"] = {["details"] = "Readme", },
  ["latex/grayhints/doc/grayhints_man.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-irish/irish.pdf"] = {["details"] = "Package documentation", },
  ["latex/uantwerpendocs/uantwerpendocs.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst-calendar/README"] = {["details"] = "Readme", },
  ["latex/uantwerpendocs/uantwerpencoursetext-example.pdf"] = {["details"] = "Example course text", },
  ["latex/signchart/signchart.pdf"] = {["details"] = "Package documentation", },
  ["latex/uantwerpendocs/README"] = {["details"] = "Readme", },
  ["latex/uaclasses/ua-example.pdf"] = {["details"] = "ua-thesis example", },
  ["latex/amscls-doc/ams-author-handbook-doc.pdf"] = {["details"] = "Compiling the AMS Author Handbooks", },
  ["latex/typogrid/typogrid.pdf"] = {["details"] = "Package documentation", },
  ["fonts/amsfonts/amsfonts.pdf"] = {["details"] = "General package documentation", },
  ["latex/method/README"] = {["details"] = "Readme", },
  ["latex/printlen/printlen-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst-eucl-translation-bg/README-bulgarian.txt"] = {["lang"] = "bg", ["details"] = "Readme", },
  ["latex/typogrid/README"] = {["details"] = "Readme", },
  ["latex/typoaid/typoaid.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cfr-lm/clm-test.pdf"] = {["details"] = "Samples from available fonts", },
  ["latex/listlbls/README"] = {["details"] = "Readme", },
  ["fonts/typicons/typicons.pdf"] = {["details"] = "Package documentation", },
  ["latex/tudscr/tudscr.pdf"] = {["details"] = "Package documentation", },
  ["latex/bclogo/bclogo-doc.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/templates-fenn/templates-fenn-de.txt"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/typeface/typeface-all-rm.pdf"] = {["details"] = "Properties of fonts that are available", },
  ["latex/genealogytree/genealogytree.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst-exa/README"] = {["details"] = "Readme", },
  ["latex/thesis-titlepage-fhac/README"] = {["details"] = "README file", },
  ["latex/dotseqn/dotseqn.pdf"] = {["details"] = "Package documentation", },
  ["latex/typed-checklist/README.md"] = {["details"] = "Readme", },
  ["latex/type1cm/type1cm-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-arrow/README"] = {["details"] = "Readme", },
  ["generic/petri-nets/pndoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/outlines/outlines.pdf"] = {["details"] = "Package documentation", },
  ["latex/footbib/README"] = {["details"] = "Readme", },
  ["latex/prftree/README"] = {["details"] = "Readme", },
  ["fonts/txfontsb/README"] = {["details"] = "Readme", },
  ["latex/lshort-persian/README"] = {["details"] = "Readme", },
  ["support/installfont/README"] = {["details"] = "Readme", },
  ["latex/bangorexam/README.txt"] = {["details"] = "Readme", },
  ["fonts/txfonts/txfontsdocA4.pdf"] = {["details"] = "Package documentation (A4)", },
  ["fonts/nkarta/fonttable.pdf"] = {["details"] = "A testfont table of the font", },
  ["latex/twoinone/twoinone.pdf"] = {["details"] = "Package documentation", },
  ["latex/getfiledate/getfiledate-guide.pdf"] = {["details"] = "Package documentation", },
  ["fonts/fontmfizz/README"] = {["details"] = "Readme", },
  ["latex/multibib/README"] = {["details"] = "Package Readme", },
  ["latex/sphdthesis/README"] = {["details"] = "Readme", },
  ["latex/europecv/europecv.pdf"] = {["details"] = "Class documentation", },
  ["latex/blindtext/blindtext.pdf"] = {["details"] = "Package documentation", },
  ["latex/cnbwp/README"] = {["details"] = "Readme", },
  ["latex/punk-latex/punk.pdf"] = {["details"] = "Package documentation", },
  ["latex/turnthepage/README"] = {["details"] = "Readme", },
  ["latex/turnstile/turnstile_article.pdf"] = {["details"] = "Article about the package (English)", },
  ["latex/turnstile/turnstile-pt.pdf"] = {["lang"] = "pt", ["details"] = "Package documentation (Portuguese)", },
  ["xelatex/simple-thesis-dissertation/README.md"] = {["details"] = "Readme", },
  ["latex/fduthesis/fduthesis.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation  (Chinese)", },
  ["generic/xint/README.html"] = {["details"] = "Readme (HTML)", },
  ["latex/turnstile/README"] = {["details"] = "Readme", },
  ["latex/turkmen/turkmen.pdf"] = {["details"] = "Package documentation", },
  ["latex/plari/README"] = {["details"] = "Readme", },
  ["latex/latexfileinfo-pkgs/README"] = {["details"] = "Readme", },
  ["bibtex/babelbib/babelbib.pdf"] = {["details"] = "Package documentation", },
  ["latex/apa6/README"] = {["details"] = "Readme", },
  ["xelatex/kurdishlipsum/kurdishlipsum.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/bidihl/README"] = {["details"] = "Readme", },
  ["latex/tkz-berge/readme-tkz-berge.txt"] = {["details"] = "Readme", },
  ["fonts/cyklop/cyklop-info.pdf"] = {["details"] = "Package documentation", },
  ["latex/fbithesis/fbithesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/calcage/calcage.pdf"] = {["details"] = "Package documentation", },
  ["latex/turabian-formatting/README"] = {["details"] = "Readme", },
  ["latex/setdeck/README"] = {["details"] = "Readme", },
  ["latex/eepic/eepic.pdf"] = {["details"] = "Package documentation", },
  ["latex/qrcode/qrcode.pdf"] = {["details"] = "Package documentation", },
  ["latex/extpfeil/README"] = {["details"] = "Readme", },
  ["latex/stanli/README.md"] = {["details"] = "Readme", },
  ["pmxchords/chordsRefCZ.pdf"] = {["lang"] = "cs", ["details"] = "Reference table (Czech)", },
  ["latex/play/README"] = {["details"] = "Readme", },
  ["latex/copyedit/copyedit.pdf"] = {["details"] = "Package documentation", },
  ["latex/arsclassica/README"] = {["details"] = "Readme", },
  ["fonts/wasy2-ps/README"] = {["details"] = "Readme", },
  ["fonts/lxfonts/LXfonts-demo.pdf"] = {["details"] = "Package outline and samples", },
  ["latex/tugboat/README"] = {["details"] = "Readme", },
  ["latex/tqft/tqft.pdf"] = {["details"] = "Package documentation", },
  ["latex/bibletext/bibletext.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/typewriter/README.md"] = {["details"] = "Readme", },
  ["latex/digiconfigs/README"] = {["details"] = "Readme", },
  ["latex/mcmthesis/README"] = {["details"] = "Readme", },
  ["latex/tucv/tucv.pdf"] = {["details"] = "Package documentation", },
  ["latex/tucv/README"] = {["details"] = "Readme", },
  ["latex/spanish-mx/README"] = {["details"] = "Readme", },
  ["latex/trsym/trsym.pdf"] = {["details"] = "Package documentation", },
  ["latex/trsym/README"] = {["details"] = "Readme", },
  ["latex/biblatex-swiss-legal/biblatex-swiss-legal.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/trivfloat/trivfloat.pdf"] = {["details"] = "Package documentation", },
  ["latex/trivfloat/README"] = {["details"] = "Readme", },
  ["context/third/french/french-demo.pdf"] = {["details"] = "Example of use", },
  ["bibtex/apalike-german/README.md"] = {["details"] = "Readme", },
  ["generic/path/path-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/trimspaces/trimspaces.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-cox/pst-coxcoor/pst-coxcoor_doc.pdf"] = {["lang"] = "en", ["details"] = "Documentation for pst-coxcoor", },
  ["support/bib2gls/README.md"] = {["details"] = "Readme", },
  ["latex/lualatex-doc-de/lualatex-doc-DE.pdf"] = {["details"] = "The translated document itself", },
  ["latex/aucklandthesis/README.txt"] = {["details"] = "Readme", },
  ["plain/treetex/readme"] = {["details"] = "Readme", },
  ["latex/tree-dvips/tree-manual.pdf"] = {["details"] = "Manual for tree-dvips", },
  ["latex/askmaps/askmaps.pdf"] = {["details"] = "Package documentation", },
  ["latex/tree-dvips/lingmacros-manual.pdf"] = {["details"] = "Manual for lingmacros", },
  ["latex/tree-dvips/README"] = {["details"] = "Readme", },
  ["latex/translator/README.md"] = {["details"] = "Readme", },
  ["latex/translations/translations_en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/plweb/README"] = {["details"] = "Readme", },
  ["latex/translation-natbib-fr/f-natbib.pdf"] = {["details"] = "Translated documentation", },
  ["latex/alertmessage/README.md"] = {["details"] = "Readme", },
  ["generic/mfpic/mfpic-doc.pdf"] = {["lang"] = "en", ["details"] = "Mfpic manual", },
  ["latex/translation-filecontents-de/filecontents-de.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/translation-enumitem-de/enumitem-de.pdf"] = {["lang"] = "de", ["details"] = "The translation itself", },
  ["latex/bibunits/bibunits.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/quran/quran-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/translation-dcolumn-fr/f-dcolumn.pdf"] = {["details"] = "The translation", },
  ["latex/mslapa/mslapa.pdf"] = {["details"] = "Package documentation", },
  ["plain/impatient/book.pdf"] = {["lang"] = "en", ["details"] = "A copy of the book (English)", },
  ["otherformats/texsis/base/README"] = {["details"] = "Readme", },
  ["latex/translation-dcolumn-fr/README"] = {["details"] = "Readme", },
  ["latex/translation-chemsym-de/chemsym-de.pdf"] = {["lang"] = "de", ["details"] = "The document itself", },
  ["latex/translation-biblatex-de/biblatex-de-Benutzerhandbuch.pdf"] = {["details"] = "The translation", },
  ["generic/ofs/ofsdoc-e.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/ku-template/README.txt"] = {["details"] = "Readme", },
  ["generic/pst-pulley/pst-pulley-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/translation-arsclassica-de/ArsClassica-de.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["generic/musixtex/README"] = {["details"] = "Readme", },
  ["generic/systeme/systeme_fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/translation-array-fr/f-array.pdf"] = {["details"] = "The translation", },
  ["latex/tram/README"] = {["details"] = "Readme", },
  ["plain/xii/README.txt"] = {["details"] = "Readme", },
  ["latex/fancybox/fancybox-doc.pdf"] = {["details"] = "Documentation -- Box tips and tricks:", },
  ["latex/cd/cd.pdf"] = {["details"] = "Package documentation", },
  ["latex/flippdf/README"] = {["details"] = "Readme", },
  ["generic/tracklang/tracklang.pdf"] = {["details"] = "Package documentation", },
  ["fonts/nimbus15/README"] = {["details"] = "Readme", },
  ["generic/tracklang/README"] = {["details"] = "Readme", },
  ["latex/tqft/tqft_code.pdf"] = {["details"] = "Annotated code", },
  ["generic/dowith/dowith.pdf"] = {["details"] = "Dowith documentation", },
  ["latex/tufte-latex/sample-book.pdf"] = {["details"] = "Sample book", },
  ["latex/totpages/totpages.pdf"] = {["details"] = "Package documentation", },
  ["latex/totpages/README"] = {["details"] = "Readme", },
  ["latex/karnaugh-map/README.md"] = {["details"] = "Readme", },
  ["latex/seqsplit/seqsplit.pdf"] = {["details"] = "Package documentation", },
  ["latex/topletter/README.md"] = {["details"] = "Readme", },
  ["latex/topletter/ExampleLetter.pdf"] = {["details"] = "Example of use (English)", },
  ["latex/feynmp-auto/README"] = {["details"] = "Readme", },
  ["latex/iso/README"] = {["details"] = "Readme", },
  ["latex/knowledge/knowledge.pdf"] = {["details"] = "Package documentation", },
  ["latex/polytable/polytable.pdf"] = {["details"] = "Package documentation", },
  ["latex/pbox/README"] = {["details"] = "Readme", },
  ["latex/incgraph/README"] = {["details"] = "Readme", },
  ["fonts/stix2-otf/STIX_2.0.0_stylistic_sets.pdf"] = {["details"] = "Package documentation", },
  ["latex/tokenizer/tokenizer.pdf"] = {["details"] = "Package documentation", },
  ["support/hook-pre-commit-pkg/README"] = {["details"] = "Readme", },
  ["latex/todonotes/todonotes.pdf"] = {["details"] = "Package documentation", },
  ["latex/soton/README"] = {["details"] = "Readme", },
  ["latex/tikz-optics/tikz-optics.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/todo/todo-spl.pdf"] = {["details"] = "Sample of use", },
  ["latex/pracjourn/README"] = {["details"] = "Readme", },
  ["latex/pbsheet/xpl/xpl-fr.pdf"] = {["lang"] = "fr", ["details"] = "Example of use", },
  ["latex/fonttable/fonttable.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries-magyar/glossaries-magyar.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-vietnamese/README"] = {["details"] = "Readme", },
  ["fonts/amiri/README-Arabic"] = {["lang"] = "ar", ["details"] = "Readme (Arabic)", },
  ["latex/dotarrow/README"] = {["details"] = "Readme", },
  ["latex/sesstime/sesstime.pdf"] = {["details"] = "Package documentation", },
  ["generic/autosp/autosp.pdf"] = {["details"] = "Package documentation", },
  ["latex/bhcexam/README"] = {["details"] = "Readme", },
  ["fonts/wadalab/mcj/README"] = {["details"] = "Readme", },
  ["latex/tocdata/README"] = {["details"] = "Readme", },
  ["latex/fullwidth/README"] = {["details"] = "Readme", },
  ["generic/midnight/README"] = {["details"] = "Package Readme", },
  ["latex/lastpage/README"] = {["details"] = "Readme", },
  ["latex/nag/README"] = {["details"] = "Readme", },
  ["latex/biblatex-claves/documentation/biblatex-claves.pdf"] = {["details"] = "Package documentation", },
  ["latex/tlc-article/README.md"] = {["details"] = "Readme", },
  ["latex/esint/esint.pdf"] = {["details"] = "Package documentation", },
  ["latex/tkz-tab/tkz-tab-screen.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/notestex/NotesTeX.pdf"] = {["details"] = "Package documentation", },
  ["latex/sourcesanspro/sourcesanspro.pdf"] = {["details"] = "Package documentation", },
  ["latex/tkz-tab/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/tkz-linknodes/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["generic/pst-node/README"] = {["details"] = "Readme", },
  ["latex/imakeidx/imakeidx.pdf"] = {["details"] = "Package documentation", },
  ["latex/tkz-graph/tkz-graph-screen.pdf"] = {["details"] = "Package documentation", },
  ["latex/chemgreek/README"] = {["details"] = "Readme", },
  ["latex/octavo/README"] = {["details"] = "Readme", },
  ["latex/marginfix/marginfix.pdf"] = {["details"] = "Package documentation", },
  ["latex/colortbl/colortbl.pdf"] = {["lang"] = "en", ["details"] = "Package manual", },
  ["platex/jsclasses/jsverb.pdf"] = {["lang"] = "ja", ["details"] = "Jsverb package documentation (Japanese)", },
  ["latex/tkz-fct/readme-tkz-fct-fr.txt"] = {["lang"] = "fr", ["details"] = "Readme", },
  ["latex/tkz-fct/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/tkz-euclide/tkz-euclide-screen.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/biblatex-cheatsheet/biblatex-cheatsheet.pdf"] = {["details"] = "The document itself", },
  ["latex/tkz-euclide/readme-tkz-euclide-fr.txt"] = {["lang"] = "fr", ["details"] = "Readme", },
  ["latex/bnumexpr/README"] = {["details"] = "Readme", },
  ["support/de-macro/README"] = {["details"] = "Readme", },
  ["latex/tkz-berge/tkz-berge-screen.pdf"] = {["details"] = "Package documentation", },
  ["latex/extpfeil/extpfeil.pdf"] = {["lang"] = "de", ["details"] = "Benutzerdoku", },
  ["support/tex4ebook/README"] = {["details"] = "Readme", },
  ["latex/tkz-base/README"] = {["details"] = "Readme", },
  ["latex/titling/titling.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-ijsra/README.md"] = {["details"] = "Readme", },
  ["latex/testidx/testidx-manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/eqlist/eqlist.pdf"] = {["details"] = "Package documentation", },
  ["latex/titlesec/titlesec.pdf"] = {["details"] = "Package documentation", },
  ["latex/titlesec/README"] = {["details"] = "Readme", },
  ["latex/titlepages/titlepages.pdf"] = {["details"] = "The document itself", },
  ["latex/filehook/README"] = {["details"] = "Readme", },
  ["latex/invoice2/README.md"] = {["details"] = "Readme", },
  ["generic/babel-portuges/portuges.pdf"] = {["details"] = "Package documentation", },
  ["latex/tipfr/tipfr-doc.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/tipa-de/tipaman-de.pdf"] = {["lang"] = "de", ["details"] = "The document itself", },
  ["latex/tipa-de/LIESMICH"] = {["lang"] = "de", ["details"] = "Readme", },
  ["latex/autonum/autonum.pdf"] = {["details"] = "Package documentation", },
  ["fonts/tipa/tipaman.pdf"] = {["lang"] = "en", ["details"] = "Package manual (English)", },
  ["latex/ledmac/ledmac.pdf"] = {["details"] = "Package documentation", },
  ["fonts/tinos/tinos-samples.pdf"] = {["details"] = "Package documentation", },
  ["latex/amsmath/technote.pdf"] = {["lang"] = "en", ["details"] = "Technical details", },
  ["latex/timing-diagrams/README"] = {["details"] = "Readme", },
  ["latex/ktv-texdata/ktv-texdata.pdf"] = {["details"] = "Package documentation", },
  ["latex/amsmath/amsldoc.pdf"] = {["lang"] = "en", ["details"] = "User guide (English)", },
  ["latex/timbreicmc/timbreicmc.pdf"] = {["details"] = "Package documentation", },
  ["platex/pxrubrica/pxrubrica.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation", },
  ["latex/ryethesis/ryethesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/timbreicmc/README.md"] = {["details"] = "Readme", },
  ["generic/bagpipe/quickref.pdf"] = {["details"] = "Package 'quick reference'", },
  ["latex/tikzsymbols/README.md"] = {["details"] = "Readme", },
  ["latex/tikzposter/tikzposter.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikzposter/README"] = {["details"] = "Readme", },
  ["latex/pageslts/pageslts-example.pdf"] = {["details"] = "Package examples", },
  ["latex/tikzpfeile/tikzpfeile.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-hebrew/datetime2-hebrew.pdf"] = {["details"] = "Package documentation", },
  ["latex/abntex2/examples/abntex2-modelo-projeto-pesquisa.pdf"] = {["lang"] = "pt-br", ["details"] = "Research project template", },
  ["latex/tikzpfeile/README"] = {["details"] = "Readme", },
  ["latex/mflogo/README"] = {["details"] = "Readme", },
  ["latex/tikzpeople/README.md"] = {["details"] = "Readme", },
  ["latex/tikzpagenodes/tikzpagenodes.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikzpagenodes/README"] = {["details"] = "Readme", },
  ["latex/fcolumn/README"] = {["details"] = "Readme", },
  ["latex/modular/modular.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikzorbital/tikzorbital.pdf"] = {["details"] = "Package documentation", },
  ["latex/pdfoverlay/pdfoverlay.pdf"] = {["details"] = "Package documentation", },
  ["latex/latexcheat-ptbr/README-ptbr"] = {["lang"] = "pt", ["details"] = "Readme (in Brazilian Portuguese)", },
  ["fonts/accanthis/accanthis-samples.pdf"] = {["details"] = "Font sample page", },
  ["latex/tikzmark/tikzmark.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-spectra/pst-spectra.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/quotchap/document.pdf"] = {["details"] = "Sample document (chapter headings only)", },
  ["latex/tikzducks/tikzducks-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/spie/README"] = {["details"] = "Readme", },
  ["latex/tikzcodeblocks/tikzcodeblocks-documentation.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["fonts/miama/README"] = {["details"] = "Readme", },
  ["latex/tikz-timing/tikz-timing.pdf"] = {["details"] = "Package documentation", },
  ["latex/classicthesis/ClassicThesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/diagbox/README"] = {["details"] = "Readme", },
  ["latex/pspicture/pspicture.pdf"] = {["details"] = "Package documentation", },
  ["latex/thesis-titlepage-fhac/fhACtitlepage.pdf"] = {["details"] = "Package documentation", },
  ["latex/xcookybooky/xcookybooky.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-catalan/catalan.pdf"] = {["details"] = "Package documentation", },
  ["latex/coverpage/CoverPage.pdf"] = {["details"] = "Package documentation", },
  ["latex/ijmart/ijmart.pdf"] = {["details"] = "Package documentation", },
  ["latex/ctex-faq/ctex-faq.pdf"] = {["lang"] = "zh", ["details"] = "The FAQ (Chinese)", },
  ["latex/tikz-palattice/tikz-palattice_documentation.pdf"] = {["details"] = "Package documentation", },
  ["latex/sansmath/sansmath.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-palattice/README"] = {["details"] = "Readme", },
  ["latex/tikz-page/README.md"] = {["details"] = "Readme", },
  ["lualatex/showhyphens/README"] = {["details"] = "Readme", },
  ["latex/tikz-optics/README"] = {["details"] = "Readme", },
  ["latex/lcyw/lcyw.pdf"] = {["details"] = "Package documentation", },
  ["latex/struktex/struktex.en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/isopt/README.md"] = {["details"] = "Readme", },
  ["latex/tikz-opm/tikz-opm.pdf"] = {["details"] = "Package documentation", },
  ["latex/skdoc/README"] = {["details"] = "Readme", },
  ["latex/pst-calendar/pst-calendar-docFR.pdf"] = {["lang"] = "fr", ["details"] = "French documentation:", },
  ["luatex/luapackageloader/README.md"] = {["details"] = "Readme", },
  ["latex/tikz-nef/README.md"] = {["details"] = "Readme", },
  ["latex/vdmlisting/vdmlisting.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-kalender/tikz-kalender-example1.pdf"] = {["details"] = "Example of use", },
  ["support/lyluatex/README.md"] = {["details"] = "Readme", },
  ["generic/listofitems/listofitems-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/dichokey/rhodocyb.pdf"] = {["details"] = "Example of use", },
  ["latex/apxproof/apxproof.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-inet/tikz-inet-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/elements/elements_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/withargs/README"] = {["details"] = "Readme", },
  ["fonts/japanese-otf/README"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["generic/m-tx/README"] = {["details"] = "Readme", },
  ["latex/tikz-feynman/tikz-feynman.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-feynman/README.md"] = {["details"] = "Readme", },
  ["plain/newsletr/newsletr.txt"] = {["details"] = "Help for newsletr format", },
  ["latex/svn-prov/svn-prov.pdf"] = {["details"] = "Package documentation", },
  ["latex/prosper/prosper-tour.pdf"] = {["details"] = "A brief example of usage", },
  ["latex/tikz-feynhand/README.md"] = {["details"] = "Readme", },
  ["latex/mathexam/README"] = {["details"] = "Readme", },
  ["latex/adrconv/README"] = {["details"] = "Readme", },
  ["latex/dvdcoll/README"] = {["details"] = "README file", },
  ["generic/pst-gr3d/pst-gr3d.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-dimline/tikz-dimline-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikz-dimline/README"] = {["details"] = "Readme", },
  ["latex/tikz-dependency/tikz-dependency-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/calrsfs/calrsfs.pdf"] = {["details"] = "Package documentation", },
  ["latex/l3kernel/l3styleguide.pdf"] = {["details"] = "LaTeX3 style guide", },
  ["latex/tikz-cd/tikz-cd-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/guitar/guitar.pdf"] = {["details"] = "Package documentation", },
  ["latex/pageslts/README"] = {["details"] = "Readme", },
  ["latex/bxtexlogo/README.md"] = {["details"] = "Readme", },
  ["latex/tikz-3dplot/tikz-3dplot_documentation.pdf"] = {["details"] = "Package documentation", },
  ["latex/linguex/linguex-doc.pdf"] = {["details"] = "Linguex manual", },
  ["latex/diadia/diadia.pdf"] = {["details"] = "PDF file", },
  ["latex/hepthesis/README"] = {["details"] = "Package README", },
  ["latex/showtags/showtags-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/factura/README"] = {["details"] = "Readme", },
  ["latex/lshort-english/lshort.pdf"] = {["details"] = "The document itself", },
  ["generic/pst-text/pst-text-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/thuthesis/thuthesis.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation  (Chinese)", },
  ["generic/babel-bulgarian/README"] = {["details"] = "Readme", },
  ["xelatex/quran/quran-test.pdf"] = {["details"] = "Example of use", },
  ["latex/examdesign/examdesign.pdf"] = {["details"] = "Package documentation", },
  ["latex/latexcheat-de/README"] = {["lang"] = "de", ["details"] = "Readme (in German)", },
  ["latex/checklistings/example.pdf"] = {["details"] = "Example of use", },
  ["latex/mattens/README"] = {["details"] = "Readme", },
  ["latex/ltxtools/README"] = {["details"] = "Readme", },
  ["latex/thumbs/README"] = {["details"] = "Readme", },
  ["generic/thumbpdf/README"] = {["details"] = "Readme", },
  ["latex/thumb/thumb.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cns/cns40-3/README"] = {["details"] = "Readme", },
  ["support/latexdiff/doc/latexdiff-man.pdf"] = {["details"] = "Package manual", },
  ["latex/numnameru/README"] = {["details"] = "Readme", },
  ["latex/fnpct/README"] = {["details"] = "Readme", },
  ["latex/threeparttable/threeparttable.pdf"] = {["details"] = "Package documentation", },
  ["metapost/threeddice/threeddice-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/mycv/mycv.pdf"] = {["details"] = "Package documentation", },
  ["latex/thmtools/thmtools.pdf"] = {["details"] = "Package documentation", },
  ["latex/spalign/spalign.pdf"] = {["details"] = "Package documentation", },
  ["fonts/nanumtype1/nanum-sampler.pdf"] = {["details"] = "Font sampler", },
  ["generic/armenian/manual.pdf"] = {["lang"] = "hy", ["details"] = "Package manual (Armenian)", },
  ["fonts/newtxsf/newtxsf-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/german/00readme.1st"] = {["lang"] = "de", ["details"] = "Readme", },
  ["latex/ucs/README"] = {["details"] = "Bundle readme", },
  ["latex/thinsp/thinsp.pdf"] = {["details"] = "Package documentation", },
  ["latex/thesis-ekf/README"] = {["details"] = "Readme", },
  ["latex/tikz-relay/README.txt"] = {["details"] = "Readme", },
  ["latex/typeface/README"] = {["details"] = "Readme", },
  ["latex/sf298/sf298.pdf"] = {["details"] = "Package documentation", },
  ["latex/css-colors/css-colors.pdf"] = {["details"] = "List of colours", },
  ["latex/formular/README"] = {["details"] = "Readme", },
  ["generic/babel-hungarian/README"] = {["details"] = "Notes about distribution", },
  ["latex/hyper/contrib/README"] = {["details"] = "Readme", },
  ["latex/fei/README"] = {["details"] = "Readme", },
  ["latex/akletter/letterdoc.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["fonts/comfortaa/comfortaa.pdf"] = {["details"] = "Package documentation", },
  ["latex/hustthesis/README.md"] = {["details"] = "Readme", },
  ["latex/pax/README"] = {["details"] = "Package README", },
  ["latex/thesis-gwu/thesis-sample.pdf"] = {["details"] = "Example thesis and user guide", },
  ["latex/hithesis/README.md"] = {["details"] = "Readme", },
  ["generic/pst-am/pst-am-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/colorweb/colorwebuser.pdf"] = {["details"] = "Documentation for users", },
  ["latex/makeplot/makeplot.pdf"] = {["details"] = "Package documentation", },
  ["latex/bxtexlogo/bxtexlogo-sample.pdf"] = {["details"] = "Samples", },
  ["latex/l2picfaq/README"] = {["lang"] = "de", ["details"] = "Package README", },
  ["generic/around-the-bend/AroundTheBend.pdf"] = {["details"] = "The document itself", },
  ["latex/thesis-gwu/README.md"] = {["details"] = "Readme", },
  ["latex/css-colors/README.txt"] = {["details"] = "Readme", },
  ["latex/russ/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/theoremref/theoremref-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ekaia/ekaia.pdf"] = {["details"] = "Package documentation", },
  ["latex/regcount/readme"] = {["details"] = "Readme", },
  ["latex/theoremref/README"] = {["details"] = "Readme", },
  ["latex/esvect/esvect.pdf"] = {["details"] = "Package documentation", },
  ["latex/thaispec/thaispec.pdf"] = {["lang"] = "th", ["details"] = "Package documentation (Thai)", },
  ["latex/thaispec/README.md"] = {["details"] = "Readme", },
  ["latex/pst-eucl-translation-bg/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/rubik/rubikpatternsLIST.pdf"] = {["details"] = "List of Rubik patterns", },
  ["fonts/tfrupee/README"] = {["details"] = "Readme", },
  ["latex/booklet/README"] = {["details"] = "Readme", },
  ["generic/pst-rputover/pst-rputover-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/mathspic/sourcecode113.pdf"] = {["details"] = "Perl source listing", },
  ["latex/pdf14/README"] = {["details"] = "Readme", },
  ["latex/texvc/README.md"] = {["details"] = "Readme", },
  ["latex/mathpartir/mathpartir.pdf"] = {["details"] = "Package documentation", },
  ["latex/geometry-de/README"] = {["lang"] = "en", ["details"] = "Package Readme", },
  ["fonts/noto/README"] = {["details"] = "Readme", },
  ["latex/adigraph/README.md"] = {["details"] = "Readme", },
  ["fonts/chemarrow/chemarrow-de.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/textualicomma/textualicomma.pdf"] = {["details"] = "Package documentation", },
  ["latex/textpos/textpos.pdf"] = {["details"] = "Package documentation (PDF)", },
  ["latex/termcal/README"] = {["details"] = "Readme", },
  ["latex/textpos/textpos.html"] = {["details"] = "Version history, etc. (HTML)", },
  ["latex/textpos/niepraschk-eso-pic.pdf"] = {["details"] = "Demo file: textpos with eso-pic", },
  ["latex/textpos/README"] = {["details"] = "Package README", },
  ["latex/algorithm2e/README"] = {["details"] = "Package Readme", },
  ["latex/bgteubner/README"] = {["details"] = "Readme", },
  ["generic/babel-turkish/turkish.pdf"] = {["details"] = "Package documentation", },
  ["latex/fduthesis/fduthesis-code.pdf"] = {["lang"] = "zh", ["details"] = "Code Implementation (Chinese)", },
  ["latex/textopo/textopo.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/xespotcolor/xespotcolor.pdf"] = {["details"] = "Package documentation", },
  ["latex/nfssext-cfr/README"] = {["details"] = "Readme", },
  ["latex/actuarialsymbol/README.md"] = {["details"] = "Readme", },
  ["latex/datetime2-german/README.md"] = {["details"] = "Readme", },
  ["latex/mentis/README"] = {["details"] = "Readme", },
  ["latex/mathastext/mathastexttestalphabets.pdf"] = {["details"] = "Output of test script", },
  ["latex/textgreek/textgreek.pdf"] = {["details"] = "Package documentation", },
  ["latex/footnotehyper/README.md"] = {["details"] = "Readme", },
  ["latex/emarks/emarks.pdf"] = {["details"] = "Package documentation", },
  ["latex/textgreek/README"] = {["details"] = "Readme", },
  ["fonts/zlmtt/zlmtt-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/midpage/midpage.pdf"] = {["details"] = "Package documentation", },
  ["latex/textglos/README"] = {["details"] = "Readme", },
  ["generic/bagpipe/README"] = {["details"] = "Readme", },
  ["latex/textfit/README"] = {["details"] = "Readme", },
  ["latex/textcase/textcase.pdf"] = {["details"] = "Package documentation", },
  ["latex/ftnxtra/README"] = {["details"] = "Readme", },
  ["latex/gsemthesis/gsemthesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/scanpages/README"] = {["details"] = "Readme", },
  ["latex/textcase/README"] = {["details"] = "Readme", },
  ["latex/texshade/tsfaq.pdf"] = {["lang"] = "en", ["details"] = "FAQs about TeXshade", },
  ["plain/metatex/mtpaper.pdf"] = {["details"] = "Package documentation", },
  ["latex/texshade/texshade.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/betababel/betatest.pdf"] = {["details"] = "Package documentation", },
  ["latex/texproposal/texproposal.pdf"] = {["lang"] = "zh", ["details"] = "The document itself", },
  ["latex/patchcmd/patchcmd.pdf"] = {["details"] = "Package documentation", },
  ["support/texosquery/README.md"] = {["details"] = "Readme", },
  ["latex/etoc/README.md"] = {["details"] = "Readme", },
  ["latex/texmate/texmate2manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries-italian/glossaries-italian.pdf"] = {["details"] = "Package documentation", },
  ["context/third/construction-plan/construction-plan-demo.pdf"] = {["details"] = "Example of use", },
  ["latex/datetime2-icelandic/datetime2-icelandic.pdf"] = {["details"] = "Package documentation", },
  ["support/texliveonfly/README"] = {["details"] = "Readme", },
  ["latex/sfg/README"] = {["details"] = "Readme", },
  ["fonts/initials/README"] = {["details"] = "Readme", },
  ["fonts/cm-super/README"] = {["details"] = "Bundle Readme", },
  ["latex/ling-macros/ling-macros-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/stex/mikoslides/README"] = {["details"] = "Readme", },
  ["support/texdoc/texdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex-brochure/README.pdf"] = {["details"] = "Readme (PDF format)", },
  ["latex/juramisc/README"] = {["lang"] = "de", ["details"] = "Package README", },
  ["support/texdirflatten/README"] = {["details"] = "Readme", },
  ["latex/mdwtools/README"] = {["details"] = "Bundle readme", },
  ["xelatex/cqubeamer/cqubeamer.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["latex/psu-thesis/manual.pdf"] = {["details"] = "Package manual", },
  ["metapost/dviincl/README"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["support/texdef/README"] = {["details"] = "Readme", },
  ["latex/dashrule/README"] = {["details"] = "Readme", },
  ["latex/scrlttr2copy/letter-copy-test.pdf"] = {["details"] = "Example document", },
  ["latex/polski/czytaj.txt"] = {["lang"] = "pl", ["details"] = "Readme (Polish)", },
  ["latex/keyvaltable/keyvaltable.pdf"] = {["details"] = "Package documentation", },
  ["generic/texdate/README"] = {["details"] = "Readme", },
  ["latex/fontaxes/fontaxes.pdf"] = {["details"] = "Package documentation", },
  ["support/texcount/doc/TeXcount.pdf"] = {["details"] = "Package documentation", },
  ["support/texcount/doc/QuickReference.pdf"] = {["details"] = "Quick reference", },
  ["support/makedtx/README.txt"] = {["details"] = "Readme", },
  ["latex/pas-crosswords/README"] = {["details"] = "Readme", },
  ["generic/texapi/README"] = {["details"] = "Readme", },
  ["generic/tex4ht/mn.html"] = {["details"] = "Package documentation", },
  ["latex/latexcheat/README"] = {["details"] = "Readme", },
  ["latex/rubik/rubikexamples.pdf"] = {["details"] = "Rubik examples", },
  ["generic/tex-virtual-academy-pl/README"] = {["details"] = "Readme", },
  ["latex/datetime2-ukrainian/README.md"] = {["details"] = "Readme", },
  ["latex/pst-poker/pst-poker-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/multibibliography/multibibliography.pdf"] = {["details"] = "Package documentation", },
  ["latex/12many/README"] = {["details"] = "Readme", },
  ["latex/glossaries/glossariesbegin.pdf"] = {["details"] = "Beginners' guide (PDF)", },
  ["latex/tex-label/tex-label-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/tex-label/tex-label-demo.pdf"] = {["details"] = "Example of use", },
  ["latex/maths-symbols/maths-symbols.pdf"] = {["details"] = "The document itself", },
  ["latex/beamer-FUBerlin/README"] = {["details"] = "Readme", },
  ["latex/dynkin-diagrams/README"] = {["details"] = "Readme", },
  ["latex/tex-label/README"] = {["details"] = "Readme", },
  ["latex/directory/README"] = {["details"] = "Package Readme", },
  ["latex/greek-fontenc/README.html"] = {["details"] = "Readme (HTML)", },
  ["generic/pst-circ/pst-circ-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/gloss/README"] = {["details"] = "Package Readme", },
  ["latex/coordsys/README"] = {["details"] = "Bundle Readme", },
  ["latex/layaureo/layaureo.pdf"] = {["lang"] = "it", ["details"] = "Package documentation", },
  ["fonts/arev/README"] = {["details"] = "Readme", },
  ["latex/correctmathalign/correctmathalign.pdf"] = {["details"] = "Package documentation", },
  ["latex/frege/README"] = {["details"] = "Readme", },
  ["latex/lshort-italian/itlshort.pdf"] = {["lang"] = "it", ["details"] = "The document itself", },
  ["latex/teubner/teubner-doc.pdf"] = {["details"] = "User manual", },
  ["latex/teubner/README"] = {["details"] = "Readme", },
  ["latex/bgreek/README"] = {["details"] = "Readme", },
  ["latex/rrgtrees/README"] = {["details"] = "Package README", },
  ["latex/biblatex-realauthor/README"] = {["details"] = "Readme", },
  ["latex/snapshot/snapshot.pdf"] = {["details"] = "Package documentation", },
  ["latex/testidx/README"] = {["details"] = "Readme", },
  ["latex/chemexec/chemexec_de.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/coseoul/coseoul.pdf"] = {["details"] = "Package documentation", },
  ["latex/termlist/termlist.pdf"] = {["details"] = "Package documentation", },
  ["latex/termlist/README"] = {["details"] = "Readme", },
  ["latex/arabi/README"] = {["details"] = "Readme", },
  ["latex/tensor/README"] = {["details"] = "Package README", },
  ["support/classpack/classpack.pdf"] = {["details"] = "Package documentation", },
  ["latex/tengwarscript/tengtest.pdf"] = {["details"] = "Crib sheet and tests", },
  ["latex/ednotes/ednotes.pdf"] = {["lang"] = "en", ["details"] = "All plain text files, as pdf", },
  ["fonts/tempora/tempora-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/xassoccnt/xassoccnt_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/templatetools/README"] = {["details"] = "Readme", },
  ["latex/templates-sommer/README.de"] = {["lang"] = "de", ["details"] = "README", },
  ["latex/cprotect/README"] = {["details"] = "Readme", },
  ["latex/typehtml/typehtml.pdf"] = {["details"] = "Package documentation", },
  ["metapost/metago/README"] = {["details"] = "Readme", },
  ["latex/tdsfrmath/tdsfrmath.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/tdsfrmath/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["generic/tds/tds.pdf"] = {["details"] = "The standard specification", },
  ["generic/pst-slpe/README"] = {["details"] = "Readme", },
  ["latex/sffms/README"] = {["details"] = "Readme", },
  ["latex/tdclock/tdclock-beamer-example.pdf"] = {["details"] = "Example Usage", },
  ["fonts/erewhon/erewhon-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/currency/currency_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/pstool/README.md"] = {["details"] = "Readme", },
  ["latex/tcolorbox/tcolorbox.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/tcldoc/examples/README.txt"] = {["details"] = "Readme", },
  ["latex/tasks/tasks_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/sffms/sffms_manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-historian/historian.pdf"] = {["details"] = "Package documentation", },
  ["latex/nlctdoc/README"] = {["details"] = "Readme", },
  ["fonts/tapir/readme.pdf"] = {["details"] = "Package documentation", },
  ["fonts/auncial-new/auncial.pdf"] = {["details"] = "Package documentation", },
  ["fonts/tapir/readme"] = {["details"] = "Readme", },
  ["latex/sectionbox/README"] = {["details"] = "Readme", },
  ["bibtex/tamethebeast/ttb_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/tamefloats/README.txt"] = {["details"] = "Readme", },
  ["latex/datetime2-it-fulltext/README"] = {["details"] = "Readme", },
  ["latex/opteng/README"] = {["details"] = "Readme", },
  ["latex/talk/talkdoc.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/arabluatex/arabluatex.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/bibexport/README"] = {["details"] = "Readme", },
  ["latex/jacow/JACoW_A4_LaTeX.pdf"] = {["details"] = "Package documentation", },
  ["latex/istgame/README.txt"] = {["details"] = "Readme", },
  ["latex/engpron/engpron.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/tagpdf/tagpdf.pdf"] = {["details"] = "Package documentation", },
  ["latex/tagpair/README"] = {["details"] = "Readme", },
  ["latex/tagging/tagging.pdf"] = {["details"] = "Package documentation", },
  ["latex/multiaudience/multiaudience.pdf"] = {["details"] = "Package documentation", },
  ["latex/tabvar/tabvar.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["fonts/tinos/README"] = {["details"] = "Readme", },
  ["latex/repltext/README"] = {["details"] = "Readme", },
  ["latex/needspace/needspace.pdf"] = {["details"] = "Package documentation", },
  ["latex/tabularew/tabularew.pdf"] = {["details"] = "Package documentation", },
  ["latex/sectsty/sectsty.pdf"] = {["details"] = "Package documentation", },
  ["latex/units/README"] = {["details"] = "Package README", },
  ["latex/tabularcalc/tabularcalc_doc_en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/tabularborder/tabularborder.pdf"] = {["details"] = "Package documentation", },
  ["latex/tabu/README"] = {["details"] = "Readme", },
  ["latex/grfpaste/dvipaste.txt"] = {["details"] = "Readme-alike", },
  ["latex/tabstackengine/tabstackengine.pdf"] = {["details"] = "Package documentation", },
  ["latex/tabstackengine/README"] = {["details"] = "Readme", },
  ["latex/tabriz-thesis/tabriz-thesis.pdf"] = {["lang"] = "fa", ["details"] = "Package documentation", },
  ["latex/datetime2-danish/README"] = {["details"] = "Readme", },
  ["latex/formular/formular.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-abnt/README.md"] = {["lang"] = "pt-br", ["details"] = "Readme", },
  ["latex/yagusylo/yagusylo-fr.pdf"] = {["lang"] = "fr", ["details"] = "User documentation (French)", },
  ["latex/tablor/README-fr.txt"] = {["lang"] = "fr", ["details"] = "Readme (French)", },
  ["latex/tablists/tablists.pdf"] = {["details"] = "Package documentation", },
  ["latex/listings/listings.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/showhyphens/showhyphens-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/makecell/README"] = {["details"] = "README file", },
  ["latex/gillcm/sample.pdf"] = {["details"] = "Sample of the fonts", },
  ["latex/newunicodechar/newunicodechar.pdf"] = {["details"] = "Package documentation", },
  ["latex/chemstyle/chemstyle.pdf"] = {["details"] = "Package documentation", },
  ["latex/structmech/doc/structmech.pdf"] = {["details"] = "Package documentation", },
  ["latex/tablefootnote/README"] = {["details"] = "Readme", },
  ["latex/tablefootnote/tablefootnote.pdf"] = {["details"] = "Package documentation", },
  ["fonts/libris/README"] = {["details"] = "Readme", },
  ["latex/komacv-rg/komacv-lco.pdf"] = {["details"] = "Package documentation komacv-lco", },
  ["latex/svg-inkscape/InkscapePDFLaTeX.pdf"] = {["details"] = "The document itself", },
  ["latex/chembst/README"] = {["details"] = "Readme", },
  ["latex/ctanify/ctanify.pdf"] = {["details"] = "Package documentation", },
  ["latex/tableof/tableof.pdf"] = {["details"] = "Package documentation", },
  ["latex/nrc/README"] = {["details"] = "Readme", },
  ["latex/oplotsymbl/oPlotSymbl-Manual-en.pdf"] = {["details"] = "Package documentation", },
  ["fonts/romande/README"] = {["details"] = "Readme", },
  ["fonts/gfssolomos/gfssolomos.pdf"] = {["details"] = "Package documentation", },
  ["latex/plantslabels/README"] = {["details"] = "Readme", },
  ["latex/table-fct/README"] = {["details"] = "Readme", },
  ["generic/pst-cox/README"] = {["details"] = "Readme", },
  ["latex/tabfigures/tabfigures.pdf"] = {["details"] = "Package documentation", },
  ["latex/ccaption/ccaption.pdf"] = {["details"] = "Package documentation", },
  ["latex/hletter/Hletter.pdf"] = {["details"] = "Package documentation", },
  ["latex/ksp-thesis/README.md"] = {["details"] = "Readme", },
  ["latex/seealso/seealso.pdf"] = {["details"] = "Package documentation", },
  ["plain/fontch/fontch.pdf"] = {["details"] = "Package documentation", },
  ["generic/t2/etc/rumkidx/README"] = {["details"] = "Readme", },
  ["support/lyluatex/lyluatex.pdf"] = {["details"] = "Package documentation", },
  ["latex/overpic/README.md"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/t-angles/t-manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/synttree/README"] = {["details"] = "Readme", },
  ["latex/syntrace/syntrace.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-czech/datetime2-czech.pdf"] = {["details"] = "Package documentation", },
  ["latex/supertabular/supertabular.pdf"] = {["details"] = "Package documentation", },
  ["latex/ed/ed.pdf"] = {["details"] = "Package documentation", },
  ["latex/blkarray/blkarray.pdf"] = {["details"] = "Package documentation", },
  ["latex/macros2e/README"] = {["details"] = "Readme", },
  ["latex/ejpecp/README"] = {["details"] = "Readme", },
  ["latex/parselines/parselines.pdf"] = {["details"] = "Package documentation", },
  ["latex/syntax/README"] = {["lang"] = "de", ["details"] = "Package README", },
  ["generic/pst-tools/README"] = {["details"] = "Readme", },
  ["latex/synproof/README"] = {["details"] = "Readme", },
  ["latex/sympytexpackage/README"] = {["details"] = "Readme", },
  ["latex/lstaddons/lstlinebgrd.pdf"] = {["details"] = "lstlinebgrd package documentation", },
  ["latex/syllogism/syllogism.pdf"] = {["details"] = "Package documentation", },
  ["ptex/pbibtex/README.md"] = {["details"] = "Readme", },
  ["latex/changebar/README"] = {["details"] = "Readme", },
  ["latex/rcs/rcs-user.pdf"] = {["details"] = "User documentation", },
  ["generic/pst-infixplot/pst-infixplot.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst-calendar/pst-calendar-docDE.pdf"] = {["lang"] = "de", ["details"] = "German documentation:", },
  ["latex/platex-tools/pxftnright.pdf"] = {["lang"] = "ja", ["details"] = "Documentation of the pxftnright package", },
  ["latex/noitcrul/noitcrul.pdf"] = {["lang"] = "de", ["details"] = "Benutzerdoku", },
  ["latex/swebib/README"] = {["details"] = "Readme", },
  ["latex/glossaries-magyar/README"] = {["details"] = "Readme", },
  ["latex/prftree/prftreedoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/listliketab/README"] = {["details"] = "Package Readme", },
  ["fonts/imfellenglish/README"] = {["details"] = "Readme", },
  ["latex/lroundrect/lroundrect.pdf"] = {["details"] = "Package documentation", },
  ["fonts/svrsymbols/README"] = {["details"] = "Readme", },
  ["latex/tikz-feynhand/tikz-feynhand.userguide.pdf"] = {["details"] = "Package documentation", },
  ["fonts/imfellenglish/imfellEnglish.pdf"] = {["details"] = "Package documentation", },
  ["support/svn-multi/svn-multi-pl.pdf"] = {["details"] = "Perl script documentation", },
  ["latex/scrjrnl/README"] = {["details"] = "Readme", },
  ["latex/datetime2-german/datetime2-german.pdf"] = {["details"] = "Package documentation", },
  ["latex/pgf-umlsd/pgf-umlsd-manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/svgcolor/README"] = {["details"] = "Readme", },
  ["latex/svg/svg.pdf"] = {["details"] = "Package documentation", },
  ["generic/arrayjobx/README"] = {["details"] = "Package Readme", },
  ["latex/supertabular/README"] = {["details"] = "Package documentation", },
  ["lualatex/luatodonotes/README.md"] = {["details"] = "Readme", },
  ["latex/gillcm/gillcm.pdf"] = {["details"] = "Package documentation", },
  ["latex/euenc/euenc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/superiors/superiors-doc.pdf"] = {["details"] = "Package documentation", },
  ["support/latexdiff/README"] = {["details"] = "Package Readme", },
  ["fonts/superiors/README"] = {["details"] = "Readme", },
  ["latex/suftesi/suftesi.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["lualatex/combofont/combofont.pdf"] = {["details"] = "Package documentation", },
  ["latex/leaflet/leaflet-manual.pdf"] = {["details"] = "Package manual (as a leaflet)", },
  ["fonts/gfsbodoni/README"] = {["details"] = "Readme", },
  ["fonts/ogham/testfont.pdf"] = {["details"] = "Testfont print", },
  ["latex/fullblck/fullblck.pdf"] = {["details"] = "Package documentation", },
  ["latex/ethiop-t1/README"] = {["details"] = "Readme", },
  ["latex/suftesi/README"] = {["details"] = "Readme", },
  ["latex/etoolbox/README.md"] = {["details"] = "Readme", },
  ["latex/sudokubundle/sudokubundle.pdf"] = {["details"] = "Package documentation", },
  ["latex/sudokubundle/README"] = {["details"] = "Readme", },
  ["latex/hagenberg-thesis/examples/HgbInternshipReport/main.pdf"] = {["lang"] = "de", ["details"] = "Example document / Tutorial (German)", },
  ["generic/pst-func/pst-func-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/delimtxt/delimtxt.pdf"] = {["details"] = "Package documentation", },
  ["latex/leftidx/leftidx.pdf"] = {["details"] = "Package documentation", },
  ["latex/lshort-french/lshort-fr.pdf"] = {["lang"] = "fr", ["details"] = "The document itself", },
  ["latex/sudoku/README"] = {["details"] = "Readme", },
  ["generic/around-the-bend/README"] = {["details"] = "Readme", },
  ["latex/subsupscripts/README"] = {["details"] = "Readme", },
  ["fonts/lobster2/lobster2-samples.pdf"] = {["details"] = "Package documentation", },
  ["latex/substr/README"] = {["details"] = "Readme", },
  ["latex/jpsj/injpsj2.pdf"] = {["details"] = "Instructions for authors", },
  ["latex/subfloat/subfloat.pdf"] = {["details"] = "Package documentation", },
  ["latex/fjodor/fjodor.pdf"] = {["details"] = "Package documentation", },
  ["latex/hanging/README"] = {["details"] = "Readme", },
  ["latex/subfloat/README"] = {["details"] = "Readme", },
  ["latex/subfiles/subfiles.pdf"] = {["details"] = "Package documentation", },
  ["latex/dvdcoll/dvdcoll.pdf"] = {["lang"] = "en", ["details"] = "Class documentation", },
  ["latex/poemscol/poemscolcheatsheet.pdf"] = {["details"] = "Cheat Sheet", },
  ["latex/subfigure/subfigure.pdf"] = {["details"] = "Package documentation", },
  ["latex/breakurl/breakurl.pdf"] = {["details"] = "Package documentation", },
  ["latex/diffcoeff/README.txt"] = {["details"] = "Readme", },
  ["latex/subfigmat/subfigmat-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/stex/rfdmeta/README"] = {["details"] = "Readme", },
  ["latex/wrapfig/multiple-span.txt"] = {["details"] = "How to use wrapfig to span multiple columns", },
  ["latex/subeqnarray/subeqnarray.pdf"] = {["details"] = "Package documentation", },
  ["latex/subeqn/subeqn.pdf"] = {["details"] = "Package documentation", },
  ["latex/subdepth/subdepth.pdf"] = {["details"] = "Package documentation", },
  ["metapost/roundrect/roundrect.pdf"] = {["details"] = "Package documentation", },
  ["fonts/stickstoo/README"] = {["details"] = "Readme", },
  ["latex/uni-wtal-ger/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/studenthandouts/README"] = {["details"] = "Readme", },
  ["latex/datetime2-samin/README"] = {["details"] = "Readme", },
  ["metapost/mp3d/gut2001.pdf"] = {["lang"] = "fr", ["details"] = "Paper on geometry, for Cahiers Gutenberg", },
  ["latex/struktex/struktex.de.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/nopageno/nopageno.pdf"] = {["details"] = "Package documentation", },
  ["metapost/blockdraw_mp/bonddemo.pdf"] = {["details"] = "Example 1", },
  ["latex/mhchem/README"] = {["details"] = "Readme", },
  ["fonts/gfsporson/gfsporson.pdf"] = {["details"] = "Package documentation", },
  ["latex/acroterm/README"] = {["details"] = "Readme", },
  ["generic/mfpic/mfpguide.pdf"] = {["lang"] = "en", ["details"] = "Short introduction", },
  ["latex/cascade/README.md"] = {["details"] = "Readme", },
  ["latex/correctmathalign/README.md"] = {["details"] = "Readme", },
  ["latex/datetime2-esperanto/README"] = {["details"] = "Readme", },
  ["latex/stringstrings/stringstrings.pdf"] = {["details"] = "Package documentation", },
  ["latex/stringstrings/README"] = {["details"] = "Readme", },
  ["latex/circuitikz/circuitikzmanual.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-ovl/pst-ovl-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/savetrees/st-sample2e.pdf"] = {["details"] = "Package showcase", },
  ["latex/storecmd/README"] = {["details"] = "Readme", },
  ["latex/bbm-macros/bbm.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-manuscripts-philology/documentation/biblatex-manuscripts-philology-example.pdf"] = {["details"] = "Example of use", },
  ["latex/makebarcode/README"] = {["details"] = "Readme", },
  ["latex/storebox/README"] = {["details"] = "Readme", },
  ["latex/contracard/contracard.pdf"] = {["details"] = "Package documentation", },
  ["latex/showcharinbox/README"] = {["details"] = "Readme", },
  ["fonts/esrelation/esrelation.pdf"] = {["details"] = "Package documentation", },
  ["latex/stix2/stix2.pdf"] = {["details"] = "Package documentation", },
  ["generic/minifp/minifp.pdf"] = {["details"] = "Package documentation", },
  ["latex/showlabels/showlabels.pdf"] = {["details"] = "Package documentation (PDF)", },
  ["latex/bigints/bigints.pdf"] = {["details"] = "Package documentation", },
  ["latex/imtekda/IMTEKda.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/lpic/instructions.pdf"] = {["details"] = "Package documentation", },
  ["latex/cell/README"] = {["details"] = "Readme", },
  ["generic/pst-fill/README"] = {["details"] = "Package README", },
  ["support/sty2dtx/README"] = {["details"] = "Readme", },
  ["latex/stex/workaddress/README"] = {["details"] = "Readme", },
  ["latex/advdate/advdate.pdf"] = {["details"] = "Package documentation", },
  ["latex/stex/structview/README"] = {["details"] = "Readme", },
  ["latex/carlisle/ltxtable.pdf"] = {["details"] = "Documentation of ltxtable", },
  ["latex/xcookybooky/README"] = {["details"] = "Readme", },
  ["latex/stex/stex/README"] = {["details"] = "Readme", },
  ["latex/stex/sref/README"] = {["details"] = "Readme", },
  ["latex/stex/smglom/README"] = {["details"] = "Readme", },
  ["latex/stex/problem/README"] = {["details"] = "Readme", },
  ["latex/stex/presentation/README"] = {["details"] = "Readme", },
  ["latex/xassoccnt/README"] = {["details"] = "Readme", },
  ["lualatex/lilyglyphs/documentation/lilyglyphs.pdf"] = {["details"] = "Package documentation", },
  ["latex/fifinddo-info/dantev45.htm"] = {["lang"] = "de", ["details"] = "The presentation document", },
  ["xelatex/bidipagegrid/bidipagegrid-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/mpgraphics/mpgraphics.pdf"] = {["details"] = "Package documentation", },
  ["latex/mcaption/mcaption.pdf"] = {["details"] = "Package documentation", },
  ["latex/anonchap/anonchap.pdf"] = {["details"] = "Package documentation", },
  ["latex/ucdavisthesis/example/ucdavisthesis_example_main.pdf"] = {["details"] = "Example of output", },
  ["latex/stex/dcm/README"] = {["details"] = "Readme", },
  ["latex/sidecap/README"] = {["details"] = "Readme", },
  ["latex/cooking-units/cooking-units.pdf"] = {["details"] = "Package documentation", },
  ["latex/interactiveworkbook/documentation/interactiveworkbookmanual.pdf"] = {["details"] = "Package documentation", },
  ["latex/stex/cmath/README"] = {["details"] = "Readme", },
  ["latex/stellenbosch/USthesis-5.0.pdf"] = {["lang"] = "en", ["details"] = "Class/package documentation", },
  ["latex/unitsdef/README.1st"] = {["details"] = "Readme", },
  ["latex/geschichtsfrkl/README"] = {["details"] = "Readme", },
  ["latex/feynmf/README"] = {["details"] = "Package Readme", },
  ["latex/stellenbosch/USnomencl-1.1.pdf"] = {["lang"] = "en", ["details"] = "USnomencl package documentation", },
  ["support/pdfjam/PDFjam-README.html"] = {["details"] = "Readme", },
  ["latex/cool/README"] = {["details"] = "Package README", },
  ["latex/fancyvrb/fvrb-ex.pdf"] = {["details"] = "Example environments", },
  ["latex/stellenbosch/README"] = {["lang"] = "en", ["details"] = "Bundle README", },
  ["latex/datetime2-magyar/datetime2-magyar.pdf"] = {["details"] = "Package documentation", },
  ["latex/stealcaps/README.md"] = {["details"] = "Readme", },
  ["generic/pst-ode/README.md"] = {["details"] = "Readme", },
  ["latex/stdpage/README"] = {["details"] = "Package README", },
  ["latex/stdclsdv/stdclsdv.pdf"] = {["details"] = "Package documentation", },
  ["latex/endnotesj/README.md"] = {["details"] = "Readme", },
  ["latex/mcexam/README.md"] = {["details"] = "Readme", },
  ["latex/pygmentex/README"] = {["details"] = "Readme", },
  ["fonts/staves/README"] = {["details"] = "Package README", },
  ["fonts/nkarta/README"] = {["details"] = "Readme", },
  ["latex/glossaries-english/README"] = {["details"] = "Readme", },
  ["latex/statmath/statmath.pdf"] = {["details"] = "Package documentation", },
  ["generic/plainpkg/plainpkg-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/statistics/README.md"] = {["details"] = "Readme", },
  ["latex/statex/statex-example.pdf"] = {["details"] = "Package documentation (by example)", },
  ["latex/coloring/README"] = {["details"] = "Readme", },
  ["otherformats/startex/guide.pdf"] = {["details"] = "Beginner's guide", },
  ["generic/pst-eps/README"] = {["details"] = "Package README", },
  ["latex/bxnewfont/README.md"] = {["details"] = "Readme", },
  ["fonts/starfont/starfont.pdf"] = {["details"] = "Package documentation", },
  ["latex/sgame/sgame.pdf"] = {["details"] = "Package documentation", },
  ["latex/stanli/stanli.pdf"] = {["details"] = "Package documentation", },
  ["latex/pm-isomath/README"] = {["details"] = "Readme", },
  ["latex/halloweenmath/halloweenmath-doc.pdf"] = {["details"] = "Package overview", },
  ["latex/gmiflink/README"] = {["details"] = "README", },
  ["latex/glossaries-extra/glossaries-extra-manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/dpfloat/dpfloat.pdf"] = {["details"] = "Package documentation", },
  ["latex/exercise/exercise.pdf"] = {["details"] = "Package documentation", },
  ["latex/nfssext-cfr/nfssext-cfr.pdf"] = {["details"] = "Package documentation", },
  ["latex/greek-inputenc/greek-utf8.pdf"] = {["details"] = "Test, demonstrating the range covered", },
  ["latex/stage/stage-documentation.pdf"] = {["details"] = "Package documentation", },
  ["latex/stage/README"] = {["details"] = "Package README", },
  ["fonts/ifsym/ifsym.ps"] = {["lang"] = "de", ["details"] = "Documentation and font tables (German)", },
  ["latex/beamer-verona/beamer-verona.pdf"] = {["details"] = "Package documentation", },
  ["latex/stackengine/README"] = {["details"] = "Readme", },
  ["latex/sseq/sseq.pdf"] = {["details"] = "Package documentation", },
  ["latex/fmtcount/README"] = {["details"] = "Package README", },
  ["latex/relsize/README"] = {["details"] = "Readme", },
  ["support/srcredact/README"] = {["details"] = "Readme", },
  ["bibtex/bookdb/README"] = {["details"] = "Readme", },
  ["latex/srcltx/srcltx.pdf"] = {["details"] = "Package documentation", },
  ["latex/srcltx/README"] = {["details"] = "Package Readme", },
  ["latex/srbook-mem/SerbianBookMem.pdf"] = {["details"] = "Package documentation", },
  ["latex/spverbatim/spverbatim.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-true-citepages-omit/README"] = {["details"] = "Readme", },
  ["generic/iftex/iftex.pdf"] = {["details"] = "Package documentation", },
  ["latex/ntheorem/ntheorem.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["generic/pst-barcode/README.md"] = {["details"] = "Readme", },
  ["latex/morehype/hypertoc.pdf"] = {["details"] = "Package documentation: hypertoc", },
  ["latex/spreadtab/README"] = {["details"] = "Readme", },
  ["latex/splitindex/splitidx.pdf"] = {["details"] = "Package documentation", },
  ["latex/cntformats/cntformats_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/splitbib/README"] = {["details"] = "Readme", },
  ["fonts/esrelation/README"] = {["details"] = "Readme", },
  ["latex/spie/article.pdf"] = {["details"] = "Template and guidelines for authors", },
  ["latex/sphdthesis/SPhdThesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-scottish/datetime2-scottish.pdf"] = {["details"] = "Package documentation", },
  ["latex/objectz/readme"] = {["details"] = "Readme", },
  ["latex/chkfloat/chkfloat.pdf"] = {["details"] = "Package documentation", },
  ["generic/autoaligne/autoaligne-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/combine/combine.pdf"] = {["details"] = "Package documentation", },
  ["luatex/spelling/CHANGES"] = {["details"] = "Record of changes", },
  ["latex/spectralsequences/manual/spectralsequencesmanual.pdf"] = {["details"] = "Package documentation", },
  ["latex/filecontentsdef/README.md"] = {["details"] = "Readme", },
  ["latex/spath3/spath3_code.pdf"] = {["details"] = "Package documentation", },
  ["latex/gmiflink/gmiflink.pdf"] = {["details"] = "Package documentation", },
  ["latex/modref/README"] = {["details"] = "Readme", },
  ["latex/cybercic/README"] = {["details"] = "Readme", },
  ["latex/spath3/README"] = {["details"] = "Readme", },
  ["latex/undolabl/undolabl.pdf"] = {["details"] = "Package documentation", },
  ["latex/leipzig/leipzig.pdf"] = {["details"] = "Package documentation", },
  ["fonts/spark-otf/README.md"] = {["details"] = "Readme", },
  ["latex/truncate/truncate.pdf"] = {["details"] = "Package documentation", },
  ["latex/urcls/README.md"] = {["details"] = "Readme", },
  ["latex/sourceserifpro/sourceserifpro.pdf"] = {["details"] = "Package documentation", },
  ["latex/soup/soup.pdf"] = {["details"] = "Package documentation", },
  ["latex/wasysym/wasysym.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cbfonts/cbgreek.pdf"] = {["details"] = "Package documentation", },
  ["latex/rcsinfo/rcsinfo.pdf"] = {["details"] = "Package documentation", },
  ["latex/todo/todo.pdf"] = {["details"] = "Package documentation", },
  ["latex/microtype/microtype.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["metapost/hatching/README"] = {["details"] = "Readme", },
  ["plain/happy4th/happy4th.pdf"] = {["details"] = "Package output", },
  ["latex/songs/songs.pdf"] = {["details"] = "Package documentation", },
  ["latex/songs/README"] = {["details"] = "Readme", },
  ["latex/stex/omdoc/README"] = {["details"] = "Readme", },
  ["latex/aomart/aomsample.pdf"] = {["details"] = "Sample article", },
  ["latex/grant/grant.pdf"] = {["details"] = "User manual", },
  ["latex/calxxxx-yyyy/cal2018-2035_DE.pdf"] = {["lang"] = "de", ["details"] = "Sample (German) Calendar", },
  ["latex/lshort-estonian/lshort-ee.pdf"] = {["lang"] = "et", ["details"] = "The document itself", },
  ["latex/snotez/README"] = {["details"] = "Readme", },
  ["latex/bibleref-parse/README"] = {["details"] = "Readme", },
  ["latex/smartunits/README.md"] = {["details"] = "Readme", },
  ["latex/datetime2-russian/datetime2-russian.pdf"] = {["details"] = "Package documentation", },
  ["latex/smartdiagram/README"] = {["details"] = "Readme", },
  ["latex/msc/manual.pdf"] = {["details"] = "User manual", },
  ["latex/changes/changes.english.withcode.pdf"] = {["lang"] = "en", ["details"] = "Package documentation, with code listing (English)", },
  ["latex/smalltableof/smalltableof-doc-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["fonts/txuprcal/README"] = {["details"] = "Readme", },
  ["fonts/svrsymbols/svrsymbols.pdf"] = {["details"] = "Package documentation", },
  ["latex/slantsc/slantsc.pdf"] = {["details"] = "Package documentation", },
  ["latex/slantsc/README"] = {["details"] = "Readme", },
  ["latex/sidenotes/README"] = {["details"] = "Readme", },
  ["latex/biblatex-ieee/biblatex-ieee.pdf"] = {["details"] = "Package documentation", },
  ["latex/skrapport/README"] = {["details"] = "Readme", },
  ["latex/filedate/README"] = {["details"] = "Readme", },
  ["latex/chemsec/README.md"] = {["details"] = "Readme", },
  ["latex/skmath/skmath.pdf"] = {["details"] = "Package documentation", },
  ["latex/harpoon/harpoon.pdf"] = {["details"] = "Package documentation", },
  ["latex/sfg/sfg-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/skmath/README"] = {["details"] = "Readme", },
  ["latex/skeyval/README"] = {["details"] = "Readme", },
  ["latex/exceltex/README"] = {["details"] = "Readme", },
  ["latex/skdoc/skdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/skb/skb.pdf"] = {["details"] = "Implementation details", },
  ["latex/skb/README"] = {["details"] = "Readme", },
  ["latex/actuarialangle/README.md"] = {["details"] = "Readme", },
  ["latex/gincltex/gincltex.pdf"] = {["details"] = "Package documentation", },
  ["fonts/drm/specimen.pdf"] = {["details"] = "Font samples (2)", },
  ["fonts/iwona/iwona-info.pdf"] = {["details"] = "Package documentation", },
  ["fonts/skaknew/README"] = {["details"] = "Readme", },
  ["latex/siunitx/siunitx.pdf"] = {["details"] = "Package documentation", },
  ["latex/siunitx/README.md"] = {["details"] = "Readme", },
  ["latex/dottex/dottex.pdf"] = {["details"] = "Package documentation", },
  ["latex/idxcmds/README"] = {["details"] = "Readme", },
  ["lualatex/simurgh/simurgh-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/gentium-tug/README"] = {["details"] = "Readme", },
  ["generic/simplekv/simplekv-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/labbook/labbook.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/simplecv/simplecv.pdf"] = {["details"] = "Package documentation", },
  ["latex/seminar/seminar-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/simplecd/simplecd.pdf"] = {["details"] = "Package documentation", },
  ["latex/simplecd/README"] = {["details"] = "Readme", },
  ["latex/intopdf/intopdf.pdf"] = {["details"] = "Package documentation", },
  ["latex/coseoul/README"] = {["details"] = "Readme", },
  ["xelatex/simple-thesis-dissertation/Thesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/elzcards/elzcards-examples.pdf"] = {["details"] = "Samples", },
  ["xelatex/simple-resume-cv/CV.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/urlbst/urlbst.pdf"] = {["details"] = "Package documentation (PDF format)", },
  ["latex/sides/README"] = {["details"] = "Readme", },
  ["latex/fancytooltips/fancytooltips.pdf"] = {["details"] = "Package documentation", },
  ["latex/sidenotes/sidenotes.pdf"] = {["details"] = "Package documentation", },
  ["latex/skrapport/skrapport.pdf"] = {["details"] = "Package documentation", },
  ["latex/export/00readme"] = {["details"] = "Readme", },
  ["latex/ticollege/ticollege-doc.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/showlabels/showlabels.html"] = {["details"] = "Package documentation (HTML)", },
  ["lualatex/showhyphens/showhyphens-sample.pdf"] = {["details"] = "Sample output", },
  ["latex/biblatex-gb7714-2015/README.md"] = {["details"] = "Readme", },
  ["latex/tablists/README"] = {["details"] = "README file", },
  ["latex/showexpl/showexpl.pdf"] = {["details"] = "Package documentation", },
  ["latex/showexpl/showexpl-test.pdf"] = {["details"] = "Package example", },
  ["latex/show2e/show2e.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/show2e/README"] = {["details"] = "Readme", },
  ["latex/shorttoc/shorttoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-ieee/README.md"] = {["details"] = "Readme", },
  ["latex/pdfpc-movie/README.md"] = {["details"] = "Readme", },
  ["lualatex/selnolig/selnolig.pdf"] = {["details"] = "Package documentation", },
  ["latex/phffullpagefigure/phffullpagefigure.pdf"] = {["details"] = "Package documentation", },
  ["fonts/shobhika/shobhika.pdf"] = {["details"] = "Font samples", },
  ["latex/asypictureb/asypictureB.pdf"] = {["details"] = "Package documentation", },
  ["latex/gitinfo/README"] = {["details"] = "Readme", },
  ["latex/shipunov/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/shdoc/shdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/shdoc/README"] = {["details"] = "Readme", },
  ["latex/datetime2-breton/datetime2-breton.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst-optexp/pst-optexp-DE.pdf"] = {["lang"] = "de", ["details"] = "Package manual (German)", },
  ["fonts/genealogy/README"] = {["details"] = "Readme", },
  ["metapost/shapes/shapes.pdf"] = {["details"] = "Package documentation", },
  ["generic/shapepar/README.shapepar"] = {["details"] = "Package README", },
  ["latex/chemmacros/README"] = {["details"] = "Readme", },
  ["latex/juramisc/doc/jmgerdoc.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/clefval/README"] = {["details"] = "Package README", },
  ["latex/mbenotes/README"] = {["details"] = "Readme", },
  ["latex/shadowtext/README"] = {["details"] = "Readme", },
  ["generic/shade/description.pdf"] = {["lang"] = "en", ["details"] = "Description of the algorithm:", },
  ["latex/serbian-def-cyr/usage.pdf"] = {["details"] = "Package documentation", },
  ["fonts/starfont/README"] = {["details"] = "Readme", },
  ["latex/dnaseq/README"] = {["details"] = "Readme", },
  ["generic/tds/README"] = {["details"] = "Readme", },
  ["latex/multirow/multirow.pdf"] = {["details"] = "Package documentation", },
  ["latex/fast-diagram/help.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["fonts/antt/README"] = {["details"] = "Readme", },
  ["latex/ean13isbn/ean13isbn.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-thai/thai.pdf"] = {["details"] = "Package documentation", },
  ["latex/lipsum/lipsum.pdf"] = {["details"] = "Package documentation", },
  ["latex/ijsra/README.md"] = {["details"] = "Readme", },
  ["xelatex/sexam/README.txt"] = {["details"] = "Readme", },
  ["latex/IEEEconf/IEEEconf.pdf"] = {["details"] = "Package documentation", },
  ["plain/present/README"] = {["details"] = "Readme", },
  ["generic/pst-poly/pst-poly-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/cdpbundl/README"] = {["details"] = "Readme", },
  ["xelatex/fontwrap/fontwrap_example.pdf"] = {["details"] = "Examples of use", },
  ["latex/turabian/userguide.txt"] = {["details"] = "Package documentation", },
  ["latex/achemso/achemso-demo.pdf"] = {["details"] = "Example of use", },
  ["latex/authorindex/authorindex.pdf"] = {["details"] = "Package documentation", },
  ["latex/oberdiek/oberdiek.pdf"] = {["details"] = "Table of contents of the bundle", },
  ["latex/nmbib/README"] = {["details"] = "Readme", },
  ["latex/pdfsync/pdfsync-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/hyperref/options.pdf"] = {["lang"] = "en", ["details"] = "Summary of options", },
  ["latex/serbian-lig/README.txt"] = {["details"] = "Readme", },
  ["generic/shade/README"] = {["details"] = "Readme", },
  ["latex/serbian-def-cyr/README"] = {["details"] = "Readme", },
  ["generic/dcpic/eurotex2001.pdf"] = {["lang"] = "en", ["details"] = "EuroTeX2001 paper", },
  ["latex/serbian-date-lat/README"] = {["details"] = "Readme", },
  ["latex/filehook/filehook.pdf"] = {["details"] = "Package documentation", },
  ["metapost/mpattern/README.PL"] = {["lang"] = "pl", ["details"] = "Package Readme (Polish)", },
  ["plain/cweb/cwebman.pdf"] = {["details"] = "Package documentation", },
  ["latex/serbian-apostrophe/README"] = {["details"] = "Readme", },
  ["latex/toptesi/toptesi-it.pdf"] = {["lang"] = "it", ["details"] = "Class documentation (Italian)", },
  ["latex/sepnum/sepnum-doc.pdf"] = {["details"] = "Package documentation", },
  ["context/third/layout/t-layout.pdf"] = {["details"] = "Package documentation", },
  ["latex/milog/README.md"] = {["details"] = "Readme", },
  ["latex/datetime2-french/README"] = {["details"] = "Readme", },
  ["latex/kotex-utils/README"] = {["details"] = "Readme", },
  ["latex/probsoln/samples/sample.pdf"] = {["details"] = "Example of use", },
  ["latex/lshort-dutch/LEESMIJ"] = {["lang"] = "nl", ["details"] = "Readme", },
  ["latex/seminar/README"] = {["details"] = "Readme", },
  ["platex/jsclasses/README.md"] = {["details"] = "Readme", },
  ["generic/babel-croatian/croatian.pdf"] = {["details"] = "Package documentation", },
  ["latex/rtklage/README"] = {["lang"] = "de", ["details"] = "Readme", },
  ["generic/pst-geometrictools/README.md"] = {["details"] = "Readme", },
  ["latex/addfont/addfont.pdf"] = {["details"] = "Package documentation", },
  ["latex/exam-n/exam-n.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-japanese/japanese-sample.pdf"] = {["lang"] = "ja", ["details"] = "Example of use (Japanese)", },
  ["latex/semantic-markup/semantic-markup.pdf"] = {["details"] = "Package documentation", },
  ["latex/coverpage/ECCV06Sample.pdf"] = {["details"] = "Another sample", },
  ["latex/semantic-markup/README"] = {["details"] = "Readme", },
  ["latex/gentombow/gentombow-ja.pdf"] = {["lang"] = "ja", ["details"] = "Documentation of the gentombow package (Japanese)", },
  ["latex/semantic/semantic.pdf"] = {["details"] = "Package documentation", },
  ["latex/short-math-guide/README"] = {["details"] = "Readme", },
  ["latex/wnri-latex/wnri.pdf"] = {["details"] = "Package documentation", },
  ["latex/tabularew/README"] = {["details"] = "Readme", },
  ["fonts/librebodoni/README"] = {["details"] = "Readme", },
  ["latex/sectionbreak/README.md"] = {["details"] = "Readme", },
  ["latex/python/README"] = {["details"] = "Readme", },
  ["latex/section/section-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/secdot/secdot.pdf"] = {["details"] = "Package documentation", },
  ["latex/sduthesis/sduthesis.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation (Chinese)", },
  ["latex/scrlttr2copy/README.md"] = {["details"] = "Readme", },
  ["latex/scrjrnl/scrjrnl.pdf"] = {["details"] = "Package documentation", },
  ["latex/scrjrnl/example.pdf"] = {["details"] = "Example of use (with lipsum)", },
  ["fonts/chemarrow/chemarrow.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/l3packages/xparse/xparse.pdf"] = {["details"] = "xparse documentation", },
  ["latex/svn-multi/README"] = {["details"] = "Readme", },
  ["latex/cryst/README"] = {["details"] = "Readme", },
  ["latex/screenplay-pkg/screenplay-pkg-example.pdf"] = {["details"] = "Example of use", },
  ["fonts/ptex-fontmaps/README"] = {["details"] = "Readme", },
  ["latex/mpostinl/mpostinl.pdf"] = {["details"] = "Package documentation", },
  ["latex/mcaption/README"] = {["details"] = "Readme", },
  ["latex/screenplay/README"] = {["details"] = "Package README", },
  ["latex/cmdtrack/cmdtrack.pdf"] = {["details"] = "Package documentation", },
  ["latex/eskd/README"] = {["details"] = "Package README", },
  ["latex/scratchx/README.txt"] = {["details"] = "Readme", },
  ["fonts/fontawesome5/README.md"] = {["details"] = "Readme", },
  ["latex/scratch/scratch-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["generic/pst-electricfield/pst-electricfield-docEN.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/handin/example.pdf"] = {["details"] = "Example of use", },
  ["latex/sclang-prettifier/README"] = {["details"] = "Readme", },
  ["latex/pgf-umlcd/pgf-umlcd-manual.pdf"] = {["details"] = "Package demo", },
  ["latex/sciposter/scipostermanual.pdf"] = {["details"] = "Package manual", },
  ["generic/babel-welsh/welsh.pdf"] = {["details"] = "Package documentation (code listing)", },
  ["latex/sciposter/README"] = {["details"] = "Readme", },
  ["latex/scientific-thesis-cover/scientific-thesis-cover.pdf"] = {["details"] = "Package documentation", },
  ["latex/ytableau/ytableau.pdf"] = {["details"] = "Package documentation", },
  ["latex/schwalbe-chess/README"] = {["details"] = "Readme", },
  ["generic/babel-sorbian/lsorbian.pdf"] = {["details"] = "Package documentation (Lower Sorbian)", },
  ["latex/logbox/logbox.pdf"] = {["details"] = "Package documentation", },
  ["latex/schule/schule.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/schule/README_DE"] = {["lang"] = "de", ["details"] = "Readme (German)", },
  ["latex/schule/README"] = {["details"] = "Readme", },
  ["generic/schemata/schemata.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/francais-bst/README"] = {["lang"] = "fr", ["details"] = "Readme (mostly in French)", },
  ["latex/schemabloc/README"] = {["details"] = "Readme", },
  ["latex/hobby/hobby_code.pdf"] = {["details"] = "Annotated code", },
  ["latex/scalerel/README"] = {["details"] = "README", },
  ["latex/lineno/lnosuppl.pdf"] = {["details"] = "Supplementary files as PDF", },
  ["latex/delimseasy/README.txt"] = {["details"] = "Readme", },
  ["latex/scale/README"] = {["details"] = "Readme", },
  ["latex/turnthepage/turnthepage.pdf"] = {["details"] = "Package documentation", },
  ["latex/savetrees/README"] = {["details"] = "README", },
  ["latex/savefnmark/savefnmark.pdf"] = {["details"] = "Package documentation", },
  ["latex/uri/uri.pdf"] = {["details"] = "Package documentation", },
  ["latex/sauerj/README"] = {["details"] = "Readme", },
  ["latex/layouts/README"] = {["details"] = "Readme", },
  ["latex/flashcards/README"] = {["details"] = "Package Readme", },
  ["latex/sasnrdisplay/SASnRdisplay.pdf"] = {["details"] = "Package documentation", },
  ["latex/sapthesis/sapthesis-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/exercises/exercises.pdf"] = {["details"] = "Package documentation (partly in German)", },
  ["fonts/sansmathfonts/sansmathfonts.pdf"] = {["details"] = "Package documentation", },
  ["fonts/sansmathfonts/README"] = {["details"] = "Readme", },
  ["latex/proba/proba.pdf"] = {["details"] = "Package documentation", },
  ["latex/thumby/thumby.pdf"] = {["details"] = "Package documentation", },
  ["latex/sanitize-umlaut/README"] = {["details"] = "Readme", },
  ["latex/fnumprint/fnumprint.pdf"] = {["details"] = "Package documentation", },
  ["latex/spverbatim/README"] = {["details"] = "Readme", },
  ["latex/etoc/etoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/sa-tikz/README"] = {["details"] = "Readme", },
  ["latex/ryethesis/ryesample.pdf"] = {["details"] = "Example output", },
  ["latex/rvwrite/README"] = {["details"] = "Readme", },
  ["fonts/gfsbaskerville/gfsbaskerville.pdf"] = {["details"] = "Package documentation", },
  ["latex/boxedminipage2e/boxedminipage2e.pdf"] = {["details"] = "Package documentation", },
  ["latex/rviewport/rviewport.pdf"] = {["details"] = "Package documentation", },
  ["latex/rutitlepage/rutitlepage.pdf"] = {["details"] = "Package documentation", },
  ["latex/rulercompass/rulercompass_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-trad/README.md"] = {["details"] = "Readme", },
  ["fonts/clearsans/README"] = {["details"] = "Readme", },
  ["latex/beamertheme-focus/demo.pdf"] = {["details"] = "Example of use", },
  ["latex/colourchange/colourchangedoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/pm-isomath/pm-isomath.pdf"] = {["details"] = "Package documentation", },
  ["latex/l2picfaq/l2picfaq.pdf"] = {["lang"] = "de", ["details"] = "The document itself", },
  ["fonts/tfrupee/tfrupee.pdf"] = {["details"] = "Package documentation", },
  ["latex/mailmerge/mailmerge.pdf"] = {["details"] = "Package documentation", },
  ["latex/rubik/rubikpatterns.pdf"] = {["details"] = "Rubikpatterns package documentation", },
  ["latex/rmpage/rmpage-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/guitartabs/guitartabs-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/recipebook/ShortSample.pdf"] = {["details"] = "Example (2)", },
  ["generic/lpform/lpform-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-dart/pst-dart-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-apa/README"] = {["details"] = "Readme", },
  ["latex/underscore/underscore.pdf"] = {["details"] = "Package documentation", },
  ["fonts/rsfso/rsfso-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/xebaposter/README"] = {["details"] = "Readme", },
  ["latex/hexgame/README"] = {["details"] = "Readme", },
  ["latex/dhua/README"] = {["details"] = "Readme", },
  ["fonts/libertinegc/README"] = {["details"] = "Readme", },
  ["fonts/rsfs/README"] = {["details"] = "Readme", },
  ["latex/rsc/rsc.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamerposter/README"] = {["details"] = "Readme", },
  ["generic/babel-slovenian/slovene.pdf"] = {["details"] = "Package documentation", },
  ["fonts/mxedruli/mxeddoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/amsmath/testmath.pdf"] = {["lang"] = "en", ["details"] = "Examples paper", },
  ["latex/rsc/README.md"] = {["details"] = "Readme", },
  ["latex/ednotes/ednotugb.pdf"] = {["lang"] = "en", ["details"] = "TUGboat paper", },
  ["latex/roundbox/README"] = {["details"] = "Readme", },
  ["latex/blowup/README.md"] = {["details"] = "Readme", },
  ["latex/rotpages/README"] = {["details"] = "Readme", },
  ["latex/rotpages/Documentation/rotpages-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/rosario/Rosario.pdf"] = {["details"] = "Package documentation", },
  ["fonts/fbb/README"] = {["details"] = "Readme", },
  ["latex/abntex2/examples/abntex2-modelo-trabalho-academico.pdf"] = {["lang"] = "pt-br", ["details"] = "Thesis template", },
  ["latex/colophon/colophon.pdf"] = {["details"] = "Package documentation", },
  ["latex/dk-bib/README"] = {["details"] = "Readme", },
  ["latex/romannum/README"] = {["details"] = "Readme", },
  ["lualatex/pgfmolbio/pgfmolbio.pdf"] = {["details"] = "Package documentation", },
  ["metapost/metaobj/README"] = {["details"] = "Readme", },
  ["latex/romanneg/romanneg.pdf"] = {["details"] = "Package documentation", },
  ["fonts/romande/romandeadf.pdf"] = {["details"] = "Package documentation", },
  ["latex/gauss/gauss-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/romanbarpagenumber/README"] = {["details"] = "Readme", },
  ["latex/romanbar/romanbar.pdf"] = {["details"] = "Package documentation", },
  ["latex/refcheck/README"] = {["details"] = "Readme", },
  ["latex/romanbar/README"] = {["details"] = "Readme", },
  ["latex/robustindex/robustsample.pdf"] = {["details"] = "Example of use", },
  ["latex/robustcommand/robustcommand.pdf"] = {["lang"] = "de", ["details"] = "Benutzerdoku", },
  ["plain/newsletr/newssamp.pdf"] = {["details"] = "Sample Newsletter", },
  ["latex/h2020proposal/template-ict/template-ict.pdf"] = {["details"] = "Template (ICT)", },
  ["latex/robustcommand/README"] = {["details"] = "Readme", },
  ["latex/musikui/README.txt"] = {["details"] = "Readme", },
  ["fonts/roboto/README"] = {["details"] = "Readme", },
  ["latex/horoscop/README"] = {["details"] = "Readme", },
  ["latex/rubik/rubikcube.pdf"] = {["details"] = "Rubikcube package documentation", },
  ["latex/marginnote/README.txt"] = {["details"] = "Package README", },
  ["latex/rmathbr/rmathbr.pdf"] = {["details"] = "Package documentation", },
  ["generic/pmx/pmx284.pdf"] = {["details"] = "Package manual", },
  ["generic/rlepsf/rlepsdoc.ps"] = {["details"] = "Package documentation", },
  ["latex/register/README"] = {["details"] = "Readme", },
  ["latex/quoting/README"] = {["details"] = "Readme", },
  ["latex/rjlparshap/rjlpshap.pdf"] = {["details"] = "Package documentation", },
  ["latex/rjlparshap/README"] = {["details"] = "Readme", },
  ["platex/pxchfon/README.md"] = {["details"] = "Readme", },
  ["latex/isodoc/isodoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/regexpatch/README"] = {["details"] = "Readme", },
  ["latex/diagmac2/README"] = {["details"] = "Readme", },
  ["latex/impnattypo/impnattypo.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/makecell/makecell.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/revquantum/revquantum.pdf"] = {["details"] = "Package documentation", },
  ["plain/reverxii/reverxii.pdf"] = {["details"] = "Package documentation", },
  ["latex/sauterfonts/README"] = {["details"] = "Readme", },
  ["latex/pdfsync/README"] = {["details"] = "Readme", },
  ["xelatex/resumecls/example/README.md"] = {["details"] = "Readme", },
  ["latex/proba/README"] = {["details"] = "Readme", },
  ["latex/lmake/README"] = {["details"] = "Readme", },
  ["latex/resphilosophica/README"] = {["details"] = "Readme", },
  ["latex/repltext/repltext.pdf"] = {["details"] = "Package documentation", },
  ["latex/tabulars-e/README"] = {["details"] = "Readme", },
  ["support/texdiff/README"] = {["details"] = "Readme", },
  ["latex/repeatindex/README"] = {["details"] = "Readme", },
  ["latex/reotex/README"] = {["details"] = "Readme", },
  ["latex/relsize/relsize-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/lfb/README"] = {["details"] = "Package README", },
  ["metapost/makecirc/README"] = {["details"] = "Readme", },
  ["latex/uni-wtal-lin/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/regstats/README"] = {["details"] = "Readme", },
  ["latex/ntgclass/briefdoc.pdf"] = {["lang"] = "nl", ["details"] = "The brief (letter) class", },
  ["latex/cooltooltips/cooltooltips.pdf"] = {["details"] = "Package documentation", },
  ["latex/register/register.pdf"] = {["details"] = "Package documentation", },
  ["latex/regexpatch/regexpatch.pdf"] = {["details"] = "Package documentation", },
  ["latex/elzcards/elzcards.pdf"] = {["details"] = "Package documentation", },
  ["latex/unamth-template/tesis.pdf"] = {["lang"] = "es", ["details"] = "Filled out sample (Mexican Spanis)", },
  ["latex/nonumonpart/nonumonpart-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/ieeepes/ieeepes_doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/gfsporson/README"] = {["details"] = "Readme", },
  ["latex/exsheets/README"] = {["details"] = "Readme", },
  ["latex/reflectgraphics/reflectgraphics.pdf"] = {["details"] = "Package documentation", },
  ["latex/reflectgraphics/README"] = {["details"] = "Readme", },
  ["metapost/exteps/exteps.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-ukrainian/ukraineb.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-multiple-dm/README"] = {["details"] = "Readme", },
  ["latex/refenums/README.pdf"] = {["details"] = "Package documentation", },
  ["latex/amsmath/README.md"] = {["lang"] = "en", ["details"] = "General README", },
  ["latex/refenums/README.md"] = {["details"] = "Readme", },
  ["latex/refcheck/refdemo.pdf"] = {["details"] = "Package demo", },
  ["latex/romanbar/romanbar-example.pdf"] = {["details"] = "Sample of usage", },
  ["fonts/recycle/README"] = {["details"] = "Readme", },
  ["latex/onlyamsmath/README"] = {["details"] = "Readme", },
  ["plain/reverxii/README"] = {["details"] = "Readme", },
  ["latex/recipecard/README"] = {["details"] = "Package README", },
  ["latex/grid/README"] = {["details"] = "Readme", },
  ["latex/curve2e/README"] = {["details"] = "Readme", },
  ["latex/recipe/sample.pdf"] = {["lang"] = "it", ["details"] = "Sample of output (Italian)", },
  ["latex/easylist/README"] = {["details"] = "Bundle readme", },
  ["latex/ghsystem/README"] = {["details"] = "Readme", },
  ["latex/rec-thy/README"] = {["details"] = "Readme", },
  ["latex/cje/cjeguide.pdf"] = {["details"] = "Package documentation", },
  ["latex/realboxes/realboxes.pdf"] = {["details"] = "Package documentation", },
  ["latex/realboxes/README"] = {["details"] = "Readme", },
  ["fonts/junicode/aelfric_job.pdf"] = {["details"] = "Example text set with Junicode", },
  ["latex/readarray/readarray.pdf"] = {["details"] = "Package documentation", },
  ["latex/idxlayout/idxlayout.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/halloweenmath/00readme.txt"] = {["details"] = "2nd Readme", },
  ["latex/soul/soul.pdf"] = {["details"] = "Package documentation", },
  ["latex/dateiliste/README-en"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/rcs-multi/rcs-multi.pdf"] = {["details"] = "Package documentation", },
  ["latex/europasscv/README.md"] = {["details"] = "Readme", },
  ["latex/l2tabu-spanish/l2tabues.pdf"] = {["lang"] = "es", ["details"] = "The document itself", },
  ["latex/rcs/rcs-conf.pdf"] = {["details"] = "Package internal interfaces", },
  ["latex/rcs/README"] = {["details"] = "Readme", },
  ["latex/braids/README"] = {["details"] = "Readme", },
  ["fonts/countriesofeurope/README"] = {["details"] = "Readme", },
  ["latex/permute/permute.pdf"] = {["details"] = "Package documentation", },
  ["latex/msc/refman.pdf"] = {["details"] = "Reference manual", },
  ["latex/pdftricks2/README"] = {["details"] = "Readme", },
  ["latex/dvdcoll/dcexample.pdf"] = {["details"] = "Example output", },
  ["metapost/mp3d/paper1997corrected.pdf"] = {["lang"] = "en", ["details"] = "Corrected TUGboat paper", },
  ["latex/rccol/README"] = {["details"] = "Readme", },
  ["latex/randtext/README"] = {["details"] = "Readme", },
  ["latex/randomwalk/randomwalk.pdf"] = {["details"] = "Package documentation", },
  ["latex/classics/README"] = {["details"] = "Readme", },
  ["latex/visualpstricks/VisualPSTricks.pdf"] = {["details"] = "Package documentation", },
  ["generic/randomlist/randomlist.pdf"] = {["details"] = "Package documentation", },
  ["latex/randbild/README"] = {["details"] = "Readme (bilingual)", },
  ["latex/eledform/eledform.pdf"] = {["details"] = "Package documentation", },
  ["latex/ran_toks/doc/rantoks_man.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-subseries/README"] = {["details"] = "Readme", },
  ["latex/listings/README"] = {["details"] = "Package Readme", },
  ["latex/ebproof/ebproof.pdf"] = {["details"] = "Package documentation", },
  ["latex/ran_toks/README.md"] = {["details"] = "Readme", },
  ["latex/raleway/raleway.pdf"] = {["details"] = "Package documentation", },
  ["latex/r_und_s/README"] = {["details"] = "Readme", },
  ["latex/translation-ecv-de/ecvde.dtx.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/fjodor/README"] = {["details"] = "Readme", },
  ["latex/normalcolor/normalcolor.pdf"] = {["details"] = "Package documentation", },
  ["support/lua2dox/README"] = {["details"] = "Readme", },
  ["latex/nolbreaks/nolbreaks.pdf"] = {["details"] = "Package documentation", },
  ["fonts/prodint/README"] = {["details"] = "Readme", },
  ["latex/quotmark/quotmark.pdf"] = {["details"] = "Package documentation", },
  ["latex/amsaddr/README"] = {["details"] = "Readme", },
  ["generic/dowith/domore.pdf"] = {["details"] = "Domore documentation", },
  ["xelatex/ptext/ptext.pdf"] = {["lang"] = "fa", ["details"] = "Package documentation (writtent in Persian)", },
  ["latex/tikzinclude/tikzinclude.pdf"] = {["details"] = "Package documentation", },
  ["latex/quicktype/README.txt"] = {["details"] = "Readme", },
  ["xelatex/persian-bib/README"] = {["details"] = "Readme", },
  ["latex/templatetools/templatetools.pdf"] = {["details"] = "Package documentation", },
  ["fonts/quattrocento/README"] = {["details"] = "Readme (the primary documentation)", },
  ["latex/perltex/perltex.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex-referenz/README"] = {["lang"] = "de", ["details"] = "Readme", },
  ["latex/gmp/README"] = {["details"] = "Readme", },
  ["fonts/wnri/old/README"] = {["details"] = "Readme", },
  ["latex/latex-bib-ex/README"] = {["details"] = "Readme", },
  ["latex/qstest/makematch.pdf"] = {["details"] = "makematch Package documentation", },
  ["latex/cntperchap/cntperchap_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/qobitree/README"] = {["details"] = "Package documentation", },
  ["latex/moreenum/README"] = {["details"] = "Readme", },
  ["latex/qcm/qcm.pdf"] = {["details"] = "Package documentation", },
  ["latex/pbsheet/README"] = {["details"] = "Readme", },
  ["latex/qcircuit/qcircuit.pdf"] = {["details"] = "Package documentation", },
  ["latex/qcircuit/README.md"] = {["details"] = "Readme", },
  ["latex/sphack/sphack-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/swimgraf/README"] = {["details"] = "Readme", },
  ["latex/latexbug/README.md"] = {["details"] = "Readme", },
  ["latex/xcolor/xcolor.pdf"] = {["details"] = "Package documentation", },
  ["latex/pythonhighlight/README.md"] = {["details"] = "Readme", },
  ["latex/a5comb/a5comb.pdf"] = {["details"] = "Package documentation", },
  ["latex/sectionbox/sectionboxmanual.pdf"] = {["details"] = "Package documentation", },
  ["latex/pygmentex/demo.pdf"] = {["details"] = "Package documentation", },
  ["fonts/staves/symbols.pdf"] = {["details"] = "List of symbols", },
  ["fonts/pxtxalfa/pxtxalfa.pdf"] = {["details"] = "Package documentation", },
  ["fonts/paratype/paratype.pdf"] = {["details"] = "Package documentation", },
  ["latex/pxpgfmark/README"] = {["details"] = "Readme", },
  ["latex/boxhandler/boxhandler.pdf"] = {["details"] = "Package documentation", },
  ["latex/chess-problem-diagrams/diagram.pdf"] = {["details"] = "Package documentation", },
  ["platex/pxjahyper/README.md"] = {["details"] = "Readme", },
  ["latex/paresse/paresse-ex-en.pdf"] = {["lang"] = "en", ["details"] = "Usage examples (English)", },
  ["generic/pst-vue3d/README"] = {["details"] = "Readme", },
  ["latex/pxcjkcat/README.md"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/lettrine/demo-de.pdf"] = {["lang"] = "de", ["details"] = "Package examples (German)", },
  ["platex/pxbase/README.md"] = {["details"] = "Readme", },
  ["latex/dashbox/dashbox.pdf"] = {["details"] = "Package documentation", },
  ["fonts/paratype/paratype-sample.pdf"] = {["details"] = "Font samples", },
  ["latex/iffont/iffont.pdf"] = {["details"] = "Package documentation", },
  ["plain/impatient-fr/README"] = {["details"] = "Basic project information", },
  ["latex/axessibility/README"] = {["details"] = "Readme", },
  ["xelatex/xecjk/README.md"] = {["details"] = "Readme", },
  ["latex/ptptex/manptp.pdf"] = {["details"] = "Package documentation", },
  ["latex/quoting/quoting.pdf"] = {["details"] = "Package documentation", },
  ["latex/draftwatermark/draftwatermark.pdf"] = {["details"] = "Package documentation", },
  ["fonts/montserrat/README"] = {["details"] = "Readme", },
  ["xelatex/ptext/README"] = {["details"] = "Readme", },
  ["latex/muthesis/README"] = {["details"] = "Readme", },
  ["latex/ntheorem/README"] = {["details"] = "Readme", },
  ["latex/screenplay/test.pdf"] = {["details"] = "Example script", },
  ["support/patgen2-tutorial/patgen2.pdf"] = {["lang"] = "en", ["details"] = "The document itself", },
  ["latex/gloss-occitan/gloss-occitan.pdf"] = {["details"] = "Package documentation", },
  ["latex/psu-thesis/README"] = {["details"] = "Readme", },
  ["metapost/mpcolornames/mpcolornames.pdf"] = {["details"] = "Package documentation", },
  ["generic/pstricks-add/pstricks-add-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/pauldoc/README"] = {["details"] = "Readme", },
  ["latex/halloweenmath/README"] = {["details"] = "Readme", },
  ["fonts/montserrat/montserrat-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pstricks/pst-user.pdf"] = {["details"] = "Package documentation", },
  ["latex/biochemistry-colors/Biochemistry-colors.pdf"] = {["details"] = "Package documentation", },
  ["generic/pstricks/README"] = {["details"] = "Readme", },
  ["plain/font-change/README.txt"] = {["details"] = "Readme", },
  ["latex/pstool/pstool.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst2pdf/pst2pdf-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/pxfonts/pxfontsdoc.pdf"] = {["details"] = "Package documentation (Letter paper format)", },
  ["latex/pst-vowel/pst-vowel.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst-vowel/README"] = {["details"] = "Readme", },
  ["latex/istgame/istgame-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-vehicle/pst-vehicle-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/frame/frame-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/xq/README"] = {["details"] = "Package README", },
  ["generic/pst-osci/pst-osci-doc.pdf"] = {["lang"] = "en", ["details"] = "English documentation", },
  ["latex/pdfreview/pdfreview.pdf"] = {["details"] = "Package documentation", },
  ["latex/h2020proposal/README.txt"] = {["details"] = "Readme", },
  ["generic/pst-uml/README"] = {["details"] = "Readme", },
  ["generic/pst-tvz/pst-tvz-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/pdfoverlay/README.md"] = {["details"] = "Readme", },
  ["generic/ukrhyph/rules_ph.pdf"] = {["lang"] = "uk", ["details"] = "Old hyphenation rules with phonetics", },
  ["generic/pst-thick/pst-thick-doc.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/xfor/README"] = {["details"] = "Readme", },
  ["generic/pst-spirograph/pst-spirograph-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/functan/README"] = {["details"] = "Readme", },
  ["generic/pst-spectra/pst-spectraEN.pdf"] = {["lang"] = "en", ["details"] = "Package outline", },
  ["latex/l3kernel/expl3.pdf"] = {["details"] = "The LaTeX3 Programming Language", },
  ["latex/addlines/README"] = {["details"] = "Readme", },
  ["generic/pst-soroban/pst-soroban-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-solarsystem/pst-solarsystem-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/koma-script-examples/LIESMICH.txt"] = {["lang"] = "de", ["details"] = "German Readme", },
  ["latex/nonumonpart/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["generic/pst-slpe/pst-slpe.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-sigsys/pst-sigsys-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/getfiledate/README"] = {["details"] = "Readme", },
  ["latex/pspicture/README"] = {["details"] = "Package Readme", },
  ["latex/cnltx/README"] = {["details"] = "Readme", },
  ["generic/pst-shell/README.md"] = {["details"] = "Readme", },
  ["latex/lstbayes/examples.pdf"] = {["details"] = "Examples of use", },
  ["fonts/orkhun/rakhimov.pdf"] = {["details"] = "Package documentation", },
  ["latex/seuthesix/seuthesix.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation (Chinese)", },
  ["generic/pst-plot/README"] = {["details"] = "Readme", },
  ["latex/ordinalpt/README"] = {["details"] = "Readme", },
  ["generic/pst-perspective/pst-perspective-docEN.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["context/third/chromato/chromato-demo.pdf"] = {["details"] = "Package usage demo", },
  ["latex/fontspec/README.md"] = {["details"] = "Package README", },
  ["generic/pst-perspective/pst-perspective-doc.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["generic/pst-perspective/README"] = {["details"] = "Readme", },
  ["latex/capt-of/capt-of.pdf"] = {["details"] = "Package documentation", },
  ["latex/dialogl/codialog.pdf"] = {["details"] = "Tugboat article, overhauled", },
  ["latex/biblist/README"] = {["details"] = "Readme", },
  ["latex/pagecont/pagecont.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-macedonian/macedonian.pdf"] = {["details"] = "Package documentation", },
  ["latex/booktabs-fr/f-booktabs.pdf"] = {["details"] = "The document itself", },
  ["latex/adjmulticol/README"] = {["details"] = "Readme", },
  ["latex/biblatex-anonymous/biblatex-anonymous.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst-pdf/README.md"] = {["details"] = "README file", },
  ["fonts/boisik/example/boisik.pdf"] = {["details"] = "Brief outline, in the font", },
  ["generic/pst-func/README.md"] = {["details"] = "Package README", },
  ["latex/floatrow/floatrow-rus.pdf"] = {["lang"] = "ru", ["details"] = "Package documentation (Russian)", },
  ["latex/listofsymbols/README"] = {["details"] = "Readme", },
  ["generic/pst-ovl/README"] = {["details"] = "Readme", },
  ["latex/pst-vectorian/README"] = {["details"] = "Readme", },
  ["latex/MemoirChapStyles/MemoirChapStyles.pdf"] = {["details"] = "The document itself", },
  ["xelatex/xgreek/xgreek.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-nature/biblatex-nature.pdf"] = {["details"] = "Examples of use", },
  ["latex/pst-optexp/pst-optexp-quickref.pdf"] = {["details"] = "Quick reference (cheat-sheet)", },
  ["latex/forest/README"] = {["details"] = "Readme", },
  ["generic/pst-ode/pst-ode-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/stdpage/stdpage.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-node/pst-node-doc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["generic/hyph-utf8/hyph-utf8.pdf"] = {["lang"] = "en", ["details"] = "Main documentation for hyph-utf8", },
  ["latex/tkz-kiviat/TKZdoc-kiviat-main.pdf"] = {["details"] = "Package documentation", },
  ["latex/floatrow/README"] = {["details"] = "Readme", },
  ["xelatex/fontbook/fontbook-freefonts.pdf"] = {["details"] = "Sample usage (free fonts book)", },
  ["generic/pst-mirror/README"] = {["details"] = "Readme", },
  ["latex/fitbox/fitbox.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-magneticfield/pst-magneticfield-docDE.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["fonts/librecaslon/samples.pdf"] = {["details"] = "Font samples", },
  ["fonts/eiad/Leigh_me.txt"] = {["details"] = "Readme", },
  ["latex/flowfram/flowfram.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-magneticfield/README"] = {["details"] = "Readme", },
  ["generic/pst-light3d/pst-light3d-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-lens/pst-lens.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-sigsys/README"] = {["details"] = "Readme", },
  ["latex/pst-layout/pst-layout-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/gridslides/README.md"] = {["details"] = "Readme", },
  ["latex/pst-layout/README"] = {["details"] = "Readme", },
  ["generic/pst-labo/pst-labo-docFR.pdf"] = {["details"] = "French documentation:", },
  ["generic/pst-labo/README"] = {["details"] = "Readme", },
  ["context/third/inifile/inifile-demo.pdf"] = {["details"] = "Example of use", },
  ["generic/pst-jtree/pst-jtree-doc.pdf"] = {["details"] = "Package documentation", },
  ["luatex/gregoriotex/GregorioRef.pdf"] = {["details"] = "Package documentation", },
  ["latex/pecha/README"] = {["details"] = "Package README", },
  ["generic/pst-infixplot/README"] = {["details"] = "Readme", },
  ["generic/xlop/xlop-doc-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/bohr/bohr_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/csbulletin/README"] = {["details"] = "Readme", },
  ["generic/pst-geometrictools/pst-geometrictools-fr-doc.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["generic/pst-geo/README"] = {["details"] = "Readme", },
  ["latex/akktex/README"] = {["details"] = "Bundle README", },
  ["generic/pst-gantt/pst-gantt-doc.pdf"] = {["details"] = "Brief documentation, with examples", },
  ["latex/apa/README"] = {["details"] = "Readme", },
  ["latex/sudoku/sudoku.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-pad/pst-pad-doc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["generic/pst-fractal/pst-fractal-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/booktabs/README"] = {["details"] = "Readme", },
  ["fonts/tex-gyre-math/README"] = {["details"] = "Readme", },
  ["latex/thaienum/README.md"] = {["details"] = "Readme", },
  ["latex/fancybox/README"] = {["details"] = "Readme", },
  ["latex/latex2e-help-texinfo/latex2e.html"] = {["details"] = "HTML conversion of the source (English)", },
  ["latex/acro/acro_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/hep/README"] = {["details"] = "Readme", },
  ["generic/babel-serbianc/README"] = {["details"] = "Readme", },
  ["latex/fixcmex/README"] = {["details"] = "Readme", },
  ["fonts/stickstoo/stickstoo-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-musuos/biblatex-musuos.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst-exa/pst-exa-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-fr3d/pst-fr3d.pdf"] = {["details"] = "Package documentation", },
  ["context/third/gantt/README"] = {["details"] = "Readme", },
  ["latex/prooftrees/prooftrees.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-eucl/README"] = {["details"] = "Readme", },
  ["otherformats/startex/README"] = {["details"] = "Readme", },
  ["latex/sclang-prettifier/sclang-prettifier.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-electricfield/README"] = {["details"] = "Readme", },
  ["latex/bchart/bchart.pdf"] = {["details"] = "Package documentation", },
  ["latex/authorarchive/brucker-authorarchive-2016.pdf"] = {["details"] = "Example", },
  ["latex/adrconv/adrconv.pdf"] = {["details"] = "Implementation details", },
  ["latex/miller/README"] = {["details"] = "Readme", },
  ["latex/rtklage/rtklage.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/trimspaces/README"] = {["details"] = "Readme", },
  ["generic/pst-bar/pst-bar-doc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["generic/pst-contourplot/pst-contourplot-docFR.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["generic/pst-coil/README"] = {["details"] = "Package README", },
  ["generic/pst-circ/README"] = {["details"] = "Readme", },
  ["generic/musixtex/musixdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/colorweb/colorwebfull.pdf"] = {["details"] = "Full package documentation", },
  ["latex/bgteubner/bgteucls.pdf"] = {["lang"] = "de", ["details"] = "Commented source", },
  ["generic/pst-cie/pst-cie-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-cie/README.md"] = {["details"] = "Readme", },
  ["latex/pythontex/pythontex.pdf"] = {["details"] = "Package documentation", },
  ["latex/memoir/README"] = {["details"] = "Readme", },
  ["support/pedigree-perl/doc/pedigree.pdf"] = {["details"] = "Package documentation", },
  ["latex/colorspace/colorspace.pdf"] = {["details"] = "Package documentation", },
  ["latex/karnaugh-map/karnaugh-map.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-calculate/pst-calculate-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-bspline/pst-bspline-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-bspline/README"] = {["details"] = "Readme", },
  ["generic/pst-blur/pst-blur.pdf"] = {["details"] = "Package documentation", },
  ["latex/lexref/lexref.pdf"] = {["details"] = "Package documentation", },
  ["man/man1/epstopdf.man1.pdf"] = {["details"] = "Manual page", },
  ["generic/pst-barcode/pst-barcode-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/delimset/README.txt"] = {["details"] = "Readme", },
  ["generic/chronosys/docchronosys_fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/spreadtab/spreadtab-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/biocon/README"] = {["details"] = "Readme", },
  ["generic/pst-bar/README"] = {["details"] = "Readme", },
  ["generic/pst-arrow/pst-arrow-doc.pdf"] = {["details"] = "Brief documentation, with examples", },
  ["plain/font-change/font-change.pdf"] = {["details"] = "Package documentation", },
  ["latex/smalltableof/README"] = {["lang"] = "fr", ["details"] = "Readme (French)", },
  ["generic/pst-antiprism/README.md"] = {["details"] = "Readme", },
  ["generic/pst-am/README"] = {["details"] = "Readme", },
  ["latex/autonum/README"] = {["details"] = "Readme", },
  ["latex/dinbrief/liesmich"] = {["lang"] = "de", ["details"] = "Package Readme", },
  ["latex/fileinfo/README.pdf"] = {["details"] = "Readme", },
  ["generic/pst-3dplot/pst-3dplot-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/nicefilelist/nicefilelist.pdf"] = {["details"] = "Package documentation", },
  ["latex/drawmatrix/drawmatrix.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-2dplot/pst-2dplot-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/pdfpagediff/doc/pdfpagediff-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-ijsra/biblatex-ijsra.pdf"] = {["details"] = "Package documentation", },
  ["fonts/noto/noto-samples.pdf"] = {["details"] = "Font samples", },
  ["generic/pst-2dplot/README"] = {["details"] = "Readme", },
  ["latex/tikz-timing/README"] = {["details"] = "Readme", },
  ["fonts/mxedruli/README"] = {["details"] = "Readme", },
  ["generic/pst-lens/README"] = {["details"] = "Readme", },
  ["latex/abraces/abraces-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/psgo/psgomanual.pdf"] = {["details"] = "Package documentation", },
  ["latex/psgo/README"] = {["details"] = "Readme", },
  ["latex/gridslides/gridslides.pdf"] = {["details"] = "Package documentation", },
  ["generic/lecturer/LecturerDemo-VisualDoc.pdf"] = {["details"] = "Visual package documentation", },
  ["latex/HA-prosper/HA-prosper.pdf"] = {["details"] = "Package documentation", },
  ["fonts/algolrevived/algolrevived-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/psfragx/README"] = {["details"] = "Readme", },
  ["latex/psfrag-italian/itpfgguide.pdf"] = {["details"] = "The document itself", },
  ["latex/psfrag/pfgguide.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/mversion/mVersion.pdf"] = {["details"] = "Package documentation", },
  ["latex/psfrag/announce.txt"] = {["details"] = "Release notice", },
  ["latex/pgfornament-han/pgfornament-han_manual.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation (Chinese)", },
  ["fonts/gillius/gillius-samples.pdf"] = {["details"] = "Gillius font samples", },
  ["latex/protocol/README"] = {["lang"] = "en", ["details"] = "Bundle readme", },
  ["latex/proposal/eu/README"] = {["details"] = "Readme", },
  ["latex/graphicxbox/examples/grfxbox_tst.pdf"] = {["details"] = "Package example", },
  ["latex/proposal/dfg/report/README"] = {["details"] = "Readme", },
  ["generic/pst-eucl/pst-eucl-doc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["fonts/inconsolata/inconsolata-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/fcolumn/fcolumn.pdf"] = {["details"] = "Package documentation", },
  ["fonts/pacioli/tryfont.ps"] = {["details"] = "Font sample", },
  ["latex/proofread/README"] = {["details"] = "Readme", },
  ["latex/underoverlap/README"] = {["details"] = "Readme", },
  ["latex/progress/README"] = {["details"] = "Readme", },
  ["latex/productbox/productbox.pdf"] = {["details"] = "Package documentation", },
  ["latex/productbox/README"] = {["details"] = "Readme", },
  ["xelatex/quran/README"] = {["details"] = "Readme", },
  ["lualatex/cstypo/README.md"] = {["details"] = "Readme", },
  ["fonts/cochineal/README"] = {["details"] = "Readme", },
  ["latex/latexcheat-esmx/latexsheet-esmx.pdf"] = {["lang"] = "es", ["details"] = "The document itself", },
  ["fonts/adforn/README"] = {["details"] = "Readme", },
  ["latex/semioneside/semioneside.pdf"] = {["details"] = "Package documentation", },
  ["latex/hagenberg-thesis/examples/HgbLabReportDE/main.pdf"] = {["lang"] = "de", ["details"] = "Example document / Tutorial (German)", },
  ["latex/preview/preview.pdf"] = {["details"] = "Package documentation", },
  ["latex/pressrelease/pressrelease.pdf"] = {["details"] = "Package documentation", },
  ["latex/hang/hang.pdf"] = {["details"] = "Package documentation", },
  ["latex/prerex/README"] = {["details"] = "Readme", },
  ["latex/pracjourn/pracjourn.pdf"] = {["details"] = "Package documentation", },
  ["latex/ppr-prv/ppr-prv.pdf"] = {["details"] = "Package documentation", },
  ["latex/faktor/faktor.pdf"] = {["lang"] = "de", ["details"] = "Benutzerdoku", },
  ["latex/bussproofs/BussGuide2.pdf"] = {["details"] = "User Guide", },
  ["latex/powerdot-FUBerlin/README"] = {["details"] = "Readme", },
  ["latex/postcards/README"] = {["details"] = "Readme", },
  ["latex/hecthese/hecthese.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/ucharcat/README"] = {["details"] = "Readme", },
  ["latex/ffslides/README.txt"] = {["details"] = "Readme", },
  ["latex/polynomial/polynomial.pdf"] = {["details"] = "Package documentation", },
  ["latex/polski/sample-polski.pdf"] = {["lang"] = "pl", ["details"] = "Example of use", },
  ["latex/nox/README"] = {["lang"] = "de", ["details"] = "Readme (German)", },
  ["latex/polski/polski.pdf"] = {["lang"] = "pl", ["details"] = "Package documentation (largely in Polish)", },
  ["latex/lccaps/README.md"] = {["details"] = "Readme", },
  ["generic/texdate/texdate.pdf"] = {["details"] = "Package documentation", },
  ["fonts/doublestroke/dsdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/jura/README"] = {["lang"] = "en", ["details"] = "README file", },
  ["latex/serbian-apostrophe/serbian-apostrophe.pdf"] = {["details"] = "Package documentation", },
  ["latex/polexpr/README.md"] = {["details"] = "Readme", },
  ["latex/gender/README"] = {["details"] = "Readme", },
  ["support/pdflatexpicscale/pdflatexpicscale.pdf"] = {["details"] = "Package documentation", },
  ["latex/pgfplots/pgfplots.pdf"] = {["details"] = "Package manual", },
  ["latex/glossaries-dutch/glossaries-dutch.pdf"] = {["details"] = "Package documentation", },
  ["latex/anyfontsize/anyfontsize.pdf"] = {["details"] = "Package documentation", },
  ["latex/poetrytex/README"] = {["details"] = "Readme", },
  ["latex/aobs-tikz/README"] = {["details"] = "Readme", },
  ["latex/poemscol/README"] = {["details"] = "Readme", },
  ["latex/authoraftertitle/authoraftertitle.pdf"] = {["details"] = "Package documentation", },
  ["pmxchords/examples/noel/pasli_ovce_valasi/README"] = {["details"] = "Readme", },
  ["latex/abntex2/examples/abntex2-modelo-relatorio-tecnico.pdf"] = {["lang"] = "pt-br", ["details"] = "Technical report template", },
  ["latex/ebong/eb.pdf"] = {["lang"] = "bn", ["details"] = "Package documentation", },
  ["latex/churchslavonic/churchslavonic-en.pdf"] = {["details"] = "Package documentation", },
  ["pmxchords/chordsRef.pdf"] = {["lang"] = "en", ["details"] = "Reference table (English)", },
  ["pmxchords/README"] = {["details"] = "Readme", },
  ["latex/csquotes/README.md"] = {["details"] = "Package README", },
  ["latex/rmathbr/README"] = {["details"] = "Readme", },
  ["generic/pmx/pmx-install.pdf"] = {["details"] = "Installation details", },
  ["latex/glossaries-serbian/README"] = {["details"] = "Readme", },
  ["latex/rubik/rubikrotation.pdf"] = {["details"] = "Rubikrotation package documentation", },
  ["latex/translations/README"] = {["details"] = "Readme", },
  ["bibtex/economic/README"] = {["details"] = "Readme", },
  ["plain/plstmary/plstmary-doc.pdf"] = {["details"] = "Package documentation and font tables", },
  ["latex/jurabib/docs/german/jbgerdoc.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/framed/framed.pdf"] = {["details"] = "Package documentation", },
  ["plain/plipsum/README"] = {["details"] = "Readme", },
  ["fonts/plex/plex-samples.pdf"] = {["details"] = "Samples", },
  ["fonts/plex/README"] = {["details"] = "Readme", },
  ["latex/turabian/README"] = {["details"] = "Readme", },
  ["latex/exsol/README"] = {["details"] = "Readme", },
  ["latex/bibtopic/bibtopic.pdf"] = {["details"] = "Package documentation", },
  ["latex/platexcheat/sample-jsclasses.pdf"] = {["lang"] = "ja", ["details"] = "Example of use", },
  ["latex/glossaries-french/glossaries-french.pdf"] = {["details"] = "Package documentation", },
  ["latex/indextools/README"] = {["details"] = "Readme", },
  ["latex/platexcheat/platexsheet.pdf"] = {["lang"] = "ja", ["details"] = "The LaTeX cheat sheet", },
  ["latex/platexcheat/README.md"] = {["details"] = "Readme", },
  ["fonts/alkalami/AlkalamiSample.pdf"] = {["details"] = "Font sample", },
  ["latex/jumplines/README"] = {["details"] = "Readme", },
  ["latex/turkmen/README"] = {["details"] = "Readme", },
  ["fonts/obnov/README"] = {["details"] = "Readme", },
  ["latex/platex-tools/plextarray.pdf"] = {["lang"] = "ja", ["details"] = "Documentation of the plextarray package", },
  ["latex/latex-refsheet/LaTeX_RefSheet.pdf"] = {["details"] = "The document itself", },
  ["platex/base/platex-en.pdf"] = {["details"] = "Package documentation", },
  ["latex/cweb-latex/cweb-conf.pdf"] = {["details"] = "Internal interfaces", },
  ["platex/base/README.md"] = {["details"] = "Readme", },
  ["latex/bardiag/bardiag.pdf"] = {["details"] = "Package documentation, sections 1-3", },
  ["latex/ltxkeys/README"] = {["details"] = "Readme", },
  ["latex/plates/plates.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cmll/cmll.pdf"] = {["details"] = "Package documentation", },
  ["latex/visualpstricks/VisuelPSTricks.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["support/epstopdf/README"] = {["details"] = "Readme", },
  ["latex/milog/milog-formular.pdf"] = {["details"] = "Example", },
  ["latex/psfragx/laprintdoc.ps"] = {["details"] = "LaPrint documentation:", },
  ["lualatex/placeat/placeat.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/placeat/README.md"] = {["details"] = "Readme", },
  ["latex/pkgloader/pkgloader.pdf"] = {["details"] = "Package documentation", },
  ["latex/esk/esk.pdf"] = {["details"] = "Package documentation", },
  ["support/pkfix-helper/README"] = {["details"] = "Readme", },
  ["latex/metre/metre.pdf"] = {["details"] = "Package documentation", },
  ["latex/pigpen/pigpendoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/pigpen/README"] = {["details"] = "Readme", },
  ["metapost/piechartmp/README"] = {["details"] = "Readme", },
  ["latex/ieeepes/README"] = {["details"] = "Readme", },
  ["latex/picinpar/picinpar-de.pdf"] = {["lang"] = "de", ["details"] = "Package usage example", },
  ["bibtex/bath-bst/README.md"] = {["details"] = "Readme", },
  ["latex/physics/physics.pdf"] = {["details"] = "Package documentation", },
  ["latex/phonrule/README.md"] = {["details"] = "Readme", },
  ["fonts/phonetic/README"] = {["details"] = "Readme", },
  ["fonts/phonetic/Doc/README"] = {["details"] = "Readme", },
  ["latex/phonenumbers/README"] = {["details"] = "Readme", },
  ["latex/vertbars/vertbars.pdf"] = {["details"] = "Package documentation", },
  ["latex/philex/philexmanual.pdf"] = {["details"] = "Package documentation", },
  ["latex/phfthm/phfthm.pdf"] = {["details"] = "Package documentation", },
  ["latex/phfsvnwatermark/phfsvnwatermark.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/persian-bib/Persian-bib-userguide.pdf"] = {["lang"] = "fa", ["details"] = "Package documentation", },
  ["latex/ctable/README"] = {["details"] = "Readme", },
  ["latex/phfquotetext/phfquotetext.pdf"] = {["details"] = "Package documentation", },
  ["latex/phfquotetext/README.md"] = {["details"] = "Readme", },
  ["latex/phfqit/README.md"] = {["details"] = "Readme", },
  ["latex/phfparen/phfparen.pdf"] = {["details"] = "Package documentation", },
  ["latex/phfparen/README.md"] = {["details"] = "Readme", },
  ["latex/longfigure/longfigure.pdf"] = {["details"] = "Package documentation", },
  ["latex/phffullpagefigure/README.md"] = {["details"] = "Readme", },
  ["fonts/phaistos/glyphTable.pdf"] = {["details"] = "Table of symbols", },
  ["xelatex/fontbook/README"] = {["details"] = "Readme", },
  ["generic/pgfplots/README"] = {["details"] = "Readme", },
  ["latex/frontespizio/frontespizio.pdf"] = {["lang"] = "it", ["details"] = "Package documentation", },
  ["latex/pgfornament-han/xiaoshan-sample.pdf"] = {["lang"] = "zh", ["details"] = "Sample of the Xiaoshan beamer theme", },
  ["latex/przechlewski-book/README"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/ctex/README.md"] = {["details"] = "Readme", },
  ["latex/pgfornament-han/heavenlyclouds-sample.pdf"] = {["lang"] = "zh", ["details"] = "Sample of the HeavenlyClouds beamer theme", },
  ["latex/pgfornament-han/README.md"] = {["details"] = "Readme", },
  ["latex/pgfornament/ornaments.pdf"] = {["details"] = "Package documentation", },
  ["latex/pgfornament/README"] = {["details"] = "Readme", },
  ["latex/beamertheme-detlevcm/beamertheme-detlevcm.pdf"] = {["details"] = "Package documentation", },
  ["latex/pgfopts/pgfopts.pdf"] = {["details"] = "Package documentation", },
  ["latex/getmap/README.md"] = {["details"] = "Readme", },
  ["latex/appendix/appendix.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/pgfmolbio/README"] = {["details"] = "Readme", },
  ["fonts/lfb/example.pdf"] = {["lang"] = "el", ["details"] = "Example of text in lfb", },
  ["latex/basque-date/README"] = {["details"] = "Readme", },
  ["latex/pgfgantt/pgfgantt.pdf"] = {["details"] = "Package documentation", },
  ["latex/pgf-spectra/README"] = {["details"] = "Readme", },
  ["latex/pgf-soroban/pgf-soroban-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex-refsheet/README.md"] = {["details"] = "Readme", },
  ["plain/gfnotation/GFnotation-doc.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/bidihl/bidihl-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/mathabx/mathtest.pdf"] = {["details"] = "Package documentation, including font lists", },
  ["latex/paper/README"] = {["details"] = "Readme", },
  ["latex/glossaries-danish/README"] = {["details"] = "Readme", },
  ["latex/graphicx-psmin/graphicx-psmin.pdf"] = {["details"] = "Package documentation", },
  ["latex/pgf-soroban/README"] = {["details"] = "Readme", },
  ["latex/cmsd/readme"] = {["details"] = "Readme", },
  ["latex/beamer2thesis/beamer2thesis.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (as presentation; in English)", },
  ["latex/canoniclayout/canoniclayout.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-occitan/occitan.pdf"] = {["details"] = "Package documentation", },
  ["latex/arabi-add/README"] = {["details"] = "Readme", },
  ["latex/simplified-latex/README"] = {["details"] = "Readme", },
  ["fonts/dsserif/dsserif-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/mpostinl/README.txt"] = {["details"] = "Readme", },
  ["metapost/mpman-ru/README"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/othelloboard/othelloboard.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/fixlatvian/fixlatvian.pdf"] = {["lang"] = "lv", ["details"] = "Package documentation (Latvian)", },
  ["latex/uspace/uspace.pdf"] = {["details"] = "Package documentation", },
  ["latex/aastex/README.md"] = {["details"] = "Readme", },
  ["latex/cjk/doc/chinese/README"] = {["details"] = "Readme file", },
  ["generic/ifxptex/ifxptex-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/colorwav/README"] = {["details"] = "Readme", },
  ["latex/fullminipage/fullminipage.pdf"] = {["details"] = "Package documentation", },
  ["latex/isomath/isomath-test.pdf"] = {["details"] = "Example of appearance", },
  ["latex/clefval/exemple.pdf"] = {["lang"] = "fr", ["details"] = "Package examples of use (French)", },
  ["latex/fouridx/README"] = {["details"] = "Readme", },
  ["latex/rccol/rccol.pdf"] = {["details"] = "Package documentation", },
  ["latex/limap/limap.pdf"] = {["details"] = "Package documentation", },
  ["asymptote/CAD.pdf"] = {["details"] = "Using Asymptote for 2D CAD", },
  ["latex/uebungsblatt/README"] = {["lang"] = "de", ["details"] = "README", },
  ["latex/delimset/delimset.pdf"] = {["details"] = "Package documentation", },
  ["fonts/amiri/README"] = {["details"] = "Readme", },
  ["latex/datetime2-usorbian/README"] = {["details"] = "Readme", },
  ["latex/ndsu-thesis/README"] = {["details"] = "Readme", },
  ["latex/qtree/qtreenotes.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/lstbayes/lstbayes.pdf"] = {["details"] = "Package documentation", },
  ["metapost/metauml/README"] = {["details"] = "Readme", },
  ["latex/avremu/README"] = {["details"] = "Readme", },
  ["latex/dynblocks/README"] = {["details"] = "Readme", },
  ["latex/perfectcut/perfectcut.pdf"] = {["details"] = "Package documentation", },
  ["latex/algorithmicx/README"] = {["details"] = "Package README", },
  ["latex/perfectcut/README"] = {["details"] = "Readme", },
  ["latex/codicefiscaleitaliano/codicefiscaleitaliano.pdf"] = {["lang"] = "it", ["details"] = "Package documentation (principally Italian)", },
  ["latex/ffslides/ffslides-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamerposter/beamerposter.pdf"] = {["details"] = "Package documentation (PDF)", },
  ["luatex/interpreter/README"] = {["details"] = "Readme", },
  ["latex/snotez/snotez_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/endnotes/endnotes.pdf"] = {["details"] = "Package documentation", },
  ["latex/nevelok/README.md"] = {["details"] = "Readme", },
  ["latex/biblatex-bookinother/README"] = {["details"] = "Readme", },
  ["latex/colourchange/README"] = {["details"] = "Readme", },
  ["latex/pecha/example.pdf"] = {["details"] = "Example usage", },
  ["latex/pst-intersect/pst-intersect-DE.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["lualatex/luacode/luacode.pdf"] = {["details"] = "Package documentation", },
  ["latex/pdfx/README"] = {["details"] = "Readme", },
  ["support/latex2nemeth/README"] = {["details"] = "Readme", },
  ["generic/babel-japanese/japanese.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation (Japanese)", },
  ["latex/cleveref/README"] = {["details"] = "Readme", },
  ["latex/sidecap/sidecap.pdf"] = {["details"] = "Package documentation", },
  ["latex/codesection/README"] = {["details"] = "Readme", },
  ["latex/amslatex-primer/README"] = {["details"] = "Readme", },
  ["latex/modernposter/README.md"] = {["details"] = "Readme", },
  ["latex/pdfscreen/manual-print.pdf"] = {["details"] = "Manual, for printing", },
  ["generic/pst-uml/pst-uml-doc.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/graphics/grfguide.pdf"] = {["details"] = "Bundle documentation", },
  ["latex/numspell/README"] = {["details"] = "Readme", },
  ["latex/flipbook/flipbook-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/datatool/datatool-code.pdf"] = {["details"] = "Documented code of the package", },
  ["plain/present/HowToTalkTeX.pdf"] = {["details"] = "Package documentation, as a presentation", },
  ["latex/harmony/README"] = {["details"] = "Readme", },
  ["latex/dvgloss/dvgloss.pdf"] = {["details"] = "Package documentation", },
  ["generic/german/gerdoc.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/pdfprivacy/README.md"] = {["details"] = "Readme", },
  ["latex/shorttoc/00readme"] = {["details"] = "Package README", },
  ["fonts/berenisadf/berenisadf.pdf"] = {["details"] = "Package documentation", },
  ["fonts/caladea/Caladea-Regular.pdf"] = {["details"] = "Package documentation", },
  ["generic/barr/diaxydoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ecv/README"] = {["details"] = "Readme", },
  ["latex/polski/README"] = {["details"] = "Readme (English)", },
  ["fonts/semaphor/example.pdf"] = {["details"] = "Listing of semaphor letters", },
  ["generic/pst-tvz/README"] = {["details"] = "Readme", },
  ["fonts/clearsans/clear-samples.pdf"] = {["details"] = "Font samples", },
  ["luatex/luaxml/luaxml.pdf"] = {["details"] = "Package documentation", },
  ["latex/pdfmarginpar/pdfmarginpar.pdf"] = {["details"] = "Package documentation", },
  ["latex/poetrytex/poetrytex.pdf"] = {["details"] = "Package documentation", },
  ["support/pdflatexpicscale/README"] = {["details"] = "Readme", },
  ["latex/mcite/mcite.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-chicago/README"] = {["details"] = "Readme and usage outline", },
  ["latex/stellenbosch/USbib-1.0.pdf"] = {["lang"] = "en", ["details"] = "USbib package documentation", },
  ["latex/diagbox/diagbox.pdf"] = {["details"] = "Package documentation (English and Chinese)", },
  ["metapost/blockdraw_mp/cascadedemo.pdf"] = {["details"] = "Example 1", },
  ["latex/datetime2-it-fulltext/datetime2-it-fulltext.pdf"] = {["details"] = "Package documentation", },
  ["latex/ghab/README"] = {["details"] = "Readme", },
  ["support/pdfcrop/README"] = {["details"] = "Readme (including manual)", },
  ["latex/pdfcprot/pdfcprot.pdf"] = {["details"] = "Package documentation", },
  ["support/dtxgen/dtxgen.pdf"] = {["details"] = "Package documentation", },
  ["latex/gmutils/gmutils.pdf"] = {["details"] = "Package documentation", },
  ["latex/umoline/README"] = {["details"] = "Readme", },
  ["latex/gaceta/README"] = {["details"] = "Readme", },
  ["generic/pst-platon/pst-platon-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/pdfcomment/example_math_markup.pdf"] = {["details"] = "Example output: math mode", },
  ["latex/lstbayes/README.md"] = {["details"] = "Readme", },
  ["latex/pkgloader/README"] = {["details"] = "Readme", },
  ["latex/fnspe/README"] = {["details"] = "Readme", },
  ["latex/pdf14/pdf14.pdf"] = {["details"] = "Package documentation", },
  ["latex/easy-todo/easy-todo.pdf"] = {["details"] = "Package documentation", },
  ["latex/tocvsec2/README"] = {["details"] = "Readme", },
  ["latex/bgreek/bgman.pdf"] = {["details"] = "Package documentation", },
  ["latex/breakcites/breakcites.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-phys/README.md"] = {["details"] = "Readme", },
  ["latex/lshort-polish/README"] = {["details"] = "Readme", },
  ["latex/IEEEtran/IEEEtran_HOWTO.pdf"] = {["details"] = "Package documentation", },
  ["latex/qcm/README"] = {["details"] = "Readme", },
  ["latex/pbox/pbox.pdf"] = {["details"] = "Package documentation", },
  ["latex/toolbox/README"] = {["details"] = "Readme", },
  ["latex/latex2e-help-texinfo-spanish/latex2e-es.html"] = {["lang"] = "es", ["details"] = "HTML conversion of the source (Spanish)", },
  ["fonts/Asana-Math/README"] = {["details"] = "Readme", },
  ["latex/pb-diagram/README"] = {["details"] = "Readme", },
  ["latex/minutes/README.md"] = {["details"] = "Readme", },
  ["latex/pawpict/README"] = {["details"] = "Readme", },
  ["metapost/mpattern/mpattern_test.pdf"] = {["details"] = "Package test output", },
  ["latex/biblatex-anonymous/README"] = {["details"] = "Readme", },
  ["latex/l3packages/README.md"] = {["details"] = "Readme", },
  ["latex/biblatex-dw/biblatex-dw.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["ptex/ptex-base/README.md"] = {["details"] = "Readme", },
  ["latex/tocvsec2/tocvsec2.pdf"] = {["details"] = "Package documentation", },
  ["latex/mcmthesis/mcmthesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/cmbright/cmbright.pdf"] = {["details"] = "Package documentation", },
  ["latex/hecthese/hecthese-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/hepthesis/hepthesis.pdf"] = {["details"] = "Package documentation", },
  ["support/arara/arara-manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/textopo/textopo.txt"] = {["details"] = "Notes on the distribution", },
  ["latex/pas-tableur/README"] = {["details"] = "Readme", },
  ["generic/babel-finnish/finnish.pdf"] = {["details"] = "Package documentation", },
  ["latex/pas-cv/pas-cv.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/screenplay/screenplay.pdf"] = {["details"] = "Package documentation", },
  ["latex/errata/README"] = {["details"] = "Readme", },
  ["latex/microtype-de/microtype-DE.pdf"] = {["details"] = "The document itself", },
  ["latex/fullminipage/README"] = {["details"] = "Readme", },
  ["latex/attachfile/README"] = {["details"] = "Readme", },
  ["latex/dyntree/dyntree.pdf"] = {["details"] = "Package documentation", },
  ["context/third/npp-for-context/README.md"] = {["details"] = "Readme", },
  ["omega/antomega/antomega.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/luatodonotes/luatodonotes.pdf"] = {["details"] = "Package documentation", },
  ["latex/jslectureplanner/jslectureplanner.pdf"] = {["details"] = "Package documentation", },
  ["latex/parskip/parskip-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ecgdraw/README"] = {["details"] = "Readme", },
  ["latex/usebib/usebib.pdf"] = {["details"] = "Package documentation", },
  ["metapost/mcf2graph/README"] = {["details"] = "Readme", },
  ["latex/flabels/README"] = {["details"] = "Readme", },
  ["latex/footnoterange/footnoterange.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-turkish/datetime2-turkish.pdf"] = {["details"] = "Package documentation", },
  ["latex/steinmetz/steinmetz.pdf"] = {["details"] = "Package documentation", },
  ["generic/gates/gates-doc.pdf"] = {["details"] = "Package documentation", },
  ["generic/tex-refs/README"] = {["details"] = "Readme", },
  ["latex/paresse/paresse-ex-fr.pdf"] = {["lang"] = "fr", ["details"] = "Usage examples (French)", },
  ["latex/pxgreeks/pxgreeks.pdf"] = {["details"] = "Package documentation", },
  ["latex/paresse/paresse-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/glossaries-english/glossaries-english.pdf"] = {["details"] = "Package documentation", },
  ["generic/langcode/README"] = {["details"] = "Readme", },
  ["bibtex/iopart-num/README"] = {["details"] = "Readme", },
  ["fonts/old-arrows/old-arrows-italian.pdf"] = {["lang"] = "it", ["details"] = "Package documentation (Italian)", },
  ["latex/leftidx/README"] = {["details"] = "Readme", },
  ["latex/hagenberg-thesis/examples/HgbLabReportEN/main.pdf"] = {["lang"] = "de", ["details"] = "Example document / Tutorial (German)", },
  ["metapost/metapost-examples/README"] = {["details"] = "Readme", },
  ["latex/axodraw2/axodraw2-man.pdf"] = {["details"] = "Package documentation", },
  ["latex/esami/README"] = {["details"] = "Readme", },
  ["latex/adigraph/adigraph_documentation.pdf"] = {["details"] = "Documentation", },
  ["fonts/boondox/boondox-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/bracketkey/bracketkey.pdf"] = {["details"] = "Package documentation", },
  ["latex/booktabs-fr/README"] = {["lang"] = "fr", ["details"] = "Readme", },
  ["latex/namespc/README"] = {["details"] = "Readme", },
  ["latex/docmfp/README"] = {["details"] = "Package Readme", },
  ["latex/rulercompass/rulercompass.pdf"] = {["details"] = "Code listing", },
  ["latex/flashcards/flashcards.pdf"] = {["details"] = "Package documentation", },
  ["latex/parades/README"] = {["details"] = "Readme", },
  ["metapost/mpcolornames/README"] = {["details"] = "Readme", },
  ["latex/paracol/README"] = {["details"] = "Readme", },
  ["latex/dithesis/sample.pdf"] = {["details"] = "Package sample, serving as documentation", },
  ["fonts/shobhika/README.md"] = {["details"] = "Readme", },
  ["latex/dateiliste/README"] = {["details"] = "Readme", },
  ["latex/papertex/papertex.pdf"] = {["details"] = "Package documentation", },
  ["latex/loops/README"] = {["details"] = "Readme", },
  ["latex/l3packages/xfrac/xfrac.pdf"] = {["details"] = "xfrac documentation", },
  ["latex/crossword/README"] = {["details"] = "Package Readme", },
  ["latex/amsaddr/amsaddr.pdf"] = {["details"] = "Package documentation", },
  ["latex/dot2texi/dot2texi.pdf"] = {["details"] = "Package documentation", },
  ["latex/lexref/README"] = {["details"] = "Readme", },
  ["latex/bookcover/README"] = {["details"] = "Readme", },
  ["latex/latexbangla/latexbangla.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-basque/basque.pdf"] = {["details"] = "Package documentation", },
  ["latex/papermas/README"] = {["details"] = "Readme", },
  ["latex/papercdcase/papercdcase.pdf"] = {["details"] = "Package documentation", },
  ["latex/pst-pdf/pst-pdf-DE.pdf"] = {["lang"] = "de", ["details"] = "German documentation", },
  ["amstex/base/README"] = {["details"] = "Readme", },
  ["fonts/antt/AntykwaTorunska-doc-en-2_03.pdf"] = {["lang"] = "en", ["details"] = "Documentation in English", },
  ["fonts/marvosym/marvodoc.pdf"] = {["details"] = "Font tables, etc.", },
  ["latex/koma-moderncvclassic/README"] = {["details"] = "Readme", },
  ["xelatex/mathspec/README.txt"] = {["details"] = "Readme", },
  ["latex/GS1/README"] = {["details"] = "Readme", },
  ["latex/pageslts/pageslts.pdf"] = {["details"] = "Package documentation", },
  ["latex/tikzpeople/tikzpeople.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cinzel/cinzel-samples.pdf"] = {["details"] = "Font samples", },
  ["latex/diagnose/README"] = {["details"] = "Package Readme", },
  ["latex/pagenote/pagenote.pdf"] = {["details"] = "Package documentation", },
  ["latex/modref/modref.pdf"] = {["details"] = "Package documentation", },
  ["latex/overlays/overlays.pdf"] = {["details"] = "Package documentation", },
  ["latex/exsheets/exsheets_en.pdf"] = {["details"] = "Package documentation", },
  ["latex/hanging/hanging.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-pdgr/README"] = {["details"] = "Readme", },
  ["latex/pagecolor/pagecolor.pdf"] = {["details"] = "Package documentation", },
  ["fonts/concmath-fonts/README"] = {["details"] = "Package Readme", },
  ["latex/biblatex-oxref/oxyear-doc.pdf"] = {["details"] = "Style documentation for oxyear", },
  ["latex/pagecolor/pagecolor-example.pdf"] = {["details"] = "Example", },
  ["support/ctan-o-mat/README.md"] = {["details"] = "Readme", },
  ["latex/mailing/mailing.pdf"] = {["details"] = "Package documentation", },
  ["latex/dateiliste/README-eo"] = {["lang"] = "eo", ["details"] = "Readme", },
  ["latex/proofread/proofread.pdf"] = {["details"] = "Package documentation", },
  ["fonts/coelacanth/coelacanth-samples.pdf"] = {["details"] = "Package documentation", },
  ["fonts/pacioli/README"] = {["details"] = "Readme", },
  ["latex/datetime/datetime.pdf"] = {["details"] = "Package documentation (PDF)", },
  ["generic/dirtree/dirtree.pdf"] = {["details"] = "Package documentation", },
  ["latex/bibleref-french/bibleref-french.pdf"] = {["lang"] = "en", ["details"] = "Package Manual", },
  ["latex/begingreek/README"] = {["details"] = "Readme", },
  ["latex/pagenote/README"] = {["details"] = "Readme", },
  ["generic/babel-interlingua/interlingua.pdf"] = {["details"] = "Package documentation", },
  ["latex/makeshape/README"] = {["details"] = "Readme", },
  ["fonts/bguq/README"] = {["details"] = "Readme", },
  ["generic/babel-russian/README"] = {["details"] = "Readme", },
  ["latex/adrconv/adrguide.pdf"] = {["lang"] = "de", ["details"] = "User guide", },
  ["latex/bardiag/bardiag1.pdf"] = {["details"] = "Package documentation, section 4", },
  ["latex/arydshln/README"] = {["details"] = "Readme", },
  ["fonts/schulschriften/schulschriften.pdf"] = {["details"] = "Package documentation", },
  ["latex/outlining/outlining.pdf"] = {["details"] = "Package documentation", },
  ["latex/blindtext/README"] = {["details"] = "Readme", },
  ["latex/gcard/gcard.pdf"] = {["details"] = "Package documentation", },
  ["latex/zhlipsum/README.md"] = {["details"] = "Readme", },
  ["latex/algorithms/algorithms.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/txgreeks/txgreeks.pdf"] = {["details"] = "Package documentation", },
  ["latex/ecclesiastic/ecclesiastic.pdf"] = {["details"] = "Package documentation", },
  ["latex/outlines/README"] = {["details"] = "Readme", },
  ["latex/foreign/README"] = {["details"] = "Readme", },
  ["latex/refman/layout_d.pdf"] = {["lang"] = "de", ["details"] = "Layout-Anderungen mit LaTeX", },
  ["latex/biblatex-subseries/documentation/biblatex-subseries.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries-spanish/README"] = {["details"] = "Readme", },
  ["generic/pst-dbicons/pst-dbicons.pdf"] = {["details"] = "Package documentation", },
  ["latex/emisa/README.md"] = {["details"] = "Readme", },
  ["latex/bold-extra/bold-extra.pdf"] = {["details"] = "Package documentation", },
  ["latex/hagenberg-thesis/examples/HgbThesisTutorial/main.pdf"] = {["lang"] = "de", ["details"] = "Example document / Tutorial (German)", },
  ["latex/oscola/oscola.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-qtree/pst-qtree-manual.pdf"] = {["details"] = "Package documentation", },
  ["latex/amslatex-primer/amshelp.pdf"] = {["details"] = "The document itself", },
  ["xelatex/langsci/README.md"] = {["details"] = "README.md", },
  ["latex/latex-via-exemplos/latex-via-exemplos.pdf"] = {["details"] = "The document itself (twoside)", },
  ["latex/fixltxhyph/fixltxhyph.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-icelandic/README.md"] = {["details"] = "Readme", },
  ["latex/textglos/textglos.pdf"] = {["details"] = "Package documentation", },
  ["latex/bookest/bookestdoc-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/glossaries-german/glossaries-german.pdf"] = {["details"] = "Package documentation", },
  ["fonts/dejavu/dejavu-sample.pdf"] = {["details"] = "Font samples", },
  ["latex/gatech-thesis/README"] = {["details"] = "Readme", },
  ["support/mf2pt1/mf2pt1.pdf"] = {["details"] = "Package documentation", },
  ["fonts/nimbus15/nimbus15-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/cfr-initials/cfr-initials.pdf"] = {["details"] = "Package documentation", },
  ["latex/crbox/README"] = {["details"] = "Readme", },
  ["metapost/bpolynomial/bpolynomial.pdf"] = {["details"] = "Package documentation", },
  ["eplain/eplain.pdf"] = {["details"] = "Package documentation (PDF)", },
  ["latex/enumitem-zref/enumitem-zref.pdf"] = {["details"] = "Package documentation", },
  ["latex/lengthconvert/lengthconvert.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/ucharclasses/ucharclasses.pdf"] = {["details"] = "Package documentation", },
  ["latex/bookcover/bookcover-example1.pdf"] = {["details"] = "Examples of use 1", },
  ["latex/opteng/OptEngInstruct.pdf"] = {["details"] = "Package documentation", },
  ["latex/tableaux/exemples.pdf"] = {["lang"] = "fr", ["details"] = "Further examples (comments in French)", },
  ["support/latexindent/README"] = {["details"] = "Readme", },
  ["latex/dashundergaps/README.md"] = {["details"] = "Readme", },
  ["fonts/anonymouspro/AnonymousPro.pdf"] = {["details"] = "Package documentation", },
  ["latex/oplotsymbl/README.md"] = {["details"] = "Readme", },
  ["latex/emf/README"] = {["details"] = "Readme", },
  ["latex/fduthesis/fduthesis-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation  (English)", },
  ["latex/threeparttablex/threeparttablex.pdf"] = {["details"] = "Package documentation", },
  ["latex/bxcalc/bxcalcize.pdf"] = {["details"] = "Package documentation", },
  ["fonts/dingbat/dingbat.pdf"] = {["details"] = "Package details including symbol table", },
  ["latex/ifmslide/ifmman.pdf"] = {["details"] = "Package manual", },
  ["generic/dehyph-exptl/dehyph-exptl.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/fn2end/fn2end.pdf"] = {["details"] = "Package documentation", },
  ["latex/fifinddo-info/LIESMICH.txt"] = {["lang"] = "de", ["details"] = "Readme", },
  ["latex/ctanify/README"] = {["details"] = "Readme", },
  ["fonts/opensans/opensans-samples.pdf"] = {["details"] = "Font samples", },
  ["latex/math-into-latex-4/README"] = {["details"] = "Readme", },
  ["latex/onrannual/README"] = {["details"] = "Readme", },
  ["latex/fcavtex/README"] = {["lang"] = "pt", ["details"] = "Readme (in Brazilian Portuguese)", },
  ["latex/clrscode3e/README"] = {["details"] = "Readme", },
  ["bibtex/biolett-bst/README.txt"] = {["details"] = "Readme", },
  ["generic/hyph-utf8/es/division.pdf"] = {["lang"] = "es", ["details"] = "Paper on word division (Spanish)", },
  ["latex/copyrightbox/copyrightbox.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries/glossary2glossaries.pdf"] = {["details"] = "How to upgrade from glossary.sty (PDF)", },
  ["latex/bibleref-german/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/latex-doc-ptr/README"] = {["details"] = "Readme", },
  ["latex/cooking-units/README.md"] = {["details"] = "Readme", },
  ["latex/onedown/onedown.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/simple-resume-cv/README.md"] = {["details"] = "Readme", },
  ["latex/onedown/README"] = {["details"] = "Readme", },
  ["latex/mnsymbol/README"] = {["details"] = "Readme", },
  ["latex/metaplot/README"] = {["details"] = "Readme", },
  ["latex/enotez/README"] = {["details"] = "Readme", },
  ["latex/anufinalexam/README"] = {["details"] = "Readme", },
  ["latex/ocr-latex/ocr.pdf"] = {["details"] = "Package documentation", },
  ["latex/boolexpr/boolexpr.pdf"] = {["details"] = "Package documentation", },
  ["latex/dtk/README.md"] = {["details"] = "Readme", },
  ["fonts/amsfonts/README"] = {["details"] = "Package readme", },
  ["latex/frletter/README"] = {["details"] = "Readme", },
  ["latex/ecltree/ecltreesample.pdf"] = {["details"] = "Package examples", },
  ["latex/oldstyle/oldstyle.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-azerbaijani/README"] = {["details"] = "Readme", },
  ["fonts/oldstandard/README"] = {["details"] = "Readme", },
  ["latex/l2tabu-italian/l2tabuit.pdf"] = {["lang"] = "it", ["details"] = "The document itself", },
  ["latex/coloring/coloring-doc.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/bidicontour/bidicontour-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/oldlatin/oldlatin.pdf"] = {["details"] = "Package documentation", },
  ["latex/sf298/README.txt"] = {["details"] = "Readme", },
  ["latex/basque-book/basque-book_EUS.pdf"] = {["lang"] = "eu", ["details"] = "Package documentation (Basque)", },
  ["latex/xargs/xargs.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/platex-tools/plextdelarray.pdf"] = {["lang"] = "ja", ["details"] = "Documentation of the plextdelarray package", },
  ["latex/decorule/decorule.pdf"] = {["details"] = "Package documentation", },
  ["latex/ctib/README"] = {["details"] = "Readme", },
  ["latex/lshort-bulgarian/README"] = {["details"] = "Readme", },
  ["latex/eledform/README.md"] = {["details"] = "Readme", },
  ["latex/translation-biblatex-de/README"] = {["details"] = "Readme", },
  ["latex/chextras/README"] = {["details"] = "Readme", },
  ["latex/tkz-graph/readme-tkz-graph.txt"] = {["lang"] = "fr", ["details"] = "Readme (French)", },
  ["latex/nidanfloat/README.md"] = {["details"] = "Readme", },
  ["latex/GS1/GS1.pdf"] = {["details"] = "Package documentation", },
  ["latex/dprogress/README"] = {["details"] = "Readme", },
  ["latex/ghab/ghab-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/jlabels/README"] = {["details"] = "Readme", },
  ["latex/emisa/emisa.pdf"] = {["details"] = "Package documentation", },
  ["latex/fncylab/fncylab.pdf"] = {["details"] = "Package documentation", },
  ["omega/ocherokee/cherokee.pdf"] = {["details"] = "Package documentation", },
  ["latex/ocgx/ocgx-manual-en.pdf"] = {["details"] = "Package documentation", },
  ["support/bundledoc/README"] = {["details"] = "Readme", },
  ["latex/memory/README"] = {["details"] = "Readme", },
  ["latex/ocg-p/README"] = {["details"] = "Readme", },
  ["fonts/obnov/example_obn.pdf"] = {["details"] = "Font samples", },
  ["latex/contour/README"] = {["details"] = "Package Readme", },
  ["latex/biblatex-claves/README"] = {["details"] = "Readme", },
  ["latex/comma/comma.pdf"] = {["details"] = "Package documentation", },
  ["latex/inversepath/README"] = {["details"] = "Readme", },
  ["latex/dox/README.md"] = {["details"] = "Readme", },
  ["latex/pythontex/pythontex_quickstart.pdf"] = {["details"] = "Quick start documentation", },
  ["latex/easyformat/easyformat.pdf"] = {["details"] = "Package documentation", },
  ["latex/objectz/ozguide.pdf"] = {["details"] = "Package documentation", },
  ["context/third/fancybreak/fancybreak-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/musixtex-fonts/README"] = {["details"] = "Readme", },
  ["latex/bibtopic/README"] = {["details"] = "Readme", },
  ["generic/commado/README"] = {["details"] = "Readme", },
  ["latex/gaceta/plantilla-articulo-de-seccion.pdf"] = {["lang"] = "es", ["details"] = "Example 2: how to write an article", },
  ["latex/acmart/sample-manuscript.pdf"] = {["details"] = "Example of use", },
  ["latex/sesamanuel/sesamanuel.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/dithesis/README"] = {["details"] = "Readme", },
  ["latex/memory/memory.pdf"] = {["details"] = "Package documentation", },
  ["latex/nwejm/examples/sample.pdf"] = {["details"] = "Example of use", },
  ["generic/babel-breton/breton.pdf"] = {["details"] = "Package documentation", },
  ["latex/animate/animate.pdf"] = {["details"] = "Package documentation", },
  ["latex/bezos/README"] = {["details"] = "Bundle Readme", },
  ["fonts/fontawesome/fontawesome.pdf"] = {["details"] = "Package documentation", },
  ["latex/bangorexam/bangorexam.pdf"] = {["details"] = "Package documentation", },
  ["latex/cyber/cyber.pdf"] = {["details"] = "Package documentation", },
  ["latex/draftwatermark/README"] = {["details"] = "Readme", },
  ["latex/dprogress/dprogress.pdf"] = {["details"] = "Package documentation", },
  ["support/bib2gls/bib2gls-begin.pdf"] = {["details"] = "Introductory Guide", },
  ["latex/l3build/l3build.pdf"] = {["details"] = "Package documentation", },
  ["latex/fmp/README"] = {["details"] = "Readme", },
  ["latex/metre/README"] = {["details"] = "Readme", },
  ["generic/vntex/vn-min.pdf"] = {["lang"] = "en", ["details"] = "Minimal steps to typeset Vietnamese", },
  ["latex/mfirstuc/mfirstuc-manual.pdf"] = {["details"] = "User manual (PDF)", },
  ["latex/decorule/README"] = {["details"] = "Readme", },
  ["fonts/dantelogo/README"] = {["details"] = "Readme", },
  ["latex/conv-xkv/README.md"] = {["details"] = "Readme", },
  ["latex/algorithms/README"] = {["details"] = "Readme", },
  ["metapost/dviincl/README.pl"] = {["lang"] = "pl", ["details"] = "Readme (Polish)", },
  ["latex/numericplots/NumericPlots.pdf"] = {["details"] = "Package documentation", },
  ["latex/factura/factura.pdf"] = {["lang"] = "es-ve", ["details"] = "Package documentation", },
  ["latex/harveyballs/README"] = {["details"] = "Readme", },
  ["latex/datetime2-irish/datetime2-irish.pdf"] = {["details"] = "Package documentation", },
  ["latex/docsurvey/docsurvey.pdf"] = {["details"] = "The document itself", },
  ["latex/eukdate/eukdate.pdf"] = {["details"] = "Package documentation", },
  ["support/fig4latex/README"] = {["details"] = "Readme", },
  ["context/third/cmttbf/cmttbf-doc.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/cstypo/cstypotest.pdf"] = {["details"] = "Example of use", },
  ["latex/nuc/nuc.pdf"] = {["details"] = "Package documentation", },
  ["latex/chemnum/chemnum_en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/spreadtab/spreadtab-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["luatex/ctablestack/README.md"] = {["details"] = "Readme", },
  ["fonts/ptex-fonts/README.md"] = {["details"] = "Readme", },
  ["latex/bbold/bbold.pdf"] = {["details"] = "Package documentation", },
  ["generic/tex-ini-files/README.md"] = {["details"] = "Readme", },
  ["latex/mcexam/mcexam.pdf"] = {["details"] = "Package documentation", },
  ["latex/ntgclass/rapdoc.pdf"] = {["lang"] = "nl", ["details"] = "The rapport (report) classes", },
  ["latex/ntgclass/classdoc.pdf"] = {["details"] = "Introduction to the classes", },
  ["latex/datetime2-french/datetime2-french.pdf"] = {["details"] = "Package documentation", },
  ["fonts/chivo/Chivo.pdf"] = {["details"] = "Package documentation", },
  ["latex/delim/delim.pdf"] = {["details"] = "Package documentation", },
  ["latex/mwe/README"] = {["details"] = "Readme", },
  ["latex/ntgclass/artdoc.pdf"] = {["lang"] = "nl", ["details"] = "The artikel (article) classes", },
  ["fonts/fpl/README"] = {["details"] = "Readme", },
  ["latex/fancylabel/fancylabel.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex4wp-it/latex4wp-it.pdf"] = {["lang"] = "it", ["details"] = "The document itself", },
  ["latex/cooking/cooking.pdf"] = {["details"] = "Package documentation", },
  ["fonts/drm/drm.pdf"] = {["details"] = "Package documentation", },
  ["latex/typed-checklist/typed-checklist.pdf"] = {["details"] = "Package documentation", },
  ["latex/aiaa/README"] = {["details"] = "Package README", },
  ["latex/tableaux/tableau.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["xelatex/bidipresentation/README"] = {["details"] = "Readme", },
  ["latex/chess-problem-diagrams/README"] = {["details"] = "Readme", },
  ["latex/dramatist/README"] = {["details"] = "Package Readme", },
  ["fonts/greektex/greektexdoc.pdf"] = {["lang"] = "el", ["details"] = "Package documentation", },
  ["latex/fetchcls/fetchcls.pdf"] = {["details"] = "Package documentation", },
  ["latex/ltablex/ltablex.pdf"] = {["details"] = "Package documentation", },
  ["latex/nox/nox.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/datetime2-icelandic/README"] = {["details"] = "Readme", },
  ["generic/xint/README"] = {["details"] = "Readme", },
  ["latex/latexcourse-rug/README"] = {["details"] = "Readme", },
  ["latex/nowidow/README"] = {["details"] = "Readme file", },
  ["fonts/arimo/arimo-samples.pdf"] = {["details"] = "Package documentation", },
  ["latex/marginfix/README"] = {["details"] = "Readme", },
  ["latex/jnuexam/jnuexam.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["latex/chemformula/README"] = {["details"] = "Readme", },
  ["support/latex2man/README"] = {["details"] = "Readme", },
  ["latex/biblatex-oxref/oxref.pdf"] = {["details"] = "Package documentation", },
  ["latex/adjustbox/adjustbox.pdf"] = {["details"] = "Package documentation", },
  ["latex/bondgraphs/bondgraphs.pdf"] = {["details"] = "Package documentation", },
  ["generic/schemata/README"] = {["details"] = "Readme", },
  ["latex/braille/README"] = {["details"] = "Readme", },
  ["latex/dickimaw/dickimaw-novices.pdf"] = {["details"] = "LaTeX for Complete Novices", },
  ["latex/aomart/aomart.pdf"] = {["details"] = "Package documentation", },
  ["latex/nostarch/nssample.pdf"] = {["details"] = "Sample book", },
  ["latex/acronym/README"] = {["details"] = "Package Readme", },
  ["latex/nostarch/nostarch.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex-veryshortguide/veryshortguide-A4-imposed.pdf"] = {["details"] = "The document itself, imposed for 1 sheet of A4 paper", },
  ["latex/he-she/he-she.pdf"] = {["details"] = "Package documentation", },
  ["latex/refman/refman.pdf"] = {["details"] = "Package documentation", },
  ["latex/collref/README.txt"] = {["details"] = "Readme", },
  ["latex/nonfloat/nonfloat.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/cutwin/README"] = {["details"] = "Readme", },
  ["latex/nomentbl/nomentbl.pdf"] = {["details"] = "Package documentation", },
  ["latex/nomentbl/example.pdf"] = {["details"] = "Example of use", },
  ["latex/manuscript/README"] = {["details"] = "Readme", },
  ["latex/nomencl/README"] = {["details"] = "Readme", },
  ["latex/pgf-blur/README"] = {["details"] = "Readme", },
  ["latex/faktor/README"] = {["details"] = "Readme and basic instructions", },
  ["luatex/fontloader-luaotfload/fontloader-luaotfload.pdf"] = {["details"] = "Package documentation", },
  ["latex/eqparbox/eqparbox.pdf"] = {["details"] = "Package documentation", },
  ["latex/fonttable/README"] = {["details"] = "Readme", },
  ["generic/pst-graphicx/README"] = {["details"] = "Readme", },
  ["fonts/cmsrb/README.md"] = {["details"] = "Readme", },
  ["latex/bxjalipsum/README.md"] = {["details"] = "Readme", },
  ["generic/xstring/README"] = {["details"] = "Readme", },
  ["latex/nmbib/nmsample.pdf"] = {["details"] = "Example of use", },
  ["latex/nmbib/nmbib.pdf"] = {["details"] = "Package documentation", },
  ["asymptote/asymptote.pdf"] = {["details"] = "Package documentation", },
  ["latex/exam-n/README"] = {["details"] = "Readme", },
  ["latex/serbian-lig/serbian-lig.pdf"] = {["details"] = "Package documentation", },
  ["fonts/almfixed/almfixed-dtk.pdf"] = {["details"] = "Package documentation", },
  ["context/third/fullpage/README"] = {["details"] = "README", },
  ["latex/ginpenc/README"] = {["details"] = "Readme", },
  ["luatex/chickenize/chickenize.pdf"] = {["details"] = "Package documentation", },
  ["latex/bibleref-german/en-bibleref-german.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/twoup/README"] = {["details"] = "Package Readme", },
  ["latex/mongolian-babel/mongolian.pdf"] = {["details"] = "Package documentation", },
  ["latex/augie/README.augie"] = {["details"] = "Readme", },
  ["latex/asciilist/README.md"] = {["details"] = "Readme", },
  ["latex/biblatex-arthistory-bonn/README.md"] = {["details"] = "Readme", },
  ["latex/yafoot/yafoot-man.pdf"] = {["details"] = "Bundle documentation", },
  ["latex/extract/extract.pdf"] = {["details"] = "Package documentation", },
  ["latex/labyrinth/labyrinth.pdf"] = {["details"] = "Package documentation", },
  ["latex/nicetext/makedoc.pdf"] = {["details"] = "Makedoc documentation", },
  ["latex/hf-tikz/README"] = {["details"] = "Readme", },
  ["generic/chemfig/chemfig-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["latex/isomath/isomath.pdf"] = {["details"] = "Package documentation (in PDF format)", },
  ["latex/graphviz/graphviz.pdf"] = {["details"] = "Package documentation", },
  ["latex/nicetext/README"] = {["details"] = "Readme", },
  ["latex/linegoal/README"] = {["details"] = "Readme", },
  ["generic/xypic/xypdf.pdf"] = {["lang"] = "en", ["details"] = "PDF output using xypdf", },
  ["latex/xtab/xtab.pdf"] = {["details"] = "Package documentation", },
  ["latex/nicefilelist/README"] = {["details"] = "Readme", },
  ["latex/extract/README"] = {["details"] = "Package README", },
  ["latex/copyrightbox/README"] = {["details"] = "Readme", },
  ["latex/IEEEtran/README"] = {["details"] = "Readme", },
  ["latex/fcavtex/manual/fcavtex.pdf"] = {["lang"] = "pt", ["details"] = "Package documentation (Brazilian Portuguese)", },
  ["latex/newverbs/README"] = {["details"] = "Readme", },
  ["latex/tasks/README"] = {["details"] = "Readme", },
  ["latex/newvbtm/newvbtm-man.pdf"] = {["details"] = "Package manual", },
  ["latex/datetime2-bulgarian/README"] = {["details"] = "Readme", },
  ["latex/bookest/bookestdoc-it.pdf"] = {["lang"] = "it", ["details"] = "Package documentation (Italian)", },
  ["latex/amsldoc-vn/amsldoc-vi.pdf"] = {["lang"] = "vi", ["details"] = "For on-screen viewing", },
  ["fonts/newtxtt/README"] = {["details"] = "Readme", },
  ["fonts/newtxsf/README"] = {["details"] = "Readme", },
  ["fonts/newtx/mathnotes.pdf"] = {["details"] = "Notes concerning math fonts", },
  ["fonts/newtx/implementation.pdf"] = {["details"] = "Implementation notes", },
  ["fonts/adfsymbols/README"] = {["details"] = "Readme", },
  ["fonts/auncial-new/README"] = {["details"] = "Readme", },
  ["latex/ccfonts/ccfonts.pdf"] = {["details"] = "Package documentation", },
  ["generic/armenian/manual-e.pdf"] = {["lang"] = "en", ["details"] = "Package manual (English)", },
  ["latex/calculation/README"] = {["details"] = "Readme", },
  ["latex/geometry-de/README-DE"] = {["lang"] = "de", ["details"] = "Package Readme (German)", },
  ["context/third/lettrine/lettrine-doc.pdf"] = {["details"] = "MAPS paper about the package", },
  ["generic/babel-italian/italian.pdf"] = {["details"] = "Package documentation", },
  ["latex/newspaper/README"] = {["details"] = "Readme", },
  ["latex/enumitem-zref/README"] = {["details"] = "Readme", },
  ["fonts/allrunes/README.txt"] = {["details"] = "Package Readme", },
  ["latex/engtlc/itengtlc.pdf"] = {["lang"] = "it", ["details"] = "Package documentation (Italian)", },
  ["latex/impnattypo/impnattypo-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/lcg/lcg.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex-veryshortguide/veryshortguide-Legal-imposed.pdf"] = {["details"] = "The document itself, imposed for 1 sheet of legal paper", },
  ["latex/niceframe/niceframe.pdf"] = {["details"] = "Package documentation", },
  ["latex/arsclassica/ArsClassica.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/bez123/README"] = {["details"] = "Readme", },
  ["fonts/comfortaa/README"] = {["details"] = "Readme", },
  ["latex/cv4tw/README"] = {["details"] = "Readme", },
  ["latex/endofproofwd/README.txt"] = {["details"] = "Readme", },
  ["fonts/newpx/newpxdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/bxcjkjatype/sample-bxcjkjatype-beamer.pdf"] = {["details"] = "Sample: Introductory beamer slides (English and Japanese)", },
  ["latex/savetrees/savetrees.pdf"] = {["details"] = "Package documentation", },
  ["latex/diagnose/diagnose.pdf"] = {["details"] = "Package documentation", },
  ["fonts/newpx/README"] = {["details"] = "Readme", },
  ["latex/bclogo/README"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/esami/doc/esami-doc-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/newfile/newfile.pdf"] = {["details"] = "Package documentation", },
  ["latex/ladder/README.md"] = {["details"] = "Readme", },
  ["latex/listings-ext/listings-ext.pdf"] = {["details"] = "Package documentation", },
  ["latex/cheatsheet/cheatsheet.pdf"] = {["details"] = "Package documentation", },
  ["latex/modiagram/modiagram_en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["generic/babel-dutch/dutch.pdf"] = {["details"] = "Package documentation", },
  ["latex/newenviron/README"] = {["details"] = "Readme", },
  ["latex/facsimile/facsimile.pdf"] = {["details"] = "Package documentation", },
  ["support/asymptote-faq-zh-cn/asymptote-faq-zh-cn.pdf"] = {["lang"] = "zh", ["details"] = "The document itself", },
  ["latex/biblatex-morenames/documentation/example-bookineditor.pdf"] = {["details"] = "Example of use", },
  ["latex/ted/ted.pdf"] = {["details"] = "Package documentation (English)", },
  ["xelatex/bidishadowtext/bidishadowtext-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/newcommand/README"] = {["details"] = "Readme", },
  ["latex/nevelok/nevelok.pdf"] = {["details"] = "Package documentation", },
  ["latex/neuralnetwork/examples/neural-networks-ebook.pdf"] = {["details"] = "Sample of a book using the package", },
  ["xelatex/facture/README"] = {["lang"] = "fr", ["details"] = "Readme (in French)", },
  ["latex/fitbox/README"] = {["details"] = "Readme", },
  ["support/dtxgen/README"] = {["details"] = "README", },
  ["latex/matc3mem/README"] = {["details"] = "Readme", },
  ["fonts/gillius/README"] = {["details"] = "Readme", },
  ["latex/ncclatex/nccnews.pdf"] = {["lang"] = "ru", ["details"] = "Package news", },
  ["latex/beamercolorthemeowl/README.md"] = {["details"] = "Readme", },
  ["latex/custom-bib/README"] = {["details"] = "Readme", },
  ["latex/mmap/README"] = {["details"] = "Readme", },
  ["latex/dateiliste/README-de"] = {["lang"] = "de", ["details"] = "Readme", },
  ["latex/hepparticles/hepparticles.pdf"] = {["details"] = "Package documentation", },
  ["latex/ncclatex/README"] = {["details"] = "Readme", },
  ["latex/navydocs/navydocs.pdf"] = {["details"] = "Package documentation", },
  ["generic/navigator/README"] = {["details"] = "Readme", },
  ["latex/euler/euler.pdf"] = {["details"] = "Package documentation", },
  ["generic/abbr/README"] = {["details"] = "Readme", },
  ["latex/catoptions/README"] = {["details"] = "Readme", },
  ["fonts/crimson/crimson-samples.pdf"] = {["details"] = "Font samples", },
  ["latex/eqnnumwarn/eqnnumwarn.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/bookdb/bookdb.pdf"] = {["details"] = "Package documentation", },
  ["latex/l3packages/xtemplate/xtemplate.pdf"] = {["details"] = "xtemplate documentation", },
  ["latex/natded/extended_doc.pdf"] = {["details"] = "Extended documentation", },
  ["fonts/boisik/README"] = {["details"] = "Readme", },
  ["latex/natbib/natbib.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["latex/clefval/clefval.pdf"] = {["details"] = "Package documentation", },
  ["latex/ftnxtra/ftnxtra.pdf"] = {["details"] = "Package documentation", },
  ["fonts/nanumtype1/README"] = {["details"] = "Readme", },
  ["fonts/fourier/fourier-doc-en.pdf"] = {["details"] = "Package documentation", },
  ["latex/nameauth/README"] = {["details"] = "Readme", },
  ["latex/cesenaexam/cesenaexam_example.pdf"] = {["details"] = "Example document", },
  ["generic/e-french/test/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/cryptocode/README"] = {["details"] = "Readme", },
  ["latex/isodate/isodate.pdf"] = {["details"] = "Package documentation", },
  ["fonts/antiqua/antiqua.txt"] = {["details"] = "Package documentation", },
  ["latex/tocbibind/README"] = {["details"] = "Readme", },
  ["latex/numberedblock/README"] = {["details"] = "Readme", },
  ["xelatex/na-position/na-position.pdf"] = {["lang"] = "ar", ["details"] = "Package documentation (Arabic)", },
  ["latex/mpgraphics/README"] = {["details"] = "Readme", },
  ["omega/antomega/README"] = {["details"] = "Readme", },
  ["latex/aastex/aassymbols.pdf"] = {["details"] = "Symbols for use with AASTeX", },
  ["latex/hrlatex/hrlatex.pdf"] = {["lang"] = "hr", ["details"] = "Package documentation:", },
  ["latex/mycv/README"] = {["details"] = "Readme", },
  ["latex/captdef/captdef.pdf"] = {["details"] = "Package documentation", },
  ["fonts/archaic/README.PRW"] = {["details"] = "A description of Peter Wilson's contribution", },
  ["fonts/hacm/hacmdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/mweights/README"] = {["details"] = "Readme", },
  ["latex/sapthesis/README"] = {["details"] = "Readme", },
  ["latex/mwe/mwe.pdf"] = {["details"] = "Package documentation", },
  ["latex/gentombow/bounddvi.pdf"] = {["lang"] = "ja", ["details"] = "Documentation of the bounddvi package (Japanese)", },
  ["latex/listings/lstdrvrs.pdf"] = {["details"] = "Language drivers details", },
  ["latex/beamertheme-cuerna/README"] = {["details"] = "Readme", },
  ["latex/fragments/README"] = {["details"] = "Readme", },
  ["latex/mwcls/CZYTAJ"] = {["lang"] = "pl", ["details"] = "Package README", },
  ["latex/simplecv/testcv.pdf"] = {["details"] = "Sample of use", },
  ["fonts/coelacanth/README"] = {["details"] = "Readme", },
  ["latex/cjk/examples/README"] = {["details"] = "Readme file", },
  ["latex/fbithesis/README"] = {["details"] = "Readme", },
  ["latex/biblatex-archaeology/README.md"] = {["details"] = "Readme", },
  ["latex/musuos/README"] = {["details"] = "Readme", },
  ["latex/keyfloat/README.txt"] = {["details"] = "Readme", },
  ["latex/bangtex/README.bangtex"] = {["details"] = "Readme", },
  ["latex/mucproc/README.md"] = {["details"] = "Readme", },
  ["latex/mciteplus/README"] = {["details"] = "Readme", },
  ["generic/musixtex/musixdoc/musixdoc.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/urlbst/README"] = {["details"] = "Package Readme", },
  ["fonts/kurier/00readme.pol"] = {["lang"] = "pl", ["details"] = "Outline in Polish", },
  ["latex/inversepath/inversepath.pdf"] = {["details"] = "Package documentation", },
  ["fonts/roboto/roboto-samples.pdf"] = {["details"] = "Package documentation", },
  ["metapost/bpolynomial/README"] = {["details"] = "Readme", },
  ["latex/calxxxx-yyyy/calxxxx-yyyy-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/biocon/manual.pdf"] = {["details"] = "Package documentation", },
  ["fonts/esstix/README"] = {["details"] = "Readme", },
  ["latex/trfsigns/README"] = {["details"] = "Package Readme", },
  ["latex/biblatex-nature/README.md"] = {["details"] = "Readme", },
  ["context/third/filter/filter.txt"] = {["details"] = "Package notes", },
  ["latex/assignment/assignment.pdf"] = {["details"] = "Package documentation, as an assignment", },
  ["generic/multido/multido-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/multidef/multidef.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-abnt/NBR10520-2002_reference.pdf"] = {["details"] = "Examples of use", },
  ["latex/cellprops/cellprops.pdf"] = {["details"] = "Package documentation", },
  ["latex/auto-pst-pdf-lua/auto-pst-pdf-lua-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-en-fulltext/datetime2-en-fulltext.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamertheme-epyt/README"] = {["details"] = "Readme", },
  ["latex/fcltxdoc/fcltxdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/multiaudience/README"] = {["details"] = "Readme", },
  ["latex/multenum/multienum.pdf"] = {["details"] = "Package documentation", },
  ["latex/mucproc/mucproc.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamertheme-epyt/epyt-demo-cn.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation (Chinese)", },
  ["latex/lccaps/lccaps.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/babelbib/tugboat-babelbib.pdf"] = {["details"] = "TUGboat paper about the package", },
  ["latex/custom-bib/makebst.pdf"] = {["details"] = "Package documentation", },
  ["fonts/boisik/example/boisik-idiot.pdf"] = {["lang"] = "cs", ["details"] = "Running text, in the font", },
  ["latex/bxorigcapt/README.md"] = {["details"] = "Readme", },
  ["latex/unicode-math/unimath-symbols.pdf"] = {["details"] = "List of symbols defined by the package", },
  ["latex/msg/msgguide.pdf"] = {["details"] = "User guide", },
  ["latex/msg/README"] = {["details"] = "Readme", },
  ["fonts/baskervillef/baskervillef-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/bitpattern/bitpattern.pdf"] = {["details"] = "Package documentation", },
  ["latex/verbatimbox/README"] = {["details"] = "Readme", },
  ["latex/chletter/README"] = {["details"] = "Readme", },
  ["latex/fancyhdr-it/README"] = {["details"] = "Readme", },
  ["generic/listofitems/listofitems-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation (French)", },
  ["luatex/lualibs/README"] = {["details"] = "Readme", },
  ["latex/abc/README"] = {["details"] = "Readme", },
  ["latex/actuarialsymbol/actuarialsymbol.pdf"] = {["details"] = "Package documentation", },
  ["metapost/mpattern/README"] = {["lang"] = "en", ["details"] = "Package Readme (English)", },
  ["generic/babel-japanese/README.md"] = {["details"] = "Readme", },
  ["latex/sttools/README.md"] = {["details"] = "Readme", },
  ["latex/widetable/README"] = {["details"] = "Readme", },
  ["latex/punk-latex/README"] = {["details"] = "Readme", },
  ["fonts/accanthis/README"] = {["details"] = "Readme", },
  ["latex/cyber/README"] = {["details"] = "Readme", },
  ["latex/movie15/README"] = {["details"] = "README file", },
  ["latex/morewrites/morewrites.pdf"] = {["details"] = "Package documentation", },
  ["fonts/fandol/README"] = {["details"] = "Readme", },
  ["latex/cooking/README"] = {["details"] = "Package Readme", },
  ["luatex/luapackageloader/luapackageloader.pdf"] = {["details"] = "Package documentation", },
  ["latex/dccpaper/dccpaper.pdf"] = {["details"] = "Package documentation", },
  ["latex/tkz-fct/tkz-fct-screen.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/bibleref-german/de-bibleref-german.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["fonts/niceframe-type1/README.md"] = {["details"] = "Readme", },
  ["latex/mandi/mandi.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-philosophy/README"] = {["details"] = "Readme", },
  ["latex/ifplatform/README"] = {["details"] = "Readme", },
  ["latex/chkfloat/README"] = {["details"] = "Readme", },
  ["latex/monofill/README"] = {["details"] = "Readme", },
  ["latex/datetime2-croatian/README"] = {["details"] = "Readme", },
  ["metapost/blockdraw_mp/README"] = {["details"] = "Readme", },
  ["latex/nihbiosketch/README.md"] = {["details"] = "Readme", },
  ["generic/modulus/modulus.pdf"] = {["details"] = "Package documentation", },
  ["latex/poemscol/poemscol.pdf"] = {["details"] = "Documented source", },
  ["latex/float/README"] = {["details"] = "Package Readme", },
  ["latex/mathalfa/mathalfa.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-danish/datetime2-danish.pdf"] = {["details"] = "Package documentation", },
  ["latex/graph35/graph35-fr.pdf"] = {["details"] = "Package documentation in French", },
  ["context/third/context-notes-zh-cn/ctxnotes.pdf"] = {["details"] = "The document itself", },
  ["latex/gnuplottex/README"] = {["details"] = "Readme", },
  ["latex/idxlayout/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/biblatex-nottsclassic/nottsclassic.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex-mr/latex-mr.pdf"] = {["details"] = "Package documentation", },
  ["latex/ecv/template/CV-template_en.pdf"] = {["lang"] = "en", ["details"] = "Template (English)", },
  ["latex/dviinfox/README.md"] = {["details"] = "Readme", },
  ["latex/guitarchordschemes/README"] = {["details"] = "Readme", },
  ["latex/spath3/calligraphy.pdf"] = {["details"] = "Use for 'calligraphy", },
  ["latex/logicpuzzle/README"] = {["details"] = "Readme", },
  ["latex/juraabbrev/juraabbrev.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["context/third/construction-plan/construction-plan-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/mhequ/example.pdf"] = {["details"] = "Examples of package use", },
  ["latex/moderntimeline/moderntimeline.pdf"] = {["details"] = "Package documentation", },
  ["latex/kantlipsum/README.md"] = {["details"] = "Readme", },
  ["latex/captcont/README"] = {["details"] = "Readme", },
  ["fonts/esstix/Esstix.pdf"] = {["details"] = "Package documentation", },
  ["latex/modernposter/modernposter.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex-course/README"] = {["details"] = "Readme", },
  ["latex/unitn-bimrep/README.md"] = {["details"] = "Readme", },
  ["latex/arabtex/readme.txt"] = {["details"] = "Readme", },
  ["generic/gtl/README.md"] = {["details"] = "Readme", },
  ["latex/mnsymbol/MnSymbol.pdf"] = {["details"] = "Package documentation", },
  ["latex/mnras/mnras_guide.pdf"] = {["details"] = "Package documentation", },
  ["support/fontools/README"] = {["details"] = "Bundle readme", },
  ["fonts/grotesq/readme.grotesq"] = {["details"] = "Copyright and licence statement", },
  ["fonts/astro/astrosym.txt"] = {["details"] = "Detailed description", },
  ["fonts/stmaryrd/stmaryrd.pdf"] = {["details"] = "Package and font documentation", },
  ["latex/formation-latex-ul/formation-latex-ul-diapos.pdf"] = {["lang"] = "fr", ["details"] = "The course itself (Slides)", },
  ["fonts/cantarell/cantarell-samples.pdf"] = {["details"] = "Font samples", },
  ["support/mkpic/mkpic.pdf"] = {["details"] = "Package manual page", },
  ["latex/latex-doc-ptr/latex-doc-ptr.pdf"] = {["details"] = "The document itself", },
  ["latex/boxedminipage2e/README"] = {["details"] = "Readme", },
  ["latex/droit-fr/droit-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/mciteplus/mciteplus_code.txt"] = {["details"] = "Details for modifying .bst code", },
  ["support/mkpic/README"] = {["details"] = "Readme", },
  ["latex/perltex/README"] = {["details"] = "Readme", },
  ["latex/morewrites/README.md"] = {["details"] = "Readme", },
  ["generic/mkjobtexmf/mkjobtexmf.txt"] = {["details"] = "Package documentation (text format)", },
  ["generic/babel-russian/russianb.pdf"] = {["details"] = "Package documentation", },
  ["latex/grayhints/README.md"] = {["details"] = "Readme", },
  ["latex/block/block.pdf"] = {["details"] = "Package documentation", },
  ["latex/lshort-dutch/lshort-nl-1.3.pdf"] = {["lang"] = "nl", ["details"] = "The document itself", },
  ["latex/cryptocode/cryptocode.pdf"] = {["details"] = "Package documentation", },
  ["support/mkgrkindex/mkgrkindex.pdf"] = {["details"] = "Package documentation", },
  ["fonts/missaali/missaali.pdf"] = {["details"] = "Package documentation", },
  ["latex/chemformula/chemformula_en.pdf"] = {["details"] = "Package documentation", },
  ["fonts/mintspirit/mintspirit2-samples.pdf"] = {["details"] = "MintSpirit2 samples", },
  ["latex/minted/minted.pdf"] = {["details"] = "Package documentation", },
  ["latex/minted/README"] = {["details"] = "Readme", },
  ["context/third/ruby/README"] = {["details"] = "Readme", },
  ["latex/minitoc/minitoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/meetingmins/README"] = {["details"] = "Readme", },
  ["latex/nrc/userguide.pdf"] = {["details"] = "Package documentation", },
  ["fonts/stix2-otf/README.md"] = {["details"] = "Readme", },
  ["latex/gzt/french/gzt-fr.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/dvdcoll/dvdcoll_de.pdf"] = {["lang"] = "de", ["details"] = "Class documentation", },
  ["generic/minifp/README"] = {["details"] = "Readme", },
  ["xelatex/bidipresentation/sample.pdf"] = {["details"] = "Sample presentation", },
  ["latex/minidocument/minidocument.pdf"] = {["details"] = "Package documentation", },
  ["latex/minidocument/README"] = {["details"] = "Readme", },
  ["latex/cachepic/cachepic.pdf"] = {["details"] = "Package documentation", },
  ["latex/csquotes-de/LIESMICH"] = {["lang"] = "de", ["details"] = "Readme", },
  ["fonts/cjk-gs-integrate/README.md"] = {["details"] = "Readme", },
  ["latex/europasscv/europasscv.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-en-fulltext/README"] = {["details"] = "Readme", },
  ["metapost/cmarrows/README"] = {["details"] = "Readme", },
  ["generic/commado/commado.pdf"] = {["details"] = "Package documentation", },
  ["latex/drawmatrix/README.md"] = {["details"] = "Readme", },
  ["latex/makeglos/README"] = {["details"] = "Readme", },
  ["latex/webguide/webguide.pdf"] = {["details"] = "The document, as PDF", },
  ["fonts/gfssolomos/README"] = {["details"] = "Readme", },
  ["latex/biblatex-oxref/README.md"] = {["details"] = "Readme", },
  ["latex/cals/test/README"] = {["details"] = "Readme", },
  ["latex/chemstyle/README"] = {["details"] = "Readme", },
  ["latex/csvsimple/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["generic/e-french/manuels/faq.pdf"] = {["lang"] = "fr", ["details"] = "Foire aux Questions", },
  ["latex/pgfkeyx/README"] = {["details"] = "Readme", },
  ["support/latexpand/README"] = {["details"] = "Readme", },
  ["latex/Tabbing/Tabbing.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["generic/es-tex-faq/FAQ-CervanTeX.pdf"] = {["lang"] = "es", ["details"] = "The document itself (PDF)", },
  ["latex/undolabl/undolabl-example.pdf"] = {["details"] = "Package examples", },
  ["latex/lshort-thai/lsh132.pdf"] = {["details"] = "The document itself", },
  ["latex/tablefootnote/tablefootnote-example.pdf"] = {["details"] = "Demonstration and outline of use", },
  ["latex/biblatex-archaeology/biblatex-archaeology.pdf"] = {["details"] = "Package documentation", },
  ["latex/mhchem/mhchem.pdf"] = {["details"] = "Package documentation", },
  ["latex/lineno/lineno.pdf"] = {["details"] = "Package documentation:", },
  ["fonts/calligra-type1/README"] = {["details"] = "Readme", },
  ["latex/mftinc/README"] = {["details"] = "Readme", },
  ["latex/bibleref-lds/bibleref-lds.pdf"] = {["details"] = "Package documentation", },
  ["latex/expdlist/expdlisg.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/struktex/README.md"] = {["details"] = "Readme", },
  ["latex/zxjafont/README.md"] = {["lang"] = "en", ["details"] = "Readme", },
  ["fonts/punknova/README"] = {["details"] = "Readme", },
  ["latex/a0poster/a0.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/beameraudience/README"] = {["details"] = "Readme", },
  ["latex/othelloboard/README"] = {["details"] = "Readme", },
  ["latex/jlabels/jlabels.pdf"] = {["details"] = "Package documentation", },
  ["fonts/wasy/wasydoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/mfirstuc/mfirstuc-code.pdf"] = {["details"] = "Package code and documentation (PDF)", },
  ["latex/babel/README.md"] = {["details"] = "Readme", },
  ["latex/computational-complexity/README"] = {["details"] = "Readme", },
  ["latex/pas-cours/README"] = {["details"] = "Readme", },
  ["latex/mfirstuc/README"] = {["details"] = "Readme", },
  ["latex/metrix/metrix.pdf"] = {["details"] = "Package documentation", },
  ["latex/metrix/README"] = {["details"] = "Readme", },
  ["plain/pitex/README"] = {["details"] = "Readme", },
  ["latex/metre/demo.pdf"] = {["details"] = "Example of usage", },
  ["generic/lecturer/README"] = {["details"] = "Readme", },
  ["fonts/cormorantgaramond/cormorantgaramond.pdf"] = {["details"] = "Package documentation", },
  ["latex/newcommand/newcommand.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-ukrainian/datetime2-ukrainian.pdf"] = {["details"] = "Package documentation", },
  ["latex/menukeys/README"] = {["details"] = "Readme", },
  ["latex/menu/menu.pdf"] = {["details"] = "Package documentation", },
  ["latex/chessfss/README"] = {["details"] = "Readme", },
  ["generic/docbytex/docby.pdf"] = {["lang"] = "cs", ["details"] = "Package documentation", },
  ["latex/datetime2-finnish/README"] = {["details"] = "Readme", },
  ["generic/babel-hebrew/hebinp.pdf"] = {["details"] = "Package documentation (Hebrew input encodings for use with LaTeX 2)", },
  ["latex/mentis/mentis.pdf"] = {["details"] = "Package documentation", },
  ["latex/dialogl/dia-driv.pdf"] = {["details"] = "Package documentation", },
  ["latex/mensa-tex/README.md"] = {["details"] = "Readme", },
  ["latex/issuulinks/issuulinks.pdf"] = {["details"] = "Package documentation", },
  ["latex/hyperbar/hyperbar.pdf"] = {["details"] = "Package documentation", },
  ["latex/footnpag/README"] = {["details"] = "Readme", },
  ["latex/clrdblpg/clrdblpg.pdf"] = {["details"] = "Package documentation", },
  ["latex/mfpic4ode/mfpic4ode.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/luatextra/README"] = {["details"] = "Readme", },
  ["latex/dotseqn/readme"] = {["details"] = "Readme", },
  ["latex/gitlog/gitlog.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/abstyles/abstdok.pdf"] = {["lang"] = "de", ["details"] = "Outline of the family", },
  ["fonts/amsfonts/eufrak.pdf"] = {["details"] = "Eufrak package documentation", },
  ["metapost/repere/README.md"] = {["details"] = "Readme", },
  ["latex/mdsymbol/mdsymbol.pdf"] = {["details"] = "Package documentation, including font tables", },
  ["latex/mdputu/mdputu.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamertheme-metropolis/metropolistheme.pdf"] = {["details"] = "Package documentation", },
  ["latex/mciteplus/mciteplus_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/bbm-macros/README"] = {["details"] = "Readme", },
  ["latex/begriff/README"] = {["details"] = "Readme", },
  ["latex/beamertheme-saintpetersburg/example.pdf"] = {["details"] = "Example of use", },
  ["metapost/mcf2graph/mcf_manual.pdf"] = {["details"] = "Package documentation", },
  ["fonts/allrunes/allrunes.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-czech/czech.pdf"] = {["details"] = "Package documentation", },
  ["metapost/mcf2graph/mcf_example.pdf"] = {["details"] = "Molecular Coding Format Examples", },
  ["latex/doclicense/README"] = {["details"] = "Readme", },
  ["latex/usebib/README"] = {["details"] = "Readme", },
  ["latex/geometry-de/geometry-de.pdf"] = {["lang"] = "de", ["details"] = "Users' manual", },
  ["fonts/unfonts-core/README.md"] = {["details"] = "Readme", },
  ["latex/mbenotes/mbenotes.pdf"] = {["details"] = "Package documentation", },
  ["latex/jmlr/jmlr.pdf"] = {["details"] = "Package documentation", },
  ["latex/dichokey/dichokey.pdf"] = {["details"] = "Package documentation", },
  ["latex/zxjatype/README.md"] = {["details"] = "Readme", },
  ["latex/mattens/mattens_sample.pdf"] = {["details"] = "Sample of use", },
  ["latex/thuthesis/README.md"] = {["details"] = "Readme", },
  ["latex/matlab-prettifier/matlab-prettifier.pdf"] = {["details"] = "Package documentation", },
  ["latex/matlab-prettifier/README"] = {["details"] = "Readme", },
  ["context/third/transliterator/transliterator.pdf"] = {["details"] = "Package documentation", },
  ["fonts/ec/00readme.txt"] = {["details"] = "Readme", },
  ["latex/mathspic/mathsPICmanual.pdf"] = {["details"] = "User documentation (Perl version)", },
  ["latex/calcage/README"] = {["details"] = "Readme", },
  ["latex/brandeis-dissertation/brandeis-dissertation.pdf"] = {["details"] = "Package documentation", },
  ["latex/collref/collref.pdf"] = {["details"] = "Package documentation", },
  ["fonts/mflogo-font/README"] = {["details"] = "Readme", },
  ["latex/pbsheet/pbsheet.pdf"] = {["details"] = "Package documentation", },
  ["fonts/dutchcal/README"] = {["details"] = "Readme", },
  ["latex/numname/numname.pdf"] = {["details"] = "Package documentation", },
  ["latex/handin/README.txt"] = {["details"] = "Readme", },
  ["latex/grundgesetze/grundgesetze.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/arabxetex/examples/ednotes_example.pdf"] = {["details"] = "Ednotes example", },
  ["fonts/cmsrb/cmsrb-doc-SR.pdf"] = {["lang"] = "sr-sp", ["details"] = "Package documentation (Serbian)", },
  ["latex/amscls/README"] = {["details"] = "Readme", },
  ["latex/makebase/README.md"] = {["details"] = "Readme", },
  ["latex/bbding/bbding.pdf"] = {["details"] = "Package documentation", },
  ["latex/vdmlisting/README"] = {["details"] = "Package Readme", },
  ["latex/mathpartir/README"] = {["details"] = "Readme", },
  ["metapost/automata/README"] = {["details"] = "Readme", },
  ["latex/mathfam256/mathfam256.pdf"] = {["details"] = "Package documentation", },
  ["generic/librarian/README"] = {["details"] = "Readme", },
  ["latex/mathexam/sample.pdf"] = {["details"] = "Sample exam", },
  ["latex/mathexam/mathexam.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-source-division/biblatex-source-division.pdf"] = {["details"] = "Package documentation", },
  ["latex/sageep/sageep.pdf"] = {["details"] = "Package documentation", },
  ["latex/longnamefilelist/longnamefilelist.pdf"] = {["details"] = "Package documentation", },
  ["latex/mathcomp/mathcomp.pdf"] = {["details"] = "Package documentation", },
  ["latex/ecobiblatex/README.txt"] = {["details"] = "Readme", },
  ["fonts/dejavu-otf/dejavu-otf-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/bibleref-french/bibleref-french-francais.pdf"] = {["lang"] = "fr", ["details"] = "Package Manual", },
  ["latex/matc3/README"] = {["details"] = "Readme", },
  ["fonts/marvosym/README"] = {["details"] = "README", },
  ["latex/karnaughmap/karnaughmap.pdf"] = {["details"] = "Package documentation", },
  ["latex/rmpage/readme"] = {["details"] = "Readme", },
  ["latex/halloweenmath/halloweenmath-man.pdf"] = {["details"] = "User manual", },
  ["latex/fancypar/README"] = {["details"] = "Readme", },
  ["xelatex/xeindex/xeindex.pdf"] = {["details"] = "Package documentation", },
  ["latex/nicetext/wikicheat.pdf"] = {["details"] = "Cheatsheet for the wiki package", },
  ["latex/marginfit/README"] = {["details"] = "Readme", },
  ["latex/auto-pst-pdf/README"] = {["details"] = "Package Readme", },
  ["latex/cursolatex/cursolatex.pdf"] = {["lang"] = "pt", ["details"] = "Package documentation", },
  ["latex/biblatex-nejm/README"] = {["details"] = "Readme", },
  ["latex/asyfig/README"] = {["details"] = "Readme", },
  ["latex/margbib/margbib.pdf"] = {["details"] = "Package documentation", },
  ["latex/xmpincl/xmpincl.pdf"] = {["details"] = "Package documentation", },
  ["latex/manyind/README.md"] = {["details"] = "Readme", },
  ["fonts/droid/README"] = {["details"] = "Readme", },
  ["latex/beamertheme-phnompenh/README"] = {["details"] = "Readme", },
  ["latex/foekfont/README"] = {["details"] = "Package README", },
  ["latex/biblatex-dw/README"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/mandi/README"] = {["details"] = "Readme", },
  ["latex/hf-tikz/hf-tikz.pdf"] = {["details"] = "Package documentation", },
  ["latex/mailmerge/README"] = {["details"] = "Readme", },
  ["latex/latex-brochure/brochure.pdf"] = {["details"] = "The document itself", },
  ["fonts/bera/bera.pdf"] = {["details"] = "Font samples", },
  ["support/texcount/README.md"] = {["details"] = "Readme", },
  ["generic/babel-german/ngermanb.pdf"] = {["details"] = "Package documentation  (new orthography)", },
  ["latex/chappg/chappg.pdf"] = {["details"] = "Package documentation", },
  ["latex/sttools/sttools.pdf"] = {["details"] = "Overview of the bundle", },
  ["xelatex/bidi/README"] = {["details"] = "Readme", },
  ["latex/concepts/README"] = {["details"] = "Readme", },
  ["latex/biblatex-realauthor/documentation/biblatex-realauthor.pdf"] = {["details"] = "Package documentation", },
  ["metapost/makecirc/MakeCirc.pdf"] = {["lang"] = "es", ["details"] = "Package documentation in Spanish", },
  ["fonts/fbb/fbb-doc.pdf"] = {["details"] = "Introduction and sampler", },
  ["latex/answers/answers.pdf"] = {["details"] = "Package documentation", },
  ["latex/ifplatform/ifplatform.pdf"] = {["details"] = "Package documentation", },
  ["latex/makebox/README"] = {["details"] = "Package Readme", },
  ["latex/eledmac/eledmac.pdf"] = {["details"] = "Package documentation", },
  ["latex/eqell/README"] = {["details"] = "Readme", },
  ["latex/rubik/rubiktwocube.pdf"] = {["details"] = "Rubiktwocube package documentation", },
  ["latex/bibtopicprefix/bibtopicprefix.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-trad/biblatex-trad.pdf"] = {["details"] = "Package documentation", },
  ["latex/etex-pkg/README"] = {["details"] = "Readme", },
  ["latex/macroswap/README"] = {["details"] = "Readme", },
  ["latex/macros2e/macros2e.pdf"] = {["details"] = "The macro list itself", },
  ["generic/t2/README"] = {["details"] = "Readme", },
  ["latex/locality/locality.pdf"] = {["details"] = "Package documentation", },
  ["latex/musicography/README.md"] = {["details"] = "Readme", },
  ["latex/datetime2-polish/README"] = {["details"] = "Readme", },
  ["fonts/ly1/README"] = {["details"] = "Outline of usage", },
  ["fonts/lxfonts/README"] = {["details"] = "Readme", },
  ["latex/medstarbeamer/README.txt"] = {["details"] = "Readme", },
  ["latex/lwarp/lwarp.pdf"] = {["details"] = "Package documentation", },
  ["latex/ednotes/README"] = {["details"] = "Readme", },
  ["luatex/luaxml/README"] = {["details"] = "Readme", },
  ["latex/lgreek/README"] = {["details"] = "Readme", },
  ["latex/bnumexpr/bnumexpr.pdf"] = {["details"] = "Package documentation", },
  ["luatex/luavlna/luavlna-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-portuges/README"] = {["details"] = "Readme", },
  ["fonts/auncial-new/tryauncial.pdf"] = {["details"] = "Font sample, etc.", },
  ["lualatex/luatextra/luatextra.pdf"] = {["details"] = "Package documentation", },
  ["latex/sageep/sample.pdf"] = {["details"] = "Formatted sample", },
  ["luatex/luatexja/luatexja-en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["generic/fltpoint/README"] = {["details"] = "Readme", },
  ["latex/citeall/README.md"] = {["details"] = "Readme", },
  ["latex/exercises/README.md"] = {["details"] = "Readme", },
  ["xelatex/bidi-atbegshi/README"] = {["details"] = "Readme", },
  ["generic/luatex85/luatex85.pdf"] = {["details"] = "Package documentation", },
  ["latex/ctex/ctex.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation", },
  ["generic/iftex/README"] = {["details"] = "Readme", },
  ["latex/cases/cases.pdf"] = {["details"] = "Package documentation", },
  ["latex/spectralsequences/README.md"] = {["details"] = "Readme", },
  ["latex/tikz-nef/tikz-nef-doc.pdf"] = {["details"] = "Package documentation", },
  ["luatex/fontloader-luaotfload/README.md"] = {["details"] = "Readme", },
  ["luatex/luaotfload/luaotfload.pdf"] = {["details"] = "Package documentation", },
  ["luatex/luaotfload/README"] = {["details"] = "Readme", },
  ["latex/autopdf/autopdf.pdf"] = {["details"] = "Package documentation", },
  ["latex/leipzig/README.md"] = {["details"] = "Readme", },
  ["lualatex/luamesh/luamesh-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/bullcntr/bullcntr-man.pdf"] = {["details"] = "Package documentation", },
  ["latex/easyfig/README"] = {["details"] = "Readme", },
  ["fonts/esint-type1/README"] = {["details"] = "Readme", },
  ["latex/cmpica/README"] = {["details"] = "Readme", },
  ["latex/dox/dox.pdf"] = {["details"] = "Package documentation", },
  ["latex/br-lex/brlex-doc.pdf"] = {["lang"] = "pt-br", ["details"] = "Package documentation", },
  ["latex/trfsigns/trfsigns.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["fonts/antt/AntykwaTorunska-doc-pl-2_03.pdf"] = {["lang"] = "pl", ["details"] = "Documentation in Polish", },
  ["bibtex/base/btxdoc.pdf"] = {["details"] = "Using BibTeX", },
  ["latex/titling/README"] = {["details"] = "Readme", },
  ["lualatex/lualatex-doc/README"] = {["details"] = "Readme", },
  ["fonts/xits/xits-specimen.pdf"] = {["details"] = "Specimen of font", },
  ["lualatex/lua-check-hyphen/doc/luacheckhyphenmanual.pdf"] = {["details"] = "Package documentation", },
  ["latex/sanskrit/README.md"] = {["details"] = "Readme", },
  ["latex/ltxnew/ltxnew.pdf"] = {["details"] = "Package documentation", },
  ["latex/ltxkeys/ltxkeys-guide.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-chem/biblatex-chem.pdf"] = {["details"] = "Package documentation", },
  ["latex/coordsys/coordsys.pdf"] = {["details"] = "Bundle documentation", },
  ["latex/komacv-rg/README.md"] = {["details"] = "Readme", },
  ["latex/bashful/bashful.pdf"] = {["details"] = "Package documentation", },
  ["latex/engrec/README"] = {["details"] = "Package Readme", },
  ["latex/awesomebox/awesomebox.pdf"] = {["details"] = "Package documentation", },
  ["latex/attachfile/attachfile.pdf"] = {["details"] = "Package documentation", },
  ["metapost/bbcard/README.calendar"] = {["details"] = "Calendar readme", },
  ["latex/datetime2-slovak/datetime2-slovak.pdf"] = {["details"] = "Package documentation", },
  ["latex/ltabptch/ltabptch.pdf"] = {["details"] = "Package documentation", },
  ["latex/chembst/chembst.pdf"] = {["details"] = "Package documentation", },
  ["fonts/wadalab/mrj/README"] = {["details"] = "Readme", },
  ["latex/beamertheme-metropolis/README.md"] = {["details"] = "Readme", },
  ["fonts/kpfonts/kpfonts-abstract.pdf"] = {["details"] = "Summary of features", },
  ["latex/calxxxx-yyyy/README"] = {["details"] = "Readme", },
  ["latex/lgreek/lgreekuse.pdf"] = {["details"] = "Package documentation", },
  ["latex/aramaic-serto/sertodoc.pdf"] = {["details"] = "Font documentation", },
  ["latex/genealogytree/genealogytree-example-1.pdf"] = {["details"] = "Example 1", },
  ["latex/lshort-turkish/lshort-tr.pdf"] = {["lang"] = "tr", ["details"] = "The document itself", },
  ["latex/tabulary/tabulary.pdf"] = {["details"] = "Package documentation", },
  ["generic/bagpipe/bagdoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/lshort-slovenian/lshort-slovenian.pdf"] = {["details"] = "The document itself", },
  ["latex/anyfontsize/README"] = {["details"] = "Readme", },
  ["latex/emp/README"] = {["details"] = "Package Readme", },
  ["latex/engtlc/engtlc.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/datetime2-esperanto/datetime2-esperanto.pdf"] = {["details"] = "Package documentation", },
  ["latex/bxcjkjatype/sample-bxcjkjatype.pdf"] = {["lang"] = "ja", ["details"] = "Small example of use (Japanese and English)", },
  ["generic/babel-swedish/swedish.pdf"] = {["details"] = "Package documentation", },
  ["latex/gamebook/gamebook-example.pdf"] = {["details"] = "Simple gamebook example", },
  ["latex/xargs/README"] = {["details"] = "Readme", },
  ["latex/tex-font-errors-cheatsheet/tex-font-cheatsheet.pdf"] = {["details"] = "Package documentation", },
  ["latex/lshort-french/README"] = {["details"] = "Readme", },
  ["latex/listbib/listbib-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/lshort-czech/lshort-cs.pdf"] = {["details"] = "The document itself", },
  ["latex/lshort-czech/README"] = {["details"] = "Readme", },
  ["latex/lshort-chinese/lshort-zh-cn.pdf"] = {["lang"] = "zh", ["details"] = "The document itself", },
  ["latex/lshort-bulgarian/lshort-bg.pdf"] = {["lang"] = "bg", ["details"] = "The document itself", },
  ["metapost/slideshow/sshowintro.pdf"] = {["details"] = "Presentation about package", },
  ["latex/biblatex-opcit-booktitle/README"] = {["details"] = "Readme", },
  ["support/latexindent/latexindent.pdf"] = {["details"] = "Package documentation", },
  ["latex/lplfitch/lplfitch.pdf"] = {["details"] = "Package documentation", },
  ["latex/disser/README.md"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/beamerthemenirma/README"] = {["details"] = "Readme", },
  ["latex/genealogytree/genealogytree-example-3.pdf"] = {["details"] = "Example 3", },
  ["latex/lpic/README"] = {["details"] = "Readme", },
  ["latex/datetime2-italian/README"] = {["details"] = "Readme", },
  ["latex/multiobjective/multiobjective.pdf"] = {["details"] = "Package documentation", },
  ["latex/codepage/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["latex/lewis/README"] = {["details"] = "Readme", },
  ["latex/longfbox/README"] = {["details"] = "Readme", },
  ["latex/logicproof/logicproof.pdf"] = {["details"] = "Package documentation", },
  ["generic/babel-samin/samin.pdf"] = {["details"] = "Package documentation", },
  ["fonts/arphic/arphic-sampler.pdf"] = {["details"] = "Font sampler", },
  ["latex/kerntest/kerntest.pdf"] = {["details"] = "Package documentation", },
  ["latex/latexcheat/latexsheet.pdf"] = {["details"] = "The document itself", },
  ["latex/first-latex-doc/README"] = {["details"] = "Readme", },
  ["context/third/cyrillicnumbers/cyrillicnumbers.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/apacite/apacite.pdf"] = {["details"] = "Package documentation", },
  ["latex/everyhook/everyhook.pdf"] = {["details"] = "Package documentation", },
  ["generic/gobble/README"] = {["details"] = "Readme", },
  ["generic/pst-shell/pst-shell-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/academicons/README"] = {["details"] = "Readme", },
  ["latex/plates/README"] = {["details"] = "Readme", },
  ["latex/lithuanian/README"] = {["details"] = "Readme", },
  ["latex/endiagram/README"] = {["details"] = "Readme", },
  ["fonts/accanthis/Accanthis-Cat.pdf"] = {["lang"] = "fr", ["details"] = "Arkandis catalogue page", },
  ["bibtex/besjournals/README"] = {["details"] = "Readme", },
  ["latex/pst-optexp/README"] = {["details"] = "Readme", },
  ["latex/uhhassignment/uhhassignment.pdf"] = {["details"] = "Package documentation", },
  ["latex/latex-via-exemplos/latex-via-exemplos-oneside.pdf"] = {["details"] = "The document itself (oneside)", },
  ["support/ketcindy/README"] = {["details"] = "Readme", },
  ["latex/biblatex-chicago/biblatex-chicago.pdf"] = {["details"] = "Package documentation", },
  ["fonts/ebgaramond/README"] = {["details"] = "Readme", },
  ["latex/glossaries-french/README"] = {["details"] = "Readme", },
  ["latex/cite/README"] = {["details"] = "Bundle readme", },
  ["latex/bbding/README"] = {["details"] = "Readme", },
  ["xelatex/sexam/sexam_wexam_doc_ar.pdf"] = {["lang"] = "ar-dz", ["details"] = "Package documentation", },
  ["latex/lion-msc/lion-msc.pdf"] = {["details"] = "Package documentation", },
  ["latex/etaremune/README"] = {["details"] = "Readme", },
  ["latex/bxdvidriver/README.md"] = {["details"] = "Readme", },
  ["latex/braket/braket.pdf"] = {["details"] = "Package documentation", },
  ["latex/embedall/embedall.pdf"] = {["details"] = "Package documentation", },
  ["fonts/rsfso/README"] = {["details"] = "Readme", },
  ["latex/scalebar/README"] = {["details"] = "Package README", },
  ["latex/vertbars/README"] = {["details"] = "Readme", },
  ["latex/limecv/limecv.pdf"] = {["details"] = "Package documentation", },
  ["latex/exam/README"] = {["details"] = "Package Readme", },
  ["latex/limecv/examples/mwe-xelatex.pdf"] = {["details"] = "Example of use", },
  ["latex/pict2e/README.md"] = {["details"] = "Readme", },
  ["latex/limap/README.md"] = {["details"] = "Readme", },
  ["latex/acroterm/acroterm.pdf"] = {["details"] = "Package documentation", },
  ["context/third/french/french-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/librebaskerville/README"] = {["details"] = "Readme", },
  ["latex/unravel/README.md"] = {["details"] = "Readme", },
  ["latex/antt/README"] = {["details"] = "Readme", },
  ["fonts/libertinust1math/LibertinusT1Math-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/lhelp/lhelp.pdf"] = {["details"] = "Package documentation", },
  ["generic/pst-bezier/pst-bezier-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ccicons/ccicons.pdf"] = {["details"] = "Package documentation", },
  ["latex/hardwrap/hardwrap.pdf"] = {["details"] = "Package documentation", },
  ["latex/hyper/scontrib/README"] = {["details"] = "Readme", },
  ["fonts/ecc/readme"] = {["details"] = "Readme", },
  ["latex/beamertheme-saintpetersburg/SaintPetersburg.pdf"] = {["details"] = "Package documentation", },
  ["latex/notespages/README"] = {["details"] = "Readme", },
  ["latex/datetime2-catalan/README"] = {["details"] = "Readme", },
  ["latex/lettre/lettre.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/biblatex-lni/README.md"] = {["details"] = "Readme", },
  ["latex/datetime2-lsorbian/datetime2-lsorbian.pdf"] = {["details"] = "Package documentation", },
  ["latex/inlinedef/README"] = {["details"] = "Readme", },
  ["latex/SIstyle/README"] = {["details"] = "Readme", },
  ["fonts/tipa/00README"] = {["details"] = "Readme", },
  ["latex/harmony/harmony.pdf"] = {["details"] = "Package documentation (by example)", },
  ["fonts/fontawesome/README.md"] = {["details"] = "Readme", },
  ["latex/ledmac/README"] = {["details"] = "Readme", },
  ["generic/mathdots/README"] = {["details"] = "Readme", },
  ["generic/babel-bosnian/README"] = {["details"] = "Readme", },
  ["luatex/luatexja/README"] = {["details"] = "Readme", },
  ["latex/continue/README"] = {["details"] = "Readme", },
  ["generic/lecturer/lecturer-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/barcodes/eandoc.pdf"] = {["details"] = "Package documentation", },
  ["latex/doctools/README"] = {["details"] = "Readme", },
  ["latex/finstrut/finstrut.pdf"] = {["details"] = "Package documentation", },
  ["latex/bxjaprnind/bxjaprnind.pdf"] = {["lang"] = "ja", ["details"] = "Package documentation", },
  ["lualatex/plantuml/README.md"] = {["details"] = "Readme", },
  ["generic/babel-danish/danish.pdf"] = {["details"] = "Package documentation", },
  ["latex/comma/README"] = {["details"] = "Readme", },
  ["latex/leaflet/README.md"] = {["details"] = "Readme", },
  ["latex/jamtimes/README"] = {["details"] = "Readme", },
  ["latex/layouts/layman.pdf"] = {["details"] = "Package documentation", },
  ["generic/gtl/gtl.pdf"] = {["details"] = "Package documentation", },
  ["fonts/lato/lato-samples.pdf"] = {["details"] = "Font samples", },
  ["latex/bullcntr/README"] = {["details"] = "Readme", },
  ["latex/altfont/README"] = {["details"] = "Readme", },
  ["generic/babel-english/english.pdf"] = {["details"] = "Package documentation", },
  ["latex/smartunits/smartunits.pdf"] = {["details"] = "Package documentation", },
  ["latex/latexgit/latexgit.pdf"] = {["details"] = "Package documentation", },
  ["latex/embrac/README"] = {["details"] = "Readme", },
  ["latex/ijmart/README"] = {["details"] = "Readme", },
  ["latex/latexfileinfo-pkgs/latexfileinfo_pkgs.htm"] = {["details"] = "The document itself", },
  ["latex/eltex/README"] = {["details"] = "Readme", },
  ["latex/aurical/aurical.pdf"] = {["details"] = "Package introduction and samples", },
  ["support/ctan_chk/ctan_chk.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2/README"] = {["details"] = "Readme", },
  ["latex/latexcourse-rug/latexcourse-rug.pdf"] = {["details"] = "Printable copy of the course", },
  ["latex/lwarp/README.txt"] = {["details"] = "Readme", },
  ["latex/hpsdiss/hpsdiss.pdf"] = {["details"] = "Example of package use", },
  ["latex/datetime2-swedish/README"] = {["details"] = "Readme", },
  ["latex/minibox/minibox.pdf"] = {["details"] = "Package documentation", },
  ["latex/itnumpar/README"] = {["details"] = "Readme", },
  ["latex/structmech/README.md"] = {["details"] = "Readme", },
  ["asymptote/asyRefCard.pdf"] = {["details"] = "Package 'reference card'", },
  ["latex/leaflet/leaflet.pdf"] = {["details"] = "Documented source", },
  ["latex/pdftricks2/pdftricks2-doc.pdf"] = {["details"] = "Package documentation", },
  ["support/latex2man/latex2man.pdf"] = {["details"] = "Package documentation (PDF)", },
  ["latex/celtic/README"] = {["details"] = "Readme", },
  ["latex/verbatimcopy/verbatimcopy.pdf"] = {["details"] = "Package documentation", },
  ["bibtex/bestpapers/bestpapers-guide.pdf"] = {["details"] = "Package documentation", },
  ["latex/bredzenie/bredzenie.pdf"] = {["details"] = "Package documentation", },
  ["latex/llncsconf/example.pdf"] = {["details"] = "Package documentation", },
  ["latex/ducksay/README.md"] = {["details"] = "Readme", },
  ["latex/greek-inputenc/README.html"] = {["details"] = "Readme (HTML)", },
  ["latex/beamertheme-cuerna/beamertheme-cuerna.pdf"] = {["details"] = "Package documentation", },
  ["latex/altfont/altfont.pdf"] = {["details"] = "Package documentation", },
  ["generic/epsf/README"] = {["details"] = "Readme", },
  ["latex/bgteubner/bgteubner.pdf"] = {["lang"] = "de", ["details"] = "Class documentation", },
  ["latex/ecothesis/ecothesis.pdf"] = {["lang"] = "pt-br", ["details"] = "Package documentation", },
  ["latex/pst-pdf/pst-pdf-example.pdf"] = {["details"] = "Examples of use", },
  ["generic/upzhkinsoku/README"] = {["details"] = "Readme", },
  ["latex/changes/README"] = {["lang"] = "en", ["details"] = "Readme", },
  ["context/third/typescripts/README"] = {["details"] = "Readme", },
  ["latex/platex-tools/plarray.pdf"] = {["lang"] = "ja", ["details"] = "Documentation of the plarray package", },
  ["latex/latex-mr/README.md"] = {["details"] = "Readme", },
  ["latex/egplot/egplot.pdf"] = {["details"] = "Package documentation", },
  ["fonts/aecc/README"] = {["details"] = "Readme", },
  ["lualatex/plantuml/plantuml.pdf"] = {["details"] = "Package documentation", },
  ["support/latex-git-log/README"] = {["details"] = "Readme", },
  ["latex/footnotebackref/footnotebackref.pdf"] = {["details"] = "Package documentation", },
  ["latex/includernw/includeRnw-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/dozenal/dozenal.pdf"] = {["details"] = "Package documentation", },
  ["latex/float/float.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cmbright/README"] = {["details"] = "Readme", },
  ["latex/afthesis/usethesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/qsymbols/qsymbols.pdf"] = {["details"] = "Package documentation", },
  ["latex/lastpackage/lastpackage.pdf"] = {["details"] = "Package documentation", },
  ["latex/rrgtrees/rrgtrees.pdf"] = {["details"] = "Package documentation", },
  ["fonts/arev/arevdoc.pdf"] = {["details"] = "Package documentation", },
  ["generic/langcode/langcode.pdf"] = {["details"] = "Package documentation", },
  ["latex/bibleref/bibleref.pdf"] = {["details"] = "Package documentation", },
  ["latex/forest-quickstart/ForestQuickstart.pdf"] = {["details"] = "Package documentation", },
  ["latex/juraabbrev/README"] = {["details"] = "Readme", },
  ["latex/ntgclass/00readme.txt"] = {["details"] = "Readme", },
  ["fonts/ebgaramond-maths/README"] = {["details"] = "Readme", },
  ["latex/multibbl/multibbl.pdf"] = {["details"] = "Package documentation", },
  ["latex/grfpaste/doc/grfpaste.pdf"] = {["details"] = "Package documentation", },
  ["latex/dlfltxb/dlfltxbmarkup.pdf"] = {["details"] = "dlfltxbmarkup package documentation", },
  ["latex/labbook/README"] = {["details"] = "Readme", },
  ["latex/l3kernel/interface3.pdf"] = {["details"] = "The LaTeX3 interfaces", },
  ["latex/dlfltxb/README"] = {["details"] = "Readme", },
  ["fonts/kurier/kurier-info.pdf"] = {["details"] = "Package documentation", },
  ["latex/l2tabu-french/README"] = {["details"] = "Readme", },
  ["lualatex/lualatex-math/README"] = {["details"] = "Readme", },
  ["latex/datetime2-welsh/README"] = {["details"] = "Readme", },
  ["latex/gu/README"] = {["details"] = "Package README", },
  ["generic/e-french/manuels/french_doc.pdf"] = {["details"] = "Package documentation (full version)", },
  ["latex/ktv-texdata/README"] = {["details"] = "Readme", },
  ["latex/sepfootnotes/README"] = {["details"] = "Readme", },
  ["latex/kotex-utf/README"] = {["details"] = "Readme", },
  ["plain/kotex-plain/README"] = {["details"] = "Readme", },
  ["latex/kotex-oblivoir/doc/oblivoir-simpledoc.pdf"] = {["lang"] = "ko", ["details"] = "Package documentation", },
  ["latex/datetime2-romanian/README"] = {["details"] = "Readme", },
  ["latex/gloss-occitan/README"] = {["details"] = "Readme", },
  ["latex/latex4wp/README"] = {["details"] = "Readme", },
  ["latex/epslatex-fr/fepslatex.pdf"] = {["details"] = "The document itself", },
  ["latex/mychemistry/README"] = {["details"] = "Readme", },
  ["generic/pdf-trans/example.pdf"] = {["details"] = "Examples of use", },
  ["support/epspdf/README"] = {["details"] = "Readme", },
  ["fonts/knitting/README"] = {["details"] = "Readme", },
  ["generic/chemfig/README"] = {["details"] = "Readme", },
  ["support/crossrefware/crossrefware.pdf"] = {["details"] = "Package documentation", },
  ["latex/tui/TUIdoc.pdf"] = {["lang"] = "es", ["details"] = "Package documentation (Spanish)", },
  ["latex/keyval2e/keyval2e-guide.pdf"] = {["details"] = "Package documentation", },
  ["latex/calctab/README"] = {["details"] = "Readme", },
  ["latex/keyfloat/keyfloat.pdf"] = {["details"] = "Package documentation", },
  ["latex/keycommand/keycommand.pdf"] = {["details"] = "Package documentation", },
  ["latex/bibleref-parse/bibleref-parse.pdf"] = {["details"] = "Package documentation", },
  ["latex/mparhack/mparhack.pdf"] = {["details"] = "Package documentation", },
  ["latex/ctable/ctable.pdf"] = {["details"] = "Package documentation", },
  ["latex/kerntest/README"] = {["details"] = "Readme", },
  ["latex/hobete/hobete_doc.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/mfnfss/oldgerm.pdf"] = {["details"] = "Documentation of the Old German support", },
  ["latex/biblatex-luh-ipw/de-biblatex-luh-ipw.pdf"] = {["lang"] = "de", ["details"] = "Package documentation (German)", },
  ["latex/kdgdocs/kdgcoursetext-example.pdf"] = {["details"] = "Course text example", },
  ["latex/karnaugh/kvdoc.pdf"] = {["details"] = "Package documentation", },
  ["luatex/kanaparser/README.md"] = {["details"] = "Readme", },
  ["latex/zhnumber/README.md"] = {["details"] = "Readme", },
  ["context/third/degrade/degrade-demo.pdf"] = {["details"] = "Example of use", },
  ["fonts/ipaex/README"] = {["details"] = "Readme", },
  ["generic/catcodes/catcodes.pdf"] = {["details"] = "Package documentation", },
  ["latex/platex-tools/pxeverysel.pdf"] = {["lang"] = "ja", ["details"] = "Documentation of the pxeverysel package", },
  ["latex/datetime2-spanish/README"] = {["details"] = "Readme", },
  ["latex/lexikon/lexikon-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/patchcmd/README"] = {["details"] = "Readme", },
  ["latex/h2020proposal/manual/h2020proposal.pdf"] = {["details"] = "Package documentation", },
  ["support/latex-make/README"] = {["details"] = "Readme", },
  ["etex/base/etex_man.pdf"] = {["details"] = "System documentation (v2.0)", },
  ["latex/libgreek/README"] = {["details"] = "Readme", },
  ["latex/octave/README.md"] = {["details"] = "Readme", },
  ["latex/jmlr/README"] = {["details"] = "Readme", },
  ["latex/jlreq/README.md"] = {["details"] = "Readme", },
  ["generic/apnum/apnum.pdf"] = {["details"] = "Package documentation", },
  ["latex/jkmath/jkmath.pdf"] = {["details"] = "Package documentation", },
  ["fonts/japanese-otf/readme-ja.txt"] = {["lang"] = "ja", ["details"] = "Readme (japanese)", },
  ["latex/bibletext/README.md"] = {["details"] = "Readme", },
  ["latex/karnaughmap/README"] = {["details"] = "Readme", },
  ["latex/jacow/README"] = {["details"] = "Readme", },
  ["latex/censor/README"] = {["details"] = "Readme", },
  ["latex/blox/blox.pdf"] = {["details"] = "Package documentation", },
  ["latex/erw-l3/README.txt"] = {["details"] = "Readme", },
  ["latex/beamer2thesis/README"] = {["details"] = "Readme", },
  ["generic/pst-fractal/README"] = {["details"] = "Readme", },
  ["latex/computational-complexity/cc.pdf"] = {["details"] = "Package documentation", },
  ["metapost/mp3d/README"] = {["details"] = "Readme", },
  ["latex/revtex/README"] = {["details"] = "Package Readme", },
  ["latex/biblatex-abnt/tests/README.md"] = {["lang"] = "pt-br", ["details"] = "Readme", },
  ["lualatex/odsfile/README"] = {["details"] = "Readme", },
  ["fonts/alegreya/alegreya-samples.pdf"] = {["details"] = "Font samples", },
  ["latex/musikui/musikui.pdf"] = {["details"] = "Package documentation", },
  ["latex/elmath/elmath.pdf"] = {["details"] = "Package documentation", },
  ["latex/interval/README"] = {["details"] = "Readme", },
  ["latex/comprehensive/SYMLIST"] = {["details"] = "Plain text list of symbol commands", },
  ["latex/parades/parades.pdf"] = {["details"] = "Package documentation", },
  ["generic/insbox/demo.pdf"] = {["details"] = "Examples of use", },
  ["latex/eledmac/README"] = {["details"] = "Readme", },
  ["latex/commath/commath.pdf"] = {["details"] = "Package documentation", },
  ["latex/inputtrc/README"] = {["details"] = "Readme", },
  ["fonts/concmath/concmath.pdf"] = {["details"] = "Package documentation", },
  ["latex/inlinedef/inlinedef.pdf"] = {["details"] = "Package documentation", },
  ["luatex/luamplib/luamplib.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cbfonts-fd/cbfonts-fd.pdf"] = {["details"] = "Package documentation", },
  ["latex/prooftrees/README"] = {["details"] = "Readme", },
  ["latex/cesenaexam/cesenaexam.pdf"] = {["details"] = "Package documentation", },
  ["latex/constants/constants.pdf"] = {["details"] = "Package documentation", },
  ["support/purifyeps/README"] = {["details"] = "Package README", },
  ["latex/titlecaps/titlecaps.pdf"] = {["details"] = "Package documentation", },
  ["latex/amscls-doc/Author_Handbook_ProcColl.pdf"] = {["details"] = "AMS Author Handbook for Proceedings and Collections Classes", },
  ["latex/xprintlen/README"] = {["details"] = "Readme", },
  ["latex/ibycus-babel/ibycus-babel.pdf"] = {["details"] = "Package documentation", },
  ["latex/complexity/complexity.pdf"] = {["details"] = "Package documentation", },
  ["latex/draftfigure/README.md"] = {["details"] = "Readme", },
  ["generic/babel-greek/grmath.pdf"] = {["details"] = "Maths documentation", },
  ["latex/imakeidx/README"] = {["details"] = "Readme", },
  ["latex/image-gallery/gallery-example.pdf"] = {["details"] = "Example output", },
  ["latex/dtxdescribe/dtxdescribe.pdf"] = {["details"] = "Package documentation", },
  ["generic/ifxetex/ifxetex.pdf"] = {["details"] = "Package documentation", },
  ["latex/biblatex-dw/LIESMICH"] = {["lang"] = "de", ["details"] = "Readme (German)", },
  ["latex/eqlist/README"] = {["details"] = "Readme", },
  ["latex/ifoddpage/ifoddpage.pdf"] = {["details"] = "Package documentation", },
  ["latex/curves/README"] = {["details"] = "Readme", },
  ["fonts/stix/stix.pdf"] = {["details"] = "Package documentation", },
  ["latex/caption/caption-eng.pdf"] = {["lang"] = "en", ["details"] = "English documentation", },
  ["latex/polexpr/polexpr.txt"] = {["details"] = "Package documentation (text format)", },
  ["latex/refman/layout_e.pdf"] = {["lang"] = "en", ["details"] = "Changing the layout with LaTeX", },
  ["latex/hyperxmp/hyperxmp.pdf"] = {["details"] = "Package documentation", },
  ["latex/hyperref/paper.pdf"] = {["lang"] = "en", ["details"] = "Paper on tagging and navigation", },
  ["latex/hyperbar/README.md"] = {["details"] = "Readme", },
  ["latex/elocalloc/README.txt"] = {["details"] = "Readme", },
  ["latex/newvbtm/README"] = {["details"] = "Readme", },
  ["latex/bxpapersize/README.md"] = {["details"] = "Readme", },
  ["latex/hypdvips/hypdvips.pdf"] = {["details"] = "Package documentation", },
  ["fonts/droid/droid.pdf"] = {["details"] = "Package documentation", },
  ["latex/hypdvips/README"] = {["details"] = "Readme", },
  ["latex/hvindex/hvindex-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/soup/README.md"] = {["details"] = "Readme", },
  ["generic/pst-pdgr/pst-pdgr.pdf"] = {["details"] = "Package documentation", },
  ["latex/hvfloat/README"] = {["details"] = "Readme (package history)", },
  ["generic/babel-greek/README"] = {["details"] = "Readme", },
  ["latex/ctex-faq/README"] = {["lang"] = "en", ["details"] = "Readme (English)", },
  ["latex/cybercic/cybercic.pdf"] = {["details"] = "Package documentation", },
  ["fonts/nkarta/figtable.pdf"] = {["details"] = "A table of the MetaPost images", },
  ["latex/mversion/README"] = {["details"] = "Readme", },
  ["generic/pst-diffraction/README"] = {["details"] = "Readme", },
  ["latex/drs/drsdoc.pdf"] = {["details"] = "Package documentation", },
  ["xelatex/arabxetex/README.md"] = {["details"] = "Readme", },
  ["generic/hlist/hlist-fr.pdf"] = {["details"] = "Package documentation", },
  ["generic/chronosys/docchronosys_en.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/hithesis/hithesis.pdf"] = {["lang"] = "zh", ["details"] = "Package documentation (Chinese)", },
  ["latex/jura/jura.pdf"] = {["lang"] = "de", ["details"] = "Class documentation", },
  ["context/third/chromato/chromato-doc.pdf"] = {["details"] = "Package documentation", },
  ["fonts/hfoldsty/README"] = {["details"] = "Readme", },
  ["latex/fancyhdr/README"] = {["details"] = "Readme", },
  ["latex/hexgame/hexgame.pdf"] = {["details"] = "Package documentation", },
  ["latex/aeguill/guil-test2.pdf"] = {["details"] = "Available guillemet glyphs", },
  ["latex/hepparticles/README"] = {["details"] = "Readme", },
  ["latex/eulervm/eulervm.pdf"] = {["details"] = "Package documentation", },
  ["latex/hang/README"] = {["details"] = "Readme", },
  ["latex/cbcoptic/coptfont.pdf"] = {["details"] = "Font details and samples", },
  ["latex/handout/README"] = {["details"] = "Readme", },
  ["fonts/feyn/README"] = {["details"] = "Package Readme", },
  ["latex/harveyballs/harveyballs-Manual.pdf"] = {["details"] = "Package documentation", },
  ["lualatex/cstypo/cstypo.pdf"] = {["details"] = "User Manual", },
  ["latex/probsoln/probsoln.pdf"] = {["details"] = "Package documentation", },
  ["latex/forarray/forarray.pdf"] = {["details"] = "Package documentation", },
  ["latex/hagenberg-thesis/examples/HgbArticle/main.pdf"] = {["lang"] = "de", ["details"] = "Example document / Tutorial (German)", },
  ["latex/hackthefootline/README.md"] = {["details"] = "Readme", },
  ["latex/cyrillic/README.md"] = {["details"] = "Readme", },
  ["latex/overpic/README.de.md"] = {["lang"] = "de", ["details"] = "Readme (German)", },
  ["context/third/fancybreak/README"] = {["details"] = "README", },
  ["latex/tikz-cd/README"] = {["details"] = "Readme", },
  ["latex/clrdblpg/README"] = {["details"] = "Readme", },
  ["latex/bracketkey/README"] = {["details"] = "Readme, including usage details", },
  ["latex/mcmthesis/mcmthesis-demo.pdf"] = {["details"] = "Example of use", },
  ["latex/dickimaw/README"] = {["details"] = "Readme", },
  ["latex/alertmessage/alertmessage.pdf"] = {["details"] = "Package documentation", },
  ["latex/ed/README"] = {["details"] = "Readme", },
  ["latex/zed-csp/zed2e.pdf"] = {["details"] = "Z usage documentation", },
  ["latex/cd/README"] = {["details"] = "Readme", },
  ["latex/psfragx/psfragx.pdf"] = {["details"] = "Psfragx documentation:", },
  ["latex/unamthesis/README"] = {["details"] = "Readme", },
  ["fonts/concmath/README"] = {["details"] = "Readme", },
  ["latex/topfloat/topfloat.pdf"] = {["lang"] = "it", ["details"] = "Package documentation", },
  ["fonts/greektex/README"] = {["details"] = "Readme", },
  ["latex/greektonoi/greektonoi.pdf"] = {["lang"] = "el", ["details"] = "Package documentation (Greek)", },
  ["fonts/librebodoni/librebodoni-samples.pdf"] = {["details"] = "Package documentation", },
  ["latex/graph35/graph35-en.pdf"] = {["details"] = "Package documentation in English", },
  ["fonts/gfsneohellenicmath/MathematicsCheatSheet.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cm-unicode/README"] = {["details"] = "Readme", },
  ["latex/crossreftools/crossreftools_doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/ksp-thesis/ksp-thesis.pdf"] = {["details"] = "Package documentation", },
  ["latex/europecv/README.md"] = {["details"] = "Readme", },
  ["latex/drawstack/stack-example.pdf"] = {["details"] = "Usage example", },
  ["latex/grafcet/grafcet.pdf"] = {["lang"] = "fr", ["details"] = "Package documentation", },
  ["latex/minibox/README"] = {["details"] = "Readme", },
  ["latex/spath3/knots.pdf"] = {["details"] = "Use for knot (and similar) diagrams", },
  ["latex/apprends-latex/README"] = {["details"] = "Readme", },
  ["latex/basque-book/basque-book.pdf"] = {["lang"] = "en", ["details"] = "Package documentation (English)", },
  ["latex/options/README"] = {["details"] = "Readme", },
  ["fonts/cantarell/README"] = {["details"] = "Readme", },
  ["metapost/bbcard/README.bbcard"] = {["details"] = "Bullshit bingo readme", },
  ["latex/glossaries/glossaries-user.html"] = {["details"] = "User manual (HTML)", },
  ["latex/gmverse/README"] = {["details"] = "README", },
  ["latex/longfbox/longfbox.pdf"] = {["details"] = "Package documentation", },
  ["latex/currvita/currvita.pdf"] = {["details"] = "Package documentation", },
  ["latex/gloss/gloss.pdf"] = {["details"] = "Package documentation", },
  ["latex/dvdcoll/INSTALL"] = {["details"] = "Installation instructions", },
  ["generic/babel-azerbaijani/azerbaijani.pdf"] = {["details"] = "Package documentation", },
  ["latex/embrac/embrac_en.pdf"] = {["details"] = "Package documentation", },
  ["fonts/gfsbodoni/gfsbodoni.pdf"] = {["details"] = "Package documentation", },
  ["latex/piano/README"] = {["details"] = "Readme", },
  ["latex/getitems/README.md"] = {["details"] = "Readme", },
  ["latex/stex/modules/README"] = {["details"] = "Readme", },
  ["latex/uaclasses/my-example.pdf"] = {["details"] = "my-thesis example", },
  ["latex/abc/abc.pdf"] = {["details"] = "Package documentation", },
  ["latex/iscram/README"] = {["details"] = "Readme", },
  ["latex/lshort-vietnamese/lshort-vi.pdf"] = {["details"] = "The document itself", },
  ["generic/dehyph-exptl/README"] = {["details"] = "Readme", },
  ["latex/gamebook/gamebook.pdf"] = {["details"] = "Package documentation", },
  ["latex/g-brief/beispiel2.pdf"] = {["lang"] = "de", ["details"] = "Example, new version", },
  ["latex/dnaseq/dnaseq.pdf"] = {["details"] = "Package documentation", },
  ["latex/fwlw/fwlw.pdf"] = {["details"] = "Package documentation", },
  ["fonts/eurosym/README"] = {["details"] = "Readme", },
  ["plain/plstmary/README"] = {["details"] = "Readme", },
  ["luatex/luavlna/README.md"] = {["details"] = "Readme", },
  ["fonts/skaknew/fonttables.pdf"] = {["details"] = "Font tables", },
  ["fonts/burmese/burmguide.pdf"] = {["details"] = "Package documentation", },
  ["fonts/knitting/knitting-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/bxjaprnind/README.md"] = {["details"] = "Readme", },
  ["latex/datetime2-bahasai/datetime2-bahasai.pdf"] = {["details"] = "Package documentation", },
  ["latex/footnotehyper/footnotehyper.pdf"] = {["details"] = "Package documentation", },
  ["latex/footmisc/README"] = {["details"] = "Readme", },
  ["latex/crosswrd/README"] = {["details"] = "Readme", },
  ["metapost/splines/README"] = {["details"] = "Package README", },
  ["latex/options/options.pdf"] = {["details"] = "Package documentation", },
  ["generic/fntproof/README"] = {["details"] = "Readme", },
  ["generic/babel-albanian/albanian.pdf"] = {["details"] = "Package documentation", },
  ["latex/fithesis/fithesis.pdf"] = {["details"] = "Package documentation", },
  ["fonts/newtxtt/newtxtt-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/beamer-verona/README.md"] = {["details"] = "Readme", },
  ["fonts/figbas/example.pdf"] = {["details"] = "Short example of use", },
  ["latex/fgruler/fgruler.pdf"] = {["details"] = "Package documentation", },
  ["latex/ascmac/README.md"] = {["details"] = "Readme", },
  ["latex/binarytree/README"] = {["details"] = "README", },
  ["latex/acmconf/acmconf.pdf"] = {["details"] = "Package documentation", },
  ["fonts/fetamont/fetamont-typeface.pdf"] = {["details"] = "Package design and construction", },
  ["latex/sepfootnotes/sepfootnotes.pdf"] = {["details"] = "Package documentation", },
  ["support/installfont/installfont.pdf"] = {["lang"] = "en", ["details"] = "Package documentation", },
  ["fonts/cfr-lm/cfr-lm.pdf"] = {["details"] = "Package documentation", },
  ["latex/dlfltxb/dlfltxbmarkup-showkeys.pdf"] = {["details"] = "dlfltxbmarkup key description summary", },
  ["generic/c-pascal/README.pol"] = {["lang"] = "pl", ["details"] = "Readme", },
  ["fonts/fourier/fourier-orns.pdf"] = {["details"] = "The Fourier ornaments", },
  ["latex/penrose/penrose.pdf"] = {["details"] = "User documentation", },
  ["fonts/carlito/README"] = {["details"] = "Readme", },
  ["fonts/alkalami/README.txt"] = {["details"] = "Readme", },
  ["latex/extarrows/README"] = {["details"] = "Readme", },
  ["latex/movie15/movie15.pdf"] = {["details"] = "Package Documentation", },
  ["latex/othelloboard/example-rose-chps1-2.pdf"] = {["details"] = "Usage example", },
  ["fonts/heuristica/heuristica-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/basque-date/basque-date.pdf"] = {["details"] = "Package documentation", },
  ["latex/standalone/README"] = {["details"] = "Readme", },
  ["generic/fixjfm/fixjfm-doc.pdf"] = {["details"] = "Package documentation", },
  ["latex/etoc/etoc-DE.pdf"] = {["lang"] = "de", ["details"] = "Package documentation", },
  ["latex/cfr-initials/README"] = {["details"] = "Readme", },
  ["latex/etextools/etextools.pdf"] = {["details"] = "Package documentation", },
  ["latex/aspectratio/ar.pdf"] = {["details"] = "Package documentation", },
  ["latex/esk/README"] = {["details"] = "Readme", },
  ["latex/cd-cover/README"] = {["details"] = "Readme", },
  ["latex/msc/README"] = {["details"] = "Package Readme", },
  ["lualatex/novel/novel.pdf"] = {["details"] = "Example document", },
  ["latex/endheads/endheads.pdf"] = {["details"] = "Package documentation", },
  ["latex/coverpage/SimpleSample.pdf"] = {["details"] = "Simple sample", },
  ["latex/syntax/syntaxintro.pdf"] = {["lang"] = "de", ["details"] = "Introduction to the package", },
  ["latex/catchfilebetweentags/README"] = {["details"] = "Readme", },
  ["generic/babel-ukrainian/README.md"] = {["details"] = "Readme", },
  ["latex/shadowtext/shadowtext.pdf"] = {["details"] = "Package documentation", },
  ["latex/glossaries/glossary2glossaries.html"] = {["details"] = "How to upgrade from glossary.sty (HTML)", },
  ["latex/biblatex-morenames/documentation/biblatex-morenames.pdf"] = {["details"] = "Package documentation", },
  ["fonts/cantarell/cantarell.pdf"] = {["details"] = "Package documentation", },
  ["latex/apprends-latex/Apprends_LaTeX.pdf"] = {["lang"] = "fr", ["details"] = "The document itself", },
  ["latex/cals/examples/demo.pdf"] = {["details"] = "Examples of use", },
  ["latex/braille/braille.html"] = {["details"] = "Package documentation (HTML)", },
  ["latex/IEEEconf/README"] = {["details"] = "Readme", },
  ["latex/fancynum/README"] = {["details"] = "Package README", },
  ["latex/hrefhide/hrefhide.pdf"] = {["details"] = "Package documentation", },
  ["latex/breqn/README.md"] = {["details"] = "Readme", },
  ["latex/clipboard/README"] = {["details"] = "Readme", },
  ["latex/codedoc/CodeDoc-manual.pdf"] = {["details"] = "Package manual", },
  ["latex/tex-overview/tex-overview.pdf"] = {["details"] = "The document itself", },
  ["latex/keycommand/README"] = {["details"] = "Readme", },
  ["latex/iso10303/stepman.pdf"] = {["details"] = "Package manual", },
  ["latex/vmargin/vmargin.pdf"] = {["details"] = "Package documentation", },
  ["latex/datetime2-polish/datetime2-polish.pdf"] = {["details"] = "Package documentation", },
  ["fonts/frederika2016/README"] = {["details"] = "Readme", },
  ["latex/aeguill/guil-test1.pdf"] = {["details"] = "Discussion of package", },
  ["bibtex/bibhtml/bibhtml.html"] = {["details"] = "Package documentation", },
  ["latex/eltex/pri_cz.pdf"] = {["lang"] = "cs", ["details"] = "Package documentation (Czech)", },
}
local tlp_from_runfile = {
  ["ltxtools-incluput"] = {["ltxtools"]=true,},
  ["lwarp-atbegshi"] = {["lwarp"]=true,},
  ["athnum"] = {["babel-greek"]=true,},
  ["kam-latex"] = {["enctex"]=true,},
  ["tikzlibrarygraphs.standard.code"] = {["pgf"]=true,},
  ["lwarp-hypdestopt"] = {["lwarp"]=true,},
  ["ncc"] = {["ncclatex"]=true,},
  ["memhangul-common"] = {["kotex-oblivoir"]=true,},
  ["esk"] = {["esk"]=true,},
  ["common-footnotes"] = {},
  ["eskdinfo"] = {["eskdx"]=true,},
  ["pfltrace"] = {["platex"]=true,},
  ["lollipop-document"] = {["lollipop"]=true,},
  ["cropmark"] = {},
  ["babel-maltese"] = {["babel"]=true,},
  ["babel-english-ca"] = {["babel"]=true,},
  ["caption3"] = {["caption"]=true,},
  ["armkb-u8"] = {["armtex"]=true,},
  ["fontloader-preprocessor"] = {["luaotfload"]=true,},
  ["luatexja-compat"] = {["luatexja"]=true,},
  ["octave"] = {["octave"]=true,},
  ["tikzpeople.shape.nun"] = {["tikzpeople"]=true,},
  ["fullpage"] = {["preprint"]=true,},
  ["silence"] = {["silence"]=true,},
  ["hyph-ia"] = {},
  ["pgfsyssoftpath.code"] = {["pgf"]=true,},
  ["beamerbasemisc"] = {["beamer"]=true,},
  ["pgflibraryvectorian.code"] = {["pgfornament"]=true,},
  ["tableof"] = {["tableof"]=true,},
  ["ltxcmds"] = {["oberdiek"]=true,},
  ["GFnotation"] = {["gfnotation"]=true,},
  ["abc"] = {["abc"]=true,},
  ["ecvNLS"] = {["ecv"]=true,},
  ["a0poster"] = {["a0poster"]=true,},
  ["lwarp-graphicx"] = {["lwarp"]=true,},
  ["newlfont"] = {["latex"]=true,},
  ["conteq"] = {["conteq"]=true,},
  ["beamerthemeCambridgeUS"] = {["beamer"]=true,},
  ["ltxguide"] = {["latex"]=true,},
  ["libertineMono-type1"] = {["libertine"]=true,},
  ["lwarp-fltrace"] = {["lwarp"]=true,},
  ["akkcs"] = {["akktex"]=true,},
  ["pgfmathfunctions.basic.code"] = {["pgf"]=true,},
  ["lwarp-titleps"] = {["lwarp"]=true,},
  ["UniversalisADFStd"] = {["universalis"]=true,},
  ["beamerthemeDresden"] = {["beamer"]=true,},
  ["lisp-mod-fpnum"] = {["lisp-on-tex"]=true,},
  ["babel-embu"] = {["babel"]=true,},
  ["bitelist"] = {["bitelist"]=true,},
  ["hgbthesis"] = {["hagenberg-thesis"]=true,},
  ["smartdiagramlibrarycore.commands.code"] = {["smartdiagram"]=true,},
  ["hyph-pl"] = {},
  ["blkarray"] = {["blkarray"]=true,},
  ["pgfmoduleoo.code"] = {["pgf"]=true,},
  ["ltr-navigation"] = {["lecturer"]=true,},
  ["tikzlibrarypgfplots.ternary.code"] = {["pgfplots"]=true,},
  ["begriff"] = {["begriff"]=true,},
  ["tgpagella"] = {["tex-gyre"]=true,},
  ["sugconf"] = {["sugconf"]=true,},
  ["infwarerr"] = {["oberdiek"]=true,},
  ["font_antt-condensed-medium"] = {["font-change"]=true,},
  ["mparhack"] = {["mparhack"]=true,},
  ["fancybox"] = {["fancybox"]=true,},
  ["antt-math"] = {["antt"]=true,},
  ["pgfcorelayers.code"] = {["pgf"]=true,},
  ["xob-hyper"] = {["kotex-oblivoir"]=true,},
  ["nccparskip"] = {["ncctools"]=true,},
  ["getfiledate"] = {["getfiledate"]=true,},
  ["pst-ode"] = {["pst-ode"]=true,},
  ["exercise"] = {["exercise"]=true,},
  ["appendixnumberbeamer"] = {["appendixnumberbeamer"]=true,},
  ["example-glossaries-childnoname"] = {["glossaries"]=true,},
  ["pageranges"] = {["inlinebib"]=true,},
  ["hieroglf"] = {["archaic"]=true,},
  ["papermas"] = {["papermas"]=true,},
  ["gztarticle"] = {["gzt"]=true,},
  ["babel-hausa"] = {["babel"]=true,},
  ["lltjp-fontspec"] = {["luatexja"]=true,},
  ["eskdspec"] = {["eskdx"]=true,},
  ["chemmacros.module.tikz.code"] = {["chemmacros"]=true,},
  ["llncsconf"] = {["llncsconf"]=true,},
  ["pressrelease"] = {["pressrelease"]=true,},
  ["allcmin"] = {["bookhands"]=true,},
  ["subfloat"] = {["subfloat"]=true,},
  ["revquantum"] = {["revquantum"]=true,},
  ["tipx"] = {["tipa"]=true,},
  ["tikzlibrarydecorations.shapes.code"] = {["pgf"]=true,},
  ["bxjspandoc"] = {["bxjscls"]=true,},
  ["abstbook"] = {},
  ["plain-utf8-cs"] = {["enctex"]=true,},
  ["t-pgfbpt"] = {["pgf"]=true,},
  ["beamerthemeSaintPetersburg"] = {["beamertheme-saintpetersburg"]=true,},
  ["lwarp-arabicfront"] = {["lwarp"]=true,},
  ["cspsfont"] = {["cslatex"]=true,},
  ["combine"] = {["combine"]=true,},
  ["akkcounterlabelpattern"] = {["akktex"]=true,},
  ["babel-ukenglish"] = {["babel"]=true,},
  ["simurgh-amsart"] = {["simurgh"]=true,},
  ["cc4llncs"] = {["computational-complexity"]=true,},
  ["babel-basque"] = {["babel"]=true,},
  ["nfssfox"] = {["lh"]=true,},
  ["advdate"] = {["advdate"]=true,},
  ["loadhyph-kmr"] = {},
  ["1250-t1"] = {["enctex"]=true,},
  ["gfsporson"] = {["gfsporson"]=true,},
  ["threadcol"] = {["threadcol"]=true,},
  ["enigma"] = {["enigma"]=true,},
  ["allimin"] = {["bookhands"]=true,},
  ["bpfats3"] = {["iso10303"]=true,},
  ["ctib"] = {["ctib"]=true,},
  ["beamerthemeMadrid"] = {["beamer"]=true,},
  ["cnltx-translations"] = {["cnltx"]=true,},
  ["luapackageloader"] = {["luapackageloader"]=true,},
  ["misccorr"] = {["t2"]=true,},
  ["seuthesix"] = {["seuthesix"]=true,},
  ["babel-tachelhit-latn"] = {["babel"]=true,},
  ["markblog"] = {["morehype"]=true,},
  ["bankstatement"] = {["bankstatement"]=true,},
  ["sctkzsym-base"] = {["scsnowman"]=true,},
  ["babel-oromo"] = {["babel"]=true,},
  ["pst-3dplot"] = {["pst-3dplot"]=true,},
  ["addrset"] = {["newlfm"]=true,},
  ["testfont"] = {},
  ["ebgaramond"] = {["ebgaramond"]=true,},
  ["sarabian"] = {["archaic"]=true,},
  ["droidserif"] = {["droid"]=true,},
  ["akkmathproof"] = {["akktex"]=true,},
  ["isodate"] = {["isodate"]=true,},
  ["arabicore"] = {["arabi"]=true,},
  ["cqubeamer"] = {["cqubeamer"]=true,},
  ["common-parmarkup"] = {},
  ["fontinst"] = {["fontinst"]=true,},
  ["babel-armenian"] = {["babel"]=true,},
  ["pgfcorepathprocessing.code"] = {["pgf"]=true,},
  ["easybib"] = {["easy"]=true,},
  ["testpage"] = {["latex"]=true,},
  ["babel-dutch"] = {["babel"]=true,},
  ["pgfplots.code"] = {["pgfplots"]=true,},
  ["novel-Images"] = {["novel"]=true,},
  ["s-simpleslides-default"] = {["context-simpleslides"]=true,},
  ["cmll"] = {["cmll"]=true,},
  ["lwarp-siunitx"] = {["lwarp"]=true,},
  ["sistyle"] = {["sistyle"]=true,},
  ["akkmathdisc"] = {["akktex"]=true,},
  ["FiraMono"] = {["fira"]=true,},
  ["unique"] = {["thmtools"]=true,},
  ["amsrefs"] = {["amsrefs"]=true,},
  ["authoraftertitle"] = {["authoraftertitle"]=true,},
  ["cmpj2"] = {["cmpj"]=true,},
  ["setouterhbox"] = {["oberdiek"]=true,},
  ["bidi-media9"] = {["bidi"]=true,},
  ["cmtiup"] = {["cmtiup"]=true,},
  ["ob-toclof"] = {["kotex-oblivoir"]=true,},
  ["grfpaste"] = {["grfpaste"]=true,},
  ["simplecv"] = {["simplecv"]=true,},
  ["lwarp-grffile"] = {["lwarp"]=true,},
  ["jspf"] = {["jsclasses"]=true,},
  ["hhsubfigure"] = {["bgteubner"]=true,},
  ["default"] = {},
  ["amsbsy"] = {["amsmath"]=true,},
  ["chletter"] = {["chletter"]=true,},
  ["zref-hyperref"] = {["oberdiek"]=true,},
  ["lwarp-flushend"] = {["lwarp"]=true,},
  ["tcbskinsjigsaw.code"] = {["tcolorbox"]=true,},
  ["lwarp-boxedminipage"] = {["lwarp"]=true,},
  ["binarytree"] = {["binarytree"]=true,},
  ["flushend"] = {["sttools"]=true,},
  ["komacv-addons"] = {["komacv-rg"]=true,},
  ["lastpackage"] = {["lastpackage"]=true,},
  ["pgfcorepoints.code"] = {["pgf"]=true,},
  ["pgflibrarycurvilinear.code"] = {["pgf"]=true,},
  ["pntext"] = {["petri-nets"]=true,},
  ["mdwlist"] = {["mdwtools"]=true,},
  ["hypertoc"] = {["morehype"]=true,},
  ["fusering"] = {["xymtex"]=true,},
  ["babel-portuguese-pt"] = {["babel"]=true,},
  ["scrlayer-scrpage"] = {},
  ["leipzig"] = {["leipzig"]=true,},
  ["phfthm"] = {["phfthm"]=true,},
  ["fixltx2e"] = {["latex"]=true,},
  ["multiply"] = {["bez123"]=true,},
  ["blackdvi"] = {["dvips"]=true,},
  ["nwejm"] = {["nwejm"]=true,},
  ["scrkbase"] = {},
  ["grmath"] = {["babel-greek"]=true,},
  ["gfsartemisia-euler"] = {["gfsartemisia"]=true,},
  ["lollipop"] = {["lollipop"]=true,},
  ["ltr-graphics"] = {["lecturer"]=true,},
  ["ghsystem_flame-3-black"] = {["ghsystem"]=true,},
  ["lollipop-plain"] = {["lollipop"]=true,},
  ["dblfont"] = {["bosisio"]=true,},
  ["phffullpagefigure"] = {["phffullpagefigure"]=true,},
  ["loadhyph-eu"] = {},
  ["lstbayes"] = {["lstbayes"]=true,},
  ["TXSmacs"] = {["texsis"]=true,},
  ["spanish"] = {["babel"]=true,},
  ["lwarp-shadow"] = {["lwarp"]=true,},
  ["powerdot-klope"] = {["powerdot"]=true,},
  ["ltjsbook"] = {["luatexja"]=true,},
  ["pgfplotsoldpgfsupp_pgfcoreimage.code"] = {["pgfplots"]=true,},
  ["movie"] = {["xypic"]=true,},
  ["babel-latvian"] = {["babel"]=true,},
  ["mceinleger"] = {["mceinleger"]=true,},
  ["tikzlibrarycircuits.plc.ladder.code"] = {["tikz-ladder"]=true,},
  ["stackengine"] = {["stackengine"]=true,},
  ["hyph-ga.ec"] = {},
  ["scite-context-readme"] = {["context"]=true,},
  ["jlreq-trimmarks"] = {["jlreq"]=true,},
  ["langsci-advertisement"] = {["langsci"]=true,},
  ["bpfap10"] = {["iso10303"]=true,},
  ["compare"] = {},
  ["luatex-gadgets"] = {["context"]=true,},
  ["glossary-longragged"] = {["glossaries"]=true,},
  ["skblncsppt"] = {["skb"]=true,},
  ["852-latex"] = {["enctex"]=true,},
  ["simurgh-harfi"] = {["simurgh"]=true,},
  ["elmath"] = {["elmath"]=true,},
  ["ESIEEcv"] = {["esieecv"]=true,},
  ["pgfplotstable"] = {["pgfplots"]=true,},
  ["imac"] = {["imac"]=true,},
  ["hmac"] = {["arabtex"]=true,},
  ["mdputu"] = {["mdputu"]=true,},
  ["jurabase"] = {["juramisc"]=true,},
  ["perpage"] = {["bigfoot"]=true,},
  ["amssym"] = {["amsfonts"]=true,},
  ["remreset"] = {["carlisle"]=true,},
  ["amac"] = {["arabtex"]=true,},
  ["graphicx"] = {["graphics-pln"]=true,["graphics"]=true,},
  ["lh-t2bccr"] = {["lh"]=true,},
  ["croptest"] = {},
  ["cclicenses"] = {["cclicenses"]=true,},
  ["twocolfix"] = {["tudscr"]=true,},
  ["authorarchive"] = {["authorarchive"]=true,},
  ["datetime2-it-fulltext"] = {["datetime2-it-fulltext"]=true,},
  ["makebox"] = {["makebox"]=true,},
  ["babel-assamese"] = {["babel"]=true,},
  ["simurgh-jalalical"] = {["simurgh"]=true,},
  ["chemmacros.module.orbital.code"] = {["chemmacros"]=true,},
  ["babel-icelandic"] = {["babel"]=true,},
  ["amsdtx"] = {["amscls"]=true,},
  ["hyph-be.t2a"] = {},
  ["hyph-ka.t8m"] = {},
  ["cochineal"] = {["cochineal"]=true,},
  ["pythontex"] = {["pythontex"]=true,},
  ["setdim"] = {["newlfm"]=true,},
  ["font_iwona-light"] = {["font-change"]=true,},
  ["textcomp"] = {["latex"]=true,},
  ["datenumber"] = {["datenumber"]=true,},
  ["bidituftegeneralstructure"] = {["bidi"]=true,},
  ["babel-telugu"] = {["babel"]=true,},
  ["cs-schola"] = {},
  ["plex-otf"] = {["plex-otf"]=true,},
  ["null"] = {},
  ["GS1"] = {["gs1"]=true,},
  ["lettre"] = {["lettre"]=true,},
  ["babel-asturian"] = {["babel"]=true,},
  ["spotcolorhks"] = {["spotcolor"]=true,},
  ["s-simpleslides-BottomSquares"] = {["context-simpleslides"]=true,},
  ["beamerouterthemesmoothbars"] = {["beamer"]=true,},
  ["longdivision"] = {["longdivision"]=true,},
  ["klutab"] = {["kluwer"]=true,},
  ["babel-sinhala"] = {["babel"]=true,},
  ["mathabx"] = {["mathabx"]=true,},
  ["makerobust"] = {["oberdiek"]=true,},
  ["nanumfontsel"] = {["kotex-oblivoir"]=true,},
  ["mmap"] = {["mmap"]=true,},
  ["ghsystem_flame-4-2"] = {["ghsystem"]=true,},
  ["oblivoir-xlua"] = {["kotex-oblivoir"]=true,},
  ["compsci"] = {["frankenstein"]=true,},
  ["tensind"] = {["bezos"]=true,},
  ["ghsystem_skull"] = {["ghsystem"]=true,},
  ["jkmath"] = {["jkmath"]=true,},
  ["luatexbase"] = {["luatexbase"]=true,},
  ["dtk"] = {["dtk"]=true,},
  ["easyvector"] = {["easy"]=true,},
  ["bibarts"] = {["bibarts"]=true,},
  ["beamerthemeclassic"] = {["beamer"]=true,},
  ["amsmidx"] = {["amscls"]=true,},
  ["hgbreport"] = {["hagenberg-thesis"]=true,},
  ["ragged"] = {},
  ["scrletter"] = {},
  ["ketpic"] = {["ketcindy"]=true,},
  ["pgfmanual"] = {["pgf"]=true,},
  ["simurgh-leqno"] = {["simurgh"]=true,},
  ["biolist"] = {["shipunov"]=true,},
  ["tikzlibrarycircuits.logic.CDH.code"] = {["pgf"]=true,},
  ["apptools"] = {["apptools"]=true,},
  ["lettrine"] = {["lettrine"]=true,},
  ["arevtext"] = {["arev"]=true,},
  ["italian"] = {["babel"]=true,},
  ["circ"] = {["circ"]=true,},
  ["tp-crf"] = {["gustlib"]=true,},
  ["txfontsb"] = {["txfontsb"]=true,},
  ["fxtargetlayoutcolorcb"] = {["fixme"]=true,},
  ["xymtex"] = {["xymtex"]=true,},
  ["babel-cherokee"] = {["babel"]=true,},
  ["vwcol"] = {["vwcol"]=true,},
  ["loadhyph-lv"] = {},
  ["inslrmin"] = {["bookhands"]=true,},
  ["tikzlibrarydecorations.code"] = {["pgf"]=true,},
  ["noto-mono"] = {["noto"]=true,},
  ["xsim.base.code"] = {["xsim"]=true,},
  ["babel-bemba"] = {["babel"]=true,},
  ["ucsutils"] = {["ucs"]=true,},
  ["bibcheck"] = {},
  ["greekmacros"] = {["levy"]=true,},
  ["linegoal"] = {["linegoal"]=true,},
  ["keyreader"] = {["keyreader"]=true,},
  ["drs"] = {["drs"]=true,},
  ["xyv2"] = {["xypic"]=true,},
  ["loadhyph-en-us"] = {},
  ["cnltx-names"] = {["cnltx"]=true,},
  ["forest-compat"] = {["forest"]=true,},
  ["uebungsblatt"] = {["uebungsblatt"]=true,},
  ["lwarp-hyperref"] = {["lwarp"]=true,},
  ["tikzlibraryintersections.code"] = {["pgf"]=true,},
  ["scrbookreportarticle-experts"] = {},
  ["gregoriotex-nabc"] = {["gregoriotex"]=true,},
  ["timetable"] = {},
  ["guide-english"] = {},
  ["beamerinnerthememetropolis"] = {["beamertheme-metropolis"]=true,},
  ["filemod-expmin"] = {["filemod"]=true,},
  ["univie-ling-expose"] = {["univie-ling"]=true,},
  ["ghsystem_flame-5-2-white"] = {["ghsystem"]=true,},
  ["unitsdef"] = {["unitsdef"]=true,},
  ["ifptex"] = {["ifptex"]=true,},
  ["lwarp-wallpaper"] = {["lwarp"]=true,},
  ["mpsproof"] = {["metapost"]=true,},
  ["authorindex"] = {["authorindex"]=true,},
  ["font_cm"] = {["font-change"]=true,},
  ["lwarp-romanbar"] = {["lwarp"]=true,},
  ["fp-eval"] = {["fp"]=true,},
  ["beamercolorthemesidebartab"] = {["beamer"]=true,},
  ["tex4ht"] = {["tex4ht"]=true,},
  ["hijrical"] = {["polyglossia"]=true,},
  ["german"] = {["german"]=true,},
  ["stix2"] = {["stix2-type1"]=true,},
  ["sansmath"] = {["sansmath"]=true,},
  ["example-glossaries-xr"] = {["glossaries-extra"]=true,},
  ["lwarp-inputenc"] = {["lwarp"]=true,},
  ["babel-romansh"] = {["babel"]=true,},
  ["fsbmath"] = {["hrlatex"]=true,},
  ["langcode"] = {["langcode"]=true,},
  ["textcmds"] = {["amsrefs"]=true,},
  ["olabels"] = {["labels"]=true,},
  ["mathscinet"] = {["amsrefs"]=true,},
  ["outlining"] = {["outlining"]=true,},
  ["undertilde"] = {},
  ["arfonts"] = {["arabi"]=true,},
  ["l3tl-analysis"] = {["l3kernel"]=true,},
  ["example-glossaries-symbolnames"] = {["glossaries"]=true,},
  ["CJKvert"] = {["cjk"]=true,},
  ["exaccent"] = {["tipa"]=true,},
  ["frfonts"] = {["arabi"]=true,},
  ["pgfplotsoldpgfsupp_loader.code"] = {["pgfplots"]=true,},
  ["extreport"] = {["extsizes"]=true,},
  ["ctexcap"] = {["ctex"]=true,},
  ["hyph-hr.ec"] = {},
  ["xycurve"] = {["xypic"]=true,},
  ["showcharinbox"] = {["showcharinbox"]=true,},
  ["nccltrus"] = {["ncclatex"]=true,},
  ["logicpuzzle"] = {["logicpuzzle"]=true,},
  ["beamerthemeUR"] = {["urcls"]=true,},
  ["ccicons"] = {["ccicons"]=true,},
  ["lh-t2cccr"] = {["lh"]=true,},
  ["skhyphen"] = {},
  ["changepage"] = {["changepage"]=true,},
  ["dad"] = {["dad"]=true,},
  ["lisp-gc"] = {["lisp-on-tex"]=true,},
  ["sseqmacromakers.code"] = {["spectralsequences"]=true,},
  ["txi-pl"] = {},
  ["marginfit"] = {["marginfit"]=true,},
  ["forest-lib-edges"] = {["forest"]=true,},
  ["itaxpf"] = {["economic"]=true,},
  ["hyph-fr.ec"] = {},
  ["graphbox"] = {["graphbox"]=true,},
  ["loadhyph-sr-latn"] = {},
  ["mtx-context-sql"] = {["context"]=true,},
  ["thumbs"] = {["thumbs"]=true,},
  ["852-il2"] = {["enctex"]=true,},
  ["babel-faroese"] = {["babel"]=true,},
  ["TXSletr"] = {["texsis"]=true,},
  ["pygmentex"] = {["pygmentex"]=true,},
  ["mathpartir"] = {["mathpartir"]=true,},
  ["czhyphen"] = {},
  ["pgfplotsoldpgfsupp_pgfcorescopes.code"] = {["pgfplots"]=true,},
  ["loadhyph-cy"] = {},
  ["nccfloats"] = {["ncctools"]=true,},
  ["jacow"] = {["jacow"]=true,},
  ["gatech-thesis-patch"] = {["gatech-thesis"]=true,},
  ["dblfnote"] = {["yafoot"]=true,},
  ["oplotsymbl"] = {["oplotsymbl"]=true,},
  ["babel-bambara"] = {["babel"]=true,},
  ["t-pgfbpl"] = {["pgf"]=true,},
  ["cancel"] = {["cancel"]=true,},
  ["tinos"] = {["tinos"]=true,},
  ["dolines"] = {["midnight"]=true,},
  ["tikzpeople.shape.physician"] = {["tikzpeople"]=true,},
  ["booklet"] = {["booklet"]=true,},
  ["ogrmhyph4"] = {["antomega"]=true,},
  ["lcd"] = {["lcd"]=true,},
  ["beamerbasenavigation"] = {["beamer"]=true,},
  ["ofs-cm"] = {["ofs"]=true,},
  ["options"] = {["options"]=true,},
  ["nidanfloat"] = {["nidanfloat"]=true,},
  ["hyph-pt.ec"] = {},
  ["beamercolorthemespruce"] = {["beamer"]=true,},
  ["xmpmulti"] = {["beamer"]=true,},
  ["pdfpc-movie"] = {["pdfpc-movie"]=true,},
  ["TXSconts"] = {["texsis"]=true,},
  ["xsim.auxfile.code"] = {["xsim"]=true,},
  ["biblatex-true-citepages-omit"] = {["biblatex-true-citepages-omit"]=true,},
  ["gfsneohellenic"] = {["gfsneohellenic"]=true,},
  ["bxjsbook"] = {["bxjscls"]=true,},
  ["scrjrnl"] = {["scrjrnl"]=true,},
  ["mls"] = {["montex"]=true,},
  ["beamerthemeAachen"] = {["beamerposter"]=true,},
  ["sslides"] = {["sslides"]=true,},
  ["xynecula"] = {["xypic"]=true,},
  ["bxjsreport"] = {["bxjscls"]=true,},
  ["libertineRoman"] = {["libertine"]=true,},
  ["blochsphere"] = {["blochsphere"]=true,},
  ["interfaces-titlesec"] = {["interfaces"]=true,},
  ["lwarp-hyperxmp"] = {["lwarp"]=true,},
  ["xetexko-space"] = {["xetexko"]=true,},
  ["modules"] = {["stex"]=true,},
  ["babel-french-canada"] = {["babel"]=true,},
  ["breton"] = {["babel"]=true,},
  ["tikzlibrarycircuits.logic.IEC.code"] = {["pgf"]=true,},
  ["s-simpleslides-BigNumber"] = {["context-simpleslides"]=true,},
  ["idverb"] = {["js-misc"]=true,},
  ["pspicture"] = {["pspicture"]=true,},
  ["imsproc"] = {["imsproc"]=true,},
  ["lwarp-chappg"] = {["lwarp"]=true,},
  ["ted"] = {["ted"]=true,},
  ["amsproc"] = {["amscls"]=true,},
  ["lwarp-wasysym"] = {["lwarp"]=true,},
  ["lambda"] = {["lambda-lists"]=true,},
  ["counttexruns"] = {["counttexruns"]=true,},
  ["changelayout"] = {["changelayout"]=true,},
  ["beamercolorthemerose"] = {["beamer"]=true,},
  ["kotex-varioref"] = {["kotex-utf"]=true,},
  ["cdpaddon"] = {["cdpbundl"]=true,},
  ["beamerouterthemesplit"] = {["beamer"]=true,},
  ["gfsbaskerville"] = {["gfsbaskerville"]=true,},
  ["luacode"] = {["luacode"]=true,},
  ["linearb"] = {["archaic"]=true,},
  ["beamerbaseoptions"] = {["beamer"]=true,},
  ["babel-norsk"] = {["babel"]=true,},
  ["dingbat"] = {["dingbat"]=true,},
  ["clipboard"] = {["clipboard"]=true,},
  ["showframe"] = {["eso-pic"]=true,},
  ["avr.io"] = {["avremu"]=true,},
  ["guitar"] = {["guitar"]=true,},
  ["lwarp-adjmulticol"] = {["lwarp"]=true,},
  ["volumes"] = {["volumes"]=true,},
  ["beamerouterthemehohenheimposter"] = {["hobete"]=true,},
  ["ghsystem_explos-4"] = {["ghsystem"]=true,},
  ["hyph-pa"] = {},
  ["hyph-lv.l7x"] = {},
  ["bera"] = {["bera"]=true,},
  ["skb"] = {["skb"]=true,},
  ["babel-greek"] = {["babel"]=true,},
  ["glossaries-polyglossia"] = {["glossaries"]=true,},
  ["macro-orion"] = {["pas-cv"]=true,},
  ["hawking"] = {["context"]=true,},
  ["gregoriotex-symbols"] = {["gregoriotex"]=true,},
  ["babel-nswissgerman"] = {["babel"]=true,},
  ["apreambl"] = {["abstyles"]=true,},
  ["conv-utf8-t8m"] = {["hyph-utf8"]=true,},
  ["bpfap13"] = {["iso10303"]=true,},
  ["bxjalipsum"] = {["bxjalipsum"]=true,},
  ["adjmulticol"] = {["adjmulticol"]=true,},
  ["epstopdf-base"] = {["oberdiek"]=true,},
  ["cachepic"] = {["cachepic"]=true,},
  ["glossaries-compatible-307"] = {["glossaries"]=true,},
  ["hgb"] = {["hagenberg-thesis"]=true,},
  ["shorttoc"] = {["shorttoc"]=true,},
  ["bropd"] = {["bropd"]=true,},
  ["beamerthemehohenheim"] = {["hobete"]=true,},
  ["dehypht-x-2018-03-31"] = {["dehyph-exptl"]=true,},
  ["nimbussans"] = {["nimbus15"]=true,},
  ["scrlttr2-experts"] = {},
  ["ifxetex"] = {["ifxetex"]=true,},
  ["skbbook"] = {["skb"]=true,},
  ["algorithmicx"] = {["algorithmicx"]=true,},
  ["lwarp-subfig"] = {["lwarp"]=true,},
  ["bhslabel"] = {["arabtex"]=true,},
  ["bpfap15"] = {["iso10303"]=true,},
  ["lwarp-moreverb"] = {["lwarp"]=true,},
  ["emptypage"] = {["emptypage"]=true,},
  ["bhs"] = {["arabtex"]=true,},
  ["longfigure"] = {["longfigure"]=true,},
  ["drac"] = {["drac"]=true,},
  ["lnavicol"] = {["morehype"]=true,},
  ["hyph-af"] = {},
  ["runic"] = {["archaic"]=true,},
  ["CormorantGaramond"] = {["cormorantgaramond"]=true,},
  ["juraabbrev"] = {["juraabbrev"]=true,},
  ["fontloader-swiglib-test"] = {["luaotfload"]=true,},
  ["leading"] = {["leading"]=true,},
  ["expdlist"] = {["expdlist"]=true,},
  ["dumyhyph"] = {},
  ["makecell"] = {["makecell"]=true,},
  ["pgfcircutils"] = {["circuitikz"]=true,},
  ["argetabelle"] = {["bewerbung"]=true,},
  ["7point"] = {["varisize"]=true,},
  ["babel-uzbek-cyrl"] = {["babel"]=true,},
  ["afonts"] = {["arabtex"]=true,},
  ["etextools"] = {["etextools"]=true,},
  ["color"] = {["graphics-pln"]=true,["graphics"]=true,},
  ["luatexja-fontspec"] = {["luatexja"]=true,},
  ["tikzposterNotestyles"] = {["tikzposter"]=true,},
  ["lblchng1"] = {["ednotes"]=true,},
  ["manyind"] = {["manyind"]=true,},
  ["cours"] = {["mafr"]=true,},
  ["pgfcircflow"] = {["circuitikz"]=true,},
  ["labbook"] = {["labbook"]=true,},
  ["coelacanth"] = {["coelacanth"]=true,},
  ["simurgh-artikel2"] = {["simurgh"]=true,},
  ["x"] = {["tools"]=true,},
  ["pnets"] = {["petri-nets"]=true,},
  ["eskdbiblist"] = {["eskdx"]=true,},
  ["histogr"] = {["histogr"]=true,},
  ["interlingua"] = {["babel"]=true,},
  ["ordinalpt"] = {["ordinalpt"]=true,},
  ["hepnames"] = {["hepnames"]=true,},
  ["makeglos"] = {["makeglos"]=true,},
  ["lltjp-preview"] = {["luatexja"]=true,},
  ["tocstyle"] = {},
  ["font_kp"] = {["font-change"]=true,},
  ["pxgentombow"] = {["gentombow"]=true,},
  ["babel-friulian"] = {["babel"]=true,},
  ["example-image-letter-landscape"] = {["mwe"]=true,},
  ["s"] = {["tools"]=true,},
  ["loadhyph-de-1901"] = {},
  ["TS1mac"] = {["fontch"]=true,},
  ["reflectgraphics"] = {["reflectgraphics"]=true,},
  ["utf8off"] = {["enctex"]=true,},
  ["1250-latex"] = {["enctex"]=true,},
  ["zhubuntufonts"] = {["ctex"]=true,},
  ["interfaces-embedfile"] = {["interfaces"]=true,},
  ["common-options"] = {},
  ["9point"] = {["varisize"]=true,},
  ["lwarp-morefloats"] = {["lwarp"]=true,},
  ["coursepaper"] = {},
  ["loadhyph-la-x-liturgic"] = {},
  ["hyph-mul-ethi"] = {},
  ["lwarp-sidenotes"] = {["lwarp"]=true,},
  ["tikzlibrarycalendar.code"] = {["pgf"]=true,},
  ["xob-amssymb"] = {["kotex-oblivoir"]=true,},
  ["fcuse"] = {},
  ["lwarp-tabulary"] = {["lwarp"]=true,},
  ["mycv_style"] = {["mycv"]=true,},
  ["VocalTract"] = {["vocaltract"]=true,},
  ["lmfonts"] = {},
  ["fcprefix"] = {["fmtcount"]=true,},
  ["eskdexplan"] = {["eskdx"]=true,},
  ["bondgraph"] = {["bondgraph"]=true,},
  ["musixpoi"] = {["musixtex"]=true,},
  ["fancyvrb"] = {["fancyvrb"]=true,},
  ["musixadf"] = {["musixtex"]=true,},
  ["knittingpattern"] = {["knittingpattern"]=true,},
  ["oubraces"] = {["oubraces"]=true,},
  ["flagderiv"] = {["flagderiv"]=true,},
  ["3parttable"] = {["threeparttable"]=true,},
  ["myletter"] = {["akletter"]=true,},
  ["macro-andromede"] = {["pas-cv"]=true,},
  ["cmbright"] = {["cmbright"]=true,},
  ["mlnames"] = {},
  ["cnltx-doc"] = {["cnltx"]=true,},
  ["hyph-lv"] = {},
  ["uestcthesis"] = {["uestcthesis"]=true,},
  ["gmverb"] = {["gmverb"]=true,},
  ["tikzlibrarypgfplots.colormaps.code"] = {["pgfplots"]=true,},
  ["dataplot"] = {["datatool"]=true,},
  ["interfaces-LaTeX"] = {["interfaces"]=true,},
  ["phn-AT_Ortsnamen"] = {["phonenumbers"]=true,},
  ["akkmath"] = {["akktex"]=true,},
  ["leadsheet"] = {["leadsheets"]=true,},
  ["upkcat"] = {["pxbase"]=true,},
  ["TXSfigs"] = {["texsis"]=true,},
  ["scrdatetime"] = {},
  ["HAPHA"] = {["ha-prosper"]=true,},
  ["Elzevier"] = {["cfr-initials"]=true,},
  ["loadhyph-la-x-classic"] = {},
  ["novel-ChapterScene"] = {["novel"]=true,},
  ["welsh"] = {["babel"]=true,},
  ["eskdstamp"] = {["eskdx"]=true,},
  ["etex-mac"] = {},
  ["Nouveaud"] = {["cfr-initials"]=true,},
  ["ruhyph16"] = {["antomega"]=true,},
  ["actuarialangle"] = {["actuarialangle"]=true,},
  ["lwarp-acronym"] = {["lwarp"]=true,},
  ["tikzlibrarydatavisualization.formats.functions.code"] = {["pgf"]=true,},
  ["gfsartemisia"] = {["gfsartemisia"]=true,},
  ["fr_rs"] = {["r_und_s"]=true,},
  ["alltgoth"] = {["bookhands"]=true,},
  ["langsci-bidi"] = {["langsci"]=true,},
  ["kluopen"] = {["kluwer"]=true,},
  ["tgbonum"] = {["tex-gyre"]=true,},
  ["ipaex-type1"] = {["ipaex-type1"]=true,},
  ["nccbbb"] = {["ncctools"]=true,},
  ["ghsystem_flame-3-white"] = {["ghsystem"]=true,},
  ["xysmart"] = {["xypic"]=true,},
  ["pgflibraryplotmarks"] = {["pgf"]=true,},
  ["ibyhyph"] = {},
  ["boxedminipage2e"] = {["boxedminipage2e"]=true,},
  ["pgfmathfunctions.integerarithmetics.code"] = {["pgf"]=true,},
  ["ulthese"] = {["ulthese"]=true,},
  ["breakcites"] = {["breakcites"]=true,},
  ["s-pre-26"] = {["context"]=true,},
  ["numspell-french"] = {["numspell"]=true,},
  ["gtrlib.largetrees.code"] = {["gtrlib-largetrees"]=true,},
  ["hyph-cs"] = {},
  ["ptptex"] = {["ptptex"]=true,},
  ["makecmds"] = {["makecmds"]=true,},
  ["hyrules"] = {["hyplain"]=true,},
  ["powerdot-ciment"] = {["powerdot"]=true,},
  ["gtrlang.swedish.code"] = {["genealogytree"]=true,},
  ["unicode-math-xetex"] = {["unicode-math"]=true,},
  ["lltjfont"] = {["luatexja"]=true,},
  ["arsclassica"] = {["arsclassica"]=true,},
  ["tikz-page"] = {["tikz-page"]=true,},
  ["mypsizzl"] = {},
  ["mintspirit2"] = {["mintspirit"]=true,},
  ["ibycus4"] = {["ibygrk"]=true,},
  ["hyph-eo"] = {},
  ["pst-barcode"] = {["pst-barcode"]=true,},
  ["ncclatex"] = {["ncclatex"]=true,},
  ["achemso"] = {["achemso"]=true,},
  ["zxjafbfont"] = {["zxjafbfont"]=true,},
  ["sidecap"] = {["sidecap"]=true,},
  ["abntex2"] = {["abntex2"]=true,},
  ["doafter"] = {["mdwtools"]=true,},
  ["amsmath"] = {["amsmath"]=true,},
  ["bez123"] = {["bez123"]=true,},
  ["lua-visual-debug"] = {["lua-visual-debug"]=true,},
  ["lwarp-xy"] = {["lwarp"]=true,},
  ["foekfont"] = {["foekfont"]=true,},
  ["simurgh-clss"] = {["simurgh"]=true,},
  ["muthesis"] = {["muthesis"]=true,},
  ["powerdot-jefka"] = {["powerdot"]=true,},
  ["s-simpleslides-Split"] = {["context-simpleslides"]=true,},
  ["PTSansCaption"] = {["paratype"]=true,},
  ["g-brief"] = {["g-brief"]=true,},
  ["fbb"] = {["fbb"]=true,},
  ["emtrees"] = {},
  ["pgflibrarypgfplots.colorbrewer.code"] = {["pgfplots"]=true,},
  ["babel-belarusian"] = {["babel"]=true,},
  ["babel-danish"] = {["babel"]=true,},
  ["hobsub-generic"] = {["oberdiek"]=true,},
  ["context-test"] = {["context"]=true,},
  ["Rothdn"] = {["cfr-initials"]=true,},
  ["mfirstuc-english"] = {["mfirstuc"]=true,},
  ["pmboxdraw"] = {["oberdiek"]=true,},
  ["xsim.grades.code"] = {["xsim"]=true,},
  ["Starburst"] = {["cfr-initials"]=true,},
  ["luatexbase-mcb"] = {["luatexbase"]=true,},
  ["upmethodology-extension"] = {["upmethodology"]=true,},
  ["autobreak"] = {["autobreak"]=true,},
  ["lwarp-needspace"] = {["lwarp"]=true,},
  ["babel-mexican"] = {["babel"]=true,},
  ["pxrubrica"] = {["pxrubrica"]=true,},
  ["lwarp-addlines"] = {["lwarp"]=true,},
  ["lwarp-fnpos"] = {["lwarp"]=true,},
  ["fontspec-luatex"] = {["fontspec"]=true,},
  ["bidi-symbols"] = {["context"]=true,},
  ["ojapan"] = {},
  ["pictexwd"] = {["pictex"]=true,},
  ["bgfonts"] = {["bgreek"]=true,},
  ["bpfs2"] = {["iso10303"]=true,},
  ["adtrees"] = {["adtrees"]=true,},
  ["EileenBl"] = {["cfr-initials"]=true,},
  ["jsreport"] = {["jsclasses"]=true,},
  ["univie-ling-thesis"] = {["univie-ling"]=true,},
  ["drftcite"] = {["cite"]=true,},
  ["usthesis"] = {["stellenbosch"]=true,},
  ["lwarp-listings"] = {["lwarp"]=true,},
  ["scrlayer-notecolumn-example"] = {},
  ["newenviron"] = {["newenviron"]=true,},
  ["Typocaps"] = {["cfr-initials"]=true,},
  ["l3draw"] = {["l3experimental"]=true,},
  ["s-cdr-01"] = {["context"]=true,},
  ["allhmin"] = {["bookhands"]=true,},
  ["ukrhypst"] = {["ukrhyph"]=true,},
  ["ligtable"] = {["arabtex"]=true,},
  ["curves"] = {["curves"]=true,},
  ["moderncvcompatibility"] = {["moderncv"]=true,},
  ["zref-xr"] = {["oberdiek"]=true,},
  ["hyph-cs.ec"] = {},
  ["nccmath"] = {["ncctools"]=true,},
  ["xsim.properties.code"] = {["xsim"]=true,},
  ["xfntinst"] = {["fontinst"]=true,},
  ["hcolor"] = {["fancyvrb"]=true,},
  ["gfscomplutum"] = {["gfscomplutum"]=true,},
  ["nl_rs"] = {["r_und_s"]=true,},
  ["plarray"] = {["platex-tools"]=true,},
  ["txi-sr"] = {},
  ["keyvald"] = {["cweb-latex"]=true,},
  ["bibtopicprefix"] = {["bibtopicprefix"]=true,},
  ["fontmfizz"] = {["fontmfizz"]=true,},
  ["common-dictum"] = {},
  ["wrapfig"] = {["wrapfig"]=true,},
  ["shortvrb"] = {["latex"]=true,},
  ["nhelvet"] = {["cslatex"]=true,},
  ["cfntinst"] = {["fontinst"]=true,},
  ["linguho"] = {["linguex"]=true,},
  ["hefonts"] = {["arabtex"]=true,},
  ["lcy2ucy"] = {["ukrhyph"]=true,},
  ["t4phonet"] = {},
  ["ctexheading"] = {["ctex"]=true,},
  ["utreport"] = {["uplatex"]=true,},
  ["chelvet"] = {},
  ["foilhtml"] = {["foilhtml"]=true,},
  ["eskd"] = {["eskd"]=true,},
  ["babel-aghem"] = {["babel"]=true,},
  ["babel-vunjo"] = {["babel"]=true,},
  ["PPRdefault"] = {["prosper"]=true,},
  ["doclicense-CC-by-nc-sa-4.0-latex"] = {["doclicense"]=true,},
  ["ograhyph4"] = {["antomega"]=true,},
  ["pgflibraryarrows.code"] = {["pgf"]=true,},
  ["nomencl"] = {["nomencl"]=true,},
  ["gtrlang.italian.code"] = {["genealogytree"]=true,},
  ["ifmslide"] = {["ifmslide"]=true,},
  ["zhwindowsfonts"] = {["ctex"]=true,},
  ["babel-ossetic"] = {["babel"]=true,},
  ["cs-polta"] = {},
  ["tikzlibrarymath.code"] = {["pgf"]=true,},
  ["unswcover"] = {["unswcover"]=true,},
  ["ltjltxdoc"] = {["luatexja"]=true,},
  ["mkpatter"] = {["mkpattern"]=true,},
  ["cookingsymbols"] = {["cookingsymbols"]=true,},
  ["fxthemecolor"] = {["fixme"]=true,},
  ["exam-n"] = {["exam-n"]=true,},
  ["doclicense-CC-by-nc-3.0-plaintext"] = {["doclicense"]=true,},
  ["bashful"] = {["bashful"]=true,},
  ["tikzfeynman.keys.code"] = {["tikz-feynman"]=true,},
  ["cyryoas"] = {},
  ["uni-math"] = {},
  ["unitn-bimrep"] = {["unitn-bimrep"]=true,},
  ["fakebold"] = {},
  ["zwpagelayout"] = {["zwpagelayout"]=true,},
  ["addfont"] = {["addfont"]=true,},
  ["bpfap11"] = {["iso10303"]=true,},
  ["musixgre"] = {["musixtex"]=true,},
  ["DejaVuSansMono"] = {["dejavu"]=true,},
  ["phonenumbers"] = {["phonenumbers"]=true,},
  ["font_libertine_palatino"] = {["font-change"]=true,},
  ["babel-chinese-hant-hk"] = {["babel"]=true,},
  ["hyph-zh-latn-pinyin.ec"] = {},
  ["lwarp-hang"] = {["lwarp"]=true,},
  ["pgfmathcalc.code"] = {["pgf"]=true,},
  ["l3doc"] = {["l3kernel"]=true,},
  ["bigfoot"] = {["bigfoot"]=true,},
  ["guide-ngerman"] = {},
  ["simurgh-tags"] = {["simurgh"]=true,},
  ["gauss"] = {["gauss"]=true,},
  ["fxthemecolorsig"] = {["fixme"]=true,},
  ["lwarp-realscripts"] = {["lwarp"]=true,},
  ["petiteannonce"] = {["petiteannonce"]=true,},
  ["mfpic4ode"] = {["mfpic4ode"]=true,},
  ["tkz-lib-symbols"] = {["tkz-euclide"]=true,},
  ["ling-macros"] = {["ling-macros"]=true,},
  ["croatian"] = {["babel"]=true,},
  ["lwarp-showkeys"] = {["lwarp"]=true,},
  ["pst-vowel"] = {["pst-vowel"]=true,},
  ["phfnote"] = {["phfnote"]=true,},
  ["parselines"] = {["parselines"]=true,},
  ["shellesc"] = {["tools"]=true,},
  ["inscrutable"] = {},
  ["cprotect"] = {["cprotect"]=true,},
  ["combinedgraphics"] = {["combinedgraphics"]=true,},
  ["mnhyphex"] = {["montex"]=true,},
  ["bpfap3"] = {["iso10303"]=true,},
  ["ruhyphen"] = {},
  ["il2code"] = {},
  ["t-gates"] = {["gates"]=true,},
  ["luafonts"] = {},
  ["beamerouterthemeCuerna"] = {["beamertheme-cuerna"]=true,},
  ["mtcmess"] = {["minitoc"]=true,},
  ["tikzlibraryceltic.code"] = {["celtic"]=true,},
  ["arabrep1"] = {["arabtex"]=true,},
  ["mlp-33"] = {["e-french"]=true,},
  ["hefonts0"] = {["arabtex"]=true,},
  ["bibl-ams"] = {["context"]=true,},
  ["s-simpleslides-Sunrise"] = {["context-simpleslides"]=true,},
  ["paper"] = {["paper"]=true,},
  ["skeyval-view"] = {["skeyval"]=true,},
  ["hexgame"] = {["hexgame"]=true,},
  ["tcbbreakable.code"] = {["tcolorbox"]=true,},
  ["bpfs1"] = {["iso10303"]=true,},
  ["numspell-german"] = {["numspell"]=true,},
  ["tcbdocumentation.code"] = {["tcolorbox"]=true,},
  ["thumby"] = {["thumby"]=true,},
  ["noprefnt"] = {["enctex"]=true,},
  ["tikz-qtree"] = {["tikz-qtree"]=true,},
  ["nohyperref"] = {["hyperref"]=true,},
  ["verbasef"] = {["verbasef"]=true,},
  ["acronym"] = {["acronym"]=true,},
  ["langsci-hyphenation-universal"] = {["langsci"]=true,},
  ["flashcard"] = {},
  ["beamerthemeMontpellier"] = {["beamer"]=true,},
  ["ltr-slides"] = {["lecturer"]=true,},
  ["chemmacros.module.charges.code"] = {["chemmacros"]=true,},
  ["eltex2"] = {["eltex"]=true,},
  ["hylang"] = {["hyplain"]=true,},
  ["ua-title"] = {["uaclasses"]=true,},
  ["dashundergaps"] = {["dashundergaps"]=true,},
  ["pgflibraryprofiler.code"] = {["pgf"]=true,},
  ["blockdiagram"] = {["texdraw"]=true,},
  ["mathtext"] = {["t2"]=true,},
  ["ujreport"] = {["uplatex"]=true,},
  ["url"] = {["url"]=true,},
  ["ctxdoc"] = {["ctex"]=true,},
  ["hefonts2"] = {["arabtex"]=true,},
  ["moderntimeline"] = {["moderntimeline"]=true,},
  ["figbib"] = {["figbib"]=true,},
  ["lwarp-relsize"] = {["lwarp"]=true,},
  ["pgflibraryfpu.code"] = {["pgf"]=true,},
  ["chemscheme"] = {["chemstyle"]=true,},
  ["ebsthesis"] = {["ebsthesis"]=true,},
  ["seminar"] = {["seminar"]=true,},
  ["mtx-context-meaning"] = {["context"]=true,},
  ["babel-yiddish"] = {["babel"]=true,},
  ["hacm"] = {["hacm"]=true,},
  ["beamerthemelined"] = {["beamer"]=true,},
  ["pshan"] = {["cjk"]=true,},
  ["moderncvstyleclassic"] = {["moderncv"]=true,},
  ["rtkinenc"] = {["rtkinenc"]=true,},
  ["beamerthemeI6td"] = {["beamerposter"]=true,},
  ["grhyph16"] = {["antomega"]=true,},
  ["babel-kikuyu"] = {["babel"]=true,},
  ["hyph-sh-latn.ec"] = {},
  ["tikzlibraryfit.code"] = {["pgf"]=true,},
  ["hulipsum"] = {["hulipsum"]=true,},
  ["glossaries-prefix"] = {["glossaries"]=true,},
  ["tlinfra-guit07"] = {},
  ["lltjp-unicode-math"] = {["luatexja"]=true,},
  ["loadhyph-lt"] = {},
  ["wallcalendar-french"] = {["wallcalendar"]=true,},
  ["hebrew_p"] = {["babel-hebrew"]=true,},
  ["fncychap"] = {["fncychap"]=true,},
  ["parskip"] = {["parskip"]=true,},
  ["pointers"] = {["pictex"]=true,},
  ["calendrierfpar"] = {["arabi"]=true,},
  ["hyph-cu"] = {},
  ["rmpage"] = {["rmpage"]=true,},
  ["loops"] = {["loops"]=true,},
  ["birm"] = {["burmese"]=true,},
  ["ellipsis"] = {["ellipsis"]=true,},
  ["bidituftetoc"] = {["bidi"]=true,},
  ["pgfcircmonopoles"] = {["circuitikz"]=true,},
  ["font_palatino"] = {["font-change"]=true,},
  ["merriweather"] = {["merriweather"]=true,},
  ["babel-malayalam"] = {["babel"]=true,},
  ["babel-english-newzealand"] = {["babel"]=true,},
  ["tikzpeople.shape.bob"] = {["tikzpeople"]=true,},
  ["accents"] = {["bezos"]=true,},
  ["arev"] = {["arev"]=true,},
  ["thinsp"] = {["thinsp"]=true,},
  ["frontespizio"] = {["frontespizio"]=true,},
  ["lexikon"] = {["lexikon"]=true,},
  ["chemarrow"] = {["chemarrow"]=true,},
  ["fancylabel"] = {["fancylabel"]=true,},
  ["pgfcircquadpoles"] = {["circuitikz"]=true,},
  ["colorspace"] = {["colorspace"]=true,},
  ["pgfcorepatterns.code"] = {["pgf"]=true,},
  ["lh-lcyxccr"] = {["lh"]=true,},
  ["hardwrap"] = {["hardwrap"]=true,},
  ["excludeonly"] = {["excludeonly"]=true,},
  ["lwarp-xtab"] = {["lwarp"]=true,},
  ["exsheets"] = {["exsheets"]=true,},
  ["kluedit"] = {["kluwer"]=true,},
  ["lwarp-footmisc"] = {["lwarp"]=true,},
  ["duerer"] = {["duerer-latex"]=true,},
  ["hyph-sv"] = {},
  ["asaetr"] = {["asaetr"]=true,},
  ["odev"] = {},
  ["bpfats5"] = {["iso10303"]=true,},
  ["interactiveworkbook"] = {["interactiveworkbook"]=true,},
  ["hyph-nb.ec"] = {},
  ["smartunits"] = {["smartunits"]=true,},
  ["capt-of"] = {["capt-of"]=true,},
  ["librarian"] = {["librarian"]=true,},
  ["pndraw"] = {["petri-nets"]=true,},
  ["luatodonotes"] = {["luatodonotes"]=true,},
  ["alterqcm"] = {["alterqcm"]=true,},
  ["hyph-ru"] = {},
  ["langsci-linguex"] = {["langsci"]=true,},
  ["scientific-thesis-cover"] = {["scientific-thesis-cover"]=true,},
  ["beamercolorthemefocus"] = {["beamertheme-focus"]=true,},
  ["gatherenum"] = {["gatherenum"]=true,},
  ["musixtex"] = {["musixtex"]=true,},
  ["listbib"] = {["listbib"]=true,},
  ["dantelogo"] = {["dantelogo"]=true,},
  ["babel-fulah"] = {["babel"]=true,},
  ["tikzlibrarygraphdrawing.code"] = {["pgf"]=true,},
  ["tagpdf-struct-code"] = {["tagpdf"]=true,},
  ["ctexart"] = {["ctex"]=true,},
  ["s-simpleslides-FramedTitle"] = {["context-simpleslides"]=true,},
  ["dtk-logos"] = {["dtk"]=true,},
  ["verbatimcopy"] = {["verbatimcopy"]=true,},
  ["lollipop-define"] = {["lollipop"]=true,},
  ["pgflibraryshapes.geometric.code"] = {["pgf"]=true,},
  ["carom"] = {["xymtex"]=true,},
  ["font_kurier-condensed-bold"] = {["font-change"]=true,},
  ["doclicense-CC-by-4.0-latex"] = {["doclicense"]=true,},
  ["beamerbaseauxtemplates"] = {["beamer"]=true,},
  ["gb4e"] = {["gb4e"]=true,},
  ["nccrules"] = {["ncctools"]=true,},
  ["apsstempl"] = {["iso10303"]=true,},
  ["pst-fill"] = {["pst-fill"]=true,},
  ["concmath"] = {["concmath"]=true,},
  ["luatexbase-modutils"] = {["luatexbase"]=true,},
  ["ruhyphal"] = {},
  ["reotex"] = {["reotex"]=true,},
  ["upref"] = {["amscls"]=true,},
  ["lwarp-trivfloat"] = {["lwarp"]=true,},
  ["etaremune"] = {["etaremune"]=true,},
  ["uothesis"] = {["uothesis"]=true,},
  ["xintcore"] = {["xint"]=true,},
  ["babel-slovenian"] = {["babel"]=true,},
  ["pbsi"] = {["brushscr"]=true,},
  ["verbdef"] = {["verbdef"]=true,},
  ["export"] = {["export"]=true,},
  ["daytime"] = {},
  ["xypdf-cu"] = {["xypic"]=true,},
  ["romanbar"] = {["romanbar"]=true,},
  ["xintcfrac"] = {["xint"]=true,},
  ["sindhi"] = {["arabtex"]=true,},
  ["basicarith"] = {["basicarith"]=true,},
  ["fenixpar"] = {["fenixpar"]=true,},
  ["arabaux"] = {["arabtex"]=true,},
  ["vmargin"] = {["vmargin"]=true,},
  ["beamerfontthemedefault"] = {["beamer"]=true,},
  ["font_bera_euler"] = {["font-change"]=true,},
  ["interfaces-base"] = {["interfaces"]=true,},
  ["eng_rs"] = {["r_und_s"]=true,},
  ["ieeepes"] = {["ieeepes"]=true,},
  ["wallcalendar-norwegian"] = {["wallcalendar"]=true,},
  ["babel-ewondo"] = {["babel"]=true,},
  ["pslatex"] = {},
  ["txfonts"] = {["txfonts"]=true,},
  ["loadhyph-sa"] = {["hyphen-sanskrit"]=true,},
  ["luatex-mplib"] = {["context"]=true,},
  ["pgfcircpath"] = {["circuitikz"]=true,},
  ["s-simpleslides-Embossed"] = {["context-simpleslides"]=true,},
  ["lxfonts"] = {["lxfonts"]=true,},
  ["lpic"] = {["lpic"]=true,},
  ["nfssext-cfr"] = {["nfssext-cfr"]=true,},
  ["diagxy"] = {["barr"]=true,},
  ["chronology"] = {["chronology"]=true,},
  ["babel-macedonian"] = {["babel"]=true,},
  ["t-mathsets"] = {["context-mathsets"]=true,},
  ["luavlna"] = {["luavlna"]=true,},
  ["interfaces"] = {["interfaces"]=true,},
  ["xydvips"] = {["xypic"]=true,},
  ["pantyk"] = {["ofs"]=true,},
  ["hyph-sv.ec"] = {},
  ["fontenc"] = {["latex"]=true,},
  ["venndiagram"] = {["venndiagram"]=true,},
  ["semcolor"] = {["seminar"]=true,},
  ["afoot"] = {["arabtex"]=true,["ledmac"]=true,},
  ["hrlatex"] = {["hrlatex"]=true,},
  ["bidicode"] = {["bidi"]=true,},
  ["random"] = {},
  ["tcbminted.code"] = {["tcolorbox"]=true,},
  ["dvdcoll"] = {["dvdcoll"]=true,},
  ["babel-brazilian"] = {["babel"]=true,},
  ["plaina4"] = {},
  ["omar"] = {["arabtex"]=true,},
  ["lwarp-mcaption"] = {["lwarp"]=true,},
  ["tikzpeople.shape.nurse"] = {["tikzpeople"]=true,},
  ["program"] = {["program"]=true,},
  ["bidishadowtext"] = {["bidishadowtext"]=true,},
  ["xetexko-font"] = {["xetexko"]=true,},
  ["dprogress"] = {["dprogress"]=true,},
  ["mwbk"] = {["mwcls"]=true,},
  ["float"] = {["float"]=true,},
  ["interfaces-truncate"] = {["interfaces"]=true,},
  ["babel-spanish-mx"] = {["babel"]=true,},
  ["beamerthemebars"] = {["beamer"]=true,},
  ["dcpic"] = {["dcpic"]=true,},
  ["babel-irish"] = {["babel"]=true,},
  ["babel-tasawaq"] = {["babel"]=true,},
  ["font_bera_fnc"] = {["font-change"]=true,},
  ["colonequals"] = {["oberdiek"]=true,},
  ["gmRCS"] = {["gmutils"]=true,},
  ["footmisx"] = {["footmisx"]=true,},
  ["afthes10"] = {["afthesis"]=true,},
  ["methylen"] = {["xymtex"]=true,},
  ["ArtNouvc"] = {["cfr-initials"]=true,},
  ["cmdtrack"] = {["cmdtrack"]=true,},
  ["s-simpleslides-PlainCounter"] = {["context-simpleslides"]=true,},
  ["datetime2"] = {["datetime2"]=true,},
  ["pgfshade"] = {["pgf"]=true,},
  ["afthesis"] = {["afthesis"]=true,},
  ["babel-german-austria"] = {["babel"]=true,},
  ["pkgloader-error"] = {["pkgloader"]=true,},
  ["cdpbabel"] = {["cdpbundl"]=true,},
  ["upca"] = {["upca"]=true,},
  ["filehook-memoir"] = {["filehook"]=true,},
  ["pgfmathparser.code"] = {["pgf"]=true,},
  ["cmsd"] = {["cmsd"]=true,},
  ["arevsymbols"] = {["arev"]=true,},
  ["apmptempl"] = {["iso10303"]=true,},
  ["loadhyph-en-gb"] = {},
  ["moderncvcolororange"] = {["moderncv"]=true,},
  ["ugarite"] = {["archaic"]=true,},
  ["xcookybooky"] = {["xcookybooky"]=true,},
  ["simurgh-counters"] = {["simurgh"]=true,},
  ["bodegraph"] = {["bodegraph"]=true,},
  ["xkeyval"] = {["xkeyval"]=true,},
  ["pst-eucl"] = {["pst-eucl"]=true,},
  ["databib"] = {["datatool"]=true,},
  ["serbianc"] = {["babel-serbianc"]=true,},
  ["sudoku"] = {["sudoku"]=true,},
  ["texlinks"] = {["morehype"]=true,},
  ["hyper"] = {["hyper"]=true,},
  ["eqparbox"] = {["eqparbox"]=true,},
  ["babel-sanskrit-telugu"] = {["babel"]=true,},
  ["flippdf"] = {["flippdf"]=true,},
  ["hyph-tk.ec"] = {},
  ["rsphrase"] = {["mhchem"]=true,},
  ["ketlayer"] = {["ketcindy"]=true,},
  ["chngcntr"] = {["chngcntr"]=true,},
  ["beamerouterthemefibeamer-mu"] = {["fibeamer"]=true,},
  ["figsize"] = {["figsize"]=true,},
  ["sizeredc"] = {["xymtex"]=true,},
  ["loadhyph-nn"] = {},
  ["nddiss2e"] = {["nddiss"]=true,},
  ["setspace"] = {["setspace"]=true,},
  ["bxnewfont"] = {["bxnewfont"]=true,},
  ["musixref"] = {["musixtex"]=true,},
  ["inconsolata"] = {["inconsolata"]=true,},
  ["apa"] = {["apa"]=true,},
  ["hyph-uk"] = {},
  ["l_pic"] = {["treetex"]=true,},
  ["datetime-defaults"] = {["datetime"]=true,},
  ["s-pre-27"] = {["context"]=true,},
  ["semlcmss"] = {["seminar"]=true,},
  ["fithesis"] = {["fithesis"]=true,},
  ["chemmacros4"] = {["chemmacros"]=true,},
  ["conv-utf8-ec"] = {["hyph-utf8"]=true,},
  ["hithesis"] = {["hithesis"]=true,},
  ["unicode-math-table"] = {["unicode-math"]=true,},
  ["babel-russian"] = {["babel"]=true,},
  ["lh-lcy"] = {["lh"]=true,},
  ["beamerthemeWarsaw"] = {["beamer"]=true,},
  ["rkeyval"] = {["amsrefs"]=true,},
  ["juraurtl"] = {["juramisc"]=true,},
  ["chappg"] = {["chappg"]=true,},
  ["jlreq-helpers"] = {["jlreq"]=true,},
  ["underoverlap"] = {["underoverlap"]=true,},
  ["extdash"] = {["ncctools"]=true,},
  ["beamerbaseverbatim"] = {["beamer"]=true,},
  ["verbatimbox"] = {["verbatimbox"]=true,},
  ["bghyph"] = {["omega"]=true,},
  ["cs-charter"] = {},
  ["aecompl"] = {["ae"]=true,},
  ["doclicense-CC-by-sa-4.0-latex"] = {["doclicense"]=true,},
  ["luatex-preprocessor"] = {["context"]=true,},
  ["beamerouterthemefocus"] = {["beamertheme-focus"]=true,},
  ["leadsheets.library.songs.code"] = {["leadsheets"]=true,},
  ["zxjatype"] = {["zxjatype"]=true,},
  ["zxjafont"] = {["zxjafont"]=true,},
  ["wallcalendar-czech"] = {["wallcalendar"]=true,},
  ["ff-mac"] = {},
  ["zlmtt"] = {["zlmtt"]=true,},
  ["ziffer"] = {["ziffer"]=true,},
  ["PPRframes"] = {["prosper"]=true,},
  ["scrhack"] = {},
  ["zhulem"] = {["zhspacing"]=true,},
  ["zhfont"] = {["zhspacing"]=true,},
  ["babel-bosnian"] = {["babel"]=true,},
  ["srctex"] = {["srcltx"]=true,},
  ["pm-isomath"] = {["pm-isomath"]=true,},
  ["filehook-fink"] = {["filehook"]=true,},
  ["zhspacing"] = {["zhspacing"]=true,},
  ["commado"] = {["commado"]=true,},
  ["quotmark"] = {["quotmark"]=true,},
  ["bookcover"] = {["bookcover"]=true,},
  ["pst-poly"] = {["pst-poly"]=true,},
  ["knitting"] = {["knitting"]=true,},
  ["asymptote"] = {["asymptote"]=true,},
  ["zhmath"] = {["zhspacing"]=true,},
  ["beamercolorthemecrane"] = {["beamer"]=true,},
  ["jmlr"] = {["jmlr"]=true,},
  ["t-zhspacing"] = {["zhspacing"]=true,},
  ["zhnumber"] = {["zhnumber"]=true,},
  ["zhwinfonts"] = {["zhmetrics"]=true,},
  ["screenplay-pkg"] = {["screenplay-pkg"]=true,},
  ["zhlipsum"] = {["zhlipsum"]=true,},
  ["zed-csp"] = {["zed-csp"]=true,},
  ["mfirstuc"] = {["mfirstuc"]=true,},
  ["vrbexin"] = {},
  ["zebra-goodies"] = {["zebra-goodies"]=true,},
  ["PPRnuancegris"] = {["prosper"]=true,},
  ["yplan"] = {},
  ["cs-adventor"] = {},
  ["txi-nb"] = {},
  ["eskdtext"] = {["eskdx"]=true,},
  ["york-thesis"] = {["york-thesis"]=true,},
  ["yhmath"] = {["yhmath"]=true,},
  ["yfonts"] = {["yfonts"]=true,},
  ["tabularew"] = {["tabularew"]=true,},
  ["ydoc-expl"] = {["ydoc"]=true,},
  ["ofs-8c"] = {["ofs"]=true,},
  ["ydoc-doc"] = {["ydoc"]=true,},
  ["ydoc-desc"] = {["ydoc"]=true,},
  ["pst-exa"] = {["pst-exa"]=true,},
  ["ydocstrip"] = {["ydoc"]=true,},
  ["hyph-sh-cyrl"] = {},
  ["version"] = {["version"]=true,},
  ["typearea-experts"] = {},
  ["avr.numbers"] = {["avremu"]=true,},
  ["scrbook"] = {},
  ["yax"] = {["yax"]=true,},
  ["ghsystem_acid-8"] = {["ghsystem"]=true,},
  ["t-yax"] = {["yax"]=true,},
  ["arabsymb"] = {["arabtex"]=true,},
  ["lwarp-crop"] = {["lwarp"]=true,},
  ["tengwarscript"] = {["tengwarscript"]=true,},
  ["yathesis"] = {["yathesis"]=true,},
  ["yathesis-translations"] = {["yathesis"]=true,},
  ["morisawa"] = {["morisawa"]=true,},
  ["lyluatex"] = {["lyluatex"]=true,},
  ["hyph-or"] = {},
  ["yaletter"] = {["yaletter"]=true,},
  ["tikz-dependency"] = {["tikz-dependency"]=true,},
  ["babel-english-unitedkingdom"] = {["babel"]=true,},
  ["hyph-es.ec"] = {["hyphen-spanish"]=true,},
  ["loadhyph-bg"] = {["hyphen-bulgarian"]=true,},
  ["pfnote"] = {["yafoot"]=true,},
  ["CJKnumb"] = {["cjk"]=true,},
  ["tabularcalc"] = {["tabularcalc"]=true,},
  ["fnpos"] = {["yafoot"]=true,},
  ["rule-D"] = {["gs1"]=true,},
  ["metakeys"] = {["stex"]=true,},
  ["beamerouterthemeUR"] = {["urcls"]=true,},
  ["xytree"] = {["xytree"]=true,},
  ["humanist"] = {["bookhands"]=true,},
  ["loadhyph-rm"] = {},
  ["geophys"] = {["custom-bib"]=true,},
  ["grfext"] = {["oberdiek"]=true,},
  ["xytp-f"] = {["xypic"]=true,},
  ["gregoriotex"] = {["gregoriotex"]=true,},
  ["mtx-context-markdown"] = {["context"]=true,},
  ["umthesis"] = {["umthesis"]=true,},
  ["soulpos"] = {["bezos"]=true,},
  ["beamerbasetheorems"] = {["beamer"]=true,},
  ["pdfcprot"] = {["pdfcprot"]=true,},
  ["ltxtools-doc"] = {["ltxtools"]=true,},
  ["xytile"] = {["xypic"]=true,},
  ["docstrip"] = {["latex"]=true,},
  ["pgflibrarydecorations.markings.code"] = {["pgf"]=true,},
  ["t-pgfbim"] = {["pgf"]=true,},
  ["xytextures"] = {["xypic"]=true,},
  ["babel-amharic"] = {["babel"]=true,},
  ["px-ds"] = {["pxtxalfa"]=true,},
  ["xyrecat"] = {["xypic"]=true,},
  ["pxpgfmark"] = {["pxpgfmark"]=true,},
  ["lwarp-scrlayer-scrpage"] = {["lwarp"]=true,},
  ["tempora"] = {["tempora"]=true,},
  ["resumecls"] = {["resumecls"]=true,},
  ["xy16textures"] = {["xypic"]=true,},
  ["inlinebib"] = {["inlinebib"]=true,},
  ["chessboard-keys-main"] = {["chessboard"]=true,},
  ["tikzlibrarydecorations.footprints.code"] = {["pgf"]=true,},
  ["fontspec-xetex"] = {["fontspec"]=true,},
  ["xyps-t"] = {["xypic"]=true,},
  ["listofsymbols"] = {["listofsymbols"]=true,},
  ["bxdpx-beamer"] = {["bxdpx-beamer"]=true,},
  ["exceltex"] = {["exceltex"]=true,},
  ["eskdgraph"] = {["eskdx"]=true,},
  ["xyps-r"] = {["xypic"]=true,},
  ["ltr-steps"] = {["lecturer"]=true,},
  ["readprov"] = {["fileinfo"]=true,},
  ["xyps-ps"] = {["xypic"]=true,},
  ["bbm"] = {["bbm-macros"]=true,},
  ["xyps-pro"] = {["xypic"]=true,},
  ["xyps-l"] = {["xypic"]=true,},
  ["libris"] = {["libris"]=true,},
  ["komacv"] = {["komacv"]=true,},
  ["xyps-f"] = {["xypic"]=true,},
  ["luatexbase-attr"] = {["luatexbase"]=true,},
  ["demo-mps"] = {["context"]=true,},
  ["exscale"] = {["latex"]=true,},
  ["hyph-grc"] = {},
  ["xyps-col"] = {["xypic"]=true,},
  ["dcm"] = {["stex"]=true,},
  ["mandel"] = {},
  ["babel-turkish"] = {["babel"]=true,},
  ["lwarp-ccaption"] = {["lwarp"]=true,},
  ["uantwerpenmasterthesis"] = {["uantwerpendocs"]=true,},
  ["greek6cbc"] = {["archaic"]=true,},
  ["fge"] = {["fge"]=true,},
  ["xyps-c"] = {["xypic"]=true,},
  ["reporting"] = {["proposal"]=true,},
  ["xypicture"] = {["xypic"]=true,},
  ["lollipop-output"] = {["lollipop"]=true,},
  ["toptesi-magistrale"] = {["toptesi"]=true,},
  ["wallcalendar-portuguese"] = {["wallcalendar"]=true,},
  ["epic"] = {["eepic"]=true,},
  ["ushort"] = {["ushort"]=true,},
  ["sample"] = {["context"]=true,},
  ["xypdf-fr"] = {["xypic"]=true,},
  ["hyph-mn-cyrl.t2a"] = {},
  ["mcnish"] = {["context"]=true,},
  ["autopict"] = {["graphics-pln"]=true,},
  ["xypdf-co"] = {["xypic"]=true,},
  ["xyoztex"] = {["xypic"]=true,},
  ["xymovie"] = {["xypic"]=true,},
  ["xymatrix"] = {["xypic"]=true,},
  ["xyline"] = {["xypic"]=true,},
  ["phn-UK_Vorwahlen"] = {["phonenumbers"]=true,},
  ["xyimport"] = {["xypic"]=true,},
  ["ArtNouv"] = {["cfr-initials"]=true,},
  ["srhyph"] = {["omega"]=true,},
  ["pgfplotsoldpgfsupp_pgfutil-common-lists"] = {["pgfplots"]=true,},
  ["xyidioms"] = {["xypic"]=true,},
  ["lmodern"] = {["lm"]=true,},
  ["quoted"] = {["logical-markup-utils"]=true,},
  ["beamerthemeI6dv"] = {["beamerposter"]=true,},
  ["pssetiby"] = {["ibygrk"]=true,},
  ["xunicode-addon"] = {["xecjk"]=true,},
  ["logreq"] = {["logreq"]=true,},
  ["iflang"] = {["oberdiek"]=true,},
  ["begingreek"] = {["begingreek"]=true,},
  ["arabtex-doc"] = {["arabtex"]=true,},
  ["xyframe"] = {["xypic"]=true,},
  ["pst-pdf"] = {["pst-pdf"]=true,},
  ["babel-punjabi-gurmukhi"] = {["babel"]=true,},
  ["ltluatex"] = {["latex"]=true,},
  ["subsupscripts"] = {["subsupscripts"]=true,},
  ["lwarp-cutwin"] = {["lwarp"]=true,},
  ["xydvitops"] = {["xypic"]=true,},
  ["a0size"] = {["a0poster"]=true,},
  ["verbtest"] = {["tools"]=true,},
  ["hyph-quote-it"] = {},
  ["accsupp"] = {["oberdiek"]=true,},
  ["xydummy"] = {["xypic"]=true,},
  ["contracard"] = {["contracard"]=true,},
  ["xycrayon"] = {["xypic"]=true,},
  ["xycolor"] = {["xypic"]=true,},
  ["xycmtip"] = {["xypic"]=true,},
  ["arabi-add"] = {["arabi-add"]=true,},
  ["dummyels"] = {},
  ["xybarr"] = {["xypic"]=true,},
  ["xyarrow"] = {["xypic"]=true,},
  ["pdfnotiz"] = {["dvdcoll"]=true,},
  ["erae"] = {["economic"]=true,},
  ["subdocs"] = {["bezos"]=true,},
  ["xyarc"] = {["xypic"]=true,},
  ["xyall"] = {["xypic"]=true,},
  ["xy2cell"] = {["xypic"]=true,},
  ["xy17oztex"] = {["xypic"]=true,},
  ["xyps"] = {["xypic"]=true,},
  ["babel-tigrinya"] = {["babel"]=true,},
  ["xy"] = {["xypic"]=true,},
  ["xymtx-ps"] = {["xymtex"]=true,},
  ["greektex"] = {["greektex"]=true,},
  ["perfectcut"] = {["perfectcut"]=true,},
  ["chmst-ps"] = {["xymtex"]=true,},
  ["collect"] = {["sauerj"]=true,},
  ["pst-doc"] = {["pstricks"]=true,},
  ["xymtx-pdf"] = {["xymtex"]=true,},
  ["adfathesis"] = {["adfathesis"]=true,},
  ["tgtermes"] = {["tex-gyre"]=true,},
  ["pgfcirc.defines"] = {["circuitikz"]=true,},
  ["cmap-cyr-vf"] = {["lcyw"]=true,},
  ["mfparptc"] = {["ednotes"]=true,},
  ["chemtimes"] = {["xymtex"]=true,},
  ["lorem"] = {["context"]=true,},
  ["lltjp-tascmac"] = {["luatexja"]=true,},
  ["lthyph"] = {["omega"]=true,},
  ["pb-lams"] = {["pb-diagram"]=true,},
  ["oblivoir-base"] = {["kotex-oblivoir"]=true,},
  ["docidx2e"] = {["xdoc"]=true,},
  ["ams-mdbch"] = {["arev"]=true,},
  ["xymtexps"] = {["xymtex"]=true,},
  ["phn-DE_Vorwahlen"] = {["phonenumbers"]=true,},
  ["xymtexpdf"] = {["xymtex"]=true,},
  ["francais"] = {["babel"]=true,},
  ["polymers"] = {["xymtex"]=true,},
  ["pgflibrarysnakes"] = {["pgf"]=true,},
  ["abntex2abrev"] = {["abntex2"]=true,},
  ["lowcycle"] = {["xymtex"]=true,},
  ["caption"] = {["caption"]=true,},
  ["ofs-px"] = {["ofs"]=true,},
  ["xetexko-josa"] = {["xetexko"]=true,},
  ["infram"] = {["gustlib"]=true,},
  ["clrdblpg"] = {["clrdblpg"]=true,},
  ["fancypar"] = {["fancypar"]=true,},
  ["babel-chinese-hans"] = {["babel"]=true,},
  ["qzapfcha"] = {["tex-gyre"]=true,},
  ["agecon"] = {["economic"]=true,},
  ["gillius2"] = {["gillius"]=true,},
  ["vpe"] = {["vpe"]=true,},
  ["locant"] = {["xymtex"]=true,},
  ["lewisstruc"] = {["xymtex"]=true,},
  ["s-pre-07"] = {["context"]=true,},
  ["forest-lib-linguistics"] = {["forest"]=true,},
  ["cs-termes"] = {},
  ["fontch"] = {["fontch"]=true,},
  ["dblaccnt"] = {["vntex"]=true,},
  ["udesoftec-biblatex"] = {["udesoftec"]=true,},
  ["catchfilebetweentags"] = {["catchfilebetweentags"]=true,},
  ["pagesel"] = {["oberdiek"]=true,},
  ["font_mdutopia"] = {["font-change"]=true,},
  ["dvngcite"] = {["velthuis"]=true,},
  ["glossary-inline"] = {["glossaries"]=true,},
  ["hobete"] = {["hobete"]=true,},
  ["localloc"] = {["localloc"]=true,},
  ["removefr"] = {["fragments"]=true,},
  ["createdtx"] = {["makedtx"]=true,},
  ["emarks"] = {["emarks"]=true,},
  ["lwarp-sectsty"] = {["lwarp"]=true,},
  ["babel-quechua"] = {["babel"]=true,},
  ["chemstr"] = {["xymtex"]=true,},
  ["doclicense-CC-by-sa-3.0-plaintext"] = {["doclicense"]=true,},
  ["jsmembertable"] = {["jslectureplanner"]=true,},
  ["keystroke"] = {["keystroke"]=true,},
  ["pgflibraryhan.code"] = {["pgfornament-han"]=true,},
  ["pgflibrarytikztrees"] = {["pgf"]=true,},
  ["bondcolor"] = {["xymtex"]=true,},
  ["assurelatexmode"] = {["xymtex"]=true,},
  ["NumericPlots_styles"] = {["numericplots"]=true,},
  ["aliphat"] = {["xymtex"]=true,},
  ["utbook"] = {["uplatex"]=true,},
  ["qrcode"] = {["olsak-misc"]=true,["qrcode"]=true,},
  ["mycomplexity"] = {["complexity"]=true,},
  ["xwatermark"] = {["xwatermark"]=true,},
  ["diffcoeffx"] = {["diffcoeff"]=true,},
  ["pgfmoduledecorations.code"] = {["pgf"]=true,},
  ["xunicode"] = {["xunicode"]=true,},
  ["xtuformat"] = {["xtuthesis"]=true,},
  ["xtab"] = {["xtab"]=true,},
  ["xstring"] = {["xstring"]=true,},
  ["xskak"] = {["xskak"]=true,},
  ["xskak-nagdef"] = {["xskak"]=true,},
  ["simurgh-report"] = {["simurgh"]=true,},
  ["dspfunctions"] = {["dsptricks"]=true,},
  ["selnolig-english-hyphex"] = {["selnolig"]=true,},
  ["pkgloader-late"] = {["pkgloader"]=true,},
  ["xsimverb"] = {["xsim"]=true,},
  ["ae"] = {["ae"]=true,},
  ["xsim.verbwrite.code"] = {["xsim"]=true,},
  ["xsim.translations.code"] = {["xsim"]=true,},
  ["asmo449a"] = {["arabtex"]=true,},
  ["babel-vietnamese"] = {["babel"]=true,},
  ["pawpict"] = {["pawpict"]=true,},
  ["skrapport"] = {["skrapport"]=true,},
  ["xsim.templates.code"] = {["xsim"]=true,},
  ["babel-nepali"] = {["babel"]=true,},
  ["grayhints"] = {["grayhints"]=true,},
  ["opensans"] = {["opensans"]=true,},
  ["simurgh-tools"] = {["simurgh"]=true,},
  ["xsim.tags.code"] = {["xsim"]=true,},
  ["xsim.style.layouts.code"] = {["xsim"]=true,},
  ["tudscrcolor"] = {["tudscr"]=true,},
  ["ghsystem_skull-2"] = {["ghsystem"]=true,},
  ["eucal"] = {["amsfonts"]=true,},
  ["xsim"] = {["xsim"]=true,},
  ["t-pgfmat"] = {["pgf"]=true,},
  ["gensymb"] = {["was"]=true,},
  ["robustindex"] = {["robustindex"]=true,},
  ["etoc"] = {["etoc"]=true,},
  ["xsim.random.code"] = {["xsim"]=true,},
  ["ScratchX"] = {["scratchx"]=true,},
  ["cuted"] = {["sttools"]=true,},
  ["enumitem"] = {["enumitem"]=true,},
  ["luatex-test"] = {["context"]=true,},
  ["ofs-tx"] = {["ofs"]=true,},
  ["xsim.modules.code"] = {["xsim"]=true,},
  ["impnattypo"] = {["impnattypo"]=true,},
  ["bpfir2"] = {["iso10303"]=true,},
  ["xsim.interface.code"] = {["xsim"]=true,},
  ["bpfap6"] = {["iso10303"]=true,},
  ["fp-basic"] = {["fp"]=true,},
  ["flowchart"] = {["flowchart"]=true,},
  ["varvbtm"] = {["newvbtm"]=true,},
  ["xsim.goals.code"] = {["xsim"]=true,},
  ["phaistos"] = {["phaistos"]=true,},
  ["xsim.environments.code"] = {["xsim"]=true,},
  ["xsim.definitions.code"] = {["xsim"]=true,},
  ["beamercolorthemefibeamer-mu"] = {["fibeamer"]=true,},
  ["babel-cantonese"] = {["babel"]=true,},
  ["xsim.collections.code"] = {["xsim"]=true,},
  ["tablists"] = {["tablists"]=true,},
  ["textfit"] = {["textfit"]=true,},
  ["xsavebox"] = {["xsavebox"]=true,},
  ["AlProTex"] = {["protex"]=true,},
  ["xpunctuate"] = {["xpunctuate"]=true,},
  ["xprintlen"] = {["xprintlen"]=true,},
  ["skak"] = {["skak"]=true,},
  ["pgfmathode.code"] = {["pgf"]=true,},
  ["babel-english"] = {["babel"]=true,},
  ["TXSrefs"] = {["texsis"]=true,},
  ["xpicture"] = {["xpicture"]=true,},
  ["patchcmd"] = {["patchcmd"]=true,},
  ["anti"] = {},
  ["font-change-xetex"] = {["font-change-xetex"]=true,},
  ["classpack"] = {["classpack"]=true,},
  ["conv-utf8-l7x"] = {["hyph-utf8"]=true,},
  ["xpiano"] = {["xpiano"]=true,},
  ["conv-utf8-t2a"] = {["hyph-utf8"]=true,},
  ["lwarp-endfloat"] = {["lwarp"]=true,},
  ["tikzlibraryautomata.code"] = {["pgf"]=true,},
  ["mhchem"] = {["mhchem"]=true,},
  ["meashor"] = {["gustlib"]=true,},
  ["xpeek"] = {["xpeek"]=true,},
  ["xpatch"] = {["xpatch"]=true,},
  ["xoptarg"] = {["xoptarg"]=true,},
  ["zref-pagelayout"] = {["oberdiek"]=true,},
  ["newtxmath"] = {["newtx"]=true,},
  ["autofootnoterule"] = {["luabidi"]=true,},
  ["luatex-swiglib"] = {["context"]=true,},
  ["babel-rundi"] = {["babel"]=true,},
  ["pst-fp"] = {["pstricks"]=true,},
  ["asyalign"] = {["asyfig"]=true,},
  ["stepv13"] = {["iso10303"]=true,},
  ["kdgcoursetext"] = {["kdgdocs"]=true,},
  ["s-pre-66"] = {["context"]=true,},
  ["picmore"] = {["pictex"]=true,},
  ["xmpincl"] = {["xmpincl"]=true,},
  ["xmltex"] = {["xmltex"]=true,},
  ["xltxtra"] = {["xltxtra"]=true,},
  ["pgflibrarygraphdrawing.circular.code"] = {["pgf"]=true,},
  ["xltabular"] = {["xltabular"]=true,},
  ["pgfmodulesorting.code"] = {["pgf"]=true,},
  ["hebchrs"] = {["arabtex"]=true,},
  ["xlop"] = {["xlop"]=true,},
  ["simplewick"] = {["simplewick"]=true,},
  ["loadhyph-hu"] = {["hyphen-hungarian"]=true,},
  ["tikzlibrarymatrix.code"] = {["pgf"]=true,},
  ["xkvview"] = {["xkeyval"]=true,},
  ["makebst"] = {["custom-bib"]=true,},
  ["HAPcapsules"] = {["ha-prosper"]=true,},
  ["kotexutf"] = {["kotex-plain"]=true,["kotex-utf"]=true,},
  ["schwalbe"] = {["schwalbe-chess"]=true,},
  ["s-pre-96"] = {["context"]=true,},
  ["armtex"] = {["armtex"]=true,},
  ["apatch"] = {["arabtex"]=true,},
  ["moderncvcolorgrey"] = {["moderncv"]=true,},
  ["pdfx"] = {["pdfx"]=true,},
  ["t-pgfbsn"] = {["pgf"]=true,},
  ["babel-thai"] = {["babel"]=true,},
  ["xkvesb"] = {["xkeyval"]=true,},
  ["beamerswitch"] = {["beamerswitch"]=true,},
  ["lazylist"] = {["lazylist"]=true,},
  ["verbfwr"] = {["examplep"]=true,},
  ["pdfrender"] = {["oberdiek"]=true,},
  ["xkvesa"] = {["xkeyval"]=true,},
  ["xkvecb"] = {["xkeyval"]=true,},
  ["eskdplain"] = {["eskdx"]=true,},
  ["apldef"] = {},
  ["minibox"] = {["minibox"]=true,},
  ["tikzlibrarypgfplots.fillbetween.code"] = {["pgfplots"]=true,},
  ["babel-french-ch"] = {["babel"]=true,},
  ["thesis-ekf"] = {["thesis-ekf"]=true,},
  ["turabian-researchpaper"] = {["turabian-formatting"]=true,},
  ["moderncvheadvi"] = {["moderncv"]=true,},
  ["pstool"] = {["pstool"]=true,},
  ["poe"] = {["context"]=true,},
  ["xkvutils"] = {["xkeyval"]=true,},
  ["fitbox"] = {["fitbox"]=true,},
  ["caesar_book"] = {["sidenotes"]=true,},
  ["beamerfontthemefibeamer-mu"] = {["fibeamer"]=true,},
  ["xkvtxhdr"] = {["xkeyval"]=true,},
  ["font_kurier-condensed"] = {["font-change"]=true,},
  ["chessfss"] = {["chessfss"]=true,},
  ["hebcal"] = {["babel-hebrew"]=true,},
  ["afrikaans"] = {["babel"]=true,},
  ["xkvex4"] = {["xkeyval"]=true,},
  ["lwarp-fancyref"] = {["lwarp"]=true,},
  ["cantarell"] = {["cantarell"]=true,},
  ["tikz-timing-either"] = {["tikz-timing"]=true,},
  ["idcc"] = {["dccpaper"]=true,},
  ["xkvex2"] = {["xkeyval"]=true,},
  ["duotenzor"] = {["duotenzor"]=true,},
  ["scratch"] = {["scratch"]=true,},
  ["babel-soga"] = {["babel"]=true,},
  ["esami"] = {["esami"]=true,},
  ["pst-xkey"] = {["xkeyval"]=true,},
  ["hyph-de-1901"] = {},
  ["dtk-extern"] = {["dtk"]=true,},
  ["xintseries"] = {["xint"]=true,},
  ["xintkernel"] = {["xint"]=true,},
  ["xintgcd"] = {["xint"]=true,},
  ["tufte-handout"] = {["tufte-latex"]=true,},
  ["kdgmasterthesis"] = {["kdgdocs"]=true,},
  ["xintexpr"] = {["xint"]=true,},
  ["hyph-pt"] = {},
  ["postcards"] = {["postcards"]=true,},
  ["onedown"] = {["onedown"]=true,},
  ["Acorn"] = {["cfr-initials"]=true,},
  ["pgfkeyx"] = {["pgfkeyx"]=true,},
  ["autoaligne-fr"] = {["autoaligne"]=true,},
  ["inputtrc"] = {["inputtrc"]=true,},
  ["zref-counter"] = {["oberdiek"]=true,},
  ["zref-user"] = {["oberdiek"]=true,},
  ["xifthen"] = {["xifthen"]=true,},
  ["lwarp-enumitem"] = {["lwarp"]=true,},
  ["nameref"] = {["hyperref"]=true,},
  ["ketslide"] = {["ketcindy"]=true,},
  ["ghsystem_bottle"] = {["ghsystem"]=true,},
  ["cd"] = {["cd"]=true,},
  ["babel-german-switzerland"] = {["babel"]=true,},
  ["ed"] = {["ed"]=true,},
  ["xhfill"] = {["xhfill"]=true,},
  ["chemmacros.module.nomenclature.code"] = {["chemmacros"]=true,},
  ["adjustbox"] = {["adjustbox"]=true,},
  ["ruhyphvl"] = {},
  ["pdfcolparallel"] = {["oberdiek"]=true,},
  ["apv12"] = {["iso10303"]=true,},
  ["recipecard"] = {["recipecard"]=true,},
  ["xetexko"] = {["xetexko"]=true,},
  ["xetexko-vertical"] = {["xetexko"]=true,},
  ["babel-french-luxembourg"] = {["babel"]=true,},
  ["xetexko-hanging"] = {["xetexko"]=true,},
  ["pst-optic"] = {["pst-optic"]=true,},
  ["tdclock"] = {["tdclock"]=true,},
  ["opentype-info"] = {["xetexfontinfo"]=true,},
  ["nimbusserif"] = {["nimbus15"]=true,},
  ["supp-mpe"] = {["context"]=true,},
  ["hyph-en-gb"] = {},
  ["ifmtarg"] = {["ifmtarg"]=true,},
  ["xespotcolor"] = {["xespotcolor"]=true,},
  ["example-image-c"] = {["mwe"]=true,},
  ["hyph-kn"] = {},
  ["bpfap5"] = {["iso10303"]=true,},
  ["mdput"] = {["mathdesign"]=true,},
  ["xesearch"] = {["xesearch"]=true,},
  ["cc-cls"] = {["computational-complexity"]=true,},
  ["noindentafter"] = {["noindentafter"]=true,},
  ["algpascal"] = {["algorithmicx"]=true,},
  ["t-xesearch"] = {["xesearch"]=true,},
  ["pgflibraryplotmarks.code"] = {["pgf"]=true,},
  ["AlDraTex"] = {["dratex"]=true,},
  ["vietnam"] = {["vntex"]=true,},
  ["xepersian-persiancal"] = {["xepersian"]=true,},
  ["mtx-context-copy"] = {["context"]=true,},
  ["musixbbm"] = {["musixtex"]=true,},
  ["babel-sanskrit-kannada"] = {["babel"]=true,},
  ["dejavu-otf"] = {["dejavu-otf"]=true,},
  ["csenc-p"] = {},
  ["xepersian-multiplechoice"] = {["xepersian"]=true,},
  ["chngpage"] = {["changepage"]=true,},
  ["toptesi-dottorale"] = {["toptesi"]=true,},
  ["xepersian-magazine"] = {["xepersian"]=true,},
  ["sansmathaccent"] = {["sansmathaccent"]=true,},
  ["xeindex"] = {["xeindex"]=true,},
  ["numberedblock"] = {["numberedblock"]=true,},
  ["pst-3d"] = {["pst-3d"]=true,},
  ["chemmacros.module.reactions.code"] = {["chemmacros"]=true,},
  ["komacv-multilang"] = {["komacv-rg"]=true,},
  ["xecolor"] = {["xecolor"]=true,},
  ["xeCJKfntef"] = {["xecjk"]=true,},
  ["amsldoc"] = {["amscls"]=true,},
  ["lwarp-pdfpages"] = {["lwarp"]=true,},
  ["biblist"] = {["biblist"]=true,},
  ["lwarp-chapterbib"] = {["lwarp"]=true,},
  ["xechangebar"] = {["xechangebar"]=true,},
  ["gcard"] = {["gcard"]=true,},
  ["hgbbib"] = {["hagenberg-thesis"]=true,},
  ["xyps-s"] = {["xypic"]=true,},
  ["trimclip"] = {["adjustbox"]=true,},
  ["gtrcore.contour.code"] = {["genealogytree"]=true,},
  ["loadhyph-tk"] = {},
  ["xduthesis"] = {["xduthesis"]=true,},
  ["xdoc2"] = {["xdoc"]=true,},
  ["babel-german"] = {["babel"]=true,},
  ["babel-uzbek-cyrillic"] = {["babel"]=true,},
  ["xcomment"] = {["xcomment"]=true,},
  ["xcolor-solarized"] = {["xcolor-solarized"]=true,},
  ["xcolor-material"] = {["xcolor-material"]=true,},
  ["lwarp-luacolor"] = {["lwarp"]=true,},
  ["xcolor"] = {["xcolor"]=true,},
  ["xcntperchap"] = {["xcntperchap"]=true,},
  ["example-image"] = {["mwe"]=true,},
  ["graph35-pixelart"] = {["graph35"]=true,},
  ["xCJK2uni"] = {["xcjk2uni"]=true,},
  ["xcite"] = {["xcite"]=true,},
  ["XCharter"] = {["xcharter"]=true,},
  ["beamerthemeEastLansing"] = {["beamer"]=true,},
  ["ProTex"] = {["protex"]=true,},
  ["cyrecfnt"] = {},
  ["xassoccnt"] = {["xassoccnt"]=true,},
  ["pst-jtree"] = {["pst-jtree"]=true,},
  ["wallpaper"] = {["wallpaper"]=true,},
  ["akksection"] = {["akktex"]=true,},
  ["wtref"] = {["wtref"]=true,},
  ["ipalmacs"] = {["wsuipa"]=true,},
  ["ipa"] = {["wsuipa"]=true,},
  ["lstautogobble"] = {["lstaddons"]=true,},
  ["wordlike"] = {["wordlike"]=true,},
  ["exp-testopt"] = {["exp-testopt"]=true,},
  ["lollipop-float"] = {["lollipop"]=true,},
  ["wordcount"] = {["wordcount"]=true,},
  ["upmethodology-spec"] = {["upmethodology"]=true,},
  ["wnri"] = {["wnri-latex"]=true,},
  ["chessboard"] = {["chessboard"]=true,},
  ["beamerbasemodes"] = {["beamer"]=true,},
  ["calculator"] = {["calculator"]=true,},
  ["pgflibrarydecorations.footprints.code"] = {["pgf"]=true,},
  ["witharrows"] = {["witharrows"]=true,},
  ["showlabels"] = {["showlabels"]=true,},
  ["tikzlibrarypgfplots.groupplots.code"] = {["pgfplots"]=true,},
  ["luatex-core"] = {["context"]=true,},
  ["withargs"] = {["withargs"]=true,},
  ["needspace"] = {["needspace"]=true,},
  ["lwarp-booktabs"] = {["lwarp"]=true,},
  ["nopageno"] = {["nopageno"]=true,},
  ["copyfile"] = {["nicetext"]=true,},
  ["withargs-packagedoc"] = {["withargs"]=true,},
  ["pdfslide"] = {["pdfslide"]=true,},
  ["ycbook"] = {["ycbook"]=true,},
  ["french-msg"] = {["e-french"]=true,},
  ["ccreltx"] = {["computational-complexity"]=true,},
  ["toklist"] = {["williams"]=true,},
  ["antree"] = {["williams"]=true,},
  ["widetable"] = {["widetable"]=true,},
  ["luatex-basics-prepare"] = {["context"]=true,},
  ["pgfplotsmatrix.code"] = {["pgfplots"]=true,},
  ["s-simpleslides-Framed"] = {["context-simpleslides"]=true,},
  ["schulschriften_lin"] = {["schulschriften"]=true,},
  ["HAPFyma"] = {["ha-prosper"]=true,},
  ["cv4tw-scheme"] = {["cv4tw"]=true,},
  ["langsci-glyphs"] = {["langsci"]=true,},
  ["theoremref"] = {["theoremref"]=true,},
  ["wasyfont"] = {["wasy"]=true,},
  ["easytable"] = {["easy"]=true,},
  ["hyph-en-us"] = {},
  ["icomma"] = {["was"]=true,},
  ["lipsum"] = {["lipsum"]=true,},
  ["musicography"] = {["musicography"]=true,},
  ["xsim.solutions.code"] = {["xsim"]=true,},
  ["newlfm"] = {["newlfm"]=true,},
  ["fixmath"] = {["was"]=true,},
  ["tikzlibraryer.code"] = {["pgf"]=true,},
  ["warpcol"] = {["warpcol"]=true,},
  ["warning"] = {["warning"]=true,},
  ["interfaces-appendix"] = {["interfaces"]=true,},
  ["xargs"] = {["xargs"]=true,},
  ["mtx-context-module"] = {["context"]=true,},
  ["irv12"] = {["iso10303"]=true,},
  ["gfsbodoni"] = {["gfsbodoni"]=true,},
  ["fancyheadings"] = {["fancyhdr"]=true,},
  ["wallcalendar"] = {["wallcalendar"]=true,},
  ["bracketkey"] = {["bracketkey"]=true,},
  ["beamerthemeDetlevCM"] = {["beamertheme-detlevcm"]=true,},
  ["mathlig"] = {},
  ["wallcalendar-thai"] = {["wallcalendar"]=true,},
  ["eskdfootnote"] = {["eskdx"]=true,},
  ["moderncviconsletters"] = {["moderncv"]=true,},
  ["pas-tableur"] = {["pas-tableur"]=true,},
  ["font_iwona-condensed-light"] = {["font-change"]=true,},
  ["eepic"] = {["eepic"]=true,},
  ["tikz.code"] = {["pgf"]=true,},
  ["xr-hyper"] = {["hyperref"]=true,},
  ["wallcalendar-swedish"] = {["wallcalendar"]=true,},
  ["scrlayer-scrpage-experts"] = {},
  ["VCPref-default"] = {["vaucanson-g"]=true,},
  ["langsci-optional"] = {["langsci"]=true,},
  ["wallcalendar-slovenian"] = {["wallcalendar"]=true,},
  ["mtcpatchmem"] = {["minitoc"]=true,},
  ["biblatex-opcit-booktitle"] = {["biblatex-opcit-booktitle"]=true,},
  ["wallcalendar-slovakian"] = {["wallcalendar"]=true,},
  ["bidi2in1"] = {["bidi"]=true,},
  ["intopdf"] = {["intopdf"]=true,},
  ["a5toa4"] = {["pfarrei"]=true,},
  ["starfont"] = {["starfont"]=true,},
  ["poster"] = {["poster-mac"]=true,},
  ["xypdf"] = {["xypic"]=true,},
  ["mathstyle"] = {["breqn"]=true,},
  ["dhucs-interword"] = {["kotex-utf"]=true,},
  ["lato"] = {["lato"]=true,},
  ["wallcalendar-japanese"] = {["wallcalendar"]=true,},
  ["beamerthemeAnnArbor"] = {["beamer"]=true,},
  ["scottish"] = {["babel"]=true,},
  ["s-simpleslides-RainbowStripe"] = {["context-simpleslides"]=true,},
  ["eureporting"] = {["proposal"]=true,},
  ["allhuncl"] = {["bookhands"]=true,},
  ["langsci-gb4e"] = {["langsci"]=true,},
  ["hpsdiss"] = {["hpsdiss"]=true,},
  ["wallcalendar-italian"] = {["wallcalendar"]=true,},
  ["tikzlibraryshapes.callouts.code"] = {["pgf"]=true,},
  ["lwarp-indentfirst"] = {["lwarp"]=true,},
  ["iagproc"] = {},
  ["wallcalendar-german"] = {["wallcalendar"]=true,},
  ["wallcalendar-english"] = {["wallcalendar"]=true,},
  ["hvfloat-fps"] = {["hvfloat"]=true,},
  ["wallcalendar-dutch"] = {["wallcalendar"]=true,},
  ["minidocument"] = {["minidocument"]=true,},
  ["zwgetfdate"] = {["zwgetfdate"]=true,},
  ["beamerfontthemeprofessionalfonts"] = {["beamer"]=true,},
  ["pgfplotsoldpgfsupp_pgfmanual.pdflinks.code"] = {["pgfplots"]=true,},
  ["forarray"] = {["forarray"]=true,},
  ["vntexinfo"] = {["vntex"]=true,},
  ["lwarp-nonfloat"] = {["lwarp"]=true,},
  ["t5code"] = {["vntex"]=true,},
  ["lstlang3"] = {["listings"]=true,},
  ["redeffont"] = {["japanese-otf"]=true,},
  ["mynormal"] = {["germbib"]=true,},
  ["fp-random"] = {["fp"]=true,},
  ["calculus"] = {["calculator"]=true,},
  ["pgflibraryshapes.symbols.code"] = {["pgf"]=true,},
  ["tikzlibrarypetri.code"] = {["pgf"]=true,},
  ["pdfdcwebmac"] = {["cweb"]=true,},
  ["simurgh-shellescape"] = {["simurgh"]=true,},
  ["nccpic"] = {["ncctools"]=true,},
  ["BOONDOX-cal"] = {["boondox"]=true,},
  ["avr.testsuite"] = {["avremu"]=true,},
  ["skt"] = {["sanskrit"]=true,},
  ["catchdq"] = {["catcodes"]=true,},
  ["schuleab"] = {["schule"]=true,},
  ["eskdtab"] = {["eskdx"]=true,},
  ["example-glossaries-acronyms-lang"] = {["glossaries"]=true,},
  ["omega"] = {["omega"]=true,},
  ["psu-thesis"] = {["psu-thesis"]=true,},
  ["aicv1"] = {["iso10303"]=true,},
  ["sets"] = {["vhistory"]=true,},
  ["pgflibraryhobby.code"] = {["hobby"]=true,},
  ["texilikecover"] = {},
  ["luaotfload"] = {["luaotfload"]=true,},
  ["Staedte3dJG"] = {["pst-geo"]=true,},
  ["fontspec"] = {["fontspec"]=true,},
  ["ofs"] = {["ofs"]=true,},
  ["drm"] = {["drm"]=true,},
  ["pst-eps"] = {["pst-eps"]=true,},
  ["lhcyralt-rhyphen"] = {},
  ["svnkw"] = {["svn-multi"]=true,},
  ["babel"] = {["babel"]=true,},
  ["moresize"] = {["moresize"]=true,},
  ["titlecaps"] = {["titlecaps"]=true,},
  ["arm"] = {["armtex"]=true,},
  ["ydocincl"] = {["ydoc"]=true,},
  ["loadhyph-pa"] = {},
  ["VCPref-beamer"] = {["vaucanson-g"]=true,},
  ["extproc"] = {["extsizes"]=true,},
  ["fifo-stack"] = {["fifo-stack"]=true,},
  ["cypriot"] = {["archaic"]=true,},
  ["verbments"] = {["verbments"]=true,},
  ["index"] = {["index"]=true,},
  ["venturisold"] = {["venturisadf"]=true,},
  ["venturis2"] = {["venturisadf"]=true,},
  ["cu-util"] = {["churchslavonic"]=true,},
  ["lwarp-chemmacros"] = {["lwarp"]=true,},
  ["venturis"] = {["venturisadf"]=true,},
  ["hindicaptions"] = {["velthuis"]=true,},
  ["dnmacs"] = {["velthuis"]=true,},
  ["interchar"] = {["interchar"]=true,},
  ["gincltex"] = {["gincltex"]=true,},
  ["modernposter"] = {["modernposter"]=true,},
  ["hetarom"] = {["xymtex"]=true,},
  ["devanagari"] = {["velthuis"]=true,},
  ["ltjtarticle"] = {["luatexja"]=true,},
  ["dev209"] = {["velthuis"]=true,},
  ["lwarp-sectionbreak"] = {["lwarp"]=true,},
  ["amsopn"] = {["amsmath"]=true,},
  ["dev"] = {["velthuis"]=true,},
  ["hindi"] = {["velthuis"]=true,},
  ["example-image-a3"] = {["mwe"]=true,},
  ["babel-vai-vaii"] = {["babel"]=true,},
  ["epiolmec"] = {["epiolmec"]=true,},
  ["conv-utf8-il2"] = {["hyph-utf8"]=true,},
  ["beamercolorthemepenn"] = {["beamertheme-upenn-bc"]=true,},
  ["dynkin-diagrams"] = {["dynkin-diagrams"]=true,},
  ["vdmlisting"] = {["vdmlisting"]=true,},
  ["vaucanson"] = {["vaucanson-g"]=true,},
  ["person"] = {["datatool"]=true,},
  ["l3keys2e"] = {["l3packages"]=true,},
  ["PPRcorners"] = {["prosper"]=true,},
  ["play"] = {["play"]=true,},
  ["doclicense-CC-by-sa-4.0-plaintext"] = {["doclicense"]=true,},
  ["fontfam"] = {},
  ["Vaucanson-G"] = {["vaucanson-g"]=true,},
  ["VCPref-slides"] = {["vaucanson-g"]=true,},
  ["simurgh-minitoc"] = {["simurgh"]=true,},
  ["macro-architecte"] = {["pas-cv"]=true,},
  ["pgfplotstableshared.code"] = {["pgfplots"]=true,},
  ["varwidth"] = {["varwidth"]=true,},
  ["tikzpeople.shape.sailor"] = {["tikzpeople"]=true,},
  ["qcm"] = {["qcm"]=true,},
  ["multienv"] = {["multienv"]=true,},
  ["font_bera_concrete"] = {["font-change"]=true,},
  ["tikzlibrarypgfplots.clickable.code"] = {["pgfplots"]=true,},
  ["pl"] = {["plweb"]=true,},
  ["8point"] = {["varisize"]=true,},
  ["20point"] = {["varisize"]=true,},
  ["markdown"] = {["markdown"]=true,},
  ["17point"] = {["varisize"]=true,},
  ["MorrisIn"] = {["cfr-initials"]=true,},
  ["14point"] = {["varisize"]=true,},
  ["psfrag"] = {["psfrag"]=true,["graphics-pln"]=true,},
  ["babel-sangu"] = {["babel"]=true,},
  ["bxcalcux"] = {["bxcalc"]=true,},
  ["12point"] = {["varisize"]=true,},
  ["babel-khmer"] = {["babel"]=true,},
  ["nashbf"] = {["arabtex"]=true,},
  ["ltxindex"] = {["ltxindex"]=true,},
  ["lwarp-prelim2e"] = {["lwarp"]=true,},
  ["mtx-context-setups"] = {["context"]=true,},
  ["10pointss"] = {["varisize"]=true,},
  ["upzhkinsoku"] = {["upzhkinsoku"]=true,},
  ["lwarp-url"] = {["lwarp"]=true,},
  ["altfont"] = {["altfont"]=true,},
  ["extsizes"] = {["extsizes"]=true,},
  ["pgfplotsoldpgfsupp_pgfkeys.code"] = {["pgfplots"]=true,},
  ["lwarp-fullwidth"] = {["lwarp"]=true,},
  ["varindex"] = {["varindex"]=true,},
  ["gtrcore.node.code"] = {["genealogytree"]=true,},
  ["variations"] = {["variations"]=true,},
  ["cavantga"] = {},
  ["variablelm"] = {["variablelm"]=true,},
  ["hyphenat"] = {["hyphenat"]=true,},
  ["drawstack"] = {["drawstack"]=true,},
  ["lltjdefs"] = {["luatexja"]=true,},
  ["hgbarticle"] = {["hagenberg-thesis"]=true,},
  ["uwthesis"] = {["uwthesis"]=true,},
  ["uwmslide"] = {["uwmslide"]=true,},
  ["ducksay"] = {["ducksay"]=true,},
  ["utf8plsq"] = {["utf8mex"]=true,},
  ["hyph-ca.ec"] = {},
  ["ntx-math"] = {},
  ["utf8-pl"] = {["utf8mex"]=true,},
  ["ut-thesis"] = {["ut-thesis"]=true,},
  ["trimspaces"] = {["trimspaces"]=true,},
  ["courseoutline"] = {},
  ["uspatent"] = {["uspatent"]=true,},
  ["lwarp-sidecap"] = {["lwarp"]=true,},
  ["simurgh-extbook"] = {["simurgh"]=true,},
  ["esvect"] = {["esvect"]=true,},
  ["idx"] = {["latex"]=true,},
  ["waltham"] = {["context"]=true,},
  ["ar"] = {["aspectratio"]=true,},
  ["biditufte-handout"] = {["bidi"]=true,},
  ["uspace"] = {["uspace"]=true,},
  ["xypdf-ro"] = {["xypic"]=true,},
  ["hyph-tr"] = {},
  ["lwarp-lettrine"] = {["lwarp"]=true,},
  ["babel-kinyarwanda"] = {["babel"]=true,},
  ["lwarp-color"] = {["lwarp"]=true,},
  ["uri"] = {["uri"]=true,},
  ["xr"] = {["tools"]=true,},
  ["pgfplotscolor.code"] = {["pgfplots"]=true,},
  ["beamercolorthemeUR"] = {["urcls"]=true,},
  ["URspecialopts"] = {["urcls"]=true,},
  ["preview"] = {["preview"]=true,},
  ["URrules"] = {["urcls"]=true,},
  ["URpagestyles"] = {["urcls"]=true,},
  ["attachfile2"] = {["oberdiek"]=true,},
  ["ascii-jplain"] = {["ptex-base"]=true,},
  ["URletter"] = {["urcls"]=true,},
  ["suftesi"] = {["suftesi"]=true,},
  ["notoccite"] = {["notoccite"]=true,},
  ["URcolors"] = {["urcls"]=true,},
  ["URbeamer"] = {["urcls"]=true,},
  ["uptex"] = {["uptex-base"]=true,},
  ["upquote"] = {["upquote"]=true,},
  ["babel-serbian-latin-montenegro"] = {["babel"]=true,},
  ["curve2e"] = {["curve2e"]=true,},
  ["uppunctlm"] = {["uppunctlm"]=true,},
  ["font_artemisia_euler"] = {["font-change"]=true,},
  ["lwarp-chemgreek"] = {["lwarp"]=true,},
  ["upmethodology-task"] = {["upmethodology"]=true,},
  ["upmethodology-p-common"] = {["upmethodology"]=true,},
  ["upmethodology-frontpage"] = {["upmethodology"]=true,},
  ["tikzlibrarydecorations.text.code"] = {["pgf"]=true,},
  ["upmethodology-fmt"] = {["upmethodology"]=true,},
  ["upmethodology-document"] = {["upmethodology"]=true,},
  ["upmethodology-code"] = {["upmethodology"]=true,},
  ["powerdot-elcolors"] = {["powerdot"]=true,},
  ["lagally"] = {["arabi"]=true,},
  ["upmethodology-backpage"] = {["upmethodology"]=true,},
  ["multidef"] = {["multidef"]=true,},
  ["xyling"] = {["xyling"]=true,},
  ["schulschriften_ltx"] = {["schulschriften"]=true,},
  ["uptrace"] = {["uplatex"]=true,},
  ["xq"] = {["xq"]=true,},
  ["polytable"] = {["polytable"]=true,},
  ["ujbook"] = {["uplatex"]=true,},
  ["ifpdf"] = {["oberdiek"]=true,},
  ["ujarticle"] = {["uplatex"]=true,},
  ["lwarp-epstopdf-base"] = {["lwarp"]=true,},
  ["uowthesistitlepage"] = {["uowthesistitlepage"]=true,},
  ["tkz-tools-utilities"] = {["tkz-base"]=true,},
  ["simurgh-loader"] = {["simurgh"]=true,},
  ["unravel"] = {["unravel"]=true,},
  ["babel-serbian-latn-xk"] = {["babel"]=true,},
  ["babel-azerbaijani-cyrl"] = {["babel"]=true,},
  ["lapdf"] = {["lapdf"]=true,},
  ["beamerbasecolor"] = {["beamer"]=true,},
  ["HAPTCSTealBlue"] = {["ha-prosper"]=true,},
  ["tikzpeople.shape.santa"] = {["tikzpeople"]=true,},
  ["lwarp-marginfix"] = {["lwarp"]=true,},
  ["babel-sanskrit-beng"] = {["babel"]=true,},
  ["glossaries-extra-bib2gls"] = {["glossaries-extra"]=true,},
  ["univie-ling-wlg"] = {["univie-ling"]=true,},
  ["univie-ling-paper"] = {["univie-ling"]=true,},
  ["common-draftmode"] = {},
  ["universalis"] = {["universalis"]=true,},
  ["actuarialsymbol"] = {["actuarialsymbol"]=true,},
  ["kerkis"] = {["kerkis"]=true,},
  ["lwarp-watermark"] = {["lwarp"]=true,},
  ["fleqn"] = {["latex"]=true,},
  ["gridslides"] = {["gridslides"]=true,},
  ["bpfats7"] = {["iso10303"]=true,},
  ["multicol"] = {["tools"]=true,},
  ["example-image-golden"] = {["mwe"]=true,},
  ["contour"] = {["contour"]=true,},
  ["unisugar"] = {["unisugar"]=true,},
  ["repeat"] = {},
  ["unicode-math"] = {["unicode-math"]=true,},
  ["unicode-math-luatex"] = {["unicode-math"]=true,},
  ["load-unicode-xetex-classes"] = {["unicode-data"]=true,},
  ["lwarp-quoting"] = {["lwarp"]=true,},
  ["bullcntr"] = {["bullcntr"]=true,},
  ["pgfplotsoldpgfsupp_pgflibraryintersections.code"] = {["pgfplots"]=true,},
  ["recipe"] = {["recipe"]=true,},
  ["undolabl"] = {["undolabl"]=true,},
  ["gsp-default"] = {["gregoriotex"]=true,},
  ["workaddress"] = {["stex"]=true,},
  ["cybercic"] = {["cybercic"]=true,},
  ["underscore"] = {["underscore"]=true,},
  ["s-pre-08"] = {["context"]=true,},
  ["mex"] = {["mex"]=true,},
  ["s-pre-00"] = {["context"]=true,},
  ["umlaute"] = {["umlaute"]=true,},
  ["librebaskerville"] = {["librebaskerville"]=true,},
  ["esstixfrak"] = {["esstix"]=true,},
  ["uml"] = {["uml"]=true,},
  ["lwarp-pagegrid"] = {["lwarp"]=true,},
  ["legislation"] = {["umbclegislation"]=true,},
  ["novel-xmppacket"] = {["novel"]=true,},
  ["jarticle"] = {["platex"]=true,},
  ["PTSans"] = {["paratype"]=true,},
  ["phn-US_Vorwahlen"] = {["phonenumbers"]=true,},
  ["manmac"] = {},
  ["list-latin"] = {},
  ["ulem"] = {["ulem"]=true,},
  ["l3str-format"] = {["l3experimental"]=true,},
  ["loadhyph-pt"] = {},
  ["verses"] = {["arabtex"]=true,},
  ["epreuve"] = {["e-french"]=true,},
  ["ukrhypsm"] = {["ukrhyph"]=true,},
  ["lh-ot2"] = {["lh"]=true,},
  ["ukrhypmt"] = {["ukrhyph"]=true,},
  ["chbibref"] = {["chbibref"]=true,},
  ["luatexiniconfig"] = {["luatex"]=true,},
  ["aeguill"] = {["aeguill"]=true,},
  ["ukrhyph"] = {["ukrhyph"]=true,},
  ["ushyphex"] = {},
  ["onedash"] = {["logical-markup-utils"]=true,},
  ["enparen"] = {["oberdiek"]=true,},
  ["calcage"] = {["calcage"]=true,},
  ["floatflt"] = {["floatflt"]=true,},
  ["hetrans"] = {["arabtex"]=true,},
  ["collref"] = {["collref"]=true,},
  ["newtxsf"] = {["newtxsf"]=true,},
  ["ccalgo"] = {["computational-complexity"]=true,},
  ["fouriernc"] = {["fouriernc"]=true,},
  ["ukrenhyp"] = {["ukrhyph"]=true,},
  ["engord"] = {["oberdiek"]=true,},
  ["rules_ph"] = {["ukrhyph"]=true,},
  ["fourier"] = {["fourier"]=true,},
  ["concrete"] = {},
  ["rules90"] = {["ukrhyph"]=true,},
  ["cs-arev"] = {},
  ["notes2bib"] = {["notes2bib"]=true,},
  ["rules60"] = {["ukrhyph"]=true,},
  ["lcy2t2a"] = {["ukrhyph"]=true,},
  ["memhangul-x"] = {["kotex-oblivoir"]=true,},
  ["loadhyph-pms"] = {},
  ["lcy2ot2"] = {["ukrhyph"]=true,},
  ["lcy2lcy"] = {["ukrhyph"]=true,},
  ["lcy2koi"] = {["ukrhyph"]=true,},
  ["catlcy"] = {["ukrhyph"]=true,},
  ["uiucthesis"] = {["uiucthesis"]=true,},
  ["cutwin"] = {["cutwin"]=true,},
  ["babel-chinese-traditional-macausarchina"] = {["babel"]=true,},
  ["chextras"] = {["chextras"]=true,},
  ["uiucredborder"] = {["uiucredborder"]=true,},
  ["greek4cbc"] = {["archaic"]=true,},
  ["blkcntrl"] = {["frankenstein"]=true,},
  ["uhrzeit"] = {["uhrzeit"]=true,},
  ["uhhassignment"] = {["uhhassignment"]=true,},
  ["thm-llncs"] = {["thmtools"]=true,},
  ["fixcmex"] = {["fixcmex"]=true,},
  ["malay"] = {["arabtex"]=true,},
  ["udesoftec-bst"] = {["udesoftec"]=true,},
  ["hcreport"] = {["hc"]=true,},
  ["fontwrap"] = {["fontwrap"]=true,},
  ["beamercolorthememagpie"] = {["beamerdarkthemes"]=true,},
  ["mathfam256"] = {["mathfam256"]=true,},
  ["eso-pic"] = {["eso-pic"]=true,},
  ["pstricks"] = {["pstricks"]=true,},
  ["ketlayer2e"] = {["ketcindy"]=true,},
  ["hetaromh"] = {["xymtex"]=true,},
  ["beamercolorthemefibeamer-mu-sci"] = {["fibeamer"]=true,},
  ["datatool-base"] = {["datatool"]=true,},
  ["algc"] = {["algorithmicx"]=true,},
  ["sem-dem"] = {["seminar"]=true,},
  ["errata"] = {["errata"]=true,},
  ["lwarp-mdframed"] = {["lwarp"]=true,},
  ["pst-dbicons"] = {["pst-dbicons"]=true,},
  ["frhyphex"] = {["e-french"]=true,},
  ["cc4elsart"] = {["computational-complexity"]=true,},
  ["udesoftec-bibcommon"] = {["udesoftec"]=true,},
  ["ucthesis"] = {["ucthesis"]=true,},
  ["pgflibrarydecorations.shapes.code"] = {["pgf"]=true,},
  ["akklongpage"] = {["akktex"]=true,},
  ["tikzlibrarypgfplots.smithchart.code"] = {["pgfplots"]=true,},
  ["bxjscjkcat"] = {["bxjscls"]=true,},
  ["ucshyper"] = {["ucs"]=true,},
  ["loadhyph-id"] = {},
  ["ucs"] = {["ucs"]=true,},
  ["pgfmolbio.convert"] = {["pgfmolbio"]=true,},
  ["chronosys"] = {["chronosys"]=true,},
  ["interactiveworkbook-web"] = {["interactiveworkbook"]=true,},
  ["tascmac"] = {["ascmac"]=true,},
  ["lwarp-enumerate"] = {["lwarp"]=true,},
  ["newfloat"] = {["caption"]=true,},
  ["musixgui"] = {["musixtex"]=true,},
  ["FUpowerdot"] = {["powerdot-fuberlin"]=true,},
  ["heparse"] = {["arabtex"]=true,},
  ["aliascnt"] = {["oberdiek"]=true,},
  ["pgfmoduleshapes.code"] = {["pgf"]=true,},
  ["ucharcat"] = {["ucharcat"]=true,},
  ["duplicat"] = {["piff"]=true,},
  ["hyph-pms"] = {},
  ["ucdavisthesis"] = {["ucdavisthesis"]=true,},
  ["latin"] = {["babel"]=true,},
  ["ucbthesis"] = {["ucbthesis"]=true,},
  ["NumericPlots_legend"] = {["numericplots"]=true,},
  ["rubiktwocube"] = {["rubik"]=true,},
  ["CV"] = {["cv"]=true,},
  ["systeme"] = {["systeme"]=true,},
  ["tikzlibraryfpu.code"] = {["pgf"]=true,},
  ["cleveref"] = {["cleveref"]=true,},
  ["uassign"] = {["uassign"]=true,},
  ["extpfeil"] = {["extpfeil"]=true,},
  ["lwarp-rotating"] = {["lwarp"]=true,},
  ["tablestyles"] = {["tablestyles"]=true,},
  ["algcompatible"] = {["algorithmicx"]=true,},
  ["cooking-units"] = {["cooking-units"]=true,},
  ["ghsystem_explos"] = {["ghsystem"]=true,},
  ["lwarp-xurl"] = {["lwarp"]=true,},
  ["bibl-num"] = {["context"]=true,},
  ["uantwerpenletter"] = {["uantwerpendocs"]=true,},
  ["uantwerpenexam"] = {["uantwerpendocs"]=true,},
  ["uantwerpencoursetext"] = {["uantwerpendocs"]=true,},
  ["musixdbr"] = {["musixtex"]=true,},
  ["babel-samin"] = {["babel"]=true,},
  ["rmathbr"] = {["rmathbr"]=true,},
  ["dutchcal"] = {["dutchcal"]=true,},
  ["skbarticle"] = {["skb"]=true,},
  ["gtrcore.drawing.code"] = {["genealogytree"]=true,},
  ["babel-hausa-ghana"] = {["babel"]=true,},
  ["glossary-superragged"] = {["glossaries"]=true,},
  ["tikzlibraryplothandlers.code"] = {["pgf"]=true,},
  ["hyph-de-ch-1901.ec"] = {},
  ["uni-lcuc"] = {},
  ["kix"] = {["kix"]=true,},
  ["BHCexam"] = {["bhcexam"]=true,},
  ["stringstrings"] = {["stringstrings"]=true,},
  ["pst-vue3d"] = {["pst-vue3d"]=true,},
  ["babel-french"] = {["babel"]=true,},
  ["lwarp-newunicodechar"] = {["lwarp"]=true,},
  ["ua-thesis"] = {["uaclasses"]=true,},
  ["zref-runs"] = {["oberdiek"]=true,},
  ["beamerinnerthemerounded"] = {["beamer"]=true,},
  ["epigraph"] = {["epigraph"]=true,},
  ["sapolsky"] = {["context"]=true,},
  ["dijkstra"] = {["dijkstra"]=true,},
  ["my-thesis"] = {["uaclasses"]=true,},
  ["cc4svjour"] = {["computational-complexity"]=true,},
  ["typogrid"] = {["typogrid"]=true,},
  ["typoaid"] = {["typoaid"]=true,},
  ["feyn"] = {["feyn"]=true,},
  ["typewriter"] = {["typewriter"]=true,},
  ["hyph-sr-cyrl"] = {},
  ["lwarp-pdflscape"] = {["lwarp"]=true,},
  ["eltex5"] = {["eltex"]=true,},
  ["demo-xml"] = {["context"]=true,},
  ["emisa"] = {["emisa"]=true,},
  ["typehtml"] = {["typehtml"]=true,},
  ["novel-PostLayout"] = {["novel"]=true,},
  ["arseneau"] = {["nicetext"]=true,},
  ["beamerouterthememetropolis"] = {["beamertheme-metropolis"]=true,},
  ["sbl-paper"] = {["biblatex-sbl"]=true,},
  ["s-simpleslides-NarrowStripes"] = {["context-simpleslides"]=true,},
  ["lwarp-schemata"] = {["lwarp"]=true,},
  ["sseqdrawing.code"] = {["spectralsequences"]=true,},
  ["listing"] = {["listing"]=true,},
  ["type1cm"] = {["type1cm"]=true,},
  ["ltjarticle"] = {["luatexja"]=true,},
  ["mtcoff"] = {["minitoc"]=true,},
  ["marvosym"] = {["marvosym"]=true,},
  ["txuprcal"] = {["txuprcal"]=true,},
  ["piccorr"] = {["pictex"]=true,},
  ["solvesudoku"] = {["sudokubundle"]=true,},
  ["myfilist"] = {["fileinfo"]=true,},
  ["twoup"] = {["twoup"]=true,},
  ["samples"] = {["context"]=true,},
  ["2in1"] = {["twoinone"]=true,},
  ["fontawesome5-utex-helper"] = {["fontawesome5"]=true,},
  ["fithesis3"] = {["fithesis"]=true,},
  ["ltxdocext"] = {["revtex"]=true,},
  ["thesis-gwu"] = {["thesis-gwu"]=true,},
  ["turnpagewoetex"] = {["turnthepage"]=true,},
  ["tikzlibrarydatavisualization.sparklines.code"] = {["pgf"]=true,},
  ["turnpageetex"] = {["turnthepage"]=true,},
  ["xecyr"] = {["xecyr"]=true,},
  ["turabian-thesis"] = {["turabian-formatting"]=true,},
  ["turabian-formatting"] = {["turabian-formatting"]=true,},
  ["turabian"] = {["turabian"]=true,},
  ["multiaudience"] = {["multiaudience"]=true,},
  ["titletoc"] = {["titlesec"]=true,},
  ["beamerthemeSzeged"] = {["beamer"]=true,},
  ["subfigure"] = {["subfigure"]=true,},
  ["tugboat"] = {["tugboat-plain"]=true,},
  ["ltugproc"] = {["tugboat"]=true,},
  ["babel-basaa"] = {["babel"]=true,},
  ["hfoldsty"] = {["hfoldsty"]=true,},
  ["ltugcomn"] = {["tugboat"]=true,},
  ["pst-optexp"] = {["pst-optexp"]=true,},
  ["gatech-thesis-index"] = {["gatech-thesis"]=true,},
  ["ctimes"] = {},
  ["ltugboat"] = {["tugboat"]=true,},
  ["xintfrac"] = {["xint"]=true,},
  ["iwhdp"] = {["iwhdp"]=true,},
  ["babel-spanish"] = {["babel"]=true,},
  ["tudscrtutorial"] = {["tudscr"]=true,},
  ["atrans"] = {["arabtex"]=true,},
  ["beamerbasedecode"] = {["beamer"]=true,},
  ["tudscrsupervisor"] = {["tudscr"]=true,},
  ["tudscrreprt"] = {["tudscr"]=true,},
  ["etrans"] = {["arabtex"]=true,},
  ["ntimes"] = {["cslatex"]=true,},
  ["tudscrposter"] = {["tudscr"]=true,},
  ["tudscrmanual"] = {["tudscr"]=true,},
  ["pauldoc"] = {["pauldoc"]=true,},
  ["tudscrfonts"] = {["tudscr"]=true,},
  ["tudscrdoc"] = {["tudscr"]=true,},
  ["DB_process"] = {},
  ["tcbxparse.code"] = {["tcolorbox"]=true,},
  ["tikzpeople.shape.graduate"] = {["tikzpeople"]=true,},
  ["kam-il2"] = {["enctex"]=true,},
  ["lisp-read"] = {["lisp-on-tex"]=true,},
  ["qtimes"] = {["tex-gyre"]=true,},
  ["tudscrcomp-poster"] = {["tudscr"]=true,},
  ["example-glossaries-multipar"] = {["glossaries"]=true,},
  ["beamerinnerthemeCuerna"] = {["beamertheme-cuerna"]=true,},
  ["babel-makonde"] = {["babel"]=true,},
  ["neuralnetwork"] = {["neuralnetwork"]=true,},
  ["tudscrbase"] = {["tudscr"]=true,},
  ["tudscrartcl"] = {["tudscr"]=true,},
  ["ifdraft"] = {["oberdiek"]=true,},
  ["xarbskel"] = {["arabtex"]=true,},
  ["mathswap"] = {["tudscr"]=true,},
  ["biolinum"] = {["libertine"]=true,},
  ["babel-jolafonyi"] = {["babel"]=true,},
  ["small2e"] = {["latex"]=true,},
  ["babel-dzongkha"] = {["babel"]=true,},
  ["hyph-nn.ec"] = {},
  ["tsemlines"] = {},
  ["beamerbasetemplates"] = {["beamer"]=true,},
  ["truncate"] = {["truncate"]=true,},
  ["moderncvbodyv"] = {["moderncv"]=true,},
  ["pgfplotsoldpgfsupp_pgfmanual.code"] = {["pgfplots"]=true,},
  ["trigonometry"] = {["trigonometry"]=true,},
  ["lwarp-bigstrut"] = {["lwarp"]=true,},
  ["babel-hindi"] = {["babel"]=true,},
  ["trfsigns"] = {["trfsigns"]=true,},
  ["treetex"] = {["treetex"]=true,},
  ["classes"] = {["treetex"]=true,},
  ["weekday"] = {},
  ["itnumpar"] = {["itnumpar"]=true,},
  ["tree-dvips"] = {["tree-dvips"]=true,},
  ["autopdf"] = {["autopdf"]=true,},
  ["lingmacros"] = {["tree-dvips"]=true,},
  ["babel-bodo"] = {["babel"]=true,},
  ["feynmp-auto"] = {["feynmp-auto"]=true,},
  ["translator"] = {["translator"]=true,},
  ["translations"] = {["translations"]=true,},
  ["pgfornament-han"] = {["pgfornament-han"]=true,},
  ["arrow"] = {["eplain"]=true,},
  ["tram"] = {["tram"]=true,},
  ["lh-t2accr"] = {["lh"]=true,},
  ["titlefoot"] = {},
  ["frenchpro"] = {["e-french"]=true,},
  ["semaf"] = {["semaphor"]=true,},
  ["robustcommand"] = {["robustcommand"]=true,},
  ["datetime2-en-fulltext"] = {["datetime2-en-fulltext"]=true,},
  ["tracklang-scripts"] = {["tracklang"]=true,},
  ["expertpart"] = {},
  ["tracklang-region-codes"] = {["tracklang"]=true,},
  ["bidituftesidenote"] = {["bidi"]=true,},
  ["sseqparsers.code"] = {["spectralsequences"]=true,},
  ["lwarp-fancybox"] = {["lwarp"]=true,},
  ["autoaligne"] = {["autoaligne"]=true,},
  ["footnote"] = {["mdwtools"]=true,},
  ["concepts"] = {["concepts"]=true,},
  ["pagecolor"] = {["pagecolor"]=true,},
  ["xypic"] = {["xypic"]=true,},
  ["threeparttable"] = {["threeparttable"]=true,},
  ["tpslifonts"] = {["tpslifonts"]=true,},
  ["biditufte-book"] = {["bidi"]=true,},
  ["sperr"] = {["js-misc"]=true,},
  ["nccheadings"] = {["ncclatex"]=true,},
  ["totpages"] = {["totpages"]=true,},
  ["bpfs3"] = {["iso10303"]=true,},
  ["bxcjkjatype"] = {["bxcjkjatype"]=true,},
  ["ccaux"] = {["computational-complexity"]=true,},
  ["datatool-pgfmath"] = {["datatool"]=true,},
  ["sduthesis"] = {["sduthesis"]=true,},
  ["s-pre-23"] = {["context"]=true,},
  ["scrlayer"] = {},
  ["luatex-preprocessor-test"] = {["context"]=true,},
  ["frenchle"] = {["e-french"]=true,},
  ["novel-Footnotes"] = {["novel"]=true,},
  ["allegoth"] = {["bookhands"]=true,},
  ["AdobeExternalProfiles"] = {["pdfx"]=true,},
  ["cwbl-german"] = {["cweb-latex"]=true,},
  ["cvthemebidicasual"] = {["bidi"]=true,},
  ["toptesi"] = {["toptesi"]=true,},
  ["graphics"] = {["graphics"]=true,},
  ["fixseminar"] = {["texpower"]=true,},
  ["toptesi-scudo"] = {["toptesi"]=true,},
  ["horoscop"] = {["horoscop"]=true,},
  ["ndsu-thesis"] = {["ndsu-thesis"]=true,},
  ["flashcards"] = {["flashcards"]=true,},
  ["xepersian-mathsdigitspec"] = {["xepersian"]=true,},
  ["babel-malay-bn"] = {["babel"]=true,},
  ["octavo"] = {["octavo"]=true,},
  ["topfront"] = {["toptesi"]=true,},
  ["fduthesis"] = {["fduthesis"]=true,},
  ["lwarp-mparhack"] = {["lwarp"]=true,},
  ["schuleub"] = {["schule"]=true,},
  ["subfiles"] = {["subfiles"]=true,},
  ["algorithmic"] = {["algorithms"]=true,},
  ["xetexko-var"] = {["kotex-oblivoir"]=true,},
  ["aoutput"] = {["arabtex"]=true,},
  ["here"] = {["here"]=true,},
  ["mbenotes"] = {["mbenotes"]=true,},
  ["kpfonts"] = {["kpfonts"]=true,},
  ["esindex"] = {["bezos"]=true,},
  ["xspace"] = {["tools"]=true,},
  ["tokenizer"] = {["tokenizer"]=true,},
  ["auto-pst-pdf"] = {["auto-pst-pdf"]=true,},
  ["glossary-mcols"] = {["glossaries"]=true,},
  ["trace"] = {["tools"]=true,},
  ["dblfloatfix"] = {["dblfloatfix"]=true,},
  ["thp"] = {["tools"]=true,},
  ["fp-upn"] = {["fp"]=true,},
  ["thmb"] = {["tools"]=true,},
  ["settobox"] = {["oberdiek"]=true,},
  ["thm"] = {["tools"]=true,},
  ["ifnextok"] = {["ifnextok"]=true,},
  ["lwarp-intopdf"] = {["lwarp"]=true,},
  ["cnbwp"] = {["cnbwp"]=true,},
  ["babel-arabic-morocco"] = {["babel"]=true,},
  ["theorem"] = {["tools"]=true,},
  ["thcb"] = {["tools"]=true,},
  ["quevedo-es"] = {["context"]=true,},
  ["thb"] = {["tools"]=true,},
  ["mcmthesis"] = {["mcmthesis"]=true,},
  ["plain-il2-cs"] = {["enctex"]=true,},
  ["tabularx"] = {["tools"]=true,},
  ["bidiftnxtra"] = {["bidi"]=true,},
  ["mxedruli"] = {["mxedruli"]=true,},
  ["l3galley"] = {["l3experimental"]=true,},
  ["pxtx-cal"] = {["pxtxalfa"]=true,},
  ["fontsmpl"] = {["tools"]=true,},
  ["l3sort"] = {["l3kernel"]=true,},
  ["babel-luyia"] = {["babel"]=true,},
  ["r"] = {["tools"]=true,},
  ["q"] = {["tools"]=true,},
  ["curvesls"] = {["curves"]=true,},
  ["multicol-2017-04-11"] = {["tools"]=true,},
  ["deccomma"] = {["fltpoint"]=true,},
  ["longtable"] = {["tools"]=true,},
  ["multido"] = {["multido"]=true,},
  ["pgflibrarygraphdrawing.examples.code"] = {["pgf"]=true,},
  ["layout"] = {["tools"]=true,},
  ["indentfirst"] = {["tools"]=true,},
  ["hhline"] = {["tools"]=true,},
  ["cs-baskerx"] = {},
  ["h"] = {["tools"]=true,},
  ["uentities"] = {["jadetex"]=true,},
  ["figcaps"] = {["preprint"]=true,},
  ["ftnright"] = {["tools"]=true,},
  ["showkeys"] = {["tools"]=true,},
  ["enumerate"] = {["tools"]=true,},
  ["e"] = {["tools"]=true,},
  ["babel-somali"] = {["babel"]=true,},
  ["arabluatex"] = {["arabluatex"]=true,},
  ["IEEEtrantools"] = {["ieeetran"]=true,},
  ["pgfmolbio.domains"] = {["pgfmolbio"]=true,},
  ["musixeng"] = {["musixtex"]=true,},
  ["delarray"] = {["tools"]=true,},
  ["dcolumn"] = {["tools"]=true,},
  ["babel-swahili"] = {["babel"]=true,},
  ["sgmlcmpt"] = {["jknapltx"]=true,},
  ["bm"] = {["tools"]=true,},
  ["xtemplate"] = {["l3packages"]=true,},
  ["ghsystem_flame-2-black"] = {["ghsystem"]=true,},
  ["binomexp"] = {["binomexp"]=true,},
  ["array"] = {["tools"]=true,},
  ["array-2016-10-06"] = {["tools"]=true,},
  ["afterpage"] = {["tools"]=true,},
  ["amsqpx"] = {["qpxqtx"]=true,},
  ["babel-koyrachiini"] = {["babel"]=true,},
  ["isotope"] = {["isotope"]=true,},
  ["toolbox"] = {["toolbox"]=true,},
  ["varioref"] = {["tools"]=true,},
  ["lwarp-underscore"] = {["lwarp"]=true,},
  ["optidef"] = {["optidef"]=true,},
  ["todo"] = {["todo"]=true,},
  ["akktecdoc"] = {["akktex"]=true,},
  ["lwarp-xcolor"] = {["lwarp"]=true,},
  ["mucproc"] = {["mucproc"]=true,},
  ["germanb"] = {["babel"]=true,},
  ["tocvsec2"] = {["tocvsec2"]=true,},
  ["lplfitch"] = {["lplfitch"]=true,},
  ["tx-math"] = {},
  ["beamercolorthemefibeamer-mu-fss"] = {["fibeamer"]=true,},
  ["musixguit"] = {["musixguit"]=true,},
  ["tocloft"] = {["tocloft"]=true,},
  ["pgfmolbio"] = {["pgfmolbio"]=true,},
  ["spotcolor"] = {["spotcolor"]=true,},
  ["texapi"] = {["texapi"]=true,},
  ["font_iwona-medium"] = {["font-change"]=true,},
  ["tlc-article"] = {["tlc-article"]=true,},
  ["tkz-tab"] = {["tkz-tab"]=true,},
  ["hyph-sa"] = {["hyphen-sanskrit"]=true,},
  ["pgfplotssysgeneric.code"] = {["pgfplots"]=true,},
  ["tkz-kiviat"] = {["tkz-kiviat"]=true,},
  ["babel-punjabi-guru"] = {["babel"]=true,},
  ["tkz-graph"] = {["tkz-graph"]=true,},
  ["automata"] = {["texpower"]=true,},
  ["bxenclose"] = {["bxenclose"]=true,},
  ["tkz-fct"] = {["tkz-fct"]=true,},
  ["pst-all"] = {["pstricks"]=true,},
  ["tkz-tools-transformations"] = {["tkz-euclide"]=true,},
  ["tkz-tools-intersections"] = {["tkz-euclide"]=true,},
  ["tkz-obj-vectors"] = {["tkz-euclide"]=true,},
  ["bpfap2"] = {["iso10303"]=true,},
  ["tkz-obj-sectors"] = {["tkz-euclide"]=true,},
  ["tkz-obj-protractor"] = {["tkz-euclide"]=true,},
  ["example"] = {},
  ["menu"] = {["menu"]=true,},
  ["tkz-obj-lines"] = {["tkz-euclide"]=true,},
  ["diagnose"] = {["diagnose"]=true,},
  ["tkz-obj-arcs"] = {["tkz-euclide"]=true,},
  ["mensa-tex"] = {["mensa-tex"]=true,},
  ["subcaption"] = {["caption"]=true,},
  ["acmconf"] = {["acmconf"]=true,},
  ["tkz-obj-angles"] = {["tkz-euclide"]=true,},
  ["beamerfontthemestructurebold"] = {["beamer"]=true,},
  ["tkz-tools-misc"] = {["tkz-base"]=true,},
  ["font_iwona-bold"] = {["font-change"]=true,},
  ["tkz-obj-addpoints"] = {["tkz-euclide"]=true,},
  ["tkz-euclide"] = {["tkz-euclide"]=true,},
  ["simurgh-refrep"] = {["simurgh"]=true,},
  ["tkzexample"] = {["tkz-doc"]=true,},
  ["psibycus"] = {["ibygrk"]=true,},
  ["babel-cornish"] = {["babel"]=true,},
  ["tkz-berge"] = {["tkz-berge"]=true,},
  ["rapport1"] = {["ntgclass"]=true,},
  ["amsxtra"] = {["amsmath"]=true,},
  ["tfrupee"] = {["tfrupee"]=true,},
  ["colorweb"] = {["colorweb"]=true,},
  ["luatexko"] = {["luatexko"]=true,},
  ["quire"] = {["midnight"]=true,},
  ["UoWthesis"] = {["uowthesis"]=true,},
  ["babel-english-au"] = {["babel"]=true,},
  ["doclicense-CC-by-nc-nd-4.0-latex"] = {["doclicense"]=true,},
  ["tkz-tools-obsolete"] = {["tkz-base"]=true,},
  ["Baskervaldx"] = {["baskervaldx"]=true,},
  ["beamerouterthemetree"] = {["beamer"]=true,},
  ["tkz-tools-math"] = {["tkz-base"]=true,},
  ["sitem"] = {["sitem"]=true,},
  ["tkz-tools-base"] = {["tkz-base"]=true,},
  ["font_kurier-condensed-medium"] = {["font-change"]=true,},
  ["tkz-tools-arith"] = {["tkz-base"]=true,},
  ["fr-longtable"] = {["floatrow"]=true,},
  ["tkz-obj-segments"] = {["tkz-base"]=true,},
  ["tkz-obj-points"] = {["tkz-base"]=true,},
  ["tkz-obj-marks"] = {["tkz-base"]=true,},
  ["t-account"] = {["context-account"]=true,},
  ["tkz-base"] = {["tkz-base"]=true,},
  ["titling"] = {["titling"]=true,},
  ["titlesec"] = {["titlesec"]=true,},
  ["titleps"] = {["titlesec"]=true,},
  ["titleref"] = {["titleref"]=true,},
  ["ku-template"] = {["ku-template"]=true,},
  ["tikzlibrarypenrose.code"] = {["penrose"]=true,},
  ["s-faq-01"] = {["context"]=true,},
  ["arababel"] = {["arabtex"]=true,},
  ["endnotes"] = {["endnotes"]=true,},
  ["easymat"] = {["easy"]=true,},
  ["vowel"] = {["tipa"]=true,},
  ["cities"] = {["pst-geo"]=true,},
  ["babel-kazakh"] = {["babel"]=true,},
  ["cjhebrew"] = {["cjhebrew"]=true,},
  ["mex2"] = {["mex"]=true,},
  ["tikzlibrarycolorbrewer.code"] = {["pgfplots"]=true,},
  ["endofproofwd"] = {["endofproofwd"]=true,},
  ["tone"] = {["tipa"]=true,},
  ["dvgloss"] = {["dvgloss"]=true,},
  ["go"] = {},
  ["extraipa"] = {["tipa"]=true,},
  ["thaispec"] = {["thaispec"]=true,},
  ["example-image-a4-numbered"] = {["mwe"]=true,},
  ["timing-diagrams"] = {["timing-diagrams"]=true,},
  ["booktabs"] = {["booktabs"]=true,},
  ["ward"] = {["context"]=true,},
  ["sourcesanspro"] = {["sourcesanspro"]=true,},
  ["cc"] = {["computational-complexity"]=true,},
  ["t-inifile"] = {["context-inifile"]=true,},
  ["lwarp-aecc"] = {["lwarp"]=true,},
  ["dhucs-enumerate"] = {["kotex-utf"]=true,},
  ["beuron"] = {["beuron"]=true,},
  ["tikzsymbols"] = {["tikzsymbols"]=true,},
  ["lwarp-abstract"] = {["lwarp"]=true,},
  ["rotate"] = {["dvips"]=true,},
  ["doclicense-CC-by-nc-sa-3.0-latex"] = {["doclicense"]=true,},
  ["tikzscale"] = {["tikzscale"]=true,},
  ["tikzposterTitlestyles"] = {["tikzposter"]=true,},
  ["tikzposterLayoutthemes"] = {["tikzposter"]=true,},
  ["endnote"] = {},
  ["tikzposterInnerblockstyles"] = {["tikzposter"]=true,},
  ["fix2col"] = {["fix2col"]=true,},
  ["babel-persian"] = {["babel"]=true,},
  ["fltpoint"] = {["fltpoint"]=true,},
  ["tikzposterColorstyles"] = {["tikzposter"]=true,},
  ["exam"] = {["exam"]=true,},
  ["cs-iwona"] = {},
  ["ltxtable"] = {["carlisle"]=true,},
  ["overcite"] = {["cite"]=true,},
  ["lwarp-verse"] = {["lwarp"]=true,},
  ["eltex6"] = {["eltex"]=true,},
  ["calculation"] = {["calculation"]=true,},
  ["blox"] = {["blox"]=true,},
  ["tikzposterBackgroundstyles"] = {["tikzposter"]=true,},
  ["tikzposter"] = {["tikzposter"]=true,},
  ["macro-univers"] = {["pas-cv"]=true,},
  ["tikzpfeile"] = {["tikzpfeile"]=true,},
  ["babel-english-us"] = {["babel"]=true,},
  ["chemmacros.module.xfrac.code"] = {["chemmacros"]=true,},
  ["tabmac"] = {["edmac"]=true,},
  ["t-bnf"] = {["context-bnf"]=true,},
  ["wotree"] = {["dratex"]=true,},
  ["tikzpeople"] = {["tikzpeople"]=true,},
  ["lwarp-endnotes"] = {["lwarp"]=true,},
  ["CD_labeler"] = {},
  ["tikzpeople.shape.surgeon"] = {["tikzpeople"]=true,},
  ["pdfbase"] = {["media9"]=true,},
  ["gtrlib.templates.code"] = {["genealogytree"]=true,},
  ["beamerbasethemes"] = {["beamer"]=true,},
  ["tikzpeople.shape.priest"] = {["tikzpeople"]=true,},
  ["PlayfairDisplay"] = {["playfair"]=true,},
  ["syntrace"] = {["syntrace"]=true,},
  ["minipage-marginpar"] = {["minipage-marginpar"]=true,},
  ["tikzpeople.shape.pilot"] = {["tikzpeople"]=true,},
  ["luatexconfig"] = {["tex-ini-files"]=true,},
  ["tikzpeople.shape.mexican"] = {["tikzpeople"]=true,},
  ["rviewport"] = {["rviewport"]=true,},
  ["hyph-la"] = {},
  ["binary"] = {["siunits"]=true,},
  ["tikzpeople.shape.maninblack"] = {["tikzpeople"]=true,},
  ["tikzpeople.shape.judge"] = {["tikzpeople"]=true,},
  ["akkmathfun"] = {["akktex"]=true,},
  ["fenixtok"] = {["fenixpar"]=true,},
  ["karnaughmap"] = {["karnaughmap"]=true,},
  ["cjenatbib"] = {["cje"]=true,},
  ["tikzpeople.shape.guard"] = {["tikzpeople"]=true,},
  ["ltr-job"] = {["lecturer"]=true,},
  ["babel-hawaiian"] = {["babel"]=true,},
  ["cnltx"] = {["cnltx"]=true,},
  ["baskervald"] = {["baskervald"]=true,},
  ["serto"] = {["aramaic-serto"]=true,},
  ["tikzpeople.shape.groom"] = {["tikzpeople"]=true,},
  ["ctxdocstrip"] = {["ctex"]=true,},
  ["testfox"] = {["lh"]=true,},
  ["tikzpeople.shape.duck"] = {["tikzpeople"]=true,},
  ["ncccomma"] = {["ncctools"]=true,},
  ["tikzpeople.shape.devil"] = {["tikzpeople"]=true,},
  ["droit-fr"] = {["droit-fr"]=true,},
  ["mcol-ex"] = {["gustlib"]=true,},
  ["tikzpeople.shape.dave"] = {["tikzpeople"]=true,},
  ["mlcid"] = {["japanese-otf"]=true,},
  ["cv4tw"] = {["cv4tw"]=true,},
  ["hebtex"] = {["arabtex"]=true,},
  ["docindex"] = {["xdoc"]=true,},
  ["pxjahyper"] = {["pxjahyper"]=true,},
  ["stepman"] = {["iso10303"]=true,},
  ["ebezier"] = {["ebezier"]=true,},
  ["tikzpeople.shape.chef"] = {["tikzpeople"]=true,},
  ["cs-antt"] = {},
  ["asapsym.code"] = {["asapsym"]=true,},
  ["tikzpeople.shape.charlie"] = {["tikzpeople"]=true,},
  ["bibtopic"] = {["bibtopic"]=true,},
  ["verbatim-dek"] = {["gustlib"]=true,},
  ["qobitree"] = {["qobitree"]=true,},
  ["wsemclassic"] = {["wsemclassic"]=true,},
  ["tikzpeople.shape.alice"] = {["tikzpeople"]=true,},
  ["beamerthemetree"] = {["beamer"]=true,},
  ["screenplay"] = {["screenplay"]=true,},
  ["semrot"] = {["seminar"]=true,},
  ["cv4tw-theme-core"] = {["cv4tw"]=true,},
  ["glyphtounicode-cs"] = {},
  ["tikzpagenodes"] = {["tikzpagenodes"]=true,},
  ["allrust"] = {["bookhands"]=true,},
  ["mynsfc"] = {["mynsfc"]=true,},
  ["russianb"] = {["babel"]=true,},
  ["tikzorbital"] = {["tikzorbital"]=true,},
  ["lwarp-nicefrac"] = {["lwarp"]=true,},
  ["tikzlibraryfeynhand.code"] = {["tikz-feynhand"]=true,},
  ["emf"] = {["emf"]=true,},
  ["font_antt-condensed-light"] = {["font-change"]=true,},
  ["tikzlibrarymarmots.code"] = {["tikzmarmots"]=true,},
  ["shuffle"] = {["shuffle"]=true,},
  ["dfgreporting"] = {["proposal"]=true,},
  ["PPRserpaggi"] = {["prosper"]=true,},
  ["tikzinclude"] = {["tikzinclude"]=true,},
  ["tikzlibraryducks.code"] = {["tikzducks"]=true,},
  ["lwarp-nonumonpart"] = {["lwarp"]=true,},
  ["leadsheets.library.musejazz.code"] = {["leadsheets"]=true,},
  ["sourcesanspro-type1-autoinst"] = {["sourcesanspro"]=true,},
  ["svninfo"] = {["svninfo"]=true,},
  ["arabicfront"] = {["bezos"]=true,},
  ["typearea"] = {},
  ["tikz-timing"] = {["tikz-timing"]=true,},
  ["lwarp-todonotes"] = {["lwarp"]=true,},
  ["storebox"] = {["storebox"]=true,},
  ["tikz-timing-nicetabs"] = {["tikz-timing"]=true,},
  ["bbding"] = {["bbding"]=true,},
  ["beamerbaseframesize"] = {["beamer"]=true,},
  ["backgrnd"] = {},
  ["akletter"] = {["akletter"]=true,},
  ["t-pgfplots"] = {["pgfplots"]=true,},
  ["fcolumn"] = {["fcolumn"]=true,},
  ["newtxtt"] = {["newtxtt"]=true,},
  ["tikz-timing-interval"] = {["tikz-timing"]=true,},
  ["coolstr"] = {["coolstr"]=true,},
  ["syntonly"] = {["latex"]=true,},
  ["tikz-timing-ifsym"] = {["tikz-timing"]=true,},
  ["fp-pas"] = {["fp"]=true,},
  ["cherokee"] = {["cherokee"]=true,},
  ["tikz-timing-counters"] = {["tikz-timing"]=true,},
  ["tikzlibrarydatavisualization.code"] = {["pgf"]=true,},
  ["opmac-bib-iso690"] = {},
  ["tikz-timing-clockarrows"] = {["tikz-timing"]=true,},
  ["hyph-da.ec"] = {},
  ["tikz-timing-beamer"] = {["tikz-timing"]=true,},
  ["lwarp-boxedminipage2e"] = {["lwarp"]=true,},
  ["pgfplotsticks.code"] = {["pgfplots"]=true,},
  ["piano"] = {["piano"]=true,},
  ["tikz-timing-arrows"] = {["tikz-timing"]=true,},
  ["tikz-timing-advnodes"] = {["tikz-timing"]=true,},
  ["eqname"] = {},
  ["prepictex"] = {["pictex"]=true,},
  ["tikz-qtree-compat"] = {["tikz-qtree"]=true,},
  ["tikzlibraryshapes.gates.logic.IEC.code"] = {["pgf"]=true,},
  ["dlfltxbmarkupbookkeys"] = {["dlfltxb"]=true,},
  ["lecturer"] = {["lecturer"]=true,},
  ["pgftree"] = {["tikz-qtree"]=true,},
  ["lwarp-ltablex"] = {["lwarp"]=true,},
  ["artikel1"] = {["ntgclass"]=true,},
  ["eskddstu"] = {["eskdx"]=true,},
  ["frletter"] = {["frletter"]=true,},
  ["loadhyph-uk"] = {},
  ["hyperref"] = {["hyperref"]=true,},
  ["catalan"] = {["babel"]=true,},
  ["pgfsubpic"] = {["tikz-qtree"]=true,},
  ["tikz-palattice"] = {["tikz-palattice"]=true,},
  ["tikzlibraryoptics.code"] = {["tikz-optics"]=true,},
  ["tikz-opm"] = {["tikz-opm"]=true,},
  ["hobsub"] = {["oberdiek"]=true,},
  ["tikzlibrarynef.code"] = {["tikz-nef"]=true,},
  ["changebar"] = {["changebar"]=true,},
  ["tikz-layers"] = {["tikz-layers"]=true,},
  ["ean13isbn"] = {["ean13isbn"]=true,},
  ["spalign"] = {["spalign"]=true,},
  ["ajmacros"] = {["japanese-otf"]=true,},
  ["chapterbib"] = {["cite"]=true,},
  ["lwarp-scrhack"] = {["lwarp"]=true,},
  ["beamerthemeMalmoe"] = {["beamer"]=true,},
  ["tikzlibrarykarnaugh.code"] = {["tikz-karnaugh"]=true,},
  ["tikz-kalender"] = {["tikz-kalender"]=true,},
  ["tikz-inet"] = {["tikz-inet"]=true,},
  ["tikzlibraryfeynman.code"] = {["tikz-feynman"]=true,},
  ["boxedminipage"] = {["boxedminipage"]=true,},
  ["pst-fractal"] = {["pst-fractal"]=true,},
  ["babel-chinese-simplified"] = {["babel"]=true,},
  ["csbulacronym"] = {["csbulletin"]=true,},
  ["tikzfeynhand.keys.code"] = {["tikz-feynhand"]=true,},
  ["arabluatex-patch"] = {["arabluatex"]=true,},
  ["bidituftetitle"] = {["bidi"]=true,},
  ["pinlabel"] = {["pinlabel"]=true,},
  ["combelow"] = {["combelow"]=true,},
  ["tikz-feynhand"] = {["tikz-feynhand"]=true,},
  ["m-tex4ht"] = {["tex4ht"]=true,},
  ["philex"] = {["philex"]=true,},
  ["SASnRdisplay"] = {["sasnrdisplay"]=true,},
  ["fonetika"] = {["fonetika"]=true,},
  ["guha"] = {["arabtex"]=true,},
  ["tikz-3dplot"] = {["tikz-3dplot"]=true,},
  ["beamerthemedefault"] = {["beamer"]=true,},
  ["bxjsarticle"] = {["bxjscls"]=true,},
  ["lwarp-wrapfig"] = {["lwarp"]=true,},
  ["abraces"] = {["abraces"]=true,},
  ["pbox"] = {["pbox"]=true,},
  ["efrench"] = {["e-french"]=true,},
  ["csquotes"] = {["csquotes"]=true,},
  ["ticket"] = {["ticket"]=true,},
  ["thuthesis"] = {["thuthesis"]=true,},
  ["tlmgr-article"] = {},
  ["rvwrite"] = {["rvwrite"]=true,},
  ["luatexja-ruby"] = {["luatexja"]=true,},
  ["brief"] = {["ntgclass"]=true,},
  ["pst-blur"] = {["pst-blur"]=true,},
  ["pst-mirror"] = {["pst-mirror"]=true,},
  ["hyph-de-1996"] = {},
  ["iidef"] = {["thucoursework"]=true,},
  ["NumericPlots_TickLabels"] = {["numericplots"]=true,},
  ["tikzlibrarytqft.code"] = {["tqft"]=true,},
  ["thmtools"] = {["thmtools"]=true,},
  ["s-abbreviations-logos"] = {["context"]=true,},
  ["thmdef-thmbox"] = {["thmtools"]=true,},
  ["bloques"] = {["bloques"]=true,},
  ["thmdef-shaded"] = {["thmtools"]=true,},
  ["signchart"] = {["signchart"]=true,},
  ["luatex-swiglib-test"] = {["context"]=true,},
  ["babel-northernluri"] = {["babel"]=true,},
  ["lexref"] = {["lexref"]=true,},
  ["thmdef-mdframed"] = {["thmtools"]=true,},
  ["canoniclayout"] = {["canoniclayout"]=true,},
  ["koi2ot2"] = {},
  ["french"] = {["e-french"]=true,},
  ["devanagaridigits"] = {["polyglossia"]=true,},
  ["thm-restate"] = {["thmtools"]=true,},
  ["thm-patch"] = {["thmtools"]=true,},
  ["thm-ntheorem"] = {["thmtools"]=true,},
  ["babel-gusii"] = {["babel"]=true,},
  ["udesoftec"] = {["udesoftec"]=true,},
  ["thm-listof"] = {["thmtools"]=true,},
  ["transparent"] = {["oberdiek"]=true,},
  ["rapport3"] = {["ntgclass"]=true,},
  ["sesstime"] = {["sesstime"]=true,},
  ["thm-kv"] = {["thmtools"]=true,},
  ["babel-kyrgyz"] = {["babel"]=true,},
  ["lwarp-layout"] = {["lwarp"]=true,},
  ["pb-diagram"] = {["pb-diagram"]=true,},
  ["hypdestopt"] = {["oberdiek"]=true,},
  ["thm-amsthm"] = {["thmtools"]=true,},
  ["sfmath"] = {},
  ["makebase"] = {["makebase"]=true,},
  ["parseargs"] = {["thmtools"]=true,},
  ["currfile"] = {["currfile"]=true,},
  ["cu-calendar"] = {["churchslavonic"]=true,},
  ["biblatex-chicago"] = {["biblatex-chicago"]=true,},
  ["acjk"] = {["arabtex"]=true,},
  ["dirtree"] = {["dirtree"]=true,},
  ["ecv"] = {["ecv"]=true,},
  ["gloss_add"] = {["thesis-titlepage-fhac"]=true,},
  ["hyph-uk.t2a"] = {},
  ["lwarp-vertbars"] = {["lwarp"]=true,},
  ["aastex62"] = {["aastex"]=true,},
  ["figbib_add"] = {["thesis-titlepage-fhac"]=true,},
  ["fhACtitlepage"] = {["thesis-titlepage-fhac"]=true,},
  ["wasysym"] = {["wasysym"]=true,},
  ["thalie"] = {["thalie"]=true,},
  ["pgfplotsthemetol"] = {["beamertheme-metropolis"]=true,},
  ["thaienum"] = {["thaienum"]=true,},
  ["eskdchngsheet"] = {["eskdx"]=true,},
  ["tkz-arith"] = {["tkz-berge"]=true,},
  ["plprefix"] = {["polski"]=true,},
  ["tikzlibrarymindmap.code"] = {["pgf"]=true,},
  ["yathesis-demo"] = {["yathesis"]=true,},
  ["bewerbung-cv"] = {["bewerbung"]=true,},
  ["textualicomma"] = {["textualicomma"]=true,},
  ["textpos"] = {["textpos"]=true,},
  ["easyfig"] = {["easyfig"]=true,},
  ["eskdhash"] = {["eskdx"]=true,},
  ["eltex7"] = {["eltex"]=true,},
  ["totalcount"] = {["caption"]=true,},
  ["textopo"] = {["textopo"]=true,},
  ["xarbsymb"] = {["arabtex"]=true,},
  ["BOONDOX-ds"] = {["boondox"]=true,},
  ["biotex"] = {["textopo"]=true,},
  ["niceframe"] = {["niceframe"]=true,},
  ["nccbiblist"] = {["ncclatex"]=true,},
  ["textglos"] = {["textglos"]=true,},
  ["ltj-plain"] = {["luatexja"]=true,},
  ["gantt-s-mp"] = {["context-gantt"]=true,},
  ["xsim-manual"] = {["xsim"]=true,},
  ["beamerthemeCopenhagen"] = {["beamer"]=true,},
  ["simurgh-article"] = {["simurgh"]=true,},
  ["graphpap"] = {["latex"]=true,},
  ["TXStitle"] = {["texsis"]=true,},
  ["pgflibrarypgfplots.colortol.code"] = {["pgfplots"]=true,},
  ["lwarp-multitoc"] = {["lwarp"]=true,},
  ["TXStags"] = {["texsis"]=true,},
  ["supertabular"] = {["supertabular"]=true,},
  ["TXSsite"] = {["texsis"]=true,},
  ["nccboxes"] = {["ncctools"]=true,},
  ["TXSsects"] = {["texsis"]=true,},
  ["frabbrev"] = {["e-french"]=true,},
  ["font_cmbright"] = {["font-change"]=true,},
  ["TXSruled"] = {["texsis"]=true,},
  ["TXSprns"] = {["texsis"]=true,},
  ["schulekl"] = {["schule"]=true,},
  ["tikzlibrarydatavisualization.polar.code"] = {["pgf"]=true,},
  ["scrreprt"] = {},
  ["struktxp"] = {["struktex"]=true,},
  ["oldgerm"] = {["mfnfss"]=true,},
  ["awesomebox"] = {["awesomebox"]=true,},
  ["commath"] = {["commath"]=true,},
  ["csfontsm"] = {},
  ["babel-vai-latn"] = {["babel"]=true,},
  ["TXShead"] = {["texsis"]=true,},
  ["lwarp-vmargin"] = {["lwarp"]=true,},
  ["TXSfmts"] = {["texsis"]=true,},
  ["sltables"] = {["shipunov"]=true,},
  ["bryson"] = {["context"]=true,},
  ["TXSenvmt"] = {["texsis"]=true,},
  ["locality"] = {["locality"]=true,},
  ["dcounter"] = {["ncctools"]=true,},
  ["filehook"] = {["filehook"]=true,},
  ["relsize"] = {["relsize"]=true,},
  ["texshade"] = {["texshade"]=true,},
  ["tpsem-a4"] = {["texpower"]=true,},
  ["ptrace"] = {["platex"]=true,},
  ["tppstcol"] = {["texpower"]=true,},
  ["cclayout"] = {["computational-complexity"]=true,},
  ["coolthms"] = {["coolthms"]=true,},
  ["tplists"] = {["texpower"]=true,},
  ["datapie"] = {["datatool"]=true,},
  ["bpfir3"] = {["iso10303"]=true,},
  ["texpower"] = {["texpower"]=true,},
  ["tikzlibraryswitching-architectures.code"] = {["sa-tikz"]=true,},
  ["hebrew"] = {["babel"]=true,},
  ["pgfarrows"] = {["pgf"]=true,},
  ["toptesi-sss"] = {["toptesi"]=true,},
  ["texosquery"] = {["texosquery"]=true,},
  ["texments"] = {["texments"]=true,},
  ["texmate"] = {["texmate"]=true,},
  ["hhfixme"] = {["bgteubner"]=true,},
  ["lwarp-letterspace"] = {["lwarp"]=true,},
  ["txi-uk"] = {},
  ["babel-uppersorbian"] = {["babel"]=true,},
  ["lltjp-listings"] = {["luatexja"]=true,},
  ["tikzlibrarycalc.code"] = {["pgf"]=true,},
  ["gmparts"] = {["gmutils"]=true,},
  ["pgfcalendar"] = {["pgf"]=true,},
  ["txi-ru"] = {},
  ["apa6"] = {["apa6"]=true,},
  ["altxext"] = {["arabtex"]=true,},
  ["tagpdf-mc-code-lua"] = {["tagpdf"]=true,},
  ["progressbar"] = {["progressbar"]=true,},
  ["ltjsarticle"] = {["luatexja"]=true,},
  ["youngtab"] = {["youngtab"]=true,},
  ["txi-it"] = {},
  ["luatexbase-loader"] = {["luatexbase"]=true,},
  ["drawmatrix"] = {["drawmatrix"]=true,},
  ["bxjsslide"] = {["bxjscls"]=true,},
  ["gantt-s-tikz"] = {["context-gantt"]=true,},
  ["txi-es"] = {},
  ["directory"] = {["directory"]=true,},
  ["luabidi"] = {["luabidi"]=true,},
  ["comma"] = {["comma"]=true,},
  ["dawkins"] = {["context"]=true,},
  ["mptopdf"] = {["mptopdf"]=true,},
  ["doclicense-CC-by-3.0-latex"] = {["doclicense"]=true,},
  ["frame"] = {["frame"]=true,},
  ["aedpatch"] = {["arabtex"]=true,},
  ["txi-en"] = {},
  ["txi-de"] = {},
  ["captcont"] = {["captcont"]=true,},
  ["beamerinnerthemefibeamer-mu"] = {["fibeamer"]=true,},
  ["IMTEKda"] = {["imtekda"]=true,},
  ["musixext"] = {["musixtex"]=true,},
  ["txi-cs"] = {},
  ["texinfo"] = {},
  ["texilikechaps"] = {},
  ["hyph-id"] = {},
  ["txdtools"] = {["texdraw"]=true,},
  ["rotfloat"] = {["rotfloat"]=true,},
  ["tun"] = {["gustlib"]=true,},
  ["txdps"] = {["texdraw"]=true,},
  ["luatexja-preset"] = {["luatexja"]=true,},
  ["texdraw"] = {["texdraw"]=true,},
  ["bpfats2"] = {["iso10303"]=true,},
  ["ametsoc"] = {["ametsoc"]=true,},
  ["texdate"] = {["texdate"]=true,},
  ["tocbibind"] = {["tocbibind"]=true,},
  ["lwarp-a4"] = {["lwarp"]=true,},
  ["tikz-dimline"] = {["tikz-dimline"]=true,},
  ["arabrep"] = {["arabtex"]=true,},
  ["skmath"] = {["skmath"]=true,},
  ["pst-light3d"] = {["pst-light3d"]=true,},
  ["lwarp-diagbox"] = {["lwarp"]=true,},
  ["babel-newzealand"] = {["babel"]=true,},
  ["simurgh-glossaries"] = {["simurgh"]=true,},
  ["simurgh-unibidi"] = {["simurgh"]=true,},
  ["keyvaltable"] = {["keyvaltable"]=true,},
  ["babel-mundang"] = {["babel"]=true,},
  ["lwarp-memhfixc"] = {["lwarp"]=true,},
  ["refcheck"] = {["refcheck"]=true,},
  ["babel-finnish"] = {["babel"]=true,},
  ["tgschola"] = {["tex-gyre"]=true,},
  ["exsol"] = {["exsol"]=true,},
  ["ghsystem_explos-2"] = {["ghsystem"]=true,},
  ["gitinfo2"] = {["gitinfo2"]=true,},
  ["tgheros"] = {["tex-gyre"]=true,},
  ["tgcursor"] = {["tex-gyre"]=true,},
  ["otf"] = {["japanese-otf-uptex"]=true,},
  ["hangulcweb"] = {["kotex-plain"]=true,},
  ["tgchorus"] = {["tex-gyre"]=true,},
  ["avr.draw"] = {["avremu"]=true,},
  ["adfarrows"] = {["adfsymbols"]=true,},
  ["Kinigcap"] = {["cfr-initials"]=true,},
  ["laps"] = {},
  ["tgadventor"] = {["tex-gyre"]=true,},
  ["cyryovl"] = {},
  ["tikzpeople.shape.criminal"] = {["tikzpeople"]=true,},
  ["dialog"] = {["dialogl"]=true,},
  ["qcourier"] = {["tex-gyre"]=true,},
  ["HAPLakar"] = {["ha-prosper"]=true,},
  ["qbookman"] = {["tex-gyre"]=true,},
  ["dotnot"] = {["tex-ewd"]=true,},
  ["lh-lcyccr"] = {["lh"]=true,},
  ["soup"] = {["soup"]=true,},
  ["fontaxes"] = {["fontaxes"]=true,},
  ["babel-westernfrisian"] = {["babel"]=true,},
  ["cleanthesis"] = {["cleanthesis"]=true,},
  ["s-simpleslides-Swoosh"] = {["context-simpleslides"]=true,},
  ["chemmacros.module.polymers.code"] = {["chemmacros"]=true,},
  ["musixlit"] = {["musixtex"]=true,},
  ["tcblistings.code"] = {["tcolorbox"]=true,},
  ["base14flags"] = {["tetex"]=true,},
  ["beamerthemeXiaoshan"] = {["pgfornament-han"]=true,},
  ["lualatex-truncate"] = {["lualatex-truncate"]=true,},
  ["hyph-bn"] = {},
  ["intcalc"] = {["oberdiek"]=true,},
  ["bxpdfver"] = {["bxpdfver"]=true,},
  ["chemstyle"] = {["chemstyle"]=true,},
  ["tagpdf-tree-code"] = {["tagpdf"]=true,},
  ["zerohyph"] = {},
  ["termlist"] = {["termlist"]=true,},
  ["termcal-de"] = {["termcal-de"]=true,},
  ["dnaseq"] = {["dnaseq"]=true,},
  ["termcal"] = {["termcal"]=true,},
  ["substitutefont"] = {["substitutefont"]=true,},
  ["fontloader-fonts"] = {["luaotfload"]=true,},
  ["anonchap"] = {["anonchap"]=true,},
  ["pgf-umlsd"] = {["pgf-umlsd"]=true,},
  ["diagmac2"] = {["diagmac2"]=true,},
  ["pst-qtree"] = {["pst-qtree"]=true,},
  ["loadhyph-ro"] = {},
  ["templatetools"] = {["templatetools"]=true,},
  ["ltxtools-base"] = {["ltxtools"]=true,},
  ["prnthyph"] = {["cbcoptic"]=true,},
  ["technics"] = {["technics"]=true,},
  ["tdsfrmath"] = {["tdsfrmath"]=true,},
  ["ocg"] = {["asymptote"]=true,},
  ["tcolorbox"] = {["tcolorbox"]=true,},
  ["minutes"] = {["minutes"]=true,},
  ["tcbvignette.code"] = {["tcolorbox"]=true,},
  ["t1macro"] = {["enctex"]=true,},
  ["edtable"] = {["lineno"]=true,},
  ["fonttabs"] = {},
  ["gmmw"] = {["gmutils"]=true,},
  ["tcbtheorems.code"] = {["tcolorbox"]=true,},
  ["beamercolorthemetorinoth"] = {["beamer2thesis"]=true,},
  ["s-faq-03"] = {["context"]=true,},
  ["jurarsp"] = {["jurarsp"]=true,},
  ["samin"] = {["babel"]=true,},
  ["desclist"] = {["ncctools"]=true,},
  ["tcbskins.code"] = {["tcolorbox"]=true,},
  ["beamercolorthemefibeamer-mu-fsps"] = {["fibeamer"]=true,},
  ["skeyval-pstkey"] = {["skeyval"]=true,},
  ["dccpaper-base"] = {["dccpaper"]=true,},
  ["cyrcmfnt"] = {},
  ["tcbprocessing.code"] = {["tcolorbox"]=true,},
  ["esdiff"] = {["esdiff"]=true,},
  ["tcbposter.code"] = {["tcolorbox"]=true,},
  ["babel-ukrainian"] = {["babel"]=true,},
  ["tcbmagazine.code"] = {["tcolorbox"]=true,},
  ["akkstring"] = {["akktex"]=true,},
  ["collectbox"] = {["collectbox"]=true,},
  ["common-interleafpage"] = {},
  ["lwarp-afterpage"] = {["lwarp"]=true,},
  ["iso88596"] = {["arabtex"]=true,},
  ["beamercolorthemeseagull"] = {["beamer"]=true,},
  ["hyph-hu.ec"] = {["hyphen-hungarian"]=true,},
  ["bibleref-german"] = {["bibleref-german"]=true,},
  ["tcblistingscore.code"] = {["tcolorbox"]=true,},
  ["bpfap12"] = {["iso10303"]=true,},
  ["ppr-prv"] = {["ppr-prv"]=true,},
  ["tcbfitting.code"] = {["tcolorbox"]=true,},
  ["grant-darpa"] = {["grant"]=true,},
  ["pgfmanual.code"] = {["pgf"]=true,["tikz-dependency"]=true,},
  ["macro-mars"] = {["pas-cv"]=true,},
  ["testDidot"] = {["gfsdidot"]=true,},
  ["tcldoc"] = {["tcldoc"]=true,},
  ["hgblistings"] = {["hagenberg-thesis"]=true,},
  ["chemmacros.module.thermodynamics.code"] = {["chemmacros"]=true,},
  ["naustrian"] = {["babel"]=true,},
  ["viking"] = {["archaic"]=true,},
  ["simurgh-algorithmic"] = {["simurgh"]=true,},
  ["lt3graph-dry"] = {["lt3graph"]=true,},
  ["academicons"] = {["academicons"]=true,},
  ["beamerthemeBoadilla"] = {["beamer"]=true,},
  ["chemmacros.module.symbols.code"] = {["chemmacros"]=true,},
  ["plainenc"] = {},
  ["pgflibraryshapes.gates.ee.IEC.code"] = {["pgf"]=true,},
  ["tameflts"] = {["tamefloats"]=true,},
  ["gettitlestring"] = {["oberdiek"]=true,},
  ["talk"] = {["talk"]=true,},
  ["maybemath"] = {["maybemath"]=true,},
  ["sidebars"] = {["talk"]=true,},
  ["lastpage209"] = {["lastpage"]=true,},
  ["lwarp-dblfloatfix"] = {["lwarp"]=true,},
  ["852-csf"] = {["enctex"]=true,},
  ["tagpdf-user"] = {["tagpdf"]=true,},
  ["pgfautomata"] = {["pgf"]=true,},
  ["bnumexpr"] = {["bnumexpr"]=true,},
  ["adfbullets"] = {["adfsymbols"]=true,},
  ["termmenu"] = {["termmenu"]=true,},
  ["nag"] = {["nag"]=true,},
  ["ghsystem_explos-6"] = {["ghsystem"]=true,},
  ["tagpdf-roles-code"] = {["tagpdf"]=true,},
  ["beamerthemeIlmenau"] = {["beamer"]=true,},
  ["hycolor"] = {["oberdiek"]=true,},
  ["keyval"] = {["xkeyval"]=true,["graphics"]=true,},
  ["tagpdf-mc-code-shared"] = {["tagpdf"]=true,},
  ["algolrevived"] = {["algolrevived"]=true,},
  ["txi-pt"] = {},
  ["bahasam"] = {["babel"]=true,},
  ["magyar"] = {["babel"]=true,},
  ["colorinfo"] = {["colorinfo"]=true,},
  ["phn-AT_Vorwahlen"] = {["phonenumbers"]=true,},
  ["t-chromato"] = {["context-chromato"]=true,},
  ["beamerouterthemeDetlevCM"] = {["beamertheme-detlevcm"]=true,},
  ["powerdot-husky"] = {["powerdot"]=true,},
  ["css-colors"] = {["css-colors"]=true,},
  ["xgreek"] = {["xgreek"]=true,},
  ["supp-pdf"] = {["context"]=true,},
  ["tagging"] = {["tagging"]=true,},
  ["tabvar"] = {["tabvar"]=true,},
  ["beamercolorthemedove"] = {["beamer"]=true,},
  ["btxdockit"] = {["ltxdockit"]=true,},
  ["font_bookman"] = {["font-change"]=true,},
  ["ydoc"] = {["ydoc"]=true,},
  ["arabmaths"] = {["luabidi"]=true,},
  ["koma-moderncvclassic"] = {["koma-moderncvclassic"]=true,},
  ["phn-FR_Vorwahlen"] = {["phonenumbers"]=true,},
  ["pgf"] = {["pgf"]=true,},
  ["babel-bengali"] = {["babel"]=true,},
  ["musixhor"] = {["musixtex"]=true,},
  ["tabu"] = {["tabu"]=true,},
  ["allauncl"] = {["auncial-new"]=true,},
  ["tabstackengine"] = {["tabstackengine"]=true,},
  ["lwarp-bibunits"] = {["lwarp"]=true,},
  ["font_kurier"] = {["font-change"]=true,},
  ["babel-rombo"] = {["babel"]=true,},
  ["tabls"] = {["tabls"]=true,},
  ["tablor"] = {["tablor"]=true,},
  ["tablor-xetex"] = {["tablor"]=true,},
  ["xsim.blanks.code"] = {["xsim"]=true,},
  ["js-misc"] = {["js-misc"]=true,},
  ["beamercolorthemebeetle"] = {["beamer"]=true,},
  ["tablefootnote"] = {["tablefootnote"]=true,},
  ["beamerouterthemetorinoth"] = {["beamer2thesis"]=true,},
  ["afparticle"] = {["afparticle"]=true,},
  ["tableau"] = {["tableaux"]=true,},
  ["pgfcirclabel"] = {["circuitikz"]=true,},
  ["pdfcrypt"] = {["oberdiek"]=true,},
  ["lwarp-fixme"] = {["lwarp"]=true,},
  ["table-fct"] = {["table-fct"]=true,},
  ["tabfigures"] = {["tabfigures"]=true,},
  ["citehack"] = {["t2"]=true,},
  ["envbig"] = {["envbig"]=true,},
  ["comment"] = {["comment"]=true,},
  ["dhucs-gremph"] = {["kotex-utf"]=true,},
  ["luamplib"] = {["luamplib"]=true,},
  ["fnstcorr"] = {["t2"]=true,},
  ["refart"] = {["refman"]=true,},
  ["alias-wncy"] = {["t2"]=true,},
  ["alias-cmc"] = {["t2"]=true,},
  ["chemmacros.module.scheme.code"] = {["chemmacros"]=true,},
  ["flipbook"] = {["flipbook"]=true,},
  ["moderncvstylefancy"] = {["moderncv"]=true,},
  ["beamercolorthemewhale"] = {["beamer"]=true,},
  ["hobby.code"] = {["hobby"]=true,},
  ["tikzpeople.shape.police"] = {["tikzpeople"]=true,},
  ["okumacro"] = {["jsclasses"]=true,},
  ["fouridx"] = {["fouridx"]=true,},
  ["margbib"] = {["margbib"]=true,},
  ["cmsrb"] = {["cmsrb"]=true,},
  ["achicago"] = {["frankenstein"]=true,},
  ["cyryozn"] = {},
  ["hefonts1"] = {["arabtex"]=true,},
  ["syllogism"] = {["syllogism"]=true,},
  ["swimgraf"] = {["swimgraf"]=true,},
  ["dramatist"] = {["dramatist"]=true,},
  ["babel-georgian"] = {["babel"]=true,},
  ["beamerouterthemedefault"] = {["beamer"]=true,},
  ["rccol"] = {["rccol"]=true,},
  ["pst-laboObj"] = {["pst-labo"]=true,},
  ["svn-prov"] = {["svn-prov"]=true,},
  ["beamerthemeHannover"] = {["beamer"]=true,},
  ["svn-multi"] = {["svn-multi"]=true,},
  ["boolexpr"] = {["boolexpr"]=true,},
  ["font_kurier-medium"] = {["font-change"]=true,},
  ["svn"] = {["svn"]=true,},
  ["lwarp-imakeidx"] = {["lwarp"]=true,},
  ["svgcolor"] = {["svgcolor"]=true,},
  ["tex-label"] = {["tex-label"]=true,},
  ["cervantes-es"] = {["context"]=true,},
  ["punct"] = {["shipunov"]=true,},
  ["denisbdoc"] = {["denisbdoc"]=true,},
  ["europecv"] = {["europecv"]=true,},
  ["beamercolorthemeETII"] = {["beamertheme-detlevcm"]=true,},
  ["susy"] = {["susy"]=true,},
  ["biblatex-archaeology"] = {["biblatex-archaeology"]=true,},
  ["TXSsymb"] = {["texsis"]=true,},
  ["tikzlibraryangles.code"] = {["pgf"]=true,},
  ["superiors"] = {["superiors"]=true,},
  ["txgreeks"] = {["txgreeks"]=true,},
  ["powerdot-tuliplab"] = {["powerdot-tuliplab"]=true,},
  ["printsudoku"] = {["sudokubundle"]=true,},
  ["createsudoku"] = {["sudokubundle"]=true,},
  ["lhcyrwin-rhyphen"] = {},
  ["substances"] = {["substances"]=true,},
  ["subfigmat"] = {["subfigmat"]=true,},
  ["subfig"] = {["subfig"]=true,},
  ["beamerthemeplain"] = {["beamer"]=true,},
  ["askincv1"] = {["iso"]=true,},
  ["subeqnarray"] = {["subeqnarray"]=true,},
  ["imfellEnglish"] = {["imfellenglish"]=true,},
  ["gbt7714"] = {["gbt7714"]=true,},
  ["currfile-abspath"] = {["currfile"]=true,},
  ["subeqn"] = {["subeqn"]=true,},
  ["subdepth"] = {["subdepth"]=true,},
  ["easyformat"] = {["easyformat"]=true,},
  ["euscript"] = {["amsfonts"]=true,},
  ["studenthandouts"] = {["studenthandouts"]=true,},
  ["labels"] = {["midnight"]=true,["labels"]=true,},
  ["datagidx"] = {["datatool"]=true,},
  ["babel-makhuwameetto"] = {["babel"]=true,},
  ["texsort"] = {["sttools"]=true,},
  ["pgflibraryarrows.meta.code"] = {["pgf"]=true,},
  ["babel-langi"] = {["babel"]=true,},
  ["problem"] = {["stex"]=true,},
  ["stabular"] = {["sttools"]=true,},
  ["pgfplots.errorbars.code"] = {["pgfplots"]=true,},
  ["lwarp-ifoddpage"] = {["lwarp"]=true,},
  ["epltxchapno"] = {["expex"]=true,},
  ["midfloat"] = {["sttools"]=true,},
  ["loadhyph-ia"] = {},
  ["pdfscreen"] = {["pdfscreen"]=true,},
  ["floatpag"] = {["sttools"]=true,},
  ["musixdia"] = {["musixtex"]=true,},
  ["TXSinit"] = {["texsis"]=true,},
  ["parnotes"] = {["parnotes"]=true,},
  ["bpfap16"] = {["iso10303"]=true,},
  ["lips"] = {["frankenstein"]=true,},
  ["babeltools"] = {["bezos"]=true,},
  ["ecsubzcm"] = {["relenc"]=true,},
  ["struktex"] = {["struktex"]=true,},
  ["tvz-user"] = {["seminar"]=true,},
  ["lwarp-algorithmicx"] = {["lwarp"]=true,},
  ["font_antt-light"] = {["font-change"]=true,},
  ["strukdoc"] = {["struktex"]=true,},
  ["eemeir"] = {["eemeir"]=true,},
  ["simurgh-fleqn"] = {["simurgh"]=true,},
  ["showtags"] = {["showtags"]=true,},
  ["storecmd"] = {["storecmd"]=true,},
  ["epsf"] = {["epsf"]=true,},
  ["pgflibrarydecorations.fractals.code"] = {["pgf"]=true,},
  ["texnansi"] = {["ly1"]=true,},
  ["proba"] = {["proba"]=true,},
  ["hyplain"] = {["hyplain"]=true,},
  ["biocon"] = {["biocon"]=true,},
  ["beamerinnerthemeinmargin"] = {["beamer"]=true,},
  ["tikz-timing-overlays"] = {["tikz-timing"]=true,},
  ["spot"] = {["spot"]=true,},
  ["pgfnodes"] = {["pgf"]=true,},
  ["t-pgfsys"] = {["pgf"]=true,},
  ["mlp-49"] = {["e-french"]=true,},
  ["stmaryrd"] = {["stmaryrd"]=true,},
  ["indextools"] = {["indextools"]=true,},
  ["stix"] = {["stix"]=true,},
  ["stickstootext"] = {["stickstoo"]=true,},
  ["pgfmoduleplot.code"] = {["pgf"]=true,},
  ["t-navigator"] = {["navigator"]=true,},
  ["mVersion"] = {["mversion"]=true,},
  ["tikzinput"] = {["stex"]=true,},
  ["structview"] = {["stex"]=true,},
  ["mhsetup"] = {["mathtools"]=true,},
  ["statements"] = {["stex"]=true,},
  ["pgfcoreshade.code"] = {["pgf"]=true,},
  ["lollipop-fontdefs"] = {["lollipop"]=true,},
  ["bibl-apa-it"] = {["context"]=true,},
  ["babel-malay-brunei"] = {["babel"]=true,},
  ["flowfram"] = {["flowfram"]=true,},
  ["revtex4"] = {["revtex4"]=true,},
  ["pst-gr3d"] = {["pst-gr3d"]=true,},
  ["regstats"] = {["regstats"]=true,},
  ["lwarp-fnlineno"] = {["lwarp"]=true,},
  ["eqnnumwarn"] = {["eqnnumwarn"]=true,},
  ["biblatex"] = {["biblatex"]=true,},
  ["beamerthemenirma"] = {["beamerthemenirma"]=true,},
  ["bewerbung"] = {["bewerbung"]=true,},
  ["musixhou"] = {["musixtex"]=true,},
  ["endfloat"] = {["endfloat"]=true,},
  ["ascii"] = {["ascii-font"]=true,},
  ["presentation"] = {["stex"]=true,},
  ["eco"] = {["eco"]=true,},
  ["hyphsubst"] = {["oberdiek"]=true,},
  ["omtext"] = {["stex"]=true,},
  ["hanging"] = {["hanging"]=true,},
  ["common-headfootheight"] = {},
  ["pnversion"] = {["petri-nets"]=true,},
  ["beamerthemeJacobs"] = {["stex"]=true,},
  ["hwexam"] = {["stex"]=true,},
  ["pgfmathfunctions.comparison.code"] = {["pgf"]=true,},
  ["baskervillef"] = {["baskervillef"]=true,},
  ["fr-fancy"] = {["floatrow"]=true,},
  ["babel-lingala"] = {["babel"]=true,},
  ["cje"] = {["cje"]=true,["economic"]=true,},
  ["babel-akan"] = {["babel"]=true,},
  ["bletter"] = {["bangtex"]=true,},
  ["lwarp-authblk"] = {["lwarp"]=true,},
  ["ussummary"] = {["stellenbosch"]=true,},
  ["usnomencl"] = {["stellenbosch"]=true,},
  ["codedoc"] = {["codedoc"]=true,},
  ["lwarp-tocstyle"] = {["lwarp"]=true,},
  ["hletter"] = {["hletter"]=true,},
  ["aurical"] = {["aurical"]=true,},
  ["babel-morisyen"] = {["babel"]=true,},
  ["babel-manx"] = {["babel"]=true,},
  ["beamerthemefocus"] = {["beamertheme-focus"]=true,},
  ["babel-japanese"] = {["babel"]=true,},
  ["babel-afrikaans"] = {["babel"]=true,},
  ["fontawesomesymbols-pdftex"] = {["fontawesome"]=true,},
  ["lstlinebgrd"] = {["lstaddons"]=true,},
  ["lwarp-figcaps"] = {["lwarp"]=true,},
  ["bxbase"] = {["bxbase"]=true,},
  ["tikzmarmots"] = {["tikzmarmots"]=true,},
  ["fancynum"] = {["fancynum"]=true,},
  ["navigator"] = {["navigator"]=true,},
  ["statistik"] = {["statistik"]=true,},
  ["fiche"] = {["mafr"]=true,},
  ["mathdesign"] = {["mathdesign"]=true,},
  ["statex2"] = {["statex2"]=true,},
  ["isopt"] = {["isopt"]=true,},
  ["statex"] = {["statex"]=true,},
  ["classicthesis-arsclassica"] = {["classicthesis"]=true,},
  ["stanli"] = {["stanli"]=true,},
  ["standalone"] = {["standalone"]=true,},
  ["chronosyschr"] = {["chronosys"]=true,},
  ["lastpage"] = {["lastpage"]=true,},
  ["ncchdr"] = {["ncclatex"]=true,},
  ["stack"] = {},
  ["glossary-tree"] = {["glossaries"]=true,},
  ["relinput"] = {},
  ["sseq"] = {["sseq"]=true,},
  ["gtrcore.symbols.code"] = {["genealogytree"]=true,},
  ["extramarks"] = {["fancyhdr"]=true,},
  ["srcltx"] = {["srcltx"]=true,},
  ["tikzlibrarypgfplots.decorations.softclip.code"] = {["pgfplots"]=true,},
  ["norsk"] = {["babel"]=true,},
  ["farsical"] = {["polyglossia"]=true,},
  ["spverbatim"] = {["spverbatim"]=true,},
  ["eulerpx"] = {["eulerpx"]=true,},
  ["scrsource"] = {},
  ["spreadtab"] = {["spreadtab"]=true,},
  ["babel-hausa-niger"] = {["babel"]=true,},
  ["adjcalc"] = {["adjustbox"]=true,},
  ["infix-RPN"] = {["pst-infixplot"]=true,},
  ["villesItalia3d"] = {["pst-geo"]=true,},
  ["splitidx"] = {["splitindex"]=true,},
  ["cncent"] = {},
  ["lroundrect"] = {["lroundrect"]=true,},
  ["splitindex"] = {["splitindex"]=true,},
  ["splitbib"] = {["splitbib"]=true,},
  ["lwarp-natbib"] = {["lwarp"]=true,},
  ["zxbase"] = {["bxbase"]=true,},
  ["pst-fun"] = {["pst-fun"]=true,},
  ["sphack"] = {["sphack"]=true,},
  ["s-pre-06"] = {["context"]=true,},
  ["babel-serbian-cyrl-ba"] = {["babel"]=true,},
  ["fithesis-ped"] = {["fithesis"]=true,},
  ["shadowtext"] = {["shadowtext"]=true,},
  ["spelling"] = {["spelling"]=true,},
  ["pst-knot"] = {["pst-knot"]=true,},
  ["sseqmain.code"] = {["spectralsequences"]=true,},
  ["fp-exp"] = {["fp"]=true,},
  ["footnotehyper"] = {["footnotehyper"]=true,},
  ["revsymb4-1"] = {["revtex"]=true,},
  ["chapterfolder"] = {["chapterfolder"]=true,},
  ["pst-char"] = {["pst-text"]=true,},
  ["sseqkeys.code"] = {["spectralsequences"]=true,},
  ["bidipoem"] = {["bidi"]=true,},
  ["gradientframe"] = {["gradientframe"]=true,},
  ["pxbase"] = {["pxbase"]=true,},
  ["darwin"] = {["context"]=true,},
  ["font_concrete"] = {["font-change"]=true,},
  ["letterspace"] = {["microtype"]=true,},
  ["babel-masai"] = {["babel"]=true,},
  ["typed-checklist"] = {["typed-checklist"]=true,},
  ["sseqcheckdefinitions.code"] = {["spectralsequences"]=true,},
  ["lwarp-xfrac"] = {["lwarp"]=true,},
  ["tkz-obj-polygons"] = {["tkz-euclide"]=true,},
  ["ftnxtra"] = {["ftnxtra"]=true,},
  ["akkdoc"] = {["akktex"]=true,},
  ["tikzlibrarycalligraphy.code"] = {["spath3"]=true,},
  ["bpfap9"] = {["iso10303"]=true,},
  ["spath3"] = {["spath3"]=true,},
  ["sparklines"] = {["sparklines"]=true,},
  ["oblivoir"] = {["kotex-oblivoir"]=true,},
  ["biblatex-anonymous"] = {["biblatex-anonymous"]=true,},
  ["loadhyph-sv"] = {},
  ["xevlna"] = {["xevlna"]=true,},
  ["apacite"] = {["apacite"]=true,},
  ["pst-tree"] = {["pst-tree"]=true,},
  ["spanishmx"] = {["spanish-mx"]=true,},
  ["pst-solides3d"] = {["pst-solides3d"]=true,},
  ["mixcodes"] = {["enctex"]=true,},
  ["eskdpara"] = {["eskdx"]=true,},
  ["babel-malay"] = {["babel"]=true,},
  ["lwarp-everyshi"] = {["lwarp"]=true,},
  ["ghsystem_flame-4-3-black"] = {["ghsystem"]=true,},
  ["tikzducks"] = {["tikzducks"]=true,},
  ["sourcecodepro"] = {["sourcecodepro"]=true,},
  ["babel-vietnam"] = {["babel"]=true,},
  ["adigraph"] = {["adigraph"]=true,},
  ["constants"] = {["constants"]=true,},
  ["bgreek"] = {["bgreek"]=true,},
  ["t-syntax-groups"] = {["context-vim"]=true,},
  ["soton-palette"] = {["soton"]=true,},
  ["babel-portuguese-portugal"] = {["babel"]=true,},
  ["songs"] = {["songs"]=true,},
  ["csvsimple"] = {["csvsimple"]=true,},
  ["hepthesis"] = {["hepthesis"]=true,},
  ["babel-arabic-syria"] = {["babel"]=true,},
  ["songbook"] = {["songbook"]=true,},
  ["active-conf"] = {["active-conf"]=true,},
  ["interfaces-tocloft"] = {["interfaces"]=true,},
  ["gtrlib.debug.code"] = {["genealogytree"]=true,},
  ["cheatsheet"] = {["cheatsheet"]=true,},
  ["rubikrotation"] = {["rubik"]=true,},
  ["conditionals"] = {["songbook"]=true,},
  ["xfig"] = {["js-misc"]=true,},
  ["everysel"] = {["ms"]=true,},
  ["snapshot"] = {["snapshot"]=true,},
  ["smartref"] = {["smartref"]=true,},
  ["eiad"] = {["eiad-ltx"]=true,},
  ["byname"] = {["smartref"]=true,},
  ["ghsystem_bottle-2-black"] = {["ghsystem"]=true,},
  ["lwarp-fontspec"] = {["lwarp"]=true,},
  ["pgfmath"] = {["pgf"]=true,},
  ["bezierplot"] = {["bezierplot"]=true,},
  ["smartdiagramlibrarycore.styles.code"] = {["smartdiagram"]=true,},
  ["sesamanuel"] = {["sesamanuel"]=true,},
  ["smartdiagramlibrarycore.definitions.code"] = {["smartdiagram"]=true,},
  ["smartdiagramlibraryadditions.code"] = {["smartdiagram"]=true,},
  ["beamerthemeJuanLesPins"] = {["beamer"]=true,},
  ["t-chemfig"] = {["chemfig"]=true,},
  ["eskdcap"] = {["eskdx"]=true,},
  ["uantwerpenphdthesis"] = {["uantwerpendocs"]=true,},
  ["slantsc"] = {["slantsc"]=true,},
  ["colourchange"] = {["colourchange"]=true,},
  ["skull"] = {},
  ["skrapport-size-common"] = {["skrapport"]=true,},
  ["skrapport-colortheme-violet"] = {["skrapport"]=true,},
  ["skrapport-colortheme-unscathed"] = {["skrapport"]=true,},
  ["skrapport-colortheme-skdoc"] = {["skrapport"]=true,},
  ["cv4tw-theme-sharp"] = {["cv4tw"]=true,},
  ["skrapport-colortheme-default"] = {["skrapport"]=true,},
  ["skrapport-colortheme-cruelwater"] = {["skrapport"]=true,},
  ["luatex-math"] = {["context"]=true,},
  ["tex4ebook"] = {["tex4ebook"]=true,},
  ["georgian"] = {["babel-georgian"]=true,},
  ["graph35"] = {["graph35"]=true,},
  ["beamercolorthemecormorant"] = {["beamerdarkthemes"]=true,},
  ["skeyval"] = {["skeyval"]=true,},
  ["hyph-nn"] = {},
  ["bangfont"] = {["bangtex"]=true,},
  ["tx-of"] = {["pxtxalfa"]=true,},
  ["sapthesis"] = {["sapthesis"]=true,},
  ["Biochemistry-colors"] = {["biochemistry-colors"]=true,},
  ["skeyval-testpkg"] = {["skeyval"]=true,},
  ["grcodes"] = {},
  ["bpfir1"] = {["iso10303"]=true,},
  ["pgfplotsoldpgfsupp_tikzlibraryexternal.code"] = {["pgfplots"]=true,},
  ["skeyval-ltxcmds"] = {["skeyval"]=true,},
  ["skeyval-for"] = {["skeyval"]=true,},
  ["schemabloc"] = {["schemabloc"]=true,},
  ["skeyval-bc"] = {["skeyval"]=true,},
  ["codep"] = {["examplep"]=true,},
  ["skeycommand"] = {["skeycommand"]=true,},
  ["babel-uzbek-latn"] = {["babel"]=true,},
  ["jsbook"] = {["jsclasses"]=true,},
  ["skbmoderncv"] = {["skb"]=true,},
  ["moderncvcolorred"] = {["moderncv"]=true,},
  ["mtx-context-listing"] = {["context"]=true,},
  ["rutitlepage"] = {["rutitlepage"]=true,},
  ["flexisym"] = {["breqn"]=true,},
  ["ctexhook"] = {["ctex"]=true,},
  ["skblncsbeamer"] = {["skb"]=true,},
  ["eltex1"] = {["eltex"]=true,},
  ["hebsymb"] = {["arabtex"]=true,},
  ["skbbeamer"] = {["skb"]=true,},
  ["beamercolorthemefrigatebird"] = {["beamerdarkthemes"]=true,},
  ["couriers"] = {["courier-scaled"]=true,},
  ["fontch_doc"] = {["fontch"]=true,},
  ["OldStandard"] = {["oldstandard"]=true,},
  ["datetime2-calc"] = {["datetime2"]=true,},
  ["siunitx"] = {["siunitx"]=true,},
  ["libertine"] = {["libertine"]=true,},
  ["delimseasy"] = {["delimseasy"]=true,},
  ["simurgh"] = {["simurgh"]=true,},
  ["babel-nama"] = {["babel"]=true,},
  ["epsfx"] = {["tex-ps"]=true,},
  ["simurgh-tartibi"] = {["simurgh"]=true,},
  ["gaceta"] = {["gaceta"]=true,},
  ["HAPsimple"] = {["ha-prosper"]=true,},
  ["pgflibraryshapes.callouts.code"] = {["pgf"]=true,},
  ["dataref"] = {["dataref"]=true,},
  ["varioref-vi"] = {["vntex"]=true,},
  ["simurgh-scrreprt"] = {["simurgh"]=true,},
  ["lt3graph-packagedoc"] = {["lt3graph"]=true,},
  ["simurgh-scrlettr"] = {["simurgh"]=true,},
  ["simurgh-scrbook"] = {["simurgh"]=true,},
  ["ctexrep"] = {["ctex"]=true,},
  ["sfg"] = {["sfg"]=true,},
  ["texdepends"] = {["latex-make"]=true,},
  ["xskak-keys"] = {["xskak"]=true,},
  ["glossaries-babel"] = {["glossaries"]=true,},
  ["simurgh-rapport3"] = {["simurgh"]=true,},
  ["simurgh-rapport1"] = {["simurgh"]=true,},
  ["simurgh-poem"] = {["simurgh"]=true,},
  ["simurgh-pkgs"] = {["simurgh"]=true,},
  ["assoccnt"] = {["assoccnt"]=true,},
  ["simurgh-natbib"] = {["simurgh"]=true,},
  ["elbioimp"] = {["elbioimp"]=true,},
  ["VCPref-mystyle"] = {["vaucanson-g"]=true,},
  ["toptesi-monografia"] = {["toptesi"]=true,},
  ["nccstretch"] = {["ncctools"]=true,},
  ["simurgh-mathdigitspec"] = {["simurgh"]=true,},
  ["beamerbasecompatibility"] = {["beamer"]=true,},
  ["luatex-fonts"] = {["context"]=true,},
  ["doclicense-CC-by-nc-3.0-latex"] = {["doclicense"]=true,},
  ["simurgh-ltx"] = {["simurgh"]=true,},
  ["doclicense-CC-by-nd-3.0-latex"] = {["doclicense"]=true,},
  ["pst-spinner"] = {["pst-spinner"]=true,},
  ["simurgh-lettrine"] = {["simurgh"]=true,},
  ["quotation"] = {},
  ["simurgh-letter"] = {["simurgh"]=true,},
  ["chemmacros"] = {["chemmacros"]=true,},
  ["babel-bafia"] = {["babel"]=true,},
  ["albanian"] = {["babel"]=true,},
  ["simurgh-ftnxtra"] = {["simurgh"]=true,},
  ["beamercolorthememetropolis-highcontrast"] = {["beamertheme-metropolis"]=true,},
  ["zref-env"] = {["oberdiek"]=true,},
  ["bizcard"] = {["bizcard"]=true,},
  ["hyph-la-x-liturgic"] = {},
  ["simurgh-fonts"] = {["simurgh"]=true,},
  ["macroswap"] = {["macroswap"]=true,},
  ["simurgh-extreport"] = {["simurgh"]=true,},
  ["simurgh-extletter"] = {["simurgh"]=true,},
  ["hepatch"] = {["arabtex"]=true,},
  ["simurgh-extarticle"] = {["simurgh"]=true,},
  ["simurgh-empheq"] = {["simurgh"]=true,},
  ["simurgh-doc"] = {["simurgh"]=true,},
  ["topsection"] = {["ncctools"]=true,},
  ["lwarp-cuted"] = {["lwarp"]=true,},
  ["simurgh-caption3"] = {["simurgh"]=true,},
  ["chemmacros.module.units.code"] = {["chemmacros"]=true,},
  ["avr.binary"] = {["avremu"]=true,},
  ["babel-chinese-simplified-singapore"] = {["babel"]=true,},
  ["s-pre-93"] = {["context"]=true,},
  ["polski"] = {["polski"]=true,},
  ["simurgh-bookest"] = {["simurgh"]=true,},
  ["styledef"] = {["midnight"]=true,},
  ["portuges"] = {["babel"]=true,},
  ["h2020proposal"] = {["h2020proposal"]=true,},
  ["font_iwona-condensed-bold"] = {["font-change"]=true,},
  ["hecmd"] = {["arabtex"]=true,},
  ["simurgh-boek"] = {["simurgh"]=true,},
  ["simurgh-bidi"] = {["simurgh"]=true,},
  ["simurgh-backref"] = {["simurgh"]=true,},
  ["simurgh-arydshln"] = {["simurgh"]=true,},
  ["simurgh-artikel3"] = {["simurgh"]=true,},
  ["diagbox"] = {["diagbox"]=true,},
  ["mdwmath"] = {["mdwtools"]=true,},
  ["paravesp"] = {["parades"]=true,},
  ["lwarp-clrdblpg"] = {["lwarp"]=true,},
  ["simurgh-array"] = {["simurgh"]=true,},
  ["simurgh-amsthm"] = {["simurgh"]=true,},
  ["luacolor"] = {["oberdiek"]=true,},
  ["map"] = {["gustlib"]=true,},
  ["beamerbasenotes"] = {["beamer"]=true,},
  ["simurgh-amsmath"] = {["simurgh"]=true,},
  ["pgfplotsmeshplotimage.code"] = {["pgfplots"]=true,},
  ["memucs-setspace"] = {["kotex-oblivoir"]=true,},
  ["calendrierfpmodified"] = {["arabi"]=true,},
  ["simurgh-amsbook"] = {["simurgh"]=true,},
  ["tap"] = {},
  ["extbook"] = {["extsizes"]=true,},
  ["bookmark"] = {["oberdiek"]=true,},
  ["fontfam-global"] = {},
  ["ctibmantra"] = {["pecha"]=true,},
  ["simurgh-adadi"] = {["simurgh"]=true,},
  ["simurgh-abjad"] = {["simurgh"]=true,},
  ["simpler-wick"] = {["simpler-wick"]=true,},
  ["simplekv"] = {["simplekv"]=true,},
  ["tlinst-BT08"] = {},
  ["simpleinvoice"] = {["simpleinvoice"]=true,},
  ["simplecd"] = {["simplecd"]=true,},
  ["simplethesisdissertation"] = {["simple-thesis-dissertation"]=true,},
  ["simpleresumecv"] = {["simple-resume-cv"]=true,},
  ["sides"] = {["sides"]=true,},
  ["cap"] = {["c-pascal"]=true,},
  ["sidenotes"] = {["sidenotes"]=true,},
  ["babel-standardmoroccantamazight"] = {["babel"]=true,},
  ["exchars"] = {},
  ["scrlfile"] = {},
  ["tikzmarklibrarylistings.code"] = {["tikzmark"]=true,},
  ["gregoriotex-common"] = {["gregoriotex"]=true,},
  ["showexpl"] = {["showexpl"]=true,},
  ["showdim"] = {["showdim"]=true,},
  ["lisp-util"] = {["lisp-on-tex"]=true,},
  ["lltjp-geometry"] = {["luatexja"]=true,},
  ["starfn"] = {["shipunov"]=true,},
  ["TXSeqns"] = {["texsis"]=true,},
  ["grmhyph"] = {},
  ["ltxnew"] = {["ltxnew"]=true,},
  ["svg-extract"] = {["svg"]=true,},
  ["qcircuit"] = {["qcircuit"]=true,},
  ["bpfats9"] = {["iso10303"]=true,},
  ["isyntax"] = {["shipunov"]=true,},
  ["hyph-hy"] = {},
  ["sqrcaps"] = {["bookhands"]=true,},
  ["lwarp-textarea"] = {["lwarp"]=true,},
  ["etiketka"] = {["shipunov"]=true,},
  ["cjkutf8-ko"] = {["cjk-ko"]=true,},
  ["abidir"] = {["arabtex"]=true,},
  ["babel-tongan"] = {["babel"]=true,},
  ["classif2"] = {["shipunov"]=true,},
  ["cassete"] = {["shipunov"]=true,},
  ["lwarp-index"] = {["lwarp"]=true,},
  ["greekctr"] = {["jknapltx"]=true,},
  ["s-simpleslides-SideToc"] = {["context-simpleslides"]=true,},
  ["geschichtsfrkldoc"] = {["geschichtsfrkl"]=true,},
  ["pgfplotsoldpgfsupp_pgfmanual.prettyprinter.code"] = {["pgfplots"]=true,},
  ["biokey"] = {["shipunov"]=true,},
  ["ofs-slt"] = {["ofs"]=true,},
  ["bibleref-xidx"] = {["bibleref"]=true,},
  ["everypage"] = {["everypage"]=true,},
  ["hyph-quote-be"] = {},
  ["altverse"] = {["shipunov"]=true,},
  ["shdoc"] = {["shdoc"]=true,},
  ["cyber"] = {["cyber"]=true,},
  ["shapepar"] = {["shapepar"]=true,},
  ["shadow"] = {["shadow"]=true,},
  ["shadethm"] = {["shadethm"]=true,},
  ["shade"] = {["shade"]=true,},
  ["lhcyrkoi"] = {},
  ["sgamevar"] = {["sgame"]=true,},
  ["DotArrow"] = {["dotarrow"]=true,},
  ["musixesf"] = {["musixtex"]=true,},
  ["harvard"] = {["harvard"]=true,},
  ["sgame"] = {["sgame"]=true,},
  ["sffsmart"] = {["sffms"]=true,},
  ["tikzlibraryoverlay-beamer-styles.code"] = {["aobs-tikz"]=true,},
  ["noto-serif"] = {["noto"]=true,},
  ["lwarp-magaz"] = {["lwarp"]=true,},
  ["eolgrab"] = {["oberdiek"]=true,},
  ["sffdumb"] = {["sffms"]=true,},
  ["sf298"] = {["sf298"]=true,},
  ["wexam"] = {["sexam"]=true,},
  ["sexam"] = {["sexam"]=true,},
  ["ketslide2"] = {["ketcindy"]=true,},
  ["tikzlibrarychains.code"] = {["pgf"]=true,},
  ["bacex"] = {["sexam"]=true,},
  ["setdeck"] = {["setdeck"]=true,},
  ["esstixbb"] = {["esstix"]=true,},
  ["t-vim"] = {["context-vim"]=true,},
  ["loadhyph-ka"] = {},
  ["serbian-lig"] = {["serbian-lig"]=true,},
  ["serbian-def-cyr"] = {["serbian-def-cyr"]=true,},
  ["carey"] = {["context"]=true,},
  ["serbian-apostrophe"] = {["serbian-apostrophe"]=true,},
  ["kotexutf-core"] = {["kotex-plain"]=true,},
  ["othelloboard"] = {["othelloboard"]=true,},
  ["sepnum"] = {["sepnum"]=true,},
  ["smglom"] = {["stex"]=true,},
  ["ocgbase"] = {["ocgx2"]=true,},
  ["extract"] = {["extract"]=true,},
  ["lwarp-fancyhdr"] = {["lwarp"]=true,},
  ["hepnicenames"] = {["hepnames"]=true,},
  ["semioneside"] = {["semioneside"]=true,},
  ["jsarticle"] = {["jsclasses"]=true,},
  ["lithuanian"] = {["lithuanian"]=true,},
  ["lwarp-supertabular"] = {["lwarp"]=true,},
  ["tvz-hax"] = {["seminar"]=true,},
  ["DejaVuSansCondensed"] = {["dejavu"]=true,},
  ["l3docstrip"] = {["l3kernel"]=true,},
  ["interfaces-makecell"] = {["interfaces"]=true,},
  ["quotes"] = {["bosisio"]=true,},
  ["trans"] = {["tex-ps"]=true,},
  ["slidesec"] = {["seminar"]=true,},
  ["psizzl"] = {},
  ["babel-serbian-cyrl"] = {["babel"]=true,},
  ["semhelv"] = {["seminar"]=true,},
  ["fontloader-preprocessor-test"] = {["luaotfload"]=true,},
  ["trig"] = {["graphics"]=true,},
  ["stubs"] = {["stubs"]=true,},
  ["sem-a4"] = {["seminar"]=true,},
  ["fcltxdoc"] = {["fcltxdoc"]=true,},
  ["gmmeta"] = {["gmutils"]=true,},
  ["ean13"] = {["ean"]=true,},
  ["bpfats1"] = {["iso10303"]=true,},
  ["npsfont"] = {["seminar"]=true,},
  ["br-lex"] = {["br-lex"]=true,},
  ["babel-czech"] = {["babel"]=true,},
  ["young"] = {["jknapltx"]=true,},
  ["t-type-semaf"] = {["semaphor"]=true,},
  ["grant-nrl"] = {["grant"]=true,},
  ["geometry"] = {["geometry"]=true,},
  ["semantic-markup"] = {["semantic-markup"]=true,},
  ["nccfancyhdr"] = {["ncctools"]=true,},
  ["amstbold"] = {["qpxqtx"]=true,},
  ["luatexja-fontspec-25c"] = {["luatexja"]=true,},
  ["semantic"] = {["semantic"]=true,},
  ["m-ch-en"] = {["context"]=true,},
  ["ligature"] = {["semantic"]=true,},
  ["miniplot"] = {["miniplot"]=true,},
  ["fcnumparser"] = {["fmtcount"]=true,},
  ["lwarp-anonchap"] = {["lwarp"]=true,},
  ["regexpatch"] = {["regexpatch"]=true,},
  ["selnolig"] = {["selnolig"]=true,},
  ["glossaries-extra"] = {["glossaries-extra"]=true,},
  ["pst-coxcoor"] = {["pst-cox"]=true,},
  ["dk-apali"] = {["dk-bib"]=true,},
  ["selnolig-german-hyphex"] = {["selnolig"]=true,},
  ["tikzpeople.shape.builder"] = {["tikzpeople"]=true,},
  ["ncctitlepage"] = {["ncclatex"]=true,},
  ["musixhv"] = {["musixtex"]=true,},
  ["lwarp-fix2col"] = {["lwarp"]=true,},
  ["sectsty"] = {["sectsty"]=true,},
  ["sectionbreak"] = {["sectionbreak"]=true,},
  ["lualatexiniconfig"] = {},
  ["mintspirit"] = {["mintspirit"]=true,},
  ["bxjaprnind"] = {["bxjaprnind"]=true,},
  ["phn-FR_Ortsnamen"] = {["phonenumbers"]=true,},
  ["section"] = {["section"]=true,},
  ["fullminipage"] = {["fullminipage"]=true,},
  ["secdot"] = {["secdot"]=true,},
  ["beamerthemeepyt"] = {["beamertheme-epyt"]=true,},
  ["sdrt"] = {["sdrt"]=true,},
  ["shapedc"] = {["codepage"]=true,},
  ["hardmarg"] = {["screenplay"]=true,},
  ["sympytex"] = {["sympytexpackage"]=true,},
  ["alg"] = {["alg"]=true,},
  ["chemmacros5"] = {["chemmacros"]=true,},
  ["lwarp-scalefnt"] = {["lwarp"]=true,},
  ["s-abbreviations-extras"] = {["context"]=true,},
  ["sciposter"] = {["sciposter"]=true,},
  ["weva"] = {["schulschriften"]=true,},
  ["wesu"] = {["schulschriften"]=true,},
  ["armkb-a8"] = {["armtex"]=true,},
  ["wesa"] = {["schulschriften"]=true,},
  ["progress"] = {["progress"]=true,},
  ["jslogo"] = {["jsclasses"]=true,},
  ["feynmp"] = {["feynmf"]=true,},
  ["wela"] = {["schulschriften"]=true,},
  ["lwarp-tocloft"] = {["lwarp"]=true,},
  ["utarticle"] = {["uplatex"]=true,},
  ["beamerthemeBerlin"] = {["beamer"]=true,},
  ["scrdate"] = {},
  ["moderncvheadiii"] = {["moderncv"]=true,},
  ["amsfonts"] = {["amsfonts"]=true,},
  ["schulphy"] = {["schule"]=true,},
  ["lwarp-tikz"] = {["lwarp"]=true,},
  ["cap_c"] = {["c-pascal"]=true,},
  ["pagecont"] = {["pagecont"]=true,},
  ["example-glossaries-brief"] = {["glossaries"]=true,},
  ["labyrinth"] = {["labyrinth"]=true,},
  ["schullsg"] = {["schule"]=true,},
  ["mathrsfs"] = {["jknapltx"]=true,},
  ["fancytooltips"] = {["fancytooltips"]=true,},
  ["schulinf"] = {["schule"]=true,},
  ["schuleue"] = {["schule"]=true,},
  ["formular"] = {["formular"]=true,},
  ["mathcomp"] = {["mathcomp"]=true,},
  ["pst-spectra"] = {["pst-spectra"]=true,},
  ["spectralsequences"] = {["spectralsequences"]=true,},
  ["koi2ucy"] = {},
  ["schemata"] = {["schemata"]=true,},
  ["skeyval-core"] = {["skeyval"]=true,},
  ["scanpages"] = {["scanpages"]=true,},
  ["tikzposterBlockstyles"] = {["tikzposter"]=true,},
  ["scalebar"] = {["scalebar"]=true,},
  ["lwarp-backref"] = {["lwarp"]=true,},
  ["chkfloat"] = {["chkfloat"]=true,},
  ["sample2e"] = {["latex"]=true,},
  ["scale"] = {["scale"]=true,},
  ["amsbst"] = {["amsrefs"]=true,},
  ["savetrees"] = {["savetrees"]=true,},
  ["LinBiolinum_R"] = {["libertine"]=true,},
  ["raleway"] = {["raleway"]=true,},
  ["savefnmark"] = {["savefnmark"]=true,},
  ["sexscale"] = {["sauterfonts"]=true,},
  ["klups"] = {["kluwer"]=true,},
  ["zahl2string"] = {["sauerj"]=true,},
  ["processkv"] = {["sauerj"]=true,},
  ["hgbheadings"] = {["hagenberg-thesis"]=true,},
  ["font_times"] = {["font-change"]=true,},
  ["autind"] = {["index"]=true,},
  ["arabicfnt"] = {["arabi"]=true,},
  ["parcolumns"] = {["sauerj"]=true,},
  ["pgfcoretransformations.code"] = {["pgf"]=true,},
  ["lisp-latexutil"] = {["lisp-on-tex"]=true,},
  ["realscripts"] = {["realscripts"]=true,},
  ["tikzlibrarycd.code"] = {["tikz-cd"]=true,},
  ["sansmathfonts"] = {["sansmathfonts"]=true,},
  ["xellipsis"] = {["xellipsis"]=true,},
  ["tlinst-BT08-abstract"] = {},
  ["font_epigrafica_palatino"] = {["font-change"]=true,},
  ["lisp-string"] = {["lisp-on-tex"]=true,},
  ["chemmacros.module.particles.code"] = {["chemmacros"]=true,},
  ["beamercolorthemestructure"] = {["beamer"]=true,},
  ["path"] = {["path"]=true,},
  ["beamercolorthemedefault"] = {["beamer"]=true,},
  ["estcpmm"] = {["estcpmm"]=true,},
  ["ltxtools-review"] = {["ltxtools"]=true,},
  ["swpvntex"] = {["vntex"]=true,},
  ["sanitize-umlaut"] = {["sanitize-umlaut"]=true,},
  ["sageep"] = {["sageep"]=true,},
  ["lwarp-textcomp"] = {["lwarp"]=true,},
  ["cs-cursor"] = {},
  ["pgflibraryarrows"] = {["pgf"]=true,},
  ["powersem"] = {["texpower"]=true,},
  ["sa-tikz"] = {["sa-tikz"]=true,},
  ["lwarp-nowidow"] = {["lwarp"]=true,},
  ["macro-pegase"] = {["pas-cv"]=true,},
  ["lwarp-marginnote"] = {["lwarp"]=true,},
  ["thumbpdf"] = {["thumbpdf"]=true,},
  ["russ"] = {["russ"]=true,},
  ["old-arrows"] = {["old-arrows"]=true,},
  ["ruhyphzn"] = {},
  ["tikzlibrary3d.code"] = {["pgf"]=true,},
  ["xfor"] = {["xfor"]=true,},
  ["newunicodechar"] = {["newunicodechar"]=true,},
  ["allrunes"] = {["allrunes"]=true,},
  ["hyph-hsb.ec"] = {},
  ["cc4jT"] = {["computational-complexity"]=true,},
  ["fontloader-basics"] = {["luaotfload"]=true,},
  ["ruhyphct"] = {},
  ["bold-extra"] = {["bold-extra"]=true,},
  ["novel-pdfx"] = {["novel"]=true,},
  ["lisp-on-tex"] = {["lisp-on-tex"]=true,},
  ["relaycircuit"] = {["schule"]=true,},
  ["story"] = {},
  ["hyperbar"] = {["hyperbar"]=true,},
  ["koi2t2a"] = {},
  ["memexsupp"] = {["memexsupp"]=true,},
  ["enotez"] = {["enotez"]=true,},
  ["antomega"] = {["antomega"]=true,},
  ["koi2lcy"] = {},
  ["Kramer"] = {["cfr-initials"]=true,},
  ["koi2koi"] = {},
  ["saw"] = {["arabtex"]=true,},
  ["hypht2"] = {},
  ["enrhm2"] = {},
  ["raw"] = {["arabtex"]=true,},
  ["synproof"] = {["synproof"]=true,},
  ["docby"] = {["docbytex"]=true,},
  ["qswiss"] = {["tex-gyre"]=true,},
  ["cyryomg"] = {},
  ["cyryodv"] = {},
  ["cyryoct"] = {},
  ["phonrule"] = {["phonrule"]=true,},
  ["pgflibrarygraphdrawing.trees.code"] = {["pgf"]=true,},
  ["lh-t2bxccr"] = {["lh"]=true,},
  ["snotez"] = {["snotez"]=true,},
  ["roboto-mono"] = {["roboto"]=true,},
  ["rubikcube"] = {["rubik"]=true,},
  ["hyph-gl"] = {},
  ["chemmacros.module.mechanisms.code"] = {["chemmacros"]=true,},
  ["l3regex"] = {["l3kernel"]=true,},
  ["GoSans"] = {["gofonts"]=true,},
  ["asect"] = {["arabtex"]=true,},
  ["rtklage"] = {["rtklage"]=true,},
  ["rterface"] = {["rterface"]=true,},
  ["moderncvcolorpurple"] = {["moderncv"]=true,},
  ["pst-dart"] = {["pst-dart"]=true,},
  ["pst-spirograph"] = {["pst-spirograph"]=true,},
  ["rsc"] = {["rsc"]=true,},
  ["rrgtrees"] = {["rrgtrees"]=true,},
  ["alphalph"] = {["oberdiek"]=true,},
  ["beamerseminar"] = {["beamer"]=true,},
  ["doclicense-CC-by-nd-3.0-plaintext"] = {["doclicense"]=true,},
  ["romannum"] = {["romannum"]=true,},
  ["fxtargetlayoutcolor"] = {["fixme"]=true,},
  ["romanneg"] = {["romanneg"]=true,},
  ["doclicense-CC-by-nc-sa-3.0-plaintext"] = {["doclicense"]=true,},
  ["romanbarpagenumber"] = {["romanbarpagenumber"]=true,},
  ["egplot"] = {["egplot"]=true,},
  ["babel-taita"] = {["babel"]=true,},
  ["british"] = {["babel"]=true,},
  ["babel-sango"] = {["babel"]=true,},
  ["lwarp-register"] = {["lwarp"]=true,},
  ["eledform"] = {["eledform"]=true,},
  ["robustglossary"] = {["robustindex"]=true,},
  ["chessmin"] = {},
  ["roboto"] = {["roboto"]=true,},
  ["gfssolomos"] = {["gfssolomos"]=true,},
  ["rubikpatterns"] = {["rubik"]=true,},
  ["uafthesis"] = {["uafthesis"]=true,},
  ["iscram"] = {["iscram"]=true,},
  ["rlepsf"] = {["rlepsf"]=true,},
  ["rjlpshap"] = {["rjlparshap"]=true,},
  ["ribbonproofs"] = {["ribbonproofs"]=true,},
  ["revsymb"] = {["revtex4"]=true,},
  ["lwarp-chemfig"] = {["lwarp"]=true,},
  ["rusfonts"] = {},
  ["reftest4-1"] = {["revtex"]=true,},
  ["lwarp-soulutf8"] = {["lwarp"]=true,},
  ["ltxutil"] = {["revtex"]=true,},
  ["langsci-cgloss"] = {["langsci"]=true,},
  ["ltxfront"] = {["revtex"]=true,},
  ["delim"] = {["delim"]=true,},
  ["resumemac"] = {["resumemac"]=true,},
  ["lwarp-tabls"] = {["lwarp"]=true,},
  ["aucklandthesis"] = {["aucklandthesis"]=true,},
  ["cmtt"] = {["mdwtools"]=true,},
  ["resphilosophica"] = {["resphilosophica"]=true,},
  ["asapsym-generic"] = {["asapsym"]=true,},
  ["abbrevs"] = {["frankenstein"]=true,},
  ["repltext"] = {["repltext"]=true,},
  ["ctable"] = {["ctable"]=true,},
  ["tikzlibrarydecorations.softclip.code"] = {["pgfplots"]=true,},
  ["pgfmoduledatavisualization.code"] = {["pgf"]=true,},
  ["punk"] = {["punk-latex"]=true,},
  ["struktxf"] = {["struktex"]=true,},
  ["grant-nih"] = {["grant"]=true,},
  ["2sidedoc"] = {["relenc"]=true,},
  ["babel-chinese-simplified-hongkongsarchina"] = {["babel"]=true,},
  ["fxthemesignature"] = {["fixme"]=true,},
  ["tikzlibrarydecorations.markings.code"] = {["pgf"]=true,},
  ["langsci-tobi"] = {["langsci"]=true,},
  ["bohr"] = {["bohr"]=true,},
  ["babel-duala"] = {["babel"]=true,},
  ["scrbase"] = {},
  ["reledpar"] = {["reledmac"]=true,},
  ["reledmac"] = {["reledmac"]=true,},
  ["expex-demo"] = {["expex"]=true,},
  ["sproof"] = {["stex"]=true,},
  ["register"] = {["register"]=true,},
  ["datatool-fp"] = {["datatool"]=true,},
  ["topcapt"] = {},
  ["refstyle"] = {["refstyle"]=true,},
  ["eurosym"] = {["eurosym"]=true,},
  ["apmpspec"] = {["iso10303"]=true,},
  ["cyralias"] = {["t2"]=true,},
  ["cnbwp-manual"] = {["cnbwp"]=true,},
  ["ncctheorems"] = {["ncclatex"]=true,},
  ["mproof"] = {["metapost"]=true,},
  ["pagepc"] = {["refman"]=true,},
  ["babel-albanian"] = {["babel"]=true,},
  ["quoting"] = {["quoting"]=true,},
  ["refenums"] = {["refenums"]=true,},
  ["leadsheets.library.properties.code"] = {["leadsheets"]=true,},
  ["fixme"] = {["fixme"]=true,},
  ["articoletteracdp"] = {["cdpbundl"]=true,},
  ["gmnotonlypream"] = {["gmutils"]=true,},
  ["svg"] = {["svg"]=true,},
  ["recycle"] = {["recycle"]=true,},
  ["rectopma"] = {["rectopma"]=true,},
  ["spark-otf"] = {["spark-otf"]=true,},
  ["notespages"] = {["notespages"]=true,},
  ["RecipeBook"] = {["recipebook"]=true,},
  ["realboxes"] = {["realboxes"]=true,},
  ["unicode-bidi"] = {["unicode-bidi"]=true,},
  ["lstmisc"] = {["listings"]=true,},
  ["havannah"] = {["havannah"]=true,},
  ["readarray"] = {["readarray"]=true,},
  ["rbox-ex"] = {["gustlib"]=true,},
  ["gtrcore.options.code"] = {["genealogytree"]=true,},
  ["pkgloader-dry"] = {["pkgloader"]=true,},
  ["cc4siamltex"] = {["computational-complexity"]=true,},
  ["rcs-multi"] = {["rcs-multi"]=true,},
  ["lwarp-ltxgrid"] = {["lwarp"]=true,},
  ["beamerarticle"] = {["beamer"]=true,},
  ["competences"] = {["competences"]=true,},
  ["babel-nuer"] = {["babel"]=true,},
  ["randtext"] = {["randtext"]=true,},
  ["rlbicig"] = {["montex"]=true,},
  ["randomwalk"] = {["randomwalk"]=true,},
  ["zref-base"] = {["oberdiek"]=true,},
  ["facsimile"] = {["facsimile"]=true,},
  ["hyph-cy"] = {},
  ["beamerfontthemefocus"] = {["beamertheme-focus"]=true,},
  ["randbild"] = {["randbild"]=true,},
  ["cweb"] = {["cweb-latex"]=true,},
  ["nrc1"] = {["nrc"]=true,},
  ["artikel3"] = {["ntgclass"]=true,},
  ["ghsystem_flame-2-white"] = {["ghsystem"]=true,},
  ["savesym"] = {},
  ["numprint032"] = {["numprint"]=true,},
  ["raleway-type1-autoinst"] = {["raleway"]=true,},
  ["r_und_s"] = {["r_und_s"]=true,},
  ["multilang-tags"] = {["multilang"]=true,},
  ["tikzlibrarypgfplots.dateplot.code"] = {["pgfplots"]=true,},
  ["umich-thesis"] = {["umich-thesis"]=true,},
  ["perltex"] = {["perltex"]=true,},
  ["quotchap"] = {["quotchap"]=true,},
  ["accanthis"] = {["accanthis"]=true,},
  ["langsci-avm"] = {["langsci"]=true,},
  ["coptic"] = {["cbcoptic"]=true,},
  ["quicktype"] = {["quicktype"]=true,},
  ["mlp-49n"] = {["e-french"]=true,},
  ["quattrocento"] = {["quattrocento"]=true,},
  ["ifxptex"] = {["ifxptex"]=true,},
  ["beamer"] = {["beamer"]=true,},
  ["ghsystem_bottle-2-white"] = {["ghsystem"]=true,},
  ["qsymbols"] = {["qsymbols"]=true,},
  ["powerdot-BerlinFU"] = {["powerdot-fuberlin"]=true,},
  ["polyset"] = {["enctex"]=true,},
  ["leadsheets"] = {["leadsheets"]=true,},
  ["feynmf"] = {["feynmf"]=true,},
  ["macro-dynamique"] = {["pas-cv"]=true,},
  ["montgomery"] = {["context"]=true,},
  ["ebgaramond-maths"] = {["ebgaramond-maths"]=true,},
  ["qtxmath"] = {["qpxqtx"]=true,},
  ["beamerthemecompatibility"] = {["beamer"]=true,},
  ["cellprops"] = {["cellprops"]=true,},
  ["scraddr"] = {},
  ["floatpagestyle"] = {["floatrow"]=true,},
  ["lwarp-theorem"] = {["lwarp"]=true,},
  ["breqn"] = {["breqn"]=true,},
  ["shrthand"] = {["semantic"]=true,},
  ["amsqtx"] = {["qpxqtx"]=true,},
  ["amspbold"] = {["qpxqtx"]=true,},
  ["bxdvidriver"] = {["bxdvidriver"]=true,},
  ["tikzlibrarydecorations.fractals.code"] = {["pgf"]=true,},
  ["loadhyph-de-ch-1901"] = {},
  ["selnolig-english-patterns"] = {["selnolig"]=true,},
  ["varsfromjobname"] = {["varsfromjobname"]=true,},
  ["numerus"] = {["shipunov"]=true,},
  ["pgfbasesnakes"] = {["pgf"]=true,},
  ["csbulletin"] = {["csbulletin"]=true,},
  ["ltxmdf"] = {["mdframed"]=true,},
  ["pgfcoreexternal.code"] = {["pgf"]=true,},
  ["zref-abspos"] = {["oberdiek"]=true,},
  ["python"] = {["python"]=true,},
  ["common-titles"] = {},
  ["mathbbol"] = {["jknapltx"]=true,},
  ["lwarp-array"] = {["lwarp"]=true,},
  ["pxufont"] = {["pxufont"]=true,},
  ["logbox"] = {["logbox"]=true,},
  ["istgame"] = {["istgame"]=true,},
  ["pxtx-frak"] = {["pxtxalfa"]=true,},
  ["babel-english-australia"] = {["babel"]=true,},
  ["xyrotate"] = {["xypic"]=true,},
  ["pst-node"] = {["pst-node"]=true,},
  ["powerdot-fyma"] = {["powerdot"]=true,},
  ["lwarp-midfloat"] = {["lwarp"]=true,},
  ["xypspatt"] = {["xypic"]=true,},
  ["babel-lsorbian"] = {["babel"]=true,},
  ["tikzpeople.shape.cowboy"] = {["tikzpeople"]=true,},
  ["pxgreeks"] = {["pxgreeks"]=true,},
  ["pxfonts"] = {["pxfonts"]=true,},
  ["pxcjkcat"] = {["pxcjkcat"]=true,},
  ["lwarp-framed"] = {["lwarp"]=true,},
  ["bidipresentation"] = {["bidipresentation"]=true,},
  ["pxchfon"] = {["pxchfon"]=true,},
  ["pxbabel"] = {["pxbase"]=true,},
  ["wrapft"] = {["ptptex"]=true,},
  ["forest"] = {["forest"]=true,},
  ["pst-arrow"] = {["pst-arrow"]=true,},
  ["modulus"] = {["modulus"]=true,},
  ["ptext"] = {["ptext"]=true,},
  ["ptex"] = {["ptex-base"]=true,},
  ["loadhyph-grc"] = {},
  ["luatexko-core"] = {["luatexko"]=true,},
  ["URoptions"] = {["urcls"]=true,},
  ["vhistory"] = {["vhistory"]=true,},
  ["charter"] = {["psnfss"]=true,},
  ["pst-coil"] = {["pst-coil"]=true,},
  ["bitedemo"] = {["bitelist"]=true,},
  ["babel-luo"] = {["babel"]=true,},
  ["longnamefilelist"] = {["longnamefilelist"]=true,},
  ["gfsdidot"] = {["gfsdidot"]=true,},
  ["pstricks-pdf"] = {["pstricks"]=true,},
  ["pstcol"] = {["pstricks"]=true,},
  ["pst-platform"] = {["pstricks"]=true,},
  ["pstricks97"] = {["pstricks"]=true,},
  ["hyph-fi"] = {},
  ["pst-key"] = {["pstricks"]=true,},
  ["glossaries"] = {["glossaries"]=true,},
  ["incgraph"] = {["incgraph"]=true,},
  ["minitoc-hyper"] = {["hyperref"]=true,},
  ["ListVehicles"] = {["pst-vehicle"]=true,},
  ["psvectorian"] = {["pst-vectorian"]=true,},
  ["gregoriotex-spaces"] = {["gregoriotex"]=true,},
  ["pst-uml"] = {["pst-uml"]=true,},
  ["forloop"] = {["forloop"]=true,},
  ["ekaia"] = {["ekaia"]=true,},
  ["pst-tools"] = {["pst-tools"]=true,},
  ["pst-thick"] = {["pst-thick"]=true,},
  ["probsoln"] = {["probsoln"]=true,},
  ["miller"] = {["miller"]=true,},
  ["pst-text"] = {["pst-text"]=true,},
  ["newapave"] = {["confproc"]=true,},
  ["sseqloadstore.code"] = {["spectralsequences"]=true,},
  ["pst-stru"] = {["pst-stru"]=true,},
  ["schulein"] = {["schule"]=true,},
  ["babel-sanskrit-telu"] = {["babel"]=true,},
  ["hebrew_newcode"] = {["babel-hebrew"]=true,},
  ["MIKmathf"] = {["plnfss"]=true,},
  ["sourceserifpro"] = {["sourceserifpro"]=true,},
  ["pst-platon"] = {["pst-platon"]=true,},
  ["babel-ewe"] = {["babel"]=true,},
  ["forms16be"] = {["forms16be"]=true,},
  ["titles"] = {["frankenstein"]=true,},
  ["pst-solarsystem"] = {["pst-solarsystem"]=true,},
  ["mtx-context-precache"] = {["context"]=true,},
  ["pst-slpe"] = {["pst-slpe"]=true,},
  ["opmac-xetex"] = {},
  ["medstarbeamer"] = {["medstarbeamer"]=true,},
  ["pst-shell"] = {["pst-shell"]=true,},
  ["pgfplotsstackedplots.code"] = {["pgfplots"]=true,},
  ["filehook-listings"] = {["filehook"]=true,},
  ["lwarp-tocbibind"] = {["lwarp"]=true,},
  ["exercisebank"] = {["exercisebank"]=true,},
  ["pst-rubans"] = {["pst-rubans"]=true,},
  ["dspblocks"] = {["dsptricks"]=true,},
  ["denselists"] = {["nih"]=true,},
  ["dfgpdata"] = {["proposal"]=true,},
  ["pst-poker"] = {["pst-poker"]=true,},
  ["dashrule"] = {["dashrule"]=true,},
  ["PPRthomasd"] = {["prosper"]=true,},
  ["fxabbrev"] = {["e-french"]=true,},
  ["pst-plot"] = {["pst-plot"]=true,},
  ["pst-perspective"] = {["pst-perspective"]=true,},
  ["fithesis-econ"] = {["fithesis"]=true,},
  ["chemexec"] = {["chemexec"]=true,},
  ["twoblks"] = {["arabtex"]=true,},
  ["fltrace"] = {["latex"]=true,},
  ["pst-pad"] = {["pst-pad"]=true,},
  ["codeiso1"] = {["codepage"]=true,},
  ["babel-punjabi-arabic"] = {["babel"]=true,},
  ["lwarp-parskip"] = {["lwarp"]=true,},
  ["ptmxcomp"] = {["bgteubner"]=true,},
  ["pst-osci"] = {["pst-osci"]=true,},
  ["pst-ob3d"] = {["pst-ob3d"]=true,},
  ["dhucs-ucshyper"] = {["kotex-utf"]=true,},
  ["t-tikz"] = {["pgf"]=true,},
  ["pxtatescale"] = {["pxtatescale"]=true,},
  ["pst-math"] = {["pst-math"]=true,},
  ["jura"] = {["jura"]=true,},
  ["tvz-code"] = {["seminar"]=true,},
  ["pst-lens"] = {["pst-lens"]=true,},
  ["glossary-bookindex"] = {["glossaries-extra"]=true,},
  ["pst-layout"] = {["pst-layout"]=true,},
  ["svrsymbols"] = {["svrsymbols"]=true,},
  ["pst-labo"] = {["pst-labo"]=true,},
  ["ghsystem_flame-5-2-black"] = {["ghsystem"]=true,},
  ["sseqmessages.code"] = {["spectralsequences"]=true,},
  ["pst-intersect"] = {["pst-intersect"]=true,},
  ["pst-infixplot"] = {["pst-infixplot"]=true,},
  ["elpres"] = {["elpres"]=true,},
  ["gitlog"] = {["gitlog"]=true,},
  ["spotcolorpantone"] = {["spotcolor"]=true,},
  ["lua-check-hyphen"] = {["lua-check-hyphen"]=true,},
  ["pst-graphicx"] = {["pst-graphicx"]=true,},
  ["pgfplotsoldpgfsupp_pgflibraryluamath.code"] = {["pgfplots"]=true,},
  ["lgreek"] = {["lgreek"]=true,},
  ["openbib"] = {["latex"]=true,},
  ["primargs"] = {["morewrites"]=true,},
  ["babel-lakota"] = {["babel"]=true,},
  ["sref"] = {["stex"]=true,},
  ["bguq"] = {["bguq"]=true,},
  ["phn-UK_Ortsnamen"] = {["phonenumbers"]=true,},
  ["pst-ghsb"] = {["pst-ghsb"]=true,},
  ["sans"] = {["jknapltx"]=true,},
  ["ifoddpage"] = {["ifoddpage"]=true,},
  ["steinmetz"] = {["steinmetz"]=true,},
  ["fontawesome5-generic-helper"] = {["fontawesome5"]=true,},
  ["luatex-unicode-letters"] = {["luatex"]=true,},
  ["mwart"] = {["mwcls"]=true,},
  ["diffcoeff"] = {["diffcoeff"]=true,},
  ["moderncvstyleoldstyle"] = {["moderncv"]=true,},
  ["cool"] = {["cool"]=true,},
  ["villesFrance3d"] = {["pst-geo"]=true,},
  ["export-example"] = {["context"]=true,},
  ["bezier"] = {["latex"]=true,},
  ["lwarp-midpage"] = {["lwarp"]=true,},
  ["capitals"] = {["pst-geo"]=true,},
  ["vgrid"] = {["vgrid"]=true,},
  ["pst-gantt"] = {["pst-gantt"]=true,},
  ["beamercolorthemebeaver"] = {["beamer"]=true,},
  ["fnumprint"] = {["fnumprint"]=true,},
  ["t-lettrine"] = {["context-lettrine"]=true,},
  ["SPhdThesis"] = {["sphdthesis"]=true,},
  ["calctab"] = {["calctab"]=true,},
  ["captdef"] = {["captdef"]=true,},
  ["lwarp-stabular"] = {["lwarp"]=true,},
  ["loadhyph-nl"] = {},
  ["finstmsc"] = {["fontinst"]=true,},
  ["tikz-feynman"] = {["tikz-feynman"]=true,},
  ["pst-fr3d"] = {["pst-fr3d"]=true,},
  ["pst-fit"] = {["pst-fit"]=true,},
  ["fithesis-med"] = {["fithesis"]=true,},
  ["ydoc-code"] = {["ydoc"]=true,},
  ["swrule"] = {},
  ["spanglish"] = {["babel-spanglish"]=true,},
  ["versonotes"] = {["versonotes"]=true,},
  ["pst-electricfield"] = {["pst-electricfield"]=true,},
  ["plex-mono"] = {["plex"]=true,},
  ["bsymbols"] = {["fontch"]=true,},
  ["scrload"] = {["rsfs"]=true,},
  ["datatool"] = {["datatool"]=true,},
  ["ghsystem_aqpol"] = {["ghsystem"]=true,},
  ["pst-coxeterp"] = {["pst-cox"]=true,},
  ["arabart"] = {["arabtex"]=true,},
  ["selnolig-german-patterns"] = {["selnolig"]=true,},
  ["pst-contourplot"] = {["pst-contourplot"]=true,},
  ["ltjreport"] = {["luatexja"]=true,},
  ["endheads"] = {["endheads"]=true,},
  ["pstricks-add"] = {["pstricks-add"]=true,},
  ["linden"] = {["context"]=true,},
  ["fontawesome"] = {["fontawesome"]=true,},
  ["pst-circ"] = {["pst-circ"]=true,},
  ["loadhyph-el-monoton"] = {["hyphen-greek"]=true,},
  ["font_kurier-light"] = {["font-change"]=true,},
  ["beamerthememetropolis"] = {["beamertheme-metropolis"]=true,},
  ["khatt-en"] = {["context"]=true,},
  ["pst-cie"] = {["pst-cie"]=true,},
  ["balancedquotes"] = {},
  ["easy"] = {["easy"]=true,},
  ["cs-heuristica"] = {},
  ["layouts"] = {["layouts"]=true,},
  ["eskdtitle"] = {["eskdx"]=true,},
  ["pst-calculate"] = {["pst-calculate"]=true,},
  ["pst-bspline"] = {["pst-bspline"]=true,},
  ["pst-bezier"] = {["pst-bezier"]=true,},
  ["pst-bar"] = {["pst-bar"]=true,},
  ["cwebbase"] = {["cweb-latex"]=true,},
  ["oands"] = {["archaic"]=true,},
  ["doclicense-CC-by-4.0-plaintext"] = {["doclicense"]=true,},
  ["pst-asr"] = {["pst-asr"]=true,},
  ["pst-antiprism"] = {["pst-antiprism"]=true,},
  ["facture"] = {["facture"]=true,},
  ["pst-am"] = {["pst-am"]=true,},
  ["lwarp-scrlayer-notecolumn"] = {["lwarp"]=true,},
  ["pst-2dplot"] = {["pst-2dplot"]=true,},
  ["utopia"] = {["psnfss"]=true,},
  ["DSmac"] = {["fontch"]=true,},
  ["draftwatermark"] = {["draftwatermark"]=true,},
  ["babel-vai-vai"] = {["babel"]=true,},
  ["times"] = {["psnfss"]=true,},
  ["carlito"] = {["carlito"]=true,},
  ["galois"] = {["galois"]=true,},
  ["pifont"] = {["psnfss"]=true,},
  ["cc2cite"] = {["computational-complexity"]=true,},
  ["newcent"] = {["psnfss"]=true,},
  ["l3str"] = {["l3kernel"]=true,},
  ["mathptmx"] = {["psnfss"]=true,},
  ["lwarp-extramarks"] = {["lwarp"]=true,},
  ["enc-u"] = {["enctex"]=true,},
  ["rotchiffre"] = {["oberdiek"]=true,},
  ["mathptm"] = {["psnfss"]=true,},
  ["mathpple"] = {["psnfss"]=true,},
  ["interfaces-etoolbox"] = {["interfaces"]=true,},
  ["mathpazo"] = {["psnfss"]=true,},
  ["tocdata"] = {["tocdata"]=true,},
  ["courier"] = {["psnfss"]=true,},
  ["font_antt-medium"] = {["font-change"]=true,},
  ["pstring"] = {["pstring"]=true,},
  ["lamsarrow"] = {["pb-diagram"]=true,},
  ["erewhon"] = {["erewhon"]=true,},
  ["bookman"] = {["psnfss"]=true,},
  ["mergeh"] = {["hletter"]=true,},
  ["mftmac"] = {},
  ["printlen"] = {["printlen"]=true,},
  ["avant"] = {["psnfss"]=true,},
  ["s-pre-13"] = {["context"]=true,},
  ["hyph-hsb"] = {},
  ["lwarp-threeparttable"] = {["lwarp"]=true,},
  ["luatex"] = {["oberdiek"]=true,},
  ["PPRautumn"] = {["prosper"]=true,},
  ["palatino"] = {["psnfss"]=true,},
  ["babel-meta"] = {["babel"]=true,},
  ["fontawesomesymbols-generic"] = {["fontawesome"]=true,},
  ["gobble"] = {["gobble"]=true,},
  ["edmacfss"] = {["edmac"]=true,},
  ["pseudocode"] = {["pseudocode"]=true,},
  ["scrfontsizes"] = {},
  ["psbao"] = {["psbao"]=true,},
  ["milog"] = {["milog"]=true,},
  ["beletter"] = {},
  ["tarticle"] = {["platex"]=true,},
  ["protocol"] = {["protocol"]=true,},
  ["pagereference"] = {},
  ["ukinsoku"] = {["uplatex"]=true,["uptex-base"]=true,},
  ["aguplus"] = {["aguplus"]=true,},
  ["prosper"] = {["prosper"]=true,},
  ["pdfpages"] = {["pdfpages"]=true,},
  ["PPRwj"] = {["prosper"]=true,},
  ["fixltxhyph"] = {["fixltxhyph"]=true,},
  ["LobsterTwo"] = {["lobster2"]=true,},
  ["webmac"] = {},
  ["pkgloader"] = {["pkgloader"]=true,},
  ["PPRwinter"] = {["prosper"]=true,},
  ["fmtcount"] = {["fmtcount"]=true,},
  ["babel-yangben"] = {["babel"]=true,},
  ["catoptions"] = {["catoptions"]=true,},
  ["tikzlibraryshapes.misc.code"] = {["pgf"]=true,},
  ["beamerthemeSingapore"] = {["beamer"]=true,},
  ["gost732"] = {["disser"]=true,},
  ["PPRtroispoints"] = {["prosper"]=true,},
  ["jmlrbook"] = {["jmlr"]=true,},
  ["pst-plot97"] = {["pst-plot"]=true,},
  ["pgfsys"] = {["pgf"]=true,},
  ["przyklad"] = {["gustlib"]=true,},
  ["stackrel"] = {["oberdiek"]=true,},
  ["PPRprettybox"] = {["prosper"]=true,},
  ["ytableau"] = {["ytableau"]=true,},
  ["qstest"] = {["qstest"]=true,},
  ["PPRlignesbleues"] = {["prosper"]=true,},
  ["fink"] = {["fink"]=true,},
  ["PPRgyom"] = {["prosper"]=true,},
  ["PPRfyma"] = {["prosper"]=true,},
  ["bibleref-mouth"] = {["bibleref-mouth"]=true,},
  ["PPRdarkblue"] = {["prosper"]=true,},
  ["beamerthemeDarmstadt"] = {["beamer"]=true,},
  ["pgfmodulematrix.code"] = {["pgf"]=true,},
  ["TeXProject"] = {["dratex"]=true,},
  ["ot1patch"] = {["polski"]=true,},
  ["makematch"] = {["qstest"]=true,},
  ["beameraudience"] = {["beameraudience"]=true,},
  ["PPRcapsules"] = {["prosper"]=true,},
  ["hyph-quote-zh-latn-pinyin"] = {},
  ["PPRblends"] = {["prosper"]=true,},
  ["PPRazure"] = {["prosper"]=true,},
  ["PPRalienglow"] = {["prosper"]=true,},
  ["PPRalcatel"] = {["prosper"]=true,},
  ["barticle"] = {["bangtex"]=true,},
  ["tikzlibraryspy.code"] = {["pgf"]=true,},
  ["euproposal"] = {["proposal"]=true,},
  ["eupdata"] = {["proposal"]=true,},
  ["tikzlibrarytikzmark.code"] = {["tikzmark"]=true,},
  ["bibnames"] = {},
  ["njf"] = {["economic"]=true,},
  ["dfgproposal"] = {["proposal"]=true,},
  ["pst-pulley"] = {["pst-pulley"]=true,},
  ["engpron"] = {["engpron"]=true,},
  ["xypoly"] = {["xypic"]=true,},
  ["proposal"] = {["proposal"]=true,},
  ["tikzlibraryfolding.code"] = {["pgf"]=true,},
  ["pdata"] = {["proposal"]=true,},
  ["moderncvstylecasual"] = {["moderncv"]=true,},
  ["langsci-hyphenation-de"] = {["langsci"]=true,},
  ["prooftrees"] = {["prooftrees"]=true,},
  ["drftcono"] = {["chemcono"]=true,},
  ["txi-nn"] = {},
  ["totcount"] = {["totcount"]=true,},
  ["zref-savepos"] = {["oberdiek"]=true,},
  ["prodint"] = {["prodint"]=true,},
  ["listpen"] = {["floatrow"]=true,},
  ["leadsheets.library.templates.code"] = {["leadsheets"]=true,},
  ["procIAGssymp"] = {},
  ["cc4apjrnl"] = {["computational-complexity"]=true,},
  ["prettyref"] = {["prettyref"]=true,},
  ["stppdlst"] = {["iso10303"]=true,},
  ["semproc"] = {["semproc"]=true,},
  ["s-simpleslides-Rounded"] = {["context-simpleslides"]=true,},
  ["newpxmath"] = {["newpx"]=true,},
  ["optional"] = {["optional"]=true,},
  ["tlgsqq"] = {["ibygrk"]=true,},
  ["libertinust1math"] = {["libertinust1math"]=true,},
  ["hackthefootline"] = {["hackthefootline"]=true,},
  ["present"] = {["present"]=true,},
  ["sr-vorl"] = {["sr-vorl"]=true,},
  ["sublabel"] = {["preprint"]=true,},
  ["balance"] = {["preprint"]=true,},
  ["footbib"] = {["footbib"]=true,},
  ["onrannual"] = {["onrannual"]=true,},
  ["kluwer"] = {["kluwer"]=true,},
  ["francaisbst"] = {["francais-bst"]=true,},
  ["babel-bosnian-latn"] = {["babel"]=true,},
  ["lwarp-continue"] = {["lwarp"]=true,},
  ["crossreference"] = {["crossreference"]=true,},
  ["s-faq-00"] = {["context"]=true,},
  ["pracjourn"] = {["pracjourn"]=true,},
  ["fullblck"] = {["fullblck"]=true,},
  ["tcbhooks.code"] = {["tcolorbox"]=true,},
  ["plextarray"] = {["platex-tools"]=true,},
  ["multibbl"] = {["multibbl"]=true,},
  ["idxcmds"] = {["idxcmds"]=true,},
  ["PPRmancini"] = {["prosper"]=true,},
  ["powerdot"] = {["powerdot"]=true,},
  ["powerdot-upen"] = {["powerdot"]=true,},
  ["babel-nyankole"] = {["babel"]=true,},
  ["powerdot-tycja"] = {["powerdot"]=true,},
  ["powerdot-simple"] = {["powerdot"]=true,},
  ["ascelike"] = {["ascelike"]=true,},
  ["ltjbook"] = {["luatexja"]=true,},
  ["loadhyph-la"] = {},
  ["anyfontsize"] = {["anyfontsize"]=true,},
  ["babel-piedmontese"] = {["babel"]=true,},
  ["powerdot-paintings"] = {["powerdot"]=true,},
  ["copyrightbox"] = {["copyrightbox"]=true,},
  ["powerdot-horatio"] = {["powerdot"]=true,},
  ["leadsheets.library.translations.code"] = {["leadsheets"]=true,},
  ["alertmessage"] = {["alertmessage"]=true,},
  ["powerdot-default"] = {["powerdot"]=true,},
  ["hyph-de-1996.ec"] = {},
  ["pml3array"] = {["hobby"]=true,},
  ["syntax"] = {["syntax"]=true,["mdwtools"]=true,},
  ["ecca"] = {["economic"]=true,},
  ["powerdot-aggie"] = {["powerdot"]=true,},
  ["interfaces-marks"] = {["interfaces"]=true,},
  ["wallcalendar-romanian"] = {["wallcalendar"]=true,},
  ["loadhyph-hy"] = {},
  ["flacards"] = {["flacards"]=true,},
  ["hyph-ka"] = {},
  ["lwarp-placeins"] = {["lwarp"]=true,},
  ["polynomial"] = {["polynomial"]=true,},
  ["amsart"] = {["amscls"]=true,},
  ["faktor"] = {["faktor"]=true,},
  ["polynom"] = {["polynom"]=true,},
  ["algmatlab"] = {["algorithmicx"]=true,},
  ["polyglossia"] = {["polyglossia"]=true,},
  ["cwebmac"] = {["cweb"]=true,},
  ["unifam"] = {},
  ["nkonumbers"] = {["polyglossia"]=true,},
  ["hebrewcal"] = {["polyglossia"]=true,},
  ["tikzlibrarypgfplots.statistics.code"] = {["pgfplots"]=true,},
  ["prerex"] = {["prerex"]=true,},
  ["bengalidigits"] = {["polyglossia"]=true,},
  ["droidmono"] = {["droid"]=true,},
  ["apa6e"] = {["apa6e"]=true,},
  ["scrdocstrip"] = {},
  ["lwarp-forest"] = {["lwarp"]=true,},
  ["antpolt"] = {["poltawski"]=true,},
  ["novel-DropCap"] = {["novel"]=true,},
  ["doctools"] = {["doctools"]=true,},
  ["polexpr"] = {["polexpr"]=true,},
  ["poetrytex"] = {["poetrytex"]=true,},
  ["poetry"] = {["poetry"]=true,},
  ["poemscol"] = {["poemscol"]=true,},
  ["dehyphn-x-2018-03-31"] = {["dehyph-exptl"]=true,},
  ["chords"] = {["pmxchords"]=true,},
  ["knuth"] = {["context"]=true,},
  ["pmx"] = {["pmx"]=true,},
  ["electrum"] = {["electrum"]=true,},
  ["ucharclasses"] = {["ucharclasses"]=true,},
  ["pcode"] = {["plweb"]=true,},
  ["ocherokee"] = {["ocherokee"]=true,},
  ["comicneue"] = {["comicneue"]=true,},
  ["insbox"] = {["insbox"]=true,},
  ["luatex-pdf"] = {["context"]=true,},
  ["dlfltxbcodetips"] = {["dlfltxb"]=true,},
  ["pgflibraryshapes.gates.logic.code"] = {["pgf"]=true,},
  ["langscibook"] = {["langsci"]=true,},
  ["pst-soroban"] = {["pst-soroban"]=true,},
  ["HAPAggie"] = {["ha-prosper"]=true,},
  ["phn-DE_Ortsnamen"] = {["phonenumbers"]=true,},
  ["babel-chinese-hant-mo"] = {["babel"]=true,},
  ["pst-diffraction"] = {["pst-diffraction"]=true,},
  ["pxftnright"] = {["platex-tools"]=true,},
  ["pxeveryshi"] = {["platex-tools"]=true,},
  ["pxeverysel"] = {["platex-tools"]=true,},
  ["pxatbegshi"] = {["platex-tools"]=true,},
  ["s-simpleslides-SideSquares"] = {["context-simpleslides"]=true,},
  ["statya"] = {},
  ["plextdelarray"] = {["platex-tools"]=true,},
  ["xytips"] = {["xypic"]=true,},
  ["hypcap"] = {["oberdiek"]=true,},
  ["plnews"] = {["platex"]=true,},
  ["sem-page"] = {["seminar"]=true,},
  ["dayofweek"] = {},
  ["babel-meru"] = {["babel"]=true,},
  ["hyph-quote-uk"] = {},
  ["beamercolorthemewolverine"] = {["beamer"]=true,},
  ["autopagewidth"] = {["extsizes"]=true,},
  ["novel-HeadFootStyles"] = {["novel"]=true,},
  ["babel-chinese"] = {["babel"]=true,},
  ["mlp-01"] = {["e-french"]=true,},
  ["loadhyph-tr"] = {},
  ["beamercolorthemebrick"] = {["beamertheme-cuerna"]=true,},
  ["picture"] = {["oberdiek"]=true,["graphics-pln"]=true,},
  ["assyr"] = {["aramaic-serto"]=true,},
  ["stdclsdv"] = {["stdclsdv"]=true,},
  ["jreport"] = {["platex"]=true,},
  ["setiby4"] = {["ibygrk"]=true,},
  ["luatexja-ajmacros"] = {["luatexja"]=true,},
  ["coseoul"] = {["coseoul"]=true,},
  ["jltxdoc"] = {["platex"]=true,},
  ["jbook"] = {["platex"]=true,},
  ["ulqda"] = {["ulqda"]=true,},
  ["exppl2e"] = {["platex"]=true,},
  ["plates"] = {["plates"]=true,},
  ["endplate"] = {["plates"]=true,},
  ["pgfcircbipoles"] = {["circuitikz"]=true,},
  ["cs-kurier"] = {},
  ["plari"] = {["plari"]=true,},
  ["outlines"] = {["outlines"]=true,},
  ["beamerbasetranslator"] = {["beamer"]=true,},
  ["pgfcomp-version-0-65"] = {["pgf"]=true,},
  ["babel-chinese-hans-hk"] = {["babel"]=true,},
  ["lilyglyphs"] = {["lilyglyphs"]=true,},
  ["grid"] = {["grid"]=true,},
  ["plainpkg"] = {["plainpkg"]=true,},
  ["mikoslides"] = {["stex"]=true,},
  ["hyph-bg.t2a"] = {["hyphen-bulgarian"]=true,},
  ["lwarp-savetrees"] = {["lwarp"]=true,},
  ["bibunits"] = {["bibunits"]=true,},
  ["picmac"] = {},
  ["hyph-th"] = {},
  ["mptmac"] = {},
  ["mnotes"] = {["mnotes"]=true,},
  ["xmlparser"] = {["olsak-misc"]=true,},
  ["cfr-lm"] = {["cfr-lm"]=true,},
  ["opteng"] = {["opteng"]=true,},
  ["linearA"] = {["lineara"]=true,},
  ["tccompat"] = {["jknapltx"]=true,},
  ["tagpdf-mc-code-generic"] = {["tagpdf"]=true,},
  ["gkpmac"] = {},
  ["fontchart"] = {},
  ["placeins"] = {["placeins"]=true,},
  ["babel-english-canada"] = {["babel"]=true,},
  ["placeat"] = {["placeat"]=true,},
  ["lwarp-hanging"] = {["lwarp"]=true,},
  ["pkuthss"] = {["pkuthss"]=true,},
  ["fnum"] = {["arabi"]=true,},
  ["pkgloader-true"] = {["pkgloader"]=true,},
  ["tikzlibraryshapes.multipart.code"] = {["pgf"]=true,},
  ["pkgloader-false"] = {["pkgloader"]=true,},
  ["pkgloader-early"] = {["pkgloader"]=true,},
  ["pkgloader-cls-pkg"] = {["pkgloader"]=true,},
  ["arabxetex"] = {["arabxetex"]=true,},
  ["a35"] = {["ofs"]=true,},
  ["pixelart"] = {["pixelart"]=true,},
  ["pittetd"] = {["pittetd"]=true,},
  ["pitex"] = {["pitex"]=true,},
  ["moderncvdebugtools"] = {["moderncv"]=true,},
  ["babel-northndebele"] = {["babel"]=true,},
  ["pigpen"] = {["pigpen"]=true,},
  ["time"] = {["piff"]=true,},
  ["babel-vai-latin"] = {["babel"]=true,},
  ["newproof"] = {["piff"]=true,},
  ["lwarp-pfnote"] = {["lwarp"]=true,},
  ["bullenum"] = {["bullcntr"]=true,},
  ["beamercolorthemefibeamer-mu-phil"] = {["fibeamer"]=true,},
  ["bpfats8"] = {["iso10303"]=true,},
  ["babel-malay-sg"] = {["babel"]=true,},
  ["hyph-bg"] = {["hyphen-bulgarian"]=true,},
  ["tree"] = {["pictex"]=true,},
  ["gn-logic14"] = {["gene-logic"]=true,},
  ["postpictex"] = {["pictex"]=true,},
  ["loadhyph-gl"] = {},
  ["lwarp-tocenter"] = {["lwarp"]=true,},
  ["pictex"] = {["pictex"]=true,},
  ["babel-koyraborosenni"] = {["babel"]=true,},
  ["sobolev"] = {["bosisio"]=true,},
  ["xnewcommand"] = {["xnewcommand"]=true,},
  ["latexpicobjs"] = {["pictex"]=true,},
  ["bookest"] = {["bookest"]=true,},
  ["mftinc"] = {["mftinc"]=true,},
  ["urwchancal"] = {["urwchancal"]=true,},
  ["slovak"] = {["babel"]=true,["cslatex"]=true,},
  ["physics"] = {["physics"]=true,},
  ["miama"] = {["miama"]=true,},
  ["bkltprnt"] = {["booklet"]=true,},
  ["common-oddorevenpage"] = {},
  ["cyryoal"] = {},
  ["phonetic"] = {["phonetic"]=true,},
  ["dsptricks"] = {["dsptricks"]=true,},
  ["phn-US_Ortsnamen"] = {["phonenumbers"]=true,},
  ["beamerouterthemesmoothtree"] = {["beamer"]=true,},
  ["a117"] = {["ofs"]=true,},
  ["xyknot"] = {["xypic"]=true,},
  ["attrib"] = {["frankenstein"]=true,},
  ["hsetup"] = {["hletter"]=true,},
  ["natmove"] = {["achemso"]=true,},
  ["plex-sans"] = {["plex"]=true,},
  ["list"] = {},
  ["reich"] = {["context"]=true,},
  ["philokalia"] = {["philokalia"]=true,},
  ["tikz-cd"] = {["tikz-cd"]=true,},
  ["droidsans"] = {["droid"]=true,},
  ["phfsvnwatermark"] = {["phfsvnwatermark"]=true,},
  ["babel-romanian"] = {["babel"]=true,},
  ["phfqit"] = {["phfqit"]=true,},
  ["tikzlibraryscopes.code"] = {["pgf"]=true,},
  ["tikzlibrarybabel.code"] = {["pgf"]=true,},
  ["s-simpleslides-Ellipse"] = {["context-simpleslides"]=true,},
  ["xsim.exercises.code"] = {["xsim"]=true,},
  ["pgfregressiontest"] = {["pgfplots"]=true,},
  ["font_kurier-bold"] = {["font-change"]=true,},
  ["answers"] = {["answers"]=true,},
  ["pgfplots"] = {["pgfplots"]=true,},
  ["tikzlibrarypgfplotsclickable.code"] = {["pgfplots"]=true,},
  ["xypsdict"] = {["xypic"]=true,},
  ["pgfplotsutil.verb.code"] = {["pgfplots"]=true,},
  ["shadebox"] = {},
  ["pgfplotscolormap.code"] = {["pgfplots"]=true,},
  ["il2-1250"] = {["enctex"]=true,},
  ["allrtnd"] = {["bookhands"]=true,},
  ["fancyhdr"] = {["fancyhdr"]=true,},
  ["graphicxbox"] = {["graphicxbox"]=true,},
  ["beamerfontthemeUR"] = {["urcls"]=true,},
  ["loadhyph-as"] = {},
  ["xepsf"] = {},
  ["pgfplotsbinary.data.code"] = {["pgfplots"]=true,},
  ["beamercolorthemealbatross"] = {["beamer"]=true,},
  ["hyph-quote-oc"] = {},
  ["nccold"] = {["ncclatex"]=true,},
  ["pgfplots.assert"] = {["pgfplots"]=true,},
  ["nonfloat"] = {["nonfloat"]=true,},
  ["pgfplots.assert.code"] = {["pgfplots"]=true,},
  ["tkz-linknodes"] = {["tkz-linknodes"]=true,},
  ["pgfplotsplothandlers.code"] = {["pgfplots"]=true,},
  ["default-amssymbols"] = {["font-change"]=true,},
  ["lwarp-ulem"] = {["lwarp"]=true,},
  ["example-image-1x1"] = {["mwe"]=true,},
  ["beamerinnerthemefocus"] = {["beamertheme-focus"]=true,},
  ["xurl"] = {["xurl"]=true,},
  ["pgfplotscoordprocessing.code"] = {["pgfplots"]=true,},
  ["pgfplots.scaling.code"] = {["pgfplots"]=true,},
  ["arcs"] = {["arcs"]=true,},
  ["unicode"] = {},
  ["s-simpleslides-Boxed"] = {["context-simpleslides"]=true,},
  ["pdftricks2"] = {["pdftricks2"]=true,},
  ["attachfile"] = {["attachfile"]=true,},
  ["pgfbaseimage"] = {["pgf"]=true,},
  ["pgfplots.revision"] = {["pgfplots"]=true,},
  ["cmmib57"] = {["amsfonts"]=true,},
  ["pgfplots.paths.code"] = {["pgfplots"]=true,},
  ["mtx-context-select"] = {["context"]=true,},
  ["pgfcorearrows.code"] = {["pgf"]=true,},
  ["pgfplots.markers.code"] = {["pgfplots"]=true,},
  ["testidx"] = {["testidx"]=true,},
  ["repeatindex"] = {["repeatindex"]=true,},
  ["pgflibraryfillbetween.code"] = {["pgfplots"]=true,},
  ["bplain"] = {["babel"]=true,},
  ["tikzlibrarydateplot.code"] = {["pgfplots"]=true,},
  ["pgfplotsoldpgfsupp_trig_format.code"] = {["pgfplots"]=true,},
  ["fp-trigo"] = {["fp"]=true,},
  ["expl3"] = {["l3kernel"]=true,},
  ["skeyval-ltxpatch"] = {["skeyval"]=true,},
  ["musixstr"] = {["musixtex"]=true,},
  ["aurl"] = {["aurl"]=true,},
  ["pgfplotsoldpgfsupp_tikzexternalshared.code"] = {["pgfplots"]=true,},
  ["scrguide"] = {},
  ["babel-chinese-hans-sg"] = {["babel"]=true,},
  ["pgfplotsoldpgfsupp_tikzexternal"] = {["pgfplots"]=true,},
  ["pgfplotsoldpgfsupp_pgfmathfloat.code"] = {["pgfplots"]=true,},
  ["forest-doc"] = {["forest"]=true,},
  ["lstdoc"] = {["listings"]=true,},
  ["boldline"] = {["shipunov"]=true,},
  ["nowtoaux"] = {["volumes"]=true,},
  ["trivfloat"] = {["trivfloat"]=true,},
  ["doclicense-CC-zero-1.0-plaintext"] = {["doclicense"]=true,},
  ["load-unicode-data"] = {["unicode-data"]=true,},
  ["leadsheets.library.external.code"] = {["leadsheets"]=true,},
  ["osf2x"] = {["fontinst"]=true,},
  ["ghsystem_explos-3"] = {["ghsystem"]=true,},
  ["pgfplotsoldpgfsupp_pgflibraryfpu.code"] = {["pgfplots"]=true,},
  ["tikzlibrarydatavisualization.barcharts.code"] = {["pgf"]=true,},
  ["heuristica"] = {["heuristica"]=true,},
  ["beamerthemeFrankfurt"] = {["beamer"]=true,},
  ["pgfplotsoldpgfsupp_pgfkeysfiltered.code"] = {["pgfplots"]=true,},
  ["pgfplotsoldpgfsupp_pgfcorelayers.code"] = {["pgfplots"]=true,},
  ["lpenv"] = {["logicpuzzle"]=true,},
  ["font_charter"] = {["font-change"]=true,},
  ["TXSdcol"] = {["texsis"]=true,},
  ["gamebook"] = {["gamebook"]=true,},
  ["pgfplotsoldpgfsupp_misc.code"] = {["pgfplots"]=true,},
  ["pgfplotsoldpgfsupp_leq.code"] = {["pgfplots"]=true,},
  ["examdesign"] = {["examdesign"]=true,},
  ["milsymb"] = {["milsymb"]=true,},
  ["pgfplotsliststructureext.code"] = {["pgfplots"]=true,},
  ["context-performance"] = {["context"]=true,},
  ["pgfplotsdeque.code"] = {["pgfplots"]=true,},
  ["hhead"] = {["hletter"]=true,},
  ["hyph-eo.il3"] = {},
  ["pgfplotsarray.code"] = {["pgfplots"]=true,},
  ["moderncvstylebanking"] = {["moderncv"]=true,},
  ["expl3-code"] = {["l3kernel"]=true,},
  ["graphviz"] = {["graphviz"]=true,},
  ["edmac"] = {["edmac"]=true,},
  ["tikzlibrarypgfplots.units.code"] = {["pgfplots"]=true,},
  ["aramaic"] = {["archaic"]=true,},
  ["miniltx"] = {["graphics-pln"]=true,},
  ["cenccmn"] = {["ucs"]=true,},
  ["tikzlibrarypgfplots.polar.code"] = {["pgfplots"]=true,},
  ["dhucs-cmap"] = {["kotex-utf"]=true,},
  ["mdsymbol"] = {["mdsymbol"]=true,},
  ["harpoon"] = {["harpoon"]=true,},
  ["tikzlibrarypgfplots.patchplots.code"] = {["pgfplots"]=true,},
  ["tikzlibrarypgfplots.external.code"] = {["pgfplots"]=true,},
  ["demo-symbols"] = {["context"]=true,},
  ["srbook-mem"] = {["srbook-mem"]=true,},
  ["hyph-hr"] = {},
  ["fdsymbol"] = {["fdsymbol"]=true,},
  ["tikzlibrarycolortol.code"] = {["pgfplots"]=true,},
  ["engtlc"] = {["engtlc"]=true,},
  ["hcycle"] = {["xymtex"]=true,},
  ["pgflibrarypgfplots.surfshading.code"] = {["pgfplots"]=true,},
  ["t-pgfplotstable"] = {["pgfplots"]=true,},
  ["ccycle"] = {["xymtex"]=true,},
  ["tikzlibraryarrows.code"] = {["pgf"]=true,},
  ["handin"] = {["handin"]=true,},
  ["cncolours"] = {["pgfornament-han"]=true,},
  ["issuulinks"] = {["issuulinks"]=true,},
  ["tikzlibraryfillbetween.code"] = {["pgfplots"]=true,},
  ["babel-sanskrit-gujr"] = {["babel"]=true,},
  ["beamerthemeHeavenlyClouds"] = {["pgfornament-han"]=true,},
  ["combofont"] = {["combofont"]=true,},
  ["pgfornament"] = {["pgfornament"]=true,},
  ["pgflibraryam.code"] = {["pgfornament"]=true,},
  ["dlfltxbtocconfig"] = {["dlfltxb"]=true,},
  ["arabtex"] = {["arabtex"]=true,},
  ["lwarp-transparent"] = {["lwarp"]=true,},
  ["scrbookreportarticle"] = {},
  ["hypht1"] = {},
  ["helvet"] = {["psnfss"]=true,},
  ["pmgraph"] = {["pmgraph"]=true,},
  ["pgfmolbio.chromatogram"] = {["pgfmolbio"]=true,},
  ["beamercolorthemelettuce"] = {["beamertheme-cuerna"]=true,},
  ["xintbinhex"] = {["xint"]=true,},
  ["cjkutf8-josa"] = {["cjk-ko"]=true,},
  ["lhcyralt"] = {},
  ["fp-snap"] = {["fp"]=true,},
  ["pgfgantt"] = {["pgfgantt"]=true,},
  ["greenfield"] = {["context"]=true,},
  ["pgf-umlcd"] = {["pgf-umlcd"]=true,},
  ["manfnt"] = {},
  ["beramono"] = {["bera"]=true,},
  ["lwarp-makeidx"] = {["lwarp"]=true,},
  ["bbold"] = {["bbold"]=true,},
  ["spectra.data"] = {["pgf-spectra"]=true,},
  ["pgflibraryluamath.code"] = {["pgf"]=true,},
  ["infr-ex"] = {["gustlib"]=true,},
  ["pgf-soroban"] = {["pgf-soroban"]=true,},
  ["gmutils"] = {["gmutils"]=true,},
  ["tikzlibraryshadows.blur.code"] = {["pgf-blur"]=true,},
  ["xxcolor"] = {["pgf"]=true,},
  ["protecteddef"] = {["oberdiek"]=true,},
  ["talk-guit2008"] = {},
  ["dotlessi"] = {["bezos"]=true,},
  ["tikzexternal"] = {["pgf"]=true,},
  ["pgfrcs"] = {["pgf"]=true,},
  ["pgfpages"] = {["pgf"]=true,},
  ["exsheets-listings"] = {["exsheets"]=true,},
  ["pgfkeys"] = {["pgf"]=true,},
  ["acmart"] = {["acmart"]=true,},
  ["pgffor"] = {["pgf"]=true,},
  ["tikz"] = {["pgf"]=true,},
  ["pgfpict2e"] = {["pgf"]=true,},
  ["storebox-pgf"] = {["storebox"]=true,},
  ["pgfmanual.prettyprinter.code"] = {["pgf"]=true,["tikz-dependency"]=true,},
  ["pgfmanual.pdflinks.code"] = {["pgf"]=true,["tikz-dependency"]=true,},
  ["novel-Obsolete"] = {["novel"]=true,},
  ["pgflibrarytikzbackgrounds"] = {["pgf"]=true,},
  ["amsbook"] = {["amscls"]=true,},
  ["delimtxt"] = {["delimtxt"]=true,},
  ["pgflibraryplothandlers"] = {["pgf"]=true,},
  ["pgflibraryshapes.misc.code"] = {["pgf"]=true,},
  ["example-glossaries-cite"] = {["glossaries"]=true,},
  ["pgfheaps"] = {["pgf"]=true,},
  ["ketmedia"] = {["ketcindy"]=true,},
  ["at"] = {["mdwtools"]=true,},
  ["pgfcomp-version-1-18"] = {["pgf"]=true,},
  ["pgfcore"] = {["pgf"]=true,},
  ["AnonymousPro"] = {["anonymouspro"]=true,},
  ["loadhyph-cs"] = {},
  ["matc3mem"] = {["matc3mem"]=true,},
  ["plantslabels"] = {["plantslabels"]=true,},
  ["gu"] = {["gu"]=true,},
  ["example-image-a5"] = {["mwe"]=true,},
  ["tikzlibrarylindenmayersystems.code"] = {["pgf"]=true,},
  ["bidi"] = {["bidi"]=true,},
  ["comfortaa"] = {["comfortaa"]=true,},
  ["pgffor.code"] = {["pgf"]=true,},
  ["moderncvcollection"] = {["moderncv"]=true,},
  ["hyph-lt"] = {},
  ["m-ch-nl"] = {["context"]=true,},
  ["everyhook"] = {["everyhook"]=true,},
  ["pgfbasepatterns"] = {["pgf"]=true,},
  ["lhcyrkoi-rhyphen"] = {},
  ["pgfbasematrix"] = {["pgf"]=true,},
  ["lwarp-epstopdf"] = {["lwarp"]=true,},
  ["tabularborder"] = {["tabularborder"]=true,},
  ["talk-bacho08"] = {},
  ["extlabels"] = {["macros2e"]=true,},
  ["gtl"] = {["gtl"]=true,},
  ["mitpress"] = {},
  ["tufte"] = {["context"]=true,},
  ["pgfrcs.code"] = {["pgf"]=true,},
  ["gedalin"] = {["arabtex"]=true,},
  ["pgfkeysfiltered.code"] = {["pgf"]=true,},
  ["pgfkeys.code"] = {["pgf"]=true,},
  ["newpxtext"] = {["newpx"]=true,},
  ["pgfbaseplot"] = {["pgf"]=true,},
  ["pgfexternalwithdepth"] = {["pgf"]=true,},
  ["pgfexternal"] = {["pgf"]=true,},
  ["pgfcalendar.code"] = {["pgf"]=true,},
  ["oegatb"] = {["economic"]=true,},
  ["pgfsys.code"] = {["pgf"]=true,},
  ["pgfmodulesnakes.code"] = {["pgf"]=true,},
  ["babel-azerbaijani-latn"] = {["babel"]=true,},
  ["NumericPlots"] = {["numericplots"]=true,},
  ["pgfmoduleparser.code"] = {["pgf"]=true,},
  ["harveyballs"] = {["harveyballs"]=true,},
  ["microtype"] = {["microtype"]=true,},
  ["babel-sena"] = {["babel"]=true,},
  ["pgfmodulenonlineartransformations.code"] = {["pgf"]=true,},
  ["pgfplotscore.code"] = {["pgfplots"]=true,},
  ["relenc"] = {["relenc"]=true,},
  ["pgfmodulebending.code"] = {["pgf"]=true,},
  ["pgfmathutil.code"] = {["pgf"]=true,},
  ["xpinyin"] = {["xpinyin"]=true,},
  ["edfnotes"] = {["edfnotes"]=true,},
  ["pgfmathfunctions.trigonometric.code"] = {["pgf"]=true,},
  ["aiaa-tc"] = {["aiaa"]=true,},
  ["acroterm"] = {["acroterm"]=true,},
  ["soul"] = {["soul"]=true,},
  ["bidituftehyperref"] = {["bidi"]=true,},
  ["xkvex3"] = {["xkeyval"]=true,},
  ["pgfmathfunctions.misc.code"] = {["pgf"]=true,},
  ["moreenum"] = {["moreenum"]=true,},
  ["HAPTCS"] = {["ha-prosper"]=true,},
  ["protosem"] = {["archaic"]=true,},
  ["ngermanb"] = {["babel"]=true,},
  ["gatech-thesis-losa"] = {["gatech-thesis"]=true,},
  ["hyph-de-1901.ec"] = {},
  ["letteracdp"] = {["cdpbundl"]=true,},
  ["pgfmathfloat.code"] = {["pgf"]=true,},
  ["polish"] = {["babel"]=true,},
  ["utf8"] = {["arabtex"]=true,},
  ["pgflibraryautomata"] = {["pgf"]=true,},
  ["cals"] = {["cals"]=true,},
  ["mtx-context-domotica"] = {["context"]=true,},
  ["PPRwhitecross"] = {["prosper"]=true,},
  ["plipsum"] = {["plipsum"]=true,},
  ["babel-pashto"] = {["babel"]=true,},
  ["pgflibraryshapes.gates.logic.IEC.code"] = {["pgf"]=true,},
  ["tabto"] = {["tabto-ltx"]=true,},
  ["pgflibrarysvg.path.code"] = {["pgf"]=true,},
  ["hecthese"] = {["hecthese"]=true,},
  ["pgflibrarysnakes.code"] = {["pgf"]=true,},
  ["japanlco"] = {},
  ["2up"] = {["2up"]=true,},
  ["amstext"] = {["amsmath"]=true,},
  ["linguex"] = {["linguex"]=true,},
  ["isov2"] = {["iso"]=true,},
  ["tikzlibraryquotes.code"] = {["pgf"]=true,},
  ["dvipscol"] = {["oberdiek"]=true,},
  ["pgflibraryplothandlers.code"] = {["pgf"]=true,},
  ["pgflibrarypatterns.meta.code"] = {["pgf"]=true,},
  ["flafter"] = {["latex"]=true,},
  ["tikzlibraryshapes.gates.logic.US.code"] = {["pgf"]=true,},
  ["Eichenla"] = {["cfr-initials"]=true,},
  ["il2-t1"] = {["enctex"]=true,},
  ["lion-msc"] = {["lion-msc"]=true,},
  ["loadhyph-mul-ethi"] = {},
  ["lltjp-xunicode"] = {["luatexja"]=true,},
  ["english"] = {["babel"]=true,},
  ["pgflibraryfixedpointarithmetic.code"] = {["pgf"]=true,},
  ["babel-sakha"] = {["babel"]=true,},
  ["pgflibraryfadings.code"] = {["pgf"]=true,},
  ["pgflibraryarrows.spaced.code"] = {["pgf"]=true,},
  ["stfloats"] = {["sttools"]=true,},
  ["dvipaste"] = {["export"]=true,},
  ["avr.instr"] = {["avremu"]=true,},
  ["eskdtotal"] = {["eskdx"]=true,},
  ["pgf-spectra"] = {["pgf-spectra"]=true,},
  ["beamerthemeBerkeley"] = {["beamer"]=true,},
  ["pgflibrarydecorations.text.code"] = {["pgf"]=true,},
  ["babel-tibetan"] = {["babel"]=true,},
  ["Chivo"] = {["chivo"]=true,},
  ["centernot"] = {["oberdiek"]=true,},
  ["chessboard-pgf"] = {["chessboard"]=true,},
  ["pgflibrarydecorations.pathmorphing.code"] = {["pgf"]=true,},
  ["fithesis2"] = {["fithesis"]=true,},
  ["pgflibrarydatavisualization.polar.code"] = {["pgf"]=true,},
  ["biblatex-multiple-dm"] = {["biblatex-multiple-dm"]=true,},
  ["bytefield"] = {["bytefield"]=true,},
  ["tikzposterColorpalettes"] = {["tikzposter"]=true,},
  ["europroc"] = {["dcpic"]=true,},
  ["babel-urdu"] = {["babel"]=true,},
  ["calc"] = {["tools"]=true,},
  ["hebrew_oldcode"] = {["babel-hebrew"]=true,},
  ["ethiop"] = {["ethiop"]=true,},
  ["pgflibrarygraphdrawing.layered.code"] = {["pgf"]=true,},
  ["pst-node97"] = {["pst-node"]=true,},
  ["pgfcircshapes"] = {["circuitikz"]=true,},
  ["swedish"] = {["babel"]=true,},
  ["fixpdfmag"] = {},
  ["pgflibrarygraphdrawing.code"] = {["pgf"]=true,},
  ["tikzlibraryturtle.code"] = {["pgf"]=true,},
  ["beamercolorthemefibeamer-mu-ped"] = {["fibeamer"]=true,},
  ["beamericonbook"] = {["beamer"]=true,},
  ["grafcet"] = {["grafcet"]=true,},
  ["tikzlibrarytopaths.code"] = {["pgf"]=true,},
  ["tikzlibrarythrough.code"] = {["pgf"]=true,},
  ["mleftright"] = {["oberdiek"]=true,},
  ["lwarp-scrextend"] = {["lwarp"]=true,},
  ["cellspace"] = {["cellspace"]=true,},
  ["zhfandolfonts"] = {["ctex"]=true,},
  ["tikzlibrarysvg.path.code"] = {["pgf"]=true,},
  ["ean8"] = {["ean"]=true,},
  ["beamerthemefibeamer"] = {["fibeamer"]=true,},
  ["barddoc"] = {["bardiag"]=true,},
  ["jrurstud"] = {["economic"]=true,},
  ["fullwidth"] = {["fullwidth"]=true,},
  ["flags"] = {["oberdiek"]=true,},
  ["tikzlibraryshapes.symbols.code"] = {["pgf"]=true,},
  ["pkgloader-recommended"] = {["pkgloader"]=true,},
  ["ukraineb"] = {["babel"]=true,},
  ["pgflibraryshapes.arrows.code"] = {["pgf"]=true,},
  ["lwarp-titletoc"] = {["lwarp"]=true,},
  ["babel-kannada"] = {["babel"]=true,},
  ["dhucs-trivcj"] = {["kotex-utf"]=true,},
  ["axessibility"] = {["axessibility"]=true,},
  ["pazo-math"] = {},
  ["tikzlibraryshapes.code"] = {["pgf"]=true,},
  ["bpfap8"] = {["iso10303"]=true,},
  ["tikzlibraryshapes.arrows.code"] = {["pgf"]=true,},
  ["beamerthemeboxes"] = {["beamer"]=true,},
  ["asyprocess"] = {["asyfig"]=true,},
  ["dinbrief"] = {["dinbrief"]=true,},
  ["elteikthesis"] = {["elteikthesis"]=true,},
  ["tikzlibraryshadings.code"] = {["pgf"]=true,},
  ["CD_labeler_test"] = {},
  ["arabic"] = {},
  ["nth"] = {},
  ["fnspe"] = {["fnspe"]=true,},
  ["phfparen"] = {["phfparen"]=true,},
  ["pgflibraryshadings.code"] = {["pgf"]=true,},
  ["tikzlibrarybackgrounds.code"] = {["pgf"]=true,},
  ["DraTex"] = {["dratex"]=true,},
  ["tikzlibrarypositioning.code"] = {["pgf"]=true,},
  ["lisp-simple-alloc"] = {["lisp-on-tex"]=true,},
  ["tikzlibraryplotmarks.code"] = {["pgf"]=true,},
  ["ketpic2e"] = {["ketcindy"]=true,},
  ["babel-gujarati"] = {["babel"]=true,},
  ["ajae"] = {["economic"]=true,},
  ["thm-beamer"] = {["thmtools"]=true,},
  ["bibl-apa"] = {["context"]=true,},
  ["mychemistry"] = {["mychemistry"]=true,},
  ["tikzlibraryfixedpointarithmetic.code"] = {["pgf"]=true,},
  ["fixjfm"] = {["fixjfm"]=true,},
  ["tikzlibraryfadings.code"] = {["pgf"]=true,},
  ["ifthen"] = {["latex"]=true,},
  ["memory"] = {["memory"]=true,},
  ["babel-lao"] = {["babel"]=true,},
  ["tikzlibrarydecorations.pathmorphing.code"] = {["pgf"]=true,},
  ["loadhyph-te"] = {},
  ["scrlttr2"] = {},
  ["beamerthemesidebar"] = {["beamer"]=true,},
  ["txi-tr"] = {},
  ["tikzlibrarybending.code"] = {["pgf"]=true,},
  ["font_iwona-condensed"] = {["font-change"]=true,},
  ["babel-shona"] = {["babel"]=true,},
  ["tikzlibrarygraphs.code"] = {["pgf"]=true,},
  ["TXSmemo"] = {["texsis"]=true,},
  ["tikz-timing-columntype"] = {["tikz-timing"]=true,},
  ["listout"] = {["dialogl"]=true,},
  ["tikzlibrarydatavisualization.3d.code"] = {["pgf"]=true,},
  ["babel-luxembourgish"] = {["babel"]=true,},
  ["tikzlibrarycircuits.logic.code"] = {["pgf"]=true,},
  ["tikzlibrarycircuits.logic.US.code"] = {["pgf"]=true,},
  ["beamerthemeTorinoTh"] = {["beamer2thesis"]=true,},
  ["dynamicnumber"] = {["dynamicnumber"]=true,},
  ["babel-rwa"] = {["babel"]=true,},
  ["babel-uzbek-latin"] = {["babel"]=true,},
  ["beamercolorthemefibeamer-mu-med"] = {["fibeamer"]=true,},
  ["gillius"] = {["gillius"]=true,},
  ["tikzlibrarycircuits.ee.IEC.code"] = {["pgf"]=true,},
  ["babel-filipino"] = {["babel"]=true,},
  ["babel-usorbian"] = {["babel"]=true,},
  ["hyph-eu"] = {},
  ["asapsym"] = {["asapsym"]=true,},
  ["tikzlibrarycircuits.code"] = {["pgf"]=true,},
  ["pgfcoretransparency.code"] = {["pgf"]=true,},
  ["babel-malagasy"] = {["babel"]=true,},
  ["afthes12"] = {["afthesis"]=true,},
  ["mimulcol"] = {["gustlib"]=true,},
  ["optparams"] = {["sauerj"]=true,},
  ["pgfcorescopes.code"] = {["pgf"]=true,},
  ["mtx-context-ideas"] = {["context"]=true,},
  ["paralist"] = {["paralist"]=true,},
  ["NotesTeX"] = {["notestex"]=true,},
  ["pgfcorepathconstruct.code"] = {["pgf"]=true,},
  ["cmpj3"] = {["cmpj"]=true,},
  ["pgfcoreobjects.code"] = {["pgf"]=true,},
  ["pgfcoreimage.code"] = {["pgf"]=true,},
  ["pgfmathfunctions.random.code"] = {["pgf"]=true,},
  ["cryptocode"] = {["cryptocode"]=true,},
  ["pythonhighlight"] = {["pythonhighlight"]=true,},
  ["egameps"] = {["egameps"]=true,},
  ["utf8-t1"] = {["enctex"]=true,},
  ["fnpara"] = {["fnpara"]=true,},
  ["pgfcore.code"] = {["pgf"]=true,},
  ["hypdoc"] = {["oberdiek"]=true,},
  ["includeRnw"] = {["includernw"]=true,},
  ["t-pgfmod"] = {["pgf"]=true,},
  ["fontbook"] = {["fontbook"]=true,},
  ["beamerfontthemeDetlevCM"] = {["beamertheme-detlevcm"]=true,},
  ["t-pgfkey"] = {["pgf"]=true,},
  ["lollipop-lists"] = {["lollipop"]=true,},
  ["lscape"] = {["graphics"]=true,},
  ["t-pgfcal"] = {["pgf"]=true,},
  ["tikzlibraryexternal.code"] = {["pgf"]=true,},
  ["musixec"] = {["musixtex"]=true,},
  ["philosophersimprint"] = {["philosophersimprint"]=true,},
  ["hewrite"] = {["arabtex"]=true,},
  ["t-pgfbma"] = {["pgf"]=true,},
  ["t-pgfbla"] = {["pgf"]=true,},
  ["t-pgf"] = {["pgf"]=true,},
  ["pfarrei"] = {["pfarrei"]=true,},
  ["dowith"] = {["dowith"]=true,},
  ["mikoaffiliation"] = {["stex"]=true,},
  ["continue"] = {["continue"]=true,},
  ["expl3-generic"] = {["l3kernel"]=true,},
  ["tipfr"] = {["tipfr"]=true,},
  ["overpic"] = {["overpic"]=true,},
  ["nevelok"] = {["nevelok"]=true,},
  ["gregoriosyms"] = {["gregoriotex"]=true,},
  ["ginpenc"] = {["ginpenc"]=true,},
  ["pdfwin"] = {["pdfwin"]=true,},
  ["nih"] = {["nih"]=true,},
  ["pdftricks"] = {["pdftricks"]=true,},
  ["matc3"] = {["matc3"]=true,},
  ["loadhyph-cop"] = {},
  ["glyphtounicode"] = {["pdftex"]=true,},
  ["alluncl"] = {["bookhands"]=true,},
  ["konames-utf"] = {["cjk-ko"]=true,},
  ["pgfcircvoltage"] = {["circuitikz"]=true,},
  ["bpfats6"] = {["iso10303"]=true,},
  ["babel-sanskrit-knda"] = {["babel"]=true,},
  ["pdfreview"] = {["pdfreview"]=true,},
  ["beamercolorthemeorchid"] = {["beamer"]=true,},
  ["manyfoot"] = {["ncctools"]=true,},
  ["pdfprivacy"] = {["pdfprivacy"]=true,},
  ["fp-addons"] = {["fp"]=true,},
  ["mup"] = {["abc"]=true,},
  ["s-simpleslides-Shaded"] = {["context-simpleslides"]=true,},
  ["pdfoverlay"] = {["pdfoverlay"]=true,},
  ["lineno"] = {["lineno"]=true,},
  ["pdfcomment"] = {["pdfcomment"]=true,},
  ["eepicemu"] = {["eepic"]=true,},
  ["multibib"] = {["multibib"]=true,},
  ["europasscv"] = {["europasscv"]=true,},
  ["pbsheet"] = {["pbsheet"]=true,},
  ["asciilist"] = {["asciilist"]=true,},
  ["gastex"] = {["gastex"]=true,},
  ["dhucs-paralist"] = {["kotex-utf"]=true,},
  ["pb-xy"] = {["pb-diagram"]=true,},
  ["thm-autoref"] = {["thmtools"]=true,},
  ["chancery"] = {["psnfss"]=true,},
  ["pax"] = {["pax"]=true,},
  ["PTSerifCaption"] = {["paratype"]=true,},
  ["teixmlslides"] = {},
  ["teixml"] = {},
  ["nrc2"] = {["nrc"]=true,},
  ["babel-italian"] = {["babel"]=true,},
  ["xycmactex"] = {["xypic"]=true,},
  ["pas-cv"] = {["pas-cv"]=true,},
  ["macro-venus"] = {["pas-cv"]=true,},
  ["pgfplotsbinary.code"] = {["pgfplots"]=true,},
  ["macro-saturne"] = {["pas-cv"]=true,},
  ["macro-pluton"] = {["pas-cv"]=true,},
  ["uncial"] = {["bookhands"]=true,},
  ["lwarp-syntonly"] = {["lwarp"]=true,},
  ["ryethesis"] = {["ryethesis"]=true,},
  ["nicefilelist"] = {["nicefilelist"]=true,},
  ["msg"] = {["msg"]=true,},
  ["ntheorem-hyper"] = {["hyperref"]=true,},
  ["lwarp-titleref"] = {["lwarp"]=true,},
  ["tcbexternal.code"] = {["tcolorbox"]=true,},
  ["babel-igbo"] = {["babel"]=true,},
  ["babel-sanskrit-devanagari"] = {["babel"]=true,},
  ["macro-jupiter"] = {["pas-cv"]=true,},
  ["arabnovowel"] = {["arabi"]=true,},
  ["mtx-context-xml"] = {["context"]=true,},
  ["extletter"] = {["extsizes"]=true,},
  ["shapecm"] = {["codepage"]=true,},
  ["emp"] = {["emp"]=true,},
  ["fmp"] = {["fmp"]=true,},
  ["macro-gaia"] = {["pas-cv"]=true,},
  ["apxproof"] = {["apxproof"]=true,},
  ["CJKspace"] = {["cjk"]=true,},
  ["ktv-buildnum"] = {["ktv-texdata"]=true,},
  ["PPRcontemporain"] = {["prosper"]=true,},
  ["hcslides"] = {["hc"]=true,},
  ["font_libertine_kp"] = {["font-change"]=true,},
  ["biblatex-bookinarticle"] = {["biblatex-bookinarticle"]=true,},
  ["pas-crosswords"] = {["pas-crosswords"]=true,},
  ["pas-cours"] = {["pas-cours"]=true,},
  ["databar"] = {["datatool"]=true,},
  ["luatexbase-cctb"] = {["luatexbase"]=true,},
  ["worlddev"] = {["economic"]=true,},
  ["paresse"] = {["paresse"]=true,},
  ["calrsfs"] = {["calrsfs"]=true,},
  ["lwarp-parnotes"] = {["lwarp"]=true,},
  ["ucharacters"] = {},
  ["PTSerif"] = {["paratype"]=true,},
  ["tsp"] = {["gustlib"]=true,},
  ["utf8raw"] = {["enctex"]=true,},
  ["fast-diagram"] = {["fast-diagram"]=true,},
  ["easyeqn"] = {["easy"]=true,},
  ["interfaces-hypbmsec"] = {["interfaces"]=true,},
  ["disser"] = {["disser"]=true,},
  ["PTMono"] = {["paratype"]=true,},
  ["lwarp-easy-todo"] = {["lwarp"]=true,},
  ["parallel"] = {["parallel"]=true,},
  ["pgfcorequick.code"] = {["pgf"]=true,},
  ["nccfoots"] = {["ncctools"]=true,},
  ["paracol"] = {["paracol"]=true,},
  ["montserrat"] = {["montserrat"]=true,},
  ["papertex"] = {["papertex"]=true,},
  ["papercdcase"] = {["papercdcase"]=true,},
  ["babel-inarisami"] = {["babel"]=true,},
  ["linop"] = {["linop"]=true,},
  ["permute"] = {["permute"]=true,},
  ["komacv-lco"] = {["komacv-rg"]=true,},
  ["pagerange"] = {["pagerange"]=true,},
  ["pagenote"] = {["pagenote"]=true,},
  ["schullzk"] = {["schule"]=true,},
  ["lwarp-soul"] = {["lwarp"]=true,},
  ["lwarp-endheads"] = {["lwarp"]=true,},
  ["moderncvcolorgreen"] = {["moderncv"]=true,},
  ["beamercolorthemedolphin"] = {["beamer"]=true,},
  ["beamercolorthemefibeamer-mu-law"] = {["fibeamer"]=true,},
  ["rec-thy"] = {["rec-thy"]=true,},
  ["pecha"] = {["pecha"]=true,},
  ["beamerinnerthemecircles"] = {["beamer"]=true,},
  ["overlock"] = {["overlock"]=true,},
  ["lltjp-siunitx"] = {["luatexja"]=true,},
  ["overlays"] = {["overlays"]=true,},
  ["outliner"] = {},
  ["zref-pageattr"] = {["oberdiek"]=true,},
  ["outline"] = {["outline"]=true,},
  ["otibet"] = {["otibet"]=true,},
  ["seqsplit"] = {["seqsplit"]=true,},
  ["lwarp-splitidx"] = {["lwarp"]=true,},
  ["s-pre-67"] = {["context"]=true,},
  ["ot-tableau"] = {["ot-tableau"]=true,},
  ["eqlist"] = {["eqlist"]=true,},
  ["translit"] = {["arabi"]=true,},
  ["opcit"] = {["opcit"]=true,},
  ["authblk"] = {["preprint"]=true,},
  ["beamerbasearticle"] = {["beamer"]=true,},
  ["icelandic"] = {["babel"]=true,},
  ["tocenter"] = {["ncctools"]=true,},
  ["grlccode"] = {["omega"]=true,},
  ["fei"] = {["fei"]=true,},
  ["llist"] = {},
  ["niceverb"] = {["nicetext"]=true,},
  ["scancsv"] = {["olsak-misc"]=true,},
  ["tikzrput"] = {["pgfornament"]=true,},
  ["hlist"] = {["hlist"]=true,},
  ["lwarp-emptypage"] = {["lwarp"]=true,},
  ["knowledge"] = {["knowledge"]=true,},
  ["example-image-a"] = {["mwe"]=true,},
  ["stex"] = {["stex"]=true,},
  ["tikzlibraryrulercompass.code"] = {["rulercompass"]=true,},
  ["oinuit"] = {["oinuit"]=true,},
  ["bpfap14"] = {["iso10303"]=true,},
  ["txfn"] = {["ofs"]=true,},
  ["introduction"] = {},
  ["klucite"] = {["kluwer"]=true,},
  ["xyweb"] = {["xypic"]=true,},
  ["vertbars"] = {["vertbars"]=true,},
  ["autolist"] = {["shipunov"]=true,},
  ["lwarp-floatflt"] = {["lwarp"]=true,},
  ["ofs-mt"] = {["ofs"]=true,},
  ["ofs-ams"] = {["ofs"]=true,},
  ["wlist"] = {},
  ["gmdoc-enhance"] = {["gmdoc-enhance"]=true,},
  ["embedfile"] = {["oberdiek"]=true,},
  ["ofs-8x"] = {["ofs"]=true,},
  ["floatrow"] = {["floatrow"]=true,},
  ["memoir"] = {["memoir"]=true,},
  ["ofs-6y"] = {["ofs"]=true,},
  ["ofs-6x"] = {["ofs"]=true,},
  ["ifluatex"] = {["ifluatex"]=true,},
  ["ofs-6t"] = {["ofs"]=true,},
  ["elsarticle"] = {["elsarticle"]=true,},
  ["ofs-6s"] = {["ofs"]=true,},
  ["ofs-6k"] = {["ofs"]=true,},
  ["babel-esperanto"] = {["babel"]=true,},
  ["dottex"] = {["dottex"]=true,},
  ["csenc-k"] = {},
  ["tikzpeople.shape.bride"] = {["tikzpeople"]=true,},
  ["ucode"] = {},
  ["babel-serbian-latin"] = {["babel"]=true,},
  ["mtfn"] = {["ofs"]=true,},
  ["loadhyph-kn"] = {},
  ["musixper"] = {["musixtex"]=true,},
  ["guitarchordschemes"] = {["guitarchordschemes"]=true,},
  ["GoMono"] = {["gofonts"]=true,},
  ["cv4tw-theme-simple"] = {["cv4tw"]=true,},
  ["luatex-plain"] = {["context"]=true,},
  ["gmurl"] = {["gmutils"]=true,},
  ["babel-lithuanian"] = {["babel"]=true,},
  ["hyph-eu.ec"] = {},
  ["hypernat"] = {["hypernat"]=true,},
  ["ocr"] = {["ocr-latex"]=true,},
  ["l3str-convert"] = {["l3experimental"]=true,},
  ["estonian"] = {["babel"]=true,},
  ["lwarp-changebar"] = {["lwarp"]=true,},
  ["fixocgx"] = {["ocgx2"]=true,},
  ["rotunda"] = {["bookhands"]=true,},
  ["tikzlibraryocgx.code"] = {["ocgx"]=true,},
  ["loadhyph-mn-cyrl-x-lmc"] = {},
  ["kluref"] = {["kluwer"]=true,},
  ["ocgx"] = {["ocgx"]=true,},
  ["cmsdocs"] = {["biblatex-chicago"]=true,},
  ["etex"] = {["etex-pkg"]=true,},
  ["ocg-p"] = {["ocg-p"]=true,},
  ["nabatean"] = {["archaic"]=true,},
  ["bxorigcapt"] = {["bxorigcapt"]=true,},
  ["zref"] = {["oberdiek"]=true,},
  ["zref-totpages"] = {["oberdiek"]=true,},
  ["babel-french-be"] = {["babel"]=true,},
  ["zref-titleref"] = {["oberdiek"]=true,},
  ["conv-utf8-qx"] = {["hyph-utf8"]=true,},
  ["doclicense-CC-by-nc-4.0-plaintext"] = {["doclicense"]=true,},
  ["basque-date"] = {["basque-date"]=true,},
  ["hyphen"] = {},
  ["simurgh-algorithm"] = {["simurgh"]=true,},
  ["filehook-scrlfile"] = {["filehook"]=true,},
  ["loadhyph-bn"] = {},
  ["zref-nextpage"] = {["oberdiek"]=true,},
  ["beamerbasetoc"] = {["beamer"]=true,},
  ["ClearSans"] = {["clearsans"]=true,},
  ["kashmiri"] = {["arabtex"]=true,},
  ["zref-marks"] = {["oberdiek"]=true,},
  ["ifplatform"] = {["ifplatform"]=true,},
  ["logpap"] = {["logpap"]=true,},
  ["babel-serbian-cyrillic-montenegro"] = {["babel"]=true,},
  ["zref-lastpage"] = {["oberdiek"]=true,},
  ["gcite"] = {["gcite"]=true,},
  ["zref-dotfill"] = {["oberdiek"]=true,},
  ["textwidthfootnoterule"] = {["luabidi"]=true,},
  ["randomlist"] = {["randomlist"]=true,},
  ["smartmn"] = {["jknapltx"]=true,},
  ["zref-abspage"] = {["oberdiek"]=true,},
  ["xcolor-patch"] = {["oberdiek"]=true,},
  ["cs-all"] = {},
  ["datetime"] = {["datetime"]=true,},
  ["twoopt"] = {["oberdiek"]=true,},
  ["gitexinfo"] = {["gitinfo2"]=true,},
  ["nbaseprt"] = {["numprint"]=true,},
  ["codepage"] = {["codepage"]=true,},
  ["blogdot"] = {["morehype"]=true,},
  ["tabularht"] = {["oberdiek"]=true,},
  ["yiddish"] = {["arabtex"]=true,},
  ["stampinclude"] = {["oberdiek"]=true,},
  ["babel-australian"] = {["babel"]=true,},
  ["PPRrico"] = {["prosper"]=true,},
  ["beamerthemeshadow"] = {["beamer"]=true,},
  ["selinput"] = {["oberdiek"]=true,},
  ["hyph-fr"] = {},
  ["mafr"] = {["mafr"]=true,},
  ["lualatex-math"] = {["lualatex-math"]=true,},
  ["xucuri"] = {["mxedruli"]=true,},
  ["fixfoot"] = {["fixfoot"]=true,},
  ["dhucs-sectsty"] = {["kotex-utf"]=true,},
  ["emulateapj"] = {["emulateapj"]=true,},
  ["caption2"] = {["caption"]=true,},
  ["beamercolorthemeMedStarColors"] = {["medstarbeamer"]=true,},
  ["rerunfilecheck"] = {["oberdiek"]=true,},
  ["eskdlist"] = {["eskdx"]=true,},
  ["hyph-zh-latn-pinyin"] = {},
  ["loop"] = {["midnight"]=true,},
  ["pdflscape"] = {["oberdiek"]=true,},
  ["pdfcolparcolumns"] = {["oberdiek"]=true,},
  ["minitoc"] = {["minitoc"]=true,},
  ["handout"] = {["handout"]=true,},
  ["askmaps"] = {["askmaps"]=true,},
  ["pdfcolfoot"] = {["oberdiek"]=true,},
  ["pagegrid"] = {["oberdiek"]=true,},
  ["simurgh-amstext"] = {["simurgh"]=true,},
  ["fourier-orns"] = {["fourier"]=true,},
  ["loadhyph-zh-latn-pinyin"] = {},
  ["doclicense-CC-zero-1.0-latex"] = {["doclicense"]=true,},
  ["onlyamsmath"] = {["onlyamsmath"]=true,},
  ["gmp"] = {["gmp"]=true,},
  ["makeplot"] = {["makeplot"]=true,},
  ["kvoptions-patch"] = {["oberdiek"]=true,},
  ["nccthm"] = {["ncctools"]=true,},
  ["uighur"] = {["arabtex"]=true,},
  ["he-she"] = {["he-she"]=true,},
  ["pdfcwebmac"] = {["cweb"]=true,},
  ["tocbasic"] = {},
  ["t-pgfrcs"] = {["pgf"]=true,},
  ["tbook"] = {["platex"]=true,},
  ["lwarp-gridset"] = {["lwarp"]=true,},
  ["s-pre-18"] = {["context"]=true,},
  ["hopatch"] = {["oberdiek"]=true,},
  ["holtxdoc"] = {["oberdiek"]=true,},
  ["cp1256"] = {["arabtex"]=true,},
  ["cnltx-example"] = {["cnltx"]=true,},
  ["Royal"] = {["cfr-initials"]=true,},
  ["akkmathrel"] = {["akktex"]=true,},
  ["carbohydrates"] = {["carbohydrates"]=true,},
  ["method"] = {["method"]=true,},
  ["xytpic"] = {["xypic"]=true,},
  ["tikzlibrarysnakes.code"] = {["pgf"]=true,},
  ["ebook"] = {["ebook"]=true,},
  ["harmony"] = {["harmony"]=true,},
  ["barcodes"] = {["barcodes"]=true,},
  ["epstopdf"] = {["oberdiek"]=true,},
  ["dtx-attach"] = {["oberdiek"]=true,},
  ["bbook"] = {["bangtex"]=true,},
  ["loadhyph-mn-cyrl"] = {},
  ["lwarp-algorithm2e"] = {["lwarp"]=true,},
  ["pdfmarginpar"] = {["pdfmarginpar"]=true,},
  ["complexity"] = {["complexity"]=true,},
  ["babel-estonian"] = {["babel"]=true,},
  ["pgflibrarydecorations.pathreplacing.code"] = {["pgf"]=true,},
  ["dynblocks"] = {["dynblocks"]=true,},
  ["beamerthemeI6pd"] = {["beamerposter"]=true,},
  ["bmpsize-test"] = {["oberdiek"]=true,},
  ["kanaparser"] = {["kanaparser"]=true,},
  ["bmpsize-base"] = {["oberdiek"]=true,},
  ["auxhook"] = {["oberdiek"]=true,},
  ["mlp"] = {["e-french"]=true,},
  ["atveryend"] = {["oberdiek"]=true,},
  ["atenddvi"] = {["oberdiek"]=true,},
  ["lwarp-phfqit"] = {["lwarp"]=true,},
  ["esperanto"] = {["babel"]=true,},
  ["erdc"] = {["erdc"]=true,},
  ["xydvidrv"] = {["xypic"]=true,},
  ["gtrlang.dutch.code"] = {["genealogytree"]=true,},
  ["thepdfnumber"] = {["oberdiek"]=true,},
  ["telprint"] = {["oberdiek"]=true,},
  ["biblatex-source-division"] = {["biblatex-source-division"]=true,},
  ["stringenc"] = {["oberdiek"]=true,},
  ["soulutf8"] = {["oberdiek"]=true,},
  ["lwarp"] = {["lwarp"]=true,},
  ["pdftexcmds"] = {["oberdiek"]=true,},
  ["CJKulem"] = {["cjk"]=true,},
  ["pdfescape"] = {["oberdiek"]=true,},
  ["greekdates"] = {["greekdates"]=true,},
  ["relabel"] = {["computational-complexity"]=true,},
  ["minimum"] = {["tableaux"]=true,},
  ["pdfcol"] = {["oberdiek"]=true,},
  ["pgfutil-common"] = {["pgf"]=true,},
  ["akkmathset"] = {["akktex"]=true,},
  ["genealogytree"] = {["genealogytree"]=true,},
  ["kvsetkeys"] = {["oberdiek"]=true,},
  ["kvdefinekeys"] = {["oberdiek"]=true,},
  ["georgiancaps"] = {["babel-georgian"]=true,},
  ["aertt"] = {["economic"]=true,},
  ["xygraph"] = {["xypic"]=true,},
  ["moderncvcolorblack"] = {["moderncv"]=true,},
  ["hologo"] = {["oberdiek"]=true,},
  ["ghsystem_flame-O-5-1"] = {["ghsystem"]=true,},
  ["hobsub-hyperref"] = {["oberdiek"]=true,},
  ["fibnum"] = {["oberdiek"]=true,},
  ["hyph-sk.ec"] = {},
  ["code437"] = {["codepage"]=true,},
  ["cuisine"] = {["cuisine"]=true,},
  ["etexcmds"] = {["oberdiek"]=true,},
  ["fithesis-fss"] = {["fithesis"]=true,},
  ["catchfile"] = {["oberdiek"]=true,},
  ["bitset"] = {["oberdiek"]=true,},
  ["lwarp-romanbarpagenumber"] = {["lwarp"]=true,},
  ["fxenvlayoutcolorsig"] = {["fixme"]=true,},
  ["gentombow"] = {["gentombow"]=true,},
  ["exercises"] = {["exercises"]=true,},
  ["atbegshi"] = {["oberdiek"]=true,},
  ["rotpages"] = {["rotpages"]=true,},
  ["ktv-texdata"] = {["ktv-texdata"]=true,},
  ["checklistings"] = {["checklistings"]=true,},
  ["doclicense-CC-by-nc-4.0-latex"] = {["doclicense"]=true,},
  ["ifoption"] = {["amsrefs"]=true,},
  ["numspell-magyar"] = {["numspell"]=true,},
  ["numspell-italian"] = {["numspell"]=true,},
  ["numspell-english"] = {["numspell"]=true,},
  ["numprint"] = {["numprint"]=true,},
  ["khatt-ar"] = {["context"]=true,},
  ["example-glossaries-parent"] = {["glossaries"]=true,},
  ["econometrics"] = {["econometrics"]=true,},
  ["numname"] = {["numname"]=true,},
  ["NumericPlots_macros"] = {["numericplots"]=true,},
  ["NumericPlots_labels"] = {["numericplots"]=true,},
  ["beamerthemePittsburgh"] = {["beamer"]=true,},
  ["threeparttablex"] = {["threeparttablex"]=true,},
  ["arabskel"] = {["arabtex"]=true,},
  ["collcell"] = {["collcell"]=true,},
  ["turnstile"] = {["turnstile"]=true,},
  ["babel-tachelhit-tifinagh"] = {["babel"]=true,},
  ["mlp-opts"] = {["e-french"]=true,},
  ["glossary-list"] = {["glossaries"]=true,},
  ["syriac"] = {["aramaic-serto"]=true,},
  ["babel-serbian-latn"] = {["babel"]=true,},
  ["nucleardata"] = {["nucleardata"]=true,},
  ["gregoriotex-chars"] = {["gregoriotex"]=true,},
  ["babel-spanish-mexico"] = {["babel"]=true,},
  ["ltjsreport"] = {["luatexja"]=true,},
  ["t1code"] = {},
  ["boek3"] = {["ntgclass"]=true,},
  ["boek"] = {["ntgclass"]=true,},
  ["lwarp-geometry"] = {["lwarp"]=true,},
  ["ran_toks"] = {["ran_toks"]=true,},
  ["artikel2"] = {["ntgclass"]=true,},
  ["dehyphtex"] = {},
  ["crop"] = {["crop"]=true,},
  ["a4"] = {["ntgclass"]=true,},
  ["fontawesome5"] = {["fontawesome5"]=true,},
  ["idxlayout"] = {["idxlayout"]=true,},
  ["nowidow"] = {["nowidow"]=true,},
  ["hvfloat"] = {["hvfloat"]=true,},
  ["bchart"] = {["bchart"]=true,},
  ["ruenhyph"] = {},
  ["algpseudocode"] = {["algorithmicx"]=true,},
  ["isorot"] = {["isorot"]=true,},
  ["forest-index"] = {["forest"]=true,},
  ["cv4tw-theme-compact"] = {["cv4tw"]=true,},
  ["babel-ngiemboon"] = {["babel"]=true,},
  ["noto-sans"] = {["noto"]=true,},
  ["macro-neptune"] = {["pas-cv"]=true,},
  ["platexrelease"] = {["platex"]=true,},
  ["factura"] = {["factura"]=true,},
  ["eskdfreesize"] = {["eskdx"]=true,},
  ["gmrelsize"] = {["gmutils"]=true,},
  ["dcwrtbib"] = {["dvdcoll"]=true,},
  ["productbox"] = {["productbox"]=true,},
  ["novel-Fonts"] = {["novel"]=true,},
  ["novel-FileData"] = {["novel"]=true,},
  ["luaindex"] = {["luaindex"]=true,},
  ["novel-CalculateLayout"] = {["novel"]=true,},
  ["noto"] = {["noto"]=true,},
  ["pgfcorepathusage.code"] = {["pgf"]=true,},
  ["notes"] = {["notes"]=true,},
  ["bulgarian"] = {["babel"]=true,},
  ["nshyper"] = {["nostarch"]=true,},
  ["zip"] = {},
  ["hypgotoe"] = {["oberdiek"]=true,},
  ["normalcolor"] = {["normalcolor"]=true,},
  ["klein"] = {["context"]=true,},
  ["background"] = {["background"]=true,},
  ["bitpattern"] = {["bitpattern"]=true,},
  ["figflow"] = {["figflow"]=true,},
  ["plidxmac"] = {["gustlib"]=true,},
  ["nomentbl"] = {["nomentbl"]=true,},
  ["nolbreaks"] = {["nolbreaks"]=true,},
  ["noitcrul"] = {["noitcrul"]=true,},
  ["thrmappendix"] = {},
  ["loadhyph-sk"] = {},
  ["dictsym"] = {["dictsym"]=true,},
  ["nodetree"] = {["nodetree"]=true,},
  ["example-image-a5-landscape"] = {["mwe"]=true,},
  ["noconflict"] = {["noconflict"]=true,},
  ["nmbib"] = {["nmbib"]=true,},
  ["arabbook"] = {["arabtex"]=true,},
  ["lwarp-figsize"] = {["lwarp"]=true,},
  ["aat-info"] = {["xetexfontinfo"]=true,},
  ["lollipop-heading"] = {["lollipop"]=true,},
  ["nimbusmononarrow"] = {["nimbus15"]=true,},
  ["nimbusmono"] = {["nimbus15"]=true,},
  ["gmoldcomm"] = {["gmdoc"]=true,},
  ["nihbiosketch"] = {["nihbiosketch"]=true,},
  ["pandora"] = {["mfnfss"]=true,},
  ["pst-rputover"] = {["pst-rputover"]=true,},
  ["moderncvbodyiv"] = {["moderncv"]=true,},
  ["substr"] = {["substr"]=true,["nicetext"]=true,},
  ["makedoc"] = {["nicetext"]=true,},
  ["fdtxttex"] = {["nicetext"]=true,},
  ["bibletext"] = {["bibletext"]=true,},
  ["filemod"] = {["filemod"]=true,},
  ["fddial0g"] = {["nicetext"]=true,},
  ["callouts"] = {["callouts"]=true,},
  ["dpfloat"] = {["dpfloat"]=true,},
  ["typeface"] = {["typeface"]=true,},
  ["demo-tex"] = {["context"]=true,},
  ["nicematrix"] = {["nicematrix"]=true,},
  ["textmerg"] = {["textmerg"]=true,},
  ["nextpage"] = {},
  ["newverbs"] = {["newverbs"]=true,},
  ["newvbtm"] = {["newvbtm"]=true,},
  ["ruhyphmg"] = {},
  ["newtxtext"] = {["newtx"]=true,},
  ["akknum"] = {["akktex"]=true,},
  ["hf-tikz"] = {["hf-tikz"]=true,},
  ["newspaper"] = {["newspaper"]=true,},
  ["loadhyph-mr"] = {},
  ["newsletr"] = {["newsletr"]=true,},
  ["newfile"] = {["newfile"]=true,},
  ["glyphtounicode-cmr"] = {["pdfx"]=true,},
  ["nestquot"] = {},
  ["latexrelease"] = {["latex"]=true,},
  ["simurgh-memoir"] = {["simurgh"]=true,},
  ["gitinfo"] = {["gitinfo"]=true,},
  ["watermark"] = {["ncctools"]=true,},
  ["beamertexpower"] = {["beamer"]=true,},
  ["simurgh-cptns"] = {["simurgh"]=true,},
  ["kvoptions"] = {["oberdiek"]=true,},
  ["arabipoetry"] = {["arabi"]=true,},
  ["pgfplotsoldpgfsupp_pgflibraryplothandlers.code"] = {["pgfplots"]=true,},
  ["beamerthemePaloAlto"] = {["beamer"]=true,},
  ["inputenx"] = {["oberdiek"]=true,},
  ["metalogo"] = {["metalogo"]=true,},
  ["crossreftools"] = {["crossreftools"]=true,},
  ["nccsect"] = {["ncctools"]=true,},
  ["tdiagram"] = {["semantic"]=true,},
  ["ncccropmark"] = {["ncctools"]=true,},
  ["ijdc-v9"] = {["dccpaper"]=true,},
  ["l3tl-build"] = {["l3kernel"]=true,},
  ["cesenaexam"] = {["cesenaexam"]=true,},
  ["loadhyph-ml"] = {},
  ["mboxfill"] = {["ncctools"]=true,},
  ["pgfplotsoldpgfsupp_pgfcoreexternal.code"] = {["pgfplots"]=true,},
  ["afterpackage"] = {["ncctools"]=true,},
  ["sibjnm"] = {["ncclatex"]=true,},
  ["lfp"] = {["fp"]=true,},
  ["babel-tachelhit-tfng"] = {["babel"]=true,},
  ["arydshln"] = {["arydshln"]=true,},
  ["selectp"] = {["selectp"]=true,},
  ["ncctitle"] = {["ncclatex"]=true,},
  ["beamerfontthemeserif"] = {["beamer"]=true,},
  ["nccsections"] = {["ncclatex"]=true,},
  ["asypictureB"] = {["asypictureb"]=true,},
  ["gridset"] = {["gridset"]=true,},
  ["nccproc"] = {["ncclatex"]=true,},
  ["macro-uranus"] = {["pas-cv"]=true,},
  ["nccindex"] = {["ncclatex"]=true,},
  ["magaz"] = {["magaz"]=true,},
  ["stage"] = {["stage"]=true,},
  ["babel-serbian-latn-me"] = {["babel"]=true,},
  ["gtrlang.german.code"] = {["genealogytree"]=true,},
  ["textgreek"] = {["textgreek"]=true,},
  ["beamerfontthemeSaintPetersburg"] = {["beamertheme-saintpetersburg"]=true,},
  ["LKey"] = {["libertine"]=true,},
  ["navydocs"] = {["navydocs"]=true,},
  ["xfp"] = {["l3packages"]=true,},
  ["statmath"] = {["statmath"]=true,},
  ["komacv-casual"] = {["komacv"]=true,},
  ["nature"] = {["nature"]=true,},
  ["nath"] = {["nath"]=true,},
  ["examplep"] = {["examplep"]=true,},
  ["natded"] = {["natded"]=true,},
  ["ams-math"] = {},
  ["natbib"] = {["natbib"]=true,},
  ["kurier"] = {["kurier"]=true,},
  ["xkvex1"] = {["xkeyval"]=true,},
  ["breakurl"] = {["breakurl"]=true,},
  ["namespc"] = {["namespc"]=true,},
  ["nameauth"] = {["nameauth"]=true,},
  ["na-position"] = {["na-position"]=true,},
  ["na-box"] = {["na-box"]=true,},
  ["mycv_dec"] = {["mycv"]=true,},
  ["hep"] = {["hep"]=true,},
  ["eltex4"] = {["eltex"]=true,},
  ["sepfootnotes"] = {["sepfootnotes"]=true,},
  ["tikzlibrarypatterns.code"] = {["pgf"]=true,},
  ["mweights"] = {["mweights"]=true,},
  ["fxlayoutpdfmargin"] = {["fixme"]=true,},
  ["easylist"] = {["easylist"]=true,},
  ["mwe"] = {["mwe"]=true,},
  ["example-movie"] = {["mwe"]=true,},
  ["example-image-plain"] = {["mwe"]=true,},
  ["catkoi"] = {},
  ["example-image-letter-numbered"] = {["mwe"]=true,},
  ["ffslides"] = {["ffslides"]=true,},
  ["nicefrac"] = {["units"]=true,},
  ["chemmacros.module.greek.code"] = {["chemmacros"]=true,},
  ["example-image-golden-upright"] = {["mwe"]=true,},
  ["cgloss4e"] = {["gb4e"]=true,},
  ["interfaces-enumitem"] = {["interfaces"]=true,},
  ["example-image-empty"] = {["mwe"]=true,},
  ["tclldoc"] = {["tcldoc"]=true,},
  ["mpostinl"] = {["mpostinl"]=true,},
  ["babel-walser"] = {["babel"]=true,},
  ["lollipop-tools"] = {["lollipop"]=true,},
  ["hepparticles"] = {["hepparticles"]=true,},
  ["example-image-a4-landscape"] = {["mwe"]=true,},
  ["example-image-a3-landscape"] = {["mwe"]=true,},
  ["gobble-user"] = {["gobble"]=true,},
  ["example-image-9x16"] = {["mwe"]=true,},
  ["il2-csf"] = {["enctex"]=true,},
  ["AlegreyaSans"] = {["alegreya"]=true,},
  ["pgfplotsmeshplothandler.code"] = {["pgfplots"]=true,},
  ["utfcode"] = {["arabtex"]=true,},
  ["boldtensors"] = {["boldtensors"]=true,},
  ["example-image-16x9"] = {["mwe"]=true,},
  ["s-simpleslides-HorizontalStripes"] = {["context-simpleslides"]=true,},
  ["loadhyph-ca"] = {},
  ["ecfonts"] = {},
  ["ebproof"] = {["ebproof"]=true,},
  ["upmethodology-version"] = {["upmethodology"]=true,},
  ["envmath"] = {["bosisio"]=true,},
  ["example-grid-100x100pt"] = {["mwe"]=true,},
  ["example-grid-100x100bp"] = {["mwe"]=true,},
  ["lwarp-rotfloat"] = {["lwarp"]=true,},
  ["doi"] = {["doi"]=true,},
  ["fmultico"] = {["arabi"]=true,},
  ["catechis"] = {["catechis"]=true,},
  ["bondgraphs"] = {["bondgraphs"]=true,},
  ["conv-xkv"] = {["conv-xkv"]=true,},
  ["apacdoc"] = {["apacite"]=true,},
  ["grid-system"] = {["grid-system"]=true,},
  ["classicthesis"] = {["classicthesis"]=true,},
  ["fontfam-catalog"] = {},
  ["pst-calendar"] = {["pst-calendar"]=true,},
  ["berasans"] = {["bera"]=true,},
  ["musuos"] = {["musuos"]=true,},
  ["bpfap7"] = {["iso10303"]=true,},
  ["musixtnt"] = {["musixtnt"]=true,},
  ["danish"] = {["babel"]=true,},
  ["fn2end"] = {["fn2end"]=true,},
  ["babel-lubakatanga"] = {["babel"]=true,},
  ["musixtri"] = {["musixtex"]=true,},
  ["musixtmr"] = {["musixtex"]=true,},
  ["ccfonts"] = {["ccfonts"]=true,},
  ["babel-oriya"] = {["babel"]=true,},
  ["pdffcwebmac"] = {["cweb"]=true,},
  ["allsqrc"] = {["bookhands"]=true,},
  ["musixstf"] = {["musixtex"]=true,},
  ["ltxgrid"] = {["revtex"]=true,},
  ["bophook"] = {["bophook"]=true,},
  ["musixsqr"] = {["musixtex"]=true,},
  ["axodraw2"] = {["axodraw2"]=true,},
  ["getoptk"] = {["getoptk"]=true,},
  ["musixslu"] = {["musixtex"]=true,},
  ["oldpfont"] = {["platex"]=true,},
  ["babel-ganda"] = {["babel"]=true,},
  ["musixplt"] = {["musixtex"]=true,},
  ["ccproof"] = {["computational-complexity"]=true,},
  ["allfonts"] = {["ofs"]=true,},
  ["pst-pdgr"] = {["pst-pdgr"]=true,},
  ["loadhyph-af"] = {},
  ["musixinv"] = {["musixtex"]=true,},
  ["rdfmeta"] = {["stex"]=true,},
  ["musixfll"] = {["musixtex"]=true,},
  ["hpstatement"] = {["mhchem"]=true,},
  ["xob-paralist"] = {["kotex-oblivoir"]=true,},
  ["t-handlecsv"] = {["context-handlecsv"]=true,},
  ["kiyou"] = {["jsclasses"]=true,},
  ["musixdat"] = {["musixtex"]=true,},
  ["eskdsect"] = {["eskdx"]=true,},
  ["musixcrd"] = {["musixtex"]=true,},
  ["asyfig"] = {["asyfig"]=true,},
  ["beamerpatchparalist"] = {["beamer"]=true,},
  ["download"] = {["download"]=true,},
  ["galician"] = {["babel"]=true,},
  ["bibleref-lds"] = {["bibleref-lds"]=true,},
  ["babel-swedish"] = {["babel"]=true,},
  ["Konanur"] = {["cfr-initials"]=true,},
  ["fxtargetlayoutchangebar"] = {["fixme"]=true,},
  ["fjodor"] = {["fjodor"]=true,},
  ["babel-arabic-algeria"] = {["babel"]=true,},
  ["beamerbasefont"] = {["beamer"]=true,},
  ["musixbm"] = {["musixtex"]=true,},
  ["musixblx"] = {["musixtex"]=true,},
  ["musixbar"] = {["musixtex"]=true,},
  ["musixadd"] = {["musixtex"]=true,},
  ["common-pagestylemanipulation"] = {},
  ["ghsystem_explos-5"] = {["ghsystem"]=true,},
  ["lwarp-hyphenat"] = {["lwarp"]=true,},
  ["qtree"] = {["qtree"]=true,},
  ["multirow"] = {["multirow"]=true,},
  ["bigstrut"] = {["multirow"]=true,},
  ["bigdelim"] = {["multirow"]=true,},
  ["multiobjective"] = {["multiobjective"]=true,},
  ["code850"] = {["codepage"]=true,},
  ["gsemthesis"] = {["gsemthesis"]=true,},
  ["glossaries-compatible-207"] = {["glossaries"]=true,},
  ["dichokey"] = {["dichokey"]=true,},
  ["multilang-sect"] = {["multilang"]=true,},
  ["multiexpand"] = {["multiexpand"]=true,},
  ["lwarp-cancel"] = {["lwarp"]=true,},
  ["antcmlgc"] = {["cm-lgc"]=true,},
  ["multicap"] = {["multicap"]=true,},
  ["mtx-context-arrange"] = {["context"]=true,},
  ["showkeysff"] = {["chemcono"]=true,},
  ["latexgit"] = {["latexgit"]=true,},
  ["babel-marathi"] = {["babel"]=true,},
  ["tui"] = {["tui"]=true,},
  ["multienum"] = {["multenum"]=true,},
  ["babel-english-nz"] = {["babel"]=true,},
  ["memucs-enumerate"] = {["kotex-oblivoir"]=true,},
  ["zhsmyclass"] = {["zhspacing"]=true,},
  ["msu-thesis"] = {["msu-thesis"]=true,},
  ["ghsystem_flame-4-3-white"] = {["ghsystem"]=true,},
  ["utf8lata"] = {["enctex"]=true,},
  ["norsk_msg-msg"] = {["msg"]=true,},
  ["weisman"] = {["context"]=true,},
  ["arraysort"] = {["arraysort"]=true,},
  ["novel-LayoutSettings"] = {["novel"]=true,},
  ["msg-msg"] = {["msg"]=true,},
  ["ccref"] = {["computational-complexity"]=true,},
  ["rotating"] = {["graphics"]=true,},
  ["french_msg-msg"] = {["msg"]=true,},
  ["eskdfont"] = {["eskdx"]=true,},
  ["ragged2e"] = {["ms"]=true,},
  ["prelim2e"] = {["ms"]=true,},
  ["multitoc"] = {["ms"]=true,},
  ["hyph-es"] = {["hyphen-spanish"]=true,},
  ["count1to"] = {["ms"]=true,},
  ["cs-heros"] = {},
  ["karabas"] = {},
  ["kniga"] = {},
  ["babel-chinese-traditional-hongkongsarchina"] = {["babel"]=true,},
  ["mpgraphics"] = {["mpgraphics"]=true,},
  ["movie15"] = {["movie15"]=true,},
  ["morewrites"] = {["morewrites"]=true,},
  ["moreverb"] = {["moreverb"]=true,},
  ["othello"] = {},
  ["loadhyph-et"] = {},
  ["lwarp-cases"] = {["lwarp"]=true,},
  ["fonttable"] = {["fonttable"]=true,},
  ["casyltex"] = {["casyl"]=true,},
  ["SIunits"] = {["siunits"]=true,},
  ["hyph-sh-cyrl.t2a"] = {},
  ["blog"] = {["morehype"]=true,},
  ["morefloats"] = {["morefloats"]=true,},
  ["babel-interlingua"] = {["babel"]=true,},
  ["pgfmathfunctions.code"] = {["pgf"]=true,},
  ["hyph-th.lth"] = {},
  ["ob-nokoreanappendix"] = {["kotex-oblivoir"]=true,},
  ["tikzlibrarybayesnet.code"] = {["tikz-bayesnet"]=true,},
  ["lwarp-ltcaption"] = {["lwarp"]=true,},
  ["turnthepage"] = {["turnthepage"]=true,},
  ["ccaption"] = {["ccaption"]=true,},
  ["censor"] = {["censor"]=true,},
  ["mongolian"] = {["mongolian-babel"]=true,},
  ["modular"] = {["modular"]=true,},
  ["missaali"] = {["missaali"]=true,},
  ["arevmath"] = {["arev"]=true,},
  ["mathspic"] = {["mathspic"]=true,},
  ["pictex2"] = {},
  ["tweaklist"] = {["moderncv"]=true,},
  ["t-angles"] = {["t-angles"]=true,},
  ["moderncvstyleempty"] = {["moderncv"]=true,},
  ["babel-uyghur"] = {["babel"]=true,},
  ["properties"] = {["properties"]=true,},
  ["moderncviconsmarvosym"] = {["moderncv"]=true,},
  ["ltj-base"] = {["luatexja"]=true,},
  ["moderncviconsawesome"] = {["moderncv"]=true,},
  ["xkveca"] = {["xkeyval"]=true,},
  ["babel-azerbaijani"] = {["babel"]=true,},
  ["moderncvheadv"] = {["moderncv"]=true,},
  ["moderncvheadiv"] = {["moderncv"]=true,},
  ["syntaxdi"] = {["schule"]=true,},
  ["lwarp-lscape"] = {["lwarp"]=true,},
  ["textpathmp"] = {["textpath"]=true,},
  ["mdpgd"] = {["mathdesign"]=true,},
  ["csckrn2x"] = {["fontinst"]=true,},
  ["moderncvfooti"] = {["moderncv"]=true,},
  ["beamerfontthemestructureitalicserif"] = {["beamer"]=true,},
  ["rsfso"] = {["rsfso"]=true,},
  ["babel-centralkurdish"] = {["babel"]=true,},
  ["xkvesc"] = {["xkeyval"]=true,},
  ["padcount"] = {["padcount"]=true,},
  ["l3color"] = {["l3experimental"]=true,},
  ["latexbangla"] = {["latexbangla"]=true,},
  ["moderncvcolorblue"] = {["moderncv"]=true,},
  ["font_arev"] = {["font-change"]=true,},
  ["blplain"] = {["babel"]=true,},
  ["trsym"] = {["trsym"]=true,},
  ["beamerthemeVerona"] = {["beamer-verona"]=true,},
  ["plain"] = {["carlisle"]=true,},
  ["wiki"] = {["nicetext"]=true,},
  ["genmpage"] = {["genmpage"]=true,},
  ["gnuplottex"] = {["gnuplottex"]=true,},
  ["t1enc"] = {["latex"]=true,},
  ["serbian-date-lat"] = {["serbian-date-lat"]=true,},
  ["moderncvbodyi"] = {["moderncv"]=true,},
  ["fxlayoutpdfcmargin"] = {["fixme"]=true,},
  ["marginnote"] = {["marginnote"]=true,},
  ["akkmathnum"] = {["akktex"]=true,},
  ["moderncv"] = {["moderncv"]=true,},
  ["MnSymbol"] = {["mnsymbol"]=true,},
  ["mnras"] = {["mnras"]=true,},
  ["babel-lowersorbian"] = {["babel"]=true,},
  ["mlist"] = {["mlist"]=true,},
  ["mla"] = {["mla-paper"]=true,},
  ["modroman"] = {["modroman"]=true,},
  ["sectionbox"] = {["sectionbox"]=true,},
  ["minted1"] = {["minted"]=true,},
  ["minted"] = {["minted"]=true,},
  ["babel-centralatlastamazight"] = {["babel"]=true,},
  ["minorrevision"] = {["minorrevision"]=true,},
  ["coloring"] = {["coloring"]=true,},
  ["newclude"] = {["frankenstein"]=true,},
  ["minifp"] = {["minifp"]=true,},
  ["babel-french-lu"] = {["babel"]=true,},
  ["pgfplotstable.code"] = {["pgfplots"]=true,},
  ["wkmgr"] = {["przechlewski-book"]=true,},
  ["combinet"] = {["combine"]=true,},
  ["crimson"] = {["crimson"]=true,},
  ["simurgh-book"] = {["simurgh"]=true,},
  ["loadhyph-fr"] = {},
  ["border"] = {["midnight"]=true,},
  ["langsci-basic"] = {["langsci"]=true,},
  ["photo"] = {["photo"]=true,},
  ["mhequ"] = {["mhequ"]=true,},
  ["musixevo"] = {["musixtex"]=true,},
  ["mgltex"] = {["mgltex"]=true,},
  ["errorbars"] = {["pictex"]=true,},
  ["mfpicdef"] = {["mfpic"]=true,},
  ["mfpic"] = {["mfpic"]=true,},
  ["glossary-long"] = {["glossaries"]=true,},
  ["isiri"] = {["arabtex"]=true,},
  ["mflogo"] = {["mflogo"]=true,},
  ["mexconf"] = {["mex"]=true,},
  ["loadhyph-da"] = {},
  ["ctexpatch"] = {["ctex"]=true,},
  ["lamex"] = {["mex"]=true,},
  ["metrix"] = {["metrix"]=true,},
  ["t-circuitikz"] = {["circuitikz"]=true,},
  ["metre"] = {["metre"]=true,},
  ["menukeys"] = {["menukeys"]=true,},
  ["akkwidepage"] = {["akktex"]=true,},
  ["schule"] = {["schule"]=true,},
  ["extarticle"] = {["extsizes"]=true,},
  ["zhsusefulmacros"] = {["zhspacing"]=true,},
  ["memucs-interword"] = {["kotex-oblivoir"]=true,},
  ["ofs-8t"] = {["ofs"]=true,},
  ["memhfixc"] = {["memoir"]=true,},
  ["meetingmins"] = {["meetingmins"]=true,},
  ["pst-sigsys"] = {["pst-sigsys"]=true,},
  ["chemarr"] = {["oberdiek"]=true,},
  ["powerdot-bframe"] = {["powerdot"]=true,},
  ["synttree"] = {["synttree"]=true,},
  ["mdwtab"] = {["mdwtools"]=true,},
  ["mathenv"] = {["mdwtools"]=true,},
  ["mdframed"] = {["mdframed"]=true,},
  ["UNAMThesis"] = {["unamthesis"]=true,},
  ["mcite"] = {["mcite"]=true,},
  ["mcexam"] = {["mcexam"]=true,},
  ["mcaption"] = {["mcaption"]=true,},
  ["mattens"] = {["mattens"]=true,},
  ["jlreq"] = {["jlreq"]=true,},
  ["oldstyle"] = {["oldstyle"]=true,},
  ["mathtools"] = {["mathtools"]=true,},
  ["lwarp-tocbasic"] = {["lwarp"]=true,},
  ["nwejm-logos-collection"] = {["nwejm"]=true,},
  ["mathspec"] = {["mathspec"]=true,},
  ["bpfats10"] = {["iso10303"]=true,},
  ["chemnum"] = {["chemnum"]=true,},
  ["mathpunctspace"] = {["mathpunctspace"]=true,},
  ["mathfixs"] = {["mathfixs"]=true,},
  ["ruler"] = {},
  ["currvita"] = {["currvita"]=true,},
  ["mathexam"] = {["mathexam"]=true,},
  ["lwarp-patch-memoir"] = {["lwarp"]=true,},
  ["mathdots"] = {["mathdots"]=true,},
  ["mdugm"] = {["mathdesign"]=true,},
  ["mdpus"] = {["mathdesign"]=true,},
  ["apnum"] = {["apnum"]=true,},
  ["langsci-forest-setup"] = {["langsci"]=true,},
  ["split"] = {["gustlib"]=true,},
  ["grant-doe"] = {["grant"]=true,},
  ["euler"] = {["euler"]=true,},
  ["eqnarray"] = {["eqnarray"]=true,},
  ["logicproof"] = {["logicproof"]=true,},
  ["expex"] = {["expex"]=true,},
  ["mdbch"] = {["mathdesign"]=true,},
  ["atsv11"] = {["iso10303"]=true,},
  ["aecc"] = {["aecc"]=true,},
  ["appendix"] = {["appendix"]=true,},
  ["csc2x"] = {["fontinst"]=true,},
  ["eqell"] = {["eqell"]=true,},
  ["apalike"] = {["bibtex"]=true,},
  ["mathastext"] = {["mathastext"]=true,},
  ["mathalfa"] = {["mathalfa"]=true,},
  ["t-markdown"] = {["markdown"]=true,},
  ["fancytabs"] = {["fancytabs"]=true,},
  ["marginfix"] = {["marginfix"]=true,},
  ["amsppt"] = {["amstex"]=true,},
  ["gmdocc"] = {["gmdoc"]=true,},
  ["manuscript"] = {["manuscript"]=true,},
  ["mandi"] = {["mandi"]=true,},
  ["guitartabs"] = {["guitartabs"]=true,},
  ["11point"] = {["varisize"]=true,},
  ["berenis"] = {["berenisadf"]=true,},
  ["maker"] = {["maker"]=true,},
  ["conv-utf8-lmc"] = {["hyph-utf8"]=true,},
  ["idxmac"] = {["makeindex"]=true,},
  ["makebarcode"] = {["makebarcode"]=true,},
  ["mailmerge"] = {["mailmerge"]=true,},
  ["mailing"] = {["mailing"]=true,},
  ["scrindex"] = {["oberdiek"]=true,},
  ["statistics"] = {["statistics"]=true,},
  ["isodoc"] = {["isodoc"]=true,},
  ["texvc"] = {["texvc"]=true,},
  ["lwarp-zwpagelayout"] = {["lwarp"]=true,},
  ["ncccropbox"] = {["ncctools"]=true,},
  ["ctexbeamer"] = {["ctex"]=true,},
  ["beamerouterthemeinfolines"] = {["beamer"]=true,},
  ["ltjspf"] = {["luatexja"]=true,},
  ["lwarp-xltxtra"] = {["lwarp"]=true,},
  ["example-glossaries-symbols"] = {["glossaries"]=true,},
  ["lwarp-xellipsis"] = {["lwarp"]=true,},
  ["uni"] = {["universa"]=true,},
  ["font_iwona"] = {["font-change"]=true,},
  ["interfaces-pgfkeys"] = {["interfaces"]=true,},
  ["lwarp-fancyvrb"] = {["lwarp"]=true,},
  ["lwarp-vowel"] = {["lwarp"]=true,},
  ["bibgerm"] = {["germbib"]=true,},
  ["beraserif"] = {["bera"]=true,},
  ["TXSfonts"] = {["texsis"]=true,},
  ["hyph-rm"] = {},
  ["hyph-quote-rm"] = {},
  ["scalerel"] = {["scalerel"]=true,},
  ["10point"] = {["varisize"]=true,},
  ["lwarp-upref"] = {["lwarp"]=true,},
  ["fithesis-sci"] = {["fithesis"]=true,},
  ["todonotes"] = {["todonotes"]=true,},
  ["lwarp-typearea"] = {["lwarp"]=true,},
  ["lwarp-turnthepage"] = {["lwarp"]=true,},
  ["lwarp-trimclip"] = {["lwarp"]=true,},
  ["pgfbaseshapes"] = {["pgf"]=true,},
  ["usbib"] = {["stellenbosch"]=true,},
  ["hyph-mr"] = {},
  ["wedn"] = {["schulschriften"]=true,},
  ["beamerinnerthemedefault"] = {["beamer"]=true,},
  ["lwarp-titling"] = {["lwarp"]=true,},
  ["lwarp-titlesec"] = {["lwarp"]=true,},
  ["biditools"] = {["bidi"]=true,},
  ["pdfswitch"] = {["latex-make"]=true,},
  ["karnaugh-map"] = {["karnaugh-map"]=true,},
  ["lwarp-textfit"] = {["lwarp"]=true,},
  ["hyph-be"] = {},
  ["bangorcsthesis"] = {["bangorcsthesis"]=true,},
  ["bpfats11"] = {["iso10303"]=true,},
  ["lwarp-chemformula"] = {["lwarp"]=true,},
  ["babel-sanskrit-mlym"] = {["babel"]=true,},
  ["epsdice"] = {["epsdice"]=true,},
  ["calxxxx-yyyy"] = {["calxxxx-yyyy"]=true,},
  ["adforn"] = {["adforn"]=true,},
  ["iffont"] = {["iffont"]=true,},
  ["lwarp-subfigure"] = {["lwarp"]=true,},
  ["ogrphyph4"] = {["antomega"]=true,},
  ["lwarp-stfloats"] = {["lwarp"]=true,},
  ["blogligs"] = {["morehype"]=true,},
  ["embedall"] = {["embedall"]=true,},
  ["lwarp-showidx"] = {["lwarp"]=true,},
  ["lwarp-setspace"] = {["lwarp"]=true,},
  ["lwarp-section"] = {["lwarp"]=true,},
  ["lwarp-scrlayer"] = {["lwarp"]=true,},
  ["bugtracker"] = {["pgfplots"]=true,},
  ["pst-abspos"] = {["pst-abspos"]=true,},
  ["babel-galician"] = {["babel"]=true,},
  ["lwarp-hypcap"] = {["lwarp"]=true,},
  ["smalltableof"] = {["smalltableof"]=true,},
  ["babel-yoruba"] = {["babel"]=true,},
  ["beton"] = {["beton"]=true,},
  ["overrightarrow"] = {["fragments"]=true,},
  ["bigintcalc"] = {["oberdiek"]=true,},
  ["integrated-pool"] = {},
  ["dialogue"] = {["frankenstein"]=true,},
  ["lwarp-repeatindex"] = {["lwarp"]=true,},
  ["lwarp-ragged2e"] = {["lwarp"]=true,},
  ["load-unicode-math-classes"] = {["unicode-data"]=true,},
  ["lwarp-quotchap"] = {["lwarp"]=true,},
  ["cloze"] = {["cloze"]=true,},
  ["lwarp-preview"] = {["lwarp"]=true,},
  ["usorbian"] = {["babel"]=true,},
  ["ghsystem_flame-4-1"] = {["ghsystem"]=true,},
  ["askinclude"] = {["oberdiek"]=true,},
  ["lwarp-pdfx"] = {["lwarp"]=true,},
  ["lwarp-pdfsync"] = {["lwarp"]=true,},
  ["underlin"] = {["underlin"]=true,},
  ["ghsystem"] = {["ghsystem"]=true,},
  ["example-glossaries-long"] = {["glossaries"]=true,},
  ["boisik"] = {["boisik"]=true,},
  ["letter"] = {["latex"]=true,},
  ["chess"] = {["chess"]=true,},
  ["alphabeta"] = {["greek-fontenc"]=true,},
  ["lwarp-pdfrender"] = {["lwarp"]=true,},
  ["lwarp-pbox"] = {["lwarp"]=true,},
  ["luatextra"] = {["luatextra"]=true,},
  ["lwarp-patch-komascript"] = {["lwarp"]=true,},
  ["pst-ovl"] = {["pst-ovl"]=true,},
  ["paratype"] = {["paratype"]=true,},
  ["lwarp-paralist"] = {["lwarp"]=true,},
  ["avr.memory"] = {["avremu"]=true,},
  ["braids"] = {["braids"]=true,},
  ["lsorbian"] = {["babel"]=true,},
  ["lwarp-pagenote"] = {["lwarp"]=true,},
  ["quran"] = {["quran"]=true,},
  ["anttor"] = {["antt"]=true,},
  ["lwarp-ntheorem"] = {["lwarp"]=true,},
  ["lwarp-nopageno"] = {["lwarp"]=true,},
  ["luatexja-core"] = {["luatexja"]=true,},
  ["kinsoku"] = {["ptex-base"]=true,["platex"]=true,},
  ["beamerouterthemesidebar"] = {["beamer"]=true,},
  ["lwarp-nextpage"] = {["lwarp"]=true,},
  ["moredefs"] = {["frankenstein"]=true,},
  ["biblatex-shortfields"] = {["biblatex-shortfields"]=true,},
  ["version4"] = {["ibygrk"]=true,},
  ["ghsystem_acid"] = {["ghsystem"]=true,},
  ["spie"] = {["spie"]=true,},
  ["dot2texi"] = {["dot2texi"]=true,},
  ["lwarp-nameref"] = {["lwarp"]=true,},
  ["hyph-nl"] = {},
  ["pgfmathfunctions.round.code"] = {["pgf"]=true,},
  ["lwarp-multicol"] = {["lwarp"]=true,},
  ["babel-mazanderani"] = {["babel"]=true,},
  ["lwarp-morewrites"] = {["lwarp"]=true,},
  ["capitals3d"] = {["pst-geo"]=true,},
  ["dateiliste"] = {["dateiliste"]=true,},
  ["scrextend"] = {},
  ["babel-sanskrit-bengali"] = {["babel"]=true,},
  ["lwarp-microtype"] = {["lwarp"]=true,},
  ["confproc"] = {["confproc"]=true,},
  ["lwarp-mhchem"] = {["lwarp"]=true,},
  ["filesdo"] = {["commado"]=true,},
  ["lwarp-metalogo"] = {["lwarp"]=true,},
  ["cite"] = {["cite"]=true,},
  ["villesFrance"] = {["pst-geo"]=true,},
  ["sffms"] = {["sffms"]=true,},
  ["babel-uzbek-arab"] = {["babel"]=true,},
  ["lwarp-lua-check-hyphen"] = {["lwarp"]=true,},
  ["doclicense-CC-by-nc-nd-3.0-plaintext"] = {["doclicense"]=true,},
  ["lwarp-ltxtable"] = {["lwarp"]=true,},
  ["basque"] = {["babel"]=true,},
  ["babel-kashmiri"] = {["babel"]=true,},
  ["fxlayoutpdfsigmargin"] = {["fixme"]=true,},
  ["moderncvheadii"] = {["moderncv"]=true,},
  ["doclicense-CC-by-nd-4.0-plaintext"] = {["doclicense"]=true,},
  ["schuleit"] = {["schule"]=true,},
  ["domore"] = {["dowith"]=true,},
  ["babel-kalenjin"] = {["babel"]=true,},
  ["lwarp-alltt"] = {["lwarp"]=true,},
  ["lwarp-lips"] = {["lwarp"]=true,},
  ["german_french-msg"] = {["e-french"]=true,},
  ["lwarp-resizegather"] = {["lwarp"]=true,},
  ["lccaps"] = {["lccaps"]=true,},
  ["ltablex"] = {["ltablex"]=true,},
  ["usebib"] = {["usebib"]=true,},
  ["texlogos"] = {},
  ["ctexspamacro"] = {["ctex"]=true,},
  ["cpalatin"] = {},
  ["loadhyph-cu"] = {},
  ["report"] = {["latex"]=true,},
  ["fancyhandout"] = {["fancyhandout"]=true,},
  ["beamerthemeGoettingen"] = {["beamer"]=true,},
  ["lwarp-inputenx"] = {["lwarp"]=true,},
  ["loadhyph-ta"] = {},
  ["wallcalendar-hungarian"] = {["wallcalendar"]=true,},
  ["mtx-context-combine"] = {["context"]=true,},
  ["lwarp-idxlayout"] = {["lwarp"]=true,},
  ["thcc"] = {["computational-complexity"]=true,},
  ["lwarp-hypernat"] = {["lwarp"]=true,},
  ["hypbmsec"] = {["oberdiek"]=true,},
  ["lwarp-grid"] = {["lwarp"]=true,},
  ["lwarp-graphics"] = {["lwarp"]=true,},
  ["lwarp-glossary"] = {["lwarp"]=true,},
  ["longdiv"] = {},
  ["pageslts"] = {["pageslts"]=true,},
  ["qpalatin"] = {["tex-gyre"]=true,},
  ["sbbm"] = {["sauterfonts"]=true,},
  ["lwarp-ftnright"] = {["lwarp"]=true,},
  ["xgalley"] = {["l3experimental"]=true,},
  ["lwarp-blowup"] = {["lwarp"]=true,},
  ["asaesub"] = {["asaetr"]=true,},
  ["lwarp-footnpag"] = {["lwarp"]=true,},
  ["bibl-apa-de"] = {["context"]=true,},
  ["ghsystem_exclam"] = {["ghsystem"]=true,},
  ["babel-korean"] = {["babel"]=true,},
  ["lwarp-footnotehyper"] = {["lwarp"]=true,},
  ["s-pre-50"] = {["context"]=true,},
  ["lwarp-footnote"] = {["lwarp"]=true,},
  ["umoline"] = {["umoline"]=true,},
  ["lltjp-array"] = {["luatexja"]=true,},
  ["eskdafterpkg"] = {["eskdx"]=true,},
  ["babel-chinese-hant"] = {["babel"]=true,},
  ["smultiling"] = {["stex"]=true,},
  ["mtx-context-common"] = {["context"]=true,},
  ["lwarp-floatrow"] = {["lwarp"]=true,},
  ["lwarp-floatpag"] = {["lwarp"]=true,},
  ["babel-english-unitedstates"] = {["babel"]=true,},
  ["ofs-ps"] = {["ofs"]=true,},
  ["lwarp-float"] = {["lwarp"]=true,},
  ["jneurosci"] = {["jneurosci"]=true,},
  ["gregoriotex-signs"] = {["gregoriotex"]=true,},
  ["lwarp-flafter"] = {["lwarp"]=true,},
  ["lwarp-fixmetodonotes"] = {["lwarp"]=true,},
  ["seealso"] = {["seealso"]=true,},
  ["scrtime"] = {},
  ["babel-punjabi"] = {["babel"]=true,},
  ["babel-swissgerman"] = {["babel"]=true,},
  ["nlctdoc"] = {["nlctdoc"]=true,},
  ["lwarp-vwcol"] = {["lwarp"]=true,},
  ["lwarp-fancyheadings"] = {["lwarp"]=true,},
  ["sourceserifpro-type1-autoinst"] = {["sourceserifpro"]=true,},
  ["lwarp-everypage"] = {["lwarp"]=true,},
  ["metatex"] = {["metatex"]=true,["context"]=true,},
  ["lwarp-eso-pic"] = {["lwarp"]=true,},
  ["footnoterange"] = {["footnoterange"]=true,},
  ["copyedit"] = {["copyedit"]=true,},
  ["shorthnd"] = {["custom-bib"]=true,},
  ["lwarp-errata"] = {["lwarp"]=true,},
  ["conv-utf8-lth"] = {["hyph-utf8"]=true,},
  ["pgfbaselayers"] = {["pgf"]=true,},
  ["lwarp-epigraph"] = {["lwarp"]=true,},
  ["pgfplotsutil.code"] = {["pgfplots"]=true,},
  ["lwarp-xmpincl"] = {["lwarp"]=true,},
  ["babel-hebrew"] = {["babel"]=true,},
  ["addlines"] = {["addlines"]=true,},
  ["lwarp-draftwatermark"] = {["lwarp"]=true,},
  ["poligraf"] = {["tex-ps"]=true,},
  ["hyph-it"] = {},
  ["lwarp-dblfnote"] = {["lwarp"]=true,},
  ["iby4extr"] = {["ibygrk"]=true,},
  ["simurgh-chkeng"] = {["simurgh"]=true,},
  ["font_antt-condensed"] = {["font-change"]=true,},
  ["boites"] = {["boites"]=true,},
  ["lwarp-colortbl"] = {["lwarp"]=true,},
  ["codicefiscaleitaliano"] = {["codicefiscaleitaliano"]=true,},
  ["pict2e"] = {["pict2e"]=true,},
  ["simurgh-artikel1"] = {["simurgh"]=true,},
  ["lwarp-cite"] = {["lwarp"]=true,},
  ["lwarp-chngpage"] = {["lwarp"]=true,},
  ["dhucs-setspace"] = {["kotex-utf"]=true,},
  ["ruhyphdv"] = {},
  ["grabhedr"] = {["dialogl"]=true,},
  ["aparse"] = {["arabtex"]=true,},
  ["beamerposter"] = {["beamerposter"]=true,},
  ["example-image-16x10"] = {["mwe"]=true,},
  ["revtex4-1"] = {["revtex"]=true,},
  ["lwarp-longtable"] = {["lwarp"]=true,},
  ["lwarp-changepage"] = {["lwarp"]=true,},
  ["babel-turkmen"] = {["babel"]=true,},
  ["font_utopia"] = {["font-change"]=true,},
  ["dashbox"] = {["dashbox"]=true,},
  ["xparse"] = {["l3packages"]=true,},
  ["invoice2"] = {["invoice2"]=true,},
  ["hyph-ru.t2a"] = {},
  ["texlive-installer-graphics"] = {},
  ["fenglish"] = {["e-french"]=true,},
  ["ccdbs"] = {["computational-complexity"]=true,},
  ["bidituftefloat"] = {["bidi"]=true,},
  ["dehyphn"] = {},
  ["eukdate"] = {["eukdate"]=true,},
  ["xyxdvi"] = {["xypic"]=true,},
  ["gitsetinfo"] = {["gitinfo"]=true,},
  ["pdfXcwebmac"] = {["cweb"]=true,},
  ["babel-uzbek-arabic"] = {["babel"]=true,},
  ["lwarp-caption"] = {["lwarp"]=true,},
  ["tasks"] = {["tasks"]=true,},
  ["dotseqn"] = {["dotseqn"]=true,},
  ["lwarp-breakurl"] = {["lwarp"]=true,},
  ["musixltx"] = {["musixtex"]=true,},
  ["interfaces-hyperref"] = {["interfaces"]=true,},
  ["type1ec"] = {["cm-super"]=true,},
  ["simurgh-boek3"] = {["simurgh"]=true,},
  ["doclicense-CC-by-nc-nd-3.0-latex"] = {["doclicense"]=true,},
  ["gatech-thesis-gloss"] = {["gatech-thesis"]=true,},
  ["grant-onr"] = {["grant"]=true,},
  ["babel-portuguese-brazil"] = {["babel"]=true,},
  ["witbhs"] = {["arabtex"]=true,},
  ["gtrlang.spanish.code"] = {["genealogytree"]=true,},
  ["etoolbox"] = {["etoolbox"]=true,},
  ["cstypo"] = {["cstypo"]=true,},
  ["852-t1"] = {["enctex"]=true,},
  ["t-construction-plan"] = {["context-construction-plan"]=true,},
  ["hyph-ca"] = {},
  ["lwarp-bigdelim"] = {["lwarp"]=true,},
  ["lwarp-balance"] = {["lwarp"]=true,},
  ["fontloader-math"] = {["luaotfload"]=true,},
  ["abnt"] = {["abnt"]=true,},
  ["cntformats"] = {["cntformats"]=true,},
  ["beamericonarticle"] = {["beamer"]=true,},
  ["lwarp-axodraw2"] = {["lwarp"]=true,},
  ["DejaVuSerifCondensed"] = {["dejavu"]=true,},
  ["ustitle"] = {["stellenbosch"]=true,},
  ["babel-konkani"] = {["babel"]=true,},
  ["codesection"] = {["codesection"]=true,},
  ["footmisc"] = {["footmisc"]=true,},
  ["bxcalc"] = {["bxcalc"]=true,},
  ["lwarp-attachfile"] = {["lwarp"]=true,},
  ["lwarp-arydshln"] = {["lwarp"]=true,},
  ["listings-ext"] = {["listings-ext"]=true,},
  ["lh-x2xccr"] = {["lh"]=true,},
  ["lwarp-appendix"] = {["lwarp"]=true,},
  ["ltxcheck"] = {["latex"]=true,},
  ["lwarp-anysize"] = {["lwarp"]=true,},
  ["lwarp-amsthm"] = {["lwarp"]=true,},
  ["babel-arabic-ma"] = {["babel"]=true,},
  ["pgflibrarydatavisualization.formats.functions.code"] = {["pgf"]=true,},
  ["babel-serbian-cyrl-xk"] = {["babel"]=true,},
  ["fnlineno"] = {["lineno"]=true,},
  ["csenc-u"] = {},
  ["flower"] = {["shipunov"]=true,},
  ["ob-koreanappendix"] = {["kotex-oblivoir"]=true,},
  ["cooking"] = {["cooking"]=true,},
  ["babel-bosnian-cyrl"] = {["babel"]=true,},
  ["lwarp-lineno"] = {["lwarp"]=true,},
  ["conv-utf8-il3"] = {["hyph-utf8"]=true,},
  ["kurdishlipsum"] = {["kurdishlipsum"]=true,},
  ["extcode"] = {},
  ["biblotex"] = {["gustlib"]=true,},
  ["correctmathalign"] = {["correctmathalign"]=true,},
  ["t-gantt"] = {["context-gantt"]=true,},
  ["anysize"] = {["anysize"]=true,},
  ["beamercolorthemefibeamer-mu-fi"] = {["fibeamer"]=true,},
  ["babel-mongolian"] = {["babel"]=true,},
  ["timbreicmc"] = {["timbreicmc"]=true,},
  ["lwarp-ae"] = {["lwarp"]=true,},
  ["elocalloc"] = {["elocalloc"]=true,},
  ["mciteplus"] = {["mciteplus"]=true,},
  ["loadhyph-el-polyton"] = {["hyphen-greek"]=true,},
  ["babel-german-at"] = {["babel"]=true,},
  ["lwarp-textpos"] = {["lwarp"]=true,},
  ["lwarp-acro"] = {["lwarp"]=true,},
  ["multislot"] = {["fontinst"]=true,},
  ["beamerinnerthemerectangles"] = {["beamer"]=true,},
  ["chemmacros.module.spectroscopy.code"] = {["chemmacros"]=true,},
  ["hyph-sl.ec"] = {},
  ["xu-ruenhyph"] = {["xecyr"]=true,},
  ["cs-pagella"] = {},
  ["s-faq-02"] = {["context"]=true,},
  ["harnon-cv"] = {["harnon-cv"]=true,},
  ["tracklang"] = {["tracklang"]=true,},
  ["my-title"] = {["uaclasses"]=true,},
  ["lualatexquotejobname"] = {},
  ["pdf-trans"] = {["pdf-trans"]=true,},
  ["interfaces-tikz"] = {["interfaces"]=true,},
  ["fonts-tlwg"] = {["fonts-tlwg"]=true,},
  ["babel-polish"] = {["babel"]=true,},
  ["dk-bib"] = {["dk-bib"]=true,},
  ["wiggly"] = {},
  ["babel-slovak"] = {["babel"]=true,},
  ["memhangul-ucs"] = {["kotex-oblivoir"]=true,},
  ["lisp-prim"] = {["lisp-on-tex"]=true,},
  ["hyph-mn-cyrl-x-lmc.lmc"] = {},
  ["curve"] = {["curve"]=true,},
  ["IEEEtran"] = {["ieeetran"]=true,},
  ["pgflibrarylindenmayersystems.code"] = {["pgf"]=true,},
  ["chemist"] = {["xymtex"]=true,},
  ["lltjp-stfloats"] = {["luatexja"]=true,},
  ["hrefhide"] = {["hrefhide"]=true,},
  ["eufrak"] = {["amsfonts"]=true,},
  ["gentium"] = {["gentium-tug"]=true,},
  ["dirtytalk"] = {["dirtytalk"]=true,},
  ["colophon"] = {["colophon"]=true,},
  ["show2e"] = {["show2e"]=true,},
  ["hyph-cop"] = {},
  ["fbithesis"] = {["fbithesis"]=true,},
  ["animate"] = {["animate"]=true,},
  ["lltjp-fontspec-immediate"] = {["luatexja"]=true,},
  ["beamerfontthememetropolis"] = {["beamertheme-metropolis"]=true,},
  ["picinpar"] = {["picinpar"]=true,},
  ["hyph-gu"] = {},
  ["halloweenmath"] = {["halloweenmath"]=true,},
  ["pgfcirctripoles"] = {["circuitikz"]=true,},
  ["efbox"] = {["efbox"]=true,},
  ["bpfats12"] = {["iso10303"]=true,},
  ["lltjcore"] = {["luatexja"]=true,},
  ["ltjtreport"] = {["luatexja"]=true,},
  ["scrjuraexample"] = {},
  ["ltjtbook"] = {["luatexja"]=true,},
  ["ntheorem"] = {["ntheorem"]=true,},
  ["nonumonpart"] = {["nonumonpart"]=true,},
  ["chickenize"] = {["chickenize"]=true,},
  ["hyph-et"] = {},
  ["numspell"] = {["numspell"]=true,},
  ["ltj-latex"] = {["luatexja"]=true,},
  ["ltjskiyou"] = {["luatexja"]=true,},
  ["FiraSans"] = {["fira"]=true,},
  ["crosswrd"] = {["crosswrd"]=true,},
  ["ofsdef"] = {["ofs"]=true,},
  ["novel"] = {["novel"]=true,},
  ["txi-nl"] = {},
  ["clrscode3e"] = {["clrscode3e"]=true,},
  ["bahasa"] = {["babel"]=true,},
  ["germkorr"] = {["germkorr"]=true,},
  ["qpxmath"] = {["qpxqtx"]=true,},
  ["babel-azerbaijani-cyrillic"] = {["babel"]=true,},
  ["babel-arabic"] = {["babel"]=true,},
  ["import"] = {["import"]=true,},
  ["bussproofs"] = {["bussproofs"]=true,},
  ["zi4"] = {["inconsolata"]=true,},
  ["lollipop-fonts"] = {["lollipop"]=true,},
  ["plantuml"] = {["plantuml"]=true,},
  ["xkvltxp"] = {["xkeyval"]=true,},
  ["edstanza"] = {["edmac"]=true,},
  ["loadhyph-ga"] = {},
  ["hyph-af.ec"] = {},
  ["acmd"] = {["arabtex"]=true,},
  ["figlatex"] = {["latex-make"]=true,},
  ["grant-arl"] = {["grant"]=true,},
  ["bbox"] = {["fontinst"]=true,},
  ["bewerbung-cv-casual"] = {["bewerbung"]=true,},
  ["letltxmacro"] = {["oberdiek"]=true,},
  ["gchords"] = {["gchords"]=true,},
  ["ghsystem_flame"] = {["ghsystem"]=true,},
  ["calligra"] = {["fundus-calligra"]=true,},
  ["lltjext"] = {["luatexja"]=true,},
  ["txdexamp"] = {["texdraw"]=true,},
  ["luatexja-otf"] = {["luatexja"]=true,},
  ["tikzlibraryhobby.code"] = {["hobby"]=true,},
  ["oldlfont"] = {["latex"]=true,},
  ["liturg"] = {["liturg"]=true,},
  ["tkz-orm"] = {["tkz-orm"]=true,},
  ["ifuptex"] = {["ifptex"]=true,},
  ["prftree"] = {["prftree"]=true,},
  ["pocoec"] = {["economic"]=true,},
  ["classlist"] = {["oberdiek"]=true,},
  ["babel-arabic-sy"] = {["babel"]=true,},
  ["decimal"] = {["decimal"]=true,},
  ["musixsty"] = {["musixtex"]=true,},
  ["colorwav"] = {["colorwav"]=true,},
  ["lwarp-luatodonotes"] = {["lwarp"]=true,},
  ["babel-kamba"] = {["babel"]=true,},
  ["luatexbase-compat"] = {["luatexbase"]=true,},
  ["parrun"] = {["parrun"]=true,},
  ["filecontentsdef"] = {["filecontentsdef"]=true,},
  ["babel-serbian-latin-bosniaherzegovina"] = {["babel"]=true,},
  ["ecgdraw"] = {["ecgdraw"]=true,},
  ["luatex85"] = {["luatex85"]=true,},
  ["ukrhypmp"] = {["ukrhyph"]=true,},
  ["villesItalia"] = {["pst-geo"]=true,},
  ["gillcm"] = {["gillcm"]=true,},
  ["babel-nynorsk"] = {["babel"]=true,},
  ["chemmacros.module.all.code"] = {["chemmacros"]=true,},
  ["luasseq"] = {["luasseq"]=true,},
  ["babel-kabuverdianu"] = {["babel"]=true,},
  ["lwarp-multirow"] = {["lwarp"]=true,},
  ["limap"] = {["limap"]=true,},
  ["gradstudentresume"] = {["gradstudentresume"]=true,},
  ["mtx-context-compare"] = {["context"]=true,},
  ["bxeepic"] = {["bxeepic"]=true,},
  ["fontloader-test"] = {["luaotfload"]=true,},
  ["fontloader-swiglib"] = {["luaotfload"]=true,},
  ["fontloader-plain"] = {["luaotfload"]=true,},
  ["gmiflink"] = {["gmiflink"]=true,},
  ["braket"] = {["braket"]=true,},
  ["colortab"] = {["colortab"]=true,},
  ["asmo449"] = {["arabtex"]=true,},
  ["akkgerman"] = {["akktex"]=true,},
  ["fclfont"] = {},
  ["fontloader-mplib"] = {["luaotfload"]=true,},
  ["Alegreya"] = {["alegreya"]=true,},
  ["fduthesis-en"] = {["fduthesis"]=true,},
  ["babel-bulgarian"] = {["babel"]=true,},
  ["compactbib"] = {},
  ["odsfile"] = {["odsfile"]=true,},
  ["filedate"] = {["filedate"]=true,},
  ["boites_exemples"] = {["boites"]=true,},
  ["zhadobefonts"] = {["ctex"]=true,},
  ["xob-dotemph"] = {["kotex-oblivoir"]=true,},
  ["context-todo"] = {["context"]=true,},
  ["interfaces-environ"] = {["interfaces"]=true,},
  ["lwarp-chemnum"] = {["lwarp"]=true,},
  ["babel-samburu"] = {["babel"]=true,},
  ["apmptbl"] = {["iso10303"]=true,},
  ["dsfont"] = {["doublestroke"]=true,},
  ["luamesh"] = {["luamesh"]=true,},
  ["luainputenc"] = {["luainputenc"]=true,},
  ["luahyphenrules"] = {["luahyphenrules"]=true,},
  ["TOPletter"] = {["topletter"]=true,},
  ["tlmgr-dtk"] = {},
  ["covington"] = {["covington"]=true,},
  ["Carrickc"] = {["cfr-initials"]=true,},
  ["luabibentry"] = {["luabibentry"]=true,},
  ["thuan"] = {["context"]=true,},
  ["LinLibertine_I"] = {["libertine"]=true,},
  ["babel-serbian"] = {["babel"]=true,},
  ["psfont"] = {["altfont"]=true,},
  ["easy-todo"] = {["easy-todo"]=true,},
  ["font_kp-light"] = {["font-change"]=true,},
  ["utf8-csf"] = {["enctex"]=true,},
  ["ltxtools-trace"] = {["ltxtools"]=true,},
  ["ltxtools-index"] = {["ltxtools"]=true,},
  ["french_french-msg"] = {["e-french"]=true,},
  ["eqnalign"] = {["eqnalign"]=true,},
  ["babel-serbian-cyrillic"] = {["babel"]=true,},
  ["cap_pyt"] = {["c-pascal"]=true,},
  ["cmlgc"] = {["cm-lgc"]=true,},
  ["luatex-basics"] = {["context"]=true,},
  ["tagpdf-checks-code"] = {["tagpdf"]=true,},
  ["luatexbase-regs"] = {["luatexbase"]=true,},
  ["regcount"] = {["regcount"]=true,},
  ["lwarp-ebook"] = {["lwarp"]=true,},
  ["babel-teso"] = {["babel"]=true,},
  ["pdfpagediff"] = {["pdfpagediff"]=true,},
  ["pgfutil-common-lists"] = {["pgf"]=true,},
  ["cwebarray"] = {["cweb-latex"]=true,},
  ["linsys"] = {},
  ["autonum"] = {["autonum"]=true,},
  ["babel-serbian-cyrl-me"] = {["babel"]=true,},
  ["klunote"] = {["kluwer"]=true,},
  ["beamerbaseframe"] = {["beamer"]=true,},
  ["multibibliography"] = {["multibibliography"]=true,},
  ["babel-tachelhit-latin"] = {["babel"]=true,},
  ["blowup"] = {["blowup"]=true,},
  ["upmgr"] = {["przechlewski-book"]=true,},
  ["tudscrcomp"] = {["tudscr"]=true,},
  ["tikzlibraryshadows.code"] = {["pgf"]=true,},
  ["beamercolorthemelily"] = {["beamer"]=true,},
  ["ltxkeys"] = {["ltxkeys"]=true,},
  ["example-glossaries-acronym-desc"] = {["glossaries"]=true,},
  ["har2nat"] = {["har2nat"]=true,},
  ["ltxdockit"] = {["ltxdockit"]=true,},
  ["crbox"] = {["crbox"]=true,},
  ["reserved"] = {["semantic"]=true,},
  ["tabulary"] = {["tabulary"]=true,},
  ["alphanum"] = {["jura"]=true,},
  ["luatexja-zhfonts"] = {["luatexja"]=true,},
  ["ltb2bib"] = {["ltb2bib"]=true,},
  ["ltabptch"] = {["ltabptch"]=true,},
  ["lt3graph"] = {["lt3graph"]=true,},
  ["lwarp-keyfloat"] = {["lwarp"]=true,},
  ["babel-vai"] = {["babel"]=true,},
  ["hyph-de-ch-1901"] = {},
  ["acro"] = {["acro"]=true,},
  ["babel-serbian-latin-kosovo"] = {["babel"]=true,},
  ["stdpage"] = {["stdpage"]=true,},
  ["fifinddo"] = {["nicetext"]=true,},
  ["ofs-6a"] = {["ofs"]=true,},
  ["lps"] = {["lps"]=true,},
  ["ghsystem_flame-O"] = {["ghsystem"]=true,},
  ["doclicense-CC-by-nd-4.0-latex"] = {["doclicense"]=true,},
  ["lpform"] = {["lpform"]=true,},
  ["tikzpeople.shape.businessman"] = {["tikzpeople"]=true,},
  ["gtrlib.largetrees"] = {["gtrlib-largetrees"]=true,},
  ["beamercolorthemegoeagles"] = {["beamertheme-upenn-bc"]=true,},
  ["staves"] = {["staves"]=true,},
  ["eulervm"] = {["eulervm"]=true,},
  ["texpictex"] = {["pictex"]=true,},
  ["flabels"] = {["flabels"]=true,},
  ["beamerthemeJLTree"] = {},
  ["engpron-tools"] = {["engpron"]=true,},
  ["example-image-a4"] = {["mwe"]=true,},
  ["pgflibraryshapes.multipart.code"] = {["pgf"]=true,},
  ["fithesis-base"] = {["fithesis"]=true,},
  ["lollipop-text"] = {["lollipop"]=true,},
  ["extarrows"] = {["extarrows"]=true,},
  ["tqft"] = {["tqft"]=true,},
  ["beamerouterthemeshadow"] = {["beamer"]=true,},
  ["cwpuzzle"] = {["crossword"]=true,},
  ["draftfigure"] = {["draftfigure"]=true,},
  ["steroid"] = {["xymtex"]=true,},
  ["t-pgffor"] = {["pgf"]=true,},
  ["nostarch"] = {["nostarch"]=true,},
  ["xint"] = {["xint"]=true,},
  ["loadhyph-ru"] = {},
  ["chicago"] = {},
  ["hcletter"] = {["hc"]=true,},
  ["dyntree"] = {["dyntree"]=true,},
  ["cnltx-base"] = {["cnltx"]=true,},
  ["gatech-thesis"] = {["gatech-thesis"]=true,},
  ["apecon"] = {["economic"]=true,},
  ["babel-bosnian-cyrillic"] = {["babel"]=true,},
  ["slovene"] = {["babel"]=true,},
  ["cmap"] = {["cmap"]=true,},
  ["example-image-4x3"] = {["mwe"]=true,},
  ["i-readme"] = {["context"]=true,},
  ["texsis"] = {["texsis"]=true,},
  ["x-chronosys"] = {["chronosys"]=true,},
  ["arabext"] = {["arabtex"]=true,},
  ["babel-sanskrit-malayalam"] = {["babel"]=true,},
  ["hanoi"] = {},
  ["mlstrans"] = {["montex"]=true,},
  ["fxlayoutmarginnote"] = {["fixme"]=true,},
  ["checkend"] = {["bezos"]=true,},
  ["memucs-interword-x"] = {["kotex-oblivoir"]=true,},
  ["alatex"] = {["arabtex"]=true,},
  ["cs-bera"] = {},
  ["pacioli"] = {["pacioli"]=true,},
  ["aboxes"] = {["arabtex"]=true,},
  ["edmargin"] = {["edmargin"]=true,},
  ["davis"] = {["context"]=true,},
  ["powerdot-ikeda"] = {["powerdot"]=true,},
  ["s-pre-16"] = {["context"]=true,},
  ["gitfile-info"] = {["gitfile-info"]=true,},
  ["abjad"] = {["arabtex"]=true,},
  ["utf8warn"] = {["enctex"]=true,},
  ["t-transliterator"] = {["context-transliterator"]=true,},
  ["lwarp-attachfile2"] = {["lwarp"]=true,},
  ["oz"] = {["objectz"]=true,},
  ["keyval2e"] = {["keyval2e"]=true,},
  ["proof"] = {["lkproof"]=true,},
  ["leadsheets.library.chords.code"] = {["leadsheets"]=true,},
  ["backnaur"] = {["backnaur"]=true,},
  ["amsthm"] = {["amscls"]=true,},
  ["ijmart"] = {["ijmart"]=true,},
  ["listofitems"] = {["listofitems"]=true,},
  ["regression-test"] = {["l3build"]=true,},
  ["chemschemex"] = {["chemschemex"]=true,},
  ["cyklop"] = {["cyklop"]=true,},
  ["rustic"] = {["bookhands"]=true,},
  ["listliketab"] = {["listliketab"]=true,},
  ["listlbls"] = {["listlbls"]=true,},
  ["dsserif"] = {["dsserif"]=true,},
  ["csfonts"] = {},
  ["lhelp"] = {["lhelp"]=true,},
  ["numnameru"] = {["numnameru"]=true,},
  ["lstlang2"] = {["listings"]=true,},
  ["simurgh-scrartcl"] = {["simurgh"]=true,},
  ["enumitem-zref"] = {["enumitem-zref"]=true,},
  ["listings"] = {["listings"]=true,},
  ["hyph-is.ec"] = {},
  ["docmute"] = {["docmute"]=true,},
  ["holtpolt"] = {["jknapltx"]=true,},
  ["ngerman"] = {["german"]=true,},
  ["babel-zulu"] = {["babel"]=true,},
  ["txxdefs"] = {},
  ["lisp-mod-l3regex"] = {["lisp-on-tex"]=true,},
  ["metainfo"] = {["sauerj"]=true,},
  ["topcoman"] = {["toptesi"]=true,},
  ["lisp-arith"] = {["lisp-on-tex"]=true,},
  ["journal"] = {["paper"]=true,},
  ["ps-trees"] = {["linguex"]=true,},
  ["makeshape"] = {["makeshape"]=true,},
  ["colo-asy"] = {["asymptote"]=true,},
  ["authorpart"] = {},
  ["adrconvnote"] = {},
  ["vplref"] = {["lineno"]=true,},
  ["lmake"] = {["lmake"]=true,},
  ["opmac-luatex"] = {},
  ["lwarp-ellipsis"] = {["lwarp"]=true,},
  ["ednmath0"] = {["lineno"]=true,},
  ["babel-hungarian"] = {["babel"]=true,},
  ["beamerbaseoverlay"] = {["beamer"]=true,},
  ["opmac-bib"] = {},
  ["klumono"] = {["kluwer"]=true,},
  ["font_century"] = {["font-change"]=true,},
  ["cyrchars"] = {},
  ["fxlayoutpdfsignote"] = {["fixme"]=true,},
  ["formlett"] = {["formlett"]=true,},
  ["citeref"] = {["citeref"]=true,},
  ["amstex"] = {["amsmath"]=true,["amstex"]=true,},
  ["lwarp-todo"] = {["lwarp"]=true,},
  ["beamerfoils"] = {["beamer"]=true,},
  ["LibreBodoni"] = {["librebodoni"]=true,},
  ["babel-french-belgium"] = {["babel"]=true,},
  ["linkalias"] = {},
  ["t-librarian"] = {["librarian"]=true,},
  ["simurgh-footnotes"] = {["simurgh"]=true,},
  ["babel-kalaallisut"] = {["babel"]=true,},
  ["libgreek"] = {["libgreek"]=true,},
  ["allpgoth"] = {["bookhands"]=true,},
  ["afonts2"] = {["arabtex"]=true,},
  ["libertinus-otf"] = {["libertinus-otf"]=true,},
  ["dozenal"] = {["dozenal"]=true,},
  ["epigrafica"] = {["epigrafica"]=true,},
  ["libertinegc"] = {["libertinegc"]=true,},
  ["a5comb"] = {["a5comb"]=true,},
  ["hyph-sh-latn"] = {},
  ["libertineotf"] = {["libertine"]=true,},
  ["bidipagegrid"] = {["bidipagegrid"]=true,},
  ["libertineMono"] = {["libertine"]=true,},
  ["fgruler"] = {["fgruler"]=true,},
  ["fix-tudscrfonts"] = {["tudscr"]=true,},
  ["biolinum-type1"] = {["libertine"]=true,},
  ["stacklet"] = {["catcodes"]=true,},
  ["LinLibertine_R"] = {["libertine"]=true,},
  ["fetchcls"] = {["fetchcls"]=true,},
  ["pdfwebmac"] = {["cweb"]=true,},
  ["ascan"] = {["arabtex"]=true,},
  ["pismo"] = {},
  ["bibleref-french"] = {["bibleref-french"]=true,},
  ["otchet"] = {},
  ["lhcyrwin"] = {},
  ["sseqforeach.code"] = {["spectralsequences"]=true,},
  ["irish"] = {["babel"]=true,},
  ["ifthenx"] = {["ifthenx"]=true,},
  ["codemac"] = {["codepage"]=true,},
  ["versions"] = {["versions"]=true,},
  ["kam-csf"] = {["enctex"]=true,},
  ["hyph-ro"] = {},
  ["font_iwona-condensed-medium"] = {["font-change"]=true,},
  ["ctablestack"] = {["ctablestack"]=true,},
  ["s-simpleslides-FuzzyTopic"] = {["context-simpleslides"]=true,},
  ["DejaVuSerif"] = {["dejavu"]=true,},
  ["musixppff"] = {["musixtex"]=true,},
  ["currency"] = {["currency"]=true,},
  ["t1enc-u"] = {},
  ["esstixcal"] = {["esstix"]=true,},
  ["testkern"] = {["lh"]=true,},
  ["pst-tvz"] = {["pst-tvz"]=true,},
  ["modref"] = {["modref"]=true,},
  ["luatex-languages"] = {["context"]=true,},
  ["akkparskip"] = {["akktex"]=true,},
  ["luatexja"] = {["luatexja"]=true,},
  ["bibl-apa-fr"] = {["context"]=true,},
  ["lh-t2cxccr"] = {["lh"]=true,},
  ["beamerthemeRochester"] = {["beamer"]=true,},
  ["PPRarabic"] = {["arabi"]=true,},
  ["chemmacros.module.acid-base.code"] = {["chemmacros"]=true,},
  ["kolabels-utf"] = {["cjk-ko"]=true,},
  ["afonts0"] = {["arabtex"]=true,},
  ["multimediasymbols"] = {["beamer"]=true,},
  ["avremu"] = {["avremu"]=true,},
  ["macro-centaure"] = {["pas-cv"]=true,},
  ["auto-pst-pdf-lua"] = {["auto-pst-pdf-lua"]=true,},
  ["lh-t2axccr"] = {["lh"]=true,},
  ["cases"] = {["cases"]=true,},
  ["verse"] = {["verse"]=true,},
  ["lh-ot2ccr"] = {["lh"]=true,},
  ["atabg"] = {["arabtex"]=true,},
  ["doclicense"] = {["doclicense"]=true,},
  ["chemmacros.module.phases.code"] = {["chemmacros"]=true,},
  ["loadhyph-it"] = {},
  ["psfragx"] = {["psfragx"]=true,},
  ["akklecture"] = {["akktex"]=true,},
  ["caladea"] = {["caladea"]=true,},
  ["pst-grad"] = {["pst-grad"]=true,},
  ["skdoc"] = {["skdoc"]=true,},
  ["rcsinfo"] = {["rcsinfo"]=true,},
  ["xinttools"] = {["xint"]=true,},
  ["algobox"] = {["algobox"]=true,},
  ["beamercolorthemeCuerna"] = {["beamertheme-cuerna"]=true,},
  ["pgfmath.code"] = {["pgf"]=true,},
  ["s-simpleslides-ThickStripes"] = {["context-simpleslides"]=true,},
  ["hyph-et.ec"] = {},
  ["hyph-el-polyton"] = {["hyphen-greek"]=true,},
  ["gmampulex"] = {["gmutils"]=true,},
  ["lengthconvert"] = {["lengthconvert"]=true,},
  ["CJK"] = {["cjk"]=true,},
  ["ledpar"] = {["ledmac"]=true,},
  ["ledmac"] = {["ledmac"]=true,},
  ["menus"] = {["dialogl"]=true,},
  ["hepunits"] = {["hepunits"]=true,},
  ["leaflet"] = {["leaflet"]=true,},
  ["graphfig"] = {["bosisio"]=true,},
  ["epsfig"] = {["graphics"]=true,},
  ["example-image-10x16"] = {["mwe"]=true,},
  ["fvrb-ex"] = {["fancyvrb"]=true,},
  ["fcavtex"] = {["fcavtex"]=true,},
  ["ledarab"] = {["ledmac"]=true,},
  ["t-pgfcor"] = {["pgf"]=true,},
  ["lewis"] = {["lewis"]=true,},
  ["pgfplotslibrary.code"] = {["pgfplots"]=true,},
  ["eledpar"] = {["eledmac"]=true,},
  ["DejaVuSans"] = {["dejavu"]=true,},
  ["ltr-areas"] = {["lecturer"]=true,},
  ["zref-thepage"] = {["oberdiek"]=true,},
  ["loadhyph-th"] = {},
  ["babel-tamil"] = {["babel"]=true,},
  ["jurabib"] = {["jurabib"]=true,},
  ["loadhyph-is"] = {},
  ["amssymb"] = {["amsfonts"]=true,},
  ["rcs"] = {["rcs"]=true,},
  ["mfparxsp"] = {["ednotes"]=true,},
  ["leadsheets.library.transposing.code"] = {["leadsheets"]=true,},
  ["gmdoc"] = {["gmdoc"]=true,},
  ["pdfcolor"] = {["pdftex"]=true,},
  ["fp"] = {["fp"]=true,},
  ["lwarp-pagesel"] = {["lwarp"]=true,},
  ["leadsheets.library.musicsymbols.code"] = {["leadsheets"]=true,},
  ["gtrlang.danish.code"] = {["genealogytree"]=true,},
  ["dhucs-enumitem"] = {["kotex-utf"]=true,},
  ["fixmetodonotes"] = {["fixmetodonotes"]=true,},
  ["ednotes"] = {["ednotes"]=true,},
  ["zapf"] = {["context"]=true,},
  ["leadsheets.library.chordnames.code"] = {["leadsheets"]=true,},
  ["scalefnt"] = {["carlisle"]=true,},
  ["beamerthemePhnomPenh"] = {["beamertheme-phnompenh"]=true,},
  ["phfquotetext"] = {["phfquotetext"]=true,},
  ["bewerbung-cv-oldstyle"] = {["bewerbung"]=true,},
  ["lcg"] = {["lcg"]=true,},
  ["zhfounderfonts"] = {["ctex"]=true,},
  ["gmenvir"] = {["gmutils"]=true,},
  ["selectpage"] = {},
  ["layaureo"] = {["layaureo"]=true,},
  ["dlfltxbmarkup"] = {["dlfltxb"]=true,},
  ["calxxxx-yyyy-doc"] = {["calxxxx-yyyy"]=true,},
  ["doclicense-CC-by-sa-3.0-latex"] = {["doclicense"]=true,},
  ["font_pagella"] = {["font-change"]=true,},
  ["encmacro"] = {["enctex"]=true,},
  ["gtrcore.parser.code"] = {["genealogytree"]=true,},
  ["fithesis-fsps"] = {["fithesis"]=true,},
  ["mtgreek"] = {["mtgreek"]=true,},
  ["lualatex-patch-kernel"] = {},
  ["regstud"] = {["economic"]=true,},
  ["combcite"] = {["combine"]=true,},
  ["latexbug"] = {["latexbug"]=true,},
  ["beamerthemeBergen"] = {["beamer"]=true,},
  ["babel-catalan"] = {["babel"]=true,},
  ["moderncvcolorburgundy"] = {["moderncv"]=true,},
  ["startex"] = {["startex"]=true,},
  ["babel-french-ca"] = {["babel"]=true,},
  ["texgraphicx"] = {["latex-make"]=true,},
  ["epsf-dvipdfmx"] = {["epsf-dvipdfmx"]=true,},
  ["lstlang1"] = {["listings"]=true,},
  ["sourcecodepro-type1-autoinst"] = {["sourcecodepro"]=true,},
  ["bpfats14"] = {["iso10303"]=true,},
  ["libertine-type1"] = {["libertine"]=true,},
  ["tracefnt"] = {["latex"]=true,},
  ["moderncvbodyiii"] = {["moderncv"]=true,},
  ["eskdtitlebase"] = {["eskdx"]=true,},
  ["fithesis-phil"] = {["fithesis"]=true,},
  ["chemsec"] = {["chemsec"]=true,},
  ["fnpct"] = {["fnpct"]=true,},
  ["begriff-bguq"] = {["bguq"]=true,},
  ["babel-bena"] = {["babel"]=true,},
  ["beamerinnerthemetorinoth"] = {["beamer2thesis"]=true,},
  ["showidx"] = {["latex"]=true,},
  ["soton-beamer"] = {["soton"]=true,},
  ["colordvi"] = {["dvips"]=true,},
  ["proc"] = {["latex"]=true,},
  ["cntperchap"] = {["cntperchap"]=true,},
  ["onepagem"] = {["piff"]=true,},
  ["babel-serbian-cyrillic-kosovo"] = {["babel"]=true,},
  ["BOONDOX-frak"] = {["boondox"]=true,},
  ["endiagram"] = {["endiagram"]=true,},
  ["alltt"] = {["latex"]=true,},
  ["guide"] = {["getoptk"]=true,},
  ["nfssfont"] = {["latex"]=true,},
  ["elzcards"] = {["elzcards"]=true,},
  ["xoblivoir"] = {["kotex-oblivoir"]=true,},
  ["arimo"] = {["arimo"]=true,},
  ["babel-norwegianbokmal"] = {["babel"]=true,},
  ["supp-mis"] = {["context"]=true,},
  ["minimal"] = {["latex"]=true,},
  ["makeidx"] = {["latex"]=true,},
  ["akkmathpaper"] = {["akktex"]=true,},
  ["ltxdoc"] = {["latex"]=true,},
  ["ltnews"] = {["latex"]=true,},
  ["xyemtex"] = {["xypic"]=true,},
  ["leqno"] = {["latex"]=true,},
  ["latexsym"] = {["latex"]=true,},
  ["scrlayer-notecolumn"] = {},
  ["hyph-quote-pms"] = {},
  ["hyph-ta"] = {},
  ["circuitikz"] = {["circuitikz"]=true,},
  ["greektonoi"] = {["greektonoi"]=true,},
  ["lablst"] = {["latex"]=true,},
  ["amsbooka"] = {["amscls"]=true,},
  ["afonts1"] = {["arabtex"]=true,},
  ["carrol"] = {["context"]=true,},
  ["diadia"] = {["diadia"]=true,},
  ["fancyref"] = {["fancyref"]=true,},
  ["oldprsn"] = {["archaic"]=true,},
  ["mlsgalig"] = {["montex"]=true,},
  ["inputenc"] = {["latex"]=true,},
  ["kantlipsum"] = {["kantlipsum"]=true,},
  ["AdobeColorProfiles"] = {["pdfx"]=true,},
  ["musixmad"] = {["musixtex"]=true,},
  ["moderncvheadi"] = {["moderncv"]=true,},
  ["cs-libertine"] = {},
  ["common-textmarkup"] = {},
  ["latexdemo"] = {["latexdemo"]=true,},
  ["endnotesj"] = {["endnotesj"]=true,},
  ["bibref"] = {["index"]=true,},
  ["units"] = {["units"]=true,},
  ["pgflibrarypatterns.code"] = {["pgf"]=true,},
  ["fix-cm"] = {["latex"]=true,},
  ["chemfig"] = {["chemfig"]=true,},
  ["inslrmaj"] = {["bookhands"]=true,},
  ["cbookman"] = {},
  ["book"] = {["latex"]=true,},
  ["lwarp-marginfit"] = {["lwarp"]=true,},
  ["fwlw"] = {["fwlw"]=true,},
  ["article"] = {["latex"]=true,},
  ["jsverb"] = {["jsclasses"]=true,},
  ["tagpdf"] = {["tagpdf"]=true,},
  ["langsci-tbls"] = {["langsci"]=true,},
  ["betababel"] = {["betababel"]=true,},
  ["beamerthemeAntibes"] = {["beamer"]=true,},
  ["langsci-subparts"] = {["langsci"]=true,},
  ["semtrans"] = {["jknapltx"]=true,},
  ["langsci-jambox"] = {["langsci"]=true,},
  ["langsci-hyphenation-pt"] = {["langsci"]=true,},
  ["langsci-hyphenation-fr"] = {["langsci"]=true,},
  ["graphicxpsd"] = {["graphicxpsd"]=true,},
  ["musixste"] = {["musixtex"]=true,},
  ["lwarp-xpiano"] = {["lwarp"]=true,},
  ["opmac-bib-simple"] = {},
  ["kotexplain"] = {["kotex-plain"]=true,},
  ["chemmacros.module.isotopes.code"] = {["chemmacros"]=true,},
  ["iftex"] = {["iftex"]=true,},
  ["elhyph16"] = {},
  ["bardiag"] = {["bardiag"]=true,},
  ["romanian"] = {["babel"]=true,},
  ["babel-norwegiannynorsk"] = {["babel"]=true,},
  ["thmbox"] = {["thmbox"]=true,},
  ["ladder"] = {["ladder"]=true,},
  ["labelschanged"] = {["labelschanged"]=true,},
  ["udesoftec-extra"] = {["udesoftec"]=true,},
  ["beamerthemeCuerna"] = {["beamertheme-cuerna"]=true,},
  ["serbian"] = {["babel"]=true,},
  ["xfrac"] = {["l3packages"]=true,},
  ["hyph-la.ec"] = {},
  ["hyph-lt.l7x"] = {},
  ["rawfonts"] = {["tools"]=true,},
  ["BOONDOX-calo"] = {["boondox"]=true,},
  ["Rosario"] = {["rosario"]=true,},
  ["pgflibrarygraphdrawing.force.code"] = {["pgf"]=true,},
  ["lwarp-glossaries"] = {["lwarp"]=true,},
  ["amsaddr"] = {["amsaddr"]=true,},
  ["GoudyIn"] = {["cfr-initials"]=true,},
  ["pgflibraryshapes.gates.logic.US.code"] = {["pgf"]=true,},
  ["somedefs"] = {["tools"]=true,},
  ["gtrlang.french.code"] = {["genealogytree"]=true,},
  ["amscd"] = {["amsmath"]=true,},
  ["kmath"] = {["kerkis"]=true,},
  ["trajan"] = {["trajan"]=true,},
  ["loadhyph-sr-cyrl"] = {},
  ["ocgx2"] = {["ocgx2"]=true,},
  ["proofread"] = {["proofread"]=true,},
  ["upgreek"] = {["was"]=true,},
  ["kurier-math"] = {["kurier"]=true,},
  ["akkmathbasic"] = {["akktex"]=true,},
  ["materie"] = {["context"]=true,},
  ["hyph-kmr"] = {},
  ["titlepic"] = {["titlepic"]=true,},
  ["nwejmart"] = {["nwejm"]=true,},
  ["ksp-thesis"] = {["ksp-thesis"]=true,},
  ["topfloat"] = {["topfloat"]=true,},
  ["utf8cseq"] = {["enctex"]=true,},
  ["finnish"] = {["babel"]=true,},
  ["kosections-utf"] = {["kotex-utf"]=true,},
  ["dhucs"] = {["kotex-utf"]=true,},
  ["beamercolorthemeowl"] = {["beamercolorthemeowl"]=true,},
  ["dhucs-nanumfont"] = {["kotex-utf"]=true,},
  ["kotex-logo"] = {["kotex-utf"]=true,},
  ["dhucsfn"] = {["kotex-utf"]=true,},
  ["autoarea"] = {["autoarea"]=true,},
  ["eplain"] = {["eplain"]=true,},
  ["AnnSton"] = {["cfr-initials"]=true,},
  ["chemformula"] = {["chemformula"]=true,},
  ["jlabels"] = {["jlabels"]=true,},
  ["gates"] = {["gates"]=true,},
  ["plain-1250-cs"] = {["enctex"]=true,},
  ["huncial"] = {["bookhands"]=true,},
  ["teubner"] = {["teubner"]=true,},
  ["frcursive"] = {["frcursive"]=true,},
  ["lwarp-dcolumn"] = {["lwarp"]=true,},
  ["HAPciment"] = {["ha-prosper"]=true,},
  ["xob-font"] = {["kotex-oblivoir"]=true,},
  ["beamerthemeI6pd2"] = {["beamerposter"]=true,},
  ["lwarp-caption2"] = {["lwarp"]=true,},
  ["babel-hausa-gh"] = {["babel"]=true,},
  ["fp-eqn"] = {["fp"]=true,},
  ["documentation"] = {["documentation"]=true,},
  ["cmath"] = {["stex"]=true,},
  ["autofe"] = {["ucs"]=true,},
  ["CJKpunct"] = {["cjkpunct"]=true,},
  ["common-lists"] = {},
  ["modiagram"] = {["modiagram"]=true,},
  ["cu-kinovar"] = {["churchslavonic"]=true,},
  ["s-simpleslides-BoxedTitle"] = {["context-simpleslides"]=true,},
  ["mugsthesis"] = {["mugsthesis"]=true,},
  ["memucs-gremph"] = {["kotex-oblivoir"]=true,},
  ["cvthemebidiclassic"] = {["bidi"]=true,},
  ["HA-prosper"] = {["ha-prosper"]=true,},
  ["assignment"] = {["assignment"]=true,},
  ["fxlayoutpdfcsigmargin"] = {["fixme"]=true,},
  ["embrac"] = {["embrac"]=true,},
  ["auncial"] = {["auncial-new"]=true,},
  ["plnfss"] = {["plnfss"]=true,},
  ["memhangul-patch"] = {["kotex-oblivoir"]=true,},
  ["example-glossaries-images"] = {["glossaries"]=true,},
  ["hfontsel"] = {["kotex-oblivoir"]=true,},
  ["tensor"] = {["tensor"]=true,},
  ["doclicense-CC-by-nc-nd-4.0-plaintext"] = {["doclicense"]=true,},
  ["cnltx-listings"] = {["cnltx"]=true,},
  ["babel-british"] = {["babel"]=true,},
  ["10_5"] = {["kotex-oblivoir"]=true,},
  ["lwarp-a4wide"] = {["lwarp"]=true,},
  ["xbmks"] = {["xbmks"]=true,},
  ["cyr"] = {},
  ["babel-colognian"] = {["babel"]=true,},
  ["babel-french-switzerland"] = {["babel"]=true,},
  ["lcy"] = {["cyrillic"]=true,},
  ["komacv-classic"] = {["komacv"]=true,},
  ["powerdot-sailor"] = {["powerdot"]=true,},
  ["glossaries-accsupp"] = {["glossaries"]=true,},
  ["Romantik"] = {["cfr-initials"]=true,},
  ["glossary-hypernav"] = {["glossaries"]=true,},
  ["beamercolorthemeseahorse"] = {["beamer"]=true,},
  ["pgflibraryshapes.gates.ee.code"] = {["pgf"]=true,},
  ["scrdoc"] = {},
  ["bxtoolbox"] = {["bxbase"]=true,},
  ["HAPTycja"] = {["ha-prosper"]=true,},
  ["chemmacros.module.newman.code"] = {["chemmacros"]=true,},
  ["babel-kako"] = {["babel"]=true,},
  ["ifsym"] = {["ifsym"]=true,},
  ["arabicnumbers"] = {["polyglossia"]=true,},
  ["scrpage2"] = {},
  ["dtxdescribe"] = {["dtxdescribe"]=true,},
  ["cquthesis"] = {["cquthesis"]=true,},
  ["chemcono"] = {["chemcono"]=true,},
  ["tikzcodeblocks"] = {["tikzcodeblocks"]=true,},
  ["bidicontour"] = {["bidicontour"]=true,},
  ["bclogo"] = {["bclogo"]=true,},
  ["decorule"] = {["decorule"]=true,},
  ["loadhyph-sl"] = {},
  ["cap_pas"] = {["c-pascal"]=true,},
  ["wallcalendar-spanish"] = {["wallcalendar"]=true,},
  ["beamerbaselocalstructure"] = {["beamer"]=true,},
  ["hyph-el-monoton"] = {["hyphen-greek"]=true,},
  ["fvextra"] = {["fvextra"]=true,},
  ["scrjura"] = {},
  ["textcase"] = {["textcase"]=true,},
  ["babel-bosnian-latin"] = {["babel"]=true,},
  ["aconfig"] = {["arabtex"]=true,},
  ["preface"] = {},
  ["babel-punjabi-arab"] = {["babel"]=true,},
  ["douglas"] = {["context"]=true,},
  ["dlfltxbmisc"] = {["dlfltxb"]=true,},
  ["tikzlibrarytrees.code"] = {["pgf"]=true,},
  ["common-compatibility"] = {},
  ["sesamanuelTIKZ"] = {["sesamanuel"]=true,},
  ["common-typearea"] = {},
  ["beamerthemesplit"] = {["beamer"]=true,},
  ["buck"] = {["arabtex"]=true,},
  ["backref"] = {["hyperref"]=true,},
  ["gfsneohellenicot"] = {["gfsneohellenicmath"]=true,},
  ["s-pre-22"] = {["context"]=true,},
  ["cjkutf8-nanummjhanja"] = {["cjk-ko"]=true,},
  ["il2-852"] = {["enctex"]=true,},
  ["pdfuni"] = {},
  ["babel-occitan"] = {["babel"]=true,},
  ["abbr"] = {["abbr"]=true,},
  ["slgreek"] = {["levy"]=true,},
  ["ctexbook"] = {["ctex"]=true,},
  ["bibentry"] = {["natbib"]=true,},
  ["pdftexmagfix"] = {},
  ["common-fontsize"] = {},
  ["komabug"] = {},
  ["pgfcirccurrent"] = {["circuitikz"]=true,},
  ["tugproc"] = {["tugboat-plain"]=true,},
  ["beamerprosper"] = {["beamer"]=true,},
  ["klusec"] = {["kluwer"]=true,},
  ["pgflibraryshapes"] = {["pgf"]=true,},
  ["chars-8z"] = {},
  ["babel-northernsami"] = {["babel"]=true,},
  ["allimaj"] = {["bookhands"]=true,},
  ["babel-sanskrit-deva"] = {["babel"]=true,},
  ["lwarp-fullpage"] = {["lwarp"]=true,},
  ["esrelation"] = {["esrelation"]=true,},
  ["dehypht"] = {},
  ["ifvtex"] = {["oberdiek"]=true,},
  ["babel-malay-singapore"] = {["babel"]=true,},
  ["phoenician"] = {["archaic"]=true,},
  ["filecontents"] = {["filecontents"]=true,},
  ["pgfmathfunctions.base.code"] = {["pgf"]=true,},
  ["klumath"] = {["kluwer"]=true,},
  ["xcoffins"] = {["l3experimental"]=true,},
  ["babel-welsh"] = {["babel"]=true,},
  ["klumac"] = {["kluwer"]=true,},
  ["tikzexternalshared.code"] = {["pgf"]=true,},
  ["babel-burmese"] = {["babel"]=true,},
  ["colordoc"] = {["colordoc"]=true,},
  ["klulist"] = {["kluwer"]=true,},
  ["babel-german-ch"] = {["babel"]=true,},
  ["biocon-old"] = {["biocon"]=true,},
  ["fxlayoutpdfnote"] = {["fixme"]=true,},
  ["hausarbeit-jura"] = {["hausarbeit-jura"]=true,},
  ["t-cmttbf"] = {["context-cmttbf"]=true,},
  ["csenc-w"] = {},
  ["postage"] = {["postage"]=true,},
  ["keyfloat"] = {["keyfloat"]=true,},
  ["luatex-loader"] = {["oberdiek"]=true,},
  ["bgteubner"] = {["bgteubner"]=true,},
  ["grffile"] = {["oberdiek"]=true,},
  ["alists"] = {["arabtex"]=true,},
  ["loadhyph-eo"] = {},
  ["babel-usenglish"] = {["babel"]=true,},
  ["chemmacros.module.redox.code"] = {["chemmacros"]=true,},
  ["tikzlibraryknots.code"] = {["spath3"]=true,},
  ["loadhyph-fur"] = {},
  ["interfaces-bookmark"] = {["interfaces"]=true,},
  ["hyph-hi"] = {},
  ["kerntest"] = {["kerntest"]=true,},
  ["bidi-atbegshi"] = {["bidi-atbegshi"]=true,},
  ["tikzlibrarycircuits.ee.code"] = {["pgf"]=true,},
  ["plex-serif"] = {["plex"]=true,},
  ["kvmacros"] = {["karnaugh"]=true,},
  ["btxmac"] = {["eplain"]=true,},
  ["ijsra"] = {["ijsra"]=true,},
  ["jvlisting"] = {["jvlisting"]=true,},
  ["juraovw"] = {["juramisc"]=true,},
  ["initcar"] = {["codepage"]=true,},
  ["hyph-oc"] = {},
  ["interfaces-fancyhdr"] = {["interfaces"]=true,},
  ["babel-ngomba"] = {["babel"]=true,},
  ["t-pgfbsh"] = {["pgf"]=true,},
  ["jurabook"] = {["juramisc"]=true,},
  ["Tabbing"] = {["tabbing"]=true,},
  ["pst-magneticfield"] = {["pst-magneticfield"]=true,},
  ["farsifnt"] = {["arabi"]=true,},
  ["loadhyph-de-1996"] = {},
  ["aliasctr"] = {["thmtools"]=true,},
  ["lwarp-fwlw"] = {["lwarp"]=true,},
  ["resizegather"] = {["oberdiek"]=true,},
  ["txxextra"] = {},
  ["hyph-as"] = {},
  ["babel-chiga"] = {["babel"]=true,},
  ["jslectureplanner"] = {["jslectureplanner"]=true,},
  ["okuverb"] = {["jsclasses"]=true,},
  ["abstract"] = {["abstract"]=true,},
  ["avr.bitops"] = {["avremu"]=true,},
  ["BOONDOX-uprscr"] = {["boondox"]=true,},
  ["utf8unkn"] = {["enctex"]=true,},
  ["example-image-letter"] = {["mwe"]=true,},
  ["pgflibraryintersections.code"] = {["pgf"]=true,},
  ["xepersian"] = {["xepersian"]=true,},
  ["kotex"] = {["cjk-ko"]=true,},
  ["schild"] = {["js-misc"]=true,},
  ["cs-bonum"] = {},
  ["abntex2cite"] = {["abntex2"]=true,},
  ["ghab"] = {["ghab"]=true,},
  ["fotex"] = {},
  ["jpsj2"] = {["jpsj"]=true,},
  ["testhyphens"] = {["testhyphens"]=true,},
  ["cascadilla"] = {["cascadilla"]=true,},
  ["inversepath"] = {["inversepath"]=true,},
  ["jmlrutils"] = {["jmlr"]=true,},
  ["assurechemist"] = {["xymtex"]=true,},
  ["lcydefs"] = {["cyrillic"]=true,},
  ["matlab-prettifier"] = {["matlab-prettifier"]=true,},
  ["thumb"] = {["thumb"]=true,},
  ["babel-austrian"] = {["babel"]=true,},
  ["hyph-nl.ec"] = {},
  ["pgflibrarydatavisualization.barcharts.code"] = {["pgf"]=true,},
  ["pst-geometrictools"] = {["pst-geometrictools"]=true,},
  ["parboxx"] = {["jknapltx"]=true,},
  ["mlutf"] = {["japanese-otf-uptex"]=true,},
  ["babel-arabic-dz"] = {["babel"]=true,},
  ["vntex"] = {["vntex"]=true,},
  ["jamtimes"] = {["jamtimes"]=true,},
  ["eskdappsheet"] = {["eskdx"]=true,},
  ["akkmathtext"] = {["akktex"]=true,},
  ["iwona-math"] = {["iwona"]=true,},
  ["iwona"] = {["iwona"]=true,},
  ["tufte-book"] = {["tufte-latex"]=true,},
  ["fetamont"] = {["fetamont"]=true,},
  [""] = {["tools"]=true,},
  ["pdf14"] = {["pdf14"]=true,},
  ["third-rep"] = {["muthesis"]=true,},
  ["novel-TextMacros"] = {["novel"]=true,},
  ["isonums"] = {["isonums"]=true,},
  ["isomath"] = {["isomath"]=true,},
  ["mtx"] = {["m-tx"]=true,},
  ["pmfrench"] = {["e-french"]=true,},
  ["musixcho"] = {["musixtex"]=true,},
  ["semlayer"] = {["seminar"]=true,},
  ["tikzpeople.shape.conductor"] = {["tikzpeople"]=true,},
  ["skeyval-testclass"] = {["skeyval"]=true,},
  ["pdfsync"] = {["pdfsync"]=true,},
  ["pgfcoregraphicstate.code"] = {["pgf"]=true,},
  ["lwarp-t1enc"] = {["lwarp"]=true,},
  ["chemmacros.module.chemformula.code"] = {["chemmacros"]=true,},
  ["beamerfontthemetorinoth"] = {["beamer2thesis"]=true,},
  ["12many"] = {["12many"]=true,},
  ["cu-num"] = {["churchslavonic"]=true,},
  ["amsfn"] = {["ofs"]=true,},
  ["bpfap4"] = {["iso10303"]=true,},
  ["plext"] = {["platex"]=true,},
  ["bpfap1"] = {["iso10303"]=true,},
  ["omdoc"] = {["stex"]=true,},
  ["fithesis-law"] = {["fithesis"]=true,},
  ["plain-852-cs"] = {["enctex"]=true,},
  ["keycommand"] = {["keycommand"]=true,},
  ["vruler"] = {["vruler"]=true,},
  ["refrep"] = {["refman"]=true,},
  ["apendint"] = {["iso10303"]=true,},
  ["greek"] = {["babel"]=true,},
  ["tudscrbook"] = {["tudscr"]=true,},
  ["IEEEconf"] = {["ieeeconf"]=true,},
  ["beamerthemeZH"] = {["beamerposter"]=true,},
  ["lualatex-reset-codes"] = {},
  ["ionumbers"] = {["ionumbers"]=true,},
  ["czech"] = {["babel"]=true,["cslatex"]=true,},
  ["ecclesiastic"] = {["ecclesiastic"]=true,},
  ["wallcalendar-serbian"] = {["wallcalendar"]=true,},
  ["ellipse"] = {["ellipse"]=true,},
  ["nuc"] = {["nuc"]=true,},
  ["amsppt1"] = {["amstex"]=true,},
  ["elements"] = {["elements"]=true,},
  ["interpreter"] = {["interpreter"]=true,},
  ["interfaces-scrlfile"] = {["interfaces"]=true,},
  ["cstypo-tex"] = {["cstypo"]=true,},
  ["cascade"] = {["cascade"]=true,},
  ["alocal"] = {["arabtex"]=true,},
  ["eskdspecii"] = {["eskdx"]=true,},
  ["bounddvi"] = {["gentombow"]=true,},
  ["ibycusps"] = {["ibygrk"]=true,},
  ["pxjafont"] = {["pxchfon"]=true,},
  ["churchslavonic"] = {["churchslavonic"]=true,},
  ["isodateo"] = {["isodate"]=true,},
  ["jumplines"] = {["jumplines"]=true,},
  ["tgothic"] = {["bookhands"]=true,},
  ["grant"] = {["grant"]=true,},
  ["articleingud"] = {["articleingud"]=true,},
  ["inlinedef"] = {["inlinedef"]=true,},
  ["1250-csf"] = {["enctex"]=true,},
  ["edcntwd0"] = {["ednotes"]=true,},
  ["mltex"] = {["mltex"]=true,},
  ["bibleref"] = {["bibleref"]=true,},
  ["frege"] = {["frege"]=true,},
  ["suffix"] = {["bigfoot"]=true,},
  ["algorithm"] = {["algorithms"]=true,},
  ["babel-uzbek"] = {["babel"]=true,},
  ["pgothic"] = {["bookhands"]=true,},
  ["hescan"] = {["arabtex"]=true,},
  ["lh-x2ccr"] = {["lh"]=true,},
  ["verbatim"] = {["tools"]=true,},
  ["imakeidx"] = {["imakeidx"]=true,},
  ["babelbib"] = {["babelbib"]=true,},
  ["t-cmscbf"] = {["context-cmscbf"]=true,},
  ["image-gallery"] = {["image-gallery"]=true,},
  ["iitem"] = {["iitem"]=true,},
  ["invoice"] = {["invoice"]=true,},
  ["citeall"] = {["citeall"]=true,},
  ["bewerbung-cv-classic"] = {["bewerbung"]=true,},
  ["lsc"] = {["lsc"]=true,},
  ["xypdf-li"] = {["xypic"]=true,},
  ["musikui"] = {["musikui"]=true,},
  ["hbaw"] = {["fancyvrb"]=true,},
  ["utf8lat1"] = {["enctex"]=true,},
  ["t-typearea"] = {["context-typearea"]=true,},
  ["CoverPage"] = {["coverpage"]=true,},
  ["chessboard-keys-pgf"] = {["chessboard"]=true,},
  ["fxenvlayoutcolor"] = {["fixme"]=true,},
  ["ifetex"] = {["ifetex"]=true,},
  ["diagram"] = {["chess-problem-diagrams"]=true,},
  ["lltjp-footmisc"] = {["luatexja"]=true,},
  ["beamerbaseframecomponents"] = {["beamer"]=true,},
  ["babel-zarma"] = {["babel"]=true,},
  ["arabiftoday"] = {["arabi"]=true,},
  ["beamerouterthememiniframes"] = {["beamer"]=true,},
  ["nox"] = {["nox"]=true,},
  ["lwarp-newclude"] = {["lwarp"]=true,},
  ["euro"] = {["euro"]=true,},
  ["hyph-oc.ec"] = {},
  ["komacv-oldstyle"] = {["komacv"]=true,},
  ["eltex3"] = {["eltex"]=true,},
  ["dejavu"] = {["dejavu"]=true,},
  ["environ"] = {["environ"]=true,},
  ["dotlessj"] = {["carlisle"]=true,},
  ["langsci-lgr"] = {["langsci"]=true,},
  ["babel-chinese-hans-mo"] = {["babel"]=true,},
  ["msc"] = {["msc"]=true,},
  ["chet"] = {["chet"]=true,},
  ["mfpextra"] = {["minifp"]=true,},
  ["foreign"] = {["foreign"]=true,},
  ["t-floatnumber"] = {["context-account"]=true,},
  ["hypdvips"] = {["hypdvips"]=true,},
  ["pst-geo"] = {["pst-geo"]=true,},
  ["gtrlang.english.code"] = {["genealogytree"]=true,},
  ["psgo"] = {["psgo"]=true,},
  ["loadhyph-hsb"] = {},
  ["babel-american"] = {["babel"]=true,},
  ["babel-kabyle"] = {["babel"]=true,},
  ["babel-indonesian"] = {["babel"]=true,},
  ["ki-math"] = {},
  ["hyph-tk"] = {},
  ["fontloader-languages"] = {["luaotfload"]=true,},
  ["bidihl"] = {["bidihl"]=true,},
  ["moodle"] = {["moodle"]=true,},
  ["hyph-quote-fr"] = {},
  ["chemmacros.module.base.code"] = {["chemmacros"]=true,},
  ["bpchem"] = {["bpchem"]=true,},
  ["bpfats4"] = {["iso10303"]=true,},
  ["beamerthemeMarburg"] = {["beamer"]=true,},
  ["everyshi"] = {["ms"]=true,},
  ["hustthesis"] = {["hustthesis"]=true,},
  ["cell"] = {["cell"]=true,},
  ["loadhyph-es"] = {["hyphen-spanish"]=true,},
  ["hyph-sl"] = {},
  ["lwarp-a5comb"] = {["lwarp"]=true,},
  ["hyph-sk"] = {},
  ["icsv"] = {["icsv"]=true,},
  ["blogexec"] = {["morehype"]=true,},
  ["letterspacing"] = {},
  ["pcatcode"] = {["amsrefs"]=true,},
  ["logsys"] = {["coordsys"]=true,},
  ["hyph-ro.ec"] = {},
  ["LinBiolinum_K"] = {["libertine"]=true,},
  ["fapapersize"] = {["kotex-oblivoir"]=true,},
  ["scrartcl"] = {},
  ["hgbabbrev"] = {["hagenberg-thesis"]=true,},
  ["akkconditional"] = {["akktex"]=true,},
  ["uniquecounter"] = {["oberdiek"]=true,},
  ["arrayjob"] = {["arrayjobx"]=true,},
  ["hyph-pl.qx"] = {},
  ["babel-chinese-simplified-macausarchina"] = {["babel"]=true,},
  ["loadhyph-pl"] = {},
  ["babel-sanskrit-gujarati"] = {["babel"]=true,},
  ["loadhyph-oc"] = {},
  ["hyph-no"] = {},
  ["chmst-pdf"] = {["xymtex"]=true,},
  ["tucv"] = {["tucv"]=true,},
  ["loadhyph-nb"] = {},
  ["hyph-mn-cyrl-x-lmc"] = {},
  ["babel-canadian"] = {["babel"]=true,},
  ["babel-scottishgaelic"] = {["babel"]=true,},
  ["babel-machame"] = {["babel"]=true,},
  ["pstfp"] = {["bardiag"]=true,},
  ["1250-il2"] = {["enctex"]=true,},
  ["il2-kam"] = {["enctex"]=true,},
  ["hyph-la-x-classic"] = {},
  ["hyph-la-x-liturgic.ec"] = {},
  ["common-marginpar"] = {},
  ["chemmacros.module.lang.code"] = {["chemmacros"]=true,},
  ["hyph-kmr.ec"] = {},
  ["cnltx-tools"] = {["cnltx"]=true,},
  ["luatexja-adjust"] = {["luatexja"]=true,},
  ["hyph-nb"] = {},
  ["ecltree"] = {["ecltree"]=true,},
  ["tagpair"] = {["tagpair"]=true,},
  ["dehyphts-x-2018-03-31"] = {["dehyph-exptl"]=true,},
  ["beamerbasetwoscreens"] = {["beamer"]=true,},
  ["framed"] = {["framed"]=true,},
  ["asycolors"] = {["asymptote"]=true,},
  ["beamerouterthemehohenheim"] = {["hobete"]=true,},
  ["bxcalcize"] = {["bxcalc"]=true,},
  ["dox"] = {["dox"]=true,},
  ["doipubmed"] = {["doipubmed"]=true,},
  ["fsbispit"] = {["hrlatex"]=true,},
  ["cap_comm"] = {["c-pascal"]=true,},
  ["dcfonts"] = {},
  ["testidx-glossaries"] = {["testidx"]=true,},
  ["ejpecp"] = {["ejpecp"]=true,},
  ["lh-ot2xccr"] = {["lh"]=true,},
  ["loadhyph-or"] = {},
  ["s-simpleslides-FuzzyFrame"] = {["context-simpleslides"]=true,},
  ["cabin"] = {["cabin"]=true,},
  ["grant-nsf"] = {["grant"]=true,},
  ["babel-portuguese"] = {["babel"]=true,},
  ["hyph-hu"] = {["hyphen-hungarian"]=true,},
  ["tcblistingsutf8.code"] = {["tcolorbox"]=true,},
  ["bibl-ssa"] = {["context"]=true,},
  ["t-degrade"] = {["context-degrade"]=true,},
  ["footnpag"] = {["footnpag"]=true,},
  ["classics"] = {["classics"]=true,},
  ["grphyph5"] = {["hyphen-greek"]=true,},
  ["cmpj"] = {["cmpj"]=true,},
  ["grmhyph5"] = {["hyphen-greek"]=true,},
  ["xeCJK-listings"] = {["xecjk"]=true,},
  ["tikzlibrarydecorations.pathreplacing.code"] = {["pgf"]=true,},
  ["duckuments"] = {["duckuments"]=true,},
  ["ofs-6c"] = {["ofs"]=true,},
  ["awrite"] = {["arabtex"]=true,},
  ["hyph-quote-af"] = {},
  ["hyph-gl.ec"] = {},
  ["hyph-fur"] = {},
  ["hyph-quote-fur"] = {},
  ["hyph-fur.ec"] = {},
  ["jojomayer"] = {["context"]=true,},
  ["pgfsysprotocol.code"] = {["pgf"]=true,},
  ["kp-fonts"] = {},
  ["gmtypos"] = {["gmutils"]=true,},
  ["german_msg-msg"] = {["msg"]=true,},
  ["babel-croatian"] = {["babel"]=true,},
  ["USenglish"] = {["babel"]=true,},
  ["austrian"] = {["babel"]=true,},
  ["loadhyph-fi"] = {},
  ["cooltooltips"] = {["cooltooltips"]=true,},
  ["smartdiagram"] = {["smartdiagram"]=true,},
  ["scanbase"] = {["olsak-misc"]=true,},
  ["mex1"] = {["mex"]=true,},
  ["hebfont"] = {["babel-hebrew"]=true,},
  ["babel-tachelhit"] = {["babel"]=true,},
  ["lwarp-overpic"] = {["lwarp"]=true,},
  ["loadhyph-hr"] = {},
  ["doc"] = {["latex"]=true,},
  ["clefval"] = {["clefval"]=true,},
  ["CountriesOfEurope"] = {["countriesofeurope"]=true,},
  ["fntproof"] = {["fntproof"]=true,},
  ["hyph-is"] = {},
  ["limecv"] = {["limecv"]=true,},
  ["scsnowman"] = {["scsnowman"]=true,},
  ["refcount"] = {["oberdiek"]=true,},
  ["multilang"] = {["multilang"]=true,},
  ["babel-sichuanyi"] = {["babel"]=true,},
  ["arabtoks"] = {["arabtex"]=true,},
  ["copthyph"] = {},
  ["ofs-8z"] = {["ofs"]=true,},
  ["yagusylo"] = {["yagusylo"]=true,},
  ["lwarp-tabularx"] = {["lwarp"]=true,},
  ["loadhyph-be"] = {},
  ["epspdfconversion"] = {["epspdfconversion"]=true,},
  ["babel-asu"] = {["babel"]=true,},
  ["cyrtex"] = {},
  ["pgfopts"] = {["pgfopts"]=true,},
  ["grahyph5"] = {},
  ["musixlyr"] = {["musixtex"]=true,},
  ["Zallman"] = {["cfr-initials"]=true,},
  ["beamerbaseboxes"] = {["beamer"]=true,},
  ["ghsystem_skull-6"] = {["ghsystem"]=true,},
  ["fnbreak"] = {["fnbreak"]=true,},
  ["feupphdteses"] = {["feupphdteses"]=true,},
  ["hyperxmp"] = {["hyperxmp"]=true,},
  ["biolinum-otf"] = {["libertine"]=true,},
  ["hxt-bc"] = {["hyper"]=true,},
  ["hvindex"] = {["hvindex"]=true,},
  ["treport"] = {["platex"]=true,},
  ["Eileen"] = {["cfr-initials"]=true,},
  ["agupp"] = {["aguplus"]=true,},
  ["beamercolorthemehohenheim"] = {["hobete"]=true,},
  ["babel-azerbaijani-latin"] = {["babel"]=true,},
  ["bagpipe"] = {["bagpipe"]=true,},
  ["lni"] = {["lni"]=true,},
  ["jnuexam"] = {["jnuexam"]=true,},
  ["eledmac"] = {["eledmac"]=true,},
  ["esint"] = {["esint-type1"]=true,["esint"]=true,},
  ["MULEenc"] = {["cjk"]=true,},
  ["mycv"] = {["mycv"]=true,},
  ["sverb"] = {["mdwtools"]=true,},
  ["braille"] = {["braille"]=true,},
  ["cassette"] = {["js-misc"]=true,},
  ["heppennames"] = {["hepnames"]=true,},
  ["midpage"] = {["midpage"]=true,},
  ["librecaslon"] = {["librecaslon"]=true,},
  ["m-ch-de"] = {["context"]=true,},
  ["makesc8y"] = {["afm2pl"]=true,},
  ["chordsCZ"] = {["pmxchords"]=true,},
  ["textalpha"] = {["greek-fontenc"]=true,},
  ["loadhyph-hi"] = {},
  ["eskdlang"] = {["eskdx"]=true,},
  ["sclang-prettifier"] = {["sclang-prettifier"]=true,},
  ["harvmac"] = {["harvmac"]=true,},
  ["fixlatvian"] = {["fixlatvian"]=true,},
  ["txi-fr"] = {},
  ["romande"] = {["romande"]=true,},
  ["bidimoderncv"] = {["bidi"]=true,},
  ["hang"] = {["hang"]=true,},
  ["beamerbasesection"] = {["beamer"]=true,},
  ["g-brief2"] = {["g-brief"]=true,},
  ["beamercolorthemefly"] = {["beamer"]=true,},
  ["beng"] = {["bengali"]=true,},
  ["ctex"] = {["ctex"]=true,},
  ["gzt"] = {["gzt"]=true,},
  ["roundbox"] = {["roundbox"]=true,["gustlib"]=true,},
  ["mdici"] = {["mathdesign"]=true,},
  ["guit"] = {["guitlogo"]=true,},
  ["interfaces-umrand"] = {["interfaces"]=true,},
  ["gtrcrd"] = {["gtrcrd"]=true,},
  ["ftcap"] = {["ftcap"]=true,},
  ["magicnum"] = {["oberdiek"]=true,},
  ["grundgesetze"] = {["grundgesetze"]=true,},
  ["gregoriotex-syllable"] = {["gregoriotex"]=true,},
  ["gregoriotex-main"] = {["gregoriotex"]=true,},
  ["showhyphens"] = {["showhyphens"]=true,},
  ["etharab"] = {["ethiop"]=true,},
  ["egothic"] = {["bookhands"]=true,},
  ["ruby"] = {["cjk"]=true,},
  ["fxlayoutpdfcnote"] = {["fixme"]=true,},
  ["minijs"] = {["jsclasses"]=true,},
  ["encxvlna"] = {["encxvlna"]=true,},
  ["multimedia"] = {["beamer"]=true,},
  ["brandeis-dissertation"] = {["brandeis-dissertation"]=true,},
  ["lwarp-bookmark"] = {["lwarp"]=true,},
  ["font_libertine_times"] = {["font-change"]=true,},
  ["graph35-keys"] = {["graph35"]=true,},
  ["klufloa"] = {["kluwer"]=true,},
  ["beamercolorthememonarca"] = {["beamer"]=true,},
  ["vaucanson-g"] = {["vaucanson-g"]=true,},
  ["loadhyph-gu"] = {},
  ["gotoh"] = {["gotoh"]=true,},
  ["babel-hausa-ne"] = {["babel"]=true,},
  ["block"] = {["block"]=true,},
  ["clock"] = {["clock"]=true,},
  ["tipa"] = {["tipa"]=true,},
  ["gmverse"] = {["gmverse"]=true,},
  ["digiconfigs"] = {["digiconfigs"]=true,},
  ["gmlogos"] = {["gmutils"]=true,},
  ["hyph-cy.ec"] = {},
  ["bangorexam"] = {["bangorexam"]=true,},
  ["gmcommand"] = {["gmutils"]=true,},
  ["gmbase"] = {["gmutils"]=true,},
  ["antyktor"] = {["antt"]=true,},
  ["glossaries-extra-stylemods"] = {["glossaries-extra"]=true,},
  ["getitems"] = {["getitems"]=true,},
  ["arrayjobx"] = {["arrayjobx"]=true,},
  ["example-glossaries-url"] = {["glossaries"]=true,},
  ["hcart"] = {["hc"]=true,},
  ["lwarp-xltabular"] = {["lwarp"]=true,},
  ["tudscrcomp-book"] = {["tudscr"]=true,},
  ["chemmacros.module.errorcheck.code"] = {["chemmacros"]=true,},
  ["romanidx"] = {["babel-spanish"]=true,},
  ["babel-kwasio"] = {["babel"]=true,},
  ["pgfplotsliststructure.code"] = {["pgfplots"]=true,},
  ["glossary-super"] = {["glossaries"]=true,},
  ["s-simpleslides-BlackBoard"] = {["context-simpleslides"]=true,},
  ["glossary-longbooktabs"] = {["glossaries"]=true,},
  ["t-lecturer"] = {["lecturer"]=true,},
  ["zref-perpage"] = {["oberdiek"]=true,},
  ["pst-vehicle"] = {["pst-vehicle"]=true,},
  ["gloss"] = {["gloss"]=true,["midnight"]=true,},
  ["tabularkv"] = {["oberdiek"]=true,},
  ["ascmac"] = {["ascmac"]=true,},
  ["xeCJK"] = {["xecjk"]=true,},
  ["moderncvbodyii"] = {["moderncv"]=true,},
  ["beamersubframe"] = {["beamersubframe"]=true,},
  ["ghsystem_health"] = {["ghsystem"]=true,},
  ["mslapa"] = {["mslapa"]=true,},
  ["lwarp-prettyref"] = {["lwarp"]=true,},
  ["actcodes"] = {["catcodes"]=true,},
  ["ghsystem_explos-1"] = {["ghsystem"]=true,},
  ["font_antp_euler"] = {["font-change"]=true,},
  ["aesop-de"] = {["context"]=true,},
  ["graphicx-psmin"] = {["graphicx-psmin"]=true,},
  ["example-image-b"] = {["mwe"]=true,},
  ["dtk-url"] = {["dtk"]=true,},
  ["eskdindent"] = {["eskdx"]=true,},
  ["beamerbaserequires"] = {["beamer"]=true,},
  ["cc4amsart"] = {["computational-complexity"]=true,},
  ["chscite"] = {["chscite"]=true,},
  ["leadsheets.library.shorthands.code"] = {["leadsheets"]=true,},
  ["hhtensor"] = {["hhtensor"]=true,},
  ["bxjscompat"] = {["bxjscls"]=true,},
  ["bxtexlogo"] = {["bxtexlogo"]=true,},
  ["getmap"] = {["getmap"]=true,},
  ["vncaps"] = {["vntex"]=true,},
  ["outerhbox"] = {},
  ["haparabica"] = {["arabi"]=true,},
  ["dow"] = {},
  ["ctexsize"] = {["ctex"]=true,},
  ["colortbl"] = {["colortbl"]=true,},
  ["aomart"] = {["aomart"]=true,},
  ["beamerfontthemestructuresmallcapsserif"] = {["beamer"]=true,},
  ["draftcopy"] = {["draftcopy"]=true,},
  ["font_kurier-condensed-light"] = {["font-change"]=true,},
  ["nccdefaults"] = {["ncclatex"]=true,},
  ["ConcProg"] = {},
  ["beamerbasetitle"] = {["beamer"]=true,},
  ["gender"] = {["gender"]=true,},
  ["fontdoc"] = {["fontinst"]=true,},
  ["subscript"] = {["fragments"]=true,},
  ["amsgen"] = {["amsmath"]=true,},
  ["t-layout"] = {["context-layout"]=true,},
  ["HAPTCSgrad"] = {["ha-prosper"]=true,},
  ["kp-math"] = {},
  ["suetterl"] = {["fundus-sueterlin"]=true,},
  ["functan"] = {["functan"]=true,},
  ["slemph"] = {["frankenstein"]=true,},
  ["phn-Landeskennzahlen"] = {["phonenumbers"]=true,},
  ["delimset"] = {["delimset"]=true,},
  ["checklab"] = {["fragments"]=true,},
  ["babel-chinese-traditional"] = {["babel"]=true,},
  ["s-pre-12"] = {["context"]=true,},
  ["musixps"] = {["musixtex"]=true,},
  ["csfmacro"] = {["enctex"]=true,},
  ["slides"] = {["latex"]=true,},
  ["defpattern"] = {["fp"]=true,},
  ["ukrhypfa"] = {["ukrhyph"]=true,},
  ["listingsutf8"] = {["oberdiek"]=true,},
  ["arabchrs"] = {["arabtex"]=true,},
  ["install08text"] = {},
  ["evenpage"] = {["bosisio"]=true,},
  ["tikzlibraryshapes.geometric.code"] = {["pgf"]=true,},
  ["babel-chechen"] = {["babel"]=true,},
  ["arwindoc"] = {["arabtex"]=true,},
  ["mtx-context-trim"] = {["context"]=true,},
  ["tkz-doc"] = {["tkz-doc"]=true,},
  ["teubnertx"] = {["teubner"]=true,},
  ["hyph-da"] = {},
  ["fontawesomesymbols-xeluatex"] = {["fontawesome"]=true,},
  ["alnumsec"] = {["alnumsec"]=true,},
  ["stealcaps"] = {["stealcaps"]=true,},
  ["coordsys"] = {["coordsys"]=true,},
  ["tabriz-thesis"] = {["tabriz-thesis"]=true,},
  ["changes"] = {["changes"]=true,},
  ["stmary"] = {["plstmary"]=true,},
  ["font_epigrafica_euler"] = {["font-change"]=true,},
  ["thc"] = {["tools"]=true,},
  ["hyph-tr.ec"] = {},
  ["hyph-mn-cyrl"] = {},
  ["babel-breton"] = {["babel"]=true,},
  ["cinzel"] = {["cinzel"]=true,},
  ["beamercolorthemeSaintPetersburg"] = {["beamertheme-saintpetersburg"]=true,},
  ["blindtext"] = {["blindtext"]=true,},
  ["doclicense-CC-by-nc-sa-4.0-plaintext"] = {["doclicense"]=true,},
  ["sotoku"] = {["arabtex"]=true,},
  ["pst-func"] = {["pst-func"]=true,},
  ["combnat"] = {["combine"]=true,},
  ["mpgmpar"] = {["minipage-marginpar"]=true,},
  ["lwarp-fontenc"] = {["lwarp"]=true,},
  ["cccite"] = {["computational-complexity"]=true,},
  ["fncylab"] = {["fncylab"]=true,},
  ["tuplet"] = {["musixtex"]=true,},
  ["hyph-ml"] = {},
  ["ccqed"] = {["computational-complexity"]=true,},
  ["infernce"] = {["semantic"]=true,},
  ["bxpapersize"] = {["bxpapersize"]=true,},
  ["ctexmakespa"] = {["ctex"]=true,},
  ["cjeupmath"] = {["cje"]=true,},
  ["footnotebackref"] = {["footnotebackref"]=true,},
  ["hyph-ga"] = {},
  ["empheq"] = {["mathtools"]=true,},
  ["pinyin"] = {["cjk"]=true,},
  ["flashmovie"] = {["flashmovie"]=true,},
  ["dutch"] = {["babel"]=true,},
  ["l7xenc"] = {["lithuanian"]=true,},
  ["opmac"] = {},
  ["babel-sanskrit"] = {["babel"]=true,},
  ["boxhandler"] = {["boxhandler"]=true,},
  ["aer"] = {["economic"]=true,},
  ["aligned-overset"] = {["aligned-overset"]=true,},
  ["UKenglish"] = {["babel"]=true,},
  ["ruhyphas"] = {},
  ["fithesis-fi"] = {["fithesis"]=true,},
  ["chemcompounds"] = {["chemcompounds"]=true,},
  ["finstrut"] = {["finstrut"]=true,},
  ["chemgreek"] = {["chemgreek"]=true,},
  ["pgf-filehook"] = {["filehook"]=true,},
  ["tikzlibrarypatterns.meta.code"] = {["pgf"]=true,},
  ["novel-Sandbox"] = {["novel"]=true,},
  ["fdudoc"] = {["fduthesis"]=true,},
  ["beamercolorthemefibeamer-mu-econ"] = {["fibeamer"]=true,},
  ["libertine-otf"] = {["libertine"]=true,},
  ["typicons"] = {["typicons"]=true,},
  ["tikzpeople.shape.jester"] = {["tikzpeople"]=true,},
  ["turkish"] = {["babel"]=true,},
  ["fdulogo"] = {["fduthesis"]=true,},
  ["latex2man"] = {["latex2man"]=true,},
  ["PTSansNarrow"] = {["paratype"]=true,},
  ["leftidx"] = {["leftidx"]=true,},
  ["babel-serbian-cyrillic-bosniaherzegovina"] = {["babel"]=true,},
  ["fr-subfig"] = {["floatrow"]=true,},
  ["fancyslides"] = {["fancyslides"]=true,},
  ["aligs"] = {["arabtex"]=true,},
  ["ccthm"] = {["computational-complexity"]=true,},
  ["pressrelease-symbols"] = {["pressrelease"]=true,},
  ["mathcmd"] = {["bosisio"]=true,},
  ["mwrep"] = {["mwcls"]=true,},
  ["gtrcore.processing.code"] = {["genealogytree"]=true,},
  ["lwarp-bytefield"] = {["lwarp"]=true,},
  ["longbox"] = {["longfbox"]=true,},
  ["xebaposter"] = {["xebaposter"]=true,},
  ["pgfplotstable.coltype.code"] = {["pgfplots"]=true,},
  ["ltxtools-environ"] = {["ltxtools"]=true,},
  ["docmfp"] = {["docmfp"]=true,},
  ["etsvthor"] = {["etsvthor"]=true,},
  ["CJKutf8"] = {["cjk"]=true,},
  ["clrscode"] = {["clrscode"]=true,},
  ["erw-l3"] = {["erw-l3"]=true,},
  ["longfbox"] = {["longfbox"]=true,},
  ["fxlayoutpdfcsignote"] = {["fixme"]=true,},
  ["cstug08"] = {},
  ["beamerthemeLuebeck"] = {["beamer"]=true,},
  ["babel-serbian-latn-ba"] = {["babel"]=true,},
  ["babel-portuguese-br"] = {["babel"]=true,},
  ["beamercolorthemebluesimplex"] = {["beamertheme-cuerna"]=true,},
  ["tikzlibrarycircuits.plc.sfc.code"] = {["tikz-sfc"]=true,},
  ["envlab"] = {["envlab"]=true,},
  ["powerdot-pazik"] = {["powerdot"]=true,},
  ["textarea"] = {["ncctools"]=true,},
  ["epigram"] = {},
  ["akkgermanabbreviations"] = {["akktex"]=true,},
  ["binhex"] = {["kastrup"]=true,},
  ["coollist"] = {["coollist"]=true,},
  ["drcaps"] = {["shipunov"]=true,},
  ["media9"] = {["media9"]=true,},
  ["cmdstring"] = {["cmdstring"]=true,},
  ["utf8math"] = {["enctex"]=true,},
  ["easybmat"] = {["easy"]=true,},
  ["akkscript"] = {["akktex"]=true,},
  ["plain-utf8-ec"] = {["enctex"]=true,},
  ["basque-book"] = {["basque-book"]=true,},
  ["texnames"] = {},
  ["ltcaption"] = {["caption"]=true,},
  ["kam-t1"] = {["enctex"]=true,},
  ["bibl-aps"] = {["context"]=true,},
  ["carolmin"] = {["bookhands"]=true,},
  ["lwarp-soulpos"] = {["lwarp"]=true,},
  ["musixcpt"] = {["musixtex"]=true,},
  ["dhua"] = {["dhua"]=true,},
  ["babel-slovene"] = {["babel"]=true,},
  ["easyReview"] = {["easyreview"]=true,},
  ["ltxtools"] = {["ltxtools"]=true,},
  ["CJKfntef"] = {["cjk"]=true,},
  ["hyph-fi.ec"] = {},
  ["pdfcolmk"] = {["oberdiek"]=true,},
  ["beilstein"] = {["beilstein"]=true,},
  ["cmyk-hax"] = {["tex-ps"]=true,},
  ["bmpsize"] = {["oberdiek"]=true,},
  ["monofill"] = {["monofill"]=true,},
  ["mentis"] = {["mentis"]=true,},
  ["pdftex-dvi"] = {["pdftex"]=true,},
  ["bibl-num-fr"] = {["context"]=true,},
  ["eptexfn"] = {["expex"]=true,},
  ["pgflibraryshapes.code"] = {["pgf"]=true,},
  ["bigints"] = {["bigints"]=true,},
  ["accenti"] = {["bosisio"]=true,},
  ["engrec"] = {["engrec"]=true,},
  ["dithesis"] = {["dithesis"]=true,},
  ["doclicense-CC-by-3.0-plaintext"] = {["doclicense"]=true,},
  ["bibleref-parse"] = {["bibleref-parse"]=true,},
  ["tkz-obj-circles"] = {["tkz-euclide"]=true,},
  ["american"] = {["babel"]=true,},
  ["droid"] = {["droid"]=true,},
  ["afthes11"] = {["afthesis"]=true,},
  ["bredzenie"] = {["bredzenie"]=true,},
  ["tcbraster.code"] = {["tcolorbox"]=true,},
  ["interval"] = {["interval"]=true,},
  ["pdftexconfig"] = {["tex-ini-files"]=true,},
  ["etruscan"] = {["archaic"]=true,},
  ["pdficwebmac"] = {["cweb"]=true,},
  ["example-glossaries-acronym"] = {["glossaries"]=true,},
  ["beamercolorthememetropolis"] = {["beamertheme-metropolis"]=true,},
  ["Sanremo"] = {["cfr-initials"]=true,},
  ["ticollege"] = {["ticollege"]=true,},
  ["m-pictex"] = {["context"]=true,},
  ["algorithm2e"] = {["algorithm2e"]=true,},
  ["epltxfn"] = {["expex"]=true,},
  ["cd-cover"] = {["cd-cover"]=true,},
  ["hitec"] = {["hitec"]=true,},
  ["mtx-context-timing"] = {["context"]=true,},
  ["babel-english-gb"] = {["babel"]=true,},
  ["GotIn"] = {["cfr-initials"]=true,},
  ["withargs-dry"] = {["withargs"]=true,},
  ["font_antt"] = {["font-change"]=true,},
  ["a4wide"] = {["a4wide"]=true,},
  ["scrwfile"] = {},
  ["akktex"] = {["akktex"]=true,},
  ["slashed"] = {["carlisle"]=true,},
  ["hgbmath"] = {["hagenberg-thesis"]=true,},
  ["hyph-te"] = {},
  ["babel-shambala"] = {["babel"]=true,},
  ["lwarp-fncychap"] = {["lwarp"]=true,},
  ["bicaption"] = {["caption"]=true,},
  ["plain-kam-cs"] = {["enctex"]=true,},
}
local tlp_doclist = {
  ["formular"] = {
    "texmf-dist/doc/latex/formular/README",
    "texmf-dist/doc/latex/formular/formular.pdf",
  },
  ["image-gallery"] = {
    "texmf-dist/doc/latex/image-gallery/README",
    "texmf-dist/doc/latex/image-gallery/gallery-example.pdf",
    "texmf-dist/doc/latex/image-gallery/mypics.txt",
  },
  ["yathesis"] = {
    "texmf-dist/doc/latex/yathesis/README.md",
    "texmf-dist/doc/latex/yathesis/french/documentation/latexmkrc",
    "texmf-dist/doc/latex/yathesis/french/documentation/yathesis-fr.pdf",
    "texmf-dist/doc/latex/yathesis/french/exemples/specimen/a-plat/comue.pdf",
    "texmf-dist/doc/latex/yathesis/french/exemples/specimen/a-plat/labo.pdf",
    "texmf-dist/doc/latex/yathesis/french/exemples/specimen/a-plat/latexmkrc",
    "texmf-dist/doc/latex/yathesis/french/exemples/specimen/a-plat/paris13.pdf",
    "texmf-dist/doc/latex/yathesis/french/exemples/specimen/a-plat/these.pdf",
    "texmf-dist/doc/latex/yathesis/french/exemples/specimen/a-plat/tiger.pdf",
    "texmf-dist/doc/latex/yathesis/french/exemples/specimen/a-plat/ulco.pdf",
  },
  ["ecc"] = {
    "texmf-dist/doc/fonts/ecc/copyrite",
    "texmf-dist/doc/fonts/ecc/liesmich",
    "texmf-dist/doc/fonts/ecc/readme",
  },
  ["octave"] = {
    "texmf-dist/doc/latex/octave/README.md",
    "texmf-dist/doc/latex/octave/octave.pdf",
  },
  ["greek-fontenc"] = {
    "texmf-dist/doc/latex/greek-fontenc/README",
    "texmf-dist/doc/latex/greek-fontenc/README.html",
    "texmf-dist/doc/latex/greek-fontenc/alphabeta-doc.pdf",
    "texmf-dist/doc/latex/greek-fontenc/alphabeta-euenc.def.html",
    "texmf-dist/doc/latex/greek-fontenc/alphabeta-lgr.def.html",
    "texmf-dist/doc/latex/greek-fontenc/alphabeta.sty.html",
    "texmf-dist/doc/latex/greek-fontenc/diacritics.pdf",
    "texmf-dist/doc/latex/greek-fontenc/greek-euenc-doc.pdf",
    "texmf-dist/doc/latex/greek-fontenc/greek-euenc.def.html",
    "texmf-dist/doc/latex/greek-fontenc/greek-fontenc.def.html",
    "texmf-dist/doc/latex/greek-fontenc/greekhyperref.pdf",
    "texmf-dist/doc/latex/greek-fontenc/lgr2licr.lua.html",
    "texmf-dist/doc/latex/greek-fontenc/lgrenc-test.pdf",
    "texmf-dist/doc/latex/greek-fontenc/lgrenc.def.html",
    "texmf-dist/doc/latex/greek-fontenc/test-active-semicolon.pdf",
    "texmf-dist/doc/latex/greek-fontenc/test-nameclashes.pdf",
    "texmf-dist/doc/latex/greek-fontenc/textalpha-doc.pdf",
    "texmf-dist/doc/latex/greek-fontenc/textalpha.sty.html",
  },
  ["silence"] = {
    "texmf-dist/doc/latex/silence/README",
    "texmf-dist/doc/latex/silence/silence-doc.pdf",
  },
  ["mpcolornames"] = {
    "texmf-dist/doc/metapost/mpcolornames/CHANGES",
    "texmf-dist/doc/metapost/mpcolornames/LICENSE",
    "texmf-dist/doc/metapost/mpcolornames/README",
    "texmf-dist/doc/metapost/mpcolornames/mpcolornames.pdf",
  },
  ["tableof"] = {
    "texmf-dist/doc/latex/tableof/README",
    "texmf-dist/doc/latex/tableof/README.md",
    "texmf-dist/doc/latex/tableof/tableof.pdf",
  },
  ["abc"] = {
    "texmf-dist/doc/latex/abc/README",
    "texmf-dist/doc/latex/abc/abc.pdf",
  },
  ["a0poster"] = {
    "texmf-dist/doc/latex/a0poster/a0.pdf",
    "texmf-dist/doc/latex/a0poster/a0_eng.pdf",
  },
  ["qobitree"] = {
    "texmf-dist/doc/latex/qobitree/README",
  },
  ["minifp"] = {
    "texmf-dist/doc/generic/minifp/README",
    "texmf-dist/doc/generic/minifp/minifp.pdf",
  },
  ["faq-en"] = {
    "texmf-dist/doc/generic/FAQ-en/ChangeLog",
    "texmf-dist/doc/generic/FAQ-en/Makefile",
    "texmf-dist/doc/generic/FAQ-en/Makefile-CTAN",
    "texmf-dist/doc/generic/FAQ-en/README",
    "texmf-dist/doc/generic/FAQ-en/install-CTAN",
    "texmf-dist/doc/generic/FAQ-en/letterfaq.pdf",
    "texmf-dist/doc/generic/FAQ-en/markup-syntax",
    "texmf-dist/doc/generic/FAQ-en/newfaq.pdf",
  },
  ["phonrule"] = {
    "texmf-dist/doc/latex/phonrule/README.md",
    "texmf-dist/doc/latex/phonrule/phonrule-doc.pdf",
  },
  ["missaali"] = {
    "texmf-dist/doc/fonts/missaali/MANIFEST-Missaali.txt",
    "texmf-dist/doc/fonts/missaali/OFL.txt",
    "texmf-dist/doc/fonts/missaali/README",
    "texmf-dist/doc/fonts/missaali/lppl.txt",
    "texmf-dist/doc/fonts/missaali/missaali.pdf",
  },
  ["tex-ps"] = {
    "texmf-dist/doc/generic/tex-ps/cmyk-hax/README",
    "texmf-dist/doc/generic/tex-ps/cmyk-hax/cmyk-doc.pdf",
    "texmf-dist/doc/generic/tex-ps/poligraf/README",
  },
  ["datetime2-basque"] = {
    "texmf-dist/doc/latex/datetime2-basque/CHANGES",
    "texmf-dist/doc/latex/datetime2-basque/Makefile",
    "texmf-dist/doc/latex/datetime2-basque/README.txt",
  },
  ["tikz-kalender"] = {
    "texmf-dist/doc/latex/tikz-kalender/README.md",
    "texmf-dist/doc/latex/tikz-kalender/tikz-kalender-example1.pdf",
    "texmf-dist/doc/latex/tikz-kalender/tikz-kalender-example2.pdf",
  },
  ["economic"] = {
    "texmf-dist/doc/bibtex/economic/CHANGELOG",
    "texmf-dist/doc/bibtex/economic/NEWS",
    "texmf-dist/doc/bibtex/economic/README",
    "texmf-dist/doc/bibtex/economic/ajae-ex.pdf",
    "texmf-dist/doc/bibtex/economic/apecon-ex.pdf",
    "texmf-dist/doc/bibtex/economic/ecca-ex.pdf",
    "texmf-dist/doc/bibtex/economic/erae-ex.pdf",
    "texmf-dist/doc/bibtex/economic/ier-bib-test.pdf",
    "texmf-dist/doc/bibtex/economic/itaxpf-ex-title.pdf",
    "texmf-dist/doc/bibtex/economic/itaxpf-ex.pdf",
    "texmf-dist/doc/bibtex/economic/oegatb-ex.pdf",
  },
  ["bitelist"] = {
    "texmf-dist/doc/generic/bitelist/README",
    "texmf-dist/doc/generic/bitelist/bitelist.pdf",
  },
  ["blkarray"] = {
    "texmf-dist/doc/latex/blkarray/README",
    "texmf-dist/doc/latex/blkarray/blkarray.pdf",
  },
  ["begriff"] = {
    "texmf-dist/doc/latex/begriff/COPYING",
    "texmf-dist/doc/latex/begriff/README",
    "texmf-dist/doc/latex/begriff/examples.pdf",
  },
  ["sugconf"] = {
    "texmf-dist/doc/latex/sugconf/README",
    "texmf-dist/doc/latex/sugconf/SUGI-paper-example.txt",
    "texmf-dist/doc/latex/sugconf/article-example.pdf",
    "texmf-dist/doc/latex/sugconf/sugconf-example.pdf",
  },
  ["mparhack"] = {
    "texmf-dist/doc/latex/mparhack/README",
    "texmf-dist/doc/latex/mparhack/mparhack.pdf",
  },
  ["fancybox"] = {
    "texmf-dist/doc/latex/fancybox/Changes",
    "texmf-dist/doc/latex/fancybox/Makefile",
    "texmf-dist/doc/latex/fancybox/README",
    "texmf-dist/doc/latex/fancybox/fancybox-doc.pdf",
  },
  ["fonts-churchslavonic"] = {
    "texmf-dist/doc/fonts/fonts-churchslavonic/GPL.txt",
    "texmf-dist/doc/fonts/fonts-churchslavonic/LICENSE",
    "texmf-dist/doc/fonts/fonts-churchslavonic/OFL.txt",
    "texmf-dist/doc/fonts/fonts-churchslavonic/README",
    "texmf-dist/doc/fonts/fonts-churchslavonic/fonts-churchslavonic.pdf",
  },
  ["getfiledate"] = {
    "texmf-dist/doc/latex/getfiledate/README",
    "texmf-dist/doc/latex/getfiledate/getfiledate-guide.pdf",
  },
  ["pst-ode"] = {
    "texmf-dist/doc/generic/pst-ode/ChangeLog",
    "texmf-dist/doc/generic/pst-ode/README.md",
    "texmf-dist/doc/generic/pst-ode/pst-ode-doc.pdf",
  },
  ["exercise"] = {
    "texmf-dist/doc/latex/exercise/README",
    "texmf-dist/doc/latex/exercise/exercise.pdf",
  },
  ["epigraph"] = {
    "texmf-dist/doc/latex/epigraph/README",
    "texmf-dist/doc/latex/epigraph/epigraph.pdf",
  },
  ["plain-doc"] = {
    "texmf-dist/doc/plain/plain-doc/csname.txt",
  },
  ["datetime2-portuges"] = {
    "texmf-dist/doc/latex/datetime2-portuges/README",
    "texmf-dist/doc/latex/datetime2-portuges/datetime2-portuges.pdf",
  },
  ["resumemac"] = {
    "texmf-dist/doc/plain/resumemac/README",
  },
  ["lobster2"] = {
    "texmf-dist/doc/fonts/lobster2/OFL.txt",
    "texmf-dist/doc/fonts/lobster2/README",
    "texmf-dist/doc/fonts/lobster2/lobster2-samples.pdf",
  },
  ["curve"] = {
    "texmf-dist/doc/latex/curve/NEWS",
    "texmf-dist/doc/latex/curve/README",
    "texmf-dist/doc/latex/curve/THANKS",
    "texmf-dist/doc/latex/curve/curve.pdf",
    "texmf-dist/doc/latex/curve/examples/Makefile",
  },
  ["pressrelease"] = {
    "texmf-dist/doc/latex/pressrelease/CHANGES",
    "texmf-dist/doc/latex/pressrelease/README",
    "texmf-dist/doc/latex/pressrelease/pressrelease.pdf",
    "texmf-dist/doc/latex/pressrelease/samples/sample-pressrelease.pdf",
    "texmf-dist/doc/latex/pressrelease/samples/sample-pressrelease2.pdf",
    "texmf-dist/doc/latex/pressrelease/samples/sample-pressrelease3.pdf",
    "texmf-dist/doc/latex/pressrelease/samples/sample-pressrelease4.pdf",
    "texmf-dist/doc/latex/pressrelease/samples/sample-pressrelease5.pdf",
    "texmf-dist/doc/latex/pressrelease/samples/sample-pressrelease6.pdf",
    "texmf-dist/doc/latex/pressrelease/samples/sample-pressrelease7.pdf",
  },
  ["revquantum"] = {
    "texmf-dist/doc/latex/revquantum/README.md",
    "texmf-dist/doc/latex/revquantum/example.pdf",
    "texmf-dist/doc/latex/revquantum/revquantum.pdf",
  },
  ["uplatex"] = {
    "texmf-dist/doc/man/man1/uplatex.man1.pdf",
    "texmf-dist/doc/uplatex/base/LICENSE",
    "texmf-dist/doc/uplatex/base/README.md",
    "texmf-dist/doc/uplatex/base/README_uplatex.txt",
    "texmf-dist/doc/uplatex/base/uplatex-en.pdf",
    "texmf-dist/doc/uplatex/base/uplatex.pdf",
    "texmf-dist/doc/uplatex/base/upldoc.pdf",
  },
  ["combine"] = {
    "texmf-dist/doc/latex/combine/README",
    "texmf-dist/doc/latex/combine/combine.pdf",
  },
  ["babel-basque"] = {
    "texmf-dist/doc/generic/babel-basque/basque.pdf",
  },
  ["advdate"] = {
    "texmf-dist/doc/latex/advdate/advdate.pdf",
    "texmf-dist/doc/latex/advdate/manifest.txt",
  },
  ["beamer2thesis"] = {
    "texmf-dist/doc/latex/beamer2thesis/README",
    "texmf-dist/doc/latex/beamer2thesis/beamer2thesis.pdf",
    "texmf-dist/doc/latex/beamer2thesis/beamer2thesis_ita.pdf",
    "texmf-dist/doc/latex/beamer2thesis/license",
  },
  ["gfsporson"] = {
    "texmf-dist/doc/fonts/gfsporson/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gfsporson/OFL.txt",
    "texmf-dist/doc/fonts/gfsporson/README",
    "texmf-dist/doc/fonts/gfsporson/gfsporson.pdf",
  },
  ["threadcol"] = {
    "texmf-dist/doc/latex/threadcol/README",
    "texmf-dist/doc/latex/threadcol/threadcol.pdf",
  },
  ["breqn"] = {
    "texmf-dist/doc/latex/breqn/README.md",
    "texmf-dist/doc/latex/breqn/breqn.pdf",
    "texmf-dist/doc/latex/breqn/flexisym.pdf",
    "texmf-dist/doc/latex/breqn/mathstyle.pdf",
  },
  ["lualibs"] = {
    "texmf-dist/doc/luatex/lualibs/LICENSE",
    "texmf-dist/doc/luatex/lualibs/NEWS",
    "texmf-dist/doc/luatex/lualibs/README",
    "texmf-dist/doc/luatex/lualibs/lualibs.pdf",
  },
  ["mhequ"] = {
    "texmf-dist/doc/latex/mhequ/README",
    "texmf-dist/doc/latex/mhequ/example.pdf",
  },
  ["ctib"] = {
    "texmf-dist/doc/latex/ctib/README",
    "texmf-dist/doc/latex/ctib/ctib4tex.pdf",
  },
  ["armtex"] = {
    "texmf-dist/doc/generic/armenian/manual-e.pdf",
    "texmf-dist/doc/generic/armenian/manual.pdf",
    "texmf-dist/doc/generic/armenian/readme.txt",
  },
  ["luapackageloader"] = {
    "texmf-dist/doc/luatex/luapackageloader/README.md",
    "texmf-dist/doc/luatex/luapackageloader/luapackageloader.pdf",
  },
  ["datetime2-samin"] = {
    "texmf-dist/doc/latex/datetime2-samin/README",
    "texmf-dist/doc/latex/datetime2-samin/datetime2-samin.pdf",
  },
  ["iopart-num"] = {
    "texmf-dist/doc/bibtex/iopart-num/README",
    "texmf-dist/doc/bibtex/iopart-num/iopart-num.pdf",
  },
  ["bankstatement"] = {
    "texmf-dist/doc/latex/bankstatement/README.md",
    "texmf-dist/doc/latex/bankstatement/bankstatement-example.pdf",
    "texmf-dist/doc/latex/bankstatement/bankstatement.pdf",
    "texmf-dist/doc/latex/bankstatement/makefile",
  },
  ["borceux"] = {
    "texmf-dist/doc/generic/borceux/Diagram_Mode_d_Emploi",
    "texmf-dist/doc/generic/borceux/Diagram_Read_Me",
    "texmf-dist/doc/generic/borceux/README",
    "texmf-dist/doc/generic/borceux/compatibility/OldDiagram",
    "texmf-dist/doc/generic/borceux/compatibility/OldMaxiDiagram",
    "texmf-dist/doc/generic/borceux/compatibility/OldMicroDiagram",
    "texmf-dist/doc/generic/borceux/compatibility/OldMiniDiagram",
    "texmf-dist/doc/generic/borceux/compatibility/OldMultipleArrows",
  },
  ["ebong"] = {
    "texmf-dist/doc/latex/ebong/ANNOUNCE.txt",
    "texmf-dist/doc/latex/ebong/README",
    "texmf-dist/doc/latex/ebong/eb.pdf",
  },
  ["tex-refs"] = {
    "texmf-dist/doc/generic/tex-refs/README",
    "texmf-dist/doc/generic/tex-refs/tex-refs.html",
    "texmf-dist/doc/generic/tex-refs/tex-refs.txt",
  },
  ["pst-3dplot"] = {
    "texmf-dist/doc/generic/pst-3dplot/Changes",
    "texmf-dist/doc/generic/pst-3dplot/README",
    "texmf-dist/doc/generic/pst-3dplot/pst-3dplot-doc.pdf",
  },
  ["texpower"] = {
    "texmf-dist/doc/latex/texpower/00readme.txt",
    "texmf-dist/doc/latex/texpower/01install.txt",
    "texmf-dist/doc/latex/texpower/02changes.txt",
    "texmf-dist/doc/latex/texpower/contrib/00readme.txt",
    "texmf-dist/doc/latex/texpower/manual.pdf",
    "texmf-dist/doc/latex/texpower/tpslifonts/00readme.txt",
    "texmf-dist/doc/latex/texpower/tpslifonts/01install.txt",
    "texmf-dist/doc/latex/texpower/tpslifonts/Makefile",
  },
  ["ebgaramond"] = {
    "texmf-dist/doc/fonts/ebgaramond/COPYING",
    "texmf-dist/doc/fonts/ebgaramond/Changes",
    "texmf-dist/doc/fonts/ebgaramond/EBGaramond-Initials-Glyphs.pdf",
    "texmf-dist/doc/fonts/ebgaramond/EBGaramond12-Italic-Glyphs.pdf",
    "texmf-dist/doc/fonts/ebgaramond/EBGaramond12-Regular-Glyphs.pdf",
    "texmf-dist/doc/fonts/ebgaramond/README",
    "texmf-dist/doc/fonts/ebgaramond/README.xelualatex",
    "texmf-dist/doc/fonts/ebgaramond/Specimen.pdf",
    "texmf-dist/doc/fonts/ebgaramond/samples.pdf",
  },
  ["isodate"] = {
    "texmf-dist/doc/latex/isodate/ChangeLog",
    "texmf-dist/doc/latex/isodate/README",
    "texmf-dist/doc/latex/isodate/isodate.pdf",
    "texmf-dist/doc/latex/isodate/isodateo.pdf",
    "texmf-dist/doc/latex/isodate/testdate.pdf",
  },
  ["mpgraphics"] = {
    "texmf-dist/doc/latex/mpgraphics/README",
    "texmf-dist/doc/latex/mpgraphics/mpgraphics.pdf",
  },
  ["casyl"] = {
    "texmf-dist/doc/latex/casyl/README",
    "texmf-dist/doc/latex/casyl/casyldoc.pdf",
  },
  ["cqubeamer"] = {
    "texmf-dist/doc/xelatex/cqubeamer/LICENSE",
    "texmf-dist/doc/xelatex/cqubeamer/README.md",
    "texmf-dist/doc/xelatex/cqubeamer/cqubeamer.pdf",
    "texmf-dist/doc/xelatex/cqubeamer/logo/cqulogo.pdf",
    "texmf-dist/doc/xelatex/cqubeamer/logo/cquname.pdf",
  },
  ["booktabs-de"] = {
    "texmf-dist/doc/latex/booktabs-de/README-DE",
    "texmf-dist/doc/latex/booktabs-de/booktabs-de.pdf",
  },
  ["fontinst"] = {
    "texmf-dist/doc/fonts/fontinst/README",
    "texmf-dist/doc/fonts/fontinst/examples/eurofont/Makefile",
    "texmf-dist/doc/fonts/fontinst/examples/mathptm/Makefile",
    "texmf-dist/doc/fonts/fontinst/examples/mathptmx/Makefile",
    "texmf-dist/doc/fonts/fontinst/manual/fontinst.pdf",
    "texmf-dist/doc/fonts/fontinst/manual/intro98.pdf",
    "texmf-dist/doc/fonts/fontinst/talks/et99-font-tables.pdf",
    "texmf-dist/doc/fonts/fontinst/talks/et99-font-tutorial.pdf",
    "texmf-dist/doc/man/man1/fontinst.man1.pdf",
  },
  ["tikz-sfc"] = {
    "texmf-dist/doc/latex/tikz-sfc/README.txt",
    "texmf-dist/doc/latex/tikz-sfc/tikz-sfc-doc.pdf",
  },
  ["pst-asr"] = {
    "texmf-dist/doc/generic/pst-asr/README",
    "texmf-dist/doc/generic/pst-asr/pst-asr-doc.pdf",
    "texmf-dist/doc/generic/pst-asr/pst-asr-examples.pdf",
  },
  ["ecobiblatex"] = {
    "texmf-dist/doc/latex/ecobiblatex/README.txt",
    "texmf-dist/doc/latex/ecobiblatex/ecobiblatex.pdf",
  },
  ["cmll"] = {
    "texmf-dist/doc/fonts/cmll/README",
    "texmf-dist/doc/fonts/cmll/cmll.pdf",
  },
  ["sistyle"] = {
    "texmf-dist/doc/latex/SIstyle/README",
    "texmf-dist/doc/latex/SIstyle/SIstyle-2.3a.pdf",
  },
  ["adigraph"] = {
    "texmf-dist/doc/latex/adigraph/LICENSE",
    "texmf-dist/doc/latex/adigraph/README.md",
    "texmf-dist/doc/latex/adigraph/adigraph_documentation.pdf",
    "texmf-dist/doc/latex/adigraph/adigraph_working_test.pdf",
  },
  ["ftnxtra"] = {
    "texmf-dist/doc/latex/ftnxtra/README",
    "texmf-dist/doc/latex/ftnxtra/ftnxtra.pdf",
  },
  ["authoraftertitle"] = {
    "texmf-dist/doc/latex/authoraftertitle/authoraftertitle.pdf",
  },
  ["tamefloats"] = {
    "texmf-dist/doc/latex/tamefloats/README.txt",
  },
  ["cmtiup"] = {
    "texmf-dist/doc/latex/cmtiup/README",
    "texmf-dist/doc/latex/cmtiup/cmtiup.pdf",
  },
  ["biblatex-caspervector"] = {
    "texmf-dist/doc/latex/biblatex-caspervector/ChangeLog.txt",
    "texmf-dist/doc/latex/biblatex-caspervector/Makefile",
    "texmf-dist/doc/latex/biblatex-caspervector/caspervector-ay.pdf",
    "texmf-dist/doc/latex/biblatex-caspervector/caspervector.pdf",
  },
  ["babel-bulgarian"] = {
    "texmf-dist/doc/generic/babel-bulgarian/README",
    "texmf-dist/doc/generic/babel-bulgarian/bulgarian.pdf",
  },
  ["simplecv"] = {
    "texmf-dist/doc/latex/simplecv/README",
    "texmf-dist/doc/latex/simplecv/simplecv.pdf",
    "texmf-dist/doc/latex/simplecv/testcv.pdf",
  },
  ["chletter"] = {
    "texmf-dist/doc/latex/chletter/README",
    "texmf-dist/doc/latex/chletter/chletter.pdf",
  },
  ["cjk"] = {
    "texmf-dist/doc/latex/cjk/ChangeLog",
    "texmf-dist/doc/latex/cjk/MANIFEST",
    "texmf-dist/doc/latex/cjk/Makefile",
    "texmf-dist/doc/latex/cjk/README",
    "texmf-dist/doc/latex/cjk/TODO",
    "texmf-dist/doc/latex/cjk/doc/CEF.txt",
    "texmf-dist/doc/latex/cjk/doc/CJK.txt",
    "texmf-dist/doc/latex/cjk/doc/CJKnumb.txt",
    "texmf-dist/doc/latex/cjk/doc/CJKspace.txt",
    "texmf-dist/doc/latex/cjk/doc/CJKutf8.txt",
    "texmf-dist/doc/latex/cjk/doc/COPYING",
    "texmf-dist/doc/latex/cjk/doc/INSTALL",
    "texmf-dist/doc/latex/cjk/doc/TDS.txt",
    "texmf-dist/doc/latex/cjk/doc/chinese/README",
    "texmf-dist/doc/latex/cjk/doc/chinese/emTeXb5.txt",
    "texmf-dist/doc/latex/cjk/doc/chinese/teTeXb5.txt",
    "texmf-dist/doc/latex/cjk/doc/cjk-enc.txt",
    "texmf-dist/doc/latex/cjk/doc/cjk/README",
    "texmf-dist/doc/latex/cjk/doc/commands.txt",
    "texmf-dist/doc/latex/cjk/doc/dvidrv.txt",
    "texmf-dist/doc/latex/cjk/doc/fdxfiles.txt",
    "texmf-dist/doc/latex/cjk/doc/fonts.txt",
    "texmf-dist/doc/latex/cjk/doc/hbf2gf.txt",
    "texmf-dist/doc/latex/cjk/doc/history.txt",
    "texmf-dist/doc/latex/cjk/doc/japanese/README",
    "texmf-dist/doc/latex/cjk/doc/japanese/ascii.txt",
    "texmf-dist/doc/latex/cjk/doc/japanese/japanese.txt",
    "texmf-dist/doc/latex/cjk/doc/japanese/jp-fonts.txt",
    "texmf-dist/doc/latex/cjk/doc/japanese/jp-tex.txt",
    "texmf-dist/doc/latex/cjk/doc/japanese/preview.txt",
    "texmf-dist/doc/latex/cjk/doc/japanese/shibuaki.txt",
    "texmf-dist/doc/latex/cjk/doc/pdf/READMEb5.pdf",
    "texmf-dist/doc/latex/cjk/doc/pdf/READMEgb.pdf",
    "texmf-dist/doc/latex/cjk/doc/pdfhowto/HOWTO.txt",
    "texmf-dist/doc/latex/cjk/doc/pinyin.txt",
    "texmf-dist/doc/latex/cjk/doc/pyhyphen.txt",
    "texmf-dist/doc/latex/cjk/doc/reftex.txt",
    "texmf-dist/doc/latex/cjk/doc/ruby.txt",
    "texmf-dist/doc/latex/cjk/doc/thaifont.txt",
    "texmf-dist/doc/latex/cjk/doc/vertical.txt",
    "texmf-dist/doc/latex/cjk/examples/README",
    "texmf-dist/doc/latex/cjk/examples/pdf/Big5.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/Big5vert.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/CEF_test.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/CJKbabel.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/CJKfntef.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/CJKmixed.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/CJKspace.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/CJKutf8.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/GB.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/JIS.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/KS.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/SJIS.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/UTF8.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/muletest.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/py_test.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/pytest.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/rubytest.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/thai.pdf",
    "texmf-dist/doc/latex/cjk/examples/pdf/thai_utf8.pdf",
  },
  ["binarytree"] = {
    "texmf-dist/doc/latex/binarytree/README",
    "texmf-dist/doc/latex/binarytree/binarytree.pdf",
    "texmf-dist/doc/latex/binarytree/examples/binarytree-ex1.pdf",
    "texmf-dist/doc/latex/binarytree/examples/binarytree-ex2.pdf",
    "texmf-dist/doc/latex/binarytree/examples/binarytree-ex3.pdf",
    "texmf-dist/doc/latex/binarytree/examples/binarytree-ex4.pdf",
    "texmf-dist/doc/latex/binarytree/examples/btree-5_up_0,0,0_3729359_7458719_655360_0.7_0.7_-lrr-x--_-llrr-x--_-rll-x--_-rrll-x--.pdf",
  },
  ["context-degrade"] = {
    "texmf-dist/doc/context/third/degrade/VERSION",
    "texmf-dist/doc/context/third/degrade/degrade-demo.pdf",
    "texmf-dist/doc/context/third/degrade/degrade-doc.pdf",
  },
  ["lastpackage"] = {
    "texmf-dist/doc/latex/lastpackage/README",
    "texmf-dist/doc/latex/lastpackage/lastpackage.pdf",
  },
  ["leipzig"] = {
    "texmf-dist/doc/latex/leipzig/README.md",
    "texmf-dist/doc/latex/leipzig/leipzig.pdf",
  },
  ["phfthm"] = {
    "texmf-dist/doc/latex/phfthm/Makefile",
    "texmf-dist/doc/latex/phfthm/README.md",
    "texmf-dist/doc/latex/phfthm/phfthm.pdf",
  },
  ["xetexko"] = {
    "texmf-dist/doc/xetex/xetexko/ChangeLog",
    "texmf-dist/doc/xetex/xetexko/README",
    "texmf-dist/doc/xetex/xetexko/xetexko-doc.pdf",
  },
  ["nwejm"] = {
    "texmf-dist/doc/latex/nwejm/README.md",
    "texmf-dist/doc/latex/nwejm/examples/article-in-dutch.pdf",
    "texmf-dist/doc/latex/nwejm/examples/article-in-english.pdf",
    "texmf-dist/doc/latex/nwejm/examples/article-in-french.pdf",
    "texmf-dist/doc/latex/nwejm/examples/article-in-german.pdf",
    "texmf-dist/doc/latex/nwejm/examples/issue.pdf",
    "texmf-dist/doc/latex/nwejm/examples/sample.pdf",
    "texmf-dist/doc/latex/nwejm/french/documentation/latexmkrc",
    "texmf-dist/doc/latex/nwejm/french/documentation/nwejm-fr.pdf",
  },
  ["lollipop"] = {
    "texmf-dist/doc/otherformats/lollipop/README",
    "texmf-dist/doc/otherformats/lollipop/manual/lollipop-manual.pdf",
  },
  ["phffullpagefigure"] = {
    "texmf-dist/doc/latex/phffullpagefigure/Makefile",
    "texmf-dist/doc/latex/phffullpagefigure/README.md",
    "texmf-dist/doc/latex/phffullpagefigure/phffullpagefigure.pdf",
  },
  ["grotesq"] = {
    "texmf-dist/doc/fonts/grotesq/grotesq.txt",
    "texmf-dist/doc/fonts/grotesq/readme.grotesq",
    "texmf-dist/doc/fonts/grotesq/ugq.txt",
  },
  ["lstbayes"] = {
    "texmf-dist/doc/latex/lstbayes/Makefile",
    "texmf-dist/doc/latex/lstbayes/README.md",
    "texmf-dist/doc/latex/lstbayes/examples.pdf",
    "texmf-dist/doc/latex/lstbayes/lstbayes.pdf",
  },
  ["uowthesistitlepage"] = {
    "texmf-dist/doc/latex/uowthesistitlepage/README.md",
    "texmf-dist/doc/latex/uowthesistitlepage/uowthesistitlepage_doc.pdf",
  },
  ["beamertheme-focus"] = {
    "texmf-dist/doc/latex/beamertheme-focus/LICENSE",
    "texmf-dist/doc/latex/beamertheme-focus/README.md",
    "texmf-dist/doc/latex/beamertheme-focus/demo.pdf",
    "texmf-dist/doc/latex/beamertheme-focus/focuslogo.pdf",
  },
  ["movie15"] = {
    "texmf-dist/doc/latex/movie15/README",
    "texmf-dist/doc/latex/movie15/files/3dsystem.pdf",
    "texmf-dist/doc/latex/movie15/movie15.pdf",
    "texmf-dist/doc/latex/movie15/overlay-example.pdf",
  },
  ["babel-latvian"] = {
    "texmf-dist/doc/generic/babel-latvian/README.md",
    "texmf-dist/doc/generic/babel-latvian/latvian.pdf",
  },
  ["mceinleger"] = {
    "texmf-dist/doc/latex/mceinleger/mceinleger.pdf",
  },
  ["stackengine"] = {
    "texmf-dist/doc/latex/stackengine/README",
    "texmf-dist/doc/latex/stackengine/stackengine.pdf",
  },
  ["fandol"] = {
    "texmf-dist/doc/fonts/fandol/COPYING",
    "texmf-dist/doc/fonts/fandol/README",
  },
  ["sectionbreak"] = {
    "texmf-dist/doc/latex/sectionbreak/README.md",
    "texmf-dist/doc/latex/sectionbreak/sectionbreak-doc.pdf",
  },
  ["guide-to-latex"] = {
    "texmf-dist/doc/latex/guide-to-latex/README.txt",
    "texmf-dist/doc/latex/guide-to-latex/demo.pdf",
    "texmf-dist/doc/latex/guide-to-latex/demodoc.pdf",
    "texmf-dist/doc/latex/guide-to-latex/demodoc.ps",
    "texmf-dist/doc/latex/guide-to-latex/palette.pdf",
    "texmf-dist/doc/latex/guide-to-latex/palette.ps",
  },
  ["upmethodology"] = {
    "texmf-dist/doc/latex/upmethodology/AUTHORS",
    "texmf-dist/doc/latex/upmethodology/COPYING",
    "texmf-dist/doc/latex/upmethodology/Changelog",
    "texmf-dist/doc/latex/upmethodology/README",
    "texmf-dist/doc/latex/upmethodology/README.textile",
    "texmf-dist/doc/latex/upmethodology/VERSION",
    "texmf-dist/doc/latex/upmethodology/figure_and_tex.pdf",
    "texmf-dist/doc/latex/upmethodology/upmethodology-doc.pdf",
  },
  ["elmath"] = {
    "texmf-dist/doc/latex/elmath/elmath.pdf",
  },
  ["imac"] = {
    "texmf-dist/doc/latex/imac/imac.pdf",
    "texmf-dist/doc/latex/imac/readme.txt",
  },
  ["mdputu"] = {
    "texmf-dist/doc/latex/mdputu/README",
    "texmf-dist/doc/latex/mdputu/mdputu.pdf",
    "texmf-dist/doc/latex/mdputu/sample.pdf",
  },
  ["finstrut"] = {
    "texmf-dist/doc/latex/finstrut/README",
    "texmf-dist/doc/latex/finstrut/SRCFILEs.txt",
    "texmf-dist/doc/latex/finstrut/finstrut.pdf",
    "texmf-dist/doc/latex/finstrut/fstrutst.pdf",
  },
  ["flabels"] = {
    "texmf-dist/doc/latex/flabels/README",
    "texmf-dist/doc/latex/flabels/flabels.pdf",
    "texmf-dist/doc/latex/flabels/makedoc",
    "texmf-dist/doc/latex/flabels/makedoc-patched",
  },
  ["authorarchive"] = {
    "texmf-dist/doc/latex/authorarchive/LICENSE",
    "texmf-dist/doc/latex/authorarchive/README.md",
    "texmf-dist/doc/latex/authorarchive/brucker-authorarchive-2016.pdf",
  },
  ["datetime2-it-fulltext"] = {
    "texmf-dist/doc/latex/datetime2-it-fulltext/README",
    "texmf-dist/doc/latex/datetime2-it-fulltext/datetime2-it-fulltext.pdf",
  },
  ["makebox"] = {
    "texmf-dist/doc/latex/makebox/ChangeLog",
    "texmf-dist/doc/latex/makebox/README",
    "texmf-dist/doc/latex/makebox/makebox.pdf",
  },
  ["babel-icelandic"] = {
    "texmf-dist/doc/generic/babel-icelandic/README.md",
    "texmf-dist/doc/generic/babel-icelandic/icelandic.pdf",
  },
  ["cochineal"] = {
    "texmf-dist/doc/fonts/cochineal/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/cochineal/OFL.txt",
    "texmf-dist/doc/fonts/cochineal/README",
    "texmf-dist/doc/fonts/cochineal/cochineal-doc.pdf",
  },
  ["fancyhdr-it"] = {
    "texmf-dist/doc/latex/fancyhdr-it/Makefile",
    "texmf-dist/doc/latex/fancyhdr-it/README",
    "texmf-dist/doc/latex/fancyhdr-it/itfancyhdr.pdf",
  },
  ["datenumber"] = {
    "texmf-dist/doc/latex/datenumber/README.txt",
    "texmf-dist/doc/latex/datenumber/doc.pdf",
    "texmf-dist/doc/latex/datenumber/docgerman.pdf",
  },
  ["animate"] = {
    "texmf-dist/doc/latex/animate/ChangeLog",
    "texmf-dist/doc/latex/animate/README",
    "texmf-dist/doc/latex/animate/animate.pdf",
  },
  ["venn"] = {
    "texmf-dist/doc/metapost/venn/README",
  },
  ["table-fct"] = {
    "texmf-dist/doc/latex/table-fct/README",
    "texmf-dist/doc/latex/table-fct/README.TEXLIVE",
  },
  ["cmap"] = {
    "texmf-dist/doc/latex/cmap/README",
  },
  ["lettre"] = {
    "texmf-dist/doc/latex/lettre/3.000-release-notes.txt",
    "texmf-dist/doc/latex/lettre/ALIRE_TDS",
    "texmf-dist/doc/latex/lettre/README_TDS",
    "texmf-dist/doc/latex/lettre/lettre.pdf",
  },
  ["lshort-japanese"] = {
    "texmf-dist/doc/latex/lshort-japanese/00README",
    "texmf-dist/doc/latex/lshort-japanese/jlshort.pdf",
  },
  ["longdivision"] = {
    "texmf-dist/doc/latex/longdivision/README",
    "texmf-dist/doc/latex/longdivision/longdivision_example.pdf",
  },
  ["mathabx"] = {
    "texmf-dist/doc/fonts/mathabx/README",
    "texmf-dist/doc/fonts/mathabx/mathtest.pdf",
  },
  ["umlaute"] = {
    "texmf-dist/doc/latex/umlaute/umlaute.pdf",
  },
  ["gitfile-info"] = {
    "texmf-dist/doc/support/gitfile-info/README",
    "texmf-dist/doc/support/gitfile-info/gitfile-info.pdf",
  },
  ["mmap"] = {
    "texmf-dist/doc/latex/mmap/README",
  },
  ["cfr-initials"] = {
    "texmf-dist/doc/latex/cfr-initials/README",
    "texmf-dist/doc/latex/cfr-initials/cfr-initials.pdf",
    "texmf-dist/doc/latex/cfr-initials/manifest.txt",
  },
  ["jkmath"] = {
    "texmf-dist/doc/latex/jkmath/README.md",
    "texmf-dist/doc/latex/jkmath/jkmath.pdf",
  },
  ["luatexbase"] = {
    "texmf-dist/doc/luatex/luatexbase/README.md",
    "texmf-dist/doc/luatex/luatexbase/luatexbase.pdf",
  },
  ["dtk"] = {
    "texmf-dist/doc/latex/dtk/README.md",
    "texmf-dist/doc/latex/dtk/doc/beispiel.pdf",
  },
  ["bibarts"] = {
    "texmf-dist/doc/latex/bibarts/COPYING",
    "texmf-dist/doc/latex/bibarts/ba-short.pdf",
    "texmf-dist/doc/latex/bibarts/bibarts.pdf",
    "texmf-dist/doc/latex/bibarts/readme.txt",
  },
  ["mathexam"] = {
    "texmf-dist/doc/latex/mathexam/README",
    "texmf-dist/doc/latex/mathexam/mathexam.pdf",
    "texmf-dist/doc/latex/mathexam/sample.pdf",
  },
  ["translation-ecv-de"] = {
    "texmf-dist/doc/latex/translation-ecv-de/ecvde.dtx.pdf",
  },
  ["lettrine"] = {
    "texmf-dist/doc/latex/lettrine/README",
    "texmf-dist/doc/latex/lettrine/W.pdf",
    "texmf-dist/doc/latex/lettrine/demo-de.pdf",
    "texmf-dist/doc/latex/lettrine/demo.pdf",
    "texmf-dist/doc/latex/lettrine/lettrine.pdf",
  },
  ["luabibentry"] = {
    "texmf-dist/doc/lualatex/luabibentry/News",
    "texmf-dist/doc/lualatex/luabibentry/README",
    "texmf-dist/doc/lualatex/luabibentry/luabibentry.pdf",
  },
  ["circ"] = {
    "texmf-dist/doc/latex/circ/COPYING",
    "texmf-dist/doc/latex/circ/README",
    "texmf-dist/doc/latex/circ/circ.pdf",
    "texmf-dist/doc/latex/circ/circ.txt",
  },
  ["serbian-lig"] = {
    "texmf-dist/doc/latex/serbian-lig/Changes.txt",
    "texmf-dist/doc/latex/serbian-lig/README.txt",
    "texmf-dist/doc/latex/serbian-lig/lig-list.pdf",
    "texmf-dist/doc/latex/serbian-lig/serbian-lig.pdf",
  },
  ["latex-papersize"] = {
    "texmf-dist/doc/support/latex-papersize/README",
  },
  ["dsserif"] = {
    "texmf-dist/doc/fonts/dsserif/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/dsserif/OFL.txt",
    "texmf-dist/doc/fonts/dsserif/README",
    "texmf-dist/doc/fonts/dsserif/dsserif-doc.pdf",
  },
  ["datetime2-welsh"] = {
    "texmf-dist/doc/latex/datetime2-welsh/README",
    "texmf-dist/doc/latex/datetime2-welsh/datetime2-welsh.pdf",
  },
  ["amsmath-it"] = {
    "texmf-dist/doc/latex/amsmath-it/README",
    "texmf-dist/doc/latex/amsmath-it/diffs-m_it.txt",
  },
  ["imtekda"] = {
    "texmf-dist/doc/latex/imtekda/IMTEKda.pdf",
    "texmf-dist/doc/latex/imtekda/README",
    "texmf-dist/doc/latex/imtekda/figures/bild.pdf",
  },
  ["cweb-latex"] = {
    "texmf-dist/doc/latex/cweb-latex/CATALOG",
    "texmf-dist/doc/latex/cweb-latex/History",
    "texmf-dist/doc/latex/cweb-latex/INSTALL",
    "texmf-dist/doc/latex/cweb-latex/License",
    "texmf-dist/doc/latex/cweb-latex/MANIFEST",
    "texmf-dist/doc/latex/cweb-latex/Makefile",
    "texmf-dist/doc/latex/cweb-latex/README",
    "texmf-dist/doc/latex/cweb-latex/contrib/Index",
    "texmf-dist/doc/latex/cweb-latex/contrib/cweb-hy/README.txt",
    "texmf-dist/doc/latex/cweb-latex/cweb-conf.pdf",
    "texmf-dist/doc/latex/cweb-latex/cweb-user.pdf",
    "texmf-dist/doc/latex/cweb-latex/examples/Makefile",
    "texmf-dist/doc/latex/cweb-latex/examples/compare/wcltx.dvi",
    "texmf-dist/doc/latex/cweb-latex/examples/tex-it",
    "texmf-dist/doc/latex/cweb-latex/src/Imakefile",
    "texmf-dist/doc/latex/cweb-latex/src/README",
    "texmf-dist/doc/latex/cweb-latex/src/TODO",
    "texmf-dist/doc/latex/cweb-latex/src/test/Imakefile",
  },
  ["r_und_s"] = {
    "texmf-dist/doc/latex/r_und_s/README",
  },
  ["linegoal"] = {
    "texmf-dist/doc/latex/linegoal/README",
    "texmf-dist/doc/latex/linegoal/linegoal.pdf",
  },
  ["keyreader"] = {
    "texmf-dist/doc/latex/keyreader/README",
    "texmf-dist/doc/latex/keyreader/keyreader-guide.pdf",
  },
  ["drs"] = {
    "texmf-dist/doc/latex/drs/README",
    "texmf-dist/doc/latex/drs/drsdoc.pdf",
  },
  ["latex-web-companion"] = {
    "texmf-dist/doc/latex/latex-web-companion/apa/README.apa",
    "texmf-dist/doc/latex/latex-web-companion/apa/teched.html",
    "texmf-dist/doc/latex/latex-web-companion/apb/README.apb",
    "texmf-dist/doc/latex/latex-web-companion/apc/README.apc",
    "texmf-dist/doc/latex/latex-web-companion/apc/utf8.html",
    "texmf-dist/doc/latex/latex-web-companion/apc/utf8raw.html",
    "texmf-dist/doc/latex/latex-web-companion/ch2/latexexa.pdf",
    "texmf-dist/doc/latex/latex-web-companion/ch2/phys332-1.pdf",
    "texmf-dist/doc/latex/latex-web-companion/ch2/phys332-2.pdf",
    "texmf-dist/doc/latex/latex-web-companion/ch3/README.ch3",
    "texmf-dist/doc/latex/latex-web-companion/ch4/latexexa.dvi",
    "texmf-dist/doc/latex/latex-web-companion/ch4/latexexa.html",
    "texmf-dist/doc/latex/latex-web-companion/ch4/tmp.ps",
    "texmf-dist/doc/latex/latex-web-companion/ch6/README.ch6",
    "texmf-dist/doc/latex/latex-web-companion/ch6/catalog",
    "texmf-dist/doc/latex/latex-web-companion/ch7/README.ch7",
    "texmf-dist/doc/latex/latex-web-companion/ch7/catalog",
    "texmf-dist/doc/latex/latex-web-companion/ch7/debug.txt",
    "texmf-dist/doc/latex/latex-web-companion/ch7/invfo1.pdf",
  },
  ["latex-course"] = {
    "texmf-dist/doc/latex/latex-course/LaTeX-Course.pdf",
    "texmf-dist/doc/latex/latex-course/README",
  },
  ["gnu-freefont"] = {
    "texmf-dist/doc/fonts/gnu-freefont/AUTHORS",
    "texmf-dist/doc/fonts/gnu-freefont/BUILDING",
    "texmf-dist/doc/fonts/gnu-freefont/COPYING",
    "texmf-dist/doc/fonts/gnu-freefont/CREDITS",
    "texmf-dist/doc/fonts/gnu-freefont/ChangeLog",
    "texmf-dist/doc/fonts/gnu-freefont/INSTALL",
    "texmf-dist/doc/fonts/gnu-freefont/Makefile",
    "texmf-dist/doc/fonts/gnu-freefont/README",
    "texmf-dist/doc/fonts/gnu-freefont/TROUBLESHOOTING",
    "texmf-dist/doc/fonts/gnu-freefont/USAGE",
    "texmf-dist/doc/fonts/gnu-freefont/notes/README-downloads.txt",
    "texmf-dist/doc/fonts/gnu-freefont/notes/building.txt",
    "texmf-dist/doc/fonts/gnu-freefont/notes/features.txt",
    "texmf-dist/doc/fonts/gnu-freefont/notes/maintenance.txt",
    "texmf-dist/doc/fonts/gnu-freefont/notes/troubleshooting.txt",
    "texmf-dist/doc/fonts/gnu-freefont/notes/usage.txt",
    "texmf-dist/doc/fonts/gnu-freefont/notes/webfont_guidelines.txt",
    "texmf-dist/doc/fonts/gnu-freefont/tools/generate/MacTT",
    "texmf-dist/doc/fonts/gnu-freefont/tools/generate/OpenType",
    "texmf-dist/doc/fonts/gnu-freefont/tools/generate/TrueType",
    "texmf-dist/doc/fonts/gnu-freefont/tools/generate/WOFF",
    "texmf-dist/doc/fonts/gnu-freefont/tools/report/OS2UnicodeRange",
    "texmf-dist/doc/fonts/gnu-freefont/tools/test/MES-Conformance/MES-1.txt",
    "texmf-dist/doc/fonts/gnu-freefont/tools/test/MES-Conformance/MES-2.txt",
    "texmf-dist/doc/fonts/gnu-freefont/tools/test/MES-Conformance/MES-3B.txt",
  },
  ["amscls-doc"] = {
    "texmf-dist/doc/latex/amscls-doc/Author_Handbook_Journals.pdf",
    "texmf-dist/doc/latex/amscls-doc/Author_Handbook_Memo.pdf",
    "texmf-dist/doc/latex/amscls-doc/Author_Handbook_Mono.pdf",
    "texmf-dist/doc/latex/amscls-doc/Author_Handbook_ProcColl.pdf",
    "texmf-dist/doc/latex/amscls-doc/Color2Gray.pdf",
    "texmf-dist/doc/latex/amscls-doc/README-AH.txt",
    "texmf-dist/doc/latex/amscls-doc/ams-author-handbook-doc.pdf",
    "texmf-dist/doc/latex/amscls-doc/gamuts.pdf",
    "texmf-dist/doc/latex/amscls-doc/manifest.txt",
    "texmf-dist/doc/latex/amscls-doc/rgb-cmyk.pdf",
    "texmf-dist/doc/latex/amscls-doc/spectrum.pdf",
  },
  ["ifptex"] = {
    "texmf-dist/doc/generic/ifptex/LICENSE",
    "texmf-dist/doc/generic/ifptex/README.md",
  },
  ["authorindex"] = {
    "texmf-dist/doc/latex/authorindex/COPYING",
    "texmf-dist/doc/latex/authorindex/NEWS",
    "texmf-dist/doc/latex/authorindex/README",
    "texmf-dist/doc/latex/authorindex/authorindex.pdf",
  },
  ["biblatex-historian"] = {
    "texmf-dist/doc/latex/biblatex-historian/README.txt",
    "texmf-dist/doc/latex/biblatex-historian/historian.pdf",
  },
  ["polyglossia"] = {
    "texmf-dist/doc/latex/polyglossia/README",
    "texmf-dist/doc/latex/polyglossia/example-arabic.pdf",
    "texmf-dist/doc/latex/polyglossia/example-korean.pdf",
    "texmf-dist/doc/latex/polyglossia/example-thai.pdf",
    "texmf-dist/doc/latex/polyglossia/examples.pdf",
    "texmf-dist/doc/latex/polyglossia/gloss-latin.pdf",
    "texmf-dist/doc/latex/polyglossia/polyglossia.pdf",
    "texmf-dist/doc/latex/polyglossia/test-welsh.pdf",
  },
  ["t2"] = {
    "texmf-dist/doc/generic/t2/Makefile",
    "texmf-dist/doc/generic/t2/README",
    "texmf-dist/doc/generic/t2/amscyr.txt",
    "texmf-dist/doc/generic/t2/broken1.txt",
    "texmf-dist/doc/generic/t2/broken2.txt",
    "texmf-dist/doc/generic/t2/cyrcset7.txt",
    "texmf-dist/doc/generic/t2/cyrcset8.txt",
    "texmf-dist/doc/generic/t2/etc/rubibtex/README",
    "texmf-dist/doc/generic/t2/etc/rubibtex/rubibtex",
    "texmf-dist/doc/generic/t2/etc/ruinpenc",
    "texmf-dist/doc/generic/t2/etc/rumkidx/README",
    "texmf-dist/doc/generic/t2/etc/rumkidx/rumakeindex",
    "texmf-dist/doc/generic/t2/mtcyr.txt",
    "texmf-dist/doc/generic/t2/t2cyr.txt",
    "texmf-dist/doc/generic/t2/t2lat.txt",
    "texmf-dist/doc/generic/t2/urwcyr.txt",
  },
  ["german"] = {
    "texmf-dist/doc/generic/german/00readme.1st",
    "texmf-dist/doc/generic/german/betatest/00readme.1st",
    "texmf-dist/doc/generic/german/gerdoc.pdf",
  },
  ["wintools.win32"] = {
    "texmf-dist/doc/support/tlaunch/COPYING",
    "texmf-dist/doc/support/tlaunch/Changes",
    "texmf-dist/doc/support/tlaunch/README",
    "texmf-dist/doc/support/tlaunch/tlaunch.pdf",
    "texmf-dist/doc/support/wintools/fc-cache.pdf",
    "texmf-dist/doc/support/wintools/fc-cat.pdf",
    "texmf-dist/doc/support/wintools/fc-list.pdf",
    "texmf-dist/doc/support/wintools/fc-match.pdf",
    "texmf-dist/doc/support/wintools/fc-pattern.pdf",
    "texmf-dist/doc/support/wintools/fc-query.pdf",
    "texmf-dist/doc/support/wintools/fc-scan.pdf",
    "texmf-dist/doc/support/wintools/fc-validate.pdf",
    "texmf-dist/doc/support/wintools/gzip.pdf",
    "texmf-dist/doc/support/wintools/pdfdetach.pdf",
    "texmf-dist/doc/support/wintools/pdffonts.pdf",
    "texmf-dist/doc/support/wintools/pdfimages.pdf",
    "texmf-dist/doc/support/wintools/pdfinfo.pdf",
    "texmf-dist/doc/support/wintools/pdfseparate.pdf",
    "texmf-dist/doc/support/wintools/pdfsig.pdf",
    "texmf-dist/doc/support/wintools/pdftocairo.pdf",
    "texmf-dist/doc/support/wintools/pdftohtml.pdf",
    "texmf-dist/doc/support/wintools/pdftoppm.pdf",
    "texmf-dist/doc/support/wintools/pdftops.pdf",
    "texmf-dist/doc/support/wintools/pdftotext.pdf",
    "texmf-dist/doc/support/wintools/pdfunite.pdf",
    "texmf-dist/doc/support/wintools/unzip.pdf",
    "texmf-dist/doc/support/wintools/zip.pdf",
  },
  ["sansmath"] = {
    "texmf-dist/doc/latex/sansmath/sansmath.pdf",
  },
  ["epigrafica"] = {
    "texmf-dist/doc/fonts/epigrafica/README",
    "texmf-dist/doc/fonts/epigrafica/epigrafica.pdf",
  },
  ["jlabels"] = {
    "texmf-dist/doc/latex/jlabels/README",
    "texmf-dist/doc/latex/jlabels/jlabels.pdf",
  },
  ["langcode"] = {
    "texmf-dist/doc/generic/langcode/README",
    "texmf-dist/doc/generic/langcode/SrcFILEs.txt",
    "texmf-dist/doc/generic/langcode/langcode.pdf",
  },
  ["a5comb"] = {
    "texmf-dist/doc/latex/a5comb/a5comb.pdf",
  },
  ["uiucthesis"] = {
    "texmf-dist/doc/latex/uiucthesis/README",
    "texmf-dist/doc/latex/uiucthesis/thesis-ex.pdf",
    "texmf-dist/doc/latex/uiucthesis/uiucthesis.pdf",
  },
  ["impatient"] = {
    "texmf-dist/doc/plain/impatient/Makefile",
    "texmf-dist/doc/plain/impatient/NEWS",
    "texmf-dist/doc/plain/impatient/README",
    "texmf-dist/doc/plain/impatient/book.pdf",
    "texmf-dist/doc/plain/impatient/configure",
  },
  ["pdfcomment"] = {
    "texmf-dist/doc/latex/pdfcomment/CHANGES",
    "texmf-dist/doc/latex/pdfcomment/INSTALL",
    "texmf-dist/doc/latex/pdfcomment/README.md",
    "texmf-dist/doc/latex/pdfcomment/example.pdf",
    "texmf-dist/doc/latex/pdfcomment/example_latex_dvipdfmx.pdf",
    "texmf-dist/doc/latex/pdfcomment/example_latex_dvips_ps2pdf.pdf",
    "texmf-dist/doc/latex/pdfcomment/example_math_markup.pdf",
    "texmf-dist/doc/latex/pdfcomment/example_xelatex.pdf",
    "texmf-dist/doc/latex/pdfcomment/manifest.txt",
    "texmf-dist/doc/latex/pdfcomment/pdfcomment.pdf",
    "texmf-dist/doc/latex/pdfcomment/pdfcomment_de.pdf",
  },
  ["bidipagegrid"] = {
    "texmf-dist/doc/xelatex/bidipagegrid/README",
    "texmf-dist/doc/xelatex/bidipagegrid/bidipagegrid-doc.pdf",
  },
  ["beamertheme-phnompenh"] = {
    "texmf-dist/doc/latex/beamertheme-phnompenh/README",
    "texmf-dist/doc/latex/beamertheme-phnompenh/beamerthemePhnomPenh.pdf",
  },
  ["presentations-en"] = {
    "texmf-dist/doc/latex/presentations-en/README",
    "texmf-dist/doc/latex/presentations-en/images/beamer/TeX.pdf",
    "texmf-dist/doc/latex/presentations-en/images/beamer/beamer0.pdf",
    "texmf-dist/doc/latex/presentations-en/images/beamer/beamer1.pdf",
    "texmf-dist/doc/latex/presentations-en/images/beamer/beamer2.pdf",
    "texmf-dist/doc/latex/presentations-en/images/beamer/beamernavsymbols.pdf",
    "texmf-dist/doc/latex/presentations-en/images/beamer/zedat.pdf",
    "texmf-dist/doc/latex/presentations-en/images/pdfscreen/Tore3d.pdf",
    "texmf-dist/doc/latex/presentations-en/images/pdfscreen/mp.pdf",
    "texmf-dist/doc/latex/presentations-en/images/pdfscreen/pst.pdf",
    "texmf-dist/doc/latex/presentations-en/runAll",
  },
  ["fetchcls"] = {
    "texmf-dist/doc/latex/fetchcls/README.md",
    "texmf-dist/doc/latex/fetchcls/fetchcls.pdf",
  },
  ["showcharinbox"] = {
    "texmf-dist/doc/latex/showcharinbox/README",
    "texmf-dist/doc/latex/showcharinbox/showcharinbox.pdf",
  },
  ["logicpuzzle"] = {
    "texmf-dist/doc/latex/logicpuzzle/README",
    "texmf-dist/doc/latex/logicpuzzle/logicpuzzle.pdf",
    "texmf-dist/doc/latex/logicpuzzle/manifest.txt",
  },
  ["context-layout"] = {
    "texmf-dist/doc/context/third/layout/VERSION",
    "texmf-dist/doc/context/third/layout/t-layout.pdf",
  },
  ["ccicons"] = {
    "texmf-dist/doc/fonts/ccicons/FONTLOG.txt",
    "texmf-dist/doc/fonts/ccicons/OFL.txt",
    "texmf-dist/doc/latex/ccicons/ccicons.pdf",
  },
  ["changepage"] = {
    "texmf-dist/doc/latex/changepage/README",
    "texmf-dist/doc/latex/changepage/changepage.pdf",
  },
  ["dad"] = {
    "texmf-dist/doc/fonts/dad/README",
    "texmf-dist/doc/fonts/dad/dad-user-guide.pdf",
  },
  ["marginfit"] = {
    "texmf-dist/doc/latex/marginfit/Makefile",
    "texmf-dist/doc/latex/marginfit/README",
    "texmf-dist/doc/latex/marginfit/marginfit.pdf",
  },
  ["ucdavisthesis"] = {
    "texmf-dist/doc/latex/ucdavisthesis/README",
    "texmf-dist/doc/latex/ucdavisthesis/example/ucdavisthesis_example_figure.pdf",
    "texmf-dist/doc/latex/ucdavisthesis/example/ucdavisthesis_example_main.pdf",
    "texmf-dist/doc/latex/ucdavisthesis/ucdavisthesis.pdf",
  },
  ["graphbox"] = {
    "texmf-dist/doc/latex/graphbox/README.txt",
    "texmf-dist/doc/latex/graphbox/graphbox.pdf",
  },
  ["cns"] = {
    "texmf-dist/doc/fonts/cns/cns40-1/README",
    "texmf-dist/doc/fonts/cns/cns40-2/README",
    "texmf-dist/doc/fonts/cns/cns40-3/README",
    "texmf-dist/doc/fonts/cns/cns40-4/README",
    "texmf-dist/doc/fonts/cns/cns40-5/README",
    "texmf-dist/doc/fonts/cns/cns40-6/README",
    "texmf-dist/doc/fonts/cns/cns40-7/README",
    "texmf-dist/doc/fonts/cns/cns40-b5/README",
  },
  ["thumbs"] = {
    "texmf-dist/doc/latex/thumbs/README",
    "texmf-dist/doc/latex/thumbs/thumbs-example.pdf",
    "texmf-dist/doc/latex/thumbs/thumbs.pdf",
  },
  ["pygmentex"] = {
    "texmf-dist/doc/latex/pygmentex/README",
    "texmf-dist/doc/latex/pygmentex/demo.pdf",
  },
  ["dialogl"] = {
    "texmf-dist/doc/latex/dialogl/Makefile",
    "texmf-dist/doc/latex/dialogl/README",
    "texmf-dist/doc/latex/dialogl/codialog.pdf",
    "texmf-dist/doc/latex/dialogl/dia-driv.pdf",
    "texmf-dist/doc/latex/dialogl/manifest.txt",
  },
  ["mathpartir"] = {
    "texmf-dist/doc/latex/mathpartir/COPYING",
    "texmf-dist/doc/latex/mathpartir/README",
    "texmf-dist/doc/latex/mathpartir/mathpartir.pdf",
  },
  ["meetingmins"] = {
    "texmf-dist/doc/latex/meetingmins/README",
    "texmf-dist/doc/latex/meetingmins/meetingmins.pdf",
    "texmf-dist/doc/latex/meetingmins/samples/agenda.pdf",
    "texmf-dist/doc/latex/meetingmins/samples/chair.pdf",
    "texmf-dist/doc/latex/meetingmins/samples/minutes.pdf",
  },
  ["jacow"] = {
    "texmf-dist/doc/latex/jacow/JACoW_A4_LaTeX.pdf",
    "texmf-dist/doc/latex/jacow/JACoW_US_LaTeX.pdf",
    "texmf-dist/doc/latex/jacow/JACpic_mc.pdf",
    "texmf-dist/doc/latex/jacow/README",
    "texmf-dist/doc/latex/jacow/jacow-collaboration-2018.pdf",
  },
  ["oplotsymbl"] = {
    "texmf-dist/doc/latex/oplotsymbl/README.md",
    "texmf-dist/doc/latex/oplotsymbl/oPlotSymbl-Manual-en.pdf",
  },
  ["tinos"] = {
    "texmf-dist/doc/fonts/tinos/LICENSE.txt",
    "texmf-dist/doc/fonts/tinos/README",
    "texmf-dist/doc/fonts/tinos/tinos-samples.pdf",
  },
  ["booklet"] = {
    "texmf-dist/doc/latex/booklet/README",
    "texmf-dist/doc/latex/booklet/booklet.pdf",
  },
  ["lcd"] = {
    "texmf-dist/doc/latex/lcd/00readme",
    "texmf-dist/doc/latex/lcd/example.pdf",
  },
  ["biblatex-bookinarticle"] = {
    "texmf-dist/doc/latex/biblatex-bookinarticle/README",
    "texmf-dist/doc/latex/biblatex-bookinarticle/documentation/biblatex-bookinarticle-crossref.pdf",
    "texmf-dist/doc/latex/biblatex-bookinarticle/documentation/biblatex-bookinarticle.pdf",
    "texmf-dist/doc/latex/biblatex-bookinarticle/documentation/makefile",
    "texmf-dist/doc/latex/biblatex-bookinarticle/makefile",
  },
  ["options"] = {
    "texmf-dist/doc/latex/options/README",
    "texmf-dist/doc/latex/options/options.html",
    "texmf-dist/doc/latex/options/options.pdf",
  },
  ["nidanfloat"] = {
    "texmf-dist/doc/latex/nidanfloat/LICENSE",
    "texmf-dist/doc/latex/nidanfloat/README.md",
    "texmf-dist/doc/latex/nidanfloat/nidanfloat.pdf",
  },
  ["pst-support"] = {
    "texmf-dist/doc/generic/pst-support/README",
  },
  ["cases"] = {
    "texmf-dist/doc/latex/cases/cases.pdf",
  },
  ["pdfpc-movie"] = {
    "texmf-dist/doc/latex/pdfpc-movie/README.md",
    "texmf-dist/doc/latex/pdfpc-movie/pdfpc-movie-doc.pdf",
  },
  ["levy"] = {
    "texmf-dist/doc/fonts/levy/README",
    "texmf-dist/doc/fonts/levy/makeall",
    "texmf-dist/doc/fonts/levy/makefont",
    "texmf-dist/doc/fonts/levy/testfont",
  },
  ["biblatex-true-citepages-omit"] = {
    "texmf-dist/doc/latex/biblatex-true-citepages-omit/README",
    "texmf-dist/doc/latex/biblatex-true-citepages-omit/biblatex-true-citepages-omit-example.pdf",
    "texmf-dist/doc/latex/biblatex-true-citepages-omit/biblatex-true-citepages-omit.pdf",
    "texmf-dist/doc/latex/biblatex-true-citepages-omit/latexmkrc",
    "texmf-dist/doc/latex/biblatex-true-citepages-omit/makefile",
  },
  ["gfsneohellenic"] = {
    "texmf-dist/doc/fonts/gfsneohellenic/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gfsneohellenic/OFL.txt",
    "texmf-dist/doc/fonts/gfsneohellenic/README",
    "texmf-dist/doc/fonts/gfsneohellenic/README.TEXLIVE",
    "texmf-dist/doc/fonts/gfsneohellenic/VERSION",
  },
  ["sslides"] = {
    "texmf-dist/doc/latex/sslides/land-sample-slide.pdf",
    "texmf-dist/doc/latex/sslides/port-sample-slide.pdf",
  },
  ["blochsphere"] = {
    "texmf-dist/doc/latex/blochsphere/LICENSE",
    "texmf-dist/doc/latex/blochsphere/README.md",
    "texmf-dist/doc/latex/blochsphere/blochsphere.pdf",
    "texmf-dist/doc/latex/blochsphere/example.pdf",
  },
  ["lualatex-doc-de"] = {
    "texmf-dist/doc/latex/lualatex-doc-de/lualatex-doc-DE.pdf",
  },
  ["pspicture"] = {
    "texmf-dist/doc/latex/pspicture/README",
    "texmf-dist/doc/latex/pspicture/pspicture.pdf",
  },
  ["imsproc"] = {
    "texmf-dist/doc/xelatex/imsproc/README",
  },
  ["reverxii"] = {
    "texmf-dist/doc/plain/reverxii/README",
    "texmf-dist/doc/plain/reverxii/reverxii.pdf",
  },
  ["tikz-karnaugh"] = {
    "texmf-dist/doc/latex/tikz-karnaugh/README.txt",
    "texmf-dist/doc/latex/tikz-karnaugh/tikz-karnaugh-doc.pdf",
  },
  ["counttexruns"] = {
    "texmf-dist/doc/latex/counttexruns/README",
    "texmf-dist/doc/latex/counttexruns/counttexruns.pdf",
  },
  ["jurabib"] = {
    "texmf-dist/doc/latex/jurabib/changes.txt",
    "texmf-dist/doc/latex/jurabib/docs/english/jbendoc.pdf",
    "texmf-dist/doc/latex/jurabib/docs/german/jbgerdoc.pdf",
    "texmf-dist/doc/latex/jurabib/jbtest.pdf",
  },
  ["latexpand"] = {
    "texmf-dist/doc/support/latexpand/LICENCE",
    "texmf-dist/doc/support/latexpand/README",
    "texmf-dist/doc/support/latexpand/version.txt",
  },
  ["gfsbaskerville"] = {
    "texmf-dist/doc/fonts/gfsbaskerville/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gfsbaskerville/OFL.txt",
    "texmf-dist/doc/fonts/gfsbaskerville/README",
    "texmf-dist/doc/fonts/gfsbaskerville/README.TEXLIVE",
    "texmf-dist/doc/fonts/gfsbaskerville/gfsbaskerville.pdf",
  },
  ["luacode"] = {
    "texmf-dist/doc/lualatex/luacode/News",
    "texmf-dist/doc/lualatex/luacode/README",
    "texmf-dist/doc/lualatex/luacode/luacode.pdf",
  },
  ["babel-norsk"] = {
    "texmf-dist/doc/generic/babel-norsk/norsk.pdf",
  },
  ["dingbat"] = {
    "texmf-dist/doc/fonts/dingbat/README",
    "texmf-dist/doc/fonts/dingbat/dingbat.pdf",
  },
  ["clipboard"] = {
    "texmf-dist/doc/latex/clipboard/README",
    "texmf-dist/doc/latex/clipboard/clipboard.pdf",
  },
  ["guitar"] = {
    "texmf-dist/doc/latex/guitar/README",
    "texmf-dist/doc/latex/guitar/guitar.pdf",
    "texmf-dist/doc/latex/guitar/guitar.txt",
  },
  ["lcdftypetools"] = {
    "texmf-dist/doc/man/man1/cfftot1.man1.pdf",
    "texmf-dist/doc/man/man1/mmafm.man1.pdf",
    "texmf-dist/doc/man/man1/mmpfb.man1.pdf",
    "texmf-dist/doc/man/man1/otfinfo.man1.pdf",
    "texmf-dist/doc/man/man1/otftotfm.man1.pdf",
    "texmf-dist/doc/man/man1/t1dotlessj.man1.pdf",
    "texmf-dist/doc/man/man1/t1lint.man1.pdf",
    "texmf-dist/doc/man/man1/t1rawafm.man1.pdf",
    "texmf-dist/doc/man/man1/t1reencode.man1.pdf",
    "texmf-dist/doc/man/man1/t1testpage.man1.pdf",
    "texmf-dist/doc/man/man1/ttftotype42.man1.pdf",
  },
  ["bera"] = {
    "texmf-dist/doc/fonts/bera/LICENSE",
    "texmf-dist/doc/fonts/bera/README",
    "texmf-dist/doc/fonts/bera/bera.pdf",
    "texmf-dist/doc/fonts/bera/bera.txt",
  },
  ["skb"] = {
    "texmf-dist/doc/latex/skb/HISTORY.TXT",
    "texmf-dist/doc/latex/skb/LICENSE.TXT",
    "texmf-dist/doc/latex/skb/MANIFEST.TXT",
    "texmf-dist/doc/latex/skb/README",
    "texmf-dist/doc/latex/skb/TODO.TXT",
    "texmf-dist/doc/latex/skb/makefile",
    "texmf-dist/doc/latex/skb/skb.pdf",
    "texmf-dist/doc/latex/skb/user-guide.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-0.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-1.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-10.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-11.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-12.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-13.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-14.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-2.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-3.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-4.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-5.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-6.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-7.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-8.pdf",
    "texmf-dist/doc/latex/skb/user-guide/figures/multiexample/dpe-9.pdf",
  },
  ["babel-greek"] = {
    "texmf-dist/doc/generic/babel-greek/README",
    "texmf-dist/doc/generic/babel-greek/README.html",
    "texmf-dist/doc/generic/babel-greek/athnum.pdf",
    "texmf-dist/doc/generic/babel-greek/babel-greek.pdf",
    "texmf-dist/doc/generic/babel-greek/grmath.pdf",
    "texmf-dist/doc/generic/babel-greek/test-greek.pdf",
    "texmf-dist/doc/generic/babel-greek/test-unicode-greek.pdf",
    "texmf-dist/doc/generic/babel-greek/test-unicode-lgr.pdf",
    "texmf-dist/doc/generic/babel-greek/usage.pdf",
  },
  ["makedtx"] = {
    "texmf-dist/doc/support/makedtx/README.txt",
    "texmf-dist/doc/support/makedtx/makedtx.pdf",
  },
  ["texapi"] = {
    "texmf-dist/doc/generic/texapi/README",
    "texmf-dist/doc/generic/texapi/texapi-doc.pdf",
  },
  ["latexfileinfo-pkgs"] = {
    "texmf-dist/doc/latex/latexfileinfo-pkgs/README",
    "texmf-dist/doc/latex/latexfileinfo-pkgs/latexfileinfo_pkgs.htm",
  },
  ["biblatex-publist"] = {
    "texmf-dist/doc/latex/biblatex-publist/README",
    "texmf-dist/doc/latex/biblatex-publist/biblatex-publist.pdf",
  },
  ["adjmulticol"] = {
    "texmf-dist/doc/latex/adjmulticol/Makefile",
    "texmf-dist/doc/latex/adjmulticol/README",
    "texmf-dist/doc/latex/adjmulticol/adjmulticol.pdf",
    "texmf-dist/doc/latex/adjmulticol/sample.pdf",
  },
  ["cachepic"] = {
    "texmf-dist/doc/latex/cachepic/README",
    "texmf-dist/doc/latex/cachepic/cachepic.pdf",
  },
  ["initials"] = {
    "texmf-dist/doc/fonts/initials/README",
  },
  ["shorttoc"] = {
    "texmf-dist/doc/latex/shorttoc/00readme",
    "texmf-dist/doc/latex/shorttoc/shorttoc.pdf",
  },
  ["context-notes-zh-cn"] = {
    "texmf-dist/doc/context/third/context-notes-zh-cn/README",
    "texmf-dist/doc/context/third/context-notes-zh-cn/ctxnotes.pdf",
    "texmf-dist/doc/context/third/context-notes-zh-cn/src/Makefile",
    "texmf-dist/doc/context/third/context-notes-zh-cn/src/figures/cow.pdf",
  },
  ["glossaries-magyar"] = {
    "texmf-dist/doc/latex/glossaries-magyar/README",
    "texmf-dist/doc/latex/glossaries-magyar/glossaries-magyar.pdf",
  },
  ["ifxetex"] = {
    "texmf-dist/doc/generic/ifxetex/README",
    "texmf-dist/doc/generic/ifxetex/ifxetex.pdf",
  },
  ["biblatex-bwl"] = {
    "texmf-dist/doc/latex/biblatex-bwl/Changes",
    "texmf-dist/doc/latex/biblatex-bwl/doc/bwl-FU.pdf",
  },
  ["datetime2-english"] = {
    "texmf-dist/doc/latex/datetime2-english/CHANGES",
    "texmf-dist/doc/latex/datetime2-english/README",
    "texmf-dist/doc/latex/datetime2-english/datetime2-english-sample.pdf",
    "texmf-dist/doc/latex/datetime2-english/datetime2-english.pdf",
  },
  ["algorithmicx"] = {
    "texmf-dist/doc/latex/algorithmicx/README",
    "texmf-dist/doc/latex/algorithmicx/algorithmicx.pdf",
  },
  ["datetime2-irish"] = {
    "texmf-dist/doc/latex/datetime2-irish/README",
    "texmf-dist/doc/latex/datetime2-irish/datetime2-irish.pdf",
  },
  ["l3kernel"] = {
    "texmf-dist/doc/latex/l3kernel/README.md",
    "texmf-dist/doc/latex/l3kernel/expl3.pdf",
    "texmf-dist/doc/latex/l3kernel/interface3.pdf",
    "texmf-dist/doc/latex/l3kernel/l3docstrip.pdf",
    "texmf-dist/doc/latex/l3kernel/l3obsolete.txt",
    "texmf-dist/doc/latex/l3kernel/l3styleguide.pdf",
    "texmf-dist/doc/latex/l3kernel/l3syntax-changes.pdf",
    "texmf-dist/doc/latex/l3kernel/source3.pdf",
  },
  ["emptypage"] = {
    "texmf-dist/doc/latex/emptypage/README",
    "texmf-dist/doc/latex/emptypage/emptypage.pdf",
  },
  ["longfigure"] = {
    "texmf-dist/doc/latex/longfigure/README",
    "texmf-dist/doc/latex/longfigure/longfigure.pdf",
  },
  ["templates-fenn"] = {
    "texmf-dist/doc/latex/templates-fenn/templates-fenn-de.txt",
    "texmf-dist/doc/latex/templates-fenn/templates-fenn-en.txt",
  },
  ["poemscol"] = {
    "texmf-dist/doc/latex/poemscol/README",
    "texmf-dist/doc/latex/poemscol/poemscol.pdf",
    "texmf-dist/doc/latex/poemscol/poemscolcheatsheet.pdf",
  },
  ["juraabbrev"] = {
    "texmf-dist/doc/latex/juraabbrev/README",
    "texmf-dist/doc/latex/juraabbrev/juraabbrev.pdf",
  },
  ["bhcexam"] = {
    "texmf-dist/doc/latex/bhcexam/Makefile",
    "texmf-dist/doc/latex/bhcexam/README",
  },
  ["leading"] = {
    "texmf-dist/doc/latex/leading/README",
    "texmf-dist/doc/latex/leading/leading.pdf",
  },
  ["expdlist"] = {
    "texmf-dist/doc/latex/expdlist/expdlisg.pdf",
    "texmf-dist/doc/latex/expdlist/expdlist.pdf",
    "texmf-dist/doc/latex/expdlist/readme.txt",
  },
  ["makecell"] = {
    "texmf-dist/doc/latex/makecell/README",
    "texmf-dist/doc/latex/makecell/makecell-rus.pdf",
    "texmf-dist/doc/latex/makecell/makecell.pdf",
  },
  ["pgf-umlcd"] = {
    "texmf-dist/doc/latex/pgf-umlcd/COPYING",
    "texmf-dist/doc/latex/pgf-umlcd/README",
    "texmf-dist/doc/latex/pgf-umlcd/pgf-umlcd-manual.pdf",
  },
  ["etextools"] = {
    "texmf-dist/doc/latex/etextools/README",
    "texmf-dist/doc/latex/etextools/etextools-examples.pdf",
    "texmf-dist/doc/latex/etextools/etextools.pdf",
  },
  ["papertex"] = {
    "texmf-dist/doc/latex/papertex/CHANGES",
    "texmf-dist/doc/latex/papertex/README",
    "texmf-dist/doc/latex/papertex/example/example.pdf",
    "texmf-dist/doc/latex/papertex/papertex.pdf",
  },
  ["manyind"] = {
    "texmf-dist/doc/latex/manyind/README.md",
    "texmf-dist/doc/latex/manyind/mind.html",
    "texmf-dist/doc/latex/manyind/mindsample.pdf",
  },
  ["labbook"] = {
    "texmf-dist/doc/latex/labbook/README",
    "texmf-dist/doc/latex/labbook/labboode.pdf",
    "texmf-dist/doc/latex/labbook/labbook.pdf",
  },
  ["coelacanth"] = {
    "texmf-dist/doc/fonts/coelacanth/README",
    "texmf-dist/doc/fonts/coelacanth/coelacanth-samples.pdf",
  },
  ["histogr"] = {
    "texmf-dist/doc/latex/histogr/histogr.pdf",
  },
  ["prosper"] = {
    "texmf-dist/doc/latex/prosper/AUTHORS",
    "texmf-dist/doc/latex/prosper/ChangeLog",
    "texmf-dist/doc/latex/prosper/FAQ",
    "texmf-dist/doc/latex/prosper/INSTALL",
    "texmf-dist/doc/latex/prosper/NEWS",
    "texmf-dist/doc/latex/prosper/README",
    "texmf-dist/doc/latex/prosper/TODO",
    "texmf-dist/doc/latex/prosper/manifest.txt",
    "texmf-dist/doc/latex/prosper/prosper-doc.pdf",
    "texmf-dist/doc/latex/prosper/prosper-tour.pdf",
  },
  ["gofonts"] = {
    "texmf-dist/doc/fonts/gofonts/License",
    "texmf-dist/doc/fonts/gofonts/README",
    "texmf-dist/doc/fonts/gofonts/go-samples.pdf",
    "texmf-dist/doc/fonts/gofonts/gofonts.pdf",
  },
  ["hepnames"] = {
    "texmf-dist/doc/latex/hepnames/ChangeLog",
    "texmf-dist/doc/latex/hepnames/Makefile",
    "texmf-dist/doc/latex/hepnames/README",
    "texmf-dist/doc/latex/hepnames/hepnames.pdf",
    "texmf-dist/doc/latex/hepnames/hepnicenames-it.pdf",
    "texmf-dist/doc/latex/hepnames/hepnicenames-rm.pdf",
    "texmf-dist/doc/latex/hepnames/heppennames-it.pdf",
    "texmf-dist/doc/latex/hepnames/heppennames-rm.pdf",
    "texmf-dist/doc/latex/hepnames/mkmacrotables",
  },
  ["makeglos"] = {
    "texmf-dist/doc/latex/makeglos/README",
    "texmf-dist/doc/latex/makeglos/makeglos.pdf",
  },
  ["texdoc"] = {
    "texmf-dist/doc/man/man1/texdoc.man1.pdf",
    "texmf-dist/doc/support/texdoc/COPYING",
    "texmf-dist/doc/support/texdoc/NEWS",
    "texmf-dist/doc/support/texdoc/README.md",
    "texmf-dist/doc/support/texdoc/texdoc.pdf",
  },
  ["fancyref"] = {
    "texmf-dist/doc/latex/fancyref/COPYING",
    "texmf-dist/doc/latex/fancyref/README",
    "texmf-dist/doc/latex/fancyref/fancyref.pdf",
  },
  ["tabto-ltx"] = {
    "texmf-dist/doc/latex/tabto-ltx/tabto-doc.pdf",
  },
  ["fira"] = {
    "texmf-dist/doc/fonts/fira/Fira_4_2_Version_Log.pdf",
    "texmf-dist/doc/fonts/fira/LICENSE",
    "texmf-dist/doc/fonts/fira/README",
    "texmf-dist/doc/fonts/fira/TechnicalReportFiraSans4203.pdf",
    "texmf-dist/doc/fonts/fira/fira-samples.pdf",
  },
  ["endnotesj"] = {
    "texmf-dist/doc/latex/endnotesj/LICENSE",
    "texmf-dist/doc/latex/endnotesj/Makefile",
    "texmf-dist/doc/latex/endnotesj/README.md",
    "texmf-dist/doc/latex/endnotesj/endnotesj-ja.pdf",
  },
  ["sepnum"] = {
    "texmf-dist/doc/latex/sepnum/sepnum-doc.pdf",
  },
  ["latexcheat-de"] = {
    "texmf-dist/doc/latex/latexcheat-de/README",
    "texmf-dist/doc/latex/latexcheat-de/latexsheet-de.pdf",
  },
  ["fibeamer"] = {
    "texmf-dist/doc/latex/fibeamer/fibeamer.pdf",
    "texmf-dist/doc/latex/fibeamer/guide/mu/econ.pdf",
    "texmf-dist/doc/latex/fibeamer/guide/mu/fi.pdf",
    "texmf-dist/doc/latex/fibeamer/guide/mu/fsps.pdf",
    "texmf-dist/doc/latex/fibeamer/guide/mu/fss.pdf",
    "texmf-dist/doc/latex/fibeamer/guide/mu/law.pdf",
    "texmf-dist/doc/latex/fibeamer/guide/mu/med.pdf",
    "texmf-dist/doc/latex/fibeamer/guide/mu/ped.pdf",
    "texmf-dist/doc/latex/fibeamer/guide/mu/phil.pdf",
    "texmf-dist/doc/latex/fibeamer/guide/mu/sci.pdf",
  },
  ["bondgraph"] = {
    "texmf-dist/doc/latex/bondgraph/License.txt",
    "texmf-dist/doc/latex/bondgraph/README",
    "texmf-dist/doc/latex/bondgraph/bondgraph_example.pdf",
  },
  ["lfb"] = {
    "texmf-dist/doc/fonts/lfb/README",
    "texmf-dist/doc/fonts/lfb/example.pdf",
  },
  ["knittingpattern"] = {
    "texmf-dist/doc/latex/knittingpattern/README",
    "texmf-dist/doc/latex/knittingpattern/introduction.pdf",
    "texmf-dist/doc/latex/knittingpattern/template.pdf",
  },
  ["oubraces"] = {
    "texmf-dist/doc/latex/oubraces/oubraces.pdf",
  },
  ["flagderiv"] = {
    "texmf-dist/doc/latex/flagderiv/README",
    "texmf-dist/doc/latex/flagderiv/flagderiv.pdf",
  },
  ["cmbright"] = {
    "texmf-dist/doc/fonts/cmbright/LICENSE",
    "texmf-dist/doc/fonts/cmbright/README",
    "texmf-dist/doc/fonts/cmbright/cmbright.txt",
    "texmf-dist/doc/fonts/cmbright/manifest.txt",
    "texmf-dist/doc/latex/cmbright/cmbright.pdf",
  },
  ["uestcthesis"] = {
    "texmf-dist/doc/latex/uestcthesis/README",
    "texmf-dist/doc/latex/uestcthesis/doc/uestcthesis-doc.pdf",
  },
  ["gmverb"] = {
    "texmf-dist/doc/latex/gmverb/README",
    "texmf-dist/doc/latex/gmverb/gmverb.pdf",
  },
  ["dickimaw"] = {
    "texmf-dist/doc/latex/dickimaw/ERRATA",
    "texmf-dist/doc/latex/dickimaw/README",
    "texmf-dist/doc/latex/dickimaw/dickimaw-minexample.pdf",
    "texmf-dist/doc/latex/dickimaw/dickimaw-novices.pdf",
    "texmf-dist/doc/latex/dickimaw/dickimaw-thesis.pdf",
    "texmf-dist/doc/latex/dickimaw/src/novices/pictures/circle.pdf",
    "texmf-dist/doc/latex/dickimaw/src/novices/pictures/rectangle.pdf",
    "texmf-dist/doc/latex/dickimaw/src/novices/pictures/shapes.pdf",
    "texmf-dist/doc/latex/dickimaw/src/thesis/pictures/titlepage.pdf",
  },
  ["brushscr"] = {
    "texmf-dist/doc/fonts/brushscr/Makefile",
    "texmf-dist/doc/fonts/brushscr/README",
    "texmf-dist/doc/fonts/brushscr/kern.txt",
  },
  ["fontname"] = {
    "texmf-dist/doc/fonts/fontname/8a.html",
    "texmf-dist/doc/fonts/fontname/8r.html",
    "texmf-dist/doc/fonts/fontname/Adobe-fonts.html",
    "texmf-dist/doc/fonts/fontname/Apple-fonts.html",
    "texmf-dist/doc/fonts/fontname/Bitstream-fonts.html",
    "texmf-dist/doc/fonts/fontname/ChangeLog",
    "texmf-dist/doc/fonts/fontname/DTC-fonts.html",
    "texmf-dist/doc/fonts/fontname/Encodings.html",
    "texmf-dist/doc/fonts/fontname/Filenames-for-fonts.html",
    "texmf-dist/doc/fonts/fontname/Font-legalities.html",
    "texmf-dist/doc/fonts/fontname/Font-name-lists.html",
    "texmf-dist/doc/fonts/fontname/History.html",
    "texmf-dist/doc/fonts/fontname/ITC-fonts.html",
    "texmf-dist/doc/fonts/fontname/Introduction.html",
    "texmf-dist/doc/fonts/fontname/Linotype-fonts.html",
    "texmf-dist/doc/fonts/fontname/Long-names.html",
    "texmf-dist/doc/fonts/fontname/Long-naming-scheme.html",
    "texmf-dist/doc/fonts/fontname/Makefile",
    "texmf-dist/doc/fonts/fontname/Monotype-fonts.html",
    "texmf-dist/doc/fonts/fontname/Name-mapping-file.html",
    "texmf-dist/doc/fonts/fontname/References.html",
    "texmf-dist/doc/fonts/fontname/Standard-PostScript-fonts.html",
    "texmf-dist/doc/fonts/fontname/Suppliers.html",
    "texmf-dist/doc/fonts/fontname/Typefaces.html",
    "texmf-dist/doc/fonts/fontname/URW-fonts.html",
    "texmf-dist/doc/fonts/fontname/Variants.html",
    "texmf-dist/doc/fonts/fontname/Weights.html",
    "texmf-dist/doc/fonts/fontname/Widths.html",
    "texmf-dist/doc/fonts/fontname/cork.html",
    "texmf-dist/doc/fonts/fontname/dvips.html",
    "texmf-dist/doc/fonts/fontname/fontname.html",
    "texmf-dist/doc/fonts/fontname/fontname.pdf",
    "texmf-dist/doc/fonts/fontname/index.html",
    "texmf-dist/doc/fonts/fontname/texmext.html",
    "texmf-dist/doc/fonts/fontname/texmital.html",
    "texmf-dist/doc/fonts/fontname/texmsym.html",
    "texmf-dist/doc/fonts/fontname/texnansi.html",
    "texmf-dist/doc/fonts/fontname/texnansx.html",
    "texmf-dist/doc/fonts/fontname/xl2.html",
    "texmf-dist/doc/fonts/fontname/xt2.html",
  },
  ["besjournals"] = {
    "texmf-dist/doc/bibtex/besjournals/README",
  },
  ["cmexb"] = {
    "texmf-dist/doc/fonts/cmexb/README-cmexb",
  },
  ["actuarialangle"] = {
    "texmf-dist/doc/latex/actuarialangle/README.md",
    "texmf-dist/doc/latex/actuarialangle/actuarialangle.pdf",
  },
  ["gfsartemisia"] = {
    "texmf-dist/doc/fonts/gfsartemisia/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gfsartemisia/OFL.txt",
    "texmf-dist/doc/fonts/gfsartemisia/README",
    "texmf-dist/doc/fonts/gfsartemisia/README.TEXLIVE",
    "texmf-dist/doc/fonts/gfsartemisia/gfsartemisia.pdf",
  },
  ["context-inifile"] = {
    "texmf-dist/doc/context/third/inifile/VERSION",
    "texmf-dist/doc/context/third/inifile/inifile-demo.pdf",
    "texmf-dist/doc/context/third/inifile/inifile-doc.pdf",
  },
  ["ipaex-type1"] = {
    "texmf-dist/doc/fonts/ipaex-type1/LICENSE",
    "texmf-dist/doc/fonts/ipaex-type1/README.md",
    "texmf-dist/doc/fonts/ipaex-type1/sample-ipaex-type1.pdf",
  },
  ["newpx"] = {
    "texmf-dist/doc/fonts/newpx/MANIFEST-newpx.txt",
    "texmf-dist/doc/fonts/newpx/README",
    "texmf-dist/doc/fonts/newpx/newpxdoc.pdf",
    "texmf-dist/doc/fonts/newpx/newpxeg-crop.pdf",
    "texmf-dist/doc/fonts/newpx/pxfontseg-crop.pdf",
  },
  ["imfellenglish"] = {
    "texmf-dist/doc/fonts/imfellenglish/COPYING",
    "texmf-dist/doc/fonts/imfellenglish/README",
    "texmf-dist/doc/fonts/imfellenglish/imfellEnglish.pdf",
  },
  ["boxedminipage2e"] = {
    "texmf-dist/doc/latex/boxedminipage2e/README",
    "texmf-dist/doc/latex/boxedminipage2e/boxedminipage2e.pdf",
  },
  ["beamer-verona"] = {
    "texmf-dist/doc/latex/beamer-verona/README.md",
    "texmf-dist/doc/latex/beamer-verona/beamer-verona-default.pdf",
    "texmf-dist/doc/latex/beamer-verona/beamer-verona-sidebar.pdf",
    "texmf-dist/doc/latex/beamer-verona/beamer-verona.pdf",
  },
  ["breakcites"] = {
    "texmf-dist/doc/latex/breakcites/README",
    "texmf-dist/doc/latex/breakcites/breakcites.pdf",
  },
  ["ptptex"] = {
    "texmf-dist/doc/latex/ptptex/README",
    "texmf-dist/doc/latex/ptptex/README.TEXLIVE",
    "texmf-dist/doc/latex/ptptex/manptp.pdf",
  },
  ["makecmds"] = {
    "texmf-dist/doc/latex/makecmds/README",
    "texmf-dist/doc/latex/makecmds/makecmds.pdf",
  },
  ["custom-bib"] = {
    "texmf-dist/doc/latex/custom-bib/README",
    "texmf-dist/doc/latex/custom-bib/makebst.pdf",
    "texmf-dist/doc/latex/custom-bib/merlin.pdf",
    "texmf-dist/doc/latex/custom-bib/readme.mkb",
  },
  ["textpos"] = {
    "texmf-dist/doc/latex/textpos/LICENCE",
    "texmf-dist/doc/latex/textpos/README",
    "texmf-dist/doc/latex/textpos/examples/README.examples",
    "texmf-dist/doc/latex/textpos/niepraschk-eso-pic.pdf",
    "texmf-dist/doc/latex/textpos/textpos.html",
    "texmf-dist/doc/latex/textpos/textpos.pdf",
  },
  ["kdgdocs"] = {
    "texmf-dist/doc/latex/kdgdocs/LICENSE",
    "texmf-dist/doc/latex/kdgdocs/README",
    "texmf-dist/doc/latex/kdgdocs/kdg_color.pdf",
    "texmf-dist/doc/latex/kdgdocs/kdgcoursetext-example.pdf",
    "texmf-dist/doc/latex/kdgdocs/kdgdocs.pdf",
    "texmf-dist/doc/latex/kdgdocs/kdgmasterthesis-example.pdf",
    "texmf-dist/doc/latex/kdgdocs/manifest.txt",
  },
  ["featpost"] = {
    "texmf-dist/doc/metapost/featpost/MPexamples.html",
    "texmf-dist/doc/metapost/featpost/README",
    "texmf-dist/doc/metapost/featpost/doc/Exemplifier.pdf",
    "texmf-dist/doc/metapost/featpost/doc/alldefs.txt",
    "texmf-dist/doc/metapost/featpost/doc/featpostbeamer.pdf",
    "texmf-dist/doc/metapost/featpost/doc/featpostmanual.pdf",
    "texmf-dist/doc/metapost/featpost/doc/lastfiguretest.pdf",
    "texmf-dist/doc/metapost/featpost/doc/letterfor3Dand4D.pdf",
    "texmf-dist/doc/metapost/featpost/doc/metapostpropaganda.pdf",
    "texmf-dist/doc/metapost/featpost/doc/metapostpropagandafiles/minimal-1.pdf",
    "texmf-dist/doc/metapost/featpost/doc/otherinfo/BezierApprox.pdf",
    "texmf-dist/doc/metapost/featpost/doc/otherinfo/CHANGESv1.803.txt",
    "texmf-dist/doc/metapost/featpost/doc/otherinfo/Zpravodaj2009-4.pdf",
    "texmf-dist/doc/metapost/featpost/doc/otherinfo/tb88henderson.pdf",
  },
  ["iwhdp"] = {
    "texmf-dist/doc/latex/iwhdp/Back_2015.pdf",
    "texmf-dist/doc/latex/iwhdp/Deckblatt_2015.pdf",
    "texmf-dist/doc/latex/iwhdp/README",
    "texmf-dist/doc/latex/iwhdp/iwhdp_Manual.pdf",
  },
  ["pst-barcode"] = {
    "texmf-dist/doc/generic/pst-barcode/Changes",
    "texmf-dist/doc/generic/pst-barcode/README.md",
    "texmf-dist/doc/generic/pst-barcode/pst-barcode-doc.pdf",
  },
  ["ncclatex"] = {
    "texmf-dist/doc/latex/ncclatex/README",
    "texmf-dist/doc/latex/ncclatex/changes.txt",
    "texmf-dist/doc/latex/ncclatex/manifest.txt",
    "texmf-dist/doc/latex/ncclatex/ncclatex.pdf",
    "texmf-dist/doc/latex/ncclatex/nccnews.pdf",
  },
  ["memoirchapstyles"] = {
    "texmf-dist/doc/latex/MemoirChapStyles/MemoirChapStyles.pdf",
    "texmf-dist/doc/latex/MemoirChapStyles/README",
  },
  ["zxjafbfont"] = {
    "texmf-dist/doc/latex/zxjafbfont/LICENSE",
    "texmf-dist/doc/latex/zxjafbfont/README",
  },
  ["abntex2"] = {
    "texmf-dist/doc/latex/abntex2/README",
    "texmf-dist/doc/latex/abntex2/abntex2.pdf",
    "texmf-dist/doc/latex/abntex2/abntex2cite-alf.pdf",
    "texmf-dist/doc/latex/abntex2/abntex2cite.pdf",
    "texmf-dist/doc/latex/abntex2/examples/abntex2-modelo-artigo.pdf",
    "texmf-dist/doc/latex/abntex2/examples/abntex2-modelo-glossarios.pdf",
    "texmf-dist/doc/latex/abntex2/examples/abntex2-modelo-img-grafico.pdf",
    "texmf-dist/doc/latex/abntex2/examples/abntex2-modelo-img-marca.pdf",
    "texmf-dist/doc/latex/abntex2/examples/abntex2-modelo-livro.pdf",
    "texmf-dist/doc/latex/abntex2/examples/abntex2-modelo-projeto-pesquisa.pdf",
    "texmf-dist/doc/latex/abntex2/examples/abntex2-modelo-relatorio-tecnico.pdf",
    "texmf-dist/doc/latex/abntex2/examples/abntex2-modelo-slides.pdf",
    "texmf-dist/doc/latex/abntex2/examples/abntex2-modelo-trabalho-academico.pdf",
  },
  ["przechlewski-book"] = {
    "texmf-dist/doc/latex/przechlewski-book/LICENSE",
    "texmf-dist/doc/latex/przechlewski-book/Makefile",
    "texmf-dist/doc/latex/przechlewski-book/README",
    "texmf-dist/doc/latex/przechlewski-book/README.pl",
    "texmf-dist/doc/latex/przechlewski-book/TAM-pl.pdf",
    "texmf-dist/doc/latex/przechlewski-book/wkmgr.html",
    "texmf-dist/doc/latex/przechlewski-book/wkmgr1.pdf",
    "texmf-dist/doc/latex/przechlewski-book/wkmgr2.pdf",
  },
  ["amsmath"] = {
    "texmf-dist/doc/latex/amsmath/README.md",
    "texmf-dist/doc/latex/amsmath/amsbsy.pdf",
    "texmf-dist/doc/latex/amsmath/amscd.pdf",
    "texmf-dist/doc/latex/amsmath/amsgen.pdf",
    "texmf-dist/doc/latex/amsmath/amsldoc.pdf",
    "texmf-dist/doc/latex/amsmath/amsmath.pdf",
    "texmf-dist/doc/latex/amsmath/amsopn.pdf",
    "texmf-dist/doc/latex/amsmath/amstext.pdf",
    "texmf-dist/doc/latex/amsmath/amsxtra.pdf",
    "texmf-dist/doc/latex/amsmath/changes.txt",
    "texmf-dist/doc/latex/amsmath/diffs-m.txt",
    "texmf-dist/doc/latex/amsmath/manifest.txt",
    "texmf-dist/doc/latex/amsmath/subeqn.pdf",
    "texmf-dist/doc/latex/amsmath/technote.pdf",
    "texmf-dist/doc/latex/amsmath/testmath.pdf",
  },
  ["bez123"] = {
    "texmf-dist/doc/latex/bez123/README",
    "texmf-dist/doc/latex/bez123/bez123.pdf",
  },
  ["lua-visual-debug"] = {
    "texmf-dist/doc/luatex/lua-visual-debug/README.doc",
    "texmf-dist/doc/luatex/lua-visual-debug/lvdebug-doc.pdf",
    "texmf-dist/doc/luatex/lua-visual-debug/sample-plain.pdf",
    "texmf-dist/doc/luatex/lua-visual-debug/sample.pdf",
  },
  ["foekfont"] = {
    "texmf-dist/doc/latex/foekfont/README",
    "texmf-dist/doc/latex/foekfont/foekfont.pdf",
  },
  ["muthesis"] = {
    "texmf-dist/doc/latex/muthesis/README",
  },
  ["g-brief"] = {
    "texmf-dist/doc/latex/g-brief/beispiel.pdf",
    "texmf-dist/doc/latex/g-brief/beispiel2.pdf",
    "texmf-dist/doc/latex/g-brief/g-brief.pdf",
  },
  ["fbb"] = {
    "texmf-dist/doc/fonts/fbb/OFL.txt",
    "texmf-dist/doc/fonts/fbb/README",
    "texmf-dist/doc/fonts/fbb/fbb-doc.pdf",
  },
  ["babel-belarusian"] = {
    "texmf-dist/doc/generic/babel-belarusian/README",
    "texmf-dist/doc/generic/babel-belarusian/belarusianb.pdf",
  },
  ["babel-danish"] = {
    "texmf-dist/doc/generic/babel-danish/danish.pdf",
  },
  ["adfsymbols"] = {
    "texmf-dist/doc/fonts/adfsymbols/COPYING",
    "texmf-dist/doc/fonts/adfsymbols/NOTICE",
    "texmf-dist/doc/fonts/adfsymbols/README",
    "texmf-dist/doc/fonts/adfsymbols/adfsymbols.pdf",
    "texmf-dist/doc/fonts/adfsymbols/manifest.txt",
  },
  ["lshort-italian"] = {
    "texmf-dist/doc/latex/lshort-italian/CHANGES",
    "texmf-dist/doc/latex/lshort-italian/MANIFEST",
    "texmf-dist/doc/latex/lshort-italian/Makefile",
    "texmf-dist/doc/latex/lshort-italian/README",
    "texmf-dist/doc/latex/lshort-italian/TRANSLATIONS",
    "texmf-dist/doc/latex/lshort-italian/itlshort.pdf",
  },
  ["biblatex-mla"] = {
    "texmf-dist/doc/latex/biblatex-mla/CHANGES",
    "texmf-dist/doc/latex/biblatex-mla/README",
    "texmf-dist/doc/latex/biblatex-mla/doc/biblatex-mla.pdf",
    "texmf-dist/doc/latex/biblatex-mla/doc/citation-examples-new.pdf",
    "texmf-dist/doc/latex/biblatex-mla/doc/citation-examples.pdf",
    "texmf-dist/doc/latex/biblatex-mla/doc/handbooksamplebib-new.pdf",
    "texmf-dist/doc/latex/biblatex-mla/doc/handbooksamplebib.pdf",
  },
  ["gfnotation"] = {
    "texmf-dist/doc/plain/gfnotation/COPYING",
    "texmf-dist/doc/plain/gfnotation/GFnotation-doc.pdf",
    "texmf-dist/doc/plain/gfnotation/README",
  },
  ["tlcockpit"] = {
    "texmf-dist/doc/support/tlcockpit/LICENSE",
    "texmf-dist/doc/support/tlcockpit/NEWS",
    "texmf-dist/doc/support/tlcockpit/README.md",
  },
  ["autobreak"] = {
    "texmf-dist/doc/latex/autobreak/README.md",
    "texmf-dist/doc/latex/autobreak/autobreak.pdf",
  },
  ["pxrubrica"] = {
    "texmf-dist/doc/platex/pxrubrica/LICENSE",
    "texmf-dist/doc/platex/pxrubrica/README.md",
    "texmf-dist/doc/platex/pxrubrica/pxrubrica-en.pdf",
    "texmf-dist/doc/platex/pxrubrica/pxrubrica.pdf",
    "texmf-dist/doc/platex/pxrubrica/sample/test-jlreq.pdf",
    "texmf-dist/doc/platex/pxrubrica/sample/test-sample.pdf",
    "texmf-dist/doc/platex/pxrubrica/sample/test-sf.pdf",
    "texmf-dist/doc/platex/pxrubrica/sample/test-toc.pdf",
  },
  ["c-pascal"] = {
    "texmf-dist/doc/generic/c-pascal/README.eng",
    "texmf-dist/doc/generic/c-pascal/README.pol",
  },
  ["adtrees"] = {
    "texmf-dist/doc/latex/adtrees/README",
    "texmf-dist/doc/latex/adtrees/adtreesdoc.pdf",
  },
  ["latex4wp-it"] = {
    "texmf-dist/doc/latex/latex4wp-it/COME-COMPORRE",
    "texmf-dist/doc/latex/latex4wp-it/README",
    "texmf-dist/doc/latex/latex4wp-it/dat2tex",
    "texmf-dist/doc/latex/latex4wp-it/gnuplot.pdf",
    "texmf-dist/doc/latex/latex4wp-it/latex4wp-it.pdf",
    "texmf-dist/doc/latex/latex4wp-it/tbx.pdf",
  },
  ["navydocs"] = {
    "texmf-dist/doc/latex/navydocs/Makefile",
    "texmf-dist/doc/latex/navydocs/README.txt",
    "texmf-dist/doc/latex/navydocs/navydocs.pdf",
  },
  ["babel-friulan"] = {
    "texmf-dist/doc/generic/babel-friulan/README.txt",
    "texmf-dist/doc/generic/babel-friulan/friulan.pdf",
  },
  ["newenviron"] = {
    "texmf-dist/doc/latex/newenviron/README",
  },
  ["curves"] = {
    "texmf-dist/doc/latex/curves/README",
    "texmf-dist/doc/latex/curves/curves.pdf",
    "texmf-dist/doc/latex/curves/latex2pdf",
  },
  ["gfscomplutum"] = {
    "texmf-dist/doc/fonts/gfscomplutum/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gfscomplutum/OFL.txt",
    "texmf-dist/doc/fonts/gfscomplutum/README",
    "texmf-dist/doc/fonts/gfscomplutum/README.TEXLIVE",
    "texmf-dist/doc/fonts/gfscomplutum/gfscomplutum.pdf",
  },
  ["bibtopicprefix"] = {
    "texmf-dist/doc/latex/bibtopicprefix/README",
    "texmf-dist/doc/latex/bibtopicprefix/bibtopicprefix.pdf",
  },
  ["fontmfizz"] = {
    "texmf-dist/doc/fonts/fontmfizz/LICENSE",
    "texmf-dist/doc/fonts/fontmfizz/README",
    "texmf-dist/doc/fonts/fontmfizz/fontmfizz.pdf",
  },
  ["wrapfig"] = {
    "texmf-dist/doc/latex/wrapfig/multiple-span.txt",
    "texmf-dist/doc/latex/wrapfig/wrapfig-doc.pdf",
  },
  ["pst-poker"] = {
    "texmf-dist/doc/latex/pst-poker/Changes",
    "texmf-dist/doc/latex/pst-poker/README",
    "texmf-dist/doc/latex/pst-poker/pst-poker-doc.pdf",
  },
  ["charter"] = {
    "texmf-dist/doc/fonts/charter/readme.charter",
  },
  ["dtxdescribe"] = {
    "texmf-dist/doc/latex/dtxdescribe/README.txt",
    "texmf-dist/doc/latex/dtxdescribe/dtxdescribe.pdf",
  },
  ["eskd"] = {
    "texmf-dist/doc/latex/eskd/README",
    "texmf-dist/doc/latex/eskd/eskd.pdf",
  },
  ["setdeck"] = {
    "texmf-dist/doc/latex/setdeck/README",
    "texmf-dist/doc/latex/setdeck/setdeck.pdf",
  },
  ["gfsneohellenicmath"] = {
    "texmf-dist/doc/fonts/gfsneohellenicmath/MathematicsCheatSheet.pdf",
    "texmf-dist/doc/fonts/gfsneohellenicmath/README",
  },
  ["l2tabu-french"] = {
    "texmf-dist/doc/latex/l2tabu-french/README",
    "texmf-dist/doc/latex/l2tabu-french/l2tabufr.pdf",
  },
  ["lshort-portuguese"] = {
    "texmf-dist/doc/latex/lshort-portuguese/pt-lshort.pdf",
  },
  ["fancynum"] = {
    "texmf-dist/doc/latex/fancynum/CHANGES",
    "texmf-dist/doc/latex/fancynum/README",
    "texmf-dist/doc/latex/fancynum/aue.txt",
    "texmf-dist/doc/latex/fancynum/ctt.txt",
    "texmf-dist/doc/latex/fancynum/fancynum.pdf",
  },
  ["ifmslide"] = {
    "texmf-dist/doc/latex/ifmslide/README",
    "texmf-dist/doc/latex/ifmslide/genbutton",
    "texmf-dist/doc/latex/ifmslide/ifmman.pdf",
  },
  ["unswcover"] = {
    "texmf-dist/doc/latex/unswcover/COPYING",
    "texmf-dist/doc/latex/unswcover/Makefile",
    "texmf-dist/doc/latex/unswcover/README",
    "texmf-dist/doc/latex/unswcover/logo_unsw_short.pdf",
    "texmf-dist/doc/latex/unswcover/thesis.pdf",
  },
  ["lua2dox"] = {
    "texmf-dist/doc/support/lua2dox/ChangeLog",
    "texmf-dist/doc/support/lua2dox/Doxyfile",
    "texmf-dist/doc/support/lua2dox/README",
    "texmf-dist/doc/support/lua2dox/docs/html/annotated.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTApp-members.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTApp.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTCore__Clock-members.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTCore__Clock.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTCore__Commandline-members.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTCore__Commandline.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTCore__IO.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTLua2DoX__filter-members.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTLua2DoX__filter.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTStream__Read-members.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTStream__Read.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTStream__Write-members.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classTStream__Write.html",
    "texmf-dist/doc/support/lua2dox/docs/html/classes.html",
    "texmf-dist/doc/support/lua2dox/docs/html/files.html",
    "texmf-dist/doc/support/lua2dox/docs/html/functions.html",
    "texmf-dist/doc/support/lua2dox/docs/html/functions_func.html",
    "texmf-dist/doc/support/lua2dox/docs/html/globals.html",
    "texmf-dist/doc/support/lua2dox/docs/html/globals_func.html",
    "texmf-dist/doc/support/lua2dox/docs/html/index.html",
    "texmf-dist/doc/support/lua2dox/docs/html/lua2dox_8lua.html",
    "texmf-dist/doc/support/lua2dox/docs/html/lua2dox_8lua_source.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/Doxyfile",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/animals_8lua.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/animals_8lua_source.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/annotated.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classAnimal-members.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classAnimal.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classBird-members.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classBird.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classCat-members.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classCat.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classDog-members.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classDog.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classMammal-members.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classMammal.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classPigeon-members.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classPigeon.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classRedKite-members.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classRedKite.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/class_8lua.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/class_8lua_source.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/classes.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/files.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/functions.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/functions_func.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/globals.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/globals_func.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/hierarchy.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/index.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/main_8lua.html",
    "texmf-dist/doc/support/lua2dox/examples/lua/docs/html/main_8lua_source.html",
    "texmf-dist/doc/support/lua2dox/examples/php/Doxyfile",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/animals_8php.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/animals_8php_source.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/annotated.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classAnimal-members.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classAnimal.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classBird-members.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classBird.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classCat-members.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classCat.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classDog-members.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classDog.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classMammal-members.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classMammal.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classPigeon-members.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classPigeon.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classRedKite-members.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classRedKite.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/classes.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/files.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/functions.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/functions_func.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/globals.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/globals_func.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/globals_vars.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/hierarchy.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/index.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/main_8php.html",
    "texmf-dist/doc/support/lua2dox/examples/php/docs/html/main_8php_source.html",
    "texmf-dist/doc/support/lua2dox/lua2dox-refm.pdf",
  },
  ["cookingsymbols"] = {
    "texmf-dist/doc/latex/cookingsymbols/README",
    "texmf-dist/doc/latex/cookingsymbols/cookingsymbols.pdf",
  },
  ["exam-n"] = {
    "texmf-dist/doc/latex/exam-n/README",
    "texmf-dist/doc/latex/exam-n/README.ctan",
    "texmf-dist/doc/latex/exam-n/exam-n.html",
    "texmf-dist/doc/latex/exam-n/exam-n.pdf",
    "texmf-dist/doc/latex/exam-n/lppl.txt",
    "texmf-dist/doc/latex/exam-n/notes-for-authors.pdf",
    "texmf-dist/doc/latex/exam-n/release-notes.html",
    "texmf-dist/doc/latex/exam-n/sample/Makefile",
    "texmf-dist/doc/latex/exam-n/sample/fig/spiral.pdf",
    "texmf-dist/doc/latex/exam-n/sample_exam.pdf",
    "texmf-dist/doc/latex/exam-n/sample_exam_solution.pdf",
  },
  ["bashful"] = {
    "texmf-dist/doc/latex/bashful/Makefile",
    "texmf-dist/doc/latex/bashful/README",
    "texmf-dist/doc/latex/bashful/bashful.pdf",
  },
  ["verbments"] = {
    "texmf-dist/doc/latex/verbments/README",
    "texmf-dist/doc/latex/verbments/verbments.pdf",
  },
  ["unitn-bimrep"] = {
    "texmf-dist/doc/latex/unitn-bimrep/README.md",
    "texmf-dist/doc/latex/unitn-bimrep/README.pdf",
    "texmf-dist/doc/latex/unitn-bimrep/examples/example.pdf",
    "texmf-dist/doc/latex/unitn-bimrep/examples/example_bib.pdf",
  },
  ["zwpagelayout"] = {
    "texmf-dist/doc/latex/zwpagelayout/License.txt",
    "texmf-dist/doc/latex/zwpagelayout/README",
    "texmf-dist/doc/latex/zwpagelayout/adjustfoot.pdf",
    "texmf-dist/doc/latex/zwpagelayout/adjusthead.pdf",
    "texmf-dist/doc/latex/zwpagelayout/coversample.pdf",
    "texmf-dist/doc/latex/zwpagelayout/zwpagelayout.pdf",
  },
  ["addfont"] = {
    "texmf-dist/doc/latex/addfont/README",
    "texmf-dist/doc/latex/addfont/addfont.pdf",
    "texmf-dist/doc/latex/addfont/license",
  },
  ["texshade"] = {
    "texmf-dist/doc/latex/texshade/README",
    "texmf-dist/doc/latex/texshade/texshade.pdf",
    "texmf-dist/doc/latex/texshade/tsfaq.pdf",
  },
  ["uwmslide"] = {
    "texmf-dist/doc/latex/uwmslide/README",
  },
  ["quattrocento"] = {
    "texmf-dist/doc/fonts/quattrocento/OFL.txt",
    "texmf-dist/doc/fonts/quattrocento/README",
    "texmf-dist/doc/fonts/quattrocento/samples.pdf",
  },
  ["bartel-chess-fonts"] = {
    "texmf-dist/doc/fonts/bartel-chess-fonts/README",
  },
  ["bigfoot"] = {
    "texmf-dist/doc/latex/bigfoot/COPYING",
    "texmf-dist/doc/latex/bigfoot/Makefile",
    "texmf-dist/doc/latex/bigfoot/README",
    "texmf-dist/doc/latex/bigfoot/bigfoot.pdf",
    "texmf-dist/doc/latex/bigfoot/perpage.pdf",
    "texmf-dist/doc/latex/bigfoot/suffix.pdf",
  },
  ["gauss"] = {
    "texmf-dist/doc/latex/gauss/README",
    "texmf-dist/doc/latex/gauss/gauss-doc.pdf",
    "texmf-dist/doc/latex/gauss/gauss-ex.pdf",
  },
  ["mfpic4ode"] = {
    "texmf-dist/doc/latex/mfpic4ode/README",
    "texmf-dist/doc/latex/mfpic4ode/demo/demo-plain.pdf",
    "texmf-dist/doc/latex/mfpic4ode/demo/demo.pdf",
    "texmf-dist/doc/latex/mfpic4ode/mfpic4ode.pdf",
  },
  ["parselines"] = {
    "texmf-dist/doc/latex/parselines/README",
    "texmf-dist/doc/latex/parselines/parselines.pdf",
  },
  ["texlive-common"] = {
    "texmf-dist/doc/texlive/index.html",
  },
  ["lpform"] = {
    "texmf-dist/doc/generic/lpform/README",
    "texmf-dist/doc/generic/lpform/lpform-doc.pdf",
  },
  ["prooftrees"] = {
    "texmf-dist/doc/latex/prooftrees/README",
    "texmf-dist/doc/latex/prooftrees/manifest.txt",
    "texmf-dist/doc/latex/prooftrees/prooftrees.pdf",
  },
  ["gtl"] = {
    "texmf-dist/doc/generic/gtl/README.md",
    "texmf-dist/doc/generic/gtl/gtl.pdf",
  },
  ["hexgame"] = {
    "texmf-dist/doc/latex/hexgame/README",
    "texmf-dist/doc/latex/hexgame/hexgame.pdf",
  },
  ["thumby"] = {
    "texmf-dist/doc/latex/thumby/LICENSE",
    "texmf-dist/doc/latex/thumby/README",
    "texmf-dist/doc/latex/thumby/example.pdf",
    "texmf-dist/doc/latex/thumby/latexmkrc",
    "texmf-dist/doc/latex/thumby/thumby.pdf",
  },
  ["ltxfileinfo"] = {
    "texmf-dist/doc/support/ltxfileinfo/README",
    "texmf-dist/doc/support/ltxfileinfo/ltxfileinfo.pdf",
  },
  ["verbasef"] = {
    "texmf-dist/doc/latex/verbasef/verbasef-doc.pdf",
  },
  ["acronym"] = {
    "texmf-dist/doc/latex/acronym/README",
    "texmf-dist/doc/latex/acronym/acronym.pdf",
  },
  ["dtl"] = {
    "texmf-dist/doc/man/man1/dt2dv.man1.pdf",
    "texmf-dist/doc/man/man1/dv2dt.man1.pdf",
  },
  ["otibet"] = {
    "texmf-dist/doc/latex/otibet/README",
    "texmf-dist/doc/latex/otibet/allbasic.dvi",
    "texmf-dist/doc/latex/otibet/tiblatex.dvi",
    "texmf-dist/doc/latex/otibet/unidoc.dvi",
    "texmf-dist/doc/latex/otibet/yugpacan.dvi",
  },
  ["url"] = {
    "texmf-dist/doc/latex/url/url.pdf",
  },
  ["lshort-finnish"] = {
    "texmf-dist/doc/latex/lshort-finnish/README",
    "texmf-dist/doc/latex/lshort-finnish/lyhyt2e.pdf",
  },
  ["layaureo"] = {
    "texmf-dist/doc/latex/layaureo/README",
    "texmf-dist/doc/latex/layaureo/layaureo.pdf",
  },
  ["moderntimeline"] = {
    "texmf-dist/doc/latex/moderntimeline/README",
    "texmf-dist/doc/latex/moderntimeline/moderntimeline.pdf",
  },
  ["figbib"] = {
    "texmf-dist/doc/latex/figbib/README",
    "texmf-dist/doc/latex/figbib/figbib_doc.pdf",
    "texmf-dist/doc/latex/figbib/figbib_sample.pdf",
  },
  ["pst-2dplot"] = {
    "texmf-dist/doc/generic/pst-2dplot/README",
    "texmf-dist/doc/generic/pst-2dplot/pst-2dplot-doc.pdf",
  },
  ["ebsthesis"] = {
    "texmf-dist/doc/latex/ebsthesis/README",
    "texmf-dist/doc/latex/ebsthesis/ebsthesis.pdf",
  },
  ["seminar"] = {
    "texmf-dist/doc/latex/seminar/Changes",
    "texmf-dist/doc/latex/seminar/README",
    "texmf-dist/doc/latex/seminar/seminar-doc.pdf",
    "texmf-dist/doc/latex/seminar/semsamp1.pdf",
    "texmf-dist/doc/latex/seminar/semsamp2.pdf",
    "texmf-dist/doc/latex/seminar/semsamp3.pdf",
  },
  ["hacm"] = {
    "texmf-dist/doc/fonts/hacm/README",
    "texmf-dist/doc/fonts/hacm/hacmdoc.pdf",
  },
  ["lacheck"] = {
    "texmf-dist/doc/man/man1/lacheck.man1.pdf",
  },
  ["turabian-formatting"] = {
    "texmf-dist/doc/latex/turabian-formatting/README",
    "texmf-dist/doc/latex/turabian-formatting/turabian-formatting-doc.pdf",
  },
  ["hulipsum"] = {
    "texmf-dist/doc/latex/hulipsum/README",
    "texmf-dist/doc/latex/hulipsum/hulipsum.pdf",
  },
  ["shadethm"] = {
    "texmf-dist/doc/latex/shadethm/shadetest.pdf",
    "texmf-dist/doc/latex/shadethm/shadethm-doc.pdf",
  },
  ["fncychap"] = {
    "texmf-dist/doc/latex/fncychap/README",
    "texmf-dist/doc/latex/fncychap/fncychap.pdf",
    "texmf-dist/doc/latex/fncychap/manifest.txt",
  },
  ["parskip"] = {
    "texmf-dist/doc/latex/parskip/parskip-doc.pdf",
  },
  ["loops"] = {
    "texmf-dist/doc/latex/loops/README",
  },
  ["ellipsis"] = {
    "texmf-dist/doc/latex/ellipsis/README",
    "texmf-dist/doc/latex/ellipsis/ellipsis.pdf",
  },
  ["mychemistry"] = {
    "texmf-dist/doc/latex/mychemistry/README",
    "texmf-dist/doc/latex/mychemistry/examples.pdf",
    "texmf-dist/doc/latex/mychemistry/mychemistry_en.pdf",
  },
  ["abstract"] = {
    "texmf-dist/doc/latex/abstract/README",
    "texmf-dist/doc/latex/abstract/abstract.pdf",
  },
  ["dccpaper"] = {
    "texmf-dist/doc/latex/dccpaper/README.md",
    "texmf-dist/doc/latex/dccpaper/dccpaper.pdf",
  },
  ["e-french"] = {
    "texmf-dist/doc/generic/e-french/Probl_Msg.pdf",
    "texmf-dist/doc/generic/e-french/Probl_Msg.txt",
    "texmf-dist/doc/generic/e-french/README",
    "texmf-dist/doc/generic/e-french/README-TDS",
    "texmf-dist/doc/generic/e-french/bin/linux/makefrindex",
    "texmf-dist/doc/generic/e-french/changements.html",
    "texmf-dist/doc/generic/e-french/histoire.html",
    "texmf-dist/doc/generic/e-french/install/MiniDocEf_Lnx.pdf",
    "texmf-dist/doc/generic/e-french/install/MiniDocEf_Win.pdf",
    "texmf-dist/doc/generic/e-french/install/MiniDocMkfr.pdf",
    "texmf-dist/doc/generic/e-french/install/MiniDoc_Unx.txt",
    "texmf-dist/doc/generic/e-french/install/MiniDoc_Win.txt",
    "texmf-dist/doc/generic/e-french/install/Probl_Msg.pdf",
    "texmf-dist/doc/generic/e-french/install/Probl_Msg.txt",
    "texmf-dist/doc/generic/e-french/licence.html",
    "texmf-dist/doc/generic/e-french/lisez-moi.html",
    "texmf-dist/doc/generic/e-french/manuels/FrenchLe.html",
    "texmf-dist/doc/generic/e-french/manuels/FrenchPro.html",
    "texmf-dist/doc/generic/e-french/manuels/efrench.pdf",
    "texmf-dist/doc/generic/e-french/manuels/faq.pdf",
    "texmf-dist/doc/generic/e-french/manuels/french_doc.pdf",
    "texmf-dist/doc/generic/e-french/manuels/frenchle.pdf",
    "texmf-dist/doc/generic/e-french/src/french_doc.txt",
    "texmf-dist/doc/generic/e-french/test/README",
    "texmf-dist/doc/generic/e-french/test/frenchrf.pdf",
    "texmf-dist/doc/generic/e-french/test/tests-formats/README",
    "texmf-dist/doc/generic/e-french/test/tests-formats/ltesthyp",
    "texmf-dist/doc/generic/e-french/test/tests-formats/testhyp",
  },
  ["arev"] = {
    "texmf-dist/doc/fonts/arev/ArevSansLicense.txt",
    "texmf-dist/doc/fonts/arev/BitstreamVeraLicense.txt",
    "texmf-dist/doc/fonts/arev/ChangeLog",
    "texmf-dist/doc/fonts/arev/GPLv2.txt",
    "texmf-dist/doc/fonts/arev/LPPLv1-3a.txt",
    "texmf-dist/doc/fonts/arev/Makefile",
    "texmf-dist/doc/fonts/arev/README",
    "texmf-dist/doc/fonts/arev/arevdoc.pdf",
    "texmf-dist/doc/fonts/arev/mathtesty.pdf",
  },
  ["thinsp"] = {
    "texmf-dist/doc/latex/thinsp/COPYRIGHT",
    "texmf-dist/doc/latex/thinsp/README",
    "texmf-dist/doc/latex/thinsp/thinsp.pdf",
  },
  ["frontespizio"] = {
    "texmf-dist/doc/latex/frontespizio/README",
    "texmf-dist/doc/latex/frontespizio/examplea.pdf",
    "texmf-dist/doc/latex/frontespizio/exampleasuf.pdf",
    "texmf-dist/doc/latex/frontespizio/exampleb.pdf",
    "texmf-dist/doc/latex/frontespizio/examplec.pdf",
    "texmf-dist/doc/latex/frontespizio/exampled.pdf",
    "texmf-dist/doc/latex/frontespizio/examplee.pdf",
    "texmf-dist/doc/latex/frontespizio/fakelogo.pdf",
    "texmf-dist/doc/latex/frontespizio/frontespizio.pdf",
  },
  ["lexikon"] = {
    "texmf-dist/doc/latex/lexikon/lexikon-doc.pdf",
  },
  ["biblatex-manuscripts-philology"] = {
    "texmf-dist/doc/latex/biblatex-manuscripts-philology/README",
    "texmf-dist/doc/latex/biblatex-manuscripts-philology/documentation/biblatex-manuscripts-philology-example.pdf",
    "texmf-dist/doc/latex/biblatex-manuscripts-philology/documentation/biblatex-manuscripts-philology.pdf",
    "texmf-dist/doc/latex/biblatex-manuscripts-philology/documentation/makefile",
    "texmf-dist/doc/latex/biblatex-manuscripts-philology/makefile",
  },
  ["glossaries-dutch"] = {
    "texmf-dist/doc/latex/glossaries-dutch/README",
    "texmf-dist/doc/latex/glossaries-dutch/glossaries-dutch.pdf",
  },
  ["fancylabel"] = {
    "texmf-dist/doc/latex/fancylabel/README",
    "texmf-dist/doc/latex/fancylabel/fancylabel.pdf",
  },
  ["colorspace"] = {
    "texmf-dist/doc/latex/colorspace/README",
    "texmf-dist/doc/latex/colorspace/colorspace.pdf",
  },
  ["version"] = {
    "texmf-dist/doc/latex/version/version-doc.pdf",
  },
  ["hardwrap"] = {
    "texmf-dist/doc/latex/hardwrap/README",
    "texmf-dist/doc/latex/hardwrap/hardwrap.pdf",
  },
  ["excludeonly"] = {
    "texmf-dist/doc/latex/excludeonly/excludeonly.pdf",
  },
  ["isonums"] = {
    "texmf-dist/doc/latex/isonums/isonums.pdf",
  },
  ["duerer"] = {
    "texmf-dist/doc/fonts/duerer/README",
  },
  ["asaetr"] = {
    "texmf-dist/doc/latex/asaetr/MANIFEST",
    "texmf-dist/doc/latex/asaetr/asaetr.pdf",
  },
  ["proposal"] = {
    "texmf-dist/doc/latex/proposal/base/README",
    "texmf-dist/doc/latex/proposal/dfg/README",
    "texmf-dist/doc/latex/proposal/dfg/dfgproposal.pdf",
    "texmf-dist/doc/latex/proposal/dfg/proposal/Makefile",
    "texmf-dist/doc/latex/proposal/dfg/proposal/proposal.pdf",
    "texmf-dist/doc/latex/proposal/dfg/proposal/public.pdf",
    "texmf-dist/doc/latex/proposal/dfg/proposal/submit.pdf",
    "texmf-dist/doc/latex/proposal/dfg/report/Makefile",
    "texmf-dist/doc/latex/proposal/dfg/report/README",
    "texmf-dist/doc/latex/proposal/dfg/report/finalreport.pdf",
    "texmf-dist/doc/latex/proposal/dfg/report/letter_submission.pdf",
    "texmf-dist/doc/latex/proposal/eu/README",
    "texmf-dist/doc/latex/proposal/eu/euproposal.pdf",
    "texmf-dist/doc/latex/proposal/eu/strep/Makefile",
    "texmf-dist/doc/latex/proposal/eu/strep/README",
    "texmf-dist/doc/latex/proposal/eu/strep/propB.pdf",
    "texmf-dist/doc/latex/proposal/eu/strep/public.pdf",
    "texmf-dist/doc/latex/proposal/eu/strep/submit.pdf",
  },
  ["interactiveworkbook"] = {
    "texmf-dist/doc/latex/interactiveworkbook/documentation/interactiveworkbookmanual.pdf",
    "texmf-dist/doc/latex/interactiveworkbook/samplefiles/check.pdf",
    "texmf-dist/doc/latex/interactiveworkbook/samplefiles/field.pdf",
    "texmf-dist/doc/latex/interactiveworkbook/samplefiles/ndex.pdf",
    "texmf-dist/doc/latex/interactiveworkbook/samplefiles/popup.pdf",
    "texmf-dist/doc/latex/interactiveworkbook/samplefiles/radio.pdf",
  },
  ["underscore"] = {
    "texmf-dist/doc/latex/underscore/underscore.pdf",
  },
  ["smartunits"] = {
    "texmf-dist/doc/latex/smartunits/README.md",
    "texmf-dist/doc/latex/smartunits/smartunits.pdf",
  },
  ["capt-of"] = {
    "texmf-dist/doc/latex/capt-of/README",
    "texmf-dist/doc/latex/capt-of/capt-of.pdf",
  },
  ["librarian"] = {
    "texmf-dist/doc/generic/librarian/README",
    "texmf-dist/doc/generic/librarian/librarian-doc.pdf",
  },
  ["happy4th"] = {
    "texmf-dist/doc/plain/happy4th/happy4th.pdf",
  },
  ["alterqcm"] = {
    "texmf-dist/doc/latex/alterqcm/README",
    "texmf-dist/doc/latex/alterqcm/README.doc",
    "texmf-dist/doc/latex/alterqcm/doc_aq-screen.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/AntillesESjuin2006.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/alea.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/annexe.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/correct.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/doc_aq.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/example_1.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/example_2.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/language.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/points.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/sep.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/transparent-final.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/transparent-init.pdf",
    "texmf-dist/doc/latex/alterqcm/examples/pdf/verb.pdf",
  },
  ["dtxgen"] = {
    "texmf-dist/doc/support/dtxgen/README",
    "texmf-dist/doc/support/dtxgen/dtxgen.pdf",
  },
  ["gatherenum"] = {
    "texmf-dist/doc/latex/gatherenum/LICENSE",
    "texmf-dist/doc/latex/gatherenum/README.md",
    "texmf-dist/doc/latex/gatherenum/gatherenum.pdf",
  },
  ["listbib"] = {
    "texmf-dist/doc/latex/listbib/listbib-doc.pdf",
  },
  ["dantelogo"] = {
    "texmf-dist/doc/fonts/dantelogo/Changes",
    "texmf-dist/doc/fonts/dantelogo/README",
    "texmf-dist/doc/fonts/dantelogo/dantelogo-doc.pdf",
  },
  ["computational-complexity"] = {
    "texmf-dist/doc/latex/computational-complexity/README",
    "texmf-dist/doc/latex/computational-complexity/cc.pdf",
  },
  ["verbatimcopy"] = {
    "texmf-dist/doc/latex/verbatimcopy/README",
    "texmf-dist/doc/latex/verbatimcopy/verbatimcopy.pdf",
  },
  ["luainputenc"] = {
    "texmf-dist/doc/lualatex/luainputenc/NEWS",
    "texmf-dist/doc/lualatex/luainputenc/README",
    "texmf-dist/doc/lualatex/luainputenc/luainputenc.pdf",
  },
  ["gb4e"] = {
    "texmf-dist/doc/latex/gb4e/README",
    "texmf-dist/doc/latex/gb4e/gb4e-doc.pdf",
  },
  ["ctie"] = {
    "texmf-dist/doc/man/man1/ctie.man1.pdf",
  },
  ["pst-fill"] = {
    "texmf-dist/doc/generic/pst-fill/Changes",
    "texmf-dist/doc/generic/pst-fill/README",
    "texmf-dist/doc/generic/pst-fill/pst-fill.pdf",
  },
  ["concmath"] = {
    "texmf-dist/doc/fonts/concmath/CATALOGUE",
    "texmf-dist/doc/fonts/concmath/README",
    "texmf-dist/doc/fonts/concmath/concmath.pdf",
  },
  ["morehype"] = {
    "texmf-dist/doc/latex/morehype/README",
    "texmf-dist/doc/latex/morehype/README.pdf",
    "texmf-dist/doc/latex/morehype/RELEASEs.txt",
    "texmf-dist/doc/latex/morehype/SrcFILEs.txt",
    "texmf-dist/doc/latex/morehype/blog.pdf",
    "texmf-dist/doc/latex/morehype/blogdemo/hellowor/hellowor.htm",
    "texmf-dist/doc/latex/morehype/blogexec.pdf",
    "texmf-dist/doc/latex/morehype/hypertoc.pdf",
    "texmf-dist/doc/latex/morehype/markblog.htm",
    "texmf-dist/doc/latex/morehype/texlinks.pdf",
    "texmf-dist/doc/latex/morehype/wiki_mwe.pdf",
  },
  ["greek-inputenc"] = {
    "texmf-dist/doc/latex/greek-inputenc/README",
    "texmf-dist/doc/latex/greek-inputenc/README.html",
    "texmf-dist/doc/latex/greek-inputenc/greek-utf8.pdf",
    "texmf-dist/doc/latex/greek-inputenc/lgrenc.dfu.html",
    "texmf-dist/doc/latex/greek-inputenc/test-iso-8859-7.pdf",
    "texmf-dist/doc/latex/greek-inputenc/unicode-licr.txt",
  },
  ["etaremune"] = {
    "texmf-dist/doc/latex/etaremune/README",
    "texmf-dist/doc/latex/etaremune/etaremune.pdf",
  },
  ["babel-slovenian"] = {
    "texmf-dist/doc/generic/babel-slovenian/slovene.pdf",
  },
  ["kantlipsum"] = {
    "texmf-dist/doc/latex/kantlipsum/README.md",
    "texmf-dist/doc/latex/kantlipsum/kantlipsum.pdf",
  },
  ["verbdef"] = {
    "texmf-dist/doc/latex/verbdef/verbdef.pdf",
  },
  ["isotope"] = {
    "texmf-dist/doc/latex/isotope/README",
  },
  ["latex2e-help-texinfo"] = {
    "texmf-dist/doc/latex/latex2e-help-texinfo/ChangeLog",
    "texmf-dist/doc/latex/latex2e-help-texinfo/Makefile",
    "texmf-dist/doc/latex/latex2e-help-texinfo/NEWS",
    "texmf-dist/doc/latex/latex2e-help-texinfo/README",
    "texmf-dist/doc/latex/latex2e-help-texinfo/graphics/README",
    "texmf-dist/doc/latex/latex2e-help-texinfo/latex2e-figures/list.pdf",
    "texmf-dist/doc/latex/latex2e-help-texinfo/latex2e-figures/list.txt",
    "texmf-dist/doc/latex/latex2e-help-texinfo/latex2e.html",
    "texmf-dist/doc/latex/latex2e-help-texinfo/latex2e.pdf",
    "texmf-dist/doc/latex/latex2e-help-texinfo/latex2e.txt",
  },
  ["fenixpar"] = {
    "texmf-dist/doc/generic/fenixpar/README",
  },
  ["vmargin"] = {
    "texmf-dist/doc/latex/vmargin/vmargin.pdf",
  },
  ["cslatex"] = {
    "texmf-dist/doc/man/man1/cslatex.man1.pdf",
    "texmf-dist/doc/man/man1/pdfcslatex.man1.pdf",
  },
  ["ieeepes"] = {
    "texmf-dist/doc/latex/ieeepes/README",
    "texmf-dist/doc/latex/ieeepes/ieeepes_doc.pdf",
  },
  ["metago"] = {
    "texmf-dist/doc/metapost/metago/README",
    "texmf-dist/doc/metapost/metago/example-program.pdf",
    "texmf-dist/doc/metapost/metago/example-script-1.pdf",
    "texmf-dist/doc/metapost/metago/example-script-2.pdf",
    "texmf-dist/doc/metapost/metago/example-script-3.pdf",
    "texmf-dist/doc/metapost/metago/example-script-4.pdf",
    "texmf-dist/doc/metapost/metago/example-script-5.pdf",
    "texmf-dist/doc/metapost/metago/example-script-6.pdf",
    "texmf-dist/doc/metapost/metago/example-script-7.pdf",
    "texmf-dist/doc/metapost/metago/example-script-8.pdf",
  },
  ["kastrup"] = {
    "texmf-dist/doc/generic/kastrup/binhex.pdf",
  },
  ["unravel"] = {
    "texmf-dist/doc/latex/unravel/README.md",
    "texmf-dist/doc/latex/unravel/unravel.pdf",
  },
  ["pxfonts"] = {
    "texmf-dist/doc/fonts/pxfonts/00bug_fix.txt",
    "texmf-dist/doc/fonts/pxfonts/COPYRIGHT",
    "texmf-dist/doc/fonts/pxfonts/pxfontsdoc.pdf",
    "texmf-dist/doc/fonts/pxfonts/pxfontsdocA4.pdf",
  },
  ["bibtexu"] = {
    "texmf-dist/doc/bibtexu/00readme.txt",
    "texmf-dist/doc/bibtexu/HISTORY",
    "texmf-dist/doc/bibtexu/csfile.txt",
  },
  ["lxfonts"] = {
    "texmf-dist/doc/fonts/lxfonts/LXfonts-demo.pdf",
    "texmf-dist/doc/fonts/lxfonts/README",
    "texmf-dist/doc/fonts/lxfonts/lxfonts.pdf",
    "texmf-dist/doc/fonts/lxfonts/manifest.txt",
  },
  ["nameauth"] = {
    "texmf-dist/doc/latex/nameauth/Makefile",
    "texmf-dist/doc/latex/nameauth/README",
    "texmf-dist/doc/latex/nameauth/README.txt",
    "texmf-dist/doc/latex/nameauth/nameauth.pdf",
  },
  ["chronology"] = {
    "texmf-dist/doc/latex/chronology/README.md",
    "texmf-dist/doc/latex/chronology/example.pdf",
  },
  ["babel-macedonian"] = {
    "texmf-dist/doc/generic/babel-macedonian/README.md",
    "texmf-dist/doc/generic/babel-macedonian/macedonian.pdf",
  },
  ["ijqc"] = {
    "texmf-dist/doc/bibtex/ijqc/README",
    "texmf-dist/doc/bibtex/ijqc/makefile",
    "texmf-dist/doc/bibtex/ijqc/xampl.pdf",
  },
  ["luavlna"] = {
    "texmf-dist/doc/luatex/luavlna/README.md",
    "texmf-dist/doc/luatex/luavlna/luavlna-doc.pdf",
  },
  ["interfaces"] = {
    "texmf-dist/doc/latex/interfaces/README",
    "texmf-dist/doc/latex/interfaces/interfaces.pdf",
  },
  ["venndiagram"] = {
    "texmf-dist/doc/latex/venndiagram/CHANGES",
    "texmf-dist/doc/latex/venndiagram/README",
    "texmf-dist/doc/latex/venndiagram/samples/venn-sample.pdf",
    "texmf-dist/doc/latex/venndiagram/venndiagram.pdf",
  },
  ["datetime2-czech"] = {
    "texmf-dist/doc/latex/datetime2-czech/README",
    "texmf-dist/doc/latex/datetime2-czech/datetime2-czech.pdf",
  },
  ["dvdcoll"] = {
    "texmf-dist/doc/latex/dvdcoll/CHANGES",
    "texmf-dist/doc/latex/dvdcoll/INSTALL",
    "texmf-dist/doc/latex/dvdcoll/README",
    "texmf-dist/doc/latex/dvdcoll/dcexample.pdf",
    "texmf-dist/doc/latex/dvdcoll/dvdcoll.pdf",
    "texmf-dist/doc/latex/dvdcoll/dvdcoll_de.pdf",
    "texmf-dist/doc/latex/dvdcoll/manifest.txt",
  },
  ["bidishadowtext"] = {
    "texmf-dist/doc/xelatex/bidishadowtext/bidishadowtext-demo.pdf",
    "texmf-dist/doc/xelatex/bidishadowtext/bidishadowtext-doc.pdf",
  },
  ["dprogress"] = {
    "texmf-dist/doc/latex/dprogress/README",
    "texmf-dist/doc/latex/dprogress/dprogress.pdf",
  },
  ["float"] = {
    "texmf-dist/doc/latex/float/README",
    "texmf-dist/doc/latex/float/float.pdf",
  },
  ["stanli"] = {
    "texmf-dist/doc/latex/stanli/README.md",
    "texmf-dist/doc/latex/stanli/stanli.pdf",
  },
  ["dcpic"] = {
    "texmf-dist/doc/generic/dcpic/README",
    "texmf-dist/doc/generic/dcpic/eurotex2001.pdf",
    "texmf-dist/doc/generic/dcpic/examples.pdf",
    "texmf-dist/doc/generic/dcpic/manDCPiC.pdf",
    "texmf-dist/doc/generic/dcpic/manDCPiCpt.pdf",
  },
  ["babel-irish"] = {
    "texmf-dist/doc/generic/babel-irish/irish.pdf",
  },
  ["reledmac"] = {
    "texmf-dist/doc/latex/reledmac/README",
    "texmf-dist/doc/latex/reledmac/doc-more/latexmkrc",
    "texmf-dist/doc/latex/reledmac/doc-more/makefile",
    "texmf-dist/doc/latex/reledmac/doc-more/page-typesetting-columns.pdf",
    "texmf-dist/doc/latex/reledmac/examples/1-criticalendotes.pdf",
    "texmf-dist/doc/latex/reledmac/examples/1-criticalnotes.pdf",
    "texmf-dist/doc/latex/reledmac/examples/1-sidenotes.pdf",
    "texmf-dist/doc/latex/reledmac/examples/1-tabular.pdf",
    "texmf-dist/doc/latex/reledmac/examples/1-verses-doublenumbering.pdf",
    "texmf-dist/doc/latex/reledmac/examples/1-verses.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-cross_referencing.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-footnote_spacing.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-glossaries.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-indexing.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-lemma_disambigution.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-line_numbers_in_header.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-linespacing.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-manuscript-apparatus.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-notes-width.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-one_series_per_pstart.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-performances.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-reledmac-right-to-left.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-titles_in_line_numbering_with_notes.pdf",
    "texmf-dist/doc/latex/reledmac/examples/2-titles_not_in_line_numbering.pdf",
    "texmf-dist/doc/latex/reledmac/examples/3-reledpar_columns_different_languages.pdf",
    "texmf-dist/doc/latex/reledmac/examples/3-reledpar_mwe.pdf",
    "texmf-dist/doc/latex/reledmac/examples/3-reledpar_pages_different_languages_lualatex.pdf",
    "texmf-dist/doc/latex/reledmac/examples/3-reledpar_same_page_number_in_both_side.pdf",
    "texmf-dist/doc/latex/reledmac/examples/3-reledpar_sync_setting_advancedshifted-nomax.pdf",
    "texmf-dist/doc/latex/reledmac/examples/3-reledpar_sync_setting_advancedshifted.pdf",
    "texmf-dist/doc/latex/reledmac/examples/3-reledpar_sync_setting_default.pdf",
    "texmf-dist/doc/latex/reledmac/examples/3-reledpar_sync_setting_nomax-shifted.pdf",
    "texmf-dist/doc/latex/reledmac/examples/3-reledpar_sync_setting_nomax.pdf",
    "texmf-dist/doc/latex/reledmac/examples/3-reledpar_sync_setting_nosync.pdf",
    "texmf-dist/doc/latex/reledmac/examples/3-reledpar_sync_setting_shifted.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_column_mix_with_not_column-continuous-numbering.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_column_mix_with_not_column.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_columns_alignment.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_columns_titles_in_line_numbering_with_notes.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_inside-outside-columns.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_one_series_per_pstart.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_pages_long_notes.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_pages_notes_leftpage.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_pages_paragraph_separator_between.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_pages_titles_in_line_numbering_with_notes.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_titles_not_in_line_numbering.pdf",
    "texmf-dist/doc/latex/reledmac/examples/4-reledpar_verse_text_between.pdf",
    "texmf-dist/doc/latex/reledmac/examples/latexmkrc",
    "texmf-dist/doc/latex/reledmac/examples/makefile",
    "texmf-dist/doc/latex/reledmac/latexmkrc",
    "texmf-dist/doc/latex/reledmac/makefile",
    "texmf-dist/doc/latex/reledmac/migration.pdf",
    "texmf-dist/doc/latex/reledmac/reledmac.pdf",
    "texmf-dist/doc/latex/reledmac/reledpar.pdf",
  },
  ["footmisx"] = {
    "texmf-dist/doc/latex/footmisx/README",
  },
  ["libertinus-otf"] = {
    "texmf-dist/doc/fonts/libertinus-otf/Changes",
    "texmf-dist/doc/fonts/libertinus-otf/README.md",
    "texmf-dist/doc/fonts/libertinus-otf/libertinus-otf-doc.pdf",
  },
  ["psfrag-italian"] = {
    "texmf-dist/doc/latex/psfrag-italian/itpfgguide.pdf",
  },
  ["cmdtrack"] = {
    "texmf-dist/doc/latex/cmdtrack/Makefile",
    "texmf-dist/doc/latex/cmdtrack/README",
    "texmf-dist/doc/latex/cmdtrack/cmdtrack.pdf",
  },
  ["ucthesis"] = {
    "texmf-dist/doc/latex/ucthesis/README",
  },
  ["afthesis"] = {
    "texmf-dist/doc/latex/afthesis/README",
    "texmf-dist/doc/latex/afthesis/usethesis.pdf",
  },
  ["einfuehrung"] = {
    "texmf-dist/doc/latex/einfuehrung/B.pdf",
    "texmf-dist/doc/latex/einfuehrung/README",
    "texmf-dist/doc/latex/einfuehrung/doc_data.txt",
  },
  ["cmsd"] = {
    "texmf-dist/doc/latex/cmsd/liesmich",
    "texmf-dist/doc/latex/cmsd/readme",
  },
  ["xcookybooky"] = {
    "texmf-dist/doc/latex/xcookybooky/README",
    "texmf-dist/doc/latex/xcookybooky/example/example.pdf",
    "texmf-dist/doc/latex/xcookybooky/example/pic/background.pdf",
    "texmf-dist/doc/latex/xcookybooky/xcookybooky.pdf",
  },
  ["bodegraph"] = {
    "texmf-dist/doc/latex/bodegraph/README",
    "texmf-dist/doc/latex/bodegraph/bodegraph.pdf",
  },
  ["sudoku"] = {
    "texmf-dist/doc/latex/sudoku/CHANGES",
    "texmf-dist/doc/latex/sudoku/README",
    "texmf-dist/doc/latex/sudoku/sudoku.pdf",
  },
  ["hyper"] = {
    "texmf-dist/doc/latex/hyper/README",
    "texmf-dist/doc/latex/hyper/TODO",
    "texmf-dist/doc/latex/hyper/contrib/README",
    "texmf-dist/doc/latex/hyper/hyper.pdf",
    "texmf-dist/doc/latex/hyper/scontrib/README",
  },
  ["eqparbox"] = {
    "texmf-dist/doc/latex/eqparbox/README",
    "texmf-dist/doc/latex/eqparbox/eqparbox.pdf",
  },
  ["flippdf"] = {
    "texmf-dist/doc/latex/flippdf/README",
    "texmf-dist/doc/latex/flippdf/flippdf.pdf",
  },
  ["chngcntr"] = {
    "texmf-dist/doc/latex/chngcntr/README.md",
    "texmf-dist/doc/latex/chngcntr/chngcntr.pdf",
  },
  ["latexmk"] = {
    "texmf-dist/doc/man/man1/latexmk.man1.pdf",
    "texmf-dist/doc/support/latexmk/CHANGES",
    "texmf-dist/doc/support/latexmk/COPYING",
    "texmf-dist/doc/support/latexmk/INSTALL",
    "texmf-dist/doc/support/latexmk/README",
    "texmf-dist/doc/support/latexmk/example_rcfiles/README-latexmk-rcfiles",
    "texmf-dist/doc/support/latexmk/example_rcfiles/asymptote_latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/bib2gls_latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/clean_pattern_latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/exceltex_latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/glossary_latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/lualatex_latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/mpost_latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/nomenclature_latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/pdflatexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/pythontex-latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/sagetex_latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/sweave_latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/tex4ht-latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/texinfo-latexmkrc",
    "texmf-dist/doc/support/latexmk/example_rcfiles/xelatex_latexmkrc",
    "texmf-dist/doc/support/latexmk/extra-scripts/README-latexmk-scripts",
    "texmf-dist/doc/support/latexmk/extra-scripts/dvipdfm_call",
    "texmf-dist/doc/support/latexmk/extra-scripts/dvipdfmx_call",
    "texmf-dist/doc/support/latexmk/extra-scripts/htlatexonly",
    "texmf-dist/doc/support/latexmk/extra-scripts/kickxdvi",
    "texmf-dist/doc/support/latexmk/extra-scripts/l1",
    "texmf-dist/doc/support/latexmk/extra-scripts/l2",
    "texmf-dist/doc/support/latexmk/extra-scripts/myhtlatex2",
    "texmf-dist/doc/support/latexmk/extra-scripts/pst2pdf_for_latexmk",
    "texmf-dist/doc/support/latexmk/extra-scripts/pst2pdf_for_latexmk_README.txt",
    "texmf-dist/doc/support/latexmk/extra-scripts/startacroread",
    "texmf-dist/doc/support/latexmk/latexmk.pdf",
    "texmf-dist/doc/support/latexmk/latexmk.txt",
  },
  ["bxnewfont"] = {
    "texmf-dist/doc/latex/bxnewfont/LICENSE",
    "texmf-dist/doc/latex/bxnewfont/README.md",
  },
  ["inconsolata"] = {
    "texmf-dist/doc/fonts/inconsolata/OFL.txt",
    "texmf-dist/doc/fonts/inconsolata/README",
    "texmf-dist/doc/fonts/inconsolata/afmcmds.txt",
    "texmf-dist/doc/fonts/inconsolata/inconsolata-doc.pdf",
    "texmf-dist/doc/fonts/inconsolata/novarqu-crop.pdf",
    "texmf-dist/doc/fonts/inconsolata/novarqu-noupq-crop.pdf",
    "texmf-dist/doc/fonts/inconsolata/varqu-noupq-crop.pdf",
  },
  ["apa"] = {
    "texmf-dist/doc/latex/apa/CHANGELOG.txt",
    "texmf-dist/doc/latex/apa/LICENCE",
    "texmf-dist/doc/latex/apa/README",
    "texmf-dist/doc/latex/apa/apacls.html",
    "texmf-dist/doc/latex/apa/apacls.txt",
    "texmf-dist/doc/latex/apa/apaenum.txt",
    "texmf-dist/doc/latex/apa/examples.txt",
  },
  ["fithesis"] = {
    "texmf-dist/doc/latex/fithesis/fithesis.pdf",
    "texmf-dist/doc/latex/fithesis/guide/mu/econ.pdf",
    "texmf-dist/doc/latex/fithesis/guide/mu/fi.pdf",
    "texmf-dist/doc/latex/fithesis/guide/mu/fsps.pdf",
    "texmf-dist/doc/latex/fithesis/guide/mu/fss.pdf",
    "texmf-dist/doc/latex/fithesis/guide/mu/law.pdf",
    "texmf-dist/doc/latex/fithesis/guide/mu/med.pdf",
    "texmf-dist/doc/latex/fithesis/guide/mu/ped.pdf",
    "texmf-dist/doc/latex/fithesis/guide/mu/phil.pdf",
    "texmf-dist/doc/latex/fithesis/guide/mu/sci.pdf",
  },
  ["dithesis"] = {
    "texmf-dist/doc/latex/dithesis/README",
    "texmf-dist/doc/latex/dithesis/sample.pdf",
    "texmf-dist/doc/latex/dithesis/sampleNoArial.pdf",
  },
  ["hithesis"] = {
    "texmf-dist/doc/latex/hithesis/Makefile",
    "texmf-dist/doc/latex/hithesis/README.md",
    "texmf-dist/doc/latex/hithesis/hithesis.pdf",
    "texmf-dist/doc/latex/hithesis/latexmkrc",
    "texmf-dist/doc/latex/hithesis/main.pdf",
  },
  ["babel-russian"] = {
    "texmf-dist/doc/generic/babel-russian/README",
    "texmf-dist/doc/generic/babel-russian/russianb.pdf",
  },
  ["pas-cv"] = {
    "texmf-dist/doc/latex/pas-cv/examples/cv-andromede.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-architecte.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-centaure.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-dynamique.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-gaia.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-jupiter.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-mars.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-neptune.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-orion.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-pegase.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-pluton.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-saturne.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-univers.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-uranus.pdf",
    "texmf-dist/doc/latex/pas-cv/examples/cv-venus.pdf",
    "texmf-dist/doc/latex/pas-cv/pas-cv.pdf",
  },
  ["chappg"] = {
    "texmf-dist/doc/latex/chappg/chappg.pdf",
  },
  ["underoverlap"] = {
    "texmf-dist/doc/latex/underoverlap/README",
    "texmf-dist/doc/latex/underoverlap/underoverlap.pdf",
  },
  ["epsincl"] = {
    "texmf-dist/doc/metapost/epsincl/0info.txt",
    "texmf-dist/doc/metapost/epsincl/README",
  },
  ["verbatimbox"] = {
    "texmf-dist/doc/latex/verbatimbox/README",
    "texmf-dist/doc/latex/verbatimbox/verbatimbox.pdf",
  },
  ["jamtimes"] = {
    "texmf-dist/doc/latex/jamtimes/README",
    "texmf-dist/doc/latex/jamtimes/jamtimes.pdf",
    "texmf-dist/doc/latex/jamtimes/mathsample.pdf",
    "texmf-dist/doc/latex/jamtimes/mathsample_ps.pdf",
    "texmf-dist/doc/latex/jamtimes/textsample.pdf",
    "texmf-dist/doc/latex/jamtimes/textsample_ps.pdf",
  },
  ["commado"] = {
    "texmf-dist/doc/generic/commado/README",
    "texmf-dist/doc/generic/commado/SrcFILEs.txt",
    "texmf-dist/doc/generic/commado/commado.pdf",
  },
  ["bookcover"] = {
    "texmf-dist/doc/latex/bookcover/README",
    "texmf-dist/doc/latex/bookcover/bookcover-example1.pdf",
    "texmf-dist/doc/latex/bookcover/bookcover-example2.pdf",
    "texmf-dist/doc/latex/bookcover/bookcover.pdf",
    "texmf-dist/doc/latex/bookcover/figures/background1.pdf",
    "texmf-dist/doc/latex/bookcover/figures/background2.pdf",
    "texmf-dist/doc/latex/bookcover/figures/background3.pdf",
    "texmf-dist/doc/latex/bookcover/figures/background4.pdf",
    "texmf-dist/doc/latex/bookcover/figures/background5.pdf",
    "texmf-dist/doc/latex/bookcover/figures/barcode.pdf",
    "texmf-dist/doc/latex/bookcover/figures/bg1.pdf",
    "texmf-dist/doc/latex/bookcover/figures/bg4.pdf",
    "texmf-dist/doc/latex/bookcover/figures/cover.pdf",
    "texmf-dist/doc/latex/bookcover/figures/coverscheme.pdf",
    "texmf-dist/doc/latex/bookcover/figures/foreground1.pdf",
    "texmf-dist/doc/latex/bookcover/figures/foreground2.pdf",
    "texmf-dist/doc/latex/bookcover/figures/newpart.pdf",
    "texmf-dist/doc/latex/bookcover/figures/parts.pdf",
    "texmf-dist/doc/latex/bookcover/figures/result.pdf",
    "texmf-dist/doc/latex/bookcover/figures/ruler.pdf",
    "texmf-dist/doc/latex/bookcover/figures/sizes.pdf",
    "texmf-dist/doc/latex/bookcover/figures/summa.pdf",
    "texmf-dist/doc/latex/bookcover/figures/tikz.pdf",
    "texmf-dist/doc/latex/bookcover/figures/tikzclip.pdf",
  },
  ["pst-poly"] = {
    "texmf-dist/doc/generic/pst-poly/Changes",
    "texmf-dist/doc/generic/pst-poly/README",
    "texmf-dist/doc/generic/pst-poly/pst-poly-doc.pdf",
  },
  ["knitting"] = {
    "texmf-dist/doc/fonts/knitting/README",
    "texmf-dist/doc/fonts/knitting/knitting-doc.pdf",
  },
  ["jmlr"] = {
    "texmf-dist/doc/latex/jmlr/CHANGES",
    "texmf-dist/doc/latex/jmlr/INSTALL",
    "texmf-dist/doc/latex/jmlr/README",
    "texmf-dist/doc/latex/jmlr/jmlr.pdf",
    "texmf-dist/doc/latex/jmlr/sample-books/jmlrbook-sample.pdf",
    "texmf-dist/doc/latex/jmlr/sample-books/proceedings-sample.pdf",
    "texmf-dist/doc/latex/jmlr/sample-papers/jmlr-sample.pdf",
    "texmf-dist/doc/latex/jmlr/sample-papers/jmlrwcp-sample.pdf",
  },
  ["screenplay-pkg"] = {
    "texmf-dist/doc/latex/screenplay-pkg/README",
    "texmf-dist/doc/latex/screenplay-pkg/screenplay-pkg-example.pdf",
    "texmf-dist/doc/latex/screenplay-pkg/screenplay-pkg.pdf",
  },
  ["glossaries-english"] = {
    "texmf-dist/doc/latex/glossaries-english/README",
    "texmf-dist/doc/latex/glossaries-english/glossaries-english.pdf",
  },
  ["iso10303"] = {
    "texmf-dist/doc/latex/iso10303/README",
    "texmf-dist/doc/latex/iso10303/step4ht.pdf",
    "texmf-dist/doc/latex/iso10303/stepe.pdf",
    "texmf-dist/doc/latex/iso10303/stepman.pdf",
  },
  ["hook-pre-commit-pkg"] = {
    "texmf-dist/doc/support/hook-pre-commit-pkg/README",
    "texmf-dist/doc/support/hook-pre-commit-pkg/pre-commit-latex",
  },
  ["morisawa"] = {
    "texmf-dist/doc/fonts/morisawa/LICENSE",
    "texmf-dist/doc/fonts/morisawa/Makefile",
    "texmf-dist/doc/fonts/morisawa/README.md",
    "texmf-dist/doc/fonts/morisawa/morisawa.pdf",
  },
  ["tikz-dependency"] = {
    "texmf-dist/doc/latex/tikz-dependency/AUTHORS",
    "texmf-dist/doc/latex/tikz-dependency/LICENSE",
    "texmf-dist/doc/latex/tikz-dependency/README",
    "texmf-dist/doc/latex/tikz-dependency/VERSION",
    "texmf-dist/doc/latex/tikz-dependency/tikz-dependency-doc.pdf",
  },
  ["gregoriotex"] = {
    "texmf-dist/doc/luatex/gregoriotex/GregorioNabcRef.pdf",
    "texmf-dist/doc/luatex/gregoriotex/GregorioRef.pdf",
  },
  ["glossaries-irish"] = {
    "texmf-dist/doc/latex/glossaries-irish/README",
    "texmf-dist/doc/latex/glossaries-irish/glossaries-irish.pdf",
  },
  ["pdfcprot"] = {
    "texmf-dist/doc/latex/pdfcprot/00CONTEN",
    "texmf-dist/doc/latex/pdfcprot/00README",
    "texmf-dist/doc/latex/pdfcprot/INSTALL.txt",
    "texmf-dist/doc/latex/pdfcprot/LEGAL.txt",
    "texmf-dist/doc/latex/pdfcprot/README.txt",
    "texmf-dist/doc/latex/pdfcprot/TODO",
    "texmf-dist/doc/latex/pdfcprot/pdfcprot.pdf",
  },
  ["pdfoverlay"] = {
    "texmf-dist/doc/latex/pdfoverlay/README.md",
    "texmf-dist/doc/latex/pdfoverlay/pdfoverlay.pdf",
  },
  ["varisize"] = {
    "texmf-dist/doc/plain/varisize/README",
  },
  ["nddiss"] = {
    "texmf-dist/doc/latex/nddiss/LICENSE",
    "texmf-dist/doc/latex/nddiss/README.md",
    "texmf-dist/doc/latex/nddiss/nddiss2e.pdf",
  },
  ["inlinebib"] = {
    "texmf-dist/doc/bibtex/inlinebib/MANIFEST",
    "texmf-dist/doc/bibtex/inlinebib/inlinebib.htm",
    "texmf-dist/doc/bibtex/inlinebib/inlinebib.txt",
  },
  ["scanpages"] = {
    "texmf-dist/doc/latex/scanpages/README",
    "texmf-dist/doc/latex/scanpages/pic1.pdf",
    "texmf-dist/doc/latex/scanpages/pic2.pdf",
    "texmf-dist/doc/latex/scanpages/scanpages-doc.pdf",
  },
  ["listofsymbols"] = {
    "texmf-dist/doc/latex/listofsymbols/README",
    "texmf-dist/doc/latex/listofsymbols/listofsymbols.pdf",
  },
  ["bxdpx-beamer"] = {
    "texmf-dist/doc/latex/bxdpx-beamer/LICENSE",
    "texmf-dist/doc/latex/bxdpx-beamer/README.md",
  },
  ["toptesi"] = {
    "texmf-dist/doc/latex/toptesi/FrontespiziAssemblati.pdf",
    "texmf-dist/doc/latex/toptesi/FrontespiziAssemblati1.pdf",
    "texmf-dist/doc/latex/toptesi/FrontespiziAssemblati2.pdf",
    "texmf-dist/doc/latex/toptesi/Frontespizio-sss.pdf",
    "texmf-dist/doc/latex/toptesi/FrontespizioLandscape.pdf",
    "texmf-dist/doc/latex/toptesi/FrontespizioScudo.pdf",
    "texmf-dist/doc/latex/toptesi/Logodue.pdf",
    "texmf-dist/doc/latex/toptesi/Logoquattro.pdf",
    "texmf-dist/doc/latex/toptesi/Logotre.pdf",
    "texmf-dist/doc/latex/toptesi/Logouno.pdf",
    "texmf-dist/doc/latex/toptesi/README",
    "texmf-dist/doc/latex/toptesi/README.TEXLIVE",
    "texmf-dist/doc/latex/toptesi/TITDocScCropped.pdf",
    "texmf-dist/doc/latex/toptesi/Toptesi-con-topfront.pdf",
    "texmf-dist/doc/latex/toptesi/Toptesi-example-topfront.pdf",
    "texmf-dist/doc/latex/toptesi/VeraPDFinitial.pdf",
    "texmf-dist/doc/latex/toptesi/manifest.txt",
    "texmf-dist/doc/latex/toptesi/topfront-example.pdf",
    "texmf-dist/doc/latex/toptesi/toptesi-example-con-frontespizio.pdf",
    "texmf-dist/doc/latex/toptesi/toptesi-example-dottorale.pdf",
    "texmf-dist/doc/latex/toptesi/toptesi-example-luatex.pdf",
    "texmf-dist/doc/latex/toptesi/toptesi-example-magistrale.pdf",
    "texmf-dist/doc/latex/toptesi/toptesi-example-sss.pdf",
    "texmf-dist/doc/latex/toptesi/toptesi-example-triennale.pdf",
    "texmf-dist/doc/latex/toptesi/toptesi-example.pdf",
    "texmf-dist/doc/latex/toptesi/toptesi-it.pdf",
    "texmf-dist/doc/latex/toptesi/toptesi.pdf",
    "texmf-dist/doc/latex/toptesi/veraPDFconformance.pdf",
    "texmf-dist/doc/latex/toptesi/veraPDFnoconformance.pdf",
  },
  ["bbm"] = {
    "texmf-dist/doc/fonts/bbm/README",
  },
  ["libris"] = {
    "texmf-dist/doc/fonts/libris/COPYING",
    "texmf-dist/doc/fonts/libris/NOTICE.txt",
    "texmf-dist/doc/fonts/libris/README",
    "texmf-dist/doc/fonts/libris/librisadf.pdf",
    "texmf-dist/doc/fonts/libris/manifest.txt",
  },
  ["komacv"] = {
    "texmf-dist/doc/latex/komacv/README",
    "texmf-dist/doc/latex/komacv/examples/komacv_example.pdf",
    "texmf-dist/doc/latex/komacv/examples/komacv_example_casual.pdf",
    "texmf-dist/doc/latex/komacv/examples/komacv_example_classic.pdf",
    "texmf-dist/doc/latex/komacv/examples/komacv_example_oldstyle.pdf",
    "texmf-dist/doc/latex/komacv/komacv.pdf",
  },
  ["einfuehrung2"] = {
    "texmf-dist/doc/latex/einfuehrung2/README",
    "texmf-dist/doc/latex/einfuehrung2/doc_data.txt",
    "texmf-dist/doc/latex/einfuehrung2/musterlogo.ps",
  },
  ["context-fancybreak"] = {
    "texmf-dist/doc/context/third/fancybreak/README",
    "texmf-dist/doc/context/third/fancybreak/VERSION",
    "texmf-dist/doc/context/third/fancybreak/fancybreak-doc.pdf",
  },
  ["babel-turkish"] = {
    "texmf-dist/doc/generic/babel-turkish/README",
    "texmf-dist/doc/generic/babel-turkish/turkish.pdf",
  },
  ["qcm"] = {
    "texmf-dist/doc/latex/qcm/NEWS",
    "texmf-dist/doc/latex/qcm/README",
    "texmf-dist/doc/latex/qcm/qcm.pdf",
  },
  ["fge"] = {
    "texmf-dist/doc/fonts/fge/README",
    "texmf-dist/doc/fonts/fge/fge-doc.pdf",
  },
  ["aobs-tikz"] = {
    "texmf-dist/doc/latex/aobs-tikz/README",
    "texmf-dist/doc/latex/aobs-tikz/aobs-tikz.pdf",
  },
  ["anonchap"] = {
    "texmf-dist/doc/latex/anonchap/anonchap.pdf",
  },
  ["translation-natbib-fr"] = {
    "texmf-dist/doc/latex/translation-natbib-fr/f-natbib.pdf",
  },
  ["enctex"] = {
    "texmf-dist/doc/generic/enctex/COPYING",
    "texmf-dist/doc/generic/enctex/INSTALL",
    "texmf-dist/doc/generic/enctex/README",
    "texmf-dist/doc/generic/enctex/changes.txt",
    "texmf-dist/doc/generic/enctex/encdoc-e.pdf",
    "texmf-dist/doc/generic/enctex/encdoc.pdf",
  },
  ["logreq"] = {
    "texmf-dist/doc/latex/logreq/README",
  },
  ["begingreek"] = {
    "texmf-dist/doc/latex/begingreek/README",
    "texmf-dist/doc/latex/begingreek/begingreek.pdf",
    "texmf-dist/doc/latex/begingreek/manifest.txt",
  },
  ["subsupscripts"] = {
    "texmf-dist/doc/latex/subsupscripts/README",
    "texmf-dist/doc/latex/subsupscripts/SubSupScripts.pdf",
  },
  ["nodetree"] = {
    "texmf-dist/doc/luatex/nodetree/README.md",
    "texmf-dist/doc/luatex/nodetree/nodetree.pdf",
  },
  ["contracard"] = {
    "texmf-dist/doc/latex/contracard/LICENSE",
    "texmf-dist/doc/latex/contracard/Makefile",
    "texmf-dist/doc/latex/contracard/README",
    "texmf-dist/doc/latex/contracard/README.md",
    "texmf-dist/doc/latex/contracard/contracard.pdf",
  },
  ["arabi-add"] = {
    "texmf-dist/doc/latex/arabi-add/README",
    "texmf-dist/doc/latex/arabi-add/arabi-add-doc.pdf",
    "texmf-dist/doc/latex/arabi-add/arabi-add-example.pdf",
  },
  ["context-transliterator"] = {
    "texmf-dist/doc/context/third/transliterator/COPYING",
    "texmf-dist/doc/context/third/transliterator/VERSION",
    "texmf-dist/doc/context/third/transliterator/transliterator.pdf",
  },
  ["greektex"] = {
    "texmf-dist/doc/fonts/greektex/README",
    "texmf-dist/doc/fonts/greektex/greektexdoc.pdf",
  },
  ["adfathesis"] = {
    "texmf-dist/doc/latex/adfathesis/README",
    "texmf-dist/doc/latex/adfathesis/adfathesis.pdf",
  },
  ["texmate"] = {
    "texmf-dist/doc/latex/texmate/README",
    "texmf-dist/doc/latex/texmate/texmate2manual.pdf",
  },
  ["caption"] = {
    "texmf-dist/doc/latex/caption/CHANGELOG",
    "texmf-dist/doc/latex/caption/README",
    "texmf-dist/doc/latex/caption/SUMMARY",
    "texmf-dist/doc/latex/caption/bicaption.pdf",
    "texmf-dist/doc/latex/caption/caption-deu.pdf",
    "texmf-dist/doc/latex/caption/caption-eng.pdf",
    "texmf-dist/doc/latex/caption/caption-rus.pdf",
    "texmf-dist/doc/latex/caption/ltcaption.pdf",
    "texmf-dist/doc/latex/caption/newfloat.pdf",
    "texmf-dist/doc/latex/caption/subcaption.pdf",
    "texmf-dist/doc/latex/caption/totalcount.pdf",
  },
  ["figbas"] = {
    "texmf-dist/doc/fonts/figbas/README",
    "texmf-dist/doc/fonts/figbas/example.pdf",
    "texmf-dist/doc/fonts/figbas/figbas.pdf",
  },
  ["clrdblpg"] = {
    "texmf-dist/doc/latex/clrdblpg/CHANGES",
    "texmf-dist/doc/latex/clrdblpg/README",
    "texmf-dist/doc/latex/clrdblpg/clrdblpg.pdf",
    "texmf-dist/doc/latex/clrdblpg/lppl.txt",
  },
  ["fancypar"] = {
    "texmf-dist/doc/latex/fancypar/README",
    "texmf-dist/doc/latex/fancypar/fancypar.pdf",
  },
  ["vpe"] = {
    "texmf-dist/doc/latex/vpe/README",
  },
  ["fontch"] = {
    "texmf-dist/doc/plain/fontch/README",
    "texmf-dist/doc/plain/fontch/fontch.pdf",
  },
  ["catchfilebetweentags"] = {
    "texmf-dist/doc/latex/catchfilebetweentags/README",
    "texmf-dist/doc/latex/catchfilebetweentags/catchfilebetweentags.pdf",
  },
  ["hobete"] = {
    "texmf-dist/doc/latex/hobete/README",
    "texmf-dist/doc/latex/hobete/hobete_doc.pdf",
  },
  ["context-visualcounter"] = {
    "texmf-dist/doc/context/third/visualcounter/VERSION",
    "texmf-dist/doc/context/third/visualcounter/visualcounter.pdf",
  },
  ["showdim"] = {
    "texmf-dist/doc/latex/showdim/README",
  },
  ["japanese-otf"] = {
    "texmf-dist/doc/fonts/japanese-otf/COPYRIGHT",
    "texmf-dist/doc/fonts/japanese-otf/README",
    "texmf-dist/doc/fonts/japanese-otf/readme-ja.txt",
  },
  ["keystroke"] = {
    "texmf-dist/doc/latex/keystroke/README",
    "texmf-dist/doc/latex/keystroke/key-test.pdf",
  },
  ["pictexsum"] = {
    "texmf-dist/doc/latex/pictexsum/Makefile",
    "texmf-dist/doc/latex/pictexsum/README",
    "texmf-dist/doc/latex/pictexsum/pictexsum.pdf",
  },
  ["qrcode"] = {
    "texmf-dist/doc/latex/qrcode/README",
    "texmf-dist/doc/latex/qrcode/qrcode.pdf",
  },
  ["ean"] = {
    "texmf-dist/doc/generic/ean/README",
  },
  ["context-simpleslides"] = {
    "texmf-dist/doc/context/third/simpleslides/VERSION",
    "texmf-dist/doc/context/third/simpleslides/example.pdf",
    "texmf-dist/doc/context/third/simpleslides/simpleslides.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/BigNumber-blue.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/BigNumber-red.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/BottomSquares.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Boxed.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Ellipse.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Embossed.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Framed-square.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Framed-stripe.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/FramedTitle.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/HorizontalStripes-blue.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/HorizontalStripes-green.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/HorizontalStripes-red.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/NarrowStripes-blue.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/NarrowStripes-green.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/NarrowStripes-red.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/RainbowStripe.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Rounded.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Shaded-blue.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Shaded-bluered.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Shaded-green.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/SideSquares.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/SideToc.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Split.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Sunrise.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/Swoosh.pdf",
    "texmf-dist/doc/context/third/simpleslides/styles/ThickStripes.pdf",
  },
  ["latex-refsheet"] = {
    "texmf-dist/doc/latex/latex-refsheet/LaTeX_RefSheet.pdf",
    "texmf-dist/doc/latex/latex-refsheet/README.md",
    "texmf-dist/doc/latex/latex-refsheet/acknowledgements.pdf",
    "texmf-dist/doc/latex/latex-refsheet/header-graph.pdf",
    "texmf-dist/doc/latex/latex-refsheet/layout.pdf",
    "texmf-dist/doc/latex/latex-refsheet/thesis.pdf",
  },
  ["ae"] = {
    "texmf-dist/doc/fonts/ae/COPYING",
    "texmf-dist/doc/fonts/ae/MANIFEST",
    "texmf-dist/doc/fonts/ae/README",
  },
  ["babel-vietnamese"] = {
    "texmf-dist/doc/generic/babel-vietnamese/README",
    "texmf-dist/doc/generic/babel-vietnamese/vietnamese.pdf",
  },
  ["pawpict"] = {
    "texmf-dist/doc/latex/pawpict/README",
  },
  ["grayhints"] = {
    "texmf-dist/doc/latex/grayhints/README.md",
    "texmf-dist/doc/latex/grayhints/doc/grayhints_man.pdf",
    "texmf-dist/doc/latex/grayhints/examples/gh-eforms.pdf",
  },
  ["opensans"] = {
    "texmf-dist/doc/fonts/opensans/CHANGES",
    "texmf-dist/doc/fonts/opensans/README",
    "texmf-dist/doc/fonts/opensans/manifest.txt",
    "texmf-dist/doc/fonts/opensans/opensans-samples.pdf",
    "texmf-dist/doc/fonts/opensans/opensans.pdf",
  },
  ["robustindex"] = {
    "texmf-dist/doc/latex/robustindex/README.md",
    "texmf-dist/doc/latex/robustindex/multisample.pdf",
    "texmf-dist/doc/latex/robustindex/robustmanual.pdf",
    "texmf-dist/doc/latex/robustindex/robustsample.pdf",
    "texmf-dist/doc/latex/robustindex/stind.html",
  },
  ["etoc"] = {
    "texmf-dist/doc/latex/etoc/README.md",
    "texmf-dist/doc/latex/etoc/etoc-DE.pdf",
    "texmf-dist/doc/latex/etoc/etoc.pdf",
  },
  ["latex2nemeth"] = {
    "texmf-dist/doc/support/latex2nemeth/README",
    "texmf-dist/doc/support/latex2nemeth/gpl-3.0.txt",
  },
  ["enumitem"] = {
    "texmf-dist/doc/latex/enumitem/README",
    "texmf-dist/doc/latex/enumitem/enumitem.pdf",
  },
  ["pst-cox"] = {
    "texmf-dist/doc/generic/pst-cox/README",
    "texmf-dist/doc/generic/pst-cox/gpl.txt",
    "texmf-dist/doc/generic/pst-cox/lgpl.txt",
    "texmf-dist/doc/generic/pst-cox/pst-coxcoor/pst-coxcoor_doc.pdf",
    "texmf-dist/doc/generic/pst-cox/pst-coxeterp/pst-coxeterp_doc.pdf",
  },
  ["impnattypo"] = {
    "texmf-dist/doc/latex/impnattypo/impnattypo-fr.pdf",
    "texmf-dist/doc/latex/impnattypo/impnattypo.pdf",
  },
  ["flowchart"] = {
    "texmf-dist/doc/latex/flowchart/README",
    "texmf-dist/doc/latex/flowchart/flowchart.pdf",
  },
  ["mkpic"] = {
    "texmf-dist/doc/support/mkpic/README",
    "texmf-dist/doc/support/mkpic/mkpic.pdf",
    "texmf-dist/doc/support/mkpic/mkpicdoc.pdf",
  },
  ["sf298"] = {
    "texmf-dist/doc/latex/sf298/Makefile",
    "texmf-dist/doc/latex/sf298/README.txt",
    "texmf-dist/doc/latex/sf298/sf298.pdf",
  },
  ["babel-english"] = {
    "texmf-dist/doc/generic/babel-english/README",
    "texmf-dist/doc/generic/babel-english/english.pdf",
  },
  ["patchcmd"] = {
    "texmf-dist/doc/latex/patchcmd/README",
    "texmf-dist/doc/latex/patchcmd/patchcmd.pdf",
  },
  ["font-change-xetex"] = {
    "texmf-dist/doc/xetex/font-change-xetex/README.txt",
    "texmf-dist/doc/xetex/font-change-xetex/font-change-xetex.pdf",
  },
  ["formlett"] = {
    "texmf-dist/doc/generic/formlett/prog_manual.pdf",
    "texmf-dist/doc/generic/formlett/user_manual.pdf",
  },
  ["simplewick"] = {
    "texmf-dist/doc/latex/simplewick/README",
    "texmf-dist/doc/latex/simplewick/simplewick.pdf",
  },
  ["xpatch"] = {
    "texmf-dist/doc/latex/xpatch/README",
    "texmf-dist/doc/latex/xpatch/xpatch.pdf",
  },
  ["tikzscale"] = {
    "texmf-dist/doc/latex/tikzscale/README",
    "texmf-dist/doc/latex/tikzscale/test-tikzscale.pdf",
    "texmf-dist/doc/latex/tikzscale/tikzscale.pdf",
  },
  ["pseudocode"] = {
    "texmf-dist/doc/latex/pseudocode/README",
    "texmf-dist/doc/latex/pseudocode/pseudocode.pdf",
  },
  ["penrose"] = {
    "texmf-dist/doc/latex/penrose/README.txt",
    "texmf-dist/doc/latex/penrose/penrose.pdf",
    "texmf-dist/doc/latex/penrose/penrose_code.pdf",
  },
  ["tex-font-errors-cheatsheet"] = {
    "texmf-dist/doc/latex/tex-font-errors-cheatsheet/README",
    "texmf-dist/doc/latex/tex-font-errors-cheatsheet/tex-font-cheatsheet.pdf",
  },
  ["pdfx"] = {
    "texmf-dist/doc/latex/pdfx/README",
    "texmf-dist/doc/latex/pdfx/manifest.txt",
    "texmf-dist/doc/latex/pdfx/pdfx.pdf",
  },
  ["babel-thai"] = {
    "texmf-dist/doc/generic/babel-thai/thai.pdf",
  },
  ["beamerswitch"] = {
    "texmf-dist/doc/latex/beamerswitch/Makefile",
    "texmf-dist/doc/latex/beamerswitch/README.md",
    "texmf-dist/doc/latex/beamerswitch/beamerswitch-example-article.pdf",
    "texmf-dist/doc/latex/beamerswitch/beamerswitch-example-handout.pdf",
    "texmf-dist/doc/latex/beamerswitch/beamerswitch-example-trans.pdf",
    "texmf-dist/doc/latex/beamerswitch/beamerswitch-example.pdf",
    "texmf-dist/doc/latex/beamerswitch/beamerswitch.pdf",
  },
  ["alkalami"] = {
    "texmf-dist/doc/fonts/alkalami/AlkalamiSample.pdf",
    "texmf-dist/doc/fonts/alkalami/OFL.txt",
    "texmf-dist/doc/fonts/alkalami/README.TEXLIVE",
    "texmf-dist/doc/fonts/alkalami/README.txt",
  },
  ["jlreq"] = {
    "texmf-dist/doc/latex/jlreq/LICENSE",
    "texmf-dist/doc/latex/jlreq/README.md",
    "texmf-dist/doc/latex/jlreq/jlreq-ja.html",
    "texmf-dist/doc/latex/jlreq/jlreq-trimmarks-ja.html",
    "texmf-dist/doc/latex/jlreq/jlreq-trimmarks.html",
    "texmf-dist/doc/latex/jlreq/jlreq.html",
  },
  ["minibox"] = {
    "texmf-dist/doc/latex/minibox/README",
    "texmf-dist/doc/latex/minibox/minibox.pdf",
  },
  ["pstool"] = {
    "texmf-dist/doc/latex/pstool/README.md",
    "texmf-dist/doc/latex/pstool/pstool.pdf",
  },
  ["fitbox"] = {
    "texmf-dist/doc/latex/fitbox/Makefile",
    "texmf-dist/doc/latex/fitbox/README",
    "texmf-dist/doc/latex/fitbox/fitbox.pdf",
    "texmf-dist/doc/latex/fitbox/sample.pdf",
  },
  ["textualicomma"] = {
    "texmf-dist/doc/latex/textualicomma/README.md",
    "texmf-dist/doc/latex/textualicomma/textualicomma.pdf",
  },
  ["chessfss"] = {
    "texmf-dist/doc/latex/chessfss/README",
    "texmf-dist/doc/latex/chessfss/README.TEXLIVE",
  },
  ["cantarell"] = {
    "texmf-dist/doc/fonts/cantarell/CHANGES",
    "texmf-dist/doc/fonts/cantarell/README",
    "texmf-dist/doc/fonts/cantarell/cantarell-samples.pdf",
    "texmf-dist/doc/fonts/cantarell/cantarell.pdf",
    "texmf-dist/doc/fonts/cantarell/manifest.txt",
  },
  ["duotenzor"] = {
    "texmf-dist/doc/latex/duotenzor/README",
    "texmf-dist/doc/latex/duotenzor/duotenzormanual.pdf",
  },
  ["esami"] = {
    "texmf-dist/doc/latex/esami/README",
    "texmf-dist/doc/latex/esami/VERSION",
    "texmf-dist/doc/latex/esami/doc/esami-doc-en.pdf",
    "texmf-dist/doc/latex/esami/doc/esami-doc-it.pdf",
    "texmf-dist/doc/latex/esami/doc/examples/testA-luatex-sol.pdf",
    "texmf-dist/doc/latex/esami/doc/examples/testA-luatex.pdf",
    "texmf-dist/doc/latex/esami/doc/examples/testA-sol.pdf",
    "texmf-dist/doc/latex/esami/doc/examples/testA-xetex-sol.pdf",
    "texmf-dist/doc/latex/esami/doc/examples/testA-xetex.pdf",
    "texmf-dist/doc/latex/esami/doc/examples/testA.pdf",
  },
  ["onedown"] = {
    "texmf-dist/doc/latex/onedown/README",
    "texmf-dist/doc/latex/onedown/examples/onedown-examples.pdf",
    "texmf-dist/doc/latex/onedown/examples/onedown-ref.pdf",
    "texmf-dist/doc/latex/onedown/onedown.pdf",
  },
  ["inputtrc"] = {
    "texmf-dist/doc/latex/inputtrc/README",
    "texmf-dist/doc/latex/inputtrc/README.pdf",
    "texmf-dist/doc/latex/inputtrc/RELEASE.txt",
    "texmf-dist/doc/latex/inputtrc/SrcFILEs.txt",
    "texmf-dist/doc/latex/inputtrc/inputtrc.pdf",
  },
  ["ocgx"] = {
    "texmf-dist/doc/latex/ocgx/README",
    "texmf-dist/doc/latex/ocgx/demo-ocgx.pdf",
    "texmf-dist/doc/latex/ocgx/ocgx-manual-en.pdf",
  },
  ["pst-vectorian"] = {
    "texmf-dist/doc/latex/pst-vectorian/README",
    "texmf-dist/doc/latex/pst-vectorian/psvectorian.pdf",
  },
  ["cd"] = {
    "texmf-dist/doc/latex/cd/README",
    "texmf-dist/doc/latex/cd/cd.pdf",
  },
  ["ed"] = {
    "texmf-dist/doc/latex/ed/Makefile",
    "texmf-dist/doc/latex/ed/README",
    "texmf-dist/doc/latex/ed/ed.pdf",
  },
  ["pst-perspective"] = {
    "texmf-dist/doc/generic/pst-perspective/Parallelprojektion-Kreis.pdf",
    "texmf-dist/doc/generic/pst-perspective/README",
    "texmf-dist/doc/generic/pst-perspective/pst-perspective-doc.pdf",
    "texmf-dist/doc/generic/pst-perspective/pst-perspective-docEN.pdf",
  },
  ["pst-optic"] = {
    "texmf-dist/doc/generic/pst-optic/Changes",
    "texmf-dist/doc/generic/pst-optic/README",
    "texmf-dist/doc/generic/pst-optic/pst-optic-doc.pdf",
  },
  ["tdclock"] = {
    "texmf-dist/doc/latex/tdclock/Changelog",
    "texmf-dist/doc/latex/tdclock/README",
    "texmf-dist/doc/latex/tdclock/tdclock-beamer-example.pdf",
    "texmf-dist/doc/latex/tdclock/tdclock-doc.pdf",
  },
  ["sphdthesis"] = {
    "texmf-dist/doc/latex/sphdthesis/LICENSE",
    "texmf-dist/doc/latex/sphdthesis/README",
    "texmf-dist/doc/latex/sphdthesis/SPhdThesis.pdf",
    "texmf-dist/doc/latex/sphdthesis/example/border.pdf",
    "texmf-dist/doc/latex/sphdthesis/example/thesis.pdf",
  },
  ["multicap"] = {
    "texmf-dist/doc/latex/multicap/multicap.pdf",
  },
  ["latex-brochure"] = {
    "texmf-dist/doc/latex/latex-brochure/README",
    "texmf-dist/doc/latex/latex-brochure/README.pdf",
    "texmf-dist/doc/latex/latex-brochure/brochure-a3folderFB.pdf",
    "texmf-dist/doc/latex/latex-brochure/brochure-a3folderIN.pdf",
    "texmf-dist/doc/latex/latex-brochure/brochure-a4.pdf",
    "texmf-dist/doc/latex/latex-brochure/brochure-ledgerfolderFB.pdf",
    "texmf-dist/doc/latex/latex-brochure/brochure-ledgerfolderIN.pdf",
    "texmf-dist/doc/latex/latex-brochure/brochure-letter.pdf",
    "texmf-dist/doc/latex/latex-brochure/brochure.pdf",
    "texmf-dist/doc/latex/latex-brochure/build",
    "texmf-dist/doc/latex/latex-brochure/diagram-crop.pdf",
    "texmf-dist/doc/latex/latex-brochure/index.html",
    "texmf-dist/doc/latex/latex-brochure/sample-crop.pdf",
  },
  ["ifmtarg"] = {
    "texmf-dist/doc/latex/ifmtarg/README",
    "texmf-dist/doc/latex/ifmtarg/ifmtarg.pdf",
  },
  ["unicode-data"] = {
    "texmf-dist/doc/generic/unicode-data/README.md",
  },
  ["noindentafter"] = {
    "texmf-dist/doc/latex/noindentafter/README",
    "texmf-dist/doc/latex/noindentafter/noindentafter.pdf",
  },
  ["bewerbung"] = {
    "texmf-dist/doc/latex/bewerbung/Foto.pdf",
    "texmf-dist/doc/latex/bewerbung/README",
    "texmf-dist/doc/latex/bewerbung/README_DE",
    "texmf-dist/doc/latex/bewerbung/argetabelle-example.pdf",
    "texmf-dist/doc/latex/bewerbung/bewerbung-example.pdf",
    "texmf-dist/doc/latex/bewerbung/bewerbung.pdf",
  },
  ["braille"] = {
    "texmf-dist/doc/latex/braille/README",
    "texmf-dist/doc/latex/braille/braille.html",
    "texmf-dist/doc/latex/braille/summary.pdf",
  },
  ["xechangebar"] = {
    "texmf-dist/doc/xelatex/xechangebar/README",
  },
  ["dejavu-otf"] = {
    "texmf-dist/doc/fonts/dejavu-otf/Changes",
    "texmf-dist/doc/fonts/dejavu-otf/README.md",
    "texmf-dist/doc/fonts/dejavu-otf/dejavu-otf-doc.pdf",
  },
  ["make4ht"] = {
    "texmf-dist/doc/support/make4ht/README",
    "texmf-dist/doc/support/make4ht/make4ht-doc.pdf",
    "texmf-dist/doc/support/make4ht/readme.tex",
  },
  ["pst-3d"] = {
    "texmf-dist/doc/generic/pst-3d/Changes",
    "texmf-dist/doc/generic/pst-3d/README",
    "texmf-dist/doc/generic/pst-3d/pst-3d-doc.pdf",
  },
  ["gcard"] = {
    "texmf-dist/doc/latex/gcard/README",
    "texmf-dist/doc/latex/gcard/gcard.pdf",
  },
  ["thmtools"] = {
    "texmf-dist/doc/latex/thmtools/COPYING",
    "texmf-dist/doc/latex/thmtools/README",
    "texmf-dist/doc/latex/thmtools/TODO",
    "texmf-dist/doc/latex/thmtools/thmtools.pdf",
  },
  ["plstmary"] = {
    "texmf-dist/doc/plain/plstmary/README",
    "texmf-dist/doc/plain/plstmary/plstmary-doc.pdf",
  },
  ["ogham"] = {
    "texmf-dist/doc/fonts/ogham/testfont.pdf",
  },
  ["exp-testopt"] = {
    "texmf-dist/doc/latex/exp-testopt/README",
    "texmf-dist/doc/latex/exp-testopt/exp-testopt.pdf",
  },
  ["chessboard"] = {
    "texmf-dist/doc/latex/chessboard/README",
    "texmf-dist/doc/latex/chessboard/README.TEXLIVE",
  },
  ["calculator"] = {
    "texmf-dist/doc/latex/calculator/README",
    "texmf-dist/doc/latex/calculator/calculator.pdf",
  },
  ["lm"] = {
    "texmf-dist/doc/fonts/lm/GUST-FONT-LICENSE.TXT",
    "texmf-dist/doc/fonts/lm/MANIFEST-Latin-Modern.TXT",
    "texmf-dist/doc/fonts/lm/README-Latin-Modern.TXT",
    "texmf-dist/doc/fonts/lm/lm-hist.txt",
    "texmf-dist/doc/fonts/lm/lm-info.pdf",
    "texmf-dist/doc/fonts/lm/tstlmot1.pdf",
    "texmf-dist/doc/fonts/lm/tstlmot4.pdf",
    "texmf-dist/doc/fonts/lm/tstlmqx.pdf",
    "texmf-dist/doc/fonts/lm/tstlmt1.pdf",
    "texmf-dist/doc/fonts/lm/tstlmts1.pdf",
  },
  ["needspace"] = {
    "texmf-dist/doc/latex/needspace/README",
    "texmf-dist/doc/latex/needspace/needspace.pdf",
  },
  ["babel-kurmanji"] = {
    "texmf-dist/doc/generic/babel-kurmanji/kurmanji.pdf",
  },
  ["cm"] = {
    "texmf-dist/doc/fonts/cm/README",
    "texmf-dist/doc/fonts/cm/README-cmps.txt",
  },
  ["pdfslide"] = {
    "texmf-dist/doc/latex/pdfslide/demo.pdf",
    "texmf-dist/doc/latex/pdfslide/mpgraph.pdf",
  },
  ["ycbook"] = {
    "texmf-dist/doc/latex/ycbook/README.md",
    "texmf-dist/doc/latex/ycbook/ycbook-doc.pdf",
  },
  ["amsldoc-it"] = {
    "texmf-dist/doc/latex/amsldoc-it/itamsldoc.pdf",
  },
  ["frege"] = {
    "texmf-dist/doc/latex/frege/GNU.txt",
    "texmf-dist/doc/latex/frege/INSTALL",
    "texmf-dist/doc/latex/frege/README",
    "texmf-dist/doc/latex/frege/frege.pdf",
  },
  ["lipsum"] = {
    "texmf-dist/doc/latex/lipsum/README",
    "texmf-dist/doc/latex/lipsum/lipsum.pdf",
  },
  ["newlfm"] = {
    "texmf-dist/doc/latex/newlfm/README",
    "texmf-dist/doc/latex/newlfm/draft.pdf",
    "texmf-dist/doc/latex/newlfm/lvb.pdf",
    "texmf-dist/doc/latex/newlfm/make_clean",
    "texmf-dist/doc/latex/newlfm/make_unix",
    "texmf-dist/doc/latex/newlfm/manual.pdf",
    "texmf-dist/doc/latex/newlfm/newlfm.pdf",
    "texmf-dist/doc/latex/newlfm/newlfm.txt",
    "texmf-dist/doc/latex/newlfm/palm.pdf",
    "texmf-dist/doc/latex/newlfm/testms.pdf",
    "texmf-dist/doc/latex/newlfm/wine.pdf",
  },
  ["lh"] = {
    "texmf-dist/doc/fonts/lh/README",
    "texmf-dist/doc/fonts/lh/install",
    "texmf-dist/doc/fonts/lh/manifest.txt",
  },
  ["xargs"] = {
    "texmf-dist/doc/latex/xargs/README",
    "texmf-dist/doc/latex/xargs/xargs-fr.pdf",
    "texmf-dist/doc/latex/xargs/xargs.pdf",
  },
  ["bracketkey"] = {
    "texmf-dist/doc/latex/bracketkey/Malva.pdf",
    "texmf-dist/doc/latex/bracketkey/README",
    "texmf-dist/doc/latex/bracketkey/bracketkey.pdf",
  },
  ["tabvar"] = {
    "texmf-dist/doc/latex/tabvar/README",
    "texmf-dist/doc/latex/tabvar/demo.pdf",
    "texmf-dist/doc/latex/tabvar/tabvar.pdf",
  },
  ["pas-tableur"] = {
    "texmf-dist/doc/latex/pas-tableur/README",
    "texmf-dist/doc/latex/pas-tableur/README.TEXLIVE",
  },
  ["eepic"] = {
    "texmf-dist/doc/latex/eepic/eepic.pdf",
    "texmf-dist/doc/latex/eepic/fig2eepic/makefile",
    "texmf-dist/doc/latex/eepic/fig2eepic/readme",
    "texmf-dist/doc/latex/eepic/readme",
  },
  ["biblatex-opcit-booktitle"] = {
    "texmf-dist/doc/latex/biblatex-opcit-booktitle/README",
    "texmf-dist/doc/latex/biblatex-opcit-booktitle/documentation/biblatex-opcit-booktitle-example.pdf",
    "texmf-dist/doc/latex/biblatex-opcit-booktitle/documentation/biblatex-opcit-booktitle.pdf",
    "texmf-dist/doc/latex/biblatex-opcit-booktitle/documentation/latexmkrc",
    "texmf-dist/doc/latex/biblatex-opcit-booktitle/documentation/makefile",
    "texmf-dist/doc/latex/biblatex-opcit-booktitle/makefile",
  },
  ["slideshow"] = {
    "texmf-dist/doc/metapost/slideshow/slideshow.txt",
    "texmf-dist/doc/metapost/slideshow/sshowex.pdf",
    "texmf-dist/doc/metapost/slideshow/sshowex2.pdf",
    "texmf-dist/doc/metapost/slideshow/sshowex3.pdf",
    "texmf-dist/doc/metapost/slideshow/sshowintro.pdf",
  },
  ["pdfxup"] = {
    "texmf-dist/doc/man/man1/pdfxup.man1.pdf",
    "texmf-dist/doc/support/pdfxup/README",
    "texmf-dist/doc/support/pdfxup/RELEASES",
    "texmf-dist/doc/support/pdfxup/pdfxup.pdf",
  },
  ["lato"] = {
    "texmf-dist/doc/fonts/lato/CHANGES",
    "texmf-dist/doc/fonts/lato/README",
    "texmf-dist/doc/fonts/lato/lato-samples.pdf",
    "texmf-dist/doc/fonts/lato/lato.pdf",
    "texmf-dist/doc/fonts/lato/manifest.txt",
  },
  ["hpsdiss"] = {
    "texmf-dist/doc/latex/hpsdiss/hpsdiss.pdf",
  },
  ["minidocument"] = {
    "texmf-dist/doc/latex/minidocument/Makefile",
    "texmf-dist/doc/latex/minidocument/README",
    "texmf-dist/doc/latex/minidocument/minidocument.pdf",
  },
  ["forarray"] = {
    "texmf-dist/doc/latex/forarray/README.txt",
    "texmf-dist/doc/latex/forarray/forarray",
    "texmf-dist/doc/latex/forarray/forarray-test.pdf",
    "texmf-dist/doc/latex/forarray/forarray.pdf",
  },
  ["biblatex-gb7714-2015"] = {
    "texmf-dist/doc/latex/biblatex-gb7714-2015/README.md",
    "texmf-dist/doc/latex/biblatex-gb7714-2015/biblatex-gb7714-2015.pdf",
    "texmf-dist/doc/latex/biblatex-gb7714-2015/example/egbeamer.pdf",
    "texmf-dist/doc/latex/biblatex-gb7714-2015/example/egbeameray.pdf",
    "texmf-dist/doc/latex/biblatex-gb7714-2015/example/egtest.pdf",
    "texmf-dist/doc/latex/biblatex-gb7714-2015/example/testaligngb7714-2015.pdf",
    "texmf-dist/doc/latex/biblatex-gb7714-2015/example/testaligngb7714-2015ay.pdf",
  },
  ["latex-referenz"] = {
    "texmf-dist/doc/latex/latex-referenz/README",
    "texmf-dist/doc/latex/latex-referenz/images/B.pdf",
  },
  ["bold-extra"] = {
    "texmf-dist/doc/latex/bold-extra/bold-extra.pdf",
  },
  ["mcexam"] = {
    "texmf-dist/doc/latex/mcexam/README.md",
    "texmf-dist/doc/latex/mcexam/mcexam.pdf",
  },
  ["serbian-def-cyr"] = {
    "texmf-dist/doc/latex/serbian-def-cyr/README",
    "texmf-dist/doc/latex/serbian-def-cyr/proba.pdf",
    "texmf-dist/doc/latex/serbian-def-cyr/usage.pdf",
  },
  ["luaotfload"] = {
    "texmf-dist/doc/luatex/luaotfload/COPYING",
    "texmf-dist/doc/luatex/luaotfload/NEWS",
    "texmf-dist/doc/luatex/luaotfload/README",
    "texmf-dist/doc/luatex/luaotfload/filegraph.pdf",
    "texmf-dist/doc/luatex/luaotfload/luaotfload.pdf",
    "texmf-dist/doc/man/man1/luaotfload-tool.man1.pdf",
    "texmf-dist/doc/man/man5/luaotfload.conf.man5.pdf",
  },
  ["fontspec"] = {
    "texmf-dist/doc/latex/fontspec/README.md",
    "texmf-dist/doc/latex/fontspec/fontspec-code.pdf",
    "texmf-dist/doc/latex/fontspec/fontspec.pdf",
  },
  ["drm"] = {
    "texmf-dist/doc/fonts/drm/CHANGES",
    "texmf-dist/doc/fonts/drm/OFL.txt",
    "texmf-dist/doc/fonts/drm/README",
    "texmf-dist/doc/fonts/drm/drm.pdf",
    "texmf-dist/doc/fonts/drm/drm_font_tables.pdf",
    "texmf-dist/doc/fonts/drm/drmfilelist.txt",
    "texmf-dist/doc/fonts/drm/gpl-3.0.txt",
    "texmf-dist/doc/fonts/drm/small_specimen.pdf",
    "texmf-dist/doc/fonts/drm/specimen.pdf",
  },
  ["titlecaps"] = {
    "texmf-dist/doc/latex/titlecaps/README",
    "texmf-dist/doc/latex/titlecaps/titlecaps.pdf",
  },
  ["fifo-stack"] = {
    "texmf-dist/doc/latex/fifo-stack/README",
    "texmf-dist/doc/latex/fifo-stack/fifo-stack.pdf",
  },
  ["index"] = {
    "texmf-dist/doc/latex/index/README",
    "texmf-dist/doc/latex/index/TODO",
    "texmf-dist/doc/latex/index/index.pdf",
  },
  ["germbib"] = {
    "texmf-dist/doc/bibtex/germbib/README.bibgerm",
  },
  ["din1505"] = {
    "texmf-dist/doc/latex/din1505/README.TEXLIVE",
  },
  ["modernposter"] = {
    "texmf-dist/doc/latex/modernposter/README.md",
    "texmf-dist/doc/latex/modernposter/demo.pdf",
    "texmf-dist/doc/latex/modernposter/modernposter.pdf",
  },
  ["jsclasses"] = {
    "texmf-dist/doc/platex/jsclasses/LICENSE",
    "texmf-dist/doc/platex/jsclasses/README.md",
    "texmf-dist/doc/platex/jsclasses/jsclasses.pdf",
    "texmf-dist/doc/platex/jsclasses/jslogo.pdf",
    "texmf-dist/doc/platex/jsclasses/jsverb.pdf",
    "texmf-dist/doc/platex/jsclasses/okumacro.pdf",
    "texmf-dist/doc/platex/jsclasses/okuverb.pdf",
  },
  ["guitlogo"] = {
    "texmf-dist/doc/latex/guitlogo/README",
    "texmf-dist/doc/latex/guitlogo/guit.pdf",
    "texmf-dist/doc/latex/guitlogo/guittest.pdf",
  },
  ["epiolmec"] = {
    "texmf-dist/doc/latex/epiolmec/GlyphAccessCommands.pdf",
  },
  ["l2tabu-italian"] = {
    "texmf-dist/doc/latex/l2tabu-italian/l2tabuit.pdf",
  },
  ["glossaries-spanish"] = {
    "texmf-dist/doc/latex/glossaries-spanish/README",
    "texmf-dist/doc/latex/glossaries-spanish/glossaries-spanish.pdf",
  },
  ["dynkin-diagrams"] = {
    "texmf-dist/doc/latex/dynkin-diagrams/README",
    "texmf-dist/doc/latex/dynkin-diagrams/dynkin-diagrams.pdf",
  },
  ["latex-bib-ex"] = {
    "texmf-dist/doc/latex/latex-bib-ex/README",
  },
  ["warpcol"] = {
    "texmf-dist/doc/latex/warpcol/README",
    "texmf-dist/doc/latex/warpcol/warpcol.pdf",
  },
  ["protex"] = {
    "texmf-dist/doc/latex/protex/ChangeLog",
    "texmf-dist/doc/latex/protex/README",
  },
  ["multienv"] = {
    "texmf-dist/doc/latex/multienv/README",
    "texmf-dist/doc/latex/multienv/multienv.pdf",
  },
  ["pl"] = {
    "texmf-dist/doc/fonts/pl/README.ENG",
    "texmf-dist/doc/fonts/pl/README.POL",
  },
  ["mfpic"] = {
    "texmf-dist/doc/generic/mfpic/README",
    "texmf-dist/doc/generic/mfpic/changes.txt",
    "texmf-dist/doc/generic/mfpic/install.txt",
    "texmf-dist/doc/generic/mfpic/mfpcard.pdf",
    "texmf-dist/doc/generic/mfpic/mfpguide.pdf",
    "texmf-dist/doc/generic/mfpic/mfpic-doc.pdf",
  },
  ["ltxindex"] = {
    "texmf-dist/doc/latex/ltxindex/README",
    "texmf-dist/doc/latex/ltxindex/copying.txt",
    "texmf-dist/doc/latex/ltxindex/ltxindex.pdf",
  },
  ["upzhkinsoku"] = {
    "texmf-dist/doc/generic/upzhkinsoku/README",
    "texmf-dist/doc/generic/upzhkinsoku/upzhkinsoku-doc.pdf",
  },
  ["doc-pictex"] = {
    "texmf-dist/doc/generic/doc-pictex/Doc-PiCTeX.txt",
  },
  ["bbm-macros"] = {
    "texmf-dist/doc/latex/bbm-macros/README",
    "texmf-dist/doc/latex/bbm-macros/bbm.pdf",
  },
  ["extsizes"] = {
    "texmf-dist/doc/latex/extsizes/README",
    "texmf-dist/doc/latex/extsizes/extsizes.pdf",
    "texmf-dist/doc/latex/extsizes/readme.extsizes",
  },
  ["amscls"] = {
    "texmf-dist/doc/latex/amscls/00LICENSE.txt",
    "texmf-dist/doc/latex/amscls/README",
    "texmf-dist/doc/latex/amscls/amsbooka.pdf",
    "texmf-dist/doc/latex/amscls/amsclass.pdf",
    "texmf-dist/doc/latex/amscls/amsdtx.pdf",
    "texmf-dist/doc/latex/amscls/amsmidx.pdf",
    "texmf-dist/doc/latex/amscls/amsthdoc.pdf",
    "texmf-dist/doc/latex/amscls/diffs-c.txt",
    "texmf-dist/doc/latex/amscls/manifest.txt",
    "texmf-dist/doc/latex/amscls/thmtest.pdf",
    "texmf-dist/doc/latex/amscls/upref.pdf",
  },
  ["hyphenat"] = {
    "texmf-dist/doc/latex/hyphenat/README",
    "texmf-dist/doc/latex/hyphenat/hyphenat.pdf",
  },
  ["drawstack"] = {
    "texmf-dist/doc/latex/drawstack/Makefile",
    "texmf-dist/doc/latex/drawstack/README",
    "texmf-dist/doc/latex/drawstack/stack-example.pdf",
  },
  ["imakeidx"] = {
    "texmf-dist/doc/latex/imakeidx/README",
    "texmf-dist/doc/latex/imakeidx/imakeidx.pdf",
    "texmf-dist/doc/latex/imakeidx/manifest.txt",
  },
  ["ms"] = {
    "texmf-dist/doc/latex/ms/count1to.pdf",
    "texmf-dist/doc/latex/ms/everysel.pdf",
    "texmf-dist/doc/latex/ms/everyshi.pdf",
    "texmf-dist/doc/latex/ms/multitoc.pdf",
    "texmf-dist/doc/latex/ms/prelim2e.pdf",
    "texmf-dist/doc/latex/ms/ragged2e.pdf",
  },
  ["calligra"] = {
    "texmf-dist/doc/latex/calligra/README",
    "texmf-dist/doc/latex/calligra/testfont.pdf",
  },
  ["trimspaces"] = {
    "texmf-dist/doc/latex/trimspaces/README",
    "texmf-dist/doc/latex/trimspaces/trimspaces.pdf",
  },
  ["esvect"] = {
    "texmf-dist/doc/latex/esvect/README",
    "texmf-dist/doc/latex/esvect/esvect.pdf",
  },
  ["philex"] = {
    "texmf-dist/doc/latex/philex/README",
    "texmf-dist/doc/latex/philex/philexmanual.pdf",
  },
  ["etex-pkg"] = {
    "texmf-dist/doc/latex/etex-pkg/README",
  },
  ["bibhtml"] = {
    "texmf-dist/doc/bibtex/bibhtml/LICENCE",
    "texmf-dist/doc/bibtex/bibhtml/README",
    "texmf-dist/doc/bibtex/bibhtml/bibhtml",
    "texmf-dist/doc/bibtex/bibhtml/bibhtml.html",
  },
  ["datetime2-french"] = {
    "texmf-dist/doc/latex/datetime2-french/README",
    "texmf-dist/doc/latex/datetime2-french/datetime2-french-test-luatex.pdf",
    "texmf-dist/doc/latex/datetime2-french/datetime2-french-test-pdftex.pdf",
    "texmf-dist/doc/latex/datetime2-french/datetime2-french-test-xetex.pdf",
    "texmf-dist/doc/latex/datetime2-french/datetime2-french.pdf",
  },
  ["preview"] = {
    "texmf-dist/doc/latex/preview/README",
    "texmf-dist/doc/latex/preview/preview.pdf",
  },
  ["numericplots"] = {
    "texmf-dist/doc/latex/numericplots/License.txt",
    "texmf-dist/doc/latex/numericplots/NumericPlots.pdf",
    "texmf-dist/doc/latex/numericplots/README",
    "texmf-dist/doc/latex/numericplots/README.TEXLIVE",
  },
  ["notoccite"] = {
    "texmf-dist/doc/latex/notoccite/notoccite.pdf",
  },
  ["fontools"] = {
    "texmf-dist/doc/man/man1/afm2afm.man1.pdf",
    "texmf-dist/doc/man/man1/autoinst.man1.pdf",
    "texmf-dist/doc/man/man1/ot2kpx.man1.pdf",
    "texmf-dist/doc/support/fontools/GPLv2.txt",
    "texmf-dist/doc/support/fontools/README",
  },
  ["curve2e"] = {
    "texmf-dist/doc/latex/curve2e/README",
    "texmf-dist/doc/latex/curve2e/README.txt",
    "texmf-dist/doc/latex/curve2e/curve2e.pdf",
    "texmf-dist/doc/latex/curve2e/manifest.txt",
  },
  ["cstex"] = {
    "texmf-dist/doc/cstex/00-README-cslatex",
    "texmf-dist/doc/cstex/README-csplain",
    "texmf-dist/doc/cstex/README-cspsfont",
    "texmf-dist/doc/cstex/README-doc",
    "texmf-dist/doc/cstex/README-opmac",
    "texmf-dist/doc/cstex/README.cslatex",
    "texmf-dist/doc/cstex/cspsfonts-gen/README",
    "texmf-dist/doc/cstex/cspsfonts-gen/fontgen",
    "texmf-dist/doc/cstex/cspsfonts-gen/mkf",
    "texmf-dist/doc/cstex/cspsfonts-gen/mkfc",
    "texmf-dist/doc/cstex/cstexman.pdf",
    "texmf-dist/doc/cstex/jemny.pdf",
    "texmf-dist/doc/cstex/opmac-u-en.pdf",
    "texmf-dist/doc/cstex/opmac-u.pdf",
    "texmf-dist/doc/cstex/pdfuni-article.pdf",
    "texmf-dist/doc/cstex/prvni.pdf",
    "texmf-dist/doc/cstex/test8z.pdf",
    "texmf-dist/doc/cstex/testgyre8z.pdf",
    "texmf-dist/doc/cstex/zmeny.txt",
  },
  ["multiobjective"] = {
    "texmf-dist/doc/latex/multiobjective/README",
    "texmf-dist/doc/latex/multiobjective/multiobjective.pdf",
  },
  ["multidef"] = {
    "texmf-dist/doc/latex/multidef/README",
    "texmf-dist/doc/latex/multidef/multidef.pdf",
  },
  ["xyling"] = {
    "texmf-dist/doc/latex/xyling/README",
    "texmf-dist/doc/latex/xyling/xyli-doc.pdf",
  },
  ["polytable"] = {
    "texmf-dist/doc/latex/polytable/README",
    "texmf-dist/doc/latex/polytable/polytable.pdf",
  },
  ["glossaries-italian"] = {
    "texmf-dist/doc/latex/glossaries-italian/README",
    "texmf-dist/doc/latex/glossaries-italian/glossaries-italian.pdf",
  },
  ["vancouver"] = {
    "texmf-dist/doc/bibtex/vancouver/FAQ",
    "texmf-dist/doc/bibtex/vancouver/README",
    "texmf-dist/doc/bibtex/vancouver/vancouver.pdf",
  },
  ["lapdf"] = {
    "texmf-dist/doc/latex/lapdf/README",
    "texmf-dist/doc/latex/lapdf/arcs.pdf",
    "texmf-dist/doc/latex/lapdf/bezier.pdf",
    "texmf-dist/doc/latex/lapdf/bezinfo.pdf",
    "texmf-dist/doc/latex/lapdf/buttrfly.pdf",
    "texmf-dist/doc/latex/lapdf/cfamily.pdf",
    "texmf-dist/doc/latex/lapdf/chrysant.pdf",
    "texmf-dist/doc/latex/lapdf/circle.pdf",
    "texmf-dist/doc/latex/lapdf/colors.pdf",
    "texmf-dist/doc/latex/lapdf/conic.pdf",
    "texmf-dist/doc/latex/lapdf/curve.pdf",
    "texmf-dist/doc/latex/lapdf/curveto.pdf",
    "texmf-dist/doc/latex/lapdf/cycloid.pdf",
    "texmf-dist/doc/latex/lapdf/drawing.pdf",
    "texmf-dist/doc/latex/lapdf/ellipse.pdf",
    "texmf-dist/doc/latex/lapdf/fplot.pdf",
    "texmf-dist/doc/latex/lapdf/geometry.pdf",
    "texmf-dist/doc/latex/lapdf/grids.pdf",
    "texmf-dist/doc/latex/lapdf/hippo.pdf",
    "texmf-dist/doc/latex/lapdf/lapdf.pdf",
    "texmf-dist/doc/latex/lapdf/licence.txt",
    "texmf-dist/doc/latex/lapdf/line.pdf",
    "texmf-dist/doc/latex/lapdf/polygon.pdf",
    "texmf-dist/doc/latex/lapdf/polynom.pdf",
    "texmf-dist/doc/latex/lapdf/pplot.pdf",
    "texmf-dist/doc/latex/lapdf/pythagor.pdf",
    "texmf-dist/doc/latex/lapdf/qcircle.pdf",
    "texmf-dist/doc/latex/lapdf/quartic.pdf",
    "texmf-dist/doc/latex/lapdf/rational.pdf",
    "texmf-dist/doc/latex/lapdf/rcircle.pdf",
    "texmf-dist/doc/latex/lapdf/rcurve.pdf",
    "texmf-dist/doc/latex/lapdf/recttria.pdf",
    "texmf-dist/doc/latex/lapdf/roundtri.pdf",
    "texmf-dist/doc/latex/lapdf/rparams.pdf",
    "texmf-dist/doc/latex/lapdf/superell.pdf",
    "texmf-dist/doc/latex/lapdf/tplot.pdf",
    "texmf-dist/doc/latex/lapdf/trochoid.pdf",
    "texmf-dist/doc/latex/lapdf/turtle.pdf",
    "texmf-dist/doc/latex/lapdf/vector.pdf",
  },
  ["fp"] = {
    "texmf-dist/doc/latex/fp/README",
  },
  ["fifinddo-info"] = {
    "texmf-dist/doc/latex/fifinddo-info/LIESMICH.txt",
    "texmf-dist/doc/latex/fifinddo-info/README",
    "texmf-dist/doc/latex/fifinddo-info/SrcFILEs.txt",
    "texmf-dist/doc/latex/fifinddo-info/dante-mv45-lueck.pdf",
    "texmf-dist/doc/latex/fifinddo-info/dantev45.htm",
    "texmf-dist/doc/latex/fifinddo-info/mdoccorr.pdf",
    "texmf-dist/doc/latex/fifinddo-info/variants/dantev45-1180-clean.html",
    "texmf-dist/doc/latex/fifinddo-info/variants/dantev45-1180-com.html",
    "texmf-dist/doc/latex/fifinddo-info/variants/dantev45-768-com.html",
    "texmf-dist/doc/latex/fifinddo-info/variants/dantev45-768-exact-frame.html",
    "texmf-dist/doc/latex/fifinddo-info/variants/dantev45-768-exact-show.html",
    "texmf-dist/doc/latex/fifinddo-info/variants/dantev45-768-filltype-show.html",
    "texmf-dist/doc/latex/fifinddo-info/variants/dantev45-992-com.html",
    "texmf-dist/doc/latex/fifinddo-info/variants/dantev45-992-exact-frame.html",
    "texmf-dist/doc/latex/fifinddo-info/variants/dantev45-992-exact.html",
  },
  ["bibexport"] = {
    "texmf-dist/doc/bibtex/bibexport/README",
    "texmf-dist/doc/bibtex/bibexport/bibexport.pdf",
  },
  ["actuarialsymbol"] = {
    "texmf-dist/doc/latex/actuarialsymbol/README.md",
    "texmf-dist/doc/latex/actuarialsymbol/actuarialsymbol.pdf",
  },
  ["gridslides"] = {
    "texmf-dist/doc/latex/gridslides/Makefile",
    "texmf-dist/doc/latex/gridslides/README.md",
    "texmf-dist/doc/latex/gridslides/example.pdf",
    "texmf-dist/doc/latex/gridslides/figures/cnontrivial.pdf",
    "texmf-dist/doc/latex/gridslides/figures/ctrivial.pdf",
    "texmf-dist/doc/latex/gridslides/figures/qahe.pdf",
    "texmf-dist/doc/latex/gridslides/figures/qshe.pdf",
    "texmf-dist/doc/latex/gridslides/figures/skyrmion.pdf",
    "texmf-dist/doc/latex/gridslides/figures/trivial.pdf",
    "texmf-dist/doc/latex/gridslides/gridslides.pdf",
  },
  ["txuprcal"] = {
    "texmf-dist/doc/fonts/txuprcal/README",
    "texmf-dist/doc/fonts/txuprcal/txuprcal-doc.pdf",
  },
  ["aspectratio"] = {
    "texmf-dist/doc/latex/aspectratio/ar.pdf",
  },
  ["bullcntr"] = {
    "texmf-dist/doc/latex/bullcntr/00readme.txt",
    "texmf-dist/doc/latex/bullcntr/README",
    "texmf-dist/doc/latex/bullcntr/bullcntr-man.pdf",
    "texmf-dist/doc/latex/bullcntr/manifest.txt",
  },
  ["tex"] = {
    "texmf-dist/doc/man/man1/initex.man1.pdf",
    "texmf-dist/doc/man/man1/tex.man1.pdf",
  },
  ["cybercic"] = {
    "texmf-dist/doc/latex/cybercic/LICENSE",
    "texmf-dist/doc/latex/cybercic/README",
    "texmf-dist/doc/latex/cybercic/cybercic.pdf",
  },
  ["mex"] = {
    "texmf-dist/doc/mex/base/00readme",
    "texmf-dist/doc/mex/base/mex.html",
  },
  ["librebaskerville"] = {
    "texmf-dist/doc/fonts/librebaskerville/FONTLOG.txt",
    "texmf-dist/doc/fonts/librebaskerville/OFL.txt",
    "texmf-dist/doc/fonts/librebaskerville/README",
    "texmf-dist/doc/fonts/librebaskerville/librebaskerville-samples.pdf",
  },
  ["algorithms"] = {
    "texmf-dist/doc/latex/algorithms/COPYING",
    "texmf-dist/doc/latex/algorithms/README",
    "texmf-dist/doc/latex/algorithms/THANKS",
    "texmf-dist/doc/latex/algorithms/algorithms.pdf",
  },
  ["chbibref"] = {
    "texmf-dist/doc/latex/chbibref/chbibref.pdf",
  },
  ["aeguill"] = {
    "texmf-dist/doc/latex/aeguill/README",
    "texmf-dist/doc/latex/aeguill/guil-test1.pdf",
    "texmf-dist/doc/latex/aeguill/guil-test2.pdf",
    "texmf-dist/doc/latex/aeguill/license.txt",
  },
  ["calcage"] = {
    "texmf-dist/doc/latex/calcage/README",
    "texmf-dist/doc/latex/calcage/calcage.pdf",
  },
  ["floatflt"] = {
    "texmf-dist/doc/latex/floatflt/README",
    "texmf-dist/doc/latex/floatflt/floatfig.txt",
    "texmf-dist/doc/latex/floatflt/floatflt.pdf",
  },
  ["collref"] = {
    "texmf-dist/doc/latex/collref/README.txt",
    "texmf-dist/doc/latex/collref/collref.pdf",
  },
  ["biblatex-chem"] = {
    "texmf-dist/doc/latex/biblatex-chem/README.md",
    "texmf-dist/doc/latex/biblatex-chem/biblatex-chem-acs.pdf",
    "texmf-dist/doc/latex/biblatex-chem/biblatex-chem-angew.pdf",
    "texmf-dist/doc/latex/biblatex-chem/biblatex-chem-biochem.pdf",
    "texmf-dist/doc/latex/biblatex-chem/biblatex-chem-rsc.pdf",
    "texmf-dist/doc/latex/biblatex-chem/biblatex-chem.pdf",
  },
  ["fourier"] = {
    "texmf-dist/doc/fonts/fourier/README",
    "texmf-dist/doc/fonts/fourier/fourier-doc-en.pdf",
    "texmf-dist/doc/fonts/fourier/fourier-orns.pdf",
  },
  ["concrete"] = {
    "texmf-dist/doc/fonts/concrete/CATALOGUE",
    "texmf-dist/doc/fonts/concrete/Makefile",
    "texmf-dist/doc/fonts/concrete/README",
  },
  ["notes2bib"] = {
    "texmf-dist/doc/latex/notes2bib/README",
    "texmf-dist/doc/latex/notes2bib/notes2bib.pdf",
  },
  ["luaxml"] = {
    "texmf-dist/doc/luatex/luaxml/README",
    "texmf-dist/doc/luatex/luaxml/luaxml.pdf",
  },
  ["cutwin"] = {
    "texmf-dist/doc/latex/cutwin/README",
    "texmf-dist/doc/latex/cutwin/cutwin.pdf",
  },
  ["texlive-en"] = {
    "texmf-dist/doc/texlive/texlive-en/Makefile",
    "texmf-dist/doc/texlive/texlive-en/README",
    "texmf-dist/doc/texlive/texlive-en/archive/changemail",
    "texmf-dist/doc/texlive/texlive-en/texlive-en.html",
    "texmf-dist/doc/texlive/texlive-en/texlive-en.pdf",
    "texmf-dist/doc/texlive/tlbuild/tlbuild.html",
    "texmf-dist/doc/texlive/tlbuild/tlbuild.pdf",
  },
  ["fixcmex"] = {
    "texmf-dist/doc/latex/fixcmex/README",
    "texmf-dist/doc/latex/fixcmex/fixcmex.pdf",
  },
  ["biber"] = {
    "texmf-dist/doc/bibtex/biber/biber.pdf",
  },
  ["mathfam256"] = {
    "texmf-dist/doc/latex/mathfam256/LICENSE",
    "texmf-dist/doc/latex/mathfam256/README.md",
    "texmf-dist/doc/latex/mathfam256/mathfam256.pdf",
  },
  ["eso-pic"] = {
    "texmf-dist/doc/latex/eso-pic/README",
    "texmf-dist/doc/latex/eso-pic/eso-pic.pdf",
  },
  ["ltxkeys"] = {
    "texmf-dist/doc/latex/ltxkeys/README",
    "texmf-dist/doc/latex/ltxkeys/ltxkeys-guide.pdf",
  },
  ["errata"] = {
    "texmf-dist/doc/latex/errata/README",
    "texmf-dist/doc/latex/errata/errata.pdf",
  },
  ["pst-dbicons"] = {
    "texmf-dist/doc/generic/pst-dbicons/README",
    "texmf-dist/doc/generic/pst-dbicons/pst-dbicons.pdf",
  },
  ["biblatex-realauthor"] = {
    "texmf-dist/doc/latex/biblatex-realauthor/README",
    "texmf-dist/doc/latex/biblatex-realauthor/documentation/biblatex-realauthor.pdf",
    "texmf-dist/doc/latex/biblatex-realauthor/documentation/example-realauthor.pdf",
    "texmf-dist/doc/latex/biblatex-realauthor/documentation/makefile",
    "texmf-dist/doc/latex/biblatex-realauthor/makefile",
  },
  ["schwalbe-chess"] = {
    "texmf-dist/doc/latex/schwalbe-chess/README",
    "texmf-dist/doc/latex/schwalbe-chess/schwalbe.pdf",
  },
  ["doublestroke"] = {
    "texmf-dist/doc/fonts/doublestroke/README",
    "texmf-dist/doc/fonts/doublestroke/dsdoc.pdf",
  },
  ["chronosys"] = {
    "texmf-dist/doc/generic/chronosys/README",
    "texmf-dist/doc/generic/chronosys/docchronosys_en.pdf",
    "texmf-dist/doc/generic/chronosys/docchronosys_fr.pdf",
  },
  ["nkarta"] = {
    "texmf-dist/doc/fonts/nkarta/README",
    "texmf-dist/doc/fonts/nkarta/figtable.pdf",
    "texmf-dist/doc/fonts/nkarta/fonttable.pdf",
  },
  ["xytree"] = {
    "texmf-dist/doc/latex/xytree/README",
    "texmf-dist/doc/latex/xytree/xytree-doc-en.pdf",
  },
  ["geometry-de"] = {
    "texmf-dist/doc/latex/geometry-de/README",
    "texmf-dist/doc/latex/geometry-de/README-DE",
    "texmf-dist/doc/latex/geometry-de/geometry-de.pdf",
  },
  ["systeme"] = {
    "texmf-dist/doc/generic/systeme/README",
    "texmf-dist/doc/generic/systeme/systeme_fr.pdf",
  },
  ["cleveref"] = {
    "texmf-dist/doc/latex/cleveref/README",
    "texmf-dist/doc/latex/cleveref/cleveref.pdf",
  },
  ["bxjscls"] = {
    "texmf-dist/doc/latex/bxjscls/LICENSE",
    "texmf-dist/doc/latex/bxjscls/README.md",
    "texmf-dist/doc/latex/bxjscls/bxjscls-manual.pdf",
    "texmf-dist/doc/latex/bxjscls/bxjscls.pdf",
  },
  ["extpfeil"] = {
    "texmf-dist/doc/latex/extpfeil/README",
    "texmf-dist/doc/latex/extpfeil/extpfeil.pdf",
  },
  ["tablestyles"] = {
    "texmf-dist/doc/latex/tablestyles/README",
    "texmf-dist/doc/latex/tablestyles/tablestyles.pdf",
  },
  ["cooking-units"] = {
    "texmf-dist/doc/latex/cooking-units/README.md",
    "texmf-dist/doc/latex/cooking-units/cooking-units.pdf",
  },
  ["babel-samin"] = {
    "texmf-dist/doc/generic/babel-samin/samin.pdf",
  },
  ["dutchcal"] = {
    "texmf-dist/doc/fonts/dutchcal/README",
  },
  ["memoir"] = {
    "texmf-dist/doc/latex/memoir/Makeidxglo",
    "texmf-dist/doc/latex/memoir/README",
    "texmf-dist/doc/latex/memoir/memman.pdf",
  },
  ["kix"] = {
    "texmf-dist/doc/latex/kix/kix.pdf",
  },
  ["pst-vue3d"] = {
    "texmf-dist/doc/generic/pst-vue3d/Changes",
    "texmf-dist/doc/generic/pst-vue3d/README",
    "texmf-dist/doc/generic/pst-vue3d/pst-vue3d-doc.pdf",
  },
  ["babel-french"] = {
    "texmf-dist/doc/generic/babel-french/README",
    "texmf-dist/doc/generic/babel-french/frenchb.pdf",
  },
  ["dosepsbin"] = {
    "texmf-dist/doc/man/man1/dosepsbin.man1.pdf",
    "texmf-dist/doc/support/dosepsbin/README",
    "texmf-dist/doc/support/dosepsbin/dosepsbin.html",
    "texmf-dist/doc/support/dosepsbin/dosepsbin.pdf",
    "texmf-dist/doc/support/dosepsbin/dosepsbin.txt",
  },
  ["mathalfa"] = {
    "texmf-dist/doc/latex/mathalfa/README",
    "texmf-dist/doc/latex/mathalfa/mathalfa.pdf",
  },
  ["datetime2-bahasai"] = {
    "texmf-dist/doc/latex/datetime2-bahasai/README",
    "texmf-dist/doc/latex/datetime2-bahasai/datetime2-bahasai.pdf",
  },
  ["emisa"] = {
    "texmf-dist/doc/latex/emisa/README.TEXLIVE",
    "texmf-dist/doc/latex/emisa/README.md",
    "texmf-dist/doc/latex/emisa/emisa.pdf",
    "texmf-dist/doc/latex/emisa/manifest.txt",
  },
  ["context-french"] = {
    "texmf-dist/doc/context/third/french/VERSION",
    "texmf-dist/doc/context/third/french/french-demo.pdf",
    "texmf-dist/doc/context/third/french/french-doc.pdf",
  },
  ["datetime2-slovak"] = {
    "texmf-dist/doc/latex/datetime2-slovak/README",
    "texmf-dist/doc/latex/datetime2-slovak/datetime2-slovak.pdf",
  },
  ["calligra-type1"] = {
    "texmf-dist/doc/fonts/calligra-type1/README",
  },
  ["marvosym"] = {
    "texmf-dist/doc/fonts/marvosym/FONTLOG.txt",
    "texmf-dist/doc/fonts/marvosym/Makefile",
    "texmf-dist/doc/fonts/marvosym/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/marvosym/OFL.txt",
    "texmf-dist/doc/fonts/marvosym/README",
    "texmf-dist/doc/fonts/marvosym/marvodoc.pdf",
    "texmf-dist/doc/fonts/marvosym/marvosym-doc.pdf",
  },
  ["chembst"] = {
    "texmf-dist/doc/latex/chembst/README",
    "texmf-dist/doc/latex/chembst/chembst.pdf",
  },
  ["anonymouspro"] = {
    "texmf-dist/doc/fonts/anonymouspro/AnonymousPro.pdf",
    "texmf-dist/doc/fonts/anonymouspro/FONTLOG.txt",
    "texmf-dist/doc/fonts/anonymouspro/OFL.txt",
    "texmf-dist/doc/fonts/anonymouspro/README",
    "texmf-dist/doc/fonts/anonymouspro/README_Mark-Simonson.txt",
  },
  ["thesis-gwu"] = {
    "texmf-dist/doc/latex/thesis-gwu/README.md",
    "texmf-dist/doc/latex/thesis-gwu/thesis-sample.pdf",
  },
  ["mversion"] = {
    "texmf-dist/doc/latex/mversion/README",
    "texmf-dist/doc/latex/mversion/mVersion.pdf",
  },
  ["hfoldsty"] = {
    "texmf-dist/doc/fonts/hfoldsty/ChangeLog",
    "texmf-dist/doc/fonts/hfoldsty/Makefile",
    "texmf-dist/doc/fonts/hfoldsty/README",
    "texmf-dist/doc/fonts/hfoldsty/TODO",
    "texmf-dist/doc/fonts/hfoldsty/gpl.txt",
    "texmf-dist/doc/fonts/hfoldsty/hfoldsty.pdf",
  },
  ["ecgdraw"] = {
    "texmf-dist/doc/latex/ecgdraw/README",
    "texmf-dist/doc/latex/ecgdraw/ecgdraw.pdf",
    "texmf-dist/doc/latex/ecgdraw/ecgdraw_ita.pdf",
  },
  ["babel-spanish"] = {
    "texmf-dist/doc/generic/babel-spanish/README.md",
    "texmf-dist/doc/generic/babel-spanish/spanish.pdf",
  },
  ["visualpstricks"] = {
    "texmf-dist/doc/latex/visualpstricks/README.txt",
    "texmf-dist/doc/latex/visualpstricks/Source/VPST.ps",
    "texmf-dist/doc/latex/visualpstricks/Source/XXX1.ps",
    "texmf-dist/doc/latex/visualpstricks/Source/XXX2.ps",
    "texmf-dist/doc/latex/visualpstricks/Source/data3d.txt",
    "texmf-dist/doc/latex/visualpstricks/Source/logoiut-eps-converted-to.pdf",
    "texmf-dist/doc/latex/visualpstricks/Source/xxx.txt",
    "texmf-dist/doc/latex/visualpstricks/VisualPSTricks.pdf",
    "texmf-dist/doc/latex/visualpstricks/VisuelPSTricks.pdf",
  },
  ["tudscr"] = {
    "texmf-dist/doc/latex/tudscr/README.md",
    "texmf-dist/doc/latex/tudscr/tudscr.pdf",
    "texmf-dist/doc/latex/tudscr/tudscr_print.pdf",
    "texmf-dist/doc/latex/tudscr/tudscrsource.pdf",
    "texmf-dist/doc/latex/tudscr/tutorials/mathswap.pdf",
    "texmf-dist/doc/latex/tudscr/tutorials/mathtype.pdf",
    "texmf-dist/doc/latex/tudscr/tutorials/treatise.pdf",
  },
  ["neuralnetwork"] = {
    "texmf-dist/doc/latex/neuralnetwork/examples/neural-networks-ebook.pdf",
    "texmf-dist/doc/latex/neuralnetwork/examples/neuralnetwork.pdf",
    "texmf-dist/doc/latex/neuralnetwork/examples/xor.pdf",
  },
  ["tfrupee"] = {
    "texmf-dist/doc/fonts/tfrupee/LICENSE",
    "texmf-dist/doc/fonts/tfrupee/README",
    "texmf-dist/doc/fonts/tfrupee/tfrupee.pdf",
  },
  ["bib-fr"] = {
    "texmf-dist/doc/bibtex/bib-fr/CHANGELOG",
    "texmf-dist/doc/bibtex/bib-fr/README",
  },
  ["ginpenc"] = {
    "texmf-dist/doc/latex/ginpenc/ChangeLog",
    "texmf-dist/doc/latex/ginpenc/Makefile",
    "texmf-dist/doc/latex/ginpenc/README",
    "texmf-dist/doc/latex/ginpenc/ginpenc.pdf",
    "texmf-dist/doc/latex/ginpenc/news-message.txt",
  },
  ["babel-spanglish"] = {
    "texmf-dist/doc/generic/babel-spanglish/README",
  },
  ["baekmuk"] = {
    "texmf-dist/doc/fonts/baekmuk/COPYRIGHT",
    "texmf-dist/doc/fonts/baekmuk/README.md",
    "texmf-dist/doc/fonts/baekmuk/README_original",
  },
  ["fetamont"] = {
    "texmf-dist/doc/fonts/fetamont/README",
    "texmf-dist/doc/fonts/fetamont/fetamont-typeface.pdf",
    "texmf-dist/doc/fonts/fetamont/fetamont.pdf",
  },
  ["blockdraw_mp"] = {
    "texmf-dist/doc/metapost/blockdraw_mp/README",
    "texmf-dist/doc/metapost/blockdraw_mp/blockdraw_mp.pdf",
    "texmf-dist/doc/metapost/blockdraw_mp/bonddemo.pdf",
    "texmf-dist/doc/metapost/blockdraw_mp/cascadedemo.pdf",
  },
  ["autopdf"] = {
    "texmf-dist/doc/latex/autopdf/README.txt",
    "texmf-dist/doc/latex/autopdf/autopdf.pdf",
  },
  ["seuthesis"] = {
    "texmf-dist/doc/latex/seuthesis/Makefile",
    "texmf-dist/doc/latex/seuthesis/figures/doctor-hwzs.pdf",
    "texmf-dist/doc/latex/seuthesis/figures/master-hwzs.pdf",
    "texmf-dist/doc/latex/seuthesis/figures/seu-badge-logo.pdf",
    "texmf-dist/doc/latex/seuthesis/sample-bachelor.pdf",
    "texmf-dist/doc/latex/seuthesis/sample-doctor.pdf",
    "texmf-dist/doc/latex/seuthesis/sample-master.pdf",
    "texmf-dist/doc/latex/seuthesis/seuthesis.pdf",
  },
  ["feynmp-auto"] = {
    "texmf-dist/doc/latex/feynmp-auto/README",
    "texmf-dist/doc/latex/feynmp-auto/feynmp-auto.pdf",
  },
  ["semantic"] = {
    "texmf-dist/doc/latex/semantic/semantic.pdf",
  },
  ["robustcommand"] = {
    "texmf-dist/doc/latex/robustcommand/README",
    "texmf-dist/doc/latex/robustcommand/robustcommand.pdf",
  },
  ["datetime2-esperanto"] = {
    "texmf-dist/doc/latex/datetime2-esperanto/README",
    "texmf-dist/doc/latex/datetime2-esperanto/datetime2-esperanto.pdf",
  },
  ["autoaligne"] = {
    "texmf-dist/doc/generic/autoaligne/README",
    "texmf-dist/doc/generic/autoaligne/autoaligne-fr.pdf",
  },
  ["pagecolor"] = {
    "texmf-dist/doc/latex/pagecolor/README",
    "texmf-dist/doc/latex/pagecolor/pagecolor-example.pdf",
    "texmf-dist/doc/latex/pagecolor/pagecolor.pdf",
  },
  ["xypic"] = {
    "texmf-dist/doc/generic/xypic/CATALOG",
    "texmf-dist/doc/generic/xypic/COPYING",
    "texmf-dist/doc/generic/xypic/FONTCOPYING",
    "texmf-dist/doc/generic/xypic/INSTALL",
    "texmf-dist/doc/generic/xypic/MANIFEST",
    "texmf-dist/doc/generic/xypic/README",
    "texmf-dist/doc/generic/xypic/TRAILER",
    "texmf-dist/doc/generic/xypic/VERSIONS",
    "texmf-dist/doc/generic/xypic/Xy-pic.html",
    "texmf-dist/doc/generic/xypic/barrdoc.pdf",
    "texmf-dist/doc/generic/xypic/support/dvitogif89a",
    "texmf-dist/doc/generic/xypic/support/install-tds",
    "texmf-dist/doc/generic/xypic/xyguide.pdf",
    "texmf-dist/doc/generic/xypic/xypdf.pdf",
    "texmf-dist/doc/generic/xypic/xyrefer.pdf",
    "texmf-dist/doc/generic/xypic/xysource.pdf",
  },
  ["bxcjkjatype"] = {
    "texmf-dist/doc/latex/bxcjkjatype/LICENSE",
    "texmf-dist/doc/latex/bxcjkjatype/README.md",
    "texmf-dist/doc/latex/bxcjkjatype/sample-bxcjkjatype-beamer.pdf",
    "texmf-dist/doc/latex/bxcjkjatype/sample-bxcjkjatype.pdf",
  },
  ["ocr-latex"] = {
    "texmf-dist/doc/latex/ocr-latex/README",
    "texmf-dist/doc/latex/ocr-latex/ocr.pdf",
  },
  ["sduthesis"] = {
    "texmf-dist/doc/latex/sduthesis/LICENSE",
    "texmf-dist/doc/latex/sduthesis/README",
    "texmf-dist/doc/latex/sduthesis/README.md",
    "texmf-dist/doc/latex/sduthesis/sduthesis-demo.pdf",
    "texmf-dist/doc/latex/sduthesis/sduthesis.pdf",
  },
  ["pgf"] = {
    "texmf-dist/doc/generic/pgf/AUTHORS",
    "texmf-dist/doc/generic/pgf/ChangeLog",
    "texmf-dist/doc/generic/pgf/FILES",
    "texmf-dist/doc/generic/pgf/INSTALL",
    "texmf-dist/doc/generic/pgf/README",
    "texmf-dist/doc/generic/pgf/images/pgfmanual-mindmap-1.pdf",
    "texmf-dist/doc/generic/pgf/images/pgfmanual-mindmap-2.pdf",
    "texmf-dist/doc/generic/pgf/licenses/LICENSE",
    "texmf-dist/doc/generic/pgf/licenses/gnu-free-documentation-license-1.2.txt",
    "texmf-dist/doc/generic/pgf/licenses/gnu-public-license-2.txt",
    "texmf-dist/doc/generic/pgf/licenses/latex-project-public-license-1.3c.txt",
    "texmf-dist/doc/generic/pgf/licenses/manifest-code.txt",
    "texmf-dist/doc/generic/pgf/licenses/manifest-documentation.txt",
    "texmf-dist/doc/generic/pgf/pgfmanual.pdf",
    "texmf-dist/doc/generic/pgf/version-for-dvipdfm/en/Makefile",
    "texmf-dist/doc/generic/pgf/version-for-dvipdfmx/en/Makefile",
    "texmf-dist/doc/generic/pgf/version-for-dvips/en/Makefile",
    "texmf-dist/doc/generic/pgf/version-for-luatex/en/Makefile",
    "texmf-dist/doc/generic/pgf/version-for-pdftex/en/Makefile",
    "texmf-dist/doc/generic/pgf/version-for-tex4ht/en/Makefile",
    "texmf-dist/doc/generic/pgf/version-for-vtex/en/Makefile",
    "texmf-dist/doc/generic/pgf/version-for-xetex/en/Makefile",
  },
  ["boxhandler"] = {
    "texmf-dist/doc/latex/boxhandler/README",
    "texmf-dist/doc/latex/boxhandler/boxhandler.pdf",
  },
  ["concmath-fonts"] = {
    "texmf-dist/doc/fonts/concmath-fonts/CATALOGUE",
    "texmf-dist/doc/fonts/concmath-fonts/Makefile",
    "texmf-dist/doc/fonts/concmath-fonts/README",
  },
  ["graphics"] = {
    "texmf-dist/doc/latex/graphics/README.md",
    "texmf-dist/doc/latex/graphics/changes.txt",
    "texmf-dist/doc/latex/graphics/color.pdf",
    "texmf-dist/doc/latex/graphics/drivers.pdf",
    "texmf-dist/doc/latex/graphics/epsfig.pdf",
    "texmf-dist/doc/latex/graphics/graphics.pdf",
    "texmf-dist/doc/latex/graphics/graphicx.pdf",
    "texmf-dist/doc/latex/graphics/grfguide.pdf",
    "texmf-dist/doc/latex/graphics/keyval.pdf",
    "texmf-dist/doc/latex/graphics/lscape.pdf",
    "texmf-dist/doc/latex/graphics/rotating.pdf",
    "texmf-dist/doc/latex/graphics/rotex.pdf",
    "texmf-dist/doc/latex/graphics/trig.pdf",
  },
  ["horoscop"] = {
    "texmf-dist/doc/latex/horoscop/README",
    "texmf-dist/doc/latex/horoscop/horoscop.pdf",
  },
  ["flashcards"] = {
    "texmf-dist/doc/latex/flashcards/COPYING",
    "texmf-dist/doc/latex/flashcards/README",
    "texmf-dist/doc/latex/flashcards/flashcards.pdf",
  },
  ["octavo"] = {
    "texmf-dist/doc/latex/octavo/README",
    "texmf-dist/doc/latex/octavo/changes",
    "texmf-dist/doc/latex/octavo/tub-octavo.pdf",
  },
  ["context-filter"] = {
    "texmf-dist/doc/context/third/filter/VERSION",
    "texmf-dist/doc/context/third/filter/filter.txt",
  },
  ["fduthesis"] = {
    "texmf-dist/doc/latex/fduthesis/README.md",
    "texmf-dist/doc/latex/fduthesis/fduthesis-code.pdf",
    "texmf-dist/doc/latex/fduthesis/fduthesis-en.pdf",
    "texmf-dist/doc/latex/fduthesis/fduthesis.pdf",
    "texmf-dist/doc/latex/fduthesis/latexmkrc",
    "texmf-dist/doc/latex/fduthesis/latexmkrc-en",
  },
  ["subfiles"] = {
    "texmf-dist/doc/latex/subfiles/README",
    "texmf-dist/doc/latex/subfiles/subfiles.pdf",
  },
  ["beamertheme-saintpetersburg"] = {
    "texmf-dist/doc/latex/beamertheme-saintpetersburg/README.md",
    "texmf-dist/doc/latex/beamertheme-saintpetersburg/SaintPetersburg.pdf",
    "texmf-dist/doc/latex/beamertheme-saintpetersburg/example.pdf",
  },
  ["bib2gls"] = {
    "texmf-dist/doc/support/bib2gls/CHANGES",
    "texmf-dist/doc/support/bib2gls/README.md",
    "texmf-dist/doc/support/bib2gls/bib2gls-begin.pdf",
    "texmf-dist/doc/support/bib2gls/bib2gls.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-authors.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-bacteria.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-chemical.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-citations.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-constants.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-languages.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-maths.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-media.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-msymbols.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-multi1.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-multi2.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-people.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-textsymbols.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-units1.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-units2.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-units3.pdf",
    "texmf-dist/doc/support/bib2gls/examples/sample-usergroups.pdf",
  },
  ["hanging"] = {
    "texmf-dist/doc/latex/hanging/README",
    "texmf-dist/doc/latex/hanging/hanging.pdf",
  },
  ["auto-pst-pdf"] = {
    "texmf-dist/doc/latex/auto-pst-pdf/README",
    "texmf-dist/doc/latex/auto-pst-pdf/auto-pst-pdf-DE.pdf",
    "texmf-dist/doc/latex/auto-pst-pdf/auto-pst-pdf.pdf",
  },
  ["dblfloatfix"] = {
    "texmf-dist/doc/latex/dblfloatfix/dblfloatfix.pdf",
  },
  ["delimset"] = {
    "texmf-dist/doc/latex/delimset/README.txt",
    "texmf-dist/doc/latex/delimset/delimset.pdf",
  },
  ["ifnextok"] = {
    "texmf-dist/doc/latex/ifnextok/README",
    "texmf-dist/doc/latex/ifnextok/RELEASEs.txt",
    "texmf-dist/doc/latex/ifnextok/SRCFILEs.txt",
    "texmf-dist/doc/latex/ifnextok/ifnextok.pdf",
    "texmf-dist/doc/latex/ifnextok/testIfNT.pdf",
  },
  ["cnbwp"] = {
    "texmf-dist/doc/latex/cnbwp/README",
    "texmf-dist/doc/latex/cnbwp/cnbpaper.pdf",
    "texmf-dist/doc/latex/cnbwp/cnbwp-manual-cs.pdf",
    "texmf-dist/doc/latex/cnbwp/cnbwp-manual-en.pdf",
    "texmf-dist/doc/latex/cnbwp/graph18.pdf",
  },
  ["voss-mathcol"] = {
    "texmf-dist/doc/latex/voss-mathcol/Changes",
    "texmf-dist/doc/latex/voss-mathcol/mathCol.pdf",
  },
  ["mcmthesis"] = {
    "texmf-dist/doc/latex/mcmthesis/LICENSE",
    "texmf-dist/doc/latex/mcmthesis/README",
    "texmf-dist/doc/latex/mcmthesis/README.tex",
    "texmf-dist/doc/latex/mcmthesis/figures/mcmthesis-aaa-eps-converted-to.pdf",
    "texmf-dist/doc/latex/mcmthesis/figures/mcmthesis-logo.pdf",
    "texmf-dist/doc/latex/mcmthesis/mcmthesis-demo.pdf",
    "texmf-dist/doc/latex/mcmthesis/mcmthesis.pdf",
  },
  ["mxedruli"] = {
    "texmf-dist/doc/fonts/mxedruli/README",
    "texmf-dist/doc/fonts/mxedruli/mxeddoc.pdf",
  },
  ["wordlike"] = {
    "texmf-dist/doc/latex/wordlike/README",
    "texmf-dist/doc/latex/wordlike/wordlike.pdf",
  },
  ["multido"] = {
    "texmf-dist/doc/generic/multido/Changes",
    "texmf-dist/doc/generic/multido/multido-doc.pdf",
    "texmf-dist/doc/generic/multido/multido.pdf",
  },
  ["l3build"] = {
    "texmf-dist/doc/latex/l3build/README.md",
    "texmf-dist/doc/latex/l3build/l3build.pdf",
    "texmf-dist/doc/man/man1/l3build.man1.pdf",
  },
  ["pedigree-perl"] = {
    "texmf-dist/doc/man/man1/pedigree.man1.pdf",
    "texmf-dist/doc/support/pedigree-perl/LICENSE",
    "texmf-dist/doc/support/pedigree-perl/Makefile",
    "texmf-dist/doc/support/pedigree-perl/NEWS",
    "texmf-dist/doc/support/pedigree-perl/Pedigree/Makefile",
    "texmf-dist/doc/support/pedigree-perl/README",
    "texmf-dist/doc/support/pedigree-perl/doc/Makefile",
    "texmf-dist/doc/support/pedigree-perl/doc/pedigree.pdf",
    "texmf-dist/doc/support/pedigree-perl/doc/pedigree.ps",
  },
  ["fncylab"] = {
    "texmf-dist/doc/latex/fncylab/fncylab.pdf",
  },
  ["syntax"] = {
    "texmf-dist/doc/latex/syntax/README",
    "texmf-dist/doc/latex/syntax/syntaxintro.pdf",
    "texmf-dist/doc/latex/syntax/syntaxtest.pdf",
  },
  ["binomexp"] = {
    "texmf-dist/doc/latex/binomexp/README",
    "texmf-dist/doc/latex/binomexp/binomexp.pdf",
  },
  ["optidef"] = {
    "texmf-dist/doc/latex/optidef/README.md",
    "texmf-dist/doc/latex/optidef/optidef.pdf",
  },
  ["gzt"] = {
    "texmf-dist/doc/latex/gzt/README.md",
    "texmf-dist/doc/latex/gzt/french/gzt-fr.pdf",
    "texmf-dist/doc/latex/gzt/french/latexmkrc",
  },
  ["lplfitch"] = {
    "texmf-dist/doc/latex/lplfitch/README",
    "texmf-dist/doc/latex/lplfitch/lplfitch.pdf",
  },
  ["musixguit"] = {
    "texmf-dist/doc/latex/musixguit/README",
    "texmf-dist/doc/latex/musixguit/musixguit_de.pdf",
  },
  ["arabi"] = {
    "texmf-dist/doc/latex/arabi/README",
    "texmf-dist/doc/latex/arabi/big2.pdf",
    "texmf-dist/doc/latex/arabi/fontchart_arabic.pdf",
    "texmf-dist/doc/latex/arabi/fontchart_farsi.pdf",
    "texmf-dist/doc/latex/arabi/lion.pdf",
    "texmf-dist/doc/latex/arabi/samplebook.html",
    "texmf-dist/doc/latex/arabi/samplebook.pdf",
    "texmf-dist/doc/latex/arabi/test_beamer.pdf",
    "texmf-dist/doc/latex/arabi/testplaintex.pdf",
    "texmf-dist/doc/latex/arabi/user_guide.pdf",
  },
  ["automata"] = {
    "texmf-dist/doc/metapost/automata/README",
    "texmf-dist/doc/metapost/automata/example.pdf",
  },
  ["bxenclose"] = {
    "texmf-dist/doc/latex/bxenclose/LICENSE",
    "texmf-dist/doc/latex/bxenclose/README.md",
  },
  ["bibtex"] = {
    "texmf-dist/doc/bibtex/base/README",
    "texmf-dist/doc/bibtex/base/btxdoc.pdf",
    "texmf-dist/doc/bibtex/base/btxhak.pdf",
    "texmf-dist/doc/man/man1/bibtex.man1.pdf",
  },
  ["acmconf"] = {
    "texmf-dist/doc/latex/acmconf/README",
    "texmf-dist/doc/latex/acmconf/acmconf.pdf",
  },
  ["gender"] = {
    "texmf-dist/doc/latex/gender/README",
    "texmf-dist/doc/latex/gender/gender.pdf",
  },
  ["colorweb"] = {
    "texmf-dist/doc/latex/colorweb/README",
    "texmf-dist/doc/latex/colorweb/colorweb.pdf",
    "texmf-dist/doc/latex/colorweb/colorwebfull.pdf",
    "texmf-dist/doc/latex/colorweb/colorwebuser.pdf",
  },
  ["luatexko"] = {
    "texmf-dist/doc/luatex/luatexko/ChangeLog",
    "texmf-dist/doc/luatex/luatexko/README",
    "texmf-dist/doc/luatex/luatexko/luatexko-doc.pdf",
  },
  ["bbcard"] = {
    "texmf-dist/doc/metapost/bbcard/README.TEXLIVE",
    "texmf-dist/doc/metapost/bbcard/README.bbcard",
    "texmf-dist/doc/metapost/bbcard/README.calendar",
    "texmf-dist/doc/metapost/bbcard/README.scorecard",
  },
  ["fgruler"] = {
    "texmf-dist/doc/latex/fgruler/README",
    "texmf-dist/doc/latex/fgruler/fgruler.pdf",
  },
  ["sitem"] = {
    "texmf-dist/doc/latex/sitem/sitem.pdf",
  },
  ["textfit"] = {
    "texmf-dist/doc/latex/textfit/README",
    "texmf-dist/doc/latex/textfit/makefile",
    "texmf-dist/doc/latex/textfit/manifest",
    "texmf-dist/doc/latex/textfit/textfit.pdf",
  },
  ["anyfontsize"] = {
    "texmf-dist/doc/latex/anyfontsize/README",
    "texmf-dist/doc/latex/anyfontsize/anyfontsize.pdf",
  },
  ["endnotes"] = {
    "texmf-dist/doc/latex/endnotes/endnotes.pdf",
  },
  ["cjhebrew"] = {
    "texmf-dist/doc/latex/cjhebrew/README",
    "texmf-dist/doc/latex/cjhebrew/cjhebrew.pdf",
  },
  ["endofproofwd"] = {
    "texmf-dist/doc/latex/endofproofwd/README.txt",
    "texmf-dist/doc/latex/endofproofwd/endofproofwd.pdf",
  },
  ["dvgloss"] = {
    "texmf-dist/doc/latex/dvgloss/README",
    "texmf-dist/doc/latex/dvgloss/dvgloss.pdf",
  },
  ["lshort-turkish"] = {
    "texmf-dist/doc/latex/lshort-turkish/README",
    "texmf-dist/doc/latex/lshort-turkish/lshort-tr.pdf",
  },
  ["oscola"] = {
    "texmf-dist/doc/latex/oscola/README.md",
    "texmf-dist/doc/latex/oscola/oscola.pdf",
  },
  ["arphic"] = {
    "texmf-dist/doc/fonts/arphic/arphic-sampler.pdf",
    "texmf-dist/doc/fonts/arphic/bkaiu/README",
    "texmf-dist/doc/fonts/arphic/bsmiu/README",
    "texmf-dist/doc/fonts/arphic/gbsnu/README",
    "texmf-dist/doc/fonts/arphic/gkaiu/README",
  },
  ["ec"] = {
    "texmf-dist/doc/fonts/ec/00bugs.txt",
    "texmf-dist/doc/fonts/ec/00error.txt",
    "texmf-dist/doc/fonts/ec/00files.txt",
    "texmf-dist/doc/fonts/ec/00inst.txt",
    "texmf-dist/doc/fonts/ec/00readme.txt",
    "texmf-dist/doc/fonts/ec/copyrite.txt",
    "texmf-dist/doc/fonts/ec/dc-chg.txt",
    "texmf-dist/doc/fonts/ec/ec-chg.txt",
    "texmf-dist/doc/fonts/ec/tc-chg.txt",
  },
  ["flashmovie"] = {
    "texmf-dist/doc/latex/flashmovie/README",
    "texmf-dist/doc/latex/flashmovie/flv-player-license/license.txt",
    "texmf-dist/doc/latex/flashmovie/test-beamer-0.pdf",
    "texmf-dist/doc/latex/flashmovie/test-flv.pdf",
  },
  ["crossword"] = {
    "texmf-dist/doc/latex/crossword/Makefile",
    "texmf-dist/doc/latex/crossword/README",
    "texmf-dist/doc/latex/crossword/cwpuzzle.pdf",
  },
  ["beuron"] = {
    "texmf-dist/doc/fonts/beuron/README",
    "texmf-dist/doc/fonts/beuron/beuron-de.pdf",
    "texmf-dist/doc/fonts/beuron/beuron-en.pdf",
  },
  ["fix2col"] = {
    "texmf-dist/doc/latex/fix2col/README",
    "texmf-dist/doc/latex/fix2col/fix2col.pdf",
  },
  ["superiors"] = {
    "texmf-dist/doc/fonts/superiors/README",
    "texmf-dist/doc/fonts/superiors/libfoot0-crop.pdf",
    "texmf-dist/doc/fonts/superiors/libfoot1-crop.pdf",
    "texmf-dist/doc/fonts/superiors/stempelfoot0-crop.pdf",
    "texmf-dist/doc/fonts/superiors/stempelfoot1-crop.pdf",
    "texmf-dist/doc/fonts/superiors/superiors-doc.pdf",
  },
  ["fltpoint"] = {
    "texmf-dist/doc/generic/fltpoint/README",
    "texmf-dist/doc/generic/fltpoint/fltpoint.pdf",
  },
  ["exam"] = {
    "texmf-dist/doc/latex/exam/README",
    "texmf-dist/doc/latex/exam/examdoc.pdf",
  },
  ["pas-cours"] = {
    "texmf-dist/doc/latex/pas-cours/README",
    "texmf-dist/doc/latex/pas-cours/README.TEXLIVE",
  },
  ["context-annotation"] = {
    "texmf-dist/doc/context/third/annotation/README",
    "texmf-dist/doc/context/third/annotation/VERSION",
    "texmf-dist/doc/context/third/annotation/annotation-doc.pdf",
  },
  ["calculation"] = {
    "texmf-dist/doc/latex/calculation/README",
    "texmf-dist/doc/latex/calculation/calculation.pdf",
  },
  ["blox"] = {
    "texmf-dist/doc/latex/blox/README",
    "texmf-dist/doc/latex/blox/blox.pdf",
  },
  ["lualatex-doc"] = {
    "texmf-dist/doc/lualatex/lualatex-doc/News",
    "texmf-dist/doc/lualatex/lualatex-doc/README",
    "texmf-dist/doc/lualatex/lualatex-doc/lualatex-doc.pdf",
  },
  ["basque-book"] = {
    "texmf-dist/doc/latex/basque-book/README",
    "texmf-dist/doc/latex/basque-book/basque-book.pdf",
    "texmf-dist/doc/latex/basque-book/basque-book_EUS.pdf",
  },
  ["t1utils"] = {
    "texmf-dist/doc/man/man1/t1ascii.man1.pdf",
    "texmf-dist/doc/man/man1/t1asm.man1.pdf",
    "texmf-dist/doc/man/man1/t1binary.man1.pdf",
    "texmf-dist/doc/man/man1/t1disasm.man1.pdf",
    "texmf-dist/doc/man/man1/t1mac.man1.pdf",
    "texmf-dist/doc/man/man1/t1unmac.man1.pdf",
  },
  ["pdfreview"] = {
    "texmf-dist/doc/latex/pdfreview/README.md",
    "texmf-dist/doc/latex/pdfreview/pdfreview.pdf",
    "texmf-dist/doc/latex/pdfreview/sample-for-docs.pdf",
  },
  ["objectz"] = {
    "texmf-dist/doc/latex/objectz/catalog",
    "texmf-dist/doc/latex/objectz/makefile",
    "texmf-dist/doc/latex/objectz/manifest",
    "texmf-dist/doc/latex/objectz/ozguide.pdf",
    "texmf-dist/doc/latex/objectz/readme",
  },
  ["minipage-marginpar"] = {
    "texmf-dist/doc/latex/minipage-marginpar/README",
    "texmf-dist/doc/latex/minipage-marginpar/minipage-marginpar.pdf",
  },
  ["tikz-feynhand"] = {
    "texmf-dist/doc/latex/tikz-feynhand/README.md",
    "texmf-dist/doc/latex/tikz-feynhand/changes.txt",
    "texmf-dist/doc/latex/tikz-feynhand/tikz-feynhand.userguide.pdf",
  },
  ["pdftools"] = {
    "texmf-dist/doc/man/man1/e2pall.man1.pdf",
    "texmf-dist/doc/man/man1/pdfclose.man1.pdf",
    "texmf-dist/doc/man/man1/pdfopen.man1.pdf",
    "texmf-dist/doc/man/man1/pdftosrc.man1.pdf",
  },
  ["rviewport"] = {
    "texmf-dist/doc/latex/rviewport/Makefile",
    "texmf-dist/doc/latex/rviewport/README",
    "texmf-dist/doc/latex/rviewport/rviewport.pdf",
  },
  ["spath3"] = {
    "texmf-dist/doc/latex/spath3/README",
    "texmf-dist/doc/latex/spath3/calligraphy.pdf",
    "texmf-dist/doc/latex/spath3/knots.pdf",
    "texmf-dist/doc/latex/spath3/spath3_code.pdf",
  },
  ["invoice2"] = {
    "texmf-dist/doc/latex/invoice2/README.md",
    "texmf-dist/doc/latex/invoice2/invoice2.pdf",
  },
  ["karnaughmap"] = {
    "texmf-dist/doc/latex/karnaughmap/README",
    "texmf-dist/doc/latex/karnaughmap/karnaughmap.pdf",
  },
  ["cnltx"] = {
    "texmf-dist/doc/latex/cnltx/README",
    "texmf-dist/doc/latex/cnltx/cnltx_en.pdf",
  },
  ["baskervald"] = {
    "texmf-dist/doc/fonts/baskervald/COPYING",
    "texmf-dist/doc/fonts/baskervald/NOTICE.txt",
    "texmf-dist/doc/fonts/baskervald/README",
    "texmf-dist/doc/fonts/baskervald/baskervaldadf.pdf",
    "texmf-dist/doc/fonts/baskervald/manifest.txt",
  },
  ["blacklettert1"] = {
    "texmf-dist/doc/fonts/blacklettert1/COPYING",
    "texmf-dist/doc/fonts/blacklettert1/INSTALL",
    "texmf-dist/doc/fonts/blacklettert1/README",
    "texmf-dist/doc/fonts/blacklettert1/blacklettert1.pdf",
  },
  ["lshort-thai"] = {
    "texmf-dist/doc/latex/lshort-thai/lsh132.pdf",
    "texmf-dist/doc/latex/lshort-thai/readme",
  },
  ["droit-fr"] = {
    "texmf-dist/doc/latex/droit-fr/droit-fr.pdf",
    "texmf-dist/doc/latex/droit-fr/example/latexmkrc",
    "texmf-dist/doc/latex/droit-fr/latexmkrc",
  },
  ["metafont-beginners"] = {
    "texmf-dist/doc/fonts/metafont-beginners/metafont-for-beginners.pdf",
  },
  ["randomwalk"] = {
    "texmf-dist/doc/latex/randomwalk/README.md",
    "texmf-dist/doc/latex/randomwalk/randomwalk.pdf",
  },
  ["cv4tw"] = {
    "texmf-dist/doc/latex/cv4tw/README",
    "texmf-dist/doc/latex/cv4tw/sample-jules-verne-compact.pdf",
    "texmf-dist/doc/latex/cv4tw/sample-jules-verne-sharp.pdf",
    "texmf-dist/doc/latex/cv4tw/sample-jules-verne-simple.pdf",
  },
  ["ebezier"] = {
    "texmf-dist/doc/latex/ebezier/ebezier.pdf",
  },
  ["pdflatexpicscale"] = {
    "texmf-dist/doc/support/pdflatexpicscale/README",
    "texmf-dist/doc/support/pdflatexpicscale/pdflatexpicscale.pdf",
    "texmf-dist/doc/support/pdflatexpicscale/testprinter.ps",
  },
  ["ifxptex"] = {
    "texmf-dist/doc/generic/ifxptex/README",
    "texmf-dist/doc/generic/ifxptex/ifxptex-doc.pdf",
  },
  ["iitem"] = {
    "texmf-dist/doc/latex/iitem/README",
    "texmf-dist/doc/latex/iitem/iitem.pdf",
  },
  ["ttfutils"] = {
    "texmf-dist/doc/man/man1/ttf2afm.man1.pdf",
    "texmf-dist/doc/man/man1/ttf2pk.man1.pdf",
    "texmf-dist/doc/man/man1/ttf2tfm.man1.pdf",
    "texmf-dist/doc/man/man1/ttfdump.man1.pdf",
    "texmf-dist/doc/ttf2pk/ttf2pk.txt",
    "texmf-dist/doc/ttf2pk/ttf2tfm.txt",
  },
  ["emf"] = {
    "texmf-dist/doc/latex/emf/README",
    "texmf-dist/doc/latex/emf/emf.pdf",
    "texmf-dist/doc/latex/emf/license",
  },
  ["includernw"] = {
    "texmf-dist/doc/latex/includernw/README.txt",
    "texmf-dist/doc/latex/includernw/includeRnw-doc.pdf",
  },
  ["pst-gr3d"] = {
    "texmf-dist/doc/generic/pst-gr3d/Changes",
    "texmf-dist/doc/generic/pst-gr3d/README",
    "texmf-dist/doc/generic/pst-gr3d/pst-gr3d.pdf",
  },
  ["bbding"] = {
    "texmf-dist/doc/latex/bbding/README",
    "texmf-dist/doc/latex/bbding/bbding.pdf",
  },
  ["akletter"] = {
    "texmf-dist/doc/latex/akletter/letterdoc.pdf",
    "texmf-dist/doc/latex/akletter/lettereng.pdf",
  },
  ["fcolumn"] = {
    "texmf-dist/doc/latex/fcolumn/README",
    "texmf-dist/doc/latex/fcolumn/fcolumn.pdf",
  },
  ["context-account"] = {
    "texmf-dist/doc/context/third/account/README",
    "texmf-dist/doc/context/third/account/VERSION",
    "texmf-dist/doc/context/third/account/account-doc.pdf",
  },
  ["coolstr"] = {
    "texmf-dist/doc/latex/coolstr/README",
    "texmf-dist/doc/latex/coolstr/coolstr.pdf",
  },
  ["webguide"] = {
    "texmf-dist/doc/latex/webguide/README",
    "texmf-dist/doc/latex/webguide/webguide.html",
    "texmf-dist/doc/latex/webguide/webguide.pdf",
  },
  ["etsvthor"] = {
    "texmf-dist/doc/latex/etsvthor/README.md",
  },
  ["beilstein"] = {
    "texmf-dist/doc/latex/beilstein/BJNANO_Technical_Handbook.pdf",
    "texmf-dist/doc/latex/beilstein/README.md",
    "texmf-dist/doc/latex/beilstein/figure1.pdf",
    "texmf-dist/doc/latex/beilstein/scheme1.pdf",
    "texmf-dist/doc/latex/beilstein/scheme2.pdf",
  },
  ["zxjatype"] = {
    "texmf-dist/doc/latex/zxjatype/LICENSE",
    "texmf-dist/doc/latex/zxjatype/README.md",
    "texmf-dist/doc/latex/zxjatype/example/example.pdf",
    "texmf-dist/doc/latex/zxjatype/example/xetexsamp01.pdf",
  },
  ["zxjafont"] = {
    "texmf-dist/doc/latex/zxjafont/LICENSE",
    "texmf-dist/doc/latex/zxjafont/README.md",
    "texmf-dist/doc/latex/zxjafont/zxjafont.pdf",
  },
  ["piano"] = {
    "texmf-dist/doc/latex/piano/README",
  },
  ["zwgetfdate"] = {
    "texmf-dist/doc/latex/zwgetfdate/License.txt",
    "texmf-dist/doc/latex/zwgetfdate/README",
    "texmf-dist/doc/latex/zwgetfdate/zwgetfdate.pdf",
  },
  ["zlmtt"] = {
    "texmf-dist/doc/fonts/zlmtt/README",
    "texmf-dist/doc/fonts/zlmtt/zlmtt-doc.pdf",
  },
  ["ziffer"] = {
    "texmf-dist/doc/latex/ziffer/README",
  },
  ["zhspacing"] = {
    "texmf-dist/doc/generic/zhspacing/README.md",
    "texmf-dist/doc/generic/zhspacing/zhs-man.pdf",
  },
  ["impatient-cn"] = {
    "texmf-dist/doc/plain/impatient-cn/Makefile",
    "texmf-dist/doc/plain/impatient-cn/cnbook.pdf",
  },
  ["ctan_chk"] = {
    "texmf-dist/doc/support/ctan_chk/README",
    "texmf-dist/doc/support/ctan_chk/ctan_chk.pdf",
    "texmf-dist/doc/support/ctan_chk/ctan_chk_bash",
  },
  ["frletter"] = {
    "texmf-dist/doc/latex/frletter/README",
  },
  ["hyperref"] = {
    "texmf-dist/doc/latex/hyperref/ChangeLog",
    "texmf-dist/doc/latex/hyperref/ChangeLog.pdf",
    "texmf-dist/doc/latex/hyperref/README",
    "texmf-dist/doc/latex/hyperref/README.pdf",
    "texmf-dist/doc/latex/hyperref/backref.pdf",
    "texmf-dist/doc/latex/hyperref/hyperref.pdf",
    "texmf-dist/doc/latex/hyperref/manual.html",
    "texmf-dist/doc/latex/hyperref/manual.pdf",
    "texmf-dist/doc/latex/hyperref/manual2.html",
    "texmf-dist/doc/latex/hyperref/manual3.html",
    "texmf-dist/doc/latex/hyperref/nameref.pdf",
    "texmf-dist/doc/latex/hyperref/options.pdf",
    "texmf-dist/doc/latex/hyperref/paper.pdf",
    "texmf-dist/doc/latex/hyperref/slides.pdf",
  },
  ["zhlipsum"] = {
    "texmf-dist/doc/latex/zhlipsum/README.md",
    "texmf-dist/doc/latex/zhlipsum/zhlipsum-en.pdf",
    "texmf-dist/doc/latex/zhlipsum/zhlipsum.pdf",
  },
  ["zed-csp"] = {
    "texmf-dist/doc/latex/zed-csp/csp2e.pdf",
    "texmf-dist/doc/latex/zed-csp/zed2e.pdf",
  },
  ["beamer-fuberlin"] = {
    "texmf-dist/doc/latex/beamer-FUBerlin/Changes",
    "texmf-dist/doc/latex/beamer-FUBerlin/README",
    "texmf-dist/doc/latex/beamer-FUBerlin/doc/README.doc",
    "texmf-dist/doc/latex/beamer-FUBerlin/doc/exampleClass.pdf",
    "texmf-dist/doc/latex/beamer-FUBerlin/doc/exampleTheme.pdf",
  },
  ["zebra-goodies"] = {
    "texmf-dist/doc/latex/zebra-goodies/README.md",
  },
  ["ytableau"] = {
    "texmf-dist/doc/latex/ytableau/README",
    "texmf-dist/doc/latex/ytableau/ytableau.pdf",
  },
  ["changebar"] = {
    "texmf-dist/doc/latex/changebar/CATALOG",
    "texmf-dist/doc/latex/changebar/MANIFEST",
    "texmf-dist/doc/latex/changebar/README",
    "texmf-dist/doc/latex/changebar/cbtest1-ltx.pdf",
    "texmf-dist/doc/latex/changebar/cbtest1-pdf.pdf",
    "texmf-dist/doc/latex/changebar/cbtest1good.pdf",
    "texmf-dist/doc/latex/changebar/cbtest1wrong.pdf",
    "texmf-dist/doc/latex/changebar/cbtest2-ltx.pdf",
    "texmf-dist/doc/latex/changebar/cbtest2-pdf.pdf",
    "texmf-dist/doc/latex/changebar/changebar.pdf",
  },
  ["ean13isbn"] = {
    "texmf-dist/doc/latex/ean13isbn/License.txt",
    "texmf-dist/doc/latex/ean13isbn/README",
    "texmf-dist/doc/latex/ean13isbn/ean13isbn.pdf",
  },
  ["intopdf"] = {
    "texmf-dist/doc/latex/intopdf/README.md",
    "texmf-dist/doc/latex/intopdf/intopdf.pdf",
  },
  ["bxorigcapt"] = {
    "texmf-dist/doc/latex/bxorigcapt/LICENSE",
    "texmf-dist/doc/latex/bxorigcapt/README.md",
  },
  ["babel-bosnian"] = {
    "texmf-dist/doc/generic/babel-bosnian/README",
    "texmf-dist/doc/generic/babel-bosnian/bosnian.pdf",
  },
  ["fonetika"] = {
    "texmf-dist/doc/fonts/fonetika/README",
    "texmf-dist/doc/fonts/fonetika/fonetika.pdf",
  },
  ["cmarrows"] = {
    "texmf-dist/doc/metapost/cmarrows/README",
    "texmf-dist/doc/metapost/cmarrows/cmarrows.pdf",
  },
  ["xetexfontinfo"] = {
    "texmf-dist/doc/xetex/xetexfontinfo/README",
  },
  ["latex-git-log"] = {
    "texmf-dist/doc/man/man1/latex-git-log.man1.pdf",
    "texmf-dist/doc/support/latex-git-log/README",
    "texmf-dist/doc/support/latex-git-log/example.pdf",
  },
  ["pbox"] = {
    "texmf-dist/doc/latex/pbox/AUTHORS",
    "texmf-dist/doc/latex/pbox/COPYING",
    "texmf-dist/doc/latex/pbox/ChangeLog",
    "texmf-dist/doc/latex/pbox/INSTALL",
    "texmf-dist/doc/latex/pbox/Makefile",
    "texmf-dist/doc/latex/pbox/README",
    "texmf-dist/doc/latex/pbox/pbox.pdf",
  },
  ["york-thesis"] = {
    "texmf-dist/doc/latex/york-thesis/README",
    "texmf-dist/doc/latex/york-thesis/york-thesis.pdf",
  },
  ["csquotes"] = {
    "texmf-dist/doc/latex/csquotes/README.md",
    "texmf-dist/doc/latex/csquotes/csquotes.pdf",
  },
  ["yinit-otf"] = {
    "texmf-dist/doc/fonts/yinit-otf/README.md",
  },
  ["yhmath"] = {
    "texmf-dist/doc/fonts/yhmath/LICENSE",
    "texmf-dist/doc/fonts/yhmath/Makefile",
    "texmf-dist/doc/fonts/yhmath/README",
    "texmf-dist/doc/fonts/yhmath/yhmath.pdf",
  },
  ["yfonts-t1"] = {
    "texmf-dist/doc/fonts/yfonts-t1/README",
  },
  ["yfonts"] = {
    "texmf-dist/doc/latex/yfonts/liesmich",
    "texmf-dist/doc/latex/yfonts/readme",
  },
  ["pst-blur"] = {
    "texmf-dist/doc/generic/pst-blur/Changes",
    "texmf-dist/doc/generic/pst-blur/README",
    "texmf-dist/doc/generic/pst-blur/pst-blur.pdf",
  },
  ["sepfootnotes"] = {
    "texmf-dist/doc/latex/sepfootnotes/README",
    "texmf-dist/doc/latex/sepfootnotes/sepfootnotes.pdf",
  },
  ["yannisgr"] = {
    "texmf-dist/doc/fonts/yannisgr/00changes.txt",
    "texmf-dist/doc/fonts/yannisgr/README",
    "texmf-dist/doc/fonts/yannisgr/README.TEXLIVE",
    "texmf-dist/doc/fonts/yannisgr/monsyl.txt",
    "texmf-dist/doc/fonts/yannisgr/tomakeformat.txt",
  },
  ["yaletter"] = {
    "texmf-dist/doc/latex/yaletter/CHANGES",
    "texmf-dist/doc/latex/yaletter/README",
    "texmf-dist/doc/latex/yaletter/lppl.txt",
    "texmf-dist/doc/latex/yaletter/yaletter.pdf",
  },
  ["latex-notes-zh-cn"] = {
    "texmf-dist/doc/generic/latex-notes-zh-cn/README",
    "texmf-dist/doc/generic/latex-notes-zh-cn/history.txt",
    "texmf-dist/doc/generic/latex-notes-zh-cn/latex-notes-zh-cn.pdf",
    "texmf-dist/doc/generic/latex-notes-zh-cn/license.txt",
  },
  ["bloques"] = {
    "texmf-dist/doc/latex/bloques/README",
    "texmf-dist/doc/latex/bloques/example.pdf",
  },
  ["yafoot"] = {
    "texmf-dist/doc/latex/yafoot/README",
    "texmf-dist/doc/latex/yafoot/yafoot-man.pdf",
  },
  ["signchart"] = {
    "texmf-dist/doc/latex/signchart/README.txt",
    "texmf-dist/doc/latex/signchart/signchart.pdf",
  },
  ["lexref"] = {
    "texmf-dist/doc/latex/lexref/README",
    "texmf-dist/doc/latex/lexref/lexref.pdf",
  },
  ["bosisio"] = {
    "texmf-dist/doc/latex/bosisio/README",
    "texmf-dist/doc/latex/bosisio/accenti.html",
    "texmf-dist/doc/latex/bosisio/accenti.pdf",
    "texmf-dist/doc/latex/bosisio/dblfont.html",
    "texmf-dist/doc/latex/bosisio/dblfont.pdf",
    "texmf-dist/doc/latex/bosisio/envmath.html",
    "texmf-dist/doc/latex/bosisio/envmath.pdf",
    "texmf-dist/doc/latex/bosisio/evenpage.html",
    "texmf-dist/doc/latex/bosisio/evenpage.pdf",
    "texmf-dist/doc/latex/bosisio/graphfig.html",
    "texmf-dist/doc/latex/bosisio/graphfig.pdf",
    "texmf-dist/doc/latex/bosisio/index.html",
    "texmf-dist/doc/latex/bosisio/makedoc",
    "texmf-dist/doc/latex/bosisio/mathcmd.html",
    "texmf-dist/doc/latex/bosisio/mathcmd.pdf",
    "texmf-dist/doc/latex/bosisio/quotes.html",
    "texmf-dist/doc/latex/bosisio/quotes.pdf",
    "texmf-dist/doc/latex/bosisio/sobolev.html",
    "texmf-dist/doc/latex/bosisio/sobolev.pdf",
  },
  ["canoniclayout"] = {
    "texmf-dist/doc/latex/canoniclayout/README",
    "texmf-dist/doc/latex/canoniclayout/canoniclayout.pdf",
  },
  ["frankenstein"] = {
    "texmf-dist/doc/latex/frankenstein/ChangeLog",
    "texmf-dist/doc/latex/frankenstein/Frankenfile",
    "texmf-dist/doc/latex/frankenstein/INSTALL",
    "texmf-dist/doc/latex/frankenstein/README",
    "texmf-dist/doc/latex/frankenstein/abbrevs.pdf",
    "texmf-dist/doc/latex/frankenstein/achicago-bst.pdf",
    "texmf-dist/doc/latex/frankenstein/achicago.pdf",
    "texmf-dist/doc/latex/frankenstein/attrib.pdf",
    "texmf-dist/doc/latex/frankenstein/blkcntrl.pdf",
    "texmf-dist/doc/latex/frankenstein/compsci.pdf",
    "texmf-dist/doc/latex/frankenstein/dialogue.pdf",
    "texmf-dist/doc/latex/frankenstein/lips.pdf",
    "texmf-dist/doc/latex/frankenstein/moredefs.pdf",
    "texmf-dist/doc/latex/frankenstein/newclude.pdf",
    "texmf-dist/doc/latex/frankenstein/slemph.pdf",
    "texmf-dist/doc/latex/frankenstein/titles.pdf",
    "texmf-dist/doc/latex/frankenstein/unsupported/README-unsupported",
    "texmf-dist/doc/latex/frankenstein/unsupported/bits.pdf",
    "texmf-dist/doc/latex/frankenstein/unsupported/drama.pdf",
    "texmf-dist/doc/latex/frankenstein/unsupported/includex.pdf",
  },
  ["ethiop-t1"] = {
    "texmf-dist/doc/latex/ethiop-t1/COPYING",
    "texmf-dist/doc/latex/ethiop-t1/README",
  },
  ["xymtex"] = {
    "texmf-dist/doc/latex/xymtex/doc/README.TEXLIVE",
  },
  ["xwatermark"] = {
    "texmf-dist/doc/latex/xwatermark/README",
    "texmf-dist/doc/latex/xwatermark/xwatermark-examples2.pdf",
    "texmf-dist/doc/latex/xwatermark/xwatermark-guide.pdf",
  },
  ["dynblocks"] = {
    "texmf-dist/doc/latex/dynblocks/README",
    "texmf-dist/doc/latex/dynblocks/dynblocks.pdf",
    "texmf-dist/doc/latex/dynblocks/images/alert_1.pdf",
    "texmf-dist/doc/latex/dynblocks/images/alert_2.pdf",
    "texmf-dist/doc/latex/dynblocks/images/align.pdf",
    "texmf-dist/doc/latex/dynblocks/images/basic_1.pdf",
    "texmf-dist/doc/latex/dynblocks/images/basic_2.pdf",
    "texmf-dist/doc/latex/dynblocks/images/cmbx_1.pdf",
    "texmf-dist/doc/latex/dynblocks/images/cmbx_2.pdf",
    "texmf-dist/doc/latex/dynblocks/images/cmby_1.pdf",
    "texmf-dist/doc/latex/dynblocks/images/cmby_2.pdf",
    "texmf-dist/doc/latex/dynblocks/images/custcol_1.pdf",
    "texmf-dist/doc/latex/dynblocks/images/custcol_2.pdf",
    "texmf-dist/doc/latex/dynblocks/images/custcol_3.pdf",
    "texmf-dist/doc/latex/dynblocks/images/custcol_4.pdf",
    "texmf-dist/doc/latex/dynblocks/images/estl.pdf",
    "texmf-dist/doc/latex/dynblocks/images/shdrndc_1.pdf",
    "texmf-dist/doc/latex/dynblocks/images/shdrndc_2.pdf",
    "texmf-dist/doc/latex/dynblocks/images/szeg_1.pdf",
    "texmf-dist/doc/latex/dynblocks/images/szeg_2.pdf",
  },
  ["udesoftec"] = {
    "texmf-dist/doc/latex/udesoftec/udesoftec-doc.pdf",
  },
  ["xurl"] = {
    "texmf-dist/doc/latex/xurl/Changes",
    "texmf-dist/doc/latex/xurl/README.md",
    "texmf-dist/doc/latex/xurl/xurl.pdf",
  },
  ["xunicode"] = {
    "texmf-dist/doc/xelatex/xunicode/README",
  },
  ["sesstime"] = {
    "texmf-dist/doc/latex/sesstime/README.txt",
    "texmf-dist/doc/latex/sesstime/sesstime.pdf",
  },
  ["xtuthesis"] = {
    "texmf-dist/doc/latex/xtuthesis/README.md",
    "texmf-dist/doc/latex/xtuthesis/figures/Correct_Ratio.pdf",
    "texmf-dist/doc/latex/xtuthesis/figures/Lenna.pdf",
    "texmf-dist/doc/latex/xtuthesis/figures/Lenna_e.pdf",
    "texmf-dist/doc/latex/xtuthesis/xtu.pdf",
    "texmf-dist/doc/latex/xtuthesis/xtuthesis.pdf",
  },
  ["xtab"] = {
    "texmf-dist/doc/latex/xtab/README",
    "texmf-dist/doc/latex/xtab/xtab.pdf",
  },
  ["texlive-de"] = {
    "texmf-dist/doc/texlive/texlive-de/Makefile",
    "texmf-dist/doc/texlive/texlive-de/texlive-de.html",
    "texmf-dist/doc/texlive/texlive-de/texlive-de.pdf",
  },
  ["pxufont"] = {
    "texmf-dist/doc/latex/pxufont/LICENSE",
    "texmf-dist/doc/latex/pxufont/README.md",
  },
  ["makebase"] = {
    "texmf-dist/doc/latex/makebase/MANIFEST",
    "texmf-dist/doc/latex/makebase/README.md",
    "texmf-dist/doc/latex/makebase/makebase.pdf",
  },
  ["xskak"] = {
    "texmf-dist/doc/latex/xskak/README",
    "texmf-dist/doc/latex/xskak/README.TEXLIVE",
  },
  ["currfile"] = {
    "texmf-dist/doc/latex/currfile/README",
    "texmf-dist/doc/latex/currfile/currfile.pdf",
  },
  ["xsim"] = {
    "texmf-dist/doc/latex/xsim/README",
    "texmf-dist/doc/latex/xsim/examples/xsim.blanks.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.boxed-headings.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.code-and-output.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.collections.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.crossref.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.description-list.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.different-point-types.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.difficulties.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.floating.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.golatex-80640.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.golatex-91339.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.grade-distribution.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.hints.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.listings.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.listofexercises.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.multiplechoice.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.pointsums.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.randomexercises.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-13635.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-155630.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-199360.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-299534.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-305110.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-308883.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-338165.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-350028.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-369065.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-369636.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-369803.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-370642.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-391530.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texsx-395273.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texwelt-15093.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.texwelt-6698.pdf",
    "texmf-dist/doc/latex/xsim/examples/xsim.various.pdf",
    "texmf-dist/doc/latex/xsim/xsim_manual.pdf",
  },
  ["xsavebox"] = {
    "texmf-dist/doc/latex/xsavebox/ChangeLog",
    "texmf-dist/doc/latex/xsavebox/README",
    "texmf-dist/doc/latex/xsavebox/xsavebox.pdf",
  },
  ["m-tx"] = {
    "texmf-dist/doc/generic/m-tx/Makefile",
    "texmf-dist/doc/generic/m-tx/README",
    "texmf-dist/doc/generic/m-tx/README.documentation",
    "texmf-dist/doc/generic/m-tx/make-dvi",
    "texmf-dist/doc/generic/m-tx/make-pdf",
    "texmf-dist/doc/generic/m-tx/make-target",
    "texmf-dist/doc/generic/m-tx/mtx-install.pdf",
    "texmf-dist/doc/generic/m-tx/mtxdoc.pdf",
    "texmf-dist/doc/generic/m-tx/pdfcat",
    "texmf-dist/doc/generic/m-tx/prepmx.pdf",
    "texmf-dist/doc/man/man1/prepmx.man1.pdf",
  },
  ["dirtree"] = {
    "texmf-dist/doc/generic/dirtree/README",
    "texmf-dist/doc/generic/dirtree/dirtree.pdf",
  },
  ["iwona"] = {
    "texmf-dist/doc/fonts/iwona/00readme.eng",
    "texmf-dist/doc/fonts/iwona/00readme.pol",
    "texmf-dist/doc/fonts/iwona/GUST-FONT-LICENSE.txt",
    "texmf-dist/doc/fonts/iwona/iwona-info.pdf",
    "texmf-dist/doc/fonts/iwona/manifest.txt",
  },
  ["xq"] = {
    "texmf-dist/doc/fonts/xq/ChangeLog",
    "texmf-dist/doc/fonts/xq/Makefile",
    "texmf-dist/doc/fonts/xq/README",
    "texmf-dist/doc/fonts/xq/bugs-0-3.pdf",
    "texmf-dist/doc/fonts/xq/xqexample.pdf",
  },
  ["normalcolor"] = {
    "texmf-dist/doc/latex/normalcolor/README",
    "texmf-dist/doc/latex/normalcolor/normalcolor.pdf",
  },
  ["xprintlen"] = {
    "texmf-dist/doc/latex/xprintlen/LICENSE",
    "texmf-dist/doc/latex/xprintlen/README",
    "texmf-dist/doc/latex/xprintlen/xprintlen.pdf",
  },
  ["mathabx-type1"] = {
    "texmf-dist/doc/fonts/mathabx-type1/README",
  },
  ["xpicture"] = {
    "texmf-dist/doc/latex/xpicture/README",
    "texmf-dist/doc/latex/xpicture/xpicture-doc.pdf",
    "texmf-dist/doc/latex/xpicture/xpicture.pdf",
  },
  ["xpiano"] = {
    "texmf-dist/doc/latex/xpiano/README.txt",
    "texmf-dist/doc/latex/xpiano/xpiano.pdf",
  },
  ["xpeek"] = {
    "texmf-dist/doc/latex/xpeek/README",
    "texmf-dist/doc/latex/xpeek/xpeek.pdf",
  },
  ["easyfig"] = {
    "texmf-dist/doc/latex/easyfig/README",
    "texmf-dist/doc/latex/easyfig/easyfig.pdf",
  },
  ["xnewcommand"] = {
    "texmf-dist/doc/latex/xnewcommand/README",
    "texmf-dist/doc/latex/xnewcommand/xnewcommand.pdf",
  },
  ["exteps"] = {
    "texmf-dist/doc/metapost/exteps/LICENSE",
    "texmf-dist/doc/metapost/exteps/README",
    "texmf-dist/doc/metapost/exteps/delfin",
    "texmf-dist/doc/metapost/exteps/exteps.pdf",
  },
  ["fontloader-luaotfload"] = {
    "texmf-dist/doc/luatex/fontloader-luaotfload/README.md",
    "texmf-dist/doc/luatex/fontloader-luaotfload/fontloader-luaotfload.pdf",
    "texmf-dist/doc/luatex/fontloader-luaotfload/test-fontloader-2017-02-11-stix.pdf",
    "texmf-dist/doc/luatex/fontloader-luaotfload/test-fontloader-2017-07-28.pdf",
    "texmf-dist/doc/luatex/fontloader-luaotfload/test-fontloader-2017-08-18.pdf",
  },
  ["amslatex-primer"] = {
    "texmf-dist/doc/latex/amslatex-primer/README",
    "texmf-dist/doc/latex/amslatex-primer/amshelp.pdf",
  },
  ["match_parens"] = {
    "texmf-dist/doc/support/match_parens/README",
    "texmf-dist/doc/support/match_parens/match_parens.pdf",
  },
  ["bookdb"] = {
    "texmf-dist/doc/bibtex/bookdb/README",
    "texmf-dist/doc/bibtex/bookdb/bookdb.pdf",
  },
  ["tkz-tab"] = {
    "texmf-dist/doc/latex/tkz-tab/README",
    "texmf-dist/doc/latex/tkz-tab/readme-us.txt",
    "texmf-dist/doc/latex/tkz-tab/tkz-tab-screen.pdf",
  },
  ["texlive-pl"] = {
    "texmf-dist/doc/texlive/texlive-pl/Makefile",
    "texmf-dist/doc/texlive/texlive-pl/texlive-pl.html",
    "texmf-dist/doc/texlive/texlive-pl/texlive-pl.pdf",
  },
  ["babel-serbianc"] = {
    "texmf-dist/doc/generic/babel-serbianc/COPYING",
    "texmf-dist/doc/generic/babel-serbianc/ChangeLog",
    "texmf-dist/doc/generic/babel-serbianc/Copyright",
    "texmf-dist/doc/generic/babel-serbianc/INSTALL",
    "texmf-dist/doc/generic/babel-serbianc/README",
    "texmf-dist/doc/generic/babel-serbianc/README.Files",
    "texmf-dist/doc/generic/babel-serbianc/README.serbianc",
    "texmf-dist/doc/generic/babel-serbianc/sample.pdf",
  },
  ["awesomebox"] = {
    "texmf-dist/doc/latex/awesomebox/LICENSE",
    "texmf-dist/doc/latex/awesomebox/README.md",
    "texmf-dist/doc/latex/awesomebox/awesomebox.pdf",
  },
  ["commath"] = {
    "texmf-dist/doc/latex/commath/README",
    "texmf-dist/doc/latex/commath/commath.pdf",
  },
  ["datetime2-finnish"] = {
    "texmf-dist/doc/latex/datetime2-finnish/CHANGES",
    "texmf-dist/doc/latex/datetime2-finnish/README",
    "texmf-dist/doc/latex/datetime2-finnish/datetime2-finnish.pdf",
  },
  ["xmltex"] = {
    "texmf-dist/doc/otherformats/xmltex/base/manual.html",
    "texmf-dist/doc/otherformats/xmltex/base/readme.txt",
  },
  ["xltxtra"] = {
    "texmf-dist/doc/latex/xltxtra/README",
    "texmf-dist/doc/latex/xltxtra/xltxtra.pdf",
  },
  ["zhmetrics-uptex"] = {
    "texmf-dist/doc/fonts/zhmetrics-uptex/README.md",
    "texmf-dist/doc/fonts/zhmetrics-uptex/doc/README.md",
    "texmf-dist/doc/fonts/zhmetrics-uptex/upzhfandolfonts-test.pdf",
  },
  ["xlop"] = {
    "texmf-dist/doc/generic/xlop/LISEZMOI",
    "texmf-dist/doc/generic/xlop/README",
    "texmf-dist/doc/generic/xlop/xlop-doc-fr.pdf",
    "texmf-dist/doc/generic/xlop/xlop-doc.pdf",
  },
  ["locality"] = {
    "texmf-dist/doc/latex/locality/README",
    "texmf-dist/doc/latex/locality/changes.txt",
    "texmf-dist/doc/latex/locality/locality.pdf",
  },
  ["xkeyval"] = {
    "texmf-dist/doc/latex/xkeyval/README",
    "texmf-dist/doc/latex/xkeyval/xkeyval.pdf",
  },
  ["filehook"] = {
    "texmf-dist/doc/latex/filehook/README",
    "texmf-dist/doc/latex/filehook/filehook.pdf",
  },
  ["biblatex-subseries"] = {
    "texmf-dist/doc/latex/biblatex-subseries/README",
    "texmf-dist/doc/latex/biblatex-subseries/documentation/biblatex-subseries-example.pdf",
    "texmf-dist/doc/latex/biblatex-subseries/documentation/biblatex-subseries.pdf",
    "texmf-dist/doc/latex/biblatex-subseries/documentation/makefile",
    "texmf-dist/doc/latex/biblatex-subseries/makefile",
  },
  ["xint"] = {
    "texmf-dist/doc/generic/xint/CHANGES.html",
    "texmf-dist/doc/generic/xint/CHANGES.pdf",
    "texmf-dist/doc/generic/xint/README",
    "texmf-dist/doc/generic/xint/README.html",
    "texmf-dist/doc/generic/xint/README.pdf",
    "texmf-dist/doc/generic/xint/sourcexint.pdf",
    "texmf-dist/doc/generic/xint/xint.pdf",
  },
  ["wasysym"] = {
    "texmf-dist/doc/latex/wasysym/wasysym.pdf",
  },
  ["coolthms"] = {
    "texmf-dist/doc/latex/coolthms/README",
    "texmf-dist/doc/latex/coolthms/coolthms.pdf",
  },
  ["splines"] = {
    "texmf-dist/doc/metapost/splines/README",
    "texmf-dist/doc/metapost/splines/splines.pdf",
  },
  ["xindy"] = {
    "texmf-dist/doc/man/man1/tex2xindy.man1.pdf",
    "texmf-dist/doc/man/man1/texindy.man1.pdf",
    "texmf-dist/doc/man/man1/xindy.man1.pdf",
    "texmf-dist/doc/xindy/README",
    "texmf-dist/doc/xindy/alphabets-doc.pdf",
    "texmf-dist/doc/xindy/faq-1.html",
    "texmf-dist/doc/xindy/faq-2.html",
    "texmf-dist/doc/xindy/faq-3.html",
    "texmf-dist/doc/xindy/faq-4.html",
    "texmf-dist/doc/xindy/faq.html",
    "texmf-dist/doc/xindy/makeindex4.pdf",
    "texmf-dist/doc/xindy/manual-1.html",
    "texmf-dist/doc/xindy/manual-2.html",
    "texmf-dist/doc/xindy/manual-3.html",
    "texmf-dist/doc/xindy/manual-4.html",
    "texmf-dist/doc/xindy/manual-5.html",
    "texmf-dist/doc/xindy/manual-6.html",
    "texmf-dist/doc/xindy/manual-7.html",
    "texmf-dist/doc/xindy/manual.html",
    "texmf-dist/doc/xindy/style-tutorial-1.html",
    "texmf-dist/doc/xindy/style-tutorial-2.html",
    "texmf-dist/doc/xindy/style-tutorial-3.html",
    "texmf-dist/doc/xindy/style-tutorial-4.html",
    "texmf-dist/doc/xindy/style-tutorial.html",
    "texmf-dist/doc/xindy/tex2xindy.pdf",
    "texmf-dist/doc/xindy/texindy.pdf",
    "texmf-dist/doc/xindy/xindy.pdf",
  },
  ["xii"] = {
    "texmf-dist/doc/plain/xii/README.txt",
  },
  ["xifthen"] = {
    "texmf-dist/doc/latex/xifthen/README",
    "texmf-dist/doc/latex/xifthen/xifthen.pdf",
  },
  ["pdftex"] = {
    "texmf-dist/doc/man/man1/pdfetex.man1.pdf",
    "texmf-dist/doc/man/man1/pdftex.man1.pdf",
    "texmf-dist/doc/pdftex/NEWS",
    "texmf-dist/doc/pdftex/README",
    "texmf-dist/doc/pdftex/manual/ChangeLog",
    "texmf-dist/doc/pdftex/manual/Makefile",
    "texmf-dist/doc/pdftex/manual/README",
    "texmf-dist/doc/pdftex/manual/pdftex-a.pdf",
    "texmf-dist/doc/pdftex/manual/pdftex-syntax.txt",
    "texmf-dist/doc/pdftex/samplepdftex/pic.pdf",
    "texmf-dist/doc/pdftex/samplepdftex/tmp.pdf",
    "texmf-dist/doc/pdftex/tests/01-fake-interword-space/Makefile",
    "texmf-dist/doc/pdftex/tests/02-pdfmatch/Makefile",
    "texmf-dist/doc/pdftex/tests/03-deterministic-output/Makefile",
    "texmf-dist/doc/pdftex/tests/04-pdfsuppressptexinfo/Makefile",
    "texmf-dist/doc/pdftex/tests/05-mediabox/Makefile",
    "texmf-dist/doc/pdftex/tests/06-pkmap/Makefile",
    "texmf-dist/doc/pdftex/tests/06-pkmap/prepatch.pdf",
    "texmf-dist/doc/pdftex/tests/07-mapwarn/Makefile",
  },
  ["xgreek"] = {
    "texmf-dist/doc/xelatex/xgreek/README",
    "texmf-dist/doc/xelatex/xgreek/xgreek.pdf",
  },
  ["cyber"] = {
    "texmf-dist/doc/latex/cyber/LICENSE",
    "texmf-dist/doc/latex/cyber/README",
    "texmf-dist/doc/latex/cyber/cyber.pdf",
  },
  ["apa6"] = {
    "texmf-dist/doc/latex/apa6/README",
    "texmf-dist/doc/latex/apa6/apa6.pdf",
    "texmf-dist/doc/latex/apa6/samples/Figure1.pdf",
  },
  ["dvipdfmx"] = {
    "texmf-dist/doc/dvipdfm/Makefile",
    "texmf-dist/doc/dvipdfm/dvipdfm.pdf",
    "texmf-dist/doc/dvipdfm/something.pdf",
    "texmf-dist/doc/dvipdfm/transistor.pdf",
    "texmf-dist/doc/dvipdfmx/dvipdfmx-special.pdf",
    "texmf-dist/doc/dvipdfmx/dvipdfmx.pdf",
    "texmf-dist/doc/dvipdfmx/fdl-1.3.txt",
    "texmf-dist/doc/dvipdfmx/tug2003-slides.pdf",
    "texmf-dist/doc/dvipdfmx/tug2005.pdf",
    "texmf-dist/doc/man/man1/dvipdfm.man1.pdf",
    "texmf-dist/doc/man/man1/dvipdfmx.man1.pdf",
    "texmf-dist/doc/man/man1/dvipdft.man1.pdf",
    "texmf-dist/doc/man/man1/ebb.man1.pdf",
    "texmf-dist/doc/man/man1/extractbb.man1.pdf",
    "texmf-dist/doc/man/man1/xdvipdfmx.man1.pdf",
  },
  ["urcls"] = {
    "texmf-dist/doc/latex/urcls/README.md",
    "texmf-dist/doc/latex/urcls/URbeamer-DEMO.pdf",
    "texmf-dist/doc/latex/urcls/URletter-DEMO.pdf",
    "texmf-dist/doc/latex/urcls/URpagestyles-DEMO.pdf",
  },
  ["xetexref"] = {
    "texmf-dist/doc/xetex/xetexref/README",
    "texmf-dist/doc/xetex/xetexref/xetex-reference.pdf",
  },
  ["youngtab"] = {
    "texmf-dist/doc/latex/youngtab/README",
    "texmf-dist/doc/latex/youngtab/makeydoc",
    "texmf-dist/doc/latex/youngtab/youngtab.pdf",
  },
  ["drawmatrix"] = {
    "texmf-dist/doc/latex/drawmatrix/README.md",
    "texmf-dist/doc/latex/drawmatrix/drawmatrix.pdf",
  },
  ["directory"] = {
    "texmf-dist/doc/latex/directory/README",
    "texmf-dist/doc/latex/directory/directory.pdf",
  },
  ["luabidi"] = {
    "texmf-dist/doc/lualatex/luabidi/README",
  },
  ["comma"] = {
    "texmf-dist/doc/latex/comma/README",
    "texmf-dist/doc/latex/comma/comma.pdf",
  },
  ["findhyph"] = {
    "texmf-dist/doc/man/man1/findhyph.man1.pdf",
    "texmf-dist/doc/support/findhyph/README",
    "texmf-dist/doc/support/findhyph/findhyph.txt",
  },
  ["frame"] = {
    "texmf-dist/doc/generic/frame/Changes",
    "texmf-dist/doc/generic/frame/Makefile",
    "texmf-dist/doc/generic/frame/README",
    "texmf-dist/doc/generic/frame/frame-doc.pdf",
  },
  ["newtxtt"] = {
    "texmf-dist/doc/fonts/newtxtt/README",
    "texmf-dist/doc/fonts/newtxtt/newtxtt-doc.pdf",
  },
  ["captcont"] = {
    "texmf-dist/doc/latex/captcont/README",
    "texmf-dist/doc/latex/captcont/captcont.pdf",
  },
  ["astro"] = {
    "texmf-dist/doc/fonts/astro/astrosym.txt",
  },
  ["uni-wtal-ger"] = {
    "texmf-dist/doc/latex/uni-wtal-ger/CHANGES",
    "texmf-dist/doc/latex/uni-wtal-ger/LIESMICH",
    "texmf-dist/doc/latex/uni-wtal-ger/README",
    "texmf-dist/doc/latex/uni-wtal-ger/germanistik.pdf",
  },
  ["rotfloat"] = {
    "texmf-dist/doc/latex/rotfloat/rotfloat.pdf",
  },
  ["baskervaldx"] = {
    "texmf-dist/doc/fonts/baskervaldx/COPYING",
    "texmf-dist/doc/fonts/baskervaldx/NOTICE.txt",
    "texmf-dist/doc/fonts/baskervaldx/README",
    "texmf-dist/doc/fonts/baskervaldx/baskervaldx-doc.pdf",
    "texmf-dist/doc/fonts/baskervaldx/baskervaldxmatheg-crop.pdf",
  },
  ["ametsoc"] = {
    "texmf-dist/doc/latex/ametsoc/README",
    "texmf-dist/doc/latex/ametsoc/README.TEXLIVE",
  },
  ["tocbibind"] = {
    "texmf-dist/doc/latex/tocbibind/README",
    "texmf-dist/doc/latex/tocbibind/tocbibind.pdf",
  },
  ["texlive-es"] = {
    "texmf-dist/doc/texlive/texlive-es/Makefile",
    "texmf-dist/doc/texlive/texlive-es/archive/texlive-es.html",
    "texmf-dist/doc/texlive/texlive-es/archive/texlive-es.pdf",
    "texmf-dist/doc/texlive/texlive-es/texlive-es.html",
    "texmf-dist/doc/texlive/texlive-es/texlive-es.pdf",
  },
  ["xetex-tibetan"] = {
    "texmf-dist/doc/xetex/xetex-tibetan/Changelog",
    "texmf-dist/doc/xetex/xetex-tibetan/README",
  },
  ["xetex-pstricks"] = {
    "texmf-dist/doc/xetex/xetex-pstricks/README",
  },
  ["pst-spirograph"] = {
    "texmf-dist/doc/generic/pst-spirograph/Changes",
    "texmf-dist/doc/generic/pst-spirograph/README",
    "texmf-dist/doc/generic/pst-spirograph/animation/README.anim",
    "texmf-dist/doc/generic/pst-spirograph/pst-spirograph-doc.pdf",
  },
  ["keyvaltable"] = {
    "texmf-dist/doc/latex/keyvaltable/README.md",
    "texmf-dist/doc/latex/keyvaltable/keyvaltable.pdf",
  },
  ["lua-alt-getopt"] = {
    "texmf-dist/doc/support/lua-alt-getopt/ChangeLog",
    "texmf-dist/doc/support/lua-alt-getopt/Makefile",
    "texmf-dist/doc/support/lua-alt-getopt/NEWS",
    "texmf-dist/doc/support/lua-alt-getopt/README",
    "texmf-dist/doc/support/lua-alt-getopt/alt_getopt",
  },
  ["bath-bst"] = {
    "texmf-dist/doc/bibtex/bath-bst/Makefile",
    "texmf-dist/doc/bibtex/bath-bst/README.md",
    "texmf-dist/doc/bibtex/bath-bst/bath-bst-v1.pdf",
    "texmf-dist/doc/bibtex/bath-bst/bath-bst.pdf",
  },
  ["xetex"] = {
    "texmf-dist/doc/man/man1/xelatex.man1.pdf",
    "texmf-dist/doc/xetex/base/NEWS",
    "texmf-dist/doc/xetex/base/README",
    "texmf-dist/doc/xetex/base/XeTeX-notes.pdf",
  },
  ["xespotcolor"] = {
    "texmf-dist/doc/xelatex/xespotcolor/README",
    "texmf-dist/doc/xelatex/xespotcolor/jatestspot.pdf",
    "texmf-dist/doc/xelatex/xespotcolor/spotcolor-test.pdf",
    "texmf-dist/doc/xelatex/xespotcolor/xespotcolor.pdf",
  },
  ["exsol"] = {
    "texmf-dist/doc/latex/exsol/LICENSE",
    "texmf-dist/doc/latex/exsol/README",
    "texmf-dist/doc/latex/exsol/example-formulacollection.pdf",
    "texmf-dist/doc/latex/exsol/example-local.pdf",
    "texmf-dist/doc/latex/exsol/example-solutionbook.pdf",
    "texmf-dist/doc/latex/exsol/example.pdf",
    "texmf-dist/doc/latex/exsol/exsol.pdf",
  },
  ["gitinfo2"] = {
    "texmf-dist/doc/latex/gitinfo2/README",
    "texmf-dist/doc/latex/gitinfo2/gitinfo2.pdf",
    "texmf-dist/doc/latex/gitinfo2/post-xxx-sample.txt",
  },
  ["cbfonts-fd"] = {
    "texmf-dist/doc/fonts/cbfonts-fd/README",
    "texmf-dist/doc/fonts/cbfonts-fd/cbfonts-fd.pdf",
    "texmf-dist/doc/fonts/cbfonts-fd/manifest.txt",
  },
  ["xesearch"] = {
    "texmf-dist/doc/xetex/xesearch/README",
    "texmf-dist/doc/xetex/xesearch/xesearch.pdf",
  },
  ["subdepth"] = {
    "texmf-dist/doc/latex/subdepth/README",
    "texmf-dist/doc/latex/subdepth/subdepth.pdf",
  },
  ["xepersian"] = {
    "texmf-dist/doc/xelatex/xepersian/README",
    "texmf-dist/doc/xelatex/xepersian/xepersian-doc.pdf",
    "texmf-dist/doc/xelatex/xepersian/xepersian.pdf",
  },
  ["xellipsis"] = {
    "texmf-dist/doc/latex/xellipsis/CHANGES",
    "texmf-dist/doc/latex/xellipsis/README",
    "texmf-dist/doc/latex/xellipsis/lppl.txt",
    "texmf-dist/doc/latex/xellipsis/xellipsis.pdf",
  },
  ["xeindex"] = {
    "texmf-dist/doc/xelatex/xeindex/README",
    "texmf-dist/doc/xelatex/xeindex/xeindex.pdf",
  },
  ["datetime2-german"] = {
    "texmf-dist/doc/latex/datetime2-german/README.md",
    "texmf-dist/doc/latex/datetime2-german/datetime2-german.pdf",
  },
  ["chicago-annote"] = {
    "texmf-dist/doc/bibtex/chicago-annote/README",
  },
  ["pst-calculate"] = {
    "texmf-dist/doc/generic/pst-calculate/Changes",
    "texmf-dist/doc/generic/pst-calculate/README.md",
    "texmf-dist/doc/generic/pst-calculate/pst-calculate-doc.pdf",
  },
  ["xecyr"] = {
    "texmf-dist/doc/xelatex/xecyr/NEWS",
    "texmf-dist/doc/xelatex/xecyr/README",
    "texmf-dist/doc/xelatex/xecyr/listings-utf8-ex.pdf",
    "texmf-dist/doc/xelatex/xecyr/pict2e-ex.pdf",
    "texmf-dist/doc/xelatex/xecyr/rubibtex-ex-x.pdf",
    "texmf-dist/doc/xelatex/xecyr/rumakeindex-ex-x.pdf",
    "texmf-dist/doc/xelatex/xecyr/xecyr-doc-ru.pdf",
    "texmf-dist/doc/xelatex/xecyr/xecyr-ex1-ru-x.pdf",
    "texmf-dist/doc/xelatex/xecyr/xecyr-ex2-ru-x.pdf",
    "texmf-dist/doc/xelatex/xecyr/xecyr-ex3-ru-x.pdf",
    "texmf-dist/doc/xelatex/xecyr/xecyr-ex4-ru-x.pdf",
    "texmf-dist/doc/xelatex/xecyr/xecyr-ex5-ru-x.pdf",
    "texmf-dist/doc/xelatex/xecyr/xecyr-ex6-ru-x.pdf",
    "texmf-dist/doc/xelatex/xecyr/xecyr-ex7-ru-x.pdf",
  },
  ["pst-dart"] = {
    "texmf-dist/doc/generic/pst-dart/Changes",
    "texmf-dist/doc/generic/pst-dart/README.md",
    "texmf-dist/doc/generic/pst-dart/pst-dart-doc.pdf",
  },
  ["xecolor"] = {
    "texmf-dist/doc/xelatex/xecolor/README",
  },
  ["semioneside"] = {
    "texmf-dist/doc/latex/semioneside/README",
    "texmf-dist/doc/latex/semioneside/semioneside.pdf",
  },
  ["enigma"] = {
    "texmf-dist/doc/context/third/enigma/enigma/COPYING",
    "texmf-dist/doc/context/third/enigma/enigma/README",
    "texmf-dist/doc/context/third/enigma/enigma/enigma-doc.pdf",
  },
  ["apprends-latex"] = {
    "texmf-dist/doc/latex/apprends-latex/Apprends_LaTeX.pdf",
    "texmf-dist/doc/latex/apprends-latex/README",
    "texmf-dist/doc/latex/apprends-latex/exemples/Makefile",
    "texmf-dist/doc/latex/apprends-latex/latexmkrc",
  },
  ["lualatex-truncate"] = {
    "texmf-dist/doc/lualatex/lualatex-truncate/README.md",
    "texmf-dist/doc/lualatex/lualatex-truncate/lualatex-truncate-doc.pdf",
  },
  ["etoolbox-de"] = {
    "texmf-dist/doc/latex/etoolbox-de/etoolbox-DE.pdf",
  },
  ["theatre"] = {
    "texmf-dist/doc/latex/theatre/README",
    "texmf-dist/doc/latex/theatre/doc/Help_Theatre.pdf",
  },
  ["chemstyle"] = {
    "texmf-dist/doc/latex/chemstyle/README",
    "texmf-dist/doc/latex/chemstyle/chemstyle.pdf",
  },
  ["dnaseq"] = {
    "texmf-dist/doc/latex/dnaseq/README",
    "texmf-dist/doc/latex/dnaseq/dnaseq.pdf",
  },
  ["substitutefont"] = {
    "texmf-dist/doc/latex/substitutefont/README",
    "texmf-dist/doc/latex/substitutefont/cyrillic-lm-lgc.pdf",
    "texmf-dist/doc/latex/substitutefont/cyrillic-paratype.pdf",
    "texmf-dist/doc/latex/substitutefont/greek-palatino-didot.pdf",
    "texmf-dist/doc/latex/substitutefont/greek-times-artemisia.pdf",
    "texmf-dist/doc/latex/substitutefont/substitutefont-doc.html",
    "texmf-dist/doc/latex/substitutefont/substitutefont-test.pdf",
    "texmf-dist/doc/latex/substitutefont/substitutefont.sty.html",
  },
  ["pgf-umlsd"] = {
    "texmf-dist/doc/latex/pgf-umlsd/README",
    "texmf-dist/doc/latex/pgf-umlsd/diagrams.pdf",
    "texmf-dist/doc/latex/pgf-umlsd/pgf-umlsd-manual.pdf",
  },
  ["diagmac2"] = {
    "texmf-dist/doc/latex/diagmac2/README",
    "texmf-dist/doc/latex/diagmac2/doc/diagmac2.pdf",
    "texmf-dist/doc/latex/diagmac2/doc/diagmactest.pdf",
  },
  ["pst-qtree"] = {
    "texmf-dist/doc/generic/pst-qtree/CHANGES",
    "texmf-dist/doc/generic/pst-qtree/LICENSE",
    "texmf-dist/doc/generic/pst-qtree/pst-qtree-manual.pdf",
  },
  ["lcg"] = {
    "texmf-dist/doc/latex/lcg/lcg.pdf",
  },
  ["xduthesis"] = {
    "texmf-dist/doc/latex/xduthesis/README.md",
    "texmf-dist/doc/latex/xduthesis/examples/thesis-doctor.pdf",
    "texmf-dist/doc/latex/xduthesis/xduthesis.pdf",
  },
  ["jurarsp"] = {
    "texmf-dist/doc/latex/jurarsp/README",
    "texmf-dist/doc/latex/jurarsp/jurarsp.pdf",
  },
  ["latex4wp"] = {
    "texmf-dist/doc/latex/latex4wp/HOW-TO-TYPESET",
    "texmf-dist/doc/latex/latex4wp/README",
    "texmf-dist/doc/latex/latex4wp/gnuplot.pdf",
    "texmf-dist/doc/latex/latex4wp/latex4wp.pdf",
    "texmf-dist/doc/latex/latex4wp/tbx.pdf",
  },
  ["xdoc"] = {
    "texmf-dist/doc/latex/xdoc/README",
    "texmf-dist/doc/latex/xdoc/docindex.pdf",
    "texmf-dist/doc/latex/xdoc/xdoc2.pdf",
    "texmf-dist/doc/latex/xdoc/xdocdemo.pdf",
  },
  ["tree-dvips"] = {
    "texmf-dist/doc/latex/tree-dvips/Makefile",
    "texmf-dist/doc/latex/tree-dvips/README",
    "texmf-dist/doc/latex/tree-dvips/README.TEXLIVE",
    "texmf-dist/doc/latex/tree-dvips/lingmacros-manual.pdf",
    "texmf-dist/doc/latex/tree-dvips/tree-manual.pdf",
  },
  ["xcomment"] = {
    "texmf-dist/doc/generic/xcomment/Changes",
    "texmf-dist/doc/generic/xcomment/Makefile",
    "texmf-dist/doc/generic/xcomment/README",
    "texmf-dist/doc/generic/xcomment/xcomment-doc.pdf",
  },
  ["esdiff"] = {
    "texmf-dist/doc/latex/esdiff/README",
    "texmf-dist/doc/latex/esdiff/esdiff.pdf",
  },
  ["xcolor-solarized"] = {
    "texmf-dist/doc/latex/xcolor-solarized/README",
    "texmf-dist/doc/latex/xcolor-solarized/xcolor-solarized.pdf",
  },
  ["babel-ukrainian"] = {
    "texmf-dist/doc/generic/babel-ukrainian/README.md",
    "texmf-dist/doc/generic/babel-ukrainian/ukraineb.pdf",
  },
  ["xcolor-material"] = {
    "texmf-dist/doc/latex/xcolor-material/README.md",
    "texmf-dist/doc/latex/xcolor-material/xcolor-material.pdf",
  },
  ["xcolor"] = {
    "texmf-dist/doc/latex/xcolor/ChangeLog",
    "texmf-dist/doc/latex/xcolor/README",
    "texmf-dist/doc/latex/xcolor/xcolor.pdf",
    "texmf-dist/doc/latex/xcolor/xcolor2.pdf",
  },
  ["collectbox"] = {
    "texmf-dist/doc/latex/collectbox/README",
    "texmf-dist/doc/latex/collectbox/collectbox.pdf",
  },
  ["xcntperchap"] = {
    "texmf-dist/doc/latex/xcntperchap/README",
    "texmf-dist/doc/latex/xcntperchap/xcntperchap_basicusage.pdf",
    "texmf-dist/doc/latex/xcntperchap/xcntperchap_doc.pdf",
    "texmf-dist/doc/latex/xcntperchap/xcntperchap_driver.pdf",
    "texmf-dist/doc/latex/xcntperchap/xcntperchap_labelusage.pdf",
  },
  ["xcjk2uni"] = {
    "texmf-dist/doc/latex/xcjk2uni/README.md",
    "texmf-dist/doc/latex/xcjk2uni/xCJK2uni.pdf",
  },
  ["bibleref-german"] = {
    "texmf-dist/doc/latex/bibleref-german/CHANGES",
    "texmf-dist/doc/latex/bibleref-german/LIESMICH",
    "texmf-dist/doc/latex/bibleref-german/README",
    "texmf-dist/doc/latex/bibleref-german/de-bibleref-german.pdf",
    "texmf-dist/doc/latex/bibleref-german/en-bibleref-german.pdf",
  },
  ["rtkinenc"] = {
    "texmf-dist/doc/latex/rtkinenc/README.txt",
    "texmf-dist/doc/latex/rtkinenc/rtkinenc-doc.pdf",
  },
  ["matlab-prettifier"] = {
    "texmf-dist/doc/latex/matlab-prettifier/README",
    "texmf-dist/doc/latex/matlab-prettifier/matlab-prettifier.pdf",
  },
  ["siunits"] = {
    "texmf-dist/doc/latex/SIunits/README",
    "texmf-dist/doc/latex/SIunits/SIunits.pdf",
  },
  ["wtref"] = {
    "texmf-dist/doc/latex/wtref/LICENSE",
    "texmf-dist/doc/latex/wtref/README.md",
    "texmf-dist/doc/latex/wtref/wtref-ja.pdf",
    "texmf-dist/doc/latex/wtref/wtref.pdf",
  },
  ["wsuipa"] = {
    "texmf-dist/doc/fonts/wsuipa/LICENCE-wsuipa.txt",
    "texmf-dist/doc/fonts/wsuipa/README",
    "texmf-dist/doc/fonts/wsuipa/compilefonts",
    "texmf-dist/doc/fonts/wsuipa/ipaman.ps",
  },
  ["wsemclassic"] = {
    "texmf-dist/doc/latex/wsemclassic/LICENSE",
    "texmf-dist/doc/latex/wsemclassic/Makefile",
    "texmf-dist/doc/latex/wsemclassic/README",
    "texmf-dist/doc/latex/wsemclassic/wsemclassic-test.pdf",
    "texmf-dist/doc/latex/wsemclassic/wsemclassic.pdf",
  },
  ["academicons"] = {
    "texmf-dist/doc/fonts/academicons/README",
    "texmf-dist/doc/fonts/academicons/academicons.pdf",
  },
  ["pgf-soroban"] = {
    "texmf-dist/doc/latex/pgf-soroban/Changes",
    "texmf-dist/doc/latex/pgf-soroban/README",
    "texmf-dist/doc/latex/pgf-soroban/pgf-soroban-doc.pdf",
  },
  ["pkuthss"] = {
    "texmf-dist/doc/latex/pkuthss/example.pdf",
    "texmf-dist/doc/latex/pkuthss/example/Makefile",
    "texmf-dist/doc/latex/pkuthss/readme.pdf",
    "texmf-dist/doc/latex/pkuthss/readme/ChangeLog-upto-1.3.txt",
    "texmf-dist/doc/latex/pkuthss/readme/ChangeLog.txt",
    "texmf-dist/doc/latex/pkuthss/readme/Makefile",
  },
  ["context-handlecsv"] = {
    "texmf-dist/doc/context/third/handlecsv/VERSION",
    "texmf-dist/doc/context/third/handlecsv/handlecsv.pdf",
  },
  ["typehtml"] = {
    "texmf-dist/doc/latex/typehtml/README",
    "texmf-dist/doc/latex/typehtml/typehtml.pdf",
  },
  ["wordcount"] = {
    "texmf-dist/doc/latex/wordcount/README",
  },
  ["wnri-latex"] = {
    "texmf-dist/doc/latex/wnri-latex/README",
    "texmf-dist/doc/latex/wnri-latex/wnri.pdf",
  },
  ["bnumexpr"] = {
    "texmf-dist/doc/latex/bnumexpr/README",
    "texmf-dist/doc/latex/bnumexpr/README.md",
    "texmf-dist/doc/latex/bnumexpr/bnumexpr.pdf",
  },
  ["abraces"] = {
    "texmf-dist/doc/latex/abraces/abraces-doc.pdf",
  },
  ["termmenu"] = {
    "texmf-dist/doc/generic/termmenu/README",
    "texmf-dist/doc/generic/termmenu/termmenu.pdf",
  },
  ["nag"] = {
    "texmf-dist/doc/latex/nag/README",
    "texmf-dist/doc/latex/nag/nag.pdf",
  },
  ["wnri"] = {
    "texmf-dist/doc/fonts/wnri/README",
    "texmf-dist/doc/fonts/wnri/old/README",
  },
  ["witharrows"] = {
    "texmf-dist/doc/latex/witharrows/README.md",
    "texmf-dist/doc/latex/witharrows/witharrows.pdf",
  },
  ["coverpage"] = {
    "texmf-dist/doc/latex/coverpage/CoverPage.pdf",
    "texmf-dist/doc/latex/coverpage/ECCV06Sample.pdf",
    "texmf-dist/doc/latex/coverpage/README",
    "texmf-dist/doc/latex/coverpage/SimpleSample.BibTeX.txt",
    "texmf-dist/doc/latex/coverpage/SimpleSample.pdf",
  },
  ["withargs"] = {
    "texmf-dist/doc/latex/withargs/README",
    "texmf-dist/doc/latex/withargs/withargs.pdf",
  },
  ["perfectcut"] = {
    "texmf-dist/doc/latex/perfectcut/README",
    "texmf-dist/doc/latex/perfectcut/perfectcut.pdf",
  },
  ["colorinfo"] = {
    "texmf-dist/doc/latex/colorinfo/README",
  },
  ["fancyslides"] = {
    "texmf-dist/doc/latex/fancyslides/README",
    "texmf-dist/doc/latex/fancyslides/doc/example.pdf",
    "texmf-dist/doc/latex/fancyslides/doc/fancyslides.pdf",
    "texmf-dist/doc/latex/fancyslides/doc/fs.pdf",
  },
  ["widetable"] = {
    "texmf-dist/doc/latex/widetable/README",
    "texmf-dist/doc/latex/widetable/manifest.txt",
    "texmf-dist/doc/latex/widetable/widetable.pdf",
  },
  ["css-colors"] = {
    "texmf-dist/doc/latex/css-colors/README.txt",
    "texmf-dist/doc/latex/css-colors/css-colors.pdf",
  },
  ["cherokee"] = {
    "texmf-dist/doc/fonts/cherokee/README",
  },
  ["statex2"] = {
    "texmf-dist/doc/latex/statex2/statex2-example.pdf",
  },
  ["pst-graphicx"] = {
    "texmf-dist/doc/generic/pst-graphicx/Changes",
    "texmf-dist/doc/generic/pst-graphicx/README",
  },
  ["ku-template"] = {
    "texmf-dist/doc/latex/ku-template/README.txt",
    "texmf-dist/doc/latex/ku-template/images/hum-da.pdf",
    "texmf-dist/doc/latex/ku-template/images/hum-en.pdf",
    "texmf-dist/doc/latex/ku-template/images/jur-da.pdf",
    "texmf-dist/doc/latex/ku-template/images/jur-en.pdf",
    "texmf-dist/doc/latex/ku-template/images/ku-da.pdf",
    "texmf-dist/doc/latex/ku-template/images/ku-en.pdf",
    "texmf-dist/doc/latex/ku-template/images/sam-da.pdf",
    "texmf-dist/doc/latex/ku-template/images/sam-en.pdf",
    "texmf-dist/doc/latex/ku-template/images/sci-da.pdf",
    "texmf-dist/doc/latex/ku-template/images/sci-en.pdf",
    "texmf-dist/doc/latex/ku-template/images/sun-da.pdf",
    "texmf-dist/doc/latex/ku-template/images/sun-en.pdf",
    "texmf-dist/doc/latex/ku-template/images/teo-da.pdf",
    "texmf-dist/doc/latex/ku-template/images/teo-en.pdf",
  },
  ["ydoc"] = {
    "texmf-dist/doc/latex/ydoc/README",
    "texmf-dist/doc/latex/ydoc/ydoc.pdf",
  },
  ["koma-moderncvclassic"] = {
    "texmf-dist/doc/latex/koma-moderncvclassic/README",
    "texmf-dist/doc/latex/koma-moderncvclassic/changelog",
    "texmf-dist/doc/latex/koma-moderncvclassic/cvbasic.pdf",
  },
  ["glossaries"] = {
    "texmf-dist/doc/latex/glossaries/CHANGES",
    "texmf-dist/doc/latex/glossaries/INSTALL",
    "texmf-dist/doc/latex/glossaries/README",
    "texmf-dist/doc/latex/glossaries/glossaries-code.pdf",
    "texmf-dist/doc/latex/glossaries/glossaries-user.html",
    "texmf-dist/doc/latex/glossaries/glossaries-user.pdf",
    "texmf-dist/doc/latex/glossaries/glossariesbegin.html",
    "texmf-dist/doc/latex/glossaries/glossariesbegin.pdf",
    "texmf-dist/doc/latex/glossaries/glossary2glossaries.html",
    "texmf-dist/doc/latex/glossaries/glossary2glossaries.pdf",
    "texmf-dist/doc/latex/glossaries/samples/README-samples",
    "texmf-dist/doc/latex/glossaries/samples/glossary-lipsum-examples.pdf",
    "texmf-dist/doc/latex/glossaries/samples/minimalgls.pdf",
    "texmf-dist/doc/latex/glossaries/samples/mwe-acr-desc.pdf",
    "texmf-dist/doc/latex/glossaries/samples/mwe-acr.pdf",
    "texmf-dist/doc/latex/glossaries/samples/mwe-gls.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-FnDesc.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-chap-hyperfirst.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-crossref.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-custom-acronym.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-dot-abbr.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-dual.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-entrycount.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-entryfmt.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-font-abbr.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-ignored.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-index.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-inline.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-langdict.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-newkeys.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-noidxapp-utf8.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-noidxapp.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-nomathhyper.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-numberlist.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-prefix.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-si.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-storage-abbr-desc.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample-storage-abbr.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sample4col.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleAcr.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleAcrDesc.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleCustomAcr.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleDB.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleDesc.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleEq.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleEqPg.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleFnAcrDesc.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleNtn.pdf",
    "texmf-dist/doc/latex/glossaries/samples/samplePeople.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleSec.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleSort.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleaccsupp.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleacronyms.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampletree.pdf",
    "texmf-dist/doc/latex/glossaries/samples/sampleutf8.pdf",
    "texmf-dist/doc/latex/glossaries/samples/samplexdy.pdf",
    "texmf-dist/doc/latex/glossaries/samples/samplexdy2.pdf",
    "texmf-dist/doc/latex/glossaries/samples/samplexdy3.pdf",
    "texmf-dist/doc/man/man1/makeglossaries-lite.man1.pdf",
    "texmf-dist/doc/man/man1/makeglossaries.man1.pdf",
  },
  ["xypic-tut-pt"] = {
    "texmf-dist/doc/generic/xypic-tut-pt/README",
    "texmf-dist/doc/generic/xypic-tut-pt/xypic-tutorial.pdf",
  },
  ["glossaries-german"] = {
    "texmf-dist/doc/latex/glossaries-german/README",
    "texmf-dist/doc/latex/glossaries-german/glossaries-german.pdf",
  },
  ["js-misc"] = {
    "texmf-dist/doc/plain/js-misc/History",
    "texmf-dist/doc/plain/js-misc/INSTALL",
    "texmf-dist/doc/plain/js-misc/Imakefile",
    "texmf-dist/doc/plain/js-misc/License",
    "texmf-dist/doc/plain/js-misc/Makefile",
    "texmf-dist/doc/plain/js-misc/README",
    "texmf-dist/doc/plain/js-misc/TODO",
  },
  ["web"] = {
    "texmf-dist/doc/man/man1/tangle.man1.pdf",
    "texmf-dist/doc/man/man1/weave.man1.pdf",
  },
  ["afparticle"] = {
    "texmf-dist/doc/latex/afparticle/Makefile",
    "texmf-dist/doc/latex/afparticle/README",
    "texmf-dist/doc/latex/afparticle/afparticle.pdf",
    "texmf-dist/doc/latex/afparticle/afpsample.pdf",
  },
  ["latexbangla"] = {
    "texmf-dist/doc/latex/latexbangla/README",
    "texmf-dist/doc/latex/latexbangla/latexbangla.pdf",
  },
  ["de-macro"] = {
    "texmf-dist/doc/support/de-macro/README",
    "texmf-dist/doc/support/de-macro/user-guide.pdf",
  },
  ["envbig"] = {
    "texmf-dist/doc/latex/envbig/README",
  },
  ["comment"] = {
    "texmf-dist/doc/latex/comment/Makefile",
    "texmf-dist/doc/latex/comment/README",
    "texmf-dist/doc/latex/comment/comm_test.pdf",
    "texmf-dist/doc/latex/comment/comment.pdf",
    "texmf-dist/doc/latex/comment/t1test.pdf",
    "texmf-dist/doc/latex/comment/writeup.pdf",
  },
  ["luamplib"] = {
    "texmf-dist/doc/luatex/luamplib/NEWS",
    "texmf-dist/doc/luatex/luamplib/README",
    "texmf-dist/doc/luatex/luamplib/luamplib.pdf",
  },
  ["wasy2-ps"] = {
    "texmf-dist/doc/fonts/wasy2-ps/README",
  },
  ["wasy"] = {
    "texmf-dist/doc/fonts/wasy/legal.txt",
    "texmf-dist/doc/fonts/wasy/wasydoc.pdf",
  },
  ["was"] = {
    "texmf-dist/doc/latex/was/fixmath.pdf",
    "texmf-dist/doc/latex/was/gensymb.pdf",
    "texmf-dist/doc/latex/was/icomma.pdf",
    "texmf-dist/doc/latex/was/readme.1st",
    "texmf-dist/doc/latex/was/upgreek.pdf",
  },
  ["warning"] = {
    "texmf-dist/doc/latex/warning/warning-doc.pdf",
  },
  ["wallpaper"] = {
    "texmf-dist/doc/latex/wallpaper/README",
    "texmf-dist/doc/latex/wallpaper/wallpapermanual.pdf",
  },
  ["niceframe-type1"] = {
    "texmf-dist/doc/fonts/niceframe-type1/README.md",
  },
  ["tableaux"] = {
    "texmf-dist/doc/latex/tableaux/exemples.pdf",
    "texmf-dist/doc/latex/tableaux/tableau.pdf",
  },
  ["biblatex-musuos"] = {
    "texmf-dist/doc/latex/biblatex-musuos/README",
    "texmf-dist/doc/latex/biblatex-musuos/biblatex-musuos.pdf",
  },
  ["fouridx"] = {
    "texmf-dist/doc/latex/fouridx/README",
    "texmf-dist/doc/latex/fouridx/fouridx.pdf",
  },
  ["margbib"] = {
    "texmf-dist/doc/latex/margbib/margbib.pdf",
  },
  ["a4wide"] = {
    "texmf-dist/doc/latex/a4wide/a4wide.pdf",
  },
  ["vwcol"] = {
    "texmf-dist/doc/latex/vwcol/README",
    "texmf-dist/doc/latex/vwcol/vwcol.pdf",
  },
  ["vruler"] = {
    "texmf-dist/doc/latex/vruler/vruler.pdf",
  },
  ["dramatist"] = {
    "texmf-dist/doc/latex/dramatist/README",
    "texmf-dist/doc/latex/dramatist/dramatist.pdf",
  },
  ["babel-georgian"] = {
    "texmf-dist/doc/generic/babel-georgian/README",
  },
  ["rccol"] = {
    "texmf-dist/doc/latex/rccol/README",
    "texmf-dist/doc/latex/rccol/rccol.pdf",
  },
  ["williams"] = {
    "texmf-dist/doc/latex/williams/README",
  },
  ["pythontex"] = {
    "texmf-dist/doc/latex/pythontex/README",
    "texmf-dist/doc/latex/pythontex/pythontex.pdf",
    "texmf-dist/doc/latex/pythontex/pythontex_gallery.pdf",
    "texmf-dist/doc/latex/pythontex/pythontex_quickstart.pdf",
  },
  ["boolexpr"] = {
    "texmf-dist/doc/latex/boolexpr/README",
    "texmf-dist/doc/latex/boolexpr/boolexpr.pdf",
  },
  ["tex-label"] = {
    "texmf-dist/doc/latex/tex-label/README",
    "texmf-dist/doc/latex/tex-label/tex-label-demo.pdf",
    "texmf-dist/doc/latex/tex-label/tex-label-doc.pdf",
  },
  ["denisbdoc"] = {
    "texmf-dist/doc/latex/denisbdoc/README.md",
  },
  ["biblatex-gost"] = {
    "texmf-dist/doc/latex/biblatex-gost/README.md",
    "texmf-dist/doc/latex/biblatex-gost/biblatex-gost-examples.pdf",
    "texmf-dist/doc/latex/biblatex-gost/biblatex-gost.pdf",
  },
  ["biblatex-archaeology"] = {
    "texmf-dist/doc/latex/biblatex-archaeology/README.htm",
    "texmf-dist/doc/latex/biblatex-archaeology/README.md",
    "texmf-dist/doc/latex/biblatex-archaeology/biblatex-archaeology.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/aefkw-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/afwl-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/amit-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/archa-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/authoryear-archaeology-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/authoryear-comp-archaeology-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/authoryear-ibid-archaeology-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/authoryear-icomp-archaeology-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/dguf-alt-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/dguf-apa-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/dguf-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/eaz-alt-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/eaz-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/foe-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/jb-halle-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/jb-kreis-neuss-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/karl-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/maja-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/mpk-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/mpkoeaw-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/nnu-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/numeric-comp-archaeology-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/offa-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/rgk-inline-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/rgk-numeric-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/rgk-verbose-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/rgzm-inline-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/rgzm-numeric-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/rgzm-verbose-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/ufg-muenster-inline-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/ufg-muenster-numeric-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/ufg-muenster-verbose-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/verbose-archaeology-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/verbose-ibid-archaeology-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/verbose-trad2note-archaeology-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/volkskunde-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/zaak-example.pdf",
    "texmf-dist/doc/latex/biblatex-archaeology/example/zaes-example.pdf",
  },
  ["txgreeks"] = {
    "texmf-dist/doc/latex/txgreeks/README",
    "texmf-dist/doc/latex/txgreeks/txgreeks.pdf",
  },
  ["ghab"] = {
    "texmf-dist/doc/latex/ghab/README",
    "texmf-dist/doc/latex/ghab/ghab-doc.pdf",
  },
  ["gbt7714"] = {
    "texmf-dist/doc/bibtex/gbt7714/LICENSE",
    "texmf-dist/doc/bibtex/gbt7714/README.md",
    "texmf-dist/doc/bibtex/gbt7714/gbt7714.pdf",
  },
  ["easyformat"] = {
    "texmf-dist/doc/latex/easyformat/README",
    "texmf-dist/doc/latex/easyformat/easyformat.pdf",
  },
  ["volumes"] = {
    "texmf-dist/doc/latex/volumes/README",
    "texmf-dist/doc/latex/volumes/volumes.pdf",
  },
  ["labels"] = {
    "texmf-dist/doc/latex/labels/README",
    "texmf-dist/doc/latex/labels/labels.pdf",
  },
  ["latexcheat-esmx"] = {
    "texmf-dist/doc/latex/latexcheat-esmx/README",
    "texmf-dist/doc/latex/latexcheat-esmx/latexsheet-esmx.pdf",
  },
  ["datetime2-greek"] = {
    "texmf-dist/doc/latex/datetime2-greek/README",
    "texmf-dist/doc/latex/datetime2-greek/datetime2-greek.pdf",
  },
  ["vntex"] = {
    "texmf-dist/doc/generic/vntex/INSTALL",
    "texmf-dist/doc/generic/vntex/ReleaseNotes.pdf",
    "texmf-dist/doc/generic/vntex/vn-fonts-print.pdf",
    "texmf-dist/doc/generic/vntex/vn-fonts.pdf",
    "texmf-dist/doc/generic/vntex/vn-min-print.pdf",
    "texmf-dist/doc/generic/vntex/vn-min.pdf",
    "texmf-dist/doc/generic/vntex/vntex-man-print.pdf",
    "texmf-dist/doc/generic/vntex/vntex-man.pdf",
    "texmf-dist/doc/generic/vntex/vntex-print.pdf",
    "texmf-dist/doc/generic/vntex/vntex-update-maps",
    "texmf-dist/doc/generic/vntex/vntex.pdf",
  },
  ["ieeeconf"] = {
    "texmf-dist/doc/latex/IEEEconf/IEEEconf.pdf",
    "texmf-dist/doc/latex/IEEEconf/README",
  },
  ["euro-ce"] = {
    "texmf-dist/doc/fonts/euro-ce/euro-ce.dvi",
  },
  ["pdfscreen"] = {
    "texmf-dist/doc/latex/pdfscreen/logo.pdf",
    "texmf-dist/doc/latex/pdfscreen/manual-print.pdf",
    "texmf-dist/doc/latex/pdfscreen/manual-screen.pdf",
    "texmf-dist/doc/latex/pdfscreen/nopanel.pdf",
    "texmf-dist/doc/latex/pdfscreen/portrait.pdf",
    "texmf-dist/doc/latex/pdfscreen/print.pdf",
    "texmf-dist/doc/latex/pdfscreen/slide.pdf",
    "texmf-dist/doc/latex/pdfscreen/square.pdf",
    "texmf-dist/doc/latex/pdfscreen/widepanel.pdf",
  },
  ["parnotes"] = {
    "texmf-dist/doc/latex/parnotes/README.md",
    "texmf-dist/doc/latex/parnotes/parnotes.pdf",
  },
  ["classpack"] = {
    "texmf-dist/doc/support/classpack/MANIFEST",
    "texmf-dist/doc/support/classpack/README",
    "texmf-dist/doc/support/classpack/classpack.pdf",
    "texmf-dist/doc/support/classpack/readme.xml",
  },
  ["exceltex"] = {
    "texmf-dist/doc/latex/exceltex/CHANGES",
    "texmf-dist/doc/latex/exceltex/INSTALL",
    "texmf-dist/doc/latex/exceltex/README",
    "texmf-dist/doc/latex/exceltex/README.TEXLIVE",
  },
  ["kerntest"] = {
    "texmf-dist/doc/latex/kerntest/ChangeLog",
    "texmf-dist/doc/latex/kerntest/README",
    "texmf-dist/doc/latex/kerntest/ToDo",
    "texmf-dist/doc/latex/kerntest/kerntest.pdf",
  },
  ["showtags"] = {
    "texmf-dist/doc/latex/showtags/showtags-doc.pdf",
  },
  ["epsf"] = {
    "texmf-dist/doc/generic/epsf/LICENSE",
    "texmf-dist/doc/generic/epsf/Makefile",
    "texmf-dist/doc/generic/epsf/README",
    "texmf-dist/doc/generic/epsf/bboxgrid.ps",
    "texmf-dist/doc/generic/epsf/epsf-doc.pdf",
    "texmf-dist/doc/generic/epsf/fndbadps",
    "texmf-dist/doc/generic/epsf/okay/tepsf1.dvi",
    "texmf-dist/doc/generic/epsf/okay/tepsf2.dvi",
    "texmf-dist/doc/generic/epsf/okay/tepsf3.dvi",
  },
  ["preprint"] = {
    "texmf-dist/doc/latex/preprint/README",
    "texmf-dist/doc/latex/preprint/authblk.pdf",
    "texmf-dist/doc/latex/preprint/balance.pdf",
    "texmf-dist/doc/latex/preprint/figcaps.pdf",
    "texmf-dist/doc/latex/preprint/fullpage.pdf",
    "texmf-dist/doc/latex/preprint/sublabel.pdf",
  },
  ["proba"] = {
    "texmf-dist/doc/latex/proba/README",
    "texmf-dist/doc/latex/proba/proba.pdf",
  },
  ["biocon"] = {
    "texmf-dist/doc/latex/biocon/COPYING",
    "texmf-dist/doc/latex/biocon/INSTALL",
    "texmf-dist/doc/latex/biocon/README",
    "texmf-dist/doc/latex/biocon/manual-old.pdf",
    "texmf-dist/doc/latex/biocon/manual.pdf",
    "texmf-dist/doc/latex/biocon/source.pdf",
  },
  ["hvindex"] = {
    "texmf-dist/doc/latex/hvindex/Changes",
    "texmf-dist/doc/latex/hvindex/README",
    "texmf-dist/doc/latex/hvindex/hvindex-doc.pdf",
  },
  ["orkhun"] = {
    "texmf-dist/doc/fonts/orkhun/README_Orkhun.txt",
    "texmf-dist/doc/fonts/orkhun/rakhimov.pdf",
  },
  ["spot"] = {
    "texmf-dist/doc/latex/spot/README",
    "texmf-dist/doc/latex/spot/spot.pdf",
  },
  ["visualtikz"] = {
    "texmf-dist/doc/latex/visualtikz/README.txt",
    "texmf-dist/doc/latex/visualtikz/Source/data3d.txt",
    "texmf-dist/doc/latex/visualtikz/Source/xxx.txt",
    "texmf-dist/doc/latex/visualtikz/VisualTikZ-fr.pdf",
    "texmf-dist/doc/latex/visualtikz/VisualTikZ.pdf",
  },
  ["indextools"] = {
    "texmf-dist/doc/latex/indextools/README",
    "texmf-dist/doc/latex/indextools/indextools.pdf",
    "texmf-dist/doc/latex/indextools/issue5.pdf",
    "texmf-dist/doc/latex/indextools/latexmkrc",
    "texmf-dist/doc/latex/indextools/makefile",
  },
  ["visualfaq"] = {
    "texmf-dist/doc/latex/visualfaq/README",
    "texmf-dist/doc/latex/visualfaq/source/README",
    "texmf-dist/doc/latex/visualfaq/source/anotherarticle.pdf",
    "texmf-dist/doc/latex/visualfaq/source/fuzzytext.pdf",
    "texmf-dist/doc/latex/visualfaq/source/labelgraph.pdf",
    "texmf-dist/doc/latex/visualfaq/source/watermark.pdf",
    "texmf-dist/doc/latex/visualfaq/troubleshoot-vlf.pdf",
    "texmf-dist/doc/latex/visualfaq/visualFAQ.pdf",
  },
  ["ntgclass"] = {
    "texmf-dist/doc/latex/ntgclass/00readme.txt",
    "texmf-dist/doc/latex/ntgclass/a4.pdf",
    "texmf-dist/doc/latex/ntgclass/artdoc.pdf",
    "texmf-dist/doc/latex/ntgclass/brief.pdf",
    "texmf-dist/doc/latex/ntgclass/briefdoc.pdf",
    "texmf-dist/doc/latex/ntgclass/catalog.txt",
    "texmf-dist/doc/latex/ntgclass/changes.txt",
    "texmf-dist/doc/latex/ntgclass/classdoc.pdf",
    "texmf-dist/doc/latex/ntgclass/manifest.txt",
    "texmf-dist/doc/latex/ntgclass/ntgclass.pdf",
    "texmf-dist/doc/latex/ntgclass/rapdoc.pdf",
  },
  ["protocol"] = {
    "texmf-dist/doc/latex/protocol/README",
    "texmf-dist/doc/latex/protocol/protocol.pdf",
  },
  ["calrsfs"] = {
    "texmf-dist/doc/latex/calrsfs/README",
    "texmf-dist/doc/latex/calrsfs/calrsfs.pdf",
  },
  ["texlive.infra"] = {
    "README",
    "README.usergroups",
    "index.html",
    "readme-html.dir/readme.cs.html",
    "readme-html.dir/readme.de.html",
    "readme-html.dir/readme.en.html",
    "readme-html.dir/readme.es.html",
    "readme-html.dir/readme.fr.html",
    "readme-html.dir/readme.it.html",
    "readme-html.dir/readme.ja.html",
    "readme-html.dir/readme.pl.html",
    "readme-html.dir/readme.pt-br.html",
    "readme-html.dir/readme.ru.html",
    "readme-html.dir/readme.sr.html",
    "readme-html.dir/readme.zh-cn.html",
    "readme-txt.dir/README.CS",
    "readme-txt.dir/README.DE",
    "readme-txt.dir/README.EN",
    "readme-txt.dir/README.ES",
    "readme-txt.dir/README.FR",
    "readme-txt.dir/README.IT",
    "readme-txt.dir/README.JA",
    "readme-txt.dir/README.PL",
    "readme-txt.dir/README.PT-BR",
    "readme-txt.dir/README.RU",
    "readme-txt.dir/README.RU-cp1251",
    "readme-txt.dir/README.RU-koi8",
    "readme-txt.dir/README.SK-ascii",
    "readme-txt.dir/README.SK-cp1250",
    "readme-txt.dir/README.SK-il2",
    "readme-txt.dir/README.SR",
    "readme-txt.dir/README.ZH-CN",
    "texmf-dist/doc/man/man1/tlmgr.man1.pdf",
    "texmf-dist/scripts/texlive/NEWS",
    "tlpkg/README",
    "tlpkg/installer/COPYING.MinGW-runtime.txt",
  },
  ["flowfram"] = {
    "texmf-dist/doc/latex/flowfram/CHANGES",
    "texmf-dist/doc/latex/flowfram/README",
    "texmf-dist/doc/latex/flowfram/ffuserguide.pdf",
    "texmf-dist/doc/latex/flowfram/flowfram.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sample-article.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sample-brochure.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sample-news.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sample-news2.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sample-pages.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sample-poster.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sample-rot.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sample.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sample1.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sample2.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sample3.pdf",
    "texmf-dist/doc/latex/flowfram/samples/sampleRL.pdf",
  },
  ["revtex4"] = {
    "texmf-dist/doc/latex/revtex4/DOWNLOAD",
    "texmf-dist/doc/latex/revtex4/README",
    "texmf-dist/doc/latex/revtex4/ltxdocext.pdf",
    "texmf-dist/doc/latex/revtex4/ltxgrid.pdf",
    "texmf-dist/doc/latex/revtex4/ltxutil.pdf",
    "texmf-dist/doc/latex/revtex4/revtex4.pdf",
  },
  ["cm-super"] = {
    "texmf-dist/doc/fonts/cm-super/COPYING",
    "texmf-dist/doc/fonts/cm-super/ChangeLog",
    "texmf-dist/doc/fonts/cm-super/FAQ",
    "texmf-dist/doc/fonts/cm-super/INSTALL",
    "texmf-dist/doc/fonts/cm-super/README",
    "texmf-dist/doc/fonts/cm-super/TODO",
  },
  ["mfnfss"] = {
    "texmf-dist/doc/latex/mfnfss/README.md",
    "texmf-dist/doc/latex/mfnfss/changes.txt",
    "texmf-dist/doc/latex/mfnfss/manifest.txt",
    "texmf-dist/doc/latex/mfnfss/oldgerm.pdf",
    "texmf-dist/doc/latex/mfnfss/pandora.pdf",
  },
  ["eqnnumwarn"] = {
    "texmf-dist/doc/latex/eqnnumwarn/README.md",
    "texmf-dist/doc/latex/eqnnumwarn/eqnnumwarn.pdf",
  },
  ["biblatex"] = {
    "texmf-dist/doc/latex/biblatex/README",
    "texmf-dist/doc/latex/biblatex/biblatex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/01-introduction-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/01-introduction-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/02-annotations-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/02-annotations-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/03-localization-keys-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/03-localization-keys-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/04-delimiters-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/04-delimiters-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/10-references-per-section-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/10-references-per-section-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/11-references-by-section-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/11-references-by-section-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/12-references-by-segment-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/12-references-by-segment-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/13-references-by-keyword-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/13-references-by-keyword-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/14-references-by-category-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/14-references-by-category-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/15-references-by-type-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/15-references-by-type-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/16-numeric-prefixed-1-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/16-numeric-prefixed-1-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/17-numeric-prefixed-2-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/17-numeric-prefixed-2-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/18-numeric-hybrid-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/18-numeric-hybrid-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/19-alphabetic-prefixed-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/19-alphabetic-prefixed-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/20-indexing-single-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/20-indexing-single-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/21-indexing-multiple-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/21-indexing-multiple-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/22-indexing-subentry-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/22-indexing-subentry-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/30-style-numeric-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/30-style-numeric-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/31-style-numeric-comp-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/31-style-numeric-comp-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/32-style-numeric-verb-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/32-style-numeric-verb-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/40-style-alphabetic-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/40-style-alphabetic-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/41-style-alphabetic-verb-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/41-style-alphabetic-verb-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/42-style-alphabetic-template-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/42-style-alphabetic-template-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/50-style-authoryear-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/50-style-authoryear-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/51-style-authoryear-ibid-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/51-style-authoryear-ibid-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/52-style-authoryear-comp-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/52-style-authoryear-comp-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/53-style-authoryear-icomp-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/53-style-authoryear-icomp-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/60-style-authortitle-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/60-style-authortitle-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/61-style-authortitle-ibid-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/61-style-authortitle-ibid-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/62-style-authortitle-comp-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/62-style-authortitle-comp-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/63-style-authortitle-icomp-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/63-style-authortitle-icomp-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/64-style-authortitle-terse-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/64-style-authortitle-terse-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/65-style-authortitle-tcomp-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/65-style-authortitle-tcomp-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/66-style-authortitle-ticomp-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/66-style-authortitle-ticomp-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/70-style-verbose-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/70-style-verbose-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/71-style-verbose-ibid-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/71-style-verbose-ibid-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/72-style-verbose-note-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/72-style-verbose-note-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/73-style-verbose-inote-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/73-style-verbose-inote-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/74-style-verbose-trad1-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/74-style-verbose-trad1-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/75-style-verbose-trad2-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/75-style-verbose-trad2-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/76-style-verbose-trad3-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/76-style-verbose-trad3-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/80-style-reading-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/80-style-reading-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/81-style-draft-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/81-style-draft-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/82-style-debug-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/82-style-debug-bibtex.pdf",
    "texmf-dist/doc/latex/biblatex/examples/90-related-entries-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/91-sorting-schemes-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/92-bibliographylists-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/93-nameparts-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/94-labelprefix-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/95-customlists-biber.pdf",
    "texmf-dist/doc/latex/biblatex/examples/96-dates-biber.pdf",
  },
  ["beamerthemenirma"] = {
    "texmf-dist/doc/latex/beamerthemenirma/README",
  },
  ["venturisadf"] = {
    "texmf-dist/doc/fonts/venturisadf/LICENSE-utopia.txt",
    "texmf-dist/doc/fonts/venturisadf/LIST-Venturis.txt",
    "texmf-dist/doc/fonts/venturisadf/README",
    "texmf-dist/doc/fonts/venturisadf/manifest.txt",
    "texmf-dist/doc/fonts/venturisadf/venturisadf.pdf",
  },
  ["endfloat"] = {
    "texmf-dist/doc/latex/endfloat/COPYING",
    "texmf-dist/doc/latex/endfloat/README",
    "texmf-dist/doc/latex/endfloat/endfloat.pdf",
  },
  ["eco"] = {
    "texmf-dist/doc/fonts/eco/CHANGES",
    "texmf-dist/doc/fonts/eco/COPYING",
    "texmf-dist/doc/fonts/eco/INSTALL",
    "texmf-dist/doc/fonts/eco/README",
    "texmf-dist/doc/fonts/eco/VERSION",
  },
  ["knowledge"] = {
    "texmf-dist/doc/latex/knowledge/README.md",
    "texmf-dist/doc/latex/knowledge/knowledge.pdf",
    "texmf-dist/doc/latex/knowledge/makefile",
  },
  ["baskervillef"] = {
    "texmf-dist/doc/fonts/baskervillef/FONTLOG.txt",
    "texmf-dist/doc/fonts/baskervillef/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/baskervillef/OFL.txt",
    "texmf-dist/doc/fonts/baskervillef/README",
    "texmf-dist/doc/fonts/baskervillef/baskervillef-doc.pdf",
  },
  ["pmgraph"] = {
    "texmf-dist/doc/latex/pmgraph/COPYING",
    "texmf-dist/doc/latex/pmgraph/README",
    "texmf-dist/doc/latex/pmgraph/pmgraph.pdf",
  },
  ["cje"] = {
    "texmf-dist/doc/latex/cje/README.txt",
    "texmf-dist/doc/latex/cje/canadian-flag.pdf",
    "texmf-dist/doc/latex/cje/cjeguide.pdf",
  },
  ["unisugar"] = {
    "texmf-dist/doc/xelatex/unisugar/Makefile",
    "texmf-dist/doc/xelatex/unisugar/README",
    "texmf-dist/doc/xelatex/unisugar/unisugar.pdf",
  },
  ["codedoc"] = {
    "texmf-dist/doc/latex/codedoc/CodeDoc-manual.pdf",
    "texmf-dist/doc/latex/codedoc/README",
  },
  ["aurical"] = {
    "texmf-dist/doc/latex/aurical/aurical.pdf",
  },
  ["skeyval"] = {
    "texmf-dist/doc/latex/skeyval/README",
    "texmf-dist/doc/latex/skeyval/skeyval-pokayoke1.pdf",
    "texmf-dist/doc/latex/skeyval/skeyval-pokayoke2.pdf",
    "texmf-dist/doc/latex/skeyval/skeyval-view-pokayoke1.pdf",
  },
  ["babel-japanese"] = {
    "texmf-dist/doc/generic/babel-japanese/Makefile",
    "texmf-dist/doc/generic/babel-japanese/README.md",
    "texmf-dist/doc/generic/babel-japanese/japanese-sample.pdf",
    "texmf-dist/doc/generic/babel-japanese/japanese.pdf",
  },
  ["bxbase"] = {
    "texmf-dist/doc/latex/bxbase/LICENSE",
    "texmf-dist/doc/latex/bxbase/README.md",
    "texmf-dist/doc/latex/bxbase/bxbase-ja.pdf",
  },
  ["tikzmarmots"] = {
    "texmf-dist/doc/latex/tikzmarmots/LICENSE.txt",
    "texmf-dist/doc/latex/tikzmarmots/README.md",
    "texmf-dist/doc/latex/tikzmarmots/tikzmarmots-doc.pdf",
  },
  ["vdmlisting"] = {
    "texmf-dist/doc/latex/vdmlisting/README",
    "texmf-dist/doc/latex/vdmlisting/vdmlisting.pdf",
  },
  ["afm2pl"] = {
    "texmf-dist/doc/man/man1/afm2pl.man1.pdf",
  },
  ["mathdesign"] = {
    "texmf-dist/doc/fonts/mathdesign/MD-Adobe-Utopia-example.pdf",
    "texmf-dist/doc/fonts/mathdesign/MD-Bitstream-Bitstream-Charter-example.pdf",
    "texmf-dist/doc/fonts/mathdesign/README",
    "texmf-dist/doc/fonts/mathdesign/mathdesign-doc.pdf",
  },
  ["isopt"] = {
    "texmf-dist/doc/latex/isopt/Changes",
    "texmf-dist/doc/latex/isopt/README.md",
    "texmf-dist/doc/latex/isopt/isopt-doc.pdf",
  },
  ["vaucanson-g"] = {
    "texmf-dist/doc/generic/vaucanson-g/CHANGES",
    "texmf-dist/doc/generic/vaucanson-g/README",
    "texmf-dist/doc/generic/vaucanson-g/VCManual.pdf",
  },
  ["varwidth"] = {
    "texmf-dist/doc/latex/varwidth/varwidth-doc.pdf",
  },
  ["varsfromjobname"] = {
    "texmf-dist/doc/latex/varsfromjobname/README.md",
    "texmf-dist/doc/latex/varsfromjobname/varsfromjobname.pdf",
  },
  ["showexpl"] = {
    "texmf-dist/doc/latex/showexpl/README",
    "texmf-dist/doc/latex/showexpl/result-picture.pdf",
    "texmf-dist/doc/latex/showexpl/showexpl-test.pdf",
    "texmf-dist/doc/latex/showexpl/showexpl.pdf",
  },
  ["lastpage"] = {
    "texmf-dist/doc/latex/lastpage/README",
    "texmf-dist/doc/latex/lastpage/lastpage-example.pdf",
    "texmf-dist/doc/latex/lastpage/lastpage.pdf",
  },
  ["varindex"] = {
    "texmf-dist/doc/latex/varindex/README",
    "texmf-dist/doc/latex/varindex/varindex.pdf",
    "texmf-dist/doc/latex/varindex/varindex.txt",
  },
  ["variations"] = {
    "texmf-dist/doc/generic/variations/ALIRE",
    "texmf-dist/doc/generic/variations/COPYING",
    "texmf-dist/doc/generic/variations/README",
    "texmf-dist/doc/generic/variations/docvariations.pdf",
  },
  ["variablelm"] = {
    "texmf-dist/doc/fonts/variablelm/README.md",
    "texmf-dist/doc/fonts/variablelm/control_scaling_lm_fonts.pdf",
  },
  ["context-simplefonts"] = {
    "texmf-dist/doc/context/third/simplefonts/README",
    "texmf-dist/doc/context/third/simplefonts/VERSION",
  },
  ["uwthesis"] = {
    "texmf-dist/doc/latex/uwthesis/LICENSE",
    "texmf-dist/doc/latex/uwthesis/README",
    "texmf-dist/doc/latex/uwthesis/uwthesis.pdf",
  },
  ["utopia"] = {
    "texmf-dist/doc/fonts/utopia/LICENSE-utopia.txt",
    "texmf-dist/doc/fonts/utopia/README-utopia.txt",
  },
  ["utf8mex"] = {
    "texmf-dist/doc/mex/utf8mex/Makefile",
    "texmf-dist/doc/mex/utf8mex/README",
    "texmf-dist/doc/mex/utf8mex/examples/Makefile",
  },
  ["eulerpx"] = {
    "texmf-dist/doc/fonts/eulerpx/README",
  },
  ["ut-thesis"] = {
    "texmf-dist/doc/latex/ut-thesis/README",
  },
  ["uspatent"] = {
    "texmf-dist/doc/latex/uspatent/PatentApplication.pdf",
    "texmf-dist/doc/latex/uspatent/PatentApplicationGuide.pdf",
    "texmf-dist/doc/latex/uspatent/README",
    "texmf-dist/doc/latex/uspatent/VisioDrawing.pdf",
  },
  ["biblatex-phys"] = {
    "texmf-dist/doc/latex/biblatex-phys/README.md",
    "texmf-dist/doc/latex/biblatex-phys/biblatex-phys.pdf",
  },
  ["uspace"] = {
    "texmf-dist/doc/latex/uspace/LICENSE",
    "texmf-dist/doc/latex/uspace/README.md",
    "texmf-dist/doc/latex/uspace/uspace-test-lualatex.pdf",
    "texmf-dist/doc/latex/uspace/uspace-test-pdflatex.pdf",
    "texmf-dist/doc/latex/uspace/uspace-test-xelatex.pdf",
    "texmf-dist/doc/latex/uspace/uspace.pdf",
  },
  ["ushort"] = {
    "texmf-dist/doc/latex/ushort/README",
    "texmf-dist/doc/latex/ushort/ushort.pdf",
    "texmf-dist/doc/latex/ushort/ushort.txt",
  },
  ["opcit"] = {
    "texmf-dist/doc/latex/opcit/README",
    "texmf-dist/doc/latex/opcit/opcit.pdf",
  },
  ["lroundrect"] = {
    "texmf-dist/doc/latex/lroundrect/README",
    "texmf-dist/doc/latex/lroundrect/lppl.txt",
    "texmf-dist/doc/latex/lroundrect/lroundrect.pdf",
  },
  ["urwchancal"] = {
    "texmf-dist/doc/fonts/urwchancal/README",
    "texmf-dist/doc/fonts/urwchancal/urwchancal-doc.pdf",
  },
  ["genealogy"] = {
    "texmf-dist/doc/fonts/genealogy/README",
    "texmf-dist/doc/fonts/genealogy/licence.txt",
  },
  ["babel-swedish"] = {
    "texmf-dist/doc/generic/babel-swedish/swedish.pdf",
  },
  ["footnotehyper"] = {
    "texmf-dist/doc/latex/footnotehyper/README.md",
    "texmf-dist/doc/latex/footnotehyper/footnotehyper.pdf",
  },
  ["concepts"] = {
    "texmf-dist/doc/latex/concepts/README",
    "texmf-dist/doc/latex/concepts/concepts.pdf",
  },
  ["chapterfolder"] = {
    "texmf-dist/doc/latex/chapterfolder/chapterfolder.pdf",
  },
  ["graphics-pln"] = {
    "texmf-dist/doc/plain/graphics-pln/README.txt",
  },
  ["gradientframe"] = {
    "texmf-dist/doc/latex/gradientframe/README",
    "texmf-dist/doc/latex/gradientframe/gradientframe.pdf",
  },
  ["typed-checklist"] = {
    "texmf-dist/doc/latex/typed-checklist/README.md",
    "texmf-dist/doc/latex/typed-checklist/typed-checklist.pdf",
  },
  ["adforn"] = {
    "texmf-dist/doc/fonts/adforn/COPYING",
    "texmf-dist/doc/fonts/adforn/NOTICE",
    "texmf-dist/doc/fonts/adforn/README",
    "texmf-dist/doc/fonts/adforn/adforn.pdf",
    "texmf-dist/doc/fonts/adforn/manifest.txt",
  },
  ["psu-thesis"] = {
    "texmf-dist/doc/latex/psu-thesis/README",
    "texmf-dist/doc/latex/psu-thesis/manual.pdf",
    "texmf-dist/doc/latex/psu-thesis/mssample.pdf",
    "texmf-dist/doc/latex/psu-thesis/phdsample.pdf",
  },
  ["urlbst"] = {
    "texmf-dist/doc/bibtex/urlbst/README",
    "texmf-dist/doc/bibtex/urlbst/VERSION",
    "texmf-dist/doc/bibtex/urlbst/urlbst.html",
    "texmf-dist/doc/bibtex/urlbst/urlbst.pdf",
  },
  ["biblatex-anonymous"] = {
    "texmf-dist/doc/latex/biblatex-anonymous/README",
    "texmf-dist/doc/latex/biblatex-anonymous/biblatex-anonymous.pdf",
    "texmf-dist/doc/latex/biblatex-anonymous/latexmkrc",
    "texmf-dist/doc/latex/biblatex-anonymous/makefile",
  },
  ["uri"] = {
    "texmf-dist/doc/latex/uri/README",
    "texmf-dist/doc/latex/uri/uri-example.pdf",
    "texmf-dist/doc/latex/uri/uri.pdf",
  },
  ["xevlna"] = {
    "texmf-dist/doc/xelatex/xevlna/License.txt",
    "texmf-dist/doc/xelatex/xevlna/README",
    "texmf-dist/doc/xelatex/xevlna/xevlna.pdf",
  },
  ["apacite"] = {
    "texmf-dist/doc/bibtex/apacite/README",
    "texmf-dist/doc/bibtex/apacite/apacite.pdf",
  },
  ["babel-malay"] = {
    "texmf-dist/doc/generic/babel-malay/README",
    "texmf-dist/doc/generic/babel-malay/malay.pdf",
  },
  ["relsize"] = {
    "texmf-dist/doc/latex/relsize/README",
    "texmf-dist/doc/latex/relsize/relsize-doc.pdf",
  },
  ["constants"] = {
    "texmf-dist/doc/latex/constants/README",
    "texmf-dist/doc/latex/constants/constants.pdf",
  },
  ["bgreek"] = {
    "texmf-dist/doc/latex/bgreek/MANIFEST.TXT",
    "texmf-dist/doc/latex/bgreek/README",
    "texmf-dist/doc/latex/bgreek/bgman.pdf",
  },
  ["lshort-persian"] = {
    "texmf-dist/doc/latex/lshort-persian/README",
    "texmf-dist/doc/latex/lshort-persian/lshort.pdf",
  },
  ["csvsimple"] = {
    "texmf-dist/doc/latex/csvsimple/CHANGES",
    "texmf-dist/doc/latex/csvsimple/README",
    "texmf-dist/doc/latex/csvsimple/csvsimple-example.pdf",
    "texmf-dist/doc/latex/csvsimple/csvsimple.pdf",
  },
  ["biblatex-fiwi"] = {
    "texmf-dist/doc/latex/biblatex-fiwi/README",
    "texmf-dist/doc/latex/biblatex-fiwi/biblatex-fiwi.pdf",
    "texmf-dist/doc/latex/biblatex-fiwi/example-biblatex-fiwi-options.pdf",
    "texmf-dist/doc/latex/biblatex-fiwi/example-biblatex-fiwi-xindy.pdf",
    "texmf-dist/doc/latex/biblatex-fiwi/example-biblatex-fiwi.pdf",
    "texmf-dist/doc/latex/biblatex-fiwi/example-biblatex-fiwi2-options.pdf",
  },
  ["active-conf"] = {
    "texmf-dist/doc/latex/active-conf/README",
    "texmf-dist/doc/latex/active-conf/active-conf.pdf",
    "texmf-dist/doc/latex/active-conf/example/header-logo-2006.pdf",
  },
  ["cheatsheet"] = {
    "texmf-dist/doc/latex/cheatsheet/README.md",
    "texmf-dist/doc/latex/cheatsheet/cheatsheet.pdf",
  },
  ["fpl"] = {
    "texmf-dist/doc/fonts/fpl/COPYING",
    "texmf-dist/doc/fonts/fpl/README",
  },
  ["eiad"] = {
    "texmf-dist/doc/fonts/eiad/Leigh_me.txt",
    "texmf-dist/doc/fonts/eiad/amhrawn.pdf",
    "texmf-dist/doc/fonts/eiad/evening.pdf",
    "texmf-dist/doc/fonts/eiad/micheal.ps",
    "texmf-dist/doc/fonts/eiad/recreat.pdf",
  },
  ["uptex-fonts"] = {
    "texmf-dist/doc/fonts/uptex-fonts/LICENSE",
    "texmf-dist/doc/fonts/uptex-fonts/README.md",
    "texmf-dist/doc/fonts/uptex-fonts/README_ASCII_Corp.txt",
  },
  ["uptex-base"] = {
    "texmf-dist/doc/uptex/uptex-base/00readme_uptex.txt",
    "texmf-dist/doc/uptex/uptex-base/01uptex_doc_utf8.txt",
    "texmf-dist/doc/uptex/uptex-base/02uptex_changelog_utf8.txt",
    "texmf-dist/doc/uptex/uptex-base/LICENSE",
    "texmf-dist/doc/uptex/uptex-base/Makefile",
    "texmf-dist/doc/uptex/uptex-base/README.md",
    "texmf-dist/doc/uptex/uptex-base/README_uptex.txt",
    "texmf-dist/doc/uptex/uptex-base/samples/00readme_uptex_samples.txt",
    "texmf-dist/doc/uptex/uptex-base/samples/Makefile",
    "texmf-dist/doc/uptex/uptex-base/samples/bkmk-docinfo.txt",
    "texmf-dist/doc/uptex/uptex-base/samples/iotext.txt",
  },
  ["bezierplot"] = {
    "texmf-dist/doc/lualatex/bezierplot/README",
    "texmf-dist/doc/lualatex/bezierplot/bezierplot-doc.pdf",
  },
  ["sesamanuel"] = {
    "texmf-dist/doc/latex/sesamanuel/sesamanuel.pdf",
    "texmf-dist/doc/latex/sesamanuel/sesamath-doc-fr.pdf",
  },
  ["uptex"] = {
    "texmf-dist/doc/man/man1/euptex.man1.pdf",
    "texmf-dist/doc/man/man1/uptex.man1.pdf",
    "texmf-dist/doc/upmendex/README.md",
    "texmf-dist/doc/upmendex/samples/Makefile",
  },
  ["upquote"] = {
    "texmf-dist/doc/latex/upquote/README",
    "texmf-dist/doc/latex/upquote/upquote.pdf",
  },
  ["colourchange"] = {
    "texmf-dist/doc/latex/colourchange/README",
    "texmf-dist/doc/latex/colourchange/colourchangedoc.pdf",
  },
  ["a2ping"] = {
    "texmf-dist/doc/man/man1/a2ping.man1.pdf",
    "texmf-dist/doc/support/a2ping/HISTORY.txt",
  },
  ["jvlisting"] = {
    "texmf-dist/doc/latex/jvlisting/README",
    "texmf-dist/doc/latex/jvlisting/jvlisting.pdf",
  },
  ["tex4ebook"] = {
    "texmf-dist/doc/support/tex4ebook/README",
    "texmf-dist/doc/support/tex4ebook/readme.tex",
    "texmf-dist/doc/support/tex4ebook/tex4ebook-doc.pdf",
  },
  ["biblatex-abnt"] = {
    "texmf-dist/doc/latex/biblatex-abnt/NBR10520-2002_reference.pdf",
    "texmf-dist/doc/latex/biblatex-abnt/NBR6023-2002_reference.pdf",
    "texmf-dist/doc/latex/biblatex-abnt/README.md",
    "texmf-dist/doc/latex/biblatex-abnt/biblatex-abnt.pdf",
    "texmf-dist/doc/latex/biblatex-abnt/tests/README.md",
  },
  ["koma-script-sfs"] = {
    "texmf-dist/doc/latex/koma-script-sfs/README",
    "texmf-dist/doc/latex/koma-script-sfs/SFSesim.pdf",
  },
  ["sapthesis"] = {
    "texmf-dist/doc/latex/sapthesis/README",
    "texmf-dist/doc/latex/sapthesis/sapthesis-doc.pdf",
  },
  ["upca"] = {
    "texmf-dist/doc/generic/upca/README",
  },
  ["uowthesis"] = {
    "texmf-dist/doc/latex/uowthesis/README",
    "texmf-dist/doc/latex/uowthesis/mythesis.pdf",
  },
  ["uothesis"] = {
    "texmf-dist/doc/latex/uothesis/README",
    "texmf-dist/doc/latex/uothesis/README.txt",
    "texmf-dist/doc/latex/uothesis/uothesis.pdf",
  },
  ["hc"] = {
    "texmf-dist/doc/latex/hc/COPYING",
    "texmf-dist/doc/latex/hc/FILES",
    "texmf-dist/doc/latex/hc/README",
    "texmf-dist/doc/latex/hc/hc.ps",
  },
  ["ted"] = {
    "texmf-dist/doc/latex/ted/README",
    "texmf-dist/doc/latex/ted/ted-fr.pdf",
    "texmf-dist/doc/latex/ted/ted.pdf",
  },
  ["platexcheat"] = {
    "texmf-dist/doc/latex/platexcheat/LICENSE",
    "texmf-dist/doc/latex/platexcheat/README.md",
    "texmf-dist/doc/latex/platexcheat/platexsheet-jsclasses.pdf",
    "texmf-dist/doc/latex/platexcheat/platexsheet.pdf",
    "texmf-dist/doc/latex/platexcheat/sample-jsclasses.pdf",
    "texmf-dist/doc/latex/platexcheat/sample.pdf",
  },
  ["unitsdef"] = {
    "texmf-dist/doc/latex/unitsdef/README.1st",
    "texmf-dist/doc/latex/unitsdef/unitsdef.pdf",
  },
  ["translation-array-fr"] = {
    "texmf-dist/doc/latex/translation-array-fr/Copyright",
    "texmf-dist/doc/latex/translation-array-fr/README",
    "texmf-dist/doc/latex/translation-array-fr/f-array.pdf",
  },
  ["unicode-math"] = {
    "texmf-dist/doc/latex/unicode-math/LICENSE",
    "texmf-dist/doc/latex/unicode-math/README.md",
    "texmf-dist/doc/latex/unicode-math/unicode-math-code.pdf",
    "texmf-dist/doc/latex/unicode-math/unicode-math.pdf",
    "texmf-dist/doc/latex/unicode-math/unimath-example.pdf",
    "texmf-dist/doc/latex/unicode-math/unimath-symbols.pdf",
  },
  ["unicode-bidi"] = {
    "texmf-dist/doc/xelatex/unicode-bidi/README.md",
  },
  ["uni-wtal-lin"] = {
    "texmf-dist/doc/latex/uni-wtal-lin/CHANGES",
    "texmf-dist/doc/latex/uni-wtal-lin/LIESMICH",
    "texmf-dist/doc/latex/uni-wtal-lin/README",
    "texmf-dist/doc/latex/uni-wtal-lin/uni-wtal-lin.pdf",
  },
  ["pst-fr3d"] = {
    "texmf-dist/doc/generic/pst-fr3d/Changes",
    "texmf-dist/doc/generic/pst-fr3d/README",
    "texmf-dist/doc/generic/pst-fr3d/pst-fr3d.pdf",
  },
  ["na-position"] = {
    "texmf-dist/doc/xelatex/na-position/README",
    "texmf-dist/doc/xelatex/na-position/na-position.pdf",
  },
  ["unfonts-core"] = {
    "texmf-dist/doc/fonts/unfonts-core/COPYING",
    "texmf-dist/doc/fonts/unfonts-core/README.md",
    "texmf-dist/doc/fonts/unfonts-core/README_original",
  },
  ["l2tabu"] = {
    "texmf-dist/doc/latex/l2tabu/CHANGES",
    "texmf-dist/doc/latex/l2tabu/README",
    "texmf-dist/doc/latex/l2tabu/l2tabu.pdf",
  },
  ["underlin"] = {
    "texmf-dist/doc/latex/underlin/underlin.pdf",
  },
  ["pst-pad"] = {
    "texmf-dist/doc/generic/pst-pad/CHANGES",
    "texmf-dist/doc/generic/pst-pad/README",
    "texmf-dist/doc/generic/pst-pad/pst-pad-doc.pdf",
    "texmf-dist/doc/generic/pst-pad/pst-pad-docDE.pdf",
  },
  ["libertine"] = {
    "texmf-dist/doc/fonts/libertine/Bugs.txt",
    "texmf-dist/doc/fonts/libertine/ChangeLog.txt",
    "texmf-dist/doc/fonts/libertine/GPL.txt",
    "texmf-dist/doc/fonts/libertine/LICENCE.txt",
    "texmf-dist/doc/fonts/libertine/README",
    "texmf-dist/doc/fonts/libertine/ToDo.txt",
    "texmf-dist/doc/fonts/libertine/libertine.pdf",
    "texmf-dist/doc/fonts/libertine/rename_enc",
    "texmf-dist/doc/fonts/libertine/samples.pdf",
  },
  ["delimseasy"] = {
    "texmf-dist/doc/latex/delimseasy/README.txt",
    "texmf-dist/doc/latex/delimseasy/delimseasyMan.pdf",
  },
  ["unamthesis"] = {
    "texmf-dist/doc/latex/unamthesis/Escudo-FC.pdf",
    "texmf-dist/doc/latex/unamthesis/Escudo-IBT.pdf",
    "texmf-dist/doc/latex/unamthesis/Escudo-UNAM.pdf",
    "texmf-dist/doc/latex/unamthesis/LEEME",
    "texmf-dist/doc/latex/unamthesis/License",
    "texmf-dist/doc/latex/unamthesis/README",
    "texmf-dist/doc/latex/unamthesis/UNAMThesis.pdf",
  },
  ["biochemistry-colors"] = {
    "texmf-dist/doc/latex/biochemistry-colors/Biochemistry-colors.pdf",
    "texmf-dist/doc/latex/biochemistry-colors/README.txt",
  },
  ["unamth-template"] = {
    "texmf-dist/doc/latex/unamth-template/LICENSE",
    "texmf-dist/doc/latex/unamth-template/Latex/Classes/Escudos/fi_azul.pdf",
    "texmf-dist/doc/latex/unamth-template/Latex/Classes/Escudos/fi_negro.pdf",
    "texmf-dist/doc/latex/unamth-template/Latex/Classes/Escudos/unam_azul.pdf",
    "texmf-dist/doc/latex/unamth-template/Latex/Classes/Escudos/unam_azul_5x5cm.pdf",
    "texmf-dist/doc/latex/unamth-template/Latex/Classes/Escudos/unam_negro.pdf",
    "texmf-dist/doc/latex/unamth-template/Latex/Classes/Escudos/unam_negro_5x5cm.pdf",
    "texmf-dist/doc/latex/unamth-template/Latex/Classes/urlbst",
    "texmf-dist/doc/latex/unamth-template/README.md",
    "texmf-dist/doc/latex/unamth-template/latex_intro.pdf",
    "texmf-dist/doc/latex/unamth-template/tesis.pdf",
  },
  ["gaceta"] = {
    "texmf-dist/doc/latex/gaceta/README",
    "texmf-dist/doc/latex/gaceta/plantilla-articulo-de-seccion.pdf",
    "texmf-dist/doc/latex/gaceta/plantilla-articulo-suelto.pdf",
  },
  ["umthesis"] = {
    "texmf-dist/doc/latex/umthesis/README",
    "texmf-dist/doc/latex/umthesis/example.pdf",
  },
  ["dataref"] = {
    "texmf-dist/doc/latex/dataref/README.md",
    "texmf-dist/doc/latex/dataref/dataref-doc.pdf",
  },
  ["umoline"] = {
    "texmf-dist/doc/latex/umoline/README",
    "texmf-dist/doc/latex/umoline/umoline.pdf",
  },
  ["uml"] = {
    "texmf-dist/doc/latex/uml/Changes",
    "texmf-dist/doc/latex/uml/README",
    "texmf-dist/doc/latex/uml/uml.pdf",
  },
  ["thalie"] = {
    "texmf-dist/doc/latex/thalie/LICENSE.txt",
    "texmf-dist/doc/latex/thalie/README.md",
    "texmf-dist/doc/latex/thalie/thalie.pdf",
  },
  ["sfg"] = {
    "texmf-dist/doc/latex/sfg/Changes",
    "texmf-dist/doc/latex/sfg/README",
    "texmf-dist/doc/latex/sfg/sfg-doc.pdf",
  },
  ["kotex-utils"] = {
    "texmf-dist/doc/latex/kotex-utils/ChangeLog",
    "texmf-dist/doc/latex/kotex-utils/README",
  },
  ["umbclegislation"] = {
    "texmf-dist/doc/latex/umbclegislation/COPYING",
    "texmf-dist/doc/latex/umbclegislation/Makefile",
    "texmf-dist/doc/latex/umbclegislation/README",
  },
  ["ulthese"] = {
    "texmf-dist/doc/latex/ulthese/README.md",
    "texmf-dist/doc/latex/ulthese/ulthese.pdf",
  },
  ["txfontsb"] = {
    "texmf-dist/doc/fonts/txfontsb/README",
    "texmf-dist/doc/fonts/txfontsb/txfontsb.pdf",
  },
  ["typeoutfileinfo"] = {
    "texmf-dist/doc/support/typeoutfileinfo/README",
  },
  ["msg"] = {
    "texmf-dist/doc/latex/msg/CHANGES",
    "texmf-dist/doc/latex/msg/README",
    "texmf-dist/doc/latex/msg/README_msg_doc.txt",
    "texmf-dist/doc/latex/msg/msg.pdf",
    "texmf-dist/doc/latex/msg/msgguide.pdf",
  },
  ["elbioimp"] = {
    "texmf-dist/doc/latex/elbioimp/Makefile",
    "texmf-dist/doc/latex/elbioimp/README",
    "texmf-dist/doc/latex/elbioimp/elbioimp.pdf",
  },
  ["amsthdoc-it"] = {
    "texmf-dist/doc/latex/amsthdoc-it/README",
    "texmf-dist/doc/latex/amsthdoc-it/amsthdoc_it.pdf",
  },
  ["latexfileversion"] = {
    "texmf-dist/doc/support/latexfileversion/ChangeLog",
    "texmf-dist/doc/support/latexfileversion/README",
  },
  ["pst-spinner"] = {
    "texmf-dist/doc/generic/pst-spinner/Changes",
    "texmf-dist/doc/generic/pst-spinner/README",
    "texmf-dist/doc/generic/pst-spinner/pst-spinner-doc.pdf",
  },
  ["context-gantt"] = {
    "texmf-dist/doc/context/third/gantt/README",
    "texmf-dist/doc/context/third/gantt/VERSION",
    "texmf-dist/doc/context/third/gantt/gantt.txt",
  },
  ["framed"] = {
    "texmf-dist/doc/latex/framed/framed.pdf",
  },
  ["chemmacros"] = {
    "texmf-dist/doc/latex/chemmacros/README",
    "texmf-dist/doc/latex/chemmacros/chemmacros_en.pdf",
  },
  ["ulem"] = {
    "texmf-dist/doc/generic/ulem/README",
    "texmf-dist/doc/generic/ulem/ulem.pdf",
  },
  ["bizcard"] = {
    "texmf-dist/doc/latex/bizcard/bizcard.pdf",
    "texmf-dist/doc/latex/bizcard/bizex.pdf",
  },
  ["thesis-titlepage-fhac"] = {
    "texmf-dist/doc/latex/thesis-titlepage-fhac/LogoFH.pdf",
    "texmf-dist/doc/latex/thesis-titlepage-fhac/README",
    "texmf-dist/doc/latex/thesis-titlepage-fhac/fhACtitlepage.pdf",
  },
  ["macroswap"] = {
    "texmf-dist/doc/latex/macroswap/Makefile",
    "texmf-dist/doc/latex/macroswap/README",
    "texmf-dist/doc/latex/macroswap/macroswap.pdf",
  },
  ["uhrzeit"] = {
    "texmf-dist/doc/latex/uhrzeit/README.txt",
    "texmf-dist/doc/latex/uhrzeit/uhrzeit-doc.pdf",
  },
  ["uhhassignment"] = {
    "texmf-dist/doc/latex/uhhassignment/README.md",
    "texmf-dist/doc/latex/uhhassignment/uhhassignment.pdf",
  },
  ["uhc"] = {
    "texmf-dist/doc/fonts/uhc/umj/README",
  },
  ["latex-veryshortguide"] = {
    "texmf-dist/doc/latex/latex-veryshortguide/MANIFEST",
    "texmf-dist/doc/latex/latex-veryshortguide/README.md",
    "texmf-dist/doc/latex/latex-veryshortguide/veryshortguide-A4-imposed.pdf",
    "texmf-dist/doc/latex/latex-veryshortguide/veryshortguide-Legal-imposed.pdf",
    "texmf-dist/doc/latex/latex-veryshortguide/veryshortguide.pdf",
    "texmf-dist/doc/latex/latex-veryshortguide/vsg.pdf",
  },
  ["uebungsblatt"] = {
    "texmf-dist/doc/latex/uebungsblatt/README",
    "texmf-dist/doc/latex/uebungsblatt/history.txt",
    "texmf-dist/doc/latex/uebungsblatt/uebungsblatt-doc.pdf",
  },
  ["ucharcat"] = {
    "texmf-dist/doc/latex/ucharcat/README",
    "texmf-dist/doc/latex/ucharcat/ucharcat.pdf",
  },
  ["ucbthesis"] = {
    "texmf-dist/doc/latex/ucbthesis/README",
    "texmf-dist/doc/latex/ucbthesis/ucbthesis.pdf",
  },
  ["polski"] = {
    "texmf-dist/doc/latex/polski/README",
    "texmf-dist/doc/latex/polski/conowego.txt",
    "texmf-dist/doc/latex/polski/czytaj.txt",
    "texmf-dist/doc/latex/polski/polski.pdf",
    "texmf-dist/doc/latex/polski/sample-polski.pdf",
  },
  ["uafthesis"] = {
    "texmf-dist/doc/latex/uafthesis/Makefile",
    "texmf-dist/doc/latex/uafthesis/README.md",
  },
  ["uaclasses"] = {
    "texmf-dist/doc/latex/uaclasses/README",
    "texmf-dist/doc/latex/uaclasses/my-example.pdf",
    "texmf-dist/doc/latex/uaclasses/ua-example.pdf",
  },
  ["typogrid"] = {
    "texmf-dist/doc/latex/typogrid/ChangeLog",
    "texmf-dist/doc/latex/typogrid/Makefile",
    "texmf-dist/doc/latex/typogrid/README",
    "texmf-dist/doc/latex/typogrid/typogrid.pdf",
  },
  ["typoaid"] = {
    "texmf-dist/doc/latex/typoaid/README.md",
    "texmf-dist/doc/latex/typoaid/typoaid.pdf",
  },
  ["typewriter"] = {
    "texmf-dist/doc/lualatex/typewriter/README.md",
    "texmf-dist/doc/lualatex/typewriter/typewriter-guide.pdf",
  },
  ["checkcites"] = {
    "texmf-dist/doc/support/checkcites/README",
    "texmf-dist/doc/support/checkcites/checkcites-doc.pdf",
  },
  ["cm-unicode"] = {
    "texmf-dist/doc/fonts/cm-unicode/Changes",
    "texmf-dist/doc/fonts/cm-unicode/FAQ",
    "texmf-dist/doc/fonts/cm-unicode/FontLog.txt",
    "texmf-dist/doc/fonts/cm-unicode/INSTALL",
    "texmf-dist/doc/fonts/cm-unicode/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/cm-unicode/OFL.txt",
    "texmf-dist/doc/fonts/cm-unicode/README",
    "texmf-dist/doc/fonts/cm-unicode/README.doc",
    "texmf-dist/doc/fonts/cm-unicode/TODO",
    "texmf-dist/doc/fonts/cm-unicode/cmunrm.pdf",
    "texmf-dist/doc/fonts/cm-unicode/cmunti.pdf",
  },
  ["diagbox"] = {
    "texmf-dist/doc/latex/diagbox/README",
    "texmf-dist/doc/latex/diagbox/diagbox.pdf",
  },
  ["omegaware"] = {
    "texmf-dist/doc/man/man1/odvicopy.man1.pdf",
    "texmf-dist/doc/man/man1/odvitype.man1.pdf",
    "texmf-dist/doc/man/man1/ofm2opl.man1.pdf",
    "texmf-dist/doc/man/man1/opl2ofm.man1.pdf",
    "texmf-dist/doc/man/man1/otangle.man1.pdf",
    "texmf-dist/doc/man/man1/otp2ocp.man1.pdf",
    "texmf-dist/doc/man/man1/outocp.man1.pdf",
    "texmf-dist/doc/man/man1/ovf2ovp.man1.pdf",
    "texmf-dist/doc/man/man1/ovp2ovf.man1.pdf",
  },
  ["dehyph-exptl"] = {
    "texmf-dist/doc/generic/dehyph-exptl/CHANGES",
    "texmf-dist/doc/generic/dehyph-exptl/INSTALL",
    "texmf-dist/doc/generic/dehyph-exptl/README",
    "texmf-dist/doc/generic/dehyph-exptl/dehyph-exptl.pdf",
  },
  ["adhocfilelist"] = {
    "texmf-dist/doc/support/adhocfilelist/README",
    "texmf-dist/doc/support/adhocfilelist/RELEASEs.txt",
    "texmf-dist/doc/support/adhocfilelist/SrcFILEs.txt",
    "texmf-dist/doc/support/adhocfilelist/adhocfilelist.htm",
    "texmf-dist/doc/support/adhocfilelist/demo/herelist.txt",
  },
  ["metapost-examples"] = {
    "texmf-dist/doc/metapost/metapost-examples/Makefile",
    "texmf-dist/doc/metapost/metapost-examples/README",
    "texmf-dist/doc/metapost/metapost-examples/data1",
    "texmf-dist/doc/metapost/metapost-examples/data2",
    "texmf-dist/doc/metapost/metapost-examples/data3",
  },
  ["biblatex-claves"] = {
    "texmf-dist/doc/latex/biblatex-claves/README",
    "texmf-dist/doc/latex/biblatex-claves/documentation/biblatex-claves.pdf",
    "texmf-dist/doc/latex/biblatex-claves/documentation/latexmkrc",
    "texmf-dist/doc/latex/biblatex-claves/documentation/makefile",
    "texmf-dist/doc/latex/biblatex-claves/makefile",
  },
  ["txfonts"] = {
    "texmf-dist/doc/fonts/txfonts/00bug_fix.txt",
    "texmf-dist/doc/fonts/txfonts/COPYRIGHT",
    "texmf-dist/doc/fonts/txfonts/README",
    "texmf-dist/doc/fonts/txfonts/txfontsdoc.pdf",
    "texmf-dist/doc/fonts/txfonts/txfontsdocA4.pdf",
  },
  ["twoup"] = {
    "texmf-dist/doc/latex/twoup/README",
    "texmf-dist/doc/latex/twoup/twoup.pdf",
  },
  ["twoinone"] = {
    "texmf-dist/doc/latex/twoinone/twoinone.pdf",
  },
  ["zhmetrics"] = {
    "texmf-dist/doc/fonts/zhmetrics/README",
  },
  ["correctmathalign"] = {
    "texmf-dist/doc/latex/correctmathalign/README.md",
    "texmf-dist/doc/latex/correctmathalign/aligned-org.pdf",
    "texmf-dist/doc/latex/correctmathalign/correctmathalign.pdf",
    "texmf-dist/doc/latex/correctmathalign/eqnarray-org.pdf",
  },
  ["turnstile"] = {
    "texmf-dist/doc/latex/turnstile/README",
    "texmf-dist/doc/latex/turnstile/README.en",
    "texmf-dist/doc/latex/turnstile/README.pt",
    "texmf-dist/doc/latex/turnstile/turnstile-en.pdf",
    "texmf-dist/doc/latex/turnstile/turnstile-pt.pdf",
    "texmf-dist/doc/latex/turnstile/turnstile_article.pdf",
    "texmf-dist/doc/latex/turnstile/turnstile_artigo.pdf",
  },
  ["songs"] = {
    "texmf-dist/doc/latex/songs/Makefile",
    "texmf-dist/doc/latex/songs/README",
    "texmf-dist/doc/latex/songs/history.txt",
    "texmf-dist/doc/latex/songs/license.txt",
    "texmf-dist/doc/latex/songs/sample/Makefile",
    "texmf-dist/doc/latex/songs/songs.pdf",
  },
  ["turabian"] = {
    "texmf-dist/doc/latex/turabian/README",
    "texmf-dist/doc/latex/turabian/userguide.txt",
  },
  ["playfair"] = {
    "texmf-dist/doc/fonts/playfair/OFL.txt",
    "texmf-dist/doc/fonts/playfair/Playfair_Display_A4_specimen.pdf",
    "texmf-dist/doc/fonts/playfair/README",
    "texmf-dist/doc/fonts/playfair/playfair-samples.pdf",
  },
  ["biblatex-chicago"] = {
    "texmf-dist/doc/latex/biblatex-chicago/README",
    "texmf-dist/doc/latex/biblatex-chicago/RELEASE",
    "texmf-dist/doc/latex/biblatex-chicago/biblatex-chicago.pdf",
    "texmf-dist/doc/latex/biblatex-chicago/cms-dates-intro.pdf",
    "texmf-dist/doc/latex/biblatex-chicago/cms-dates-sample.pdf",
    "texmf-dist/doc/latex/biblatex-chicago/cms-legal-sample.pdf",
    "texmf-dist/doc/latex/biblatex-chicago/cms-notes-intro.pdf",
    "texmf-dist/doc/latex/biblatex-chicago/cms-notes-sample.pdf",
    "texmf-dist/doc/latex/biblatex-chicago/cms-trad-appendix.pdf",
    "texmf-dist/doc/latex/biblatex-chicago/cms-trad-sample.pdf",
  },
  ["tugboat"] = {
    "texmf-dist/doc/latex/tugboat/README",
    "texmf-dist/doc/latex/tugboat/ltubguid.pdf",
    "texmf-dist/doc/latex/tugboat/manifest.txt",
    "texmf-dist/doc/latex/tugboat/tugboat.pdf",
  },
  ["tufte-latex"] = {
    "texmf-dist/doc/latex/tufte-latex/History.txt",
    "texmf-dist/doc/latex/tufte-latex/Manifest.txt",
    "texmf-dist/doc/latex/tufte-latex/README.txt",
    "texmf-dist/doc/latex/tufte-latex/graphics/be-contents.pdf",
    "texmf-dist/doc/latex/tufte-latex/graphics/be-title.pdf",
    "texmf-dist/doc/latex/tufte-latex/graphics/ei-contents.pdf",
    "texmf-dist/doc/latex/tufte-latex/graphics/ei-title.pdf",
    "texmf-dist/doc/latex/tufte-latex/graphics/helix.pdf",
    "texmf-dist/doc/latex/tufte-latex/graphics/hilbertcurves.pdf",
    "texmf-dist/doc/latex/tufte-latex/graphics/sine.pdf",
    "texmf-dist/doc/latex/tufte-latex/graphics/vdqi-contents.pdf",
    "texmf-dist/doc/latex/tufte-latex/graphics/vdqi-title.pdf",
    "texmf-dist/doc/latex/tufte-latex/graphics/ve-contents.pdf",
    "texmf-dist/doc/latex/tufte-latex/graphics/ve-title.pdf",
    "texmf-dist/doc/latex/tufte-latex/sample-book.pdf",
    "texmf-dist/doc/latex/tufte-latex/sample-handout.pdf",
  },
  ["ltxnew"] = {
    "texmf-dist/doc/latex/ltxnew/README",
    "texmf-dist/doc/latex/ltxnew/ltxnew.pdf",
  },
  ["pmxchords"] = {
    "texmf-dist/doc/man/man1/pmxchords.man1.pdf",
    "texmf-dist/doc/pmxchords/README",
    "texmf-dist/doc/pmxchords/chordsRef.pdf",
    "texmf-dist/doc/pmxchords/chordsRefCZ.pdf",
    "texmf-dist/doc/pmxchords/examples/jazz/misty/misty.pdf",
    "texmf-dist/doc/pmxchords/examples/jazz/schema/schema.pdf",
    "texmf-dist/doc/pmxchords/examples/jazz/schema/schema_full.pdf",
    "texmf-dist/doc/pmxchords/examples/noel/aj_co_to_hlasaju/aj_co_to_hlasaju.pdf",
    "texmf-dist/doc/pmxchords/examples/noel/pasli_ovce_valasi/README",
    "texmf-dist/doc/pmxchords/examples/noel/pasli_ovce_valasi/pasli_ovce_valasi.pdf",
    "texmf-dist/doc/pmxchords/gpl-2.0.txt",
    "texmf-dist/doc/pmxchords/pmxchords-install.pdf",
  },
  ["ifluatex"] = {
    "texmf-dist/doc/latex/oberdiek/ifluatex.pdf",
  },
  ["europasscv"] = {
    "texmf-dist/doc/latex/europasscv/README.md",
    "texmf-dist/doc/latex/europasscv/europasscv.pdf",
    "texmf-dist/doc/latex/europasscv/example/europasscv_en.pdf",
  },
  ["trfsigns"] = {
    "texmf-dist/doc/latex/trfsigns/COPYING",
    "texmf-dist/doc/latex/trfsigns/README",
    "texmf-dist/doc/latex/trfsigns/trfsigns.pdf",
  },
  ["babel-latin"] = {
    "texmf-dist/doc/generic/babel-latin/latin.pdf",
  },
  ["rcs-multi"] = {
    "texmf-dist/doc/latex/rcs-multi/example.pdf",
    "texmf-dist/doc/latex/rcs-multi/rcs-multi.pdf",
  },
  ["bbold-type1"] = {
    "texmf-dist/doc/fonts/bbold-type1/README",
    "texmf-dist/doc/fonts/bbold-type1/test.pdf",
  },
  ["lshort-french"] = {
    "texmf-dist/doc/latex/lshort-french/README",
    "texmf-dist/doc/latex/lshort-french/historique",
    "texmf-dist/doc/latex/lshort-french/lshort-fr.pdf",
  },
  ["apalike-german"] = {
    "texmf-dist/doc/bibtex/apalike-german/README.html",
    "texmf-dist/doc/bibtex/apalike-german/README.md",
  },
  ["cjkpunct"] = {
    "texmf-dist/doc/latex/cjkpunct/CJKpunct.pdf",
    "texmf-dist/doc/latex/cjkpunct/README.md",
    "texmf-dist/doc/latex/cjkpunct/setpunct/README.txt",
  },
  ["truncate"] = {
    "texmf-dist/doc/latex/truncate/truncate.pdf",
  },
  ["syntrace"] = {
    "texmf-dist/doc/latex/syntrace/README",
    "texmf-dist/doc/latex/syntrace/syntrace.pdf",
  },
  ["harvard"] = {
    "texmf-dist/doc/latex/harvard/INSTALL",
    "texmf-dist/doc/latex/harvard/README",
    "texmf-dist/doc/latex/harvard/harvard.pdf",
    "texmf-dist/doc/latex/harvard/manifest.txt",
  },
  ["trigonometry"] = {
    "texmf-dist/doc/generic/trigonometry/README.txt",
  },
  ["xmpincl"] = {
    "texmf-dist/doc/latex/xmpincl/README",
    "texmf-dist/doc/latex/xmpincl/xmpincl.pdf",
  },
  ["biblatex-morenames"] = {
    "texmf-dist/doc/latex/biblatex-morenames/README",
    "texmf-dist/doc/latex/biblatex-morenames/documentation/biblatex-morenames.pdf",
    "texmf-dist/doc/latex/biblatex-morenames/documentation/example-bookineditor-BookineditorFromEditor.pdf",
    "texmf-dist/doc/latex/biblatex-morenames/documentation/example-bookineditor.pdf",
    "texmf-dist/doc/latex/biblatex-morenames/documentation/example-maineditor.pdf",
    "texmf-dist/doc/latex/biblatex-morenames/documentation/latexmkrc",
    "texmf-dist/doc/latex/biblatex-morenames/documentation/makefile",
    "texmf-dist/doc/latex/biblatex-morenames/makefile",
  },
  ["translator"] = {
    "texmf-dist/doc/latex/translator/README.md",
    "texmf-dist/doc/latex/translator/translator.pdf",
  },
  ["translations"] = {
    "texmf-dist/doc/latex/translations/README",
    "texmf-dist/doc/latex/translations/translations_en.pdf",
  },
  ["macros2e"] = {
    "texmf-dist/doc/latex/macros2e/Makefile",
    "texmf-dist/doc/latex/macros2e/README",
    "texmf-dist/doc/latex/macros2e/macros2e.pdf",
  },
  ["translation-moreverb-de"] = {
    "texmf-dist/doc/latex/translation-moreverb-de/filecontens-de.ins.txt",
    "texmf-dist/doc/latex/translation-moreverb-de/moreverb-de.dtx.pdf",
  },
  ["translation-filecontents-de"] = {
    "texmf-dist/doc/latex/translation-filecontents-de/filecontents-de.pdf",
  },
  ["latex-via-exemplos"] = {
    "texmf-dist/doc/latex/latex-via-exemplos/README.md",
    "texmf-dist/doc/latex/latex-via-exemplos/changelog.txt",
    "texmf-dist/doc/latex/latex-via-exemplos/latex-via-exemplos-oneside.pdf",
    "texmf-dist/doc/latex/latex-via-exemplos/latex-via-exemplos.pdf",
  },
  ["xetex-itrans"] = {
    "texmf-dist/doc/xelatex/xetex-itrans/README",
    "texmf-dist/doc/xelatex/xetex-itrans/itrans-tamil-sample.pdf",
  },
  ["booktabs-fr"] = {
    "texmf-dist/doc/latex/booktabs-fr/README",
    "texmf-dist/doc/latex/booktabs-fr/f-booktabs.pdf",
  },
  ["translation-enumitem-de"] = {
    "texmf-dist/doc/latex/translation-enumitem-de/enumitem-de.pdf",
  },
  ["translation-dcolumn-fr"] = {
    "texmf-dist/doc/latex/translation-dcolumn-fr/Copyright",
    "texmf-dist/doc/latex/translation-dcolumn-fr/README",
    "texmf-dist/doc/latex/translation-dcolumn-fr/f-dcolumn.pdf",
  },
  ["translation-chemsym-de"] = {
    "texmf-dist/doc/latex/translation-chemsym-de/chemsym-de.pdf",
  },
  ["translation-biblatex-de"] = {
    "texmf-dist/doc/latex/translation-biblatex-de/README",
    "texmf-dist/doc/latex/translation-biblatex-de/biblatex-de-Benutzerhandbuch.pdf",
  },
  ["translation-arsclassica-de"] = {
    "texmf-dist/doc/latex/translation-arsclassica-de/ArsClassica-de.pdf",
  },
  ["tram"] = {
    "texmf-dist/doc/latex/tram/README",
    "texmf-dist/doc/latex/tram/tram-doc.pdf",
  },
  ["kpathsea"] = {
    "texmf-dist/doc/info/dir",
    "texmf-dist/doc/kpathsea/kpathsea.html",
    "texmf-dist/doc/kpathsea/kpathsea.pdf",
    "texmf-dist/doc/man/man1/kpseaccess.man1.pdf",
    "texmf-dist/doc/man/man1/kpsereadlink.man1.pdf",
    "texmf-dist/doc/man/man1/kpsestat.man1.pdf",
    "texmf-dist/doc/man/man1/kpsewhich.man1.pdf",
    "texmf-dist/doc/man/man1/mkocp.man1.pdf",
    "texmf-dist/doc/man/man1/mkofm.man1.pdf",
    "texmf-dist/doc/man/man1/mktexfmt.man1.pdf",
    "texmf-dist/doc/man/man1/mktexlsr.man1.pdf",
    "texmf-dist/doc/man/man1/mktexmf.man1.pdf",
    "texmf-dist/doc/man/man1/mktexpk.man1.pdf",
    "texmf-dist/doc/man/man1/mktextfm.man1.pdf",
    "texmf-dist/doc/man/man1/texhash.man1.pdf",
    "texmf-dist/doc/web2c/web2c.html",
    "texmf-dist/doc/web2c/web2c.pdf",
  },
  ["biblatex-swiss-legal"] = {
    "texmf-dist/doc/latex/biblatex-swiss-legal/README",
    "texmf-dist/doc/latex/biblatex-swiss-legal/biblatex-swiss-legal.pdf",
  },
  ["floatrow"] = {
    "texmf-dist/doc/latex/floatrow/README",
    "texmf-dist/doc/latex/floatrow/floatrow-rus.pdf",
    "texmf-dist/doc/latex/floatrow/floatrow.pdf",
  },
  ["tracklang"] = {
    "texmf-dist/doc/generic/tracklang/CHANGES",
    "texmf-dist/doc/generic/tracklang/README",
    "texmf-dist/doc/generic/tracklang/samples/animals/sample-animals-babel.pdf",
    "texmf-dist/doc/generic/tracklang/samples/animals/sample-animals-de-poly.pdf",
    "texmf-dist/doc/generic/tracklang/samples/animals/sample-animals-de.pdf",
    "texmf-dist/doc/generic/tracklang/samples/animals/sample-animals-poly.pdf",
    "texmf-dist/doc/generic/tracklang/samples/animals/sample-animals.pdf",
    "texmf-dist/doc/generic/tracklang/samples/animals/sample-animals2-de.pdf",
    "texmf-dist/doc/generic/tracklang/samples/animals/sample-animals2-de2.pdf",
    "texmf-dist/doc/generic/tracklang/samples/animals/sample-animals2-de3.pdf",
    "texmf-dist/doc/generic/tracklang/samples/animals/sample-animals2-de4.pdf",
    "texmf-dist/doc/generic/tracklang/samples/animals/sample-animals2-poly.pdf",
    "texmf-dist/doc/generic/tracklang/samples/animals/sample-animals2.pdf",
    "texmf-dist/doc/generic/tracklang/samples/regions/sample-regions.pdf",
    "texmf-dist/doc/generic/tracklang/samples/regions/sample-regions2-manx.pdf",
    "texmf-dist/doc/generic/tracklang/samples/regions/sample-regions2-map.pdf",
    "texmf-dist/doc/generic/tracklang/samples/regions/sample-regions2-map2.pdf",
    "texmf-dist/doc/generic/tracklang/samples/regions/sample-regions2-map3.pdf",
    "texmf-dist/doc/generic/tracklang/samples/regions/sample-regions2-nomap.pdf",
    "texmf-dist/doc/generic/tracklang/samples/regions/sample-regions2.pdf",
    "texmf-dist/doc/generic/tracklang/samples/sample-tracklang.pdf",
    "texmf-dist/doc/generic/tracklang/samples/sample-tracklang2.pdf",
    "texmf-dist/doc/generic/tracklang/samples/sample-tracklang3.pdf",
    "texmf-dist/doc/generic/tracklang/samples/sample-tracklang4.pdf",
    "texmf-dist/doc/generic/tracklang/samples/sample-tracklang5.pdf",
    "texmf-dist/doc/generic/tracklang/samples/sample-tracklang6.pdf",
    "texmf-dist/doc/generic/tracklang/tracklang.pdf",
  },
  ["tqft"] = {
    "texmf-dist/doc/latex/tqft/README",
    "texmf-dist/doc/latex/tqft/tqft.pdf",
    "texmf-dist/doc/latex/tqft/tqft_code.pdf",
  },
  ["tpslifonts"] = {
    "texmf-dist/doc/latex/tpslifonts/00readme.txt",
    "texmf-dist/doc/latex/tpslifonts/01install.txt",
    "texmf-dist/doc/latex/tpslifonts/Makefile",
  },
  ["lithuanian"] = {
    "texmf-dist/doc/latex/lithuanian/COPYING",
    "texmf-dist/doc/latex/lithuanian/ChangeLog",
    "texmf-dist/doc/latex/lithuanian/Copyright",
    "texmf-dist/doc/latex/lithuanian/README",
    "texmf-dist/doc/latex/lithuanian/latin7x.pdf",
  },
  ["texliveonfly"] = {
    "texmf-dist/doc/support/texliveonfly/README",
  },
  ["rterface"] = {
    "texmf-dist/doc/latex/rterface/README",
    "texmf-dist/doc/latex/rterface/rterface.pdf",
  },
  ["ltablex"] = {
    "texmf-dist/doc/latex/ltablex/README",
    "texmf-dist/doc/latex/ltablex/ltablex.pdf",
  },
  ["ellipse"] = {
    "texmf-dist/doc/latex/ellipse/README",
    "texmf-dist/doc/latex/ellipse/ellipse.pdf",
  },
  ["biblatex-ieee"] = {
    "texmf-dist/doc/latex/biblatex-ieee/README.md",
    "texmf-dist/doc/latex/biblatex-ieee/biblatex-ieee-alphabetic.pdf",
    "texmf-dist/doc/latex/biblatex-ieee/biblatex-ieee.pdf",
  },
  ["germkorr"] = {
    "texmf-dist/doc/latex/germkorr/COPYING",
    "texmf-dist/doc/latex/germkorr/README",
    "texmf-dist/doc/latex/germkorr/germkorr.pdf",
  },
  ["stubs"] = {
    "texmf-dist/doc/latex/stubs/COPYING",
    "texmf-dist/doc/latex/stubs/README",
    "texmf-dist/doc/latex/stubs/README.TEXLIVE",
    "texmf-dist/doc/latex/stubs/changelog.txt",
    "texmf-dist/doc/latex/stubs/stubs_ex.pdf",
  },
  ["fcltxdoc"] = {
    "texmf-dist/doc/latex/fcltxdoc/fcltxdoc.pdf",
  },
  ["scalebar"] = {
    "texmf-dist/doc/latex/scalebar/README",
    "texmf-dist/doc/latex/scalebar/scalebar_examples.pdf",
  },
  ["simurgh"] = {
    "texmf-dist/doc/lualatex/simurgh/ChangeLog",
    "texmf-dist/doc/lualatex/simurgh/README.doc",
    "texmf-dist/doc/lualatex/simurgh/simurgh-doc.pdf",
    "texmf-dist/doc/lualatex/simurgh/simurgh-logo.pdf",
  },
  ["br-lex"] = {
    "texmf-dist/doc/latex/br-lex/README.md",
    "texmf-dist/doc/latex/br-lex/brlex-doc.pdf",
    "texmf-dist/doc/latex/br-lex/leis-exemplo.pdf",
  },
  ["babel-czech"] = {
    "texmf-dist/doc/generic/babel-czech/czech.pdf",
  },
  ["ppr-prv"] = {
    "texmf-dist/doc/latex/ppr-prv/README",
    "texmf-dist/doc/latex/ppr-prv/ppr-prv.pdf",
  },
  ["geometry"] = {
    "texmf-dist/doc/latex/geometry/README.md",
    "texmf-dist/doc/latex/geometry/changes.txt",
    "texmf-dist/doc/latex/geometry/geometry.pdf",
  },
  ["tpic2pdftex"] = {
    "texmf-dist/doc/man/man1/tpic2pdftex.man1.pdf",
    "texmf-dist/doc/tpic2pdftex/Makefile",
    "texmf-dist/doc/tpic2pdftex/beamerexample.pdf",
    "texmf-dist/doc/tpic2pdftex/example.pdf",
  },
  ["totpages"] = {
    "texmf-dist/doc/latex/totpages/README",
    "texmf-dist/doc/latex/totpages/totpages.pdf",
  },
  ["miniplot"] = {
    "texmf-dist/doc/latex/miniplot/disclaimer.txt",
    "texmf-dist/doc/latex/miniplot/miniplot.pdf",
  },
  ["topletter"] = {
    "texmf-dist/doc/latex/topletter/EsempioLettera.pdf",
    "texmf-dist/doc/latex/topletter/ExampleLetter.pdf",
    "texmf-dist/doc/latex/topletter/LICENSE",
    "texmf-dist/doc/latex/topletter/LogoPolitoBlu.pdf",
    "texmf-dist/doc/latex/topletter/README.md",
    "texmf-dist/doc/latex/topletter/TOPletter.pdf",
    "texmf-dist/doc/latex/topletter/Walt_Disney_1942_signature.pdf",
  },
  ["regexpatch"] = {
    "texmf-dist/doc/latex/regexpatch/README",
    "texmf-dist/doc/latex/regexpatch/regexpatch.pdf",
  },
  ["glossaries-extra"] = {
    "texmf-dist/doc/latex/glossaries-extra/CHANGES",
    "texmf-dist/doc/latex/glossaries-extra/README",
    "texmf-dist/doc/latex/glossaries-extra/glossaries-extra-code.pdf",
    "texmf-dist/doc/latex/glossaries-extra/glossaries-extra-manual.html",
    "texmf-dist/doc/latex/glossaries-extra/glossaries-extra-manual.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-abbr-styles.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-abbrv.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-accsupp.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-acronym-desc.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-acronym.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-alias.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-altmodifier.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-alttree-marginpar.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-alttree-sym.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-alttree.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-autoindex-hyp.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-autoindex.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-crossref.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-crossref2.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-docdef.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-entrycount.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-external.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-fmt.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-footnote.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-header.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-indexhook.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-initialisms.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-linkcount.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-mixed-abbrv-styles.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-mixedsort.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-mixture.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-name-font.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-nested.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-noidx-restricteddocdefs.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-onelink.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-onthefly-utf8.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-onthefly-xetex.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-onthefly.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-pages.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-postdot.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-postlink.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-prefix.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-record.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-restricteddocdefs.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-suppl-hyp.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-suppl-main-hyp.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-suppl-main.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-suppl.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-trans.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-undef.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample-unitentrycount.pdf",
    "texmf-dist/doc/latex/glossaries-extra/sample.pdf",
  },
  ["tools"] = {
    "texmf-dist/doc/latex/tools/README.md",
    "texmf-dist/doc/latex/tools/README.txt",
    "texmf-dist/doc/latex/tools/afterpage.pdf",
    "texmf-dist/doc/latex/tools/array.pdf",
    "texmf-dist/doc/latex/tools/bm.pdf",
    "texmf-dist/doc/latex/tools/calc.pdf",
    "texmf-dist/doc/latex/tools/changes.txt",
    "texmf-dist/doc/latex/tools/dcolumn.pdf",
    "texmf-dist/doc/latex/tools/delarray.pdf",
    "texmf-dist/doc/latex/tools/enumerate.pdf",
    "texmf-dist/doc/latex/tools/fileerr.pdf",
    "texmf-dist/doc/latex/tools/fontsmpl.pdf",
    "texmf-dist/doc/latex/tools/ftnright.pdf",
    "texmf-dist/doc/latex/tools/hhline.pdf",
    "texmf-dist/doc/latex/tools/indentfirst.pdf",
    "texmf-dist/doc/latex/tools/layout.pdf",
    "texmf-dist/doc/latex/tools/longtable.pdf",
    "texmf-dist/doc/latex/tools/manifest.txt",
    "texmf-dist/doc/latex/tools/multicol.pdf",
    "texmf-dist/doc/latex/tools/rawfonts.pdf",
    "texmf-dist/doc/latex/tools/shellesc.pdf",
    "texmf-dist/doc/latex/tools/showkeys.pdf",
    "texmf-dist/doc/latex/tools/somedefs.pdf",
    "texmf-dist/doc/latex/tools/tabularx.pdf",
    "texmf-dist/doc/latex/tools/theorem.pdf",
    "texmf-dist/doc/latex/tools/tools-overview.pdf",
    "texmf-dist/doc/latex/tools/trace.pdf",
    "texmf-dist/doc/latex/tools/varioref.pdf",
    "texmf-dist/doc/latex/tools/verbatim.pdf",
    "texmf-dist/doc/latex/tools/xr.pdf",
    "texmf-dist/doc/latex/tools/xspace.pdf",
  },
  ["biblatex-nottsclassic"] = {
    "texmf-dist/doc/latex/biblatex-nottsclassic/README.md",
    "texmf-dist/doc/latex/biblatex-nottsclassic/nottsclassic.pdf",
  },
  ["stmaryrd"] = {
    "texmf-dist/doc/fonts/stmaryrd/INSTALL",
    "texmf-dist/doc/fonts/stmaryrd/README",
    "texmf-dist/doc/fonts/stmaryrd/README.hoekwater",
    "texmf-dist/doc/fonts/stmaryrd/stmaryrd.pdf",
  },
  ["tokenizer"] = {
    "texmf-dist/doc/latex/tokenizer/tokenizer.pdf",
  },
  ["todo"] = {
    "texmf-dist/doc/latex/todo/README",
    "texmf-dist/doc/latex/todo/todo-spl.pdf",
    "texmf-dist/doc/latex/todo/todo.pdf",
  },
  ["tocloft"] = {
    "texmf-dist/doc/latex/tocloft/README",
    "texmf-dist/doc/latex/tocloft/tocloft.pdf",
  },
  ["bxjaprnind"] = {
    "texmf-dist/doc/latex/bxjaprnind/LICENSE",
    "texmf-dist/doc/latex/bxjaprnind/README.md",
    "texmf-dist/doc/latex/bxjaprnind/bxjaprnind.pdf",
    "texmf-dist/doc/latex/bxjaprnind/sample-bxjaprnind.pdf",
  },
  ["fullminipage"] = {
    "texmf-dist/doc/latex/fullminipage/COPYING",
    "texmf-dist/doc/latex/fullminipage/Makefile",
    "texmf-dist/doc/latex/fullminipage/README",
    "texmf-dist/doc/latex/fullminipage/fullminipage.pdf",
    "texmf-dist/doc/latex/fullminipage/fullminipage_test.pdf",
  },
  ["alg"] = {
    "texmf-dist/doc/latex/alg/readme.txt",
  },
  ["tabularcalc"] = {
    "texmf-dist/doc/latex/tabularcalc/README",
    "texmf-dist/doc/latex/tabularcalc/tabularcalc_doc_en.pdf",
    "texmf-dist/doc/latex/tabularcalc/tabularcalc_doc_fr.pdf",
    "texmf-dist/doc/latex/tabularcalc/tabularcalc_doc_vn.pdf",
  },
  ["progress"] = {
    "texmf-dist/doc/latex/progress/README",
    "texmf-dist/doc/latex/progress/progress.pdf",
    "texmf-dist/doc/latex/progress/progress20030701.html",
  },
  ["thaienum"] = {
    "texmf-dist/doc/latex/thaienum/README.md",
    "texmf-dist/doc/latex/thaienum/thaienum.pdf",
  },
  ["amsfonts"] = {
    "texmf-dist/doc/fonts/amsfonts/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/amsfonts/OFL.txt",
    "texmf-dist/doc/fonts/amsfonts/README",
    "texmf-dist/doc/fonts/amsfonts/amsfndoc.pdf",
    "texmf-dist/doc/fonts/amsfonts/amsfonts.pdf",
    "texmf-dist/doc/fonts/amsfonts/amssymb.pdf",
    "texmf-dist/doc/fonts/amsfonts/cmmib57.pdf",
    "texmf-dist/doc/fonts/amsfonts/eufrak.pdf",
    "texmf-dist/doc/fonts/amsfonts/euscript.pdf",
  },
  ["tlshell"] = {
    "texmf-dist/doc/support/tlshell/README",
  },
  ["labyrinth"] = {
    "texmf-dist/doc/latex/labyrinth/README",
    "texmf-dist/doc/latex/labyrinth/labyrinth.pdf",
  },
  ["lps"] = {
    "texmf-dist/doc/latex/lps/README",
    "texmf-dist/doc/latex/lps/lps.pdf",
  },
  ["mathcomp"] = {
    "texmf-dist/doc/latex/mathcomp/mathcomp.pdf",
  },
  ["langsci"] = {
    "texmf-dist/doc/xelatex/langsci/README.md",
    "texmf-dist/doc/xelatex/langsci/documentation/langsci-doc.pdf",
    "texmf-dist/doc/xelatex/langsci/storagelogo.pdf",
  },
  ["spectralsequences"] = {
    "texmf-dist/doc/latex/spectralsequences/README.md",
    "texmf-dist/doc/latex/spectralsequences/examples/ANSS-S_2.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/ANSS-S_3.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/ASS-S_2.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/ASS-S_3.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/ASS-tmf_2.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/HFPSS-EO2_3.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/HFPSS-KO.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/J_RPinfty.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/MaySS-bo-A1.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/MaySS-tmf-A2.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/SSS-KF3n.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/SSS-KZ3.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/hatcher.pdf",
    "texmf-dist/doc/latex/spectralsequences/examples/unstable_ASS_SO.pdf",
    "texmf-dist/doc/latex/spectralsequences/manual/spectralsequences-howitworks.pdf",
    "texmf-dist/doc/latex/spectralsequences/manual/spectralsequencesmanual.pdf",
  },
  ["pst-fun"] = {
    "texmf-dist/doc/generic/pst-fun/Changes",
    "texmf-dist/doc/generic/pst-fun/README",
    "texmf-dist/doc/generic/pst-fun/pst-fun-doc.pdf",
  },
  ["poetrytex"] = {
    "texmf-dist/doc/latex/poetrytex/README",
    "texmf-dist/doc/latex/poetrytex/poetrytex.pdf",
  },
  ["iso"] = {
    "texmf-dist/doc/latex/iso/README",
    "texmf-dist/doc/latex/iso/iso4ht.pdf",
    "texmf-dist/doc/latex/iso/isoe.pdf",
    "texmf-dist/doc/latex/iso/isoman.pdf",
  },
  ["chkfloat"] = {
    "texmf-dist/doc/latex/chkfloat/README",
    "texmf-dist/doc/latex/chkfloat/chkfloat.pdf",
  },
  ["latex-graphics-companion"] = {
    "texmf-dist/doc/latex/latex-graphics-companion/Makefile",
    "texmf-dist/doc/latex/latex-graphics-companion/README",
    "texmf-dist/doc/latex/latex-graphics-companion/config.ps",
  },
  ["lshort-estonian"] = {
    "texmf-dist/doc/latex/lshort-estonian/README",
    "texmf-dist/doc/latex/lshort-estonian/lshort-ee-a5.pdf",
    "texmf-dist/doc/latex/lshort-estonian/lshort-ee.pdf",
  },
  ["pdfprivacy"] = {
    "texmf-dist/doc/latex/pdfprivacy/LICENSE",
    "texmf-dist/doc/latex/pdfprivacy/README.md",
    "texmf-dist/doc/latex/pdfprivacy/pdfprivacy.pdf",
  },
  ["jadetex"] = {
    "texmf-dist/doc/man/man1/jadetex.man1.pdf",
    "texmf-dist/doc/man/man1/pdfjadetex.man1.pdf",
    "texmf-dist/doc/otherformats/jadetex/base/ChangeLog",
    "texmf-dist/doc/otherformats/jadetex/base/ChangeLog-old",
    "texmf-dist/doc/otherformats/jadetex/base/Makefile",
    "texmf-dist/doc/otherformats/jadetex/base/index.html",
  },
  ["uantwerpendocs"] = {
    "texmf-dist/doc/latex/uantwerpendocs/LICENSE",
    "texmf-dist/doc/latex/uantwerpendocs/README",
    "texmf-dist/doc/latex/uantwerpendocs/uantwerpencoursetext-example.pdf",
    "texmf-dist/doc/latex/uantwerpendocs/uantwerpendocs.pdf",
    "texmf-dist/doc/latex/uantwerpendocs/uantwerpenexam-example1.pdf",
    "texmf-dist/doc/latex/uantwerpendocs/uantwerpenexam-example2.pdf",
    "texmf-dist/doc/latex/uantwerpendocs/uantwerpenletter-example.pdf",
    "texmf-dist/doc/latex/uantwerpendocs/uantwerpenmasterthesis-example.pdf",
    "texmf-dist/doc/latex/uantwerpendocs/uantwerpenphdthesis-example1.pdf",
    "texmf-dist/doc/latex/uantwerpendocs/uantwerpenphdthesis-example2.pdf",
  },
  ["glossaries-danish"] = {
    "texmf-dist/doc/latex/glossaries-danish/README",
    "texmf-dist/doc/latex/glossaries-danish/glossaries-danish.pdf",
  },
  ["tlc2"] = {
    "texmf-dist/doc/latex/tlc2/Escher.ps",
    "texmf-dist/doc/latex/tlc2/cat.ps",
    "texmf-dist/doc/latex/tlc2/elephant.ps",
    "texmf-dist/doc/latex/tlc2/rosette.ps",
    "texmf-dist/doc/latex/tlc2/w.ps",
  },
  ["here"] = {
    "texmf-dist/doc/latex/here/README",
  },
  ["nath"] = {
    "texmf-dist/doc/latex/nath/README",
    "texmf-dist/doc/latex/nath/nathguide.pdf",
  },
  ["path"] = {
    "texmf-dist/doc/generic/path/path-doc.pdf",
  },
  ["scratchx"] = {
    "texmf-dist/doc/latex/scratchx/Explanations_ScratchX.pdf",
    "texmf-dist/doc/latex/scratchx/Explications_ScratchX.pdf",
    "texmf-dist/doc/latex/scratchx/README.txt",
  },
  ["math-e"] = {
    "texmf-dist/doc/latex/math-e/README",
  },
  ["tkz-orm"] = {
    "texmf-dist/doc/latex/tkz-orm/LICENSE",
    "texmf-dist/doc/latex/tkz-orm/Makefile",
    "texmf-dist/doc/latex/tkz-orm/README",
    "texmf-dist/doc/latex/tkz-orm/tkz-orm.pdf",
  },
  ["tabls"] = {
    "texmf-dist/doc/latex/tabls/tabls.pdf",
  },
  ["tkz-kiviat"] = {
    "texmf-dist/doc/latex/tkz-kiviat/TKZdoc-kiviat-main.pdf",
  },
  ["tkz-graph"] = {
    "texmf-dist/doc/latex/tkz-graph/readme-tkz-graph.txt",
    "texmf-dist/doc/latex/tkz-graph/tkz-graph-screen.pdf",
  },
  ["lineara"] = {
    "texmf-dist/doc/fonts/linearA/README",
    "texmf-dist/doc/fonts/linearA/linearA_glyphs.pdf",
  },
  ["tkz-fct"] = {
    "texmf-dist/doc/latex/tkz-fct/README",
    "texmf-dist/doc/latex/tkz-fct/examples/Makefile",
    "texmf-dist/doc/latex/tkz-fct/examples/info_fct_tex.txt",
    "texmf-dist/doc/latex/tkz-fct/readme-tkz-fct-fr.txt",
    "texmf-dist/doc/latex/tkz-fct/readme-tkz-fct.txt",
    "texmf-dist/doc/latex/tkz-fct/readme.tex",
    "texmf-dist/doc/latex/tkz-fct/tkz-fct-screen.pdf",
  },
  ["thumbpdf"] = {
    "texmf-dist/doc/generic/thumbpdf/README",
    "texmf-dist/doc/man/man1/thumbpdf.man1.pdf",
  },
  ["listofitems"] = {
    "texmf-dist/doc/generic/listofitems/README",
    "texmf-dist/doc/generic/listofitems/listofitems-en.pdf",
    "texmf-dist/doc/generic/listofitems/listofitems-fr.pdf",
  },
  ["hausarbeit-jura"] = {
    "texmf-dist/doc/latex/hausarbeit-jura/README.md",
    "texmf-dist/doc/latex/hausarbeit-jura/hausarbeit-jura.pdf",
  },
  ["xfor"] = {
    "texmf-dist/doc/latex/xfor/CHANGES",
    "texmf-dist/doc/latex/xfor/README",
    "texmf-dist/doc/latex/xfor/xfor.pdf",
  },
  ["allrunes"] = {
    "texmf-dist/doc/fonts/allrunes/README.txt",
    "texmf-dist/doc/fonts/allrunes/allrunes.pdf",
  },
  ["tkz-base"] = {
    "texmf-dist/doc/latex/tkz-base/README",
    "texmf-dist/doc/latex/tkz-base/examples/Makefile",
    "texmf-dist/doc/latex/tkz-base/examples/info_base_tex.txt",
    "texmf-dist/doc/latex/tkz-base/tkz-base-screen.pdf",
  },
  ["titling"] = {
    "texmf-dist/doc/latex/titling/README",
    "texmf-dist/doc/latex/titling/titling.pdf",
  },
  ["titlesec"] = {
    "texmf-dist/doc/latex/titlesec/CHANGES",
    "texmf-dist/doc/latex/titlesec/README",
    "texmf-dist/doc/latex/titlesec/titleps.pdf",
    "texmf-dist/doc/latex/titlesec/titlesec.pdf",
  },
  ["titleref"] = {
    "texmf-dist/doc/latex/titleref/titleref.pdf",
  },
  ["lisp-on-tex"] = {
    "texmf-dist/doc/latex/lisp-on-tex/LICENSE",
    "texmf-dist/doc/latex/lisp-on-tex/README.md",
    "texmf-dist/doc/latex/lisp-on-tex/examples/div2.pdf",
    "texmf-dist/doc/latex/lisp-on-tex/examples/fact.pdf",
    "texmf-dist/doc/latex/lisp-on-tex/examples/fpnummodule-mandelbrot.pdf",
    "texmf-dist/doc/latex/lisp-on-tex/examples/nqueen.pdf",
    "texmf-dist/doc/latex/lisp-on-tex/examples/rocket.pdf",
    "texmf-dist/doc/latex/lisp-on-tex/examples/showfont.pdf",
  },
  ["vlna"] = {
    "texmf-dist/doc/man/man1/vlna.man1.pdf",
    "texmf-dist/doc/vlna/vlna.pdf",
  },
  ["hyperbar"] = {
    "texmf-dist/doc/latex/hyperbar/README.md",
    "texmf-dist/doc/latex/hyperbar/example.pdf",
    "texmf-dist/doc/latex/hyperbar/hyperbar.pdf",
  },
  ["memexsupp"] = {
    "texmf-dist/doc/latex/memexsupp/README",
    "texmf-dist/doc/latex/memexsupp/memexsupp.pdf",
  },
  ["enotez"] = {
    "texmf-dist/doc/latex/enotez/README",
    "texmf-dist/doc/latex/enotez/enotez_en.pdf",
  },
  ["tipfr"] = {
    "texmf-dist/doc/latex/tipfr/README",
    "texmf-dist/doc/latex/tipfr/tipfr-doc.pdf",
  },
  ["tipa-de"] = {
    "texmf-dist/doc/latex/tipa-de/LIESMICH",
    "texmf-dist/doc/latex/tipa-de/tipaman-de.pdf",
  },
  ["timing-diagrams"] = {
    "texmf-dist/doc/latex/timing-diagrams/Makefile",
    "texmf-dist/doc/latex/timing-diagrams/README",
    "texmf-dist/doc/latex/timing-diagrams/diagrams-examples.pdf",
    "texmf-dist/doc/latex/timing-diagrams/version.txt",
  },
  ["francais-bst"] = {
    "texmf-dist/doc/bibtex/francais-bst/README",
  },
  ["synproof"] = {
    "texmf-dist/doc/latex/synproof/README",
    "texmf-dist/doc/latex/synproof/synproof-doc.pdf",
  },
  ["tikzposter"] = {
    "texmf-dist/doc/latex/tikzposter/README",
    "texmf-dist/doc/latex/tikzposter/tikzposter.pdf",
  },
  ["tikzpfeile"] = {
    "texmf-dist/doc/latex/tikzpfeile/README",
    "texmf-dist/doc/latex/tikzpfeile/tikzpfeile.pdf",
  },
  ["tikzpeople"] = {
    "texmf-dist/doc/latex/tikzpeople/README.md",
    "texmf-dist/doc/latex/tikzpeople/tikzpeople.pdf",
  },
  ["tikzpagenodes"] = {
    "texmf-dist/doc/latex/tikzpagenodes/README",
    "texmf-dist/doc/latex/tikzpagenodes/tikzpagenodes.pdf",
  },
  ["termcal-de"] = {
    "texmf-dist/doc/latex/termcal-de/README.md",
    "texmf-dist/doc/latex/termcal-de/termcal-de-doc.pdf",
  },
  ["blowup"] = {
    "texmf-dist/doc/latex/blowup/README.md",
    "texmf-dist/doc/latex/blowup/blowup-ex1.pdf",
    "texmf-dist/doc/latex/blowup/blowup-ex2.pdf",
    "texmf-dist/doc/latex/blowup/blowup-ex3.pdf",
    "texmf-dist/doc/latex/blowup/blowup-ex4.pdf",
    "texmf-dist/doc/latex/blowup/blowup-ex5.pdf",
    "texmf-dist/doc/latex/blowup/blowup.pdf",
  },
  ["snotez"] = {
    "texmf-dist/doc/latex/snotez/README",
    "texmf-dist/doc/latex/snotez/snotez_en.pdf",
  },
  ["tikzorbital"] = {
    "texmf-dist/doc/latex/tikzorbital/README.rst",
    "texmf-dist/doc/latex/tikzorbital/tikzorbital.pdf",
  },
  ["tikzmark"] = {
    "texmf-dist/doc/latex/tikzmark/README",
    "texmf-dist/doc/latex/tikzmark/tikzmark.pdf",
  },
  ["paralist"] = {
    "texmf-dist/doc/latex/paralist/README",
    "texmf-dist/doc/latex/paralist/paralist.pdf",
  },
  ["tikzinclude"] = {
    "texmf-dist/doc/latex/tikzinclude/README",
    "texmf-dist/doc/latex/tikzinclude/tikzinclude.pdf",
  },
  ["tikzducks"] = {
    "texmf-dist/doc/latex/tikzducks/LICENSE.txt",
    "texmf-dist/doc/latex/tikzducks/README.md",
    "texmf-dist/doc/latex/tikzducks/tikzducks-doc.pdf",
  },
  ["tikzcodeblocks"] = {
    "texmf-dist/doc/latex/tikzcodeblocks/README.md",
    "texmf-dist/doc/latex/tikzcodeblocks/tikzcodeblocks-documentation.pdf",
  },
  ["fjodor"] = {
    "texmf-dist/doc/latex/fjodor/README",
    "texmf-dist/doc/latex/fjodor/dostojevski.pdf",
    "texmf-dist/doc/latex/fjodor/fjodor.pdf",
  },
  ["ptex2pdf"] = {
    "texmf-dist/doc/latex/ptex2pdf/COPYING",
    "texmf-dist/doc/latex/ptex2pdf/README.md",
  },
  ["tikz-qtree"] = {
    "texmf-dist/doc/latex/tikz-qtree/README",
    "texmf-dist/doc/latex/tikz-qtree/tikz-qtree-manual.pdf",
  },
  ["context-fullpage"] = {
    "texmf-dist/doc/context/third/fullpage/README",
    "texmf-dist/doc/context/third/fullpage/VERSION",
    "texmf-dist/doc/context/third/fullpage/fullpage-doc.pdf",
  },
  ["tikz-palattice"] = {
    "texmf-dist/doc/latex/tikz-palattice/Makefile",
    "texmf-dist/doc/latex/tikz-palattice/README",
    "texmf-dist/doc/latex/tikz-palattice/tikz-palattice_documentation.pdf",
  },
  ["latex-tabellen"] = {
    "texmf-dist/doc/latex/latex-tabellen/README",
  },
  ["tikz-page"] = {
    "texmf-dist/doc/latex/tikz-page/Makefile",
    "texmf-dist/doc/latex/tikz-page/README",
    "texmf-dist/doc/latex/tikz-page/README.md",
    "texmf-dist/doc/latex/tikz-page/tikz-page.pdf",
  },
  ["biblatex-cheatsheet"] = {
    "texmf-dist/doc/latex/biblatex-cheatsheet/README",
    "texmf-dist/doc/latex/biblatex-cheatsheet/biblatex-cheatsheet.pdf",
  },
  ["tikz-optics"] = {
    "texmf-dist/doc/latex/tikz-optics/README",
    "texmf-dist/doc/latex/tikz-optics/tikz-optics.pdf",
  },
  ["tikz-opm"] = {
    "texmf-dist/doc/latex/tikz-opm/README",
    "texmf-dist/doc/latex/tikz-opm/tikz-opm.pdf",
  },
  ["pst-solides3d"] = {
    "texmf-dist/doc/generic/pst-solides3d/Changes",
    "texmf-dist/doc/generic/pst-solides3d/README",
    "texmf-dist/doc/generic/pst-solides3d/pst-solides3d-doc.pdf",
  },
  ["egplot"] = {
    "texmf-dist/doc/latex/egplot/egplot.pdf",
  },
  ["sauerj"] = {
    "texmf-dist/doc/latex/sauerj/README",
    "texmf-dist/doc/latex/sauerj/collect.pdf",
    "texmf-dist/doc/latex/sauerj/metainfo.pdf",
    "texmf-dist/doc/latex/sauerj/optparams.pdf",
    "texmf-dist/doc/latex/sauerj/parcolumns.pdf",
    "texmf-dist/doc/latex/sauerj/processkv.pdf",
    "texmf-dist/doc/latex/sauerj/zahl2string.pdf",
  },
  ["tikz-ladder"] = {
    "texmf-dist/doc/latex/tikz-ladder/README.txt",
    "texmf-dist/doc/latex/tikz-ladder/tikz-ladder-doc.pdf",
  },
  ["tikz-inet"] = {
    "texmf-dist/doc/latex/tikz-inet/README",
    "texmf-dist/doc/latex/tikz-inet/tikz-inet-doc.pdf",
  },
  ["eledform"] = {
    "texmf-dist/doc/latex/eledform/README.md",
    "texmf-dist/doc/latex/eledform/eledform.pdf",
    "texmf-dist/doc/latex/eledform/example.pdf",
    "texmf-dist/doc/latex/eledform/makefile",
  },
  ["tikz-dimline"] = {
    "texmf-dist/doc/latex/tikz-dimline/README",
    "texmf-dist/doc/latex/tikz-dimline/tikz-dimline-doc.pdf",
  },
  ["tikz-cd"] = {
    "texmf-dist/doc/latex/tikz-cd/README",
    "texmf-dist/doc/latex/tikz-cd/tikz-cd-doc.pdf",
  },
  ["gfssolomos"] = {
    "texmf-dist/doc/fonts/gfssolomos/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gfssolomos/OFL.txt",
    "texmf-dist/doc/fonts/gfssolomos/README",
    "texmf-dist/doc/fonts/gfssolomos/gfssolomos.pdf",
  },
  ["gsftopk"] = {
    "texmf-dist/doc/man/man1/gsftopk.man1.pdf",
  },
  ["iscram"] = {
    "texmf-dist/doc/latex/iscram/HMI.pdf",
    "texmf-dist/doc/latex/iscram/README",
    "texmf-dist/doc/latex/iscram/iscram-class-doc.pdf",
  },
  ["logical-markup-utils"] = {
    "texmf-dist/doc/latex/logical-markup-utils/COPYING",
    "texmf-dist/doc/latex/logical-markup-utils/README",
    "texmf-dist/doc/latex/logical-markup-utils/TODO",
  },
  ["tikz-bayesnet"] = {
    "texmf-dist/doc/latex/tikz-bayesnet/LICENSE_GPL",
    "texmf-dist/doc/latex/tikz-bayesnet/LICENSE_LPPL",
    "texmf-dist/doc/latex/tikz-bayesnet/README.rst",
  },
  ["tikz-3dplot"] = {
    "texmf-dist/doc/latex/tikz-3dplot/CHANGELOG",
    "texmf-dist/doc/latex/tikz-3dplot/README",
    "texmf-dist/doc/latex/tikz-3dplot/tikz-3dplot_documentation.pdf",
  },
  ["datetime2-swedish"] = {
    "texmf-dist/doc/latex/datetime2-swedish/README",
    "texmf-dist/doc/latex/datetime2-swedish/datetime2-swedish.pdf",
  },
  ["newsletr"] = {
    "texmf-dist/doc/plain/newsletr/newsletr.txt",
    "texmf-dist/doc/plain/newsletr/newssamp.pdf",
    "texmf-dist/doc/plain/newsletr/sample.pdf",
  },
  ["delim"] = {
    "texmf-dist/doc/latex/delim/README",
    "texmf-dist/doc/latex/delim/delim.pdf",
  },
  ["aucklandthesis"] = {
    "texmf-dist/doc/latex/aucklandthesis/README.txt",
    "texmf-dist/doc/latex/aucklandthesis/aucklandthesis_userguide.pdf",
  },
  ["makebarcode"] = {
    "texmf-dist/doc/latex/makebarcode/License.txt",
    "texmf-dist/doc/latex/makebarcode/README",
    "texmf-dist/doc/latex/makebarcode/makebarcode.pdf",
  },
  ["gentle"] = {
    "texmf-dist/doc/plain/gentle/gentle.pdf",
  },
  ["lshort-german"] = {
    "texmf-dist/doc/latex/lshort-german/CHANGES",
    "texmf-dist/doc/latex/lshort-german/README.l2kurz",
    "texmf-dist/doc/latex/lshort-german/fontspec.pdf",
    "texmf-dist/doc/latex/lshort-german/l2kurz.pdf",
    "texmf-dist/doc/latex/lshort-german/latexmkrc",
    "texmf-dist/doc/latex/lshort-german/ozean.pdf",
  },
  ["morewrites"] = {
    "texmf-dist/doc/latex/morewrites/README.md",
    "texmf-dist/doc/latex/morewrites/morewrites.pdf",
    "texmf-dist/doc/latex/morewrites/primargs.pdf",
  },
  ["tie"] = {
    "texmf-dist/doc/man/man1/tie.man1.pdf",
  },
  ["bohr"] = {
    "texmf-dist/doc/latex/bohr/README",
    "texmf-dist/doc/latex/bohr/bohr_en.pdf",
  },
  ["junicode"] = {
    "texmf-dist/doc/fonts/junicode/Junicode.pdf",
    "texmf-dist/doc/fonts/junicode/README",
    "texmf-dist/doc/fonts/junicode/aelfric_job.pdf",
    "texmf-dist/doc/fonts/junicode/fleurons",
    "texmf-dist/doc/fonts/junicode/homer_sample.pdf",
    "texmf-dist/doc/fonts/junicode/replacements",
  },
  ["ticket"] = {
    "texmf-dist/doc/latex/ticket/README.txt",
    "texmf-dist/doc/latex/ticket/comment",
    "texmf-dist/doc/latex/ticket/ex_file.pdf",
    "texmf-dist/doc/latex/ticket/ex_flashcard.pdf",
    "texmf-dist/doc/latex/ticket/ex_flashcard_dup.pdf",
    "texmf-dist/doc/latex/ticket/ex_flashcard_rm.pdf",
    "texmf-dist/doc/latex/ticket/ex_marks.pdf",
    "texmf-dist/doc/latex/ticket/ex_pin.pdf",
    "texmf-dist/doc/latex/ticket/ex_vcard.pdf",
    "texmf-dist/doc/latex/ticket/manual.pdf",
  },
  ["thuthesis"] = {
    "texmf-dist/doc/latex/thuthesis/Makefile",
    "texmf-dist/doc/latex/thuthesis/README.md",
    "texmf-dist/doc/latex/thuthesis/figures/scan-auth.pdf",
    "texmf-dist/doc/latex/thuthesis/figures/scan-record.pdf",
    "texmf-dist/doc/latex/thuthesis/figures/scan-statement.pdf",
    "texmf-dist/doc/latex/thuthesis/figures/thu-fig-logo.pdf",
    "texmf-dist/doc/latex/thuthesis/figures/thu-lib-logo.pdf",
    "texmf-dist/doc/latex/thuthesis/figures/thu-text-logo.pdf",
    "texmf-dist/doc/latex/thuthesis/figures/thu-whole-logo.pdf",
    "texmf-dist/doc/latex/thuthesis/latexmkrc",
    "texmf-dist/doc/latex/thuthesis/main.pdf",
    "texmf-dist/doc/latex/thuthesis/shuji.pdf",
    "texmf-dist/doc/latex/thuthesis/thuthesis.pdf",
    "texmf-dist/doc/latex/thuthesis/tsinghua.pdf",
  },
  ["context-mathsets"] = {
    "texmf-dist/doc/context/third/mathsets/VERSION",
    "texmf-dist/doc/context/third/mathsets/mathsets-doc.pdf",
  },
  ["eurosym"] = {
    "texmf-dist/doc/fonts/eurosym/COPYING",
    "texmf-dist/doc/fonts/eurosym/Changes",
    "texmf-dist/doc/fonts/eurosym/Makefile",
    "texmf-dist/doc/fonts/eurosym/README",
    "texmf-dist/doc/fonts/eurosym/README.type1",
    "texmf-dist/doc/fonts/eurosym/c/Makefile",
    "texmf-dist/doc/fonts/eurosym/makemfs",
    "texmf-dist/doc/fonts/eurosym/rundvips",
    "texmf-dist/doc/fonts/eurosym/runlatex",
    "texmf-dist/doc/fonts/eurosym/src/Makefile",
    "texmf-dist/doc/fonts/eurosym/testeuro.pdf",
  },
  ["lshort-dutch"] = {
    "texmf-dist/doc/latex/lshort-dutch/LEESMIJ",
    "texmf-dist/doc/latex/lshort-dutch/README",
    "texmf-dist/doc/latex/lshort-dutch/WIJZIGINGEN",
    "texmf-dist/doc/latex/lshort-dutch/lshort-nl-1.3.pdf",
  },
  ["babel-albanian"] = {
    "texmf-dist/doc/generic/babel-albanian/albanian.pdf",
  },
  ["quoting"] = {
    "texmf-dist/doc/latex/quoting/README",
    "texmf-dist/doc/latex/quoting/quoting.pdf",
  },
  ["musixtex"] = {
    "texmf-dist/doc/generic/musixtex/ChangeLog-114.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-115.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-116.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-117.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-118.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-119.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-120.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-121.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-122.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-123.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-124.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-125.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-126.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-127.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-128.txt",
    "texmf-dist/doc/generic/musixtex/ChangeLog-musixdoc.txt",
    "texmf-dist/doc/generic/musixtex/README",
    "texmf-dist/doc/generic/musixtex/musixcrd/doc.pdf",
    "texmf-dist/doc/generic/musixtex/musixdoc.pdf",
    "texmf-dist/doc/generic/musixtex/musixdoc/musixdoc.pdf",
    "texmf-dist/doc/generic/musixtex/musixlyr/README.musixlyr",
    "texmf-dist/doc/generic/musixtex/musixlyr/examples/nonmoriar.pdf",
    "texmf-dist/doc/generic/musixtex/musixlyr/mxlyrdoc.pdf",
    "texmf-dist/doc/generic/musixtex/musixtex-install.pdf",
    "texmf-dist/doc/generic/musixtex/notationMistakes/sottieng.pdf",
    "texmf-dist/doc/generic/musixtex/notationMistakes/sottiger.pdf",
    "texmf-dist/doc/generic/musixtex/notationMistakes/sottigra.pdf",
    "texmf-dist/doc/generic/musixtex/psslurs/README.psslurs",
    "texmf-dist/doc/generic/musixtex/psslurs/slurs.pdf",
    "texmf-dist/doc/generic/musixtex/reference/musixre1.pdf",
    "texmf-dist/doc/generic/musixtex/scripts/musixflx.pdf",
    "texmf-dist/doc/generic/musixtex/scripts/musixtex.pdf",
    "texmf-dist/doc/man/man1/musixflx.man1.pdf",
    "texmf-dist/doc/man/man1/musixtex.man1.pdf",
  },
  ["hyph-utf8"] = {
    "texmf-dist/doc/generic/hyph-utf8/CHANGES",
    "texmf-dist/doc/generic/hyph-utf8/HISTORY",
    "texmf-dist/doc/generic/hyph-utf8/hyph-utf8.pdf",
    "texmf-dist/doc/generic/hyph-utf8/hyphenation-distribution.pdf",
    "texmf-dist/doc/luatex/hyph-utf8/README",
    "texmf-dist/doc/luatex/hyph-utf8/luatex-hyphen.pdf",
  },
  ["svg"] = {
    "texmf-dist/doc/latex/svg/README.md",
    "texmf-dist/doc/latex/svg/svg.pdf",
  },
  ["context-chromato"] = {
    "texmf-dist/doc/context/third/chromato/VERSION",
    "texmf-dist/doc/context/third/chromato/chromato-demo.pdf",
    "texmf-dist/doc/context/third/chromato/chromato-doc.pdf",
  },
  ["spark-otf"] = {
    "texmf-dist/doc/fonts/spark-otf/Changes",
    "texmf-dist/doc/fonts/spark-otf/README.md",
    "texmf-dist/doc/fonts/spark-otf/spark-otf-doc.pdf",
  },
  ["realboxes"] = {
    "texmf-dist/doc/latex/realboxes/README",
    "texmf-dist/doc/latex/realboxes/realboxes.pdf",
  },
  ["havannah"] = {
    "texmf-dist/doc/latex/havannah/README",
    "texmf-dist/doc/latex/havannah/havannah.pdf",
  },
  ["font-change"] = {
    "texmf-dist/doc/plain/font-change/README.txt",
    "texmf-dist/doc/plain/font-change/font-change.pdf",
    "texmf-dist/doc/plain/font-change/font-change_FRENCH.pdf",
  },
  ["thumb"] = {
    "texmf-dist/doc/latex/thumb/README",
    "texmf-dist/doc/latex/thumb/thumb.pdf",
  },
  ["thucoursework"] = {
    "texmf-dist/doc/latex/thucoursework/Makefile",
    "texmf-dist/doc/latex/thucoursework/README.md",
    "texmf-dist/doc/latex/thucoursework/iihw.pdf",
    "texmf-dist/doc/latex/thucoursework/ithw.pdf",
    "texmf-dist/doc/latex/thucoursework/thucoursework.pdf",
  },
  ["competences"] = {
    "texmf-dist/doc/latex/competences/README.md",
  },
  ["threeparttable"] = {
    "texmf-dist/doc/latex/threeparttable/threeparttable.pdf",
  },
  ["placeins"] = {
    "texmf-dist/doc/latex/placeins/placeins-doc.pdf",
    "texmf-dist/doc/latex/placeins/placeins.txt",
  },
  ["facsimile"] = {
    "texmf-dist/doc/latex/facsimile/README",
    "texmf-dist/doc/latex/facsimile/facsimile.pdf",
  },
  ["cweb"] = {
    "texmf-dist/doc/man/man1/ctangle.man1.pdf",
    "texmf-dist/doc/man/man1/cweave.man1.pdf",
    "texmf-dist/doc/man/man1/cweb.man1.pdf",
    "texmf-dist/doc/plain/cweb/cwebman.pdf",
  },
  ["thmbox"] = {
    "texmf-dist/doc/latex/thmbox/README",
    "texmf-dist/doc/latex/thmbox/thmbox.pdf",
  },
  ["uiucredborder"] = {
    "texmf-dist/doc/latex/uiucredborder/uiucredborder.pdf",
  },
  ["thesis-ekf"] = {
    "texmf-dist/doc/latex/thesis-ekf/README",
    "texmf-dist/doc/latex/thesis-ekf/thesis-ekf-templates/figures/eke-logo.pdf",
    "texmf-dist/doc/latex/thesis-ekf/thesis-ekf-templates/figures/eku-logo.pdf",
    "texmf-dist/doc/latex/thesis-ekf/thesis-ekf.pdf",
  },
  ["cmpica"] = {
    "texmf-dist/doc/latex/cmpica/README",
  },
  ["theoremref"] = {
    "texmf-dist/doc/latex/theoremref/COPYING",
    "texmf-dist/doc/latex/theoremref/README",
    "texmf-dist/doc/latex/theoremref/theoremref-doc.pdf",
  },
  ["umich-thesis"] = {
    "texmf-dist/doc/latex/umich-thesis/README",
  },
  ["perltex"] = {
    "texmf-dist/doc/latex/perltex/README",
    "texmf-dist/doc/latex/perltex/perltex.pdf",
    "texmf-dist/doc/man/man1/perltex.man1.pdf",
  },
  ["thaispec"] = {
    "texmf-dist/doc/latex/thaispec/README.md",
    "texmf-dist/doc/latex/thaispec/thaispec.pdf",
  },
  ["accanthis"] = {
    "texmf-dist/doc/fonts/accanthis/Accanthis-Cat.pdf",
    "texmf-dist/doc/fonts/accanthis/COPYING",
    "texmf-dist/doc/fonts/accanthis/NOTICE.txt",
    "texmf-dist/doc/fonts/accanthis/README",
    "texmf-dist/doc/fonts/accanthis/accanthis-samples.pdf",
  },
  ["texworks"] = {
    "texmf-dist/doc/texworks/README",
  },
  ["carlito"] = {
    "texmf-dist/doc/fonts/carlito/Carlito-Regular.pdf",
    "texmf-dist/doc/fonts/carlito/LICENSE",
    "texmf-dist/doc/fonts/carlito/README",
    "texmf-dist/doc/fonts/carlito/samples.pdf",
  },
  ["beamer"] = {
    "texmf-dist/doc/latex/beamer/README.md",
    "texmf-dist/doc/latex/beamer/beamerexample-conference-talk.pdf",
    "texmf-dist/doc/latex/beamer/beamerexample-lecture-beamer-version.pdf",
    "texmf-dist/doc/latex/beamer/beamerexample-lecture-logo.pdf",
    "texmf-dist/doc/latex/beamer/beamerexample-lecture-print-version.pdf",
    "texmf-dist/doc/latex/beamer/beamerlogo.pdf",
    "texmf-dist/doc/latex/beamer/beameruserguide.pdf",
    "texmf-dist/doc/latex/beamer/fdl.txt",
    "texmf-dist/doc/latex/beamer/gpl-2.0.txt",
    "texmf-dist/doc/latex/beamer/lppl-1-3c.txt",
    "texmf-dist/doc/latex/beamer/manifest-code.txt",
    "texmf-dist/doc/latex/beamer/manifest-documentation.txt",
  },
  ["refcheck"] = {
    "texmf-dist/doc/latex/refcheck/README",
    "texmf-dist/doc/latex/refcheck/refdemo.pdf",
  },
  ["leadsheets"] = {
    "texmf-dist/doc/latex/leadsheets/README",
    "texmf-dist/doc/latex/leadsheets/leadsheets_en.pdf",
  },
  ["feynmf"] = {
    "texmf-dist/doc/latex/feynmf/Announce",
    "texmf-dist/doc/latex/feynmf/COPYING",
    "texmf-dist/doc/latex/feynmf/README",
    "texmf-dist/doc/latex/feynmf/Tutorial",
    "texmf-dist/doc/latex/feynmf/fmfman.pdf",
  },
  ["ebgaramond-maths"] = {
    "texmf-dist/doc/fonts/ebgaramond-maths/README",
    "texmf-dist/doc/fonts/ebgaramond-maths/ebgaramond-maths.pdf",
    "texmf-dist/doc/fonts/ebgaramond-maths/manifest.txt",
  },
  ["cellprops"] = {
    "texmf-dist/doc/latex/cellprops/LICENSE",
    "texmf-dist/doc/latex/cellprops/README.md",
    "texmf-dist/doc/latex/cellprops/cellprops.pdf",
  },
  ["qsymbols"] = {
    "texmf-dist/doc/latex/qsymbols/CATALOG",
    "texmf-dist/doc/latex/qsymbols/COPYING",
    "texmf-dist/doc/latex/qsymbols/MANIFEST",
    "texmf-dist/doc/latex/qsymbols/README",
    "texmf-dist/doc/latex/qsymbols/qsymbols.pdf",
  },
  ["latexdiff"] = {
    "texmf-dist/doc/man/man1/latexdiff-vc.man1.pdf",
    "texmf-dist/doc/man/man1/latexdiff.man1.pdf",
    "texmf-dist/doc/man/man1/latexrevise.man1.pdf",
    "texmf-dist/doc/support/latexdiff/COPYING",
    "texmf-dist/doc/support/latexdiff/Makefile",
    "texmf-dist/doc/support/latexdiff/README",
    "texmf-dist/doc/support/latexdiff/contrib/README.latexchanges",
    "texmf-dist/doc/support/latexdiff/contrib/latexdiff-wrap",
    "texmf-dist/doc/support/latexdiff/doc/latexdiff-man.pdf",
    "texmf-dist/doc/support/latexdiff/latexdiff",
    "texmf-dist/doc/support/latexdiff/latexdiff-fast",
  },
  ["texvc"] = {
    "texmf-dist/doc/latex/texvc/LICENSE",
    "texmf-dist/doc/latex/texvc/README.md",
    "texmf-dist/doc/latex/texvc/texvc.pdf",
  },
  ["gtrlib-largetrees"] = {
    "texmf-dist/doc/latex/gtrlib-largetrees/README.md",
    "texmf-dist/doc/latex/gtrlib-largetrees/gtrlib.largetrees.pdf",
  },
  ["gloss-occitan"] = {
    "texmf-dist/doc/latex/gloss-occitan/README",
    "texmf-dist/doc/latex/gloss-occitan/gloss-occitan.pdf",
  },
  ["textpath"] = {
    "texmf-dist/doc/metapost/textpath/CHANGES",
    "texmf-dist/doc/metapost/textpath/README",
    "texmf-dist/doc/metapost/textpath/textpath.pdf",
  },
  ["textopo"] = {
    "texmf-dist/doc/latex/textopo/textopo.pdf",
    "texmf-dist/doc/latex/textopo/textopo.txt",
  },
  ["logbox"] = {
    "texmf-dist/doc/latex/logbox/README",
    "texmf-dist/doc/latex/logbox/logbox.pdf",
  },
  ["istgame"] = {
    "texmf-dist/doc/latex/istgame/README.txt",
    "texmf-dist/doc/latex/istgame/istgame-doc.pdf",
  },
  ["npp-for-context"] = {
    "texmf-dist/doc/context/third/npp-for-context/LICENSE",
    "texmf-dist/doc/context/third/npp-for-context/README.md",
    "texmf-dist/doc/context/third/npp-for-context/doc/npp-context-manual.pdf",
  },
  ["pst-coil"] = {
    "texmf-dist/doc/generic/pst-coil/Changes",
    "texmf-dist/doc/generic/pst-coil/README",
    "texmf-dist/doc/generic/pst-coil/pst-coil-doc.pdf",
  },
  ["textglos"] = {
    "texmf-dist/doc/latex/textglos/README",
    "texmf-dist/doc/latex/textglos/README.txt",
    "texmf-dist/doc/latex/textglos/textglos.pdf",
  },
  ["textcase"] = {
    "texmf-dist/doc/latex/textcase/README",
    "texmf-dist/doc/latex/textcase/textcase.pdf",
  },
  ["makecirc"] = {
    "texmf-dist/doc/metapost/makecirc/MakeCirc-en.pdf",
    "texmf-dist/doc/metapost/makecirc/MakeCirc.pdf",
    "texmf-dist/doc/metapost/makecirc/README",
    "texmf-dist/doc/metapost/makecirc/ejemplos.pdf",
  },
  ["forest"] = {
    "texmf-dist/doc/latex/forest/LICENCE",
    "texmf-dist/doc/latex/forest/README",
    "texmf-dist/doc/latex/forest/forest-doc.pdf",
    "texmf-dist/doc/latex/forest/forest.pdf",
  },
  ["pst-arrow"] = {
    "texmf-dist/doc/generic/pst-arrow/Changes",
    "texmf-dist/doc/generic/pst-arrow/README",
    "texmf-dist/doc/generic/pst-arrow/pst-arrow-doc.pdf",
  },
  ["modulus"] = {
    "texmf-dist/doc/generic/modulus/CHANGES",
    "texmf-dist/doc/generic/modulus/README",
    "texmf-dist/doc/generic/modulus/lppl.txt",
    "texmf-dist/doc/generic/modulus/modulus.pdf",
  },
  ["phfqit"] = {
    "texmf-dist/doc/latex/phfqit/Makefile",
    "texmf-dist/doc/latex/phfqit/README.md",
    "texmf-dist/doc/latex/phfqit/phfqit.pdf",
  },
  ["vhistory"] = {
    "texmf-dist/doc/latex/vhistory/CHANGES",
    "texmf-dist/doc/latex/vhistory/README",
    "texmf-dist/doc/latex/vhistory/README.doc",
    "texmf-dist/doc/latex/vhistory/vh_set_example.pdf",
    "texmf-dist/doc/latex/vhistory/vh_sets_de.pdf",
    "texmf-dist/doc/latex/vhistory/vh_sets_en.pdf",
  },
  ["easy-todo"] = {
    "texmf-dist/doc/latex/easy-todo/LICENSE",
    "texmf-dist/doc/latex/easy-todo/README",
    "texmf-dist/doc/latex/easy-todo/easy-todo.pdf",
  },
  ["longnamefilelist"] = {
    "texmf-dist/doc/latex/longnamefilelist/README",
    "texmf-dist/doc/latex/longnamefilelist/SrcFILEs.txt",
    "texmf-dist/doc/latex/longnamefilelist/longnamefilelist.pdf",
  },
  ["gfsdidot"] = {
    "texmf-dist/doc/fonts/gfsdidot/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gfsdidot/OFL.txt",
    "texmf-dist/doc/fonts/gfsdidot/README",
    "texmf-dist/doc/fonts/gfsdidot/README.TEXLIVE",
    "texmf-dist/doc/fonts/gfsdidot/Readme.txt",
  },
  ["texproposal"] = {
    "texmf-dist/doc/latex/texproposal/README.md",
    "texmf-dist/doc/latex/texproposal/figures/CQU-Example.pdf",
    "texmf-dist/doc/latex/texproposal/figures/CQUThesis-poster.pdf",
    "texmf-dist/doc/latex/texproposal/figures/TeX-Word-Diff.pdf",
    "texmf-dist/doc/latex/texproposal/figures/TeX-Word-Vote.pdf",
    "texmf-dist/doc/latex/texproposal/figures/pubfig0.pdf",
    "texmf-dist/doc/latex/texproposal/texproposal.pdf",
  },
  ["accfonts"] = {
    "texmf-dist/doc/fonts/accfonts/CHANGES",
    "texmf-dist/doc/fonts/accfonts/COPYING",
    "texmf-dist/doc/fonts/accfonts/README",
  },
  ["incgraph"] = {
    "texmf-dist/doc/latex/incgraph/CHANGES",
    "texmf-dist/doc/latex/incgraph/README",
    "texmf-dist/doc/latex/incgraph/incgraph-example-a.pdf",
    "texmf-dist/doc/latex/incgraph/incgraph-example-b.pdf",
    "texmf-dist/doc/latex/incgraph/incgraph-example-c.pdf",
    "texmf-dist/doc/latex/incgraph/incgraph.pdf",
  },
  ["rosario"] = {
    "texmf-dist/doc/fonts/rosario/OFL.txt",
    "texmf-dist/doc/fonts/rosario/README.md",
    "texmf-dist/doc/fonts/rosario/Rosario.pdf",
  },
  ["forloop"] = {
    "texmf-dist/doc/latex/forloop/README",
    "texmf-dist/doc/latex/forloop/forloop.pdf",
  },
  ["texlive-cz"] = {
    "texmf-dist/doc/texlive/texlive-cz/Makefile",
    "texmf-dist/doc/texlive/texlive-cz/texlive-cz.html",
    "texmf-dist/doc/texlive/texlive-cz/texlive-cz.pdf",
  },
  ["probsoln"] = {
    "texmf-dist/doc/latex/probsoln/CHANGES",
    "texmf-dist/doc/latex/probsoln/INSTALL",
    "texmf-dist/doc/latex/probsoln/README",
    "texmf-dist/doc/latex/probsoln/probsoln.pdf",
    "texmf-dist/doc/latex/probsoln/samples/sample-exclude.pdf",
    "texmf-dist/doc/latex/probsoln/samples/sample.pdf",
    "texmf-dist/doc/latex/probsoln/samples/sample2.pdf",
    "texmf-dist/doc/latex/probsoln/samples/sample3.pdf",
    "texmf-dist/doc/latex/probsoln/samples/sample4.pdf",
    "texmf-dist/doc/latex/probsoln/samples/sample5.pdf",
    "texmf-dist/doc/latex/probsoln/samples/sample6.pdf",
    "texmf-dist/doc/latex/probsoln/samples/sample7.pdf",
    "texmf-dist/doc/latex/probsoln/samples/sample8.pdf",
    "texmf-dist/doc/latex/probsoln/samples/sample9.pdf",
  },
  ["cjk-gs-integrate"] = {
    "texmf-dist/doc/fonts/cjk-gs-integrate/ChangeLog",
    "texmf-dist/doc/fonts/cjk-gs-integrate/Makefile",
    "texmf-dist/doc/fonts/cjk-gs-integrate/README.md",
  },
  ["savetrees"] = {
    "texmf-dist/doc/latex/savetrees/README",
    "texmf-dist/doc/latex/savetrees/savetrees.pdf",
    "texmf-dist/doc/latex/savetrees/st-sample2e.pdf",
  },
  ["datetime2-serbian"] = {
    "texmf-dist/doc/latex/datetime2-serbian/README",
    "texmf-dist/doc/latex/datetime2-serbian/datetime2-serbian.pdf",
  },
  ["pst-platon"] = {
    "texmf-dist/doc/generic/pst-platon/Changes",
    "texmf-dist/doc/generic/pst-platon/README",
    "texmf-dist/doc/generic/pst-platon/pst-platon-doc.pdf",
  },
  ["forms16be"] = {
    "texmf-dist/doc/latex/forms16be/README.md",
    "texmf-dist/doc/latex/forms16be/doc/forms16be-man.pdf",
    "texmf-dist/doc/latex/forms16be/examples/forms16b3-ef.pdf",
  },
  ["texments"] = {
    "texmf-dist/doc/latex/texments/README",
    "texmf-dist/doc/latex/texments/texments.pdf",
  },
  ["recipecard"] = {
    "texmf-dist/doc/latex/recipecard/README",
    "texmf-dist/doc/latex/recipecard/recipecard.pdf",
    "texmf-dist/doc/latex/recipecard/test2.pdf",
  },
  ["exercisebank"] = {
    "texmf-dist/doc/latex/exercisebank/README.txt",
    "texmf-dist/doc/latex/exercisebank/exercisebank-doc.pdf",
  },
  ["babel-hungarian"] = {
    "texmf-dist/doc/generic/babel-hungarian/README",
  },
  ["dashrule"] = {
    "texmf-dist/doc/latex/dashrule/README",
    "texmf-dist/doc/latex/dashrule/dashrule.pdf",
  },
  ["chemexec"] = {
    "texmf-dist/doc/latex/chemexec/README",
    "texmf-dist/doc/latex/chemexec/chemexec_de.pdf",
    "texmf-dist/doc/latex/chemexec/chemexec_en.pdf",
  },
  ["archaic"] = {
    "texmf-dist/doc/fonts/archaic/README.PRW",
    "texmf-dist/doc/fonts/archaic/aramaic-README",
    "texmf-dist/doc/fonts/archaic/aramaic.pdf",
    "texmf-dist/doc/fonts/archaic/asamples.pdf",
    "texmf-dist/doc/fonts/archaic/cypriot-README",
    "texmf-dist/doc/fonts/archaic/cypriot.pdf",
    "texmf-dist/doc/fonts/archaic/etruscan-README",
    "texmf-dist/doc/fonts/archaic/etruscan.pdf",
    "texmf-dist/doc/fonts/archaic/greek4cbc-README",
    "texmf-dist/doc/fonts/archaic/greek4cbc-trygivbc.pdf",
    "texmf-dist/doc/fonts/archaic/greek4cbc.pdf",
    "texmf-dist/doc/fonts/archaic/greek6cbc-README",
    "texmf-dist/doc/fonts/archaic/greek6cbc-trygvibc.pdf",
    "texmf-dist/doc/fonts/archaic/greek6cbc.pdf",
    "texmf-dist/doc/fonts/archaic/hieroglf-README",
    "texmf-dist/doc/fonts/archaic/hieroglf-trypmhg.pdf",
    "texmf-dist/doc/fonts/archaic/hieroglf.pdf",
    "texmf-dist/doc/fonts/archaic/linearb-README",
    "texmf-dist/doc/fonts/archaic/linearb.pdf",
    "texmf-dist/doc/fonts/archaic/nabatean-README",
    "texmf-dist/doc/fonts/archaic/nabatean.pdf",
    "texmf-dist/doc/fonts/archaic/oands-README",
    "texmf-dist/doc/fonts/archaic/oands.pdf",
    "texmf-dist/doc/fonts/archaic/oldprsn-README",
    "texmf-dist/doc/fonts/archaic/oldprsn.pdf",
    "texmf-dist/doc/fonts/archaic/phoenician-README",
    "texmf-dist/doc/fonts/archaic/phoenician-tryphnc.pdf",
    "texmf-dist/doc/fonts/archaic/phoenician.pdf",
    "texmf-dist/doc/fonts/archaic/protosem-README",
    "texmf-dist/doc/fonts/archaic/protosem.pdf",
    "texmf-dist/doc/fonts/archaic/runic-README",
    "texmf-dist/doc/fonts/archaic/runic.pdf",
    "texmf-dist/doc/fonts/archaic/sarabian-README",
    "texmf-dist/doc/fonts/archaic/sarabian.pdf",
    "texmf-dist/doc/fonts/archaic/tryaramaic.pdf",
    "texmf-dist/doc/fonts/archaic/trycypriot.pdf",
    "texmf-dist/doc/fonts/archaic/tryetruscan.pdf",
    "texmf-dist/doc/fonts/archaic/trylinearb.pdf",
    "texmf-dist/doc/fonts/archaic/trynabatean.pdf",
    "texmf-dist/doc/fonts/archaic/tryoands.pdf",
    "texmf-dist/doc/fonts/archaic/tryoldprsn.pdf",
    "texmf-dist/doc/fonts/archaic/tryprotosem.pdf",
    "texmf-dist/doc/fonts/archaic/tryrunic.pdf",
    "texmf-dist/doc/fonts/archaic/trysarabian.pdf",
    "texmf-dist/doc/fonts/archaic/tryugarite.pdf",
    "texmf-dist/doc/fonts/archaic/ugarite-README",
    "texmf-dist/doc/fonts/archaic/ugarite.pdf",
    "texmf-dist/doc/fonts/archaic/viking-README",
  },
  ["texloganalyser"] = {
    "texmf-dist/doc/support/texloganalyser/README",
  },
  ["svrsymbols"] = {
    "texmf-dist/doc/fonts/svrsymbols/README",
    "texmf-dist/doc/fonts/svrsymbols/svrsymbols.pdf",
  },
  ["eukdate"] = {
    "texmf-dist/doc/latex/eukdate/README",
    "texmf-dist/doc/latex/eukdate/eukdate.pdf",
  },
  ["elpres"] = {
    "texmf-dist/doc/latex/elpres/README",
    "texmf-dist/doc/latex/elpres/elpres-manual.pdf",
  },
  ["gitlog"] = {
    "texmf-dist/doc/latex/gitlog/README.md",
    "texmf-dist/doc/latex/gitlog/gitlog.pdf",
  },
  ["sanskrit"] = {
    "texmf-dist/doc/latex/sanskrit/README.md",
    "texmf-dist/doc/latex/sanskrit/README.pdf",
    "texmf-dist/doc/latex/sanskrit/sktdoc.pdf",
  },
  ["lua-check-hyphen"] = {
    "texmf-dist/doc/lualatex/lua-check-hyphen/doc/README.md",
    "texmf-dist/doc/lualatex/lua-check-hyphen/doc/luacheckhyphenmanual.pdf",
    "texmf-dist/doc/lualatex/lua-check-hyphen/doc/mit-license.txt",
    "texmf-dist/doc/lualatex/lua-check-hyphen/doc/sample.pdf",
  },
  ["latexindent"] = {
    "texmf-dist/doc/support/latexindent/README",
    "texmf-dist/doc/support/latexindent/latexindent.pdf",
  },
  ["archaeologie"] = {
    "texmf-dist/doc/latex/archaeologie/README.md",
    "texmf-dist/doc/latex/archaeologie/archaeologie-ger.pdf",
    "texmf-dist/doc/latex/archaeologie/archaeologie.pdf",
  },
  ["tablefootnote"] = {
    "texmf-dist/doc/latex/tablefootnote/README",
    "texmf-dist/doc/latex/tablefootnote/tablefootnote-example.pdf",
    "texmf-dist/doc/latex/tablefootnote/tablefootnote.pdf",
  },
  ["fn2end"] = {
    "texmf-dist/doc/latex/fn2end/fn2end.pdf",
  },
  ["ifoddpage"] = {
    "texmf-dist/doc/latex/ifoddpage/README",
    "texmf-dist/doc/latex/ifoddpage/ifoddpage.pdf",
  },
  ["steinmetz"] = {
    "texmf-dist/doc/latex/steinmetz/README",
    "texmf-dist/doc/latex/steinmetz/steinmetz.pdf",
  },
  ["maker"] = {
    "texmf-dist/doc/latex/maker/README",
    "texmf-dist/doc/latex/maker/README.TEXLIVE",
  },
  ["cool"] = {
    "texmf-dist/doc/latex/cool/Content_LaTeX_Package_Demo.pdf",
    "texmf-dist/doc/latex/cool/README",
    "texmf-dist/doc/latex/cool/cool.pdf",
  },
  ["vgrid"] = {
    "texmf-dist/doc/latex/vgrid/vgrid.pdf",
  },
  ["dotseqn"] = {
    "texmf-dist/doc/latex/dotseqn/dotseqn.pdf",
    "texmf-dist/doc/latex/dotseqn/readme",
  },
  ["midnight"] = {
    "texmf-dist/doc/generic/midnight/README",
    "texmf-dist/doc/generic/midnight/midnight.pdf",
  },
  ["calctab"] = {
    "texmf-dist/doc/latex/calctab/README",
    "texmf-dist/doc/latex/calctab/calctab_manual.pdf",
  },
  ["captdef"] = {
    "texmf-dist/doc/latex/captdef/captdef.pdf",
  },
  ["ctablestack"] = {
    "texmf-dist/doc/luatex/ctablestack/README.md",
    "texmf-dist/doc/luatex/ctablestack/ctablestack.pdf",
  },
  ["tikz-feynman"] = {
    "texmf-dist/doc/latex/tikz-feynman/LICENSE",
    "texmf-dist/doc/latex/tikz-feynman/README.md",
    "texmf-dist/doc/latex/tikz-feynman/tikz-feynman.pdf",
  },
  ["biblatex-apa"] = {
    "texmf-dist/doc/latex/biblatex-apa/README",
    "texmf-dist/doc/latex/biblatex-apa/biblatex-apa-test.pdf",
    "texmf-dist/doc/latex/biblatex-apa/biblatex-apa.pdf",
  },
  ["versonotes"] = {
    "texmf-dist/doc/latex/versonotes/LICENCE.txt",
    "texmf-dist/doc/latex/versonotes/README",
    "texmf-dist/doc/latex/versonotes/sample.pdf",
    "texmf-dist/doc/latex/versonotes/versonotes.pdf",
  },
  ["texlive-zh-cn"] = {
    "texmf-dist/doc/texlive/texlive-zh-cn/Makefile",
    "texmf-dist/doc/texlive/texlive-zh-cn/texlive-zh-cn.pdf",
  },
  ["ekaia"] = {
    "texmf-dist/doc/latex/ekaia/README",
    "texmf-dist/doc/latex/ekaia/ekaia.pdf",
    "texmf-dist/doc/latex/ekaia/ekaia_EUS.pdf",
    "texmf-dist/doc/latex/ekaia/ekaia_example.pdf",
  },
  ["texlive-sr"] = {
    "texmf-dist/doc/texlive/texlive-sr/Makefile",
    "texmf-dist/doc/texlive/texlive-sr/README-SR.txt",
    "texmf-dist/doc/texlive/texlive-sr/texlive-sr.html",
    "texmf-dist/doc/texlive/texlive-sr/texlive-sr.pdf",
  },
  ["memdesign"] = {
    "texmf-dist/doc/fonts/memdesign/README",
    "texmf-dist/doc/fonts/memdesign/memdesign.pdf",
  },
  ["endheads"] = {
    "texmf-dist/doc/latex/endheads/README.md",
    "texmf-dist/doc/latex/endheads/endheads.pdf",
  },
  ["pstricks-add"] = {
    "texmf-dist/doc/generic/pstricks-add/Changes",
    "texmf-dist/doc/generic/pstricks-add/README",
    "texmf-dist/doc/generic/pstricks-add/data/olympic.pdf",
    "texmf-dist/doc/generic/pstricks-add/pstricks-add-doc.pdf",
  },
  ["easy"] = {
    "texmf-dist/doc/latex/easy/README",
    "texmf-dist/doc/latex/easy/README.TEXLIVE",
  },
  ["avremu"] = {
    "texmf-dist/doc/latex/avremu/README",
    "texmf-dist/doc/latex/avremu/avremu.pdf",
  },
  ["layouts"] = {
    "texmf-dist/doc/latex/layouts/README",
    "texmf-dist/doc/latex/layouts/layman.pdf",
    "texmf-dist/doc/latex/layouts/layouts.pdf",
  },
  ["texdoctk"] = {
    "texmf-dist/doc/man/man1/texdoctk.man1.pdf",
  },
  ["stix2-type1"] = {
    "texmf-dist/doc/latex/stix2/stix2.pdf",
  },
  ["auto-pst-pdf-lua"] = {
    "texmf-dist/doc/latex/auto-pst-pdf-lua/Changes",
    "texmf-dist/doc/latex/auto-pst-pdf-lua/README",
    "texmf-dist/doc/latex/auto-pst-pdf-lua/auto-pst-pdf-lua-doc.pdf",
  },
  ["texlive-scripts"] = {
    "texmf-dist/doc/man/man1/install-tl.man1.pdf",
  },
  ["facture"] = {
    "texmf-dist/doc/xelatex/facture/README",
    "texmf-dist/doc/xelatex/facture/exemple.pdf",
    "texmf-dist/doc/xelatex/facture/exemplesansTVA.pdf",
    "texmf-dist/doc/xelatex/facture/exemplesansremise.pdf",
    "texmf-dist/doc/xelatex/facture/facture.pdf",
    "texmf-dist/doc/xelatex/facture/makefile",
  },
  ["texlive-ru"] = {
    "texmf-dist/doc/texlive/texlive-ru/Makefile",
    "texmf-dist/doc/texlive/texlive-ru/texlive-ru.html",
    "texmf-dist/doc/texlive/texlive-ru/texlive-ru.pdf",
  },
  ["berenisadf"] = {
    "texmf-dist/doc/fonts/berenisadf/COPYING",
    "texmf-dist/doc/fonts/berenisadf/NOTICE.txt",
    "texmf-dist/doc/fonts/berenisadf/README.doc",
    "texmf-dist/doc/fonts/berenisadf/berenisadf.pdf",
    "texmf-dist/doc/fonts/berenisadf/manifest.txt",
  },
  ["docbytex"] = {
    "texmf-dist/doc/generic/docbytex/README",
    "texmf-dist/doc/generic/docbytex/annonce",
    "texmf-dist/doc/generic/docbytex/docby-e.pdf",
    "texmf-dist/doc/generic/docbytex/docby.pdf",
    "texmf-dist/doc/generic/docbytex/lup.pdf",
  },
  ["draftwatermark"] = {
    "texmf-dist/doc/latex/draftwatermark/README",
    "texmf-dist/doc/latex/draftwatermark/draftwatermark.pdf",
  },
  ["statex"] = {
    "texmf-dist/doc/latex/statex/statex-example.pdf",
  },
  ["texlive-fr"] = {
    "texmf-dist/doc/texlive/texlive-fr/Makefile",
    "texmf-dist/doc/texlive/texlive-fr/notes",
    "texmf-dist/doc/texlive/texlive-fr/texlive-fr.html",
    "texmf-dist/doc/texlive/texlive-fr/texlive-fr.pdf",
  },
  ["caladea"] = {
    "texmf-dist/doc/fonts/caladea/Caladea-Regular.pdf",
    "texmf-dist/doc/fonts/caladea/README",
    "texmf-dist/doc/fonts/caladea/samples.pdf",
  },
  ["texlive-docindex"] = {
    "doc.html",
  },
  ["ltximg"] = {
    "texmf-dist/doc/support/ltximg/README.md",
    "texmf-dist/doc/support/ltximg/ltximg-doc.pdf",
  },
  ["xstring"] = {
    "texmf-dist/doc/generic/xstring/README",
    "texmf-dist/doc/generic/xstring/xstring_doc_en.pdf",
    "texmf-dist/doc/generic/xstring/xstring_doc_fr.pdf",
  },
  ["lshort-vietnamese"] = {
    "texmf-dist/doc/latex/lshort-vietnamese/README",
    "texmf-dist/doc/latex/lshort-vietnamese/lshort-vi.pdf",
    "texmf-dist/doc/latex/lshort-vietnamese/src/LocalVariables",
    "texmf-dist/doc/latex/lshort-vietnamese/src/Makefile",
    "texmf-dist/doc/latex/lshort-vietnamese/src/README.txt",
  },
  ["datetime2-scottish"] = {
    "texmf-dist/doc/latex/datetime2-scottish/README",
    "texmf-dist/doc/latex/datetime2-scottish/datetime2-scottish.pdf",
  },
  ["tocdata"] = {
    "texmf-dist/doc/latex/tocdata/README",
    "texmf-dist/doc/latex/tocdata/images/symbol_warning.pdf",
    "texmf-dist/doc/latex/tocdata/tocdata.pdf",
  },
  ["pstring"] = {
    "texmf-dist/doc/latex/pstring/README.md",
    "texmf-dist/doc/latex/pstring/pstring-doc.pdf",
  },
  ["roundrect"] = {
    "texmf-dist/doc/metapost/roundrect/CHANGES",
    "texmf-dist/doc/metapost/roundrect/README",
    "texmf-dist/doc/metapost/roundrect/lppl.txt",
    "texmf-dist/doc/metapost/roundrect/roundrect.pdf",
  },
  ["mkjobtexmf"] = {
    "texmf-dist/doc/generic/mkjobtexmf/README",
    "texmf-dist/doc/generic/mkjobtexmf/mkjobtexmf.html",
    "texmf-dist/doc/generic/mkjobtexmf/mkjobtexmf.pdf",
    "texmf-dist/doc/generic/mkjobtexmf/mkjobtexmf.txt",
    "texmf-dist/doc/man/man1/mkjobtexmf.man1.pdf",
  },
  ["printlen"] = {
    "texmf-dist/doc/latex/printlen/printlen-doc.pdf",
  },
  ["luatex"] = {
    "texmf-dist/doc/luatex/base/luatex.pdf",
    "texmf-dist/doc/man/man1/luatex.man1.pdf",
    "texmf-dist/doc/man/man1/texlua.man1.pdf",
    "texmf-dist/doc/man/man1/texluac.man1.pdf",
  },
  ["shuffle"] = {
    "texmf-dist/doc/latex/shuffle/README",
    "texmf-dist/doc/latex/shuffle/shuffle.pdf",
  },
  ["mftinc"] = {
    "texmf-dist/doc/latex/mftinc/README",
    "texmf-dist/doc/latex/mftinc/mftinc.pdf",
  },
  ["gobble"] = {
    "texmf-dist/doc/generic/gobble/README",
    "texmf-dist/doc/generic/gobble/gobble.pdf",
  },
  ["texosquery"] = {
    "texmf-dist/doc/support/texosquery/CHANGES",
    "texmf-dist/doc/support/texosquery/README.md",
    "texmf-dist/doc/support/texosquery/texosquery.pdf",
  },
  ["fcavtex"] = {
    "texmf-dist/doc/latex/fcavtex/README",
    "texmf-dist/doc/latex/fcavtex/examples/basico-principal.pdf",
    "texmf-dist/doc/latex/fcavtex/manual/fcavtex.pdf",
  },
  ["nomencl"] = {
    "texmf-dist/doc/latex/nomencl/README",
    "texmf-dist/doc/latex/nomencl/nomencl.pdf",
  },
  ["sty2dtx"] = {
    "texmf-dist/doc/man/man1/sty2dtx.man1.pdf",
    "texmf-dist/doc/support/sty2dtx/README",
    "texmf-dist/doc/support/sty2dtx/sty2dtx.pdf",
  },
  ["aguplus"] = {
    "texmf-dist/doc/latex/aguplus/README",
    "texmf-dist/doc/latex/aguplus/README.aguplus",
    "texmf-dist/doc/latex/aguplus/aguplus.pdf",
  },
  ["pdfpages"] = {
    "texmf-dist/doc/latex/pdfpages/dummy-l.pdf",
    "texmf-dist/doc/latex/pdfpages/dummy.pdf",
    "texmf-dist/doc/latex/pdfpages/pdfpages.pdf",
  },
  ["fixltxhyph"] = {
    "texmf-dist/doc/latex/fixltxhyph/README",
    "texmf-dist/doc/latex/fixltxhyph/fixltxhyph.pdf",
  },
  ["biblatex-iso690"] = {
    "texmf-dist/doc/latex/biblatex-iso690/README.md",
    "texmf-dist/doc/latex/biblatex-iso690/biblatex-iso690.pdf",
  },
  ["pkgloader"] = {
    "texmf-dist/doc/latex/pkgloader/README",
    "texmf-dist/doc/latex/pkgloader/pkgloader.pdf",
  },
  ["texfot"] = {
    "texmf-dist/doc/man/man1/texfot.man1.pdf",
    "texmf-dist/doc/support/texfot/README",
  },
  ["fmtcount"] = {
    "texmf-dist/doc/latex/fmtcount/CHANGES",
    "texmf-dist/doc/latex/fmtcount/README",
    "texmf-dist/doc/latex/fmtcount/fmtcount.pdf",
  },
  ["texdraw"] = {
    "texmf-dist/doc/support/texdraw/ChangeLog",
    "texmf-dist/doc/support/texdraw/README",
    "texmf-dist/doc/support/texdraw/texdraw.ps",
  },
  ["catoptions"] = {
    "texmf-dist/doc/latex/catoptions/README",
  },
  ["nicematrix"] = {
    "texmf-dist/doc/latex/nicematrix/README.md",
    "texmf-dist/doc/latex/nicematrix/nicematrix.pdf",
  },
  ["cbcoptic"] = {
    "texmf-dist/doc/latex/cbcoptic/README",
    "texmf-dist/doc/latex/cbcoptic/coptfont.pdf",
    "texmf-dist/doc/latex/cbcoptic/testcopOK.pdf",
  },
  ["b1encoding"] = {
    "texmf-dist/doc/latex/b1encoding/README",
    "texmf-dist/doc/latex/b1encoding/b1encoding.pdf",
  },
  ["schemata"] = {
    "texmf-dist/doc/generic/schemata/Makefile",
    "texmf-dist/doc/generic/schemata/README",
    "texmf-dist/doc/generic/schemata/README.txt",
    "texmf-dist/doc/generic/schemata/schemata.pdf",
  },
  ["diffcoeff"] = {
    "texmf-dist/doc/latex/diffcoeff/README.txt",
    "texmf-dist/doc/latex/diffcoeff/diffcoeff.pdf",
    "texmf-dist/doc/latex/diffcoeff/diffcoeffx.pdf",
  },
  ["ncctools"] = {
    "texmf-dist/doc/latex/ncctools/README",
    "texmf-dist/doc/latex/ncctools/README.source",
    "texmf-dist/doc/latex/ncctools/afterpackage.pdf",
    "texmf-dist/doc/latex/ncctools/changes.txt",
    "texmf-dist/doc/latex/ncctools/dcounter.pdf",
    "texmf-dist/doc/latex/ncctools/desclist.pdf",
    "texmf-dist/doc/latex/ncctools/extdash.pdf",
    "texmf-dist/doc/latex/ncctools/manifest.txt",
    "texmf-dist/doc/latex/ncctools/manyfoot.pdf",
    "texmf-dist/doc/latex/ncctools/mboxfill.pdf",
    "texmf-dist/doc/latex/ncctools/nccbbb.pdf",
    "texmf-dist/doc/latex/ncctools/nccboxes.pdf",
    "texmf-dist/doc/latex/ncctools/ncccomma.pdf",
    "texmf-dist/doc/latex/ncctools/ncccropbox.pdf",
    "texmf-dist/doc/latex/ncctools/ncccropmark.pdf",
    "texmf-dist/doc/latex/ncctools/nccfancyhdr.pdf",
    "texmf-dist/doc/latex/ncctools/nccfloats.pdf",
    "texmf-dist/doc/latex/ncctools/nccfoots.pdf",
    "texmf-dist/doc/latex/ncctools/nccmath.pdf",
    "texmf-dist/doc/latex/ncctools/nccparskip.pdf",
    "texmf-dist/doc/latex/ncctools/nccpic.pdf",
    "texmf-dist/doc/latex/ncctools/nccrules.pdf",
    "texmf-dist/doc/latex/ncctools/nccsect.pdf",
    "texmf-dist/doc/latex/ncctools/nccstretch.pdf",
    "texmf-dist/doc/latex/ncctools/nccthm.pdf",
    "texmf-dist/doc/latex/ncctools/ncctools.pdf",
    "texmf-dist/doc/latex/ncctools/textarea.pdf",
    "texmf-dist/doc/latex/ncctools/tocenter.pdf",
    "texmf-dist/doc/latex/ncctools/topsection.pdf",
    "texmf-dist/doc/latex/ncctools/watermark.pdf",
  },
  ["qstest"] = {
    "texmf-dist/doc/latex/qstest/README",
    "texmf-dist/doc/latex/qstest/makematch.pdf",
    "texmf-dist/doc/latex/qstest/qstest.pdf",
  },
  ["fink"] = {
    "texmf-dist/doc/latex/fink/NEWS",
    "texmf-dist/doc/latex/fink/README",
    "texmf-dist/doc/latex/fink/THANKS",
    "texmf-dist/doc/latex/fink/fink.pdf",
  },
  ["bibleref-mouth"] = {
    "texmf-dist/doc/latex/bibleref-mouth/README",
    "texmf-dist/doc/latex/bibleref-mouth/bibleref-mouth.pdf",
  },
  ["l2tabu-spanish"] = {
    "texmf-dist/doc/latex/l2tabu-spanish/README",
    "texmf-dist/doc/latex/l2tabu-spanish/l2tabues.pdf",
  },
  ["realscripts"] = {
    "texmf-dist/doc/latex/realscripts/README.md",
    "texmf-dist/doc/latex/realscripts/realscripts.pdf",
  },
  ["beameraudience"] = {
    "texmf-dist/doc/latex/beameraudience/README",
  },
  ["xoptarg"] = {
    "texmf-dist/doc/latex/xoptarg/README",
    "texmf-dist/doc/latex/xoptarg/xoptarg.pdf",
  },
  ["carlisle"] = {
    "texmf-dist/doc/latex/carlisle/README.txt",
    "texmf-dist/doc/latex/carlisle/ltxtable.pdf",
  },
  ["romanneg"] = {
    "texmf-dist/doc/latex/romanneg/romanneg.pdf",
  },
  ["sauterfonts"] = {
    "texmf-dist/doc/latex/sauterfonts/README",
  },
  ["pkfix-helper"] = {
    "texmf-dist/doc/man/man1/pkfix-helper.man1.pdf",
    "texmf-dist/doc/support/pkfix-helper/README",
    "texmf-dist/doc/support/pkfix-helper/encoding-samples.pdf",
  },
  ["rtklage"] = {
    "texmf-dist/doc/latex/rtklage/README",
    "texmf-dist/doc/latex/rtklage/rtklage.pdf",
  },
  ["pst-pulley"] = {
    "texmf-dist/doc/generic/pst-pulley/Changes",
    "texmf-dist/doc/generic/pst-pulley/README.md",
    "texmf-dist/doc/generic/pst-pulley/pst-pulley-doc.pdf",
  },
  ["engpron"] = {
    "texmf-dist/doc/latex/engpron/LISEZMOI",
    "texmf-dist/doc/latex/engpron/README",
    "texmf-dist/doc/latex/engpron/engpron-en.pdf",
    "texmf-dist/doc/latex/engpron/engpron-ex-en.pdf",
    "texmf-dist/doc/latex/engpron/engpron-ex-fr.pdf",
    "texmf-dist/doc/latex/engpron/engpron-fr.pdf",
    "texmf-dist/doc/latex/engpron/engpron.pdf",
  },
  ["totcount"] = {
    "texmf-dist/doc/latex/totcount/README",
    "texmf-dist/doc/latex/totcount/totcount.pdf",
  },
  ["context-animation"] = {
    "texmf-dist/doc/context/third/animation/README",
    "texmf-dist/doc/context/third/animation/VERSION",
  },
  ["libertinus"] = {
    "texmf-dist/doc/fonts/libertinus/AUTHORS.txt",
    "texmf-dist/doc/fonts/libertinus/FONTLOG.txt",
    "texmf-dist/doc/fonts/libertinus/OFL.txt",
    "texmf-dist/doc/fonts/libertinus/README.txt",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinus-sample.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinus-testmath.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinuskeyboard-regular-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinusmath-regular-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinusmono-regular-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinussans-bold-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinussans-italic-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinussans-regular-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinusserif-bold-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinusserif-bolditalic-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinusserif-italic-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinusserif-regular-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinusserif-semibold-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinusserif-semibolditalic-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinusserifdisplay-regular-table.pdf",
    "texmf-dist/doc/fonts/libertinus/documentation/libertinusserifinitials-regular-table.pdf",
  },
  ["pst-solarsystem"] = {
    "texmf-dist/doc/generic/pst-solarsystem/Changes",
    "texmf-dist/doc/generic/pst-solarsystem/README",
    "texmf-dist/doc/generic/pst-solarsystem/pst-solarsystem-doc.pdf",
  },
  ["optional"] = {
    "texmf-dist/doc/latex/optional/optional.pdf",
  },
  ["linop"] = {
    "texmf-dist/doc/latex/linop/README.md",
    "texmf-dist/doc/latex/linop/linop.pdf",
  },
  ["hackthefootline"] = {
    "texmf-dist/doc/latex/hackthefootline/README.md",
    "texmf-dist/doc/latex/hackthefootline/doc/hackthefootline-doc.pdf",
    "texmf-dist/doc/latex/hackthefootline/doc/hackthefootline-examples.pdf",
  },
  ["sr-vorl"] = {
    "texmf-dist/doc/latex/sr-vorl/README",
    "texmf-dist/doc/latex/sr-vorl/sr-vorl.pdf",
  },
  ["spanish-mx"] = {
    "texmf-dist/doc/latex/spanish-mx/README",
  },
  ["footbib"] = {
    "texmf-dist/doc/latex/footbib/README",
    "texmf-dist/doc/latex/footbib/footbib.pdf",
  },
  ["kluwer"] = {
    "texmf-dist/doc/latex/kluwer/00readme",
    "texmf-dist/doc/latex/kluwer/sampkluw.dvi",
    "texmf-dist/doc/latex/kluwer/sampopen.dvi",
    "texmf-dist/doc/latex/kluwer/usrman.dvi",
  },
  ["texdirflatten"] = {
    "texmf-dist/doc/man/man1/texdirflatten.man1.pdf",
    "texmf-dist/doc/support/texdirflatten/README",
  },
  ["crossreference"] = {
    "texmf-dist/doc/latex/crossreference/README",
    "texmf-dist/doc/latex/crossreference/crossreference.pdf",
  },
  ["fullblck"] = {
    "texmf-dist/doc/latex/fullblck/README",
    "texmf-dist/doc/latex/fullblck/fullblck.pdf",
  },
  ["ctable"] = {
    "texmf-dist/doc/latex/ctable/01a",
    "texmf-dist/doc/latex/ctable/01b",
    "texmf-dist/doc/latex/ctable/02k",
    "texmf-dist/doc/latex/ctable/02l",
    "texmf-dist/doc/latex/ctable/02m",
    "texmf-dist/doc/latex/ctable/03a",
    "texmf-dist/doc/latex/ctable/03b",
    "texmf-dist/doc/latex/ctable/04a",
    "texmf-dist/doc/latex/ctable/04b",
    "texmf-dist/doc/latex/ctable/05a",
    "texmf-dist/doc/latex/ctable/05b",
    "texmf-dist/doc/latex/ctable/05c",
    "texmf-dist/doc/latex/ctable/06a",
    "texmf-dist/doc/latex/ctable/06b",
    "texmf-dist/doc/latex/ctable/07a",
    "texmf-dist/doc/latex/ctable/07b",
    "texmf-dist/doc/latex/ctable/08a",
    "texmf-dist/doc/latex/ctable/08b",
    "texmf-dist/doc/latex/ctable/09b",
    "texmf-dist/doc/latex/ctable/10a",
    "texmf-dist/doc/latex/ctable/10b",
    "texmf-dist/doc/latex/ctable/12a",
    "texmf-dist/doc/latex/ctable/12b",
    "texmf-dist/doc/latex/ctable/13a",
    "texmf-dist/doc/latex/ctable/13b",
    "texmf-dist/doc/latex/ctable/13c",
    "texmf-dist/doc/latex/ctable/14a",
    "texmf-dist/doc/latex/ctable/README",
    "texmf-dist/doc/latex/ctable/ctable.pdf",
    "texmf-dist/doc/latex/ctable/doit",
    "texmf-dist/doc/latex/ctable/inst",
  },
  ["multibbl"] = {
    "texmf-dist/doc/latex/multibbl/multibbl.pdf",
  },
  ["pst-node"] = {
    "texmf-dist/doc/generic/pst-node/Changes",
    "texmf-dist/doc/generic/pst-node/README",
    "texmf-dist/doc/generic/pst-node/psmatrix-docDE.pdf",
    "texmf-dist/doc/generic/pst-node/pst-node-doc.pdf",
  },
  ["ascelike"] = {
    "texmf-dist/doc/latex/ascelike/README",
    "texmf-dist/doc/latex/ascelike/ascexmpl.pdf",
  },
  ["adrconv"] = {
    "texmf-dist/doc/latex/adrconv/README",
    "texmf-dist/doc/latex/adrconv/adrconv.pdf",
    "texmf-dist/doc/latex/adrconv/adrconv_pages08.pages/Contents/PkgInfo",
    "texmf-dist/doc/latex/adrconv/adrguide.pdf",
  },
  ["babel-piedmontese"] = {
    "texmf-dist/doc/generic/babel-piedmontese/piedmontese.pdf",
  },
  ["texdiff"] = {
    "texmf-dist/doc/man/man1/texdiff.man1.pdf",
    "texmf-dist/doc/support/texdiff/README",
  },
  ["alertmessage"] = {
    "texmf-dist/doc/latex/alertmessage/README.md",
    "texmf-dist/doc/latex/alertmessage/alertmessage.pdf",
  },
  ["dviasm"] = {
    "texmf-dist/doc/latex/dviasm/README",
  },
  ["platex-tools"] = {
    "texmf-dist/doc/latex/platex-tools/LICENSE",
    "texmf-dist/doc/latex/platex-tools/Makefile",
    "texmf-dist/doc/latex/platex-tools/README.md",
    "texmf-dist/doc/latex/platex-tools/plarray.pdf",
    "texmf-dist/doc/latex/platex-tools/plextarray.pdf",
    "texmf-dist/doc/latex/platex-tools/plextdelarray.pdf",
    "texmf-dist/doc/latex/platex-tools/pxatbegshi.pdf",
    "texmf-dist/doc/latex/platex-tools/pxeverysel.pdf",
    "texmf-dist/doc/latex/platex-tools/pxeveryshi.pdf",
    "texmf-dist/doc/latex/platex-tools/pxftnright.pdf",
  },
  ["faktor"] = {
    "texmf-dist/doc/latex/faktor/README",
    "texmf-dist/doc/latex/faktor/faktor.pdf",
  },
  ["texdef"] = {
    "texmf-dist/doc/support/texdef/CHANGELOG",
    "texmf-dist/doc/support/texdef/INSTALL",
    "texmf-dist/doc/support/texdef/README",
    "texmf-dist/doc/support/texdef/texdef.pdf",
  },
  ["texdate"] = {
    "texmf-dist/doc/generic/texdate/CHANGES",
    "texmf-dist/doc/generic/texdate/README",
    "texmf-dist/doc/generic/texdate/lppl.txt",
    "texmf-dist/doc/generic/texdate/texdate.pdf",
  },
  ["babel-occitan"] = {
    "texmf-dist/doc/generic/babel-occitan/README",
    "texmf-dist/doc/generic/babel-occitan/occitan.pdf",
  },
  ["prerex"] = {
    "texmf-dist/doc/latex/prerex/README",
    "texmf-dist/doc/latex/prerex/chart.pdf",
    "texmf-dist/doc/latex/prerex/intro.pdf",
    "texmf-dist/doc/latex/prerex/prerex.pdf",
    "texmf-dist/doc/latex/prerex/prerex.sty.7.pdf",
    "texmf-dist/doc/man/man5/prerex.man5.pdf",
  },
  ["apa6e"] = {
    "texmf-dist/doc/latex/apa6e/README",
    "texmf-dist/doc/latex/apa6e/apa6e.pdf",
  },
  ["pst-diffraction"] = {
    "texmf-dist/doc/generic/pst-diffraction/Changes",
    "texmf-dist/doc/generic/pst-diffraction/README",
    "texmf-dist/doc/generic/pst-diffraction/pst-diffraction-docDE.pdf",
    "texmf-dist/doc/generic/pst-diffraction/pst-diffraction-docE.pdf",
    "texmf-dist/doc/generic/pst-diffraction/pst-diffraction-docFR.pdf",
  },
  ["ptex-fonts"] = {
    "texmf-dist/doc/fonts/ptex-fonts/Changes_makejvf",
    "texmf-dist/doc/fonts/ptex-fonts/LICENSE",
    "texmf-dist/doc/fonts/ptex-fonts/README.md",
    "texmf-dist/doc/fonts/ptex-fonts/README_makejvf",
  },
  ["doctools"] = {
    "texmf-dist/doc/latex/doctools/README",
    "texmf-dist/doc/latex/doctools/doctools.pdf",
  },
  ["scientific-thesis-cover"] = {
    "texmf-dist/doc/latex/scientific-thesis-cover/README.TEXLIVE",
    "texmf-dist/doc/latex/scientific-thesis-cover/README.md",
    "texmf-dist/doc/latex/scientific-thesis-cover/scientific-thesis-cover.pdf",
  },
  ["knuth"] = {
    "texmf-dist/doc/generic/knuth/errata/errata.pdf",
    "texmf-dist/doc/generic/knuth/errata/errorlog.pdf",
    "texmf-dist/doc/generic/knuth/etc/vftovp.pdf",
    "texmf-dist/doc/generic/knuth/etc/vptovf.pdf",
    "texmf-dist/doc/generic/knuth/mf/mf.pdf",
    "texmf-dist/doc/generic/knuth/mf/trapman.pdf",
    "texmf-dist/doc/generic/knuth/mfware/gftodvi.pdf",
    "texmf-dist/doc/generic/knuth/mfware/gftopk.pdf",
    "texmf-dist/doc/generic/knuth/mfware/gftype.pdf",
    "texmf-dist/doc/generic/knuth/mfware/mft.pdf",
    "texmf-dist/doc/generic/knuth/tex/glue.pdf",
    "texmf-dist/doc/generic/knuth/tex/tex.pdf",
    "texmf-dist/doc/generic/knuth/tex/tripman.pdf",
    "texmf-dist/doc/generic/knuth/texware/dvitype.pdf",
    "texmf-dist/doc/generic/knuth/texware/pltotf.pdf",
    "texmf-dist/doc/generic/knuth/texware/pooltype.pdf",
    "texmf-dist/doc/generic/knuth/texware/tftopl.pdf",
    "texmf-dist/doc/generic/knuth/web/tangle.pdf",
    "texmf-dist/doc/generic/knuth/web/weave.pdf",
    "texmf-dist/doc/generic/knuth/web/webman.pdf",
  },
  ["electrum"] = {
    "texmf-dist/doc/fonts/electrum/COPYING",
    "texmf-dist/doc/fonts/electrum/NOTICE.txt",
    "texmf-dist/doc/fonts/electrum/README",
    "texmf-dist/doc/fonts/electrum/electrumadf.pdf",
    "texmf-dist/doc/fonts/electrum/manifest.txt",
  },
  ["ucharclasses"] = {
    "texmf-dist/doc/xelatex/ucharclasses/README",
    "texmf-dist/doc/xelatex/ucharclasses/ucharclasses.pdf",
  },
  ["ocherokee"] = {
    "texmf-dist/doc/omega/ocherokee/cherokee.pdf",
  },
  ["insbox"] = {
    "texmf-dist/doc/generic/insbox/demo.pdf",
  },
  ["circuitikz"] = {
    "texmf-dist/doc/context/third/circuitikz/circuitikz-context.pdf",
    "texmf-dist/doc/generic/circuitikz/README.md",
    "texmf-dist/doc/latex/circuitikz/circuitikzmanual.pdf",
  },
  ["tdsfrmath"] = {
    "texmf-dist/doc/latex/tdsfrmath/LISEZMOI",
    "texmf-dist/doc/latex/tdsfrmath/README",
    "texmf-dist/doc/latex/tdsfrmath/tdsfrmath.pdf",
  },
  ["pst-soroban"] = {
    "texmf-dist/doc/generic/pst-soroban/Changes",
    "texmf-dist/doc/generic/pst-soroban/README",
    "texmf-dist/doc/generic/pst-soroban/pst-soroban-doc.pdf",
  },
  ["texcount"] = {
    "texmf-dist/doc/support/texcount/README.md",
    "texmf-dist/doc/support/texcount/doc/QuickReference.pdf",
    "texmf-dist/doc/support/texcount/doc/TeXcount.pdf",
    "texmf-dist/doc/support/texcount/doc/TechDoc.pdf",
  },
  ["texbytopic"] = {
    "texmf-dist/doc/plain/texbytopic/README",
    "texmf-dist/doc/plain/texbytopic/TeXbyTopic.pdf",
  },
  ["plex-otf"] = {
    "texmf-dist/doc/fonts/plex-otf/Changes",
    "texmf-dist/doc/fonts/plex-otf/README.md",
    "texmf-dist/doc/fonts/plex-otf/plex-otf-doc.pdf",
  },
  ["ecothesis"] = {
    "texmf-dist/doc/latex/ecothesis/README.txt",
    "texmf-dist/doc/latex/ecothesis/capitulos/cap01/introduction.pdf",
    "texmf-dist/doc/latex/ecothesis/capitulos/cap02/cap02.pdf",
    "texmf-dist/doc/latex/ecothesis/capitulos/cap03/cap03.pdf",
    "texmf-dist/doc/latex/ecothesis/ecothesis.pdf",
    "texmf-dist/doc/latex/ecothesis/main.pdf",
  },
  ["ps2pk"] = {
    "texmf-dist/doc/man/man1/mag.man1.pdf",
    "texmf-dist/doc/man/man1/pfb2pfa.man1.pdf",
    "texmf-dist/doc/man/man1/pk2bm.man1.pdf",
    "texmf-dist/doc/man/man1/ps2pk.man1.pdf",
  },
  ["biblatex-dw"] = {
    "texmf-dist/doc/latex/biblatex-dw/CHANGES",
    "texmf-dist/doc/latex/biblatex-dw/LIESMICH",
    "texmf-dist/doc/latex/biblatex-dw/README",
    "texmf-dist/doc/latex/biblatex-dw/biblatex-dw.pdf",
    "texmf-dist/doc/latex/biblatex-dw/de-biblatex-dw.pdf",
    "texmf-dist/doc/latex/biblatex-dw/examples/de-authortitle-dw.pdf",
    "texmf-dist/doc/latex/biblatex-dw/examples/de-footnote-dw.pdf",
    "texmf-dist/doc/latex/biblatex-dw/examples/en-authortitle-dw.pdf",
    "texmf-dist/doc/latex/biblatex-dw/examples/en-footnote-dw.pdf",
  },
  ["dviincl"] = {
    "texmf-dist/doc/metapost/dviincl/README",
    "texmf-dist/doc/metapost/dviincl/README.pl",
  },
  ["context-ruby"] = {
    "texmf-dist/doc/context/third/ruby/README",
    "texmf-dist/doc/context/third/ruby/VERSION",
  },
  ["tex4ht"] = {
    "texmf-dist/doc/generic/tex4ht/bugfixes.html",
    "texmf-dist/doc/generic/tex4ht/index.html",
    "texmf-dist/doc/generic/tex4ht/mn-htf.html",
    "texmf-dist/doc/generic/tex4ht/mn-index.html",
    "texmf-dist/doc/generic/tex4ht/mn-mswin.html",
    "texmf-dist/doc/generic/tex4ht/mn-port.html",
    "texmf-dist/doc/generic/tex4ht/mn-unix.html",
    "texmf-dist/doc/generic/tex4ht/mn.html",
    "texmf-dist/doc/generic/tex4ht/mn10.html",
    "texmf-dist/doc/generic/tex4ht/mn11.html",
    "texmf-dist/doc/generic/tex4ht/mn12.html",
    "texmf-dist/doc/generic/tex4ht/mn13.html",
    "texmf-dist/doc/generic/tex4ht/mn14.html",
    "texmf-dist/doc/generic/tex4ht/mn15.html",
    "texmf-dist/doc/generic/tex4ht/mn16.html",
    "texmf-dist/doc/generic/tex4ht/mn17.html",
    "texmf-dist/doc/generic/tex4ht/mn19.html",
    "texmf-dist/doc/generic/tex4ht/mn2.html",
    "texmf-dist/doc/generic/tex4ht/mn20.html",
    "texmf-dist/doc/generic/tex4ht/mn21.html",
    "texmf-dist/doc/generic/tex4ht/mn23.html",
    "texmf-dist/doc/generic/tex4ht/mn24.html",
    "texmf-dist/doc/generic/tex4ht/mn25.html",
    "texmf-dist/doc/generic/tex4ht/mn27.html",
    "texmf-dist/doc/generic/tex4ht/mn28.html",
    "texmf-dist/doc/generic/tex4ht/mn29.html",
    "texmf-dist/doc/generic/tex4ht/mn3.html",
    "texmf-dist/doc/generic/tex4ht/mn31.html",
    "texmf-dist/doc/generic/tex4ht/mn32.html",
    "texmf-dist/doc/generic/tex4ht/mn33.html",
    "texmf-dist/doc/generic/tex4ht/mn34.html",
    "texmf-dist/doc/generic/tex4ht/mn35.html",
    "texmf-dist/doc/generic/tex4ht/mn36.html",
    "texmf-dist/doc/generic/tex4ht/mn37.html",
    "texmf-dist/doc/generic/tex4ht/mn38.html",
    "texmf-dist/doc/generic/tex4ht/mn39.html",
    "texmf-dist/doc/generic/tex4ht/mn4.html",
    "texmf-dist/doc/generic/tex4ht/mn40.html",
    "texmf-dist/doc/generic/tex4ht/mn41.html",
    "texmf-dist/doc/generic/tex4ht/mn42.html",
    "texmf-dist/doc/generic/tex4ht/mn43.html",
    "texmf-dist/doc/generic/tex4ht/mn44.html",
    "texmf-dist/doc/generic/tex4ht/mn45.html",
    "texmf-dist/doc/generic/tex4ht/mn46.html",
    "texmf-dist/doc/generic/tex4ht/mn47.html",
    "texmf-dist/doc/generic/tex4ht/mn48.html",
    "texmf-dist/doc/generic/tex4ht/mn49.html",
    "texmf-dist/doc/generic/tex4ht/mn5.html",
    "texmf-dist/doc/generic/tex4ht/mn50.html",
    "texmf-dist/doc/generic/tex4ht/mn51.html",
    "texmf-dist/doc/generic/tex4ht/mn52.html",
    "texmf-dist/doc/generic/tex4ht/mn53.html",
    "texmf-dist/doc/generic/tex4ht/mn54.html",
    "texmf-dist/doc/generic/tex4ht/mn55.html",
    "texmf-dist/doc/generic/tex4ht/mn56.html",
    "texmf-dist/doc/generic/tex4ht/mn57.html",
    "texmf-dist/doc/generic/tex4ht/mn58.html",
    "texmf-dist/doc/generic/tex4ht/mn59.html",
    "texmf-dist/doc/generic/tex4ht/mn6.html",
    "texmf-dist/doc/generic/tex4ht/mn60.html",
    "texmf-dist/doc/generic/tex4ht/mn7.html",
    "texmf-dist/doc/generic/tex4ht/mn8.html",
    "texmf-dist/doc/generic/tex4ht/mn9.html",
  },
  ["stdclsdv"] = {
    "texmf-dist/doc/latex/stdclsdv/README",
    "texmf-dist/doc/latex/stdclsdv/stdclsdv.pdf",
  },
  ["chemfig"] = {
    "texmf-dist/doc/generic/chemfig/README",
    "texmf-dist/doc/generic/chemfig/chemfig-en.pdf",
    "texmf-dist/doc/generic/chemfig/chemfig-fr.pdf",
  },
  ["coseoul"] = {
    "texmf-dist/doc/latex/coseoul/README",
    "texmf-dist/doc/latex/coseoul/coseoul.pdf",
    "texmf-dist/doc/latex/coseoul/cosexamp.pdf",
  },
  ["ulqda"] = {
    "texmf-dist/doc/latex/ulqda/README",
    "texmf-dist/doc/latex/ulqda/ulqda.pdf",
  },
  ["outlines"] = {
    "texmf-dist/doc/latex/outlines/README",
    "texmf-dist/doc/latex/outlines/outlines.pdf",
  },
  ["svn-multi"] = {
    "texmf-dist/doc/latex/svn-multi/README",
    "texmf-dist/doc/latex/svn-multi/svn-multi.pdf",
    "texmf-dist/doc/support/svn-multi/svn-multi-pl.pdf",
  },
  ["tcldoc"] = {
    "texmf-dist/doc/latex/tcldoc/README.txt",
    "texmf-dist/doc/latex/tcldoc/examples/README.txt",
    "texmf-dist/doc/latex/tcldoc/tclldoc.pdf",
    "texmf-dist/doc/latex/tcldoc/tools/README.txt",
  },
  ["bibunits"] = {
    "texmf-dist/doc/latex/bibunits/README",
    "texmf-dist/doc/latex/bibunits/bibtexall",
    "texmf-dist/doc/latex/bibunits/bibunits.pdf",
  },
  ["plates"] = {
    "texmf-dist/doc/latex/plates/README",
    "texmf-dist/doc/latex/plates/plates.pdf",
  },
  ["math-into-latex-4"] = {
    "texmf-dist/doc/latex/math-into-latex-4/README",
    "texmf-dist/doc/latex/math-into-latex-4/README.TEXLIVE",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer1.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer10.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer2.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer3.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer4.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer5.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer6.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer6block.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer6mod.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer6mod2.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer7.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer8.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/babybeamer9.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/basem3-1.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/basem3-2.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/basem3-3.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/basem3-4.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/beamerstructure1.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/beamerstructure2.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/cube.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/gratzer",
    "texmf-dist/doc/latex/math-into-latex-4/notation.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/products.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/quickbeamer.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/quickbeamer1.pdf",
    "texmf-dist/doc/latex/math-into-latex-4/quickbeamer2.pdf",
  },
  ["latex-make"] = {
    "texmf-dist/doc/support/latex-make/README",
    "texmf-dist/doc/support/latex-make/figlatex.pdf",
    "texmf-dist/doc/support/latex-make/latex-make.pdf",
    "texmf-dist/doc/support/latex-make/texdepends.pdf",
  },
  ["opteng"] = {
    "texmf-dist/doc/latex/opteng/OptEngInstruct.pdf",
    "texmf-dist/doc/latex/opteng/README",
  },
  ["tex-ini-files"] = {
    "texmf-dist/doc/generic/tex-ini-files/README.md",
  },
  ["ifthenx"] = {
    "texmf-dist/doc/latex/ifthenx/README",
  },
  ["arabxetex"] = {
    "texmf-dist/doc/xelatex/arabxetex/README.md",
    "texmf-dist/doc/xelatex/arabxetex/arabxetex.pdf",
    "texmf-dist/doc/xelatex/arabxetex/examples/ednotes_example.pdf",
  },
  ["tex-gyre"] = {
    "texmf-dist/doc/fonts/tex-gyre/GUST-FONT-LICENSE.txt",
    "texmf-dist/doc/fonts/tex-gyre/MANIFEST-TeX-Gyre-Adventor.txt",
    "texmf-dist/doc/fonts/tex-gyre/MANIFEST-TeX-Gyre-Bonum.txt",
    "texmf-dist/doc/fonts/tex-gyre/MANIFEST-TeX-Gyre-Chorus.txt",
    "texmf-dist/doc/fonts/tex-gyre/MANIFEST-TeX-Gyre-Cursor.txt",
    "texmf-dist/doc/fonts/tex-gyre/MANIFEST-TeX-Gyre-Heros.txt",
    "texmf-dist/doc/fonts/tex-gyre/MANIFEST-TeX-Gyre-Pagella.txt",
    "texmf-dist/doc/fonts/tex-gyre/MANIFEST-TeX-Gyre-Schola.txt",
    "texmf-dist/doc/fonts/tex-gyre/MANIFEST-TeX-Gyre-Termes.txt",
    "texmf-dist/doc/fonts/tex-gyre/README-TeX-Gyre-Adventor.txt",
    "texmf-dist/doc/fonts/tex-gyre/README-TeX-Gyre-Bonum.txt",
    "texmf-dist/doc/fonts/tex-gyre/README-TeX-Gyre-Chorus.txt",
    "texmf-dist/doc/fonts/tex-gyre/README-TeX-Gyre-Cursor.txt",
    "texmf-dist/doc/fonts/tex-gyre/README-TeX-Gyre-Heros.txt",
    "texmf-dist/doc/fonts/tex-gyre/README-TeX-Gyre-Pagella.txt",
    "texmf-dist/doc/fonts/tex-gyre/README-TeX-Gyre-Schola.txt",
    "texmf-dist/doc/fonts/tex-gyre/README-TeX-Gyre-Termes.txt",
    "texmf-dist/doc/fonts/tex-gyre/qag-hist.txt",
    "texmf-dist/doc/fonts/tex-gyre/qag-info.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qbk-hist.txt",
    "texmf-dist/doc/fonts/tex-gyre/qbk-info.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qbk-test.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qcr-hist.txt",
    "texmf-dist/doc/fonts/tex-gyre/qcr-info.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qcr-test.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qcs-hist.txt",
    "texmf-dist/doc/fonts/tex-gyre/qcs-info.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qcs-test.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qhv-hist.txt",
    "texmf-dist/doc/fonts/tex-gyre/qhv-info.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qhv-test.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qpl-hist.txt",
    "texmf-dist/doc/fonts/tex-gyre/qpl-info.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qtm-hist.txt",
    "texmf-dist/doc/fonts/tex-gyre/qtm-info.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qtm-test.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qzc-hist.txt",
    "texmf-dist/doc/fonts/tex-gyre/qzc-info.pdf",
    "texmf-dist/doc/fonts/tex-gyre/qzc-test.pdf",
  },
  ["tex-ewd"] = {
    "texmf-dist/doc/generic/tex-ewd/README",
    "texmf-dist/doc/generic/tex-ewd/bsdlic.txt",
  },
  ["teubner"] = {
    "texmf-dist/doc/latex/teubner/README",
    "texmf-dist/doc/latex/teubner/teubner-doc.pdf",
    "texmf-dist/doc/latex/teubner/teubner.pdf",
    "texmf-dist/doc/latex/teubner/teubner.txt",
  },
  ["epslatex-fr"] = {
    "texmf-dist/doc/latex/epslatex-fr/README",
    "texmf-dist/doc/latex/epslatex-fr/README.TEXLIVE",
    "texmf-dist/doc/latex/epslatex-fr/cm.ps",
    "texmf-dist/doc/latex/epslatex-fr/fepslatex.pdf",
    "texmf-dist/doc/latex/epslatex-fr/pretzel.ps",
    "texmf-dist/doc/latex/epslatex-fr/rosette.ps",
  },
  ["arsclassica"] = {
    "texmf-dist/doc/latex/arsclassica/ArsClassica.pdf",
    "texmf-dist/doc/latex/arsclassica/Changes",
    "texmf-dist/doc/latex/arsclassica/Graphics/TFZSuperEllisse.pdf",
    "texmf-dist/doc/latex/arsclassica/README",
  },
  ["type1cm"] = {
    "texmf-dist/doc/latex/type1cm/type1cm-doc.pdf",
    "texmf-dist/doc/latex/type1cm/type1cm.txt",
  },
  ["lyluatex"] = {
    "texmf-dist/doc/support/lyluatex/LICENSE",
    "texmf-dist/doc/support/lyluatex/README.md",
    "texmf-dist/doc/support/lyluatex/latexmkrc",
    "texmf-dist/doc/support/lyluatex/lyluatex.pdf",
  },
  ["cfr-lm"] = {
    "texmf-dist/doc/fonts/cfr-lm/README",
    "texmf-dist/doc/fonts/cfr-lm/cfr-lm.pdf",
    "texmf-dist/doc/fonts/cfr-lm/clm-test.pdf",
    "texmf-dist/doc/fonts/cfr-lm/manifest.txt",
  },
  ["testhyphens"] = {
    "texmf-dist/doc/latex/testhyphens/README",
    "texmf-dist/doc/latex/testhyphens/manifest.txt",
    "texmf-dist/doc/latex/testhyphens/testhyphens.pdf",
  },
  ["seetexk"] = {
    "texmf-dist/doc/man/man1/dvibook.man1.pdf",
    "texmf-dist/doc/man/man1/dviconcat.man1.pdf",
    "texmf-dist/doc/man/man1/dviselect.man1.pdf",
    "texmf-dist/doc/man/man1/dvitodvi.man1.pdf",
  },
  ["termcal"] = {
    "texmf-dist/doc/latex/termcal/Contents",
    "texmf-dist/doc/latex/termcal/README",
    "texmf-dist/doc/latex/termcal/termcal.pdf",
  },
  ["miama"] = {
    "texmf-dist/doc/fonts/miama/README",
    "texmf-dist/doc/fonts/miama/miama.pdf",
  },
  ["swimgraf"] = {
    "texmf-dist/doc/latex/swimgraf/100br1.pdf",
    "texmf-dist/doc/latex/swimgraf/100br2.pdf",
    "texmf-dist/doc/latex/swimgraf/README",
    "texmf-dist/doc/latex/swimgraf/README.TEXLIVE",
    "texmf-dist/doc/latex/swimgraf/text1.pdf",
    "texmf-dist/doc/latex/swimgraf/text2.pdf",
  },
  ["dsptricks"] = {
    "texmf-dist/doc/latex/dsptricks/README",
    "texmf-dist/doc/latex/dsptricks/dspTricksManual.pdf",
  },
  ["tengwarscript"] = {
    "texmf-dist/doc/latex/tengwarscript/COPYING",
    "texmf-dist/doc/latex/tengwarscript/README",
    "texmf-dist/doc/latex/tengwarscript/quetta.pdf",
    "texmf-dist/doc/latex/tengwarscript/tengfonts.pdf",
    "texmf-dist/doc/latex/tengwarscript/tengtest.pdf",
    "texmf-dist/doc/latex/tengwarscript/tengwarscript.pdf",
  },
  ["tempora"] = {
    "texmf-dist/doc/fonts/tempora/COPYING",
    "texmf-dist/doc/fonts/tempora/README",
    "texmf-dist/doc/fonts/tempora/tempora-doc.pdf",
  },
  ["type1fonts"] = {
    "texmf-dist/doc/fonts/Type1fonts/README",
    "texmf-dist/doc/fonts/Type1fonts/fontinstallationguide.pdf",
  },
  ["templatetools"] = {
    "texmf-dist/doc/latex/templatetools/README",
    "texmf-dist/doc/latex/templatetools/templatetools.pdf",
  },
  ["templates-sommer"] = {
    "texmf-dist/doc/latex/templates-sommer/README.de",
  },
  ["sourcecodepro"] = {
    "texmf-dist/doc/latex/sourcecodepro/LICENSE.txt",
    "texmf-dist/doc/latex/sourcecodepro/sourcecodepro-otf-specimen.pdf",
    "texmf-dist/doc/latex/sourcecodepro/sourcecodepro-type1-specimen.pdf",
    "texmf-dist/doc/latex/sourcecodepro/sourcecodepro.pdf",
  },
  ["cmcyr"] = {
    "texmf-dist/doc/fonts/cmcyr/README",
    "texmf-dist/doc/fonts/cmcyr/cmalt",
    "texmf-dist/doc/fonts/cmcyr/cmalte",
    "texmf-dist/doc/fonts/cmcyr/cmiso",
    "texmf-dist/doc/fonts/cmcyr/cmisoe",
    "texmf-dist/doc/fonts/cmcyr/cmkde",
    "texmf-dist/doc/fonts/cmcyr/cmkdee",
    "texmf-dist/doc/fonts/cmcyr/cmkoi8",
    "texmf-dist/doc/fonts/cmcyr/cmkoi8e",
    "texmf-dist/doc/fonts/cmcyr/cmwin",
    "texmf-dist/doc/fonts/cmcyr/cmwine",
    "texmf-dist/doc/fonts/cmcyr/cyralt",
    "texmf-dist/doc/fonts/cmcyr/cyralte",
    "texmf-dist/doc/fonts/cmcyr/cyriso",
    "texmf-dist/doc/fonts/cmcyr/cyrisoe",
    "texmf-dist/doc/fonts/cmcyr/cyrkde",
    "texmf-dist/doc/fonts/cmcyr/cyrkdee",
    "texmf-dist/doc/fonts/cmcyr/cyrkoi8",
    "texmf-dist/doc/fonts/cmcyr/cyrkoi8e",
    "texmf-dist/doc/fonts/cmcyr/cyrwin",
    "texmf-dist/doc/fonts/cmcyr/cyrwine",
    "texmf-dist/doc/fonts/cmcyr/wncalt",
    "texmf-dist/doc/fonts/cmcyr/wncalte",
    "texmf-dist/doc/fonts/cmcyr/wnciso",
    "texmf-dist/doc/fonts/cmcyr/wncisoe",
    "texmf-dist/doc/fonts/cmcyr/wnckoi8",
    "texmf-dist/doc/fonts/cmcyr/wnckoi8e",
  },
  ["universa"] = {
    "texmf-dist/doc/fonts/universa/README.uni",
  },
  ["technics"] = {
    "texmf-dist/doc/latex/technics/technics.pdf",
  },
  ["tcolorbox"] = {
    "texmf-dist/doc/latex/tcolorbox/CHANGES",
    "texmf-dist/doc/latex/tcolorbox/README",
    "texmf-dist/doc/latex/tcolorbox/tcolorbox-example-poster.pdf",
    "texmf-dist/doc/latex/tcolorbox/tcolorbox-example.pdf",
    "texmf-dist/doc/latex/tcolorbox/tcolorbox-tutorial-poster.pdf",
    "texmf-dist/doc/latex/tcolorbox/tcolorbox.pdf",
  },
  ["newfile"] = {
    "texmf-dist/doc/latex/newfile/README",
    "texmf-dist/doc/latex/newfile/newfile.pdf",
  },
  ["amsaddr"] = {
    "texmf-dist/doc/latex/amsaddr/README",
    "texmf-dist/doc/latex/amsaddr/amsaddr.pdf",
  },
  ["rutitlepage"] = {
    "texmf-dist/doc/latex/rutitlepage/README.TEXLIVE",
    "texmf-dist/doc/latex/rutitlepage/README.md",
    "texmf-dist/doc/latex/rutitlepage/rutitlepage.pdf",
  },
  ["tamethebeast"] = {
    "texmf-dist/doc/bibtex/tamethebeast/CHANGES",
    "texmf-dist/doc/bibtex/tamethebeast/Makefile",
    "texmf-dist/doc/bibtex/tamethebeast/README",
    "texmf-dist/doc/bibtex/tamethebeast/ttb_en.pdf",
  },
  ["answers"] = {
    "texmf-dist/doc/latex/answers/answers.pdf",
  },
  ["talk"] = {
    "texmf-dist/doc/latex/talk/README",
    "texmf-dist/doc/latex/talk/talkdoc.pdf",
  },
  ["tagpair"] = {
    "texmf-dist/doc/latex/tagpair/COPYING",
    "texmf-dist/doc/latex/tagpair/ChangeLog",
    "texmf-dist/doc/latex/tagpair/README",
    "texmf-dist/doc/latex/tagpair/sample.pdf",
    "texmf-dist/doc/latex/tagpair/tagpair.pdf",
  },
  ["tagging"] = {
    "texmf-dist/doc/latex/tagging/README",
    "texmf-dist/doc/latex/tagging/tagging.pdf",
  },
  ["fancyhdr"] = {
    "texmf-dist/doc/latex/fancyhdr/README",
    "texmf-dist/doc/latex/fancyhdr/fancyhdr.pdf",
  },
  ["graphicxbox"] = {
    "texmf-dist/doc/latex/graphicxbox/README",
    "texmf-dist/doc/latex/graphicxbox/doc/graphicxbox.pdf",
    "texmf-dist/doc/latex/graphicxbox/examples/graphics/IndianBlanket.pdf",
    "texmf-dist/doc/latex/graphicxbox/examples/graphics/Wood-Brown.pdf",
    "texmf-dist/doc/latex/graphicxbox/examples/graphics/grandcanyon.pdf",
    "texmf-dist/doc/latex/graphicxbox/examples/graphics/news_bgr.pdf",
    "texmf-dist/doc/latex/graphicxbox/examples/grfxbox_tst.pdf",
    "texmf-dist/doc/latex/graphicxbox/examples/grfxbox_tst_indians.pdf",
    "texmf-dist/doc/latex/graphicxbox/examples/grfxbox_tst_sp.pdf",
  },
  ["tabulars-e"] = {
    "texmf-dist/doc/latex/tabulars-e/README",
  },
  ["tabu"] = {
    "texmf-dist/doc/latex/tabu/README",
    "texmf-dist/doc/latex/tabu/tabu.pdf",
  },
  ["pst-knot"] = {
    "texmf-dist/doc/generic/pst-knot/Changes",
    "texmf-dist/doc/generic/pst-knot/README",
    "texmf-dist/doc/generic/pst-knot/pst-knot-doc.pdf",
  },
  ["nonfloat"] = {
    "texmf-dist/doc/latex/nonfloat/nonfloat-en.pdf",
    "texmf-dist/doc/latex/nonfloat/nonfloat.pdf",
  },
  ["tabriz-thesis"] = {
    "texmf-dist/doc/latex/tabriz-thesis/README",
    "texmf-dist/doc/latex/tabriz-thesis/tabriz-thesis.pdf",
  },
  ["tkz-linknodes"] = {
    "texmf-dist/doc/latex/tkz-linknodes/README",
    "texmf-dist/doc/latex/tkz-linknodes/examples/equation.pdf",
    "texmf-dist/doc/latex/tkz-linknodes/examples/quadratic.pdf",
    "texmf-dist/doc/latex/tkz-linknodes/examples/system.pdf",
    "texmf-dist/doc/latex/tkz-linknodes/readme-linknodes.txt",
    "texmf-dist/doc/latex/tkz-linknodes/tkz-linknodes-screen.pdf",
  },
  ["tablor"] = {
    "texmf-dist/doc/latex/tablor/README",
    "texmf-dist/doc/latex/tablor/README-fr.txt",
    "texmf-dist/doc/latex/tablor/tablor.html",
    "texmf-dist/doc/latex/tablor/tablor.pdf",
  },
  ["historische-zeitschrift"] = {
    "texmf-dist/doc/latex/historische-zeitschrift/CHANGES",
    "texmf-dist/doc/latex/historische-zeitschrift/LIESMICH",
    "texmf-dist/doc/latex/historische-zeitschrift/README",
  },
  ["tablists"] = {
    "texmf-dist/doc/latex/tablists/README",
    "texmf-dist/doc/latex/tablists/tablists-rus.pdf",
    "texmf-dist/doc/latex/tablists/tablists.pdf",
  },
  ["wallcalendar"] = {
    "texmf-dist/doc/lualatex/wallcalendar/LICENSE.txt",
    "texmf-dist/doc/lualatex/wallcalendar/README.md",
    "texmf-dist/doc/lualatex/wallcalendar/doc/Makefile",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/Makefile",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-plain-01.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-plain-02.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-plain-03.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-showframe-01.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-showframe-02.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-showframe-03.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-showtrims-01.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-showtrims-02.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-showtrims-03.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-varnishmask-01.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-varnishmask-02.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-burst/cal-varnishmask-03.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-marks.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-plain.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-showframe.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-showtrims.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-thumbnails.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-translations-english.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-translations-hungarian.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-translations-japanese.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-varnishmask.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/examples/cal-year-planner.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/doc/latexmkrc",
    "texmf-dist/doc/lualatex/wallcalendar/wallcalendar-code.pdf",
    "texmf-dist/doc/lualatex/wallcalendar/wallcalendar.pdf",
  },
  ["tabfigures"] = {
    "texmf-dist/doc/latex/tabfigures/README",
    "texmf-dist/doc/latex/tabfigures/tabfigures.pdf",
  },
  ["arcs"] = {
    "texmf-dist/doc/latex/arcs/README",
    "texmf-dist/doc/latex/arcs/arcs.pdf",
  },
  ["pdftricks2"] = {
    "texmf-dist/doc/latex/pdftricks2/README",
    "texmf-dist/doc/latex/pdftricks2/pdftricks2-doc.pdf",
  },
  ["attachfile"] = {
    "texmf-dist/doc/latex/attachfile/README",
    "texmf-dist/doc/latex/attachfile/attachfile.pdf",
  },
  ["testidx"] = {
    "texmf-dist/doc/latex/testidx/CHANGES",
    "texmf-dist/doc/latex/testidx/README",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-a4.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-amsmath.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-babel-german.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-german.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-hyp.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-letter.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-lua.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-subset.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-t1.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-utf8.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-xe-digraphs.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx-xe.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idx.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idxgloss-bib2gls-xe.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idxgloss-bib2gls.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idxgloss-desc.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idxgloss-xe-digraphs.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idxgloss-xindy.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-idxgloss.pdf",
    "texmf-dist/doc/latex/testidx/samples/sample-noidxgloss.pdf",
    "texmf-dist/doc/latex/testidx/testidx-code.pdf",
    "texmf-dist/doc/latex/testidx/testidx-manual.pdf",
  },
  ["repeatindex"] = {
    "texmf-dist/doc/latex/repeatindex/README",
  },
  ["courier-scaled"] = {
    "texmf-dist/doc/fonts/courier-scaled/Couriers.pdf",
    "texmf-dist/doc/fonts/courier-scaled/README",
  },
  ["eplain"] = {
    "texmf-dist/doc/eplain/AUTHORS",
    "texmf-dist/doc/eplain/COPYING",
    "texmf-dist/doc/eplain/ChangeLog",
    "texmf-dist/doc/eplain/INSTALL",
    "texmf-dist/doc/eplain/NEWS",
    "texmf-dist/doc/eplain/README",
    "texmf-dist/doc/eplain/demo/Makefile",
    "texmf-dist/doc/eplain/eplain.html",
    "texmf-dist/doc/eplain/eplain.pdf",
    "texmf-dist/doc/eplain/util/idxuniq",
    "texmf-dist/doc/eplain/util/trimsee",
    "texmf-dist/doc/man/man1/eplain.man1.pdf",
  },
  ["aurl"] = {
    "texmf-dist/doc/latex/aurl/README.md",
  },
  ["chemformula"] = {
    "texmf-dist/doc/latex/chemformula/README",
    "texmf-dist/doc/latex/chemformula/chemformula_en.pdf",
  },
  ["latex2e-help-texinfo-spanish"] = {
    "texmf-dist/doc/latex/latex2e-help-texinfo-spanish/latex2e-es.html",
    "texmf-dist/doc/latex/latex2e-help-texinfo-spanish/latex2e-es.pdf",
    "texmf-dist/doc/latex/latex2e-help-texinfo-spanish/latex2e-es.txt",
  },
  ["pinlabel"] = {
    "texmf-dist/doc/latex/pinlabel/pinlabdoc.pdf",
    "texmf-dist/doc/latex/pinlabel/src/fig3.pdf",
    "texmf-dist/doc/latex/pinlabel/src/fig6.pdf",
    "texmf-dist/doc/latex/pinlabel/src/put.pdf",
    "texmf-dist/doc/latex/pinlabel/src/put2.pdf",
    "texmf-dist/doc/latex/pinlabel/src/screen.pdf",
  },
  ["kotex-plain"] = {
    "texmf-dist/doc/plain/kotex-plain/ChangeLog",
    "texmf-dist/doc/plain/kotex-plain/README",
  },
  ["trivfloat"] = {
    "texmf-dist/doc/latex/trivfloat/README",
    "texmf-dist/doc/latex/trivfloat/trivfloat.pdf",
  },
  ["fileinfo"] = {
    "texmf-dist/doc/latex/fileinfo/README",
    "texmf-dist/doc/latex/fileinfo/README.pdf",
    "texmf-dist/doc/latex/fileinfo/RELEASEs.txt",
    "texmf-dist/doc/latex/fileinfo/SrcFILEs.txt",
    "texmf-dist/doc/latex/fileinfo/myfilist.pdf",
    "texmf-dist/doc/latex/fileinfo/readprov.pdf",
  },
  ["heuristica"] = {
    "texmf-dist/doc/fonts/heuristica/FontLog.txt",
    "texmf-dist/doc/fonts/heuristica/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/heuristica/OFL.txt",
    "texmf-dist/doc/fonts/heuristica/README",
    "texmf-dist/doc/fonts/heuristica/heuristica-doc.pdf",
  },
  ["nostarch"] = {
    "texmf-dist/doc/latex/nostarch/README",
    "texmf-dist/doc/latex/nostarch/nostarch.pdf",
    "texmf-dist/doc/latex/nostarch/nssample.pdf",
  },
  ["pbsheet"] = {
    "texmf-dist/doc/latex/pbsheet/LPPL",
    "texmf-dist/doc/latex/pbsheet/README",
    "texmf-dist/doc/latex/pbsheet/pbsheet.pdf",
    "texmf-dist/doc/latex/pbsheet/xpl/GNUmakefile",
    "texmf-dist/doc/latex/pbsheet/xpl/img/simbin.pdf",
    "texmf-dist/doc/latex/pbsheet/xpl/xpl-fr.dvi",
    "texmf-dist/doc/latex/pbsheet/xpl/xpl-fr.pdf",
    "texmf-dist/doc/latex/pbsheet/xpl/xpl-fr.ps",
  },
  ["gamebook"] = {
    "texmf-dist/doc/latex/gamebook/README",
    "texmf-dist/doc/latex/gamebook/gamebook-example.pdf",
    "texmf-dist/doc/latex/gamebook/gamebook.pdf",
    "texmf-dist/doc/latex/gamebook/lppl.txt",
  },
  ["biblist"] = {
    "texmf-dist/doc/latex/biblist/README",
    "texmf-dist/doc/latex/biblist/biblist.pdf",
  },
  ["sympytexpackage"] = {
    "texmf-dist/doc/latex/sympytexpackage/README",
    "texmf-dist/doc/latex/sympytexpackage/README.TEXLIVE",
  },
  ["nolbreaks"] = {
    "texmf-dist/doc/latex/nolbreaks/nolbreaks.pdf",
  },
  ["syllogism"] = {
    "texmf-dist/doc/latex/syllogism/Examples.pdf",
    "texmf-dist/doc/latex/syllogism/README",
    "texmf-dist/doc/latex/syllogism/syllogism.pdf",
  },
  ["swebib"] = {
    "texmf-dist/doc/latex/swebib/README",
  },
  ["svninfo"] = {
    "texmf-dist/doc/latex/svninfo/README",
    "texmf-dist/doc/latex/svninfo/svninfo.pdf",
  },
  ["edmac"] = {
    "texmf-dist/doc/latex/edmac/COPYRIGHT",
    "texmf-dist/doc/latex/edmac/copying",
    "texmf-dist/doc/latex/edmac/ed-nfss.txt",
    "texmf-dist/doc/latex/edmac/readme",
  },
  ["simplecd"] = {
    "texmf-dist/doc/latex/simplecd/README",
    "texmf-dist/doc/latex/simplecd/examples.pdf",
    "texmf-dist/doc/latex/simplecd/simplecd.pdf",
  },
  ["struktex"] = {
    "texmf-dist/doc/latex/struktex/README.md",
    "texmf-dist/doc/latex/struktex/struktex.de.pdf",
    "texmf-dist/doc/latex/struktex/struktex.en.pdf",
  },
  ["mdsymbol"] = {
    "texmf-dist/doc/fonts/mdsymbol/FONTLOG.txt",
    "texmf-dist/doc/fonts/mdsymbol/OFL.txt",
    "texmf-dist/doc/latex/mdsymbol/mdsymbol.pdf",
  },
  ["harpoon"] = {
    "texmf-dist/doc/latex/harpoon/harpoon.pdf",
  },
  ["fdsymbol"] = {
    "texmf-dist/doc/fonts/fdsymbol/FONTLOG.txt",
    "texmf-dist/doc/fonts/fdsymbol/OFL.txt",
    "texmf-dist/doc/latex/fdsymbol/fdsymbol.pdf",
  },
  ["engtlc"] = {
    "texmf-dist/doc/latex/engtlc/README",
    "texmf-dist/doc/latex/engtlc/engtlc.pdf",
    "texmf-dist/doc/latex/engtlc/itengtlc.pdf",
  },
  ["fragmaster"] = {
    "texmf-dist/doc/support/fragmaster/AUTHORS",
    "texmf-dist/doc/support/fragmaster/COPYING",
    "texmf-dist/doc/support/fragmaster/CREDITS",
    "texmf-dist/doc/support/fragmaster/Changes",
    "texmf-dist/doc/support/fragmaster/README",
    "texmf-dist/doc/support/fragmaster/README.de",
    "texmf-dist/doc/support/fragmaster/example/document.pdf",
    "texmf-dist/doc/support/fragmaster/example/document.ps",
    "texmf-dist/doc/support/fragmaster/example/parabel.pdf",
    "texmf-dist/doc/support/fragmaster/example/parabel_fm",
    "texmf-dist/doc/support/fragmaster/example/parabel_fm.pdf",
    "texmf-dist/doc/support/fragmaster/fragmaster.pdf",
  },
  ["issuulinks"] = {
    "texmf-dist/doc/latex/issuulinks/Makefile",
    "texmf-dist/doc/latex/issuulinks/README",
    "texmf-dist/doc/latex/issuulinks/issuulinks.pdf",
    "texmf-dist/doc/latex/issuulinks/sample.pdf",
  },
  ["esstix"] = {
    "texmf-dist/doc/fonts/esstix/Esstix.pdf",
    "texmf-dist/doc/fonts/esstix/README",
  },
  ["svn"] = {
    "texmf-dist/doc/latex/svn/README",
    "texmf-dist/doc/latex/svn/svn.pdf",
  },
  ["svgcolor"] = {
    "texmf-dist/doc/latex/svgcolor/README",
  },
  ["svg-inkscape"] = {
    "texmf-dist/doc/latex/svg-inkscape/InkscapePDFLaTeX.pdf",
    "texmf-dist/doc/latex/svg-inkscape/README",
    "texmf-dist/doc/latex/svg-inkscape/image-normal.pdf",
    "texmf-dist/doc/latex/svg-inkscape/image.pdf",
  },
  ["arabtex"] = {
    "texmf-dist/doc/latex/arabtex/announce.txt",
    "texmf-dist/doc/latex/arabtex/arabtex-doc.pdf",
    "texmf-dist/doc/latex/arabtex/arabtex.htm",
    "texmf-dist/doc/latex/arabtex/arabtex1.htm",
    "texmf-dist/doc/latex/arabtex/arabtex2.htm",
    "texmf-dist/doc/latex/arabtex/changes.htm",
    "texmf-dist/doc/latex/arabtex/changes.txt",
    "texmf-dist/doc/latex/arabtex/changes2.txt",
    "texmf-dist/doc/latex/arabtex/chg311.htm",
    "texmf-dist/doc/latex/arabtex/chg311a.htm",
    "texmf-dist/doc/latex/arabtex/chg311b.htm",
    "texmf-dist/doc/latex/arabtex/chg311c.htm",
    "texmf-dist/doc/latex/arabtex/chg311d.htm",
    "texmf-dist/doc/latex/arabtex/guha.ps",
    "texmf-dist/doc/latex/arabtex/install.txt",
    "texmf-dist/doc/latex/arabtex/lppl.txt",
    "texmf-dist/doc/latex/arabtex/malay.ps",
    "texmf-dist/doc/latex/arabtex/manifest.txt",
    "texmf-dist/doc/latex/arabtex/miktex.htm",
    "texmf-dist/doc/latex/arabtex/readme.305",
    "texmf-dist/doc/latex/arabtex/readme.txt",
    "texmf-dist/doc/latex/arabtex/refer.htm",
    "texmf-dist/doc/latex/arabtex/sindhi.ps",
    "texmf-dist/doc/latex/arabtex/tetex.txt",
    "texmf-dist/doc/latex/arabtex/uighur.ps",
  },
  ["esint-type1"] = {
    "texmf-dist/doc/fonts/esint-type1/README",
    "texmf-dist/doc/fonts/esint-type1/README.plainTeX",
    "texmf-dist/doc/fonts/esint-type1/table.pdf",
  },
  ["susy"] = {
    "texmf-dist/doc/latex/susy/README",
    "texmf-dist/doc/latex/susy/README.TEXLIVE",
  },
  ["supertabular"] = {
    "texmf-dist/doc/latex/supertabular/CATALOG",
    "texmf-dist/doc/latex/supertabular/ChangeLog",
    "texmf-dist/doc/latex/supertabular/MANIFEST",
    "texmf-dist/doc/latex/supertabular/README",
    "texmf-dist/doc/latex/supertabular/supertabular.pdf",
  },
  ["suftesi"] = {
    "texmf-dist/doc/latex/suftesi/README",
    "texmf-dist/doc/latex/suftesi/suftesi.pdf",
  },
  ["substr"] = {
    "texmf-dist/doc/latex/substr/ChangeLog",
    "texmf-dist/doc/latex/substr/README",
    "texmf-dist/doc/latex/substr/testsubstr.pdf",
  },
  ["schemabloc"] = {
    "texmf-dist/doc/latex/schemabloc/README",
    "texmf-dist/doc/latex/schemabloc/schemabloc.pdf",
  },
  ["substances"] = {
    "texmf-dist/doc/latex/substances/README",
    "texmf-dist/doc/latex/substances/substances_en.pdf",
  },
  ["subfloat"] = {
    "texmf-dist/doc/latex/subfloat/ChangeLog",
    "texmf-dist/doc/latex/subfloat/README",
    "texmf-dist/doc/latex/subfloat/subfloat.pdf",
  },
  ["subfigure"] = {
    "texmf-dist/doc/latex/subfigure/README",
    "texmf-dist/doc/latex/subfigure/subfigure.pdf",
  },
  ["subfigmat"] = {
    "texmf-dist/doc/latex/subfigmat/subfigmat-doc.pdf",
  },
  ["bbold"] = {
    "texmf-dist/doc/latex/bbold/INSTALL",
    "texmf-dist/doc/latex/bbold/README",
    "texmf-dist/doc/latex/bbold/bbold.pdf",
  },
  ["subfig"] = {
    "texmf-dist/doc/latex/subfig/README",
    "texmf-dist/doc/latex/subfig/subfig.pdf",
  },
  ["subeqn"] = {
    "texmf-dist/doc/latex/subeqn/manifest.txt",
    "texmf-dist/doc/latex/subeqn/subeqn.pdf",
  },
  ["suanpan"] = {
    "texmf-dist/doc/metapost/suanpan/README",
    "texmf-dist/doc/metapost/suanpan/article.txt",
  },
  ["gmutils"] = {
    "texmf-dist/doc/latex/gmutils/README",
    "texmf-dist/doc/latex/gmutils/gmutils.pdf",
  },
  ["stix2-otf"] = {
    "texmf-dist/doc/fonts/stix2-otf/README.md",
    "texmf-dist/doc/fonts/stix2-otf/STIX_2.0.0_license.pdf",
    "texmf-dist/doc/fonts/stix2-otf/STIX_2.0.0_release_notes.txt",
    "texmf-dist/doc/fonts/stix2-otf/STIX_2.0.0_stylistic_sets.pdf",
  },
  ["acmart"] = {
    "texmf-dist/doc/latex/acmart/Makefile",
    "texmf-dist/doc/latex/acmart/README",
    "texmf-dist/doc/latex/acmart/acmart.pdf",
    "texmf-dist/doc/latex/acmart/acmguide.pdf",
    "texmf-dist/doc/latex/acmart/flies-eps-converted-to.pdf",
    "texmf-dist/doc/latex/acmart/fly-eps-converted-to.pdf",
    "texmf-dist/doc/latex/acmart/mouse-eps-converted-to.pdf",
    "texmf-dist/doc/latex/acmart/rosette-eps-converted-to.pdf",
    "texmf-dist/doc/latex/acmart/sample-acmlarge.pdf",
    "texmf-dist/doc/latex/acmart/sample-acmsmall.pdf",
    "texmf-dist/doc/latex/acmart/sample-acmtog.pdf",
    "texmf-dist/doc/latex/acmart/sample-manuscript.pdf",
    "texmf-dist/doc/latex/acmart/sample-sigchi-a.pdf",
    "texmf-dist/doc/latex/acmart/sample-sigchi.pdf",
    "texmf-dist/doc/latex/acmart/sample-sigconf-authordraft.pdf",
    "texmf-dist/doc/latex/acmart/sample-sigconf-xelatex.pdf",
    "texmf-dist/doc/latex/acmart/sample-sigconf.pdf",
    "texmf-dist/doc/latex/acmart/sample-sigplan.pdf",
    "texmf-dist/doc/latex/acmart/sampleteaser.pdf",
  },
  ["romannum"] = {
    "texmf-dist/doc/latex/romannum/README",
    "texmf-dist/doc/latex/romannum/romannum.pdf",
  },
  ["delimtxt"] = {
    "texmf-dist/doc/latex/delimtxt/delimtxt.pdf",
  },
  ["studenthandouts"] = {
    "texmf-dist/doc/latex/studenthandouts/README",
    "texmf-dist/doc/latex/studenthandouts/studenthandouts-doc.pdf",
  },
  ["sttools"] = {
    "texmf-dist/doc/latex/sttools/README.md",
    "texmf-dist/doc/latex/sttools/cuted.pdf",
    "texmf-dist/doc/latex/sttools/floatpag.pdf",
    "texmf-dist/doc/latex/sttools/flushend.pdf",
    "texmf-dist/doc/latex/sttools/midfloat.pdf",
    "texmf-dist/doc/latex/sttools/stabular.pdf",
    "texmf-dist/doc/latex/sttools/stfloats.pdf",
    "texmf-dist/doc/latex/sttools/sttools.pdf",
    "texmf-dist/doc/latex/sttools/texsort.pdf",
  },
  ["toolbox"] = {
    "texmf-dist/doc/latex/toolbox/README",
    "texmf-dist/doc/latex/toolbox/toolbox.pdf",
    "texmf-dist/doc/latex/toolbox/toolbox.txt",
  },
  ["stringstrings"] = {
    "texmf-dist/doc/latex/stringstrings/README",
    "texmf-dist/doc/latex/stringstrings/stringstrings.pdf",
  },
  ["storecmd"] = {
    "texmf-dist/doc/latex/storecmd/README",
    "texmf-dist/doc/latex/storecmd/storecmd-guide.pdf",
  },
  ["storebox"] = {
    "texmf-dist/doc/latex/storebox/README",
    "texmf-dist/doc/latex/storebox/storebox.pdf",
  },
  ["structmech"] = {
    "texmf-dist/doc/latex/structmech/README.md",
    "texmf-dist/doc/latex/structmech/doc/structmech.pdf",
  },
  ["matc3mem"] = {
    "texmf-dist/doc/latex/matc3mem/Makefile",
    "texmf-dist/doc/latex/matc3mem/README",
    "texmf-dist/doc/latex/matc3mem/matc3mem.pdf",
  },
  ["plantslabels"] = {
    "texmf-dist/doc/latex/plantslabels/README",
    "texmf-dist/doc/latex/plantslabels/doc/pdf/plantslabels.pdf",
    "texmf-dist/doc/latex/plantslabels/doc/tex/Makefile",
    "texmf-dist/doc/latex/plantslabels/example/pdf/example.pdf",
  },
  ["chemcono"] = {
    "texmf-dist/doc/latex/chemcono/chemcono.pdf",
  },
  ["comfortaa"] = {
    "texmf-dist/doc/fonts/comfortaa/CHANGES",
    "texmf-dist/doc/fonts/comfortaa/README",
    "texmf-dist/doc/fonts/comfortaa/comfortaa-samples.pdf",
    "texmf-dist/doc/fonts/comfortaa/comfortaa.pdf",
    "texmf-dist/doc/fonts/comfortaa/manifest.txt",
  },
  ["stix"] = {
    "texmf-dist/doc/fonts/stix/README",
    "texmf-dist/doc/fonts/stix/STIX_Font_1.1.1-latex_Release_Documentation.pdf",
    "texmf-dist/doc/fonts/stix/STIX_Font_1.1.1_Release_Documentation.pdf",
    "texmf-dist/doc/fonts/stix/STIX_Font_License_2010.txt",
    "texmf-dist/doc/fonts/stix/stix.pdf",
  },
  ["bidicontour"] = {
    "texmf-dist/doc/xelatex/bidicontour/README",
    "texmf-dist/doc/xelatex/bidicontour/bidicontour-doc.pdf",
    "texmf-dist/doc/xelatex/bidicontour/bidicontour-example-copies.pdf",
    "texmf-dist/doc/xelatex/bidicontour/bidicontour-example-outline.pdf",
  },
  ["everyhook"] = {
    "texmf-dist/doc/latex/everyhook/README",
    "texmf-dist/doc/latex/everyhook/everyhook.pdf",
  },
  ["stickstoo"] = {
    "texmf-dist/doc/fonts/stickstoo/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/stickstoo/OFL.txt",
    "texmf-dist/doc/fonts/stickstoo/README",
    "texmf-dist/doc/fonts/stickstoo/stickstoo-doc.pdf",
  },
  ["reotex"] = {
    "texmf-dist/doc/latex/reotex/README",
    "texmf-dist/doc/latex/reotex/reotex.pdf",
  },
  ["stellenbosch"] = {
    "texmf-dist/doc/latex/stellenbosch/CHANGELOG",
    "texmf-dist/doc/latex/stellenbosch/README",
    "texmf-dist/doc/latex/stellenbosch/USbib-1.0.pdf",
    "texmf-dist/doc/latex/stellenbosch/USlogos-4.0.pdf",
    "texmf-dist/doc/latex/stellenbosch/USnomencl-1.1.pdf",
    "texmf-dist/doc/latex/stellenbosch/USsummary-1.0a.pdf",
    "texmf-dist/doc/latex/stellenbosch/USthesis-5.0.pdf",
    "texmf-dist/doc/latex/stellenbosch/UStitle-1.0.pdf",
  },
  ["tabularborder"] = {
    "texmf-dist/doc/latex/tabularborder/tabularborder.pdf",
  },
  ["stdpage"] = {
    "texmf-dist/doc/latex/stdpage/README",
    "texmf-dist/doc/latex/stdpage/stdpage.pdf",
  },
  ["cclicenses"] = {
    "texmf-dist/doc/latex/cclicenses/README",
    "texmf-dist/doc/latex/cclicenses/cclicenses_short.pdf",
  },
  ["statmath"] = {
    "texmf-dist/doc/latex/statmath/README",
    "texmf-dist/doc/latex/statmath/statmath.pdf",
  },
  ["statistik"] = {
    "texmf-dist/doc/latex/statistik/statistik.pdf",
  },
  ["xhfill"] = {
    "texmf-dist/doc/latex/xhfill/Changes",
    "texmf-dist/doc/latex/xhfill/Makefile",
    "texmf-dist/doc/latex/xhfill/xhfill-doc.pdf",
  },
  ["startex"] = {
    "texmf-dist/doc/otherformats/startex/INSTALL",
    "texmf-dist/doc/otherformats/startex/README",
    "texmf-dist/doc/otherformats/startex/README.TEXLIVE",
    "texmf-dist/doc/otherformats/startex/guide.pdf",
    "texmf-dist/doc/otherformats/startex/ideas.pdf",
    "texmf-dist/doc/otherformats/startex/startex.pdf",
  },
  ["starfont"] = {
    "texmf-dist/doc/fonts/starfont/COPYING",
    "texmf-dist/doc/fonts/starfont/Makefile",
    "texmf-dist/doc/fonts/starfont/README",
    "texmf-dist/doc/fonts/starfont/starfont.pdf",
    "texmf-dist/doc/fonts/starfont/table.pdf",
  },
  ["standalone"] = {
    "texmf-dist/doc/latex/standalone/README",
    "texmf-dist/doc/latex/standalone/standalone.pdf",
  },
  ["pst-electricfield"] = {
    "texmf-dist/doc/generic/pst-electricfield/Changes",
    "texmf-dist/doc/generic/pst-electricfield/README",
    "texmf-dist/doc/generic/pst-electricfield/pst-electricfield-docDE.pdf",
    "texmf-dist/doc/generic/pst-electricfield/pst-electricfield-docEN.pdf",
    "texmf-dist/doc/generic/pst-electricfield/pst-electricfield-docFR.pdf",
  },
  ["flipbook"] = {
    "texmf-dist/doc/latex/flipbook/Images/Anims/an1b/im01.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an1b/im02.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an1b/im03.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an1b/im04.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an1b/im05.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an1b/im06.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im0.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im1.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im10.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im11.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im12.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im13.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im14.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im15.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im16.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im2.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im3.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im4.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im5.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im6.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im7.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im8.pdf",
    "texmf-dist/doc/latex/flipbook/Images/Anims/an2/im9.pdf",
    "texmf-dist/doc/latex/flipbook/Makefile",
    "texmf-dist/doc/latex/flipbook/README",
    "texmf-dist/doc/latex/flipbook/flipbook-doc.pdf",
    "texmf-dist/doc/latex/flipbook/flipbook-ex.pdf",
  },
  ["burmese"] = {
    "texmf-dist/doc/fonts/burmese/burmguide.pdf",
  },
  ["anufinalexam"] = {
    "texmf-dist/doc/latex/anufinalexam/README",
  },
  ["numspell"] = {
    "texmf-dist/doc/latex/numspell/README",
    "texmf-dist/doc/latex/numspell/numspell.pdf",
  },
  ["harveyballs"] = {
    "texmf-dist/doc/latex/harveyballs/README",
    "texmf-dist/doc/latex/harveyballs/harveyballs-Manual.pdf",
  },
  ["microtype"] = {
    "texmf-dist/doc/latex/microtype/README.md",
    "texmf-dist/doc/latex/microtype/microtype.pdf",
  },
  ["relenc"] = {
    "texmf-dist/doc/latex/relenc/README",
  },
  ["xpinyin"] = {
    "texmf-dist/doc/latex/xpinyin/README.md",
    "texmf-dist/doc/latex/xpinyin/xpinyin.pdf",
  },
  ["acroterm"] = {
    "texmf-dist/doc/latex/acroterm/README",
    "texmf-dist/doc/latex/acroterm/acroterm.pdf",
  },
  ["soul"] = {
    "texmf-dist/doc/latex/soul/soul.pdf",
    "texmf-dist/doc/latex/soul/soul.txt",
  },
  ["unfonts-extra"] = {
    "texmf-dist/doc/fonts/unfonts-extra/COPYING",
    "texmf-dist/doc/fonts/unfonts-extra/README.md",
    "texmf-dist/doc/fonts/unfonts-extra/README_original",
  },
  ["srcredact"] = {
    "texmf-dist/doc/man/man1/srcredact.man1.pdf",
    "texmf-dist/doc/support/srcredact/Makefile",
    "texmf-dist/doc/support/srcredact/README",
    "texmf-dist/doc/support/srcredact/srcredact.pdf",
    "texmf-dist/doc/support/srcredact/twain1-default.pdf",
    "texmf-dist/doc/support/srcredact/twain1-nobonds.pdf",
    "texmf-dist/doc/support/srcredact/twain1-uppercase.pdf",
    "texmf-dist/doc/support/srcredact/twain2-default.pdf",
    "texmf-dist/doc/support/srcredact/twain2-nobonds.pdf",
    "texmf-dist/doc/support/srcredact/twain2-uppercase.pdf",
  },
  ["srbook-mem"] = {
    "texmf-dist/doc/latex/srbook-mem/README",
    "texmf-dist/doc/latex/srbook-mem/SerbianBookMem.pdf",
    "texmf-dist/doc/latex/srbook-mem/Test.pdf",
    "texmf-dist/doc/latex/srbook-mem/TestLight.pdf",
  },
  ["spverbatim"] = {
    "texmf-dist/doc/latex/spverbatim/README",
    "texmf-dist/doc/latex/spverbatim/spverbatim.pdf",
  },
  ["spreadtab"] = {
    "texmf-dist/doc/latex/spreadtab/README",
    "texmf-dist/doc/latex/spreadtab/spreadtab-en.pdf",
    "texmf-dist/doc/latex/spreadtab/spreadtab-fr.pdf",
  },
  ["spotcolor"] = {
    "texmf-dist/doc/latex/spotcolor/README",
    "texmf-dist/doc/latex/spotcolor/readme.pdf",
    "texmf-dist/doc/latex/spotcolor/readme.tcp",
    "texmf-dist/doc/latex/spotcolor/readme.tex",
  },
  ["pdfcrop"] = {
    "texmf-dist/doc/support/pdfcrop/README",
  },
  ["impatient-fr"] = {
    "texmf-dist/doc/plain/impatient-fr/README",
    "texmf-dist/doc/plain/impatient-fr/fbook.pdf",
  },
  ["plipsum"] = {
    "texmf-dist/doc/plain/plipsum/README",
    "texmf-dist/doc/plain/plipsum/plipsum-doc.pdf",
    "texmf-dist/doc/plain/plipsum/stripc",
  },
  ["barr"] = {
    "texmf-dist/doc/generic/barr/README",
    "texmf-dist/doc/generic/barr/diaxydoc.pdf",
  },
  ["first-latex-doc"] = {
    "texmf-dist/doc/latex/first-latex-doc/README",
    "texmf-dist/doc/latex/first-latex-doc/first-latex-doc.pdf",
  },
  ["linguex"] = {
    "texmf-dist/doc/latex/linguex/README",
    "texmf-dist/doc/latex/linguex/linguex-doc.pdf",
    "texmf-dist/doc/latex/linguex/ps-trees-doc.pdf",
  },
  ["lion-msc"] = {
    "texmf-dist/doc/latex/lion-msc/LICENSE",
    "texmf-dist/doc/latex/lion-msc/README.md",
    "texmf-dist/doc/latex/lion-msc/lion-msc.pdf",
    "texmf-dist/doc/latex/lion-msc/minimal.pdf",
  },
  ["inlinedef"] = {
    "texmf-dist/doc/latex/inlinedef/README",
    "texmf-dist/doc/latex/inlinedef/inlinedef.pdf",
  },
  ["splitbib"] = {
    "texmf-dist/doc/latex/splitbib/README",
    "texmf-dist/doc/latex/splitbib/splitbib.pdf",
  },
  ["glossaries-portuges"] = {
    "texmf-dist/doc/latex/glossaries-portuges/README",
    "texmf-dist/doc/latex/glossaries-portuges/glossaries-portuges.pdf",
  },
  ["spie"] = {
    "texmf-dist/doc/latex/spie/README",
    "texmf-dist/doc/latex/spie/article.pdf",
  },
  ["sphack"] = {
    "texmf-dist/doc/latex/sphack/sphack-doc.pdf",
  },
  ["spelling"] = {
    "texmf-dist/doc/luatex/spelling/CHANGES",
    "texmf-dist/doc/luatex/spelling/LICENSE",
    "texmf-dist/doc/luatex/spelling/README",
    "texmf-dist/doc/luatex/spelling/spelling-doc.pdf",
  },
  ["moreenum"] = {
    "texmf-dist/doc/latex/moreenum/README",
    "texmf-dist/doc/latex/moreenum/moreenum-doc.pdf",
  },
  ["latexcheat-ptbr"] = {
    "texmf-dist/doc/latex/latexcheat-ptbr/README-ptbr",
    "texmf-dist/doc/latex/latexcheat-ptbr/latexsheet-ptbr.pdf",
  },
  ["pgf-spectra"] = {
    "texmf-dist/doc/latex/pgf-spectra/README",
    "texmf-dist/doc/latex/pgf-spectra/pgf-spectraManual.pdf",
  },
  ["spalign"] = {
    "texmf-dist/doc/latex/spalign/README.md",
    "texmf-dist/doc/latex/spalign/spalign.pdf",
  },
  ["pb-diagram"] = {
    "texmf-dist/doc/latex/pb-diagram/COPYING",
    "texmf-dist/doc/latex/pb-diagram/README",
    "texmf-dist/doc/latex/pb-diagram/pb-manual.pdf",
  },
  ["sourceserifpro"] = {
    "texmf-dist/doc/latex/sourceserifpro/LICENSE.txt",
    "texmf-dist/doc/latex/sourceserifpro/sourceserifpro-otf-specimen.pdf",
    "texmf-dist/doc/latex/sourceserifpro/sourceserifpro-type1-specimen.pdf",
    "texmf-dist/doc/latex/sourceserifpro/sourceserifpro.pdf",
  },
  ["sourcesanspro"] = {
    "texmf-dist/doc/latex/sourcesanspro/LICENSE.txt",
    "texmf-dist/doc/latex/sourcesanspro/LPPL.txt",
    "texmf-dist/doc/latex/sourcesanspro/sourcesanspro-otf-specimen.pdf",
    "texmf-dist/doc/latex/sourcesanspro/sourcesanspro-type1-specimen.pdf",
    "texmf-dist/doc/latex/sourcesanspro/sourcesanspro.pdf",
  },
  ["soup"] = {
    "texmf-dist/doc/latex/soup/README.md",
    "texmf-dist/doc/latex/soup/soup.pdf",
  },
  ["biblatex-multiple-dm"] = {
    "texmf-dist/doc/latex/biblatex-multiple-dm/README",
    "texmf-dist/doc/latex/biblatex-multiple-dm/biblatex-multiple-dm.pdf",
    "texmf-dist/doc/latex/biblatex-multiple-dm/latexmkrc",
    "texmf-dist/doc/latex/biblatex-multiple-dm/makefile",
  },
  ["bytefield"] = {
    "texmf-dist/doc/latex/bytefield/README",
    "texmf-dist/doc/latex/bytefield/bf-example.pdf",
    "texmf-dist/doc/latex/bytefield/bytefield.pdf",
  },
  ["soton"] = {
    "texmf-dist/doc/latex/soton/README",
  },
  ["sort-by-letters"] = {
    "texmf-dist/doc/bibtex/sort-by-letters/README",
  },
  ["expressg"] = {
    "texmf-dist/doc/metapost/expressg/README",
    "texmf-dist/doc/metapost/expressg/aamfigs.pdf",
    "texmf-dist/doc/metapost/expressg/expeg.pdf",
    "texmf-dist/doc/metapost/expressg/expressg.pdf",
  },
  ["ethiop"] = {
    "texmf-dist/doc/latex/ethiop/MANIFEST",
    "texmf-dist/doc/latex/ethiop/README",
    "texmf-dist/doc/latex/ethiop/ethiodoc.pdf",
  },
  ["turkmen"] = {
    "texmf-dist/doc/latex/turkmen/README",
    "texmf-dist/doc/latex/turkmen/turkmen.pdf",
  },
  ["songbook"] = {
    "texmf-dist/doc/latex/songbook/LesserGPL.txt",
    "texmf-dist/doc/latex/songbook/README",
    "texmf-dist/doc/latex/songbook/contrib/CarolBook/CarolBookOH.pdf",
    "texmf-dist/doc/latex/songbook/contrib/CarolBook/CarolBookWB.pdf",
    "texmf-dist/doc/latex/songbook/contrib/README",
    "texmf-dist/doc/latex/songbook/contrib/crd2sb/crd2sb",
    "texmf-dist/doc/latex/songbook/contrib/crd2sb/crd2sb.txt",
    "texmf-dist/doc/latex/songbook/contrib/modulate",
    "texmf-dist/doc/latex/songbook/install.txt",
    "texmf-dist/doc/latex/songbook/mksbadx",
    "texmf-dist/doc/latex/songbook/mksbkdx",
    "texmf-dist/doc/latex/songbook/mksbtdx",
    "texmf-dist/doc/latex/songbook/sampleAdx.pdf",
    "texmf-dist/doc/latex/songbook/sampleCBK.pdf",
    "texmf-dist/doc/latex/songbook/sampleCSBK.pdf",
    "texmf-dist/doc/latex/songbook/sampleKdx.pdf",
    "texmf-dist/doc/latex/songbook/sampleOH.pdf",
    "texmf-dist/doc/latex/songbook/sampleTdx.pdf",
    "texmf-dist/doc/latex/songbook/sampleToc.pdf",
    "texmf-dist/doc/latex/songbook/sampleWBK.pdf",
    "texmf-dist/doc/latex/songbook/songbook.pdf",
  },
  ["snapshot"] = {
    "texmf-dist/doc/latex/snapshot/snapshot.pdf",
  },
  ["smartref"] = {
    "texmf-dist/doc/latex/smartref/README",
    "texmf-dist/doc/latex/smartref/smartref-doc.pdf",
  },
  ["microtype-de"] = {
    "texmf-dist/doc/latex/microtype-de/microtype-DE.pdf",
  },
  ["grafcet"] = {
    "texmf-dist/doc/latex/grafcet/README",
    "texmf-dist/doc/latex/grafcet/grafcet.pdf",
  },
  ["fancytooltips"] = {
    "texmf-dist/doc/latex/fancytooltips/examples/fancy-preview-demo.pdf",
    "texmf-dist/doc/latex/fancytooltips/examples/fancy-preview-demo2.pdf",
    "texmf-dist/doc/latex/fancytooltips/examples/fancytooltips-example.pdf",
    "texmf-dist/doc/latex/fancytooltips/examples/pics/tecna2.pdf",
    "texmf-dist/doc/latex/fancytooltips/examples/pics/tooltipy.pdf",
    "texmf-dist/doc/latex/fancytooltips/examples/readme",
    "texmf-dist/doc/latex/fancytooltips/fancy-preview",
    "texmf-dist/doc/latex/fancytooltips/fancytipmark1.pdf",
    "texmf-dist/doc/latex/fancytooltips/fancytipmark2.pdf",
    "texmf-dist/doc/latex/fancytooltips/fancytipmark3.pdf",
    "texmf-dist/doc/latex/fancytooltips/fancytipmark4.pdf",
    "texmf-dist/doc/latex/fancytooltips/fancytooltips.pdf",
    "texmf-dist/doc/latex/fancytooltips/readme",
    "texmf-dist/doc/latex/fancytooltips/tip.pdf",
  },
  ["slantsc"] = {
    "texmf-dist/doc/latex/slantsc/ChangeLog",
    "texmf-dist/doc/latex/slantsc/Makefile",
    "texmf-dist/doc/latex/slantsc/README",
    "texmf-dist/doc/latex/slantsc/slantsc.pdf",
  },
  ["skrapport"] = {
    "texmf-dist/doc/latex/skrapport/README",
    "texmf-dist/doc/latex/skrapport/skrapport.pdf",
  },
  ["cellspace"] = {
    "texmf-dist/doc/latex/cellspace/README",
    "texmf-dist/doc/latex/cellspace/cellspace.pdf",
  },
  ["pst-cie"] = {
    "texmf-dist/doc/generic/pst-cie/Changes",
    "texmf-dist/doc/generic/pst-cie/README.md",
    "texmf-dist/doc/generic/pst-cie/pst-cie-doc.pdf",
  },
  ["skmath"] = {
    "texmf-dist/doc/latex/skmath/README",
    "texmf-dist/doc/latex/skmath/skmath.pdf",
  },
  ["randbild"] = {
    "texmf-dist/doc/latex/randbild/README",
    "texmf-dist/doc/latex/randbild/randbild.pdf",
  },
  ["fullwidth"] = {
    "texmf-dist/doc/latex/fullwidth/README",
    "texmf-dist/doc/latex/fullwidth/fullwidth-test.pdf",
    "texmf-dist/doc/latex/fullwidth/fullwidth.pdf",
  },
  ["mflogo"] = {
    "texmf-dist/doc/latex/mflogo/CATALOGUE",
    "texmf-dist/doc/latex/mflogo/Makefile",
    "texmf-dist/doc/latex/mflogo/README",
    "texmf-dist/doc/latex/mflogo/mflogo.pdf",
  },
  ["axessibility"] = {
    "texmf-dist/doc/latex/axessibility/README",
    "texmf-dist/doc/latex/axessibility/axessibility.pdf",
  },
  ["skak"] = {
    "texmf-dist/doc/latex/skak/LICENSE",
    "texmf-dist/doc/latex/skak/README",
    "texmf-dist/doc/latex/skak/THANKS",
    "texmf-dist/doc/latex/skak/informator.pdf",
    "texmf-dist/doc/latex/skak/refman.pdf",
    "texmf-dist/doc/latex/skak/skakdoc.pdf",
    "texmf-dist/doc/latex/skak/tuggame.pdf",
  },
  ["esieecv"] = {
    "texmf-dist/doc/latex/ESIEEcv/ESIEEcv.pdf",
    "texmf-dist/doc/latex/ESIEEcv/cvtest.pdf",
  },
  ["simplified-latex"] = {
    "texmf-dist/doc/latex/simplified-latex/README",
    "texmf-dist/doc/latex/simplified-latex/simplified-intro.pdf",
  },
  ["dinbrief"] = {
    "texmf-dist/doc/latex/dinbrief/dinbrief.pdf",
    "texmf-dist/doc/latex/dinbrief/liesmich",
    "texmf-dist/doc/latex/dinbrief/readme",
  },
  ["elteikthesis"] = {
    "texmf-dist/doc/latex/elteikthesis/README",
    "texmf-dist/doc/latex/elteikthesis/elteikthesis.pdf",
    "texmf-dist/doc/latex/elteikthesis/example.pdf",
  },
  ["simpler-wick"] = {
    "texmf-dist/doc/latex/simpler-wick/LICENSE",
    "texmf-dist/doc/latex/simpler-wick/README.md",
    "texmf-dist/doc/latex/simpler-wick/simpler-wick.pdf",
  },
  ["simplekv"] = {
    "texmf-dist/doc/generic/simplekv/README",
    "texmf-dist/doc/generic/simplekv/simplekv-fr.pdf",
  },
  ["simpleinvoice"] = {
    "texmf-dist/doc/latex/simpleinvoice/LICENSE",
    "texmf-dist/doc/latex/simpleinvoice/README.md",
    "texmf-dist/doc/latex/simpleinvoice/doc/examples/english.pdf",
    "texmf-dist/doc/latex/simpleinvoice/doc/examples/norwegian.pdf",
    "texmf-dist/doc/latex/simpleinvoice/doc/simpleinvoice.pdf",
  },
  ["fnspe"] = {
    "texmf-dist/doc/latex/fnspe/README",
    "texmf-dist/doc/latex/fnspe/fnspe.pdf",
  },
  ["phfparen"] = {
    "texmf-dist/doc/latex/phfparen/Makefile",
    "texmf-dist/doc/latex/phfparen/README.md",
    "texmf-dist/doc/latex/phfparen/phfparen.pdf",
  },
  ["svn-prov"] = {
    "texmf-dist/doc/latex/svn-prov/svn-prov.pdf",
  },
  ["simple-thesis-dissertation"] = {
    "texmf-dist/doc/xelatex/simple-thesis-dissertation/Figures/Figure-SampleVectorGraphic.pdf",
    "texmf-dist/doc/xelatex/simple-thesis-dissertation/LICENSE",
    "texmf-dist/doc/xelatex/simple-thesis-dissertation/README.md",
    "texmf-dist/doc/xelatex/simple-thesis-dissertation/Thesis.pdf",
  },
  ["simple-resume-cv"] = {
    "texmf-dist/doc/xelatex/simple-resume-cv/CV.pdf",
    "texmf-dist/doc/xelatex/simple-resume-cv/LICENSE",
    "texmf-dist/doc/xelatex/simple-resume-cv/README.md",
  },
  ["sidenotes"] = {
    "texmf-dist/doc/latex/sidenotes/README",
    "texmf-dist/doc/latex/sidenotes/caesar_example.pdf",
    "texmf-dist/doc/latex/sidenotes/sidenotes.pdf",
  },
  ["sidecap"] = {
    "texmf-dist/doc/latex/sidecap/README",
    "texmf-dist/doc/latex/sidecap/sidecap.pdf",
  },
  ["showlabels"] = {
    "texmf-dist/doc/latex/showlabels/README",
    "texmf-dist/doc/latex/showlabels/VERSION",
    "texmf-dist/doc/latex/showlabels/lppl.txt",
    "texmf-dist/doc/latex/showlabels/showlabels.html",
    "texmf-dist/doc/latex/showlabels/showlabels.pdf",
  },
  ["ktv-texdata"] = {
    "texmf-dist/doc/latex/ktv-texdata/README",
    "texmf-dist/doc/latex/ktv-texdata/README.txt.doc",
    "texmf-dist/doc/latex/ktv-texdata/ktv-texdata.pdf",
  },
  ["countriesofeurope"] = {
    "texmf-dist/doc/fonts/countriesofeurope/CountriesOfEurope.pdf",
    "texmf-dist/doc/fonts/countriesofeurope/README",
  },
  ["fixjfm"] = {
    "texmf-dist/doc/generic/fixjfm/README",
    "texmf-dist/doc/generic/fixjfm/fixjfm-doc.pdf",
  },
  ["biblatex-ijsra"] = {
    "texmf-dist/doc/latex/biblatex-ijsra/README.md",
    "texmf-dist/doc/latex/biblatex-ijsra/biblatex-ijsra.pdf",
  },
  ["mathtools"] = {
    "texmf-dist/doc/latex/mathtools/README",
    "texmf-dist/doc/latex/mathtools/empheq.pdf",
    "texmf-dist/doc/latex/mathtools/mathtools.pdf",
    "texmf-dist/doc/latex/mathtools/mhsetup.pdf",
  },
  ["shobhika"] = {
    "texmf-dist/doc/fonts/shobhika/AUTHORS.txt",
    "texmf-dist/doc/fonts/shobhika/CONTRIBUTORS.txt",
    "texmf-dist/doc/fonts/shobhika/Copyright.txt",
    "texmf-dist/doc/fonts/shobhika/OFL.txt",
    "texmf-dist/doc/fonts/shobhika/README.md",
    "texmf-dist/doc/fonts/shobhika/shobhika.pdf",
  },
  ["shipunov"] = {
    "texmf-dist/doc/latex/shipunov/NEWS",
    "texmf-dist/doc/latex/shipunov/README",
    "texmf-dist/doc/latex/shipunov/altverse-ex-en.pdf",
    "texmf-dist/doc/latex/shipunov/altverse-ex1-ru.pdf",
    "texmf-dist/doc/latex/shipunov/altverse-ex2-ru.pdf",
    "texmf-dist/doc/latex/shipunov/autolist-ex-en.pdf",
    "texmf-dist/doc/latex/shipunov/autolist-ex-ru.pdf",
    "texmf-dist/doc/latex/shipunov/biokey-doc-en.pdf",
    "texmf-dist/doc/latex/shipunov/biokey2html-doc-en.pdf",
    "texmf-dist/doc/latex/shipunov/biokey2html-ex-en.html",
    "texmf-dist/doc/latex/shipunov/biokey2html-ex-en1.pdf",
    "texmf-dist/doc/latex/shipunov/biokey2html-ex-en2.pdf",
    "texmf-dist/doc/latex/shipunov/biolist-ex-en.pdf",
    "texmf-dist/doc/latex/shipunov/boldline-ex-en.pdf",
    "texmf-dist/doc/latex/shipunov/cassete-ex-ru.pdf",
    "texmf-dist/doc/latex/shipunov/classif2-ex-en.pdf",
    "texmf-dist/doc/latex/shipunov/drcaps-ex-en.pdf",
    "texmf-dist/doc/latex/shipunov/etiketka-ex-ru.pdf",
    "texmf-dist/doc/latex/shipunov/etiketka-ex1-en.pdf",
    "texmf-dist/doc/latex/shipunov/etiketka-ex2-en.pdf",
    "texmf-dist/doc/latex/shipunov/field-form-ex1-ru.pdf",
    "texmf-dist/doc/latex/shipunov/field-form-ex2-ru.pdf",
    "texmf-dist/doc/latex/shipunov/flower-ex-en-x.pdf",
    "texmf-dist/doc/latex/shipunov/flower-ex-en.pdf",
    "texmf-dist/doc/latex/shipunov/numerus-ex-ru.pdf",
    "texmf-dist/doc/latex/shipunov/punct-ex-en.pdf",
    "texmf-dist/doc/latex/shipunov/rusnat-doc-ru.pdf",
    "texmf-dist/doc/latex/shipunov/rusnat-ex1-ru.pdf",
    "texmf-dist/doc/latex/shipunov/rusnat-ex2-ru.pdf",
    "texmf-dist/doc/latex/shipunov/sltables-doc-en.pdf",
    "texmf-dist/doc/latex/shipunov/starfn-ex-ru.pdf",
  },
  ["shapes"] = {
    "texmf-dist/doc/metapost/shapes/CHANGES",
    "texmf-dist/doc/metapost/shapes/README",
    "texmf-dist/doc/metapost/shapes/lppl.txt",
    "texmf-dist/doc/metapost/shapes/shapes.pdf",
  },
  ["shapepar"] = {
    "texmf-dist/doc/generic/shapepar/README.shapepar",
    "texmf-dist/doc/generic/shapepar/shapepar.pdf",
  },
  ["shadowtext"] = {
    "texmf-dist/doc/latex/shadowtext/README",
    "texmf-dist/doc/latex/shadowtext/shadowtext.pdf",
  },
  ["dynamicnumber"] = {
    "texmf-dist/doc/latex/dynamicnumber/LICENSE.txt",
    "texmf-dist/doc/latex/dynamicnumber/dynamicnumber.pdf",
  },
  ["ladder"] = {
    "texmf-dist/doc/latex/ladder/README.md",
    "texmf-dist/doc/latex/ladder/ladder.pdf",
  },
  ["gillius"] = {
    "texmf-dist/doc/fonts/gillius/COPYING",
    "texmf-dist/doc/fonts/gillius/Gillius-cat.pdf",
    "texmf-dist/doc/fonts/gillius/README",
    "texmf-dist/doc/fonts/gillius/gillius-samples.pdf",
    "texmf-dist/doc/fonts/gillius/gillius2-samples.pdf",
  },
  ["mweights"] = {
    "texmf-dist/doc/latex/mweights/README",
  },
  ["abstyles"] = {
    "texmf-dist/doc/bibtex/abstyles/README",
    "texmf-dist/doc/bibtex/abstyles/abstdok.pdf",
  },
  ["shade"] = {
    "texmf-dist/doc/generic/shade/README",
    "texmf-dist/doc/generic/shade/description.pdf",
  },
  ["sgame"] = {
    "texmf-dist/doc/latex/sgame/README",
    "texmf-dist/doc/latex/sgame/sgame.pdf",
  },
  ["assignment"] = {
    "texmf-dist/doc/latex/assignment/Changelog",
    "texmf-dist/doc/latex/assignment/LICENSE",
    "texmf-dist/doc/latex/assignment/README",
    "texmf-dist/doc/latex/assignment/assignment.pdf",
  },
  ["babel-german"] = {
    "texmf-dist/doc/generic/babel-german/README",
    "texmf-dist/doc/generic/babel-german/germanb.pdf",
    "texmf-dist/doc/generic/babel-german/ngermanb.pdf",
  },
  ["emulateapj"] = {
    "texmf-dist/doc/latex/emulateapj/README",
    "texmf-dist/doc/latex/emulateapj/sample.pdf",
  },
  ["pagecont"] = {
    "texmf-dist/doc/latex/pagecont/README",
    "texmf-dist/doc/latex/pagecont/pagecont.pdf",
  },
  ["sexam"] = {
    "texmf-dist/doc/xelatex/sexam/README.txt",
    "texmf-dist/doc/xelatex/sexam/bac_template-DZ.pdf",
    "texmf-dist/doc/xelatex/sexam/exam_with_sexam_ar-DZ.pdf",
    "texmf-dist/doc/xelatex/sexam/exam_with_wexam_ar-DZ.pdf",
    "texmf-dist/doc/xelatex/sexam/sexam_wexam_doc_ar.pdf",
  },
  ["seuthesix"] = {
    "texmf-dist/doc/latex/seuthesix/LICENCE",
    "texmf-dist/doc/latex/seuthesix/README",
    "texmf-dist/doc/latex/seuthesix/figures/doctor-hwzs.pdf",
    "texmf-dist/doc/latex/seuthesix/figures/master-hwzs.pdf",
    "texmf-dist/doc/latex/seuthesix/figures/seu-badge-logo.pdf",
    "texmf-dist/doc/latex/seuthesix/rules.pdf",
    "texmf-dist/doc/latex/seuthesix/sample_engineering.pdf",
    "texmf-dist/doc/latex/seuthesix/sample_masters.pdf",
    "texmf-dist/doc/latex/seuthesix/sample_phd.pdf",
    "texmf-dist/doc/latex/seuthesix/seuthesix.pdf",
  },
  ["setspace"] = {
    "texmf-dist/doc/latex/setspace/README",
  },
  ["everypage"] = {
    "texmf-dist/doc/latex/everypage/README",
    "texmf-dist/doc/latex/everypage/everypage.pdf",
  },
  ["ribbonproofs"] = {
    "texmf-dist/doc/latex/ribbonproofs/README",
    "texmf-dist/doc/latex/ribbonproofs/ribbonproofsmanual.pdf",
  },
  ["bengali"] = {
    "texmf-dist/doc/fonts/bengali/README",
    "texmf-dist/doc/fonts/bengali/bengdoc.pdf",
    "texmf-dist/doc/fonts/bengali/example.pdf",
    "texmf-dist/doc/fonts/bengali/manifest.txt",
  },
  ["cryptocode"] = {
    "texmf-dist/doc/latex/cryptocode/README",
    "texmf-dist/doc/latex/cryptocode/cryptocode.pdf",
  },
  ["pythonhighlight"] = {
    "texmf-dist/doc/latex/pythonhighlight/README.md",
  },
  ["egameps"] = {
    "texmf-dist/doc/latex/egameps/README",
    "texmf-dist/doc/latex/egameps/egameps.pdf",
  },
  ["yax"] = {
    "texmf-dist/doc/generic/yax/README",
    "texmf-dist/doc/generic/yax/yax-doc.pdf",
  },
  ["fnpara"] = {
    "texmf-dist/doc/latex/fnpara/fnpara-doc.pdf",
  },
  ["semproc"] = {
    "texmf-dist/doc/latex/semproc/README.txt",
    "texmf-dist/doc/latex/semproc/semproc.pdf",
  },
  ["semantic-markup"] = {
    "texmf-dist/doc/latex/semantic-markup/README",
    "texmf-dist/doc/latex/semantic-markup/semantic-markup.pdf",
  },
  ["fontbook"] = {
    "texmf-dist/doc/xelatex/fontbook/README",
    "texmf-dist/doc/xelatex/fontbook/fontbook-freefonts.pdf",
    "texmf-dist/doc/xelatex/fontbook/fontbook.pdf",
  },
  ["tabbing"] = {
    "texmf-dist/doc/latex/Tabbing/00readme",
    "texmf-dist/doc/latex/Tabbing/Tabbing.pdf",
  },
  ["selnolig"] = {
    "texmf-dist/doc/lualatex/selnolig/README",
    "texmf-dist/doc/lualatex/selnolig/selnolig-english-test.pdf",
    "texmf-dist/doc/lualatex/selnolig/selnolig-german-test.pdf",
    "texmf-dist/doc/lualatex/selnolig/selnolig.pdf",
  },
  ["selectp"] = {
    "texmf-dist/doc/latex/selectp/selectp-doc.pdf",
  },
  ["philosophersimprint"] = {
    "texmf-dist/doc/latex/philosophersimprint/Makefile",
    "texmf-dist/doc/latex/philosophersimprint/README",
    "texmf-dist/doc/latex/philosophersimprint/philosophersimprint.pdf",
    "texmf-dist/doc/latex/philosophersimprint/sample.pdf",
  },
  ["datetime2-norsk"] = {
    "texmf-dist/doc/latex/datetime2-norsk/README",
    "texmf-dist/doc/latex/datetime2-norsk/datetime2-norsk.pdf",
  },
  ["pst-tvz"] = {
    "texmf-dist/doc/generic/pst-tvz/Changes",
    "texmf-dist/doc/generic/pst-tvz/README",
    "texmf-dist/doc/generic/pst-tvz/pst-tvz-doc.pdf",
  },
  ["envlab"] = {
    "texmf-dist/doc/latex/envlab/elguide.pdf",
    "texmf-dist/doc/latex/envlab/envlab.pdf",
    "texmf-dist/doc/latex/envlab/readme.v12",
  },
  ["termlist"] = {
    "texmf-dist/doc/latex/termlist/README",
    "texmf-dist/doc/latex/termlist/termlist.pdf",
  },
  ["nanumtype1"] = {
    "texmf-dist/doc/fonts/nanumtype1/COPYING",
    "texmf-dist/doc/fonts/nanumtype1/README",
    "texmf-dist/doc/fonts/nanumtype1/nanum-sampler.pdf",
  },
  ["seealso"] = {
    "texmf-dist/doc/latex/seealso/README",
    "texmf-dist/doc/latex/seealso/seealso.pdf",
  },
  ["nih"] = {
    "texmf-dist/doc/latex/nih/README",
    "texmf-dist/doc/latex/nih/example-biosketch.pdf",
    "texmf-dist/doc/latex/nih/example-nih-cls.pdf",
  },
  ["sectsty"] = {
    "texmf-dist/doc/latex/sectsty/sectsty.pdf",
  },
  ["matc3"] = {
    "texmf-dist/doc/latex/matc3/Makefile",
    "texmf-dist/doc/latex/matc3/README",
    "texmf-dist/doc/latex/matc3/matc3.pdf",
  },
  ["section"] = {
    "texmf-dist/doc/latex/section/section-doc.pdf",
  },
  ["sansmathfonts"] = {
    "texmf-dist/doc/fonts/sansmathfonts/README",
    "texmf-dist/doc/fonts/sansmathfonts/sansmathfonts.pdf",
  },
  ["sdrt"] = {
    "texmf-dist/doc/latex/sdrt/README",
    "texmf-dist/doc/latex/sdrt/sdrt-doc.pdf",
  },
  ["refenums"] = {
    "texmf-dist/doc/latex/refenums/LICENSE",
    "texmf-dist/doc/latex/refenums/README.md",
    "texmf-dist/doc/latex/refenums/README.pdf",
    "texmf-dist/doc/latex/refenums/demo.pdf",
  },
  ["scsnowman"] = {
    "texmf-dist/doc/latex/scsnowman/LICENSE",
    "texmf-dist/doc/latex/scsnowman/Makefile",
    "texmf-dist/doc/latex/scsnowman/README.md",
    "texmf-dist/doc/latex/scsnowman/scsnowman-sample.pdf",
    "texmf-dist/doc/latex/scsnowman/scsnowman-zrtest.pdf",
    "texmf-dist/doc/latex/scsnowman/scsnowman.pdf",
  },
  ["extract"] = {
    "texmf-dist/doc/latex/extract/README",
    "texmf-dist/doc/latex/extract/extract.pdf",
  },
  ["scrjrnl"] = {
    "texmf-dist/doc/latex/scrjrnl/README",
    "texmf-dist/doc/latex/scrjrnl/example.pdf",
    "texmf-dist/doc/latex/scrjrnl/scrjrnl.pdf",
  },
  ["screenplay"] = {
    "texmf-dist/doc/latex/screenplay/COPYING",
    "texmf-dist/doc/latex/screenplay/README",
    "texmf-dist/doc/latex/screenplay/screenplay.pdf",
    "texmf-dist/doc/latex/screenplay/test.pdf",
  },
  ["tlc-article"] = {
    "texmf-dist/doc/latex/tlc-article/LICENSE",
    "texmf-dist/doc/latex/tlc-article/README.md",
  },
  ["scratch"] = {
    "texmf-dist/doc/latex/scratch/README",
    "texmf-dist/doc/latex/scratch/scratch-fr.pdf",
  },
  ["sclang-prettifier"] = {
    "texmf-dist/doc/latex/sclang-prettifier/README",
    "texmf-dist/doc/latex/sclang-prettifier/sclang-prettifier.pdf",
  },
  ["sciposter"] = {
    "texmf-dist/doc/latex/sciposter/README",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocks1.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocks1a.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocks1mx.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocks1vx.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocks2.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocks2mx.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocks3.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocks3mx.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocks3op.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocks3rec.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocks3vx.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocksopen3a.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocksopen3vx.pdf",
    "texmf-dist/doc/latex/sciposter/sciposterexample/blocksopen3vy.pdf",
    "texmf-dist/doc/latex/sciposter/scipostermanual.pdf",
  },
  ["aastex"] = {
    "texmf-dist/doc/latex/aastex/CSS081007-eps-converted-to.pdf",
    "texmf-dist/doc/latex/aastex/KT_Eri-eps-converted-to.pdf",
    "texmf-dist/doc/latex/aastex/LMC_2009-eps-converted-to.pdf",
    "texmf-dist/doc/latex/aastex/README.md",
    "texmf-dist/doc/latex/aastex/RS_Oph-eps-converted-to.pdf",
    "texmf-dist/doc/latex/aastex/U_Sco-eps-converted-to.pdf",
    "texmf-dist/doc/latex/aastex/V2491_Cyg-eps-converted-to.pdf",
    "texmf-dist/doc/latex/aastex/aassymbols.pdf",
    "texmf-dist/doc/latex/aastex/cost-eps-converted-to.pdf",
    "texmf-dist/doc/latex/aastex/sample62.pdf",
    "texmf-dist/doc/latex/aastex/video3-eps-converted-to.pdf",
  },
  ["schulschriften"] = {
    "texmf-dist/doc/fonts/schulschriften/README",
    "texmf-dist/doc/fonts/schulschriften/schulschriften.pdf",
  },
  ["multibib"] = {
    "texmf-dist/doc/latex/multibib/README",
    "texmf-dist/doc/latex/multibib/bibtexall",
    "texmf-dist/doc/latex/multibib/multibib.pdf",
  },
  ["asymptote-manual-zh-cn"] = {
    "texmf-dist/doc/support/asymptote-manual-zh-cn/README",
    "texmf-dist/doc/support/asymptote-manual-zh-cn/asymptote-manual-zh-cn.pdf",
    "texmf-dist/doc/support/asymptote-manual-zh-cn/src/cleantmp",
    "texmf-dist/doc/support/asymptote-manual-zh-cn/src/makepdf",
  },
  ["asciilist"] = {
    "texmf-dist/doc/latex/asciilist/README.md",
    "texmf-dist/doc/latex/asciilist/asciilist.pdf",
  },
  ["gastex"] = {
    "texmf-dist/doc/latex/gastex/README",
  },
  ["savefnmark"] = {
    "texmf-dist/doc/latex/savefnmark/savefnmark.pdf",
  },
  ["tikz-layers"] = {
    "texmf-dist/doc/latex/tikz-layers/README",
    "texmf-dist/doc/latex/tikz-layers/README.TEXLIVE",
    "texmf-dist/doc/latex/tikz-layers/manifest.txt",
  },
  ["secdot"] = {
    "texmf-dist/doc/latex/secdot/secdot.pdf",
  },
  ["ryethesis"] = {
    "texmf-dist/doc/latex/ryethesis/Makefile",
    "texmf-dist/doc/latex/ryethesis/README",
    "texmf-dist/doc/latex/ryethesis/figure1.pdf",
    "texmf-dist/doc/latex/ryethesis/ryesample.pdf",
    "texmf-dist/doc/latex/ryethesis/ryethesis.pdf",
  },
  ["nicefilelist"] = {
    "texmf-dist/doc/latex/nicefilelist/README",
    "texmf-dist/doc/latex/nicefilelist/SrcFILEs.txt",
    "texmf-dist/doc/latex/nicefilelist/nicefilelist.pdf",
  },
  ["sansmathaccent"] = {
    "texmf-dist/doc/fonts/sansmathaccent/README",
    "texmf-dist/doc/fonts/sansmathaccent/sansmathaccent.pdf",
  },
  ["sanskrit-t1"] = {
    "texmf-dist/doc/fonts/sanskrit-t1/README",
    "texmf-dist/doc/fonts/sanskrit-t1/sktdoc.pdf",
  },
  ["parallel"] = {
    "texmf-dist/doc/latex/parallel/parallel.pdf",
    "texmf-dist/doc/latex/parallel/readme",
  },
  ["present"] = {
    "texmf-dist/doc/plain/present/HowToTalkTeX.pdf",
    "texmf-dist/doc/plain/present/Pfeil1.pdf",
    "texmf-dist/doc/plain/present/Pfeil2.pdf",
    "texmf-dist/doc/plain/present/Pfeil3.pdf",
    "texmf-dist/doc/plain/present/Pfeil4.pdf",
    "texmf-dist/doc/plain/present/README",
    "texmf-dist/doc/plain/present/Sagnac.pdf",
  },
  ["scale"] = {
    "texmf-dist/doc/latex/scale/COPYING",
    "texmf-dist/doc/latex/scale/README",
  },
  ["rvwrite"] = {
    "texmf-dist/doc/latex/rvwrite/Makefile",
    "texmf-dist/doc/latex/rvwrite/README",
    "texmf-dist/doc/latex/rvwrite/rvwrite-doc.pdf",
  },
  ["emp"] = {
    "texmf-dist/doc/latex/emp/COPYING",
    "texmf-dist/doc/latex/emp/Makefile",
    "texmf-dist/doc/latex/emp/README",
    "texmf-dist/doc/latex/emp/emp.pdf",
    "texmf-dist/doc/latex/emp/empman.pdf",
  },
  ["tapir"] = {
    "texmf-dist/doc/fonts/tapir/readme",
    "texmf-dist/doc/fonts/tapir/readme.pdf",
  },
  ["apxproof"] = {
    "texmf-dist/doc/latex/apxproof/LICENSE",
    "texmf-dist/doc/latex/apxproof/Makefile",
    "texmf-dist/doc/latex/apxproof/README.md",
    "texmf-dist/doc/latex/apxproof/apxproof.pdf",
  },
  ["russ"] = {
    "texmf-dist/doc/latex/russ/README",
    "texmf-dist/doc/latex/russ/readme.RU.txt",
    "texmf-dist/doc/latex/russ/russ_doc.pdf",
  },
  ["rulercompass"] = {
    "texmf-dist/doc/latex/rulercompass/README.txt",
    "texmf-dist/doc/latex/rulercompass/rulercompass.pdf",
    "texmf-dist/doc/latex/rulercompass/rulercompass_doc.pdf",
  },
  ["program"] = {
    "texmf-dist/doc/latex/program/README",
    "texmf-dist/doc/latex/program/gpl-3.0.txt",
    "texmf-dist/doc/latex/program/program-doc.pdf",
  },
  ["dvipng"] = {
    "texmf-dist/doc/dvipng/dvipng.html",
    "texmf-dist/doc/dvipng/dvipng.pdf",
    "texmf-dist/doc/man/man1/dvigif.man1.pdf",
    "texmf-dist/doc/man/man1/dvipng.man1.pdf",
  },
  ["pst-geo"] = {
    "texmf-dist/doc/generic/pst-geo/Changes",
    "texmf-dist/doc/generic/pst-geo/README",
    "texmf-dist/doc/generic/pst-geo/pst-geo-doc.pdf",
  },
  ["clearsans"] = {
    "texmf-dist/doc/fonts/clearsans/LICENSE-2.0.txt",
    "texmf-dist/doc/fonts/clearsans/README",
    "texmf-dist/doc/fonts/clearsans/clear-samples.pdf",
  },
  ["jslectureplanner"] = {
    "texmf-dist/doc/latex/jslectureplanner/README",
    "texmf-dist/doc/latex/jslectureplanner/examples/English/lecturebib-english.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/English/lecturehandout-english.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/English/lecturepresentation-english.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/English/lectureprogram-english.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/English/lecturescript-english.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/English/membertable-blank-english.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/English/membertable-filled-english.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/English/presencetable-blank-english.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/English/presencetable-filled-english.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/English/presentation-list-english.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/German/lecturebib.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/German/lecturehandout.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/German/lecturepresentation.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/German/lectureprogram.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/German/lecturescript.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/German/membertable-blank.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/German/membertable-filled.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/German/presencetable-blank.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/German/presencetable-filled.pdf",
    "texmf-dist/doc/latex/jslectureplanner/examples/German/presentation-list.pdf",
    "texmf-dist/doc/latex/jslectureplanner/jslectureplanner.pdf",
  },
  ["context-rst"] = {
    "texmf-dist/doc/context/third/rst/OMakefile",
    "texmf-dist/doc/context/third/rst/VERSION",
    "texmf-dist/doc/context/third/rst/manual.pdf",
  },
  ["disser"] = {
    "texmf-dist/doc/latex/disser/ChangeLog",
    "texmf-dist/doc/latex/disser/Makefile",
    "texmf-dist/doc/latex/disser/README.md",
    "texmf-dist/doc/latex/disser/templates-utf8/Makefile",
    "texmf-dist/doc/latex/disser/templates-utf8/bachelor/Makefile",
    "texmf-dist/doc/latex/disser/templates-utf8/bachelor/fig/Makefile",
    "texmf-dist/doc/latex/disser/templates-utf8/candidate/Makefile",
    "texmf-dist/doc/latex/disser/templates-utf8/candidate/fig/Makefile",
    "texmf-dist/doc/latex/disser/templates-utf8/doctor/Makefile",
    "texmf-dist/doc/latex/disser/templates-utf8/doctor/fig/Makefile",
    "texmf-dist/doc/latex/disser/templates-utf8/master/Makefile",
    "texmf-dist/doc/latex/disser/templates-utf8/master/fig/Makefile",
    "texmf-dist/doc/latex/disser/templates-utf8/specialist/Makefile",
    "texmf-dist/doc/latex/disser/templates-utf8/specialist/fig/Makefile",
    "texmf-dist/doc/latex/disser/templates/Makefile",
    "texmf-dist/doc/latex/disser/templates/bachelor/Makefile",
    "texmf-dist/doc/latex/disser/templates/bachelor/fig/Makefile",
    "texmf-dist/doc/latex/disser/templates/candidate/Makefile",
    "texmf-dist/doc/latex/disser/templates/candidate/fig/Makefile",
    "texmf-dist/doc/latex/disser/templates/doctor/Makefile",
    "texmf-dist/doc/latex/disser/templates/doctor/fig/Makefile",
    "texmf-dist/doc/latex/disser/templates/master/Makefile",
    "texmf-dist/doc/latex/disser/templates/master/fig/Makefile",
    "texmf-dist/doc/latex/disser/templates/specialist/Makefile",
    "texmf-dist/doc/latex/disser/templates/specialist/fig/Makefile",
  },
  ["musixtex-fonts"] = {
    "texmf-dist/doc/fonts/musixtex-fonts/README",
    "texmf-dist/doc/fonts/musixtex-fonts/README.psfonts",
    "texmf-dist/doc/fonts/musixtex-fonts/musixtex-fonts-install.pdf",
  },
  ["rubik"] = {
    "texmf-dist/doc/latex/rubik/README.txt",
    "texmf-dist/doc/latex/rubik/rubik-doc-figA.pdf",
    "texmf-dist/doc/latex/rubik/rubik-doc-figB.pdf",
    "texmf-dist/doc/latex/rubik/rubik-doc-figC.pdf",
    "texmf-dist/doc/latex/rubik/rubik-doc-figD.pdf",
    "texmf-dist/doc/latex/rubik/rubik-doc-figE.pdf",
    "texmf-dist/doc/latex/rubik/rubik-doc-figF.pdf",
    "texmf-dist/doc/latex/rubik/rubikcube.pdf",
    "texmf-dist/doc/latex/rubik/rubikexamples.pdf",
    "texmf-dist/doc/latex/rubik/rubikpatterns-doc-figA.pdf",
    "texmf-dist/doc/latex/rubik/rubikpatterns.pdf",
    "texmf-dist/doc/latex/rubik/rubikpatternsLIST.pdf",
    "texmf-dist/doc/latex/rubik/rubikrot-doc-figA.pdf",
    "texmf-dist/doc/latex/rubik/rubikrot-doc-figB.pdf",
    "texmf-dist/doc/latex/rubik/rubikrot-doc-figC.pdf",
    "texmf-dist/doc/latex/rubik/rubikrot-doc-figD.pdf",
    "texmf-dist/doc/latex/rubik/rubikrotation.pdf",
    "texmf-dist/doc/latex/rubik/rubikrotationPL.pdf",
    "texmf-dist/doc/latex/rubik/rubiktwo-doc-figA.pdf",
    "texmf-dist/doc/latex/rubik/rubiktwocube.pdf",
    "texmf-dist/doc/man/man1/rubikrotation.man1.pdf",
  },
  ["xcharter"] = {
    "texmf-dist/doc/fonts/xcharter/README",
    "texmf-dist/doc/fonts/xcharter/xcharter-doc.pdf",
  },
  ["montserrat"] = {
    "texmf-dist/doc/fonts/montserrat/FONTLOG.txt",
    "texmf-dist/doc/fonts/montserrat/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/montserrat/OFL.txt",
    "texmf-dist/doc/fonts/montserrat/README",
    "texmf-dist/doc/fonts/montserrat/montserrat-doc.pdf",
  },
  ["phfnote"] = {
    "texmf-dist/doc/latex/phfnote/Makefile",
    "texmf-dist/doc/latex/phfnote/README.md",
    "texmf-dist/doc/latex/phfnote/phfnote.pdf",
  },
  ["functan"] = {
    "texmf-dist/doc/latex/functan/README",
    "texmf-dist/doc/latex/functan/functan.pdf",
  },
  ["rsc"] = {
    "texmf-dist/doc/latex/rsc/README.md",
    "texmf-dist/doc/latex/rsc/rsc.pdf",
  },
  ["rrgtrees"] = {
    "texmf-dist/doc/latex/rrgtrees/Makefile",
    "texmf-dist/doc/latex/rrgtrees/README",
    "texmf-dist/doc/latex/rrgtrees/rrgtrees.pdf",
  },
  ["roundbox"] = {
    "texmf-dist/doc/latex/roundbox/README",
  },
  ["lcyw"] = {
    "texmf-dist/doc/latex/lcyw/README",
    "texmf-dist/doc/latex/lcyw/README.koi8-r",
    "texmf-dist/doc/latex/lcyw/lcyw.pdf",
  },
  ["mp3d"] = {
    "texmf-dist/doc/metapost/mp3d/README",
    "texmf-dist/doc/metapost/mp3d/README.doc",
    "texmf-dist/doc/metapost/mp3d/gut2001.pdf",
    "texmf-dist/doc/metapost/mp3d/paper1997corrected.pdf",
    "texmf-dist/doc/metapost/mp3d/tb57roeg.pdf",
    "texmf-dist/doc/metapost/mp3d/tugboat-geometry-space.pdf",
  },
  ["bibtex8"] = {
    "texmf-dist/doc/bibtex8/00readme.txt",
    "texmf-dist/doc/bibtex8/HISTORY",
    "texmf-dist/doc/bibtex8/csfile.txt",
  },
  ["rec-thy"] = {
    "texmf-dist/doc/latex/rec-thy/README",
    "texmf-dist/doc/latex/rec-thy/rec-thy.pdf",
  },
  ["pecha"] = {
    "texmf-dist/doc/latex/pecha/CHANGES",
    "texmf-dist/doc/latex/pecha/COPYING",
    "texmf-dist/doc/latex/pecha/README",
    "texmf-dist/doc/latex/pecha/example.pdf",
    "texmf-dist/doc/latex/pecha/pecha_docu.pdf",
  },
  ["undolabl"] = {
    "texmf-dist/doc/latex/undolabl/README",
    "texmf-dist/doc/latex/undolabl/undolabl-example.pdf",
    "texmf-dist/doc/latex/undolabl/undolabl.pdf",
  },
  ["romanbarpagenumber"] = {
    "texmf-dist/doc/latex/romanbarpagenumber/README",
    "texmf-dist/doc/latex/romanbarpagenumber/romanbarpagenumber.pdf",
  },
  ["romanbar"] = {
    "texmf-dist/doc/latex/romanbar/README",
    "texmf-dist/doc/latex/romanbar/romanbar-example.pdf",
    "texmf-dist/doc/latex/romanbar/romanbar.pdf",
  },
  ["roboto"] = {
    "texmf-dist/doc/fonts/roboto/COPYRIGHT.txt",
    "texmf-dist/doc/fonts/roboto/DESCRIPTION.en_us.html",
    "texmf-dist/doc/fonts/roboto/LICENSE.txt",
    "texmf-dist/doc/fonts/roboto/README",
    "texmf-dist/doc/fonts/roboto/RobotoSpecimenBook.pdf",
    "texmf-dist/doc/fonts/roboto/roboto-samples.pdf",
  },
  ["rmpage"] = {
    "texmf-dist/doc/latex/rmpage/readme",
    "texmf-dist/doc/latex/rmpage/rmpage-doc.pdf",
  },
  ["rmathbr"] = {
    "texmf-dist/doc/latex/rmathbr/README",
    "texmf-dist/doc/latex/rmathbr/rmathbr.pdf",
  },
  ["rjlparshap"] = {
    "texmf-dist/doc/latex/rjlparshap/README",
    "texmf-dist/doc/latex/rjlparshap/rjlpshap.pdf",
  },
  ["seqsplit"] = {
    "texmf-dist/doc/latex/seqsplit/README",
    "texmf-dist/doc/latex/seqsplit/seqsplit.pdf",
  },
  ["treetex"] = {
    "texmf-dist/doc/plain/treetex/readme",
  },
  ["eqlist"] = {
    "texmf-dist/doc/latex/eqlist/README",
    "texmf-dist/doc/latex/eqlist/eqlist.pdf",
  },
  ["lambda-lists"] = {
    "texmf-dist/doc/plain/lambda-lists/README",
    "texmf-dist/doc/plain/lambda-lists/lambda-lists.pdf",
  },
  ["pigpen"] = {
    "texmf-dist/doc/latex/pigpen/README",
    "texmf-dist/doc/latex/pigpen/pigpendoc.pdf",
  },
  ["resumecls"] = {
    "texmf-dist/doc/xelatex/resumecls/Makefile",
    "texmf-dist/doc/xelatex/resumecls/README.md",
    "texmf-dist/doc/xelatex/resumecls/example/Makefile",
    "texmf-dist/doc/xelatex/resumecls/example/README.md",
    "texmf-dist/doc/xelatex/resumecls/example/resume-en.pdf",
    "texmf-dist/doc/xelatex/resumecls/example/resume-zh.pdf",
    "texmf-dist/doc/xelatex/resumecls/resumecls.pdf",
  },
  ["resphilosophica"] = {
    "texmf-dist/doc/latex/resphilosophica/Makefile",
    "texmf-dist/doc/latex/resphilosophica/README",
    "texmf-dist/doc/latex/resphilosophica/resphilosophica.pdf",
    "texmf-dist/doc/latex/resphilosophica/rpsample.pdf",
  },
  ["12many"] = {
    "texmf-dist/doc/latex/12many/12many.pdf",
    "texmf-dist/doc/latex/12many/README",
  },
  ["gs1"] = {
    "texmf-dist/doc/latex/GS1/GS1.pdf",
    "texmf-dist/doc/latex/GS1/README",
    "texmf-dist/doc/latex/GS1/examples/EANBarcode.pdf",
    "texmf-dist/doc/latex/GS1/examples/EANControlDigit.pdf",
    "texmf-dist/doc/latex/GS1/examples/GSSetup.pdf",
    "texmf-dist/doc/latex/GS1/examples/GS_cut_EAN_control_digit.pdf",
    "texmf-dist/doc/latex/GS1/examples/GS_set_EAN_control_digit.pdf",
    "texmf-dist/doc/latex/GS1/examples/GS_set_code_digit_seq.pdf",
    "texmf-dist/doc/latex/GS1/examples/int_set_to_EAN_control_digit.pdf",
    "texmf-dist/doc/latex/GS1/rule-D.pdf",
  },
  ["repltext"] = {
    "texmf-dist/doc/latex/repltext/README",
    "texmf-dist/doc/latex/repltext/repltext.pdf",
  },
  ["hlist"] = {
    "texmf-dist/doc/generic/hlist/README",
    "texmf-dist/doc/generic/hlist/hlist-fr.pdf",
  },
  ["repere"] = {
    "texmf-dist/doc/metapost/repere/README.md",
    "texmf-dist/doc/metapost/repere/repere-doc.pdf",
  },
  ["stex"] = {
    "texmf-dist/doc/latex/stex/cmath/README",
    "texmf-dist/doc/latex/stex/cmath/cmath.pdf",
    "texmf-dist/doc/latex/stex/dcm/README",
    "texmf-dist/doc/latex/stex/dcm/dcm.pdf",
    "texmf-dist/doc/latex/stex/example/Makefile",
    "texmf-dist/doc/latex/stex/example/README",
    "texmf-dist/doc/latex/stex/example/background/Makefile",
    "texmf-dist/doc/latex/stex/example/background/all.pdf",
    "texmf-dist/doc/latex/stex/example/background/functions.pdf",
    "texmf-dist/doc/latex/stex/example/background/reals.pdf",
    "texmf-dist/doc/latex/stex/example/paper/Makefile",
    "texmf-dist/doc/latex/stex/example/paper/continuous.pdf",
    "texmf-dist/doc/latex/stex/example/paper/differentiable.pdf",
    "texmf-dist/doc/latex/stex/example/paper/intro.pdf",
    "texmf-dist/doc/latex/stex/example/paper/paper.pdf",
    "texmf-dist/doc/latex/stex/hwexam/README",
    "texmf-dist/doc/latex/stex/hwexam/hwexam.pdf",
    "texmf-dist/doc/latex/stex/metakeys/README",
    "texmf-dist/doc/latex/stex/metakeys/metakeys.pdf",
    "texmf-dist/doc/latex/stex/mikoslides/README",
    "texmf-dist/doc/latex/stex/mikoslides/mikoslides.pdf",
    "texmf-dist/doc/latex/stex/modules/README",
    "texmf-dist/doc/latex/stex/modules/modules.pdf",
    "texmf-dist/doc/latex/stex/omdoc/README",
    "texmf-dist/doc/latex/stex/omdoc/omdoc.pdf",
    "texmf-dist/doc/latex/stex/omtext/README",
    "texmf-dist/doc/latex/stex/omtext/omtext.pdf",
    "texmf-dist/doc/latex/stex/presentation/README",
    "texmf-dist/doc/latex/stex/presentation/presentation.pdf",
    "texmf-dist/doc/latex/stex/problem/README",
    "texmf-dist/doc/latex/stex/problem/problem.pdf",
    "texmf-dist/doc/latex/stex/rfdmeta/README",
    "texmf-dist/doc/latex/stex/rfdmeta/certification.pdf",
    "texmf-dist/doc/latex/stex/rfdmeta/rdfmeta.pdf",
    "texmf-dist/doc/latex/stex/smglom/README",
    "texmf-dist/doc/latex/stex/smglom/smglom.pdf",
    "texmf-dist/doc/latex/stex/smultiling/README",
    "texmf-dist/doc/latex/stex/smultiling/smultiling.pdf",
    "texmf-dist/doc/latex/stex/sproof/README",
    "texmf-dist/doc/latex/stex/sproof/sproof.pdf",
    "texmf-dist/doc/latex/stex/sref/README",
    "texmf-dist/doc/latex/stex/sref/book.pdf",
    "texmf-dist/doc/latex/stex/sref/idc.pdf",
    "texmf-dist/doc/latex/stex/sref/scr.pdf",
    "texmf-dist/doc/latex/stex/sref/sref.pdf",
    "texmf-dist/doc/latex/stex/statements/README",
    "texmf-dist/doc/latex/stex/statements/statements.pdf",
    "texmf-dist/doc/latex/stex/stex/README",
    "texmf-dist/doc/latex/stex/stex/stex.pdf",
    "texmf-dist/doc/latex/stex/structview/README",
    "texmf-dist/doc/latex/stex/structview/structview.pdf",
    "texmf-dist/doc/latex/stex/tikzinput/README",
    "texmf-dist/doc/latex/stex/tikzinput/tikzinput.pdf",
    "texmf-dist/doc/latex/stex/workaddress/README",
    "texmf-dist/doc/latex/stex/workaddress/workaddress.pdf",
  },
  ["regstats"] = {
    "texmf-dist/doc/latex/regstats/README",
    "texmf-dist/doc/latex/regstats/regstats-example.pdf",
    "texmf-dist/doc/latex/regstats/regstats.pdf",
  },
  ["register"] = {
    "texmf-dist/doc/latex/register/README",
    "texmf-dist/doc/latex/register/register.pdf",
  },
  ["refstyle"] = {
    "texmf-dist/doc/latex/refstyle/README",
    "texmf-dist/doc/latex/refstyle/refconfig.pdf",
    "texmf-dist/doc/latex/refstyle/refstyle.pdf",
  },
  ["vertbars"] = {
    "texmf-dist/doc/latex/vertbars/README",
    "texmf-dist/doc/latex/vertbars/vertbars.pdf",
  },
  ["reflectgraphics"] = {
    "texmf-dist/doc/latex/reflectgraphics/README",
    "texmf-dist/doc/latex/reflectgraphics/reflectgraphics.pdf",
  },
  ["recycle"] = {
    "texmf-dist/doc/fonts/recycle/README",
  },
  ["pxjahyper"] = {
    "texmf-dist/doc/platex/pxjahyper/LICENSE",
    "texmf-dist/doc/platex/pxjahyper/README.md",
    "texmf-dist/doc/platex/pxjahyper/pxjahyper.pdf",
  },
  ["gmdoc-enhance"] = {
    "texmf-dist/doc/latex/gmdoc-enhance/README",
    "texmf-dist/doc/latex/gmdoc-enhance/gmdoc-enhance.pdf",
  },
  ["newcommand"] = {
    "texmf-dist/doc/latex/newcommand/README",
    "texmf-dist/doc/latex/newcommand/newcommand.pdf",
  },
  ["rectopma"] = {
    "texmf-dist/doc/latex/rectopma/TestTitle.pdf",
  },
  ["dvidvi"] = {
    "texmf-dist/doc/man/man1/dvidvi.man1.pdf",
  },
  ["recipe"] = {
    "texmf-dist/doc/latex/recipe/README",
    "texmf-dist/doc/latex/recipe/sample.pdf",
  },
  ["elsarticle"] = {
    "texmf-dist/doc/latex/elsarticle/1pseperateaug.pdf",
    "texmf-dist/doc/latex/elsarticle/1psingleauthorgroup.pdf",
    "texmf-dist/doc/latex/elsarticle/README",
    "texmf-dist/doc/latex/elsarticle/elsdoc.pdf",
    "texmf-dist/doc/latex/elsarticle/elstest-1p.pdf",
    "texmf-dist/doc/latex/elsarticle/elstest-1pdoubleblind.pdf",
    "texmf-dist/doc/latex/elsarticle/elstest-3p.pdf",
    "texmf-dist/doc/latex/elsarticle/elstest-3pd.pdf",
    "texmf-dist/doc/latex/elsarticle/elstest-5p.pdf",
    "texmf-dist/doc/latex/elsarticle/jfigs.pdf",
    "texmf-dist/doc/latex/elsarticle/makefile",
    "texmf-dist/doc/latex/elsarticle/manifest.txt",
  },
  ["babel-esperanto"] = {
    "texmf-dist/doc/generic/babel-esperanto/esperanto.pdf",
  },
  ["dottex"] = {
    "texmf-dist/doc/latex/dottex/README",
    "texmf-dist/doc/latex/dottex/dottex.pdf",
    "texmf-dist/doc/latex/dottex/gpl.txt",
  },
  ["readarray"] = {
    "texmf-dist/doc/latex/readarray/README",
    "texmf-dist/doc/latex/readarray/readarray.pdf",
  },
  ["mfirstuc"] = {
    "texmf-dist/doc/latex/mfirstuc/CHANGES",
    "texmf-dist/doc/latex/mfirstuc/README",
    "texmf-dist/doc/latex/mfirstuc/mfirstuc-code.pdf",
    "texmf-dist/doc/latex/mfirstuc/mfirstuc-manual.html",
    "texmf-dist/doc/latex/mfirstuc/mfirstuc-manual.pdf",
    "texmf-dist/doc/latex/mfirstuc/sample-mfirstuc.pdf",
  },
  ["xcite"] = {
    "texmf-dist/doc/latex/xcite/README",
    "texmf-dist/doc/latex/xcite/xcite.pdf",
  },
  ["randtext"] = {
    "texmf-dist/doc/latex/randtext/README",
  },
  ["randomlist"] = {
    "texmf-dist/doc/generic/randomlist/Makefile",
    "texmf-dist/doc/generic/randomlist/README.txt",
    "texmf-dist/doc/generic/randomlist/randomlist.pdf",
    "texmf-dist/doc/generic/randomlist/test-fr.pdf",
    "texmf-dist/doc/generic/randomlist/test.pdf",
    "texmf-dist/doc/generic/randomlist/testtex.pdf",
  },
  ["guitarchordschemes"] = {
    "texmf-dist/doc/latex/guitarchordschemes/README",
    "texmf-dist/doc/latex/guitarchordschemes/guitarchordschemes_en.pdf",
  },
  ["luasseq"] = {
    "texmf-dist/doc/lualatex/luasseq/README",
    "texmf-dist/doc/lualatex/luasseq/luasseq.pdf",
  },
  ["lhelp"] = {
    "texmf-dist/doc/latex/lhelp/Makefile",
    "texmf-dist/doc/latex/lhelp/lhelp.pdf",
  },
  ["raleway"] = {
    "texmf-dist/doc/latex/raleway/LPPL.txt",
    "texmf-dist/doc/latex/raleway/OFL.txt",
    "texmf-dist/doc/latex/raleway/raleway-otf-specimen.pdf",
    "texmf-dist/doc/latex/raleway/raleway-type1-specimen.pdf",
    "texmf-dist/doc/latex/raleway/raleway.pdf",
  },
  ["asymptote"] = {
    "texmf-dist/doc/asymptote/CAD.pdf",
    "texmf-dist/doc/asymptote/TeXShopAndAsymptote.pdf",
    "texmf-dist/doc/asymptote/asy-latex.pdf",
    "texmf-dist/doc/asymptote/asyRefCard.pdf",
    "texmf-dist/doc/asymptote/asymptote.pdf",
    "texmf-dist/doc/asymptote/examples/pixel.pdf",
    "texmf-dist/doc/man/man1/asy.man1.pdf",
    "texmf-dist/doc/man/man1/xasy.man1.pdf",
  },
  ["quotmark"] = {
    "texmf-dist/doc/latex/quotmark/CHANGES",
    "texmf-dist/doc/latex/quotmark/README",
    "texmf-dist/doc/latex/quotmark/quotmark.pdf",
  },
  ["hypernat"] = {
    "texmf-dist/doc/latex/hypernat/hypernat.pdf",
  },
  ["quotchap"] = {
    "texmf-dist/doc/latex/quotchap/README",
    "texmf-dist/doc/latex/quotchap/document.pdf",
    "texmf-dist/doc/latex/quotchap/quotchap.pdf",
  },
  ["quicktype"] = {
    "texmf-dist/doc/latex/quicktype/Quicktype.pdf",
    "texmf-dist/doc/latex/quicktype/QuicktypeSample.pdf",
    "texmf-dist/doc/latex/quicktype/README.txt",
  },
  ["ctex"] = {
    "texmf-dist/doc/latex/ctex/README.md",
    "texmf-dist/doc/latex/ctex/ctex.pdf",
  },
  ["qcircuit"] = {
    "texmf-dist/doc/latex/qcircuit/README.md",
    "texmf-dist/doc/latex/qcircuit/qcircuit.pdf",
  },
  ["sparklines"] = {
    "texmf-dist/doc/latex/sparklines/README",
    "texmf-dist/doc/latex/sparklines/sparklines.pdf",
  },
  ["datetime2-usorbian"] = {
    "texmf-dist/doc/latex/datetime2-usorbian/README",
    "texmf-dist/doc/latex/datetime2-usorbian/datetime2-usorbian.pdf",
  },
  ["graphicxpsd"] = {
    "texmf-dist/doc/latex/graphicxpsd/LICENSE",
    "texmf-dist/doc/latex/graphicxpsd/README.md",
    "texmf-dist/doc/latex/graphicxpsd/graphicxpsd.pdf",
  },
  ["listings"] = {
    "texmf-dist/doc/latex/listings/Makefile",
    "texmf-dist/doc/latex/listings/README",
    "texmf-dist/doc/latex/listings/listings.pdf",
    "texmf-dist/doc/latex/listings/lstdrvrs.pdf",
  },
  ["basque-date"] = {
    "texmf-dist/doc/latex/basque-date/README",
    "texmf-dist/doc/latex/basque-date/basque-date.pdf",
  },
  ["mcite"] = {
    "texmf-dist/doc/latex/mcite/COPYING",
    "texmf-dist/doc/latex/mcite/README",
    "texmf-dist/doc/latex/mcite/mcite.pdf",
  },
  ["ifplatform"] = {
    "texmf-dist/doc/latex/ifplatform/README",
    "texmf-dist/doc/latex/ifplatform/ifplatform.pdf",
  },
  ["logpap"] = {
    "texmf-dist/doc/latex/logpap/README",
    "texmf-dist/doc/latex/logpap/example.pdf",
  },
  ["tex-overview"] = {
    "texmf-dist/doc/latex/tex-overview/README.md",
    "texmf-dist/doc/latex/tex-overview/tex-overview.pdf",
  },
  ["gcite"] = {
    "texmf-dist/doc/latex/gcite/CHANGES",
    "texmf-dist/doc/latex/gcite/README",
    "texmf-dist/doc/latex/gcite/gcite.pdf",
  },
  ["dtxgallery"] = {
    "texmf-dist/doc/latex/dtxgallery/README",
    "texmf-dist/doc/latex/dtxgallery/conditional-code.pdf",
    "texmf-dist/doc/latex/dtxgallery/dtxgallery.pdf",
    "texmf-dist/doc/latex/dtxgallery/rearrange.pdf",
    "texmf-dist/doc/latex/dtxgallery/single-source.pdf",
  },
  ["datetime2-hebrew"] = {
    "texmf-dist/doc/latex/datetime2-hebrew/README",
    "texmf-dist/doc/latex/datetime2-hebrew/datetime2-hebrew.pdf",
  },
  ["datetime"] = {
    "texmf-dist/doc/latex/datetime/CHANGES",
    "texmf-dist/doc/latex/datetime/README",
    "texmf-dist/doc/latex/datetime/datetime-manual.html",
    "texmf-dist/doc/latex/datetime/datetime.pdf",
    "texmf-dist/doc/latex/datetime/samples/dt-lang.pdf",
    "texmf-dist/doc/latex/datetime/samples/dt-sampl.pdf",
  },
  ["python"] = {
    "texmf-dist/doc/latex/python/README",
  },
  ["codepage"] = {
    "texmf-dist/doc/latex/codepage/LISEZMOI",
    "texmf-dist/doc/latex/codepage/README",
    "texmf-dist/doc/latex/codepage/codepage.pdf",
    "texmf-dist/doc/latex/codepage/frintro.pdf",
  },
  ["pxtxalfa"] = {
    "texmf-dist/doc/fonts/pxtxalfa/README",
    "texmf-dist/doc/fonts/pxtxalfa/pxtxalfa.pdf",
  },
  ["pxtatescale"] = {
    "texmf-dist/doc/latex/pxtatescale/LICENSE",
    "texmf-dist/doc/latex/pxtatescale/README",
  },
  ["lualatex-math"] = {
    "texmf-dist/doc/lualatex/lualatex-math/MANIFEST",
    "texmf-dist/doc/lualatex/lualatex-math/README",
    "texmf-dist/doc/lualatex/lualatex-math/lualatex-math.pdf",
  },
  ["pxpgfmark"] = {
    "texmf-dist/doc/latex/pxpgfmark/LICENSE",
    "texmf-dist/doc/latex/pxpgfmark/README",
  },
  ["fixfoot"] = {
    "texmf-dist/doc/latex/fixfoot/README",
    "texmf-dist/doc/latex/fixfoot/fixfoot.pdf",
  },
  ["pxgreeks"] = {
    "texmf-dist/doc/latex/pxgreeks/README",
    "texmf-dist/doc/latex/pxgreeks/pxgreeks.pdf",
  },
  ["pxcjkcat"] = {
    "texmf-dist/doc/latex/pxcjkcat/LICENSE",
    "texmf-dist/doc/latex/pxcjkcat/README.md",
    "texmf-dist/doc/latex/pxcjkcat/pxcjkcat.pdf",
  },
  ["pxchfon"] = {
    "texmf-dist/doc/platex/pxchfon/LICENSE",
    "texmf-dist/doc/platex/pxchfon/README.md",
    "texmf-dist/doc/platex/pxchfon/pxchfon.pdf",
    "texmf-dist/doc/platex/pxchfon/sample-2000jis.pdf",
    "texmf-dist/doc/platex/pxchfon/sample-2004jis.pdf",
    "texmf-dist/doc/platex/pxchfon/sample-pxchfon.pdf",
  },
  ["pxbase"] = {
    "texmf-dist/doc/platex/pxbase/LICENSE",
    "texmf-dist/doc/platex/pxbase/README.md",
    "texmf-dist/doc/platex/pxbase/pxbabel.pdf",
  },
  ["dviljk"] = {
    "texmf-dist/doc/man/man1/dvihp.man1.pdf",
    "texmf-dist/doc/man/man1/dvilj.man1.pdf",
    "texmf-dist/doc/man/man1/dvilj2p.man1.pdf",
    "texmf-dist/doc/man/man1/dvilj4.man1.pdf",
    "texmf-dist/doc/man/man1/dvilj4l.man1.pdf",
    "texmf-dist/doc/man/man1/dvilj6.man1.pdf",
  },
  ["minitoc"] = {
    "texmf-dist/doc/latex/minitoc/INSTALL",
    "texmf-dist/doc/latex/minitoc/README.md",
    "texmf-dist/doc/latex/minitoc/TODO",
    "texmf-dist/doc/latex/minitoc/cmk",
    "texmf-dist/doc/latex/minitoc/emk",
    "texmf-dist/doc/latex/minitoc/examples/mtc-2c.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-2nd.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-3co.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-add.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-ads.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-amm.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-apx.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-art.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-bk.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-bo.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-ch0.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-cri.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-fko.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-fo1.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-fo2.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-gap.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-hi1.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-hi2.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-hia.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-hir.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-hop.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-liv.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-mem.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-mm1.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-mu.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-nom.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-ocf.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-ofs.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-sbf.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-scr.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-syn.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-tbi.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-tlc.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-tsf.pdf",
    "texmf-dist/doc/latex/minitoc/examples/mtc-vti.pdf",
    "texmf-dist/doc/latex/minitoc/fmk",
    "texmf-dist/doc/latex/minitoc/imk",
    "texmf-dist/doc/latex/minitoc/minitoc-fr.pdf",
    "texmf-dist/doc/latex/minitoc/minitoc.pdf",
    "texmf-dist/doc/latex/minitoc/pmk",
    "texmf-dist/doc/latex/minitoc/rmk",
    "texmf-dist/doc/latex/minitoc/tmk",
    "texmf-dist/doc/latex/minitoc/xmk",
  },
  ["handout"] = {
    "texmf-dist/doc/latex/handout/README",
    "texmf-dist/doc/latex/handout/examples/example1-minimal.pdf",
    "texmf-dist/doc/latex/handout/examples/example2-cancel-quotation.pdf",
    "texmf-dist/doc/latex/handout/examples/example3-defined-path.pdf",
    "texmf-dist/doc/latex/handout/examples/example4-sectioning.pdf",
    "texmf-dist/doc/latex/handout/examples/example5-numbering.pdf",
    "texmf-dist/doc/latex/handout/examples/example6-not-and-only.pdf",
    "texmf-dist/doc/latex/handout/examples/example7-biblatex.pdf",
    "texmf-dist/doc/latex/handout/examples/latexmkrc",
    "texmf-dist/doc/latex/handout/examples/makefile",
    "texmf-dist/doc/latex/handout/handout.pdf",
    "texmf-dist/doc/latex/handout/latexmkrc",
    "texmf-dist/doc/latex/handout/makefile",
  },
  ["makeplot"] = {
    "texmf-dist/doc/latex/makeplot/README",
    "texmf-dist/doc/latex/makeplot/makeplot.pdf",
  },
  ["he-she"] = {
    "texmf-dist/doc/latex/he-she/README",
    "texmf-dist/doc/latex/he-she/he-she.pdf",
  },
  ["dviinfox"] = {
    "texmf-dist/doc/latex/dviinfox/LICENSE",
    "texmf-dist/doc/latex/dviinfox/README.md",
  },
  ["purifyeps"] = {
    "texmf-dist/doc/man/man1/purifyeps.man1.pdf",
    "texmf-dist/doc/support/purifyeps/README",
  },
  ["punknova"] = {
    "texmf-dist/doc/fonts/punknova/Makefile",
    "texmf-dist/doc/fonts/punknova/NEWS",
    "texmf-dist/doc/fonts/punknova/README",
    "texmf-dist/doc/fonts/punknova/documentation/sample.pdf",
  },
  ["punk-latex"] = {
    "texmf-dist/doc/latex/punk-latex/README",
    "texmf-dist/doc/latex/punk-latex/punk.pdf",
  },
  ["context-cmscbf"] = {
    "texmf-dist/doc/context/third/cmscbf/VERSION",
    "texmf-dist/doc/context/third/cmscbf/cmscbf-demo.pdf",
    "texmf-dist/doc/context/third/cmscbf/cmscbf-doc.pdf",
  },
  ["ptext"] = {
    "texmf-dist/doc/xelatex/ptext/README",
    "texmf-dist/doc/xelatex/ptext/ptext.pdf",
  },
  ["ly1"] = {
    "texmf-dist/doc/fonts/ly1/README",
  },
  ["carbohydrates"] = {
    "texmf-dist/doc/latex/carbohydrates/README",
    "texmf-dist/doc/latex/carbohydrates/carbohydrates_en.pdf",
  },
  ["method"] = {
    "texmf-dist/doc/latex/method/README",
    "texmf-dist/doc/latex/method/method.pdf",
  },
  ["biblatex-oxref"] = {
    "texmf-dist/doc/latex/biblatex-oxref/Makefile",
    "texmf-dist/doc/latex/biblatex-oxref/README.md",
    "texmf-dist/doc/latex/biblatex-oxref/oxalph-doc.pdf",
    "texmf-dist/doc/latex/biblatex-oxref/oxnotes-doc.pdf",
    "texmf-dist/doc/latex/biblatex-oxref/oxnum-doc.pdf",
    "texmf-dist/doc/latex/biblatex-oxref/oxref.pdf",
    "texmf-dist/doc/latex/biblatex-oxref/oxyear-doc.pdf",
  },
  ["ebook"] = {
    "texmf-dist/doc/latex/ebook/README",
  },
  ["harmony"] = {
    "texmf-dist/doc/latex/harmony/README",
    "texmf-dist/doc/latex/harmony/harmony.pdf",
  },
  ["barcodes"] = {
    "texmf-dist/doc/latex/barcodes/README",
    "texmf-dist/doc/latex/barcodes/changes",
    "texmf-dist/doc/latex/barcodes/eandoc.pdf",
  },
  ["clrscode3e"] = {
    "texmf-dist/doc/latex/clrscode3e/README",
    "texmf-dist/doc/latex/clrscode3e/README.TEXLIVE",
  },
  ["pdfmarginpar"] = {
    "texmf-dist/doc/latex/pdfmarginpar/README",
    "texmf-dist/doc/latex/pdfmarginpar/pdfmarginpar.pdf",
    "texmf-dist/doc/latex/pdfmarginpar/pdfmarginparexample.pdf",
  },
  ["complexity"] = {
    "texmf-dist/doc/latex/complexity/README",
    "texmf-dist/doc/latex/complexity/complexity.pdf",
  },
  ["ketcindy"] = {
    "texmf-dist/doc/support/ketcindy/LICENSE",
    "texmf-dist/doc/support/ketcindy/README",
    "texmf-dist/doc/support/ketcindy/README.TeXLive",
    "texmf-dist/doc/support/ketcindy/ScriptKeytyped.txt",
    "texmf-dist/doc/support/ketcindy/Scriptfigure.txt",
    "texmf-dist/doc/support/ketcindy/Scriptkelib.txt",
    "texmf-dist/doc/support/ketcindy/Scriptketlibsci.txt",
    "texmf-dist/doc/support/ketcindy/ketcindyjs/ketcindylib.txt",
    "texmf-dist/doc/support/ketcindy/ketmanual/KeTCindyReferenceJ.pdf",
    "texmf-dist/doc/support/ketcindy/ketmanual/helpbuttonJ.pdf",
    "texmf-dist/doc/support/ketcindy/ketmanual/helpbuttonsE.pdf",
    "texmf-dist/doc/support/ketcindy/ketmanual/helplist3dJ.pdf",
    "texmf-dist/doc/support/ketcindy/ketmanual/helplistJ.pdf",
    "texmf-dist/doc/support/ketcindy/ketmanual/referenceR4_3_1_ltd.pdf",
    "texmf-dist/doc/support/ketcindy/ketmanual/refketlayer20161211J.pdf",
    "texmf-dist/doc/support/ketcindy/ketmanual/refketpic161211J.pdf",
    "texmf-dist/doc/support/ketcindy/ketmanual/refketslide161211J.pdf",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/howtouseslideE.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/howtouseslideJ.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/s0701basic.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/s0701basicE.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/s0702graph.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/s0702graphE.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/s0703repeat.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/s0703repeatE.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/s0704thin.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/s0705para.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/s0705paraE.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/s0706animate.txt",
    "texmf-dist/doc/support/ketcindy/samples/s07slides/s0706animateE.txt",
    "texmf-dist/doc/support/ketcindy/samples/s16ketcindyJS/s1602diffeq2.html",
    "texmf-dist/doc/support/ketcindy/samples/s16ketcindyJS/s1602diffeq2ketcindy.html",
    "texmf-dist/doc/support/ketcindy/samples/s16ketcindyJS/s1602diffeq2ketcindyon.html",
    "texmf-dist/doc/support/ketcindy/source/cdy-templates/Makefile",
    "texmf-dist/doc/support/ketcindy/source/ketmanual/Fig/setcolor.pdf",
  },
  ["kanaparser"] = {
    "texmf-dist/doc/luatex/kanaparser/README.md",
    "texmf-dist/doc/luatex/kanaparser/description.pdf",
    "texmf-dist/doc/luatex/kanaparser/examples.pdf",
  },
  ["nonumonpart"] = {
    "texmf-dist/doc/latex/nonumonpart/LISEZMOI",
    "texmf-dist/doc/latex/nonumonpart/README",
    "texmf-dist/doc/latex/nonumonpart/nonumonpart-en.pdf",
    "texmf-dist/doc/latex/nonumonpart/nonumonpart-fr.pdf",
    "texmf-dist/doc/latex/nonumonpart/nonumonpart.pdf",
  },
  ["tikz-relay"] = {
    "texmf-dist/doc/latex/tikz-relay/README.txt",
    "texmf-dist/doc/latex/tikz-relay/doc/BeamerAnimation.pdf",
    "texmf-dist/doc/latex/tikz-relay/doc/tikz-relay-doc.pdf",
  },
  ["erdc"] = {
    "texmf-dist/doc/latex/erdc/README",
    "texmf-dist/doc/latex/erdc/erdc.pdf",
    "texmf-dist/doc/latex/erdc/nola.pdf",
    "texmf-dist/doc/latex/erdc/red_corps_castle2.pdf",
    "texmf-dist/doc/latex/erdc/sample.pdf",
  },
  ["beamerdarkthemes"] = {
    "texmf-dist/doc/latex/beamerdarkthemes/README",
    "texmf-dist/doc/latex/beamerdarkthemes/beamerdarkthemesuserguide.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/cormorantexampledefault.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/cormorantexampleinfolines.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/cormorantexamplesidebar.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/cormorantexampletree.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/frigatebirdexampledefault.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/frigatebirdexampleinfolines.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/frigatebirdexamplesidebar.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/frigatebirdexampletree.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/magpieexampledefault.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/magpieexampleinfolines.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/magpieexamplesidebar.pdf",
    "texmf-dist/doc/latex/beamerdarkthemes/magpieexampletree.pdf",
  },
  ["datetime2-turkish"] = {
    "texmf-dist/doc/latex/datetime2-turkish/README",
    "texmf-dist/doc/latex/datetime2-turkish/datetime2-turkish.pdf",
  },
  ["biblatex-source-division"] = {
    "texmf-dist/doc/latex/biblatex-source-division/README",
    "texmf-dist/doc/latex/biblatex-source-division/biblatex-source-division.pdf",
    "texmf-dist/doc/latex/biblatex-source-division/latexmkrc",
    "texmf-dist/doc/latex/biblatex-source-division/makefile",
  },
  ["lwarp"] = {
    "texmf-dist/doc/latex/lwarp/README.txt",
    "texmf-dist/doc/latex/lwarp/lwarp.pdf",
    "texmf-dist/doc/latex/lwarp/lwarp_tutorial.txt",
  },
  ["koma-script-examples"] = {
    "texmf-dist/doc/latex/koma-script-examples/Anhang-B/result/chapterthumbexample.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/briefDIN.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/briefSN.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/briefbak3.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/briefdup.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/brieflft.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/brieflfu.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/briefnow.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/briefrag2.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/briefwbk.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/briefwbk2.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/briefwbt2.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/letter-nipponA.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/result/letter-nipponB.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Anhang-C/source/CaptainHook-1.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-11/result/scrjuraexample.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-15/result/remarkexample.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-19/result/scrlayer-notecolumn-example.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-0.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-1.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-10.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-11.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-12.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-13.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-14.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-15.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-16.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-17.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-18.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-19.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-2.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-20.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-21.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-22.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-23.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-3.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-4.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-5.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-6.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-7.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-8.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/result/letter-9.pdf",
    "texmf-dist/doc/latex/koma-script-examples/Kapitel-4/source/musterlogo.pdf",
    "texmf-dist/doc/latex/koma-script-examples/LEGAL.txt",
    "texmf-dist/doc/latex/koma-script-examples/LEGAL_DE.txt",
    "texmf-dist/doc/latex/koma-script-examples/LIESMICH.txt",
    "texmf-dist/doc/latex/koma-script-examples/README.txt",
  },
  ["drv"] = {
    "texmf-dist/doc/metapost/drv/README",
    "texmf-dist/doc/metapost/drv/doc/makefile",
    "texmf-dist/doc/metapost/drv/doc/readme.sh",
    "texmf-dist/doc/metapost/drv/drv-guide.pdf",
    "texmf-dist/doc/metapost/drv/sample/makefile",
    "texmf-dist/doc/metapost/drv/sample/readme.sh",
    "texmf-dist/doc/metapost/drv/template/makefile",
    "texmf-dist/doc/metapost/drv/template/readme.sh",
  },
  ["genealogytree"] = {
    "texmf-dist/doc/latex/genealogytree/README",
    "texmf-dist/doc/latex/genealogytree/genealogytree-example-1.pdf",
    "texmf-dist/doc/latex/genealogytree/genealogytree-example-2.pdf",
    "texmf-dist/doc/latex/genealogytree/genealogytree-example-3.pdf",
    "texmf-dist/doc/latex/genealogytree/genealogytree-languages.pdf",
    "texmf-dist/doc/latex/genealogytree/genealogytree.pdf",
  },
  ["mathdots"] = {
    "texmf-dist/doc/generic/mathdots/README",
    "texmf-dist/doc/generic/mathdots/mathdots.pdf",
    "texmf-dist/doc/generic/mathdots/mdotest.pdf",
  },
  ["ptex-fontmaps"] = {
    "texmf-dist/doc/fonts/ptex-fontmaps/ChangeLog",
    "texmf-dist/doc/fonts/ptex-fontmaps/README",
  },
  ["nucleardata"] = {
    "texmf-dist/doc/latex/nucleardata/README",
    "texmf-dist/doc/latex/nucleardata/nuccommandtest.pdf",
    "texmf-dist/doc/latex/nucleardata/nucleardata.pdf",
  },
  ["ptex"] = {
    "texmf-dist/doc/man/man1/eptex.man1.pdf",
    "texmf-dist/doc/man/man1/makejvf.man1.pdf",
    "texmf-dist/doc/man/man1/mendex.man1.pdf",
    "texmf-dist/doc/man/man1/ptex.man1.pdf",
  },
  ["ipaex"] = {
    "texmf-dist/doc/fonts/ipaex/IPA_Font_License_Agreement_v1.0.txt",
    "texmf-dist/doc/fonts/ipaex/README",
    "texmf-dist/doc/fonts/ipaex/Readme_IPAexfont00301.txt",
    "texmf-dist/doc/fonts/ipaex/Readme_IPAfont00303.txt",
  },
  ["euro"] = {
    "texmf-dist/doc/latex/euro/Makefile",
    "texmf-dist/doc/latex/euro/euro.pdf",
    "texmf-dist/doc/latex/euro/euro.txt",
  },
  ["cuisine"] = {
    "texmf-dist/doc/latex/cuisine/README",
    "texmf-dist/doc/latex/cuisine/cuisine.pdf",
  },
  ["mpostinl"] = {
    "texmf-dist/doc/latex/mpostinl/README.txt",
    "texmf-dist/doc/latex/mpostinl/mpostinl.pdf",
  },
  ["datetime2-en-fulltext"] = {
    "texmf-dist/doc/latex/datetime2-en-fulltext/README",
    "texmf-dist/doc/latex/datetime2-en-fulltext/datetime2-en-fulltext.pdf",
    "texmf-dist/doc/latex/datetime2-en-fulltext/sample-en-fulltext.pdf",
  },
  ["sa-tikz"] = {
    "texmf-dist/doc/latex/sa-tikz/README",
    "texmf-dist/doc/latex/sa-tikz/sa-tikz-doc.pdf",
  },
  ["gentombow"] = {
    "texmf-dist/doc/latex/gentombow/LICENSE",
    "texmf-dist/doc/latex/gentombow/Makefile",
    "texmf-dist/doc/latex/gentombow/README.md",
    "texmf-dist/doc/latex/gentombow/bounddvi-en.pdf",
    "texmf-dist/doc/latex/gentombow/bounddvi.pdf",
    "texmf-dist/doc/latex/gentombow/gentombow-ja.pdf",
    "texmf-dist/doc/latex/gentombow/gentombow.pdf",
    "texmf-dist/doc/latex/gentombow/pxgentombow.pdf",
  },
  ["exercises"] = {
    "texmf-dist/doc/latex/exercises/README.md",
    "texmf-dist/doc/latex/exercises/exercises.pdf",
  },
  ["rotpages"] = {
    "texmf-dist/doc/latex/rotpages/Documentation/rotpages-doc.pdf",
    "texmf-dist/doc/latex/rotpages/Examples/rotpages-doublecolumn-ex.pdf",
    "texmf-dist/doc/latex/rotpages/Examples/rotpages-fancy-ex.pdf",
    "texmf-dist/doc/latex/rotpages/Examples/rotpages-singlecolumn-ex.pdf",
    "texmf-dist/doc/latex/rotpages/README",
  },
  ["hyphen-bulgarian"] = {
    "texmf-dist/doc/generic/hyph-utf8/bg/azbukaExtended.pdf",
  },
  ["econometrics"] = {
    "texmf-dist/doc/latex/econometrics/README.txt",
    "texmf-dist/doc/latex/econometrics/econometrics.pdf",
  },
  ["psutils"] = {
    "texmf-dist/doc/man/man1/epsffit.man1.pdf",
    "texmf-dist/doc/man/man1/extractres.man1.pdf",
    "texmf-dist/doc/man/man1/includeres.man1.pdf",
    "texmf-dist/doc/man/man1/psbook.man1.pdf",
    "texmf-dist/doc/man/man1/psjoin.man1.pdf",
    "texmf-dist/doc/man/man1/psnup.man1.pdf",
    "texmf-dist/doc/man/man1/psresize.man1.pdf",
    "texmf-dist/doc/man/man1/psselect.man1.pdf",
    "texmf-dist/doc/man/man1/pstops.man1.pdf",
    "texmf-dist/doc/man/man1/psutils.man1.pdf",
  },
  ["threeparttablex"] = {
    "texmf-dist/doc/latex/threeparttablex/README",
    "texmf-dist/doc/latex/threeparttablex/threeparttablex.pdf",
  },
  ["pstricks"] = {
    "texmf-dist/doc/generic/pstricks/README",
    "texmf-dist/doc/generic/pstricks/images/tiger.pdf",
    "texmf-dist/doc/generic/pstricks/pst-doc.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news05.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news06.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news08.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news09.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news10.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news11.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news12.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news13.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news14.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news15.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news16.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news17.pdf",
    "texmf-dist/doc/generic/pstricks/pst-news18.pdf",
    "texmf-dist/doc/generic/pstricks/pst-user.pdf",
    "texmf-dist/doc/generic/pstricks/pstricks-doc.pdf",
    "texmf-dist/doc/generic/pstricks/test-pst.pdf",
  },
  ["kixfont"] = {
    "texmf-dist/doc/fonts/kixfont/kixtable.pdf",
  },
  ["l3experimental"] = {
    "texmf-dist/doc/latex/l3experimental/README.md",
    "texmf-dist/doc/latex/l3experimental/l3color/l3color.pdf",
    "texmf-dist/doc/latex/l3experimental/l3draw/l3draw-code.pdf",
    "texmf-dist/doc/latex/l3experimental/l3draw/l3draw.pdf",
    "texmf-dist/doc/latex/l3experimental/l3str/l3str-convert.pdf",
    "texmf-dist/doc/latex/l3experimental/l3str/l3str-format.pdf",
    "texmf-dist/doc/latex/l3experimental/xcoffins/xcoffins.pdf",
    "texmf-dist/doc/latex/l3experimental/xgalley/l3galley.pdf",
    "texmf-dist/doc/latex/l3experimental/xgalley/xgalley.pdf",
  },
  ["titlepages"] = {
    "texmf-dist/doc/latex/titlepages/README",
    "texmf-dist/doc/latex/titlepages/titlepages.pdf",
  },
  ["ran_toks"] = {
    "texmf-dist/doc/latex/ran_toks/README.md",
    "texmf-dist/doc/latex/ran_toks/doc/rantoks_man.pdf",
  },
  ["ltb2bib"] = {
    "texmf-dist/doc/latex/ltb2bib/README",
    "texmf-dist/doc/latex/ltb2bib/ltb2bib.pdf",
  },
  ["crop"] = {
    "texmf-dist/doc/latex/crop/crop.pdf",
    "texmf-dist/doc/latex/crop/crop.txt",
  },
  ["latex-fonts"] = {
    "texmf-dist/doc/fonts/latex-fonts/README",
    "texmf-dist/doc/fonts/latex-fonts/legal.txt",
  },
  ["fontawesome5"] = {
    "texmf-dist/doc/fonts/fontawesome5/README.md",
    "texmf-dist/doc/fonts/fontawesome5/fontawesome5.pdf",
  },
  ["perception"] = {
    "texmf-dist/doc/bibtex/perception/README",
  },
  ["hvfloat"] = {
    "texmf-dist/doc/latex/hvfloat/Changes",
    "texmf-dist/doc/latex/hvfloat/README",
    "texmf-dist/doc/latex/hvfloat/hvfloat.pdf",
  },
  ["ionumbers"] = {
    "texmf-dist/doc/latex/ionumbers/COPYING",
    "texmf-dist/doc/latex/ionumbers/Makefile",
    "texmf-dist/doc/latex/ionumbers/README",
    "texmf-dist/doc/latex/ionumbers/ionumbers.pdf",
    "texmf-dist/doc/latex/ionumbers/ionumbers_test.pdf",
  },
  ["pst2pdf"] = {
    "texmf-dist/doc/latex/pst2pdf/Changes",
    "texmf-dist/doc/latex/pst2pdf/README",
    "texmf-dist/doc/latex/pst2pdf/pst2pdf-doc.pdf",
    "texmf-dist/doc/latex/pst2pdf/test1-pdf.pdf",
    "texmf-dist/doc/latex/pst2pdf/test2-pdf.pdf",
    "texmf-dist/doc/latex/pst2pdf/test3-pdf.pdf",
  },
  ["pst-vowel"] = {
    "texmf-dist/doc/latex/pst-vowel/README",
    "texmf-dist/doc/latex/pst-vowel/pst-vowel.pdf",
  },
  ["factura"] = {
    "texmf-dist/doc/latex/factura/README",
    "texmf-dist/doc/latex/factura/factura-ejemplo-prefactura.pdf",
    "texmf-dist/doc/latex/factura/factura.pdf",
  },
  ["lineno"] = {
    "texmf-dist/doc/latex/lineno/CHANGEs.txt",
    "texmf-dist/doc/latex/lineno/COPYING.txt",
    "texmf-dist/doc/latex/lineno/README",
    "texmf-dist/doc/latex/lineno/README.txt",
    "texmf-dist/doc/latex/lineno/SRCFILEs.txt",
    "texmf-dist/doc/latex/lineno/fnlineno.pdf",
    "texmf-dist/doc/latex/lineno/lineno.pdf",
    "texmf-dist/doc/latex/lineno/lnosuppl.pdf",
    "texmf-dist/doc/latex/lineno/ulineno.pdf",
  },
  ["pst-uml"] = {
    "texmf-dist/doc/generic/pst-uml/Changes",
    "texmf-dist/doc/generic/pst-uml/README",
    "texmf-dist/doc/generic/pst-uml/pst-uml-doc.pdf",
    "texmf-dist/doc/generic/pst-uml/pst-uml-encapsuled-pdf-fig.pdf",
    "texmf-dist/doc/generic/pst-uml/pst-uml-exemples.pdf",
  },
  ["productbox"] = {
    "texmf-dist/doc/latex/productbox/Makefile",
    "texmf-dist/doc/latex/productbox/README",
    "texmf-dist/doc/latex/productbox/productbox.pdf",
  },
  ["pst-tree"] = {
    "texmf-dist/doc/generic/pst-tree/Changes",
    "texmf-dist/doc/generic/pst-tree/README",
    "texmf-dist/doc/generic/pst-tree/pst-tree-doc.pdf",
  },
  ["luaindex"] = {
    "texmf-dist/doc/lualatex/luaindex/README",
    "texmf-dist/doc/lualatex/luaindex/luaindex-example.pdf",
    "texmf-dist/doc/lualatex/luaindex/luaindex.pdf",
  },
  ["piff"] = {
    "texmf-dist/doc/latex/piff/README",
    "texmf-dist/doc/latex/piff/duplicat-doc.pdf",
    "texmf-dist/doc/latex/piff/newproof-doc.pdf",
    "texmf-dist/doc/latex/piff/onepagem-doc.pdf",
    "texmf-dist/doc/latex/piff/time-doc.pdf",
  },
  ["pst-tools"] = {
    "texmf-dist/doc/generic/pst-tools/Changes",
    "texmf-dist/doc/generic/pst-tools/README",
    "texmf-dist/doc/generic/pst-tools/pst-tools-doc.pdf",
  },
  ["lshort-ukr"] = {
    "texmf-dist/doc/latex/lshort-ukr/lshort-ukr.pdf",
  },
  ["pst-thick"] = {
    "texmf-dist/doc/generic/pst-thick/Changes",
    "texmf-dist/doc/generic/pst-thick/README",
    "texmf-dist/doc/generic/pst-thick/pst-thick-doc.pdf",
  },
  ["pst-text"] = {
    "texmf-dist/doc/generic/pst-text/Changes",
    "texmf-dist/doc/generic/pst-text/README",
    "texmf-dist/doc/generic/pst-text/pst-text-doc.pdf",
  },
  ["bitpattern"] = {
    "texmf-dist/doc/latex/bitpattern/README.md",
    "texmf-dist/doc/latex/bitpattern/bitpattern.pdf",
  },
  ["figflow"] = {
    "texmf-dist/doc/plain/figflow/README.figflow",
  },
  ["mdframed"] = {
    "texmf-dist/doc/latex/mdframed/README.txt",
    "texmf-dist/doc/latex/mdframed/mdframed-example-default.pdf",
    "texmf-dist/doc/latex/mdframed/mdframed-example-pstricks.pdf",
    "texmf-dist/doc/latex/mdframed/mdframed-example-texsx.pdf",
    "texmf-dist/doc/latex/mdframed/mdframed-example-tikz.pdf",
    "texmf-dist/doc/latex/mdframed/mdframed.pdf",
  },
  ["translation-europecv-de"] = {
    "texmf-dist/doc/latex/translation-europecv-de/Beispiele/at.pdf",
    "texmf-dist/doc/latex/translation-europecv-de/Beispiele/greek-utf8.pdf",
    "texmf-dist/doc/latex/translation-europecv-de/Beispiele/minimal.pdf",
    "texmf-dist/doc/latex/translation-europecv-de/europecv-de.pdf",
    "texmf-dist/doc/latex/translation-europecv-de/templates/cv_template_de.pdf",
    "texmf-dist/doc/latex/translation-europecv-de/templates/cv_template_en.pdf",
    "texmf-dist/doc/latex/translation-europecv-de/templates/cv_template_it.pdf",
    "texmf-dist/doc/latex/translation-europecv-de/templates/cv_template_pl.pdf",
  },
  ["tikz-nef"] = {
    "texmf-dist/doc/latex/tikz-nef/LICENSE",
    "texmf-dist/doc/latex/tikz-nef/README.md",
    "texmf-dist/doc/latex/tikz-nef/tikz-nef-doc.pdf",
  },
  ["dictsym"] = {
    "texmf-dist/doc/fonts/dictsym/README",
    "texmf-dist/doc/fonts/dictsym/dictsym.pdf",
  },
  ["epspdf"] = {
    "texmf-dist/doc/support/epspdf/COPYING",
    "texmf-dist/doc/support/epspdf/Changelog",
    "texmf-dist/doc/support/epspdf/README",
    "texmf-dist/doc/support/epspdf/epspdf.pdf",
    "texmf-dist/doc/support/epspdf/images/logo.pdf",
    "texmf-dist/doc/support/epspdf/index.html",
  },
  ["pst-slpe"] = {
    "texmf-dist/doc/generic/pst-slpe/Changes",
    "texmf-dist/doc/generic/pst-slpe/README",
    "texmf-dist/doc/generic/pst-slpe/pst-slpe.pdf",
  },
  ["pst-sigsys"] = {
    "texmf-dist/doc/generic/pst-sigsys/Changes",
    "texmf-dist/doc/generic/pst-sigsys/README",
    "texmf-dist/doc/generic/pst-sigsys/pst-sigsys-doc.pdf",
  },
  ["uppunctlm"] = {
    "texmf-dist/doc/fonts/uppunctlm/README.md",
    "texmf-dist/doc/fonts/uppunctlm/always_upright_punctuation_marks.pdf",
  },
  ["pst-rubans"] = {
    "texmf-dist/doc/generic/pst-rubans/Changes",
    "texmf-dist/doc/generic/pst-rubans/README",
    "texmf-dist/doc/generic/pst-rubans/pst-rubans-doc.pdf",
  },
  ["pst-rputover"] = {
    "texmf-dist/doc/generic/pst-rputover/Changes.txt",
    "texmf-dist/doc/generic/pst-rputover/README.md",
    "texmf-dist/doc/generic/pst-rputover/pst-rputover-doc.pdf",
  },
  ["pst-plot"] = {
    "texmf-dist/doc/generic/pst-plot/Changes",
    "texmf-dist/doc/generic/pst-plot/README",
    "texmf-dist/doc/generic/pst-plot/data/demo.txt",
    "texmf-dist/doc/generic/pst-plot/pst-plot-doc.pdf",
  },
  ["dowith"] = {
    "texmf-dist/doc/generic/dowith/Announce.txt",
    "texmf-dist/doc/generic/dowith/README",
    "texmf-dist/doc/generic/dowith/SrcFILEs.txt",
    "texmf-dist/doc/generic/dowith/domore.pdf",
    "texmf-dist/doc/generic/dowith/dowith.pdf",
  },
  ["greekdates"] = {
    "texmf-dist/doc/latex/greekdates/README",
    "texmf-dist/doc/latex/greekdates/greekdates.pdf",
  },
  ["filemod"] = {
    "texmf-dist/doc/latex/filemod/README",
    "texmf-dist/doc/latex/filemod/filemod.pdf",
  },
  ["lshort-chinese"] = {
    "texmf-dist/doc/latex/lshort-chinese/README.md",
    "texmf-dist/doc/latex/lshort-chinese/lshort-zh-cn.pdf",
    "texmf-dist/doc/latex/lshort-chinese/src/Makefile",
  },
  ["isomath"] = {
    "texmf-dist/doc/latex/isomath/README.html",
    "texmf-dist/doc/latex/isomath/README.txt",
    "texmf-dist/doc/latex/isomath/isomath-test.pdf",
    "texmf-dist/doc/latex/isomath/isomath.html",
    "texmf-dist/doc/latex/isomath/isomath.pdf",
    "texmf-dist/doc/latex/isomath/isomath.sty.html",
    "texmf-dist/doc/latex/isomath/isomath.sty.txt",
    "texmf-dist/doc/latex/isomath/isomath.txt",
  },
  ["dpfloat"] = {
    "texmf-dist/doc/latex/dpfloat/README",
    "texmf-dist/doc/latex/dpfloat/dpfloat.pdf",
  },
  ["typeface"] = {
    "texmf-dist/doc/latex/typeface/README",
    "texmf-dist/doc/latex/typeface/typeface-all-rm.pdf",
    "texmf-dist/doc/latex/typeface/typeface.pdf",
  },
  ["textmerg"] = {
    "texmf-dist/doc/generic/textmerg/textmerg.pdf",
  },
  ["pst-pdf"] = {
    "texmf-dist/doc/latex/pst-pdf/CHANGES",
    "texmf-dist/doc/latex/pst-pdf/Makefile",
    "texmf-dist/doc/latex/pst-pdf/README.md",
    "texmf-dist/doc/latex/pst-pdf/pst-pdf-DE.pdf",
    "texmf-dist/doc/latex/pst-pdf/pst-pdf-example.pdf",
    "texmf-dist/doc/latex/pst-pdf/pst-pdf.pdf",
  },
  ["undergradmath"] = {
    "texmf-dist/doc/latex/undergradmath/LICENSE",
    "texmf-dist/doc/latex/undergradmath/README",
    "texmf-dist/doc/latex/undergradmath/undergradmath.pdf",
  },
  ["cyrillic"] = {
    "texmf-dist/doc/latex/cyrillic/README.md",
    "texmf-dist/doc/latex/cyrillic/changes.txt",
    "texmf-dist/doc/latex/cyrillic/cyinpenc.pdf",
    "texmf-dist/doc/latex/cyrillic/cyoutenc.pdf",
    "texmf-dist/doc/latex/cyrillic/lcy.pdf",
    "texmf-dist/doc/latex/cyrillic/lcycmlh.pdf",
    "texmf-dist/doc/latex/cyrillic/manifest.txt",
    "texmf-dist/doc/latex/cyrillic/ot2.pdf",
    "texmf-dist/doc/latex/cyrillic/ot2cmams.pdf",
    "texmf-dist/doc/latex/cyrillic/ot2cmlh.pdf",
    "texmf-dist/doc/latex/cyrillic/t2lhfnt.pdf",
  },
  ["esk"] = {
    "texmf-dist/doc/latex/esk/COPYING",
    "texmf-dist/doc/latex/esk/README",
    "texmf-dist/doc/latex/esk/esk.pdf",
    "texmf-dist/doc/latex/esk/eskman.pdf",
  },
  ["pst-osci"] = {
    "texmf-dist/doc/generic/pst-osci/Changes",
    "texmf-dist/doc/generic/pst-osci/README",
    "texmf-dist/doc/generic/pst-osci/oscilloscope.pdf",
    "texmf-dist/doc/generic/pst-osci/pst-osci-doc.pdf",
  },
  ["hf-tikz"] = {
    "texmf-dist/doc/latex/hf-tikz/README",
    "texmf-dist/doc/latex/hf-tikz/hf-tikz.pdf",
  },
  ["pst-optexp"] = {
    "texmf-dist/doc/latex/pst-optexp/Changes",
    "texmf-dist/doc/latex/pst-optexp/Makefile",
    "texmf-dist/doc/latex/pst-optexp/README",
    "texmf-dist/doc/latex/pst-optexp/pst-optexp-DE.pdf",
    "texmf-dist/doc/latex/pst-optexp/pst-optexp-quickref.pdf",
    "texmf-dist/doc/latex/pst-optexp/pst-optexp.pdf",
  },
  ["pst-mirror"] = {
    "texmf-dist/doc/generic/pst-mirror/Changes",
    "texmf-dist/doc/generic/pst-mirror/README",
    "texmf-dist/doc/generic/pst-mirror/createEPS/test.pdf",
    "texmf-dist/doc/generic/pst-mirror/pst-mirror-doc.pdf",
  },
  ["gitinfo"] = {
    "texmf-dist/doc/latex/gitinfo/README",
    "texmf-dist/doc/latex/gitinfo/gitinfo.pdf",
    "texmf-dist/doc/latex/gitinfo/post-xxx-sample.txt",
  },
  ["pst-math"] = {
    "texmf-dist/doc/generic/pst-math/Changes",
    "texmf-dist/doc/generic/pst-math/README",
    "texmf-dist/doc/generic/pst-math/pst-math-doc.pdf",
  },
  ["aligned-overset"] = {
    "texmf-dist/doc/latex/aligned-overset/README.md",
    "texmf-dist/doc/latex/aligned-overset/aligned-overset.pdf",
  },
  ["pst-lens"] = {
    "texmf-dist/doc/generic/pst-lens/Changes",
    "texmf-dist/doc/generic/pst-lens/README",
    "texmf-dist/doc/generic/pst-lens/pst-lens.pdf",
  },
  ["pst-layout"] = {
    "texmf-dist/doc/latex/pst-layout/README",
    "texmf-dist/doc/latex/pst-layout/pst-layout-doc.pdf",
  },
  ["pst-labo"] = {
    "texmf-dist/doc/generic/pst-labo/Changes",
    "texmf-dist/doc/generic/pst-labo/Makefile",
    "texmf-dist/doc/generic/pst-labo/README",
    "texmf-dist/doc/generic/pst-labo/pst-labo-docDE.pdf",
    "texmf-dist/doc/generic/pst-labo/pst-labo-docEN.pdf",
    "texmf-dist/doc/generic/pst-labo/pst-labo-docFR.pdf",
  },
  ["tabstackengine"] = {
    "texmf-dist/doc/latex/tabstackengine/README",
    "texmf-dist/doc/latex/tabstackengine/tabstackengine.pdf",
  },
  ["metalogo"] = {
    "texmf-dist/doc/latex/metalogo/README",
    "texmf-dist/doc/latex/metalogo/TeXoutline.pdf",
    "texmf-dist/doc/latex/metalogo/eLaToutline.pdf",
    "texmf-dist/doc/latex/metalogo/metalogo.pdf",
  },
  ["crossreftools"] = {
    "texmf-dist/doc/latex/crossreftools/README",
    "texmf-dist/doc/latex/crossreftools/crossreftools_doc.pdf",
    "texmf-dist/doc/latex/crossreftools/crossreftools_driver.pdf",
  },
  ["pst-jtree"] = {
    "texmf-dist/doc/generic/pst-jtree/README",
    "texmf-dist/doc/generic/pst-jtree/pst-jtree-doc-add.pdf",
    "texmf-dist/doc/generic/pst-jtree/pst-jtree-doc.pdf",
  },
  ["pst-intersect"] = {
    "texmf-dist/doc/latex/pst-intersect/Changes",
    "texmf-dist/doc/latex/pst-intersect/README",
    "texmf-dist/doc/latex/pst-intersect/pst-intersect-DE.pdf",
    "texmf-dist/doc/latex/pst-intersect/pst-intersect.pdf",
  },
  ["fundus-sueterlin"] = {
    "texmf-dist/doc/latex/fundus-sueterlin/suetterl.pdf",
  },
  ["cesenaexam"] = {
    "texmf-dist/doc/latex/cesenaexam/Makefile",
    "texmf-dist/doc/latex/cesenaexam/README.md",
    "texmf-dist/doc/latex/cesenaexam/cesenaexam.pdf",
    "texmf-dist/doc/latex/cesenaexam/cesenaexam_example.pdf",
  },
  ["pst-geometrictools"] = {
    "texmf-dist/doc/generic/pst-geometrictools/Changes.txt",
    "texmf-dist/doc/generic/pst-geometrictools/README.md",
    "texmf-dist/doc/generic/pst-geometrictools/pst-geometrictools-doc.pdf",
    "texmf-dist/doc/generic/pst-geometrictools/pst-geometrictools-fr-doc.pdf",
  },
  ["chess-problem-diagrams"] = {
    "texmf-dist/doc/latex/chess-problem-diagrams/README",
    "texmf-dist/doc/latex/chess-problem-diagrams/diagram.pdf",
  },
  ["pst-gantt"] = {
    "texmf-dist/doc/generic/pst-gantt/Changes",
    "texmf-dist/doc/generic/pst-gantt/README",
    "texmf-dist/doc/generic/pst-gantt/pst-gantt-doc.pdf",
  },
  ["pst-func"] = {
    "texmf-dist/doc/generic/pst-func/Changes",
    "texmf-dist/doc/generic/pst-func/README.md",
    "texmf-dist/doc/generic/pst-func/pst-func-doc.pdf",
  },
  ["jablantile"] = {
    "texmf-dist/doc/fonts/jablantile/README",
    "texmf-dist/doc/fonts/jablantile/jablantile.pdf",
  },
  ["bundledoc"] = {
    "texmf-dist/doc/man/man1/arlatex.man1.pdf",
    "texmf-dist/doc/man/man1/bundledoc.man1.pdf",
    "texmf-dist/doc/support/bundledoc/README",
  },
  ["pst-fractal"] = {
    "texmf-dist/doc/generic/pst-fractal/Changes",
    "texmf-dist/doc/generic/pst-fractal/README",
    "texmf-dist/doc/generic/pst-fractal/pst-fractal-doc.pdf",
  },
  ["pst-fit"] = {
    "texmf-dist/doc/generic/pst-fit/Changes",
    "texmf-dist/doc/generic/pst-fit/README",
    "texmf-dist/doc/generic/pst-fit/pst-fit-doc.pdf",
  },
  ["pst-bspline"] = {
    "texmf-dist/doc/generic/pst-bspline/README",
    "texmf-dist/doc/generic/pst-bspline/pst-bspline-doc.pdf",
  },
  ["gridset"] = {
    "texmf-dist/doc/latex/gridset/gridset.pdf",
  },
  ["pst-eucl-translation-bg"] = {
    "texmf-dist/doc/latex/pst-eucl-translation-bg/README",
    "texmf-dist/doc/latex/pst-eucl-translation-bg/README-bulgarian.txt",
    "texmf-dist/doc/latex/pst-eucl-translation-bg/pst-eucl-docBG.pdf",
  },
  ["pst-eucl"] = {
    "texmf-dist/doc/generic/pst-eucl/Changes",
    "texmf-dist/doc/generic/pst-eucl/README",
    "texmf-dist/doc/generic/pst-eucl/pst-eucl-doc.pdf",
  },
  ["pst-eps"] = {
    "texmf-dist/doc/generic/pst-eps/Changes",
    "texmf-dist/doc/generic/pst-eps/README",
    "texmf-dist/doc/generic/pst-eps/pst-eps-doc.pdf",
  },
  ["magaz"] = {
    "texmf-dist/doc/latex/magaz/magaz.pdf",
  },
  ["stage"] = {
    "texmf-dist/doc/latex/stage/README",
    "texmf-dist/doc/latex/stage/stage-documentation.pdf",
  },
  ["pst-ob3d"] = {
    "texmf-dist/doc/generic/pst-ob3d/Changes",
    "texmf-dist/doc/generic/pst-ob3d/README",
    "texmf-dist/doc/generic/pst-ob3d/pst-ob3d.pdf",
  },
  ["pst-contourplot"] = {
    "texmf-dist/doc/generic/pst-contourplot/README",
    "texmf-dist/doc/generic/pst-contourplot/examples/Les-Ovales-de-Descartes.pdf",
    "texmf-dist/doc/generic/pst-contourplot/examples/README",
    "texmf-dist/doc/generic/pst-contourplot/examples/bourke.pdf",
    "texmf-dist/doc/generic/pst-contourplot/examples/courbe-du-diable.pdf",
    "texmf-dist/doc/generic/pst-contourplot/examples/four-leminscates.pdf",
    "texmf-dist/doc/generic/pst-contourplot/pst-contourplot-docEN.pdf",
    "texmf-dist/doc/generic/pst-contourplot/pst-contourplot-docFR.pdf",
  },
  ["textgreek"] = {
    "texmf-dist/doc/latex/textgreek/README",
    "texmf-dist/doc/latex/textgreek/textgreek.pdf",
  },
  ["pst-circ"] = {
    "texmf-dist/doc/generic/pst-circ/Changes",
    "texmf-dist/doc/generic/pst-circ/README",
    "texmf-dist/doc/generic/pst-circ/pst-circ-doc.pdf",
  },
  ["pst-exa"] = {
    "texmf-dist/doc/latex/pst-exa/Changes",
    "texmf-dist/doc/latex/pst-exa/README",
    "texmf-dist/doc/latex/pst-exa/pst-exa-doc.pdf",
  },
  ["pst-bezier"] = {
    "texmf-dist/doc/generic/pst-bezier/Changes",
    "texmf-dist/doc/generic/pst-bezier/README.md",
    "texmf-dist/doc/generic/pst-bezier/pst-bezier-doc.pdf",
  },
  ["pst-bar"] = {
    "texmf-dist/doc/generic/pst-bar/README",
    "texmf-dist/doc/generic/pst-bar/pst-bar-doc.pdf",
    "texmf-dist/doc/generic/pst-bar/pst-bar-docDE.pdf",
  },
  ["examplep"] = {
    "texmf-dist/doc/latex/examplep/README",
    "texmf-dist/doc/latex/examplep/eurotex_2005_examplep.pdf",
    "texmf-dist/doc/latex/examplep/houses.pdf",
    "texmf-dist/doc/latex/examplep/pexaminipage.pdf",
    "texmf-dist/doc/latex/examplep/shorthyp_t1xtts.pdf",
  },
  ["pst-antiprism"] = {
    "texmf-dist/doc/generic/pst-antiprism/Changes",
    "texmf-dist/doc/generic/pst-antiprism/README.md",
    "texmf-dist/doc/generic/pst-antiprism/pst-antiprism-doc.pdf",
  },
  ["pst-am"] = {
    "texmf-dist/doc/generic/pst-am/Changes",
    "texmf-dist/doc/generic/pst-am/README",
    "texmf-dist/doc/generic/pst-am/pst-am-doc.pdf",
  },
  ["pdf-trans"] = {
    "texmf-dist/doc/generic/pdf-trans/example.pdf",
  },
  ["kurier"] = {
    "texmf-dist/doc/fonts/kurier/00readme.eng",
    "texmf-dist/doc/fonts/kurier/00readme.pol",
    "texmf-dist/doc/fonts/kurier/GUST-FONT-LICENSE.txt",
    "texmf-dist/doc/fonts/kurier/kurier-info.pdf",
    "texmf-dist/doc/fonts/kurier/manifest.txt",
  },
  ["cleanthesis"] = {
    "texmf-dist/doc/latex/cleanthesis/README.md",
    "texmf-dist/doc/latex/cleanthesis/doc/cleanthesis-doc.pdf",
    "texmf-dist/doc/latex/cleanthesis/gfx/Clean-Thesis-Figure.pdf",
    "texmf-dist/doc/latex/cleanthesis/gfx/Clean-Thesis-Logo.pdf",
    "texmf-dist/doc/latex/cleanthesis/thesis-example.pdf",
  },
  ["breakurl"] = {
    "texmf-dist/doc/latex/breakurl/README",
    "texmf-dist/doc/latex/breakurl/breakurl.pdf",
  },
  ["gost"] = {
    "texmf-dist/doc/bibtex/gost/README.md",
    "texmf-dist/doc/bibtex/gost/examples/cp1251/gost2003.pdf",
    "texmf-dist/doc/bibtex/gost/examples/cp1251/gost2008-customized.pdf",
    "texmf-dist/doc/bibtex/gost/examples/cp1251/gost2008.pdf",
    "texmf-dist/doc/bibtex/gost/examples/cp1251/gost2008l.pdf",
    "texmf-dist/doc/bibtex/gost/examples/cp1251/gost2008n.pdf",
    "texmf-dist/doc/bibtex/gost/examples/cp1251/gost2008ns.pdf",
    "texmf-dist/doc/bibtex/gost/examples/cp1251/gost780.pdf",
    "texmf-dist/doc/bibtex/gost/examples/utf8/ugost2003.pdf",
    "texmf-dist/doc/bibtex/gost/examples/utf8/ugost2008-customized.pdf",
    "texmf-dist/doc/bibtex/gost/examples/utf8/ugost2008.pdf",
    "texmf-dist/doc/bibtex/gost/examples/utf8/ugost2008l.pdf",
    "texmf-dist/doc/bibtex/gost/examples/utf8/ugost2008n.pdf",
    "texmf-dist/doc/bibtex/gost/examples/utf8/ugost2008ns.pdf",
    "texmf-dist/doc/bibtex/gost/examples/utf8/ugost2008s.pdf",
    "texmf-dist/doc/bibtex/gost/gost.pdf",
  },
  ["datatool"] = {
    "texmf-dist/doc/latex/datatool/CHANGES",
    "texmf-dist/doc/latex/datatool/INSTALL",
    "texmf-dist/doc/latex/datatool/README",
    "texmf-dist/doc/latex/datatool/datatool-code.pdf",
    "texmf-dist/doc/latex/datatool/datatool-user.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-autokeys.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-barchart.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-datatooltk.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-dict.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-europecv-bib.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-gidx.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-glossary.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-index.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-mail-merge.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-piechart.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-sort.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-student-scores.pdf",
    "texmf-dist/doc/latex/datatool/samples/sample-two-per-row.pdf",
  },
  ["hep"] = {
    "texmf-dist/doc/latex/hep/ChangeLog",
    "texmf-dist/doc/latex/hep/README",
  },
  ["psfrag"] = {
    "texmf-dist/doc/latex/psfrag/00readme.txt",
    "texmf-dist/doc/latex/psfrag/announce.txt",
    "texmf-dist/doc/latex/psfrag/pfgguide.pdf",
  },
  ["psbao"] = {
    "texmf-dist/doc/latex/psbao/Changes",
    "texmf-dist/doc/latex/psbao/README",
    "texmf-dist/doc/latex/psbao/psbaomanual.pdf",
  },
  ["adjustbox"] = {
    "texmf-dist/doc/latex/adjustbox/README",
    "texmf-dist/doc/latex/adjustbox/adjcalc.pdf",
    "texmf-dist/doc/latex/adjustbox/adjustbox.pdf",
    "texmf-dist/doc/latex/adjustbox/trimclip.pdf",
  },
  ["easylist"] = {
    "texmf-dist/doc/latex/easylist/README",
    "texmf-dist/doc/latex/easylist/easylist-doc.pdf",
  },
  ["ffslides"] = {
    "texmf-dist/doc/latex/ffslides/README.txt",
    "texmf-dist/doc/latex/ffslides/bground-example.txt",
    "texmf-dist/doc/latex/ffslides/ffslides-doc.pdf",
    "texmf-dist/doc/latex/ffslides/figure.pdf",
    "texmf-dist/doc/latex/ffslides/footer-example.txt",
    "texmf-dist/doc/latex/ffslides/header-example.txt",
    "texmf-dist/doc/latex/ffslides/tiny_example_1.pdf",
    "texmf-dist/doc/latex/ffslides/tiny_example_2.pdf",
  },
  ["cyrillic-bin"] = {
    "texmf-dist/doc/man/man1/rubibtex.man1.pdf",
    "texmf-dist/doc/man/man1/rumakeindex.man1.pdf",
  },
  ["hepparticles"] = {
    "texmf-dist/doc/latex/hepparticles/ChangeLog",
    "texmf-dist/doc/latex/hepparticles/README",
    "texmf-dist/doc/latex/hepparticles/hepparticles.pdf",
    "texmf-dist/doc/latex/hepparticles/testhepparticles.pdf",
  },
  ["properties"] = {
    "texmf-dist/doc/latex/properties/readme.de",
  },
  ["proofread"] = {
    "texmf-dist/doc/latex/proofread/README",
    "texmf-dist/doc/latex/proofread/README.txt",
    "texmf-dist/doc/latex/proofread/example.pdf",
    "texmf-dist/doc/latex/proofread/proofread.pdf",
  },
  ["progressbar"] = {
    "texmf-dist/doc/latex/progressbar/README",
    "texmf-dist/doc/latex/progressbar/progressbar.pdf",
  },
  ["latexcheat"] = {
    "texmf-dist/doc/latex/latexcheat/README",
    "texmf-dist/doc/latex/latexcheat/latexsheet.pdf",
  },
  ["prodint"] = {
    "texmf-dist/doc/fonts/prodint/README",
    "texmf-dist/doc/fonts/prodint/prodint.pdf",
  },
  ["hagenberg-thesis"] = {
    "texmf-dist/doc/latex/hagenberg-thesis/README.md",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbArticle/main.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbInternshipReport/images/logo.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbInternshipReport/main.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbLabReportDE/main.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbLabReportEN/main.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbTermReport/images/PLACE_IMAGES_HERE.txt",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbTermReport/main.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisDE/images/logo.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisDE/main.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisEN/images/logo.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisEN/main.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisTutorial/images/ellipse-parameters-1.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisTutorial/images/fragebogen.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisTutorial/images/inkscape-template-orig.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisTutorial/images/inkscape-template.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisTutorial/images/logo.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisTutorial/images/mathematica-example.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisTutorial/images/workflow-cm.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/HgbThesisTutorial/main.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/examples/Manual/main.pdf",
    "texmf-dist/doc/latex/hagenberg-thesis/hagenberg-thesis.pdf",
  },
  ["labelschanged"] = {
    "texmf-dist/doc/latex/labelschanged/README.txt",
    "texmf-dist/doc/latex/labelschanged/labelschanged.pdf",
  },
  ["presentations"] = {
    "texmf-dist/doc/latex/presentations/README",
    "texmf-dist/doc/latex/presentations/images/beamer/FUlogo.pdf",
    "texmf-dist/doc/latex/presentations/images/beamer/TeX.pdf",
    "texmf-dist/doc/latex/presentations/images/beamer/beamer0.pdf",
    "texmf-dist/doc/latex/presentations/images/beamer/beamer1.pdf",
    "texmf-dist/doc/latex/presentations/images/beamer/beamer2.pdf",
    "texmf-dist/doc/latex/presentations/images/beamer/beamernavsymbols.pdf",
    "texmf-dist/doc/latex/presentations/images/beamer/fu-berlin-air.pdf",
    "texmf-dist/doc/latex/presentations/images/beamer/fu-berlin.pdf",
    "texmf-dist/doc/latex/presentations/images/beamer/zedat.pdf",
    "texmf-dist/doc/latex/presentations/images/pd/fuBIB10.pdf",
    "texmf-dist/doc/latex/presentations/images/pdfscreen/Tore3d.pdf",
    "texmf-dist/doc/latex/presentations/images/pdfscreen/mp.pdf",
    "texmf-dist/doc/latex/presentations/images/pdfscreen/pst.pdf",
  },
  ["sanitize-umlaut"] = {
    "texmf-dist/doc/latex/sanitize-umlaut/README",
    "texmf-dist/doc/latex/sanitize-umlaut/sanitize-umlaut.pdf",
  },
  ["mailmerge"] = {
    "texmf-dist/doc/latex/mailmerge/README",
    "texmf-dist/doc/latex/mailmerge/mailmerge.pdf",
  },
  ["ebproof"] = {
    "texmf-dist/doc/latex/ebproof/README.md",
    "texmf-dist/doc/latex/ebproof/ebproof.pdf",
  },
  ["biblatex-philosophy"] = {
    "texmf-dist/doc/latex/biblatex-philosophy/README",
    "texmf-dist/doc/latex/biblatex-philosophy/biblatex-philosophy.pdf",
  },
  ["pracjourn"] = {
    "texmf-dist/doc/latex/pracjourn/README",
    "texmf-dist/doc/latex/pracjourn/pjsample.pdf",
    "texmf-dist/doc/latex/pracjourn/pracjourn.pdf",
  },
  ["powerdot-tuliplab"] = {
    "texmf-dist/doc/latex/powerdot-tuliplab/README.md",
  },
  ["doi"] = {
    "texmf-dist/doc/latex/doi/README",
  },
  ["powerdot-fuberlin"] = {
    "texmf-dist/doc/latex/powerdot-FUBerlin/README",
    "texmf-dist/doc/latex/powerdot-FUBerlin/exampleClass.pdf",
    "texmf-dist/doc/latex/powerdot-FUBerlin/exampleStyle.pdf",
  },
  ["bondgraphs"] = {
    "texmf-dist/doc/latex/bondgraphs/Makefile",
    "texmf-dist/doc/latex/bondgraphs/README",
    "texmf-dist/doc/latex/bondgraphs/bondgraphs.pdf",
  },
  ["conv-xkv"] = {
    "texmf-dist/doc/latex/conv-xkv/README.md",
    "texmf-dist/doc/latex/conv-xkv/doc/conv-xkv.pdf",
  },
  ["powerdot"] = {
    "texmf-dist/doc/latex/powerdot/README",
    "texmf-dist/doc/latex/powerdot/manifest.txt",
    "texmf-dist/doc/latex/powerdot/powerdot.pdf",
    "texmf-dist/doc/latex/powerdot/powerdotDE.pdf",
  },
  ["grid-system"] = {
    "texmf-dist/doc/latex/grid-system/LICENSE",
    "texmf-dist/doc/latex/grid-system/README",
    "texmf-dist/doc/latex/grid-system/grid-system.pdf",
  },
  ["classicthesis"] = {
    "texmf-dist/doc/latex/classicthesis/CHANGES",
    "texmf-dist/doc/latex/classicthesis/COPYING",
    "texmf-dist/doc/latex/classicthesis/ClassicThesis.pdf",
    "texmf-dist/doc/latex/classicthesis/Examples/classicthesis-arsclassica-book.pdf",
    "texmf-dist/doc/latex/classicthesis/Examples/classicthesis-article.pdf",
    "texmf-dist/doc/latex/classicthesis/Examples/classicthesis-book-xelatex.pdf",
    "texmf-dist/doc/latex/classicthesis/Examples/classicthesis-book.pdf",
    "texmf-dist/doc/latex/classicthesis/Examples/classicthesis-cv.pdf",
    "texmf-dist/doc/latex/classicthesis/LISTOFFILES",
    "texmf-dist/doc/latex/classicthesis/README",
    "texmf-dist/doc/latex/classicthesis/gfx/TFZsuperellipse_bw.pdf",
  },
  ["biblatex-science"] = {
    "texmf-dist/doc/latex/biblatex-science/README.md",
    "texmf-dist/doc/latex/biblatex-science/biblatex-science.pdf",
  },
  ["pst-calendar"] = {
    "texmf-dist/doc/latex/pst-calendar/Changes",
    "texmf-dist/doc/latex/pst-calendar/README",
    "texmf-dist/doc/latex/pst-calendar/pst-calendar-docDE.pdf",
    "texmf-dist/doc/latex/pst-calendar/pst-calendar-docFR.pdf",
  },
  ["postcards"] = {
    "texmf-dist/doc/latex/postcards/README",
    "texmf-dist/doc/latex/postcards/datasmp.txt",
  },
  ["luaintro"] = {
    "texmf-dist/doc/luatex/luaintro/README",
  },
  ["eiad-ltx"] = {
    "texmf-dist/doc/latex/eiad-ltx/README",
  },
  ["polynomial"] = {
    "texmf-dist/doc/latex/polynomial/README",
    "texmf-dist/doc/latex/polynomial/polynomial.pdf",
  },
  ["polynom"] = {
    "texmf-dist/doc/latex/polynom/README",
    "texmf-dist/doc/latex/polynom/polydemo.pdf",
    "texmf-dist/doc/latex/polynom/polynom.pdf",
  },
  ["beamer-tut-pt"] = {
    "texmf-dist/doc/latex/beamer-tut-pt/AnnArbor1.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/AnnArbor2.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/CambridgeUS1.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/CambridgeUS2.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/LEIAME",
    "texmf-dist/doc/latex/beamer-tut-pt/README",
    "texmf-dist/doc/latex/beamer-tut-pt/anim1.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/anim2.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/anim3.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/anim4.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/berkeley1.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/berkeley2.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/blocos.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/boadilla1.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/boadilla2.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/madrid1.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/madrid2.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/montpellier1.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/montpellier2.ps",
    "texmf-dist/doc/latex/beamer-tut-pt/tutorialbeamer.pdf",
  },
  ["poltawski"] = {
    "texmf-dist/doc/fonts/poltawski/GUST-FONT-LICENSE.txt",
    "texmf-dist/doc/fonts/poltawski/MANIFEST-Antykwa-Poltawskiego.txt",
    "texmf-dist/doc/fonts/poltawski/README-Antykwa-Poltawskiego.txt",
    "texmf-dist/doc/fonts/poltawski/ap-hist.txt",
    "texmf-dist/doc/fonts/poltawski/ap-info.pdf",
    "texmf-dist/doc/fonts/poltawski/ap-logo.pdf",
    "texmf-dist/doc/fonts/poltawski/tstapot1.pdf",
    "texmf-dist/doc/fonts/poltawski/tstapot4.pdf",
    "texmf-dist/doc/fonts/poltawski/tstapqx.pdf",
    "texmf-dist/doc/fonts/poltawski/tstapt1.pdf",
    "texmf-dist/doc/fonts/poltawski/tstapts1.pdf",
  },
  ["ccfonts"] = {
    "texmf-dist/doc/latex/ccfonts/ccfonts.pdf",
    "texmf-dist/doc/latex/ccfonts/readme",
  },
  ["poetry"] = {
    "texmf-dist/doc/latex/poetry/CHANGES",
    "texmf-dist/doc/latex/poetry/README",
    "texmf-dist/doc/latex/poetry/lppl.txt",
    "texmf-dist/doc/latex/poetry/poetry.pdf",
  },
  ["qpxqtx"] = {
    "texmf-dist/doc/fonts/qpxqtx/00README",
  },
  ["bophook"] = {
    "texmf-dist/doc/latex/bophook/bophook.pdf",
  },
  ["axodraw2"] = {
    "texmf-dist/doc/latex/axodraw2/AUTHORS",
    "texmf-dist/doc/latex/axodraw2/COPYING",
    "texmf-dist/doc/latex/axodraw2/ChangeLog",
    "texmf-dist/doc/latex/axodraw2/INSTALL",
    "texmf-dist/doc/latex/axodraw2/README",
    "texmf-dist/doc/latex/axodraw2/README.TEXLIVE",
    "texmf-dist/doc/latex/axodraw2/axodraw2-man.pdf",
    "texmf-dist/doc/man/man1/axohelp.man1.pdf",
  },
  ["getoptk"] = {
    "texmf-dist/doc/plain/getoptk/COPYING",
    "texmf-dist/doc/plain/getoptk/COPYING-FR",
    "texmf-dist/doc/plain/getoptk/README",
    "texmf-dist/doc/plain/getoptk/guide.pdf",
  },
  ["lshort-russian"] = {
    "texmf-dist/doc/latex/lshort-russian/lshortru.pdf",
  },
  ["shadow"] = {
    "texmf-dist/doc/latex/shadow/shadow-doc.pdf",
  },
  ["pst-pdgr"] = {
    "texmf-dist/doc/generic/pst-pdgr/NEWS",
    "texmf-dist/doc/generic/pst-pdgr/README",
    "texmf-dist/doc/generic/pst-pdgr/pst-pdgr.pdf",
  },
  ["biblatex-nature"] = {
    "texmf-dist/doc/latex/biblatex-nature/README.md",
    "texmf-dist/doc/latex/biblatex-nature/biblatex-nature.pdf",
  },
  ["bibleref-parse"] = {
    "texmf-dist/doc/latex/bibleref-parse/README",
    "texmf-dist/doc/latex/bibleref-parse/bibleref-parse.pdf",
  },
  ["skeycommand"] = {
    "texmf-dist/doc/latex/skeycommand/README",
    "texmf-dist/doc/latex/skeycommand/skeycommand-guide.pdf",
  },
  ["xetex-devanagari"] = {
    "texmf-dist/doc/xetex/xetex-devanagari/Changelog",
  },
  ["pst-shell"] = {
    "texmf-dist/doc/generic/pst-shell/Changes",
    "texmf-dist/doc/generic/pst-shell/README.md",
    "texmf-dist/doc/generic/pst-shell/images/shell.pdf",
    "texmf-dist/doc/generic/pst-shell/pst-shell-doc.pdf",
  },
  ["bibleref-lds"] = {
    "texmf-dist/doc/latex/bibleref-lds/README",
    "texmf-dist/doc/latex/bibleref-lds/bibleref-lds.pdf",
  },
  ["cbfonts"] = {
    "texmf-dist/doc/fonts/cbfonts/README",
    "texmf-dist/doc/fonts/cbfonts/cbgreek.pdf",
    "texmf-dist/doc/fonts/cbfonts/grmn1000table.pdf",
  },
  ["doipubmed"] = {
    "texmf-dist/doc/latex/doipubmed/CHANGES",
    "texmf-dist/doc/latex/doipubmed/README",
    "texmf-dist/doc/latex/doipubmed/doipubmed-manual.html",
    "texmf-dist/doc/latex/doipubmed/doipubmed.pdf",
  },
  ["qtree"] = {
    "texmf-dist/doc/latex/qtree/README",
    "texmf-dist/doc/latex/qtree/qarrows.pdf",
    "texmf-dist/doc/latex/qtree/qtreenotes.pdf",
  },
  ["ocgx2"] = {
    "texmf-dist/doc/latex/ocgx2/ChangeLog",
    "texmf-dist/doc/latex/ocgx2/README",
  },
  ["gene-logic"] = {
    "texmf-dist/doc/latex/gene-logic/gn-logic14.pdf",
  },
  ["multenum"] = {
    "texmf-dist/doc/latex/multenum/README",
    "texmf-dist/doc/latex/multenum/multienum.pdf",
  },
  ["gsemthesis"] = {
    "texmf-dist/doc/latex/gsemthesis/README",
    "texmf-dist/doc/latex/gsemthesis/gsemthesis.pdf",
  },
  ["pm-isomath"] = {
    "texmf-dist/doc/latex/pm-isomath/README",
    "texmf-dist/doc/latex/pm-isomath/manifest.txt",
    "texmf-dist/doc/latex/pm-isomath/pm-isomath.pdf",
  },
  ["dichokey"] = {
    "texmf-dist/doc/latex/dichokey/dichokey.pdf",
    "texmf-dist/doc/latex/dichokey/rhodocyb.pdf",
  },
  ["plweb"] = {
    "texmf-dist/doc/latex/plweb/README",
    "texmf-dist/doc/latex/plweb/pl.pdf",
  },
  ["plnfss"] = {
    "texmf-dist/doc/plain/plnfss/LPPL.txt",
    "texmf-dist/doc/plain/plnfss/plnfss.txt",
  },
  ["plex"] = {
    "texmf-dist/doc/fonts/plex/LICENSE.txt",
    "texmf-dist/doc/fonts/plex/README",
    "texmf-dist/doc/fonts/plex/plex-samples.pdf",
  },
  ["context-construction-plan"] = {
    "texmf-dist/doc/context/third/construction-plan/VERSION",
    "texmf-dist/doc/context/third/construction-plan/construction-plan-demo.pdf",
    "texmf-dist/doc/context/third/construction-plan/construction-plan-doc.pdf",
  },
  ["latexgit"] = {
    "texmf-dist/doc/latex/latexgit/LICENSE",
    "texmf-dist/doc/latex/latexgit/README.md",
    "texmf-dist/doc/latex/latexgit/latexgit.pdf",
  },
  ["tui"] = {
    "texmf-dist/doc/latex/tui/README",
    "texmf-dist/doc/latex/tui/TUIdoc.pdf",
    "texmf-dist/doc/latex/tui/example/imagen.pdf",
  },
  ["play"] = {
    "texmf-dist/doc/latex/play/README",
  },
  ["hletter"] = {
    "texmf-dist/doc/latex/hletter/Hletter.pdf",
    "texmf-dist/doc/latex/hletter/README",
  },
  ["japanese-otf-uptex"] = {
    "texmf-dist/doc/fonts/japanese-otf-uptex/00otf-uptex.txt",
    "texmf-dist/doc/fonts/japanese-otf-uptex/COPYRIGHT",
    "texmf-dist/doc/fonts/japanese-otf-uptex/README",
    "texmf-dist/doc/fonts/japanese-otf-uptex/test/uplatex/Makefile",
  },
  ["arraysort"] = {
    "texmf-dist/doc/latex/arraysort/Makefile",
    "texmf-dist/doc/latex/arraysort/README",
    "texmf-dist/doc/latex/arraysort/arraysort.pdf",
  },
  ["datetime2-breton"] = {
    "texmf-dist/doc/latex/datetime2-breton/README",
    "texmf-dist/doc/latex/datetime2-breton/datetime2-breton.pdf",
  },
  ["combofont"] = {
    "texmf-dist/doc/lualatex/combofont/README.md",
    "texmf-dist/doc/lualatex/combofont/combofont-test-fira-math.pdf",
    "texmf-dist/doc/lualatex/combofont/combofont.pdf",
  },
  ["platex"] = {
    "texmf-dist/doc/man/man1/platex.man1.pdf",
    "texmf-dist/doc/platex/base/Changes_asciimw.txt",
    "texmf-dist/doc/platex/base/LICENSE",
    "texmf-dist/doc/platex/base/README.md",
    "texmf-dist/doc/platex/base/README2_asciimw.txt",
    "texmf-dist/doc/platex/base/README_asciimw.txt",
    "texmf-dist/doc/platex/base/exppl2e.pdf",
    "texmf-dist/doc/platex/base/platex-en.pdf",
    "texmf-dist/doc/platex/base/platex.pdf",
    "texmf-dist/doc/platex/base/platexrelease.pdf",
    "texmf-dist/doc/platex/base/pldoc.pdf",
  },
  ["plari"] = {
    "texmf-dist/doc/latex/plari/COPYING",
    "texmf-dist/doc/latex/plari/README",
    "texmf-dist/doc/latex/plari/plari.pdf",
  },
  ["plainpkg"] = {
    "texmf-dist/doc/generic/plainpkg/README",
    "texmf-dist/doc/generic/plainpkg/SrcFILEs.txt",
    "texmf-dist/doc/generic/plainpkg/plainpkg-doc.pdf",
  },
  ["pdfsync"] = {
    "texmf-dist/doc/latex/pdfsync/README",
    "texmf-dist/doc/latex/pdfsync/pdfsync-doc.pdf",
  },
  ["forest-quickstart"] = {
    "texmf-dist/doc/latex/forest-quickstart/ForestQuickstart.pdf",
    "texmf-dist/doc/latex/forest-quickstart/README",
  },
  ["asypictureb"] = {
    "texmf-dist/doc/latex/asypictureb/README",
    "texmf-dist/doc/latex/asypictureb/asypictureB.pdf",
  },
  ["threeddice"] = {
    "texmf-dist/doc/metapost/threeddice/README",
    "texmf-dist/doc/metapost/threeddice/threeddice-doc.pdf",
  },
  ["gustprog"] = {
    "texmf-dist/doc/support/gustprog/README",
  },
  ["fonttable"] = {
    "texmf-dist/doc/latex/fonttable/README",
    "texmf-dist/doc/latex/fonttable/fonttable.pdf",
  },
  ["babel-interlingua"] = {
    "texmf-dist/doc/generic/babel-interlingua/interlingua.pdf",
  },
  ["ukrhyph"] = {
    "texmf-dist/doc/generic/ukrhyph/README",
    "texmf-dist/doc/generic/ukrhyph/rules60.pdf",
    "texmf-dist/doc/generic/ukrhyph/rules90.pdf",
    "texmf-dist/doc/generic/ukrhyph/rules_ph.pdf",
  },
  ["turnthepage"] = {
    "texmf-dist/doc/latex/turnthepage/Makefile",
    "texmf-dist/doc/latex/turnthepage/README",
    "texmf-dist/doc/latex/turnthepage/turnthepage.pdf",
  },
  ["ccaption"] = {
    "texmf-dist/doc/latex/ccaption/README",
    "texmf-dist/doc/latex/ccaption/ccaption.pdf",
  },
  ["pkfix"] = {
    "texmf-dist/doc/support/pkfix/README",
  },
  ["pixelart"] = {
    "texmf-dist/doc/latex/pixelart/LICENSE.txt",
    "texmf-dist/doc/latex/pixelart/README.md",
    "texmf-dist/doc/latex/pixelart/pixelart.pdf",
  },
  ["pittetd"] = {
    "texmf-dist/doc/latex/pittetd/pittetd.pdf",
  },
  ["t-angles"] = {
    "texmf-dist/doc/latex/t-angles/README",
    "texmf-dist/doc/latex/t-angles/t-manual.pdf",
  },
  ["piechartmp"] = {
    "texmf-dist/doc/metapost/piechartmp/INSTALL",
    "texmf-dist/doc/metapost/piechartmp/LEGAL",
    "texmf-dist/doc/metapost/piechartmp/README",
    "texmf-dist/doc/metapost/piechartmp/README.TEXLIVE",
    "texmf-dist/doc/metapost/piechartmp/examples/wec-mfun.pdf",
    "texmf-dist/doc/metapost/piechartmp/examples/wec.pdf",
  },
  ["pictex"] = {
    "texmf-dist/doc/generic/pictex/00index",
    "texmf-dist/doc/generic/pictex/README",
    "texmf-dist/doc/generic/pictex/pictexzusatz.txt",
    "texmf-dist/doc/generic/pictex/readme.errorbars",
  },
  ["pict2e"] = {
    "texmf-dist/doc/latex/pict2e/README.md",
    "texmf-dist/doc/latex/pict2e/manifest.txt",
    "texmf-dist/doc/latex/pict2e/p2e-drivers.pdf",
    "texmf-dist/doc/latex/pict2e/pict2e.pdf",
  },
  ["multirow"] = {
    "texmf-dist/doc/latex/multirow/README",
    "texmf-dist/doc/latex/multirow/multirow.pdf",
  },
  ["paper"] = {
    "texmf-dist/doc/latex/paper/README",
    "texmf-dist/doc/latex/paper/install",
    "texmf-dist/doc/latex/paper/paper.pdf",
  },
  ["babel-azerbaijani"] = {
    "texmf-dist/doc/generic/babel-azerbaijani/README",
    "texmf-dist/doc/generic/babel-azerbaijani/azerbaijani.pdf",
  },
  ["photo"] = {
    "texmf-dist/doc/latex/photo/Makefile",
    "texmf-dist/doc/latex/photo/photo.pdf",
  },
  ["phonetic"] = {
    "texmf-dist/doc/fonts/phonetic/Doc/README",
    "texmf-dist/doc/fonts/phonetic/README",
    "texmf-dist/doc/fonts/phonetic/makefile",
    "texmf-dist/doc/fonts/phonetic/phonetic-table.pdf",
  },
  ["phonenumbers"] = {
    "texmf-dist/doc/latex/phonenumbers/README",
    "texmf-dist/doc/latex/phonenumbers/phonenumbers-de.pdf",
    "texmf-dist/doc/latex/phonenumbers/phonenumbers-en.pdf",
  },
  ["texlive-it"] = {
    "texmf-dist/doc/texlive/texlive-it/Makefile",
    "texmf-dist/doc/texlive/texlive-it/texlive-it.html",
    "texmf-dist/doc/texlive/texlive-it/texlive-it.pdf",
  },
  ["phfsvnwatermark"] = {
    "texmf-dist/doc/latex/phfsvnwatermark/Makefile",
    "texmf-dist/doc/latex/phfsvnwatermark/README.md",
    "texmf-dist/doc/latex/phfsvnwatermark/phfsvnwatermark.pdf",
  },
  ["ha-prosper"] = {
    "texmf-dist/doc/latex/HA-prosper/HA-prosper.pdf",
    "texmf-dist/doc/latex/HA-prosper/README",
  },
  ["phfquotetext"] = {
    "texmf-dist/doc/latex/phfquotetext/Makefile",
    "texmf-dist/doc/latex/phfquotetext/README.md",
    "texmf-dist/doc/latex/phfquotetext/phfquotetext.pdf",
  },
  ["rsfso"] = {
    "texmf-dist/doc/fonts/rsfso/README",
    "texmf-dist/doc/fonts/rsfso/rsfso-doc.pdf",
  },
  ["datetime2-italian"] = {
    "texmf-dist/doc/latex/datetime2-italian/README",
  },
  ["pgfplots"] = {
    "texmf-dist/doc/context/third/pgfplots/Makefile",
    "texmf-dist/doc/context/third/pgfplots/pgfplotsexample-context.pdf",
    "texmf-dist/doc/generic/pgfplots/README",
    "texmf-dist/doc/latex/pgfplots/TeX-programming-notes.pdf",
    "texmf-dist/doc/latex/pgfplots/pgfplots.pdf",
    "texmf-dist/doc/latex/pgfplots/pgfplotsexample.pdf",
    "texmf-dist/doc/latex/pgfplots/pgfplotstable.pdf",
    "texmf-dist/doc/latex/pgfplots/pgfplotstodo.pdf",
    "texmf-dist/doc/plain/pgfplots/pgfplotsexample-plain.pdf",
  },
  ["pgfornament-han"] = {
    "texmf-dist/doc/latex/pgfornament-han/README.md",
    "texmf-dist/doc/latex/pgfornament-han/heavenlyclouds-sample.pdf",
    "texmf-dist/doc/latex/pgfornament-han/lppl-1-3c.txt",
    "texmf-dist/doc/latex/pgfornament-han/pgfornament-han_manual.pdf",
    "texmf-dist/doc/latex/pgfornament-han/xiaoshan-sample.pdf",
  },
  ["pgfornament"] = {
    "texmf-dist/doc/latex/pgfornament/README",
    "texmf-dist/doc/latex/pgfornament/ornaments.pdf",
    "texmf-dist/doc/latex/pgfornament/tikzrput.pdf",
  },
  ["trsym"] = {
    "texmf-dist/doc/latex/trsym/README",
    "texmf-dist/doc/latex/trsym/manifest.txt",
    "texmf-dist/doc/latex/trsym/trsym.pdf",
  },
  ["pgfkeyx"] = {
    "texmf-dist/doc/latex/pgfkeyx/README",
  },
  ["genmpage"] = {
    "texmf-dist/doc/latex/genmpage/README",
    "texmf-dist/doc/latex/genmpage/genmpage.pdf",
  },
  ["gnuplottex"] = {
    "texmf-dist/doc/latex/gnuplottex/README",
    "texmf-dist/doc/latex/gnuplottex/SomeValuesForGnuplot.txt",
    "texmf-dist/doc/latex/gnuplottex/gnuplottex.pdf",
    "texmf-dist/doc/latex/gnuplottex/gpl.txt",
  },
  ["serbian-date-lat"] = {
    "texmf-dist/doc/latex/serbian-date-lat/README",
    "texmf-dist/doc/latex/serbian-date-lat/SerbianDateLat.pdf",
    "texmf-dist/doc/latex/serbian-date-lat/TestDateLat.pdf",
  },
  ["marginnote"] = {
    "texmf-dist/doc/latex/marginnote/README.txt",
    "texmf-dist/doc/latex/marginnote/marginnote.pdf",
  },
  ["srcltx"] = {
    "texmf-dist/doc/latex/srcltx/README",
    "texmf-dist/doc/latex/srcltx/srcltx.pdf",
  },
  ["modroman"] = {
    "texmf-dist/doc/latex/modroman/modroman-en.pdf",
    "texmf-dist/doc/latex/modroman/modroman-fr.pdf",
    "texmf-dist/doc/latex/modroman/modroman.pdf",
  },
  ["sectionbox"] = {
    "texmf-dist/doc/latex/sectionbox/README",
    "texmf-dist/doc/latex/sectionbox/example/README.TEXLIVE",
    "texmf-dist/doc/latex/sectionbox/sectionboxmanual.pdf",
  },
  ["around-the-bend"] = {
    "texmf-dist/doc/generic/around-the-bend/AroundTheBend.pdf",
    "texmf-dist/doc/generic/around-the-bend/README",
  },
  ["moderncv"] = {
    "texmf-dist/doc/latex/moderncv/CHANGELOG",
    "texmf-dist/doc/latex/moderncv/KNOWN_BUGS",
    "texmf-dist/doc/latex/moderncv/README",
    "texmf-dist/doc/latex/moderncv/examples/Makefile",
    "texmf-dist/doc/latex/moderncv/examples/template-es.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template-multibib.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template-zh.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template_banking_red-fullrules-center.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template_banking_red-norules.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template_banking_red-shortrules-right.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template_banking_red.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template_casual_orange-left.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template_casual_orange.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template_classic_green-right.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template_classic_green.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template_fancy_purple.pdf",
    "texmf-dist/doc/latex/moderncv/examples/template_oldstyle_grey.pdf",
  },
  ["manuscript"] = {
    "texmf-dist/doc/latex/manuscript/Makefile",
    "texmf-dist/doc/latex/manuscript/README",
    "texmf-dist/doc/latex/manuscript/manuscript.pdf",
  },
  ["crimson"] = {
    "texmf-dist/doc/fonts/crimson/OFL.txt",
    "texmf-dist/doc/fonts/crimson/README",
    "texmf-dist/doc/fonts/crimson/crimson-samples.pdf",
  },
  ["petri-nets"] = {
    "texmf-dist/doc/generic/petri-nets/COPYING",
    "texmf-dist/doc/generic/petri-nets/ChangeLog",
    "texmf-dist/doc/generic/petri-nets/README",
    "texmf-dist/doc/generic/petri-nets/pndoc.pdf",
  },
  ["petiteannonce"] = {
    "texmf-dist/doc/latex/petiteannonce/petiteannonce.doc.pdf",
    "texmf-dist/doc/latex/petiteannonce/petiteannonceexample.pdf",
  },
  ["persian-bib"] = {
    "texmf-dist/doc/xelatex/persian-bib/Persian-bib-userguide.pdf",
    "texmf-dist/doc/xelatex/persian-bib/README",
  },
  ["permute"] = {
    "texmf-dist/doc/latex/permute/permute.pdf",
  },
  ["fontware"] = {
    "texmf-dist/doc/man/man1/pltotf.man1.pdf",
    "texmf-dist/doc/man/man1/tftopl.man1.pdf",
    "texmf-dist/doc/man/man1/vftovp.man1.pdf",
    "texmf-dist/doc/man/man1/vptovf.man1.pdf",
  },
  ["nuc"] = {
    "texmf-dist/doc/latex/nuc/README",
    "texmf-dist/doc/latex/nuc/nuc.pdf",
  },
  ["bredzenie"] = {
    "texmf-dist/doc/latex/bredzenie/README",
    "texmf-dist/doc/latex/bredzenie/bredzenie.pdf",
  },
  ["pstricks_calcnotes"] = {
    "texmf-dist/doc/latex/pstricks_calcnotes/AppThreePDF.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/Convert_PstricksCode_To_Pdf/Readme.txt",
    "texmf-dist/doc/latex/pstricks_calcnotes/Convert_PstricksCode_To_Pdf/test.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/BZ1.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/BZ10.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/BZ2.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/BZ3.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/BZ4.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/BZ5.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/BZ6.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/BZ7.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/BZ8.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/BZ9.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig13.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig14.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig1a.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig1b.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig1c.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig2a.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig2b.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig3.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig4.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig5.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig6.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig7.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig8.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/Fig9.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/b6of1.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/b6of2.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/b6of3.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/b6of4.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/vecb1.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/vecb2.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/vecb3.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/vecb4.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/vecb5.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/For_Pdf_Output/vecb6.pdf",
    "texmf-dist/doc/latex/pstricks_calcnotes/README",
    "texmf-dist/doc/latex/pstricks_calcnotes/ThreeAppsPDF.pdf",
  },
  ["mendex-doc"] = {
    "texmf-dist/doc/support/mendex/LICENSE",
    "texmf-dist/doc/support/mendex/README.md",
    "texmf-dist/doc/support/mendex/mendex.pdf",
  },
  ["schule"] = {
    "texmf-dist/doc/latex/schule/README",
    "texmf-dist/doc/latex/schule/README_DE",
    "texmf-dist/doc/latex/schule/doc/abFahrplan.pdf",
    "texmf-dist/doc/latex/schule/doc/abbott.pdf",
    "texmf-dist/doc/latex/schule/doc/abstandThread.pdf",
    "texmf-dist/doc/latex/schule/doc/nandGatter.pdf",
    "texmf-dist/doc/latex/schule/doc/schaltplan.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-abFahrplan.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-abbott.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-abstandThread.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-nandGatter.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-schaltplan.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-schuleit-aufgaben-crop.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-schuleit-aufgaben.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-skalierungSequenzdiagramm.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-syntaxPython.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-telefonnummern.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-ub.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-ubSource.pdf",
    "texmf-dist/doc/latex/schule/doc/schule-wasIstInformatik.pdf",
    "texmf-dist/doc/latex/schule/doc/skalierungSequenzdiagramm.pdf",
    "texmf-dist/doc/latex/schule/doc/syntaxPython.pdf",
    "texmf-dist/doc/latex/schule/doc/telefonnummern.pdf",
    "texmf-dist/doc/latex/schule/doc/ub.pdf",
    "texmf-dist/doc/latex/schule/doc/ubSource.pdf",
    "texmf-dist/doc/latex/schule/doc/wasIstInformatik.pdf",
    "texmf-dist/doc/latex/schule/schule.pdf",
  },
  ["synttree"] = {
    "texmf-dist/doc/latex/synttree/README",
    "texmf-dist/doc/latex/synttree/synttree.pdf",
  },
  ["ocg-p"] = {
    "texmf-dist/doc/latex/ocg-p/README",
    "texmf-dist/doc/latex/ocg-p/examples/ocg-p_example_1.pdf",
    "texmf-dist/doc/latex/ocg-p/examples/ocg-p_example_2.pdf",
    "texmf-dist/doc/latex/ocg-p/examples/ocg-p_example_3.pdf",
    "texmf-dist/doc/latex/ocg-p/examples/ocg-p_example_4.pdf",
    "texmf-dist/doc/latex/ocg-p/ocg-p.pdf",
  },
  ["pdfwin"] = {
    "texmf-dist/doc/latex/pdfwin/BucResampling.pdf",
    "texmf-dist/doc/latex/pdfwin/BucSystem1.pdf",
    "texmf-dist/doc/latex/pdfwin/BucSystem2.pdf",
    "texmf-dist/doc/latex/pdfwin/BucSystem3.pdf",
    "texmf-dist/doc/latex/pdfwin/BucSystem4.pdf",
    "texmf-dist/doc/latex/pdfwin/BucSystem5.pdf",
    "texmf-dist/doc/latex/pdfwin/BucSystem6.pdf",
  },
  ["pdftricks"] = {
    "texmf-dist/doc/latex/pdftricks/makefile",
    "texmf-dist/doc/latex/pdftricks/manual.pdf",
    "texmf-dist/doc/latex/pdftricks/pst2pdf",
    "texmf-dist/doc/latex/pdftricks/test.pdf",
  },
  ["chemnum"] = {
    "texmf-dist/doc/latex/chemnum/README",
    "texmf-dist/doc/latex/chemnum/chemnum_en.pdf",
    "texmf-dist/doc/latex/chemnum/scheme-bla.ps",
    "texmf-dist/doc/latex/chemnum/scheme-tmp.ps",
  },
  ["comicneue"] = {
    "texmf-dist/doc/latex/comicneue/SIL-License.txt",
    "texmf-dist/doc/latex/comicneue/comicneue-otf-specimen.pdf",
    "texmf-dist/doc/latex/comicneue/comicneue-type1-specimen.pdf",
    "texmf-dist/doc/latex/comicneue/comicneue.pdf",
  },
  ["context-lettrine"] = {
    "texmf-dist/doc/context/third/lettrine/VERSION",
    "texmf-dist/doc/context/third/lettrine/W.pdf",
    "texmf-dist/doc/context/third/lettrine/lettrine-doc.pdf",
  },
  ["augie"] = {
    "texmf-dist/doc/latex/augie/README.augie",
    "texmf-dist/doc/latex/augie/augie.txt",
  },
  ["newtx"] = {
    "texmf-dist/doc/fonts/newtx/MANIFEST-newtx.txt",
    "texmf-dist/doc/fonts/newtx/README",
    "texmf-dist/doc/fonts/newtx/implementation.pdf",
    "texmf-dist/doc/fonts/newtx/mathnotes.pdf",
    "texmf-dist/doc/fonts/newtx/newtxdoc.pdf",
    "texmf-dist/doc/fonts/newtx/sample-lib-crop.pdf",
    "texmf-dist/doc/fonts/newtx/sample-libmtp-crop.pdf",
    "texmf-dist/doc/fonts/newtx/sample-mtp-crop.pdf",
    "texmf-dist/doc/fonts/newtx/sample-ntx-crop.pdf",
    "texmf-dist/doc/fonts/newtx/sample-ptmx-crop.pdf",
    "texmf-dist/doc/fonts/newtx/sample-tx-crop.pdf",
  },
  ["apnum"] = {
    "texmf-dist/doc/generic/apnum/README",
    "texmf-dist/doc/generic/apnum/apnum.pdf",
  },
  ["xits"] = {
    "texmf-dist/doc/fonts/xits/FONTLOG.txt",
    "texmf-dist/doc/fonts/xits/Makefile",
    "texmf-dist/doc/fonts/xits/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/xits/OFL.txt",
    "texmf-dist/doc/fonts/xits/README",
    "texmf-dist/doc/fonts/xits/user-guide.pdf",
    "texmf-dist/doc/fonts/xits/xits-specimen.pdf",
  },
  ["euler"] = {
    "texmf-dist/doc/latex/euler/euler.pdf",
    "texmf-dist/doc/latex/euler/legal.txt",
  },
  ["eqnarray"] = {
    "texmf-dist/doc/latex/eqnarray/README",
    "texmf-dist/doc/latex/eqnarray/eqnarray.pdf",
  },
  ["logicproof"] = {
    "texmf-dist/doc/latex/logicproof/README",
    "texmf-dist/doc/latex/logicproof/logicproof.pdf",
  },
  ["expex"] = {
    "texmf-dist/doc/generic/expex/README",
    "texmf-dist/doc/generic/expex/expex-doc.pdf",
  },
  ["foreign"] = {
    "texmf-dist/doc/latex/foreign/README",
    "texmf-dist/doc/latex/foreign/foreign.pdf",
  },
  ["appendix"] = {
    "texmf-dist/doc/latex/appendix/README",
    "texmf-dist/doc/latex/appendix/appendix.pdf",
  },
  ["gotoh"] = {
    "texmf-dist/doc/latex/gotoh/LICENSE",
    "texmf-dist/doc/latex/gotoh/README.md",
    "texmf-dist/doc/latex/gotoh/gotoh.pdf",
  },
  ["eqell"] = {
    "texmf-dist/doc/latex/eqell/README",
    "texmf-dist/doc/latex/eqell/eqell.pdf",
  },
  ["fancytabs"] = {
    "texmf-dist/doc/latex/fancytabs/README",
    "texmf-dist/doc/latex/fancytabs/fancytabs.pdf",
  },
  ["guitartabs"] = {
    "texmf-dist/doc/latex/guitartabs/README.md",
    "texmf-dist/doc/latex/guitartabs/guitartabs-doc.pdf",
    "texmf-dist/doc/latex/guitartabs/nothingelsematters.pdf",
  },
  ["ocr-b"] = {
    "texmf-dist/doc/fonts/ocr-b/README",
  },
  ["csquotes-de"] = {
    "texmf-dist/doc/latex/csquotes-de/LIESMICH",
    "texmf-dist/doc/latex/csquotes-de/csquotes-DE.pdf",
  },
  ["statistics"] = {
    "texmf-dist/doc/latex/statistics/LICENSE",
    "texmf-dist/doc/latex/statistics/README.md",
    "texmf-dist/doc/latex/statistics/statistics.pdf",
  },
  ["autonum"] = {
    "texmf-dist/doc/latex/autonum/README",
    "texmf-dist/doc/latex/autonum/autonum.pdf",
    "texmf-dist/doc/latex/autonum/test-autonum.pdf",
  },
  ["makeindex"] = {
    "texmf-dist/doc/man/man1/makeindex.man1.pdf",
    "texmf-dist/doc/man/man1/mkindex.man1.pdf",
    "texmf-dist/doc/support/makeindex/ind.pdf",
    "texmf-dist/doc/support/makeindex/makeindex.pdf",
  },
  ["nmbib"] = {
    "texmf-dist/doc/latex/nmbib/Makefile",
    "texmf-dist/doc/latex/nmbib/README",
    "texmf-dist/doc/latex/nmbib/nmbib.pdf",
    "texmf-dist/doc/latex/nmbib/nmsample.pdf",
  },
  ["isodoc"] = {
    "texmf-dist/doc/latex/isodoc/README",
    "texmf-dist/doc/latex/isodoc/isodoc.pdf",
  },
  ["datetime2-spanish"] = {
    "texmf-dist/doc/latex/datetime2-spanish/README",
    "texmf-dist/doc/latex/datetime2-spanish/datetime2-spanish.pdf",
  },
  ["zhnumber"] = {
    "texmf-dist/doc/latex/zhnumber/README.md",
    "texmf-dist/doc/latex/zhnumber/zhnumber.pdf",
  },
  ["scalerel"] = {
    "texmf-dist/doc/latex/scalerel/README",
    "texmf-dist/doc/latex/scalerel/scalerel.pdf",
  },
  ["mongolian-babel"] = {
    "texmf-dist/doc/latex/mongolian-babel/README",
    "texmf-dist/doc/latex/mongolian-babel/mongolian.pdf",
  },
  ["todonotes"] = {
    "texmf-dist/doc/latex/todonotes/README",
    "texmf-dist/doc/latex/todonotes/examples/alterAppearenceOfListOfTodos.pdf",
    "texmf-dist/doc/latex/todonotes/examples/externalize.pdf",
    "texmf-dist/doc/latex/todonotes/examples/saveColorByUsingLayers.pdf",
    "texmf-dist/doc/latex/todonotes/todonotes.pdf",
  },
  ["dlfltxb"] = {
    "texmf-dist/doc/latex/dlfltxb/README",
    "texmf-dist/doc/latex/dlfltxb/dlfltxbcodetips.pdf",
    "texmf-dist/doc/latex/dlfltxb/dlfltxbmarkup-showkeys.pdf",
    "texmf-dist/doc/latex/dlfltxb/dlfltxbmarkup.pdf",
    "texmf-dist/doc/latex/dlfltxb/dlfltxbmisc.pdf",
    "texmf-dist/doc/latex/dlfltxb/dlfltxbtocconfig.pdf",
  },
  ["mhchem"] = {
    "texmf-dist/doc/latex/mhchem/README",
    "texmf-dist/doc/latex/mhchem/lppl-1-3c.txt",
    "texmf-dist/doc/latex/mhchem/manifest.txt",
    "texmf-dist/doc/latex/mhchem/mhchem.pdf",
  },
  ["karnaugh-map"] = {
    "texmf-dist/doc/latex/karnaugh-map/README.md",
    "texmf-dist/doc/latex/karnaugh-map/karnaugh-map.pdf",
  },
  ["bangorcsthesis"] = {
    "texmf-dist/doc/latex/bangorcsthesis/README",
    "texmf-dist/doc/latex/bangorcsthesis/bangorcsthesis.pdf",
  },
  ["biblatex-luh-ipw"] = {
    "texmf-dist/doc/latex/biblatex-luh-ipw/README",
    "texmf-dist/doc/latex/biblatex-luh-ipw/de-biblatex-luh-ipw.pdf",
  },
  ["fast-diagram"] = {
    "texmf-dist/doc/latex/fast-diagram/README",
    "texmf-dist/doc/latex/fast-diagram/help.pdf",
  },
  ["epsdice"] = {
    "texmf-dist/doc/latex/epsdice/README",
    "texmf-dist/doc/latex/epsdice/dice.pdf",
    "texmf-dist/doc/latex/epsdice/epsdice.pdf",
  },
  ["calxxxx-yyyy"] = {
    "texmf-dist/doc/latex/calxxxx-yyyy/README",
    "texmf-dist/doc/latex/calxxxx-yyyy/cal2018-2035_DE.pdf",
    "texmf-dist/doc/latex/calxxxx-yyyy/cal2018-2035_DK.pdf",
    "texmf-dist/doc/latex/calxxxx-yyyy/cal2018-2035_EN.pdf",
    "texmf-dist/doc/latex/calxxxx-yyyy/calxxxx-yyyy-doc.pdf",
  },
  ["hyphen-hungarian"] = {
    "texmf-dist/doc/generic/huhyphen/huhyphn.pdf",
    "texmf-dist/doc/generic/hyph-utf8/hu/huhyphn.pdf",
  },
  ["iffont"] = {
    "texmf-dist/doc/latex/iffont/README.md",
    "texmf-dist/doc/latex/iffont/iffont.pdf",
  },
  ["biblatex-juradiss"] = {
    "texmf-dist/doc/latex/biblatex-juradiss/README",
    "texmf-dist/doc/latex/biblatex-juradiss/biblatex-juradiss.pdf",
  },
  ["algobox"] = {
    "texmf-dist/doc/latex/algobox/LICENSE",
    "texmf-dist/doc/latex/algobox/README.md",
    "texmf-dist/doc/latex/algobox/algobox.pdf",
  },
  ["mensa-tex"] = {
    "texmf-dist/doc/latex/mensa-tex/README.md",
    "texmf-dist/doc/latex/mensa-tex/doc/mensa-tex-doc.pdf",
    "texmf-dist/doc/latex/mensa-tex/doc/mensa-tex-example.pdf",
  },
  ["pst-abspos"] = {
    "texmf-dist/doc/generic/pst-abspos/Changes",
    "texmf-dist/doc/generic/pst-abspos/README",
    "texmf-dist/doc/generic/pst-abspos/pst-abspos-doc.pdf",
  },
  ["babel-galician"] = {
    "texmf-dist/doc/generic/babel-galician/galician.pdf",
  },
  ["pacioli"] = {
    "texmf-dist/doc/fonts/pacioli/README",
    "texmf-dist/doc/fonts/pacioli/tryfont.ps",
  },
  ["smalltableof"] = {
    "texmf-dist/doc/latex/smalltableof/README",
    "texmf-dist/doc/latex/smalltableof/smalltableof-doc-fr.pdf",
  },
  ["beton"] = {
    "texmf-dist/doc/latex/beton/beton.pdf",
    "texmf-dist/doc/latex/beton/legal.txt",
  },
  ["cloze"] = {
    "texmf-dist/doc/lualatex/cloze/README",
    "texmf-dist/doc/lualatex/cloze/README.md",
    "texmf-dist/doc/lualatex/cloze/cloze.pdf",
  },
  ["pax"] = {
    "texmf-dist/doc/latex/pax/README",
  },
  ["lt3graph"] = {
    "texmf-dist/doc/latex/lt3graph/README",
    "texmf-dist/doc/latex/lt3graph/lt3graph.pdf",
  },
  ["ghsystem"] = {
    "texmf-dist/doc/latex/ghsystem/README",
    "texmf-dist/doc/latex/ghsystem/ghsystem_en.pdf",
  },
  ["pauldoc"] = {
    "texmf-dist/doc/latex/pauldoc/README",
    "texmf-dist/doc/latex/pauldoc/pauldoc.pdf",
  },
  ["boisik"] = {
    "texmf-dist/doc/fonts/boisik/README",
    "texmf-dist/doc/fonts/boisik/example/boisik-idiot.pdf",
    "texmf-dist/doc/fonts/boisik/example/boisik.pdf",
    "texmf-dist/doc/fonts/boisik/example/table.pdf",
    "texmf-dist/doc/fonts/boisik/example/testfont.pdf",
  },
  ["patgen2-tutorial"] = {
    "texmf-dist/doc/support/patgen2-tutorial/patgen2.pdf",
  },
  ["chess"] = {
    "texmf-dist/doc/fonts/chess/changes12",
    "texmf-dist/doc/fonts/chess/copyright",
    "texmf-dist/doc/fonts/chess/installation",
    "texmf-dist/doc/fonts/chess/makefile",
    "texmf-dist/doc/fonts/chess/readme",
  },
  ["universalis"] = {
    "texmf-dist/doc/fonts/universalis/COPYING",
    "texmf-dist/doc/fonts/universalis/NOTICE.txt",
    "texmf-dist/doc/fonts/universalis/README",
    "texmf-dist/doc/fonts/universalis/universalis-samples.pdf",
  },
  ["pst-ovl"] = {
    "texmf-dist/doc/generic/pst-ovl/Changes",
    "texmf-dist/doc/generic/pst-ovl/README",
    "texmf-dist/doc/generic/pst-ovl/pst-ovl-doc.pdf",
  },
  ["paratype"] = {
    "texmf-dist/doc/fonts/paratype/CHANGELOG",
    "texmf-dist/doc/fonts/paratype/OT_TT_Install_E.txt",
    "texmf-dist/doc/fonts/paratype/OT_TT_Install_R.txt",
    "texmf-dist/doc/fonts/paratype/PT_Free_Font_License_eng_1.3.txt",
    "texmf-dist/doc/fonts/paratype/PT_Free_Font_License_rus_1.3.txt",
    "texmf-dist/doc/fonts/paratype/README",
    "texmf-dist/doc/fonts/paratype/manifest.txt",
    "texmf-dist/doc/fonts/paratype/paratype-sample.pdf",
    "texmf-dist/doc/fonts/paratype/paratype.pdf",
  },
  ["quran"] = {
    "texmf-dist/doc/xelatex/quran/README",
    "texmf-dist/doc/xelatex/quran/defaulttext.pdf",
    "texmf-dist/doc/xelatex/quran/quran-doc.pdf",
    "texmf-dist/doc/xelatex/quran/quran-test.pdf",
    "texmf-dist/doc/xelatex/quran/quran-test1.pdf",
    "texmf-dist/doc/xelatex/quran/quran-test2.pdf",
    "texmf-dist/doc/xelatex/quran/uthmanitext.pdf",
  },
  ["patgen"] = {
    "texmf-dist/doc/man/man1/patgen.man1.pdf",
  },
  ["beamertheme-epyt"] = {
    "texmf-dist/doc/latex/beamertheme-epyt/README",
    "texmf-dist/doc/latex/beamertheme-epyt/epyt-demo-cn.pdf",
    "texmf-dist/doc/latex/beamertheme-epyt/epyt-demo.pdf",
  },
  ["pas-crosswords"] = {
    "texmf-dist/doc/latex/pas-crosswords/README",
    "texmf-dist/doc/latex/pas-crosswords/pas-crosswords.pdf",
  },
  ["biblatex-shortfields"] = {
    "texmf-dist/doc/latex/biblatex-shortfields/README",
    "texmf-dist/doc/latex/biblatex-shortfields/documentation/biblatex-shortfields-example.pdf",
    "texmf-dist/doc/latex/biblatex-shortfields/documentation/biblatex-shortfields.pdf",
    "texmf-dist/doc/latex/biblatex-shortfields/documentation/latexmkrc",
    "texmf-dist/doc/latex/biblatex-shortfields/documentation/makefile",
    "texmf-dist/doc/latex/biblatex-shortfields/makefile",
  },
  ["parrun"] = {
    "texmf-dist/doc/latex/parrun/parrun.pdf",
    "texmf-dist/doc/latex/parrun/readme",
  },
  ["paresse"] = {
    "texmf-dist/doc/latex/paresse/paresse-en.pdf",
    "texmf-dist/doc/latex/paresse/paresse-ex-en.pdf",
    "texmf-dist/doc/latex/paresse/paresse-ex-fr.pdf",
    "texmf-dist/doc/latex/paresse/paresse-fr.pdf",
    "texmf-dist/doc/latex/paresse/paresse.pdf",
  },
  ["dot2texi"] = {
    "texmf-dist/doc/latex/dot2texi/README",
    "texmf-dist/doc/latex/dot2texi/dot2texi.pdf",
    "texmf-dist/doc/latex/dot2texi/examples/d2tpstexamples.pdf",
    "texmf-dist/doc/latex/dot2texi/examples/d2ttikzexamples.pdf",
    "texmf-dist/doc/latex/dot2texi/examples/docgraphs.pdf",
    "texmf-dist/doc/latex/dot2texi/examples/docgraphsorig.pdf",
    "texmf-dist/doc/latex/dot2texi/gpl.txt",
  },
  ["papermas"] = {
    "texmf-dist/doc/latex/papermas/README",
    "texmf-dist/doc/latex/papermas/papermas-example.pdf",
    "texmf-dist/doc/latex/papermas/papermas.pdf",
  },
  ["papercdcase"] = {
    "texmf-dist/doc/latex/papercdcase/papercdcase.pdf",
  },
  ["dateiliste"] = {
    "texmf-dist/doc/latex/dateiliste/README",
    "texmf-dist/doc/latex/dateiliste/README-de",
    "texmf-dist/doc/latex/dateiliste/README-en",
    "texmf-dist/doc/latex/dateiliste/README-eo",
    "texmf-dist/doc/latex/dateiliste/dateiliste.pdf",
  },
  ["biblatex-sbl"] = {
    "texmf-dist/doc/latex/biblatex-sbl/README.md",
    "texmf-dist/doc/latex/biblatex-sbl/biblatex-sbl-examples.pdf",
    "texmf-dist/doc/latex/biblatex-sbl/biblatex-sbl-ibid.pdf",
    "texmf-dist/doc/latex/biblatex-sbl/biblatex-sbl.pdf",
    "texmf-dist/doc/latex/biblatex-sbl/sbl-paper.pdf",
  },
  ["physics"] = {
    "texmf-dist/doc/latex/physics/README",
    "texmf-dist/doc/latex/physics/physics.pdf",
  },
  ["confproc"] = {
    "texmf-dist/doc/latex/confproc/README",
    "texmf-dist/doc/latex/confproc/confproc.pdf",
    "texmf-dist/doc/latex/confproc/confproc_diag.pdf",
    "texmf-dist/doc/latex/confproc/example/papers/p_001.pdf",
    "texmf-dist/doc/latex/confproc/example/papers/p_003.pdf",
    "texmf-dist/doc/latex/confproc/example/papers/p_005.pdf",
    "texmf-dist/doc/latex/confproc/example/papers/p_007.pdf",
    "texmf-dist/doc/latex/confproc/example/papers/p_009.pdf",
    "texmf-dist/doc/latex/confproc/example/pictures/ex_1stpage.pdf",
  },
  ["pagerange"] = {
    "texmf-dist/doc/latex/pagerange/README",
    "texmf-dist/doc/latex/pagerange/pagerange-guide.pdf",
  },
  ["pagenote"] = {
    "texmf-dist/doc/latex/pagenote/README",
    "texmf-dist/doc/latex/pagenote/pagenote.pdf",
  },
  ["hang"] = {
    "texmf-dist/doc/latex/hang/COPYING",
    "texmf-dist/doc/latex/hang/ChangeLog",
    "texmf-dist/doc/latex/hang/README",
    "texmf-dist/doc/latex/hang/hang.pdf",
    "texmf-dist/doc/latex/hang/sample.pdf",
  },
  ["sffms"] = {
    "texmf-dist/doc/latex/sffms/README",
    "texmf-dist/doc/latex/sffms/sffms_manual.pdf",
  },
  ["padcount"] = {
    "texmf-dist/doc/latex/padcount/CHANGES",
    "texmf-dist/doc/latex/padcount/README",
    "texmf-dist/doc/latex/padcount/lppl.txt",
    "texmf-dist/doc/latex/padcount/padcount.pdf",
  },
  ["padauk"] = {
    "texmf-dist/doc/fonts/padauk/FONTLOG.txt",
    "texmf-dist/doc/fonts/padauk/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/padauk/OFL.txt",
    "texmf-dist/doc/fonts/padauk/README",
    "texmf-dist/doc/fonts/padauk/README.TEXLIVE",
  },
  ["pbibtex-base"] = {
    "texmf-dist/doc/ptex/pbibtex/LICENSE",
    "texmf-dist/doc/ptex/pbibtex/README.md",
    "texmf-dist/doc/ptex/pbibtex/README_original",
    "texmf-dist/doc/ptex/pbibtex/jbibtex.pdf",
    "texmf-dist/doc/ptex/pbibtex/jbtxdoc.pdf",
    "texmf-dist/doc/ptex/pbibtex/jbtxhak.pdf",
  },
  ["overpic"] = {
    "texmf-dist/doc/latex/overpic/README.de.md",
    "texmf-dist/doc/latex/overpic/README.md",
    "texmf-dist/doc/latex/overpic/overpic.pdf",
  },
  ["overlock"] = {
    "texmf-dist/doc/fonts/overlock/License.txt",
    "texmf-dist/doc/fonts/overlock/README",
    "texmf-dist/doc/fonts/overlock/overlock-samples.pdf",
  },
  ["overlays"] = {
    "texmf-dist/doc/latex/overlays/COPYING",
    "texmf-dist/doc/latex/overlays/ChangeLog",
    "texmf-dist/doc/latex/overlays/README",
    "texmf-dist/doc/latex/overlays/overlays.pdf",
    "texmf-dist/doc/latex/overlays/sample.pdf",
  },
  ["outlining"] = {
    "texmf-dist/doc/latex/outlining/README.md",
    "texmf-dist/doc/latex/outlining/outlining.pdf",
  },
  ["moreverb"] = {
    "texmf-dist/doc/latex/moreverb/README",
    "texmf-dist/doc/latex/moreverb/moreverb.pdf",
  },
  ["latexmp"] = {
    "texmf-dist/doc/metapost/latexmp/README",
    "texmf-dist/doc/metapost/latexmp/latexmp.pdf",
  },
  ["othelloboard"] = {
    "texmf-dist/doc/latex/othelloboard/README",
    "texmf-dist/doc/latex/othelloboard/example-rose-chps1-2.pdf",
    "texmf-dist/doc/latex/othelloboard/othelloboard.pdf",
  },
  ["ot-tableau"] = {
    "texmf-dist/doc/latex/ot-tableau/README",
    "texmf-dist/doc/latex/ot-tableau/ot-tableau.pdf",
  },
  ["fundus-calligra"] = {
    "texmf-dist/doc/latex/fundus-calligra/calligra.pdf",
  },
  ["ordinalpt"] = {
    "texmf-dist/doc/latex/ordinalpt/README",
    "texmf-dist/doc/latex/ordinalpt/ordinalpt.pdf",
  },
  ["lccaps"] = {
    "texmf-dist/doc/latex/lccaps/README.md",
    "texmf-dist/doc/latex/lccaps/lccaps.pdf",
  },
  ["ibycus-babel"] = {
    "texmf-dist/doc/latex/ibycus-babel/README",
    "texmf-dist/doc/latex/ibycus-babel/ibycus-babel.pdf",
  },
  ["usebib"] = {
    "texmf-dist/doc/latex/usebib/README",
    "texmf-dist/doc/latex/usebib/usebib.pdf",
  },
  ["gentium-tug"] = {
    "texmf-dist/doc/fonts/gentium-tug/ChangeLog",
    "texmf-dist/doc/fonts/gentium-tug/FONTLOG.txt",
    "texmf-dist/doc/fonts/gentium-tug/Gentium/FONTLOG.txt",
    "texmf-dist/doc/fonts/gentium-tug/Gentium/GENTIUM-FAQ.txt",
    "texmf-dist/doc/fonts/gentium-tug/Gentium/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gentium-tug/Gentium/OFL.txt",
    "texmf-dist/doc/fonts/gentium-tug/Gentium/QUOTES.txt",
    "texmf-dist/doc/fonts/gentium-tug/Gentium/README.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumBasic/FONTLOG.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumBasic/GENTIUM-FAQ.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumBasic/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumBasic/OFL.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumPlus/FONTLOG.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumPlus/GENTIUM-FAQ.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumPlus/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumPlus/OFL.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumPlus/README.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumPlus/documentation/DOCUMENTATION.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumPlus/documentation/GentiumPlus-features.pdf",
    "texmf-dist/doc/fonts/gentium-tug/GentiumPlusCompact/FONTLOG.txt",
    "texmf-dist/doc/fonts/gentium-tug/GentiumPlusCompact/README.txt",
    "texmf-dist/doc/fonts/gentium-tug/Makefile",
    "texmf-dist/doc/fonts/gentium-tug/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gentium-tug/OFL.txt",
    "texmf-dist/doc/fonts/gentium-tug/README",
    "texmf-dist/doc/fonts/gentium-tug/gentium.pdf",
  },
  ["onrannual"] = {
    "texmf-dist/doc/latex/onrannual/README",
    "texmf-dist/doc/latex/onrannual/samplefigure.pdf",
    "texmf-dist/doc/latex/onrannual/skeleton.pdf",
  },
  ["onlyamsmath"] = {
    "texmf-dist/doc/latex/onlyamsmath/ChangeLog",
    "texmf-dist/doc/latex/onlyamsmath/Makefile",
    "texmf-dist/doc/latex/onlyamsmath/README",
    "texmf-dist/doc/latex/onlyamsmath/onlyamsmath.pdf",
  },
  ["fancyhandout"] = {
    "texmf-dist/doc/latex/fancyhandout/README.md",
    "texmf-dist/doc/latex/fancyhandout/fancyhandout-doc.pdf",
  },
  ["xii-lat"] = {
    "texmf-dist/doc/plain/xii-lat/README.txt",
  },
  ["pageslts"] = {
    "texmf-dist/doc/latex/pageslts/README",
    "texmf-dist/doc/latex/pageslts/pageslts-example.pdf",
    "texmf-dist/doc/latex/pageslts/pageslts.pdf",
  },
  ["omega"] = {
    "texmf-dist/doc/omega/base/torture.ps",
  },
  ["olsak-misc"] = {
    "texmf-dist/doc/generic/olsak-misc/README",
  },
  ["oldstyle"] = {
    "texmf-dist/doc/latex/oldstyle/oldstyle.pdf",
  },
  ["oldlatin"] = {
    "texmf-dist/doc/fonts/oldlatin/README",
    "texmf-dist/doc/fonts/oldlatin/oldlatin.pdf",
    "texmf-dist/doc/fonts/oldlatin/test_ol_all.pdf",
    "texmf-dist/doc/fonts/oldlatin/test_ol_bf.pdf",
    "texmf-dist/doc/fonts/oldlatin/test_ol_rm.pdf",
    "texmf-dist/doc/fonts/oldlatin/test_ol_sl.pdf",
    "texmf-dist/doc/fonts/oldlatin/test_ol_ss.pdf",
    "texmf-dist/doc/fonts/oldlatin/test_ol_tt.pdf",
  },
  ["old-arrows"] = {
    "texmf-dist/doc/fonts/old-arrows/README",
    "texmf-dist/doc/fonts/old-arrows/old-arrows-italian.pdf",
    "texmf-dist/doc/fonts/old-arrows/old-arrows.pdf",
  },
  ["prettyref"] = {
    "texmf-dist/doc/latex/prettyref/README",
    "texmf-dist/doc/latex/prettyref/prettyref.pdf",
  },
  ["ofs"] = {
    "texmf-dist/doc/generic/ofs/changes.txt",
    "texmf-dist/doc/generic/ofs/eurotex2003-ofs.pdf",
    "texmf-dist/doc/generic/ofs/ofs-slt.pdf",
    "texmf-dist/doc/generic/ofs/ofsdoc-e.pdf",
    "texmf-dist/doc/generic/ofs/ofsdoc.pdf",
    "texmf-dist/doc/generic/ofs/readme.ofs",
  },
  ["obnov"] = {
    "texmf-dist/doc/fonts/obnov/README",
    "texmf-dist/doc/fonts/obnov/example_obn.pdf",
  },
  ["oberdiek"] = {
    "texmf-dist/doc/latex/oberdiek/accsupp.pdf",
    "texmf-dist/doc/latex/oberdiek/aliascnt.pdf",
    "texmf-dist/doc/latex/oberdiek/alphalph.pdf",
    "texmf-dist/doc/latex/oberdiek/askinclude.pdf",
    "texmf-dist/doc/latex/oberdiek/atbegshi.pdf",
    "texmf-dist/doc/latex/oberdiek/atenddvi.pdf",
    "texmf-dist/doc/latex/oberdiek/attachfile2.pdf",
    "texmf-dist/doc/latex/oberdiek/atveryend.pdf",
    "texmf-dist/doc/latex/oberdiek/auxhook.pdf",
    "texmf-dist/doc/latex/oberdiek/bigintcalc.pdf",
    "texmf-dist/doc/latex/oberdiek/bitset.pdf",
    "texmf-dist/doc/latex/oberdiek/bmpsize.pdf",
    "texmf-dist/doc/latex/oberdiek/bookmark.pdf",
    "texmf-dist/doc/latex/oberdiek/catchfile.pdf",
    "texmf-dist/doc/latex/oberdiek/centernot.pdf",
    "texmf-dist/doc/latex/oberdiek/chemarr.pdf",
    "texmf-dist/doc/latex/oberdiek/classlist.pdf",
    "texmf-dist/doc/latex/oberdiek/colonequals.pdf",
    "texmf-dist/doc/latex/oberdiek/dvipscol.pdf",
    "texmf-dist/doc/latex/oberdiek/embedfile.pdf",
    "texmf-dist/doc/latex/oberdiek/engord.pdf",
    "texmf-dist/doc/latex/oberdiek/enparen.pdf",
    "texmf-dist/doc/latex/oberdiek/eolgrab.pdf",
    "texmf-dist/doc/latex/oberdiek/epstopdf.pdf",
    "texmf-dist/doc/latex/oberdiek/etexcmds.pdf",
    "texmf-dist/doc/latex/oberdiek/fibnum.pdf",
    "texmf-dist/doc/latex/oberdiek/flags.pdf",
    "texmf-dist/doc/latex/oberdiek/gettitlestring.pdf",
    "texmf-dist/doc/latex/oberdiek/grfext.pdf",
    "texmf-dist/doc/latex/oberdiek/grffile.pdf",
    "texmf-dist/doc/latex/oberdiek/hobsub.pdf",
    "texmf-dist/doc/latex/oberdiek/hologo.pdf",
    "texmf-dist/doc/latex/oberdiek/holtxdoc.pdf",
    "texmf-dist/doc/latex/oberdiek/hopatch.pdf",
    "texmf-dist/doc/latex/oberdiek/hycolor.pdf",
    "texmf-dist/doc/latex/oberdiek/hypbmsec.pdf",
    "texmf-dist/doc/latex/oberdiek/hypcap.pdf",
    "texmf-dist/doc/latex/oberdiek/hypdestopt.pdf",
    "texmf-dist/doc/latex/oberdiek/hypdoc.pdf",
    "texmf-dist/doc/latex/oberdiek/hypgotoe.pdf",
    "texmf-dist/doc/latex/oberdiek/hyphsubst.pdf",
    "texmf-dist/doc/latex/oberdiek/ifdraft.pdf",
    "texmf-dist/doc/latex/oberdiek/iflang.pdf",
    "texmf-dist/doc/latex/oberdiek/ifpdf.pdf",
    "texmf-dist/doc/latex/oberdiek/ifvtex.pdf",
    "texmf-dist/doc/latex/oberdiek/infwarerr.pdf",
    "texmf-dist/doc/latex/oberdiek/inputenx-licrcmds.txt",
    "texmf-dist/doc/latex/oberdiek/inputenx-utf8enc.txt",
    "texmf-dist/doc/latex/oberdiek/inputenx.pdf",
    "texmf-dist/doc/latex/oberdiek/intcalc.pdf",
    "texmf-dist/doc/latex/oberdiek/kvdefinekeys.pdf",
    "texmf-dist/doc/latex/oberdiek/kvoptions.pdf",
    "texmf-dist/doc/latex/oberdiek/kvsetkeys.pdf",
    "texmf-dist/doc/latex/oberdiek/letltxmacro.pdf",
    "texmf-dist/doc/latex/oberdiek/listingsutf8.pdf",
    "texmf-dist/doc/latex/oberdiek/ltxcmds.pdf",
    "texmf-dist/doc/latex/oberdiek/luacolor.pdf",
    "texmf-dist/doc/latex/oberdiek/luatex.pdf",
    "texmf-dist/doc/latex/oberdiek/magicnum.pdf",
    "texmf-dist/doc/latex/oberdiek/magicnum.txt",
    "texmf-dist/doc/latex/oberdiek/makerobust.pdf",
    "texmf-dist/doc/latex/oberdiek/mleftright.pdf",
    "texmf-dist/doc/latex/oberdiek/oberdiek.pdf",
    "texmf-dist/doc/latex/oberdiek/pagegrid.pdf",
    "texmf-dist/doc/latex/oberdiek/pagesel.pdf",
    "texmf-dist/doc/latex/oberdiek/pdfcol.pdf",
    "texmf-dist/doc/latex/oberdiek/pdfcolfoot.pdf",
    "texmf-dist/doc/latex/oberdiek/pdfcolmk.pdf",
    "texmf-dist/doc/latex/oberdiek/pdfcolparallel.pdf",
    "texmf-dist/doc/latex/oberdiek/pdfcolparcolumns.pdf",
    "texmf-dist/doc/latex/oberdiek/pdfcrypt.pdf",
    "texmf-dist/doc/latex/oberdiek/pdfescape.pdf",
    "texmf-dist/doc/latex/oberdiek/pdflscape.pdf",
    "texmf-dist/doc/latex/oberdiek/pdfrender.pdf",
    "texmf-dist/doc/latex/oberdiek/pdftexcmds.pdf",
    "texmf-dist/doc/latex/oberdiek/picture.pdf",
    "texmf-dist/doc/latex/oberdiek/pmboxdraw.pdf",
    "texmf-dist/doc/latex/oberdiek/protecteddef.pdf",
    "texmf-dist/doc/latex/oberdiek/refcount.pdf",
    "texmf-dist/doc/latex/oberdiek/rerunfilecheck.pdf",
    "texmf-dist/doc/latex/oberdiek/resizegather.pdf",
    "texmf-dist/doc/latex/oberdiek/rotchiffre.pdf",
    "texmf-dist/doc/latex/oberdiek/scrindex.pdf",
    "texmf-dist/doc/latex/oberdiek/selinput.pdf",
    "texmf-dist/doc/latex/oberdiek/setouterhbox.pdf",
    "texmf-dist/doc/latex/oberdiek/settobox.pdf",
    "texmf-dist/doc/latex/oberdiek/soulutf8.pdf",
    "texmf-dist/doc/latex/oberdiek/stackrel.pdf",
    "texmf-dist/doc/latex/oberdiek/stampinclude.pdf",
    "texmf-dist/doc/latex/oberdiek/stringenc.pdf",
    "texmf-dist/doc/latex/oberdiek/tabularht.pdf",
    "texmf-dist/doc/latex/oberdiek/tabularkv.pdf",
    "texmf-dist/doc/latex/oberdiek/telprint.pdf",
    "texmf-dist/doc/latex/oberdiek/test/pdflscape-test6.txt",
    "texmf-dist/doc/latex/oberdiek/thepdfnumber.pdf",
    "texmf-dist/doc/latex/oberdiek/transparent.pdf",
    "texmf-dist/doc/latex/oberdiek/twoopt.pdf",
    "texmf-dist/doc/latex/oberdiek/uniquecounter.pdf",
    "texmf-dist/doc/latex/oberdiek/zref.pdf",
  },
  ["numprint"] = {
    "texmf-dist/doc/latex/numprint/Makefile",
    "texmf-dist/doc/latex/numprint/README",
    "texmf-dist/doc/latex/numprint/nbaseprt.pdf",
    "texmf-dist/doc/latex/numprint/numprint.pdf",
  },
  ["jneurosci"] = {
    "texmf-dist/doc/latex/jneurosci/README",
    "texmf-dist/doc/latex/jneurosci/jneurosci.pdf",
  },
  ["numname"] = {
    "texmf-dist/doc/latex/numname/README",
    "texmf-dist/doc/latex/numname/numname.pdf",
  },
  ["numberedblock"] = {
    "texmf-dist/doc/latex/numberedblock/README",
    "texmf-dist/doc/latex/numberedblock/testnumberedblock.pdf",
  },
  ["nlctdoc"] = {
    "texmf-dist/doc/latex/nlctdoc/README",
  },
  ["interchar"] = {
    "texmf-dist/doc/xelatex/interchar/README",
    "texmf-dist/doc/xelatex/interchar/interchar.pdf",
    "texmf-dist/doc/xelatex/interchar/interchardemo1.pdf",
    "texmf-dist/doc/xelatex/interchar/interchartest.pdf",
  },
  ["background"] = {
    "texmf-dist/doc/latex/background/README",
    "texmf-dist/doc/latex/background/background.pdf",
  },
  ["nox"] = {
    "texmf-dist/doc/latex/nox/README",
    "texmf-dist/doc/latex/nox/nox.pdf",
  },
  ["metatex"] = {
    "texmf-dist/doc/plain/metatex/README",
    "texmf-dist/doc/plain/metatex/gpl.txt",
    "texmf-dist/doc/plain/metatex/mtpaper.pdf",
  },
  ["nowidow"] = {
    "texmf-dist/doc/latex/nowidow/README",
    "texmf-dist/doc/latex/nowidow/nowidow.pdf",
  },
  ["anysize"] = {
    "texmf-dist/doc/latex/anysize/README",
    "texmf-dist/doc/latex/anysize/anysize.pdf",
  },
  ["mintspirit"] = {
    "texmf-dist/doc/fonts/mintspirit/MintSpirit-Cat.pdf",
    "texmf-dist/doc/fonts/mintspirit/OFL.txt",
    "texmf-dist/doc/fonts/mintspirit/README",
    "texmf-dist/doc/fonts/mintspirit/mintspirit-samples.pdf",
    "texmf-dist/doc/fonts/mintspirit/mintspirit2-samples.pdf",
  },
  ["cc-pl"] = {
    "texmf-dist/doc/fonts/cc-pl/README.cc-pl",
  },
  ["babel-hebrew"] = {
    "texmf-dist/doc/generic/babel-hebrew/00readme.heb",
    "texmf-dist/doc/generic/babel-hebrew/heb209.pdf",
    "texmf-dist/doc/generic/babel-hebrew/hebinp.pdf",
    "texmf-dist/doc/generic/babel-hebrew/hebrew.pdf",
  },
  ["addlines"] = {
    "texmf-dist/doc/latex/addlines/README",
    "texmf-dist/doc/latex/addlines/addlines.pdf",
  },
  ["noto"] = {
    "texmf-dist/doc/fonts/noto/LICENSE_OFL.txt",
    "texmf-dist/doc/fonts/noto/README",
    "texmf-dist/doc/fonts/noto/noto-samples.pdf",
  },
  ["notespages"] = {
    "texmf-dist/doc/latex/notespages/README",
    "texmf-dist/doc/latex/notespages/notespages.pdf",
    "texmf-dist/doc/latex/notespages/np-test.pdf",
  },
  ["notes"] = {
    "texmf-dist/doc/latex/notes/makedoc",
    "texmf-dist/doc/latex/notes/notes.pdf",
  },
  ["polexpr"] = {
    "texmf-dist/doc/latex/polexpr/README.md",
    "texmf-dist/doc/latex/polexpr/polexpr.html",
    "texmf-dist/doc/latex/polexpr/polexpr.txt",
  },
  ["boites"] = {
    "texmf-dist/doc/latex/boites/Makefile",
    "texmf-dist/doc/latex/boites/README",
    "texmf-dist/doc/latex/boites/README.docu",
    "texmf-dist/doc/latex/boites/boites.pdf",
    "texmf-dist/doc/latex/boites/demo.pdf",
  },
  ["codicefiscaleitaliano"] = {
    "texmf-dist/doc/latex/codicefiscaleitaliano/README",
    "texmf-dist/doc/latex/codicefiscaleitaliano/codicefiscaleitaliano.pdf",
  },
  ["context-typescripts"] = {
    "texmf-dist/doc/context/third/typescripts/README",
    "texmf-dist/doc/context/third/typescripts/VERSION",
  },
  ["nopageno"] = {
    "texmf-dist/doc/latex/nopageno/nopageno.pdf",
  },
  ["nomentbl"] = {
    "texmf-dist/doc/latex/nomentbl/README",
    "texmf-dist/doc/latex/nomentbl/example.pdf",
    "texmf-dist/doc/latex/nomentbl/nomentbl.pdf",
  },
  ["noitcrul"] = {
    "texmf-dist/doc/latex/noitcrul/README",
    "texmf-dist/doc/latex/noitcrul/noitcrul.pdf",
  },
  ["context-algorithmic"] = {
    "texmf-dist/doc/context/third/algorithmic/VERSION",
  },
  ["noconflict"] = {
    "texmf-dist/doc/latex/noconflict/README",
  },
  ["beamerposter"] = {
    "texmf-dist/doc/latex/beamerposter/README",
    "texmf-dist/doc/latex/beamerposter/beamerposter.pdf",
  },
  ["nimbus15"] = {
    "texmf-dist/doc/fonts/nimbus15/COPYING",
    "texmf-dist/doc/fonts/nimbus15/LICENSE",
    "texmf-dist/doc/fonts/nimbus15/README",
    "texmf-dist/doc/fonts/nimbus15/nimbus15-doc.pdf",
    "texmf-dist/doc/fonts/nimbus15/ot2trans.pdf",
  },
  ["carolmin-ps"] = {
    "texmf-dist/doc/fonts/carolmin-ps/README",
  },
  ["nihbiosketch"] = {
    "texmf-dist/doc/latex/nihbiosketch/README.md",
    "texmf-dist/doc/latex/nihbiosketch/example-nihbiosketch.pdf",
  },
  ["cite"] = {
    "texmf-dist/doc/latex/cite/README",
    "texmf-dist/doc/latex/cite/chapterbib.pdf",
    "texmf-dist/doc/latex/cite/cite.pdf",
  },
  ["dashbox"] = {
    "texmf-dist/doc/latex/dashbox/dashbox.pdf",
  },
  ["niceframe"] = {
    "texmf-dist/doc/latex/niceframe/niceframe.pdf",
  },
  ["nfssext-cfr"] = {
    "texmf-dist/doc/latex/nfssext-cfr/README",
    "texmf-dist/doc/latex/nfssext-cfr/nfssext-cfr.pdf",
  },
  ["xltabular"] = {
    "texmf-dist/doc/latex/xltabular/Changes",
    "texmf-dist/doc/latex/xltabular/README.md",
    "texmf-dist/doc/latex/xltabular/xltabular-doc.pdf",
  },
  ["newverbs"] = {
    "texmf-dist/doc/latex/newverbs/README",
    "texmf-dist/doc/latex/newverbs/newverbs.pdf",
  },
  ["easyreview"] = {
    "texmf-dist/doc/latex/easyreview/README.txt",
    "texmf-dist/doc/latex/easyreview/doc/easyReview.pdf",
  },
  ["newvbtm"] = {
    "texmf-dist/doc/latex/newvbtm/README",
    "texmf-dist/doc/latex/newvbtm/newvbtm-man.pdf",
  },
  ["newunicodechar"] = {
    "texmf-dist/doc/latex/newunicodechar/README",
    "texmf-dist/doc/latex/newunicodechar/newunicodechar.pdf",
  },
  ["newtxsf"] = {
    "texmf-dist/doc/fonts/newtxsf/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/newtxsf/OFL.txt",
    "texmf-dist/doc/fonts/newtxsf/README",
    "texmf-dist/doc/fonts/newtxsf/newtxsf-doc.pdf",
  },
  ["newspaper"] = {
    "texmf-dist/doc/latex/newspaper/Figure1.pdf",
    "texmf-dist/doc/latex/newspaper/Figure2.pdf",
    "texmf-dist/doc/latex/newspaper/README",
    "texmf-dist/doc/latex/newspaper/newspaper.pdf",
  },
  ["tasks"] = {
    "texmf-dist/doc/latex/tasks/README",
    "texmf-dist/doc/latex/tasks/tasks_en.pdf",
  },
  ["nevelok"] = {
    "texmf-dist/doc/latex/nevelok/README.md",
    "texmf-dist/doc/latex/nevelok/nevelok.pdf",
  },
  ["datetime2-slovene"] = {
    "texmf-dist/doc/latex/datetime2-slovene/README",
    "texmf-dist/doc/latex/datetime2-slovene/datetime2-slovene.pdf",
  },
  ["ndsu-thesis"] = {
    "texmf-dist/doc/latex/ndsu-thesis/README",
    "texmf-dist/doc/latex/ndsu-thesis/ndsu-thesis.pdf",
  },
  ["sudokubundle"] = {
    "texmf-dist/doc/latex/sudokubundle/README",
    "texmf-dist/doc/latex/sudokubundle/sudokubundle.pdf",
  },
  ["navigator"] = {
    "texmf-dist/doc/generic/navigator/README",
    "texmf-dist/doc/generic/navigator/navigator-doc.pdf",
  },
  ["libgreek"] = {
    "texmf-dist/doc/latex/libgreek/README",
    "texmf-dist/doc/latex/libgreek/libgreek.pdf",
  },
  ["bigints"] = {
    "texmf-dist/doc/latex/bigints/Makefile",
    "texmf-dist/doc/latex/bigints/README",
    "texmf-dist/doc/latex/bigints/bigints.pdf",
  },
  ["isorot"] = {
    "texmf-dist/doc/latex/isorot/README",
    "texmf-dist/doc/latex/isorot/rotman.pdf",
  },
  ["etoolbox"] = {
    "texmf-dist/doc/latex/etoolbox/README.md",
    "texmf-dist/doc/latex/etoolbox/etoolbox.pdf",
  },
  ["cstypo"] = {
    "texmf-dist/doc/lualatex/cstypo/LICENSE",
    "texmf-dist/doc/lualatex/cstypo/README.md",
    "texmf-dist/doc/lualatex/cstypo/cstypo.pdf",
    "texmf-dist/doc/lualatex/cstypo/cstypotest.pdf",
  },
  ["natbib"] = {
    "texmf-dist/doc/latex/natbib/README.1st",
    "texmf-dist/doc/latex/natbib/README.v831b",
    "texmf-dist/doc/latex/natbib/natbib.pdf",
    "texmf-dist/doc/latex/natbib/natnotes.pdf",
  },
  ["namespc"] = {
    "texmf-dist/doc/latex/namespc/README",
    "texmf-dist/doc/latex/namespc/namespc.pdf",
  },
  ["na-box"] = {
    "texmf-dist/doc/xelatex/na-box/README",
    "texmf-dist/doc/xelatex/na-box/na-box_doc-back.pdf",
    "texmf-dist/doc/xelatex/na-box/na-box_doc.pdf",
  },
  ["mynsfc"] = {
    "texmf-dist/doc/xelatex/mynsfc/my-nsfc-proposal.pdf",
    "texmf-dist/doc/xelatex/mynsfc/mynsfc.pdf",
  },
  ["abnt"] = {
    "texmf-dist/doc/latex/abnt/README.md",
    "texmf-dist/doc/latex/abnt/abnt-doc.pdf",
  },
  ["mylatexformat"] = {
    "texmf-dist/doc/latex/mylatexformat/README",
    "texmf-dist/doc/latex/mylatexformat/mylatexformat.pdf",
  },
  ["l3packages"] = {
    "texmf-dist/doc/latex/l3packages/README.md",
    "texmf-dist/doc/latex/l3packages/l3keys2e/l3keys2e.pdf",
    "texmf-dist/doc/latex/l3packages/xfp/xfp.pdf",
    "texmf-dist/doc/latex/l3packages/xfrac/xfrac.pdf",
    "texmf-dist/doc/latex/l3packages/xparse/xparse.pdf",
    "texmf-dist/doc/latex/l3packages/xtemplate/xtemplate.pdf",
  },
  ["mwe"] = {
    "texmf-dist/doc/latex/mwe/INSTALL",
    "texmf-dist/doc/latex/mwe/README",
    "texmf-dist/doc/latex/mwe/mwe.pdf",
  },
  ["mnsymbol"] = {
    "texmf-dist/doc/latex/mnsymbol/MnSymbol.pdf",
    "texmf-dist/doc/latex/mnsymbol/README",
  },
  ["mwcls"] = {
    "texmf-dist/doc/latex/mwcls/CZYTAJ",
    "texmf-dist/doc/latex/mwcls/ChangeLog",
    "texmf-dist/doc/latex/mwcls/README",
    "texmf-dist/doc/latex/mwcls/mwclsdoc.pdf",
  },
  ["codesection"] = {
    "texmf-dist/doc/latex/codesection/README",
    "texmf-dist/doc/latex/codesection/codesection.pdf",
  },
  ["footmisc"] = {
    "texmf-dist/doc/latex/footmisc/README",
    "texmf-dist/doc/latex/footmisc/footmisc.pdf",
  },
  ["bxcalc"] = {
    "texmf-dist/doc/latex/bxcalc/LICENSE",
    "texmf-dist/doc/latex/bxcalc/README.md",
    "texmf-dist/doc/latex/bxcalc/bxcalcize.pdf",
    "texmf-dist/doc/latex/bxcalc/bxcalcux.pdf",
    "texmf-dist/doc/latex/bxcalc/sample-bxcalc.pdf",
  },
  ["grid"] = {
    "texmf-dist/doc/latex/grid/README",
    "texmf-dist/doc/latex/grid/grid.pdf",
    "texmf-dist/doc/latex/grid/manifest.txt",
  },
  ["musuos"] = {
    "texmf-dist/doc/latex/musuos/README",
    "texmf-dist/doc/latex/musuos/musuos.pdf",
  },
  ["itnumpar"] = {
    "texmf-dist/doc/latex/itnumpar/README",
    "texmf-dist/doc/latex/itnumpar/itnumpar.pdf",
  },
  ["context-title"] = {
    "texmf-dist/doc/context/third/title/VERSION",
    "texmf-dist/doc/context/third/title/title-doc.pdf",
  },
  ["assoccnt"] = {
    "texmf-dist/doc/latex/assoccnt/README",
    "texmf-dist/doc/latex/assoccnt/assoccnt_doc.pdf",
    "texmf-dist/doc/latex/assoccnt/assoccnt_example.pdf",
  },
  ["etex"] = {
    "texmf-dist/doc/etex/base/NTS-FAQ",
    "texmf-dist/doc/etex/base/etex_man.pdf",
    "texmf-dist/doc/etex/base/etex_ref.html",
    "texmf-dist/doc/etex/base/etex_src.html",
    "texmf-dist/doc/etex/base/legal.html",
    "texmf-dist/doc/etex/base/nts-group.html",
    "texmf-dist/doc/man/man1/etex.man1.pdf",
  },
  ["cooking"] = {
    "texmf-dist/doc/latex/cooking/COPYING",
    "texmf-dist/doc/latex/cooking/README",
    "texmf-dist/doc/latex/cooking/cooking.pdf",
  },
  ["kurdishlipsum"] = {
    "texmf-dist/doc/xelatex/kurdishlipsum/README",
    "texmf-dist/doc/xelatex/kurdishlipsum/kurdishlipsum.pdf",
  },
  ["beamertheme-metropolis"] = {
    "texmf-dist/doc/latex/beamertheme-metropolis/README.md",
    "texmf-dist/doc/latex/beamertheme-metropolis/demo.pdf",
    "texmf-dist/doc/latex/beamertheme-metropolis/metropolistheme.pdf",
  },
  ["aomart"] = {
    "texmf-dist/doc/latex/aomart/Makefile",
    "texmf-dist/doc/latex/aomart/README",
    "texmf-dist/doc/latex/aomart/aomart.pdf",
    "texmf-dist/doc/latex/aomart/aomfrench.pdf",
    "texmf-dist/doc/latex/aomart/aomsample.pdf",
    "texmf-dist/doc/latex/aomart/aomsample1.pdf",
  },
  ["timbreicmc"] = {
    "texmf-dist/doc/latex/timbreicmc/README.md",
    "texmf-dist/doc/latex/timbreicmc/timbreicmc.pdf",
  },
  ["multilang"] = {
    "texmf-dist/doc/latex/multilang/README.md",
    "texmf-dist/doc/latex/multilang/multilang.pdf",
  },
  ["elocalloc"] = {
    "texmf-dist/doc/latex/elocalloc/README.txt",
    "texmf-dist/doc/latex/elocalloc/elocalloc.pdf",
  },
  ["mciteplus"] = {
    "texmf-dist/doc/latex/mciteplus/README",
    "texmf-dist/doc/latex/mciteplus/changelog.txt",
    "texmf-dist/doc/latex/mciteplus/mciteplus_code.txt",
    "texmf-dist/doc/latex/mciteplus/mciteplus_doc.pdf",
  },
  ["pgfgantt"] = {
    "texmf-dist/doc/latex/pgfgantt/README",
    "texmf-dist/doc/latex/pgfgantt/pgfgantt.pdf",
  },
  ["musixtnt"] = {
    "texmf-dist/doc/generic/musixtnt/README",
    "texmf-dist/doc/generic/musixtnt/TransformNotes.pdf",
    "texmf-dist/doc/generic/musixtnt/msxlint.pdf",
    "texmf-dist/doc/man/man1/msxlint.man1.pdf",
  },
  ["dinat"] = {
    "texmf-dist/doc/bibtex/dinat/dinat-index.html",
    "texmf-dist/doc/bibtex/dinat/history.html",
  },
  ["babel"] = {
    "texmf-dist/doc/latex/babel/README.md",
    "texmf-dist/doc/latex/babel/babel.pdf",
  },
  ["ducksay"] = {
    "texmf-dist/doc/latex/ducksay/README.TEXLIVE",
    "texmf-dist/doc/latex/ducksay/README.md",
  },
  ["cm-lgc"] = {
    "texmf-dist/doc/fonts/cm-lgc/COPYING",
    "texmf-dist/doc/fonts/cm-lgc/HISTORY",
    "texmf-dist/doc/fonts/cm-lgc/INSTALL",
    "texmf-dist/doc/fonts/cm-lgc/README",
  },
  ["fouriernc"] = {
    "texmf-dist/doc/fonts/fouriernc/README",
    "texmf-dist/doc/fonts/fouriernc/test_fouriernc.pdf",
  },
  ["harnon-cv"] = {
    "texmf-dist/doc/latex/harnon-cv/README",
    "texmf-dist/doc/latex/harnon-cv/sample.pdf",
  },
  ["parades"] = {
    "texmf-dist/doc/latex/parades/README",
    "texmf-dist/doc/latex/parades/example/README.example",
    "texmf-dist/doc/latex/parades/example/SConstruct",
    "texmf-dist/doc/latex/parades/example/udhr.pdf",
    "texmf-dist/doc/latex/parades/parades.pdf",
  },
  ["glossaries-polish"] = {
    "texmf-dist/doc/latex/glossaries-polish/README",
    "texmf-dist/doc/latex/glossaries-polish/glossaries-polish.pdf",
  },
  ["fonts-tlwg"] = {
    "texmf-dist/doc/fonts/fonts-tlwg/README.latex",
  },
  ["babel-polish"] = {
    "texmf-dist/doc/generic/babel-polish/polish.pdf",
  },
  ["babel-slovak"] = {
    "texmf-dist/doc/generic/babel-slovak/slovak.pdf",
  },
  ["conteq"] = {
    "texmf-dist/doc/latex/conteq/README.txt",
    "texmf-dist/doc/latex/conteq/conteq.pdf",
  },
  ["pdfjam"] = {
    "texmf-dist/doc/man/man1/pdf180.man1.pdf",
    "texmf-dist/doc/man/man1/pdf270.man1.pdf",
    "texmf-dist/doc/man/man1/pdf90.man1.pdf",
    "texmf-dist/doc/man/man1/pdfbook.man1.pdf",
    "texmf-dist/doc/man/man1/pdfflip.man1.pdf",
    "texmf-dist/doc/man/man1/pdfjam-pocketmod.man1.pdf",
    "texmf-dist/doc/man/man1/pdfjam-slides3up.man1.pdf",
    "texmf-dist/doc/man/man1/pdfjam-slides6up.man1.pdf",
    "texmf-dist/doc/man/man1/pdfjam.man1.pdf",
    "texmf-dist/doc/man/man1/pdfjoin.man1.pdf",
    "texmf-dist/doc/man/man1/pdfnup.man1.pdf",
    "texmf-dist/doc/man/man1/pdfpun.man1.pdf",
    "texmf-dist/doc/support/pdfjam/COPYING",
    "texmf-dist/doc/support/pdfjam/PDFjam-README.html",
    "texmf-dist/doc/support/pdfjam/VERSION",
  },
  ["hrefhide"] = {
    "texmf-dist/doc/latex/hrefhide/README",
    "texmf-dist/doc/latex/hrefhide/hrefhide-example.pdf",
    "texmf-dist/doc/latex/hrefhide/hrefhide.pdf",
  },
  ["colophon"] = {
    "texmf-dist/doc/latex/colophon/CHANGES",
    "texmf-dist/doc/latex/colophon/README",
    "texmf-dist/doc/latex/colophon/colophon.pdf",
    "texmf-dist/doc/latex/colophon/lppl.txt",
  },
  ["show2e"] = {
    "texmf-dist/doc/latex/show2e/README",
    "texmf-dist/doc/latex/show2e/show2e-fr.pdf",
    "texmf-dist/doc/latex/show2e/show2e.pdf",
  },
  ["musikui"] = {
    "texmf-dist/doc/latex/musikui/README.txt",
    "texmf-dist/doc/latex/musikui/kake.pdf",
    "texmf-dist/doc/latex/musikui/musikui.pdf",
    "texmf-dist/doc/latex/musikui/wari.pdf",
  },
  ["fbithesis"] = {
    "texmf-dist/doc/latex/fbithesis/README",
    "texmf-dist/doc/latex/fbithesis/example.pdf",
    "texmf-dist/doc/latex/fbithesis/fbithesis.pdf",
  },
  ["musicography"] = {
    "texmf-dist/doc/latex/musicography/README.md",
    "texmf-dist/doc/latex/musicography/musicography.pdf",
  },
  ["munich"] = {
    "texmf-dist/doc/latex/munich/README",
    "texmf-dist/doc/latex/munich/documenation_munich_Bibtex_style.pdf",
    "texmf-dist/doc/latex/munich/example_munich_Bibtex_style.pdf",
  },
  ["formation-latex-ul"] = {
    "texmf-dist/doc/latex/formation-latex-ul/README.md",
    "texmf-dist/doc/latex/formation-latex-ul/console-screenshot.pdf",
    "texmf-dist/doc/latex/formation-latex-ul/formation-latex-ul-diapos.pdf",
    "texmf-dist/doc/latex/formation-latex-ul/formation-latex-ul.pdf",
  },
  ["picinpar"] = {
    "texmf-dist/doc/latex/picinpar/picinpar-de.pdf",
    "texmf-dist/doc/latex/picinpar/picinpar-en.pdf",
  },
  ["halloweenmath"] = {
    "texmf-dist/doc/latex/halloweenmath/00readme.txt",
    "texmf-dist/doc/latex/halloweenmath/Makefile",
    "texmf-dist/doc/latex/halloweenmath/README",
    "texmf-dist/doc/latex/halloweenmath/halloweenmath-doc.pdf",
    "texmf-dist/doc/latex/halloweenmath/halloweenmath-man.pdf",
    "texmf-dist/doc/latex/halloweenmath/manifest.txt",
  },
  ["efbox"] = {
    "texmf-dist/doc/latex/efbox/README",
    "texmf-dist/doc/latex/efbox/efbox.pdf",
  },
  ["achemso"] = {
    "texmf-dist/doc/latex/achemso/README.md",
    "texmf-dist/doc/latex/achemso/achemso-demo.pdf",
    "texmf-dist/doc/latex/achemso/achemso.pdf",
  },
  ["cascade"] = {
    "texmf-dist/doc/latex/cascade/README.md",
    "texmf-dist/doc/latex/cascade/cascade.pdf",
  },
  ["ntheorem"] = {
    "texmf-dist/doc/latex/ntheorem/README",
    "texmf-dist/doc/latex/ntheorem/ntheorem.pdf",
  },
  ["chickenize"] = {
    "texmf-dist/doc/luatex/chickenize/README.md",
    "texmf-dist/doc/luatex/chickenize/chickenize.pdf",
  },
  ["lshort-spanish"] = {
    "texmf-dist/doc/latex/lshort-spanish/fuente/CAMBIOS",
    "texmf-dist/doc/latex/lshort-spanish/fuente/MANIFEST",
    "texmf-dist/doc/latex/lshort-spanish/lshort-a4.pdf",
  },
  ["appendixnumberbeamer"] = {
    "texmf-dist/doc/latex/appendixnumberbeamer/README.md",
    "texmf-dist/doc/latex/appendixnumberbeamer/VERSION",
  },
  ["serbian-apostrophe"] = {
    "texmf-dist/doc/latex/serbian-apostrophe/README",
    "texmf-dist/doc/latex/serbian-apostrophe/apostrophe-list.pdf",
    "texmf-dist/doc/latex/serbian-apostrophe/serbian-apostrophe.pdf",
  },
  ["crosswrd"] = {
    "texmf-dist/doc/latex/crosswrd/README",
    "texmf-dist/doc/latex/crosswrd/crosswrd.pdf",
  },
  ["tkz-berge"] = {
    "texmf-dist/doc/latex/tkz-berge/NamedGraphs/README",
    "texmf-dist/doc/latex/tkz-berge/NamedGraphs/doc/NamedGraphs.pdf",
    "texmf-dist/doc/latex/tkz-berge/NamedGraphs/readme-namedgraph.txt",
    "texmf-dist/doc/latex/tkz-berge/examples/Grid.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/gr-Circulant.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/gr-Complet-16.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/gr-edgeingraphmodloop.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/grCLadder.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/grDoubleMod.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/grExtraChords.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/grLadder.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/grSQCycle.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/grStar.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/grWheel.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/hypercube.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/hypercube_simple.pdf",
    "texmf-dist/doc/latex/tkz-berge/examples/hypercubed.pdf",
    "texmf-dist/doc/latex/tkz-berge/readme-tkz-berge.txt",
    "texmf-dist/doc/latex/tkz-berge/tkz-berge-screen.pdf",
  },
  ["novel"] = {
    "texmf-dist/doc/lualatex/novel/extras/NovelDeco-glyphs.pdf",
    "texmf-dist/doc/lualatex/novel/extras/novel-Gallery-ChapterStart.pdf",
    "texmf-dist/doc/lualatex/novel/extras/novel-extras-README.html",
    "texmf-dist/doc/lualatex/novel/novel-documentation.html",
    "texmf-dist/doc/lualatex/novel/novel.pdf",
  },
  ["latex-tds"] = {
    "texmf-dist/doc/latex/latex-tds/README",
    "texmf-dist/doc/latex/latex-tds/README.html",
    "texmf-dist/doc/latex/latex-tds/README.pdf",
  },
  ["context-typearea"] = {
    "texmf-dist/doc/context/third/typearea/VERSION",
    "texmf-dist/doc/context/third/typearea/typearea-demo.pdf",
    "texmf-dist/doc/context/third/typearea/typearea-doc.pdf",
  },
  ["bussproofs"] = {
    "texmf-dist/doc/latex/bussproofs/BussGuide2.pdf",
    "texmf-dist/doc/latex/bussproofs/README.txt",
    "texmf-dist/doc/latex/bussproofs/testbp2.pdf",
  },
  ["fei"] = {
    "texmf-dist/doc/latex/fei/README",
    "texmf-dist/doc/latex/fei/README.txt",
    "texmf-dist/doc/latex/fei/fei.pdf",
  },
  ["plantuml"] = {
    "texmf-dist/doc/lualatex/plantuml/README.md",
    "texmf-dist/doc/lualatex/plantuml/plantuml.pdf",
  },
  ["fvextra"] = {
    "texmf-dist/doc/latex/fvextra/README",
    "texmf-dist/doc/latex/fvextra/fvextra.pdf",
  },
  ["datetime2-bulgarian"] = {
    "texmf-dist/doc/latex/datetime2-bulgarian/README",
    "texmf-dist/doc/latex/datetime2-bulgarian/datetime2-bulgarian.pdf",
  },
  ["metapost"] = {
    "texmf-dist/doc/man/man1/dvitomp.man1.pdf",
    "texmf-dist/doc/man/man1/mpost.man1.pdf",
    "texmf-dist/doc/metapost/base/CHANGES",
    "texmf-dist/doc/metapost/base/grdemo-doc.pdf",
    "texmf-dist/doc/metapost/base/grdemo.pdf",
    "texmf-dist/doc/metapost/base/mpboxes.pdf",
    "texmf-dist/doc/metapost/base/mpgraph.pdf",
    "texmf-dist/doc/metapost/base/mpintro.pdf",
    "texmf-dist/doc/metapost/base/mplibapi.pdf",
    "texmf-dist/doc/metapost/base/mpman.pdf",
    "texmf-dist/doc/metapost/base/source-manual/Makefile",
    "texmf-dist/doc/metapost/base/source-manual/README",
    "texmf-dist/doc/metapost/base/source-manual/TODO",
    "texmf-dist/doc/metapost/base/source-manual/grdemo-doc.ps",
    "texmf-dist/doc/metapost/base/source-tutorial/Makefile",
    "texmf-dist/doc/metapost/base/source-tutorial/previewer.pdf",
  },
  ["hatching"] = {
    "texmf-dist/doc/metapost/hatching/README",
  },
  ["gchords"] = {
    "texmf-dist/doc/latex/gchords/README",
    "texmf-dist/doc/latex/gchords/gchords_doc.pdf",
  },
  ["mucproc"] = {
    "texmf-dist/doc/latex/mucproc/README.md",
    "texmf-dist/doc/latex/mucproc/mucproc.pdf",
  },
  ["l2tabu-english"] = {
    "texmf-dist/doc/latex/l2tabu-english/l2tabuen.pdf",
  },
  ["bchart"] = {
    "texmf-dist/doc/latex/bchart/CHANGES.txt",
    "texmf-dist/doc/latex/bchart/LICENSE.txt",
    "texmf-dist/doc/latex/bchart/README.md",
    "texmf-dist/doc/latex/bchart/bchart.pdf",
  },
  ["msu-thesis"] = {
    "texmf-dist/doc/latex/msu-thesis/README",
    "texmf-dist/doc/latex/msu-thesis/msu-thesis.pdf",
    "texmf-dist/doc/latex/msu-thesis/samples/MSU-thesis-template.pdf",
    "texmf-dist/doc/latex/msu-thesis/samples/MSU-thesis-testfile.pdf",
  },
  ["ascmac"] = {
    "texmf-dist/doc/latex/ascmac/LICENSE",
    "texmf-dist/doc/latex/ascmac/README.md",
    "texmf-dist/doc/latex/ascmac/ascmac.pdf",
  },
  ["xecjk"] = {
    "texmf-dist/doc/xelatex/xecjk/README.md",
    "texmf-dist/doc/xelatex/xecjk/xeCJK.pdf",
    "texmf-dist/doc/xelatex/xecjk/xunicode-symbols.pdf",
  },
  ["liturg"] = {
    "texmf-dist/doc/latex/liturg/README",
    "texmf-dist/doc/latex/liturg/liturg.pdf",
  },
  ["prftree"] = {
    "texmf-dist/doc/latex/prftree/README",
    "texmf-dist/doc/latex/prftree/prftreedoc.pdf",
  },
  ["mptrees"] = {
    "texmf-dist/doc/metapost/mptrees/README.md",
    "texmf-dist/doc/metapost/mptrees/mptrees-doc.pdf",
  },
  ["colorwav"] = {
    "texmf-dist/doc/latex/colorwav/README",
    "texmf-dist/doc/latex/colorwav/colorwav.pdf",
  },
  ["dijkstra"] = {
    "texmf-dist/doc/latex/dijkstra/README",
    "texmf-dist/doc/latex/dijkstra/dijkstra-fr.pdf",
  },
  ["hfbright"] = {
    "texmf-dist/doc/fonts/hfbright/README",
  },
  ["filecontentsdef"] = {
    "texmf-dist/doc/latex/filecontentsdef/README.md",
    "texmf-dist/doc/latex/filecontentsdef/filecontentsdef.pdf",
  },
  ["asymptote-by-example-zh-cn"] = {
    "texmf-dist/doc/support/asymptote-by-example-zh-cn/README",
    "texmf-dist/doc/support/asymptote-by-example-zh-cn/asymptote-by-example-zh-cn.pdf",
    "texmf-dist/doc/support/asymptote-by-example-zh-cn/src/cleantmp",
    "texmf-dist/doc/support/asymptote-by-example-zh-cn/src/makepdf",
    "texmf-dist/doc/support/asymptote-by-example-zh-cn/src/tiling.pdf",
    "texmf-dist/doc/support/asymptote-by-example-zh-cn/src/xiantu-ancient.pdf",
    "texmf-dist/doc/support/asymptote-by-example-zh-cn/src/xiantu.pdf",
  },
  ["gillcm"] = {
    "texmf-dist/doc/latex/gillcm/README",
    "texmf-dist/doc/latex/gillcm/gillcm.pdf",
    "texmf-dist/doc/latex/gillcm/sample.pdf",
  },
  ["babel-dutch"] = {
    "texmf-dist/doc/generic/babel-dutch/dutch.pdf",
  },
  ["dk-bib"] = {
    "texmf-dist/doc/latex/dk-bib/COPYRIGHT",
    "texmf-dist/doc/latex/dk-bib/README",
    "texmf-dist/doc/latex/dk-bib/dk-bib.pdf",
  },
  ["bxeepic"] = {
    "texmf-dist/doc/latex/bxeepic/LICENSE",
    "texmf-dist/doc/latex/bxeepic/README",
    "texmf-dist/doc/latex/bxeepic/sample-bxeepic.pdf",
  },
  ["mparrows"] = {
    "texmf-dist/doc/metapost/mparrows/LICENSE",
    "texmf-dist/doc/metapost/mparrows/README",
    "texmf-dist/doc/metapost/mparrows/mparrows.pdf",
  },
  ["gmiflink"] = {
    "texmf-dist/doc/latex/gmiflink/README",
    "texmf-dist/doc/latex/gmiflink/gmiflink.pdf",
  },
  ["colortab"] = {
    "texmf-dist/doc/generic/colortab/Changes",
    "texmf-dist/doc/generic/colortab/Makefile",
    "texmf-dist/doc/generic/colortab/colortab-doc.pdf",
  },
  ["mptopdf"] = {
    "texmf-dist/doc/man/man1/mptopdf.man1.pdf",
  },
  ["amsrefs"] = {
    "texmf-dist/doc/latex/amsrefs/amsrdoc.pdf",
    "texmf-dist/doc/latex/amsrefs/amsrefs.pdf",
    "texmf-dist/doc/latex/amsrefs/amsxport.pdf",
    "texmf-dist/doc/latex/amsrefs/changes.pdf",
    "texmf-dist/doc/latex/amsrefs/ifoption.pdf",
    "texmf-dist/doc/latex/amsrefs/mathscinet.pdf",
    "texmf-dist/doc/latex/amsrefs/pcatcode.pdf",
    "texmf-dist/doc/latex/amsrefs/rkeyval.pdf",
    "texmf-dist/doc/latex/amsrefs/textcmds.pdf",
  },
  ["mpman-ru"] = {
    "texmf-dist/doc/metapost/mpman-ru/Makefile",
    "texmf-dist/doc/metapost/mpman-ru/README",
    "texmf-dist/doc/metapost/mpman-ru/README.ru.koi8-r",
    "texmf-dist/doc/metapost/mpman-ru/mpman-ru.pdf",
  },
  ["dratex"] = {
    "texmf-dist/doc/generic/dratex/README",
  },
  ["odsfile"] = {
    "texmf-dist/doc/lualatex/odsfile/README",
    "texmf-dist/doc/lualatex/odsfile/odsfile.pdf",
  },
  ["filedate"] = {
    "texmf-dist/doc/latex/filedate/Announce.txt",
    "texmf-dist/doc/latex/filedate/README",
    "texmf-dist/doc/latex/filedate/SrcFILEs.txt",
    "texmf-dist/doc/latex/filedate/filedate.pdf",
  },
  ["mpattern"] = {
    "texmf-dist/doc/metapost/mpattern/README",
    "texmf-dist/doc/metapost/mpattern/README.PL",
    "texmf-dist/doc/metapost/mpattern/mpattern_test.pdf",
  },
  ["outline"] = {
    "texmf-dist/doc/latex/outline/README",
    "texmf-dist/doc/latex/outline/outline.pdf",
  },
  ["moresize"] = {
    "texmf-dist/doc/latex/moresize/moresize.pdf",
  },
  ["libertinegc"] = {
    "texmf-dist/doc/fonts/libertinegc/README",
    "texmf-dist/doc/fonts/libertinegc/libertinegc-doc.pdf",
  },
  ["morefloats"] = {
    "texmf-dist/doc/latex/morefloats/README",
    "texmf-dist/doc/latex/morefloats/morefloats-example.pdf",
    "texmf-dist/doc/latex/morefloats/morefloats.pdf",
  },
  ["texware"] = {
    "texmf-dist/doc/man/man1/dvitype.man1.pdf",
    "texmf-dist/doc/man/man1/pooltype.man1.pdf",
  },
  ["contour"] = {
    "texmf-dist/doc/latex/contour/ChangeLog",
    "texmf-dist/doc/latex/contour/Makefile",
    "texmf-dist/doc/latex/contour/README",
    "texmf-dist/doc/latex/contour/contour.pdf",
  },
  ["estcpmm"] = {
    "texmf-dist/doc/latex/estcpmm/README",
    "texmf-dist/doc/latex/estcpmm/estcp.pdf",
    "texmf-dist/doc/latex/estcpmm/estcpmm.pdf",
    "texmf-dist/doc/latex/estcpmm/red_corps_castle2.pdf",
    "texmf-dist/doc/latex/estcpmm/sample.pdf",
  },
  ["modular"] = {
    "texmf-dist/doc/latex/modular/README.md",
    "texmf-dist/doc/latex/modular/modular.pdf",
  },
  ["covington"] = {
    "texmf-dist/doc/latex/covington/README",
    "texmf-dist/doc/latex/covington/covington.pdf",
  },
  ["biblatex-arthistory-bonn"] = {
    "texmf-dist/doc/latex/biblatex-arthistory-bonn/README.md",
    "texmf-dist/doc/latex/biblatex-arthistory-bonn/arthistory-bonn.pdf",
  },
  ["modiagram"] = {
    "texmf-dist/doc/latex/modiagram/README",
    "texmf-dist/doc/latex/modiagram/modiagram_en.pdf",
  },
  ["cormorantgaramond"] = {
    "texmf-dist/doc/fonts/cormorantgaramond/OFL.txt",
    "texmf-dist/doc/fonts/cormorantgaramond/cormorantgaramond.pdf",
  },
  ["babel-serbian"] = {
    "texmf-dist/doc/generic/babel-serbian/serbian.pdf",
  },
  ["luamesh"] = {
    "texmf-dist/doc/lualatex/luamesh/README",
    "texmf-dist/doc/lualatex/luamesh/animation-crop.pdf",
    "texmf-dist/doc/lualatex/luamesh/fond.pdf",
    "texmf-dist/doc/lualatex/luamesh/luamesh-doc.pdf",
    "texmf-dist/doc/lualatex/luamesh/luamesh-title.pdf",
    "texmf-dist/doc/lualatex/luamesh/mesh.txt",
    "texmf-dist/doc/lualatex/luamesh/meshgarde.txt",
  },
  ["pfarrei"] = {
    "texmf-dist/doc/latex/pfarrei/pfarrei.pdf",
  },
  ["splitindex"] = {
    "texmf-dist/doc/latex/splitindex/README",
    "texmf-dist/doc/latex/splitindex/install.txt",
    "texmf-dist/doc/latex/splitindex/manifest.txt",
    "texmf-dist/doc/latex/splitindex/splitidx.pdf",
    "texmf-dist/doc/man/man1/splitindex.man1.pdf",
  },
  ["mnras"] = {
    "texmf-dist/doc/latex/mnras/example.ps",
    "texmf-dist/doc/latex/mnras/legacy/legacy.txt",
    "texmf-dist/doc/latex/mnras/mnras_guide.pdf",
    "texmf-dist/doc/latex/mnras/readme.txt",
  },
  ["mnotes"] = {
    "texmf-dist/doc/latex/mnotes/Makefile",
    "texmf-dist/doc/latex/mnotes/README",
    "texmf-dist/doc/latex/mnotes/mnotes.pdf",
  },
  ["alegreya"] = {
    "texmf-dist/doc/fonts/alegreya/OFL.txt",
    "texmf-dist/doc/fonts/alegreya/README",
    "texmf-dist/doc/fonts/alegreya/alegreya-samples.pdf",
  },
  ["eqnalign"] = {
    "texmf-dist/doc/latex/eqnalign/README.md",
    "texmf-dist/doc/latex/eqnalign/eqnalign.pdf",
  },
  ["context-gnuplot"] = {
    "texmf-dist/doc/context/third/gnuplot/VERSION",
    "texmf-dist/doc/context/third/gnuplot/fullpage-example.pdf",
    "texmf-dist/doc/context/third/gnuplot/gnuplot-context-doc.pdf",
  },
  ["mla-paper"] = {
    "texmf-dist/doc/latex/mla-paper/README",
  },
  ["antomega"] = {
    "texmf-dist/doc/omega/antomega/README",
    "texmf-dist/doc/omega/antomega/antomega.pdf",
  },
  ["regcount"] = {
    "texmf-dist/doc/latex/regcount/readme",
    "texmf-dist/doc/latex/regcount/regcount.pdf",
  },
  ["pst-ghsb"] = {
    "texmf-dist/doc/generic/pst-ghsb/README",
    "texmf-dist/doc/generic/pst-ghsb/t-ghsb.pdf",
    "texmf-dist/doc/generic/pst-ghsb/t2-ghsb.pdf",
  },
  ["pdfpagediff"] = {
    "texmf-dist/doc/latex/pdfpagediff/README",
    "texmf-dist/doc/latex/pdfpagediff/doc/Makefile",
    "texmf-dist/doc/latex/pdfpagediff/doc/pdfpagediff-doc.pdf",
    "texmf-dist/doc/latex/pdfpagediff/example/file1.pdf",
    "texmf-dist/doc/latex/pdfpagediff/example/file2.pdf",
    "texmf-dist/doc/latex/pdfpagediff/example/ltest.pdf",
    "texmf-dist/doc/latex/pdfpagediff/manifest.txt",
  },
  ["erewhon"] = {
    "texmf-dist/doc/fonts/erewhon/FontLog.txt",
    "texmf-dist/doc/fonts/erewhon/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/erewhon/OFL.txt",
    "texmf-dist/doc/fonts/erewhon/README",
    "texmf-dist/doc/fonts/erewhon/erewhon-doc.pdf",
  },
  ["mlist"] = {
    "texmf-dist/doc/latex/mlist/README",
    "texmf-dist/doc/latex/mlist/mlist.pdf",
  },
  ["multibibliography"] = {
    "texmf-dist/doc/latex/multibibliography/Makefile",
    "texmf-dist/doc/latex/multibibliography/README",
    "texmf-dist/doc/latex/multibibliography/figure.pdf",
    "texmf-dist/doc/latex/multibibliography/multibibliography.pdf",
    "texmf-dist/doc/latex/multibibliography/tug-paper.pdf",
  },
  ["velthuis"] = {
    "texmf-dist/doc/generic/velthuis/README",
    "texmf-dist/doc/generic/velthuis/changes",
    "texmf-dist/doc/generic/velthuis/copying",
    "texmf-dist/doc/generic/velthuis/examples.pdf",
    "texmf-dist/doc/generic/velthuis/hindi.pdf",
    "texmf-dist/doc/generic/velthuis/manual.pdf",
    "texmf-dist/doc/generic/velthuis/misspaal.pdf",
    "texmf-dist/doc/generic/velthuis/vedasample.pdf",
    "texmf-dist/doc/generic/velthuis/xetex-examples.pdf",
    "texmf-dist/doc/generic/velthuis/xetex-misspaal.pdf",
    "texmf-dist/doc/man/man1/devnag.man1.pdf",
  },
  ["acro"] = {
    "texmf-dist/doc/latex/acro/README",
    "texmf-dist/doc/latex/acro/acro_en.pdf",
  },
  ["har2nat"] = {
    "texmf-dist/doc/latex/har2nat/README",
    "texmf-dist/doc/latex/har2nat/har2nat.pdf",
  },
  ["leaflet"] = {
    "texmf-dist/doc/latex/leaflet/README.md",
    "texmf-dist/doc/latex/leaflet/leaflet-manual.pdf",
    "texmf-dist/doc/latex/leaflet/leaflet.pdf",
  },
  ["crbox"] = {
    "texmf-dist/doc/latex/crbox/README",
    "texmf-dist/doc/latex/crbox/crbox-doc.pdf",
  },
  ["lshort-bulgarian"] = {
    "texmf-dist/doc/latex/lshort-bulgarian/README",
    "texmf-dist/doc/latex/lshort-bulgarian/lshort-bg.pdf",
  },
  ["tabulary"] = {
    "texmf-dist/doc/latex/tabulary/README",
    "texmf-dist/doc/latex/tabulary/tabulary.pdf",
  },
  ["latex-doc-ptr"] = {
    "texmf-dist/doc/latex/latex-doc-ptr/Makefile",
    "texmf-dist/doc/latex/latex-doc-ptr/README",
    "texmf-dist/doc/latex/latex-doc-ptr/latex-doc-ptr.pdf",
  },
  ["hecthese"] = {
    "texmf-dist/doc/latex/hecthese/README.md",
    "texmf-dist/doc/latex/hecthese/hecthese-en.pdf",
    "texmf-dist/doc/latex/hecthese/hecthese.pdf",
  },
  ["bibtopic"] = {
    "texmf-dist/doc/latex/bibtopic/README",
    "texmf-dist/doc/latex/bibtopic/bibtopic.pdf",
  },
  ["lkproof"] = {
    "texmf-dist/doc/latex/lkproof/lkproof-doc.pdf",
  },
  ["staves"] = {
    "texmf-dist/doc/fonts/staves/README",
    "texmf-dist/doc/fonts/staves/staves.pdf",
    "texmf-dist/doc/fonts/staves/symbols.pdf",
  },
  ["ctanupload"] = {
    "texmf-dist/doc/support/ctanupload/README",
    "texmf-dist/doc/support/ctanupload/ctanupload.pdf",
  },
  ["auncial-new"] = {
    "texmf-dist/doc/fonts/auncial-new/README",
    "texmf-dist/doc/fonts/auncial-new/auncial.pdf",
    "texmf-dist/doc/fonts/auncial-new/tryauncial.pdf",
  },
  ["subeqnarray"] = {
    "texmf-dist/doc/latex/subeqnarray/manifest.txt",
    "texmf-dist/doc/latex/subeqnarray/subeqnarray.pdf",
  },
  ["mkpattern"] = {
    "texmf-dist/doc/plain/mkpattern/README",
    "texmf-dist/doc/plain/mkpattern/mkpatter.pdf",
  },
  ["coloring"] = {
    "texmf-dist/doc/latex/coloring/README",
    "texmf-dist/doc/latex/coloring/coloring-doc.pdf",
  },
  ["extarrows"] = {
    "texmf-dist/doc/latex/extarrows/README",
    "texmf-dist/doc/latex/extarrows/extarrows-test.pdf",
  },
  ["mf2pt1"] = {
    "texmf-dist/doc/support/mf2pt1/ChangeLog",
    "texmf-dist/doc/support/mf2pt1/README",
    "texmf-dist/doc/support/mf2pt1/mf2pt1.pdf",
  },
  ["draftfigure"] = {
    "texmf-dist/doc/latex/draftfigure/README.md",
    "texmf-dist/doc/latex/draftfigure/draftfigure.pdf",
  },
  ["paracol"] = {
    "texmf-dist/doc/latex/paracol/README",
    "texmf-dist/doc/latex/paracol/paracol-man.pdf",
  },
  ["gmverse"] = {
    "texmf-dist/doc/latex/gmverse/README",
    "texmf-dist/doc/latex/gmverse/gmverse.pdf",
  },
  ["dyntree"] = {
    "texmf-dist/doc/latex/dyntree/README",
    "texmf-dist/doc/latex/dyntree/dyntree.pdf",
  },
  ["fixme"] = {
    "texmf-dist/doc/latex/fixme/NEWS",
    "texmf-dist/doc/latex/fixme/README.md",
    "texmf-dist/doc/latex/fixme/THANKS",
    "texmf-dist/doc/latex/fixme/fixme.pdf",
  },
  ["lstaddons"] = {
    "texmf-dist/doc/latex/lstaddons/README",
    "texmf-dist/doc/latex/lstaddons/lstautogobble.pdf",
    "texmf-dist/doc/latex/lstaddons/lstlinebgrd.pdf",
  },
  ["texsis"] = {
    "texmf-dist/doc/man/man1/texsis.man1.pdf",
    "texmf-dist/doc/otherformats/texsis/base/COPYING",
    "texmf-dist/doc/otherformats/texsis/base/INSTALL",
    "texmf-dist/doc/otherformats/texsis/base/MANIFEST",
    "texmf-dist/doc/otherformats/texsis/base/NEWS",
    "texmf-dist/doc/otherformats/texsis/base/README",
    "texmf-dist/doc/otherformats/texsis/base/letr",
  },
  ["gatech-thesis"] = {
    "texmf-dist/doc/latex/gatech-thesis/CHANGES",
    "texmf-dist/doc/latex/gatech-thesis/COMPLIANCE",
    "texmf-dist/doc/latex/gatech-thesis/COPYING",
    "texmf-dist/doc/latex/gatech-thesis/ChangeLog",
    "texmf-dist/doc/latex/gatech-thesis/INSTALL",
    "texmf-dist/doc/latex/gatech-thesis/MANIFEST",
    "texmf-dist/doc/latex/gatech-thesis/NEWS",
    "texmf-dist/doc/latex/gatech-thesis/NOTES",
    "texmf-dist/doc/latex/gatech-thesis/README",
    "texmf-dist/doc/latex/gatech-thesis/TODO",
    "texmf-dist/doc/latex/gatech-thesis/example-thesis.pdf",
  },
  ["checklistings"] = {
    "texmf-dist/doc/latex/checklistings/Makefile",
    "texmf-dist/doc/latex/checklistings/README.md",
    "texmf-dist/doc/latex/checklistings/checklistings.pdf",
    "texmf-dist/doc/latex/checklistings/example.html",
    "texmf-dist/doc/latex/checklistings/example.pdf",
  },
  ["exsheets"] = {
    "texmf-dist/doc/latex/exsheets/README",
    "texmf-dist/doc/latex/exsheets/exsheets_en.pdf",
    "texmf-dist/doc/latex/exsheets/grading-table.pdf",
  },
  ["edmargin"] = {
    "texmf-dist/doc/latex/edmargin/README",
    "texmf-dist/doc/latex/edmargin/edmargin.pdf",
  },
  ["listlbls"] = {
    "texmf-dist/doc/latex/listlbls/Makefile",
    "texmf-dist/doc/latex/listlbls/README",
    "texmf-dist/doc/latex/listlbls/README.md",
    "texmf-dist/doc/latex/listlbls/README.txt",
    "texmf-dist/doc/latex/listlbls/listlbls.pdf",
  },
  ["bibletext"] = {
    "texmf-dist/doc/latex/bibletext/LICENSE",
    "texmf-dist/doc/latex/bibletext/README.md",
    "texmf-dist/doc/latex/bibletext/bibletext.pdf",
  },
  ["comprehensive"] = {
    "texmf-dist/doc/latex/comprehensive/README",
    "texmf-dist/doc/latex/comprehensive/README.TEXLIVE",
    "texmf-dist/doc/latex/comprehensive/SYMLIST",
    "texmf-dist/doc/latex/comprehensive/rawtables-a4.pdf",
    "texmf-dist/doc/latex/comprehensive/source/Makefile",
    "texmf-dist/doc/latex/comprehensive/source/makefakeMnSymbol",
    "texmf-dist/doc/latex/comprehensive/source/makefakeboisik",
    "texmf-dist/doc/latex/comprehensive/source/makefakefdsymbol",
    "texmf-dist/doc/latex/comprehensive/source/makefakestarfont",
    "texmf-dist/doc/latex/comprehensive/source/makefakestix",
    "texmf-dist/doc/latex/comprehensive/source/makerawtables",
    "texmf-dist/doc/latex/comprehensive/symbols-a4.pdf",
  },
  ["celtic"] = {
    "texmf-dist/doc/latex/celtic/README",
    "texmf-dist/doc/latex/celtic/celtic.pdf",
    "texmf-dist/doc/latex/celtic/celtic_code.pdf",
  },
  ["backnaur"] = {
    "texmf-dist/doc/latex/backnaur/README",
    "texmf-dist/doc/latex/backnaur/backnaur.pdf",
  },
  ["ijmart"] = {
    "texmf-dist/doc/latex/ijmart/Makefile",
    "texmf-dist/doc/latex/ijmart/README",
    "texmf-dist/doc/latex/ijmart/ijmart.pdf",
    "texmf-dist/doc/latex/ijmart/ijmsample.pdf",
  },
  ["amiri"] = {
    "texmf-dist/doc/fonts/amiri/NEWS",
    "texmf-dist/doc/fonts/amiri/NEWS-Arabic",
    "texmf-dist/doc/fonts/amiri/OFL.txt",
    "texmf-dist/doc/fonts/amiri/README",
    "texmf-dist/doc/fonts/amiri/README-Arabic",
    "texmf-dist/doc/fonts/amiri/amiri-table.pdf",
    "texmf-dist/doc/fonts/amiri/documentation-arabic.pdf",
  },
  ["chemschemex"] = {
    "texmf-dist/doc/latex/chemschemex/C2H4.pdf",
    "texmf-dist/doc/latex/chemschemex/CH2CH-R2.pdf",
    "texmf-dist/doc/latex/chemschemex/R1-CHCH2.pdf",
    "texmf-dist/doc/latex/chemschemex/R2-CHCH-R1.pdf",
    "texmf-dist/doc/latex/chemschemex/README",
    "texmf-dist/doc/latex/chemschemex/RuCH-R1.pdf",
    "texmf-dist/doc/latex/chemschemex/RuCH2.pdf",
    "texmf-dist/doc/latex/chemschemex/chemschemex.pdf",
    "texmf-dist/doc/latex/chemschemex/cp.pdf",
    "texmf-dist/doc/latex/chemschemex/maleimid.pdf",
    "texmf-dist/doc/latex/chemschemex/product.pdf",
    "texmf-dist/doc/latex/chemschemex/product_num.pdf",
  },
  ["antt"] = {
    "texmf-dist/doc/fonts/antt/AntykwaTorunska-doc-en-2_03.pdf",
    "texmf-dist/doc/fonts/antt/AntykwaTorunska-doc-pl-2_03.pdf",
    "texmf-dist/doc/fonts/antt/GUST-FONT-NOSOURCE-LICENSE.txt",
    "texmf-dist/doc/fonts/antt/MANIFEST.txt",
    "texmf-dist/doc/fonts/antt/README",
    "texmf-dist/doc/latex/antt/README",
  },
  ["ascii-font"] = {
    "texmf-dist/doc/fonts/ascii-font/README.TEXLIVE",
  },
  ["mathastext"] = {
    "texmf-dist/doc/latex/mathastext/README.md",
    "texmf-dist/doc/latex/mathastext/mathastext.pdf",
    "texmf-dist/doc/latex/mathastext/mathastexttestalphabets.pdf",
  },
  ["ieeetran"] = {
    "texmf-dist/doc/latex/IEEEtran/IEEEtran_HOWTO.pdf",
    "texmf-dist/doc/latex/IEEEtran/IEEEtran_bst_HOWTO.pdf",
    "texmf-dist/doc/latex/IEEEtran/IEEEtrantools_doc.txt",
    "texmf-dist/doc/latex/IEEEtran/README",
    "texmf-dist/doc/latex/IEEEtran/README.bibtex",
    "texmf-dist/doc/latex/IEEEtran/README.extras",
    "texmf-dist/doc/latex/IEEEtran/README.testflow",
    "texmf-dist/doc/latex/IEEEtran/README.tools",
    "texmf-dist/doc/latex/IEEEtran/bibtex/changelog.txt",
    "texmf-dist/doc/latex/IEEEtran/changelog.txt",
    "texmf-dist/doc/latex/IEEEtran/testflow_ctl_A4.pdf",
    "texmf-dist/doc/latex/IEEEtran/testflow_ctl_LTR.pdf",
    "texmf-dist/doc/latex/IEEEtran/testflow_doc.pdf",
    "texmf-dist/doc/latex/IEEEtran/tools/changelog.txt",
    "texmf-dist/doc/latex/IEEEtran/tux.pdf",
  },
  ["numnameru"] = {
    "texmf-dist/doc/latex/numnameru/README",
    "texmf-dist/doc/latex/numnameru/numnameru.pdf",
  },
  ["enumitem-zref"] = {
    "texmf-dist/doc/latex/enumitem-zref/README",
    "texmf-dist/doc/latex/enumitem-zref/enumitem-zref.pdf",
  },
  ["latexbug"] = {
    "texmf-dist/doc/latex/latexbug/README.md",
    "texmf-dist/doc/latex/latexbug/latexbug.pdf",
  },
  ["docmute"] = {
    "texmf-dist/doc/latex/docmute/README",
    "texmf-dist/doc/latex/docmute/docmute.pdf",
  },
  ["grundgesetze"] = {
    "texmf-dist/doc/latex/grundgesetze/README",
    "texmf-dist/doc/latex/grundgesetze/grundgesetze.pdf",
  },
  ["media9"] = {
    "texmf-dist/doc/latex/media9/ChangeLog",
    "texmf-dist/doc/latex/media9/README",
    "texmf-dist/doc/latex/media9/media9.pdf",
  },
  ["bxdvidriver"] = {
    "texmf-dist/doc/latex/bxdvidriver/LICENSE",
    "texmf-dist/doc/latex/bxdvidriver/README.md",
  },
  ["cquthesis"] = {
    "texmf-dist/doc/latex/cquthesis/Makefile",
    "texmf-dist/doc/latex/cquthesis/README",
    "texmf-dist/doc/latex/cquthesis/cquthesis.pdf",
    "texmf-dist/doc/latex/cquthesis/figures/CQUbadge.pdf",
    "texmf-dist/doc/latex/cquthesis/figures/README",
  },
  ["apptools"] = {
    "texmf-dist/doc/latex/apptools/apptools.pdf",
  },
  ["csbulletin"] = {
    "texmf-dist/doc/latex/csbulletin/LICENSE.txt",
    "texmf-dist/doc/latex/csbulletin/README",
    "texmf-dist/doc/latex/csbulletin/csbulletin.pdf",
  },
  ["makeshape"] = {
    "texmf-dist/doc/latex/makeshape/README",
    "texmf-dist/doc/latex/makeshape/makeshape.pdf",
    "texmf-dist/doc/latex/makeshape/ontesting.pdf",
    "texmf-dist/doc/latex/makeshape/testsample.pdf",
  },
  ["minutes"] = {
    "texmf-dist/doc/latex/minutes/README.md",
    "texmf-dist/doc/latex/minutes/minutes.pdf",
  },
  ["minted"] = {
    "texmf-dist/doc/latex/minted/Makefile",
    "texmf-dist/doc/latex/minted/README",
    "texmf-dist/doc/latex/minted/minted.pdf",
  },
  ["minorrevision"] = {
    "texmf-dist/doc/latex/minorrevision/README",
    "texmf-dist/doc/latex/minorrevision/minorrevision.pdf",
  },
  ["lmake"] = {
    "texmf-dist/doc/latex/lmake/README",
    "texmf-dist/doc/latex/lmake/lmake.pdf",
  },
  ["chemgreek"] = {
    "texmf-dist/doc/latex/chemgreek/README",
    "texmf-dist/doc/latex/chemgreek/chemgreek_en.pdf",
  },
  ["milsymb"] = {
    "texmf-dist/doc/latex/milsymb/README.md",
    "texmf-dist/doc/latex/milsymb/milsymb.pdf",
  },
  ["milog"] = {
    "texmf-dist/doc/latex/milog/README.md",
    "texmf-dist/doc/latex/milog/makefile",
    "texmf-dist/doc/latex/milog/manifest.txt",
    "texmf-dist/doc/latex/milog/milog",
    "texmf-dist/doc/latex/milog/milog-example.pdf",
    "texmf-dist/doc/latex/milog/milog-formular.pdf",
    "texmf-dist/doc/latex/milog/milog.pdf",
  },
  ["miller"] = {
    "texmf-dist/doc/latex/miller/ChangeLog",
    "texmf-dist/doc/latex/miller/Makefile",
    "texmf-dist/doc/latex/miller/README",
    "texmf-dist/doc/latex/miller/miller.pdf",
  },
  ["midpage"] = {
    "texmf-dist/doc/latex/midpage/midpage.pdf",
  },
  ["citeref"] = {
    "texmf-dist/doc/latex/citeref/README.pdf",
    "texmf-dist/doc/latex/citeref/README.txt",
  },
  ["amstex"] = {
    "texmf-dist/doc/amstex/base/README",
    "texmf-dist/doc/amstex/base/amsguide.pdf",
    "texmf-dist/doc/man/man1/amstex.man1.pdf",
  },
  ["boondox"] = {
    "texmf-dist/doc/fonts/boondox/README",
    "texmf-dist/doc/fonts/boondox/boondox-doc.pdf",
  },
  ["beamertheme-upenn-bc"] = {
    "texmf-dist/doc/latex/beamertheme-upenn-bc/README",
    "texmf-dist/doc/latex/beamertheme-upenn-bc/beamerBCstyle.pdf",
    "texmf-dist/doc/latex/beamertheme-upenn-bc/beamerPENNstyle.pdf",
  },
  ["mflogo-font"] = {
    "texmf-dist/doc/fonts/mflogo-font/README",
  },
  ["dozenal"] = {
    "texmf-dist/doc/fonts/dozenal/CHANGES",
    "texmf-dist/doc/fonts/dozenal/README",
    "texmf-dist/doc/fonts/dozenal/dozenal.pdf",
    "texmf-dist/doc/fonts/dozenal/dozenalfilelist.txt",
    "texmf-dist/doc/fonts/dozenal/lppl.txt",
  },
  ["datetime2-danish"] = {
    "texmf-dist/doc/latex/datetime2-danish/README",
    "texmf-dist/doc/latex/datetime2-danish/datetime2-danish.pdf",
  },
  ["mgltex"] = {
    "texmf-dist/doc/latex/mgltex/README",
    "texmf-dist/doc/latex/mgltex/Recompilation_decision.pdf",
    "texmf-dist/doc/latex/mgltex/mgltex.pdf",
    "texmf-dist/doc/latex/mgltex/sample.pdf",
  },
  ["ietfbibs"] = {
    "texmf-dist/doc/bibtex/ietfbibs/Makefile",
    "texmf-dist/doc/bibtex/ietfbibs/README.md",
    "texmf-dist/doc/bibtex/ietfbibs/id2bib",
    "texmf-dist/doc/bibtex/ietfbibs/rfc2bib",
  },
  ["ltabptch"] = {
    "texmf-dist/doc/latex/ltabptch/README",
    "texmf-dist/doc/latex/ltabptch/ltabptch.pdf",
  },
  ["2up"] = {
    "texmf-dist/doc/generic/2up/2up-doc.pdf",
    "texmf-dist/doc/generic/2up/Changes",
    "texmf-dist/doc/generic/2up/Makefile",
    "texmf-dist/doc/generic/2up/README",
  },
  ["babel-romansh"] = {
    "texmf-dist/doc/generic/babel-romansh/romansh.pdf",
  },
  ["context-bnf"] = {
    "texmf-dist/doc/context/third/bnf/NEWS",
    "texmf-dist/doc/context/third/bnf/README",
    "texmf-dist/doc/context/third/bnf/VERSION",
    "texmf-dist/doc/context/third/bnf/t-bnf.pdf",
  },
  ["mfware"] = {
    "texmf-dist/doc/man/man1/gftodvi.man1.pdf",
    "texmf-dist/doc/man/man1/gftopk.man1.pdf",
    "texmf-dist/doc/man/man1/gftype.man1.pdf",
    "texmf-dist/doc/man/man1/mft.man1.pdf",
    "texmf-dist/doc/man/man1/pktogf.man1.pdf",
    "texmf-dist/doc/man/man1/pktype.man1.pdf",
  },
  ["versions"] = {
    "texmf-dist/doc/latex/versions/versions-doc.pdf",
  },
  ["garrigues"] = {
    "texmf-dist/doc/metapost/garrigues/README",
    "texmf-dist/doc/metapost/garrigues/article.txt",
  },
  ["currency"] = {
    "texmf-dist/doc/latex/currency/README.md",
    "texmf-dist/doc/latex/currency/currency.pdf",
    "texmf-dist/doc/latex/currency/currency_doc.pdf",
  },
  ["modref"] = {
    "texmf-dist/doc/latex/modref/README",
    "texmf-dist/doc/latex/modref/modref.pdf",
  },
  ["marginfix"] = {
    "texmf-dist/doc/latex/marginfix/README",
    "texmf-dist/doc/latex/marginfix/marginfix.pdf",
  },
  ["luatexja"] = {
    "texmf-dist/doc/luatex/luatexja/COPYING",
    "texmf-dist/doc/luatex/luatexja/README",
    "texmf-dist/doc/luatex/luatexja/lltjp-geometry.pdf",
    "texmf-dist/doc/luatex/luatexja/ltjclasses.pdf",
    "texmf-dist/doc/luatex/luatexja/ltjltxdoc.pdf",
    "texmf-dist/doc/luatex/luatexja/ltjsclasses.pdf",
    "texmf-dist/doc/luatex/luatexja/luatexja-en.pdf",
    "texmf-dist/doc/luatex/luatexja/luatexja-ja.pdf",
    "texmf-dist/doc/luatex/luatexja/luatexja-ruby.pdf",
    "texmf-dist/doc/luatex/luatexja/luatexja-zhfonts.pdf",
  },
  ["bibleref-french"] = {
    "texmf-dist/doc/latex/bibleref-french/Lacroux-Bible.pdf",
    "texmf-dist/doc/latex/bibleref-french/README",
    "texmf-dist/doc/latex/bibleref-french/bibleref-french-francais.pdf",
    "texmf-dist/doc/latex/bibleref-french/bibleref-french.pdf",
    "texmf-dist/doc/latex/bibleref-french/makefile",
  },
  ["keyfloat"] = {
    "texmf-dist/doc/latex/keyfloat/README.txt",
    "texmf-dist/doc/latex/keyfloat/images/image.pdf",
    "texmf-dist/doc/latex/keyfloat/images/image2.pdf",
    "texmf-dist/doc/latex/keyfloat/keyfloat.pdf",
  },
  ["antiqua"] = {
    "texmf-dist/doc/fonts/antiqua/antiqua.txt",
  },
  ["verse"] = {
    "texmf-dist/doc/latex/verse/README",
    "texmf-dist/doc/latex/verse/verse.pdf",
  },
  ["doclicense"] = {
    "texmf-dist/doc/latex/doclicense/README",
    "texmf-dist/doc/latex/doclicense/doclicense.pdf",
  },
  ["psfragx"] = {
    "texmf-dist/doc/latex/psfragx/README",
    "texmf-dist/doc/latex/psfragx/README.laprint-3.16",
    "texmf-dist/doc/latex/psfragx/laprintdoc.ps",
    "texmf-dist/doc/latex/psfragx/psfragx.pdf",
    "texmf-dist/doc/latex/psfragx/psfragx_example.pdf",
    "texmf-dist/doc/latex/psfragx/readmePFX.txt",
  },
  ["catechis"] = {
    "texmf-dist/doc/latex/catechis/CHANGES",
    "texmf-dist/doc/latex/catechis/README",
    "texmf-dist/doc/latex/catechis/catechis.pdf",
    "texmf-dist/doc/latex/catechis/lppl.txt",
  },
  ["lshort-polish"] = {
    "texmf-dist/doc/latex/lshort-polish/README",
    "texmf-dist/doc/latex/lshort-polish/lshort2e.pdf",
  },
  ["pst-grad"] = {
    "texmf-dist/doc/generic/pst-grad/Changes",
    "texmf-dist/doc/generic/pst-grad/pst-grad-doc.pdf",
  },
  ["skdoc"] = {
    "texmf-dist/doc/latex/skdoc/README",
    "texmf-dist/doc/latex/skdoc/skdoc.pdf",
  },
  ["rcsinfo"] = {
    "texmf-dist/doc/latex/rcsinfo/README",
    "texmf-dist/doc/latex/rcsinfo/rcsinfo.pdf",
  },
  ["ucs"] = {
    "texmf-dist/doc/latex/ucs/FAQ",
    "texmf-dist/doc/latex/ucs/GNUmakefile",
    "texmf-dist/doc/latex/ucs/INSTALL",
    "texmf-dist/doc/latex/ucs/LICENSE",
    "texmf-dist/doc/latex/ucs/README",
    "texmf-dist/doc/latex/ucs/README.TEXLIVE",
    "texmf-dist/doc/latex/ucs/VERSION",
    "texmf-dist/doc/latex/ucs/ltxmacrs.txt",
    "texmf-dist/doc/latex/ucs/ucs.pdf",
    "texmf-dist/doc/latex/ucs/unsupported/README",
    "texmf-dist/doc/latex/ucs/unsupported/sym-to-fontenc.txt",
    "texmf-dist/doc/latex/ucs/unsupported/u2ps",
  },
  ["cancel"] = {
    "texmf-dist/doc/latex/cancel/cancel.pdf",
  },
  ["rlepsf"] = {
    "texmf-dist/doc/generic/rlepsf/rlepsdoc.ps",
  },
  ["asana-math"] = {
    "texmf-dist/doc/fonts/Asana-Math/FontLog.txt",
    "texmf-dist/doc/fonts/Asana-Math/README",
  },
  ["hepunits"] = {
    "texmf-dist/doc/latex/hepunits/ChangeLog",
    "texmf-dist/doc/latex/hepunits/README",
    "texmf-dist/doc/latex/hepunits/hepunits.pdf",
  },
  ["vocaltract"] = {
    "texmf-dist/doc/latex/vocaltract/README",
    "texmf-dist/doc/latex/vocaltract/vtLatex_FigureDemo.pdf",
  },
  ["galois"] = {
    "texmf-dist/doc/latex/galois/README",
    "texmf-dist/doc/latex/galois/galois.pdf",
  },
  ["babel-portuges"] = {
    "texmf-dist/doc/generic/babel-portuges/portuges.pdf",
  },
  ["rcs"] = {
    "texmf-dist/doc/latex/rcs/CATALOG",
    "texmf-dist/doc/latex/rcs/History",
    "texmf-dist/doc/latex/rcs/INSTALL",
    "texmf-dist/doc/latex/rcs/License",
    "texmf-dist/doc/latex/rcs/MANIFEST",
    "texmf-dist/doc/latex/rcs/README",
    "texmf-dist/doc/latex/rcs/rcs-conf.pdf",
    "texmf-dist/doc/latex/rcs/rcs-user.pdf",
  },
  ["tds"] = {
    "texmf-dist/doc/generic/tds/ChangeLog",
    "texmf-dist/doc/generic/tds/Makefile",
    "texmf-dist/doc/generic/tds/README",
    "texmf-dist/doc/generic/tds/index.html",
    "texmf-dist/doc/generic/tds/tds.dvi",
    "texmf-dist/doc/generic/tds/tds.html",
    "texmf-dist/doc/generic/tds/tds.pdf",
  },
  ["ecv"] = {
    "texmf-dist/doc/latex/ecv/COPYING",
    "texmf-dist/doc/latex/ecv/README",
    "texmf-dist/doc/latex/ecv/ecv.pdf",
    "texmf-dist/doc/latex/ecv/template/CV-template_de.pdf",
    "texmf-dist/doc/latex/ecv/template/CV-template_en.pdf",
    "texmf-dist/doc/latex/ecv/template/Makefile",
  },
  ["gmdoc"] = {
    "texmf-dist/doc/latex/gmdoc/README",
    "texmf-dist/doc/latex/gmdoc/gmdoc.pdf",
  },
  ["hobby"] = {
    "texmf-dist/doc/latex/hobby/README",
    "texmf-dist/doc/latex/hobby/hobby.pdf",
    "texmf-dist/doc/latex/hobby/hobby_code.pdf",
  },
  ["changelayout"] = {
    "texmf-dist/doc/latex/changelayout/README",
    "texmf-dist/doc/latex/changelayout/changelayout-guide.pdf",
  },
  ["datetime2-catalan"] = {
    "texmf-dist/doc/latex/datetime2-catalan/README",
    "texmf-dist/doc/latex/datetime2-catalan/datetime2-catalan.pdf",
  },
  ["glossaries-serbian"] = {
    "texmf-dist/doc/latex/glossaries-serbian/README",
    "texmf-dist/doc/latex/glossaries-serbian/glossaries-serbian.pdf",
  },
  ["metrix"] = {
    "texmf-dist/doc/latex/metrix/README",
    "texmf-dist/doc/latex/metrix/metrix.pdf",
  },
  ["fragments"] = {
    "texmf-dist/doc/latex/fragments/README",
  },
  ["metauml"] = {
    "texmf-dist/doc/metapost/metauml/README",
    "texmf-dist/doc/metapost/metauml/metauml_manual_0.2.5.pdf",
  },
  ["braids"] = {
    "texmf-dist/doc/latex/braids/README",
    "texmf-dist/doc/latex/braids/braids_doc.pdf",
  },
  ["mtgreek"] = {
    "texmf-dist/doc/latex/mtgreek/mtgreek.pdf",
  },
  ["aleph"] = {
    "texmf-dist/doc/aleph/base/News",
    "texmf-dist/doc/aleph/base/readme.txt",
    "texmf-dist/doc/man/man1/aleph.man1.pdf",
    "texmf-dist/doc/man/man1/lamed.man1.pdf",
  },
  ["glossaries-french"] = {
    "texmf-dist/doc/latex/glossaries-french/README",
    "texmf-dist/doc/latex/glossaries-french/glossaries-french.pdf",
  },
  ["docsurvey"] = {
    "texmf-dist/doc/latex/docsurvey/README.txt",
    "texmf-dist/doc/latex/docsurvey/docsurvey.pdf",
  },
  ["babel-catalan"] = {
    "texmf-dist/doc/generic/babel-catalan/catalan.pdf",
  },
  ["erw-l3"] = {
    "texmf-dist/doc/latex/erw-l3/README.txt",
    "texmf-dist/doc/latex/erw-l3/erw-l3.pdf",
  },
  ["epsf-dvipdfmx"] = {
    "texmf-dist/doc/plain/epsf-dvipdfmx/README",
  },
  ["latex"] = {
    "texmf-dist/doc/latex/base/README.txt",
    "texmf-dist/doc/latex/base/alltt.pdf",
    "texmf-dist/doc/latex/base/bugs.txt",
    "texmf-dist/doc/latex/base/cfgguide.pdf",
    "texmf-dist/doc/latex/base/changes-doc.txt",
    "texmf-dist/doc/latex/base/changes.old.txt",
    "texmf-dist/doc/latex/base/changes.txt",
    "texmf-dist/doc/latex/base/classes.pdf",
    "texmf-dist/doc/latex/base/clsguide.pdf",
    "texmf-dist/doc/latex/base/cmfonts.pdf",
    "texmf-dist/doc/latex/base/cyrguide.pdf",
    "texmf-dist/doc/latex/base/doc.pdf",
    "texmf-dist/doc/latex/base/docstrip.pdf",
    "texmf-dist/doc/latex/base/encguide.pdf",
    "texmf-dist/doc/latex/base/exscale.pdf",
    "texmf-dist/doc/latex/base/fix-cm.pdf",
    "texmf-dist/doc/latex/base/fntguide.pdf",
    "texmf-dist/doc/latex/base/graphpap.pdf",
    "texmf-dist/doc/latex/base/ifthen.pdf",
    "texmf-dist/doc/latex/base/inputenc.pdf",
    "texmf-dist/doc/latex/base/latexchanges.pdf",
    "texmf-dist/doc/latex/base/latexrelease.pdf",
    "texmf-dist/doc/latex/base/latexsym.pdf",
    "texmf-dist/doc/latex/base/lb2.pdf",
    "texmf-dist/doc/latex/base/legal.txt",
    "texmf-dist/doc/latex/base/letter.pdf",
    "texmf-dist/doc/latex/base/lgc2.pdf",
    "texmf-dist/doc/latex/base/lppl-1-0.txt",
    "texmf-dist/doc/latex/base/lppl-1-1.txt",
    "texmf-dist/doc/latex/base/lppl-1-2.txt",
    "texmf-dist/doc/latex/base/lppl.pdf",
    "texmf-dist/doc/latex/base/lppl.txt",
    "texmf-dist/doc/latex/base/ltluatex.pdf",
    "texmf-dist/doc/latex/base/ltnews.pdf",
    "texmf-dist/doc/latex/base/ltx3info.pdf",
    "texmf-dist/doc/latex/base/ltxdoc.pdf",
    "texmf-dist/doc/latex/base/makeindx.pdf",
    "texmf-dist/doc/latex/base/manifest.txt",
    "texmf-dist/doc/latex/base/manual.pdf",
    "texmf-dist/doc/latex/base/modguide.pdf",
    "texmf-dist/doc/latex/base/nfssfont.pdf",
    "texmf-dist/doc/latex/base/proc.pdf",
    "texmf-dist/doc/latex/base/slides.pdf",
    "texmf-dist/doc/latex/base/slifonts.pdf",
    "texmf-dist/doc/latex/base/source2e.pdf",
    "texmf-dist/doc/latex/base/syntonly.pdf",
    "texmf-dist/doc/latex/base/tex2.txt",
    "texmf-dist/doc/latex/base/texpert.txt",
    "texmf-dist/doc/latex/base/tlc2.pdf",
    "texmf-dist/doc/latex/base/tulm.pdf",
    "texmf-dist/doc/latex/base/usrguide.pdf",
    "texmf-dist/doc/latex/base/utf8ienc.pdf",
    "texmf-dist/doc/latex/base/webcomp.pdf",
  },
  ["duerer-latex"] = {
    "texmf-dist/doc/latex/duerer-latex/README",
    "texmf-dist/doc/latex/duerer-latex/duerer.pdf",
  },
  ["c90"] = {
    "texmf-dist/doc/fonts/enc/c90/c90.pdf",
  },
  ["babel-romanian"] = {
    "texmf-dist/doc/generic/babel-romanian/romanian.pdf",
  },
  ["bxjalipsum"] = {
    "texmf-dist/doc/latex/bxjalipsum/LICENSE",
    "texmf-dist/doc/latex/bxjalipsum/README.md",
  },
  ["datetime2-estonian"] = {
    "texmf-dist/doc/latex/datetime2-estonian/README",
    "texmf-dist/doc/latex/datetime2-estonian/datetime2-estonian.pdf",
  },
  ["babel-estonian"] = {
    "texmf-dist/doc/generic/babel-estonian/README.txt",
    "texmf-dist/doc/generic/babel-estonian/estonian.pdf",
  },
  ["dvisvgm"] = {
    "texmf-dist/doc/man/man1/dvisvgm.man1.pdf",
  },
  ["cmdstring"] = {
    "texmf-dist/doc/latex/cmdstring/README",
    "texmf-dist/doc/latex/cmdstring/cmdstring.pdf",
  },
  ["endiagram"] = {
    "texmf-dist/doc/latex/endiagram/README",
    "texmf-dist/doc/latex/endiagram/endiagram_en.pdf",
  },
  ["bropd"] = {
    "texmf-dist/doc/latex/bropd/README",
    "texmf-dist/doc/latex/bropd/bropd.pdf",
  },
  ["libertinust1math"] = {
    "texmf-dist/doc/fonts/libertinust1math/FONTLOG.txt",
    "texmf-dist/doc/fonts/libertinust1math/LibertinusT1Math-doc.pdf",
    "texmf-dist/doc/fonts/libertinust1math/OFL.txt",
    "texmf-dist/doc/fonts/libertinust1math/README",
    "texmf-dist/doc/fonts/libertinust1math/sample1-crop.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample1.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample10-crop.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample10.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample11-crop.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample11.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample2-crop.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample2.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample3-crop.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample3.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample4-crop.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample4.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample5-crop.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample5.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample6-crop.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample6.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample7-crop.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample7.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample8-crop.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample8.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample9-crop.pdf",
    "texmf-dist/doc/fonts/libertinust1math/sample9.pdf",
  },
  ["elzcards"] = {
    "texmf-dist/doc/latex/elzcards/README",
    "texmf-dist/doc/latex/elzcards/elzcards-examples.pdf",
    "texmf-dist/doc/latex/elzcards/elzcards.pdf",
  },
  ["notestex"] = {
    "texmf-dist/doc/latex/notestex/NotesTeX.pdf",
    "texmf-dist/doc/latex/notestex/README.md",
  },
  ["arimo"] = {
    "texmf-dist/doc/fonts/arimo/LICENSE.txt",
    "texmf-dist/doc/fonts/arimo/README",
    "texmf-dist/doc/fonts/arimo/arimo-samples.pdf",
  },
  ["cjk-ko"] = {
    "texmf-dist/doc/latex/cjk-ko/ChangeLog",
    "texmf-dist/doc/latex/cjk-ko/README",
    "texmf-dist/doc/latex/cjk-ko/cjk-ko-doc.pdf",
  },
  ["idxcmds"] = {
    "texmf-dist/doc/latex/idxcmds/README",
    "texmf-dist/doc/latex/idxcmds/idxcmds_en.pdf",
  },
  ["babel-sorbian"] = {
    "texmf-dist/doc/generic/babel-sorbian/lsorbian.pdf",
    "texmf-dist/doc/generic/babel-sorbian/usorbian.pdf",
  },
  ["metaobj"] = {
    "texmf-dist/doc/metapost/metaobj/README",
    "texmf-dist/doc/metapost/metaobj/license.txt",
    "texmf-dist/doc/metapost/metaobj/momanual.pdf",
  },
  ["arphic-ttf"] = {
    "texmf-dist/doc/fonts/arphic-ttf/ARPHICPL.txt",
    "texmf-dist/doc/fonts/arphic-ttf/README.md",
    "texmf-dist/doc/fonts/arphic-ttf/release.txt",
  },
  ["lshort-english"] = {
    "texmf-dist/doc/latex/lshort-english/CHANGES",
    "texmf-dist/doc/latex/lshort-english/README",
    "texmf-dist/doc/latex/lshort-english/lshort.pdf",
  },
  ["greektonoi"] = {
    "texmf-dist/doc/latex/greektonoi/README.txt",
    "texmf-dist/doc/latex/greektonoi/greektonoi-en.pdf",
    "texmf-dist/doc/latex/greektonoi/greektonoi.pdf",
  },
  ["datetime2-romanian"] = {
    "texmf-dist/doc/latex/datetime2-romanian/README",
    "texmf-dist/doc/latex/datetime2-romanian/datetime2-romanian-test-luatex.pdf",
    "texmf-dist/doc/latex/datetime2-romanian/datetime2-romanian-test-pdftex.pdf",
    "texmf-dist/doc/latex/datetime2-romanian/datetime2-romanian-test-xetex.pdf",
    "texmf-dist/doc/latex/datetime2-romanian/datetime2-romanian.pdf",
  },
  ["ntheorem-vn"] = {
    "texmf-dist/doc/latex/ntheorem-vn/COPYING",
    "texmf-dist/doc/latex/ntheorem-vn/FILELIST",
    "texmf-dist/doc/latex/ntheorem-vn/README",
    "texmf-dist/doc/latex/ntheorem-vn/ntheorem-doc-vn.pdf",
    "texmf-dist/doc/latex/ntheorem-vn/src/Makefile",
    "texmf-dist/doc/latex/ntheorem-vn/src/README.src",
    "texmf-dist/doc/latex/ntheorem-vn/src/TODO",
    "texmf-dist/doc/latex/ntheorem-vn/test.pdf",
  },
  ["diadia"] = {
    "texmf-dist/doc/latex/diadia/README",
    "texmf-dist/doc/latex/diadia/diadia-example.pdf",
    "texmf-dist/doc/latex/diadia/diadia.pdf",
    "texmf-dist/doc/latex/diadia/makefile",
    "texmf-dist/doc/latex/diadia/manifest.txt",
  },
  ["gmp"] = {
    "texmf-dist/doc/latex/gmp/README",
    "texmf-dist/doc/latex/gmp/gmp.pdf",
  },
  ["diagnose"] = {
    "texmf-dist/doc/latex/diagnose/INSTALL",
    "texmf-dist/doc/latex/diagnose/README",
    "texmf-dist/doc/latex/diagnose/diagnose.pdf",
  },
  ["decorule"] = {
    "texmf-dist/doc/latex/decorule/MANIFEST",
    "texmf-dist/doc/latex/decorule/README",
    "texmf-dist/doc/latex/decorule/decorule.pdf",
  },
  ["tabularew"] = {
    "texmf-dist/doc/latex/tabularew/README",
    "texmf-dist/doc/latex/tabularew/tabularew.pdf",
  },
  ["bookhands"] = {
    "texmf-dist/doc/fonts/bookhands/README.PRW",
    "texmf-dist/doc/fonts/bookhands/allsqrcaps.pdf",
    "texmf-dist/doc/fonts/bookhands/auncial-README",
    "texmf-dist/doc/fonts/bookhands/carolmin-README",
    "texmf-dist/doc/fonts/bookhands/egothic-README",
    "texmf-dist/doc/fonts/bookhands/humanist-README",
    "texmf-dist/doc/fonts/bookhands/huncial-README",
    "texmf-dist/doc/fonts/bookhands/inslrmaj-README",
    "texmf-dist/doc/fonts/bookhands/inslrmin-README",
    "texmf-dist/doc/fonts/bookhands/pgothic-README",
    "texmf-dist/doc/fonts/bookhands/pgothic-tryfont.pdf",
    "texmf-dist/doc/fonts/bookhands/rotunda-README",
    "texmf-dist/doc/fonts/bookhands/rustic-README",
    "texmf-dist/doc/fonts/bookhands/sqrcaps-README",
    "texmf-dist/doc/fonts/bookhands/sqrcaps.pdf",
    "texmf-dist/doc/fonts/bookhands/tgothic-README",
    "texmf-dist/doc/fonts/bookhands/trysqrcaps.pdf",
    "texmf-dist/doc/fonts/bookhands/uncial-README",
  },
  ["units"] = {
    "texmf-dist/doc/latex/units/COPYING",
    "texmf-dist/doc/latex/units/README",
    "texmf-dist/doc/latex/units/units.pdf",
  },
  ["ascii-chart"] = {
    "texmf-dist/doc/support/ascii-chart/ascii.pdf",
  },
  ["metafont"] = {
    "texmf-dist/doc/man/man1/inimf.man1.pdf",
    "texmf-dist/doc/man/man1/mf-nowin.man1.pdf",
    "texmf-dist/doc/man/man1/mf.man1.pdf",
  },
  ["hyphen-spanish"] = {
    "texmf-dist/doc/generic/hyph-utf8/es/README",
    "texmf-dist/doc/generic/hyph-utf8/es/division.pdf",
  },
  ["datetime2-ukrainian"] = {
    "texmf-dist/doc/latex/datetime2-ukrainian/README.md",
    "texmf-dist/doc/latex/datetime2-ukrainian/datetime2-ukrainian.pdf",
  },
  ["cryst"] = {
    "texmf-dist/doc/latex/cryst/README",
    "texmf-dist/doc/latex/cryst/cryst1.pdf",
  },
  ["tagpdf"] = {
    "texmf-dist/doc/latex/tagpdf/README.md",
    "texmf-dist/doc/latex/tagpdf/ex-mc-manual-para-split.pdf",
    "texmf-dist/doc/latex/tagpdf/ex-patch-list-luatex.pdf",
    "texmf-dist/doc/latex/tagpdf/ex-patch-list.pdf",
    "texmf-dist/doc/latex/tagpdf/ex-patch-sectioning-koma-luamode.pdf",
    "texmf-dist/doc/latex/tagpdf/ex-patch-sectioning-koma.pdf",
    "texmf-dist/doc/latex/tagpdf/ex-structure.pdf",
    "texmf-dist/doc/latex/tagpdf/ex-tagpdf-template.pdf",
    "texmf-dist/doc/latex/tagpdf/tagpdf.pdf",
  },
  ["merriweather"] = {
    "texmf-dist/doc/fonts/merriweather/OFL.txt",
    "texmf-dist/doc/fonts/merriweather/README",
    "texmf-dist/doc/fonts/merriweather/merriweather-samples.pdf",
  },
  ["betababel"] = {
    "texmf-dist/doc/latex/betababel/betatest.pdf",
  },
  ["menukeys"] = {
    "texmf-dist/doc/latex/menukeys/README",
    "texmf-dist/doc/latex/menukeys/menukeys.pdf",
  },
  ["menu"] = {
    "texmf-dist/doc/latex/menu/README",
    "texmf-dist/doc/latex/menu/menu.pdf",
  },
  ["mentis"] = {
    "texmf-dist/doc/latex/mentis/README",
    "texmf-dist/doc/latex/mentis/mentis.pdf",
  },
  ["pdfbook2"] = {
    "texmf-dist/doc/man/man1/pdfbook2.man1.pdf",
    "texmf-dist/doc/support/pdfbook2/README",
  },
  ["memory"] = {
    "texmf-dist/doc/latex/memory/README",
    "texmf-dist/doc/latex/memory/memory.pdf",
  },
  ["iftex"] = {
    "texmf-dist/doc/generic/iftex/README",
    "texmf-dist/doc/generic/iftex/iftex.pdf",
  },
  ["medstarbeamer"] = {
    "texmf-dist/doc/latex/medstarbeamer/README.txt",
  },
  ["examdesign"] = {
    "texmf-dist/doc/latex/examdesign/Bugs",
    "texmf-dist/doc/latex/examdesign/Changes",
    "texmf-dist/doc/latex/examdesign/INSTALL",
    "texmf-dist/doc/latex/examdesign/README",
    "texmf-dist/doc/latex/examdesign/examdesign.pdf",
    "texmf-dist/doc/latex/examdesign/examplea.pdf",
    "texmf-dist/doc/latex/examdesign/exampleb.pdf",
    "texmf-dist/doc/latex/examdesign/examplec.pdf",
  },
  ["pst-stru"] = {
    "texmf-dist/doc/generic/pst-stru/Changes",
    "texmf-dist/doc/generic/pst-stru/README",
    "texmf-dist/doc/generic/pst-stru/pst-stru-doc.pdf",
  },
  ["biblatex-trad"] = {
    "texmf-dist/doc/latex/biblatex-trad/README.md",
    "texmf-dist/doc/latex/biblatex-trad/biblatex-trad.pdf",
  },
  ["getmap"] = {
    "texmf-dist/doc/latex/getmap/README.md",
    "texmf-dist/doc/latex/getmap/getmap.pdf",
    "texmf-dist/doc/latex/getmap/makefile",
    "texmf-dist/doc/latex/getmap/manifest.txt",
  },
  ["tex-gyre-math"] = {
    "texmf-dist/doc/fonts/tex-gyre-math/GUST-FONT-LICENSE.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/INSTALL.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/LICENSE-DejaVu.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/MANIFEST-TeX-Gyre-Bonum-Math.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/MANIFEST-TeX-Gyre-DejaVu-Math.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/MANIFEST-TeX-Gyre-Pagella-Math.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/MANIFEST-TeX-Gyre-Schola-Math.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/MANIFEST-TeX-Gyre-Termes-Math.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/README",
    "texmf-dist/doc/fonts/tex-gyre-math/README-TeX-Gyre-Bonum-Math.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/README-TeX-Gyre-DejaVu-Math.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/README-TeX-Gyre-Pagella-Math.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/README-TeX-Gyre-Schola-Math.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/README-TeX-Gyre-Termes-Math.txt",
    "texmf-dist/doc/fonts/tex-gyre-math/test-context-texgyre_bonum_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-context-texgyre_dejavu_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-context-texgyre_pagella_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-context-texgyre_schola_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-context-texgyre_termes_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-lualatex-texgyre_bonum_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-lualatex-texgyre_dejavu_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-lualatex-texgyre_pagella_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-lualatex-texgyre_schola_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-lualatex-texgyre_termes_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-word-texgyre_bonum_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-word-texgyre_dejavu_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-word-texgyre_pagella_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-word-texgyre_schola_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-word-texgyre_termes_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-xelatex-texgyre_bonum_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-xelatex-texgyre_dejavu_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-xelatex-texgyre_pagella_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-xelatex-texgyre_schola_math.pdf",
    "texmf-dist/doc/fonts/tex-gyre-math/test-xelatex-texgyre_termes_math.pdf",
  },
  ["fwlw"] = {
    "texmf-dist/doc/latex/fwlw/README",
    "texmf-dist/doc/latex/fwlw/fwlw.pdf",
  },
  ["mcaption"] = {
    "texmf-dist/doc/latex/mcaption/CHANGES",
    "texmf-dist/doc/latex/mcaption/README",
    "texmf-dist/doc/latex/mcaption/mcaption.pdf",
  },
  ["mbenotes"] = {
    "texmf-dist/doc/latex/mbenotes/README",
    "texmf-dist/doc/latex/mbenotes/mbenotes.pdf",
  },
  ["maybemath"] = {
    "texmf-dist/doc/latex/maybemath/README",
    "texmf-dist/doc/latex/maybemath/maybemath.pdf",
  },
  ["mattens"] = {
    "texmf-dist/doc/latex/mattens/README",
    "texmf-dist/doc/latex/mattens/mattens.pdf",
    "texmf-dist/doc/latex/mattens/mattens_sample.pdf",
  },
  ["xassoccnt"] = {
    "texmf-dist/doc/latex/xassoccnt/README",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_counterformats_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_counterhierarchy_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_counternames_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_coupledcounters_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_doc.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_driver_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_getparentcounter_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_label_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_parentcounterchain_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_periodiccounters_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_resetlist_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_resetloop_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_suspendedcounters_example.pdf",
    "texmf-dist/doc/latex/xassoccnt/xassoccnt_totalcounters_example.pdf",
  },
  ["copyrightbox"] = {
    "texmf-dist/doc/latex/copyrightbox/README",
    "texmf-dist/doc/latex/copyrightbox/copyrightbox.pdf",
  },
  ["crossrefware"] = {
    "texmf-dist/doc/man/man1/bbl2bib.man1.pdf",
    "texmf-dist/doc/man/man1/bibdoiadd.man1.pdf",
    "texmf-dist/doc/man/man1/bibmradd.man1.pdf",
    "texmf-dist/doc/man/man1/biburl2doi.man1.pdf",
    "texmf-dist/doc/man/man1/bibzbladd.man1.pdf",
    "texmf-dist/doc/man/man1/ltx2crossrefxml.man1.pdf",
    "texmf-dist/doc/support/crossrefware/Makefile",
    "texmf-dist/doc/support/crossrefware/README",
    "texmf-dist/doc/support/crossrefware/crossrefware.pdf",
  },
  ["tetex"] = {
    "texmf-dist/doc/man/man1/allcm.man1.pdf",
    "texmf-dist/doc/man/man1/allec.man1.pdf",
    "texmf-dist/doc/man/man1/allneeded.man1.pdf",
    "texmf-dist/doc/man/man1/dvi2fax.man1.pdf",
    "texmf-dist/doc/man/man1/dvired.man1.pdf",
    "texmf-dist/doc/man/man1/fmtutil-sys.man1.pdf",
    "texmf-dist/doc/man/man1/fmtutil-user.man1.pdf",
    "texmf-dist/doc/man/man1/fmtutil.man1.pdf",
    "texmf-dist/doc/man/man1/kpsepath.man1.pdf",
    "texmf-dist/doc/man/man1/kpsetool.man1.pdf",
    "texmf-dist/doc/man/man1/kpsewhere.man1.pdf",
    "texmf-dist/doc/man/man1/kpsexpand.man1.pdf",
    "texmf-dist/doc/man/man1/texconfig-sys.man1.pdf",
    "texmf-dist/doc/man/man1/texconfig.man1.pdf",
    "texmf-dist/doc/man/man1/texlinks.man1.pdf",
    "texmf-dist/doc/man/man1/updmap-sys.man1.pdf",
    "texmf-dist/doc/man/man1/updmap-user.man1.pdf",
    "texmf-dist/doc/man/man1/updmap.man1.pdf",
    "texmf-dist/doc/man/man5/fmtutil.cnf.man5.pdf",
    "texmf-dist/doc/man/man5/updmap.cfg.man5.pdf",
    "texmf-dist/doc/tetex/TETEXDOC.pdf",
    "texmf-dist/doc/tetex/teTeX-FAQ",
  },
  ["datetime2-galician"] = {
    "texmf-dist/doc/latex/datetime2-galician/README",
    "texmf-dist/doc/latex/datetime2-galician/datetime2-galician.pdf",
  },
  ["trajan"] = {
    "texmf-dist/doc/latex/trajan/README",
    "texmf-dist/doc/latex/trajan/trajan.pdf",
    "texmf-dist/doc/latex/trajan/trytrajan.pdf",
  },
  ["datetime2-latin"] = {
    "texmf-dist/doc/latex/datetime2-latin/README",
    "texmf-dist/doc/latex/datetime2-latin/datetime2-latin.pdf",
  },
  ["titlepic"] = {
    "texmf-dist/doc/latex/titlepic/README.md",
    "texmf-dist/doc/latex/titlepic/titlepic-manual.pdf",
  },
  ["detex"] = {
    "texmf-dist/doc/man/man1/detex.man1.pdf",
  },
  ["topfloat"] = {
    "texmf-dist/doc/latex/topfloat/topfloat.pdf",
  },
  ["bardiag"] = {
    "texmf-dist/doc/latex/bardiag/README",
    "texmf-dist/doc/latex/bardiag/bardiag.pdf",
    "texmf-dist/doc/latex/bardiag/bardiag1.pdf",
    "texmf-dist/doc/latex/bardiag/bardiag2.pdf",
    "texmf-dist/doc/latex/bardiag/example/altdiags.ps",
    "texmf-dist/doc/latex/bardiag/example/diagrams.dvi",
    "texmf-dist/doc/latex/bardiag/example/diagrams.ps",
    "texmf-dist/doc/latex/bardiag/example/diagramsbw.ps",
  },
  ["beamercolorthemeowl"] = {
    "texmf-dist/doc/latex/beamercolorthemeowl/Makefile",
    "texmf-dist/doc/latex/beamercolorthemeowl/README.md",
    "texmf-dist/doc/latex/beamercolorthemeowl/beamercolorthemeowl.pdf",
    "texmf-dist/doc/latex/beamercolorthemeowl/ex/readme.jpg",
  },
  ["mathspic"] = {
    "texmf-dist/doc/latex/mathspic/HELP.txt",
    "texmf-dist/doc/latex/mathspic/README.txt",
    "texmf-dist/doc/latex/mathspic/mathsPICmanual.pdf",
    "texmf-dist/doc/latex/mathspic/sourcecode113.html",
    "texmf-dist/doc/latex/mathspic/sourcecode113.pdf",
    "texmf-dist/doc/man/man1/mathspic.man1.pdf",
  },
  ["bookest"] = {
    "texmf-dist/doc/latex/bookest/README",
    "texmf-dist/doc/latex/bookest/bookestdoc-en.pdf",
    "texmf-dist/doc/latex/bookest/bookestdoc-it.pdf",
  },
  ["mathspec"] = {
    "texmf-dist/doc/xelatex/mathspec/README.txt",
    "texmf-dist/doc/xelatex/mathspec/mathspec.pdf",
  },
  ["autoarea"] = {
    "texmf-dist/doc/latex/autoarea/ANNOUNCE.txt",
    "texmf-dist/doc/latex/autoarea/README.aa",
    "texmf-dist/doc/latex/autoarea/autodemo/README.autodemo",
    "texmf-dist/doc/latex/autoarea/autodemo/autodemo+.pdf",
    "texmf-dist/doc/latex/autoarea/autodemo/autodemo-.pdf",
  },
  ["dirtytalk"] = {
    "texmf-dist/doc/latex/dirtytalk/README",
    "texmf-dist/doc/latex/dirtytalk/dirtytalk.pdf",
  },
  ["mathpazo"] = {
    "texmf-dist/doc/latex/mathpazo/README.txt",
    "texmf-dist/doc/latex/mathpazo/gpl.txt",
    "texmf-dist/doc/latex/mathpazo/pazotest.pdf",
  },
  ["mathfixs"] = {
    "texmf-dist/doc/latex/mathfixs/README.txt",
    "texmf-dist/doc/latex/mathfixs/mathfixs.pdf",
  },
  ["cv"] = {
    "texmf-dist/doc/latex/cv/ApplicationLetter.pdf",
    "texmf-dist/doc/latex/cv/CVCTAN.pdf",
    "texmf-dist/doc/latex/cv/README",
  },
  ["frcursive"] = {
    "texmf-dist/doc/fonts/frcursive/COPYING",
    "texmf-dist/doc/fonts/frcursive/README",
    "texmf-dist/doc/fonts/frcursive/frcursive.pdf",
  },
  ["asyfig"] = {
    "texmf-dist/doc/latex/asyfig/README",
    "texmf-dist/doc/latex/asyfig/asyfig.pdf",
  },
  ["fixmetodonotes"] = {
    "texmf-dist/doc/latex/fixmetodonotes/LICENSE",
    "texmf-dist/doc/latex/fixmetodonotes/README",
  },
  ["documentation"] = {
    "texmf-dist/doc/latex/documentation/README",
    "texmf-dist/doc/latex/documentation/documentation.pdf",
  },
  ["biblatex-nejm"] = {
    "texmf-dist/doc/latex/biblatex-nejm/README",
    "texmf-dist/doc/latex/biblatex-nejm/biblatex-nejm.pdf",
  },
  ["embedall"] = {
    "texmf-dist/doc/latex/embedall/embedall.pdf",
  },
  ["mugsthesis"] = {
    "texmf-dist/doc/latex/mugsthesis/README",
    "texmf-dist/doc/latex/mugsthesis/mugsthesis.pdf",
    "texmf-dist/doc/latex/mugsthesis/sample/mugsthesis_sample.pdf",
  },
  ["mdwtools"] = {
    "texmf-dist/doc/latex/mdwtools/COPYING",
    "texmf-dist/doc/latex/mdwtools/README",
    "texmf-dist/doc/latex/mdwtools/at.pdf",
    "texmf-dist/doc/latex/mdwtools/cmtt.pdf",
    "texmf-dist/doc/latex/mdwtools/doafter.pdf",
    "texmf-dist/doc/latex/mdwtools/footnote.pdf",
    "texmf-dist/doc/latex/mdwtools/mdwlist.pdf",
    "texmf-dist/doc/latex/mdwtools/mdwmath.pdf",
    "texmf-dist/doc/latex/mdwtools/mdwtab.pdf",
    "texmf-dist/doc/latex/mdwtools/sverb.pdf",
    "texmf-dist/doc/latex/mdwtools/syntax.pdf",
  },
  ["embrac"] = {
    "texmf-dist/doc/latex/embrac/README",
    "texmf-dist/doc/latex/embrac/embrac_en.pdf",
  },
  ["beamertheme-cuerna"] = {
    "texmf-dist/doc/latex/beamertheme-cuerna/README",
    "texmf-dist/doc/latex/beamertheme-cuerna/beamertheme-cuerna.pdf",
    "texmf-dist/doc/latex/beamertheme-cuerna/pictures/bluesimplexexample.pdf",
    "texmf-dist/doc/latex/beamertheme-cuerna/pictures/brickexample.pdf",
    "texmf-dist/doc/latex/beamertheme-cuerna/pictures/defaultexample.pdf",
    "texmf-dist/doc/latex/beamertheme-cuerna/pictures/lettuceexample.pdf",
  },
  ["context-cyrillicnumbers"] = {
    "texmf-dist/doc/context/third/cyrillicnumbers/COPYING",
    "texmf-dist/doc/context/third/cyrillicnumbers/README.rst",
    "texmf-dist/doc/context/third/cyrillicnumbers/VERSION",
    "texmf-dist/doc/context/third/cyrillicnumbers/cyrillicnumbers.pdf",
  },
  ["tensor"] = {
    "texmf-dist/doc/latex/tensor/README",
    "texmf-dist/doc/latex/tensor/tensor.pdf",
  },
  ["markdown"] = {
    "texmf-dist/doc/generic/markdown/markdown.html",
    "texmf-dist/doc/generic/markdown/markdown.pdf",
  },
  ["multiexpand"] = {
    "texmf-dist/doc/generic/multiexpand/README.md",
    "texmf-dist/doc/generic/multiexpand/multiexpand.pdf",
  },
  ["xbmks"] = {
    "texmf-dist/doc/latex/xbmks/README.md",
    "texmf-dist/doc/latex/xbmks/docs/xbmks.pdf",
    "texmf-dist/doc/latex/xbmks/docs/xbmksman.pdf",
    "texmf-dist/doc/latex/xbmks/examples/doc1.pdf",
    "texmf-dist/doc/latex/xbmks/examples/doc2.pdf",
    "texmf-dist/doc/latex/xbmks/examples/master.pdf",
  },
  ["mandi"] = {
    "texmf-dist/doc/latex/mandi/README",
    "texmf-dist/doc/latex/mandi/mandi.pdf",
  },
  ["drac"] = {
    "texmf-dist/doc/latex/drac/drac-fr.pdf",
    "texmf-dist/doc/latex/drac/drac.pdf",
  },
  ["montex"] = {
    "texmf-dist/doc/latex/montex/00README",
    "texmf-dist/doc/latex/montex/00readme.mfinput.km",
    "texmf-dist/doc/latex/montex/ANNOUNCE",
    "texmf-dist/doc/latex/montex/EMTEX",
    "texmf-dist/doc/latex/montex/HISTORY",
    "texmf-dist/doc/latex/montex/INSTALL",
    "texmf-dist/doc/latex/montex/MIKTEX",
    "texmf-dist/doc/latex/montex/TODO",
    "texmf-dist/doc/latex/montex/UPDATE",
    "texmf-dist/doc/latex/montex/mlsquick.pdf",
    "texmf-dist/doc/latex/montex/montex.pdf",
    "texmf-dist/doc/latex/montex/zanabazr.pdf",
  },
  ["duckuments"] = {
    "texmf-dist/doc/latex/duckuments/README.md",
    "texmf-dist/doc/latex/duckuments/duckuments.pdf",
  },
  ["xdvi"] = {
    "texmf-dist/doc/man/man1/xdvi.man1.pdf",
  },
  ["jura"] = {
    "texmf-dist/doc/latex/jura/README",
    "texmf-dist/doc/latex/jura/jura.pdf",
    "texmf-dist/doc/latex/jura/jura2html",
  },
  ["components-of-tex"] = {
    "texmf-dist/doc/generic/components-of-TeX/README",
  },
  ["ifsym"] = {
    "texmf-dist/doc/fonts/ifsym/ifsym.ps",
  },
  ["tikz-timing"] = {
    "texmf-dist/doc/latex/tikz-timing/README",
    "texmf-dist/doc/latex/tikz-timing/tikz-timing.pdf",
  },
  ["graphviz"] = {
    "texmf-dist/doc/latex/graphviz/LICENSE",
    "texmf-dist/doc/latex/graphviz/Makefile",
    "texmf-dist/doc/latex/graphviz/README",
    "texmf-dist/doc/latex/graphviz/README.md",
    "texmf-dist/doc/latex/graphviz/graphviz.pdf",
    "texmf-dist/doc/latex/graphviz/test/Makefile",
  },
  ["luahyphenrules"] = {
    "texmf-dist/doc/lualatex/luahyphenrules/README.md",
    "texmf-dist/doc/lualatex/luahyphenrules/luahyphenrules.pdf",
  },
  ["bclogo"] = {
    "texmf-dist/doc/latex/bclogo/LISEZ-MOI",
    "texmf-dist/doc/latex/bclogo/README",
    "texmf-dist/doc/latex/bclogo/bclogo-doc.pdf",
    "texmf-dist/doc/latex/bclogo/bclogo.pdf",
    "texmf-dist/doc/latex/bclogo/fond.pdf",
    "texmf-dist/doc/latex/bclogo/syracuse-crop.pdf",
    "texmf-dist/doc/latex/bclogo/syracuse.pdf",
  },
  ["cascadilla"] = {
    "texmf-dist/doc/latex/cascadilla/LICENSE.txt",
    "texmf-dist/doc/latex/cascadilla/README",
    "texmf-dist/doc/latex/cascadilla/example.pdf",
  },
  ["handin"] = {
    "texmf-dist/doc/latex/handin/README.txt",
    "texmf-dist/doc/latex/handin/example.pdf",
    "texmf-dist/doc/latex/handin/handin-doc.pdf",
    "texmf-dist/doc/latex/handin/layout.pdf",
  },
  ["latex-mr"] = {
    "texmf-dist/doc/latex/latex-mr/README.md",
    "texmf-dist/doc/latex/latex-mr/latex-mr.pdf",
  },
  ["biblatex-socialscienceshuberlin"] = {
    "texmf-dist/doc/latex/biblatex-socialscienceshuberlin/README.md",
    "texmf-dist/doc/latex/biblatex-socialscienceshuberlin/socialscienceshuberlin.pdf",
  },
  ["biolett-bst"] = {
    "texmf-dist/doc/bibtex/biolett-bst/README.txt",
  },
  ["sides"] = {
    "texmf-dist/doc/latex/sides/README",
    "texmf-dist/doc/latex/sides/sides-sample.pdf",
  },
  ["censor"] = {
    "texmf-dist/doc/latex/censor/README",
    "texmf-dist/doc/latex/censor/censor.pdf",
  },
  ["hitec"] = {
    "texmf-dist/doc/latex/hitec/README",
    "texmf-dist/doc/latex/hitec/hitec_doc.pdf",
  },
  ["wadalab"] = {
    "texmf-dist/doc/fonts/wadalab/README",
    "texmf-dist/doc/fonts/wadalab/dgj/README",
    "texmf-dist/doc/fonts/wadalab/dmj/README",
    "texmf-dist/doc/fonts/wadalab/mc2j/README",
    "texmf-dist/doc/fonts/wadalab/mcj/README",
    "texmf-dist/doc/fonts/wadalab/mr2j/README",
    "texmf-dist/doc/fonts/wadalab/mrj/README",
    "texmf-dist/doc/fonts/wadalab/wadalab-sampler.pdf",
  },
  ["babel-scottish"] = {
    "texmf-dist/doc/generic/babel-scottish/scottish.pdf",
  },
  ["abbr"] = {
    "texmf-dist/doc/generic/abbr/README",
  },
  ["pgfmolbio"] = {
    "texmf-dist/doc/lualatex/pgfmolbio/README",
    "texmf-dist/doc/lualatex/pgfmolbio/SampleUniprot.txt",
    "texmf-dist/doc/lualatex/pgfmolbio/pgfmolbio.pdf",
  },
  ["limap"] = {
    "texmf-dist/doc/latex/limap/Makefile",
    "texmf-dist/doc/latex/limap/README.md",
    "texmf-dist/doc/latex/limap/limap.pdf",
    "texmf-dist/doc/latex/limap/lppl.txt",
    "texmf-dist/doc/latex/limap/samples/boxed-toc.pdf",
    "texmf-dist/doc/latex/limap/samples/hyper.pdf",
    "texmf-dist/doc/latex/limap/samples/nolines.pdf",
    "texmf-dist/doc/latex/limap/samples/sample.pdf",
  },
  ["mailing"] = {
    "texmf-dist/doc/latex/mailing/mailing.pdf",
    "texmf-dist/doc/latex/mailing/manifest.txt",
  },
  ["revtex"] = {
    "texmf-dist/doc/latex/revtex/00readme.tex",
    "texmf-dist/doc/latex/revtex/DOWNLOAD",
    "texmf-dist/doc/latex/revtex/README",
    "texmf-dist/doc/latex/revtex/aip/aipguide4-1.pdf",
    "texmf-dist/doc/latex/revtex/aps/apsguide4-1.pdf",
    "texmf-dist/doc/latex/revtex/auguide/auguide4-1.pdf",
    "texmf-dist/doc/latex/revtex/auguide/summary4-1.pdf",
    "texmf-dist/doc/latex/revtex/auguide/whatsnew4-1.pdf",
    "texmf-dist/doc/latex/revtex/sample/aip/aipsamp.pdf",
    "texmf-dist/doc/latex/revtex/sample/aps/apssamp.pdf",
    "texmf-dist/doc/latex/revtex/source/aip.pdf",
    "texmf-dist/doc/latex/revtex/source/ltxdocext.pdf",
    "texmf-dist/doc/latex/revtex/source/ltxfront.pdf",
    "texmf-dist/doc/latex/revtex/source/ltxgrid.pdf",
    "texmf-dist/doc/latex/revtex/source/ltxutil.pdf",
    "texmf-dist/doc/latex/revtex/source/revtex4-1.pdf",
  },
  ["mafr"] = {
    "texmf-dist/doc/latex/mafr/ALIRE",
    "texmf-dist/doc/latex/mafr/COPYING",
    "texmf-dist/doc/latex/mafr/README",
    "texmf-dist/doc/latex/mafr/docmafr.pdf",
  },
  ["bidi"] = {
    "texmf-dist/doc/xelatex/bidi/README",
    "texmf-dist/doc/xelatex/bidi/bidi-doc.pdf",
    "texmf-dist/doc/xelatex/bidi/bidi.pdf",
  },
  ["pmx"] = {
    "texmf-dist/doc/generic/pmx/README",
    "texmf-dist/doc/generic/pmx/gpl.txt",
    "texmf-dist/doc/generic/pmx/pmx-install.pdf",
    "texmf-dist/doc/generic/pmx/pmx25-284.html",
    "texmf-dist/doc/generic/pmx/pmx284.pdf",
    "texmf-dist/doc/generic/pmx/pmxab.pdf",
    "texmf-dist/doc/generic/pmx/ref284.pdf",
    "texmf-dist/doc/generic/pmx/scor2prt.pdf",
    "texmf-dist/doc/man/man1/pmxab.man1.pdf",
    "texmf-dist/doc/man/man1/scor2prt.man1.pdf",
  },
  ["datetime2-polish"] = {
    "texmf-dist/doc/latex/datetime2-polish/README",
    "texmf-dist/doc/latex/datetime2-polish/datetime2-polish.pdf",
  },
  ["pitex"] = {
    "texmf-dist/doc/plain/pitex/README",
    "texmf-dist/doc/plain/pitex/pitex-doc.pdf",
    "texmf-dist/doc/plain/pitex/pitex-doc.txt",
  },
  ["pgf-blur"] = {
    "texmf-dist/doc/latex/pgf-blur/README",
    "texmf-dist/doc/latex/pgf-blur/pgf-blur.pdf",
  },
  ["babel-welsh"] = {
    "texmf-dist/doc/generic/babel-welsh/README",
    "texmf-dist/doc/generic/babel-welsh/welsh.pdf",
  },
  ["edfnotes"] = {
    "texmf-dist/doc/latex/edfnotes/PdUsample.pdf",
    "texmf-dist/doc/latex/edfnotes/README",
    "texmf-dist/doc/latex/edfnotes/README.pdf",
    "texmf-dist/doc/latex/edfnotes/README.txt",
    "texmf-dist/doc/latex/edfnotes/SRCFILEs.txt",
    "texmf-dist/doc/latex/edfnotes/edfnotes.pdf",
  },
  ["brandeis-dissertation"] = {
    "texmf-dist/doc/latex/brandeis-dissertation/README",
    "texmf-dist/doc/latex/brandeis-dissertation/brandeis-dissertation.pdf",
  },
  ["fontwrap"] = {
    "texmf-dist/doc/xelatex/fontwrap/README",
    "texmf-dist/doc/xelatex/fontwrap/fontwrap.pdf",
    "texmf-dist/doc/xelatex/fontwrap/fontwrap_example.pdf",
  },
  ["postage"] = {
    "texmf-dist/doc/latex/postage/README.txt",
    "texmf-dist/doc/latex/postage/postage.pdf",
    "texmf-dist/doc/latex/postage/sample-stamps.pdf",
  },
  ["epstopdf"] = {
    "texmf-dist/doc/man/man1/epstopdf.man1.pdf",
    "texmf-dist/doc/man/man1/repstopdf.man1.pdf",
    "texmf-dist/doc/support/epstopdf/README",
  },
  ["bgteubner"] = {
    "texmf-dist/doc/latex/bgteubner/ChangeLog",
    "texmf-dist/doc/latex/bgteubner/LIESMICH",
    "texmf-dist/doc/latex/bgteubner/README",
    "texmf-dist/doc/latex/bgteubner/README.hhsubfigure",
    "texmf-dist/doc/latex/bgteubner/ToDo",
    "texmf-dist/doc/latex/bgteubner/bgteubner-cm.pdf",
    "texmf-dist/doc/latex/bgteubner/bgteubner.pdf",
    "texmf-dist/doc/latex/bgteubner/bgteucls.pdf",
    "texmf-dist/doc/latex/bgteubner/bild_ganz.pdf",
    "texmf-dist/doc/latex/bgteubner/bild_mitte.pdf",
    "texmf-dist/doc/latex/bgteubner/bild_oben.pdf",
    "texmf-dist/doc/latex/bgteubner/bild_oben_unten.pdf",
    "texmf-dist/doc/latex/bgteubner/bild_umflossen.pdf",
    "texmf-dist/doc/latex/bgteubner/bild_unten.pdf",
    "texmf-dist/doc/latex/bgteubner/bild_zu_lang.pdf",
    "texmf-dist/doc/latex/bgteubner/bild_zu_lang2.pdf",
    "texmf-dist/doc/latex/bgteubner/manifest.txt",
    "texmf-dist/doc/latex/bgteubner/math-cm.pdf",
    "texmf-dist/doc/latex/bgteubner/math-mathtime.pdf",
    "texmf-dist/doc/latex/bgteubner/math.pdf",
    "texmf-dist/doc/latex/bgteubner/umbruch1.pdf",
    "texmf-dist/doc/latex/bgteubner/umbruch1a.pdf",
    "texmf-dist/doc/latex/bgteubner/umbruch2.pdf",
    "texmf-dist/doc/latex/bgteubner/umbruch2a.pdf",
    "texmf-dist/doc/latex/bgteubner/umbruch3.pdf",
    "texmf-dist/doc/latex/bgteubner/umbruch3a.pdf",
    "texmf-dist/doc/latex/bgteubner/usefiles.txt",
  },
  ["luatodonotes"] = {
    "texmf-dist/doc/lualatex/luatodonotes/README.md",
    "texmf-dist/doc/lualatex/luatodonotes/luatodonotes.pdf",
  },
  ["luatextra"] = {
    "texmf-dist/doc/lualatex/luatextra/News",
    "texmf-dist/doc/lualatex/luatextra/README",
    "texmf-dist/doc/lualatex/luatextra/luatextra.pdf",
  },
  ["bidi-atbegshi"] = {
    "texmf-dist/doc/xelatex/bidi-atbegshi/README",
    "texmf-dist/doc/xelatex/bidi-atbegshi/bidi-atbegshi-doc.pdf",
    "texmf-dist/doc/xelatex/bidi-atbegshi/test-LTR.pdf",
    "texmf-dist/doc/xelatex/bidi-atbegshi/test-RTL.pdf",
    "texmf-dist/doc/xelatex/bidi-atbegshi/test-foreground-LTR.pdf",
    "texmf-dist/doc/xelatex/bidi-atbegshi/test-foreground-RTL.pdf",
  },
  ["combinedgraphics"] = {
    "texmf-dist/doc/latex/combinedgraphics/COPYING",
    "texmf-dist/doc/latex/combinedgraphics/Makefile",
    "texmf-dist/doc/latex/combinedgraphics/README",
    "texmf-dist/doc/latex/combinedgraphics/combinedgraphics.pdf",
    "texmf-dist/doc/latex/combinedgraphics/test/Makefile",
    "texmf-dist/doc/latex/combinedgraphics/test/combinedgraphics_test.pdf",
    "texmf-dist/doc/latex/combinedgraphics/test/gnuplot42.pdf",
    "texmf-dist/doc/latex/combinedgraphics/test/xfig325.pdf",
  },
  ["luatex85"] = {
    "texmf-dist/doc/generic/luatex85/README.md",
    "texmf-dist/doc/generic/luatex85/luatex85.pdf",
  },
  ["natded"] = {
    "texmf-dist/doc/latex/natded/README.md",
    "texmf-dist/doc/latex/natded/extended_doc.pdf",
    "texmf-dist/doc/latex/natded/natded.pdf",
  },
  ["colordoc"] = {
    "texmf-dist/doc/latex/colordoc/README",
    "texmf-dist/doc/latex/colordoc/colordoc.pdf",
  },
  ["pst-magneticfield"] = {
    "texmf-dist/doc/generic/pst-magneticfield/Changes",
    "texmf-dist/doc/generic/pst-magneticfield/README",
    "texmf-dist/doc/generic/pst-magneticfield/pst-magneticfield-docDE.pdf",
    "texmf-dist/doc/generic/pst-magneticfield/pst-magneticfield-docEN.pdf",
    "texmf-dist/doc/generic/pst-magneticfield/pst-magneticfield-docFR.pdf",
  },
  ["chivo"] = {
    "texmf-dist/doc/fonts/chivo/Chivo.pdf",
    "texmf-dist/doc/fonts/chivo/OFL.txt",
    "texmf-dist/doc/fonts/chivo/README.md",
  },
  ["ednotes"] = {
    "texmf-dist/doc/latex/ednotes/CHANGES.txt",
    "texmf-dist/doc/latex/ednotes/CHANGING.txt",
    "texmf-dist/doc/latex/ednotes/README",
    "texmf-dist/doc/latex/ednotes/README.TEXLIVE",
    "texmf-dist/doc/latex/ednotes/README4U.txt",
    "texmf-dist/doc/latex/ednotes/READMORE.txt",
    "texmf-dist/doc/latex/ednotes/ednotes.pdf",
    "texmf-dist/doc/latex/ednotes/ednotugb.pdf",
    "texmf-dist/doc/latex/ednotes/visible.txt",
  },
  ["eskdx"] = {
    "texmf-dist/doc/latex/eskdx/ChangeLog",
    "texmf-dist/doc/latex/eskdx/Makefile",
    "texmf-dist/doc/latex/eskdx/NEWS",
    "texmf-dist/doc/latex/eskdx/README",
    "texmf-dist/doc/latex/eskdx/README.in",
    "texmf-dist/doc/latex/eskdx/manifest.txt",
    "texmf-dist/doc/latex/eskdx/manual/Makefile",
    "texmf-dist/doc/latex/eskdx/manual/eskdx.pdf",
    "texmf-dist/doc/latex/eskdx/manual/latex2html-init",
    "texmf-dist/doc/latex/eskdx/source/Makefile",
    "texmf-dist/doc/latex/eskdx/test/Makefile",
  },
  ["idxlayout"] = {
    "texmf-dist/doc/latex/idxlayout/README",
    "texmf-dist/doc/latex/idxlayout/idxlayout.pdf",
  },
  ["bpolynomial"] = {
    "texmf-dist/doc/metapost/bpolynomial/CHANGES",
    "texmf-dist/doc/metapost/bpolynomial/README",
    "texmf-dist/doc/metapost/bpolynomial/TODO",
    "texmf-dist/doc/metapost/bpolynomial/bpolynomial.pdf",
  },
  ["dashundergaps"] = {
    "texmf-dist/doc/latex/dashundergaps/README.md",
    "texmf-dist/doc/latex/dashundergaps/dashundergaps-code.pdf",
    "texmf-dist/doc/latex/dashundergaps/dashundergaps.pdf",
  },
  ["context-vim"] = {
    "texmf-dist/doc/context/third/vim/VERSION",
    "texmf-dist/doc/context/third/vim/vim.txt",
  },
  ["ctex-faq"] = {
    "texmf-dist/doc/latex/ctex-faq/README",
    "texmf-dist/doc/latex/ctex-faq/ctex-faq.pdf",
  },
  ["lengthconvert"] = {
    "texmf-dist/doc/latex/lengthconvert/lengthconvert.pdf",
  },
  ["context-letter"] = {
    "texmf-dist/doc/context/third/letter/README",
    "texmf-dist/doc/context/third/letter/VERSION",
  },
  ["bezos"] = {
    "texmf-dist/doc/latex/bezos/README",
    "texmf-dist/doc/latex/bezos/accents.pdf",
    "texmf-dist/doc/latex/bezos/babeltools.pdf",
    "texmf-dist/doc/latex/bezos/bezos.pdf",
    "texmf-dist/doc/latex/bezos/esindex.pdf",
    "texmf-dist/doc/latex/bezos/soulpos.pdf",
    "texmf-dist/doc/latex/bezos/tensind.pdf",
  },
  ["dvips"] = {
    "texmf-dist/doc/dvips/dvips.html",
    "texmf-dist/doc/dvips/dvips.pdf",
    "texmf-dist/doc/man/man1/afm2tfm.man1.pdf",
    "texmf-dist/doc/man/man1/dvips.man1.pdf",
  },
  ["datetime2-icelandic"] = {
    "texmf-dist/doc/latex/datetime2-icelandic/README",
    "texmf-dist/doc/latex/datetime2-icelandic/datetime2-icelandic.pdf",
  },
  ["inversepath"] = {
    "texmf-dist/doc/latex/inversepath/README",
    "texmf-dist/doc/latex/inversepath/inversepath.pdf",
  },
  ["elvish"] = {
    "texmf-dist/doc/fonts/elvish/README",
  },
  ["multiaudience"] = {
    "texmf-dist/doc/latex/multiaudience/Makefile",
    "texmf-dist/doc/latex/multiaudience/README",
    "texmf-dist/doc/latex/multiaudience/multiaudience.pdf",
    "texmf-dist/doc/latex/multiaudience/sample-admins.pdf",
    "texmf-dist/doc/latex/multiaudience/sample-devs.pdf",
    "texmf-dist/doc/latex/multiaudience/sample-execs.pdf",
  },
  ["lshort-czech"] = {
    "texmf-dist/doc/latex/lshort-czech/CHANGES",
    "texmf-dist/doc/latex/lshort-czech/MANIFEST",
    "texmf-dist/doc/latex/lshort-czech/Makefile",
    "texmf-dist/doc/latex/lshort-czech/README",
    "texmf-dist/doc/latex/lshort-czech/lshort-cs.pdf",
  },
  ["oldstandard"] = {
    "texmf-dist/doc/fonts/oldstandard/FONTLOG.txt",
    "texmf-dist/doc/fonts/oldstandard/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/oldstandard/OFL.txt",
    "texmf-dist/doc/fonts/oldstandard/README",
    "texmf-dist/doc/fonts/oldstandard/oldstand-manual.pdf",
    "texmf-dist/doc/fonts/oldstandard/oldstandard-samples.pdf",
  },
  ["tugboat-plain"] = {
    "texmf-dist/doc/plain/tugboat-plain/README",
    "texmf-dist/doc/plain/tugboat-plain/tubguide.pdf",
  },
  ["cals"] = {
    "texmf-dist/doc/latex/cals/README",
    "texmf-dist/doc/latex/cals/cals.pdf",
    "texmf-dist/doc/latex/cals/examples/bididemo.pdf",
    "texmf-dist/doc/latex/cals/examples/demo.pdf",
    "texmf-dist/doc/latex/cals/notes.pdf",
    "texmf-dist/doc/latex/cals/tb101parashchenko.pdf",
    "texmf-dist/doc/latex/cals/test/README",
    "texmf-dist/doc/latex/cals/test/template.txt",
  },
  ["pst-infixplot"] = {
    "texmf-dist/doc/generic/pst-infixplot/README",
    "texmf-dist/doc/generic/pst-infixplot/pst-infixplot-doc.ps",
    "texmf-dist/doc/generic/pst-infixplot/pst-infixplot.pdf",
  },
  ["biblatex-bookinother"] = {
    "texmf-dist/doc/latex/biblatex-bookinother/README",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/biblatex-bookinother.pdf",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/example-bookinarticle.pdf",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/example-bookincollection.pdf",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/example-bookininarticle.pdf",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/example-bookininbook.pdf",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/example-bookinincollection.pdf",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/example-bookininproceedings.pdf",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/example-bookinjournal.pdf",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/example-bookinproceedings.pdf",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/example-bookinthesis.pdf",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/latexmkrc",
    "texmf-dist/doc/latex/biblatex-bookinother/documentation/makefile",
    "texmf-dist/doc/latex/biblatex-bookinother/makefile",
  },
  ["context-cmttbf"] = {
    "texmf-dist/doc/context/third/cmttbf/VERSION",
    "texmf-dist/doc/context/third/cmttbf/cmttbf-demo.pdf",
    "texmf-dist/doc/context/third/cmttbf/cmttbf-doc.pdf",
  },
  ["pdf14"] = {
    "texmf-dist/doc/latex/pdf14/README",
    "texmf-dist/doc/latex/pdf14/pdf14.pdf",
  },
  ["continue"] = {
    "texmf-dist/doc/latex/continue/README",
    "texmf-dist/doc/latex/continue/continue.pdf",
    "texmf-dist/doc/latex/continue/trycontinue.pdf",
  },
  ["bxpdfver"] = {
    "texmf-dist/doc/latex/bxpdfver/LICENSE",
    "texmf-dist/doc/latex/bxpdfver/README.md",
  },
  ["dotarrow"] = {
    "texmf-dist/doc/latex/dotarrow/DotArrow.pdf",
    "texmf-dist/doc/latex/dotarrow/README",
  },
  ["uassign"] = {
    "texmf-dist/doc/latex/uassign/README.md",
    "texmf-dist/doc/latex/uassign/uassign.pdf",
  },
  ["biblatex-enc"] = {
    "texmf-dist/doc/latex/biblatex-enc/README",
  },
  ["ctan-o-mat"] = {
    "texmf-dist/doc/man/man1/ctan-o-mat.man1.pdf",
    "texmf-dist/doc/support/ctan-o-mat/LICENSE",
    "texmf-dist/doc/support/ctan-o-mat/README.md",
    "texmf-dist/doc/support/ctan-o-mat/makefile",
  },
  ["fmp"] = {
    "texmf-dist/doc/latex/fmp/README",
    "texmf-dist/doc/latex/fmp/fmp.pdf",
  },
  ["gothic"] = {
    "texmf-dist/doc/fonts/gothic/README.sueterlin",
    "texmf-dist/doc/fonts/gothic/README.yinit",
    "texmf-dist/doc/fonts/gothic/suet.pdf",
    "texmf-dist/doc/fonts/gothic/yinit.pdf",
  },
  ["ltxdockit"] = {
    "texmf-dist/doc/latex/ltxdockit/README",
  },
  ["refman"] = {
    "texmf-dist/doc/latex/refman/00Contents",
    "texmf-dist/doc/latex/refman/layout_d.pdf",
    "texmf-dist/doc/latex/refman/layout_e.pdf",
    "texmf-dist/doc/latex/refman/refman.pdf",
  },
  ["export"] = {
    "texmf-dist/doc/latex/export/00readme",
    "texmf-dist/doc/latex/export/export.pdf",
  },
  ["fontawesome"] = {
    "texmf-dist/doc/fonts/fontawesome/README.md",
    "texmf-dist/doc/fonts/fontawesome/fontawesome.pdf",
  },
  ["dvi2tty"] = {
    "texmf-dist/doc/man/man1/disdvi.man1.pdf",
    "texmf-dist/doc/man/man1/dvi2tty.man1.pdf",
  },
  ["lshort-slovenian"] = {
    "texmf-dist/doc/latex/lshort-slovenian/README",
    "texmf-dist/doc/latex/lshort-slovenian/lshort-slovenian.pdf",
  },
  ["lshort-slovak"] = {
    "texmf-dist/doc/latex/lshort-slovak/slshorte.pdf",
  },
  ["ecclesiastic"] = {
    "texmf-dist/doc/latex/ecclesiastic/README",
    "texmf-dist/doc/latex/ecclesiastic/ecclesiastic.pdf",
    "texmf-dist/doc/latex/ecclesiastic/manifest.txt",
  },
  ["basicarith"] = {
    "texmf-dist/doc/latex/basicarith/CHANGES",
    "texmf-dist/doc/latex/basicarith/README",
    "texmf-dist/doc/latex/basicarith/basicarith.pdf",
    "texmf-dist/doc/latex/basicarith/lppl.txt",
  },
  ["ctanify"] = {
    "texmf-dist/doc/latex/ctanify/README",
    "texmf-dist/doc/latex/ctanify/ctanify.pdf",
    "texmf-dist/doc/man/man1/ctanify.man1.pdf",
  },
  ["translation-tabbing-fr"] = {
    "texmf-dist/doc/latex/translation-tabbing-fr/f-Tabbing.pdf",
  },
  ["elements"] = {
    "texmf-dist/doc/latex/elements/README",
    "texmf-dist/doc/latex/elements/elements_en.pdf",
  },
  ["lshort-mongol"] = {
    "texmf-dist/doc/latex/lshort-mongol/00README",
    "texmf-dist/doc/latex/lshort-mongol/CHANGES",
    "texmf-dist/doc/latex/lshort-mongol/MANIFEST",
    "texmf-dist/doc/latex/lshort-mongol/Makefile",
    "texmf-dist/doc/latex/lshort-mongol/README",
    "texmf-dist/doc/latex/lshort-mongol/TODO",
    "texmf-dist/doc/latex/lshort-mongol/TRANSLATIONS",
    "texmf-dist/doc/latex/lshort-mongol/lshort-mn.pdf",
  },
  ["lshort-korean"] = {
    "texmf-dist/doc/latex/lshort-korean/README.ko",
    "texmf-dist/doc/latex/lshort-korean/lshort-kr.pdf",
  },
  ["lpic"] = {
    "texmf-dist/doc/latex/lpic/README",
    "texmf-dist/doc/latex/lpic/instructions-differential.pdf",
    "texmf-dist/doc/latex/lpic/instructions.pdf",
  },
  ["datetime2-lsorbian"] = {
    "texmf-dist/doc/latex/datetime2-lsorbian/README",
    "texmf-dist/doc/latex/datetime2-lsorbian/datetime2-lsorbian.pdf",
  },
  ["churchslavonic"] = {
    "texmf-dist/doc/latex/churchslavonic/LICENSE",
    "texmf-dist/doc/latex/churchslavonic/README",
    "texmf-dist/doc/latex/churchslavonic/churchslavonic-en.pdf",
    "texmf-dist/doc/latex/churchslavonic/churchslavonic-ru.pdf",
  },
  ["jumplines"] = {
    "texmf-dist/doc/latex/jumplines/README",
    "texmf-dist/doc/latex/jumplines/jumplines_doc.pdf",
    "texmf-dist/doc/latex/jumplines/jumplines_example.pdf",
  },
  ["shdoc"] = {
    "texmf-dist/doc/latex/shdoc/README",
    "texmf-dist/doc/latex/shdoc/README.txt",
    "texmf-dist/doc/latex/shdoc/shdoc.pdf",
  },
  ["grant"] = {
    "texmf-dist/doc/latex/grant/LICENSE",
    "texmf-dist/doc/latex/grant/README.md",
    "texmf-dist/doc/latex/grant/VERSION",
    "texmf-dist/doc/latex/grant/grant.pdf",
  },
  ["articleingud"] = {
    "texmf-dist/doc/latex/articleingud/README",
    "texmf-dist/doc/latex/articleingud/articleingud.pdf",
  },
  ["filecontents"] = {
    "texmf-dist/doc/latex/filecontents/README",
    "texmf-dist/doc/latex/filecontents/filecontents.pdf",
  },
  ["mltex"] = {
    "texmf-dist/doc/latex/mltex/README",
    "texmf-dist/doc/latex/mltex/mltex.txt",
    "texmf-dist/doc/latex/mltex/testmlft.dvi",
    "texmf-dist/doc/latex/mltex/testmlsw.dvi",
  },
  ["bibleref"] = {
    "texmf-dist/doc/latex/bibleref/CHANGES",
    "texmf-dist/doc/latex/bibleref/README",
    "texmf-dist/doc/latex/bibleref/bibleref.pdf",
    "texmf-dist/doc/latex/bibleref/makefile",
    "texmf-dist/doc/latex/bibleref/samples/sample-categories.pdf",
    "texmf-dist/doc/latex/bibleref/samples/sample-indextools.pdf",
    "texmf-dist/doc/latex/bibleref/samples/sample-xidx.pdf",
    "texmf-dist/doc/latex/bibleref/samples/sample.pdf",
  },
  ["localloc"] = {
    "texmf-dist/doc/latex/localloc/localloc.pdf",
  },
  ["datetime2"] = {
    "texmf-dist/doc/latex/datetime2/CHANGES",
    "texmf-dist/doc/latex/datetime2/README",
    "texmf-dist/doc/latex/datetime2/datetime2.pdf",
    "texmf-dist/doc/latex/datetime2/samples/datetime2-sample-babel.pdf",
    "texmf-dist/doc/latex/datetime2/samples/datetime2-sample-calc.pdf",
    "texmf-dist/doc/latex/datetime2/samples/datetime2-sample-filedate.pdf",
    "texmf-dist/doc/latex/datetime2/samples/datetime2-sample-hyperref.pdf",
    "texmf-dist/doc/latex/datetime2/samples/datetime2-sample-journal.pdf",
    "texmf-dist/doc/latex/datetime2/samples/datetime2-sample-styles.pdf",
    "texmf-dist/doc/latex/datetime2/samples/datetime2-sample-xe.pdf",
  },
  ["synctex"] = {
    "texmf-dist/doc/man/man1/synctex.man1.pdf",
    "texmf-dist/doc/man/man5/synctex.man5.pdf",
  },
  ["babelbib"] = {
    "texmf-dist/doc/bibtex/babelbib/ChangeLog",
    "texmf-dist/doc/bibtex/babelbib/Makefile",
    "texmf-dist/doc/bibtex/babelbib/README",
    "texmf-dist/doc/bibtex/babelbib/babelbib.pdf",
    "texmf-dist/doc/bibtex/babelbib/tugboat-babelbib.pdf",
  },
  ["llncsconf"] = {
    "texmf-dist/doc/latex/llncsconf/LICENSE",
    "texmf-dist/doc/latex/llncsconf/README.md",
    "texmf-dist/doc/latex/llncsconf/example.pdf",
  },
  ["invoice"] = {
    "texmf-dist/doc/latex/invoice/00README.tex",
    "texmf-dist/doc/latex/invoice/copying",
    "texmf-dist/doc/latex/invoice/history",
    "texmf-dist/doc/latex/invoice/install",
    "texmf-dist/doc/latex/invoice/invoice.pdf",
  },
  ["citeall"] = {
    "texmf-dist/doc/latex/citeall/README.md",
    "texmf-dist/doc/latex/citeall/citeall.pdf",
  },
  ["lecturer"] = {
    "texmf-dist/doc/generic/lecturer/LecturerDemo-BeamerCambridgeUS.pdf",
    "texmf-dist/doc/generic/lecturer/LecturerDemo-KitschScienceFiction.pdf",
    "texmf-dist/doc/generic/lecturer/LecturerDemo-Mondrian.pdf",
    "texmf-dist/doc/generic/lecturer/LecturerDemo-SimplePresentation.pdf",
    "texmf-dist/doc/generic/lecturer/LecturerDemo-SquaresOfAs.pdf",
    "texmf-dist/doc/generic/lecturer/LecturerDemo-ThePoodleLectures.pdf",
    "texmf-dist/doc/generic/lecturer/LecturerDemo-VisualDoc.pdf",
    "texmf-dist/doc/generic/lecturer/README",
    "texmf-dist/doc/generic/lecturer/lecturer-doc.pdf",
  },
  ["lsc"] = {
    "texmf-dist/doc/latex/lsc/README",
    "texmf-dist/doc/latex/lsc/lsc.pdf",
  },
  ["glossaries-finnish"] = {
    "texmf-dist/doc/latex/glossaries-finnish/README",
    "texmf-dist/doc/latex/glossaries-finnish/glossaries-finnish.pdf",
  },
  ["tkz-euclide"] = {
    "texmf-dist/doc/latex/tkz-euclide/README",
    "texmf-dist/doc/latex/tkz-euclide/examples/Makefile",
    "texmf-dist/doc/latex/tkz-euclide/examples/info_euclide_tex.txt",
    "texmf-dist/doc/latex/tkz-euclide/readme-tkz-euclide-fr.txt",
    "texmf-dist/doc/latex/tkz-euclide/readme-tkz-euclide.txt",
    "texmf-dist/doc/latex/tkz-euclide/tkz-euclide-screen.pdf",
  },
  ["listliketab"] = {
    "texmf-dist/doc/latex/listliketab/README",
    "texmf-dist/doc/latex/listliketab/listliketab.pdf",
  },
  ["babel-finnish"] = {
    "texmf-dist/doc/generic/babel-finnish/finnish.pdf",
  },
  ["latex2e-help-texinfo-fr"] = {
    "texmf-dist/doc/latex/latex2e-help-texinfo-fr/ChangeLog",
    "texmf-dist/doc/latex/latex2e-help-texinfo-fr/Makefile",
    "texmf-dist/doc/latex/latex2e-help-texinfo-fr/NEWS-fr",
    "texmf-dist/doc/latex/latex2e-help-texinfo-fr/README",
    "texmf-dist/doc/latex/latex2e-help-texinfo-fr/latex2e-fr.html",
    "texmf-dist/doc/latex/latex2e-help-texinfo-fr/latex2e-fr.pdf",
    "texmf-dist/doc/latex/latex2e-help-texinfo-fr/latex2e-fr.txt",
  },
  ["figsize"] = {
    "texmf-dist/doc/latex/figsize/README",
    "texmf-dist/doc/latex/figsize/figsize.pdf",
  },
  ["biblatex-ext"] = {
    "texmf-dist/doc/latex/biblatex-ext/README.md",
    "texmf-dist/doc/latex/biblatex-ext/biblatex-ext.pdf",
  },
  ["listings-ext"] = {
    "texmf-dist/doc/latex/listings-ext/README",
    "texmf-dist/doc/latex/listings-ext/THIS_IS_VERSION_v67",
    "texmf-dist/doc/latex/listings-ext/listings-ext.pdf",
  },
  ["listing"] = {
    "texmf-dist/doc/latex/listing/listing.pdf",
  },
  ["dejavu"] = {
    "texmf-dist/doc/fonts/dejavu/AUTHORS",
    "texmf-dist/doc/fonts/dejavu/BUGS",
    "texmf-dist/doc/fonts/dejavu/CHANGELOG",
    "texmf-dist/doc/fonts/dejavu/LICENSE",
    "texmf-dist/doc/fonts/dejavu/NEWS",
    "texmf-dist/doc/fonts/dejavu/README-font",
    "texmf-dist/doc/fonts/dejavu/README.doc",
    "texmf-dist/doc/fonts/dejavu/dejavu-sample.pdf",
    "texmf-dist/doc/fonts/dejavu/dejavu.pdf",
    "texmf-dist/doc/fonts/dejavu/langcover.txt",
    "texmf-dist/doc/fonts/dejavu/manifest.txt",
    "texmf-dist/doc/fonts/dejavu/status.txt",
    "texmf-dist/doc/fonts/dejavu/unicover.txt",
  },
  ["environ"] = {
    "texmf-dist/doc/latex/environ/README",
    "texmf-dist/doc/latex/environ/environ.pdf",
  },
  ["msc"] = {
    "texmf-dist/doc/latex/msc/COPYRIGHT.txt",
    "texmf-dist/doc/latex/msc/README",
    "texmf-dist/doc/latex/msc/maintenance.pdf",
    "texmf-dist/doc/latex/msc/manual.pdf",
    "texmf-dist/doc/latex/msc/refman.pdf",
  },
  ["chet"] = {
    "texmf-dist/doc/latex/chet/README",
    "texmf-dist/doc/latex/chet/chetdoc.pdf",
  },
  ["ling-macros"] = {
    "texmf-dist/doc/latex/ling-macros/README.txt",
    "texmf-dist/doc/latex/ling-macros/ling-macros-doc.pdf",
  },
  ["l2picfaq"] = {
    "texmf-dist/doc/latex/l2picfaq/README",
    "texmf-dist/doc/latex/l2picfaq/ctanlion.pdf",
    "texmf-dist/doc/latex/l2picfaq/l2picfaq.pdf",
  },
  ["hypdvips"] = {
    "texmf-dist/doc/latex/hypdvips/README",
    "texmf-dist/doc/latex/hypdvips/hypdvips.pdf",
    "texmf-dist/doc/latex/hypdvips/manifest.txt",
  },
  ["graphics-def"] = {
    "texmf-dist/doc/latex/graphics-def/README.md",
  },
  ["psgo"] = {
    "texmf-dist/doc/latex/psgo/Changes",
    "texmf-dist/doc/latex/psgo/README",
    "texmf-dist/doc/latex/psgo/psgomanual.pdf",
  },
  ["bidipresentation"] = {
    "texmf-dist/doc/xelatex/bidipresentation/README",
    "texmf-dist/doc/xelatex/bidipresentation/fig-1.pdf",
    "texmf-dist/doc/xelatex/bidipresentation/sample.pdf",
  },
  ["collcell"] = {
    "texmf-dist/doc/latex/collcell/collcell.pdf",
  },
  ["babel-indonesian"] = {
    "texmf-dist/doc/generic/babel-indonesian/README",
    "texmf-dist/doc/generic/babel-indonesian/indonesian.pdf",
  },
  ["moodle"] = {
    "texmf-dist/doc/latex/moodle/README.md",
    "texmf-dist/doc/latex/moodle/moodle.pdf",
  },
  ["lilyglyphs"] = {
    "texmf-dist/doc/lualatex/lilyglyphs/INSTALL",
    "texmf-dist/doc/lualatex/lilyglyphs/README",
    "texmf-dist/doc/lualatex/lilyglyphs/README-scripts",
    "texmf-dist/doc/lualatex/lilyglyphs/VERSION",
    "texmf-dist/doc/lualatex/lilyglyphs/documentation/lilyglyphs-example-de.pdf",
    "texmf-dist/doc/lualatex/lilyglyphs/documentation/lilyglyphs-example.pdf",
    "texmf-dist/doc/lualatex/lilyglyphs/documentation/lilyglyphs.pdf",
    "texmf-dist/doc/lualatex/lilyglyphs/documentation/lilyglyphs_logo/lilyglyphs_logo.pdf",
    "texmf-dist/doc/lualatex/lilyglyphs/documentation/the-feta-font-2-18-0.html",
    "texmf-dist/doc/lualatex/lilyglyphs/license/MANIFEST",
  },
  ["librebodoni"] = {
    "texmf-dist/doc/fonts/librebodoni/OFL.txt",
    "texmf-dist/doc/fonts/librebodoni/README",
    "texmf-dist/doc/fonts/librebodoni/librebodoni-samples.pdf",
  },
  ["hustthesis"] = {
    "texmf-dist/doc/latex/hustthesis/README.md",
    "texmf-dist/doc/latex/hustthesis/fig-example.pdf",
    "texmf-dist/doc/latex/hustthesis/hustthesis-en-example.pdf",
    "texmf-dist/doc/latex/hustthesis/hustthesis-zh-example.pdf",
    "texmf-dist/doc/latex/hustthesis/hustthesis.pdf",
  },
  ["cell"] = {
    "texmf-dist/doc/latex/cell/README",
  },
  ["pstools"] = {
    "texmf-dist/doc/man/man1/bbox.man1.pdf",
    "texmf-dist/doc/man/man1/ps2eps.man1.pdf",
    "texmf-dist/doc/man/man1/ps2frag.man1.pdf",
    "texmf-dist/doc/man/man1/pslatex.man1.pdf",
  },
  ["lgreek"] = {
    "texmf-dist/doc/latex/lgreek/README",
    "texmf-dist/doc/latex/lgreek/lgreekuse.pdf",
  },
  ["bpchem"] = {
    "texmf-dist/doc/latex/bpchem/README.md",
    "texmf-dist/doc/latex/bpchem/bpchem.pdf",
  },
  ["lewis"] = {
    "texmf-dist/doc/latex/lewis/README",
    "texmf-dist/doc/latex/lewis/lewis.pdf",
  },
  ["icsv"] = {
    "texmf-dist/doc/latex/icsv/README",
    "texmf-dist/doc/latex/icsv/icsv.pdf",
  },
  ["ledmac"] = {
    "texmf-dist/doc/latex/ledmac/Makefile",
    "texmf-dist/doc/latex/ledmac/README",
    "texmf-dist/doc/latex/ledmac/ledarab.pdf",
    "texmf-dist/doc/latex/ledmac/ledmac.pdf",
    "texmf-dist/doc/latex/ledmac/ledpar.pdf",
  },
  ["dvicopy"] = {
    "texmf-dist/doc/man/man1/dvicopy.man1.pdf",
  },
  ["nrc"] = {
    "texmf-dist/doc/latex/nrc/00-2013-feb-authors.txt",
    "texmf-dist/doc/latex/nrc/README",
    "texmf-dist/doc/latex/nrc/userguide.pdf",
  },
  ["gu"] = {
    "texmf-dist/doc/latex/gu/README",
    "texmf-dist/doc/latex/gu/gudoc.pdf",
  },
  ["import"] = {
    "texmf-dist/doc/latex/import/import.pdf",
  },
  ["tocvsec2"] = {
    "texmf-dist/doc/latex/tocvsec2/README",
    "texmf-dist/doc/latex/tocvsec2/tocvsec2.pdf",
  },
  ["eulervm"] = {
    "texmf-dist/doc/latex/eulervm/README.eulervm",
    "texmf-dist/doc/latex/eulervm/eulervm.pdf",
    "texmf-dist/doc/latex/eulervm/manifest.txt",
  },
  ["callouts"] = {
    "texmf-dist/doc/latex/callouts/README.md",
    "texmf-dist/doc/latex/callouts/callouts.pdf",
  },
  ["bguq"] = {
    "texmf-dist/doc/fonts/bguq/INSTALL.txt",
    "texmf-dist/doc/fonts/bguq/Makefile",
    "texmf-dist/doc/fonts/bguq/README",
    "texmf-dist/doc/fonts/bguq/bguq-doc.pdf",
  },
  ["lazylist"] = {
    "texmf-dist/doc/latex/lazylist/lazylist.pdf",
  },
  ["convbkmk"] = {
    "texmf-dist/doc/support/convbkmk/README",
  },
  ["tucv"] = {
    "texmf-dist/doc/latex/tucv/README",
    "texmf-dist/doc/latex/tucv/tucv.pdf",
    "texmf-dist/doc/latex/tucv/tucv_ex.pdf",
  },
  ["biblatex-lni"] = {
    "texmf-dist/doc/latex/biblatex-lni/LICENSE",
    "texmf-dist/doc/latex/biblatex-lni/README.md",
  },
  ["vak"] = {
    "texmf-dist/doc/bibtex/vak/README",
  },
  ["pst-light3d"] = {
    "texmf-dist/doc/generic/pst-light3d/Changes",
    "texmf-dist/doc/generic/pst-light3d/README",
    "texmf-dist/doc/generic/pst-light3d/pst-light3d-doc.pdf",
  },
  ["arydshln"] = {
    "texmf-dist/doc/latex/arydshln/README",
    "texmf-dist/doc/latex/arydshln/arydshln-man.pdf",
    "texmf-dist/doc/latex/arydshln/arydshln.pdf",
  },
  ["currvita"] = {
    "texmf-dist/doc/latex/currvita/README",
    "texmf-dist/doc/latex/currvita/currvita.pdf",
  },
  ["ecltree"] = {
    "texmf-dist/doc/latex/ecltree/ecltreesample.pdf",
  },
  ["cjkutils"] = {
    "texmf-dist/doc/man/man1/bg5conv.man1.pdf",
    "texmf-dist/doc/man/man1/cef5conv.man1.pdf",
    "texmf-dist/doc/man/man1/cefconv.man1.pdf",
    "texmf-dist/doc/man/man1/cefsconv.man1.pdf",
    "texmf-dist/doc/man/man1/extconv.man1.pdf",
    "texmf-dist/doc/man/man1/hbf2gf.man1.pdf",
    "texmf-dist/doc/man/man1/sjisconv.man1.pdf",
  },
  ["h2020proposal"] = {
    "texmf-dist/doc/latex/h2020proposal/README.txt",
    "texmf-dist/doc/latex/h2020proposal/gpl.txt",
    "texmf-dist/doc/latex/h2020proposal/manual/h2020proposal.pdf",
    "texmf-dist/doc/latex/h2020proposal/template-fet/template-fet.pdf",
    "texmf-dist/doc/latex/h2020proposal/template-ict/implementation.pdf",
    "texmf-dist/doc/latex/h2020proposal/template-ict/template-ict.pdf",
  },
  ["maths-symbols"] = {
    "texmf-dist/doc/latex/maths-symbols/README",
    "texmf-dist/doc/latex/maths-symbols/maths-symbols.pdf",
    "texmf-dist/doc/latex/maths-symbols/scriptfonts.pdf",
  },
  ["childdoc"] = {
    "texmf-dist/doc/latex/childdoc/README.txt",
    "texmf-dist/doc/latex/childdoc/childdoc.pdf",
  },
  ["dox"] = {
    "texmf-dist/doc/latex/dox/NEWS",
    "texmf-dist/doc/latex/dox/README.md",
    "texmf-dist/doc/latex/dox/THANKS",
    "texmf-dist/doc/latex/dox/dox.pdf",
  },
  ["chemarrow"] = {
    "texmf-dist/doc/fonts/chemarrow/Liesmich.txt",
    "texmf-dist/doc/fonts/chemarrow/README",
    "texmf-dist/doc/fonts/chemarrow/chemarrow-de.pdf",
    "texmf-dist/doc/fonts/chemarrow/chemarrow.pdf",
  },
  ["poster-mac"] = {
    "texmf-dist/doc/generic/poster-mac/Changes",
    "texmf-dist/doc/generic/poster-mac/Makefile",
    "texmf-dist/doc/generic/poster-mac/README",
    "texmf-dist/doc/generic/poster-mac/poster-doc.pdf",
    "texmf-dist/doc/generic/poster-mac/poster1.pdf",
    "texmf-dist/doc/generic/poster-mac/poster2.pdf",
  },
  ["chemsec"] = {
    "texmf-dist/doc/latex/chemsec/README.md",
    "texmf-dist/doc/latex/chemsec/chemsec.pdf",
  },
  ["ejpecp"] = {
    "texmf-dist/doc/latex/ejpecp/LPPL",
    "texmf-dist/doc/latex/ejpecp/README",
    "texmf-dist/doc/latex/ejpecp/ejpecp.pdf",
    "texmf-dist/doc/latex/ejpecp/sample.pdf",
  },
  ["cabin"] = {
    "texmf-dist/doc/fonts/cabin/OFL.txt",
    "texmf-dist/doc/fonts/cabin/README",
    "texmf-dist/doc/fonts/cabin/samples-condensed.pdf",
    "texmf-dist/doc/fonts/cabin/samples.pdf",
  },
  ["datetime2-dutch"] = {
    "texmf-dist/doc/latex/datetime2-dutch/README",
    "texmf-dist/doc/latex/datetime2-dutch/datetime2-dutch.pdf",
  },
  ["sageep"] = {
    "texmf-dist/doc/latex/sageep/README",
    "texmf-dist/doc/latex/sageep/sageep.pdf",
    "texmf-dist/doc/latex/sageep/sample.pdf",
  },
  ["boldtensors"] = {
    "texmf-dist/doc/latex/boldtensors/boldtensors.pdf",
  },
  ["chextras"] = {
    "texmf-dist/doc/latex/chextras/README",
    "texmf-dist/doc/latex/chextras/chextras.pdf",
  },
  ["babel-italian"] = {
    "texmf-dist/doc/generic/babel-italian/italian.pdf",
  },
  ["gincltex"] = {
    "texmf-dist/doc/latex/gincltex/README",
    "texmf-dist/doc/latex/gincltex/gincltex.pdf",
  },
  ["footnpag"] = {
    "texmf-dist/doc/latex/footnpag/CATALOG",
    "texmf-dist/doc/latex/footnpag/History",
    "texmf-dist/doc/latex/footnpag/INSTALL",
    "texmf-dist/doc/latex/footnpag/Imakefile",
    "texmf-dist/doc/latex/footnpag/License",
    "texmf-dist/doc/latex/footnpag/MANIFEST",
    "texmf-dist/doc/latex/footnpag/README",
    "texmf-dist/doc/latex/footnpag/TODO",
    "texmf-dist/doc/latex/footnpag/footnpag-user.pdf",
    "texmf-dist/doc/latex/footnpag/test/Imakefile",
  },
  ["cmpj"] = {
    "texmf-dist/doc/latex/cmpj/README",
    "texmf-dist/doc/latex/cmpj/eps_demo.pdf",
    "texmf-dist/doc/latex/cmpj/icmphome.pdf",
    "texmf-dist/doc/latex/cmpj/template.pdf",
  },
  ["arara"] = {
    "texmf-dist/doc/support/arara/README.md",
    "texmf-dist/doc/support/arara/arara-manual.pdf",
    "texmf-dist/doc/support/arara/figures/dropdown1.pdf",
    "texmf-dist/doc/support/arara/figures/dropdown2.pdf",
    "texmf-dist/doc/support/arara/figures/inputbox1.pdf",
    "texmf-dist/doc/support/arara/figures/inputbox2.pdf",
    "texmf-dist/doc/support/arara/figures/messagebox1.pdf",
    "texmf-dist/doc/support/arara/figures/messagebox2.pdf",
    "texmf-dist/doc/support/arara/figures/optionbox1.pdf",
    "texmf-dist/doc/support/arara/figures/optionbox2.pdf",
    "texmf-dist/doc/support/arara/logos/bird.pdf",
    "texmf-dist/doc/support/arara/logos/logo1.pdf",
    "texmf-dist/doc/support/arara/logos/logo2.pdf",
  },
  ["etdipa"] = {
    "texmf-dist/doc/latex/etdipa/Data/folder_description.txt",
    "texmf-dist/doc/latex/etdipa/Help/etdipa_doc_DE.pdf",
    "texmf-dist/doc/latex/etdipa/Help/etdipa_doc_EN.pdf",
    "texmf-dist/doc/latex/etdipa/Help/files.txt",
    "texmf-dist/doc/latex/etdipa/Images/Ordnerstruktur.pdf",
    "texmf-dist/doc/latex/etdipa/Images/bild.pdf",
    "texmf-dist/doc/latex/etdipa/Images/diplomand_erklaerung.pdf",
    "texmf-dist/doc/latex/etdipa/Images/subordner.pdf",
    "texmf-dist/doc/latex/etdipa/Images/titlepic.pdf",
    "texmf-dist/doc/latex/etdipa/README",
  },
  ["classics"] = {
    "texmf-dist/doc/latex/classics/README",
    "texmf-dist/doc/latex/classics/classics.pdf",
  },
  ["oinuit"] = {
    "texmf-dist/doc/fonts/oinuit/README.1ST",
    "texmf-dist/doc/fonts/oinuit/README.TEXLIVE",
  },
  ["mathpunctspace"] = {
    "texmf-dist/doc/latex/mathpunctspace/README.md",
    "texmf-dist/doc/latex/mathpunctspace/comma0mu-semicolonnat-colonnat.pdf",
    "texmf-dist/doc/latex/mathpunctspace/comma10mu-semicolon20mu-colon30mu.pdf",
    "texmf-dist/doc/latex/mathpunctspace/comma5pt-semicolon5pt-colon5pt.pdf",
    "texmf-dist/doc/latex/mathpunctspace/latexorg.pdf",
    "texmf-dist/doc/latex/mathpunctspace/mathpunctspace.pdf",
  },
  ["latexdemo"] = {
    "texmf-dist/doc/latex/latexdemo/README",
    "texmf-dist/doc/latex/latexdemo/latexdemo.pdf",
  },
  ["download"] = {
    "texmf-dist/doc/latex/download/README",
    "texmf-dist/doc/latex/download/download.pdf",
  },
  ["latexcourse-rug"] = {
    "texmf-dist/doc/latex/latexcourse-rug/README",
    "texmf-dist/doc/latex/latexcourse-rug/latexcourse-rug.pdf",
  },
  ["babel-croatian"] = {
    "texmf-dist/doc/generic/babel-croatian/croatian.pdf",
  },
  ["cooltooltips"] = {
    "texmf-dist/doc/latex/cooltooltips/README",
    "texmf-dist/doc/latex/cooltooltips/cooltooltips.pdf",
  },
  ["smartdiagram"] = {
    "texmf-dist/doc/latex/smartdiagram/README",
    "texmf-dist/doc/latex/smartdiagram/smartdiagram.pdf",
  },
  ["clefval"] = {
    "texmf-dist/doc/latex/clefval/Changements",
    "texmf-dist/doc/latex/clefval/Changes",
    "texmf-dist/doc/latex/clefval/LISEZMOI",
    "texmf-dist/doc/latex/clefval/README",
    "texmf-dist/doc/latex/clefval/clefval.pdf",
    "texmf-dist/doc/latex/clefval/example.pdf",
    "texmf-dist/doc/latex/clefval/exemple.pdf",
  },
  ["bangtex"] = {
    "texmf-dist/doc/latex/bangtex/README.bangtex",
  },
  ["fntproof"] = {
    "texmf-dist/doc/generic/fntproof/README",
    "texmf-dist/doc/generic/fntproof/fntproof-doc.pdf",
  },
  ["interpreter"] = {
    "texmf-dist/doc/luatex/interpreter/README",
    "texmf-dist/doc/luatex/interpreter/interpreter-doc.pdf",
    "texmf-dist/doc/luatex/interpreter/interpreter-doc.txt",
  },
  ["limecv"] = {
    "texmf-dist/doc/latex/limecv/LICENSE",
    "texmf-dist/doc/latex/limecv/README.md",
    "texmf-dist/doc/latex/limecv/examples/limecv-icon.pdf",
    "texmf-dist/doc/latex/limecv/examples/mwe-latex.pdf",
    "texmf-dist/doc/latex/limecv/examples/mwe-lualatex.pdf",
    "texmf-dist/doc/latex/limecv/examples/mwe-xelatex.pdf",
    "texmf-dist/doc/latex/limecv/limecv.pdf",
  },
  ["lm-math"] = {
    "texmf-dist/doc/fonts/lm-math/GUST-FONT-LICENSE.txt",
    "texmf-dist/doc/fonts/lm-math/INSTALL.txt",
    "texmf-dist/doc/fonts/lm-math/MANIFEST-Latin-Modern-Math.txt",
    "texmf-dist/doc/fonts/lm-math/README-Latin-Modern-Math.txt",
    "texmf-dist/doc/fonts/lm-math/test-context-latinmodern_math.pdf",
    "texmf-dist/doc/fonts/lm-math/test-lualatex-latinmodern_math.pdf",
    "texmf-dist/doc/fonts/lm-math/test-word-latinmodern_math.pdf",
    "texmf-dist/doc/fonts/lm-math/test-xelatex-latinmodern_math.pdf",
  },
  ["yagusylo"] = {
    "texmf-dist/doc/latex/yagusylo/LISEZMOI",
    "texmf-dist/doc/latex/yagusylo/README",
    "texmf-dist/doc/latex/yagusylo/yagusylo-en.pdf",
    "texmf-dist/doc/latex/yagusylo/yagusylo-fr.pdf",
    "texmf-dist/doc/latex/yagusylo/yagusylo.pdf",
  },
  ["epspdfconversion"] = {
    "texmf-dist/doc/latex/epspdfconversion/README",
    "texmf-dist/doc/latex/epspdfconversion/epspdfconversion.pdf",
  },
  ["emarks"] = {
    "texmf-dist/doc/latex/emarks/README",
    "texmf-dist/doc/latex/emarks/emarks.pdf",
  },
  ["pgfopts"] = {
    "texmf-dist/doc/latex/pgfopts/LICENSE",
    "texmf-dist/doc/latex/pgfopts/README",
    "texmf-dist/doc/latex/pgfopts/pgfopts.pdf",
  },
  ["beamertheme-detlevcm"] = {
    "texmf-dist/doc/latex/beamertheme-detlevcm/README.txt",
    "texmf-dist/doc/latex/beamertheme-detlevcm/beamertheme-detlevcm.pdf",
  },
  ["fnbreak"] = {
    "texmf-dist/doc/latex/fnbreak/ChangeLog",
    "texmf-dist/doc/latex/fnbreak/Makefile",
    "texmf-dist/doc/latex/fnbreak/README",
    "texmf-dist/doc/latex/fnbreak/fnbreak.pdf",
  },
  ["feupphdteses"] = {
    "texmf-dist/doc/latex/feupphdteses/Figures/uporto-feup.pdf",
    "texmf-dist/doc/latex/feupphdteses/README",
    "texmf-dist/doc/latex/feupphdteses/Template_EN.pdf",
  },
  ["intro-scientific"] = {
    "texmf-dist/doc/latex/intro-scientific/Makefile",
    "texmf-dist/doc/latex/intro-scientific/README",
    "texmf-dist/doc/latex/intro-scientific/earth-moon.pdf",
    "texmf-dist/doc/latex/intro-scientific/scidoc.pdf",
  },
  ["latex-bin"] = {
    "texmf-dist/doc/man/man1/dvilualatex.man1.pdf",
    "texmf-dist/doc/man/man1/latex.man1.pdf",
    "texmf-dist/doc/man/man1/lualatex.man1.pdf",
    "texmf-dist/doc/man/man1/pdflatex.man1.pdf",
  },
  ["metre"] = {
    "texmf-dist/doc/latex/metre/README",
    "texmf-dist/doc/latex/metre/demo.pdf",
    "texmf-dist/doc/latex/metre/metre.pdf",
  },
  ["bagpipe"] = {
    "texmf-dist/doc/generic/bagpipe/BlackDonald.pdf",
    "texmf-dist/doc/generic/bagpipe/Bonnets.pdf",
    "texmf-dist/doc/generic/bagpipe/Green.pdf",
    "texmf-dist/doc/generic/bagpipe/GreenTwo.pdf",
    "texmf-dist/doc/generic/bagpipe/README",
    "texmf-dist/doc/generic/bagpipe/Washer.pdf",
    "texmf-dist/doc/generic/bagpipe/bagdoc.pdf",
    "texmf-dist/doc/generic/bagpipe/quickref.pdf",
  },
  ["lni"] = {
    "texmf-dist/doc/latex/lni/README.md",
    "texmf-dist/doc/latex/lni/lni.pdf",
  },
  ["jnuexam"] = {
    "texmf-dist/doc/latex/jnuexam/README",
    "texmf-dist/doc/latex/jnuexam/exam-a-a3input.pdf",
    "texmf-dist/doc/latex/jnuexam/exam-a-a3split.pdf",
    "texmf-dist/doc/latex/jnuexam/exam-a-empty.pdf",
    "texmf-dist/doc/latex/jnuexam/exam-a.pdf",
    "texmf-dist/doc/latex/jnuexam/exam-b-a3input.pdf",
    "texmf-dist/doc/latex/jnuexam/exam-b-a3split.pdf",
    "texmf-dist/doc/latex/jnuexam/exam-b-empty.pdf",
    "texmf-dist/doc/latex/jnuexam/exam-b.pdf",
    "texmf-dist/doc/latex/jnuexam/jnuexam.pdf",
  },
  ["eledmac"] = {
    "texmf-dist/doc/latex/eledmac/Makefile",
    "texmf-dist/doc/latex/eledmac/README",
    "texmf-dist/doc/latex/eledmac/eledmac.pdf",
    "texmf-dist/doc/latex/eledmac/eledpar.pdf",
    "texmf-dist/doc/latex/eledmac/examples/1-criticalnotes.pdf",
    "texmf-dist/doc/latex/eledmac/examples/1-sidenotes.pdf",
    "texmf-dist/doc/latex/eledmac/examples/1-tabular.pdf",
    "texmf-dist/doc/latex/eledmac/examples/1-verses.pdf",
    "texmf-dist/doc/latex/eledmac/examples/2-cross_referencing.pdf",
    "texmf-dist/doc/latex/eledmac/examples/2-footnote_spacing.pdf",
    "texmf-dist/doc/latex/eledmac/examples/2-indexing.pdf",
    "texmf-dist/doc/latex/eledmac/examples/2-lemma_disambigution.pdf",
    "texmf-dist/doc/latex/eledmac/examples/2-line_numbers_in_header.pdf",
    "texmf-dist/doc/latex/eledmac/examples/2-one_series_per_pstart.pdf",
    "texmf-dist/doc/latex/eledmac/examples/2-performances.pdf",
    "texmf-dist/doc/latex/eledmac/examples/2-titles_in_line_numbering_with_notes.pdf",
    "texmf-dist/doc/latex/eledmac/examples/2-titles_not_in_line_numbering.pdf",
    "texmf-dist/doc/latex/eledmac/examples/3-eledpar_columns_different_languages.pdf",
    "texmf-dist/doc/latex/eledmac/examples/3-eledpar_mwe.pdf",
    "texmf-dist/doc/latex/eledmac/examples/3-eledpar_pages_different_languages_lualatex.pdf",
    "texmf-dist/doc/latex/eledmac/examples/4-eledpar_column_mix_with_not_column.pdf",
    "texmf-dist/doc/latex/eledmac/examples/4-eledpar_columns_alignment.pdf",
    "texmf-dist/doc/latex/eledmac/examples/4-eledpar_columns_titles_in_line_numbering_with_notes.pdf",
    "texmf-dist/doc/latex/eledmac/examples/4-eledpar_one_series_per_pstart.pdf",
    "texmf-dist/doc/latex/eledmac/examples/4-eledpar_page_titles_in_line_numbering_with_notes.pdf",
    "texmf-dist/doc/latex/eledmac/examples/4-eledpar_pages_long_notes.pdf",
    "texmf-dist/doc/latex/eledmac/examples/4-eledpar_pages_notes_leftpage.pdf",
    "texmf-dist/doc/latex/eledmac/examples/4-eledpar_pages_paragraph_separator_between.pdf",
    "texmf-dist/doc/latex/eledmac/examples/4-eledpar_titles_not_in_line_numbering.pdf",
    "texmf-dist/doc/latex/eledmac/examples/4-eledpar_verse_text_between.pdf",
    "texmf-dist/doc/latex/eledmac/examples/makefile",
    "texmf-dist/doc/latex/eledmac/latexmkrc",
  },
  ["esint"] = {
    "texmf-dist/doc/latex/esint/README",
    "texmf-dist/doc/latex/esint/esint.pdf",
  },
  ["mycv"] = {
    "texmf-dist/doc/latex/mycv/CHANGELOG",
    "texmf-dist/doc/latex/mycv/Examples/Notes.txt",
    "texmf-dist/doc/latex/mycv/Examples/mycv-example-dpl.pdf",
    "texmf-dist/doc/latex/mycv/Examples/mycv-example-dpl2.pdf",
    "texmf-dist/doc/latex/mycv/Examples/mycv-example-spl.pdf",
    "texmf-dist/doc/latex/mycv/Examples/mycv-example-spl2.pdf",
    "texmf-dist/doc/latex/mycv/README",
    "texmf-dist/doc/latex/mycv/mycv.pdf",
  },
  ["aecc"] = {
    "texmf-dist/doc/fonts/aecc/COPYING",
    "texmf-dist/doc/fonts/aecc/MANIFEST",
    "texmf-dist/doc/fonts/aecc/README",
    "texmf-dist/doc/fonts/aecc/clean",
    "texmf-dist/doc/fonts/aecc/go",
    "texmf-dist/doc/fonts/aecc/install",
    "texmf-dist/doc/fonts/aecc/makepl",
  },
  ["librecaslon"] = {
    "texmf-dist/doc/fonts/librecaslon/OFL.txt",
    "texmf-dist/doc/fonts/librecaslon/README",
    "texmf-dist/doc/fonts/librecaslon/samples.pdf",
  },
  ["cmsrb"] = {
    "texmf-dist/doc/fonts/cmsrb/README.md",
    "texmf-dist/doc/fonts/cmsrb/cmsrb-doc-SR.pdf",
    "texmf-dist/doc/fonts/cmsrb/cmsrb-doc.pdf",
  },
  ["datetime2-croatian"] = {
    "texmf-dist/doc/latex/datetime2-croatian/README",
    "texmf-dist/doc/latex/datetime2-croatian/datetime2-croatian.pdf",
  },
  ["ifetex"] = {
    "texmf-dist/doc/latex/ifetex/README",
    "texmf-dist/doc/latex/ifetex/ifetex.pdf",
  },
  ["fixlatvian"] = {
    "texmf-dist/doc/xelatex/fixlatvian/Makefile",
    "texmf-dist/doc/xelatex/fixlatvian/README",
    "texmf-dist/doc/xelatex/fixlatvian/fixlatvian.pdf",
  },
  ["romande"] = {
    "texmf-dist/doc/fonts/romande/COPYING",
    "texmf-dist/doc/fonts/romande/NOTICE.txt",
    "texmf-dist/doc/fonts/romande/README",
    "texmf-dist/doc/fonts/romande/manifest.txt",
    "texmf-dist/doc/fonts/romande/romandeadf.pdf",
  },
  ["bestpapers"] = {
    "texmf-dist/doc/bibtex/bestpapers/Makefile",
    "texmf-dist/doc/bibtex/bestpapers/README",
    "texmf-dist/doc/bibtex/bestpapers/bestpapers-guide.pdf",
  },
  ["kpfonts"] = {
    "texmf-dist/doc/fonts/kpfonts/Kpfonts-Doc-French.pdf",
    "texmf-dist/doc/fonts/kpfonts/kpfonts-abstract.pdf",
    "texmf-dist/doc/fonts/kpfonts/kpfonts-math-symbols-tables.pdf",
    "texmf-dist/doc/fonts/kpfonts/kpfonts.pdf",
    "texmf-dist/doc/fonts/kpfonts/readme.txt",
  },
  ["semaphor"] = {
    "texmf-dist/doc/fonts/semaphor/README",
    "texmf-dist/doc/fonts/semaphor/example.pdf",
    "texmf-dist/doc/fonts/semaphor/test-context.pdf",
  },
  ["kotex-utf"] = {
    "texmf-dist/doc/latex/kotex-utf/ChangeLog",
    "texmf-dist/doc/latex/kotex-utf/README",
    "texmf-dist/doc/latex/kotex-utf/fig/allowbreak-dhucs.pdf",
    "texmf-dist/doc/latex/kotex-utf/fig/fntexp.pdf",
    "texmf-dist/doc/latex/kotex-utf/fig/fntnormal.pdf",
    "texmf-dist/doc/latex/kotex-utf/fig/linebreaktest.pdf",
    "texmf-dist/doc/latex/kotex-utf/fig/testdhucsallowbreak.pdf",
    "texmf-dist/doc/latex/kotex-utf/kotexdoc.pdf",
    "texmf-dist/doc/latex/kotex-utf/yettext.txt",
  },
  ["ibygrk"] = {
    "texmf-dist/doc/fonts/ibygrk/COPYING",
    "texmf-dist/doc/fonts/ibygrk/NEWS",
    "texmf-dist/doc/fonts/ibygrk/README",
    "texmf-dist/doc/fonts/ibygrk/README.ibycus4",
  },
  ["ftcap"] = {
    "texmf-dist/doc/latex/ftcap/ftcap.pdf",
  },
  ["showhyphens"] = {
    "texmf-dist/doc/lualatex/showhyphens/README",
    "texmf-dist/doc/lualatex/showhyphens/showhyphens-doc.pdf",
    "texmf-dist/doc/lualatex/showhyphens/showhyphens-sample.pdf",
  },
  ["hyphen-greek"] = {
    "texmf-dist/doc/generic/elhyphen/README",
    "texmf-dist/doc/generic/elhyphen/anc-test.pdf",
    "texmf-dist/doc/generic/elhyphen/ancient.pdf",
    "texmf-dist/doc/generic/elhyphen/compound.pdf",
    "texmf-dist/doc/generic/elhyphen/copyrite.txt",
    "texmf-dist/doc/generic/elhyphen/modern.pdf",
  },
  ["graphics-cfg"] = {
    "texmf-dist/doc/latex/graphics-cfg/README.md",
  },
  ["pst-spectra"] = {
    "texmf-dist/doc/generic/pst-spectra/README",
    "texmf-dist/doc/generic/pst-spectra/pst-spectra.pdf",
    "texmf-dist/doc/generic/pst-spectra/pst-spectraEN.pdf",
  },
  ["encxvlna"] = {
    "texmf-dist/doc/generic/encxvlna/License.txt",
    "texmf-dist/doc/generic/encxvlna/README",
    "texmf-dist/doc/generic/encxvlna/encxvlna.pdf",
  },
  ["kotex-oblivoir"] = {
    "texmf-dist/doc/latex/kotex-oblivoir/ChangeLog",
    "texmf-dist/doc/latex/kotex-oblivoir/README",
    "texmf-dist/doc/latex/kotex-oblivoir/doc/oblivoir-simpledoc.pdf",
  },
  ["catcodes"] = {
    "texmf-dist/doc/generic/catcodes/README",
    "texmf-dist/doc/generic/catcodes/SrcFILEs.txt",
    "texmf-dist/doc/generic/catcodes/catcodes.pdf",
  },
  ["komacv-rg"] = {
    "texmf-dist/doc/latex/komacv-rg/README.md",
    "texmf-dist/doc/latex/komacv-rg/komacv-addons.pdf",
    "texmf-dist/doc/latex/komacv-rg/komacv-lco.pdf",
    "texmf-dist/doc/latex/komacv-rg/komacv-multilang.pdf",
  },
  ["asymptote-faq-zh-cn"] = {
    "texmf-dist/doc/support/asymptote-faq-zh-cn/README",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/asymptote-faq-zh-cn.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/4-1.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/4-2.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/4-7.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/4-8.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/4-9.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/5-1a.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/5-2.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/5-4.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-1.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-11a.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-11b.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-12.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-14.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-15.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-19.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-2.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-3.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-4.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-5.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-6.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-7b.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-7c.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-8b.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-8c.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/6-9.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/8-4.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/8-8a.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/8-8b.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/8-8c.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/8-8d.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/8-9a.pdf",
    "texmf-dist/doc/support/asymptote-faq-zh-cn/src/figures/8-9b.pdf",
  },
  ["keyval2e"] = {
    "texmf-dist/doc/latex/keyval2e/README",
    "texmf-dist/doc/latex/keyval2e/keyval2e-examples.pdf",
    "texmf-dist/doc/latex/keyval2e/keyval2e-guide.pdf",
  },
  ["dvipos"] = {
    "texmf-dist/doc/man/man1/dvipos.man1.pdf",
  },
  ["keycommand"] = {
    "texmf-dist/doc/latex/keycommand/README",
    "texmf-dist/doc/latex/keycommand/keycommand-example.pdf",
    "texmf-dist/doc/latex/keycommand/keycommand.pdf",
  },
  ["block"] = {
    "texmf-dist/doc/latex/block/block.pdf",
  },
  ["clock"] = {
    "texmf-dist/doc/latex/clock/COPYING",
    "texmf-dist/doc/latex/clock/EMTEX",
    "texmf-dist/doc/latex/clock/HISTORY",
    "texmf-dist/doc/latex/clock/INSTALL",
    "texmf-dist/doc/latex/clock/MIKTEX",
    "texmf-dist/doc/latex/clock/README",
    "texmf-dist/doc/latex/clock/clockdoc.pdf",
  },
  ["tipa"] = {
    "texmf-dist/doc/fonts/tipa/00README",
    "texmf-dist/doc/fonts/tipa/00README.doc",
    "texmf-dist/doc/fonts/tipa/Makefile",
    "texmf-dist/doc/fonts/tipa/Manifest.txt",
    "texmf-dist/doc/fonts/tipa/tipaman.pdf",
    "texmf-dist/doc/fonts/tipa/vowel.pdf",
  },
  ["dtxtut"] = {
    "texmf-dist/doc/latex/dtxtut/README",
    "texmf-dist/doc/latex/dtxtut/dtxtut.pdf",
  },
  ["bangorexam"] = {
    "texmf-dist/doc/latex/bangorexam/README.txt",
    "texmf-dist/doc/latex/bangorexam/bangorexam.pdf",
  },
  ["kerkis"] = {
    "texmf-dist/doc/latex/kerkis/License.txt",
    "texmf-dist/doc/latex/kerkis/README.html",
  },
  ["footnoterange"] = {
    "texmf-dist/doc/latex/footnoterange/README",
    "texmf-dist/doc/latex/footnoterange/footnoterange-example.pdf",
    "texmf-dist/doc/latex/footnoterange/footnoterange.pdf",
  },
  ["nicetext"] = {
    "texmf-dist/doc/latex/nicetext/ANNOUNCE.txt",
    "texmf-dist/doc/latex/nicetext/FILEs.txt",
    "texmf-dist/doc/latex/nicetext/README",
    "texmf-dist/doc/latex/nicetext/README.pdf",
    "texmf-dist/doc/latex/nicetext/RELEAS03.txt",
    "texmf-dist/doc/latex/nicetext/RELEAS04.txt",
    "texmf-dist/doc/latex/nicetext/RELEAS05.txt",
    "texmf-dist/doc/latex/nicetext/RELEASE042.txt",
    "texmf-dist/doc/latex/nicetext/RELEASE043.txt",
    "texmf-dist/doc/latex/nicetext/RELEASE044.txt",
    "texmf-dist/doc/latex/nicetext/RELEASE051.txt",
    "texmf-dist/doc/latex/nicetext/RELEASE064.txt",
    "texmf-dist/doc/latex/nicetext/RELEASE066.txt",
    "texmf-dist/doc/latex/nicetext/SrcFILEs.txt",
    "texmf-dist/doc/latex/nicetext/demo/arseneau.pdf",
    "texmf-dist/doc/latex/nicetext/demo/iso.txt",
    "texmf-dist/doc/latex/nicetext/demo/lns.txt",
    "texmf-dist/doc/latex/nicetext/demo/substr.pdf",
    "texmf-dist/doc/latex/nicetext/fifinddo.pdf",
    "texmf-dist/doc/latex/nicetext/makedoc.pdf",
    "texmf-dist/doc/latex/nicetext/mdoccheat.pdf",
    "texmf-dist/doc/latex/nicetext/niceverb.pdf",
    "texmf-dist/doc/latex/nicetext/wikicheat.pdf",
  },
  ["arrayjobx"] = {
    "texmf-dist/doc/generic/arrayjobx/README",
    "texmf-dist/doc/generic/arrayjobx/arrayjob.pdf",
    "texmf-dist/doc/generic/arrayjobx/arrayjobx.pdf",
  },
  ["digiconfigs"] = {
    "texmf-dist/doc/latex/digiconfigs/README",
    "texmf-dist/doc/latex/digiconfigs/digiconfigs.pdf",
  },
  ["gustlib"] = {
    "texmf-dist/doc/plain/gustlib/README",
    "texmf-dist/doc/plain/gustlib/readme.biblotex",
    "texmf-dist/doc/plain/gustlib/readme.plbtx993",
    "texmf-dist/doc/plain/gustlib/readme.plmac218",
  },
  ["juramisc"] = {
    "texmf-dist/doc/latex/juramisc/README",
    "texmf-dist/doc/latex/juramisc/doc/jmgerdoc.pdf",
    "texmf-dist/doc/latex/juramisc/doc/jmgerdoc_scr.pdf",
  },
  ["geschichtsfrkl"] = {
    "texmf-dist/doc/latex/geschichtsfrkl/README",
    "texmf-dist/doc/latex/geschichtsfrkl/geschichtsfrkl.pdf",
  },
  ["phaistos"] = {
    "texmf-dist/doc/fonts/phaistos/getglyphs",
    "texmf-dist/doc/fonts/phaistos/glyphTable.pdf",
  },
  ["jpsj"] = {
    "texmf-dist/doc/latex/jpsj/injpsj2.pdf",
  },
  ["short-math-guide"] = {
    "texmf-dist/doc/latex/short-math-guide/README",
    "texmf-dist/doc/latex/short-math-guide/short-math-guide.pdf",
  },
  ["datetime2-magyar"] = {
    "texmf-dist/doc/latex/datetime2-magyar/README",
    "texmf-dist/doc/latex/datetime2-magyar/datetime2-magyar.pdf",
  },
  ["pst-vehicle"] = {
    "texmf-dist/doc/generic/pst-vehicle/Changes.txt",
    "texmf-dist/doc/generic/pst-vehicle/README.md",
    "texmf-dist/doc/generic/pst-vehicle/pst-vehicle-doc-fr.pdf",
    "texmf-dist/doc/generic/pst-vehicle/pst-vehicle-doc.pdf",
  },
  ["philokalia"] = {
    "texmf-dist/doc/xelatex/philokalia/README",
    "texmf-dist/doc/xelatex/philokalia/philokalia.pdf",
  },
  ["fancyvrb"] = {
    "texmf-dist/doc/latex/fancyvrb/Changes",
    "texmf-dist/doc/latex/fancyvrb/Makefile",
    "texmf-dist/doc/latex/fancyvrb/README",
    "texmf-dist/doc/latex/fancyvrb/README.contrib",
    "texmf-dist/doc/latex/fancyvrb/fancyvrb.pdf",
    "texmf-dist/doc/latex/fancyvrb/fvrb-ex.pdf",
  },
  ["beamersubframe"] = {
    "texmf-dist/doc/latex/beamersubframe/README",
    "texmf-dist/doc/latex/beamersubframe/beamersubframe-append.pdf",
    "texmf-dist/doc/latex/beamersubframe/beamersubframe-embed.pdf",
    "texmf-dist/doc/latex/beamersubframe/beamersubframe.pdf",
  },
  ["copyedit"] = {
    "texmf-dist/doc/latex/copyedit/README",
    "texmf-dist/doc/latex/copyedit/copyedit.pdf",
    "texmf-dist/doc/latex/copyedit/example/test-copy.pdf",
    "texmf-dist/doc/latex/copyedit/manifest.txt",
  },
  ["mslapa"] = {
    "texmf-dist/doc/latex/mslapa/README",
    "texmf-dist/doc/latex/mslapa/mslapa.pdf",
  },
  ["cursolatex"] = {
    "texmf-dist/doc/latex/cursolatex/Musica.pdf",
    "texmf-dist/doc/latex/cursolatex/README",
    "texmf-dist/doc/latex/cursolatex/TeXworksPDF.pdf",
    "texmf-dist/doc/latex/cursolatex/WinEdt.pdf",
    "texmf-dist/doc/latex/cursolatex/cursolatex.pdf",
    "texmf-dist/doc/latex/cursolatex/emacs.pdf",
    "texmf-dist/doc/latex/cursolatex/integral.pdf",
    "texmf-dist/doc/latex/cursolatex/tabuleiro.pdf",
    "texmf-dist/doc/latex/cursolatex/texmaker.pdf",
    "texmf-dist/doc/latex/cursolatex/texniccenter.pdf",
    "texmf-dist/doc/latex/cursolatex/texstudio.pdf",
    "texmf-dist/doc/latex/cursolatex/tipografia.pdf",
    "texmf-dist/doc/latex/cursolatex/ubuntu.pdf",
  },
  ["nature"] = {
    "texmf-dist/doc/latex/nature/README",
  },
  ["tex-virtual-academy-pl"] = {
    "texmf-dist/doc/generic/tex-virtual-academy-pl/README",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/context/context.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/context/tytuly.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/cototex.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/fonty/font-abc.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/fonty/fonts_inst.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/fonty/index.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/fonty/nfss.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/fonty/qx-info.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/fonty/qx-table1.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/fonty/qx-table2.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/day.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/decode.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/dies.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/flags.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/id.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/index.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/jedn.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/lang.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/lower.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/mil.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/mon.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/name.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/order.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/plmindex.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/porzadek.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/program.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/setki.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/spec.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/toascii.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/typy.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/tys.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/idx/upper.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/index.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/kuchnia/hist.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/kuchnia/tex.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/latex2e.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/listy/description.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/listy/enumerate.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/listy/itemize.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/listy/klopoty.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/listy/list.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/listy/porzadek.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/listy/standard.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/3parttable.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/accents.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/acromake.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/afterpage.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/alltt.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/amsthm.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/arabic.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/array.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/balance.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/bar.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/bbm.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/blkarray.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/bm.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/bophook.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/boxedminipage.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/calc.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/capt-of.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/caption.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/caption2.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/cases.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/ccaption.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/changebar.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/chapterbib.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/cite.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/color.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/colortbl.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/comma.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/cute.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/cuted.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/dblfnote.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/dcolumn.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/delarray.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/deleq.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/dotseqn.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/dow.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/drftcite.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/dropping.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/easyeqn.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/ellipsis.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/endfloat.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/enumerate.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/enumitem.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/epsfig.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/everyship.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/expdlist.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/exscale.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/extmath.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/extramarks.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/fancybox.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/fancyhdr.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/fancyheading.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/filecontents.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/flafter.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/float.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/floatflt.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/floatpag.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/fltpage.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/flushend.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/fn2end.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/fnpara.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/fnpos.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/footmisc.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/footnpag.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/fp.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/framed.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/ftnright.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/geometry.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/graphicx.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/graphpaper.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/harpoon.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/here.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/hhline.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/hyperref.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/ifthen.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/indentfirst.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/index.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/labeldeb.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/lastpage.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/lcg.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/letterspace.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/localloc.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/longtable.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/lscape.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/ltablex.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/ltabptch.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/ltxtable.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/macro.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/macro_t.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/manyfoot.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/minitoc.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/moje_typy.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/moreverb.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/mparhack.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/multfoot.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/multibox.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/multicol.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/multirow.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/numprint.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/oldstyle.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/oubraces.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/overbrace.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/pdfscreen.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/pdfslide.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/pfnote.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/plain.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/printtim.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/pstcol.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/qobitree.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/ragged2e.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/regcount.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/relsize.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/removefromreset.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/rotate.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/rotating.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/selectpage.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/shadepar.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/shadetheorem.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/shadethm.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/shadow.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/shapepar.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/shortvrb.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/showkeys.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/sidecap.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/stmaryrd.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/subeqn.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/subeqnarray.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/subfigure.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/subfloat.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/sublabel.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/subscript.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/supertabular.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/sverb.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/tabularx.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/theorem.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/threeparttable.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/time.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/ulem.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/umoline.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/undertilde.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/url.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/verbatim.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/verbdef.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/version.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/vfrlocal.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/vmargin.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/vrbexin.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/vrflocal.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/wiggly.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/wmcropmark.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/wrapfig.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/xr.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/xspace.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/macro/xtab.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/spisy/chap.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/spisy/l_chap.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/spisy/l_sec.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/spisy/spis.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/chapter.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/count.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/liczniki.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/num.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/p3.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/p4.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/p5.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/p6.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/p7.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/part.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/poziom.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/s.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/sect.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/tc.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/tl.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/tr.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/tx.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/latex2e/tytuly/txx.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/poczatki/index.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/poczatki/latex-pl.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/poczatki/mex.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/poczatki/spw.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/poczatki/tetex.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/poczatki/web2c.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/podstawy/2-0wst1.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/podstawy/2-1coto.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/podstawy/2-1kazio.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/podstawy/2-1przyg.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/podstawy/2-1zece.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/podstawy/bib.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/podstawy/spis.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/podstawy/tex.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/podstawy/title.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/auctex.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/bibtex/bibtex-1.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/bibtex/bibtex-2.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/bibtex/bibtex-3.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/bibtex/bibtex-4.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/bibtex/bibtex-5.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/bibtex/bibtex-6.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/bibtex/bibtex-7.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/bibtex/bibtex-8.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/bibtex/bibtex.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/dvidvi.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/hyph.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/prog/prog.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/tex/odsylacze.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/tex/tex.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/tex/tex_key.html",
    "texmf-dist/doc/generic/tex-virtual-academy-pl/tex/tryby.html",
  },
  ["graphicx-psmin"] = {
    "texmf-dist/doc/latex/graphicx-psmin/README",
    "texmf-dist/doc/latex/graphicx-psmin/graphicx-psmin.pdf",
  },
  ["booktabs"] = {
    "texmf-dist/doc/latex/booktabs/README",
    "texmf-dist/doc/latex/booktabs/booktabs.pdf",
  },
  ["aiaa"] = {
    "texmf-dist/doc/latex/aiaa/README",
    "texmf-dist/doc/latex/aiaa/aiaa.pdf",
    "texmf-dist/doc/latex/aiaa/author_guide.pdf",
    "texmf-dist/doc/latex/aiaa/figure_magnet.pdf",
    "texmf-dist/doc/latex/aiaa/pre2004/CHANGES",
    "texmf-dist/doc/latex/aiaa/pre2004/MANIFEST",
    "texmf-dist/doc/latex/aiaa/pre2004/README",
    "texmf-dist/doc/latex/aiaa/pre2004/aiaa.pdf",
    "texmf-dist/doc/latex/aiaa/pre2004/demos/paper/smpaiaa.ps",
    "texmf-dist/doc/latex/aiaa/pre2004/demos/talk/smptalk.ps",
    "texmf-dist/doc/latex/aiaa/template_advanced.pdf",
    "texmf-dist/doc/latex/aiaa/template_basic.pdf",
  },
  ["chscite"] = {
    "texmf-dist/doc/latex/chscite/README",
    "texmf-dist/doc/latex/chscite/chscite.pdf",
  },
  ["installfont"] = {
    "texmf-dist/doc/support/installfont/LICENSE",
    "texmf-dist/doc/support/installfont/README",
    "texmf-dist/doc/support/installfont/installfont",
    "texmf-dist/doc/support/installfont/installfont.pdf",
    "texmf-dist/doc/support/installfont/manifest.txt",
  },
  ["hhtensor"] = {
    "texmf-dist/doc/latex/hhtensor/ChangeLog",
    "texmf-dist/doc/latex/hhtensor/Makefile",
    "texmf-dist/doc/latex/hhtensor/README",
    "texmf-dist/doc/latex/hhtensor/hhtensor.pdf",
  },
  ["cntformats"] = {
    "texmf-dist/doc/latex/cntformats/README",
    "texmf-dist/doc/latex/cntformats/cntformats_en.pdf",
  },
  ["bxtexlogo"] = {
    "texmf-dist/doc/latex/bxtexlogo/LICENSE",
    "texmf-dist/doc/latex/bxtexlogo/README.md",
    "texmf-dist/doc/latex/bxtexlogo/bxtexlogo-sample.pdf",
  },
  ["skaknew"] = {
    "texmf-dist/doc/fonts/skaknew/README",
    "texmf-dist/doc/fonts/skaknew/SkakNew.pdf",
    "texmf-dist/doc/fonts/skaknew/fonttables.pdf",
  },
  ["greenpoint"] = {
    "texmf-dist/doc/fonts/greenpoint/ChangeLog",
    "texmf-dist/doc/fonts/greenpoint/README",
  },
  ["ijsra"] = {
    "texmf-dist/doc/latex/ijsra/README.md",
    "texmf-dist/doc/latex/ijsra/ijsra.pdf",
  },
  ["feyn"] = {
    "texmf-dist/doc/fonts/feyn/LICENCE",
    "texmf-dist/doc/fonts/feyn/README",
    "texmf-dist/doc/fonts/feyn/VERSION",
    "texmf-dist/doc/fonts/feyn/exercise-font.pdf",
    "texmf-dist/doc/fonts/feyn/feyn.html",
    "texmf-dist/doc/fonts/feyn/feyn.pdf",
    "texmf-dist/doc/fonts/feyn/overheads.pdf",
    "texmf-dist/doc/fonts/feyn/release-notes.html",
  },
  ["colortbl"] = {
    "texmf-dist/doc/latex/colortbl/colortbl-DE.pdf",
    "texmf-dist/doc/latex/colortbl/colortbl.pdf",
  },
  ["draftcopy"] = {
    "texmf-dist/doc/latex/draftcopy/README",
    "texmf-dist/doc/latex/draftcopy/draftcopy.pdf",
  },
  ["hyplain"] = {
    "texmf-dist/doc/plain/hyplain/README",
    "texmf-dist/doc/plain/hyplain/hydoc.pdf",
  },
  ["eltex"] = {
    "texmf-dist/doc/latex/eltex/README",
    "texmf-dist/doc/latex/eltex/man_en.pdf",
    "texmf-dist/doc/latex/eltex/pri_cz.pdf",
  },
  ["hyphen-sanskrit"] = {
    "texmf-dist/doc/generic/hyph-utf8/sa/hyphenmin.txt",
  },
  ["metaplot"] = {
    "texmf-dist/doc/latex/metaplot/README",
    "texmf-dist/doc/latex/metaplot/examples/gpl.txt",
    "texmf-dist/doc/latex/metaplot/metaplot_preprint.pdf",
  },
  ["gradstudentresume"] = {
    "texmf-dist/doc/latex/gradstudentresume/README.txt",
  },
  ["hyperxmp"] = {
    "texmf-dist/doc/latex/hyperxmp/README",
    "texmf-dist/doc/latex/hyperxmp/hyperxmp.pdf",
  },
  ["hrlatex"] = {
    "texmf-dist/doc/latex/hrlatex/README",
    "texmf-dist/doc/latex/hrlatex/hrlatex.pdf",
  },
  ["decimal"] = {
    "texmf-dist/doc/latex/decimal/decimal.pdf",
  },
  ["jfmutil"] = {
    "texmf-dist/doc/fonts/jfmutil/LICENSE",
    "texmf-dist/doc/fonts/jfmutil/README.md",
  },
  ["braket"] = {
    "texmf-dist/doc/latex/braket/braket.pdf",
  },
  ["aramaic-serto"] = {
    "texmf-dist/doc/latex/aramaic-serto/README",
    "texmf-dist/doc/latex/aramaic-serto/sertodoc.pdf",
  },
  ["hepthesis"] = {
    "texmf-dist/doc/latex/hepthesis/ChangeLog",
    "texmf-dist/doc/latex/hepthesis/Makefile",
    "texmf-dist/doc/latex/hepthesis/README",
    "texmf-dist/doc/latex/hepthesis/TODO",
    "texmf-dist/doc/latex/hepthesis/example/Makefile",
    "texmf-dist/doc/latex/hepthesis/example/ckmfitter-alpha-combined.pdf",
    "texmf-dist/doc/latex/hepthesis/example/example.pdf",
    "texmf-dist/doc/latex/hepthesis/example/getNewBibtex",
    "texmf-dist/doc/latex/hepthesis/example/lhcb-detector-cross-section.pdf",
    "texmf-dist/doc/latex/hepthesis/hepthesis.pdf",
  },
  ["univie-ling"] = {
    "texmf-dist/doc/latex/univie-ling/README",
    "texmf-dist/doc/latex/univie-ling/templates/template-expose-deutsch.pdf",
    "texmf-dist/doc/latex/univie-ling/templates/template-expose-english.pdf",
    "texmf-dist/doc/latex/univie-ling/templates/template-paper-deutsch.pdf",
    "texmf-dist/doc/latex/univie-ling/templates/template-paper-english.pdf",
    "texmf-dist/doc/latex/univie-ling/templates/template-thesis-deutsch.pdf",
    "texmf-dist/doc/latex/univie-ling/templates/template-thesis-english.pdf",
    "texmf-dist/doc/latex/univie-ling/templates/template-wlg-article.pdf",
    "texmf-dist/doc/latex/univie-ling/univie-ling-expose.pdf",
    "texmf-dist/doc/latex/univie-ling/univie-ling-paper.pdf",
    "texmf-dist/doc/latex/univie-ling/univie-ling-thesis.pdf",
    "texmf-dist/doc/latex/univie-ling/univie-ling-wlg.pdf",
  },
  ["tkz-doc"] = {
    "texmf-dist/doc/latex/tkz-doc/README",
  },
  ["harvmac"] = {
    "texmf-dist/doc/plain/harvmac/README",
  },
  ["alnumsec"] = {
    "texmf-dist/doc/latex/alnumsec/README",
    "texmf-dist/doc/latex/alnumsec/alnumsec.pdf",
  },
  ["stealcaps"] = {
    "texmf-dist/doc/latex/stealcaps/README.md",
    "texmf-dist/doc/latex/stealcaps/stealcaps.pdf",
    "texmf-dist/doc/latex/stealcaps/testfile/stealcapitals-test.pdf",
  },
  ["coordsys"] = {
    "texmf-dist/doc/latex/coordsys/README",
    "texmf-dist/doc/latex/coordsys/coordsys.pdf",
  },
  ["changes"] = {
    "texmf-dist/doc/latex/changes/README",
    "texmf-dist/doc/latex/changes/changes.english.pdf",
    "texmf-dist/doc/latex/changes/changes.english.withcode.pdf",
    "texmf-dist/doc/latex/changes/changes.ngerman.pdf",
  },
  ["eemeir"] = {
    "texmf-dist/doc/latex/eemeir/README",
    "texmf-dist/doc/latex/eemeir/eemeir.pdf",
  },
  ["babel-breton"] = {
    "texmf-dist/doc/generic/babel-breton/breton.pdf",
  },
  ["cinzel"] = {
    "texmf-dist/doc/fonts/cinzel/README",
    "texmf-dist/doc/fonts/cinzel/SIL_Open_Font_License.txt",
    "texmf-dist/doc/fonts/cinzel/cinzel-samples.pdf",
  },
  ["latex-bib2-ex"] = {
    "texmf-dist/doc/latex/latex-bib2-ex/README",
    "texmf-dist/doc/latex/latex-bib2-ex/data/endnote-fehler.txt",
    "texmf-dist/doc/latex/latex-bib2-ex/doc_data.txt",
  },
  ["blindtext"] = {
    "texmf-dist/doc/latex/blindtext/README",
    "texmf-dist/doc/latex/blindtext/blindtext.pdf",
  },
  ["mcf2graph"] = {
    "texmf-dist/doc/metapost/mcf2graph/CHANGELOG",
    "texmf-dist/doc/metapost/mcf2graph/README",
    "texmf-dist/doc/metapost/mcf2graph/mcf_example.pdf",
    "texmf-dist/doc/metapost/mcf2graph/mcf_manual.pdf",
    "texmf-dist/doc/metapost/mcf2graph/mcf_mplib_exa.pdf",
  },
  ["grfpaste"] = {
    "texmf-dist/doc/latex/grfpaste/doc/grfpaste.pdf",
    "texmf-dist/doc/latex/grfpaste/dvipaste.txt",
  },
  ["frederika2016"] = {
    "texmf-dist/doc/fonts/frederika2016/FontInfo.pdf",
    "texmf-dist/doc/fonts/frederika2016/README",
    "texmf-dist/doc/fonts/frederika2016/testpol.pdf",
  },
  ["flacards"] = {
    "texmf-dist/doc/latex/flacards/COPYING",
    "texmf-dist/doc/latex/flacards/README",
    "texmf-dist/doc/latex/flacards/README.TEXLIVE",
    "texmf-dist/doc/latex/flacards/changelog.txt",
    "texmf-dist/doc/latex/flacards/flacards_ex.pdf",
  },
  ["graph35"] = {
    "texmf-dist/doc/latex/graph35/LICENSE.txt",
    "texmf-dist/doc/latex/graph35/README.md",
    "texmf-dist/doc/latex/graph35/graph35-en.pdf",
    "texmf-dist/doc/latex/graph35/graph35-fr.pdf",
  },
  ["cyklop"] = {
    "texmf-dist/doc/fonts/cyklop/00readme",
    "texmf-dist/doc/fonts/cyklop/00readme-pl",
    "texmf-dist/doc/fonts/cyklop/GUST-FONT-LICENSE.txt",
    "texmf-dist/doc/fonts/cyklop/MANIFEST.txt",
    "texmf-dist/doc/fonts/cyklop/cyklop-info.pdf",
  },
  ["bxpapersize"] = {
    "texmf-dist/doc/latex/bxpapersize/LICENSE",
    "texmf-dist/doc/latex/bxpapersize/README.md",
  },
  ["ocr-b-outline"] = {
    "texmf-dist/doc/fonts/ocr-b-outline/README",
  },
  ["footnotebackref"] = {
    "texmf-dist/doc/latex/footnotebackref/README",
    "texmf-dist/doc/latex/footnotebackref/footnotebackref.pdf",
  },
  ["autosp"] = {
    "texmf-dist/doc/generic/autosp/README",
    "texmf-dist/doc/generic/autosp/autosp.pdf",
    "texmf-dist/doc/generic/autosp/barsant2.pdf",
    "texmf-dist/doc/generic/autosp/geminiani.pdf",
    "texmf-dist/doc/generic/autosp/kinder2.pdf",
    "texmf-dist/doc/generic/autosp/quod2.pdf",
    "texmf-dist/doc/generic/autosp/quod2A.pdf",
    "texmf-dist/doc/generic/autosp/rebar.pdf",
    "texmf-dist/doc/generic/autosp/tex2aspc.pdf",
    "texmf-dist/doc/man/man1/autosp.man1.pdf",
    "texmf-dist/doc/man/man1/tex2aspc.man1.pdf",
  },
  ["gloss"] = {
    "texmf-dist/doc/latex/gloss/README",
    "texmf-dist/doc/latex/gloss/gloss.pdf",
  },
  ["gfsbodoni"] = {
    "texmf-dist/doc/fonts/gfsbodoni/OFL-FAQ.txt",
    "texmf-dist/doc/fonts/gfsbodoni/OFL.txt",
    "texmf-dist/doc/fonts/gfsbodoni/README",
    "texmf-dist/doc/fonts/gfsbodoni/README.TEXLIVE",
    "texmf-dist/doc/fonts/gfsbodoni/gfsbodoni.pdf",
  },
  ["getitems"] = {
    "texmf-dist/doc/latex/getitems/README.md",
    "texmf-dist/doc/latex/getitems/getitems.pdf",
  },
  ["gates"] = {
    "texmf-dist/doc/generic/gates/README",
    "texmf-dist/doc/generic/gates/gates-doc.pdf",
    "texmf-dist/doc/generic/gates/gates-doc.txt",
  },
  ["rsfs"] = {
    "texmf-dist/doc/fonts/rsfs/README",
    "texmf-dist/doc/fonts/rsfs/README.type1",
  },
  ["arabluatex"] = {
    "texmf-dist/doc/lualatex/arabluatex/arabluatex.pdf",
    "texmf-dist/doc/lualatex/arabluatex/samples/al-Jahiz.pdf",
    "texmf-dist/doc/lualatex/arabluatex/samples/template-01.pdf",
  },
  ["chemcompounds"] = {
    "texmf-dist/doc/latex/chemcompounds/README",
    "texmf-dist/doc/latex/chemcompounds/chemcompounds.pdf",
  },
  ["europecv"] = {
    "texmf-dist/doc/latex/europecv/LICENSE",
    "texmf-dist/doc/latex/europecv/README.md",
    "texmf-dist/doc/latex/europecv/europecv.pdf",
    "texmf-dist/doc/latex/europecv/examples/at.pdf",
    "texmf-dist/doc/latex/europecv/examples/greek-utf8.pdf",
    "texmf-dist/doc/latex/europecv/examples/minimal.pdf",
    "texmf-dist/doc/latex/europecv/templates/cv_template_en.pdf",
  },
  ["almfixed"] = {
    "texmf-dist/doc/fonts/almfixed/README",
    "texmf-dist/doc/fonts/almfixed/Unibook_0600.pdf",
    "texmf-dist/doc/fonts/almfixed/Unibook_0750.pdf",
    "texmf-dist/doc/fonts/almfixed/Unibook_08A0.pdf",
    "texmf-dist/doc/fonts/almfixed/Unibook_FB50-1.pdf",
    "texmf-dist/doc/fonts/almfixed/Unibook_FB50-2.pdf",
    "texmf-dist/doc/fonts/almfixed/Unibook_FB50-3.pdf",
    "texmf-dist/doc/fonts/almfixed/Unibook_FE70.pdf",
    "texmf-dist/doc/fonts/almfixed/almfixed-dtk.pdf",
    "texmf-dist/doc/fonts/almfixed/almmono10-sample-crop.pdf",
    "texmf-dist/doc/fonts/almfixed/aristoteles.pdf",
    "texmf-dist/doc/fonts/almfixed/fawaid-427-crop.pdf",
    "texmf-dist/doc/fonts/almfixed/ilahiyyat-1.pdf",
    "texmf-dist/doc/fonts/almfixed/sc-unipad.pdf",
    "texmf-dist/doc/fonts/almfixed/wa.pdf",
  },
  ["datetime2-russian"] = {
    "texmf-dist/doc/latex/datetime2-russian/README",
    "texmf-dist/doc/latex/datetime2-russian/datetime2-russian.pdf",
  },
  ["karnaugh"] = {
    "texmf-dist/doc/latex/karnaugh/kvdoc.pdf",
  },
  ["typicons"] = {
    "texmf-dist/doc/fonts/typicons/README",
    "texmf-dist/doc/fonts/typicons/typicons.pdf",
  },
  ["xpunctuate"] = {
    "texmf-dist/doc/latex/xpunctuate/README",
    "texmf-dist/doc/latex/xpunctuate/xpunctuate.pdf",
  },
  ["boxedminipage"] = {
    "texmf-dist/doc/latex/boxedminipage/boxedminipage.pdf",
  },
  ["mkgrkindex"] = {
    "texmf-dist/doc/support/mkgrkindex/mkgrkindex.pdf",
  },
  ["latex2man"] = {
    "texmf-dist/doc/man/man1/latex2man.man1.pdf",
    "texmf-dist/doc/support/latex2man/CHANGES",
    "texmf-dist/doc/support/latex2man/INSTALL",
    "texmf-dist/doc/support/latex2man/Makefile",
    "texmf-dist/doc/support/latex2man/README",
    "texmf-dist/doc/support/latex2man/latex2man-CHANGES.html",
    "texmf-dist/doc/support/latex2man/latex2man.html",
    "texmf-dist/doc/support/latex2man/latex2man.pdf",
    "texmf-dist/doc/support/latex2man/latex2man.txt",
  },
  ["leftidx"] = {
    "texmf-dist/doc/latex/leftidx/README",
    "texmf-dist/doc/latex/leftidx/leftidx.pdf",
  },
  ["gtrcrd"] = {
    "texmf-dist/doc/latex/gtrcrd/README",
    "texmf-dist/doc/latex/gtrcrd/gtrcrd-doc.pdf",
  },
  ["chktex"] = {
    "texmf-dist/doc/chktex/ChkTeX.pdf",
    "texmf-dist/doc/man/man1/chktex.man1.pdf",
    "texmf-dist/doc/man/man1/chkweb.man1.pdf",
    "texmf-dist/doc/man/man1/deweb.man1.pdf",
  },
  ["ptex-base"] = {
    "texmf-dist/doc/ptex/ptex-base/Changelog_eptex_beforeTL",
    "texmf-dist/doc/ptex/ptex-base/Changelog_ptex",
    "texmf-dist/doc/ptex/ptex-base/LICENSE",
    "texmf-dist/doc/ptex/ptex-base/Makefile",
    "texmf-dist/doc/ptex/ptex-base/README.md",
    "texmf-dist/doc/ptex/ptex-base/README_ptex",
    "texmf-dist/doc/ptex/ptex-base/eptex_resume.pdf",
    "texmf-dist/doc/ptex/ptex-base/eptexdoc.pdf",
    "texmf-dist/doc/ptex/ptex-base/jfm.pdf",
    "texmf-dist/doc/ptex/ptex-base/jtex.pdf",
    "texmf-dist/doc/ptex/ptex-base/jtexdoc.pdf",
    "texmf-dist/doc/ptex/ptex-base/ptexdoc.pdf",
    "texmf-dist/doc/ptex/ptex-base/ptexskip.pdf",
  },
  ["fontaxes"] = {
    "texmf-dist/doc/latex/fontaxes/README",
    "texmf-dist/doc/latex/fontaxes/fontaxes.pdf",
  },
  ["foilhtml"] = {
    "texmf-dist/doc/latex/foilhtml/readme.v12",
  },
  ["xebaposter"] = {
    "texmf-dist/doc/latex/xebaposter/README",
    "texmf-dist/doc/latex/xebaposter/images/docs-boxshape.pdf",
    "texmf-dist/doc/latex/xebaposter/images/docs-headerborder.pdf",
    "texmf-dist/doc/latex/xebaposter/images/docs-headershape.pdf",
    "texmf-dist/doc/latex/xebaposter/images/docs-shade.pdf",
    "texmf-dist/doc/latex/xebaposter/images/fig3.pdf",
    "texmf-dist/doc/latex/xebaposter/images/fig4.pdf",
    "texmf-dist/doc/latex/xebaposter/images/fig5.pdf",
    "texmf-dist/doc/latex/xebaposter/images/fig6.pdf",
    "texmf-dist/doc/latex/xebaposter/poster-fa.pdf",
    "texmf-dist/doc/latex/xebaposter/xebaposter-doc.pdf",
  },
  ["tikzsymbols"] = {
    "texmf-dist/doc/latex/tikzsymbols/README.md",
    "texmf-dist/doc/latex/tikzsymbols/tikzsymbols.pdf",
  },
  ["docmfp"] = {
    "texmf-dist/doc/latex/docmfp/README",
    "texmf-dist/doc/latex/docmfp/docmfp.pdf",
  },
  ["cntperchap"] = {
    "texmf-dist/doc/latex/cntperchap/README",
    "texmf-dist/doc/latex/cntperchap/cntperchap_doc.pdf",
    "texmf-dist/doc/latex/cntperchap/cntperchap_example.pdf",
  },
  ["clrscode"] = {
    "texmf-dist/doc/latex/clrscode/README",
    "texmf-dist/doc/latex/clrscode/clrscode.pdf",
  },
  ["longfbox"] = {
    "texmf-dist/doc/latex/longfbox/README",
    "texmf-dist/doc/latex/longfbox/longfbox.html",
    "texmf-dist/doc/latex/longfbox/longfbox.pdf",
  },
  ["fnumprint"] = {
    "texmf-dist/doc/latex/fnumprint/README",
    "texmf-dist/doc/latex/fnumprint/fnumprint.pdf",
  },
  ["fnpct"] = {
    "texmf-dist/doc/latex/fnpct/README",
    "texmf-dist/doc/latex/fnpct/fnpct_en.pdf",
  },
  ["sseq"] = {
    "texmf-dist/doc/latex/sseq/sseq.pdf",
  },
  ["fig4latex"] = {
    "texmf-dist/doc/support/fig4latex/CHANGES",
    "texmf-dist/doc/support/fig4latex/COPYING",
    "texmf-dist/doc/support/fig4latex/README",
    "texmf-dist/doc/support/fig4latex/example.pdf",
  },
  ["scrlttr2copy"] = {
    "texmf-dist/doc/latex/scrlttr2copy/README.md",
    "texmf-dist/doc/latex/scrlttr2copy/letter-copy-test.pdf",
  },
  ["euenc"] = {
    "texmf-dist/doc/latex/euenc/README",
    "texmf-dist/doc/latex/euenc/euenc.pdf",
  },
  ["coollist"] = {
    "texmf-dist/doc/latex/coollist/README",
    "texmf-dist/doc/latex/coollist/coollist.pdf",
  },
  ["combelow"] = {
    "texmf-dist/doc/latex/combelow/README",
    "texmf-dist/doc/latex/combelow/combelow.pdf",
  },
  ["esrelation"] = {
    "texmf-dist/doc/fonts/esrelation/README",
    "texmf-dist/doc/fonts/esrelation/README.md",
    "texmf-dist/doc/fonts/esrelation/esrelation.pdf",
  },
  ["bidihl"] = {
    "texmf-dist/doc/xelatex/bidihl/README",
    "texmf-dist/doc/xelatex/bidihl/bidihl-doc.pdf",
    "texmf-dist/doc/xelatex/bidihl/test-bidihl.pdf",
  },
  ["askmaps"] = {
    "texmf-dist/doc/latex/askmaps/README",
    "texmf-dist/doc/latex/askmaps/askmaps.pdf",
  },
  ["amsldoc-vn"] = {
    "texmf-dist/doc/latex/amsldoc-vn/Makefile",
    "texmf-dist/doc/latex/amsldoc-vn/README",
    "texmf-dist/doc/latex/amsldoc-vn/TODO",
    "texmf-dist/doc/latex/amsldoc-vn/amsldoc-print-vi.pdf",
    "texmf-dist/doc/latex/amsldoc-vn/amsldoc-vi.pdf",
  },
  ["dhua"] = {
    "texmf-dist/doc/latex/dhua/README",
    "texmf-dist/doc/latex/dhua/README.pdf",
    "texmf-dist/doc/latex/dhua/SrcFILEs.txt",
    "texmf-dist/doc/latex/dhua/dhua.pdf",
  },
  ["ltxtools"] = {
    "texmf-dist/doc/latex/ltxtools/README",
  },
  ["engrec"] = {
    "texmf-dist/doc/latex/engrec/LISEZMOI",
    "texmf-dist/doc/latex/engrec/README",
    "texmf-dist/doc/latex/engrec/engrec.pdf",
  },
  ["placeat"] = {
    "texmf-dist/doc/lualatex/placeat/README.md",
    "texmf-dist/doc/lualatex/placeat/placeat.pdf",
  },
  ["monofill"] = {
    "texmf-dist/doc/latex/monofill/Announce.txt",
    "texmf-dist/doc/latex/monofill/README",
    "texmf-dist/doc/latex/monofill/SrcFILEs.txt",
    "texmf-dist/doc/latex/monofill/monofill.pdf",
  },
  ["jknapltx"] = {
    "texmf-dist/doc/latex/jknapltx/00readme.txt",
    "texmf-dist/doc/latex/jknapltx/README.TEXLIVE",
  },
  ["siunitx"] = {
    "texmf-dist/doc/latex/siunitx/README.md",
    "texmf-dist/doc/latex/siunitx/siunitx.pdf",
  },
  ["sasnrdisplay"] = {
    "texmf-dist/doc/latex/sasnrdisplay/README",
    "texmf-dist/doc/latex/sasnrdisplay/SASnRdisplay.pdf",
  },
  ["recipebook"] = {
    "texmf-dist/doc/latex/recipebook/LongSample.pdf",
    "texmf-dist/doc/latex/recipebook/README",
    "texmf-dist/doc/latex/recipebook/ShortSample.pdf",
  },
  ["ksp-thesis"] = {
    "texmf-dist/doc/latex/ksp-thesis/LISTOFFILES",
    "texmf-dist/doc/latex/ksp-thesis/README.md",
    "texmf-dist/doc/latex/ksp-thesis/ksp-thesis.pdf",
  },
  ["droid"] = {
    "texmf-dist/doc/fonts/droid/CHANGES",
    "texmf-dist/doc/fonts/droid/README",
    "texmf-dist/doc/fonts/droid/droid.pdf",
    "texmf-dist/doc/fonts/droid/droidsans-samples.pdf",
    "texmf-dist/doc/fonts/droid/droidsansmono-samples.pdf",
    "texmf-dist/doc/fonts/droid/droidserif-samples.pdf",
    "texmf-dist/doc/fonts/droid/manifest.txt",
  },
  ["altfont"] = {
    "texmf-dist/doc/latex/altfont/README",
    "texmf-dist/doc/latex/altfont/altfont.pdf",
    "texmf-dist/doc/latex/altfont/psfont.pdf",
  },
  ["interval"] = {
    "texmf-dist/doc/latex/interval/README",
    "texmf-dist/doc/latex/interval/interval.pdf",
  },
  ["algolrevived"] = {
    "texmf-dist/doc/fonts/algolrevived/OFL.txt",
    "texmf-dist/doc/fonts/algolrevived/README",
    "texmf-dist/doc/fonts/algolrevived/algolrevived-doc.pdf",
  },
  ["cdpbundl"] = {
    "texmf-dist/doc/latex/cdpbundl/00readme.txt",
    "texmf-dist/doc/latex/cdpbundl/Makefile",
    "texmf-dist/doc/latex/cdpbundl/README",
    "texmf-dist/doc/latex/cdpbundl/cdp-ver-0-36.pdf",
    "texmf-dist/doc/latex/cdpbundl/cdpbundl-doc.pdf",
    "texmf-dist/doc/latex/cdpbundl/manifest.txt",
  },
  ["ticollege"] = {
    "texmf-dist/doc/latex/ticollege/README",
    "texmf-dist/doc/latex/ticollege/ticollege-doc.pdf",
  },
  ["algorithm2e"] = {
    "texmf-dist/doc/latex/algorithm2e/README",
    "texmf-dist/doc/latex/algorithm2e/algorithm2e.pdf",
  },
  ["cd-cover"] = {
    "texmf-dist/doc/latex/cd-cover/README",
    "texmf-dist/doc/latex/cd-cover/cd-cover.pdf",
  },
  ["es-tex-faq"] = {
    "texmf-dist/doc/generic/es-tex-faq/FAQ-CervanTeX.html",
    "texmf-dist/doc/generic/es-tex-faq/FAQ-CervanTeX.pdf",
  },
  ["context"] = {
    "texmf-dist/doc/context/documents/general/fonts/fonts-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/magazines/mag-0000-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/magazines/mag-0002-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/magazines/mag-0006-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/magazines/mag-0007-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/magazines/mag-0010-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/magazines/mag-1101-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/magazines/mag-1102-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/magazines/mag-1103-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/about.pdf",
    "texmf-dist/doc/context/documents/general/manuals/bidi.pdf",
    "texmf-dist/doc/context/documents/general/manuals/charts-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/cld-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/colors-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/columnsets.pdf",
    "texmf-dist/doc/context/documents/general/manuals/details.pdf",
    "texmf-dist/doc/context/documents/general/manuals/epub-mkiv-demo.pdf",
    "texmf-dist/doc/context/documents/general/manuals/epub-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/hybrid.pdf",
    "texmf-dist/doc/context/documents/general/manuals/languages-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/libraries-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/lua-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/luatex.pdf",
    "texmf-dist/doc/context/documents/general/manuals/ma-cb-en.pdf",
    "texmf-dist/doc/context/documents/general/manuals/math-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/metafun-p.pdf",
    "texmf-dist/doc/context/documents/general/manuals/metafun-s.pdf",
    "texmf-dist/doc/context/documents/general/manuals/mk.pdf",
    "texmf-dist/doc/context/documents/general/manuals/mkiv-publications.pdf",
    "texmf-dist/doc/context/documents/general/manuals/mmlexamp.pdf",
    "texmf-dist/doc/context/documents/general/manuals/mmlprime.pdf",
    "texmf-dist/doc/context/documents/general/manuals/mreadme.pdf",
    "texmf-dist/doc/context/documents/general/manuals/nodes.pdf",
    "texmf-dist/doc/context/documents/general/manuals/notnow.pdf",
    "texmf-dist/doc/context/documents/general/manuals/onandon.pdf",
    "texmf-dist/doc/context/documents/general/manuals/pagecolumns.pdf",
    "texmf-dist/doc/context/documents/general/manuals/rules-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/spacing-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/spreadsheets-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/sql-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/steps-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/still.pdf",
    "texmf-dist/doc/context/documents/general/manuals/swiglib-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/templates-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/tiptrick.pdf",
    "texmf-dist/doc/context/documents/general/manuals/tools-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/units-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/workflows-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/xml-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/manuals/xtables-mkiv.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-cs.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-de.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-en.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-fr.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-it.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-mapping-cs.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-mapping-de.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-mapping-en.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-mapping-fr.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-mapping-it.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-mapping-nl.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-mapping-ro.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-nl.pdf",
    "texmf-dist/doc/context/documents/general/qrcs/setup-ro.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2010/bachotex-2010-clash.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2010/bachotex-2010-move.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2011/bachotex-2011-cld-and-mkvi.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2013/bachotex-2013-bits.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2013/bachotex-2013-luatex.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2013/bachotex-2013-sense.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2013/bachotex-2013-speed.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2015/bachotex-2015-ligatures.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2016/bachotex-2016-opentype.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2016/bachotex-2016-toolbox.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2017/bachotex-2017-emoji-demo.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2017/bachotex-2017-emoji.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2017/bachotex-2017-variablefonts-demo.pdf",
    "texmf-dist/doc/context/presentations/bachotex/2017/bachotex-2017-variablefonts.pdf",
    "texmf-dist/doc/context/presentations/context/2011/context-2011-ebook-export.pdf",
    "texmf-dist/doc/context/presentations/context/2011/context-2011-mathml-update.pdf",
    "texmf-dist/doc/context/presentations/context/2011/context-2011-metapost-how-we-adapt.pdf",
    "texmf-dist/doc/context/presentations/context/2011/context-2011-sorting-registers.pdf",
    "texmf-dist/doc/context/presentations/context/2012/context-2012-after-the-cleanup.pdf",
    "texmf-dist/doc/context/presentations/context/2012/context-2012-lexing-sources.pdf",
    "texmf-dist/doc/context/presentations/context/2012/context-2012-mixed-columns.pdf",
    "texmf-dist/doc/context/presentations/context/2012/context-2012-the-script.pdf",
    "texmf-dist/doc/context/presentations/context/2012/context-2012-visual-debugging.pdf",
    "texmf-dist/doc/context/presentations/context/2012/context-2012-xml-news.pdf",
    "texmf-dist/doc/context/presentations/context/2013/context-2013-math.pdf",
    "texmf-dist/doc/context/presentations/context/2013/context-2013-speed.pdf",
    "texmf-dist/doc/context/presentations/context/2015/context-2015-status.pdf",
    "texmf-dist/doc/context/presentations/context/2016/context-2016-luatex.pdf",
    "texmf-dist/doc/context/presentations/examples/present-balls-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-colorful-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-funny-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-fuzzy-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-green-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-grow-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-organic-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-original-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-punk-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-random-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-shaded-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-split-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-stepper-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-tiles-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-weird-001.pdf",
    "texmf-dist/doc/context/presentations/examples/present-windows-001.pdf",
    "texmf-dist/doc/context/presentations/present-readme.pdf",
    "texmf-dist/doc/context/scripts/mkii/ctxtools.html",
    "texmf-dist/doc/context/scripts/mkii/imgtopdf.html",
    "texmf-dist/doc/context/scripts/mkii/mptopdf.html",
    "texmf-dist/doc/context/scripts/mkii/pdftools.html",
    "texmf-dist/doc/context/scripts/mkii/pstopdf.html",
    "texmf-dist/doc/context/scripts/mkii/rlxtools.html",
    "texmf-dist/doc/context/scripts/mkii/texexec.html",
    "texmf-dist/doc/context/scripts/mkii/texmfstart.html",
    "texmf-dist/doc/context/scripts/mkii/textools.html",
    "texmf-dist/doc/context/scripts/mkii/texutil.html",
    "texmf-dist/doc/context/scripts/mkii/tmftools.html",
    "texmf-dist/doc/context/scripts/mkii/xmltools.html",
    "texmf-dist/doc/context/scripts/mkiv/context.html",
    "texmf-dist/doc/context/scripts/mkiv/luatools.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-babel.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-base.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-bibtex.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-cache.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-chars.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-check.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-colors.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-context.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-dvi.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-epub.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-evohome.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-fcd.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-flac.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-fonts.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-grep.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-interface.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-metapost.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-metatex.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-modules.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-package.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-patterns.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-pdf.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-plain.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-profile.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-rsync.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-scite.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-server.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-texworks.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-timing.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-tools.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-unicode.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-unzip.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-update.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-watch.html",
    "texmf-dist/doc/context/scripts/mkiv/mtx-youless.html",
    "texmf-dist/doc/context/scripts/mkiv/mtxrun.html",
    "texmf-dist/doc/context/sources/general/manuals/details/cow.pdf",
    "texmf-dist/doc/context/sources/general/manuals/metafun/mycow.pdf",
    "texmf-dist/doc/context/sources/general/manuals/metafun/somecow.pdf",
    "texmf-dist/doc/context/sources/general/manuals/mk/mk-last-state.pdf",
    "texmf-dist/doc/context/sources/general/manuals/onandon/onandon-seguiemj.pdf",
    "texmf-dist/doc/context/sources/general/manuals/publications/122176.pdf",
    "texmf-dist/doc/context/sources/general/manuals/publications/124484.pdf",
    "texmf-dist/doc/context/sources/general/manuals/publications/4953942.pdf",
    "texmf-dist/doc/context/sources/general/manuals/publications/971.pdf",
    "texmf-dist/doc/context/sources/general/manuals/start/en/hasseltbook.pdf",
    "texmf-dist/doc/context/sources/general/manuals/start/en/ma-cb-en.pdf",
    "texmf-dist/doc/context/sources/general/manuals/start/graphics/cow.pdf",
    "texmf-dist/doc/context/sources/general/manuals/start/graphics/fig-page-areas-en.pdf",
    "texmf-dist/doc/context/sources/general/manuals/start/graphics/fig-page-parameters-en.pdf",
    "texmf-dist/doc/context/sources/general/manuals/start/graphics/ma-cb-00.pdf",
    "texmf-dist/doc/context/sources/general/manuals/start/graphics/minibook-en.pdf",
    "texmf-dist/doc/context/sources/general/manuals/swiglib/swiglib-mkiv-gm-1.pdf",
    "texmf-dist/doc/man/man1/context.man1.pdf",
    "texmf-dist/doc/man/man1/luatools.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-babel.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-base.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-bibtex.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-cache.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-chars.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-check.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-colors.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-context.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-dvi.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-epub.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-evohome.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-fcd.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-flac.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-fonts.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-grep.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-interface.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-metapost.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-metatex.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-modules.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-package.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-patterns.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-pdf.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-plain.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-profile.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-rsync.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-scite.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-server.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-texworks.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-timing.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-tools.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-unicode.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-unzip.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-update.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-watch.man1.pdf",
    "texmf-dist/doc/man/man1/mtx-youless.man1.pdf",
    "texmf-dist/doc/man/man1/mtxrun.man1.pdf",
    "texmf-dist/doc/man/man1/texexec.man1.pdf",
    "texmf-dist/doc/man/man1/texmfstart.man1.pdf",
  },
  ["akktex"] = {
    "texmf-dist/doc/latex/akktex/README",
    "texmf-dist/doc/latex/akktex/README.TEXLIVE",
  },
  ["cprotect"] = {
    "texmf-dist/doc/latex/cprotect/README",
    "texmf-dist/doc/latex/cprotect/README.txt",
    "texmf-dist/doc/latex/cprotect/cprotect.pdf",
  },
  ["asapsym"] = {
    "texmf-dist/doc/fonts/asapsym/README.md",
    "texmf-dist/doc/fonts/asapsym/asapsym.pdf",
  },
  ["belleek"] = {
    "texmf-dist/doc/fonts/belleek/README",
  },
  ["psnfss"] = {
    "texmf-dist/doc/latex/psnfss/00readme.txt",
    "texmf-dist/doc/latex/psnfss/changes.txt",
    "texmf-dist/doc/latex/psnfss/manifest.txt",
    "texmf-dist/doc/latex/psnfss/psfonts.pdf",
    "texmf-dist/doc/latex/psnfss/psnfss2e.pdf",
  },
}
return s_meta, tlp_from_runfile, tlp_doclist
